@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -23,6 +23,7 @@ public abstract class AntObject {
 	protected String fName;
 	private String fPluginLabel;
 	private boolean eclipseRuntime= true;
+	private String fURI= null;
 	
 	/**
 	 * Gets the className.
@@ -81,7 +82,10 @@ public abstract class AntObject {
 	 * @see java.lang.Object#toString()
 	 */
 	public String toString() {
-		return fName;
+		if (fURI == null || fURI.equals("") || fURI.equals("antlib:org.apache.tools.ant")) {  //$NON-NLS-1$//$NON-NLS-2$
+            return fName;
+        }
+		return fURI + ':' + fName;
 	}
 	/**
 	 * Returns whether this Ant object has been created because of an extension
@@ -96,7 +100,7 @@ public abstract class AntObject {
 	 * Sets that this Ant object has been created by the appropriate extension
 	 * point.
 	 * @param isDefault Whether this Ant object has been created because of an
-	 * extension point defintion.
+	 * extension point definition.
 	 * @deprecated Since 3.0 Set the plugin label to indicate a default object
 	 */
 	public void setIsDefault(boolean isDefault) {
@@ -141,4 +145,19 @@ public abstract class AntObject {
 	public void setEclipseRuntimeRequired(boolean eclipseRuntime) {
 		this.eclipseRuntime= eclipseRuntime;
 	}
+	
+	/**
+	 * Returns the URI namespace that this Ant object should live in.
+	 * Defaults value is <code>null</code>
+	 * 
+	 * @return The URI that this Ant object should live in
+     * @since 3.2
+	 */
+	public String getURI() {
+		return fURI;
+	}
+	
+	public void setURI(String uri) {
+		fURI= uri;
+	}
 }
