@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,16 +12,25 @@ package org.eclipse.core.internal.filebuffers;
 
 import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileNotFoundException;
-import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.OutputStream;
 import java.io.Reader;
-import java.io.UnsupportedEncodingException;
+import java.io.SequenceInputStream;
+import java.nio.ByteBuffer;
+import java.nio.CharBuffer;
+import java.nio.charset.CharacterCodingException;
+import java.nio.charset.Charset;
+import java.nio.charset.CharsetEncoder;
+import java.nio.charset.CodingErrorAction;
+import java.nio.charset.IllegalCharsetNameException;
+import java.nio.charset.UnmappableCharacterException;
+import java.nio.charset.UnsupportedCharsetException;
+
+import org.eclipse.core.filesystem.EFS;
+import org.eclipse.core.filesystem.IFileInfo;
+import org.eclipse.core.filesystem.IFileStore;
 
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IProgressMonitor;
@@ -35,9 +44,11 @@ import org.eclipse.core.runtime.content.IContentType;
 import org.eclipse.core.resources.IResourceStatus;
 
 import org.eclipse.core.filebuffers.FileBuffers;
+import org.eclipse.core.filebuffers.IFileBufferStatusCodes;
 import org.eclipse.core.filebuffers.IPersistableAnnotationModel;
 import org.eclipse.core.filebuffers.ITextFileBuffer;
 
+import org.eclipse.jface.text.Assert;
 import org.eclipse.jface.text.DocumentEvent;
 import org.eclipse.jface.text.IDocument;
 import org.eclipse.jface.text.IDocumentListener;
@@ -82,7 +93,7 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 	/**
 	 * Constant for representing the error status. This is considered a value object.
 	 */
-	private static final IStatus STATUS_ERROR= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.INFO, FileBuffersMessages.FileBuffer_status_error, null);
+	private static final IStatus STATUS_ERROR= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, FileBuffersMessages.FileBuffer_status_error, null);
 	/**
 	 * Constant denoting UTF-8 encoding.
 	 */
@@ -107,6 +118,16 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 	private boolean fHasBOM;
 	/** The annotation model of this file buffer */
     private IAnnotationModel fAnnotationModel;
+	/**
+	 * Lock for lazy creation of annotation model.
+	 * @since 3.2
+	 */
+	private final Object fAnnotationModelCreationLock= new Object();
+	/**
+	 * Tells whether the cache is up to date.
+	 * @since 3.2
+	 */
+	private boolean fIsCacheUpdated= false;
 
 
 	public JavaTextFileBuffer(TextFileBufferManager manager) {
@@ -114,7 +135,7 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 	}
 
 	/*
-	 * @see org.eclipse.core.buffer.text.IBufferedTextFile#getDocument()
+	 * @see org.eclipse.core.filebuffers.ITextFileBuffer#getDocument()
 	 */
 	public IDocument getDocument() {
 		return fDocument;
@@ -124,23 +145,32 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 	 * @see org.eclipse.core.filebuffers.ITextFileBuffer#getAnnotationModel()
 	 */
 	public IAnnotationModel getAnnotationModel() {
+		synchronized (fAnnotationModelCreationLock) {
+			if (fAnnotationModel == null && !isDisconnected()) {
+				fAnnotationModel= fManager.createAnnotationModel(getLocation());
+				if (fAnnotationModel != null)
+					fAnnotationModel.connect(fDocument);
+			}
+		}
 		return fAnnotationModel;
 	}
 
 	/*
-	 * @see org.eclipse.core.buffer.text.IBufferedTextFile#getEncoding()
+	 * @see org.eclipse.core.filebuffers.ITextFileBuffer#getEncoding()
 	 */
 	public String getEncoding() {
+		if (!fIsCacheUpdated)
+			cacheEncodingState(null);
 		return fEncoding;
 	}
 
 	/*
-	 * @see org.eclipse.core.buffer.text.IBufferedTextFile#setEncoding(java.lang.String)
+	 * @see org.eclipse.core.filebuffers.ITextFileBuffer#setEncoding(java.lang.String)
 	 */
 	public void setEncoding(String encoding) {
 		fExplicitEncoding= encoding;
 		if (encoding == null || encoding.equals(fEncoding))
-			cacheEncodingState(null);
+			fIsCacheUpdated= false;
 		else {
 			fEncoding= encoding;
 			fHasBOM= false;
@@ -148,7 +178,7 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 	}
 
 	/*
-	 * @see org.eclipse.core.buffer.text.IBufferedFile#getStatus()
+	 * @see org.eclipse.core.filebuffers.ITextFileBuffer#getStatus()
 	 */
 	public IStatus getStatus() {
 		if (!isDisconnected()) {
@@ -159,48 +189,42 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 		return STATUS_ERROR;
 	}
 
-	private InputStream getFileContents(File file, IProgressMonitor monitor) {
-		try {
-			if (file != null)
-				return new FileInputStream(file);
-		} catch (FileNotFoundException e) {
-		}
-		return null;
+	private InputStream getFileContents(IFileStore fileStore, IProgressMonitor monitor) throws CoreException {
+		if (fileStore == null)
+			return null;
+		
+		return fileStore.openInputStream(EFS.NONE, null);
 	}
 
-	private void setFileContents(InputStream stream, boolean overwrite, IProgressMonitor monitor) {
+	private void setFileContents(InputStream stream, boolean overwrite, IProgressMonitor monitor) throws CoreException {
+		OutputStream out= fFileStore.openOutputStream(EFS.NONE, null);
 		try {
-			OutputStream out= new FileOutputStream(fFile, false);
-
-			try {
-				byte[] buffer = new byte[8192];
-				while (true) {
-					int bytesRead = -1;
-					try {
-						bytesRead = stream.read(buffer);
-					} catch (IOException e) {
-					}
-					if (bytesRead == -1)
-						break;
-					try {
-						out.write(buffer, 0, bytesRead);
-					} catch (IOException e) {
-					}
-					if (monitor != null)
-						monitor.worked(1);
+			byte[] buffer= new byte[8192];
+			while (true) {
+				int bytesRead= -1;
+				try {
+					bytesRead= stream.read(buffer);
+				} catch (IOException e) {
 				}
+				if (bytesRead == -1)
+					break;
+				try {
+					out.write(buffer, 0, bytesRead);
+				} catch (IOException e) {
+				}
+				if (monitor != null)
+					monitor.worked(1);
+			}
+		} finally {
+			try {
+				stream.close();
+			} catch (IOException e) {
 			} finally {
 				try {
-					stream.close();
+					out.close();
 				} catch (IOException e) {
-				} finally {
-					try {
-						out.close();
-					} catch (IOException e) {
-					}
 				}
 			}
-		} catch (FileNotFoundException e) {
 		}
 	}
 
@@ -217,7 +241,7 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 		try {
 			original= fManager.createEmptyDocument(getLocation());
 			cacheEncodingState(monitor);
-			setDocumentContent(original, fFile, fEncoding, monitor);
+			setDocumentContent(original, fFileStore, fEncoding, fHasBOM, monitor);
 		} catch (CoreException x) {
 			fStatus= x.getStatus();
 		}
@@ -248,8 +272,8 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 			if (replaceContents)
 				fManager.fireBufferContentReplaced(this);
 
-			if (fFile != null)
-				fSynchronizationStamp= fFile.lastModified();
+			if (fFileStore != null)
+				fSynchronizationStamp= fFileStore.fetchInfo().getLastModified();
 
 			if (fAnnotationModel instanceof IPersistableAnnotationModel) {
 				IPersistableAnnotationModel persistableModel= (IPersistableAnnotationModel) fAnnotationModel;
@@ -274,7 +298,7 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 	 * @since 3.1
 	 */
 	public IContentType getContentType () throws CoreException {
-		if (fFile == null)
+		if (fFileStore == null)
 			return null;
 
 		InputStream stream= null;
@@ -282,7 +306,7 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 			if (isDirty()) {
 				Reader reader= new DocumentReader(getDocument());
 				try {
-					IContentDescription desc= Platform.getContentTypeManager().getDescriptionFor(reader, fFile.getName(), NO_PROPERTIES);
+					IContentDescription desc= Platform.getContentTypeManager().getDescriptionFor(reader, fFileStore.getName(), NO_PROPERTIES);
 					if (desc != null && desc.getContentType() != null)
 						return desc.getContentType();
 				} finally {
@@ -293,13 +317,13 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 					}
 				}
 			}
-			stream= new FileInputStream(fFile);
-			IContentDescription desc= Platform.getContentTypeManager().getDescriptionFor(stream, fFile.getName(), NO_PROPERTIES);
+			stream= fFileStore.openInputStream(EFS.NONE, null);
+			IContentDescription desc= Platform.getContentTypeManager().getDescriptionFor(stream, fFileStore.getName(), NO_PROPERTIES);
 			if (desc != null && desc.getContentType() != null)
 				return desc.getContentType();
 			return null;
 		} catch (IOException x) {
-			throw new CoreException(new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, NLSUtility.format(FileBuffersMessages.FileBuffer_error_queryContentDescription, fFile.getPath()), x));
+			throw new CoreException(new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, NLSUtility.format(FileBuffersMessages.FileBuffer_error_queryContentDescription, fFileStore.toString()), x));
 		} finally {
 			try {
 				if (stream != null)
@@ -332,10 +356,7 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 		try {
 			fDocument= fManager.createEmptyDocument(getLocation());
 			cacheEncodingState(monitor);
-			setDocumentContent(fDocument, fFile, fEncoding, monitor);
-
-			fAnnotationModel= fManager.createAnnotationModel(getLocation());
-
+			setDocumentContent(fDocument, fFileStore, fEncoding, fHasBOM, monitor);
 		} catch (CoreException x) {
 			fDocument= fManager.createEmptyDocument(getLocation());
 			fStatus= x.getStatus();
@@ -363,25 +384,31 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 	protected void cacheEncodingState(IProgressMonitor monitor) {
 		fEncoding= fExplicitEncoding;
 		fHasBOM= false;
+		fIsCacheUpdated= true;
 
-		InputStream stream= getFileContents(fFile, monitor);
-		if (stream != null) {
+		InputStream stream= null;
+		try {
+			stream= getFileContents(fFileStore, monitor);
+			if (stream == null)
+				return;
+			
+			QualifiedName[] options= new QualifiedName[] { IContentDescription.CHARSET, IContentDescription.BYTE_ORDER_MARK };
+			IContentDescription description= Platform.getContentTypeManager().getDescriptionFor(stream, fFileStore.getName(), options);
+			if (description != null) {
+				fHasBOM= description.getProperty(IContentDescription.BYTE_ORDER_MARK) != null;
+				if (fEncoding == null)
+					fEncoding= description.getCharset();
+			}
+		} catch (CoreException e) {
+			// do nothing
+		} catch (IOException e) {
+			// do nothing
+		} finally {
 			try {
-				QualifiedName[] options= new QualifiedName[] { IContentDescription.CHARSET, IContentDescription.BYTE_ORDER_MARK };
-				IContentDescription description= Platform.getContentTypeManager().getDescriptionFor(stream, fFile.getName(), options);
-				if (description != null) {
-					fHasBOM= description.getProperty(IContentDescription.BYTE_ORDER_MARK) != null;
-					if (fEncoding == null)
-						fEncoding= description.getCharset();
-				}
-			} catch (IOException e) {
-				// do nothing
-			} finally {
-				try {
+				if (stream != null)
 					stream.close();
-				} catch (IOException ex) {
-					FileBuffersPlugin.getDefault().getLog().log(new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, FileBuffersMessages.JavaTextFileBuffer_error_closeStream, ex));
-				}
+			} catch (IOException ex) {
+				FileBuffersPlugin.getDefault().getLog().log(new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, FileBuffersMessages.JavaTextFileBuffer_error_closeStream, ex));
 			}
 		}
 	}
@@ -391,89 +418,118 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 	 */
 	protected void commitFileBufferContent(IProgressMonitor monitor, boolean overwrite) throws CoreException {
 		String encoding= computeEncoding();
-		try {
 
-			byte[] bytes= fDocument.get().getBytes(encoding);
+		Charset charset;
+		try {
+			charset= Charset.forName(encoding);
+		} catch (UnsupportedCharsetException ex) {
+			String message= NLSUtility.format(FileBuffersMessages.ResourceTextFileBuffer_error_unsupported_encoding_message_arg, encoding);
+			IStatus s= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, message, ex);
+			throw new CoreException(s);
+		} catch (IllegalCharsetNameException ex) {
+			String message= NLSUtility.format(FileBuffersMessages.ResourceTextFileBuffer_error_illegal_encoding_message_arg, encoding);
+			IStatus s= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, message, ex);
+			throw new CoreException(s);
+		}
+
+		CharsetEncoder encoder= charset.newEncoder();
+		encoder.onMalformedInput(CodingErrorAction.REPLACE);
+		encoder.onUnmappableCharacter(CodingErrorAction.REPORT);
+
+		byte[] bytes;
+		int bytesLength;
+
+		try {
+			ByteBuffer byteBuffer= encoder.encode(CharBuffer.wrap(fDocument.get()));
+			bytesLength= byteBuffer.limit();
+			if (byteBuffer.hasArray())
+				bytes= byteBuffer.array();
+			else {
+				bytes= new byte[bytesLength];
+				byteBuffer.get(bytes);
+			}
+		} catch (CharacterCodingException ex) {
+			Assert.isTrue(ex instanceof UnmappableCharacterException);
+			String message= NLSUtility.format(FileBuffersMessages.ResourceTextFileBuffer_error_charset_mapping_failed_message_arg, encoding);
+			IStatus s= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IFileBufferStatusCodes.CHARSET_MAPPING_FAILED, message, null);
+			throw new CoreException(s);
+		}
+
+		IFileInfo fileInfo= fFileStore == null ? null : fFileStore.fetchInfo();
+		if (fileInfo != null && fileInfo.exists()) {
+
+			if (!overwrite)
+				checkSynchronizationState();
+
+			InputStream stream= new ByteArrayInputStream(bytes, 0, bytesLength);
 
 			/*
 			 * XXX:
 			 * This is a workaround for a corresponding bug in Java readers and writer,
 			 * see: http://developer.java.sun.com/developer/bugParade/bugs/4508058.html
 			 */
-			if (fHasBOM && CHARSET_UTF_8.equals(encoding)) {
-				int bomLength= IContentDescription.BOM_UTF_8.length;
-				byte[] bytesWithBOM= new byte[bytes.length + bomLength];
-				System.arraycopy(IContentDescription.BOM_UTF_8, 0, bytesWithBOM, 0, bomLength);
-				System.arraycopy(bytes, 0, bytesWithBOM, bomLength, bytes.length);
-				bytes= bytesWithBOM;
+			if (fHasBOM && CHARSET_UTF_8.equals(encoding))
+				stream= new SequenceInputStream(new ByteArrayInputStream(IContentDescription.BOM_UTF_8), stream);
+
+
+			// here the file synchronizer should actually be removed and afterwards added again. However,
+			// we are already inside an operation, so the delta is sent AFTER we have added the listener
+			setFileContents(stream, overwrite, monitor);
+			// set synchronization stamp to know whether the file synchronizer must become active
+			fSynchronizationStamp= fFileStore.fetchInfo().getLastModified();
+
+			if (fAnnotationModel instanceof IPersistableAnnotationModel) {
+				IPersistableAnnotationModel persistableModel= (IPersistableAnnotationModel) fAnnotationModel;
+				persistableModel.commit(fDocument);
 			}
 
+		} else {
 
-			if (fFile != null && fFile.exists()) {
+			fFileStore= FileBuffers.getFileStoreAtLocation(getLocation());
+			fFileStore.getParent().mkdir(EFS.NONE, null);
+			OutputStream out= fFileStore.openOutputStream(EFS.NONE, null);
+			try {
+				/*
+				 * XXX:
+				 * This is a workaround for a corresponding bug in Java readers and writer,
+				 * see: http://developer.java.sun.com/developer/bugParade/bugs/4508058.html
+				 */
+				if (fHasBOM && CHARSET_UTF_8.equals(encoding))
+					out.write(IContentDescription.BOM_UTF_8);
 
-				if (!overwrite)
-					checkSynchronizationState();
-
-				InputStream stream= new ByteArrayInputStream(bytes);
-
-				// here the file synchronizer should actually be removed and afterwards added again. However,
-				// we are already inside an operation, so the delta is sent AFTER we have added the listener
-				setFileContents(stream, overwrite, monitor);
-				// set synchronization stamp to know whether the file synchronizer must become active
-				fSynchronizationStamp= fFile.lastModified();
-
-				if (fAnnotationModel instanceof IPersistableAnnotationModel) {
-					IPersistableAnnotationModel persistableModel= (IPersistableAnnotationModel) fAnnotationModel;
-					persistableModel.commit(fDocument);
-				}
-
-			} else {
-
-				fFile= FileBuffers.getSystemFileAtLocation(getLocation());
-				fFile.getParentFile().mkdirs();
+				out.write(bytes, 0, bytesLength);
+				out.flush();
+			} catch (IOException x) {
+				IStatus s= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, x.getLocalizedMessage(), x);
+				throw new CoreException(s);
+			} finally {
 				try {
-					FileOutputStream out= new FileOutputStream(fFile);
-					try {
-						out.write(bytes);
-						out.flush();
-					} catch (IOException x) {
-						IStatus s= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, x.getLocalizedMessage(), x);
-						throw new CoreException(s);
-					} finally {
-						try {
-							out.close();
-						} catch (IOException x) {
-						}
-					}
-
-					// set synchronization stamp to know whether the file synchronizer must become active
-					fSynchronizationStamp= fFile.lastModified();
-
-				} catch (FileNotFoundException x) {
-					IStatus s= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, x.getLocalizedMessage(), x);
-					throw new CoreException(s);
+					out.close();
+				} catch (IOException x) {
 				}
-
 			}
 
-		} catch (UnsupportedEncodingException x) {
-			String message= NLSUtility.format(FileBuffersMessages.ResourceTextFileBuffer_error_unsupported_encoding_message_arg, encoding);
-			IStatus s= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, message, x);
-			throw new CoreException(s);
+			// set synchronization stamp to know whether the file synchronizer must become active
+			fSynchronizationStamp= fFileStore.fetchInfo().getLastModified();
+
 		}
 	}
 
 	private String computeEncoding() {
+		// Make sure cache is up to date
+		if (!fIsCacheUpdated)
+			cacheEncodingState(null);
+		
 		// User-defined encoding has first priority
 		if (fExplicitEncoding != null)
 			return fExplicitEncoding;
 
-		if (fFile != null) {
+		if (fFileStore != null) {
 			// Probe content
 			Reader reader= new DocumentReader(fDocument);
 			try {
 				QualifiedName[] options= new QualifiedName[] { IContentDescription.CHARSET, IContentDescription.BYTE_ORDER_MARK };
-				IContentDescription description= Platform.getContentTypeManager().getDescriptionFor(reader, fFile.getName(), options);
+				IContentDescription description= Platform.getContentTypeManager().getDescriptionFor(reader, fFileStore.getName(), options);
 				if (description != null) {
 					String encoding= description.getCharset();
 					if (encoding != null)
@@ -504,10 +560,11 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 	 * @param document the document to be initialized
 	 * @param file the file which delivers the document content
 	 * @param encoding the character encoding for reading the given stream
+	 * @param hasBOM tell whether the given file has a BOM
 	 * @param monitor the progress monitor
 	 * @exception CoreException if the given stream can not be read
 	 */
-	private void setDocumentContent(IDocument document, File file, String encoding, IProgressMonitor monitor) throws CoreException {
+	private void setDocumentContent(IDocument document, IFileStore file, String encoding, boolean hasBOM, IProgressMonitor monitor) throws CoreException {
 		InputStream contentStream= getFileContents(file, monitor);
 		if (contentStream == null)
 			return;
@@ -523,7 +580,7 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 			 * This is a workaround for a corresponding bug in Java readers and writer,
 			 * see: http://developer.java.sun.com/developer/bugParade/bugs/4508058.html
 			 */
-			if (fHasBOM && CHARSET_UTF_8.equals(encoding)) {
+			if (hasBOM && CHARSET_UTF_8.equals(encoding)) {
 				int n= 0;
 				do {
 					int bytes= contentStream.read(new byte[IContentDescription.BOM_UTF_8.length]);
@@ -560,7 +617,7 @@ public class JavaTextFileBuffer extends JavaFileBuffer implements ITextFileBuffe
 	}
 
 	/**
-	 * Checks whether the given file is synchronized with the the local file system.
+	 * Checks whether the given file is synchronized with the local file system.
 	 * If the file has been changed, a <code>CoreException</code> is thrown.
 	 *
 	 * @exception CoreException if file has been changed on the file system
