@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,21 +10,25 @@
  *******************************************************************************/
 package org.eclipse.core.internal.filebuffers;
 
-import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
+import org.eclipse.core.filesystem.EFS;
+import org.eclipse.core.filesystem.IFileInfo;
+import org.eclipse.core.filesystem.IFileStore;
+
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.ISafeRunnable;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Platform;
+import org.eclipse.core.runtime.SafeRunner;
 import org.eclipse.core.runtime.Status;
 import org.eclipse.core.runtime.content.IContentDescription;
 import org.eclipse.core.runtime.content.IContentType;
@@ -54,7 +58,6 @@ import org.eclipse.core.filebuffers.ITextFileBuffer;
 import org.eclipse.core.filebuffers.ITextFileBufferManager;
 
 import org.eclipse.jface.text.Assert;
-import org.eclipse.jface.text.Document;
 import org.eclipse.jface.text.IDocument;
 import org.eclipse.jface.text.IDocumentExtension4;
 import org.eclipse.jface.text.source.IAnnotationModel;
@@ -88,22 +91,36 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	public void connect(IPath location, IProgressMonitor monitor) throws CoreException {
 		Assert.isNotNull(location);
 		location= FileBuffers.normalizeLocation(location);
-
-		AbstractFileBuffer fileBuffer= (AbstractFileBuffer) fFilesBuffers.get(location);
-		if (fileBuffer == null)  {
-
-			fileBuffer= createFileBuffer(location);
-			if (fileBuffer == null)
-				throw new CoreException(new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IFileBufferStatusCodes.CREATION_FAILED, FileBuffersMessages.FileBufferManager_error_canNotCreateFilebuffer, null));
-
-			fileBuffer.create(location, monitor);
+		
+		AbstractFileBuffer fileBuffer= null;
+		synchronized (fFilesBuffers) {
+			fileBuffer= (AbstractFileBuffer) fFilesBuffers.get(location);
+			if (fileBuffer != null)  {
+				fileBuffer.connect();
+				return;
+			}
+		}
+			
+		fileBuffer= createFileBuffer(location);
+		if (fileBuffer == null)
+			throw new CoreException(new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IFileBufferStatusCodes.CREATION_FAILED, FileBuffersMessages.FileBufferManager_error_canNotCreateFilebuffer, null));
+		
+		fileBuffer.create(location, monitor);
+			
+		synchronized (fFilesBuffers) {
+			AbstractFileBuffer oldFileBuffer= (AbstractFileBuffer) fFilesBuffers.get(location);
+			if (oldFileBuffer != null) {
+				fileBuffer.disconnect();
+				fileBuffer.dispose();
+				oldFileBuffer.connect();
+				return;
+			}
 			fileBuffer.connect();
 			fFilesBuffers.put(location, fileBuffer);
-			fireBufferCreated(fileBuffer);
-
-		} else {
-			fileBuffer.connect();
 		}
+		
+		// Do notification outside synchronized block
+		fireBufferCreated(fileBuffer);
 	}
 
 	/*
@@ -113,21 +130,36 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 		Assert.isNotNull(location);
 		location= FileBuffers.normalizeLocation(location);
 
-		AbstractFileBuffer fileBuffer= (AbstractFileBuffer) fFilesBuffers.get(location);
-		if (fileBuffer != null) {
+		AbstractFileBuffer fileBuffer;
+		synchronized (fFilesBuffers) {
+			fileBuffer= (AbstractFileBuffer)fFilesBuffers.get(location);
+			if (fileBuffer == null)
+				return;
+			
 			fileBuffer.disconnect();
-			if (fileBuffer.isDisconnected()) {
-				fFilesBuffers.remove(location);
-				fireBufferDisposed(fileBuffer);
-				fileBuffer.dispose();
-			}
+			if (!fileBuffer.isDisconnected())
+				return;
+			
+			fFilesBuffers.remove(location);
 		}
+		
+		// Do notification outside synchronized block
+		fireBufferDisposed(fileBuffer);
+		fileBuffer.dispose();
 	}
 
 	/*
 	 * @see org.eclipse.core.filebuffers.ITextFileBufferManager#isTextFileLocation(org.eclipse.core.runtime.IPath)
 	 */
 	public boolean isTextFileLocation(IPath location) {
+		return isTextFileLocation(location, false);
+	}
+
+	/*
+	 * @see org.eclipse.core.filebuffers.ITextFileBufferManager#isTextFileLocation(org.eclipse.core.runtime.IPath, boolean)
+	 * @since 3.2
+	 */
+	public boolean isTextFileLocation(IPath location, boolean strict) {
 		Assert.isNotNull(location);
 		location= FileBuffers.normalizeLocation(location);
 
@@ -156,21 +188,24 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 					return false;
 				}
 			}
-			return true;
+			return !strict;
 		}
 
-		File externalFile= FileBuffers.getSystemFileAtLocation(location);
+		IFileStore externalFile= FileBuffers.getFileStoreAtLocation(location);
 		if (externalFile != null) {
-			if (externalFile.exists()) {
-				FileInputStream is= null;
+			IFileInfo fileInfo= externalFile.fetchInfo();
+			if (fileInfo.exists()) {
+				InputStream is= null;
 				try {
-					is= new FileInputStream(externalFile);
+					is= externalFile.openInputStream(EFS.NONE, null);
 					IContentDescription description= manager.getDescriptionFor(is, externalFile.getName(), IContentDescription.ALL);
 					if (description != null) {
 						IContentType type= description.getContentType();
 						if (type != null)
 							return type.isKindOf(text);
 					}
+				} catch (CoreException ex) {
+					// ignore: API specification tells return true if content type can't be determined
 				} catch (IOException ex) {
 					// ignore: API specification tells return true if content type can't be determined
 				} finally {
@@ -183,7 +218,7 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 					}
 				}
 
-				return true;
+				return !strict;
 
 			}
 
@@ -194,7 +229,7 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 						return true;
 				return false;
 			}
-			return true;
+			return !strict;
 		}
 
 		return false;
@@ -205,15 +240,20 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	 */
 	public IFileBuffer getFileBuffer(IPath location) {
 		location= FileBuffers.normalizeLocation(location);
-		return (IFileBuffer) fFilesBuffers.get(location);
+		return internalGetFileBuffer(location);
+	}
+	
+	private AbstractFileBuffer internalGetFileBuffer(IPath location) {
+		synchronized (fFilesBuffers) {
+			return (AbstractFileBuffer)fFilesBuffers.get(location);
+		}
 	}
 
 	/*
 	 * @see org.eclipse.core.filebuffers.ITextFileBufferManager#getTextFileBuffer(org.eclipse.core.runtime.IPath)
 	 */
 	public ITextFileBuffer getTextFileBuffer(IPath location) {
-		location= FileBuffers.normalizeLocation(location);
-		return (ITextFileBuffer) fFilesBuffers.get(location);
+		return (ITextFileBuffer)getFileBuffer(location);
 	}
 
 	/*
@@ -227,30 +267,33 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	 * @see org.eclipse.core.filebuffers.ITextFileBufferManager#createEmptyDocument(org.eclipse.core.runtime.IPath)
 	 */
 	public IDocument createEmptyDocument(IPath location) {
-		Assert.isNotNull(location);
-		location= FileBuffers.normalizeLocation(location);
-
 		final IDocument[] runnableResult= new IDocument[1];
-		final IDocumentFactory factory= fRegistry.getDocumentFactory(location);
-		if (factory != null) {
-			ISafeRunnable runnable= new ISafeRunnable() {
-				public void run() throws Exception {
-					runnableResult[0]= factory.createDocument();
-				}
-				public void handleException(Throwable t) {
-					IStatus status= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, FileBuffersMessages.TextFileBufferManager_error_documentFactoryFailed, t);
-					FileBuffersPlugin.getDefault().getLog().log(status);
-					if (t instanceof VirtualMachineError)
-						throw (VirtualMachineError)t;
-				}
-			};
-			Platform.run(runnable);
+		if (location != null) {
+			location= FileBuffers.normalizeLocation(location);
+			final IDocumentFactory factory= fRegistry.getDocumentFactory(location);
+			if (factory != null) {
+				ISafeRunnable runnable= new ISafeRunnable() {
+					public void run() throws Exception {
+						runnableResult[0]= factory.createDocument();
+					}
+					public void handleException(Throwable t) {
+						IStatus status= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, FileBuffersMessages.TextFileBufferManager_error_documentFactoryFailed, t);
+						FileBuffersPlugin.getDefault().getLog().log(status);
+						if (t instanceof VirtualMachineError)
+							throw (VirtualMachineError)t;
+					}
+				};
+				SafeRunner.run(runnable);
+			}
 		}
 		final IDocument document;
 		if (runnableResult[0] != null)
 			document= runnableResult[0];
 		else
-			document= new Document();
+			document= new SynchronizableDocument();
+		
+		if (location == null)
+			return document;
 		
 		// Set the initial line delimiter
 		if (document instanceof IDocumentExtension4) {
@@ -266,6 +309,11 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 				ISafeRunnable runnable= new ISafeRunnable() {
 					public void run() throws Exception {
 						participant.setup(document);
+						if (document.getDocumentPartitioner() != null) {
+							String message= NLSUtility.format(FileBuffersMessages.TextFileBufferManager_warning_documentSetupInstallsDefaultPartitioner, participant.getClass());
+							IStatus status= new Status(IStatus.WARNING, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, message, null);
+							FileBuffersPlugin.getDefault().getLog().log(status);
+						}
 					}
 					public void handleException(Throwable t) {
 						IStatus status= new Status(IStatus.ERROR, FileBuffersPlugin.PLUGIN_ID, IStatus.OK, FileBuffersMessages.TextFileBufferManager_error_documentSetupFailed, t);
@@ -274,7 +322,7 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 							throw (VirtualMachineError)t;
 					}
 				};
-				Platform.run(runnable);
+				SafeRunner.run(runnable);
 			}
 		}
 
@@ -298,8 +346,10 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	 */
 	public void addFileBufferListener(IFileBufferListener listener) {
 		Assert.isNotNull(listener);
-		if (!fFileBufferListeners.contains(listener))
-			fFileBufferListeners.add(listener);
+		synchronized (fFileBufferListeners) {
+			if (!fFileBufferListeners.contains(listener))
+				fFileBufferListeners.add(listener);
+		}
 	}
 
 	/*
@@ -307,7 +357,9 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	 */
 	public void removeFileBufferListener(IFileBufferListener listener) {
 		Assert.isNotNull(listener);
-		fFileBufferListeners.remove(listener);
+		synchronized (fFileBufferListeners) {
+			fFileBufferListeners.remove(listener);
+		}
 	}
 
 	/*
@@ -324,7 +376,7 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 		Assert.isNotNull(location);
 		location= FileBuffers.normalizeLocation(location);
 
-		AbstractFileBuffer fileBuffer= (AbstractFileBuffer) fFilesBuffers.get(location);
+		AbstractFileBuffer fileBuffer= internalGetFileBuffer(location);
 		if (fileBuffer != null)
 			fileBuffer.requestSynchronizationContext();
 	}
@@ -336,7 +388,7 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 		Assert.isNotNull(location);
 		location= FileBuffers.normalizeLocation(location);
 
-		AbstractFileBuffer fileBuffer= (AbstractFileBuffer) fFilesBuffers.get(location);
+		AbstractFileBuffer fileBuffer= internalGetFileBuffer(location);
 		if (fileBuffer != null)
 			fileBuffer.releaseSynchronizationContext();
 	}
@@ -357,7 +409,7 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	}
 
 	private AbstractFileBuffer createFileBuffer(IPath location) {
-		if (isTextFileLocation(location))
+		if (isTextFileLocation(location, false))
 			return createTextFileBuffer(location);
 		return createBinaryFileBuffer(location);
 	}
@@ -373,11 +425,17 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 		return createTextFileBuffer(location);
 	}
 
+	private Iterator getFileBufferListenerIterator() {
+		synchronized (fFileBufferListeners) {
+			return new ArrayList(fFileBufferListeners).iterator();
+		}
+	}
+
 	protected void fireDirtyStateChanged(final IFileBuffer buffer, final boolean isDirty) {
-		Iterator e= new ArrayList(fFileBufferListeners).iterator();
+		Iterator e= getFileBufferListenerIterator();
 		while (e.hasNext()) {
 			final IFileBufferListener l= (IFileBufferListener) e.next();
-			Platform.run(new SafeNotifier() {
+			SafeRunner.run(new SafeNotifier() {
 				public void run() {
 					l.dirtyStateChanged(buffer, isDirty);
 				}
@@ -386,10 +444,10 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	}
 
 	protected void fireBufferContentAboutToBeReplaced(final IFileBuffer buffer) {
-		Iterator e= new ArrayList(fFileBufferListeners).iterator();
+		Iterator e= getFileBufferListenerIterator();
 		while (e.hasNext()) {
 			final IFileBufferListener l= (IFileBufferListener) e.next();
-			Platform.run(new SafeNotifier() {
+			SafeRunner.run(new SafeNotifier() {
 				public void run() {
 					l.bufferContentAboutToBeReplaced(buffer);
 				}
@@ -398,10 +456,10 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	}
 
 	protected void fireBufferContentReplaced(final IFileBuffer buffer) {
-		Iterator e= new ArrayList(fFileBufferListeners).iterator();
+		Iterator e= getFileBufferListenerIterator();
 		while (e.hasNext()) {
 			final IFileBufferListener l= (IFileBufferListener) e.next();
-			Platform.run(new SafeNotifier() {
+			SafeRunner.run(new SafeNotifier() {
 				public void run() {
 					l.bufferContentReplaced(buffer);
 				}
@@ -410,10 +468,10 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	}
 
 	protected void fireUnderlyingFileMoved(final IFileBuffer buffer, final IPath target) {
-		Iterator e= new ArrayList(fFileBufferListeners).iterator();
+		Iterator e= getFileBufferListenerIterator();
 		while (e.hasNext()) {
 			final IFileBufferListener l= (IFileBufferListener) e.next();
-			Platform.run(new SafeNotifier() {
+			SafeRunner.run(new SafeNotifier() {
 				public void run() {
 					l.underlyingFileMoved(buffer, target);
 				}
@@ -422,10 +480,10 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	}
 
 	protected void fireUnderlyingFileDeleted(final IFileBuffer buffer) {
-		Iterator e= new ArrayList(fFileBufferListeners).iterator();
+		Iterator e= getFileBufferListenerIterator();
 		while (e.hasNext()) {
 			final IFileBufferListener l= (IFileBufferListener) e.next();
-			Platform.run(new SafeNotifier() {
+			SafeRunner.run(new SafeNotifier() {
 				public void run() {
 					l.underlyingFileDeleted(buffer);
 				}
@@ -434,10 +492,10 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	}
 
 	protected void fireStateValidationChanged(final IFileBuffer buffer, final boolean isStateValidated) {
-		Iterator e= new ArrayList(fFileBufferListeners).iterator();
+		Iterator e= getFileBufferListenerIterator();
 		while (e.hasNext()) {
 			final IFileBufferListener l= (IFileBufferListener) e.next();
-			Platform.run(new SafeNotifier() {
+			SafeRunner.run(new SafeNotifier() {
 				public void run() {
 					l.stateValidationChanged(buffer, isStateValidated);
 				}
@@ -446,10 +504,10 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	}
 
 	protected void fireStateChanging(final IFileBuffer buffer) {
-		Iterator e= new ArrayList(fFileBufferListeners).iterator();
+		Iterator e= getFileBufferListenerIterator();
 		while (e.hasNext()) {
 			final IFileBufferListener l= (IFileBufferListener) e.next();
-			Platform.run(new SafeNotifier() {
+			SafeRunner.run(new SafeNotifier() {
 				public void run() {
 					l.stateChanging(buffer);
 				}
@@ -458,10 +516,10 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	}
 
 	protected void fireStateChangeFailed(final IFileBuffer buffer) {
-		Iterator e= new ArrayList(fFileBufferListeners).iterator();
+		Iterator e= getFileBufferListenerIterator();
 		while (e.hasNext()) {
 			final IFileBufferListener l= (IFileBufferListener) e.next();
-			Platform.run(new SafeNotifier() {
+			SafeRunner.run(new SafeNotifier() {
 				public void run() {
 					l.stateChangeFailed(buffer);
 				}
@@ -470,10 +528,10 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	}
 
 	protected void fireBufferCreated(final IFileBuffer buffer) {
-		Iterator e= new ArrayList(fFileBufferListeners).iterator();
+		Iterator e= getFileBufferListenerIterator();
 		while (e.hasNext()) {
 			final IFileBufferListener l= (IFileBufferListener) e.next();
-			Platform.run(new SafeNotifier() {
+			SafeRunner.run(new SafeNotifier() {
 				public void run() {
 					l.bufferCreated(buffer);
 				}
@@ -482,10 +540,10 @@ public class TextFileBufferManager implements ITextFileBufferManager {
 	}
 
 	protected void fireBufferDisposed(final IFileBuffer buffer) {
-		Iterator e= new ArrayList(fFileBufferListeners).iterator();
+		Iterator e= getFileBufferListenerIterator();
 		while (e.hasNext()) {
 			final IFileBufferListener l= (IFileBufferListener) e.next();
-			Platform.run(new SafeNotifier() {
+			SafeRunner.run(new SafeNotifier() {
 				public void run() {
 					l.bufferDisposed(buffer);
 				}
