@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,8 +10,10 @@
  *******************************************************************************/
 package org.eclipse.core.internal.resources;
 
-import java.util.HashMap;
-import org.eclipse.core.internal.utils.Assert;
+import java.net.URI;
+import java.util.*;
+import org.eclipse.core.filesystem.URIUtil;
+import org.eclipse.core.internal.utils.FileUtil;
 import org.eclipse.core.resources.*;
 import org.eclipse.core.runtime.*;
 
@@ -21,19 +23,18 @@ public class WorkspaceRoot extends Container implements IWorkspaceRoot {
 	 * that have been requested from this root.  This maps project
 	 * name strings to project handles.
 	 */
-	private HashMap projectTable = new HashMap(10);
+	private final Map projectTable = Collections.synchronizedMap(new HashMap(16));
+	
+	/**
+	 * Cache of the canonicalized platform location.
+	 */
+	private final IPath workspaceLocation;
 
 	protected WorkspaceRoot(IPath path, Workspace container) {
 		super(path, container);
 		Assert.isTrue(path.equals(Path.ROOT));
-	}
-
-	/**
-	 * @see IResource#delete(boolean, IProgressMonitor)
-	 */
-	public void delete(boolean force, IProgressMonitor monitor) throws CoreException {
-		int updateFlags = force ? IResource.FORCE : IResource.NONE;
-		delete(updateFlags, monitor);
+		workspaceLocation = FileUtil.canonicalPath(Platform.getLocation());
+		Assert.isNotNull(workspaceLocation);
 	}
 
 	/**
@@ -45,21 +46,50 @@ public class WorkspaceRoot extends Container implements IWorkspaceRoot {
 		delete(updateFlags, monitor);
 	}
 
+	/**
+	 * @see org.eclipse.core.internal.resources.Resource#delete(boolean, org.eclipse.core.runtime.IProgressMonitor)
+	 */
+	public void delete(boolean force, IProgressMonitor monitor) throws CoreException {
+		int updateFlags = force ? IResource.FORCE : IResource.NONE;
+		delete(updateFlags, monitor);
+	}
+
+	/**
+	 * @see org.eclipse.core.internal.resources.Resource#exists(int, boolean)
+	 */
 	public boolean exists(int flags, boolean checkType) {
 		return true;
 	}
 
 	/**
-	 * @see IWorkspaceRoot#findContainersForLocation(IPath)
+	 * @see org.eclipse.core.resources.IWorkspaceRoot#findContainersForLocation(org.eclipse.core.runtime.IPath)
 	 */
 	public IContainer[] findContainersForLocation(IPath location) {
+		return findContainersForLocationURI(URIUtil.toURI(location.makeAbsolute()));
+	}
+
+	/**
+	 * @see org.eclipse.core.resources.IWorkspaceRoot#findContainersForLocationURI(java.net.URI)
+	 */
+	public IContainer[] findContainersForLocationURI(URI location) {
+		if (!location.isAbsolute())
+			throw new IllegalArgumentException();
 		return (IContainer[]) getLocalManager().allResourcesFor(location, false);
 	}
 
 	/**
-	 * @see IWorkspaceRoot#findFilesForLocation(IPath)
+	 * @see org.eclipse.core.resources.IWorkspaceRoot#findFilesForLocation(org.eclipse.core.runtime.IPath)
 	 */
 	public IFile[] findFilesForLocation(IPath location) {
+		return findFilesForLocationURI(URIUtil.toURI(location.makeAbsolute()));
+	}
+
+	/**
+	 * @see org.eclipse.core.resources.IWorkspaceRoot#findFilesForLocationURI(java.net.URI)
+	 */
+	public IFile[] findFilesForLocationURI(URI location) {
+		if (!location.isAbsolute())
+			throw new IllegalArgumentException();
 		return (IFile[]) getLocalManager().allResourcesFor(location, true);
 	}
 
@@ -98,7 +128,7 @@ public class WorkspaceRoot extends Container implements IWorkspaceRoot {
 	 * @see IResource#getLocation()
 	 */
 	public IPath getLocation() {
-		return Platform.getLocation();
+		return workspaceLocation;
 	}
 
 	/**
@@ -175,16 +205,25 @@ public class WorkspaceRoot extends Container implements IWorkspaceRoot {
 			((Resource) children[i]).internalSetLocal(flag, depth);
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.core.internal.resources.Resource#isLinked(int)
+	 */
+	public boolean isLinked(int options) {
+		return false;//the root is never linked
+	}
+	
 	/**
 	 * @see IResource#isLocal(int)
+	 * @deprecated
 	 */
 	public boolean isLocal(int depth) {
-		// the flags parm is ignored for the workspace root so pass anything
+		// the flags parameter is ignored for the workspace root so pass anything
 		return isLocal(-1, depth);
 	}
 
 	/**
 	 * @see IResource#isLocal(int)
+	 * @deprecated
 	 */
 	public boolean isLocal(int flags, int depth) {
 		// don't check the flags....workspace root is always local
