@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -12,6 +12,7 @@ package org.eclipse.core.resources.team;
 
 import org.eclipse.core.resources.*;
 import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.core.runtime.OperationCanceledException;
 
 /**
  * Primary interface for hooking the implementation of
@@ -38,7 +39,7 @@ public interface IMoveDeleteHook {
 	 * In broad terms, a full re-implementation should delete the file in the
 	 * local file system and then call <code>tree.deletedFile</code> to complete
 	 * the updating of the workspace resource tree to reflect this fact. If
-	 * unsuccessful in deleting the file from the the local file system, it
+	 * unsuccessful in deleting the file from the local file system, it
 	 * should instead call <code>tree.failed</code> to report the reason for
 	 * the failure. In either case, it should return <code>true</code> to
 	 * indicate that the operation was attempted. The <code>FORCE</code> update
@@ -48,21 +49,18 @@ public interface IMoveDeleteHook {
 	 * The <code>KEEP_HISTORY</code> update flag needs to be honored as well;
 	 * use <code>tree.addToLocalHistory</code> to capture the contents of the
 	 * file before deleting it from the local file system.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * An extending implementation should perform whatever pre-processing it 
 	 * needs to do and then call <code>tree.standardDeleteFile</code> to 
 	 * explicitly invoke the standard file deletion behavior, which deletes
 	 * both the file from the local file system and updates the workspace
 	 * resource tree. It should return <code>true</code> to indicate that the
 	 * operation was attempted.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * Returning <code>false</code> is the easy way for the implementation to
 	 * say "pass". It is equivalent to calling
 	 * <code>tree.standardDeleteFile</code> and returning <code>true</code>.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * The implementation of this method runs "below" the resources API and is
 	 * therefore very restricted in what resource API method it can call. The
 	 * list of useable methods includes most resource operations that read but
@@ -83,13 +81,11 @@ public interface IMoveDeleteHook {
 	 * @return <code>false</code> if this method declined to assume 
 	 *   responsibility for this operation, and <code>true</code> if this method
 	 *   attempted to carry out the operation
+	 * @exception OperationCanceledException if the operation is canceled. 
+	 * Cancelation can occur even if no progress monitor is provided.
 	 * @see IResource#delete(int,IProgressMonitor)
 	 */
-	public boolean deleteFile(
-		IResourceTree tree,
-		IFile file,
-		int updateFlags,
-		IProgressMonitor monitor);
+	public boolean deleteFile(IResourceTree tree, IFile file, int updateFlags, IProgressMonitor monitor);
 
 	/**
 	 * Implements <code>IResource.delete(int,IProgressMonitor)</code> where the
@@ -110,21 +106,18 @@ public interface IMoveDeleteHook {
 	 * The <code>KEEP_HISTORY</code> update flag needs to be honored as well; 
 	 * use <code>tree.addToLocalHistory</code> to capture the contents of any
 	 * files being deleted.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * A partial re-implementation should perform whatever pre-processing it
 	 * needs to do and then call <code>tree.standardDeleteFolder</code> to
 	 * explicitly invoke the standard folder deletion behavior, which deletes
 	 * both the folder and its descendents from the local file system and 
 	 * updates the workspace resource tree. It should return <code>true</code>
 	 * to indicate that the operation was attempted.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * Returning <code>false</code> is the easy way for the implementation to
 	 * say "pass". It is equivalent to calling
 	 * <code>tree.standardDeleteFolder</code> and returning <code>true</code>.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * The implementation of this method runs "below" the resources API and is
 	 * therefore very restricted in what resource API method it can call. The
 	 * list of useable methods includes most resource operations that read but
@@ -145,13 +138,11 @@ public interface IMoveDeleteHook {
 	 * @return <code>false</code> if this method declined to assume 
 	 *   responsibility for this operation, and <code>true</code> if this
 	 *   method attempted to carry out the operation
+	 * @exception OperationCanceledException if the operation is canceled. 
+	 * Cancelation can occur even if no progress monitor is provided.
 	 * @see IResource#delete(int,IProgressMonitor)
 	 */
-	public boolean deleteFolder(
-		IResourceTree tree,
-		IFolder folder,
-		int updateFlags,
-		IProgressMonitor monitor);
+	public boolean deleteFolder(IResourceTree tree, IFolder folder, int updateFlags, IProgressMonitor monitor);
 
 	/**
 	 * Implements <code>IResource.delete(int,IProgressMonitor)</code> where the
@@ -174,19 +165,16 @@ public interface IMoveDeleteHook {
 	 * sync before attempting to delete it. 
 	 * Note that local history is not maintained when a project is deleted,
 	 * regardless of the setting of the <code>KEEP_HISTORY</code> update flag. 
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * A partial re-implementation should perform whatever pre-processing it needs
 	 * to do and then call <code>tree.standardDeleteProject</code> to explicitly
 	 * invoke the standard project deletion behavior. It should return <code>true</code>
 	 * to indicate that the operation was attempted.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * Returning <code>false</code> is the easy way for the implementation to
 	 * say "pass". It is equivalent to calling
 	 * <code>tree.standardDeleteProject</code> and returning <code>true</code>.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * The implementation of this method runs "below" the resources API and is
 	 * therefore very restricted in what resource API method it can call. The
 	 * list of useable methods includes most resource operations that read but
@@ -207,13 +195,11 @@ public interface IMoveDeleteHook {
 	 * @return <code>false</code> if this method declined to assume 
 	 *   responsibility for this operation, and <code>true</code> if this 
 	 *   method attempted to carry out the operation
+	 * @exception OperationCanceledException if the operation is canceled. 
+	 * Cancelation can occur even if no progress monitor is provided.
 	 * @see IResource#delete(int,IProgressMonitor)
 	 */
-	public boolean deleteProject(
-		IResourceTree tree,
-		IProject project,
-		int updateFlags,
-		IProgressMonitor monitor);
+	public boolean deleteProject(IResourceTree tree, IProject project, int updateFlags, IProgressMonitor monitor);
 
 	/**
 	 * Implements <code>IResource.move(IPath,int,IProgressMonitor)</code> where
@@ -226,7 +212,7 @@ public interface IMoveDeleteHook {
 	 * accessible. In broad terms, a full re-implementation should move the file 
 	 * in the local file system and then call <code>tree.moveFile</code> to
 	 * complete the updating of the workspace resource tree to reflect this
-	 * fact. If unsuccessful in moving the file in the the local file system,
+	 * fact. If unsuccessful in moving the file in the local file system,
 	 * it should instead call <code>tree.failed</code> to report the reason for
 	 * the failure. In either case, it should return <code>true</code> to
 	 * indicate that the operation was attempted.
@@ -237,20 +223,17 @@ public interface IMoveDeleteHook {
 	 * The <code>KEEP_HISTORY</code> update flag needs to be honored as well; use
 	 * <code>tree.addToLocalHistory</code> to capture the contents of the file
 	 * (naturally, this must be before moving the file from the local file system).
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * An extending implementation should perform whatever pre-processing it needs
 	 * to do and then call <code>tree.standardMoveFile</code> to explicitly
 	 * invoke the standard file moving behavior, which moves both the file in the
 	 * local file system and updates the workspace resource tree. It should return
 	 * <code>true</code> to indicate that the operation was attempted.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * Returning <code>false</code> is the easy way for the implementation to
 	 * say "pass". It is equivalent to calling
 	 * <code>tree.standardMoveFile</code> and returning <code>true</code>.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * The implementation of this method runs "below" the resources API and is
 	 * therefore very restricted in what resource API method it can call. The
 	 * list of useable methods includes most resource operations that read but
@@ -274,14 +257,11 @@ public interface IMoveDeleteHook {
 	 * @return <code>false</code> if this method declined to assume 
 	 *   responsibility for this operation, and <code>true</code> if this
 	 *   method attempted to carry out the operation
-	 * @see IResource#move(IPath,int,IProgressMonitor)
+	 * @exception OperationCanceledException if the operation is canceled. 
+	 * Cancelation can occur even if no progress monitor is provided.
+	 * @see IResource#move(org.eclipse.core.runtime.IPath,int,IProgressMonitor)
 	 */
-	public boolean moveFile(
-		IResourceTree tree,
-		IFile source,
-		IFile destination,
-		int updateFlags,
-		IProgressMonitor monitor);
+	public boolean moveFile(IResourceTree tree, IFile source, IFile destination, int updateFlags, IProgressMonitor monitor);
 
 	/**
 	 * Implements <code>IResource.move(IPath,int,IProgressMonitor)</code> where
@@ -295,7 +275,7 @@ public interface IMoveDeleteHook {
 	 * directory tree in the local file system and then call
 	 * <code>tree.movedFolder</code> to complete the updating of the workspace
 	 * resource tree to reflect this fact. If unsuccessful in moving the 
-	 * directory or any of its descendents in the the local file system,
+	 * directory or any of its descendents in the local file system,
 	 * call <code>tree.failed</code> to report each reason for failure.
 	 * In either case, return <code>true</code> to indicate that the operation
 	 * was attempted.
@@ -306,20 +286,17 @@ public interface IMoveDeleteHook {
 	 * The <code>KEEP_HISTORY</code> update flag needs to be honored as well; use
 	 * <code>tree.addToLocalHistory</code> to capture the contents of any files being
 	 * moved.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * A partial re-implementation should perform whatever pre-processing it needs
 	 * to do and then call <code>tree.standardMoveFolder</code> to explicitly
 	 * invoke the standard folder move behavior, which move both the folder
 	 * and its descendents in the local file system and updates the workspace resource
 	 * tree. Return <code>true</code> to indicate that the operation was attempted.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * Returning <code>false</code> is the easy way for the implementation to
 	 * say "pass". It is equivalent to calling 
 	 * <code>tree.standardDeleteFolder</code> and returning <code>true</code>.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * The implementation of this method runs "below" the resources API and is
 	 * therefore very restricted in what resource API method it can call. The
 	 * list of useable methods includes most resource operations that read but
@@ -343,14 +320,11 @@ public interface IMoveDeleteHook {
 	 * @return <code>false</code> if this method declined to assume 
 	 *   responsibility for this operation, and <code>true</code> if this 
 	 *   method attempted to carry out the operation
-	 * @see IResource#move(IPath,int,IProgressMonitor)
+	 * @exception OperationCanceledException if the operation is canceled. 
+	 * Cancelation can occur even if no progress monitor is provided.
+	 * @see IResource#move(org.eclipse.core.runtime.IPath,int,IProgressMonitor)
 	 */
-	public boolean moveFolder(
-		IResourceTree tree,
-		IFolder source,
-		IFolder destination,
-		int updateFlags,
-		IProgressMonitor monitor);
+	public boolean moveFolder(IResourceTree tree, IFolder source, IFolder destination, int updateFlags, IProgressMonitor monitor);
 
 	/**
 	 * Implements <code>IResource.move(IPath,int,IProgressMonitor)</code> and
@@ -366,13 +340,11 @@ public interface IMoveDeleteHook {
 	 * project is being relocated but not renamed. When the project is being
 	 * renamed, the destination project is guaranteed not to exist in the
 	 * workspace resource tree.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * Returning <code>false</code> is the easy way for the implementation to
 	 * say "pass". It is equivalent to calling 
 	 * <code>tree.standardMoveProject</code> and returning <code>true</code>.
-	 * </p>
-	 * <p>
+	 * </p><p>
 	 * The implementation of this method runs "below" the resources API and is
 	 * therefore very restricted in what resource API method it can call. The
 	 * list of useable methods includes most resource operations that read but
@@ -402,13 +374,10 @@ public interface IMoveDeleteHook {
 	 * @return <code>false</code> if this method declined to assume 
 	 *   responsibility for this operation, and <code>true</code> if this method
 	 *   attempted to carry out the operation
-	 * @see IResource#move(IPath,int,IProgressMonitor)
+	 * @exception OperationCanceledException if the operation is canceled. 
+	 * Cancelation can occur even if no progress monitor is provided.
+	 * @see IResource#move(org.eclipse.core.runtime.IPath,int,IProgressMonitor)
 	 * @see IResource#move(IProjectDescription,int,IProgressMonitor)
 	 */
-	public boolean moveProject(
-		IResourceTree tree,
-		IProject source,
-		IProjectDescription description,
-		int updateFlags,
-		IProgressMonitor monitor);
+	public boolean moveProject(IResourceTree tree, IProject source, IProjectDescription description, int updateFlags, IProgressMonitor monitor);
 }
