@@ -1,66 +1,69 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.debug.ui;
 
 
-import java.lang.reflect.InvocationTargetException;
-
+import org.eclipse.core.resources.IResource;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.IConfigurationElement;
 import org.eclipse.core.runtime.IExtension;
 import org.eclipse.core.runtime.IExtensionPoint;
-import org.eclipse.core.runtime.IPluginDescriptor;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.Platform;
 import org.eclipse.debug.core.DebugPlugin;
 import org.eclipse.debug.core.ILaunch;
 import org.eclipse.debug.core.ILaunchConfiguration;
-import org.eclipse.debug.core.ILaunchManager;
-import org.eclipse.debug.core.IStatusHandler;
+import org.eclipse.debug.core.ILaunchConfigurationType;
 import org.eclipse.debug.core.model.IDebugElement;
 import org.eclipse.debug.core.model.IDebugTarget;
 import org.eclipse.debug.core.model.IProcess;
+import org.eclipse.debug.core.model.ISourceLocator;
 import org.eclipse.debug.internal.ui.DebugPluginImages;
-import org.eclipse.debug.internal.ui.DebugUIMessages;
 import org.eclipse.debug.internal.ui.DebugUIPlugin;
 import org.eclipse.debug.internal.ui.DefaultLabelProvider;
 import org.eclipse.debug.internal.ui.DelegatingModelPresentation;
 import org.eclipse.debug.internal.ui.LazyModelPresentation;
+import org.eclipse.debug.internal.ui.contexts.DebugContextManager;
 import org.eclipse.debug.internal.ui.launchConfigurations.LaunchConfigurationDialog;
+import org.eclipse.debug.internal.ui.launchConfigurations.LaunchConfigurationManager;
 import org.eclipse.debug.internal.ui.launchConfigurations.LaunchConfigurationPropertiesDialog;
 import org.eclipse.debug.internal.ui.launchConfigurations.LaunchConfigurationsDialog;
 import org.eclipse.debug.internal.ui.launchConfigurations.LaunchGroupExtension;
-import org.eclipse.jface.dialogs.ProgressMonitorDialog;
-import org.eclipse.jface.operation.IRunnableWithProgress;
+import org.eclipse.debug.internal.ui.memory.MemoryRenderingManager;
+import org.eclipse.debug.internal.ui.sourcelookup.SourceLookupFacility;
+import org.eclipse.debug.internal.ui.sourcelookup.SourceLookupUIUtils;
+import org.eclipse.debug.internal.ui.stringsubstitution.SelectedResourceManager;
+import org.eclipse.debug.ui.memory.IMemoryRenderingManager;
+import org.eclipse.debug.ui.sourcelookup.ISourceContainerBrowser;
+import org.eclipse.debug.ui.sourcelookup.ISourceLookupResult;
 import org.eclipse.jface.preference.IPreferenceStore;
 import org.eclipse.jface.resource.ImageDescriptor;
 import org.eclipse.jface.viewers.ISelection;
 import org.eclipse.jface.viewers.IStructuredSelection;
-import org.eclipse.jface.viewers.StructuredSelection;
-import org.eclipse.jface.viewers.Viewer;
 import org.eclipse.jface.window.Window;
 import org.eclipse.swt.custom.BusyIndicator;
+import org.eclipse.swt.graphics.Color;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.widgets.Shell;
 import org.eclipse.ui.IWorkbenchPage;
-import org.eclipse.ui.IWorkbenchPart;
 import org.eclipse.ui.IWorkbenchWindow;
-import org.eclipse.ui.PlatformUI;
+import org.eclipse.ui.console.IConsole;
 
 /**
  * This class provides utilities for clients of the debug UI.
  * <p>
  * Images retrieved from this facility should not be disposed.
- * The images will be disposed when this plugin is shutdown.
+ * The images will be disposed when this plug-in is shutdown.
  * </p>
  * <p>
  * This class is not intended to be subclassed or instantiated.
@@ -118,7 +121,7 @@ public class DebugUITools {
 	}
 	
 	/**
-	 * Returns the preference store for the debug UI plugin.
+	 * Returns the preference store for the debug UI plug-in.
 	 *
 	 * @return preference store
 	 */
@@ -155,8 +158,7 @@ public class DebugUITools {
 	 * @since 2.0
 	 */
 	public static IDebugModelPresentation newDebugModelPresentation(String identifier) {
-		IPluginDescriptor descriptor= DebugUIPlugin.getDefault().getDescriptor();
-		IExtensionPoint point= descriptor.getExtensionPoint(IDebugUIConstants.ID_DEBUG_MODEL_PRESENTATION);
+		IExtensionPoint point= Platform.getExtensionRegistry().getExtensionPoint(DebugUIPlugin.getUniqueIdentifier(), IDebugUIConstants.ID_DEBUG_MODEL_PRESENTATION);
 		if (point != null) {
 			IExtension[] extensions= point.getExtensions();
 			for (int i= 0; i < extensions.length; i++) {
@@ -178,41 +180,41 @@ public class DebugUITools {
 	 * Returns the currently selected element in the 
 	 * debug view of the current workbench page,
 	 * or <code>null</code> if there is no current
-	 * debug context, or if not called from the UI
-	 * thread.
-	 * 
+	 * debug context.
+	 * <p>
+	 * This method used to return <code>null</code> when called from a non-UI thread,
+	 * but since 3.1, this methods also works when called from a non-UI thread.
+	 * </p>
 	 * @return the currently selected debug context, or <code>null</code>
 	 * @since 2.0
 	 */
 	public static IAdaptable getDebugContext() {
-		IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow() ;
-		if (window != null) {
-			IWorkbenchPage page = window.getActivePage();
-			if (page != null) {
-				IWorkbenchPart part = page.findView(IDebugUIConstants.ID_DEBUG_VIEW);
-				if (part != null) {
-					IDebugView view = (IDebugView)part.getAdapter(IDebugView.class);
-					if (view != null) {
-						Viewer viewer = view.getViewer();
-						if (viewer != null) {
-							ISelection s = viewer.getSelection();
-							if (s != null) {
-								if (s instanceof IStructuredSelection) {
-									IStructuredSelection ss = (IStructuredSelection)s;
-									if (ss.size() == 1) {
-										Object element = ss.getFirstElement();
-										if (element instanceof IAdaptable) {
-											return (IAdaptable)element;
-										}
-									}
-								}
-							}
-						}
+	    IWorkbenchWindow activeWindow = SelectedResourceManager.getDefault().getActiveWindow();
+	    if (activeWindow != null) {
+	    	ISelection activeContext = DebugContextManager.getDefault().getActiveContext(activeWindow);
+	    	if (activeContext instanceof IStructuredSelection) {
+	    		IStructuredSelection selection = (IStructuredSelection) activeContext;
+	    		if (!selection.isEmpty()) {
+	    			Object firstElement = selection.getFirstElement();
+	    			if (firstElement instanceof IAdaptable) {
+						return (IAdaptable) firstElement;
 					}
-				}
-			}
-		}
-		return null;
+	    		}
+	    	}
+	    }
+	    return null;
+	}
+
+	/**
+	 * Returns the currently selected resource in the active workbench window,
+	 * or <code>null</code> if none. If an editor is active, the resource adapter
+	 * associated with the editor is returned, if any.
+	 * 
+	 * @return selected resource or <code>null</code>
+	 * @since 3.0
+	 */
+	public static IResource getSelectedResource() {
+		return SelectedResourceManager.getDefault().getSelectedResource();
 	}
 			
 	/**
@@ -232,13 +234,16 @@ public class DebugUITools {
 				context = launches[launches.length - 1];
 			}
 		}
+        
 		if (context instanceof IDebugElement) {
 			return ((IDebugElement)context).getDebugTarget().getProcess();
 		}
-		if (context instanceof IProcess) {
+		
+        if (context instanceof IProcess) {
 			return (IProcess)context;
 		}
-		if (context instanceof ILaunch) {
+		
+        if (context instanceof ILaunch) {
 			ILaunch launch= (ILaunch)context;
 			IDebugTarget target= launch.getDebugTarget();
 			if (target != null) {
@@ -252,6 +257,11 @@ public class DebugUITools {
 				return ps[ps.length - 1];
 			}
 		}
+        
+        if (context != null) {
+            return (IProcess) context.getAdapter(IProcess.class);
+        }
+        
 		return null;
 	}
 
@@ -277,15 +287,19 @@ public class DebugUITools {
 	 * @return the return code from opening the launch configuration dialog -
 	 *  one  of <code>Window.OK</code> or <code>Window.CANCEL</code>
 	 * @since 2.0
+	 * @deprecated use openLaunchConfigurationDialogOnGroup(Shell, IStructuredSelection, String)
+	 *  to specify the launch group that the dialog should be opened on. This method will open
+	 *  on the launch group with the specified mode and a <code>null</code> category 
 	 */
 	public static int openLaunchConfigurationDialog(Shell shell, IStructuredSelection selection, String mode) {
-		String groupId = null;
-		if (mode.equals(ILaunchManager.DEBUG_MODE)) {
-			groupId = IDebugUIConstants.ID_DEBUG_LAUNCH_GROUP;
-		} else {
-			groupId = IDebugUIConstants.ID_RUN_LAUNCH_GROUP;
+		ILaunchGroup[] groups = getLaunchGroups();
+		for (int i = 0; i < groups.length; i++) {
+			ILaunchGroup group = groups[i];
+			if (group.getMode().equals(mode) && group.getCategory() == null) {
+				return openLaunchConfigurationDialogOnGroup(shell, selection, group.getIdentifier());
+			}
 		}
-		return openLaunchConfigurationDialogOnGroup(shell, selection, groupId);
+		return Window.CANCEL;
 	}
 	
 	/**
@@ -380,59 +394,77 @@ public class DebugUITools {
 	 *
 	 * @param shell the parent shell for the launch configuration dialog
 	 * @param configuration the configuration to display
-	 * @param group identifier of the launch group the launch configuration
+	 * @param groupIdentifier group identifier of the launch group the launch configuration
 	 * belongs to
 	 * @return the return code from opening the launch configuration dialog -
 	 *  one  of <code>Window.OK</code> or <code>Window.CANCEL</code>
 	 * @since 2.1
 	 */
 	public static int openLaunchConfigurationPropertiesDialog(Shell shell, ILaunchConfiguration configuration, String groupIdentifier) {
-		LaunchGroupExtension group = DebugUIPlugin.getDefault().getLaunchConfigurationManager().getLaunchGroup(groupIdentifier);
-		if (group != null) {
-			LaunchConfigurationPropertiesDialog dialog = new LaunchConfigurationPropertiesDialog(shell, configuration, group);
-			return dialog.open();
-		} else {
-			return Window.CANCEL;
-		}
+		return openLaunchConfigurationPropertiesDialog(shell, configuration, groupIdentifier, null);
 	}
 	
 	/**
-	 * Open the launch configuration dialog on the specified launch
-	 * configuration. The dialog displays the tabs for a single configuration
-	 * only (a tree of launch configuration is not displayed), and provides a
-	 * launch (run or debug) button.
-	 * <p>
-	 * If a status is specified, a status handler is consulted to handle the
-	 * status. The status handler is passed the instance of the launch
-	 * configuration dialog that is opened. This gives the status handler an
-	 * opportunity to perform error handling/initialization as required.
-	 * </p>
+	 * Open the launch configuration properties dialog on the specified launch
+	 * configuration.
+	 *
 	 * @param shell the parent shell for the launch configuration dialog
 	 * @param configuration the configuration to display
-	 * @param group identifier of the launch group the launch configuration
+	 * @param groupIdentifier group identifier of the launch group the launch configuration
 	 * belongs to
-	 * @param status the status to display, or <code>null</code> if none 
+	 * @param status the status to display, or <code>null</code> if none
 	 * @return the return code from opening the launch configuration dialog -
 	 *  one  of <code>Window.OK</code> or <code>Window.CANCEL</code>
-	 * @since 2.1
+	 * @since 3.0
 	 */
-	public static int openLaunchConfigurationDialog(Shell shell, ILaunchConfiguration configuration, String groupIdentifier, IStatus status) {
+	public static int openLaunchConfigurationPropertiesDialog(Shell shell, ILaunchConfiguration configuration, String groupIdentifier, IStatus status) {
 		LaunchGroupExtension group = DebugUIPlugin.getDefault().getLaunchConfigurationManager().getLaunchGroup(groupIdentifier);
 		if (group != null) {
-			LaunchConfigurationDialog dialog = new LaunchConfigurationDialog(shell, configuration, group);
+			LaunchConfigurationPropertiesDialog dialog = new LaunchConfigurationPropertiesDialog(shell, configuration, group);
 			dialog.setInitialStatus(status);
 			return dialog.open();
-		} else {
-			return Window.CANCEL;
-		}
+		} 
+		
+		return Window.CANCEL;
 	}
+	
+	/**
+     * Open the launch configuration dialog on the specified launch
+     * configuration. The dialog displays the tabs for a single configuration
+     * only (a tree of launch configuration is not displayed), and provides a
+     * launch (run or debug) button.
+     * <p>
+     * If a status is specified, a status handler is consulted to handle the
+     * status. The status handler is passed the instance of the launch
+     * configuration dialog that is opened. This gives the status handler an
+     * opportunity to perform error handling/initialization as required.
+     * </p>
+     * @param shell the parent shell for the launch configuration dialog
+     * @param configuration the configuration to display
+     * @param groupIdentifier group identifier of the launch group the launch configuration
+     * belongs to
+     * @param status the status to display, or <code>null</code> if none 
+     * @return the return code from opening the launch configuration dialog -
+     *  one  of <code>Window.OK</code> or <code>Window.CANCEL</code>
+     * @since 2.1
+     */
+    public static int openLaunchConfigurationDialog(Shell shell, ILaunchConfiguration configuration, String groupIdentifier, IStatus status) {
+    	LaunchGroupExtension group = DebugUIPlugin.getDefault().getLaunchConfigurationManager().getLaunchGroup(groupIdentifier);
+    	if (group != null) {
+    		LaunchConfigurationDialog dialog = new LaunchConfigurationDialog(shell, configuration, group);
+    		dialog.setInitialStatus(status);
+    		return dialog.open();
+    	} 
+    		
+    	return Window.CANCEL;
+    }
 			
 	/**
 	 * Saves all dirty editors and builds the workspace according to current
 	 * preference settings, and returns whether a launch should proceed.
 	 * <p>
 	 * The following preferences effect whether dirty editors are saved,
-	 * and/or if the user is prompted to save dirty edtiors:<ul>
+	 * and/or if the user is prompted to save dirty editors:<ul>
 	 * <li>PREF_NEVER_SAVE_DIRTY_EDITORS_BEFORE_LAUNCH</li>
 	 * <li>PREF_PROMPT_SAVE_DIRTY_EDITORS_BEFORE_LAUNCH</li>
 	 * <li>PREF_AUTOSAVE_DIRTY_EDITORS_BEFORE_LAUNCH</li>
@@ -445,6 +477,8 @@ public class DebugUITools {
 	 * 
 	 * @return whether a launch should proceed
 	 * @since 2.0
+	 * @deprecated Saving has been moved to the launch delegate <code>LaunchConfigurationDelegate</code> to allow for scoped saving
+	 * of resources that are only involved in the current launch, no longer the entire workspace
 	 */
 	public static boolean saveAndBuildBeforeLaunch() {
 		return DebugUIPlugin.saveAndBuild();
@@ -455,7 +489,7 @@ public class DebugUITools {
 	 * preference settings, and returns whether a launch should proceed.
 	 * <p>
 	 * The following preferences effect whether dirty editors are saved,
-	 * and/or if the user is prompted to save dirty edtiors:<ul>
+	 * and/or if the user is prompted to save dirty editors:<ul>
 	 * <li>PREF_NEVER_SAVE_DIRTY_EDITORS_BEFORE_LAUNCH</li>
 	 * <li>PREF_PROMPT_SAVE_DIRTY_EDITORS_BEFORE_LAUNCH</li>
 	 * <li>PREF_AUTOSAVE_DIRTY_EDITORS_BEFORE_LAUNCH</li>
@@ -464,6 +498,8 @@ public class DebugUITools {
 	 * 
 	 * @return whether a launch should proceed
 	 * @since 2.1
+	 * @deprecated Saving has been moved to the launch delegate <code>LaunchConfigurationDelegate</code> to allow for scoped saving
+	 * of resources that are only involved in the current launch, no longer the entire workspace
 	 */
 	public static boolean saveBeforeLaunch() {
 		return DebugUIPlugin.preLaunchSave();
@@ -471,51 +507,28 @@ public class DebugUITools {
 	
 	/**
 	 * Saves and builds the workspace according to current preference settings, and
-	 * launches the given launch configuration in the specified mode with a
-	 * progress dialog. Reports any exceptions that occurr in an error dilaog.
+	 * launches the given launch configuration in the specified mode.
 	 * 
 	 * @param configuration the configuration to launch
 	 * @param mode launch mode - run or debug
 	 * @since 2.1
 	 */
 	public static void launch(final ILaunchConfiguration configuration, final String mode) {
-		if (DebugUIPlugin.preLaunchSave()) {
-			ProgressMonitorDialog dialog = new ProgressMonitorDialog(DebugUIPlugin.getShell());
-			IRunnableWithProgress runnable = new IRunnableWithProgress() {
-				public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
-					try {
-						buildAndLaunch(configuration, mode, monitor);
-					} catch (CoreException e) {
-						throw new InvocationTargetException(e);
-					}
-				}		
-			};
-			try {
-				dialog.run(true, true, runnable);
-			} catch (InvocationTargetException e) {
-				Throwable targetException = e.getTargetException();
-				Throwable t = e;
-				if (targetException instanceof CoreException) {
-					t = targetException;
-				}
-				if (t instanceof CoreException) {
-					CoreException ce = (CoreException)t;
-					IStatusHandler handler = DebugPlugin.getDefault().getStatusHandler(ce.getStatus());
-					if (handler != null) {
-						LaunchGroupExtension group = DebugUIPlugin.getDefault().getLaunchConfigurationManager().getLaunchGroup(configuration, mode);
-						if (group != null) {
-							openLaunchConfigurationDialogOnGroup(DebugUIPlugin.getShell(), new StructuredSelection(configuration), group.getIdentifier(), ce.getStatus());
-							return;
-						}
-					}
-				}
-				DebugUIPlugin.errorDialog(DebugUIPlugin.getShell(), DebugUIMessages.getString("DebugUITools.Error_1"), DebugUIMessages.getString("DebugUITools.Exception_occurred_during_launch_2"), t); //$NON-NLS-1$ //$NON-NLS-2$
-			} catch (InterruptedException e) {
-				// cancelled
-			}
+		boolean launchInBackground= true;
+		try {
+			launchInBackground= configuration.getAttribute(IDebugUIConstants.ATTR_LAUNCH_IN_BACKGROUND, true);
+		} catch (CoreException e) {
+			DebugUIPlugin.log(e);
+		}
+		if (launchInBackground) {
+			DebugUIPlugin.launchInBackground(configuration, mode);
+		} else {
+			DebugUIPlugin.launchInForeground(configuration, mode);
 		}
 	}
 	
+
+	
 	/**
 	 * Builds the workspace according to current preference settings, and launches
 	 * the given configuration in the specified mode, returning the resulting launch
@@ -538,4 +551,207 @@ public class DebugUITools {
 		return DebugUIPlugin.buildAndLaunch(configuration, mode, monitor);
 	}
 	
+	/**
+	 * Returns the perspective to switch to when a configuration of the given type
+	 * is launched in the given mode, or <code>null</code> if no switch should take
+	 * place.
+	 * 
+	 * @param type launch configuration type
+	 * @param mode launch mode identifier
+	 * @return perspective identifier or <code>null</code>
+	 * @since 3.0
+	 */
+	public static String getLaunchPerspective(ILaunchConfigurationType type, String mode) {
+		return DebugUIPlugin.getDefault().getPerspectiveManager().getLaunchPerspective(type, mode);
+	}
+	
+	/**
+	 * Sets the perspective to switch to when a configuration of the given type
+	 * is launched in the given mode. <code>PERSPECTIVE_NONE</code> indicates no
+	 * perspective switch should take place. <code>PERSPECTIVE_DEFAULT</code> indicates
+	 * a default perspective switch should take place, as defined by the associated
+	 * launch tab group extension.
+	 * 
+	 * @param type launch configuration type
+	 * @param mode launch mode identifier
+	 * @param perspective identifier, <code>PERSPECTIVE_NONE</code>, or
+	 *   <code>PERSPECTIVE_DEFAULT</code>
+	 * @since 3.0
+	 */
+	public static void setLaunchPerspective(ILaunchConfigurationType type, String mode, String perspective) {
+		DebugUIPlugin.getDefault().getPerspectiveManager().setLaunchPerspective(type, mode, perspective);
+	}	
+		
+	/**
+	 * Returns whether the given launch configuration is private. Generally,
+	 * private launch configurations should not be displayed to the user. The
+	 * private status of a launch configuration is determined by the
+	 * <code>IDebugUIConstants.ATTR_PRIVATE</code> attribute.
+	 * 
+	 * @param configuration launch configuration
+	 * @return whether the given launch configuration is private
+	 * @since 3.0
+	 */
+	public static boolean isPrivate(ILaunchConfiguration configuration) {
+		return !LaunchConfigurationManager.isVisible(configuration);
+	}
+
+	/**
+	 * Sets whether step filters should be applied to step commands. This
+	 * setting is a global option applied to all registered debug targets. 
+	 * 
+	 * @param useStepFilters whether step filters should be applied to step
+	 *  commands
+	 * @since 3.0
+	 * @see org.eclipse.debug.core.model.IStepFilters
+	 */
+	public static void setUseStepFilters(boolean useStepFilters) {
+		DebugUIPlugin.getDefault().getStepFilterManager().setUseStepFilters(useStepFilters);
+	}
+		
+	/**
+	 * Returns whether step filters are applied to step commands.
+	 * 
+	 * @return whether step filters are applied to step commands
+	 * @since 3.0
+	 * @see org.eclipse.debug.core.model.IStepFilters
+	 */
+	public static boolean isUseStepFilters() {
+		return DebugUIPlugin.getDefault().getStepFilterManager().isUseStepFilters();
+	}
+		
+	/**
+	 * Returns the console associated with the given process, or 
+	 * <code>null</code> if none.
+	 * 
+	 * @param process a process
+	 * @return console associated with the given process, or 
+	 * <code>null</code> if none
+	 * @since 3.0
+	 */
+	public static IConsole getConsole(IProcess process) {
+		return DebugUIPlugin.getDefault().getProcessConsoleManager().getConsole(process);
+	}
+	
+	/**
+	 * Returns the console associated with the given debug element, or 
+	 * <code>null</code> if none.
+	 * 
+	 * @param element a debug model element
+	 * @return console associated with the given element, or 
+	 * <code>null</code> if none
+	 * @since 3.0
+	 */
+	public static IConsole getConsole(IDebugElement element) {
+		IProcess process = element.getDebugTarget().getProcess();
+		if (process != null) {
+			return getConsole(process);
+		}
+		return null;
+	}	
+	
+	/**
+	 * Returns all registered launch group extensions.
+	 *  
+	 * @return all registered launch group extensions
+	 * @since 3.0
+	 */
+	public static ILaunchGroup[] getLaunchGroups() {
+		return DebugUIPlugin.getDefault().getLaunchConfigurationManager().getLaunchGroups();
+	}
+	
+	/**
+	 * Returns the launch group that the given launch configuration belongs
+	 * to, for the specified mode, or <code>null</code> if none.
+	 * 
+	 * @param configuration
+	 * @param mode
+	 * @return the launch group the given launch configuration belongs to, for
+	 * the specified mode, or <code>null</code> if none
+	 * @since 3.0
+	 */
+	public static ILaunchGroup getLaunchGroup(ILaunchConfiguration configuration, String mode) {
+		return DebugUIPlugin.getDefault().getLaunchConfigurationManager().getLaunchGroup(configuration, mode);
+	}
+	
+    /**
+     * Performs source lookup on the given artifact and returns the result.
+     * Optionally, a source locator may be specified.
+     *  
+     * @param artifact object for which source is to be resolved
+     * @param locator the source locator to use, or <code>null</code>. When <code>null</code>
+     *   a source locator is determined from the artifact, if possible. If the artifact
+     *   is a debug element, the source locator from its associated launch is used. 
+     * @return a source lookup result
+     * @since 3.1
+     */
+    public static ISourceLookupResult lookupSource(Object artifact, ISourceLocator locator) {	
+    	return SourceLookupFacility.getDefault().lookup(artifact, locator);
+    }
+	
+    /**
+     * Displays the given source lookup result in an editor in the given workbench
+     * page. Has no effect if the result has an unknown editor id or editor input.
+     * The editor is opened, positioned, and annotated.
+     * <p>
+     * Honors user preference for editors re-use.
+     * </p> 
+     * @param result source lookup result to display
+     * @param page the page to display the result in
+     * @since 3.1
+     */
+    public static void displaySource(ISourceLookupResult result, IWorkbenchPage page) {
+    	SourceLookupFacility.getDefault().display(result, page);
+    }
+    
+    /**
+     * Returns the memory rendering manager.
+     * 
+     * @return the memory rendering manager
+     * @since 3.1
+     */
+    public static IMemoryRenderingManager getMemoryRenderingManager() {
+        return MemoryRenderingManager.getDefault();
+    }
+    
+	/**
+	 * Returns the image associated with the specified type of source container
+	 * or <code>null</code> if none.
+	 * 
+	 * @param id unique identifier for a source container type
+	 * @return image associated with the specified type of source container
+	 *    or <code>null</code> if none
+	 * @since 3.2
+	 * @see org.eclipse.debug.core.sourcelookup.ISourceContainerType
+	 */
+	public static Image getSourceContainerImage(String id){
+		return SourceLookupUIUtils.getSourceContainerImage(id);
+	}
+	
+	/**
+	 * Returns a new source container browser for the specified type of source container
+	 * or <code>null</code> if a browser has not been registered.
+	 * 
+	 * @param id unique identifier for a source container type
+	 * @return source container browser or <code>null</code> if none
+	 * @since 3.2
+	 * @see org.eclipse.debug.ui.sourcelookup.ISourceContainerBrowser
+	 */
+	public static ISourceContainerBrowser getSourceContainerBrowser(String id) {
+		return SourceLookupUIUtils.getSourceContainerBrowser(id);
+	}
+	
+	/**
+	 * Returns the color associated with the specified preference identifier or 
+	 * <code>null</code> if none.
+	 * 
+	 * @param id preference identifier of the color
+	 * @return the color associated with the specified preference identifier
+	 * 	or <code>null</code> if none
+	 * @since 3.2
+	 * @see IDebugUIConstants
+	 */
+	public static Color getPreferenceColor(String id) {
+		return DebugUIPlugin.getPreferenceColor(id);
+	}
 }
