@@ -1,13 +1,11 @@
-/*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+/***************************************************************************************************
+ * Copyright (c) 2000, 2006 IBM Corporation and others. All rights reserved. This program and the
+ * accompanying materials are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  * 
- * Contributors:
- *     IBM Corporation - initial API and implementation
- *******************************************************************************/
+ * Contributors: IBM Corporation - initial API and implementation
+ **************************************************************************************************/
 package org.eclipse.help.internal.search;
 
 import java.io.File;
@@ -22,6 +20,7 @@ import java.nio.channels.OverlappingFileLockException;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Locale;
@@ -47,19 +46,24 @@ import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.OperationCanceledException;
 import org.eclipse.core.runtime.Platform;
 import org.eclipse.core.runtime.Status;
+import org.eclipse.help.internal.base.BaseHelpSystem;
 import org.eclipse.help.internal.base.HelpBasePlugin;
 import org.eclipse.help.internal.base.util.HelpProperties;
+import org.eclipse.help.internal.protocols.HelpURLConnection;
 import org.eclipse.help.internal.protocols.HelpURLStreamHandler;
 import org.eclipse.help.internal.toc.TocManager;
 import org.eclipse.help.internal.util.ResourceLocator;
+import org.eclipse.help.search.ISearchIndex;
+import org.eclipse.help.search.LuceneSearchParticipant;
 import org.osgi.framework.Bundle;
 import org.osgi.framework.Constants;
+import org.osgi.framework.Version;
 
 /**
- * Text search index. Documents added to this index can than be searched against
- * a search query.
+ * Text search index. Documents added to this index can than be searched against a search query.
  */
-public class SearchIndex {
+public class SearchIndex implements ISearchIndex {
+
 	private IndexReader ir;
 
 	private IndexWriter iw;
@@ -97,7 +101,7 @@ public class SearchIndex {
 
 	private File inconsistencyFile;
 
-	private HTMLDocParser parser;
+	private HTMLSearchParticipant htmlSearchParticipant;
 
 	private IndexSearcher searcher;
 
@@ -120,10 +124,8 @@ public class SearchIndex {
 	 * @param analyzerDesc
 	 *            the analyzer used to index
 	 */
-	public SearchIndex(String locale, AnalyzerDescriptor analyzerDesc,
-			TocManager tocManager) {
-		this(new File(HelpBasePlugin.getConfigurationDirectory(),
-				"index/" + locale), //$NON-NLS-1$
+	public SearchIndex(String locale, AnalyzerDescriptor analyzerDesc, TocManager tocManager) {
+		this(new File(HelpBasePlugin.getConfigurationDirectory(), "index/" + locale), //$NON-NLS-1$
 				locale, analyzerDesc, tocManager, null);
 	}
 
@@ -137,8 +139,7 @@ public class SearchIndex {
 	 * @since 3.1
 	 */
 
-	public SearchIndex(File indexDir, String locale,
-			AnalyzerDescriptor analyzerDesc, TocManager tocManager,
+	public SearchIndex(File indexDir, String locale, AnalyzerDescriptor analyzerDesc, TocManager tocManager,
 			String relativePath) {
 		this.locale = locale;
 		this.analyzerDescriptor = analyzerDesc;
@@ -146,9 +147,8 @@ public class SearchIndex {
 		this.indexDir = indexDir;
 		this.relativePath = relativePath;
 		// System.out.println("Index for a relative path: "+relativePath);
-		inconsistencyFile = new File(indexDir.getParentFile(), locale
-				+ ".inconsistent"); //$NON-NLS-1$
-		parser = new HTMLDocParser();
+		inconsistencyFile = new File(indexDir.getParentFile(), locale + ".inconsistent"); //$NON-NLS-1$
+		htmlSearchParticipant = new HTMLSearchParticipant(indexDir.getAbsolutePath());
 		if (!exists()) {
 			try {
 				if (tryLock()) {
@@ -168,8 +168,7 @@ public class SearchIndex {
 	}
 
 	/**
-	 * Indexes one document from a stream. Index has to be open and close
-	 * outside of this method
+	 * Indexes one document from a stream. Index has to be open and close outside of this method
 	 * 
 	 * @param name
 	 *            the document identifier (could be a URL)
@@ -184,43 +183,65 @@ public class SearchIndex {
 		}
 		try {
 			Document doc = new Document();
-			doc.add(Field.Keyword(FIELD_NAME, name)); //$NON-NLS-1$
+			doc.add(Field.Keyword(FIELD_NAME, name));
+			addExtraFields(doc);
+			String pluginId = SearchManager.getPluginId(name);
 			if (relativePath != null)
-				doc.add(Field.Keyword(FIELD_INDEX_ID, relativePath)); //$NON-NLS-1$
-			try {
-				try {
-					parser.openDocument(url);
-				} catch (IOException ioe) {
-					return new Status(IStatus.ERROR, HelpBasePlugin.PLUGIN_ID,
-							IStatus.ERROR, "Help document " //$NON-NLS-1$
-									+ name + " cannot be opened.", //$NON-NLS-1$
-							null);
+				doc.add(Field.Keyword(FIELD_INDEX_ID, relativePath));
+			// NEW: check for the explicit search participant.
+			LuceneSearchParticipant participant = null;
+			HelpURLConnection urlc = new HelpURLConnection(url);
+			String id = urlc.getValue("id"); //$NON-NLS-1$
+			String pid = urlc.getValue("participantId"); //$NON-NLS-1$
+			if (pid != null)
+				participant = BaseHelpSystem.getSearchManager().getGlobalParticipant(pid);
+			// NEW: check for file extension-based search participant;
+			if (participant == null)
+				participant = BaseHelpSystem.getSearchManager().getParticipant(pluginId, name);
+			if (participant != null) {
+				IStatus status = participant.addDocument(this, pluginId, name, url, id, doc);
+				if (status.getSeverity() == IStatus.OK) {
+					String filters = doc.get("filters"); //$NON-NLS-1$
+					indexedDocs.put(name, filters != null ? filters : "0"); //$NON-NLS-1$
+					if (id != null)
+						doc.add(Field.UnIndexed("id", id)); //$NON-NLS-1$
+					if (pid != null)
+						doc.add(Field.UnIndexed("participantId", pid)); //$NON-NLS-1$
+					iw.addDocument(doc);
 				}
-				ParsedDocument parsed = new ParsedDocument(parser
-						.getContentReader());
-				doc.add(Field.Text("contents", parsed.newContentReader())); //$NON-NLS-1$
-				doc.add(Field.Text("exact_contents", parsed //$NON-NLS-1$
-						.newContentReader()));
-				String title = parser.getTitle();
-				doc.add(Field.UnStored("title", title)); //$NON-NLS-1$
-				doc.add(Field.UnStored("exact_title", title)); //$NON-NLS-1$
-				doc.add(Field.UnIndexed("raw_title", title)); //$NON-NLS-1$
-				doc.add(Field.UnIndexed("summary", parser.getSummary())); //$NON-NLS-1$
-				iw.addDocument(doc);
-			} finally {
-				parser.closeDocument();
+				return status;
 			}
-			indexedDocs.put(name, "0"); //$NON-NLS-1$
-			return Status.OK_STATUS;
+			// default to html
+			IStatus status = htmlSearchParticipant.addDocument(this, pluginId, name, url, id, doc);
+			if (status.getSeverity() == IStatus.OK) {
+				String filters = doc.get("filters"); //$NON-NLS-1$
+				indexedDocs.put(name, filters != null ? filters : "0"); //$NON-NLS-1$
+				iw.addDocument(doc);
+			}
+			return status;
 		} catch (IOException e) {
-			return new Status(IStatus.ERROR, HelpBasePlugin.PLUGIN_ID,
-					IStatus.ERROR,
+			return new Status(IStatus.ERROR, HelpBasePlugin.PLUGIN_ID, IStatus.ERROR,
 					"IO exception occurred while adding document " + name //$NON-NLS-1$
 							+ " to index " + indexDir.getAbsolutePath() + ".", //$NON-NLS-1$ //$NON-NLS-2$
 					e);
 		}
+		catch (Exception e) {
+			return new Status(IStatus.ERROR, HelpBasePlugin.PLUGIN_ID, IStatus.ERROR,
+					"An unexpected internal error occurred while adding document " //$NON-NLS-1$
+							+ name + " to index " + indexDir.getAbsolutePath() //$NON-NLS-1$
+							+ ".", e); //$NON-NLS-1$
+		}
 	}
 
+	/**
+	 * Add any extra fields that need to be added to this document. Subclasses
+	 * should override to add more fields.
+	 * 
+	 * @param doc the document to add fields to
+	 */
+	protected void addExtraFields(Document doc) {
+	}
+	
 	/**
 	 * Starts additions. To be called before adding documents.
 	 */
@@ -230,9 +251,8 @@ public class SearchIndex {
 				iw.close();
 			}
 			boolean create = false;
-			if (!indexDir.exists() || !isLuceneCompatible()
-					|| !isAnalyzerCompatible() || inconsistencyFile.exists()
-					&& firstOperation) {
+			if (!indexDir.exists() || !isLuceneCompatible() || !isAnalyzerCompatible()
+					|| inconsistencyFile.exists() && firstOperation) {
 				create = true;
 				indexDir.mkdirs();
 				if (!indexDir.exists())
@@ -241,15 +261,12 @@ public class SearchIndex {
 			indexedDocs = new HelpProperties(INDEXED_DOCS_FILE, indexDir);
 			indexedDocs.restore();
 			setInconsistent(true);
-			iw = new IndexWriter(indexDir, analyzerDescriptor.getAnalyzer(),
-					create);
+			iw = new IndexWriter(indexDir, analyzerDescriptor.getAnalyzer(), create);
 			iw.mergeFactor = 20;
 			iw.maxFieldLength = 1000000;
 			return true;
 		} catch (IOException e) {
-			HelpBasePlugin
-					.logError(
-							"Exception occurred in search indexing at beginAddBatch.", e); //$NON-NLS-1$
+			HelpBasePlugin.logError("Exception occurred in search indexing at beginAddBatch.", e); //$NON-NLS-1$
 			return false;
 		}
 	}
@@ -268,9 +285,7 @@ public class SearchIndex {
 			ir = IndexReader.open(indexDir);
 			return true;
 		} catch (IOException e) {
-			HelpBasePlugin
-					.logError(
-							"Exception occurred in search indexing at beginDeleteBatch.", e); //$NON-NLS-1$
+			HelpBasePlugin.logError("Exception occurred in search indexing at beginDeleteBatch.", e); //$NON-NLS-1$
 			return false;
 		}
 	}
@@ -286,9 +301,7 @@ public class SearchIndex {
 			ir = IndexReader.open(indexDir);
 			return true;
 		} catch (IOException e) {
-			HelpBasePlugin
-					.logError(
-							"Exception occurred in search indexing at beginDeleteBatch.", e); //$NON-NLS-1$
+			HelpBasePlugin.logError("Exception occurred in search indexing at beginDeleteBatch.", e); //$NON-NLS-1$
 			return false;
 		}
 	}
@@ -304,15 +317,13 @@ public class SearchIndex {
 		if (HelpBasePlugin.DEBUG_SEARCH) {
 			System.out.println("SearchIndex.removeDocument(" + name + ")"); //$NON-NLS-1$ //$NON-NLS-2$
 		}
-		Term term = new Term(FIELD_NAME, name); //$NON-NLS-1$
+		Term term = new Term(FIELD_NAME, name);
 		try {
 			ir.delete(term);
 			indexedDocs.remove(name);
+			//System.out.println("removed: " + name); //$NON-NLS-1$
 		} catch (IOException e) {
-			return new Status(
-					IStatus.ERROR,
-					HelpBasePlugin.PLUGIN_ID,
-					IStatus.ERROR,
+			return new Status(IStatus.ERROR, HelpBasePlugin.PLUGIN_ID, IStatus.ERROR,
 					"IO exception occurred while removing document " + name //$NON-NLS-1$
 							+ " from index " + indexDir.getAbsolutePath() + ".", //$NON-NLS-1$ //$NON-NLS-2$
 					e);
@@ -323,8 +334,7 @@ public class SearchIndex {
 	/**
 	 * Finish additions. To be called after adding documents.
 	 */
-	public synchronized boolean endAddBatch(boolean optimize,
-			boolean lastOperation) {
+	public synchronized boolean endAddBatch(boolean optimize, boolean lastOperation) {
 		try {
 			if (iw == null)
 				return false;
@@ -342,10 +352,18 @@ public class SearchIndex {
 				indexedDocs = null;
 				setInconsistent(false);
 			}
+			
+			/*
+			 * The searcher's index reader has it's stuff in memory so it won't
+			 * know about this change. Close it so that it gets reloaded next search.
+			 */
+			if (searcher != null) {
+				searcher.close();
+				searcher = null;
+			}
 			return true;
 		} catch (IOException e) {
-			HelpBasePlugin.logError(
-					"Exception occurred in search indexing at endAddBatch.", e); //$NON-NLS-1$
+			HelpBasePlugin.logError("Exception occurred in search indexing at endAddBatch.", e); //$NON-NLS-1$
 			return false;
 		}
 	}
@@ -366,11 +384,18 @@ public class SearchIndex {
 			indexedDocs = null;
 			getDocPlugins().save();
 			saveDependencies();
+			
+			/*
+			 * The searcher's index reader has it's stuff in memory so it won't
+			 * know about this change. Close it so that it gets reloaded next search.
+			 */
+			if (searcher != null) {
+				searcher.close();
+				searcher = null;
+			}
 			return true;
 		} catch (IOException e) {
-			HelpBasePlugin
-					.logError(
-							"Exception occurred in search indexing at endDeleteBatch.", e); //$NON-NLS-1$
+			HelpBasePlugin.logError("Exception occurred in search indexing at endDeleteBatch.", e); //$NON-NLS-1$
 			return false;
 		}
 	}
@@ -394,9 +419,7 @@ public class SearchIndex {
 			setInconsistent(false);
 			return true;
 		} catch (IOException e) {
-			HelpBasePlugin
-					.logError(
-							"Exception occurred in search indexing at endDeleteBatch.", e); //$NON-NLS-1$
+			HelpBasePlugin.logError("Exception occurred in search indexing at endDeleteBatch.", e); //$NON-NLS-1$
 			return false;
 		}
 	}
@@ -406,9 +429,8 @@ public class SearchIndex {
 	 * 
 	 * @param dirs
 	 * @param monitor
-	 * @return Map. Keys are /pluginid/href of all merged Docs. Values are null
-	 *         for added document, or String[] of indexIds with duplicates of
-	 *         the document
+	 * @return Map. Keys are /pluginid/href of all merged Docs. Values are null for added document,
+	 *         or String[] of indexIds with duplicates of the document
 	 */
 	public Map merge(PluginIndex[] pluginIndexes, IProgressMonitor monitor) {
 		ArrayList dirList = new ArrayList(pluginIndexes.length);
@@ -438,8 +460,7 @@ public class SearchIndex {
 							.println("SearchIndex.merge merging indexId=" + indexId + ", indexPath=" + indexPath); //$NON-NLS-1$ //$NON-NLS-2$
 				}
 
-				HelpProperties prebuiltDocs = new HelpProperties(
-						INDEXED_DOCS_FILE, new File(indexPath));
+				HelpProperties prebuiltDocs = new HelpProperties(INDEXED_DOCS_FILE, new File(indexPath));
 				prebuiltDocs.restore();
 				Set prebuiltHrefs = prebuiltDocs.keySet();
 				for (Iterator it = prebuiltHrefs.iterator(); it.hasNext();) {
@@ -457,8 +478,7 @@ public class SearchIndex {
 							} else {
 								// next duplicate
 								String[] newDups = new String[dups.length + 1];
-								System.arraycopy(dups, 0, newDups, 0,
-										dups.length);
+								System.arraycopy(dups, 0, newDups, 0, dups.length);
 								newDups[dups.length] = indexId;
 								mergedDocs.put(href, newDups);
 							}
@@ -476,8 +496,7 @@ public class SearchIndex {
 		for (Iterator it = mergedDocs.keySet().iterator(); it.hasNext();) {
 			indexedDocs.put(it.next(), "0"); //$NON-NLS-1$
 		}
-		Directory[] luceneDirs = (Directory[]) dirList
-				.toArray(new Directory[dirList.size()]);
+		Directory[] luceneDirs = (Directory[]) dirList.toArray(new Directory[dirList.size()]);
 		try {
 			iw.addIndexes(luceneDirs);
 		} catch (IOException ioe) {
@@ -511,10 +530,7 @@ public class SearchIndex {
 				removeDocuments(hrefDocs, indexDocs);
 			}
 		} catch (IOException ioe) {
-			return new Status(
-					IStatus.ERROR,
-					HelpBasePlugin.PLUGIN_ID,
-					IStatus.ERROR,
+			return new Status(IStatus.ERROR, HelpBasePlugin.PLUGIN_ID, IStatus.ERROR,
 					"IO exception occurred while removing duplicates of document " + name //$NON-NLS-1$
 							+ " from index " + indexDir.getAbsolutePath() + ".", //$NON-NLS-1$ //$NON-NLS-2$
 					ioe);
@@ -542,8 +558,7 @@ public class SearchIndex {
 	 * @param docs2
 	 * @throws IOException
 	 */
-	private void removeDocuments(TermDocs doc1, TermDocs docs2)
-			throws IOException {
+	private void removeDocuments(TermDocs doc1, TermDocs docs2) throws IOException {
 		if (!doc1.next()) {
 			return;
 		}
@@ -597,26 +612,24 @@ public class SearchIndex {
 			registerSearch(Thread.currentThread());
 			if (closed)
 				return;
-			QueryBuilder queryBuilder = new QueryBuilder(searchQuery
-					.getSearchWord(), analyzerDescriptor);
-			Query luceneQuery = queryBuilder.getLuceneQuery(searchQuery
-					.getFieldNames(), searchQuery.isFieldSearch());
+			QueryBuilder queryBuilder = new QueryBuilder(searchQuery.getSearchWord(), analyzerDescriptor);
+			Query luceneQuery = queryBuilder.getLuceneQuery(searchQuery.getFieldNames(), searchQuery
+					.isFieldSearch());
 			String highlightTerms = queryBuilder.gethighlightTerms();
 			if (luceneQuery != null) {
 				if (searcher == null) {
 					openSearcher();
 				}
 				Hits hits = searcher.search(luceneQuery);
-				collector.addHits(hits, highlightTerms);
+				collector.addHits(SearchManager.asList(hits), highlightTerms);
 			}
-		} catch (BooleanQuery.TooManyClauses tmc){
+		} catch (BooleanQuery.TooManyClauses tmc) {
 			throw new QueryTooComplexException();
 		} catch (QueryTooComplexException qe) {
 			throw qe;
 		} catch (Exception e) {
-			HelpBasePlugin.logError(
-					"Exception occurred performing search for: " //$NON-NLS-1$
-							+ searchQuery.getSearchWord() + ".", e); //$NON-NLS-1$
+			HelpBasePlugin.logError("Exception occurred performing search for: " //$NON-NLS-1$
+					+ searchQuery.getSearchWord() + ".", e); //$NON-NLS-1$
 		} finally {
 			unregisterSearch(Thread.currentThread());
 		}
@@ -627,21 +640,23 @@ public class SearchIndex {
 	}
 
 	/**
-	 * Returns the list of all the plugins in this session that have declared a
-	 * help contribution.
+	 * Returns the list of all the plugins in this session that have declared a help contribution.
 	 */
 	public PluginVersionInfo getDocPlugins() {
 		if (docPlugins == null) {
+			HashSet totalIds = new HashSet();
 			Collection docPluginsIds = tocManager.getContributingPlugins();
-			docPlugins = new PluginVersionInfo(INDEXED_CONTRIBUTION_INFO_FILE,
-					docPluginsIds, indexDir, !exists());
+			Collection additionalPluginIds = BaseHelpSystem.getSearchManager()
+					.getPluginsWithSearchParticipants();
+			totalIds.addAll(docPluginsIds);
+			totalIds.addAll(additionalPluginIds);
+			docPlugins = new PluginVersionInfo(INDEXED_CONTRIBUTION_INFO_FILE, totalIds, indexDir, !exists());
 		}
 		return docPlugins;
 	}
 
 	/**
-	 * Sets the list of all plug-ns in this session. This method is used for
-	 * external indexer.
+	 * Sets the list of all plug-ns in this session. This method is used for external indexer.
 	 * 
 	 * @param docPlugins
 	 */
@@ -650,53 +665,55 @@ public class SearchIndex {
 	}
 
 	/**
-	 * We use HelpProperties, but a list would suffice. We only need the key
-	 * values.
+	 * We use HelpProperties, but a list would suffice. We only need the key values.
 	 * 
 	 * @return HelpProperties, keys are URLs of indexed documents
 	 */
 	public HelpProperties getIndexedDocs() {
-		HelpProperties indexedDocs = new HelpProperties(INDEXED_DOCS_FILE,
-				indexDir);
+		HelpProperties indexedDocs = new HelpProperties(INDEXED_DOCS_FILE, indexDir);
 		if (exists())
 			indexedDocs.restore();
 		return indexedDocs;
 	}
 
 	/**
-	 * Gets properties with versions of Lucene plugin and Analyzer used in
-	 * existing index
+	 * Gets properties with versions of Lucene plugin and Analyzer used in existing index
 	 */
 	private HelpProperties getDependencies() {
 		if (dependencies == null) {
-			dependencies = new HelpProperties(DEPENDENCIES_VERSION_FILENAME,
-					indexDir);
+			dependencies = new HelpProperties(DEPENDENCIES_VERSION_FILENAME, indexDir);
 			dependencies.restore();
 		}
 		return dependencies;
 	}
 
 	private boolean isLuceneCompatible() {
-		String usedLuceneVersion = getDependencies().getProperty(
-				DEPENDENCIES_KEY_LUCENE); //$NON-NLS-1$
+		String usedLuceneVersion = getDependencies().getProperty(DEPENDENCIES_KEY_LUCENE);
 		return isLuceneCompatible(usedLuceneVersion);
 	}
 
 	public boolean isLuceneCompatible(String luceneVersion) {
+		if (luceneVersion==null) return false;
 		String currentLuceneVersion = ""; //$NON-NLS-1$
 		Bundle lucenePluginDescriptor = Platform.getBundle(LUCENE_PLUGIN_ID);
 		if (lucenePluginDescriptor != null) {
-			currentLuceneVersion += (String) lucenePluginDescriptor
-					.getHeaders().get(Constants.BUNDLE_VERSION);
+			currentLuceneVersion += (String) lucenePluginDescriptor.getHeaders()
+					.get(Constants.BUNDLE_VERSION);
 		}
-		// Later might add code to return true for known cases
-		// of compatibility between 3.1 and post 3.1 versions.
-		return currentLuceneVersion.equals(luceneVersion);
+		//Direct comparison
+		if (currentLuceneVersion.equals(luceneVersion))
+			return true;
+		Version version = new Version(currentLuceneVersion);
+		Version currentVersion = new Version(luceneVersion);
+		// must not compare with the qualifier because they
+		// change from build to build
+		return version.getMajor() == currentVersion.getMajor()
+				&& version.getMinor() == currentVersion.getMinor()
+				&& version.getMicro() == currentVersion.getMicro();
 	}
 
 	private boolean isAnalyzerCompatible() {
-		String usedAnalyzer = getDependencies().getProperty(
-				DEPENDENCIES_KEY_ANALYZER); //$NON-NLS-1$
+		String usedAnalyzer = getDependencies().getProperty(DEPENDENCIES_KEY_ANALYZER);
 		return isAnalyzerCompatible(usedAnalyzer);
 	}
 
@@ -711,23 +728,21 @@ public class SearchIndex {
 	 * Saves Lucene version and analyzer identifier to a file.
 	 */
 	private void saveDependencies() {
-		getDependencies().put(DEPENDENCIES_KEY_ANALYZER,
-				analyzerDescriptor.getId()); //$NON-NLS-1$
+		getDependencies().put(DEPENDENCIES_KEY_ANALYZER, analyzerDescriptor.getId());
 		Bundle luceneBundle = Platform.getBundle(LUCENE_PLUGIN_ID);
 		if (luceneBundle != null) {
 			String luceneBundleVersion = "" //$NON-NLS-1$
 					+ luceneBundle.getHeaders().get(Constants.BUNDLE_VERSION);
-			getDependencies().put(DEPENDENCIES_KEY_LUCENE, luceneBundleVersion); //$NON-NLS-1$
+			getDependencies().put(DEPENDENCIES_KEY_LUCENE, luceneBundleVersion);
 		} else {
-			getDependencies().put(DEPENDENCIES_KEY_LUCENE, ""); //$NON-NLS-1$ //$NON-NLS-2$
+			getDependencies().put(DEPENDENCIES_KEY_LUCENE, ""); //$NON-NLS-1$ 
 		}
 		getDependencies().save();
 	}
 
 	/**
-	 * @return Returns true if index has been left in inconsistent state If
-	 *         analyzer has changed to incompatible one, index is treated as
-	 *         inconsistent as well.
+	 * @return Returns true if index has been left in inconsistent state If analyzer has changed to
+	 *         incompatible one, index is treated as inconsistent as well.
 	 */
 	public boolean isInconsistent() {
 		if (inconsistencyFile.exists()) {
@@ -761,9 +776,8 @@ public class SearchIndex {
 	}
 
 	/**
-	 * Closes IndexReader used by Searcher. Should be called on platform
-	 * shutdown, or when TOCs have changed when no more reading from this index
-	 * is to be performed.
+	 * Closes IndexReader used by Searcher. Should be called on platform shutdown, or when TOCs have
+	 * changed when no more reading from this index is to be performed.
 	 */
 	public void close() {
 		closed = true;
@@ -789,13 +803,11 @@ public class SearchIndex {
 	 * Finds and unzips prebuild index specified in preferences
 	 */
 	private void unzipProductIndex() {
-		String indexPluginId = HelpBasePlugin.getDefault()
-				.getPluginPreferences().getString("productIndex"); //$NON-NLS-1$
+		String indexPluginId = HelpBasePlugin.getDefault().getPluginPreferences().getString("productIndex"); //$NON-NLS-1$
 		if (indexPluginId == null || indexPluginId.length() <= 0) {
 			return;
 		}
-		InputStream zipIn = ResourceLocator.openFromPlugin(indexPluginId,
-				"doc_index.zip", getLocale()); //$NON-NLS-1$
+		InputStream zipIn = ResourceLocator.openFromPlugin(indexPluginId, "doc_index.zip", getLocale()); //$NON-NLS-1$
 		if (zipIn == null) {
 			return;
 		}
@@ -859,8 +871,7 @@ public class SearchIndex {
 	private void cleanOldIndex() {
 		IndexWriter cleaner = null;
 		try {
-			cleaner = new IndexWriter(indexDir, analyzerDescriptor
-					.getAnalyzer(), true);
+			cleaner = new IndexWriter(indexDir, analyzerDescriptor.getAnalyzer(), true);
 		} catch (IOException ioe) {
 		} finally {
 			try {
@@ -938,8 +949,7 @@ public class SearchIndex {
 	/**
 	 * Deletes the lock file. The lock must be released prior to this call.
 	 * 
-	 * @return <code>true</code> if the file has been deleted,
-	 *         <code>false</code> otherwise.
+	 * @return <code>true</code> if the file has been deleted, <code>false</code> otherwise.
 	 */
 
 	public synchronized boolean deleteLockFile() {
@@ -974,8 +984,8 @@ public class SearchIndex {
 			url = url.substring(0, url.lastIndexOf('#'));
 			// its a fragment, index whole document
 		} else {
-			// not indexable
-			return null;
+			// try search participants
+			return BaseHelpSystem.getSearchManager().isIndexable(url) ? url : null;
 		}
 		return url;
 	}
@@ -983,23 +993,61 @@ public class SearchIndex {
 	/**
 	 * Checks if document is indexable, and creates a URL to obtain contents.
 	 * 
+	 * @param locale
 	 * @param url
 	 *            specified in the navigation
 	 * @return URL to obtain document content or null
 	 */
 	public static URL getIndexableURL(String locale, String url) {
-		url = getIndexableHref(url);
+		return getIndexableURL(locale, url, null, null);
+	}
+
+	/**
+	 * Checks if document is indexable, and creates a URL to obtain contents.
+	 * 
+	 * @param locale
+	 * @param url
+	 * @param participantId
+	 *            the search participant or <code>null</code> specified in the navigation
+	 * @return URL to obtain document content or null
+	 */
+	public static URL getIndexableURL(String locale, String url, String id, String participantId) {
+		if (participantId == null)
+			url = getIndexableHref(url);
 		if (url == null)
 			return null;
 
 		try {
-			// return new URL("help:" + url + "?lang=" + index.getLocale());
+			StringBuffer query = new StringBuffer();
+			query.append("?"); //$NON-NLS-1$
+			query.append("lang=" + locale); //$NON-NLS-1$
+			if (id != null)
+				query.append("&id=" + id); //$NON-NLS-1$
+			if (participantId != null)
+				query.append("&participantId=" + participantId); //$NON-NLS-1$
 			return new URL("help", //$NON-NLS-1$
-					null, -1, url + "?lang=" + locale, //$NON-NLS-1$
-					HelpURLStreamHandler.getDefault());
+					null, -1, url + query.toString(), HelpURLStreamHandler.getDefault());
 
 		} catch (MalformedURLException mue) {
 			return null;
 		}
 	}
-}
+
+	public IStatus addDocument(String pluginId, String name, URL url, String id, Document doc) {
+		// try a registered participant for the file format
+		LuceneSearchParticipant participant = BaseHelpSystem.getSearchManager()
+				.getParticipant(pluginId, name);
+		if (participant != null) {
+			try {
+				return participant.addDocument(this, pluginId, name, url, id, doc);
+			}
+			catch (Throwable t) {
+				return new Status(IStatus.ERROR, HelpBasePlugin.PLUGIN_ID, IStatus.ERROR,
+						"Error while adding document to search participant (addDocument()): " //$NON-NLS-1$
+						+ name + ", " + url + "for participant " + participant.getClass().getName(), t); //$NON-NLS-1$ //$NON-NLS-2$
+			}
+		}
+		// default to html
+		return htmlSearchParticipant.addDocument(this, pluginId, name, url, id, doc);
+	}
+}
\ No newline at end of file
