@@ -1,13 +1,11 @@
-/*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+/***************************************************************************************************
+ * Copyright (c) 2000, 2005 IBM Corporation and others. All rights reserved. This program and the
+ * accompanying materials are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  * 
- * Contributors:
- *     IBM Corporation - initial API and implementation
- *******************************************************************************/
+ * Contributors: IBM Corporation - initial API and implementation
+ **************************************************************************************************/
 package org.eclipse.help.ui.internal.views;
 
 import java.io.IOException;
@@ -60,29 +58,27 @@ import org.eclipse.ui.forms.widgets.Section;
 import org.eclipse.ui.forms.widgets.TableWrapData;
 import org.eclipse.ui.forms.widgets.TableWrapLayout;
 
-public class SearchPart extends AbstractFormPart implements IHelpPart,
-		IHelpUIConstants {
+public class SearchPart extends AbstractFormPart implements IHelpPart, IHelpUIConstants {
+
 	private ReusableHelpPart parent;
 
-	protected static java.util.List previousSearchQueryData = new java.util.ArrayList(
-			20);
+	protected static java.util.List previousSearchQueryData = new java.util.ArrayList(20);
 
 	private static final String HREF_TOGGLE = "__toggle__"; //$NON-NLS-1$
 
 	private static final String HREF_SEARCH_HELP = "/org.eclipse.platform.doc.user/tasks/tsearch.htm"; //$NON-NLS-1$
-	
-	private static boolean SEARCH_HELP_AVAILABLE=false;
+
+	private static boolean SEARCH_HELP_AVAILABLE = false;
 
 	static {
 		InputStream is = HelpSystem.getHelpContent(HREF_SEARCH_HELP);
-		if (is!=null) {
-			//don't leak the input stream
+		if (is != null) {
+			// don't leak the input stream
 			try {
 				is.close();
 				SEARCH_HELP_AVAILABLE = true;
-			}
-			catch (IOException e) {
-				//ignore
+			} catch (IOException e) {
+				// ignore
 			}
 		}
 	}
@@ -91,6 +87,7 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 
 	private Composite container;
 
+	private Composite filteringGroup;
 	private FormText searchWordText;
 
 	private Chevron searchWordChevron;
@@ -105,17 +102,18 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 
 	private Hyperlink advancedLink;
 
-	private ScopeSetManager scopeSetManager;
+	private Observer engineObserver;
 
-	private EngineDescriptorManager descManager;
+	private ScopeSetManager scopeSetManager;
 
 	private static final int COMBO_HISTORY_SIZE = 10;
 
 	private JobListener jobListener;
-	
+
 	private boolean searchPending;
 
 	private class JobListener implements IJobChangeListener, Runnable {
+
 		private boolean searchInProgress = false;
 
 		public void aboutToRun(IJobChangeEvent event) {
@@ -126,8 +124,7 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 
 		public void done(IJobChangeEvent event) {
 			if (event.getJob().belongsTo(FederatedSearchJob.FAMILY)) {
-				Job[] searchJobs = Platform.getJobManager().find(
-						FederatedSearchJob.FAMILY);
+				Job[] searchJobs = Platform.getJobManager().find(FederatedSearchJob.FAMILY);
 				if (searchJobs.length == 0) {
 					// search finished
 					searchInProgress = false;
@@ -145,8 +142,7 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 		}
 
 		public void scheduled(IJobChangeEvent event) {
-			if (!searchInProgress
-					&& event.getJob().belongsTo(FederatedSearchJob.FAMILY)) {
+			if (!searchInProgress && event.getJob().belongsTo(FederatedSearchJob.FAMILY)) {
 				searchInProgress = true;
 				container.getDisplay().asyncExec(this);
 			}
@@ -182,19 +178,20 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 		searchWordText = toolkit.createFormText(container, false);
 		searchWordChevron = new Chevron(searchWordText, SWT.NULL);
 		toolkit.adapt(searchWordChevron, true, true);
-		searchWordChevron.setHoverDecorationColor(toolkit.getHyperlinkGroup()
-				.getActiveForeground());
-		searchWordChevron.setDecorationColor(toolkit.getColors().getColor(
-				FormColors.TB_TOGGLE));
+		searchWordChevron.setHoverDecorationColor(toolkit.getHyperlinkGroup().getActiveForeground());
+		searchWordChevron.setDecorationColor(toolkit.getColors().getColor(FormColors.TB_TOGGLE));
 		searchWordText.setControl(HREF_TOGGLE, searchWordChevron);
 		searchWordText.addHyperlinkListener(new HyperlinkAdapter() {
+
 			public void linkActivated(HyperlinkEvent e) {
 				SearchPart.this.parent.showURL(HREF_SEARCH_HELP, true);
 			}
 		});
 		searchWordChevron.addHyperlinkListener(new HyperlinkAdapter() {
+
 			public void linkActivated(HyperlinkEvent e) {
 				parent.getDisplay().asyncExec(new Runnable() {
+
 					public void run() {
 						toggleSearchWordText();
 					}
@@ -208,34 +205,36 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 		td.colspan = 2;
 		searchWordText.setLayoutData(td);
 		// Pattern combo
-		searchWordCombo = new ComboPart(container, toolkit, toolkit
-				.getBorderStyle());
+		searchWordCombo = new ComboPart(container, toolkit, toolkit.getBorderStyle());
 		updateSearchCombo(null);
 		td = new TableWrapData(TableWrapData.FILL_GRAB);
 		td.maxWidth = 100;
 		td.valign = TableWrapData.MIDDLE;
 		searchWordCombo.getControl().setLayoutData(td);
 		searchWordCombo.addSelectionListener(new SelectionAdapter() {
+
 			public void widgetSelected(SelectionEvent e) {
 				if (searchWordCombo.getSelectionIndex() < 0)
 					return;
 				searchFromHistory(searchWordCombo.getSelectionIndex());
 			}
 		});
-		goButton = toolkit.createButton(container, Messages.SearchPart_go,
-				SWT.PUSH);
+		goButton = toolkit.createButton(container, Messages.SearchPart_go, SWT.PUSH);
 		goButton.addSelectionListener(new SelectionAdapter() {
+
 			public void widgetSelected(SelectionEvent e) {
 				handleButtonPressed();
 			}
 		});
 		goButton.setEnabled(false);
 		searchWordCombo.addModifyListener(new ModifyListener() {
+
 			public void modifyText(ModifyEvent e) {
 				goButton.setEnabled(searchWordCombo.getText().length() > 0);
 			}
 		});
 		searchWordCombo.addKeyListener(new KeyAdapter() {
+
 			public void keyReleased(KeyEvent e) {
 				if (e.character == '\r') {
 					if (goButton.isEnabled())
@@ -243,14 +242,14 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 				}
 			}
 		});
-		scopeSection = toolkit.createSection(container, Section.TWISTIE
-				| Section.COMPACT | Section.LEFT_TEXT_CLIENT_ALIGNMENT);
+		scopeSection = toolkit.createSection(container, Section.TWISTIE | Section.COMPACT
+				| Section.LEFT_TEXT_CLIENT_ALIGNMENT);
 		scopeSection.setText(Messages.limit_to);
 		td = new TableWrapData();
 		td.colspan = 2;
 		td.align = TableWrapData.FILL;
 		scopeSection.setLayoutData(td);
-		Composite filteringGroup = toolkit.createComposite(scopeSection);
+		filteringGroup = toolkit.createComposite(scopeSection);
 		scopeSection.setClient(filteringGroup);
 		TableWrapLayout flayout = new TableWrapLayout();
 		flayout.numColumns = 2;
@@ -258,16 +257,14 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 		createScopeSet(scopeSection, toolkit);
 		toolkit.paintBordersFor(filteringGroup);
 		toolkit.paintBordersFor(container);
-		loadEngines(filteringGroup, toolkit);
-		createAdvancedLink(filteringGroup, toolkit);
 		jobListener = new JobListener();
 		Platform.getJobManager().addJobChangeListener(jobListener);
 	}
 
 	private void createAdvancedLink(Composite parent, FormToolkit toolkit) {
-		advancedLink = toolkit.createHyperlink(parent,
-				Messages.FederatedSearchPart_advanced, SWT.NULL); //$NON-NLS-1$
+		advancedLink = toolkit.createHyperlink(parent, Messages.FederatedSearchPart_advanced, SWT.NULL);
 		advancedLink.addHyperlinkListener(new HyperlinkAdapter() {
+
 			public void linkActivated(HyperlinkEvent e) {
 				doAdvanced();
 			}
@@ -280,12 +277,12 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 	private void createScopeSet(Section section, FormToolkit toolkit) {
 		scopeSetLink = toolkit.createHyperlink(section, null, SWT.NULL);
 		scopeSetLink.addHyperlinkListener(new HyperlinkAdapter() {
+
 			public void linkActivated(HyperlinkEvent e) {
 				doChangeScopeSet();
 			}
 		});
-		scopeSetLink
-				.setToolTipText(Messages.FederatedSearchPart_changeScopeSet); //$NON-NLS-1$
+		scopeSetLink.setToolTipText(Messages.FederatedSearchPart_changeScopeSet);
 		section.setTextClient(scopeSetLink);
 		ScopeSet active = scopeSetManager.getActiveSet();
 		setActiveScopeSet(active);
@@ -308,7 +305,7 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 			buff.append("\"/>"); //$NON-NLS-1$
 			buff.append("</p><p>"); //$NON-NLS-1$
 			buff.append(Messages.expression_label);
-			//Only add the link if available
+			// Only add the link if available
 			if (SEARCH_HELP_AVAILABLE) {
 				buff.append("</p><p>"); //$NON-NLS-1$
 				buff.append("<img href=\""); //$NON-NLS-1$
@@ -341,8 +338,7 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 	}
 
 	private void updateMasters(ScopeSet set) {
-		Control[] children = ((Composite) scopeSection.getClient())
-				.getChildren();
+		Control[] children = ((Composite) scopeSection.getClient()).getChildren();
 		for (int i = 0; i < children.length; i++) {
 			Control child = children[i];
 			if (child instanceof Button) {
@@ -356,15 +352,15 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 		}
 	}
 
-	private void loadEngines(final Composite container,
-			final FormToolkit toolkit) {
-		descManager = new EngineDescriptorManager();
+	private void loadEngines(final Composite container, final FormToolkit toolkit) {
+		EngineDescriptorManager descManager = parent.getEngineManager();
 		EngineDescriptor[] descriptors = descManager.getDescriptors();
 		for (int i = 0; i < descriptors.length; i++) {
 			EngineDescriptor desc = descriptors[i];
 			loadEngine(desc, container, toolkit);
 		}
-		descManager.addObserver(new Observer() {
+		engineObserver = new Observer() {
+
 			public void update(Observable o, Object arg) {
 				EngineDescriptorManager.DescriptorEvent event = (EngineDescriptorManager.DescriptorEvent) arg;
 				int kind = event.getKind();
@@ -380,22 +376,22 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 					updateEngine(desc);
 				}
 			}
-		});
+		};
+
+		descManager.addObserver(engineObserver);
 		updateMasters(scopeSetManager.getActiveSet());
 	}
 
-	private EngineDescriptor loadEngine(final EngineDescriptor edesc,
-			Composite container, FormToolkit toolkit) {
+	private EngineDescriptor loadEngine(final EngineDescriptor edesc, Composite container, FormToolkit toolkit) {
 		Label ilabel = toolkit.createLabel(container, null);
 		ilabel.setImage(edesc.getIconImage());
 		ilabel.setData(edesc);
-		final Button master = toolkit.createButton(container, edesc.getLabel(),
-				SWT.CHECK);
+		final Button master = toolkit.createButton(container, edesc.getLabel(), SWT.CHECK);
 		master.setData(edesc);
 		master.addSelectionListener(new SelectionAdapter() {
+
 			public void widgetSelected(SelectionEvent e) {
-				scopeSetManager.getActiveSet().setEngineEnabled(edesc,
-						master.getSelection());
+				scopeSetManager.getActiveSet().setEngineEnabled(edesc, master.getSelection());
 			}
 		});
 		String desc = edesc.getDescription();
@@ -403,9 +399,7 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 			Label spacer = toolkit.createLabel(container, null);
 			spacer.setData(edesc);
 			Label dlabel = toolkit.createLabel(container, desc, SWT.WRAP);
-			dlabel
-					.setForeground(toolkit.getColors().getColor(
-							FormColors.TITLE));
+			dlabel.setForeground(toolkit.getColors().getColor(FormColors.TITLE));
 			dlabel.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
 			dlabel.setMenu(container.getMenu());
 			dlabel.setData(edesc);
@@ -415,8 +409,7 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 
 	private void removeEngine(EngineDescriptor desc) {
 		boolean reflowNeeded = false;
-		Control[] children = ((Composite) scopeSection.getClient())
-				.getChildren();
+		Control[] children = ((Composite) scopeSection.getClient()).getChildren();
 		for (int i = 0; i < children.length; i++) {
 			Control child = children[i];
 			EngineDescriptor ed = (EngineDescriptor) child.getData();
@@ -431,8 +424,7 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 	}
 
 	private void updateEngine(EngineDescriptor desc) {
-		Control[] children = ((Composite) scopeSection.getClient())
-				.getChildren();
+		Control[] children = ((Composite) scopeSection.getClient()).getChildren();
 		boolean reflowNeeded = false;
 		for (int i = 0; i < children.length; i++) {
 			Control child = children[i];
@@ -490,8 +482,7 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 			scopeSetManager.remove(sset);
 		}
 		if (items.size() > 0)
-			searchWordCombo.setItems((String[]) items.toArray(new String[items
-					.size()]));
+			searchWordCombo.setItems((String[]) items.toArray(new String[items.size()]));
 	}
 
 	private void searchFromHistory(int index) {
@@ -511,7 +502,7 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 			stop();
 		}
 	}
-	
+
 	private void doSearch(String text) {
 		doSearch(text, false);
 	}
@@ -519,9 +510,9 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 	private void doSearch(String text, boolean fromHistory) {
 		ScopeSet set = scopeSetManager.getActiveSet();
 		if (!fromHistory && set instanceof HistoryScopeSet) {
-			String setExpression = ((HistoryScopeSet)set).getExpression();
+			String setExpression = ((HistoryScopeSet) set).getExpression();
 			if (setExpression.equals(text))
-				fromHistory=true;
+				fromHistory = true;
 		}
 		if (!fromHistory) {
 			storeSearchHistory(text);
@@ -534,28 +525,26 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 		final SearchResultsPart results = (SearchResultsPart) parent
 				.findPart(IHelpUIConstants.HV_FSEARCH_RESULT);
 		ArrayList eds = new ArrayList();
-		EngineDescriptor[] engineDescriptors = descManager.getDescriptors();
+		EngineDescriptor[] engineDescriptors = parent.getEngineManager().getDescriptors();
 		for (int i = 0; i < engineDescriptors.length; i++) {
 			final EngineDescriptor ed = engineDescriptors[i];
 			if (set.getEngineEnabled(ed) && ed.getEngine() != null) {
-				ISearchScope scope = ed.createSearchScope(set
-						.getPreferenceStore());
-				FederatedSearchEntry entry = new FederatedSearchEntry(ed
-						.getId(), ed.getLabel(), scope, ed.getEngine(),
-						new ISearchEngineResultCollector() {
-							public void accept(ISearchEngineResult searchResult) {
-								results.add(ed, searchResult);
-							}
+				ISearchScope scope = ed.createSearchScope(set.getPreferenceStore());
+				FederatedSearchEntry entry = new FederatedSearchEntry(ed.getId(), ed.getLabel(), scope, ed
+						.getEngine(), new ISearchEngineResultCollector() {
 
-							public void accept(
-									ISearchEngineResult[] searchResults) {
-								results.add(ed, searchResults);
-							}
+					public void accept(ISearchEngineResult searchResult) {
+						results.add(ed, searchResult);
+					}
 
-							public void error(IStatus status) {
-								results.error(ed, status);
-							}
-						});
+					public void accept(ISearchEngineResult[] searchResults) {
+						results.add(ed, searchResults);
+					}
+
+					public void error(IStatus status) {
+						results.error(ed, status);
+					}
+				});
 				entries.add(entry);
 				eds.add(ed);
 			}
@@ -575,20 +564,19 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 
 	private void doAdvanced() {
 		ScopeSet set = scopeSetManager.getActiveSet();
-		PreferenceManager manager = new ScopePreferenceManager(descManager, set);
-		PreferenceDialog dialog = new ScopePreferenceDialog(container
-				.getShell(), manager, descManager, set.isEditable());
+		PreferenceManager manager = new ScopePreferenceManager(parent.getEngineManager(), set);
+		PreferenceDialog dialog = new ScopePreferenceDialog(container.getShell(), manager, parent
+				.getEngineManager(), set.isEditable());
 		dialog.setPreferenceStore(set.getPreferenceStore());
 		dialog.create();
-		dialog.getShell().setText(
-				NLS.bind(Messages.ScopePreferenceDialog_wtitle, set.getName()));
+		dialog.getShell().setText(NLS.bind(Messages.ScopePreferenceDialog_wtitle, set.getName()));
 		dialog.open();
 		updateMasters(set);
 	}
 
 	private void doChangeScopeSet() {
-		ScopeSetDialog dialog = new ScopeSetDialog(container.getShell(),
-				scopeSetManager, descManager);
+		ScopeSetDialog dialog = new ScopeSetDialog(container.getShell(), scopeSetManager, parent
+				.getEngineManager());
 		dialog.setInput(scopeSetManager);
 		dialog.create();
 		dialog.getShell().setText(Messages.ScopeSetDialog_wtitle);
@@ -603,6 +591,10 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 		ScopeSet activeSet = scopeSetManager.getActiveSet();
 		if (activeSet != null)
 			activeSet.save();
+		if (engineObserver != null) {
+			parent.getEngineManager().deleteObserver(engineObserver);
+			engineObserver = null;
+		}
 		Platform.getJobManager().removeJobChangeListener(jobListener);
 		stop();
 		super.dispose();
@@ -625,6 +617,8 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 	public void init(ReusableHelpPart parent, String id, IMemento memento) {
 		this.parent = parent;
 		this.id = id;
+		loadEngines(filteringGroup, parent.getForm().getToolkit());
+		createAdvancedLink(filteringGroup, parent.getForm().getToolkit());
 		parent.hookFormText(searchWordText);
 		if (memento != null)
 			restorePart(memento);
@@ -638,9 +632,9 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 				scopeSetManager.setActiveSet(sset);
 		}
 		String expression = memento.getString("expression"); //$NON-NLS-1$
-		if (expression!=null && expression.length()>0) {
+		if (expression != null && expression.length() > 0) {
 			searchWordCombo.setText(expression);
-			searchPending=true;
+			searchPending = true;
 			markStale();
 		}
 	}
@@ -648,11 +642,11 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 	public void refresh() {
 		super.refresh();
 		if (searchPending) {
-			searchPending=false;
+			searchPending = false;
 			doSearch(searchWordCombo.getText());
 		}
 	}
-	
+
 	public String getId() {
 		return id;
 	}
@@ -681,8 +675,7 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 	 * @see org.eclipse.help.ui.internal.views.IHelpPart#hasFocusControl(org.eclipse.swt.widgets.Control)
 	 */
 	public boolean hasFocusControl(Control control) {
-		return control == searchWordText
-				|| control == searchWordCombo.getControl()
+		return control == searchWordText || control == searchWordCombo.getControl()
 				|| scopeSection.getClient() == control;
 	}
 
@@ -697,8 +690,7 @@ public class SearchPart extends AbstractFormPart implements IHelpPart,
 	}
 
 	public void stop() {
-		SearchResultsPart results = (SearchResultsPart) parent
-				.findPart(IHelpUIConstants.HV_FSEARCH_RESULT);
+		SearchResultsPart results = (SearchResultsPart) parent.findPart(IHelpUIConstants.HV_FSEARCH_RESULT);
 		results.canceling();
 		Platform.getJobManager().cancel(FederatedSearchJob.FAMILY);
 	}
