@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -15,6 +15,7 @@ import java.io.IOException;
 
 import org.eclipse.jdt.core.compiler.CharOperation;
 import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
+import org.eclipse.jdt.internal.compiler.problem.AbortCompilationUnit;
 import org.eclipse.jdt.internal.compiler.util.Util;
 
 public class CompilationUnit implements ICompilationUnit {
@@ -25,27 +26,27 @@ public class CompilationUnit implements ICompilationUnit {
 	
 public CompilationUnit(char[] contents, String fileName, String encoding) {
 	this.contents = contents;
-	if (File.separator.equals("/")) { //$NON-NLS-1$
-		if (fileName.indexOf("\\") != -1) { //$NON-NLS-1$
-			fileName = fileName.replace('\\', File.separatorChar);
-		}
-	} else {
-		// the file separator is \
-		if (fileName.indexOf('/') != -1) {
-			fileName = fileName.replace('/', File.separatorChar);
-		}
+	char[] fileNameCharArray = fileName.toCharArray();
+	switch(File.separatorChar) {
+		case '/' :
+			if (CharOperation.indexOf('\\', fileNameCharArray) != -1) {
+				CharOperation.replace(fileNameCharArray, '\\', '/');
+			}
+			break;
+		case '\\' :
+			if (CharOperation.indexOf('/', fileNameCharArray) != -1) {
+				CharOperation.replace(fileNameCharArray, '/', '\\');
+			}
 	}
-	this.fileName = fileName.toCharArray();
+	this.fileName = fileNameCharArray;
+	int start = CharOperation.lastIndexOf(File.separatorChar, fileNameCharArray) + 1; 
 
-	int start = fileName.lastIndexOf("/") + 1; //$NON-NLS-1$
-	if (start == 0 || start < fileName.lastIndexOf("\\")) //$NON-NLS-1$
-		start = fileName.lastIndexOf("\\") + 1; //$NON-NLS-1$
+	int end = CharOperation.lastIndexOf('.', fileNameCharArray);
+	if (end == -1) {
+		end = fileNameCharArray.length;
+	}
 
-	int end = fileName.lastIndexOf("."); //$NON-NLS-1$
-	if (end == -1)
-		end = fileName.length();
-
-	this.mainTypeName = fileName.substring(start, end).toCharArray();
+	this.mainTypeName = CharOperation.subarray(fileNameCharArray, start, end);
 	this.encoding = encoding;
 }
 public char[] getContents() {
@@ -56,9 +57,9 @@ public char[] getContents() {
 	try {
 		return Util.getFileCharContent(new File(new String(this.fileName)), this.encoding);
 	} catch (IOException e) {
-		// assume no content then
+		this.contents = CharOperation.NO_CHAR; // assume no source if asked again
+		throw new AbortCompilationUnit(null, e, this.encoding);
 	}
-	return CharOperation.NO_CHAR;
 }
 /**
  * @see org.eclipse.jdt.internal.compiler.env.IDependent#getFileName()
