@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,6 +11,7 @@
 package org.eclipse.jdt.internal.compiler.lookup;
 
 import org.eclipse.jdt.internal.compiler.util.HashtableOfObject;
+import org.eclipse.jdt.internal.compiler.util.SimpleSet;
 
 class MethodVerifier15 extends MethodVerifier {
 
@@ -41,30 +42,7 @@ boolean areParametersEqual(MethodBinding one, MethodBinding two) {
 	return true;
 }
 boolean areReturnTypesEqual(MethodBinding one, MethodBinding substituteTwo) {
-	if (one.returnType == substituteTwo.returnType) return true;
-
-	// short is compatible with int, but as far as covariance is concerned, its not
-	if (one.returnType.isBaseType()) return false;
-
-	if (!one.declaringClass.isInterface()) {
-		if (one.declaringClass.id == TypeIds.T_JavaLangObject)
-			return substituteTwo.returnType.isCompatibleWith(one.returnType); // interface methods inherit from Object
-		return one.returnType.isCompatibleWith(substituteTwo.returnType);
-	}
-
-	// check for methods from Object, every interface inherits from Object
-	if (substituteTwo.declaringClass.id == TypeIds.T_JavaLangObject)
-		return one.returnType.isCompatibleWith(substituteTwo.returnType);
-
-	// both are interfaces, see if they're related
-	if (one.declaringClass.implementsInterface(substituteTwo.declaringClass, true))
-		return one.returnType.isCompatibleWith(substituteTwo.returnType);
-	if (substituteTwo.declaringClass.implementsInterface(one.declaringClass, true))
-		return substituteTwo.returnType.isCompatibleWith(one.returnType);
-
-	// unrelated interfaces... one must be a subtype of the other
-	return one.returnType.isCompatibleWith(substituteTwo.returnType)
-		|| substituteTwo.returnType.isCompatibleWith(one.returnType);
+	return areReturnTypesCompatible(one, substituteTwo);
 }
 boolean areTypesEqual(TypeBinding one, TypeBinding two) {
 	if (one == two) return true;
@@ -84,7 +62,7 @@ boolean canSkipInheritedMethods() {
 	if (this.type.superclass() != null)
 		if (this.type.superclass().isAbstract() || this.type.superclass().isParameterizedType())
 			return false;
-	return this.type.superInterfaces() == NoSuperInterfaces;
+	return this.type.superInterfaces() == Binding.NO_SUPERINTERFACES;
 }
 boolean canSkipInheritedMethods(MethodBinding one, MethodBinding two) {
 	return two == null // already know one is not null
@@ -113,7 +91,8 @@ void checkConcreteInheritedMethod(MethodBinding concreteMethod, MethodBinding[]
 			}
 		}
 
-		this.type.addSyntheticBridgeMethod(originalInherited, concreteMethod.original());
+		if (concreteMethod.declaringClass.erasure().findSuperTypeWithSameErasure(originalInherited.declaringClass) == null)
+			this.type.addSyntheticBridgeMethod(originalInherited, concreteMethod.original());
 	}
 }
 void checkForBridgeMethod(MethodBinding currentMethod, MethodBinding inheritedMethod, MethodBinding[] allInheritedMethods) {
@@ -149,7 +128,7 @@ void checkForBridgeMethod(MethodBinding currentMethod, MethodBinding inheritedMe
 			MethodBinding substitute = computeSubstituteMethod(otherInheritedMethod, compareMethod);
 			if (substitute == null || doesSubstituteMethodOverride(compareMethod, substitute))
 				continue;
-			if (detectInheritedMethodClash(originalInherited, otherOriginal))
+			if (detectInheritedNameClash(originalInherited, otherOriginal))
 				return;
 		}
 	}
@@ -167,8 +146,9 @@ void checkForInheritedNameClash(MethodBinding inheritedMethod, MethodBinding oth
 	//		class Y { <T> void foo(T t) {} }
 	//		abstract class X extends Y implements I {}
 
-	if (!inheritedMethod.declaringClass.isInterface())
-		detectInheritedMethodClash(inheritedMethod, otherInheritedMethod);
+	if (inheritedMethod.declaringClass.isInterface() || inheritedMethod.isStatic()) return;
+
+	detectInheritedNameClash(inheritedMethod, otherInheritedMethod);
 }
 void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMethod) {
 	// sent from checkMethods() to compare a current method and an inherited method that are not 'equal'
@@ -203,7 +183,7 @@ void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMetho
 	//		class A implements I<Integer> { public void test(Integer i) {} }
 	//		class B extends A { public void test(Comparable i) {} }
 
-	if (currentMethod.declaringClass.isInterface()) return;
+	if (currentMethod.declaringClass.isInterface() || currentMethod.isStatic()) return;
 
 	if (!detectNameClash(currentMethod, inheritedMethod)) { // check up the hierarchy for skipped inherited methods
 		TypeBinding[] currentParams = currentMethod.parameters;
@@ -216,14 +196,13 @@ void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMetho
 				if (currentParams[i].isBaseType() != inheritedParams[i].isBaseType() || !inheritedParams[i].isCompatibleWith(currentParams[i]))
 					return; // no chance that another inherited method's bridge method can collide
 
-		ReferenceBinding[][] interfacesToVisit = new ReferenceBinding[3][];
-		int lastPosition = -1;
-		ReferenceBinding[] itsInterfaces = null;
+		ReferenceBinding[] interfacesToVisit = null;
+		int nextPosition = 0;
 		ReferenceBinding superType = inheritedMethod.declaringClass;
-		if ((itsInterfaces = superType.superInterfaces()) != NoSuperInterfaces) {
-			if (++lastPosition == interfacesToVisit.length)
-				System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[lastPosition * 2][], 0, lastPosition);
-			interfacesToVisit[lastPosition] = itsInterfaces;
+		ReferenceBinding[] itsInterfaces = superType.superInterfaces();
+		if (itsInterfaces != Binding.NO_SUPERINTERFACES) {
+			nextPosition = itsInterfaces.length;
+			interfacesToVisit = itsInterfaces;
 		}
 		superType = superType.superclass(); // now start with its superclass
 		while (superType != null && superType.isValidBinding()) {
@@ -233,29 +212,43 @@ void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMetho
 				if (substitute != null && !doesSubstituteMethodOverride(currentMethod, substitute) && detectNameClash(currentMethod, substitute))
 					return;
 			}
-			if ((itsInterfaces = superType.superInterfaces()) != NoSuperInterfaces) {
-				if (++lastPosition == interfacesToVisit.length)
-					System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[lastPosition * 2][], 0, lastPosition);
-				interfacesToVisit[lastPosition] = itsInterfaces;
+			if ((itsInterfaces = superType.superInterfaces()) != Binding.NO_SUPERINTERFACES) {
+				if (interfacesToVisit == null) {
+					interfacesToVisit = itsInterfaces;
+					nextPosition = interfacesToVisit.length;
+				} else {
+					int itsLength = itsInterfaces.length;
+					if (nextPosition + itsLength >= interfacesToVisit.length)
+						System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[nextPosition + itsLength + 5], 0, nextPosition);
+					nextInterface : for (int a = 0; a < itsLength; a++) {
+						ReferenceBinding next = itsInterfaces[a];
+						for (int b = 0; b < nextPosition; b++)
+							if (next == interfacesToVisit[b]) continue nextInterface;
+						interfacesToVisit[nextPosition++] = next;
+					}
+				}
 			}
 			superType = superType.superclass();
 		}
 
-		for (int i = 0; i <= lastPosition; i++) {
-			ReferenceBinding[] interfaces = interfacesToVisit[i];
-			for (int j = 0, l = interfaces.length; j < l; j++) {
-				superType = interfaces[j];
-				if (superType.isValidBinding()) {
-					MethodBinding[] methods = superType.getMethods(currentMethod.selector);
-					for (int m = 0, n = methods.length; m < n; m++){
-						MethodBinding substitute = computeSubstituteMethod(methods[m], currentMethod);
-						if (substitute != null && !doesSubstituteMethodOverride(currentMethod, substitute) && detectNameClash(currentMethod, substitute))
-							return;
-					}
-					if ((itsInterfaces = superType.superInterfaces()) != NoSuperInterfaces) {
-						if (++lastPosition == interfacesToVisit.length)
-							System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[lastPosition * 2][], 0, lastPosition);
-						interfacesToVisit[lastPosition] = itsInterfaces;
+		for (int i = 0; i < nextPosition; i++) {
+			superType = interfacesToVisit[i];
+			if (superType.isValidBinding()) {
+				MethodBinding[] methods = superType.getMethods(currentMethod.selector);
+				for (int m = 0, n = methods.length; m < n; m++){
+					MethodBinding substitute = computeSubstituteMethod(methods[m], currentMethod);
+					if (substitute != null && !doesSubstituteMethodOverride(currentMethod, substitute) && detectNameClash(currentMethod, substitute))
+						return;
+				}
+				if ((itsInterfaces = superType.superInterfaces()) != Binding.NO_SUPERINTERFACES) {
+					int itsLength = itsInterfaces.length;
+					if (nextPosition + itsLength >= interfacesToVisit.length)
+						System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[nextPosition + itsLength + 5], 0, nextPosition);
+					nextInterface : for (int a = 0; a < itsLength; a++) {
+						ReferenceBinding next = itsInterfaces[a];
+						for (int b = 0; b < nextPosition; b++)
+							if (next == interfacesToVisit[b]) continue nextInterface;
+						interfacesToVisit[nextPosition++] = next;
 					}
 				}
 			}
@@ -288,6 +281,26 @@ void checkInheritedMethods(MethodBinding[] methods, int length) {
 
 	super.checkInheritedMethods(methods, length);
 }
+boolean checkInheritedReturnTypes(MethodBinding[] methods, int length) {
+	if (methods[0].declaringClass.isClass())
+		return super.checkInheritedReturnTypes(methods, length);
+
+	// its possible in 1.5 that A is compatible with B & C, but B is not compatible with C
+	for (int i = 0, l = length - 1; i < l;) {
+		MethodBinding method = methods[i++];
+		for (int j = i; j <= l; j++) {
+			if (!areReturnTypesEqual(method, methods[j])) {
+				if (this.type.isInterface())
+					for (int m = length; --m >= 0;)
+						if (methods[m].declaringClass.id == TypeIds.T_JavaLangObject)
+							return false; // do not complain since the super interface already got blamed
+				problemReporter().inheritedMethodsHaveIncompatibleReturnTypes(this.type, methods, length);
+				return false;
+			}
+		}
+	}
+	return true;
+}
 void checkMethods() {
 	boolean mustImplementAbstractMethods = mustImplementAbstractMethods();
 	boolean skipInheritedMethods = mustImplementAbstractMethods && canSkipInheritedMethods(); // have a single concrete superclass so only check overridden methods
@@ -324,6 +337,9 @@ void checkMethods() {
 					}
 				}
 				if (index >= 0) {
+					// see addtional comments in https://bugs.eclipse.org/bugs/show_bug.cgi?id=122881
+					// if (index > 0 && currentMethod.declaringClass.isInterface()) // only check when inherited methods are from interfaces
+					//	checkInheritedReturnTypes(matchingInherited, index + 1);
 					checkAgainstInheritedMethods(currentMethod, matchingInherited, index + 1, inherited); // pass in the length of matching
 					while (index >= 0) matchingInherited[index--] = null; // clear the contents of the matching methods
 				}
@@ -407,7 +423,7 @@ MethodBinding computeSubstituteMethod(MethodBinding inheritedMethod, MethodBindi
 		((BinaryTypeBinding) inheritedMethod.declaringClass).resolveTypesFor(inheritedMethod);
 
 	TypeVariableBinding[] inheritedTypeVariables = inheritedMethod.typeVariables;
-	if (inheritedTypeVariables == NoTypeVariables) return inheritedMethod;
+	if (inheritedTypeVariables == Binding.NO_TYPE_VARIABLES) return inheritedMethod;
 	int inheritedLength = inheritedTypeVariables.length;
 	TypeVariableBinding[] typeVariables = currentMethod.typeVariables;
 	int length = typeVariables.length;
@@ -421,7 +437,7 @@ MethodBinding computeSubstituteMethod(MethodBinding inheritedMethod, MethodBindi
 			arguments[i] = inheritedTypeVariables[i].upperBound();
 	}
 	ParameterizedGenericMethodBinding substitute =
-		new ParameterizedGenericMethodBinding(inheritedMethod, arguments, this.environment);
+		this.environment.createParameterizedGenericMethod(inheritedMethod, arguments);
 
 	// interface I { <T> void foo(T t); }
 	// class X implements I { public <T extends I> void foo(T t) {} }
@@ -431,12 +447,20 @@ MethodBinding computeSubstituteMethod(MethodBinding inheritedMethod, MethodBindi
 		TypeBinding argument = arguments[i];
 		if (argument instanceof TypeVariableBinding) {
 			TypeVariableBinding typeVariable = (TypeVariableBinding) argument;
-			if (Scope.substitute(substitute, inheritedTypeVariable.superclass) != typeVariable.superclass)  
+			if (typeVariable.firstBound == inheritedTypeVariable.firstBound) {
+				if (typeVariable.firstBound == null)
+					continue; // both are null
+			} else if (typeVariable.firstBound != null && inheritedTypeVariable.firstBound != null) {
+				if (typeVariable.firstBound.isClass() != inheritedTypeVariable.firstBound.isClass())
+					return inheritedMethod; // not a match
+			}
+			if (Scope.substitute(substitute, inheritedTypeVariable.superclass) != typeVariable.superclass)
 				return inheritedMethod; // not a match
 			int interfaceLength = inheritedTypeVariable.superInterfaces.length;
 			ReferenceBinding[] interfaces = typeVariable.superInterfaces;
 			if (interfaceLength != interfaces.length)
 				return inheritedMethod; // not a match
+			// TODO (kent) another place where we expect the superinterfaces to be in the exact same order
 			next : for (int j = 0; j < interfaceLength; j++) {
 				TypeBinding superType = Scope.substitute(substitute, inheritedTypeVariable.superInterfaces[j]);
 				for (int k = 0; k < interfaceLength; k++)
@@ -444,28 +468,24 @@ MethodBinding computeSubstituteMethod(MethodBinding inheritedMethod, MethodBindi
 						continue next;
 				return inheritedMethod; // not a match
 			}
-		} else if (inheritedTypeVariables[i].boundCheck(substitute, argument) != TypeConstants.OK) {
-	    	return inheritedMethod;
+		} else if (inheritedTypeVariable.boundCheck(substitute, argument) != TypeConstants.OK) {
+	    		return inheritedMethod;
 		}
 	}
    return substitute;
 }
-boolean detectInheritedMethodClash(MethodBinding inherited, MethodBinding otherInherited) {
+boolean detectInheritedNameClash(MethodBinding inherited, MethodBinding otherInherited) {
 	if (!inherited.areParameterErasuresEqual(otherInherited) || inherited.returnType.erasure() != otherInherited.returnType.erasure()) return false;
-	if (doTypeVariablesClash(inherited, otherInherited) || doParametersClash(inherited, otherInherited)) {
-		problemReporter().inheritedMethodsHaveNameClash(this.type, inherited, otherInherited);
-		return true;
-	}
-	return false;
+
+	problemReporter().inheritedMethodsHaveNameClash(this.type, inherited, otherInherited);
+	return true;
 }
 boolean detectNameClash(MethodBinding current, MethodBinding inherited) {
 	MethodBinding original = inherited.original(); // can be the same as inherited
 	if (!current.areParameterErasuresEqual(original) || current.returnType.erasure() != original.returnType.erasure()) return false;
-	if (doTypeVariablesClash(current, inherited) || doParametersClash(current, original)) {
-		problemReporter(current).methodNameClash(current, original);
-		return true;
-	}
-	return false;
+
+	problemReporter(current).methodNameClash(current, original);
+	return true;
 }
 public boolean doesMethodOverride(MethodBinding method, MethodBinding inheritedMethod) {
 	MethodBinding substitute = computeSubstituteMethod(inheritedMethod, method);
@@ -483,41 +503,22 @@ boolean doesSubstituteMethodOverride(MethodBinding method, MethodBinding substit
 		return false;
 
 	// also allow a method such as Number foo(Number) to override <U> T foo(T) where T extends Number
-	if (method.typeVariables != NoTypeVariables || !substituteMethod.hasSubstitutedParameters())
-		return false;
-	if (method.declaringClass.findSuperTypeWithSameErasure(substituteMethod.declaringClass) == null)
+	if (method.typeVariables != Binding.NO_TYPE_VARIABLES || !substituteMethod.hasSubstitutedParameters())
 		return false;
 
-	for (int i = 0; i < length; i++)
-		if (params[i] != inheritedParams[i].erasure())
-			return false;
-	return true;
-}
-boolean doParametersClash(MethodBinding one, MethodBinding substituteTwo) {
-	// must check each parameter pair to see if parameterized types are compatible
-	TypeBinding[] oneParams = one.parameters;
-	TypeBinding[] twoParams = substituteTwo.parameters;
-	for (int i = 0, l = oneParams.length; i < l; i++) {
-		if (oneParams[i] == twoParams[i]) continue;
-		switch (oneParams[i].leafComponentType().kind()) {
-			case Binding.PARAMETERIZED_TYPE :
-				if (!twoParams[i].leafComponentType().isParameterizedType()
-					|| !oneParams[i].isEquivalentTo(twoParams[i])
-					|| !twoParams[i].isEquivalentTo(oneParams[i])) {
-						return true;
-				}
-				break;
-			case Binding.TYPE_PARAMETER :
-				return true; // type variables must be identical (due to substitution) given their erasures are equal
+	for (int i = 0; i < length; i++) {
+		if (inheritedParams[i].kind() == Binding.TYPE_PARAMETER) {
+			if (params[i] != ((TypeVariableBinding) inheritedParams[i]).upperBound())
+				return false;
+		} else if (params[i] != inheritedParams[i]) {
+			return false;			
 		}
-		if (twoParams[i].leafComponentType().isTypeVariable())
-			return true; // type variables must be identical (due to substitution) given their erasures are equal
 	}
-	return false;
+	return true;
 }
 boolean doTypeVariablesClash(MethodBinding one, MethodBinding substituteTwo) {
 	// one has type variables and substituteTwo did not pass bounds check in computeSubstituteMethod()
-	return one.typeVariables != NoTypeVariables && !(substituteTwo instanceof ParameterizedGenericMethodBinding);
+	return one.typeVariables != Binding.NO_TYPE_VARIABLES && !(substituteTwo instanceof ParameterizedGenericMethodBinding);
 }
 boolean isInterfaceMethodImplemented(MethodBinding inheritedMethod, MethodBinding existingMethod, ReferenceBinding superType) {
 	if (inheritedMethod.original() != inheritedMethod && existingMethod.declaringClass.isInterface())
@@ -528,6 +529,84 @@ boolean isInterfaceMethodImplemented(MethodBinding inheritedMethod, MethodBindin
 		&& inheritedMethod.returnType == existingMethod.returnType
 		&& super.isInterfaceMethodImplemented(inheritedMethod, existingMethod, superType);
 }
+SimpleSet findSuperinterfaceCollisions(ReferenceBinding superclass, ReferenceBinding[] superInterfaces) {
+	ReferenceBinding[] interfacesToVisit = null;
+	int nextPosition = 0;
+	ReferenceBinding[] itsInterfaces = superInterfaces;
+	if (itsInterfaces != Binding.NO_SUPERINTERFACES) {
+		nextPosition = itsInterfaces.length;
+		interfacesToVisit = itsInterfaces;
+	}
+
+	boolean isInconsistent = this.type.isHierarchyInconsistent();
+	ReferenceBinding superType = superclass;
+	while (superType != null && superType.isValidBinding()) {
+		isInconsistent |= superType.isHierarchyInconsistent();
+		if ((itsInterfaces = superType.superInterfaces()) != Binding.NO_SUPERINTERFACES) {
+			if (interfacesToVisit == null) {
+				interfacesToVisit = itsInterfaces;
+				nextPosition = interfacesToVisit.length;
+			} else {
+				int itsLength = itsInterfaces.length;
+				if (nextPosition + itsLength >= interfacesToVisit.length)
+					System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[nextPosition + itsLength + 5], 0, nextPosition);
+				nextInterface : for (int a = 0; a < itsLength; a++) {
+					ReferenceBinding next = itsInterfaces[a];
+					for (int b = 0; b < nextPosition; b++)
+						if (next == interfacesToVisit[b]) continue nextInterface;
+					interfacesToVisit[nextPosition++] = next;
+				}
+			}
+		}
+		superType = superType.superclass();
+	}
+
+	for (int i = 0; i < nextPosition; i++) {
+		superType = interfacesToVisit[i];
+		if (superType.isValidBinding()) {
+			isInconsistent |= superType.isHierarchyInconsistent();
+			if ((itsInterfaces = superType.superInterfaces()) != Binding.NO_SUPERINTERFACES) {
+				int itsLength = itsInterfaces.length;
+				if (nextPosition + itsLength >= interfacesToVisit.length)
+					System.arraycopy(interfacesToVisit, 0, interfacesToVisit = new ReferenceBinding[nextPosition + itsLength + 5], 0, nextPosition);
+				nextInterface : for (int a = 0; a < itsLength; a++) {
+					ReferenceBinding next = itsInterfaces[a];
+					for (int b = 0; b < nextPosition; b++)
+						if (next == interfacesToVisit[b]) continue nextInterface;
+					interfacesToVisit[nextPosition++] = next;
+				}
+			}
+		}
+	}
+
+	if (!isInconsistent) return null; // hierarchy is consistent so no collisions are possible
+	SimpleSet copy = null;
+	for (int i = 0; i < nextPosition; i++) {
+		ReferenceBinding current = interfacesToVisit[i];
+		if (current.isValidBinding()) {
+			TypeBinding erasure = current.erasure();
+			for (int j = i + 1; j < nextPosition; j++) {
+				ReferenceBinding next = interfacesToVisit[j];
+				if (next.isValidBinding() && next.erasure() == erasure) {
+					if (copy == null)
+						copy = new SimpleSet(nextPosition);
+					copy.add(interfacesToVisit[i]);
+					copy.add(interfacesToVisit[j]);
+				}
+			}
+		}
+	}
+	return copy;
+}
+boolean reportIncompatibleReturnTypeError(MethodBinding currentMethod, MethodBinding inheritedMethod) {
+	if (currentMethod.typeVariables == Binding.NO_TYPE_VARIABLES
+		&& inheritedMethod.original().typeVariables != Binding.NO_TYPE_VARIABLES
+		&& currentMethod.returnType.erasure().findSuperTypeWithSameErasure(inheritedMethod.returnType.erasure()) != null) {
+			problemReporter(currentMethod).unsafeReturnTypeOverride(currentMethod, inheritedMethod, this.type);
+			return false;
+	}
+	return super.reportIncompatibleReturnTypeError(currentMethod, inheritedMethod);
+}
 void verify(SourceTypeBinding someType) {
 	if (someType.isAnnotationType())
 		someType.detectAnnotationCycle();
@@ -537,7 +616,7 @@ void verify(SourceTypeBinding someType) {
 	for (int i = someType.typeVariables.length; --i >= 0;) {
 		TypeVariableBinding var = someType.typeVariables[i];
 		// must verify bounds if the variable has more than 1
-		if (var.superInterfaces == NoSuperInterfaces) continue;
+		if (var.superInterfaces == Binding.NO_SUPERINTERFACES) continue;
 		if (var.superInterfaces.length == 1 && var.superclass.id == TypeIds.T_JavaLangObject) continue;
 
 		this.currentMethods = new HashtableOfObject(0);
