@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -74,6 +74,18 @@ public interface ITypeBinding extends IBinding {
 	 */
 	public ITypeBinding getBound();
 
+	/**
+	 * Returns the binding representing the component type of this array type,
+	 * or <code>null</code> if this is not an array type binding. The component
+	 * type of an array might be an array type.
+	 * <p>This is subject to change before 3.2 release.</p>
+	 *
+	 * @return the component type binding, or <code>null</code> if this is
+	 *   not an array type
+	 * @since 3.2
+	 */	
+	public ITypeBinding getComponentType();
+
 	/**
 	 * Returns a list of bindings representing all the fields declared
 	 * as members of this class, interface, or enum type. These include public, 
@@ -437,15 +449,14 @@ public interface ITypeBinding extends IBinding {
 	 * <p>
 	 * Note that type arguments only occur on a type binding that represents
 	 * an instance of a generic type corresponding to a parameterized type
-	 * reference (e.g., <code>Collection&lt;String&gt;</code>) or to a raw
-	 * type reference (e.g., <code>Collection</code>) to a generic type.
+	 * reference (e.g., <code>Collection&lt;String&gt;</code>).
 	 * Do not confuse these with type parameters which only occur on the
 	 * type binding corresponding directly to the declaration of the 
 	 * generic class or interface (e.g., <code>Collection&lt;T&gt;</code>).
 	 * </p> 
 	 *
 	 * @return the list of type bindings for the type arguments used to
-	 * instantiate the corrresponding generic type, or otherwise the empty list
+	 * instantiate the corresponding generic type, or otherwise the empty list
 	 * @see #getTypeDeclaration()
 	 * @see #isGenericType()
 	 * @see #isParameterizedType()
@@ -461,6 +472,7 @@ public interface ITypeBinding extends IBinding {
      * Note that per construction, it can only contain one class or array type, 
      * at most, and then it is located in first position.
      * </p>
+     * <p>
      * Also note that array type bound may only occur in the case of a capture
      * binding, e.g. <code>capture-of ? extends Object[]</code>
      * </p>
@@ -475,10 +487,16 @@ public interface ITypeBinding extends IBinding {
 	
 	/**
 	 * Returns the binding for the type declaration corresponding to this type
-	 * binding. For parameterized types ({@link #isParameterizedType()})
-	 * and raw types ({@link #isRawType()}), this method returns the binding
-	 * for the corresponding generic type. For other type bindings, this
-	 * returns the same binding.
+	 * binding.
+	 * <p>For parameterized types ({@link #isParameterizedType()})
+	 * and most raw types ({@link #isRawType()}), this method returns the binding
+	 * for the corresponding generic type.</p>
+	 * <p>For raw member types ({@link #isRawType()}, {@link #isMember()})
+	 * of a raw declaring class, the type declaration is a generic or a non-generic
+	 * type.</p>
+	 * <p>A different non-generic binding will be returned when one of the declaring
+	 * types/methods was parameterized.</p>
+	 * <p>For other type bindings, this returns the same binding.</p>
 	 *
 	 * @return the type binding
 	 * @since 3.1
@@ -550,16 +568,16 @@ public interface ITypeBinding extends IBinding {
 	public boolean isArray();
 	
 	/**
-	 * Returns whether this type is assigment compatible with the given type,
-	 * as specified in section 5.2 of <em>The Java Language 
+	 * Returns whether an expression of this type can be assigned to a variable
+	 * of the given type, as specified in section 5.2 of <em>The Java Language 
 	 * Specification, Third Edition</em> (JLS3).
 	 * 
-	 * @param type the type to check compatibility against
-	 * @return <code>true</code> if this type is assigment compatible with the
-	 * given type, and <code>false</code> otherwise
+	 * @param variableType the type of a variable to check compatibility against
+	 * @return <code>true</code> if an expression of this type can be assigned to a
+	 *   variable of the given type, and <code>false</code> otherwise
 	 * @since 3.1
 	 */
-	public boolean isAssignmentCompatible(ITypeBinding type);
+	public boolean isAssignmentCompatible(ITypeBinding variableType);
 	
 	/**
 	 * Returns whether this type binding represents a capture binding.
@@ -738,6 +756,7 @@ public interface ITypeBinding extends IBinding {
 	 * class <code>java.lang.String</code> and whose erasure is the type
 	 * binding for the generic type <code>java.util.Collection</code>.
 	 * </p>
+	 * <p>
 	 * Note that {@link #isGenericType()},
 	 * {@link #isParameterizedType()},
 	 * and {@link #isRawType()} are mutually exclusive.
@@ -860,7 +879,7 @@ public interface ITypeBinding extends IBinding {
 	 * For example, a AST type like 
 	 * <code>Collection&lt;? extends Object&gt;</code> typically resolves to a
 	 * parameterized type binding whose type argument is a wildcard type
-	 * with upper type bound <code>java.util.Object/code>.
+	 * with upper type bound <code>java.util.Object</code>.
 	 * </p>
 	 *
 	 * @return <code>true</code> if this object represents a wildcard type,
