@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -30,11 +30,10 @@ import org.eclipse.jdt.core.JavaModelException;
 import org.eclipse.jdt.core.compiler.CharOperation;
 import org.eclipse.jdt.internal.compiler.ast.Expression;
 import org.eclipse.jdt.internal.compiler.ast.Wildcard;
-import org.eclipse.jdt.internal.compiler.env.IConstants;
+import org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.eclipse.jdt.internal.compiler.env.IDependent;
 import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
 import org.eclipse.jdt.internal.compiler.lookup.BaseTypeBinding;
-import org.eclipse.jdt.internal.compiler.lookup.BaseTypes;
 import org.eclipse.jdt.internal.compiler.lookup.Binding;
 import org.eclipse.jdt.internal.compiler.lookup.CaptureBinding;
 import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
@@ -48,6 +47,7 @@ import org.eclipse.jdt.internal.compiler.lookup.Scope;
 import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
 import org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding;
 import org.eclipse.jdt.internal.compiler.lookup.WildcardBinding;
+import org.eclipse.jdt.internal.compiler.problem.AbortCompilation;
 import org.eclipse.jdt.internal.compiler.util.SuffixConstants;
 import org.eclipse.jdt.internal.compiler.util.Util;
 import org.eclipse.jdt.internal.core.ClassFile;
@@ -75,6 +75,22 @@ class TypeBinding implements ITypeBinding {
 		this.resolver = resolver;
 	}
 
+	public IAnnotationBinding[] getAnnotations() { 
+		IAnnotationBinding[] domInstances = AnnotationBinding.NoAnnotations;
+		if (this.binding.isAnnotationType() || this.binding.isClass() || this.binding.isEnum() || this.binding.isInterface()) {
+			org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding refType = 
+				(org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding) this.binding;
+			org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding[] internalAnnotations = refType.getAnnotations();
+			int length = internalAnnotations == null ? 0 : internalAnnotations.length;
+			if (length > 0) {
+				domInstances = new AnnotationBinding[length];
+				for (int i = 0; i < length; i++)
+					domInstances[i] = this.resolver.getAnnotationInstance(internalAnnotations[i]);
+			}
+		}
+		return domInstances;
+	}
+
 	/*
 	 * @see ITypeBinding#getBinaryName()
 	 * @since 3.0
@@ -105,17 +121,17 @@ class TypeBinding implements ITypeBinding {
 	 */
 	private IClassFile getClassFile(char[] fileName) {
 		int jarSeparator = CharOperation.indexOf(IDependent.JAR_FILE_ENTRY_SEPARATOR, fileName);
-		int lastSlash = CharOperation.lastIndexOf('/', fileName);
-		if (lastSlash == -1) 
-			lastSlash = CharOperation.lastIndexOf(File.separatorChar, fileName);
-		if (jarSeparator != -1 && lastSlash < jarSeparator) // if in a jar and no slash, it is a default package -> lastSlash should be jarSeparator+1
-			lastSlash = jarSeparator+1;
-		if (lastSlash == -1)
+		int pkgEnd = CharOperation.lastIndexOf('/', fileName); // pkgEnd is exclusive
+		if (pkgEnd == -1) 
+			pkgEnd = CharOperation.lastIndexOf(File.separatorChar, fileName);
+		if (jarSeparator != -1 && pkgEnd < jarSeparator) // if in a jar and no slash, it is a default package -> pkgEnd should be equal to jarSeparator
+			pkgEnd = jarSeparator;
+		if (pkgEnd == -1)
 			return null;
-		IPackageFragment pkg = getPackageFragment(fileName, lastSlash, jarSeparator);
+		IPackageFragment pkg = getPackageFragment(fileName, pkgEnd, jarSeparator);
 		if (pkg == null) return null;
-		int start = lastSlash == jarSeparator+1 ? lastSlash : lastSlash+1;
-		return pkg.getClassFile(new String(fileName, start, fileName.length - start));
+		int start;
+		return pkg.getClassFile(new String(fileName, start = pkgEnd + 1, fileName.length - start));
 	}
 	
 	/*
@@ -124,12 +140,13 @@ class TypeBinding implements ITypeBinding {
 	 */
 	private ICompilationUnit getCompilationUnit(char[] fileName) {
 		char[] slashSeparatedFileName = CharOperation.replaceOnCopy(fileName, File.separatorChar, '/');
-		int lastSlash = CharOperation.lastIndexOf('/', slashSeparatedFileName);
-		if (lastSlash == -1) return null;
-		IPackageFragment pkg = getPackageFragment(slashSeparatedFileName, lastSlash, -1/*no jar separator for .java files*/);
+		int pkgEnd = CharOperation.lastIndexOf('/', slashSeparatedFileName); // pkgEnd is exclusive
+		if (pkgEnd == -1)
+			return null;
+		IPackageFragment pkg = getPackageFragment(slashSeparatedFileName, pkgEnd, -1/*no jar separator for .java files*/);
 		if (pkg == null) return null;
 		int start;
-		ICompilationUnit cu = pkg.getCompilationUnit(new String(slashSeparatedFileName, start =  lastSlash+1, slashSeparatedFileName.length - start));
+		ICompilationUnit cu = pkg.getCompilationUnit(new String(slashSeparatedFileName, start =  pkgEnd+1, slashSeparatedFileName.length - start));
 		if (this.resolver instanceof DefaultBindingResolver) {
 			ICompilationUnit workingCopy = cu.findWorkingCopy(((DefaultBindingResolver) this.resolver).workingCopyOwner);
 			if (workingCopy != null) 
@@ -138,6 +155,17 @@ class TypeBinding implements ITypeBinding {
 		return cu;
 	}
 
+	/*
+	 * @see ITypeBinding#getComponentType()
+	 */
+	public ITypeBinding getComponentType() {
+		if (!this.isArray()) {
+			return null;
+		}
+		ArrayBinding arrayBinding = (ArrayBinding) binding;
+		return resolver.getTypeBinding(arrayBinding.elementsType());
+	}
+
 	/*
 	 * @see ITypeBinding#getDeclaredFields()
 	 */
@@ -404,32 +432,30 @@ class TypeBinding implements ITypeBinding {
 			// local or anonymous type
 			if (Util.isClassFileName(fileName)) {
 				int jarSeparator = CharOperation.indexOf(IDependent.JAR_FILE_ENTRY_SEPARATOR, fileName);
-				int lastSlash = CharOperation.lastIndexOf('/', fileName);
-				if (lastSlash == -1) 
-					lastSlash = CharOperation.lastIndexOf(File.separatorChar, fileName);
-				if (jarSeparator != -1 && lastSlash < jarSeparator) // if in a jar and no slash, it is a default package -> lastSlash should be jarSeparator+1
-					lastSlash = jarSeparator+1;
-				if (lastSlash == -1)
+				int pkgEnd = CharOperation.lastIndexOf('/', fileName); // pkgEnd is exclusive
+				if (pkgEnd == -1) 
+					pkgEnd = CharOperation.lastIndexOf(File.separatorChar, fileName);
+				if (jarSeparator != -1 && pkgEnd < jarSeparator) // if in a jar and no slash, it is a default package -> pkgEnd should be equal to jarSeparator
+					pkgEnd = jarSeparator;
+				if (pkgEnd == -1)
 					return null;
-				IPackageFragment pkg = getPackageFragment(fileName, lastSlash, jarSeparator);
+				IPackageFragment pkg = getPackageFragment(fileName, pkgEnd, jarSeparator);
 				char[] constantPoolName = referenceBinding.constantPoolName();
 				if (constantPoolName == null) {
 					ClassFile classFile = (ClassFile) getClassFile(fileName);
 					return classFile == null ? null : (JavaElement) classFile.getType();
 				}
-				lastSlash = CharOperation.lastIndexOf('/', constantPoolName);
-				char[] classFileName = CharOperation.subarray(constantPoolName, lastSlash+1, constantPoolName.length);
+				pkgEnd = CharOperation.lastIndexOf('/', constantPoolName);
+				char[] classFileName = CharOperation.subarray(constantPoolName, pkgEnd+1, constantPoolName.length);
 				ClassFile classFile = (ClassFile) pkg.getClassFile(new String(classFileName) + SuffixConstants.SUFFIX_STRING_class);
 				return (JavaElement) classFile.getType();
 			}
 			ICompilationUnit cu = getCompilationUnit(fileName);
 			if (cu == null) return null;
-			if (!(this.resolver instanceof DefaultBindingResolver)) return null;
-			DefaultBindingResolver bindingResolver = (DefaultBindingResolver) this.resolver;
-			ASTNode node = (ASTNode) bindingResolver.bindingsToAstNodes.get(this);
 			// must use getElementAt(...) as there is no back pointer to the defining method (scope is null after resolution has ended)
 			try {
-				return (JavaElement) cu.getElementAt(node.getStartPosition());
+				int sourceStart = ((LocalTypeBinding) referenceBinding).sourceStart;
+				return (JavaElement) cu.getElementAt(sourceStart);
 			} catch (JavaModelException e) {
 				// does not exist
 				return null;
@@ -503,17 +529,17 @@ class TypeBinding implements ITypeBinding {
 			ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
 			final int accessFlags = referenceBinding.getAccessFlags() & VALID_MODIFIERS;
 			// clear the AccAbstract, AccAnnotation and the AccInterface bits
-			return accessFlags & ~(IConstants.AccAbstract | IConstants.AccInterface | IConstants.AccAnnotation);			
+			return accessFlags & ~(ClassFileConstants.AccAbstract | ClassFileConstants.AccInterface | ClassFileConstants.AccAnnotation);			
 		} else if (isInterface()) {
 			ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
 			final int accessFlags = referenceBinding.getAccessFlags() & VALID_MODIFIERS;
 			// clear the AccAbstract and the AccInterface bits
-			return accessFlags & ~(IConstants.AccAbstract | IConstants.AccInterface);
+			return accessFlags & ~(ClassFileConstants.AccAbstract | ClassFileConstants.AccInterface);
 		} else if (isEnum()) {
 			ReferenceBinding referenceBinding = (ReferenceBinding) this.binding;
 			final int accessFlags = referenceBinding.getAccessFlags() & VALID_MODIFIERS;
 			// clear the AccEnum bits
-			return accessFlags & ~IConstants.AccEnum;
+			return accessFlags & ~ClassFileConstants.AccEnum;
 		} else {
 			return 0;
 		}
@@ -611,17 +637,23 @@ class TypeBinding implements ITypeBinding {
 	
 	/*
 	 * Returns the package that includes the given file name, or null if not found.
+	 * pkgEnd == jarSeparator if default package in a jar
+	 * pkgEnd > jarSeparator if non default package in a jar
+	 * pkgEnd > 0 if package not in a jar
+	 * 
 	 * @see org.eclipse.jdt.internal.compiler.env.IDependent#getFileName()
 	 */
-	private IPackageFragment getPackageFragment(char[] fileName, int lastSlash, int jarSeparator) {
+	private IPackageFragment getPackageFragment(char[] fileName, int pkgEnd, int jarSeparator) {
 		if (jarSeparator != -1) {
 			String jarMemento = new String(fileName, 0, jarSeparator);
 			IPackageFragmentRoot root = (IPackageFragmentRoot) JavaCore.create(jarMemento);
-			char[] pkgName = CharOperation.subarray(fileName, jarSeparator+1, lastSlash);
+			if (pkgEnd == jarSeparator)
+				return root.getPackageFragment(IPackageFragment.DEFAULT_PACKAGE_NAME);
+			char[] pkgName = CharOperation.subarray(fileName, jarSeparator+1, pkgEnd);
 			CharOperation.replace(pkgName, '/', '.');
 			return root.getPackageFragment(new String(pkgName));
 		} else {
-			Path path = new Path(new String(fileName, 0, lastSlash));
+			Path path = new Path(new String(fileName, 0, pkgEnd));
 			IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();
 			IContainer folder = path.segmentCount() == 1 ? workspaceRoot.getProject(path.lastSegment()) : (IContainer) workspaceRoot.getFolder(path);
 			IJavaElement element = JavaCore.create(folder);
@@ -848,6 +880,11 @@ class TypeBinding implements ITypeBinding {
 	 * @see org.eclipse.jdt.core.dom.ITypeBinding#getTypeParameters()
 	 */
 	public ITypeBinding[] getTypeParameters() {
+		switch(this.binding.kind()) {
+			case Binding.RAW_TYPE :
+			case Binding.PARAMETERIZED_TYPE :
+				return NO_TYPE_BINDINGS;
+		}
 		TypeVariableBinding[] typeVariableBindings = this.binding.typeVariables();
 		if (typeVariableBindings != null) {
 			int typeVariableBindingsLength = typeVariableBindings.length;
@@ -916,11 +953,17 @@ class TypeBinding implements ITypeBinding {
 	 * @see ITypeBinding#isAssignmentCompatible(ITypeBinding)
 	 */
 	public boolean isAssignmentCompatible(ITypeBinding type) {
-		if (this == type) return true;
-		TypeBinding other = (TypeBinding) type;
-		Scope scope = this.resolver.scope();
-		if (scope == null) return false;
-		return this.binding.isCompatibleWith(other.binding) || scope.isBoxingCompatibleWith(this.binding, other.binding);
+		try {
+			if (this == type) return true;
+			TypeBinding other = (TypeBinding) type;
+			Scope scope = this.resolver.scope();
+			if (scope == null) return false;
+			return this.binding.isCompatibleWith(other.binding) || scope.isBoxingCompatibleWith(this.binding, other.binding);
+		} catch (AbortCompilation e) {
+			// don't surface internal exception to clients
+			// see https://bugs.eclipse.org/bugs/show_bug.cgi?id=143013
+			return false;
+		}
 	}
 	
 	/* (non-Javadoc)
@@ -934,17 +977,23 @@ class TypeBinding implements ITypeBinding {
 	 * @see ITypeBinding#isCastCompatible(ITypeBinding)
 	 */
 	public boolean isCastCompatible(ITypeBinding type) {
-		Expression expression = new Expression() {
-			public StringBuffer printExpression(int indent,StringBuffer output) {
-				return null;
-			}
-		};
-		Scope scope = this.resolver.scope();
-		if (scope == null) return false;
-		org.eclipse.jdt.internal.compiler.lookup.TypeBinding expressionType = ((TypeBinding) type).binding;
-		// simulate capture in case checked binding did not properly get extracted from a reference
-		expressionType = expressionType.capture(scope, 0);
-		return expression.checkCastTypesCompatibility(scope, this.binding, expressionType, null);
+		try {
+			Expression expression = new Expression() {
+				public StringBuffer printExpression(int indent,StringBuffer output) {
+					return null;
+				}
+			};
+			Scope scope = this.resolver.scope();
+			if (scope == null) return false;
+			org.eclipse.jdt.internal.compiler.lookup.TypeBinding expressionType = ((TypeBinding) type).binding;
+			// simulate capture in case checked binding did not properly get extracted from a reference
+			expressionType = expressionType.capture(scope, 0);
+			return expression.checkCastTypesCompatibility(scope, this.binding, expressionType, null);
+		} catch (AbortCompilation e) {
+			// don't surface internal exception to clients
+			// see https://bugs.eclipse.org/bugs/show_bug.cgi?id=143013
+			return false;
+		}
 	}
 
 	/*
@@ -1081,7 +1130,7 @@ class TypeBinding implements ITypeBinding {
 	 * @see ITypeBinding#isNullType()
 	 */
 	public boolean isNullType() {
-		return this.binding == BaseTypes.NullBinding;
+		return this.binding == org.eclipse.jdt.internal.compiler.lookup.TypeBinding.NULL;
 	}
 
 	/* (non-Javadoc)
@@ -1109,11 +1158,17 @@ class TypeBinding implements ITypeBinding {
 	 * @see ITypeBinding#isSubTypeCompatible(ITypeBinding)
 	 */
 	public boolean isSubTypeCompatible(ITypeBinding type) {
-		if (this == type) return true;
-		if (this.binding.isBaseType()) return false;
-		TypeBinding other = (TypeBinding) type;
-		if (other.binding.isBaseType()) return false;
-		return this.binding.isCompatibleWith(other.binding);
+		try {
+			if (this == type) return true;
+			if (this.binding.isBaseType()) return false;
+			TypeBinding other = (TypeBinding) type;
+			if (other.binding.isBaseType()) return false;
+			return this.binding.isCompatibleWith(other.binding);
+		} catch (AbortCompilation e) {
+			// don't surface internal exception to clients
+			// see https://bugs.eclipse.org/bugs/show_bug.cgi?id=143013
+			return false;
+		}
 	}
 	
 	/**
@@ -1158,7 +1213,7 @@ class TypeBinding implements ITypeBinding {
 	private boolean shouldBeRemoved(org.eclipse.jdt.internal.compiler.lookup.MethodBinding methodBinding) {
 		return methodBinding.isDefaultAbstract() || methodBinding.isSynthetic() || (methodBinding.isConstructor() && isInterface());
 	}
-	
+
 	/* 
 	 * For debugging purpose only.
 	 * @see java.lang.Object#toString()
