@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -42,10 +42,6 @@ public class PackageFragmentRoot extends Openable implements IPackageFragmentRoo
 	 * No source attachment property
 	 */
 	protected final static String NO_SOURCE_ATTACHMENT = ""; //$NON-NLS-1$
-	/*
-	 * No source mapper singleton
-	 */
-	protected final static SourceMapper NO_SOURCE_MAPPER = new SourceMapper();
 
 	/**
 	 * The resource associated with this root.
@@ -248,15 +244,14 @@ protected void computeFolderChildren(IContainer folder, boolean isIncluded, Stri
 			switch(member.getType()) {
 			    
 			    case IResource.FOLDER:
+					// recurse into sub folders even even parent not included as a sub folder could be included
+					// (see https://bugs.eclipse.org/bugs/show_bug.cgi?id=65637)
 					if (Util.isValidFolderNameForPackage(memberName)) {
-					    boolean isMemberIncluded = !Util.isExcluded(member, inclusionPatterns, exclusionPatterns);
-						// keep looking inside as long as included already, or may have child included due to inclusion patterns
-					    if (isMemberIncluded || inclusionPatterns != null) { 
-							// eliminate binary output only if nested inside direct subfolders
-							if (javaProject.contains(member)) {
-								String[] newNames = Util.arrayConcat(pkgName, manager.intern(memberName));
-								computeFolderChildren((IFolder) member, isMemberIncluded, newNames, vChildren, inclusionPatterns, exclusionPatterns);
-							}
+						// eliminate binary output only if nested inside direct subfolders
+						if (javaProject.contains(member)) {
+							String[] newNames = Util.arrayConcat(pkgName, manager.intern(memberName));
+							boolean isMemberIncluded = !Util.isExcluded(member, inclusionPatterns, exclusionPatterns);
+							computeFolderChildren((IFolder) member, isMemberIncluded, newNames, vChildren, inclusionPatterns, exclusionPatterns);
 						}
 					}
 			    	break;
@@ -715,28 +710,23 @@ public SourceMapper getSourceMapper() {
 		if (mapper == null) {
 			// first call to this method
 			IPath sourcePath= getSourceAttachmentPath();
-			if (sourcePath != null) {
-				IPath rootPath= getSourceAttachmentRootPath();
-				mapper = this.createSourceMapper(sourcePath, rootPath);
-				if (rootPath == null && mapper.rootPath != null) {
-					// as a side effect of calling the SourceMapper constructor, the root path was computed
-					rootPath = new Path(mapper.rootPath);
-					
-					//set the property to the path of the mapped source
-					this.setSourceAttachmentProperty(
+			IPath rootPath= getSourceAttachmentRootPath();
+			if (sourcePath == null)
+				mapper = createSourceMapper(getPath(), rootPath); // attach root to itself
+			else
+				mapper = createSourceMapper(sourcePath, rootPath);
+			if (rootPath == null && mapper.rootPath != null) {
+				// as a side effect of calling the SourceMapper constructor, the root path was computed
+				rootPath = new Path(mapper.rootPath);
+				
+				//set the property to the path of the mapped source
+				if (sourcePath != null)
+					setSourceAttachmentProperty(
 						sourcePath.toString() 
 						+ ATTACHMENT_PROPERTY_DELIMITER 
 						+ rootPath.toString());
-				}
-				rootInfo.setSourceMapper(mapper);
-			} else {
-				// remember that no source is attached
-				rootInfo.setSourceMapper(NO_SOURCE_MAPPER);
-				mapper = null;
 			}
-		} else if (mapper == NO_SOURCE_MAPPER) {
-			// a previous call to this method found out that no source was attached
-			mapper = null;
+			rootInfo.setSourceMapper(mapper);
 		}
 	} catch (JavaModelException e) {
 		// no source can be attached
