@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -22,6 +22,7 @@ import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.Path;
+import org.eclipse.core.runtime.jobs.ISchedulingRule;
 import org.eclipse.jdt.core.IClasspathEntry;
 import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IJavaElementDelta;
@@ -305,6 +306,11 @@ public class SetClasspathOperation extends JavaModelOperation {
 	protected void generateClasspathChangeDeltas() {
 
 		JavaModelManager manager = JavaModelManager.getJavaModelManager();
+		if (manager.deltaState.findJavaProject(this.project.getElementName()) == null)
+			// project doesn't exist yet (we're in an IWorkspaceRunnable)
+			// no need to create a delta here and no need to index (see https://bugs.eclipse.org/bugs/show_bug.cgi?id=133334)
+			// the delta processor will create an ADDED project delta, and index the project
+			return;
 		boolean needToUpdateDependents = false;
 		JavaElementDelta delta = new JavaElementDelta(getJavaModel());
 		boolean hasDelta = false;
@@ -557,6 +563,9 @@ public class SetClasspathOperation extends JavaModelOperation {
 			updateAffectedProjects(project.getProject().getFullPath());
 		}
 	}
+	protected ISchedulingRule getSchedulingRule() {
+		return null; // no lock taken while setting the classpath
+	}
 	/*
 	 * Returns the source attachment flag for the delta between the 2 give source paths.
 	 * Returns either F_SOURCEATTACHED, F_SOURCEDETACHED, F_SOURCEATTACHED | F_SOURCEDETACHED
@@ -792,7 +801,7 @@ public class SetClasspathOperation extends JavaModelOperation {
 		
 		if (this.newRawPath == DO_NOT_SET_ENTRIES || this.newRawPath == DO_NOT_UPDATE_PROJECT_REFS) return;
 		// will run now, or be deferred until next pre-auto-build notification if resource tree is locked
-		JavaModelManager.getJavaModelManager().deltaState.performClasspathResourceChange(
+		JavaModelManager.getJavaModelManager().deltaState.updateProjectReferences(
 		        project, 
 		        oldResolvedPath, 
 		        newResolvedPath, 
