@@ -1,23 +1,21 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.jdt.debug.ui.launchConfigurations;
 
-
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
 import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.IWorkspaceRoot;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.debug.core.ILaunchConfiguration;
@@ -25,19 +23,15 @@ import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
 import org.eclipse.debug.ui.DebugUITools;
 import org.eclipse.debug.ui.IDebugUIConstants;
 import org.eclipse.jdt.core.IJavaElement;
-import org.eclipse.jdt.core.IJavaModel;
-import org.eclipse.jdt.core.IJavaProject;
-import org.eclipse.jdt.core.JavaCore;
 import org.eclipse.jdt.core.JavaModelException;
 import org.eclipse.jdt.internal.debug.ui.IJavaDebugHelpContextIds;
 import org.eclipse.jdt.internal.debug.ui.JDIDebugUIPlugin;
+import org.eclipse.jdt.internal.debug.ui.launcher.AbstractJavaMainTab;
 import org.eclipse.jdt.internal.debug.ui.launcher.ComboFieldEditor;
-import org.eclipse.jdt.internal.debug.ui.launcher.JavaLaunchConfigurationTab;
 import org.eclipse.jdt.internal.debug.ui.launcher.LauncherMessages;
 import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;
 import org.eclipse.jdt.launching.IVMConnector;
 import org.eclipse.jdt.launching.JavaRuntime;
-import org.eclipse.jdt.ui.JavaElementLabelProvider;
 import org.eclipse.jface.preference.BooleanFieldEditor;
 import org.eclipse.jface.preference.FieldEditor;
 import org.eclipse.jface.preference.IntegerFieldEditor;
@@ -45,10 +39,7 @@ import org.eclipse.jface.preference.PreferenceStore;
 import org.eclipse.jface.preference.StringFieldEditor;
 import org.eclipse.jface.util.IPropertyChangeListener;
 import org.eclipse.jface.util.PropertyChangeEvent;
-import org.eclipse.jface.viewers.ILabelProvider;
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.events.ModifyEvent;
-import org.eclipse.swt.events.ModifyListener;
 import org.eclipse.swt.events.SelectionAdapter;
 import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.graphics.Font;
@@ -60,10 +51,7 @@ import org.eclipse.swt.widgets.Combo;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Group;
-import org.eclipse.swt.widgets.Label;
-import org.eclipse.swt.widgets.Text;
-import org.eclipse.ui.dialogs.ElementListSelectionDialog;
-import org.eclipse.ui.help.WorkbenchHelp;
+import org.eclipse.ui.PlatformUI;
 
 import com.sun.jdi.connect.Connector;
 
@@ -76,96 +64,55 @@ import com.sun.jdi.connect.Connector;
  * </p>
  * @since 2.0
  */
-public class JavaConnectTab extends JavaLaunchConfigurationTab implements IPropertyChangeListener {
-
-	// Project UI widgets
-	protected Label fProjLabel;
-	protected Text fProjText;
-	protected Button fProjButton;
+public class JavaConnectTab extends AbstractJavaMainTab implements IPropertyChangeListener {
 	
-	// Allow terminate UI widgets
-	protected Button fAllowTerminateButton;
+	// UI widgets
+	private Button fAllowTerminateButton;
+	private Map fArgumentMap;
+	private Map fFieldEditorMap = new HashMap();
+	private Composite fArgumentComposite;
+	private Combo fConnectorCombo;
 	
-	// Connector attributes for selected connector
-	protected Map fArgumentMap;
-	protected Map fFieldEditorMap = new HashMap();
-	protected Composite fArgumentComposite;
 	// the selected connector
-	protected IVMConnector fConnector;
-	
-	// Connector combo
-	protected Combo fConnectorCombo;
-	protected IVMConnector[] fConnectors = JavaRuntime.getVMConnectors();
-	
-	protected static final String EMPTY_STRING = ""; //$NON-NLS-1$
-	
-	/**
-	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#createControl(Composite)
+	private IVMConnector fConnector;
+	private IVMConnector[] fConnectors = JavaRuntime.getVMConnectors();
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#createControl(org.eclipse.swt.widgets.Composite)
 	 */
 	public void createControl(Composite parent) {
 		Font font = parent.getFont();
-		
 		Composite comp = new Composite(parent, SWT.NONE);
 		setControl(comp);
-		WorkbenchHelp.setHelp(getControl(), IJavaDebugHelpContextIds.LAUNCH_CONFIGURATION_DIALOG_CONNECT_TAB);
+		PlatformUI.getWorkbench().getHelpSystem().setHelp(getControl(), IJavaDebugHelpContextIds.LAUNCH_CONFIGURATION_DIALOG_CONNECT_TAB);
 		GridLayout topLayout = new GridLayout();
-		topLayout.marginHeight = 0;
-		comp.setLayout(topLayout);		
-		GridData gd;
-		
+		topLayout.verticalSpacing = 0;
+		comp.setLayout(topLayout);
+		comp.setFont(font);
+		createProjectEditor(comp);
 		createVerticalSpacer(comp, 1);
+		createConnectionTypeControl(comp);
+		createVerticalSpacer(comp, 1);
+		createConnectionParamsControl(comp);
+		createVerticalSpacer(comp, 2);
+		fAllowTerminateButton = createCheckButton(comp, LauncherMessages.JavaConnectTab__Allow_termination_of_remote_VM_6); 
+		fAllowTerminateButton.addSelectionListener(getDefaultListener());
+	}
 
-		Composite projComp = new Composite(comp, SWT.NONE);
-		GridLayout projLayout = new GridLayout();
-		projLayout.numColumns = 2;
-		projLayout.marginHeight = 0;
-		projLayout.marginWidth = 0;
-		projComp.setLayout(projLayout);
-		gd = new GridData(GridData.FILL_HORIZONTAL);
-		projComp.setLayoutData(gd);
-		projComp.setFont(font);
-		
-		fProjLabel = new Label(projComp, SWT.NONE);
-		fProjLabel.setText(LauncherMessages.getString("JavaConnectTab.&Project__2")); //$NON-NLS-1$
-		gd = new GridData();
+	/**
+	 * creates the connector type control group
+	 * @param parent the parent composite to add this one to
+	 */
+	private void createConnectionTypeControl(Composite parent) {
+		Font font = parent.getFont();
+		Group group = new Group(parent, SWT.NONE);
+		group.setText(LauncherMessages.JavaConnectTab_Connect_ion_Type__7); 
+		group.setLayout(new GridLayout(2, true));
+		group.setFont(font);
+		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
 		gd.horizontalSpan = 2;
-		fProjLabel.setLayoutData(gd);
-		fProjLabel.setFont(font);
-		
-		fProjText = new Text(projComp, SWT.SINGLE | SWT.BORDER);
-		gd = new GridData(GridData.FILL_HORIZONTAL);
-		fProjText.setLayoutData(gd);
-		fProjText.setFont(font);
-		fProjText.addModifyListener(new ModifyListener() {
-			public void modifyText(ModifyEvent evt) {
-				updateLaunchConfigurationDialog();
-			}
-		});
-		
-		fProjButton = createPushButton(projComp, LauncherMessages.getString("JavaConnectTab.&Browse_3"), null); //$NON-NLS-1$
-		fProjButton.addSelectionListener(new SelectionAdapter() {
-			public void widgetSelected(SelectionEvent evt) {
-				handleProjectButtonSelected();
-			}
-		});
-		
-		Composite connectorComp = new Composite(comp,SWT.NONE);
-		GridLayout y = new GridLayout();
-		y.numColumns = 2;
-		y.marginHeight = 0;
-		y.marginWidth = 0;
-		connectorComp.setLayout(y);
-		gd = new GridData(GridData.FILL_HORIZONTAL);
-		connectorComp.setLayoutData(gd);
-		
-		Label l = new Label(connectorComp, SWT.NONE);
-		l.setText(LauncherMessages.getString("JavaConnectTab.Connect&ion_Type__7")); //$NON-NLS-1$
-		gd = new GridData(GridData.BEGINNING);
-		gd.horizontalSpan = 2;
-		l.setLayoutData(gd);
-		l.setFont(font);
-		
-		fConnectorCombo = new Combo(connectorComp, SWT.READ_ONLY);
+		group.setLayoutData(gd);
+		fConnectorCombo = new Combo(group, SWT.READ_ONLY);
 		gd = new GridData(GridData.FILL_HORIZONTAL);
 		gd.horizontalSpan = 2;
 		fConnectorCombo.setLayoutData(gd);
@@ -180,42 +127,35 @@ public class JavaConnectTab extends JavaLaunchConfigurationTab implements IPrope
 				handleConnectorComboModified();
 			}
 		});
-		
-		createVerticalSpacer(comp, 2);
-		
-		Group group = new Group(comp, SWT.NONE);
-		group.setText(LauncherMessages.getString("JavaConnectTab.Connection_Properties_1")); //$NON-NLS-1$
+	}
+	
+	/**
+	 * Creates the connection parameters control group
+	 * @param parent the parent composite to add this one to
+	 */
+	private void createConnectionParamsControl(Composite parent) {
+		Font font = parent.getFont();
+		Group group = new Group(parent, SWT.NONE);
+		group.setText(LauncherMessages.JavaConnectTab_Connection_Properties_1); 
 		group.setLayout(new GridLayout());
 		group.setFont(font);
-		gd = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
+		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
 		gd.horizontalSpan = 2;
-		group.setLayoutData(gd);		
-	
+		group.setLayoutData(gd);
 		//Add in an intermediate composite to allow for spacing
 		Composite spacingComposite = new Composite(group, SWT.NONE);
-		y = new GridLayout();
-		spacingComposite.setLayout(y);
-		gd = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
+		spacingComposite.setLayout(new GridLayout(2, true)); 
+		gd = new GridData(GridData.FILL_HORIZONTAL);
 		gd.horizontalSpan = 2;
 		spacingComposite.setLayoutData(gd);	
-		fArgumentComposite= spacingComposite;
-		
-		createVerticalSpacer(comp, 2);		
-		
-		fAllowTerminateButton = new Button(comp, SWT.CHECK);
-		fAllowTerminateButton.setText(LauncherMessages.getString("JavaConnectTab.&Allow_termination_of_remote_VM_6")); //$NON-NLS-1$
-		fAllowTerminateButton.setFont(font);
-		fAllowTerminateButton.addSelectionListener(new SelectionAdapter() {
-			public void widgetSelected(SelectionEvent evt) {
-				updateLaunchConfigurationDialog();
-			}
-		});
+		fArgumentComposite = spacingComposite;
+		fArgumentComposite.setFont(font); 
 	}
-
+	
 	/**
 	 * Update the argument area to show the selected connector's arguments
 	 */
-	protected void handleConnectorComboModified() {
+	private void handleConnectorComboModified() {
 		int index = fConnectorCombo.getSelectionIndex();
 		if ( (index < 0) || (index >= fConnectors.length) ) {
 			return;
@@ -228,7 +168,7 @@ public class JavaConnectTab extends JavaLaunchConfigurationTab implements IPrope
 		try {
 			fArgumentMap = vm.getDefaultArguments();
 		} catch (CoreException e) {
-			JDIDebugUIPlugin.errorDialog(LauncherMessages.getString("JavaConnectTab.Unable_to_display_connection_arguments._2"), e.getStatus()); //$NON-NLS-1$
+			JDIDebugUIPlugin.errorDialog(LauncherMessages.JavaConnectTab_Unable_to_display_connection_arguments__2, e.getStatus()); 
 			return;
 		}
 		
@@ -236,7 +176,7 @@ public class JavaConnectTab extends JavaLaunchConfigurationTab implements IPrope
 		Control[] children = fArgumentComposite.getChildren();
 		for (int i = 0; i < children.length; i++) {
 			children[i].dispose();
-		}
+		} 
 		fFieldEditorMap.clear();
 		PreferenceStore store = new PreferenceStore();
 		// create editors
@@ -273,51 +213,47 @@ public class JavaConnectTab extends JavaLaunchConfigurationTab implements IPrope
 			field.setPropertyChangeListener(this);
 			fFieldEditorMap.put(key, field);
 		}
-		
 		fArgumentComposite.getParent().getParent().layout();
-		fArgumentComposite.layout();
+		fArgumentComposite.layout(true);
 	}
 	
 	/**
 	 * Adds a colon to the label if required
 	 */
-	protected String getLabel(String label) {
+	private String getLabel(String label) {
 		if (!label.endsWith(":")) { //$NON-NLS-1$
 			label += ":"; //$NON-NLS-1$
-		}
+		}//end if
 		return label;
 	}
 
-	/**
-	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#initializeFrom(ILaunchConfiguration)
+	 /* (non-Javadoc)
+	 * @see org.eclipse.jdt.internal.debug.ui.launcher.AbstractJavaMainTab#initializeFrom(org.eclipse.debug.core.ILaunchConfiguration)
 	 */
 	public void initializeFrom(ILaunchConfiguration config) {
-		updateProjectFromConfig(config);
+		super.initializeFrom(config);
 		updateAllowTerminateFromConfig(config);
 		updateConnectionFromConfig(config);
 	}
 	
-	protected void updateProjectFromConfig(ILaunchConfiguration config) {
-		String projectName = ""; //$NON-NLS-1$
-		try {
-			projectName = config.getAttribute(IJavaLaunchConfigurationConstants.ATTR_PROJECT_NAME, EMPTY_STRING);	
-		} catch (CoreException ce) {
-			JDIDebugUIPlugin.log(ce);
-		}
-		fProjText.setText(projectName);
-	}
-	
-	protected void updateAllowTerminateFromConfig(ILaunchConfiguration config) {
+	/**
+	 * Updates the state of the allow terminate check button from the specified configuration
+	 * @param config the config to load from
+	 */
+	private void updateAllowTerminateFromConfig(ILaunchConfiguration config) {
 		boolean allowTerminate = false;
 		try {
 			allowTerminate = config.getAttribute(IJavaLaunchConfigurationConstants.ATTR_ALLOW_TERMINATE, false);	
-		} catch (CoreException ce) {
-			JDIDebugUIPlugin.log(ce);		
-		}
+		}//end try 
+		catch (CoreException ce) {JDIDebugUIPlugin.log(ce);}
 		fAllowTerminateButton.setSelection(allowTerminate);	
 	}
 
-	protected void updateConnectionFromConfig(ILaunchConfiguration config) {
+	/**
+	 * Updates the connection argument field editors from the specified configuration
+	 * @param config the config to load from
+	 */
+	private void updateConnectionFromConfig(ILaunchConfiguration config) {
 		String id = null;
 		try {
 			id = config.getAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_CONNECTOR, JavaRuntime.getDefaultVMConnector().getIdentifier());
@@ -327,7 +263,7 @@ public class JavaConnectTab extends JavaLaunchConfigurationTab implements IPrope
 			Map attrMap = config.getAttribute(IJavaLaunchConfigurationConstants.ATTR_CONNECT_MAP, (Map)null);
 			if (attrMap == null) {
 				return;
-			}
+			}//end if
 			Iterator keys = attrMap.keySet().iterator();
 			while (keys.hasNext()) {
 				String key = (String)keys.next();
@@ -337,37 +273,28 @@ public class JavaConnectTab extends JavaLaunchConfigurationTab implements IPrope
 					String value = (String)attrMap.get(key);
 					if (arg instanceof Connector.StringArgument || arg instanceof Connector.SelectedArgument) {
 						editor.getPreferenceStore().setValue(key, value);
-					} else if (arg instanceof Connector.BooleanArgument) {
-						boolean b = new Boolean(value).booleanValue();
-						editor.getPreferenceStore().setValue(key, b);
-					} else if (arg instanceof Connector.IntegerArgument) {
-						int i = new Integer(value).intValue();
-						editor.getPreferenceStore().setValue(key, i);
-					}
+					}//end if 
+					else if (arg instanceof Connector.BooleanArgument) {
+						editor.getPreferenceStore().setValue(key, Boolean.valueOf(value).booleanValue());
+					}//end if 
+					else if (arg instanceof Connector.IntegerArgument) {
+						editor.getPreferenceStore().setValue(key, new Integer(value).intValue());
+					}//end if
 					editor.load();
-				}
-			}						
-		} catch (CoreException ce) {
-			JDIDebugUIPlugin.log(ce);
-		}	
-			
+				}//end if
+			}//end while						
+		}//end try 
+		catch (CoreException ce) {JDIDebugUIPlugin.log(ce);}	
 	}
-	
-	/**
-	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#dispose()
-	 */
-	public void dispose() {
-	}
-		
-	/**
-	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(ILaunchConfigurationWorkingCopy)
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
 	 */
 	public void performApply(ILaunchConfigurationWorkingCopy config) {
 		config.setAttribute(IJavaLaunchConfigurationConstants.ATTR_PROJECT_NAME, fProjText.getText().trim());
 		config.setAttribute(IJavaLaunchConfigurationConstants.ATTR_ALLOW_TERMINATE, fAllowTerminateButton.getSelection());
-		IVMConnector vmc = getSelectedConnector();
-		config.setAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_CONNECTOR, vmc.getIdentifier());
-		
+		config.setAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_CONNECTOR, getSelectedConnector().getIdentifier());
+		mapResources(config);
 		Map attrMap = new HashMap(fFieldEditorMap.size());
 		Iterator keys = fFieldEditorMap.keySet().iterator();
 		while (keys.hasNext()) {
@@ -375,120 +302,50 @@ public class JavaConnectTab extends JavaLaunchConfigurationTab implements IPrope
 			FieldEditor editor = (FieldEditor)fFieldEditorMap.get(key);
 			if (!editor.isValid()) {
 				return;
-			}
+			}//end if
 			Connector.Argument arg = (Connector.Argument)fArgumentMap.get(key);
 			editor.store();
 			if (arg instanceof Connector.StringArgument || arg instanceof Connector.SelectedArgument) {
-				String value = editor.getPreferenceStore().getString(key);
-				attrMap.put(key, value);
-			} else if (arg instanceof Connector.BooleanArgument) {
-				boolean value = editor.getPreferenceStore().getBoolean(key);
-				attrMap.put(key, new Boolean(value).toString());
-			} else if (arg instanceof Connector.IntegerArgument) {
-				int value = editor.getPreferenceStore().getInt(key);
-				attrMap.put(key, new Integer(value).toString());
-			}
-		}				
+				attrMap.put(key, editor.getPreferenceStore().getString(key));
+			}//end if 
+			else if (arg instanceof Connector.BooleanArgument) {
+				attrMap.put(key, Boolean.valueOf(editor.getPreferenceStore().getBoolean(key)).toString());
+			}//end if 
+			else if (arg instanceof Connector.IntegerArgument) {
+				attrMap.put(key, new Integer(editor.getPreferenceStore().getInt(key)).toString());
+			}//end if
+		}//end while				
 		config.setAttribute(IJavaLaunchConfigurationConstants.ATTR_CONNECT_MAP, attrMap);
 	}
-		
-	/**
-	 * Show a dialog that lets the user select a project.  This in turn provides
-	 * context for the main type, allowing the user to key a main type name, or
-	 * constraining the search for main types to the specified project.
-	 */
-	protected void handleProjectButtonSelected() {
-		IJavaProject project = chooseJavaProject();
-		if (project == null) {
-			return;
-		}
-		
-		String projectName = project.getElementName();
-		fProjText.setText(projectName);		
-	}
 	
-	/**
-	 * Realize a Java Project selection dialog and return the first selected project,
-	 * or null if there was none.
-	 */
-	protected IJavaProject chooseJavaProject() {
-		IJavaProject[] projects;
-		try {
-			projects= JavaCore.create(getWorkspaceRoot()).getJavaProjects();
-		} catch (JavaModelException e) {
-			JDIDebugUIPlugin.log(e);
-			projects= new IJavaProject[0];
-		}
-		
-		ILabelProvider labelProvider= new JavaElementLabelProvider(JavaElementLabelProvider.SHOW_DEFAULT);
-		ElementListSelectionDialog dialog= new ElementListSelectionDialog(getShell(), labelProvider);
-		dialog.setTitle(LauncherMessages.getString("JavaConnectTab.Project_selection_10")); //$NON-NLS-1$
-		dialog.setMessage(LauncherMessages.getString("JavaConnectTab.Choose_a_project_to_constrain_the_search_for_main_types_11")); //$NON-NLS-1$
-		dialog.setElements(projects);
-		
-		IJavaProject javaProject = getJavaProject();
-		if (javaProject != null) {
-			dialog.setInitialSelections(new Object[] { javaProject });
-		}
-		if (dialog.open() == ElementListSelectionDialog.OK) {			
-			return (IJavaProject) dialog.getFirstResult();
-		}			
-		return null;		
-	}
-	
-	/**
-	 * Return the IJavaProject corresponding to the project name in the project name
-	 * text field, or null if the text does not match a project name.
-	 */
-	protected IJavaProject getJavaProject() {
-		String projectName = fProjText.getText().trim();
-		if (projectName.length() < 1) {
-			return null;
-		}
-		return getJavaModel().getJavaProject(projectName);		
-	}
-	
-	/**
-	 * Convenience method to get the workspace root.
-	 */
-	private IWorkspaceRoot getWorkspaceRoot() {
-		return ResourcesPlugin.getWorkspace().getRoot();
-	}
-	
-	/**
-	 * Convenience method to get access to the java model.
-	 */
-	private IJavaModel getJavaModel() {
-		return JavaCore.create(getWorkspaceRoot());
-	}
-
 	/**
 	 * Initialize default settings for the given Java element
 	 */
-	protected void initializeDefaults(IJavaElement javaElement, ILaunchConfigurationWorkingCopy config) {
+	private void initializeDefaults(IJavaElement javaElement, ILaunchConfigurationWorkingCopy config) {
 		initializeJavaProject(javaElement, config);
 		initializeName(javaElement, config);
 		initializeHardCodedDefaults(config);
 	}
 
-	/**
-	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#setDefaults(ILaunchConfigurationWorkingCopy)
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#setDefaults(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
 	 */
 	public void setDefaults(ILaunchConfigurationWorkingCopy config) {
 		IJavaElement javaElement = getContext();
 		if (javaElement == null) {
 			initializeHardCodedDefaults(config);
-		} else {
+		}//end if 
+		else {
 			initializeDefaults(javaElement, config);
-		}
+		}//end else
 	}
 
 	/**
 	 * Find the first instance of a type, compilation unit, class file or project in the
 	 * specified element's parental hierarchy, and use this as the default name.
 	 */
-	protected void initializeName(IJavaElement javaElement, ILaunchConfigurationWorkingCopy config) {
-		String name = ""; //$NON-NLS-1$
+	private void initializeName(IJavaElement javaElement, ILaunchConfigurationWorkingCopy config) {
+		String name = EMPTY_STRING;
 		try {
 			IResource resource = javaElement.getUnderlyingResource();
 			if (resource != null) {
@@ -496,42 +353,39 @@ public class JavaConnectTab extends JavaLaunchConfigurationTab implements IPrope
 				int index = name.lastIndexOf('.');
 				if (index > 0) {
 					name = name.substring(0, index);
-				}
-			} else {
+				}//end if
+			}//end if 
+			else {
 				name= javaElement.getElementName();
-			}
+			}//end else
 			name = getLaunchConfigurationDialog().generateName(name);				
-		} catch (JavaModelException jme) {
-			JDIDebugUIPlugin.log(jme);
-		}
+		}//end try 
+		catch (JavaModelException jme) {JDIDebugUIPlugin.log(jme);}
 		config.rename(name);
 	}
 
 	/**
 	 * Initialize those attributes whose default values are independent of any context.
 	 */
-	protected void initializeHardCodedDefaults(ILaunchConfigurationWorkingCopy config) {
+	private void initializeHardCodedDefaults(ILaunchConfigurationWorkingCopy config) {
 		config.setAttribute(IJavaLaunchConfigurationConstants.ATTR_ALLOW_TERMINATE, false);
 		config.setAttribute(IJavaLaunchConfigurationConstants.ATTR_VM_CONNECTOR, JavaRuntime.getDefaultVMConnector().getIdentifier());
 	}
-	
-	/**
-	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#isValid(ILaunchConfiguration)
+
+	 /* (non-Javadoc)
+	 * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#isValid(org.eclipse.debug.core.ILaunchConfiguration)
 	 */
-	public boolean isValid(ILaunchConfiguration config) {
-		
+	public boolean isValid(ILaunchConfiguration config) {	
 		setErrorMessage(null);
 		setMessage(null);
-
 		// project		
 		String name = fProjText.getText().trim();
 		if (name.length() > 0) {
 			if (!ResourcesPlugin.getWorkspace().getRoot().getProject(name).exists()) {
-				setErrorMessage(LauncherMessages.getString("JavaConnectTab.Project_does_not_exist_14")); //$NON-NLS-1$
+				setErrorMessage(LauncherMessages.JavaConnectTab_Project_does_not_exist_14); 
 				return false;
-			}
-		}
-
+			}//end if
+		}//end if
 		Iterator keys = fFieldEditorMap.keySet().iterator();
 		while (keys.hasNext()) {
 			String key = (String)keys.next();
@@ -540,24 +394,23 @@ public class JavaConnectTab extends JavaLaunchConfigurationTab implements IPrope
 			if (editor instanceof StringFieldEditor) {
 				String value = ((StringFieldEditor)editor).getStringValue();
 				if (!arg.isValid(value)) {
-					setErrorMessage(arg.label() + LauncherMessages.getString("JavaConnectTab._is_invalid._5")); //$NON-NLS-1$
+					setErrorMessage(arg.label() + LauncherMessages.JavaConnectTab__is_invalid__5); 
 					return false;
-				}		
-			}
-		}				
-						
+				}//end if		
+			}//end if
+		}//end while							
 		return true;
-	}
-	
-	/**
+	}//end isValid
+
+	/* (non-Javadoc)
 	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
 	 */
 	public String getName() {
-		return LauncherMessages.getString("JavaConnectTab.Conn&ect_20"); //$NON-NLS-1$
+		return LauncherMessages.JavaConnectTab_Conn_ect_20;
 	}			
-	
-	/**
-	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getImage()
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#getImage()
 	 */
 	public Image getImage() {
 		return DebugUITools.getImage(IDebugUIConstants.IMG_LCL_DISCONNECT);
@@ -566,13 +419,14 @@ public class JavaConnectTab extends JavaLaunchConfigurationTab implements IPrope
 	/**
 	 * Returns the selected connector
 	 */
-	protected IVMConnector getSelectedConnector() {
+	private IVMConnector getSelectedConnector() {
 		return fConnector;
 	}
-	/**
-	 * @see IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
 	 */
 	public void propertyChange(PropertyChangeEvent event) {
 		updateLaunchConfigurationDialog();
 	}
-}
+}//end class
