@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -19,6 +19,8 @@ import org.eclipse.jface.viewers.ISelectionChangedListener;
 import org.eclipse.jface.viewers.ISelectionProvider;
 import org.eclipse.jface.viewers.IStructuredSelection;
 import org.eclipse.jface.viewers.StructuredSelection;
+import org.eclipse.jface.window.IShellProvider;
+import org.eclipse.swt.widgets.Shell;
 import org.eclipse.ui.IObjectActionDelegate;
 import org.eclipse.ui.IWorkbenchPart;
 import org.eclipse.ui.dialogs.PropertyDialogAction;
@@ -36,8 +38,18 @@ public class JavaBreakpointPropertiesAction implements IObjectActionDelegate {
 	 * @see IActionDelegate#run(IAction)
 	 */
 	public void run(IAction action) {
+	    IShellProvider provider;
+	    if (getActivePart() != null) {
+	        provider = getActivePart().getSite();
+	    } else {
+	        provider = new IShellProvider() {
+	            public Shell getShell() {
+	                return JDIDebugUIPlugin.getActiveWorkbenchShell();
+	            }
+	        };
+        }
 		PropertyDialogAction propertyAction= 
-			new PropertyDialogAction(JDIDebugUIPlugin.getActiveWorkbenchShell(), new ISelectionProvider() {
+			new PropertyDialogAction(provider, new ISelectionProvider() {
 				public void addSelectionChangedListener(ISelectionChangedListener listener) {
 				}
 				public ISelection getSelection() {
