@@ -1,9 +1,9 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors:
  *     IBM Corporation - initial API and implementation
@@ -11,7 +11,6 @@
 package org.eclipse.jdt.internal.debug.eval.ast.instructions;
 
 
-import org.eclipse.core.runtime.CoreException;
 
 public class ReturnInstruction extends CompoundInstruction {
 
@@ -26,12 +25,12 @@ public class ReturnInstruction extends CompoundInstruction {
 	/**
 	 * @see Instruction#execute()
 	 */
-	public void execute() throws CoreException {
+	public void execute() {
 		stop();
 	}
 	
 	public String toString() {
-		return InstructionsEvaluationMessages.getString("ReturnInstruction.return"); //$NON-NLS-1$
+		return InstructionsEvaluationMessages.ReturnInstruction_return; 
 	}
 
 }
