@@ -1,17 +1,17 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.jdt.internal.debug.core.breakpoints;
 
  
-import java.text.MessageFormat;
+import com.ibm.icu.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
@@ -22,12 +22,10 @@ import java.util.Map;
 import java.util.Set;
 
 import org.eclipse.core.resources.IMarker;
-import org.eclipse.core.resources.IWorkspaceRunnable;
-import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.MultiStatus;
+import org.eclipse.core.runtime.Preferences;
 import org.eclipse.debug.core.DebugEvent;
-import org.eclipse.debug.core.DebugException;
 import org.eclipse.debug.core.DebugPlugin;
 import org.eclipse.debug.core.IDebugEventSetListener;
 import org.eclipse.debug.core.model.Breakpoint;
@@ -54,6 +52,7 @@ import com.sun.jdi.event.Event;
 import com.sun.jdi.event.LocatableEvent;
 import com.sun.jdi.request.ClassPrepareRequest;
 import com.sun.jdi.request.EventRequest;
+import com.sun.jdi.request.EventRequestManager;
 
 public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoint, IJDIEventListener, IDebugEventSetListener {
 
@@ -87,7 +86,7 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	 * Breakpoint attribute storing suspend policy code for 
 	 * this breakpoint.
 	 * (value <code>"org.eclipse.jdt.debug.core.suspendPolicy</code>).
-	 * This attribute is an <code>int</code> correspoinding
+	 * This attribute is an <code>int</code> corresponding
 	 * to <code>IJavaBreakpoint.SUSPEND_VM</code> or
 	 * <code>IJavaBreakpoint.SUSPEND_THREAD</code>.
 	 */
@@ -138,7 +137,7 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	protected static final IJavaObject[] fgEmptyInstanceFilters = new IJavaObject[0];
 	
 	/**
-	 * Propery identifier for a breakpoint object on an event request
+	 * Property identifier for a breakpoint object on an event request
 	 */
 	public static final String JAVA_BREAKPOINT_PROPERTY = "org.eclipse.jdt.debug.breakpoint"; //$NON-NLS-1$
 	
@@ -151,16 +150,16 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		fRequestsByTarget = new HashMap(1);
 		fFilteredThreadsByTarget= new HashMap(1);
 	}	
-	
-	/**
-	 * @see IBreakpoint#getModelIdentifier()
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.model.IBreakpoint#getModelIdentifier()
 	 */
 	public String getModelIdentifier() {
 		return JDIDebugModel.getPluginIdentifier();
 	}
 
-	/**
-	 * @see IBreakpoint#setMarker(IMarker)
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.model.Breakpoint#setMarker(org.eclipse.core.resources.IMarker)
 	 */
 	public void setMarker(IMarker marker) throws CoreException {
 		super.setMarker(marker);
@@ -172,8 +171,9 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	 * or sets it as unregistered.
 	 */
 	protected void register(boolean register) throws CoreException {
-		if (register) {
-			DebugPlugin.getDefault().getBreakpointManager().addBreakpoint(this);
+        DebugPlugin plugin = DebugPlugin.getDefault();
+		if (plugin != null && register) {
+            plugin.getBreakpointManager().addBreakpoint(this);
 		} else {
 			setRegistered(false);
 		}
@@ -194,7 +194,7 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		}
 		reqs.add(request);
 		target.addJDIEventListener(this, request);
-		// update the install attibute on the breakpoint
+		// update the install attribute on the breakpoint
 		if (!(request instanceof ClassPrepareRequest)) {
 			incrementInstallCount();
 			// notification 
@@ -213,9 +213,8 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		int index = name.indexOf('$');
 		if (index == -1) {
 			return name;
-		} else {
-			return name.substring(0, index);
 		}
+		return name.substring(0, index);
 	}	
 		
 	/**
@@ -246,20 +245,39 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		}
 	}
 
-	/**
-	 * @see IJDIEventListener#handleEvent(Event, JDIDebugTarget)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.internal.debug.core.IJDIEventListener#handleEvent(com.sun.jdi.event.Event, org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget)
 	 */
 	public boolean handleEvent(Event event, JDIDebugTarget target) {
 		if (event instanceof ClassPrepareEvent) {
 			return handleClassPrepareEvent((ClassPrepareEvent)event, target);
-		} else {
-			ThreadReference threadRef= ((LocatableEvent)event).thread();
-			JDIThread thread= target.findThread(threadRef);	
-			if (thread == null || thread.isIgnoringBreakpoints()) {
-				return true;
-			}
-			return handleBreakpointEvent(event, target, thread);
-		}		
+		}
+		ThreadReference threadRef= ((LocatableEvent)event).thread();
+		JDIThread thread= target.findThread(threadRef);	
+		if (thread == null || thread.isIgnoringBreakpoints()) {
+			return true;
+		}
+		return handleBreakpointEvent(event, target, thread);		
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.internal.debug.core.IJDIEventListener#wonSuspendVote(com.sun.jdi.event.Event, org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget)
+	 */
+	public void wonSuspendVote(Event event, JDIDebugTarget target) {
+		ThreadReference threadRef = null;
+		if (event instanceof ClassPrepareEvent) {
+			threadRef = ((ClassPrepareEvent)event).thread();
+		} else if (event instanceof LocatableEvent) {
+			threadRef = ((LocatableEvent)event).thread();
+		}
+		if (threadRef == null) {
+			return;
+		}
+		JDIThread thread= target.findThread(threadRef);	
+		if (thread == null || thread.isIgnoringBreakpoints()) {
+			return;
+		}
+		thread.wonSuspendVote(this);
 	}
 
 	/**
@@ -295,7 +313,7 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	}
 	
 	/**
-	 * Deletegates to the given thread to suspend, and
+	 * Delegates to the given thread to suspend, and
 	 * returns whether the thread suspended
 	 * It is possible that the thread will not suspend
 	 * as directed by a Java breakpoint listener.
@@ -317,10 +335,14 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		if (installableType == null || queriedType == null) {
 			return false;
 		}
+		int index= queriedType.indexOf('<');
+		if (index != -1) {
+			queriedType= queriedType.substring(0, index);
+		}
 		if (installableType.equals(queriedType)) {
 			return queryInstallListeners(target, type);
 		}
-		int index= queriedType.indexOf('$', 0);
+		index= queriedType.indexOf('$', 0);
 		if (index == -1) {
 			return false;
 		}
@@ -331,21 +353,42 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	}
 	
 	/**
-	 * Called when a breakpoint event is encountered
+	 * Called when a breakpoint event is encountered. Expires the
+	 * hit count in the event's request and updates the marker.
+	 * @param event the event whose request should have its hit count
+	 * expired or <code>null</code> to only update the breakpoint marker.
 	 */
 	protected void expireHitCount(Event event) {
-		EventRequest request= event.request();
-		Integer requestCount= (Integer) request.getProperty(HIT_COUNT);
+		Integer requestCount= null;
+		EventRequest request= null;
+		if (event != null) {
+			request= event.request();
+			requestCount= (Integer) request.getProperty(HIT_COUNT);
+		}
 		if (requestCount != null) {
-			try {
+			if (request != null) {
 				request.putProperty(EXPIRED, Boolean.TRUE);
+			}
+			try {
 				setAttributes(fgExpiredEnabledAttributes, new Object[]{Boolean.TRUE, Boolean.FALSE});
 				// make a note that we auto-disabled this breakpoint.
 			} catch (CoreException ce) {
 				JDIDebugPlugin.log(ce);
 			}
 		}
-	}	
+	}
+	
+	/**
+	 * Returns whether this breakpoint should be "skipped". Breakpoints
+	 * are skipped if the breakpoint manager is disabled and the breakpoint
+	 * is registered with the manager
+	 * 
+	 * @return whether this breakpoint should be skipped
+	 */
+	public boolean shouldSkipBreakpoint() throws CoreException {
+		DebugPlugin plugin = DebugPlugin.getDefault();
+        return plugin != null && isRegistered() && !plugin.getBreakpointManager().isEnabled();
+	}
 
 	/**
 	 * Attempts to create a breakpoint request for this breakpoint in the given
@@ -354,12 +397,18 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	 * @return Whether a request was created
 	 */
 	protected boolean createRequest(JDIDebugTarget target, ReferenceType type) throws CoreException {
-		EventRequest request= newRequest(target, type);
-		if (request == null) {
+		if (shouldSkipBreakpoint()) {
+			return false;
+		}
+		EventRequest[] requests= newRequests(target, type);
+		if (requests == null) {
 			return false;
 		}
 		fInstalledTypeName = type.name();
-		registerRequest(request, target);
+		for (int i = 0; i < requests.length; i++) {
+            EventRequest request = requests[i];
+            registerRequest(request, target);    
+        }
 		return true;
 	}
 	
@@ -411,11 +460,10 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		if (hitCount > 0) {
 			request.addCountFilter(hitCount);
 			request.putProperty(HIT_COUNT, new Integer(hitCount));
-			//request.putProperty(EXPIRED, Boolean.FALSE);
 		}
 	}
 	
-	protected void configureInstanceFilters(EventRequest request, JDIDebugTarget target) throws CoreException {
+	protected void configureInstanceFilters(EventRequest request, JDIDebugTarget target) {
 		if (fInstanceFilters != null && !fInstanceFilters.isEmpty()) {
 			Iterator iter = fInstanceFilters.iterator();
 			while (iter.hasNext()) {
@@ -428,14 +476,12 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	}	
 	
 	/**
-	 * Creates and returns a breakpoint request for this breakpoint which
-	 * has been installed in the given reference type and registered
-	 * in the given target.
+	 * Creates, installs, and returns all event requests for this breakpoint
+	 * in the given reference type and and target.
 	 * 
-	 * @return the event request which was created or <code>null</code> if
-	 *  the request creation failed
+	 * @return the event requests created or <code>null</code> if creation failed
 	 */
-	protected abstract EventRequest newRequest(JDIDebugTarget target, ReferenceType type) throws CoreException;
+	protected abstract EventRequest[] newRequests(JDIDebugTarget target, ReferenceType type) throws CoreException;
 	
 	/**
 	 * Add this breakpoint to the given target. After it has been
@@ -443,20 +489,15 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	 * execution of that target as appropriate.
 	 */
 	public void addToTarget(JDIDebugTarget target) throws CoreException {
-		
-		// pre-notification
 		fireAdding(target);
-		
-		// create event requests
 		createRequests(target);
-
 	}
 	
 	/**
 	 * Creates event requests for the given target
 	 */
 	protected void createRequests(JDIDebugTarget target) throws CoreException {
-		if (target.isTerminated()) {
+		if (target.isTerminated() || shouldSkipBreakpoint()) {
 			return;
 		}
 		String referenceTypeName= getTypeName();
@@ -464,7 +505,6 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		if (referenceTypeName == null || enclosingTypeName == null) {
 			return;
 		}
-
 		// create request to listen to class loads
 		if (referenceTypeName.indexOf('$') == -1) {
 			registerRequest(target.createClassPrepareRequest(enclosingTypeName), target);
@@ -525,12 +565,6 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 			}
 		}
 	}
-	
-	/**
-	 * Do nothing - we update the request when an API call changes an attribute
-	 */
-	public void changeForTarget(JDIDebugTarget target) throws CoreException {
-	}
 			
 	/**
 	 * Returns the JDI suspend policy that corresponds to this
@@ -549,6 +583,17 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		return EventRequest.SUSPEND_ALL;
 	}
 
+	/**
+	 * returns the default suspend policy based on the pref setting on the 
+	 * Java-Debug pref page
+	 * @return the default suspend policy
+	 * @since 3.2
+	 */
+	protected int getDefaultSuspendPolicy() {
+		Preferences store = JDIDebugModel.getPreferences();
+		return store.getInt(JDIDebugPlugin.PREF_DEFAULT_BREAKPOINT_SUSPEND_POLICY);
+	}
+	
 	
 	/**
 	 * Returns whether the hitCount of this breakpoint is equal to the hitCount of
@@ -565,7 +610,7 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	}
 	
 	/**
-	 * Removes this breakpoint from the given tagret.
+	 * Removes this breakpoint from the given target.
 	 */
 	public void removeFromTarget(final JDIDebugTarget target) throws CoreException {
 		removeRequests(target);
@@ -602,7 +647,7 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	 */
 	protected void removeRequests(final JDIDebugTarget target) throws CoreException {
 		// removing was previously done is a workspace runnable, but that is
-		// not possible since it can be a resouce callback (marker deletion) that
+		// not possible since it can be a resource callback (marker deletion) that
 		// causes a breakpoint to be removed
 		ArrayList requests= (ArrayList)getRequests(target).clone();
 		// Iterate over a copy of the requests since this list of requests
@@ -613,14 +658,17 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 			req = (EventRequest)iter.next();
 			try {
 				if (target.isAvailable() && !isExpired(req)) { // cannot delete an expired request
-					target.getEventRequestManager().deleteEventRequest(req); // disable & remove
+					EventRequestManager manager = target.getEventRequestManager();
+					if (manager != null) {
+						manager.deleteEventRequest(req); // disable & remove 
+					}					
 				}
 			} catch (VMDisconnectedException e) {
 				if (target.isAvailable()) {
 					JDIDebugPlugin.log(e);
 				}
 			} catch (RuntimeException e) {
-				JDIDebugPlugin.log(e);
+                target.internalError(e);
 			} finally {
 				deregisterRequest(req, target);
 			}
@@ -652,7 +700,7 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 				}
 			} catch (VMDisconnectedException e) {
 			} catch (RuntimeException e) {
-				JDIDebugPlugin.log(e);
+				target.internalError(e);
 			}
 		}
 	}
@@ -674,9 +722,9 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		}
 		return requestExpired.booleanValue();
 	}
-	
-	/**
-	 * @see IJavaBreakpoint#isInstalled()
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#isInstalled()
 	 */
 	public boolean isInstalled() throws CoreException {
 		return ensureMarker().getAttribute(INSTALL_COUNT, 0) > 0;
@@ -722,26 +770,14 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		setAttribute(TYPE_NAME, typeName);
 	}	
 
-	/**
-	 * @see IJavaBreakpoint#getTypeName()
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#getTypeName()
 	 */
 	public String getTypeName() throws CoreException {
 		if (fInstalledTypeName == null) {
 			return ensureMarker().getAttribute(TYPE_NAME, null);
-		} else {
-			return fInstalledTypeName;
 		}
-	}
-	
-	/**
-	 * Execute the given workspace runnable
-	 */
-	protected void run(IWorkspaceRunnable wr) throws DebugException {
-		try {
-			ResourcesPlugin.getWorkspace().run(wr, null);
-		} catch (CoreException e) {
-			throw new DebugException(e.getStatus());
-		}			
+		return fInstalledTypeName;
 	}
 	
 	/**
@@ -777,15 +813,15 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		}
 	}	
 
-	/**
-	 * @see IJavaBreakpoint#getHitCount()
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#getHitCount()
 	 */
 	public int getHitCount() throws CoreException {
 		return ensureMarker().getAttribute(HIT_COUNT, -1);
 	}
 	
-	/**
-	 * @see IJavaBreakpoint#setHitCount(int)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#setHitCount(int)
 	 */
 	public void setHitCount(int count) throws CoreException {	
 		if (getHitCount() != count) {
@@ -800,17 +836,17 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		}
 	}
 	
-	protected String getMarkerMessage(int hitCount, int suspendPolicy) throws CoreException {
+	protected String getMarkerMessage(int hitCount, int suspendPolicy) {
 		StringBuffer buff= new StringBuffer();
 		if (hitCount > 0){
-			buff.append(MessageFormat.format(JDIDebugBreakpointMessages.getString("JavaBreakpoint._[Hit_Count__{0}]_1"), new Object[]{Integer.toString(hitCount)})); //$NON-NLS-1$
+			buff.append(MessageFormat.format(JDIDebugBreakpointMessages.JavaBreakpoint___Hit_Count___0___1, new Object[]{Integer.toString(hitCount)})); 
 			buff.append(' ');
 		}
 		String suspendPolicyString;
 		if (suspendPolicy == IJavaBreakpoint.SUSPEND_THREAD) {
-			suspendPolicyString= JDIDebugBreakpointMessages.getString("JavaBreakpoint.[suspend_policy__thread]_1"); //$NON-NLS-1$
+			suspendPolicyString= JDIDebugBreakpointMessages.JavaBreakpoint__suspend_policy__thread__1; 
 		} else {
-			suspendPolicyString= JDIDebugBreakpointMessages.getString("JavaBreakpoint.[suspend_policy__VM]_2"); //$NON-NLS-1$
+			suspendPolicyString= JDIDebugBreakpointMessages.JavaBreakpoint__suspend_policy__VM__2; 
 		}
 		
 		buff.append(suspendPolicyString);
@@ -824,21 +860,21 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		setAttribute(EXPIRED, expired);	
 	}	
 
-	/**
-	 * @see IJavaBreakpoint#getSuspendPolicy()
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#getSuspendPolicy()
 	 */
 	public int getSuspendPolicy() throws CoreException {
 		return ensureMarker().getAttribute(SUSPEND_POLICY, IJavaBreakpoint.SUSPEND_THREAD);
 	}
 
-	/**
-	 * @see IJavaBreakpoint#setSuspendPolicy(int)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#setSuspendPolicy(int)
 	 */
 	public void setSuspendPolicy(int suspendPolicy) throws CoreException {
-		if (getSuspendPolicy() != suspendPolicy) {
-			setAttributes(new String[]{SUSPEND_POLICY}, new Object[]{new Integer(suspendPolicy)});
+		if(getSuspendPolicy() != suspendPolicy) {
+			setAttribute(SUSPEND_POLICY, suspendPolicy);
+			recreate();
 		}
-		recreate();
 	}
 	
 	/**
@@ -848,18 +884,23 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	 * @param target debug target
 	 */
 	protected void fireAdding(IJavaDebugTarget target) {
-		JDIDebugPlugin.getDefault().fireBreakpointAdding(target, this);
+		JDIDebugPlugin plugin = JDIDebugPlugin.getDefault();
+        if (plugin != null)
+            plugin.fireBreakpointAdding(target, this);
 	}
 	
 	/**
-	 * Notifies listeners this breakpoint has been remvoed from the
+	 * Notifies listeners this breakpoint has been removed from the
 	 * given target.
 	 * 
 	 * @param target debug target
 	 */
 	protected void fireRemoved(IJavaDebugTarget target) {
-		JDIDebugPlugin.getDefault().fireBreakpointRemoved(target, this);
-		setInstalledIn(target, false);
+		JDIDebugPlugin plugin = JDIDebugPlugin.getDefault();
+        if (plugin != null) {
+            plugin.fireBreakpointRemoved(target, this);
+            setInstalledIn(target, false);
+        }
 	}	
 	
 	/**
@@ -869,8 +910,9 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	 * @param target debug target
 	 */
 	protected void fireInstalled(IJavaDebugTarget target) {
-		if (!isInstalledIn(target)) {
-			JDIDebugPlugin.getDefault().fireBreakpointInstalled(target, this);
+        JDIDebugPlugin plugin = JDIDebugPlugin.getDefault();
+		if (plugin!= null && !isInstalledIn(target)) {
+            plugin.fireBreakpointInstalled(target, this);
 			setInstalledIn(target, true);
 		}
 	}	
@@ -904,8 +946,8 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		}
 	}
 	
-	/**
-	 * @see IJavaBreakpoint#setThreadFilter(IJavaThread)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#setThreadFilter(org.eclipse.jdt.debug.core.IJavaThread)
 	 */
 	public void setThreadFilter(IJavaThread thread) throws CoreException {
 		if (!(thread.getDebugTarget() instanceof JDIDebugTarget) || !(thread instanceof JDIThread)) {
@@ -927,10 +969,8 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		}
 	}
 	
-	/**
-	 * @see IDebugEventSetListener#handleDebugEvents(DebugEvent[])
-	 * 
-	 * Cleanup on thread termination
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.IDebugEventSetListener#handleDebugEvents(org.eclipse.debug.core.DebugEvent[])
 	 */
 	public void handleDebugEvents(DebugEvent[] events) {
 		for (int i = 0; i < events.length; i++) {
@@ -980,17 +1020,17 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	 */
 	protected abstract void setRequestThreadFilter(EventRequest request, ThreadReference thread);
 	
-	/**
-	 * @see IJavaBreakpoint#getThreadFilter(IJavaDebugTarget)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#getThreadFilter(org.eclipse.jdt.debug.core.IJavaDebugTarget)
 	 */
-	public IJavaThread getThreadFilter(IJavaDebugTarget target) throws CoreException {
+	public IJavaThread getThreadFilter(IJavaDebugTarget target) {
 		return (IJavaThread)fFilteredThreadsByTarget.get(target);
 	}
 	
-	/**
-	 * @see IJavaBreakpoint#getThreadFilters()
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#getThreadFilters()
 	 */
-	public IJavaThread[] getThreadFilters() throws CoreException {
+	public IJavaThread[] getThreadFilters() {
 		IJavaThread[] threads= null;
 		Collection values= fFilteredThreadsByTarget.values();
 		threads= new IJavaThread[values.size()];
@@ -998,8 +1038,8 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		return threads;
 	}
 
-	/**
-	 * @see IJavaBreakpoint#removeThreadFilter(IJavaThread)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#removeThreadFilter(org.eclipse.jdt.debug.core.IJavaDebugTarget)
 	 */
 	public void removeThreadFilter(IJavaDebugTarget javaTarget) throws CoreException {
 		if (!(javaTarget instanceof JDIDebugTarget)) {
@@ -1021,15 +1061,19 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	 *  not installed in a specific type
 	 */
 	protected boolean queryInstallListeners(JDIDebugTarget target, ReferenceType type) {
-		IJavaType jt = null;
-		if (type != null) {
-			jt = JDIType.createType(target, type);
-		}
-		return JDIDebugPlugin.getDefault().fireInstalling(target, this, jt);
+        JDIDebugPlugin plugin = JDIDebugPlugin.getDefault();
+        if (plugin != null) {
+            IJavaType jt = null;
+            if (type != null) {
+                jt = JDIType.createType(target, type);
+            }
+            return plugin.fireInstalling(target, this, jt);
+        }
+        return false;
 	}
 	
-	/**
-	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#addInstanceFilter(IJavaObject)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#addInstanceFilter(org.eclipse.jdt.debug.core.IJavaObject)
 	 */
 	public void addInstanceFilter(IJavaObject object) throws CoreException {
 		if (fInstanceFilters == null) {
@@ -1044,27 +1088,28 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	
 	/**
 	 * Change notification when there are no marker changes. If the marker
-	 * does not exist, do not fire a change notificaiton (the marker may not
+	 * does not exist, do not fire a change notification (the marker may not
 	 * exist if the associated project was closed).
 	 */
 	protected void fireChanged() {
-		if (markerExists()) {	
-			DebugPlugin.getDefault().getBreakpointManager().fireBreakpointChanged(this);
+        DebugPlugin plugin = DebugPlugin.getDefault();
+		if (plugin != null && markerExists()) {	
+            plugin.getBreakpointManager().fireBreakpointChanged(this);
 		}					
 	}
 
-	/**
+	/* (non-Javadoc)
 	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#getInstanceFilters()
 	 */
-	public IJavaObject[] getInstanceFilters() throws CoreException {
+	public IJavaObject[] getInstanceFilters() {
 		if (fInstanceFilters == null || fInstanceFilters.isEmpty()) {
 			return fgEmptyInstanceFilters;
 		}
 		return (IJavaObject[])fInstanceFilters.toArray(new IJavaObject[fInstanceFilters.size()]);
 	}
 
-	/**
-	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#removeInstanceFilter(IJavaObject)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#removeInstanceFilter(org.eclipse.jdt.debug.core.IJavaObject)
 	 */
 	public void removeInstanceFilter(IJavaObject object) throws CoreException {
 		if (fInstanceFilters == null) {
@@ -1081,23 +1126,26 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 	 * all targets.
 	 */
 	protected void recreate() throws CoreException {
-		IDebugTarget[] targets = DebugPlugin.getDefault().getLaunchManager().getDebugTargets();
-		for (int i = 0; i < targets.length; i++) {
-			IDebugTarget target = targets[i];
-			MultiStatus multiStatus = new MultiStatus(JDIDebugPlugin.getUniqueIdentifier(), JDIDebugPlugin.INTERNAL_ERROR, JDIDebugBreakpointMessages.getString("JavaBreakpoint.Exception"), null); //$NON-NLS-1$
-			IJavaDebugTarget jdiTarget = (IJavaDebugTarget)target.getAdapter(IJavaDebugTarget.class);
-			if (jdiTarget != null) {
-				try {
-					recreate((JDIDebugTarget)jdiTarget);
-				} catch (CoreException e) {
-					multiStatus.add(e.getStatus());
-				}
-			}
-			if (!multiStatus.isOK()) {
-				throw new CoreException(multiStatus);
-			} 
-		}
-	}
+		DebugPlugin plugin = DebugPlugin.getDefault();
+        if (plugin != null) {
+            IDebugTarget[] targets = plugin.getLaunchManager().getDebugTargets();
+            for (int i = 0; i < targets.length; i++) {
+                IDebugTarget target = targets[i];
+                MultiStatus multiStatus = new MultiStatus(JDIDebugPlugin.getUniqueIdentifier(), JDIDebugPlugin.INTERNAL_ERROR, JDIDebugBreakpointMessages.JavaBreakpoint_Exception, null);
+                IJavaDebugTarget jdiTarget = (IJavaDebugTarget) target.getAdapter(IJavaDebugTarget.class);
+                if (jdiTarget != null) {
+                    try {
+                        recreate((JDIDebugTarget) jdiTarget);
+                    } catch (CoreException e) {
+                        multiStatus.add(e.getStatus());
+                    }
+                }
+                if (!multiStatus.isOK()) {
+                    throw new CoreException(multiStatus);
+                }
+            }
+        }
+    }
 	
 	/**
 	 * Recreate this breakpoint in the given target, as long as the
@@ -1112,12 +1160,24 @@ public abstract class JavaBreakpoint extends Breakpoint implements IJavaBreakpoi
 		}
 	}
 
-	/**
-	 * @see org.eclipse.debug.core.model.IBreakpoint#setEnabled(boolean)
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.model.Breakpoint#setEnabled(boolean)
 	 */
 	public void setEnabled(boolean enabled) throws CoreException {
 		super.setEnabled(enabled);
 		recreate();
 	}
 
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#supportsInstanceFilters()
+	 */
+	public boolean supportsInstanceFilters() {
+		return true;
+	}
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaBreakpoint#supportsThreadFilters()
+	 */
+	public boolean supportsThreadFilters() {
+		return true;
+	}
 }
