@@ -1,17 +1,17 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.jdt.internal.debug.core.breakpoints;
 
  
-import java.text.MessageFormat;
+import com.ibm.icu.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -23,6 +23,7 @@ import org.eclipse.core.resources.IMarker;
 import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.IWorkspaceRunnable;
 import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Status;
@@ -35,12 +36,17 @@ import org.eclipse.debug.core.model.IBreakpoint;
 import org.eclipse.debug.core.model.IDebugTarget;
 import org.eclipse.debug.core.model.ISourceLocator;
 import org.eclipse.debug.core.model.IValue;
+import org.eclipse.debug.core.sourcelookup.ISourceLookupDirector;
 import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IJavaProject;
+import org.eclipse.jdt.core.JavaCore;
 import org.eclipse.jdt.core.dom.Message;
 import org.eclipse.jdt.debug.core.IJavaDebugTarget;
 import org.eclipse.jdt.debug.core.IJavaLineBreakpoint;
 import org.eclipse.jdt.debug.core.IJavaPrimitiveValue;
+import org.eclipse.jdt.debug.core.IJavaReferenceType;
+import org.eclipse.jdt.debug.core.IJavaType;
+import org.eclipse.jdt.debug.core.JDIDebugModel;
 import org.eclipse.jdt.debug.eval.IAstEvaluationEngine;
 import org.eclipse.jdt.debug.eval.ICompiledExpression;
 import org.eclipse.jdt.debug.eval.IEvaluationListener;
@@ -62,6 +68,7 @@ import com.sun.jdi.VMDisconnectedException;
 import com.sun.jdi.event.Event;
 import com.sun.jdi.request.BreakpointRequest;
 import com.sun.jdi.request.EventRequest;
+import com.sun.jdi.request.EventRequestManager;
 
 public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreakpoint {
 
@@ -104,6 +111,12 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 	 */
 	private Map fCompiledExpressions= new HashMap();
 	
+	/**
+	 * Cache of projects for stack frames to avoid repetitive porject resolution on conditional 
+	 * breakpoints.
+	 */
+	private Map fProjectsByFrame= new HashMap();
+	
 	/**
 	 * The map of the result value of the condition (IValue) for this
 	 * breakpoint, keyed by debug target.
@@ -137,25 +150,26 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 				addLineBreakpointAttributes(attributes, getModelIdentifier(), true, lineNumber, charStart, charEnd);
 				addTypeNameAndHitCount(attributes, typeName, hitCount);
 				// set attributes
+				attributes.put(SUSPEND_POLICY, new Integer(getDefaultSuspendPolicy()));
 				ensureMarker().setAttributes(attributes);
 				
 				// add to breakpoint manager if requested
 				register(add);
 			}
 		};
-		run(wr);
+		run(getMarkerRule(resource), wr);
 	}
 	
-	/**
-	 * @see JavaBreakpoint#addToTarget(JDIDebugTarget)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#addToTarget(org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget)
 	 */
 	public void addToTarget(JDIDebugTarget target) throws CoreException {
 		clearCachedExpressionFor(target);
 		super.addToTarget(target);
 	}
-	
-	/**
-	 * @see JavaBreakpoint#removeFromTarget(JDIDebugTarget)
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#removeFromTarget(org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget)
 	 */
 	public void removeFromTarget(JDIDebugTarget target) throws CoreException {
 		clearCachedExpressionFor(target);
@@ -185,7 +199,7 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 		}
 		iter= threadsToRemove.iterator();
 		while (iter.hasNext()) {
-			map.remove((JDIThread)iter.next());
+			map.remove(iter.next());
 		}
 	}
 	
@@ -195,24 +209,41 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 	 */
 	protected void clearCachedExpressionFor(JDIDebugTarget target) {
 		removeCachedThreads(fCompiledExpressions, target);
+
+		// clean up cached projects for stack frames
+		Set frames= fProjectsByFrame.keySet();
+		List framesToRemove= new ArrayList();
+		Iterator iter= frames.iterator();
+		JDIStackFrame frame;
+		while (iter.hasNext()) {
+			frame= (JDIStackFrame)iter.next();
+			if (frame.getDebugTarget() == target) {
+				framesToRemove.add(frame);
+			}
+		}
+		iter= framesToRemove.iterator();
+		while (iter.hasNext()) {
+			fProjectsByFrame.remove(iter.next());
+		}		
+		
 	}
 	
-	/**
-	 * @see ILineBreakpoint#getLineNumber()
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.model.ILineBreakpoint#getLineNumber()
 	 */
 	public int getLineNumber() throws CoreException {
 		return ensureMarker().getAttribute(IMarker.LINE_NUMBER, -1);
 	}
 
-	/**
-	 * @see ILineBreakpoint#getCharStart()
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.model.ILineBreakpoint#getCharStart()
 	 */
 	public int getCharStart() throws CoreException {
 		return ensureMarker().getAttribute(IMarker.CHAR_START, -1);
 	}
 
-	/**
-	 * @see ILineBreakpoint#getCharEnd()
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.model.ILineBreakpoint#getCharEnd()
 	 */
 	public int getCharEnd() throws CoreException {
 		return ensureMarker().getAttribute(IMarker.CHAR_END, -1);
@@ -224,20 +255,25 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 		return JAVA_LINE_BREAKPOINT;
 	}
 		
-	/**
-	 * @see JavaBreakpoint#newRequest(JDIDebugTarget, ReferenceType)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#newRequest(org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget, com.sun.jdi.ReferenceType)
 	 */
-	protected EventRequest newRequest(JDIDebugTarget target, ReferenceType type) throws CoreException {
-		Location location= null;
-		int lineNumber= getLineNumber();			
-		location= determineLocation(lineNumber, type);
-		if (location == null) {
+	protected EventRequest[] newRequests(JDIDebugTarget target, ReferenceType type) throws CoreException {
+		int lineNumber = getLineNumber();			
+		List locations = determineLocations(lineNumber, type, target);
+		if (locations == null || locations.isEmpty()) {
 			// could be an inner type not yet loaded, or line information not available
 			return null;
 		}
-		
-		EventRequest request = createLineBreakpointRequest(location, target);	
-		return request;		
+		EventRequest[] requests = new EventRequest[locations.size()];
+		int i = 0;
+	    Iterator iterator = locations.iterator();
+	    while (iterator.hasNext()) {
+	        Location location = (Location) iterator.next();
+	        requests[i] = createLineBreakpointRequest(location, target);
+	        i++;
+	    }
+	    return requests;
 	}	
 
 	/**
@@ -246,8 +282,12 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 	 */
 	protected BreakpointRequest createLineBreakpointRequest(Location location, JDIDebugTarget target) throws CoreException {
 		BreakpointRequest request = null;
+		EventRequestManager manager = target.getEventRequestManager();
+		if (manager == null) {
+			target.requestFailed(JDIDebugBreakpointMessages.JavaLineBreakpoint_Unable_to_create_breakpoint_request___VM_disconnected__1, null);  
+		}
 		try {
-			request= target.getEventRequestManager().createBreakpointRequest(location);
+			request= manager.createBreakpointRequest(location);
 			configureRequest(request, target);
 		} catch (VMDisconnectedException e) {
 			if (!target.isAvailable()) {			
@@ -255,7 +295,7 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 			} 
 			JDIDebugPlugin.log(e);
 		} catch (RuntimeException e) {
-			JDIDebugPlugin.log(e);
+			target.internalError(e);
 			return null;
 		}
 		return request;
@@ -269,15 +309,15 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 	}
 		
 	/**
-	 * Returns a location for the line number in the given type.
-	 * Returns <code>null</code> if a location cannot be determined.
+	 * Returns a list of locations of the given line number in the given type.
+	 * Returns <code>null</code> if locations cannot be determined.
 	 */
-	protected Location determineLocation(int lineNumber, ReferenceType type) {
+	protected List determineLocations(int lineNumber, ReferenceType type, JDIDebugTarget target) {
 		List locations= null;
 		try {
 			locations= type.locationsOfLine(lineNumber);
 		} catch (AbsentInformationException aie) {
-			IStatus status= new Status(IStatus.ERROR, JDIDebugPlugin.getUniqueIdentifier(), NO_LINE_NUMBERS, JDIDebugBreakpointMessages.getString("JavaLineBreakpoint.Absent_Line_Number_Information_1"), null);  //$NON-NLS-1$
+			IStatus status= new Status(IStatus.ERROR, JDIDebugPlugin.getUniqueIdentifier(), NO_LINE_NUMBERS, JDIDebugBreakpointMessages.JavaLineBreakpoint_Absent_Line_Number_Information_1, null);  
 			IStatusHandler handler= DebugPlugin.getDefault().getStatusHandler(status);
 			if (handler != null) {
 				try {
@@ -298,15 +338,10 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 			return null;
 		} catch (RuntimeException e) {
 			// not able to retrieve line info
-			JDIDebugPlugin.log(e);
+			target.internalError(e);
 			return null;
 		}
-		
-		if (locations != null && locations.size() > 0) {
-			return (Location) locations.get(0);
-		} 
-		
-		return null;
+		return locations;
 	}
 	
 	/**
@@ -324,7 +359,7 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 	 */	
 	public void addLineBreakpointAttributes(Map attributes, String modelIdentifier, boolean enabled, int lineNumber, int charStart, int charEnd) {
 		attributes.put(IBreakpoint.ID, modelIdentifier);
-		attributes.put(IBreakpoint.ENABLED, new Boolean(enabled));
+		attributes.put(IBreakpoint.ENABLED, Boolean.valueOf(enabled));
 		attributes.put(IMarker.LINE_NUMBER, new Integer(lineNumber));
 		attributes.put(IMarker.CHAR_START, new Integer(charStart));
 		attributes.put(IMarker.CHAR_END, new Integer(charEnd)); 
@@ -348,7 +383,7 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 	}
 	
 	/**
-	 * @see JavaBreakpoint#handleBreakpointEvent(Event, JDIDebugTarget, JDIThread)
+	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#handleBreakpointEvent(com.sun.jdi.event.Event, org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget, org.eclipse.jdt.internal.debug.core.model.JDIThread)
 	 * 
 	 * (From referenced JavaDoc:
 	 * 	Returns whethers the thread should be resumed
@@ -358,12 +393,11 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 			try {
 				return handleConditionalBreakpointEvent(event, thread, target);
 			} catch (CoreException exception) {
-				// log error
+				JDIDebugPlugin.log(exception);
 				return !suspendForEvent(event, thread);
 			}
-		} else {
-			return !suspendForEvent(event, thread); // Resume if suspend fails
-		}					
+		}
+		return !suspendForEvent(event, thread); // Resume if suspend fails					
 	}
 	
 	/**
@@ -374,7 +408,7 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 			String condition = getCondition();
 			return isConditionEnabled() && condition != null && (condition.length() > 0);
 		} catch (CoreException exception) {
-			// log error
+			JDIDebugPlugin.log(exception);
 			return false;
 		}
 	}
@@ -409,52 +443,74 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 	 * the thread should resume.
 	 */
 	protected boolean handleConditionalBreakpointEvent(Event event, JDIThread thread, JDIDebugTarget target) throws CoreException {
-		if (thread.isPerformingEvaluation()) {
-			// If an evaluation is already being computed for this thread,
-			// we can't perform another
-			return !suspendForEvent(event, thread);
+		synchronized (thread) {
+			if (thread.isPerformingEvaluation()) {
+				// If an evaluation is already being computed for this thread,
+				// we can't perform another
+				return !suspendForEvent(event, thread);
+			}
+			final String condition= getCondition();
+			if (!hasCondition()) {
+				return !suspendForEvent(event, thread);
+			}
+			EvaluationListener listener= new EvaluationListener();
+	
+			int suspendPolicy= SUSPEND_THREAD;
+			try {
+				suspendPolicy= getSuspendPolicy();
+			} catch (CoreException e) {
+			}
+			if (suspendPolicy == SUSPEND_VM) {
+				((JDIDebugTarget)thread.getDebugTarget()).prepareToSuspendByBreakpoint(this);
+			} else {
+				thread.handleSuspendForBreakpointQuiet(this);
+			}
+			List frames = thread.computeNewStackFrames();
+			if (frames.size() == 0) {
+				return !suspendForEvent(event, thread);
+			}
+			JDIStackFrame frame= (JDIStackFrame)frames.get(0);
+			IJavaProject project= getJavaProject(frame);
+			if (project == null) {
+				throw new CoreException(new Status(IStatus.ERROR, JDIDebugModel.getPluginIdentifier(), DebugException.REQUEST_FAILED,
+					JDIDebugBreakpointMessages.JavaLineBreakpoint_Unable_to_compile_conditional_breakpoint___missing_Java_project_context__1, null)); 
+			}
+			IAstEvaluationEngine engine = getEvaluationEngine(target, project);
+			if (engine == null) {
+				// If no engine is available, suspend
+				return !suspendForEvent(event, thread);
+			}
+			ICompiledExpression expression= (ICompiledExpression)fCompiledExpressions.get(thread);
+			if (expression == null) {
+				expression= engine.getCompiledExpression(condition, frame);
+				fCompiledExpressions.put(thread, expression);
+			}
+			if (conditionHasErrors(expression)) {
+				fireConditionHasErrors(expression);
+				return !suspendForEvent(event, thread);
+			}
+			fSuspendEvents.put(thread, event);
+            thread.setEvaluatingConditionalBreakpoint(true);
+			engine.evaluateExpression(expression, frame, listener, DebugEvent.EVALUATION_IMPLICIT, false);
+	
+			// Do not resume. When the evaluation returns, the evaluation listener
+			// will resume the thread if necessary or update for suspension.
+			return false;
 		}
-		final String condition= getCondition();
-		if (!hasCondition()) {
-			return !suspendForEvent(event, thread);
-		}
-		EvaluationListener listener= new EvaluationListener();
-
-		int suspendPolicy= SUSPEND_THREAD;
-		try {
-			suspendPolicy= getSuspendPolicy();
-		} catch (CoreException e) {
-		}
-		if (suspendPolicy == SUSPEND_VM) {
-			((JDIDebugTarget)thread.getDebugTarget()).prepareToSuspendByBreakpoint(this);
-		} else {
-			thread.handleSuspendForBreakpointQuiet(this);
-		}
-		JDIStackFrame frame= (JDIStackFrame)thread.computeNewStackFrames().get(0);
-		IJavaProject project= getJavaProject(frame);
-		IAstEvaluationEngine engine = getEvaluationEngine(target, project);
-		if (engine == null) {
-			// If no engine is available, suspend
-			return !suspendForEvent(event, thread);
-		}
-		ICompiledExpression expression= (ICompiledExpression)fCompiledExpressions.get(thread);
-		if (expression == null) {
-			expression= engine.getCompiledExpression(condition, frame);
-			fCompiledExpressions.put(thread, expression);
-		}
-		if (conditionHasErrors(expression)) {
-			fireConditionHasErrors(expression);
-			return !suspendForEvent(event, thread);
-		}
-		fSuspendEvents.put(thread, event);
-		engine.evaluateExpression(expression, frame, listener, DebugEvent.EVALUATION_IMPLICIT, false);
-
-		// Do not resume. When the evaluation returns, the evaluation listener
-		// will resume the thread if necessary or update for suspension.
-		return false;
 	}
 	
 	private IJavaProject getJavaProject(JDIStackFrame stackFrame) {
+	    IJavaProject project= (IJavaProject) fProjectsByFrame.get(stackFrame);
+	    if (project == null) {
+	        project = computeJavaProject(stackFrame);
+	        if (project != null) {
+	            fProjectsByFrame.put(stackFrame, project);
+	        }
+	    }
+	    return project;
+	}
+	
+	private IJavaProject computeJavaProject(JDIStackFrame stackFrame) {
 		ILaunch launch = stackFrame.getLaunch();
 		if (launch == null) {
 			return null;
@@ -463,10 +519,37 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 		if (locator == null)
 			return null;
 		
-		Object sourceElement = locator.getSourceElement(stackFrame);
+		Object sourceElement= null;
+		try {
+			if (locator instanceof ISourceLookupDirector && !stackFrame.isStatic()) {
+				IJavaType thisType = stackFrame.getThis().getJavaType();
+				if (thisType instanceof IJavaReferenceType) {
+					String[] sourcePaths= ((IJavaReferenceType) thisType).getSourcePaths(null);
+					if (sourcePaths != null && sourcePaths.length > 0) {
+						sourceElement= ((ISourceLookupDirector) locator).getSourceElement(sourcePaths[0]);
+					}
+				}
+			}
+		} catch (DebugException e) {
+			DebugPlugin.log(e);
+		}
+		if (sourceElement == null) {
+			sourceElement = locator.getSourceElement(stackFrame);
+		}
+		if (!(sourceElement instanceof IJavaElement) && sourceElement instanceof IAdaptable) {
+			Object element= ((IAdaptable)sourceElement).getAdapter(IJavaElement.class);
+			if (element != null) {
+				sourceElement= element;
+			}
+		}
 		if (sourceElement instanceof IJavaElement) {
 			return ((IJavaElement) sourceElement).getJavaProject();
-		}			
+		} else if (sourceElement instanceof IResource) {
+			IJavaProject project = JavaCore.create(((IResource)sourceElement).getProject());
+			if (project.exists()) {
+				return project;
+			}
+		}
 		return null;
 	}
 	
@@ -479,12 +562,13 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 	class EvaluationListener implements IEvaluationListener {
 		public void evaluationComplete(IEvaluationResult result) {
 			JDIThread thread= (JDIThread)result.getThread();
+            thread.setEvaluatingConditionalBreakpoint(false);
 			Event event= (Event)fSuspendEvents.get(thread);
 			if (result.hasErrors()) {
 				DebugException exception= result.getException();
 				Throwable wrappedException= exception.getStatus().getException();
 				if (wrappedException instanceof VMDisconnectedException) {
-					JDIDebugPlugin.log((VMDisconnectedException)wrappedException);
+					JDIDebugPlugin.log(wrappedException);
 					try {
 						thread.resumeQuiet();
 					} catch(DebugException e) {
@@ -535,7 +619,7 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 			// Suspend when an error occurs
 			suspendForEvent(event, thread);
 		}
-	};
+	}
 	
 	private void fireConditionHasRuntimeErrors(DebugException exception) {
 		JDIDebugPlugin.getDefault().fireBreakpointHasRuntimeException(this, exception);
@@ -578,22 +662,22 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 		return ((JDIDebugTarget)vm).getEvaluationEngine(project);
 	}
 	
-	/**
-	 * @see IJavaLineBreakpoint#supportsCondition
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaLineBreakpoint#supportsCondition()
 	 */
 	public boolean supportsCondition() {
 		return true;
 	}
 	
-	/**
-	 * @see IJavaLineBreakpoint#getCondition()
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaLineBreakpoint#getCondition()
 	 */
 	public String getCondition() throws CoreException {
 		return ensureMarker().getAttribute(CONDITION, null);
 	}
 	
-	/**
-	 * @see IJavaLineBreakpoint#setCondition(String)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaLineBreakpoint#setCondition(java.lang.String)
 	 */
 	public void setCondition(String condition) throws CoreException {
 		// Clear the cached compiled expressions
@@ -607,34 +691,35 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 		recreate();
 	}
 
-	protected String getMarkerMessage(boolean conditionEnabled, String condition, int hitCount, int suspendPolicy, int lineNumber) throws CoreException {
+	protected String getMarkerMessage(boolean conditionEnabled, String condition, int hitCount, int suspendPolicy, int lineNumber) {
 		StringBuffer message= new StringBuffer(super.getMarkerMessage(hitCount, suspendPolicy));
 		if (lineNumber != -1) {
-			message.append(MessageFormat.format(JDIDebugBreakpointMessages.getString("JavaLineBreakpoint._[line__{0}]_1"), new Object[]{Integer.toString(lineNumber)})); //$NON-NLS-1$
+			message.append(MessageFormat.format(JDIDebugBreakpointMessages.JavaLineBreakpoint___line___0___1, new Object[]{Integer.toString(lineNumber)})); 
 		}
 		if (conditionEnabled && condition != null) {
-			message.append(MessageFormat.format(JDIDebugBreakpointMessages.getString("JavaLineBreakpoint._[Condition__{0}]_2"), new Object[]{condition})); //$NON-NLS-1$
+			message.append(MessageFormat.format(JDIDebugBreakpointMessages.JavaLineBreakpoint___Condition___0___2, new Object[]{condition})); 
 		}
 			
 		return message.toString();
 	}
-	
-	/**
-	 * @see IJavaLineBreakpoint#isConditionEnabled()
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaLineBreakpoint#isConditionEnabled()
 	 */
 	public boolean isConditionEnabled() throws CoreException {
 		return ensureMarker().getAttribute(CONDITION_ENABLED, false);
 	}
 	
-	/**
-	 * @see IJavaLineBreakpoint#setConditionEnabled(boolean)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaLineBreakpoint#setConditionEnabled(boolean)
 	 */
 	public void setConditionEnabled(boolean conditionEnabled) throws CoreException {	
-		setAttributes(new String[]{CONDITION_ENABLED}, new Object[]{new Boolean(conditionEnabled)});
+		setAttributes(new String[]{CONDITION_ENABLED}, new Object[]{Boolean.valueOf(conditionEnabled)});
 		recreate();
 	}
-	/**
-	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#cleanupForThreadTermination(JDIThread)
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#cleanupForThreadTermination(org.eclipse.jdt.internal.debug.core.model.JDIThread)
 	 */
 	protected void cleanupForThreadTermination(JDIThread thread) {
 		fSuspendEvents.remove(thread);
@@ -642,8 +727,8 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 		super.cleanupForThreadTermination(thread);
 	}
 	
-	/**
-	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#addInstanceFilter(EventRequest, ObjectReference)
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#addInstanceFilter(com.sun.jdi.request.EventRequest, com.sun.jdi.ObjectReference)
 	 */
 	protected void addInstanceFilter(EventRequest request,ObjectReference object) {
 		if (request instanceof BreakpointRequest) {
@@ -651,19 +736,19 @@ public class JavaLineBreakpoint extends JavaBreakpoint implements IJavaLineBreak
 		}
 	}
 
-	/**
+	/* (non-Javadoc)
 	 * @see org.eclipse.jdt.debug.core.IJavaLineBreakpoint#isConditionSuspendOnTrue()
 	 */
 	public boolean isConditionSuspendOnTrue() throws DebugException {
 		return ensureMarker().getAttribute(CONDITION_SUSPEND_ON_TRUE, true);
 	}
 
-	/**
+	/* (non-Javadoc)
 	 * @see org.eclipse.jdt.debug.core.IJavaLineBreakpoint#setConditionSuspendOnTrue(boolean)
 	 */
 	public void setConditionSuspendOnTrue(boolean suspendOnTrue) throws CoreException {
 		if (isConditionSuspendOnTrue() != suspendOnTrue) {
-			setAttributes(new String[]{CONDITION_SUSPEND_ON_TRUE}, new Object[]{new Boolean(suspendOnTrue)});
+			setAttributes(new String[]{CONDITION_SUSPEND_ON_TRUE}, new Object[]{Boolean.valueOf(suspendOnTrue)});
 			fConditionValues.clear();
 			recreate();
 		}
