@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,6 +13,7 @@ package org.eclipse.jdt.internal.debug.core.breakpoints;
 
 import java.util.HashMap;
 import java.util.Map;
+import java.util.regex.Pattern;
 
 import org.eclipse.core.resources.IMarker;
 import org.eclipse.core.resources.IResource;
@@ -22,7 +23,6 @@ import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.debug.core.model.IDebugTarget;
 import org.eclipse.jdt.debug.core.IJavaMethodBreakpoint;
 import org.eclipse.jdt.internal.debug.core.JDIDebugPlugin;
-import org.eclipse.jdt.internal.debug.core.StringMatcher;
 import org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget;
 import org.eclipse.jdt.internal.debug.core.model.JDIThread;
 
@@ -40,6 +40,7 @@ import com.sun.jdi.event.MethodEntryEvent;
 import com.sun.jdi.event.MethodExitEvent;
 import com.sun.jdi.request.BreakpointRequest;
 import com.sun.jdi.request.EventRequest;
+import com.sun.jdi.request.EventRequestManager;
 import com.sun.jdi.request.MethodEntryRequest;
 import com.sun.jdi.request.MethodExitRequest;
 
@@ -117,12 +118,12 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 	 * <li>Method exit (value <code>EXIT_EVENT</code>)</li>
 	 * </ol>
 	 */
-	private Map fLastEventTypes= new HashMap(10); //$NON-NLS-1$
+	private Map fLastEventTypes= new HashMap(10); 
 	
 	/**
 	 * Used to match type names 
 	 */
-	private StringMatcher fMatcher;
+	private Pattern fPattern;
 	
 	/**
 	 * Cache of whether this breakpoint uses a type name pattern
@@ -145,19 +146,19 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 				addLineBreakpointAttributes(attributes, getModelIdentifier(), true, lineNumber, charStart, charEnd);
 				addMethodNameAndSignature(attributes, methodName, methodSignature);
 				addTypeNameAndHitCount(attributes, typePattern, hitCount);
-				attributes.put(ENTRY, new Boolean(entry));
-				attributes.put(EXIT, new Boolean(exit));
-				attributes.put(NATIVE, new Boolean(nativeOnly));
-				
+				attributes.put(ENTRY, Boolean.valueOf(entry));
+				attributes.put(EXIT, Boolean.valueOf(exit));
+				attributes.put(NATIVE, Boolean.valueOf(nativeOnly));
+				attributes.put(SUSPEND_POLICY, new Integer(getDefaultSuspendPolicy()));
 				//set attributes
 				ensureMarker().setAttributes(attributes);
-				
 				register(register);
 			}
-
+			
 		};
-		run(wr);
-		fMatcher= new StringMatcher(typePattern, false, false);
+		run(getMarkerRule(resource), wr);
+		String type = convertToRegularExpression(typePattern);
+		fPattern = Pattern.compile(type);
 	}
 	
 	/**
@@ -246,6 +247,10 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 	 */
 	private EventRequest createMethodRequest(JDIDebugTarget target, Object classFilter, boolean entry) throws CoreException {
 		EventRequest request = null;
+		EventRequestManager manager = target.getEventRequestManager();
+		if (manager == null) {
+			target.requestFailed(JDIDebugBreakpointMessages.JavaMethodBreakpoint_Unable_to_create_breakpoint_request___VM_disconnected__1, null);  
+		}		
 		try {
 			if (entry) {
 				if (classFilter instanceof ClassType && getMethodName() != null && getMethodSignature() != null) {
@@ -257,13 +262,13 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 						if (method != null && !method.isNative()) {
 							Location location = method.location();
 							if (location != null && location.codeIndex() != -1) {
-								request = target.getEventRequestManager().createBreakpointRequest(location);
+								request = manager.createBreakpointRequest(location);
 							}
 						}
 					}
 				}
 				if (request == null) {
-					request= target.getEventRequestManager().createMethodEntryRequest();
+					request = manager.createMethodEntryRequest();
 					if (classFilter instanceof String) {
 						((MethodEntryRequest)request).addClassFilter((String)classFilter);
 					} else if (classFilter instanceof ReferenceType) {
@@ -271,7 +276,7 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 					}
 				}
 			} else {
-				request= target.getEventRequestManager().createMethodExitRequest();
+				request= manager.createMethodExitRequest();
 				if (classFilter instanceof String) {
 					((MethodExitRequest)request).addClassFilter((String)classFilter);
 				} else if (classFilter instanceof ReferenceType) {
@@ -285,7 +290,7 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 			}
 			JDIDebugPlugin.log(e);
 		} catch (RuntimeException e) {
-			JDIDebugPlugin.log(e);
+			target.internalError(e);
 		}			
 		return request;
 	}
@@ -321,7 +326,7 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 	}
 	
 	/**
-	 * @see JavaBreakpoint#updateEnabledState(EventRequest)
+	 * @see JavaBreakpoint#updateEnabledState(EventRequest, JDIDebugTarget)
 	 */
 	protected void updateEnabledState(EventRequest request, JDIDebugTarget target) throws CoreException  {
 		boolean enabled= isEnabled();
@@ -404,26 +409,36 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 			}
 			
 			if (getMethodSignature() != null) {
-				if (!method.signature().equals(getMethodSignature())) {
+				String sig = method.signature();
+				if(sig.indexOf('$') > -1) {
+					sig = sig.replace('$', '.');
+				}
+				if (!sig.equals(getMethodSignature())) {
 					return true;
 				}
 			}
 			
-			if (fMatcher != null) {
-				if (!fMatcher.match(method.declaringType().name())) {
+			if (fPattern != null) {
+				if (!fPattern.matcher(method.declaringType().name()).find()) {
 					return true;
 				}
 			}
 			
 			// simulate hit count
 			Integer count = (Integer)event.request().getProperty(HIT_COUNT);
-			if (count != null) {
-				return handleHitCount(event, count, thread);
-			} else {
-				// no hit count - suspend
-				return !suspend(thread); // resume if suspend fails
+			if (count != null && handleHitCount(event, count)) {
+				return true;
 			}
-			
+			// no hit count
+			if (hasCondition()) {
+				try {
+					return handleConditionalBreakpointEvent(event, thread, target);
+				} catch (CoreException exception) {
+					// log error
+					return !suspendForEvent(event, thread);
+				}
+			}
+			return !suspendForEvent(event, thread); // Resume if suspend fails					
 		} catch (CoreException e) {
 			JDIDebugPlugin.log(e);
 		}
@@ -435,7 +450,7 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 	 * When a method event is received, decrement the hit count
 	 * property on the request and suspend if the hit count reaches 0.
 	 */
-	private boolean handleHitCount(LocatableEvent event, Integer count, JDIThread thread) {	
+	private boolean handleHitCount(LocatableEvent event, Integer count) {	
 	// decrement count and suspend if 0
 		int hitCount = count.intValue();
 		if (hitCount > 0) {
@@ -444,7 +459,6 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 			event.request().putProperty(HIT_COUNT, count);
 			if (hitCount == 0) {
 				// the count has reached 0, breakpoint hit
-				boolean resume = !suspend(thread); // resume if suspend fails
 				try {
 					// make a note that we auto-disabled the breakpoint
 					// order is important here...see methodEntryChanged
@@ -453,26 +467,24 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 				} catch (CoreException e) {
 					JDIDebugPlugin.log(e);
 				}
-				return resume;
-			}  else {
-				// count still > 0, keep running
-				return true;		
+				return false;
 			}
-		} else {
-			// hit count expired, keep running
-			return true;
+			// count still > 0, keep running
+			return true;		
 		}
+		// hit count expired, keep running
+		return true;
 	}
 	
-	/**
-	 * @see IJavaMethodEntryBreakpoint#getMethodName()		
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaMethodEntryBreakpoint#getMethodName()		
 	 */
 	public String getMethodName() {
 		return fMethodName;
 	}
 	
-	/**
-	 * @see IJavaMethodEntryBreakpoint#getMethodSignature()		
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaMethodEntryBreakpoint#getMethodSignature()		
 	 */
 	public String getMethodSignature() {
 		return fMethodSignature;
@@ -542,7 +554,7 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 	/**
 	 * Initialize cache of attributes
 	 * 
-	 * @see IBreakpoint#setMarker(IMarker)
+	 * @see org.eclipse.debug.core.model.IBreakpoint#setMarker(IMarker)
 	 */
 	public void setMarker(IMarker marker) throws CoreException {
 		super.setMarker(marker);
@@ -550,16 +562,27 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 		fMethodSignature = marker.getAttribute(METHOD_SIGNATURE, null);
 		String typePattern= marker.getAttribute(TYPE_NAME, ""); //$NON-NLS-1$
 		if (typePattern != null) {
-			fMatcher= new StringMatcher(typePattern, false, false);
+			fPattern = Pattern.compile(convertToRegularExpression(typePattern));
 		}
-		
 	}	
 	
 	/**
-	 * @see IBreakpoint#setEnabled(boolean)
-	 * 
+	 * converts the specified string to one which has been formated to our needs
+	 * @param stringMatcherPattern the initial pattern
+	 * @return the modified pattern
+	 */
+	private String convertToRegularExpression(String stringMatcherPattern) {
+	    String regex = stringMatcherPattern.replaceAll("\\.", "\\\\.");  //$NON-NLS-1$//$NON-NLS-2$
+	    regex = regex.replaceAll("\\*", "\\.\\*");  //$NON-NLS-1$//$NON-NLS-2$
+	    regex = regex.replaceAll("\\$", "\\\\\\$"); //$NON-NLS-1$ //$NON-NLS-2$
+	    return regex;
+	}
+	
+	/**
 	 * If this breakpoint is not entry or exit enabled,
 	 * set the default (entry)
+	 * 
+	 * @see org.eclipse.debug.core.model.IBreakpoint#setEnabled(boolean)
 	 */
 	public void setEnabled(boolean enabled) throws CoreException {		
 		if (enabled) {
@@ -584,11 +607,11 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 		setAttributes(attributes, values);
 	}
 	
-	/**
-	 * @see IJavaLineBreakpoint#supportsCondition()
+	/* (non-Javadoc)
+	 * @see org.eclipse.jdt.debug.core.IJavaLineBreakpoint#supportsCondition()
 	 */
 	public boolean supportsCondition() {
-		return false;
+		return true;
 	}
 
 	/**
@@ -625,7 +648,7 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 	protected boolean usesTypePattern() throws CoreException {
 		if (fUsesTypePattern == null) {
 			String name = getTypeName();
-			fUsesTypePattern= new Boolean(name != null && (name.startsWith("*") || name.endsWith("*"))); //$NON-NLS-1$ //$NON-NLS-2$
+			fUsesTypePattern= Boolean.valueOf(name != null && (name.startsWith("*") || name.endsWith("*"))); //$NON-NLS-1$ //$NON-NLS-2$
 		}
 		return fUsesTypePattern.booleanValue();
 	}
@@ -637,17 +660,16 @@ public class JavaMethodBreakpoint extends JavaLineBreakpoint implements IJavaMet
 	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#createRequest(JDIDebugTarget, ReferenceType)
 	 */
 	protected boolean createRequest(JDIDebugTarget target, ReferenceType type) throws CoreException {
-		if (type.name().equals(getTypeName())) {
-			EventRequest entryRequest= createMethodEntryRequest(target, type);
-			EventRequest exitRequest= createMethodExitRequest(target, type);
-			
-			registerRequest(entryRequest, target);
-			registerRequest(exitRequest, target);
-			return true;
-		} else {
-			// do not create requests for inner/outer types if since this is for a specific type
+		if (!type.name().equals(getTypeName()) || shouldSkipBreakpoint()) {
+			// do not create requests for inner/outer types if this is for a specific type
 			return false;
 		}
+		EventRequest entryRequest= createMethodEntryRequest(target, type);
+		EventRequest exitRequest= createMethodExitRequest(target, type);
+		
+		registerRequest(entryRequest, target);
+		registerRequest(exitRequest, target);
+		return true;
 	}
 	/**
 	 * @see org.eclipse.jdt.internal.debug.core.breakpoints.JavaBreakpoint#setTypeName(String)
