@@ -1,18 +1,19 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.jdt.internal.debug.core.hcr;
 
 
-import java.text.MessageFormat;
+import com.ibm.icu.text.MessageFormat;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Date;
 import java.util.HashMap;
@@ -29,13 +30,13 @@ import org.eclipse.core.resources.IResourceChangeListener;
 import org.eclipse.core.resources.IResourceDelta;
 import org.eclipse.core.resources.IResourceDeltaVisitor;
 import org.eclipse.core.resources.IWorkspace;
-import org.eclipse.core.resources.IWorkspaceRunnable;
+import org.eclipse.core.resources.IncrementalProjectBuilder;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.IPath;
-import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.ListenerList;
 import org.eclipse.core.runtime.MultiStatus;
 import org.eclipse.core.runtime.Path;
 import org.eclipse.core.runtime.Status;
@@ -49,8 +50,8 @@ import org.eclipse.debug.core.ILaunchManager;
 import org.eclipse.debug.core.model.IDebugTarget;
 import org.eclipse.debug.core.model.ISourceLocator;
 import org.eclipse.debug.core.model.IThread;
-import org.eclipse.debug.internal.core.ListenerList;
 import org.eclipse.jdt.core.ICompilationUnit;
+import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IJavaModelMarker;
 import org.eclipse.jdt.core.IJavaProject;
 import org.eclipse.jdt.core.IMethod;
@@ -64,14 +65,18 @@ import org.eclipse.jdt.core.util.ISourceAttribute;
 import org.eclipse.jdt.debug.core.IJavaDebugTarget;
 import org.eclipse.jdt.debug.core.IJavaHotCodeReplaceListener;
 import org.eclipse.jdt.debug.core.IJavaStackFrame;
+import org.eclipse.jdt.debug.core.IJavaThread;
 import org.eclipse.jdt.debug.core.JDIDebugModel;
-import org.eclipse.jdt.internal.core.Util;
+import org.eclipse.jdt.internal.core.util.Util;
 import org.eclipse.jdt.internal.debug.core.JDIDebugPlugin;
+import org.eclipse.jdt.internal.debug.core.JavaDebugUtils;
 import org.eclipse.jdt.internal.debug.core.model.JDIDebugTarget;
 import org.eclipse.jdt.internal.debug.core.model.JDIStackFrame;
 import org.eclipse.jdt.internal.debug.core.model.JDIThread;
 
+import com.sun.jdi.IncompatibleThreadStateException;
 import com.sun.jdi.ReferenceType;
+import com.sun.jdi.VirtualMachine;
 
 /**
  * The hot code replace manager listens for changes to
@@ -93,13 +98,13 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 	 * The list of <code>IJavaHotCodeReplaceListeners</code> which this hot code replace 
 	 * manager will notify about hot code replace attempts.
 	 */
-	private ListenerList fHotCodeReplaceListeners= new ListenerList(1);
+	private ListenerList fHotCodeReplaceListeners= new ListenerList();
 	
 	/**
 	 * The lists of hot swap targets which support HCR and those which don't
 	 */
-	private List fHotSwapTargets= new ArrayList(1);
-	private List fNoHotSwapTargets= new ArrayList(1);
+	private ArrayList fHotSwapTargets= new ArrayList(1);
+	private ArrayList fNoHotSwapTargets= new ArrayList(1);
 	
 	/**
 	 * A mapping of the last time projects were built.
@@ -110,6 +115,12 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 	 */
 	private Map fProjectBuildTimes= new HashMap();
 	private static Date fStartupDate= new Date();
+	
+	/**
+	 * Cache of compilation unit deltas renewed on each HCR attempt.
+	 */
+	private Map fDeltaCache = new HashMap();
+	
 	/**
 	 * Utility object used for tracking build times of projects.
 	 * The HCR manager receives notification of builds AFTER
@@ -146,8 +157,6 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 		}
 	}
 	
-	protected BuiltProjectVisitor fProjectVisitor= new BuiltProjectVisitor();
-	
 	/**
 	 * Visitor for resource deltas.
 	 */
@@ -185,7 +194,7 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 		DebugPlugin.getDefault().getLaunchManager().removeLaunchListener(this);
 		DebugPlugin.getDefault().removeDebugEventListener(this);
 		getWorkspace().removeResourceChangeListener(this);
-		fHotCodeReplaceListeners.removeAll();
+		fHotCodeReplaceListeners = new ListenerList();
 		fHotSwapTargets= null;
 		fNoHotSwapTargets= null;
 	}
@@ -210,6 +219,10 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 		if (!projects.isEmpty()) {
 			updateProjectBuildTime(projects);
 		}
+		if (fHotSwapTargets.isEmpty() && fNoHotSwapTargets.isEmpty()) {
+			// If there are no targets to notify, only update the build times.
+			return;
+		}
 		ChangedClassFilesVisitor visitor = getChangedClassFiles(event);
 		if (visitor != null) {
 			List resources = visitor.getChangedClassFiles();
@@ -225,17 +238,24 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 	 */
 	protected List getBuiltProjects(IResourceChangeEvent event) {
 		IResourceDelta delta= event.getDelta();
-		if (event.getType() != IResourceChangeEvent.POST_AUTO_BUILD || delta == null) {
+		if (event.getType() != IResourceChangeEvent.POST_BUILD || delta == null || event.getBuildKind() == 0) {
 			return Collections.EMPTY_LIST;
 		}
-		fProjectVisitor.reset();
-		try {
-			delta.accept(fProjectVisitor);
-		} catch (CoreException e) {
-			JDIDebugPlugin.log(e);
+		if (event.getBuildKind() == IncrementalProjectBuilder.AUTO_BUILD && !ResourcesPlugin.getWorkspace().isAutoBuilding()) {
+			// If this is an auto build and the workspace is not autobuilding,
+			// no projects will actually be compiled.
 			return Collections.EMPTY_LIST;
 		}
-		return fProjectVisitor.getBuiltProjects();
+		Object source = event.getSource();
+		if (source instanceof IProject) {
+			List list= new ArrayList();
+			list.add(source);
+			return list;
+		} else if (source instanceof IWorkspace){
+			IProject[] allProjects = ((IWorkspace) source).getRoot().getProjects();
+			return Arrays.asList(allProjects);
+		}
+		return Collections.EMPTY_LIST;
 	}
 	
 	/**
@@ -281,24 +301,49 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 		final List hotSwapTargets= getHotSwapTargets();
 		final List noHotSwapTargets= getNoHotSwapTargets();
 		if (!hotSwapTargets.isEmpty()) {
-			IWorkspaceRunnable wRunnable= new IWorkspaceRunnable() {
-				public void run(IProgressMonitor monitor) {
+			Runnable runnable= new Runnable() {
+				public void run() {
 					doHotCodeReplace(hotSwapTargets, resources, qualifiedNames);
 				}
 			};
-			fork(wRunnable);	
+			DebugPlugin.getDefault().asyncExec(runnable);	
 		}
 		if (!noHotSwapTargets.isEmpty()) {
-			IWorkspaceRunnable wRunnable= new IWorkspaceRunnable() {
-				public void run(IProgressMonitor monitor) {
+			Runnable runnable= new Runnable() {
+				public void run() {
 					notifyUnsupportedHCR(noHotSwapTargets, resources, qualifiedNames);
 				}
 			};
-			fork(wRunnable);
+			DebugPlugin.getDefault().asyncExec(runnable);
 		}
 	}
 	
 	/**
+	 * Filters elements out of the given collections of resources and qualified names
+	 * if there is no type corresponding tyep loaded in the given debug target. This
+	 * method allows us to avoid bogus HCR attempts and "HCR failed" notifications.
+	 *   
+     * @param target the debug target
+     * @param resources the list of resources to filter
+     * @param qualifiedNames the list of qualified names to filter, which corresponds
+     *  to the list of resources on a one-to-one-basis
+     */
+    private void filterUnloadedTypes(JDIDebugTarget target, List resources, List qualifiedNames) {
+		for (int i= 0, numElements= qualifiedNames.size(); i < numElements; i++) {
+		    String name= (String) qualifiedNames.get(i);
+		    List list = target.jdiClassesByName(name);
+		    if (list.isEmpty()) {
+		        // If no classes with the given name are loaded in the VM, don't waste
+		        // cycles trying to replace.
+		        qualifiedNames.remove(i);
+		        resources.remove(i);
+		        // Decrement the index and number of elements to compensate for item removal
+		        i--;
+		        numElements--;
+		    }
+	    }
+    }
+    /**
 	 * Notify the given targets that HCR failed for classes
 	 * with the given fully qualified names.
 	 */
@@ -306,13 +351,28 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 		Iterator iter= targets.iterator();
 		JDIDebugTarget target= null;
 		while (iter.hasNext()) {	
-			target= (JDIDebugTarget) iter.next();	
-			fireHCRFailed(target, null);
-			notifyFailedHCR(target, resources, qualifiedNames);
+			target= (JDIDebugTarget) iter.next();
+			if (target.isAvailable()) {
+			    // Make a local copy of the resources/names to swap so we can filter
+			    // unloaded types on a per-target basis.
+			    List resourcesToReplace= new ArrayList(resources);
+			    List qualifiedNamesToReplace= new ArrayList(qualifiedNames);
+				filterUnloadedTypes(target, resourcesToReplace, qualifiedNamesToReplace);
+				
+				if (!qualifiedNamesToReplace.isEmpty()) {
+				    // Don't notify if the changed types aren't loaded.
+				    fireHCRFailed(target, null);
+				    notifyFailedHCR(target, qualifiedNamesToReplace);
+				}
+			} else {
+			    // Targets should be unregistered when they terminate,
+			    // but this is a fallback.
+			    deregisterTarget(target);
+			}
 		}
 	}
 	
-	protected void notifyFailedHCR(JDIDebugTarget target, List resources, List qualifiedNames) {
+	protected void notifyFailedHCR(JDIDebugTarget target, List qualifiedNames) {
 		if (target.isAvailable()) {
 			target.addOutOfSynchTypes(qualifiedNames);
 			target.fireChangeEvent(DebugEvent.STATE);
@@ -324,7 +384,7 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 	 * hot code replace.
 	 */
 	protected List getHotSwapTargets() {
-		return fHotSwapTargets;
+		return (List) fHotSwapTargets.clone();
 	}
 	
 	/**
@@ -332,7 +392,7 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 	 * not support hot code replace.
 	 */
 	protected List getNoHotSwapTargets() {
-		return fNoHotSwapTargets;
+		return (List) fNoHotSwapTargets.clone();
 	}
 	
 	/**
@@ -355,13 +415,24 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 	 * @param resources the resources which correspond to the changed classes
 	 */
 	private void doHotCodeReplace(List targets, List resources, List qualifiedNames) {
-		MultiStatus ms= new MultiStatus(JDIDebugPlugin.getUniqueIdentifier(), DebugException.TARGET_REQUEST_FAILED, JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.drop_to_frame_failed"), null); //$NON-NLS-1$
+		MultiStatus ms= new MultiStatus(JDIDebugPlugin.getUniqueIdentifier(), DebugException.TARGET_REQUEST_FAILED, JDIDebugHCRMessages.JavaHotCodeReplaceManager_drop_to_frame_failed, null); 
 		Iterator iter= targets.iterator();
 		while (iter.hasNext()) {
 			JDIDebugTarget target= (JDIDebugTarget) iter.next();
 			if (!target.isAvailable()) {
+			    deregisterTarget(target);
 				continue;
 			}
+		    // Make a local copy of the resources/names to swap so we can filter
+		    // unloaded types on a per-target basis.
+		    List resourcesToReplace= new ArrayList(resources);
+		    List qualifiedNamesToReplace= new ArrayList(qualifiedNames);
+			filterUnloadedTypes(target, resourcesToReplace, qualifiedNamesToReplace);
+			if (qualifiedNamesToReplace.isEmpty()) {
+			    // If none of the changed types are loaded, do nothing.
+			    continue;
+			}
+			
 			List poppedThreads= new ArrayList();
 			target.setIsPerformingHotCodeReplace(true);
 			try {
@@ -373,37 +444,41 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 					// Thus, pop the frames that contain affected methods
 					// *before* the class redefinition to avoid problems.
 					try {
-						attemptPopFrames(target, resources, qualifiedNames, poppedThreads);
+						attemptPopFrames(target, resourcesToReplace, qualifiedNamesToReplace, poppedThreads);
 						framesPopped= true; // No exception occurred
 					} catch (DebugException de) {
-						ms.merge(de.getStatus());
+					    if (shouldLogHCRException(de)) {
+					        ms.merge(de.getStatus());
+					    }
 					}
 				}
-				target.removeOutOfSynchTypes(qualifiedNames);
+				target.removeOutOfSynchTypes(qualifiedNamesToReplace);
 				if (target.supportsJDKHotCodeReplace()) {
-					redefineTypesJDK(target, resources, qualifiedNames);
+					redefineTypesJDK(target, resourcesToReplace, qualifiedNamesToReplace);
 				} else if (target.supportsJ9HotCodeReplace()) {
-					redefineTypesJ9(target, qualifiedNames);
+					redefineTypesJ9(target, qualifiedNamesToReplace);
 				}
 				if (containsObsoleteMethods(target)) {
 					fireObsoleteMethods(target);
 				}
-				if (target.canPopFrames() && framesPopped) {
-					// Second half of JDK 1.4 drop to frame support:
-					// All affected frames have been popped and the classes
-					// have been reloaded. Step into the first changed
-					// frame of each affected thread.
-					try {
-						// must re-set 'is doing HCR' to be able to step
-						target.setIsPerformingHotCodeReplace(false);
-						attemptStepIn(poppedThreads);
-					} catch (DebugException de) {
-						ms.merge(de.getStatus());
+				try {
+					if (target.canPopFrames() && framesPopped) {
+						// Second half of JDK 1.4 drop to frame support:
+						// All affected frames have been popped and the classes
+						// have been reloaded. Step into the first changed
+						// frame of each affected thread.
+							// must re-set 'is doing HCR' to be able to step
+							target.setIsPerformingHotCodeReplace(false);
+							attemptStepIn(poppedThreads);
+					} else {
+						// J9 drop to frame support:
+						// After redefining classes, drop to frame
+						attemptDropToFrame(target, resourcesToReplace, qualifiedNamesToReplace);
 					}
-				} else {
-					// J9 drop to frame support:
-					// After redefining classes, drop to frame
-					attemptDropToFrame(target, resources, qualifiedNames);
+				} catch (DebugException de) {
+				    if (shouldLogHCRException(de)) {
+				        ms.merge(de.getStatus());
+				    }
 				}
 				fireHCRSucceeded(target);
 			} catch (DebugException de) {
@@ -417,6 +492,19 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 		if (!ms.isOK()) {
 			JDIDebugPlugin.log(ms);
 		}
+		fDeltaCache.clear();
+	}
+	
+	/**
+	 * Returns whether the given exception, which occurred during HCR, should be logged.
+	 * We anticipate that we can get IncompatibleThreadStateExceptions if the user happens
+	 * to resume a thread at just the right moment. Since this has no ill effects for HCR,
+	 * we don't log these exceptions.
+	 */
+	private boolean shouldLogHCRException(DebugException exception) {
+	    return !(exception.getStatus().getException() instanceof IncompatibleThreadStateException ||
+	    		 exception.getStatus().getCode() == IJavaThread.ERR_INCOMPATIBLE_THREAD_STATE ||
+	    		 exception.getStatus().getCode() == IJavaThread.ERR_THREAD_NOT_SUSPENDED);
 	}
 
 	/**
@@ -440,25 +528,28 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 		if (target.supportsJ9HotCodeReplace()) {
 			target.setHCROccurred(true);
 			org.eclipse.jdi.hcr.VirtualMachine vm= (org.eclipse.jdi.hcr.VirtualMachine) target.getVM();
+			if (vm == null) {
+				target.requestFailed(JDIDebugHCRMessages.JavaHotCodeReplaceManager_Hot_code_replace_failed___VM_disconnected__1, null); 
+			}
 			int result= org.eclipse.jdi.hcr.VirtualMachine.RELOAD_FAILURE;
 			try {
 				result= vm.classesHaveChanged(typeNames);
 			} catch (RuntimeException e) {
-				target.targetRequestFailed(MessageFormat.format(JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.exception_replacing_types"), new String[] {e.toString()}), e); //$NON-NLS-1$
+				target.targetRequestFailed(MessageFormat.format(JDIDebugHCRMessages.JavaHotCodeReplaceManager_exception_replacing_types, new String[] {e.toString()}), e); 
 			}
 			switch (result) {
 				case org.eclipse.jdi.hcr.VirtualMachine.RELOAD_SUCCESS:
 					break;
 				case org.eclipse.jdi.hcr.VirtualMachine.RELOAD_IGNORED:
-					target.targetRequestFailed(JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.hcr_ignored"), null); //$NON-NLS-1$
+					target.targetRequestFailed(JDIDebugHCRMessages.JavaHotCodeReplaceManager_hcr_ignored, null); 
 					break;
 				case org.eclipse.jdi.hcr.VirtualMachine.RELOAD_FAILURE:
-					target.targetRequestFailed(JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.hcr_failed"), null); //$NON-NLS-1$
+					target.targetRequestFailed(JDIDebugHCRMessages.JavaHotCodeReplaceManager_hcr_failed, null); 
 					target.addOutOfSynchTypes(qualifiedNames);
 					break;
 			}
 		} else {
-			target.notSupported(JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.does_not_support_hcr")); //$NON-NLS-1$
+			target.notSupported(JDIDebugHCRMessages.JavaHotCodeReplaceManager_does_not_support_hcr); 
 			target.addOutOfSynchTypes(qualifiedNames);
 		}
 	}
@@ -473,30 +564,34 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 			target.setHCROccurred(true);
 			Map typesToBytes= getTypesToBytes(target, resources, qualifiedNames);
 			try {
-				target.getVM().redefineClasses(typesToBytes);
+				VirtualMachine vm = target.getVM();
+				if (vm == null) {
+					target.requestFailed(JDIDebugHCRMessages.JavaHotCodeReplaceManager_Hot_code_replace_failed___VM_disconnected__2, null); 
+				}
+				vm.redefineClasses(typesToBytes);
 			} catch (UnsupportedOperationException exception) {
 				String detail= exception.getMessage();
 				if (detail != null) {
-					redefineTypesFailedJDK(target, qualifiedNames, MessageFormat.format(JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.hcr_unsupported_operation"), new String[] {detail}), exception); //$NON-NLS-1$
+					redefineTypesFailedJDK(target, qualifiedNames, MessageFormat.format(JDIDebugHCRMessages.JavaHotCodeReplaceManager_hcr_unsupported_operation, new String[] {detail}), exception); 
 				} else {
-					redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.hcr_unsupported_redefinition"), exception); //$NON-NLS-1$
+					redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.JavaHotCodeReplaceManager_hcr_unsupported_redefinition, exception); 
 				}				
 			} catch (NoClassDefFoundError exception) {
-				redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.hcr_bad_bytes"), exception); //$NON-NLS-1$
+				redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.JavaHotCodeReplaceManager_hcr_bad_bytes, exception); 
 			} catch (VerifyError exception) {
-				redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.hcr_verify_error"), exception); //$NON-NLS-1$
+				redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.JavaHotCodeReplaceManager_hcr_verify_error, exception); 
 			} catch (UnsupportedClassVersionError exception) {
-				redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.hcr_unsupported_class_version"), exception); //$NON-NLS-1$
+				redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.JavaHotCodeReplaceManager_hcr_unsupported_class_version, exception); 
 			} catch (ClassFormatError exception) {
-				redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.hcr_class_format_error"), exception); //$NON-NLS-1$
+				redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.JavaHotCodeReplaceManager_hcr_class_format_error, exception); 
 			} catch (ClassCircularityError exception) {
-				redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.hcr_class_circularity_error"), exception); //$NON-NLS-1$
+				redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.JavaHotCodeReplaceManager_hcr_class_circularity_error, exception); 
 			} catch (RuntimeException exception) {
-				redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.hcr_failed"), exception); //$NON-NLS-1$
+				redefineTypesFailedJDK(target, qualifiedNames, JDIDebugHCRMessages.JavaHotCodeReplaceManager_hcr_failed, exception); 
 			}
 			target.reinstallBreakpointsIn(resources, qualifiedNames);
 		} else {
-			target.notSupported(JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.does_not_support_hcr")); //$NON-NLS-1$
+			target.notSupported(JDIDebugHCRMessages.JavaHotCodeReplaceManager_does_not_support_hcr); 
 		}
 	}
 	
@@ -664,7 +759,7 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 		for (int i = 0; i < numThreads; i++) {
 			thread= (JDIThread) threads[i];
 			if (thread.isSuspended()) {
-				affectedFrame= getAffectedFrame(thread, resources, replacedClassNames);
+				affectedFrame= getAffectedFrame(thread, replacedClassNames);
 				if (affectedFrame == null) {
 					// No frame to drop to in this thread
 					continue;
@@ -678,7 +773,7 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 						notifyFailedDrop(((JDIThread)threads[i]).computeStackFrames(), replacedClassNames);
 					}
 					throw new DebugException(new Status(IStatus.ERROR, JDIDebugModel.getPluginIdentifier(),
-						DebugException.NOT_SUPPORTED, JDIDebugHCRMessages.getString("JavaHotCodeReplaceManager.Drop_to_frame_not_supported"), null)); //$NON-NLS-1$
+						DebugException.NOT_SUPPORTED, JDIDebugHCRMessages.JavaHotCodeReplaceManager_Drop_to_frame_not_supported, null)); 
 				}
 			}
 		}
@@ -693,12 +788,11 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 	 * stack frames whose methods were directly affected (and not simply all frames
 	 * in affected types) will be returned.
 	 */
-	protected JDIStackFrame getAffectedFrame(JDIThread thread, IResource[] resources, List replacedClassNames) throws DebugException {
+	protected JDIStackFrame getAffectedFrame(JDIThread thread, List replacedClassNames) throws DebugException {
 		List frames= thread.computeStackFrames();
 		JDIStackFrame affectedFrame= null;
 		JDIStackFrame frame= null;
 		ICompilationUnit compilationUnit= null;
-		IMethod method= null;
 		CompilationUnitDelta delta= null;
 		IProject project= null;
 		for (int j= frames.size() - 1; j >= 0; j--) {
@@ -706,44 +800,56 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 			if (containsChangedType(frame, replacedClassNames)) {
 				// smart drop to frame support
 				compilationUnit= getCompilationUnit(frame);
-				if (compilationUnit == null) {
-					continue;
-				}
-				try {
-					project= compilationUnit.getCorrespondingResource().getProject();
-					method= getMethod(frame, compilationUnit);
-					if (method != null) {
-						delta= new CompilationUnitDelta(compilationUnit, getLastProjectBuildTime(project));
-						if (!delta.hasChanged(method)) {
+				// if we can't find the source, then do type-based drop
+				if (compilationUnit != null) {
+					try {
+						project= compilationUnit.getCorrespondingResource().getProject();
+					    delta = getDelta(compilationUnit, getLastProjectBuildTime(project));
+						if (!delta.hasChanged(frame.getName(), frame.getSignature())) {
 							continue;
 						}
+					} catch (CoreException exception) {
+						// If smart drop to frame fails, just do type-based drop	
 					}
-				} catch (CoreException exception) {
-					// If smart drop to frame fails, just do type-based drop	
 				}
 
 				if (frame.supportsDropToFrame()) {
 					affectedFrame= frame;
 					break;
-				} else {
-					// The frame we wanted to drop to cannot be popped.
-					// Set the affected frame to the next lowest poppable
-					// frame on the stack.
-					while (j > 0) {
-						j--;
-						frame= (JDIStackFrame) frames.get(j);
-						if (frame.supportsDropToFrame()) {
-							affectedFrame= frame;
-							break;
-						}
-					}
-					break;
 				}
+				// The frame we wanted to drop to cannot be popped.
+				// Set the affected frame to the next lowest poppable
+				// frame on the stack.
+				while (j > 0) {
+					j--;
+					frame= (JDIStackFrame) frames.get(j);
+					if (frame.supportsDropToFrame()) {
+						affectedFrame= frame;
+						break;
+					}
+				}
+				break;
 			}
 		}
 		return affectedFrame;
 	}
 	
+	/**
+	 * Returns the delta object for the given compilation unit
+	 * 
+	 * @param cu compilation unit
+	 * @param time time to compare to (i.e. compare to first version before this time)
+	 * @return delta object
+	 */
+	private CompilationUnitDelta getDelta(ICompilationUnit cu, long time) throws CoreException {
+	    CompilationUnitDelta delta = (CompilationUnitDelta) fDeltaCache.get(cu);
+	    if (delta == null) {
+	    	delta= new CompilationUnitDelta(cu, time);
+	    	fDeltaCache.put(cu, delta);
+	    }
+	    return delta;
+	}
+	
 	/**
 	 * Returns whether the given frame's declaring type was changed
 	 * based on the given list of changed class names.
@@ -774,7 +880,7 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 	protected void attemptStepIn(List threads) throws DebugException {
 		Iterator iter= threads.iterator();
 		while (iter.hasNext()) {
-			((JDIThread) iter.next()).stepInto();;
+			((JDIThread) iter.next()).stepInto();
 		}
 	}
 	
@@ -792,9 +898,16 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 		if (locator == null) {
 			return null;
 		}
+		IJavaDebugTarget target = (IJavaDebugTarget) frame.getDebugTarget();
+		String def = target.getDefaultStratum();
+		target.setDefaultStratum("Java"); //$NON-NLS-1$
 		Object sourceElement= locator.getSourceElement(frame);
+		target.setDefaultStratum(def);
+		if (!(sourceElement instanceof IJavaElement) && sourceElement instanceof IAdaptable) {
+			sourceElement = ((IAdaptable)sourceElement).getAdapter(IJavaElement.class);
+		}
 		if (sourceElement instanceof IType) {
-			return (ICompilationUnit)((IType)sourceElement).getCompilationUnit();
+			return ((IType)sourceElement).getCompilationUnit();
 		}
 		if (sourceElement instanceof ICompilationUnit) {
 			return (ICompilationUnit)sourceElement;
@@ -870,11 +983,11 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 	 * Returns the class file visitor after visiting the resource change.
 	 * The visitor contains the changed class files and qualified type names.
 	 * Returns <code>null</code> if the visitor encounters an exception,
-	 * or the detlta is not a POST_CHANGE.
+	 * or the delta is not a POST_BUILD.
 	 */
 	protected ChangedClassFilesVisitor getChangedClassFiles(IResourceChangeEvent event) {
 		IResourceDelta delta= event.getDelta();
-		if (event.getType() != IResourceChangeEvent.POST_CHANGE || delta == null) {
+		if (event.getType() != IResourceChangeEvent.POST_BUILD || delta == null) {
 			return null;
 		}
 		fClassfileVisitor.reset();
@@ -1005,73 +1118,30 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 		 */
 		private IResource getSourceFile(IJavaProject project, String qualifiedName, String sourceAttribute) {
 			String name = null;
-			if (sourceAttribute == null) { 
-				int nestedIndex= qualifiedName.indexOf('$');
-				if (nestedIndex != -1) {
-					// Trim nested type suffix
-					name= qualifiedName.substring(0, nestedIndex);
-				}
-				name= name + ".java"; //$NON-NLS-1$
-			} else {
-				int i = qualifiedName.lastIndexOf('/');
-				if (i > 0) {
-					name = qualifiedName.substring(0, i + 1);
-					name = name + sourceAttribute;
-				} else {
-					name = sourceAttribute;
-				}
-			}
-			ICompilationUnit unit= null;
+			IJavaElement element = null;
 			try {
-				unit= (ICompilationUnit)project.findElement(new Path(name));
-				if (unit != null) {
-					try {
-						return unit.getCorrespondingResource();
-					} catch (JavaModelException e) {
+				if (sourceAttribute == null) { 
+					element = JavaDebugUtils.findElement(qualifiedName, project);
+				} else {
+					int i = qualifiedName.lastIndexOf('/');
+					if (i > 0) {
+						name = qualifiedName.substring(0, i + 1);
+						name = name + sourceAttribute;
+					} else {
+						name = sourceAttribute;
 					}
+					element = project.findElement(new Path(name));
 				}
-			} catch (JavaModelException exception) {
+				if (element instanceof ICompilationUnit) {
+					ICompilationUnit cu = (ICompilationUnit) element;
+					return cu.getCorrespondingResource();
+				}
+			} catch (CoreException e) {
 			}
 			return null;
 		}
 	}
-	
-	class BuiltProjectVisitor implements IResourceDeltaVisitor {
-		/**
-		 * The collection of built projects
-		 */
-		protected List fProjects= new ArrayList();
-		/**
-		 * Answers whether children should be visited.
-		 * <p>
-		 * If the associated resource is a project which 
-		 * has been built, record it.
-		 */
-		public boolean visit(IResourceDelta delta) {
-			if (delta == null || 0 == (delta.getKind() & IResourceDelta.CHANGED)) {
-				return false;
-			}
-			IResource resource= delta.getResource();
-			if (resource != null && resource.getType() == IResource.PROJECT) {
-				fProjects.add(resource);
-				return false;
-			}
-			return true;
-		}
-		/**
-		 * Resets the project collection to empty
-		 */
-		public void reset() {
-			fProjects = new ArrayList();
-		}
-		
-		/**
-		 * Returns the collection of built projects
-		 */
-		public List getBuiltProjects() {
-			return fProjects;
-		}
-	}
+
 	
 	/**
 	 * Adds the given listener to the collection of hot code replace listeners.
@@ -1090,18 +1160,6 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 		fHotCodeReplaceListeners.remove(listener);
 	}
 	
-	protected void fork(final IWorkspaceRunnable wRunnable) {
-		Runnable runnable= new Runnable() {
-			public void run() {
-				try {
-					getWorkspace().run(wRunnable, null);
-				} catch (CoreException ce) {
-					JDIDebugPlugin.log(ce);
-				}
-			}
-		};
-		new Thread(runnable).start();
-	}
 	/**
 	 * @see ILaunchListener#launchRemoved(ILaunch)
 	 */
@@ -1114,11 +1172,12 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 			}
 		}
 	}
+
 	/**
 	 * Begin listening for resource changes when a launch is
 	 * registered with a hot swapable target.
 	 * 
-	 * @see ILaunchListener#launchAdded(ILaunch)
+	 * @see org.eclipse.debug.core.ILaunchListener#launchAdded(org.eclipse.debug.core.ILaunch)
 	 */
 	public void launchAdded(ILaunch launch) {
 		IDebugTarget[] debugTargets= launch.getDebugTargets();
@@ -1128,13 +1187,13 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 				JDIDebugTarget target = (JDIDebugTarget)jt;
 				if (target.supportsHotCodeReplace()) {
 					addHotSwapTarget(target);
-				} else {
+				} else if (target.isAvailable()){
 					addNonHotSwapTarget(target);
 				}				
 			}
 		}
 		if (!fHotSwapTargets.isEmpty() || !fNoHotSwapTargets.isEmpty()) {
-			getWorkspace().addResourceChangeListener(this, IResourceChangeEvent.POST_CHANGE | IResourceChangeEvent.POST_AUTO_BUILD);
+			getWorkspace().addResourceChangeListener(this, IResourceChangeEvent.POST_BUILD);
 		}
 	}
 	
@@ -1148,8 +1207,8 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 		launchAdded(launch);
 	}	
 	
-	/**
-	 * @see IDebugEventSetListener#handleDebugEvents(DebugEvent[])
+	/* (non-Javadoc)
+	 * @see org.eclipse.debug.core.IDebugEventSetListener#handleDebugEvents(org.eclipse.debug.core.DebugEvent[])
 	 */
 	public void handleDebugEvents(DebugEvent[] events) {
 		for (int i = 0; i < events.length; i++) {
@@ -1186,13 +1245,11 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 				}
 			}
 		}
-		// To get here, there must be no running JDIDebugTargets
-		getWorkspace().removeResourceChangeListener(this);
 	}
 	
 	/**
 	 * Adds the given target to the list of hot-swappable targets.
-	 * Has no effect if the target is alread registered.
+	 * Has no effect if the target is already registered.
 	 * 
 	 * @param target a target that supports hot swap
 	 */
@@ -1204,7 +1261,7 @@ public class JavaHotCodeReplaceManager implements IResourceChangeListener, ILaun
 	
 	/**
 	 * Adds the given target to the list of non hot-swappable targets.
-	 * Has no effect if the target is alread registered.
+	 * Has no effect if the target is already registered.
 	 * 
 	 * @param target a target that does not support hot swap
 	 */
