@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -119,12 +119,12 @@ public class StandardVMDebugger extends StandardVMRunner {
 		}
 		
 		IProgressMonitor subMonitor = new SubProgressMonitor(monitor, 1);
-		subMonitor.beginTask(LaunchingMessages.StandardVMDebugger_Launching_VM____1, 4); //$NON-NLS-1$
-		subMonitor.subTask(LaunchingMessages.StandardVMDebugger_Finding_free_socket____2); //$NON-NLS-1$
+		subMonitor.beginTask(LaunchingMessages.StandardVMDebugger_Launching_VM____1, 4); 
+		subMonitor.subTask(LaunchingMessages.StandardVMDebugger_Finding_free_socket____2); 
 
 		int port= SocketUtil.findFreePort();
 		if (port == -1) {
-			abort(LaunchingMessages.StandardVMDebugger_Could_not_find_a_free_socket_for_the_debugger_1, null, IJavaLaunchConfigurationConstants.ERR_NO_SOCKET_AVAILABLE); //$NON-NLS-1$
+			abort(LaunchingMessages.StandardVMDebugger_Could_not_find_a_free_socket_for_the_debugger_1, null, IJavaLaunchConfigurationConstants.ERR_NO_SOCKET_AVAILABLE); 
 		}
 		
 		subMonitor.worked(1);
@@ -134,7 +134,7 @@ public class StandardVMDebugger extends StandardVMRunner {
 			return;
 		}		
 		
-		subMonitor.subTask(LaunchingMessages.StandardVMDebugger_Constructing_command_line____3); //$NON-NLS-1$
+		subMonitor.subTask(LaunchingMessages.StandardVMDebugger_Constructing_command_line____3); 
 				
 		String program= constructProgramString(config);
 
@@ -153,15 +153,21 @@ public class StandardVMDebugger extends StandardVMRunner {
 			arguments.add("-classpath"); //$NON-NLS-1$
 			arguments.add(convertClassPath(cp));
 		}
-		arguments.add("-Xdebug"); //$NON-NLS-1$
-		arguments.add("-Xnoagent"); //$NON-NLS-1$
-		
 		double version = getJavaVersion();
+		if (version < 1.5) {
+			arguments.add("-Xdebug"); //$NON-NLS-1$
+			arguments.add("-Xnoagent"); //$NON-NLS-1$
+		}
+		
 		//check if java 1.4 or greater
 		if (version < 1.4) {
 			arguments.add("-Djava.compiler=NONE"); //$NON-NLS-1$
 		}
-		arguments.add("-Xrunjdwp:transport=dt_socket,suspend=y,address=localhost:" + port); //$NON-NLS-1$
+		if (version < 1.5) { 
+			arguments.add("-Xrunjdwp:transport=dt_socket,suspend=y,address=localhost:" + port); //$NON-NLS-1$
+		} else {
+			arguments.add("-agentlib:jdwp=transport=dt_socket,suspend=y,address=localhost:" + port); //$NON-NLS-1$
+		}
 
 		arguments.add(config.getClassToLaunch());
 		addArguments(config.getProgramArguments(), arguments);
@@ -176,11 +182,11 @@ public class StandardVMDebugger extends StandardVMRunner {
 		}		
 		
 		subMonitor.worked(1);
-		subMonitor.subTask(LaunchingMessages.StandardVMDebugger_Starting_virtual_machine____4); //$NON-NLS-1$
+		subMonitor.subTask(LaunchingMessages.StandardVMDebugger_Starting_virtual_machine____4); 
 
 		ListeningConnector connector= getConnector();
 		if (connector == null) {
-			abort(LaunchingMessages.StandardVMDebugger_Couldn__t_find_an_appropriate_debug_connector_2, null, IJavaLaunchConfigurationConstants.ERR_CONNECTOR_NOT_AVAILABLE); //$NON-NLS-1$
+			abort(LaunchingMessages.StandardVMDebugger_Couldn__t_find_an_appropriate_debug_connector_2, null, IJavaLaunchConfigurationConstants.ERR_CONNECTOR_NOT_AVAILABLE); 
 		}
 		Map map= connector.defaultArguments();
 		
@@ -210,7 +216,7 @@ public class StandardVMDebugger extends StandardVMRunner {
 				IProcess process= newProcess(launch, p, renderProcessLabel(cmdLine), getDefaultProcessMap());
 				process.setAttribute(IProcess.ATTR_CMDLINE, renderCommandLine(cmdLine));
 				subMonitor.worked(1);
-				subMonitor.subTask(LaunchingMessages.StandardVMDebugger_Establishing_debug_connection____5); //$NON-NLS-1$
+				subMonitor.subTask(LaunchingMessages.StandardVMDebugger_Establishing_debug_connection____5); 
 				boolean retry= false;
 				do  {
 					try {
@@ -283,9 +289,9 @@ public class StandardVMDebugger extends StandardVMRunner {
 				connector.stopListening(map);
 			}
 		} catch (IOException e) {
-			abort(LaunchingMessages.StandardVMDebugger_Couldn__t_connect_to_VM_4, e, IJavaLaunchConfigurationConstants.ERR_CONNECTION_FAILED);  //$NON-NLS-1$
+			abort(LaunchingMessages.StandardVMDebugger_Couldn__t_connect_to_VM_4, e, IJavaLaunchConfigurationConstants.ERR_CONNECTION_FAILED);  
 		} catch (IllegalConnectorArgumentsException e) {
-			abort(LaunchingMessages.StandardVMDebugger_Couldn__t_connect_to_VM_5, e, IJavaLaunchConfigurationConstants.ERR_CONNECTION_FAILED);  //$NON-NLS-1$
+			abort(LaunchingMessages.StandardVMDebugger_Couldn__t_connect_to_VM_5, e, IJavaLaunchConfigurationConstants.ERR_CONNECTION_FAILED);  
 		}
 		if (p != null) {
 			p.destroy();
