@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -15,8 +15,6 @@ import java.io.BufferedInputStream;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.InputStreamReader;
-import java.io.Reader;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.util.ArrayList;
@@ -175,7 +173,9 @@ public class VMDefinitionsContainer {
 	 * @return List 
 	 */
 	public List getValidVMList() {
-		List resultList = getVMList();
+		List vms = getVMList();
+		List resultList = new ArrayList(vms.size());
+		resultList.addAll(vms);
 		resultList.removeAll(fInvalidVMList);
 		return resultList;
 	}
@@ -262,7 +262,7 @@ public class VMDefinitionsContainer {
 		}
 		
 		// Serialize the Document and return the resulting String
-		return JavaLaunchConfigurationUtils.serializeDocument(doc);
+		return LaunchingPlugin.serializeDocument(doc);
 	}
 	
 	/**
@@ -360,10 +360,16 @@ public class VMDefinitionsContainer {
 		}
 		return root;
 	}
+	
+	public static VMDefinitionsContainer parseXMLIntoContainer(InputStream inputStream) throws IOException {
+		VMDefinitionsContainer container = new VMDefinitionsContainer();
+		parseXMLIntoContainer(inputStream, container);
+		return container;
+	}
 			
 	/**
-	 * Parse the VM definitions contained in the specified InputStream and return an instance
-	 * of <code>VMDefinitionsContainer</code>.
+	 * Parse the VM definitions contained in the specified InputStream into the
+	 * specified container.
 	 * <p>
 	 * The VMs in the returned container are instances of <code>VMStandin</code>.
 	 * </p>
@@ -377,39 +383,37 @@ public class VMDefinitionsContainer {
 	 * the XML contained in <code>inputStream</code>.
 	 * </p>
 	 * @param inputStream the <code>InputStream</code> containing XML that declares a set of VMs and a default VM
+	 * @param container the container to add the VM defs to
 	 * @return VMDefinitionsContainer a container for the VM objects declared in <code>inputStream</code>
 	 * @throws IOException if this method fails. Reasons include:<ul>
 	 * <li>the XML in <code>inputStream</code> was badly formatted</li>
 	 * <li>the top-level node was not 'vmSettings'</li>
 	 * </ul>
+	 * @since 3.2
 	 */
-	public static VMDefinitionsContainer parseXMLIntoContainer(InputStream inputStream) throws IOException {
-		
-		// Create the container to populate
-		VMDefinitionsContainer container = new VMDefinitionsContainer();
+	public static void parseXMLIntoContainer(InputStream inputStream, VMDefinitionsContainer container) throws IOException {
 
 		// Wrapper the stream for efficient parsing
 		InputStream stream= new BufferedInputStream(inputStream);
-		Reader reader= new InputStreamReader(stream);
 
 		// Do the parsing and obtain the top-level node
 		Element config= null;		
 		try {
 			DocumentBuilder parser= DocumentBuilderFactory.newInstance().newDocumentBuilder();
 			parser.setErrorHandler(new DefaultHandler());
-			config = parser.parse(new InputSource(reader)).getDocumentElement();
+			config = parser.parse(new InputSource(stream)).getDocumentElement();
 		} catch (SAXException e) {
-			throw new IOException(LaunchingMessages.JavaRuntime_badFormat); //$NON-NLS-1$
+			throw new IOException(LaunchingMessages.JavaRuntime_badFormat); 
 		} catch (ParserConfigurationException e) {
-			reader.close();
-			throw new IOException(LaunchingMessages.JavaRuntime_badFormat); //$NON-NLS-1$
+			stream.close();
+			throw new IOException(LaunchingMessages.JavaRuntime_badFormat); 
 		} finally {
-			reader.close();
+			stream.close();
 		}
 		
 		// If the top-level node wasn't what we expected, bail out
 		if (!config.getNodeName().equalsIgnoreCase("vmSettings")) { //$NON-NLS-1$
-			throw new IOException(LaunchingMessages.JavaRuntime_badFormat); //$NON-NLS-1$
+			throw new IOException(LaunchingMessages.JavaRuntime_badFormat); 
 		}
 		
 		// Populate the default VM-related fields
@@ -429,8 +433,6 @@ public class VMDefinitionsContainer {
 				}
 			}
 		}
-		
-		return container;
 	}
 	
 	/**
@@ -457,7 +459,7 @@ public class VMDefinitionsContainer {
 				}
 			}
 		} else {
-			LaunchingPlugin.log(LaunchingMessages.JavaRuntime_VM_type_element_with_unknown_id_1); //$NON-NLS-1$
+			LaunchingPlugin.log(LaunchingMessages.JavaRuntime_VM_type_element_with_unknown_id_1); 
 		}
 	}
 
@@ -519,7 +521,7 @@ public class VMDefinitionsContainer {
 				vmStandin.setVMArgs(vmArgs);
 			}
 		} else {
-			LaunchingPlugin.log(LaunchingMessages.JavaRuntime_VM_element_specified_with_no_id_attribute_2); //$NON-NLS-1$
+			LaunchingPlugin.log(LaunchingMessages.JavaRuntime_VM_element_specified_with_no_id_attribute_2); 
 		}
 	}	
 	
@@ -539,13 +541,13 @@ public class VMDefinitionsContainer {
 			try {
 				javadocURL= new URL(jreJavadoc);
 			} catch (MalformedURLException e) {
-				LaunchingPlugin.log(LaunchingMessages.JavaRuntime_Library_location_element_incorrectly_specified_3); //$NON-NLS-1$
+				LaunchingPlugin.log(LaunchingMessages.JavaRuntime_Library_location_element_incorrectly_specified_3); 
 			}
 		}
 		if (jreJar != null && jreSrc != null && pkgRoot != null) {
 			return new LibraryLocation(new Path(jreJar), new Path(jreSrc), new Path(pkgRoot), javadocURL);
 		}
-		LaunchingPlugin.log(LaunchingMessages.JavaRuntime_Library_location_element_incorrectly_specified_3); //$NON-NLS-1$
+		LaunchingPlugin.log(LaunchingMessages.JavaRuntime_Library_location_element_incorrectly_specified_3); 
 		return null;
 	}
 	
@@ -569,5 +571,19 @@ public class VMDefinitionsContainer {
 		}	
 		vm.setLibraryLocations((LibraryLocation[])locations.toArray(new LibraryLocation[locations.size()]));
 	}
+	
+	/**
+	 * Removes the VM from this container.
+	 * 
+	 * @param vm vm intall
+	 */
+	public void removeVM(IVMInstall vm) {
+		fVMList.remove(vm);
+		fInvalidVMList.remove(vm);
+		List list = (List) fVMTypeToVMMap.get(vm.getVMInstallType());
+		if (list != null) {
+			list.remove(vm);
+		}
+	}
 		
 }
