@@ -110,7 +110,7 @@ public abstract class AbstractVMRunner implements IVMRunner {
 		IProcess process= DebugPlugin.newProcess(launch, p, label, attributes);
 		if (process == null) {
 			p.destroy();
-			abort(LaunchingMessages.AbstractVMRunner_0, null, IJavaLaunchConfigurationConstants.ERR_INTERNAL_ERROR); //$NON-NLS-1$
+			abort(LaunchingMessages.AbstractVMRunner_0, null, IJavaLaunchConfigurationConstants.ERR_INTERNAL_ERROR); 
 		}
 		return process;
 	}
@@ -121,7 +121,7 @@ public abstract class AbstractVMRunner implements IVMRunner {
 	 * 
 	 * @param configuration runner configuration
 	 * @param vmInstall vm install
-	 * @return combined VM arguments specified by the runner confiugration
+	 * @return combined VM arguments specified by the runner configuration
 	 *  and VM install
 	 * @since 3.0
 	 */
