@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -42,17 +42,17 @@ import org.eclipse.jdt.core.IJavaProject;
 import org.eclipse.jdt.core.JavaModelException;
 import org.eclipse.jdt.core.NamingConventions;
 import org.eclipse.jdt.core.Signature;
+import org.eclipse.jdt.core.dom.rewrite.ImportRewrite;
 
-import org.eclipse.jdt.internal.corext.codemanipulation.CodeGenerationSettings;
-import org.eclipse.jdt.internal.corext.codemanipulation.ImportsStructure;
+import org.eclipse.jdt.internal.corext.codemanipulation.StubUtility;
 import org.eclipse.jdt.internal.corext.template.java.CompilationUnitCompletion.LocalVariable;
+import org.eclipse.jdt.internal.corext.util.JavaModelUtil;
 import org.eclipse.jdt.internal.corext.util.Strings;
 
 import org.eclipse.jdt.ui.JavaUI;
 import org.eclipse.jdt.ui.PreferenceConstants;
 
 import org.eclipse.jdt.internal.ui.JavaPlugin;
-import org.eclipse.jdt.internal.ui.preferences.JavaPreferencesSettings;
 import org.eclipse.jdt.internal.ui.text.template.contentassist.MultiVariable;
 import org.eclipse.jdt.internal.ui.util.ExceptionHandler;
 
@@ -73,12 +73,23 @@ public class JavaContext extends CompilationUnitContext {
 	 * @param completionLength the completion length.
 	 * @param compilationUnit the compilation unit (may be <code>null</code>).
 	 */
-	public JavaContext(TemplateContextType type, IDocument document, int completionOffset, int completionLength,
-		ICompilationUnit compilationUnit)
-	{
+	public JavaContext(TemplateContextType type, IDocument document, int completionOffset, int completionLength, ICompilationUnit compilationUnit) {
 		super(type, document, completionOffset, completionLength, compilationUnit);
 	}
 	
+	/**
+	 * Creates a java template context.
+	 * 
+	 * @param type   the context type.
+	 * @param document the document.
+	 * @param completionPosition the position defining the completion offset and length 
+	 * @param compilationUnit the compilation unit (may be <code>null</code>).
+	 * @since 3.2
+	 */
+	public JavaContext(TemplateContextType type, IDocument document, Position completionPosition, ICompilationUnit compilationUnit) {
+		super(type, document, completionPosition, compilationUnit);
+	}
+	
 	/**
 	 * Returns the indentation level at the position of code completion.
 	 * 
@@ -146,6 +157,9 @@ public class JavaContext extends CompilationUnitContext {
 	 */
 	public int getStart() {
 
+		if (fIsManaged && getCompletionLength() > 0)
+			return super.getStart();
+		
 		try {
 			IDocument document= getDocument();
 
@@ -173,7 +187,7 @@ public class JavaContext extends CompilationUnitContext {
 	 */
 	public int getEnd() {
 		
-		if (getCompletionLength() == 0)
+		if (fIsManaged || getCompletionLength() == 0)
 			return super.getEnd();
 
 		try {			
@@ -481,10 +495,9 @@ public class JavaContext extends CompilationUnitContext {
 
 			try {
 				
-				CodeGenerationSettings settings= JavaPreferencesSettings.getCodeGenerationSettings(cu.getJavaProject());
-				ImportsStructure structure= new ImportsStructure(cu, settings.importOrder, settings.importThreshold, true);
-				structure.addImport("java.util.Iterator"); //$NON-NLS-1$
-				structure.create(false, null);
+				ImportRewrite rewrite= StubUtility.createImportRewrite(cu, true);
+				rewrite.addImport("java.util.Iterator"); //$NON-NLS-1$
+				JavaModelUtil.applyEdit(cu, rewrite.rewriteImports(null), false, null);
 				
 				setCompletionOffset(position.getOffset());
 				setCompletionLength(position.getLength());
@@ -517,7 +530,7 @@ public class JavaContext extends CompilationUnitContext {
 	 */
 	public static String evaluateTemplate(Template template, ICompilationUnit compilationUnit, int position) throws CoreException, BadLocationException, TemplateException {
 
-		TemplateContextType contextType= JavaPlugin.getDefault().getTemplateContextRegistry().getContextType("java"); //$NON-NLS-1$
+		TemplateContextType contextType= JavaPlugin.getDefault().getTemplateContextRegistry().getContextType(JavaContextType.NAME);
 		if (contextType == null)
 			throw new CoreException(new Status(IStatus.ERROR, JavaUI.ID_PLUGIN, IStatus.ERROR, JavaTemplateMessages.JavaContext_error_message, null)); 
 
