@@ -1,22 +1,27 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.jdt.internal.corext.refactoring.changes;
 
-import org.eclipse.core.resources.IContainer;
-import org.eclipse.core.resources.IResource;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 
+import org.eclipse.core.resources.IContainer;
+import org.eclipse.core.resources.IResource;
+
 import org.eclipse.jdt.internal.corext.refactoring.RefactoringCoreMessages;
+import org.eclipse.jdt.internal.corext.util.Messages;
+
+import org.eclipse.ltk.core.refactoring.Change;
+import org.eclipse.ltk.core.refactoring.RefactoringStatus;
 
 public class MoveResourceChange extends ResourceReorgChange {
 	
@@ -24,14 +29,24 @@ public class MoveResourceChange extends ResourceReorgChange {
 		super(res, dest, null);
 	}
 	
+	public RefactoringStatus isValid(IProgressMonitor pm) throws CoreException {
+		// We already present a dialog to the user if he
+		// moves read-only resources. Since moving a resource
+		// doesn't do a validate edit (it actually doesn't
+		// change the content we can't check for READ only
+		// here.
+		return super.isValid(pm, DIRTY);
+	}
+	
 	/* non java-doc
 	 * @see ResourceReorgChange#doPerform(IPath, IProgressMonitor)
 	 */
-	protected void doPerform(IPath path, IProgressMonitor pm) throws CoreException{
+	protected Change doPerformReorg(IPath path, IProgressMonitor pm) throws CoreException{
 		getResource().move(path, getReorgFlags(), pm);
+		return null;
 	}
 	public String getName() {
-		return RefactoringCoreMessages.getFormattedString("MoveResourceChange.move", //$NON-NLS-1$
+		return Messages.format(RefactoringCoreMessages.MoveResourceChange_move, 
 			new String[]{getResource().getFullPath().toString(), getDestination().getName()});
 	}
 }
