@@ -1,26 +1,33 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.jdt.internal.corext.refactoring.code;
 
 import java.util.ArrayList;
+import java.util.HashSet;
 import java.util.List;
+import java.util.Set;
+
+import org.eclipse.jface.text.IRegion;
+
+import org.eclipse.ltk.core.refactoring.RefactoringStatus;
 
 import org.eclipse.jdt.core.ICompilationUnit;
-import org.eclipse.jdt.core.IJavaProject;
 import org.eclipse.jdt.core.JavaModelException;
 import org.eclipse.jdt.core.compiler.ITerminalSymbols;
 import org.eclipse.jdt.core.dom.AST;
 import org.eclipse.jdt.core.dom.ASTNode;
 import org.eclipse.jdt.core.dom.AnonymousClassDeclaration;
+import org.eclipse.jdt.core.dom.Assignment;
 import org.eclipse.jdt.core.dom.Block;
+import org.eclipse.jdt.core.dom.BodyDeclaration;
 import org.eclipse.jdt.core.dom.ClassInstanceCreation;
 import org.eclipse.jdt.core.dom.CompilationUnit;
 import org.eclipse.jdt.core.dom.ConstructorInvocation;
@@ -30,32 +37,35 @@ import org.eclipse.jdt.core.dom.ForStatement;
 import org.eclipse.jdt.core.dom.IMethodBinding;
 import org.eclipse.jdt.core.dom.ITypeBinding;
 import org.eclipse.jdt.core.dom.IVariableBinding;
+import org.eclipse.jdt.core.dom.Initializer;
 import org.eclipse.jdt.core.dom.Message;
 import org.eclipse.jdt.core.dom.MethodDeclaration;
 import org.eclipse.jdt.core.dom.Name;
-import org.eclipse.jdt.core.dom.NullLiteral;
 import org.eclipse.jdt.core.dom.PrimitiveType;
 import org.eclipse.jdt.core.dom.SimpleName;
 import org.eclipse.jdt.core.dom.SuperConstructorInvocation;
+import org.eclipse.jdt.core.dom.SwitchCase;
 import org.eclipse.jdt.core.dom.Type;
 import org.eclipse.jdt.core.dom.VariableDeclaration;
 import org.eclipse.jdt.core.dom.VariableDeclarationExpression;
 import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
 import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
+import org.eclipse.jdt.core.dom.rewrite.ImportRewrite;
 
+import org.eclipse.jdt.internal.corext.dom.ASTNodeFactory;
 import org.eclipse.jdt.internal.corext.dom.ASTNodes;
 import org.eclipse.jdt.internal.corext.dom.Bindings;
 import org.eclipse.jdt.internal.corext.dom.LocalVariableIndex;
 import org.eclipse.jdt.internal.corext.dom.Selection;
 import org.eclipse.jdt.internal.corext.refactoring.Checks;
 import org.eclipse.jdt.internal.corext.refactoring.RefactoringCoreMessages;
-import org.eclipse.jdt.internal.corext.refactoring.base.JavaSourceContext;
-import org.eclipse.jdt.internal.corext.refactoring.base.RefactoringStatus;
+import org.eclipse.jdt.internal.corext.refactoring.base.JavaStatusContext;
 import org.eclipse.jdt.internal.corext.refactoring.code.flow.FlowContext;
 import org.eclipse.jdt.internal.corext.refactoring.code.flow.FlowInfo;
 import org.eclipse.jdt.internal.corext.refactoring.code.flow.InOutFlowAnalyzer;
 import org.eclipse.jdt.internal.corext.refactoring.code.flow.InputFlowAnalyzer;
 import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
+import org.eclipse.jdt.internal.corext.util.Messages;
 
 /* package */ class ExtractMethodAnalyzer extends CodeAnalyzer {
 
@@ -68,7 +78,8 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 	public static final int RETURN_STATEMENT_VALUE=	4;
 	public static final int MULTIPLE=				5;
 
-	private MethodDeclaration fEnclosingMethod;
+	/** This is either a method declaration or an initializer */
+	private BodyDeclaration fEnclosingBodyDeclaration;
 	private IMethodBinding fEnclosingMethodBinding;
 	private int fMaxVariableId;
 
@@ -80,6 +91,7 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 	
 	private IVariableBinding[] fArguments;
 	private IVariableBinding[] fMethodLocals;
+	private ITypeBinding[] fTypeVariables;
 	
 	private IVariableBinding fReturnValue;
 	private IVariableBinding[] fCallerLocals;
@@ -88,14 +100,15 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 	private ITypeBinding[] fAllExceptions;
 	private ITypeBinding fExpressionBinding;
 
-	private boolean fForceStatic;	
+	private boolean fForceStatic;
+	private boolean fIsLastStatementSelected;
 	
 	public ExtractMethodAnalyzer(ICompilationUnit unit, Selection selection) throws JavaModelException {
 		super(unit, selection, false);
 	}
 	
-	public MethodDeclaration getEnclosingMethod() {
-		return fEnclosingMethod;
+	public BodyDeclaration getEnclosingBodyDeclaration() {
+		return fEnclosingBodyDeclaration;
 	}
 	
 	public int getReturnKind() {
@@ -147,16 +160,20 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 		return fForceStatic;
 	}
 	
+	public ITypeBinding[] getTypeVariables() {
+		return fTypeVariables;
+	}
+	
 	//---- Activation checking ---------------------------------------------------------------------------
 	
-	public RefactoringStatus checkActivation() {
+	public RefactoringStatus checkInitialConditions(ImportRewrite rewriter) {
 		RefactoringStatus result= getStatus();
 		checkExpression(result);
 		if (result.hasFatalError())
 			return result;
 			
 		fReturnKind= UNDEFINED;
-		fMaxVariableId= LocalVariableIndex.perform(fEnclosingMethod);
+		fMaxVariableId= LocalVariableIndex.perform(fEnclosingBodyDeclaration);
 		if (analyzeSelection(result).hasFatalError())
 			return result;
 
@@ -171,11 +188,11 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 		}
 			
 		if (returns > 1) {
-			result.addFatalError(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.ambiguous_return_value"), JavaSourceContext.create(fCUnit, getSelection())); //$NON-NLS-1$
+			result.addFatalError(RefactoringCoreMessages.ExtractMethodAnalyzer_ambiguous_return_value, JavaStatusContext.create(fCUnit, getSelection())); 
 			fReturnKind= MULTIPLE;
 			return result;
 		}
-		initReturnType();
+		initReturnType(rewriter);
 		return result;
 	}
 	
@@ -183,52 +200,79 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 		ASTNode[] nodes= getSelectedNodes();
 		if (nodes != null && nodes.length == 1) {
 			ASTNode node= nodes[0];
-			if (node instanceof NullLiteral) {
-				status.addFatalError(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.cannot_extract_null"), JavaSourceContext.create(fCUnit, node)); //$NON-NLS-1$
-			} else if (node instanceof Type) {
-				status.addFatalError(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.cannot_extract_type_reference"), JavaSourceContext.create(fCUnit, node)); //$NON-NLS-1$
+			if (node instanceof Type) {
+				status.addFatalError(RefactoringCoreMessages.ExtractMethodAnalyzer_cannot_extract_type_reference, JavaStatusContext.create(fCUnit, node)); 
+			} else if (node.getLocationInParent() == SwitchCase.EXPRESSION_PROPERTY) {
+				status.addFatalError(RefactoringCoreMessages.ExtractMethodAnalyzer_cannot_extract_switch_case, JavaStatusContext.create(fCUnit, node)); 
 			}
 		}
 	}
 	
-	private void initReturnType() {
-		AST ast= fEnclosingMethod.getAST();
+	private void initReturnType(ImportRewrite rewriter) {
+		AST ast= fEnclosingBodyDeclaration.getAST();
+		fReturnType= null;
 		switch (fReturnKind) {
 			case ACCESS_TO_LOCAL:
-				VariableDeclaration declaration= ASTNodes.findVariableDeclaration(fReturnValue, fEnclosingMethod);
-				fReturnType= ASTNodes.getType(declaration);
+				VariableDeclaration declaration= ASTNodes.findVariableDeclaration(fReturnValue, fEnclosingBodyDeclaration);
+				fReturnType= ASTNodeFactory.newType(ast, declaration);
 				break;
 			case EXPRESSION:
 				Expression expression= (Expression)getFirstSelectedNode();
 				if (expression.getNodeType() == ASTNode.CLASS_INSTANCE_CREATION) {
-					fExpressionBinding= ASTNodes.getTypeBinding(((ClassInstanceCreation)expression).getName());
+					fExpressionBinding= ((ClassInstanceCreation)expression).getType().resolveBinding();
 				} else {
 					fExpressionBinding= expression.resolveTypeBinding();
 				}
 				if (fExpressionBinding != null) {
-					fReturnType= Bindings.createType(fExpressionBinding, ast, true);
-					break;
+					if (fExpressionBinding.isNullType()) {
+						getStatus().addFatalError(RefactoringCoreMessages.ExtractMethodAnalyzer_cannot_extract_null_type, JavaStatusContext.create(fCUnit, expression)); 
+					} else {
+						ITypeBinding normalizedBinding= Bindings.normalizeForDeclarationUse(fExpressionBinding, ast);
+						if (normalizedBinding != null) {
+							fReturnType= rewriter.addImport(normalizedBinding, ast);
+						}
+					}
+				} else {
+					fReturnType= ast.newPrimitiveType(PrimitiveType.VOID);
+					getStatus().addError(RefactoringCoreMessages.ExtractMethodAnalyzer_cannot_determine_return_type, JavaStatusContext.create(fCUnit, expression)); 
 				}
-				fReturnType= ast.newPrimitiveType(PrimitiveType.VOID);
-				getStatus().addError(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.cannot_determine_return_type"), JavaSourceContext.create(fCUnit, expression)); //$NON-NLS-1$
 				break;	
 			case RETURN_STATEMENT_VALUE:
-				if (fEnclosingMethod.isConstructor())
-					fReturnType= null;
-				fReturnType= fEnclosingMethod.getReturnType();
+				if (fEnclosingBodyDeclaration.getNodeType() == ASTNode.METHOD_DECLARATION)
+					fReturnType= ((MethodDeclaration)fEnclosingBodyDeclaration).getReturnType2();
 				break;
 			default:
 				fReturnType= ast.newPrimitiveType(PrimitiveType.VOID);
 		}
+		if (fReturnType == null)
+			fReturnType= ast.newPrimitiveType(PrimitiveType.VOID);
 	}
 	
+	//	 !!! -- +/- same as in ExtractTempRefactoring
+	public boolean isLiteralNodeSelected() {
+		ASTNode[] nodes= getSelectedNodes();
+		if (nodes.length != 1)
+			return false;
+		ASTNode node= nodes[0];
+		switch (node.getNodeType()) {
+			case ASTNode.BOOLEAN_LITERAL :
+			case ASTNode.CHARACTER_LITERAL :
+			case ASTNode.NULL_LITERAL :
+			case ASTNode.NUMBER_LITERAL :
+				return true;
+			
+			default :
+				return false;
+		}
+	}
+
 	//---- Input checking -----------------------------------------------------------------------------------
 		
-	public void checkInput(RefactoringStatus status, String methodName, IJavaProject scope, AST ast) {
+	public void checkInput(RefactoringStatus status, String methodName, AST ast) {
 		ITypeBinding[] arguments= getArgumentTypes();
-		ITypeBinding type= fEnclosingMethodBinding != null ? fEnclosingMethodBinding.getDeclaringClass() : ast.resolveWellKnownType("void"); //$NON-NLS-1$
-		status.merge(Checks.checkMethodInType(type, methodName, arguments, scope));
-		status.merge(Checks.checkMethodInHierarchy(type.getSuperclass(), methodName, null, arguments, scope));
+		ITypeBinding type= ASTNodes.getEnclosingType(fEnclosingBodyDeclaration);
+		status.merge(Checks.checkMethodInType(type, methodName, arguments));
+		status.merge(Checks.checkMethodInHierarchy(type.getSuperclass(), methodName, null, arguments));
 	}
 	
 	private ITypeBinding[] getArgumentTypes() {
@@ -248,7 +292,7 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 		fInputFlowInfo= flowAnalyzer.perform(getSelectedNodes());
 		
 		if (fInputFlowInfo.branches()) {
-			status.addFatalError(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.branch_mismatch"), JavaSourceContext.create(fCUnit, getSelection())); //$NON-NLS-1$
+			status.addFatalError(RefactoringCoreMessages.ExtractMethodAnalyzer_branch_mismatch, JavaStatusContext.create(fCUnit, getSelection())); 
 			fReturnKind= ERROR;
 			return status;
 		}
@@ -261,49 +305,92 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 		}
 		
 		if (fReturnKind == UNDEFINED) {
-			status.addFatalError(RefactoringCoreMessages.getString("FlowAnalyzer.execution_flow"), JavaSourceContext.create(fCUnit, getSelection())); //$NON-NLS-1$
-			fReturnKind= ERROR;
-			return status;
+			status.addError(RefactoringCoreMessages.FlowAnalyzer_execution_flow, JavaStatusContext.create(fCUnit, getSelection())); 
+			fReturnKind= NO;
 		}
 		computeInput();
 		computeExceptions();
 		computeOutput(status);
-		if (!status.hasFatalError())
-			adjustArgumentsAndMethodLocals();
+		if (status.hasFatalError())
+			return status;
+		
+		adjustArgumentsAndMethodLocals();
 		compressArrays();
 		return status;
 	}
 	
 	private boolean isVoidMethod() {
+		// if we have an initializer
+		if (fEnclosingMethodBinding == null)
+			return true;
 		ITypeBinding binding= fEnclosingMethodBinding.getReturnType();
-		if (fEnclosingMethod.getAST().resolveWellKnownType("void").equals(binding)) //$NON-NLS-1$
+		if (fEnclosingBodyDeclaration.getAST().resolveWellKnownType("void").equals(binding)) //$NON-NLS-1$
 			return true;
 		return false;
 	}
 	
 	public boolean isLastStatementSelected() {
+		return fIsLastStatementSelected;
+	}
+
+	private void computeLastStatementSelected() {
 		ASTNode[] nodes= getSelectedNodes();
-		if (nodes.length == 0)
-			return false;
-		List statements= fEnclosingMethod.getBody().statements();
-		return nodes[nodes.length - 1] == statements.get(statements.size() - 1);
+		if (nodes.length == 0) {
+			fIsLastStatementSelected= false;
+		} else {
+			Block body= null;
+			if (fEnclosingBodyDeclaration instanceof MethodDeclaration) {
+				body= ((MethodDeclaration) fEnclosingBodyDeclaration).getBody();
+			} else if (fEnclosingBodyDeclaration instanceof Initializer) {
+				body= ((Initializer) fEnclosingBodyDeclaration).getBody();
+			}
+			if (body != null) {
+				List statements= body.statements();
+				fIsLastStatementSelected= nodes[nodes.length - 1] == statements.get(statements.size() - 1);
+			}
+		}
 	}
 
 	private void computeInput() {
 		int argumentMode= FlowInfo.READ | FlowInfo.READ_POTENTIAL | FlowInfo.WRITE_POTENTIAL | FlowInfo.UNKNOWN;
-		fArguments= fInputFlowInfo.get(fInputFlowContext, argumentMode);
-		removeSelectedDeclarations(fArguments);
-		fMethodLocals= fInputFlowInfo.get(fInputFlowContext, FlowInfo.WRITE | FlowInfo.WRITE_POTENTIAL);
-		removeSelectedDeclarations(fMethodLocals);
+		fArguments= removeSelectedDeclarations(fInputFlowInfo.get(fInputFlowContext, argumentMode));
+		fMethodLocals= removeSelectedDeclarations(fInputFlowInfo.get(fInputFlowContext, FlowInfo.WRITE | FlowInfo.WRITE_POTENTIAL));
+		fTypeVariables= computeTypeVariables(fInputFlowInfo.getTypeVariables());
 	}
 	
-	private void removeSelectedDeclarations(IVariableBinding[] bindings) {
+	private IVariableBinding[] removeSelectedDeclarations(IVariableBinding[] bindings) {
+		List result= new ArrayList(bindings.length);
 		Selection selection= getSelection();
 		for (int i= 0; i < bindings.length; i++) {
-			ASTNode decl= ((CompilationUnit)fEnclosingMethod.getRoot()).findDeclaringNode(bindings[i]);
-			if (selection.covers(decl))
-				bindings[i]= null;
+			ASTNode decl= ((CompilationUnit)fEnclosingBodyDeclaration.getRoot()).findDeclaringNode(bindings[i]);
+			if (!selection.covers(decl))
+				result.add(bindings[i]);
 		}
+		return (IVariableBinding[])result.toArray(new IVariableBinding[result.size()]);
+	}
+	
+	private ITypeBinding[] computeTypeVariables(ITypeBinding[] bindings) {
+		Selection selection= getSelection();
+		Set result= new HashSet();
+		// first remove all type variables that come from outside of the method
+		// or are covered by the selection
+		CompilationUnit compilationUnit= (CompilationUnit)fEnclosingBodyDeclaration.getRoot();
+		for (int i= 0; i < bindings.length; i++) {
+			ASTNode decl= compilationUnit.findDeclaringNode(bindings[i]);
+			if (decl == null || (!selection.covers(decl) && decl.getParent() instanceof MethodDeclaration))
+				result.add(bindings[i]);
+		}
+		// all all type variables which are needed since a local variable uses it
+		for (int i= 0; i < fArguments.length; i++) {
+			IVariableBinding arg= fArguments[i];
+			ITypeBinding type= arg.getType();
+			if (type != null && type.isTypeVariable()) {
+				ASTNode decl= compilationUnit.findDeclaringNode(type);
+				if (decl == null || (!selection.covers(decl) && decl.getParent() instanceof MethodDeclaration))
+					result.add(type);
+			}
+		}
+		return (ITypeBinding[])result.toArray(new ITypeBinding[result.size()]);
 	}
 	
 	private void computeOutput(RefactoringStatus status) {
@@ -314,9 +401,13 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 		FlowInfo returnInfo= new InOutFlowAnalyzer(flowContext).perform(getSelectedNodes());
 		IVariableBinding[] returnValues= returnInfo.get(flowContext, FlowInfo.WRITE | FlowInfo.WRITE_POTENTIAL | FlowInfo.UNKNOWN);
 		
+		// Compute a selection that exactly covers the selected nodes
+		IRegion region= getSelectedNodeRange();
+		Selection selection= Selection.createFromStartLength(region.getOffset(), region.getLength());
+		
 		int counter= 0;
 		flowContext.setComputeMode(FlowContext.ARGUMENTS);
-		FlowInfo argInfo= new InputFlowAnalyzer(flowContext, getSelection()).perform(fEnclosingMethod);
+		FlowInfo argInfo= new InputFlowAnalyzer(flowContext, selection, true).perform(fEnclosingBodyDeclaration);
 		IVariableBinding[] reads= argInfo.get(flowContext, FlowInfo.READ | FlowInfo.READ_POTENTIAL | FlowInfo.UNKNOWN);
 		outer: for (int i= 0; i < returnValues.length && counter <= 1; i++) {
 			IVariableBinding binding= returnValues[i];
@@ -336,24 +427,28 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 				break;
 			default:
 				fReturnValue= null;
-				status.addFatalError(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.assignments_to_local"), JavaSourceContext.create(fCUnit, getSelection())); //$NON-NLS-1$
+				status.addFatalError(RefactoringCoreMessages.ExtractMethodAnalyzer_assignments_to_local, JavaStatusContext.create(fCUnit, getSelection())); 
 				return;
 		}
 		List callerLocals= new ArrayList(5);
-		IVariableBinding[] writes= argInfo.get(flowContext, FlowInfo.WRITE);
+		FlowInfo localInfo= new InputFlowAnalyzer(flowContext, selection, false).perform(fEnclosingBodyDeclaration);
+		IVariableBinding[] writes= localInfo.get(flowContext, FlowInfo.WRITE | FlowInfo.WRITE_POTENTIAL | FlowInfo.UNKNOWN);
 		for (int i= 0; i < writes.length; i++) {
 			IVariableBinding write= writes[i];
-			if (getSelection().covers(ASTNodes.findDeclaration(write, fEnclosingMethod)))
+			if (getSelection().covers(ASTNodes.findDeclaration(write, fEnclosingBodyDeclaration)))
 				callerLocals.add(write);
 		}
 		fCallerLocals= (IVariableBinding[])callerLocals.toArray(new IVariableBinding[callerLocals.size()]);
-		if (fReturnValue != null && getSelection().covers(ASTNodes.findDeclaration(fReturnValue, fEnclosingMethod)))
+		if (fReturnValue != null && getSelection().covers(ASTNodes.findDeclaration(fReturnValue, fEnclosingBodyDeclaration)))
 			fReturnLocal= fReturnValue;
 	}
 	
 	private void adjustArgumentsAndMethodLocals() {
 		for (int i= 0; i < fArguments.length; i++) {
 			IVariableBinding argument= fArguments[i];
+			// Both arguments and locals consider FlowInfo.WRITE_POTENTIAL. But at the end a variable
+			// can either be a local of an argument. Fix this based on the compute return type which
+			// didn't exist when we computed the locals and arguments (see computeInput())
 			if (fInputFlowInfo.hasAccessMode(fInputFlowContext, argument, FlowInfo.WRITE_POTENTIAL)) {
 				if (argument != fReturnValue)
 					fArguments[i]= null;
@@ -405,7 +500,7 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 		List result= new ArrayList(fAllExceptions.length);
 		for (int i= 0; i < fAllExceptions.length; i++) {
 			ITypeBinding exception= fAllExceptions[i];
-			if (!includeRuntimeExceptions && Bindings.isRuntimeException(exception, ast))
+			if (!includeRuntimeExceptions && Bindings.isRuntimeException(exception))
 				continue;
 			result.add(exception);
 		}
@@ -413,7 +508,7 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 	}
 	
 	private void computeExceptions() {
-		fAllExceptions= ExceptionAnalyzer.perform(fEnclosingMethod, getSelectedNodes());
+		fAllExceptions= ExceptionAnalyzer.perform(getSelectedNodes());
 	}
 	
 	//---- Special visitor methods ---------------------------------------------------------------------------
@@ -423,9 +518,9 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 		checkParent(node);
 	}
 	
-	protected void handleSelectionEndsIn(ASTNode node) {
-		invalidSelection(RefactoringCoreMessages.getString("StatementAnalyzer.doesNotCover"), JavaSourceContext.create(fCUnit, node)); //$NON-NLS-1$
-		super.handleSelectionEndsIn(node);
+	protected boolean handleSelectionEndsIn(ASTNode node) {
+		invalidSelection(RefactoringCoreMessages.StatementAnalyzer_doesNotCover, JavaStatusContext.create(fCUnit, node)); 
+		return super.handleSelectionEndsIn(node);
 	}
 		
 	private void checkParent(ASTNode node) {
@@ -435,7 +530,7 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 			if (node == firstParent)
 				return;
 		} while (node != null);
-		invalidSelection(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.parent_mismatch")); //$NON-NLS-1$
+		invalidSelection(RefactoringCoreMessages.ExtractMethodAnalyzer_parent_mismatch); 
 	}
 	
 	public void endVisit(CompilationUnit node) {
@@ -449,32 +544,42 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 					MethodDeclaration methodDecl= (MethodDeclaration)coveringNode.getParent();
 					Message[] messages= ASTNodes.getMessages(methodDecl, ASTNodes.NODE_ONLY);
 					if (messages.length > 0) {
-						status.addFatalError(RefactoringCoreMessages.getFormattedString(
-							"ExtractMethodAnalyzer.compile_errors", //$NON-NLS-1$
-							methodDecl.getName().getIdentifier()), JavaSourceContext.create(fCUnit, methodDecl));
+						status.addFatalError(Messages.format(
+							RefactoringCoreMessages.ExtractMethodAnalyzer_compile_errors, 
+							methodDecl.getName().getIdentifier()), JavaStatusContext.create(fCUnit, methodDecl));
 						break superCall;
 					}
 				}
-				status.addFatalError(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.only_method_body")); //$NON-NLS-1$
+				status.addFatalError(RefactoringCoreMessages.ExtractMethodAnalyzer_only_method_body); 
 				break superCall;
 			}
-			fEnclosingMethod= (MethodDeclaration)ASTNodes.getParent(getFirstSelectedNode(), MethodDeclaration.class);
-			if (fEnclosingMethod == null) {
-				status.addFatalError(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.only_method_body")); //$NON-NLS-1$
+			fEnclosingBodyDeclaration= (BodyDeclaration)ASTNodes.getParent(getFirstSelectedNode(), BodyDeclaration.class);
+			if (fEnclosingBodyDeclaration == null || 
+				(fEnclosingBodyDeclaration.getNodeType() != ASTNode.METHOD_DECLARATION && 
+				 fEnclosingBodyDeclaration.getNodeType() != ASTNode.INITIALIZER)) {
+				status.addFatalError(RefactoringCoreMessages.ExtractMethodAnalyzer_only_method_body); 
 				break superCall;
 			} else {
-				fEnclosingMethodBinding= fEnclosingMethod.resolveBinding();
+				if (fEnclosingBodyDeclaration.getNodeType() == ASTNode.METHOD_DECLARATION)
+					fEnclosingMethodBinding= ((MethodDeclaration)fEnclosingBodyDeclaration).resolveBinding();
+			}
+			if (!isSingleExpressionOrStatementSet()) {
+				status.addFatalError(RefactoringCoreMessages.ExtractMethodAnalyzer_single_expression_or_set); 
+				break superCall;
 			}
 			if (isExpressionSelected()) {
 				ASTNode expression= getFirstSelectedNode();
 				if (expression instanceof Name) {
 					Name name= (Name)expression;
 					if (name.resolveBinding() instanceof ITypeBinding) {
-						status.addFatalError(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.cannot_extract_type_reference")); //$NON-NLS-1$
+						status.addFatalError(RefactoringCoreMessages.ExtractMethodAnalyzer_cannot_extract_type_reference); 
 						break superCall;
 					}
+					if (name.resolveBinding() instanceof IMethodBinding) {
+						status.addFatalError(RefactoringCoreMessages.ExtractMethodAnalyzer_cannot_extract_method_name_reference); 
+					}
 					if (name.isSimpleName() && ((SimpleName)name).isDeclaration()) {
-						status.addFatalError(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.cannot_extract_name_in_declaration")); //$NON-NLS-1$
+						status.addFatalError(RefactoringCoreMessages.ExtractMethodAnalyzer_cannot_extract_name_in_declaration); 
 						break superCall;
 					}
 				}
@@ -482,7 +587,8 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 					ASTNodes.getParent(expression, ASTNode.SUPER_CONSTRUCTOR_INVOCATION) != null ||
 					ASTNodes.getParent(expression, ASTNode.CONSTRUCTOR_INVOCATION) != null;
 			}
-			status.merge(LocalTypeAnalyzer.perform(fEnclosingMethod, getSelection()));
+			status.merge(LocalTypeAnalyzer.perform(fEnclosingBodyDeclaration, getSelection()));
+			computeLastStatementSelected();
 		}
 		super.endVisit(node);
 	}
@@ -490,7 +596,18 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 	public boolean visit(AnonymousClassDeclaration node) {
 		boolean result= super.visit(node);
 		if (isFirstSelectedNode(node)) {
-			invalidSelection(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.cannot_extract_anonymous_type"), JavaSourceContext.create(fCUnit, node)); //$NON-NLS-1$
+			invalidSelection(RefactoringCoreMessages.ExtractMethodAnalyzer_cannot_extract_anonymous_type, JavaStatusContext.create(fCUnit, node)); 
+			return false;
+		}
+		return result;
+	}
+	
+	public boolean visit(Assignment node) {
+		boolean result= super.visit(node);
+		if (getSelection().getVisitSelectionMode(node.getLeftHandSide()) == Selection.SELECTED) {
+			invalidSelection(
+				RefactoringCoreMessages.ExtractMethodAnalyzer_leftHandSideOfAssignment,  
+				JavaStatusContext.create(fCUnit, node));
 			return false;
 		}
 		return result;
@@ -501,7 +618,7 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 		
 		int actionStart= getBuffer().indexAfter(ITerminalSymbols.TokenNamedo, node.getStartPosition());
 		if (getSelection().getOffset() == actionStart) {
-			invalidSelection(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.after_do_keyword"), JavaSourceContext.create(fCUnit, getSelection())); //$NON-NLS-1$
+			invalidSelection(RefactoringCoreMessages.ExtractMethodAnalyzer_after_do_keyword, JavaStatusContext.create(fCUnit, getSelection())); 
 			return false;
 		}
 		
@@ -510,7 +627,13 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 
 	public boolean visit(MethodDeclaration node) {
 		Block body= node.getBody();
-		if (body == null || !getSelection().enclosedBy(body))
+		if (body == null)
+			return false;
+		Selection selection= getSelection();
+		int nodeStart= body.getStartPosition();
+		int nodeExclusiveEnd= nodeStart + body.getLength();
+		// if selection node inside of the method body ignore method
+		if (!(nodeStart < selection.getOffset() && selection.getExclusiveEnd() < nodeExclusiveEnd))
 			return false;
 		return super.visit(node);
 	}
@@ -525,7 +648,7 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 	
 	private boolean visitConstructorInvocation(ASTNode node, boolean superResult) {
 		if (getSelection().getVisitSelectionMode(node) == Selection.SELECTED) {
-			invalidSelection(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.super_or_this"), JavaSourceContext.create(fCUnit, node)); //$NON-NLS-1$
+			invalidSelection(RefactoringCoreMessages.ExtractMethodAnalyzer_super_or_this, JavaStatusContext.create(fCUnit, node)); 
 			return false;
 		}
 		return superResult;
@@ -534,7 +657,7 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 	public boolean visit(VariableDeclarationFragment node) {
 		boolean result= super.visit(node);
 		if (isFirstSelectedNode(node)) {
-			invalidSelection(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.cannot_extract_variable_declaration_fragment"), JavaSourceContext.create(fCUnit, node)); //$NON-NLS-1$
+			invalidSelection(RefactoringCoreMessages.ExtractMethodAnalyzer_cannot_extract_variable_declaration_fragment, JavaStatusContext.create(fCUnit, node)); 
 			return false;
 		}
 		return result;
@@ -543,9 +666,9 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 	public void endVisit(ForStatement node) {
 		if (getSelection().getEndVisitSelectionMode(node) == Selection.AFTER) {
 			if (node.initializers().contains(getFirstSelectedNode())) {
-				invalidSelection(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.cannot_extract_for_initializer"), JavaSourceContext.create(fCUnit, getSelection())); //$NON-NLS-1$
+				invalidSelection(RefactoringCoreMessages.ExtractMethodAnalyzer_cannot_extract_for_initializer, JavaStatusContext.create(fCUnit, getSelection())); 
 			} else if (node.updaters().contains(getLastSelectedNode())) {
-				invalidSelection(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.cannot_extract_for_updater"), JavaSourceContext.create(fCUnit, getSelection())); //$NON-NLS-1$
+				invalidSelection(RefactoringCoreMessages.ExtractMethodAnalyzer_cannot_extract_for_updater, JavaStatusContext.create(fCUnit, getSelection())); 
 			}
 		}
 		super.endVisit(node);
@@ -567,8 +690,17 @@ import org.eclipse.jdt.internal.corext.refactoring.util.CodeAnalyzer;
 	
 	private void checkTypeInDeclaration(Type node) {
 		if (getSelection().getEndVisitSelectionMode(node) == Selection.SELECTED && getFirstSelectedNode() == node) {
-			invalidSelection(RefactoringCoreMessages.getString("ExtractMethodAnalyzer.cannot_extract_variable_declaration"), JavaSourceContext.create(fCUnit, getSelection())); //$NON-NLS-1$
+			invalidSelection(RefactoringCoreMessages.ExtractMethodAnalyzer_cannot_extract_variable_declaration, JavaStatusContext.create(fCUnit, getSelection())); 
 		}
 	}
+	
+	private boolean isSingleExpressionOrStatementSet() {
+		ASTNode first= getFirstSelectedNode();
+		if (first == null)
+			return true;
+		if (first instanceof Expression && getSelectedNodes().length != 1)
+			return false;
+		return true;
+	}
 }
 
