@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -20,6 +20,7 @@ import java.util.Map;
 import java.util.Set;
 import java.util.Stack;
 
+import org.eclipse.core.runtime.Assert;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.SubProgressMonitor;
 
@@ -28,6 +29,7 @@ import org.eclipse.ltk.core.refactoring.RefactoringStatus;
 import org.eclipse.jdt.core.ICompilationUnit;
 import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IMethod;
+import org.eclipse.jdt.core.IType;
 import org.eclipse.jdt.core.JavaModelException;
 import org.eclipse.jdt.core.dom.AST;
 import org.eclipse.jdt.core.dom.ASTNode;
@@ -51,10 +53,8 @@ import org.eclipse.jdt.core.search.IJavaSearchConstants;
 import org.eclipse.jdt.core.search.SearchMatch;
 import org.eclipse.jdt.core.search.SearchPattern;
 
-import org.eclipse.jdt.internal.corext.Assert;
 import org.eclipse.jdt.internal.corext.SourceRange;
 import org.eclipse.jdt.internal.corext.dom.ASTNodes;
-import org.eclipse.jdt.internal.corext.dom.Bindings;
 import org.eclipse.jdt.internal.corext.refactoring.IRefactoringSearchRequestor;
 import org.eclipse.jdt.internal.corext.refactoring.RefactoringCoreMessages;
 import org.eclipse.jdt.internal.corext.refactoring.RefactoringScopeFactory;
@@ -63,10 +63,16 @@ import org.eclipse.jdt.internal.corext.refactoring.base.JavaStatusContext;
 import org.eclipse.jdt.internal.corext.refactoring.util.RefactoringASTParser;
 import org.eclipse.jdt.internal.corext.util.SearchUtils;
 
+/**
+ * A TargetProvider provides all targets that have to be adapted, i.e. all method invocations that should be inlined. 
+ */
 abstract class TargetProvider {
 
+	public static final boolean BUG_CORE_130317= true; // https://bugs.eclipse.org/bugs/show_bug.cgi?id=130317
+	
 	protected SourceProvider fSourceProvider;
 
+	//TODO: not used...
 	public void setSourceProvider(SourceProvider sourceProvider) {
 		Assert.isNotNull(sourceProvider);
 		fSourceProvider= sourceProvider;
@@ -85,6 +91,10 @@ abstract class TargetProvider {
 	
 	public abstract int getStatusSeverity();
 	
+	public boolean isSingle() {
+		return false;
+	}
+	
 	public static TargetProvider create(ICompilationUnit cu, MethodInvocation invocation) {
 		return new SingleCallTargetProvider(cu, invocation);
 	}
@@ -97,17 +107,27 @@ abstract class TargetProvider {
 		return new SingleCallTargetProvider(cu, invocation);
 	}
 
-	public static TargetProvider create(ICompilationUnit cu, MethodDeclaration declaration) {
+	public static TargetProvider create(MethodDeclaration declaration) {
 		IMethodBinding method= declaration.resolveBinding();
 		if (method == null)
 			return new ErrorTargetProvider(RefactoringStatus.createFatalErrorStatus(RefactoringCoreMessages.TargetProvider_method_declaration_not_unique));
 		ITypeBinding type= method.getDeclaringClass();
-		if (type.isLocal())
-			return new LocalTypeTargetProvider(cu, declaration);
-		else
-			return new MemberTypeTargetProvider(declaration);
+		if (type.isLocal()) {
+			if (((IType) type.getJavaElement()).isBinary()) {
+				return new ErrorTargetProvider(RefactoringStatus.createFatalErrorStatus(RefactoringCoreMessages.TargetProvider_cannot_local_method_in_binary));
+			} else {
+				IType declaringClassOfLocal= (IType) type.getDeclaringClass().getJavaElement();
+				return new LocalTypeTargetProvider(declaringClassOfLocal.getCompilationUnit(), declaration);
+			}
+		} else {
+			return new MemberTypeTargetProvider(declaration.resolveBinding());
+		}
 	}
 
+	public static TargetProvider create(IMethodBinding methodBinding) {
+		return new MemberTypeTargetProvider(methodBinding);
+	}
+	
 	static void fastDone(IProgressMonitor pm) {
 		if (pm == null)
 			return;
@@ -180,6 +200,9 @@ abstract class TargetProvider {
 		public int getStatusSeverity() {
 			return RefactoringStatus.FATAL;
 		}
+		public boolean isSingle() {
+			return true;
+		}
 	}
 
 	private static class BodyData {
@@ -205,8 +228,8 @@ abstract class TargetProvider {
 	}
 
 	private static class InvocationFinder extends ASTVisitor {
-		Map result= new HashMap(2);
-		Stack fBodies= new Stack();
+		Map/*<BodyDeclaration, BodyData>*/ result= new HashMap(2);
+		Stack/*<BodyData>*/ fBodies= new Stack();
 		BodyData fCurrent;
 		private IMethodBinding fBinding;
 		public InvocationFinder(IMethodBinding binding) {
@@ -301,9 +324,10 @@ abstract class TargetProvider {
 		private boolean matches(IBinding binding) {
 			if (!(binding instanceof IMethodBinding))
 				return false;
-			// https://bugs.eclipse.org/bugs/show_bug.cgi?id=87777
-			// return fBinding.isEqualTo(((IMethodBinding)binding).getMethodDeclaration());
-			return Bindings.equals(fBinding, ((IMethodBinding)binding).getMethodDeclaration());
+			if (BUG_CORE_130317)
+				return fBinding.getKey().equals(((IMethodBinding)binding).getMethodDeclaration().getKey());
+			else
+				return fBinding.isEqualTo(((IMethodBinding)binding).getMethodDeclaration());
 		}
 	}
 	
@@ -337,7 +361,7 @@ abstract class TargetProvider {
 	
 		public ASTNode[] getInvocations(BodyDeclaration declaration, IProgressMonitor pm) {
 			BodyData data= (BodyData)fBodies.get(declaration);
-			Assert.isTrue(data != null);
+			Assert.isNotNull(data);
 			fastDone(pm);
 			return data.getInvocations();
 		}
@@ -352,18 +376,18 @@ abstract class TargetProvider {
 	}
 	
 	private static class MemberTypeTargetProvider extends TargetProvider {
-		private MethodDeclaration fMethod;
+		private final IMethodBinding fMethodBinding;
 		private Map fCurrentBodies;
-		public MemberTypeTargetProvider(MethodDeclaration method) {
-			Assert.isNotNull(method);
-			fMethod= method;
+		public MemberTypeTargetProvider(IMethodBinding methodBinding) {
+			Assert.isNotNull(methodBinding);
+			fMethodBinding= methodBinding;
 		}
 		public void initialize() {
 			// do nothing.
 		}
 
 		public ICompilationUnit[] getAffectedCompilationUnits(final RefactoringStatus status, IProgressMonitor pm) throws JavaModelException {
-			IMethod method= (IMethod)fMethod.resolveBinding().getJavaElement();
+			IMethod method= (IMethod)fMethodBinding.getJavaElement();
 			Assert.isTrue(method != null);
 			final RefactoringSearchEngine2 engine= new RefactoringSearchEngine2(SearchPattern.createPattern(method, IJavaSearchConstants.REFERENCES, SearchUtils.GENERICS_AGNOSTIC_MATCH_RULE));
 			engine.setGranularity(RefactoringSearchEngine2.GRANULARITY_COMPILATION_UNIT);
@@ -397,7 +421,7 @@ abstract class TargetProvider {
 
 		public BodyDeclaration[] getAffectedBodyDeclarations(ICompilationUnit unit, IProgressMonitor pm) {
 			ASTNode root= new RefactoringASTParser(AST.JLS3).parse(unit, true);
-			InvocationFinder finder= new InvocationFinder(fMethod.resolveBinding());
+			InvocationFinder finder= new InvocationFinder(fMethodBinding);
 			root.accept(finder);
 			fCurrentBodies= finder.result;
 			Set result= fCurrentBodies.keySet();
@@ -407,7 +431,7 @@ abstract class TargetProvider {
 	
 		public ASTNode[] getInvocations(BodyDeclaration declaration, IProgressMonitor pm) {
 			BodyData data= (BodyData)fCurrentBodies.get(declaration);
-			Assert.isTrue(data != null);
+			Assert.isNotNull(data);
 			fastDone(pm);
 			return data.getInvocations();
 		}
