@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -19,6 +19,8 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import org.eclipse.core.runtime.Assert;
+
 import org.eclipse.core.resources.IFile;
 
 import org.eclipse.jdt.core.IClassFile;
@@ -27,8 +29,6 @@ import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.ISourceReference;
 import org.eclipse.jdt.core.JavaModelException;
 
-import org.eclipse.jdt.internal.corext.Assert;
-import org.eclipse.jdt.internal.corext.util.JavaModelUtil;
 
 public class SourceReferenceUtil {
 	
@@ -37,7 +37,7 @@ public class SourceReferenceUtil {
 	
 	public static IFile getFile(ISourceReference ref) {
 		ICompilationUnit unit= getCompilationUnit(ref);
-		return (IFile) JavaModelUtil.toOriginal(unit).getResource();
+		return (IFile) unit.getPrimary().getResource();
 	}
 	
 	public static ICompilationUnit getCompilationUnit(ISourceReference o){
