@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,10 +13,13 @@ package org.eclipse.jdt.internal.ui.compare;
 /**
  * Provides "Replace from local history" for Java elements.
  */
-public class JavaReplaceWithPreviousEditionAction extends JavaReplaceWithEditionAction {
+public class JavaReplaceWithPreviousEditionAction extends JavaHistoryAction {
 					
 	public JavaReplaceWithPreviousEditionAction() {
-		super(true);
 	}	
+	
+	protected JavaHistoryActionImpl createDelegate() {
+		return new JavaReplaceWithEditionActionImpl(true);
+	}
 }
 
