@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -20,14 +20,15 @@ import java.util.Set;
 import java.util.jar.Attributes;
 import java.util.jar.Manifest;
 
-import org.eclipse.core.resources.IFile;
-import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.IWorkspace;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Path;
 
+import org.eclipse.core.resources.IFile;
+import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.IWorkspace;
+
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.KeyAdapter;
 import org.eclipse.swt.events.KeyEvent;
@@ -44,22 +45,21 @@ import org.eclipse.swt.widgets.Listener;
 import org.eclipse.swt.widgets.Text;
 
 import org.eclipse.jface.dialogs.Dialog;
-import org.eclipse.jface.dialogs.DialogPage;
 import org.eclipse.jface.dialogs.IDialogSettings;
+import org.eclipse.jface.dialogs.IMessageProvider;
 import org.eclipse.jface.resource.JFaceResources;
-import org.eclipse.jface.viewers.ILabelProvider;
-import org.eclipse.jface.viewers.ITreeContentProvider;
+import org.eclipse.jface.viewers.DecoratingLabelProvider;
+import org.eclipse.jface.window.Window;
 import org.eclipse.jface.wizard.IWizardPage;
 import org.eclipse.jface.wizard.WizardPage;
 
+import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.dialogs.ElementTreeSelectionDialog;
 import org.eclipse.ui.dialogs.ISelectionStatusValidator;
 import org.eclipse.ui.dialogs.SaveAsDialog;
 import org.eclipse.ui.dialogs.SelectionDialog;
-import org.eclipse.ui.help.WorkbenchHelp;
 
 import org.eclipse.jdt.core.ICompilationUnit;
-import org.eclipse.jdt.core.IJavaElement;
 import org.eclipse.jdt.core.IPackageFragment;
 import org.eclipse.jdt.core.IPackageFragmentRoot;
 import org.eclipse.jdt.core.IType;
@@ -67,18 +67,19 @@ import org.eclipse.jdt.core.JavaCore;
 import org.eclipse.jdt.core.JavaModelException;
 import org.eclipse.jdt.core.search.IJavaSearchScope;
 
-import org.eclipse.jdt.ui.JavaElementLabelProvider;
-import org.eclipse.jdt.ui.JavaUI;
-import org.eclipse.jdt.ui.StandardJavaElementContentProvider;
+import org.eclipse.jdt.internal.corext.util.Messages;
 
+import org.eclipse.jdt.ui.JavaElementLabelProvider;
+import org.eclipse.jdt.ui.JavaElementSorter;
+import org.eclipse.jdt.ui.JavaUI;
+import org.eclipse.jdt.ui.ProblemsLabelDecorator;
+import org.eclipse.jdt.ui.StandardJavaElementContentProvider;
 import org.eclipse.jdt.ui.jarpackager.JarPackageData;
 
 import org.eclipse.jdt.internal.ui.IJavaHelpContextIds;
 import org.eclipse.jdt.internal.ui.JavaPlugin;
-
 import org.eclipse.jdt.internal.ui.dialogs.StatusInfo;
 import org.eclipse.jdt.internal.ui.filters.EmptyInnerPackageFilter;
-
 import org.eclipse.jdt.internal.ui.search.JavaSearchScopeFactory;
 import org.eclipse.jdt.internal.ui.util.ExceptionHandler;
 import org.eclipse.jdt.internal.ui.util.MainMethodSearchEngine;
@@ -152,8 +153,8 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 	 */
 	public JarManifestWizardPage(JarPackageData jarPackage) {
 		super(PAGE_NAME);
-		setTitle(JarPackagerMessages.getString("JarManifestWizardPage.title")); //$NON-NLS-1$
-		setDescription(JarPackagerMessages.getString("JarManifestWizardPage.description")); //$NON-NLS-1$
+		setTitle(JarPackagerMessages.JarManifestWizardPage_title); 
+		setDescription(JarPackagerMessages.JarManifestWizardPage_description); 
 		fJarPackage= jarPackage;
 	}
 
@@ -170,17 +171,17 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 		composite.setLayout(new GridLayout());
 		composite.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_FILL | GridData.HORIZONTAL_ALIGN_FILL));
 
-		createLabel(composite, JarPackagerMessages.getString("JarManifestWizardPage.manifestSource.label"), false); //$NON-NLS-1$
+		createLabel(composite, JarPackagerMessages.JarManifestWizardPage_manifestSource_label, false); 
 		createManifestGroup(composite);
 
 		createSpacer(composite);
 
-		fSealingHeaderLabel= createLabel(composite, JarPackagerMessages.getString("JarManifestWizardPage.sealingHeader.label"), false); //$NON-NLS-1$
+		fSealingHeaderLabel= createLabel(composite, JarPackagerMessages.JarManifestWizardPage_sealingHeader_label, false); 
 		createSealingGroup(composite);
 
 		createSpacer(composite);
 
-		fMainClassHeaderLabel= createLabel(composite, JarPackagerMessages.getString("JarManifestWizardPage.mainClassHeader.label"), false); //$NON-NLS-1$
+		fMainClassHeaderLabel= createLabel(composite, JarPackagerMessages.JarManifestWizardPage_mainClassHeader_label, false); 
 		createMainClassGroup(composite);
 
 		setEqualButtonSizes();
@@ -191,7 +192,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 		update();
 
 		Dialog.applyDialogFont(composite);
-		WorkbenchHelp.setHelp(composite, IJavaHelpContextIds.JARMANIFEST_WIZARD_PAGE);								
+		PlatformUI.getWorkbench().getHelpSystem().setHelp(composite, IJavaHelpContextIds.JARMANIFEST_WIZARD_PAGE);								
 			
 	}
 	/**
@@ -206,7 +207,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 		fManifestGroup.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
 
 		fGenerateManifestRadioButton= new Button(fManifestGroup, SWT.RADIO | SWT.LEFT);
-		fGenerateManifestRadioButton.setText(JarPackagerMessages.getString("JarManifestWizardPage.genetateManifest.text")); //$NON-NLS-1$
+		fGenerateManifestRadioButton.setText(JarPackagerMessages.JarManifestWizardPage_genetateManifest_text); 
 		fGenerateManifestRadioButton.addListener(SWT.Selection, fUntypedListener);
 
 			Composite saveOptions= new Composite(fManifestGroup, SWT.NONE);
@@ -219,17 +220,17 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 			saveOptions.setLayoutData(data);
 	
 			fSaveManifestCheckbox= new Button(saveOptions, SWT.CHECK | SWT.LEFT);
-			fSaveManifestCheckbox.setText(JarPackagerMessages.getString("JarManifestWizardPage.saveManifest.text")); //$NON-NLS-1$
+			fSaveManifestCheckbox.setText(JarPackagerMessages.JarManifestWizardPage_saveManifest_text); 
 			fSaveManifestCheckbox.addListener(SWT.MouseUp, fUntypedListener);
 	
 			fReuseManifestCheckbox= new Button(saveOptions, SWT.CHECK | SWT.LEFT);
-			fReuseManifestCheckbox.setText(JarPackagerMessages.getString("JarManifestWizardPage.reuseManifest.text")); //$NON-NLS-1$
+			fReuseManifestCheckbox.setText(JarPackagerMessages.JarManifestWizardPage_reuseManifest_text); 
 			fReuseManifestCheckbox.addListener(SWT.MouseUp, fUntypedListener);
 			
 			createNewManifestFileGroup(saveOptions);
 
 		fUseManifestRadioButton= new Button(fManifestGroup, SWT.RADIO | SWT.LEFT);
-		fUseManifestRadioButton.setText(JarPackagerMessages.getString("JarManifestWizardPage.useManifest.text")); //$NON-NLS-1$
+		fUseManifestRadioButton.setText(JarPackagerMessages.JarManifestWizardPage_useManifest_text); 
 		
 		fUseManifestRadioButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
 			Composite existingManifestGroup= new Composite(fManifestGroup, SWT.NONE);
@@ -252,7 +253,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 		manifestFileGroup.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
 
 		fNewManifestFileLabel= new Label(manifestFileGroup, SWT.NONE);
-		fNewManifestFileLabel.setText(JarPackagerMessages.getString("JarManifestWizardPage.newManifestFile.text")); //$NON-NLS-1$
+		fNewManifestFileLabel.setText(JarPackagerMessages.JarManifestWizardPage_newManifestFile_text); 
 
 		// entry field
 		fNewManifestFileText= new Text(manifestFileGroup, SWT.SINGLE | SWT.BORDER);
@@ -264,7 +265,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 
 		// browse button
 		fNewManifestFileBrowseButton= new Button(manifestFileGroup, SWT.PUSH);
-		fNewManifestFileBrowseButton.setText(JarPackagerMessages.getString("JarManifestWizardPage.newManifestFileBrowseButton.text")); //$NON-NLS-1$
+		fNewManifestFileBrowseButton.setText(JarPackagerMessages.JarManifestWizardPage_newManifestFileBrowseButton_text); 
 		fNewManifestFileBrowseButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
 		fNewManifestFileBrowseButton.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
@@ -284,7 +285,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 		manifestFileGroup.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
 
 		fManifestFileLabel= new Label(manifestFileGroup, SWT.NONE);
-		fManifestFileLabel.setText(JarPackagerMessages.getString("JarManifestWizardPage.manifestFile.text")); //$NON-NLS-1$
+		fManifestFileLabel.setText(JarPackagerMessages.JarManifestWizardPage_manifestFile_text); 
 
 		// entry field
 		fManifestFileText= new Text(manifestFileGroup, SWT.SINGLE | SWT.BORDER);
@@ -295,7 +296,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 
 		// browse button
 		fManifestFileBrowseButton= new Button(manifestFileGroup, SWT.PUSH);
-		fManifestFileBrowseButton.setText(JarPackagerMessages.getString("JarManifestWizardPage.manifestFileBrowse.text")); //$NON-NLS-1$
+		fManifestFileBrowseButton.setText(JarPackagerMessages.JarManifestWizardPage_manifestFileBrowse_text); 
 		fManifestFileBrowseButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
 		fManifestFileBrowseButton.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
@@ -323,11 +324,11 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 	/**
 	 * Creates the JAR sealing specification controls to seal the whole JAR.
 	 *
-	 * @param parent the parent control
+	 * @param sealGroup the parent control
 	 */
 	protected void createSealJarGroup(Composite sealGroup) {
 		fSealJarRadioButton= new Button(sealGroup, SWT.RADIO);
-		fSealJarRadioButton.setText(JarPackagerMessages.getString("JarManifestWizardPage.sealJar.text")); //$NON-NLS-1$
+		fSealJarRadioButton.setText(JarPackagerMessages.JarManifestWizardPage_sealJar_text); 
 		fSealJarRadioButton.addListener(SWT.Selection, fUntypedListener);
 
 		fSealJarLabel= new Label(sealGroup, SWT.RIGHT);
@@ -336,7 +337,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 		
 		fUnSealedPackagesDetailsButton= new Button(sealGroup, SWT.PUSH);
 		fUnSealedPackagesDetailsButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
-		fUnSealedPackagesDetailsButton.setText(JarPackagerMessages.getString("JarManifestWizardPage.unsealPackagesButton.text")); //$NON-NLS-1$
+		fUnSealedPackagesDetailsButton.setText(JarPackagerMessages.JarManifestWizardPage_unsealPackagesButton_text); 
 		fUnSealedPackagesDetailsButton.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
 				handleUnSealPackagesDetailsButtonPressed();
@@ -347,11 +348,11 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 	/**
 	 * Creates the JAR sealing specification controls to seal packages.
 	 *
-	 * @param parent the parent control
+	 * @param sealGroup the parent control
 	 */
 	protected void createSealPackagesGroup(Composite sealGroup) {
 		fSealPackagesRadioButton= new Button(sealGroup, SWT.RADIO);
-		fSealPackagesRadioButton.setText(JarPackagerMessages.getString("JarManifestWizardPage.sealPackagesButton.text")); //$NON-NLS-1$
+		fSealPackagesRadioButton.setText(JarPackagerMessages.JarManifestWizardPage_sealPackagesButton_text); 
 
 		fSealPackagesLabel= new Label(sealGroup, SWT.RIGHT);
 		fSealPackagesLabel.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
@@ -359,7 +360,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 
 		fSealedPackagesDetailsButton= new Button(sealGroup, SWT.PUSH);
 		fSealedPackagesDetailsButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
-		fSealedPackagesDetailsButton.setText(JarPackagerMessages.getString("JarManifestWizardPage.sealedPackagesDetailsButton.text")); //$NON-NLS-1$
+		fSealedPackagesDetailsButton.setText(JarPackagerMessages.JarManifestWizardPage_sealedPackagesDetailsButton_text); 
 		fSealedPackagesDetailsButton.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
 				handleSealPackagesDetailsButtonPressed();
@@ -376,7 +377,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 		mainClassGroup.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL));
 
 		fMainClassLabel= new Label(mainClassGroup, SWT.NONE);
-		fMainClassLabel.setText(JarPackagerMessages.getString("JarManifestWizardPage.mainClass.label")); //$NON-NLS-1$
+		fMainClassLabel.setText(JarPackagerMessages.JarManifestWizardPage_mainClass_label); 
 
 		// entry field
 		fMainClassText= new Text(mainClassGroup, SWT.SINGLE | SWT.BORDER);
@@ -396,7 +397,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 
 		// browse button
 		fMainClassBrowseButton= new Button(mainClassGroup, SWT.PUSH);
-		fMainClassBrowseButton.setText(JarPackagerMessages.getString("JarManifestWizardPage.mainClassBrowseButton.text")); //$NON-NLS-1$
+		fMainClassBrowseButton.setText(JarPackagerMessages.JarManifestWizardPage_mainClassBrowseButton_text); 
 		fMainClassBrowseButton.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL));
 		fMainClassBrowseButton.addSelectionListener(new SelectionAdapter() {
 			public void widgetSelected(SelectionEvent e) {
@@ -421,23 +422,23 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 		// Use Save As dialog to select a new file inside the workspace
 		SaveAsDialog dialog= new SaveAsDialog(getContainer().getShell());
 		dialog.create();
-		dialog.getShell().setText(JarPackagerMessages.getString("JarManifestWizardPage.saveAsDialog.title")); //$NON-NLS-1$
-		dialog.setMessage(JarPackagerMessages.getString("JarManifestWizardPage.saveAsDialog.message")); //$NON-NLS-1$
+		dialog.getShell().setText(JarPackagerMessages.JarManifestWizardPage_saveAsDialog_title); 
+		dialog.setMessage(JarPackagerMessages.JarManifestWizardPage_saveAsDialog_message); 
 		dialog.setOriginalFile(createFileHandle(fJarPackage.getManifestLocation()));
-		if (dialog.open() == SaveAsDialog.OK) {
+		if (dialog.open() == Window.OK) {
 			fJarPackage.setManifestLocation(dialog.getResult());
 			fNewManifestFileText.setText(dialog.getResult().toString());
 		}
 	}
 
 	protected void handleManifestFileBrowseButtonPressed() {
-		ElementTreeSelectionDialog dialog= createWorkspaceFileSelectionDialog(JarPackagerMessages.getString("JarManifestWizardPage.manifestSelectionDialog.title"), JarPackagerMessages.getString("JarManifestWizardPage.manifestSelectionDialog.message")); //$NON-NLS-2$ //$NON-NLS-1$
+		ElementTreeSelectionDialog dialog= createWorkspaceFileSelectionDialog(JarPackagerMessages.JarManifestWizardPage_manifestSelectionDialog_title, JarPackagerMessages.JarManifestWizardPage_manifestSelectionDialog_message); 
 		if (fJarPackage.isManifestAccessible())
 			dialog.setInitialSelections(new IResource[] {fJarPackage.getManifestFile()});
-		if (dialog.open() ==  ElementTreeSelectionDialog.OK) {
+		if (dialog.open() ==  Window.OK) {
 			Object[] resources= dialog.getResult();
 			if (resources.length != 1)
-				setErrorMessage(JarPackagerMessages.getString("JarManifestWizardPage.error.onlyOneManifestMustBeSelected")); //$NON-NLS-1$
+				setErrorMessage(JarPackagerMessages.JarManifestWizardPage_error_onlyOneManifestMustBeSelected); 
 			else {
 				setErrorMessage(""); //$NON-NLS-1$
 				fJarPackage.setManifestLocation(((IResource)resources[0]).getFullPath());
@@ -449,9 +450,11 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 	private IType findMainMethodByName(String name) {
 		if (fMainTypes == null) {
 			List resources= JarPackagerUtil.asResources(fJarPackage.getElements());
-			if (resources == null)
-				setErrorMessage(JarPackagerMessages.getString("JarManifestWizardPage.error.noResourceSelected")); //$NON-NLS-1$
-			IJavaSearchScope searchScope= JavaSearchScopeFactory.getInstance().createJavaSearchScope((IResource[])resources.toArray(new IResource[resources.size()]));
+			if (resources == null) {
+				setErrorMessage(JarPackagerMessages.JarManifestWizardPage_error_noResourceSelected);
+				return null;
+			}
+			IJavaSearchScope searchScope= JavaSearchScopeFactory.getInstance().createJavaSearchScope((IResource[])resources.toArray(new IResource[resources.size()]), true);
 			MainMethodSearchEngine engine= new MainMethodSearchEngine();
 			try {
 				fMainTypes= engine.searchMainMethods(getContainer(), searchScope, 0);
@@ -471,17 +474,17 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 	protected void handleMainClassBrowseButtonPressed() {
 		List resources= JarPackagerUtil.asResources(fJarPackage.getElements());
 		if (resources == null) {
-			setErrorMessage(JarPackagerMessages.getString("JarManifestWizardPage.error.noResourceSelected")); //$NON-NLS-1$
+			setErrorMessage(JarPackagerMessages.JarManifestWizardPage_error_noResourceSelected); 
 			return;
 		}
-		IJavaSearchScope searchScope= JavaSearchScopeFactory.getInstance().createJavaSearchScope((IResource[])resources.toArray(new IResource[resources.size()]));
+		IJavaSearchScope searchScope= JavaSearchScopeFactory.getInstance().createJavaSearchScope((IResource[])resources.toArray(new IResource[resources.size()]), true);
 		SelectionDialog dialog= JavaUI.createMainTypeDialog(getContainer().getShell(), getContainer(), searchScope, 0, false, ""); //$NON-NLS-1$
-		dialog.setTitle(JarPackagerMessages.getString("JarManifestWizardPage.mainTypeSelectionDialog.title")); //$NON-NLS-1$
-		dialog.setMessage(JarPackagerMessages.getString("JarManifestWizardPage.mainTypeSelectionDialog.message")); //$NON-NLS-1$
+		dialog.setTitle(JarPackagerMessages.JarManifestWizardPage_mainTypeSelectionDialog_title); 
+		dialog.setMessage(JarPackagerMessages.JarManifestWizardPage_mainTypeSelectionDialog_message); 
 		if (fJarPackage.getManifestMainClass() != null)
 			dialog.setInitialSelections(new Object[] {fJarPackage.getManifestMainClass()});
 
-		if (dialog.open() == SelectionDialog.OK) {
+		if (dialog.open() == Window.OK) {
 			fJarPackage.setManifestMainClass((IType)dialog.getResult()[0]);
 			fMainClassText.setText(JarPackagerUtil.getMainClassName(fJarPackage));
 		} else if (!fJarPackage.isMainClassValid(getContainer())) {
@@ -493,20 +496,20 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 
 	protected void handleSealPackagesDetailsButtonPressed() {
 		SelectionDialog dialog= createPackageDialog(getPackagesForSelectedResources(fJarPackage));
-		dialog.setTitle(JarPackagerMessages.getString("JarManifestWizardPage.sealedPackagesSelectionDialog.title")); //$NON-NLS-1$
-		dialog.setMessage(JarPackagerMessages.getString("JarManifestWizardPage.sealedPackagesSelectionDialog.message")); //$NON-NLS-1$
+		dialog.setTitle(JarPackagerMessages.JarManifestWizardPage_sealedPackagesSelectionDialog_title); 
+		dialog.setMessage(JarPackagerMessages.JarManifestWizardPage_sealedPackagesSelectionDialog_message); 
 		dialog.setInitialSelections(fJarPackage.getPackagesToSeal());
-		if (dialog.open() == SelectionDialog.OK)
+		if (dialog.open() == Window.OK)
 			fJarPackage.setPackagesToSeal(getPackagesFromDialog(dialog));
 		updateSealingInfo();
 	}
 
 	protected void handleUnSealPackagesDetailsButtonPressed() {
 		SelectionDialog dialog= createPackageDialog(getPackagesForSelectedResources(fJarPackage));
-		dialog.setTitle(JarPackagerMessages.getString("JarManifestWizardPage.unsealedPackagesSelectionDialog.title")); //$NON-NLS-1$
-		dialog.setMessage(JarPackagerMessages.getString("JarManifestWizardPage.unsealedPackagesSelectionDialog.message")); //$NON-NLS-1$
+		dialog.setTitle(JarPackagerMessages.JarManifestWizardPage_unsealedPackagesSelectionDialog_title); 
+		dialog.setMessage(JarPackagerMessages.JarManifestWizardPage_unsealedPackagesSelectionDialog_message); 
 		dialog.setInitialSelections(fJarPackage.getPackagesToUnseal());
-		if (dialog.open() == SelectionDialog.OK)
+		if (dialog.open() == Window.OK)
 			fJarPackage.setPackagesToUnseal(getPackagesFromDialog(dialog));
 		updateSealingInfo();
 	}
@@ -549,22 +552,22 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 			fSealPackagesLabel.setText(""); //$NON-NLS-1$
 			int i= fJarPackage.getPackagesToUnseal().length;
 			if (i == 0)
-				fSealJarLabel.setText(JarPackagerMessages.getString("JarManifestWizardPage.jarSealed")); //$NON-NLS-1$
+				fSealJarLabel.setText(JarPackagerMessages.JarManifestWizardPage_jarSealed); 
 			else if (i == 1)
-				fSealJarLabel.setText(JarPackagerMessages.getString("JarManifestWizardPage.jarSealedExceptOne")); //$NON-NLS-1$
+				fSealJarLabel.setText(JarPackagerMessages.JarManifestWizardPage_jarSealedExceptOne); 
 			else
-				fSealJarLabel.setText(JarPackagerMessages.getFormattedString("JarManifestWizardPage.jarSealedExceptSome", new Integer(i))); //$NON-NLS-1$
+				fSealJarLabel.setText(Messages.format(JarPackagerMessages.JarManifestWizardPage_jarSealedExceptSome, new Integer(i))); 
 				
 		}
 		else {
 			fSealJarLabel.setText(""); //$NON-NLS-1$
 			int i= fJarPackage.getPackagesToSeal().length;
 			if (i == 0)
-				fSealPackagesLabel.setText(JarPackagerMessages.getString("JarManifestWizardPage.nothingSealed")); //$NON-NLS-1$
+				fSealPackagesLabel.setText(JarPackagerMessages.JarManifestWizardPage_nothingSealed); 
 			else if (i == 1)
-				fSealPackagesLabel.setText(JarPackagerMessages.getString("JarManifestWizardPage.onePackageSealed")); //$NON-NLS-1$
+				fSealPackagesLabel.setText(JarPackagerMessages.JarManifestWizardPage_onePackageSealed); 
 			else
-				fSealPackagesLabel.setText(JarPackagerMessages.getFormattedString("JarManifestWizardPage.somePackagesSealed", new Integer(i))); //$NON-NLS-1$
+				fSealPackagesLabel.setText(Messages.format(JarPackagerMessages.JarManifestWizardPage_somePackagesSealed, new Integer(i))); 
 		}
 	}
 	/*
@@ -574,7 +577,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 		boolean isPageComplete= true;
 		setMessage(null);
 		
-		if (!fJarPackage.areClassFilesExported())
+		if (!fJarPackage.areGeneratedFilesExported())
 			return true;
 		
 		if (fJarPackage.isManifestGenerated() && fJarPackage.isManifestSaved()) {
@@ -583,17 +586,17 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 			else {
 				IPath location= fJarPackage.getManifestLocation();
 				if (!location.toString().startsWith("/")) { //$NON-NLS-1$
-					setErrorMessage(JarPackagerMessages.getString("JarManifestWizardPage.error.manifestPathMustBeAbsolute")); //$NON-NLS-1$
+					setErrorMessage(JarPackagerMessages.JarManifestWizardPage_error_manifestPathMustBeAbsolute); 
 					return false;
 				}			
 				IResource resource= findResource(location);
 				if (resource != null && resource.getType() != IResource.FILE) {
-					setErrorMessage(JarPackagerMessages.getString("JarManifestWizardPage.error.manifestMustNotBeExistingContainer")); //$NON-NLS-1$
+					setErrorMessage(JarPackagerMessages.JarManifestWizardPage_error_manifestMustNotBeExistingContainer); 
 					return false;
 				}
 				resource= findResource(location.removeLastSegments(1));
 				if (resource == null || resource.getType() == IResource.FILE) {
-					setErrorMessage(JarPackagerMessages.getString("JarManifestWizardPage.error.manifestContainerDoesNotExist")); //$NON-NLS-1$
+					setErrorMessage(JarPackagerMessages.JarManifestWizardPage_error_manifestContainerDoesNotExist); 
 					return false;
 				}
 			}
@@ -607,28 +610,28 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 					// nothing reported in the wizard
 				}
 				if (manifest != null && manifest.getMainAttributes().getValue(Attributes.Name.MANIFEST_VERSION) == null)
-					setMessage(JarPackagerMessages.getString("JarManifestWizardPage.warning.noManifestVersion"), DialogPage.WARNING); //$NON-NLS-1$
+					setMessage(JarPackagerMessages.JarManifestWizardPage_warning_noManifestVersion, IMessageProvider.WARNING); 
 			} else {
 				if (fJarPackage.getManifestLocation().toString().length() == 0)
-					setErrorMessage(JarPackagerMessages.getString("JarManifestWizardPage.error.noManifestFile")); //$NON-NLS-1$
+					setErrorMessage(JarPackagerMessages.JarManifestWizardPage_error_noManifestFile); 
 				else
-					setErrorMessage(JarPackagerMessages.getString("JarManifestWizardPage.error.invalidManifestFile")); //$NON-NLS-1$
+					setErrorMessage(JarPackagerMessages.JarManifestWizardPage_error_invalidManifestFile); 
 				return false;
 			}
 		}
 		Set selectedPackages= getPackagesForSelectedResources(fJarPackage);
 		if (fJarPackage.isJarSealed()
 				&& !selectedPackages.containsAll(Arrays.asList(fJarPackage.getPackagesToUnseal()))) {
-			setErrorMessage(JarPackagerMessages.getString("JarManifestWizardPage.error.unsealedPackagesNotInSelection")); //$NON-NLS-1$
+			setErrorMessage(JarPackagerMessages.JarManifestWizardPage_error_unsealedPackagesNotInSelection); 
 			return false;
 		}
 		if (!fJarPackage.isJarSealed()
 				&& !selectedPackages.containsAll(Arrays.asList(fJarPackage.getPackagesToSeal()))) {
-			setErrorMessage(JarPackagerMessages.getString("JarManifestWizardPage.error.sealedPackagesNotInSelection")); //$NON-NLS-1$
+			setErrorMessage(JarPackagerMessages.JarManifestWizardPage_error_sealedPackagesNotInSelection); 
 			return false;
 		}
 		if (!fJarPackage.isMainClassValid(getContainer()) || (fJarPackage.getManifestMainClass() == null && fMainClassText.getText().length() > 0)) {
-			setErrorMessage(JarPackagerMessages.getString("JarManifestWizardPage.error.invalidMainClass")); //$NON-NLS-1$
+			setErrorMessage(JarPackagerMessages.JarManifestWizardPage_error_invalidMainClass); 
 			return false;
 		}
 
@@ -815,32 +818,23 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 	 */
 	protected void setEqualButtonSizes() {
 		int width= SWTUtil.getButtonWidthHint(fManifestFileBrowseButton);
-		int height= SWTUtil.getButtonHeigthHint(fManifestFileBrowseButton);
 		int width2= SWTUtil.getButtonWidthHint(fNewManifestFileBrowseButton);
-		int height2= SWTUtil.getButtonHeigthHint(fNewManifestFileBrowseButton);
 		width= Math.max(width, width2);
-		height= Math.max(height, height2);
 
 		width2= SWTUtil.getButtonWidthHint(fSealedPackagesDetailsButton);
-		height2= SWTUtil.getButtonHeigthHint(fSealedPackagesDetailsButton);
 		width= Math.max(width, width2);
-		height= Math.max(height, height2);
 
 		width2= SWTUtil.getButtonWidthHint(fUnSealedPackagesDetailsButton);
-		height2= SWTUtil.getButtonHeigthHint(fUnSealedPackagesDetailsButton);
 		width= Math.max(width, width2);
-		height= Math.max(height, height2);
 
 		width2= SWTUtil.getButtonWidthHint(fMainClassBrowseButton);
-		height2= SWTUtil.getButtonHeigthHint(fMainClassBrowseButton);
 		width= Math.max(width, width2);
-		height= Math.max(height, height2);
 
-		setSize(fManifestFileBrowseButton, width, height);
-		setSize(fNewManifestFileBrowseButton, width, height);
-		setSize(fSealedPackagesDetailsButton, width, height);
-		setSize(fUnSealedPackagesDetailsButton, width, height);
-		setSize(fMainClassBrowseButton, width, height);		
+		setSize(fManifestFileBrowseButton, width, SWT.DEFAULT);
+		setSize(fNewManifestFileBrowseButton, width, SWT.DEFAULT);
+		setSize(fSealedPackagesDetailsButton, width, SWT.DEFAULT);
+		setSize(fUnSealedPackagesDetailsButton, width, SWT.DEFAULT);
+		setSize(fMainClassBrowseButton, width, SWT.DEFAULT);
 	}	
 	
 	/**
@@ -894,7 +888,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 				kind= fragment.getKind();
 				containsJavaElements= fragment.getChildren().length > 0;
 			} catch (JavaModelException ex) {
-				ExceptionHandler.handle(ex, getContainer().getShell(), JarPackagerMessages.getString("JarManifestWizardPage.error.jarPackageWizardError.title"), JarPackagerMessages.getFormattedString("JarManifestWizardPage.error.jarPackageWizardError.message", fragment.getElementName())); //$NON-NLS-2$ //$NON-NLS-1$
+				ExceptionHandler.handle(ex, getContainer().getShell(), JarPackagerMessages.JarManifestWizardPage_error_jarPackageWizardError_title, Messages.format(JarPackagerMessages.JarManifestWizardPage_error_jarPackageWizardError_message, fragment.getElementName())); 
 				continue;
 			}
 			if (kind != IPackageFragmentRoot.K_BINARY && containsJavaElements)
@@ -906,7 +900,8 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 				return !(element instanceof IPackageFragment) && super.hasChildren(element);
 			}
 		};
-		ElementTreeSelectionDialog dialog= new ElementTreeSelectionDialog(getContainer().getShell(), new JavaElementLabelProvider(JavaElementLabelProvider.SHOW_DEFAULT), cp);
+		final DecoratingLabelProvider provider= new DecoratingLabelProvider(new JavaElementLabelProvider(JavaElementLabelProvider.SHOW_DEFAULT), new ProblemsLabelDecorator());
+		ElementTreeSelectionDialog dialog= new ElementTreeSelectionDialog(getContainer().getShell(), provider, cp);
 		dialog.setDoubleClickSelects(false);
 		dialog.setInput(JavaCore.create(JavaPlugin.getWorkspace().getRoot()));
 		dialog.addFilter(new EmptyInnerPackageFilter());		
@@ -917,7 +912,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 				StatusInfo res= new StatusInfo();
 				for (int i= 0; i < selection.length; i++) {
 					if (!(selection[i] instanceof IPackageFragment)) {
-						res.setError(JarPackagerMessages.getString("JarManifestWizardPage.error.mustContainPackages")); //$NON-NLS-1$
+						res.setError(JarPackagerMessages.JarManifestWizardPage_error_mustContainPackages); 
 						return res;
 					}
 				}
@@ -933,7 +928,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 	 * @throws ClassCastException if results are not IPackageFragments
 	 */
 	protected IPackageFragment[] getPackagesFromDialog(SelectionDialog dialog) {
-		if (dialog.getReturnCode() == SelectionDialog.OK && dialog.getResult().length > 0)
+		if (dialog.getReturnCode() == Window.OK && dialog.getResult().length > 0)
 			return (IPackageFragment[])Arrays.asList(dialog.getResult()).toArray(new IPackageFragment[dialog.getResult().length]);
 		else
 			return new IPackageFragment[0];
@@ -945,9 +940,9 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 		int labelFlags= JavaElementLabelProvider.SHOW_BASICS
 						| JavaElementLabelProvider.SHOW_OVERLAY_ICONS
 						| JavaElementLabelProvider.SHOW_SMALL_ICONS;
-		ITreeContentProvider contentProvider= new StandardJavaElementContentProvider();
-		ILabelProvider labelProvider= new JavaElementLabelProvider(labelFlags);
-		ElementTreeSelectionDialog dialog= new ElementTreeSelectionDialog(getShell(), labelProvider, contentProvider); 
+		final DecoratingLabelProvider provider= new DecoratingLabelProvider(new JavaElementLabelProvider(labelFlags), new ProblemsLabelDecorator());
+		ElementTreeSelectionDialog dialog= new ElementTreeSelectionDialog(getShell(), provider, new StandardJavaElementContentProvider()); 
+		dialog.setSorter(new JavaElementSorter());
 		dialog.setAllowMultiple(false);
 		dialog.setValidator(new ISelectionStatusValidator() {
 			public IStatus validate(Object[] selection) {
@@ -979,9 +974,7 @@ class JarManifestWizardPage extends WizardPage implements IJarPackageWizardPage
 		for (int i= 0; i < n; i++) {
 			Object element= fJarPackage.getElements()[i];
 			if (element instanceof ICompilationUnit) {
-				IJavaElement pack= JarPackagerUtil.findParentOfKind((IJavaElement)element, org.eclipse.jdt.core.IJavaElement.PACKAGE_FRAGMENT);
-				if (pack != null)
-					packages.add(pack);
+				packages.add(((ICompilationUnit) element).getParent());
 			}
 		}
 		return packages;
