@@ -418,7 +418,7 @@ public class JavadocTreeWizardPage extends JavadocWizardPage {
 
 	private IPath[] getSourcePath(IJavaProject[] projects) {
 		HashSet res= new HashSet();
-		//loops through all projects and gets a list if of thier sourpaths
+		//loops through all projects and gets a list if of their source paths
 		for (int k= 0; k < projects.length; k++) {
 			IJavaProject iJavaProject= projects[k];
 
@@ -429,6 +429,8 @@ public class JavadocTreeWizardPage extends JavadocWizardPage {
 					if (curr.getKind() == IPackageFragmentRoot.K_SOURCE) {
 						IResource resource= curr.getResource();
 						if (resource != null) {
+							// Using get location is OK here. If the source folder
+							// isn't local we can't create Javadoc for it.
 							IPath p= resource.getLocation();
 							if (p != null) {
 								res.add(p);
@@ -452,6 +454,13 @@ public class JavadocTreeWizardPage extends JavadocWizardPage {
 			try {
 				IPath outputLocation= null;
 				
+				// Not really clear yet what to do here for EFS. See bug
+				// https://bugs.eclipse.org/bugs/show_bug.cgi?id=113233.
+				
+				// However if the output location is not local it is currently
+				// not part of JavaRuntime.computeDefaultRuntimeClassPath either
+				// so it will be simply not added to the result which would be
+				// correct.
 				IResource outputPathFolder= root.findMember(curr.getOutputLocation());
 				if (outputPathFolder != null)
 					outputLocation= outputPathFolder.getLocation();
@@ -472,7 +481,7 @@ public class JavadocTreeWizardPage extends JavadocWizardPage {
 
 	/**
 	 * Gets a list of elements to generated javadoc for from each project. 
-	 * Javadoc can be generated for either a IPackageFragmentRoot or a ICompilationUnit.
+	 * Javadoc can be generated for either a IPackageFragment or a ICompilationUnit.
 	 */
 	private IJavaElement[] getSourceElements(IJavaProject[] projects) {
 		ArrayList res= new ArrayList();
@@ -487,12 +496,14 @@ public class JavadocTreeWizardPage extends JavadocWizardPage {
 				for (int i= 0; i < roots.length; i++) {
 					IPackageFragmentRoot root= roots[i];
 					if (root.getKind() == IPackageFragmentRoot.K_SOURCE) {
+						IPath rootLocation= root.getResource().getLocation();
 						IJavaElement[] packs= root.getChildren();
 						for (int k= 0; k < packs.length; k++) {
 							IJavaElement curr= packs[k];
 							if (curr.getElementType() == IJavaElement.PACKAGE_FRAGMENT) {
 								// default packages are always incomplete
-								if (curr.getElementName().length() == 0 || !allChecked.contains(curr) || fInputGroup.isTreeItemGreyChecked(curr)) {
+								if (curr.getElementName().length() == 0 || !allChecked.contains(curr)
+										|| fInputGroup.isTreeItemGreyChecked(curr) || !isAccessibleLocation(curr.getResource().getLocation(), rootLocation)) {
 									incompletePackages.add(curr.getElementName());
 								}
 							}
@@ -533,6 +544,10 @@ public class JavadocTreeWizardPage extends JavadocWizardPage {
 		return (IJavaElement[]) res.toArray(new IJavaElement[res.size()]);
 	}
 
+	private boolean isAccessibleLocation(IPath packageLocation, IPath rootLocation) {
+		return rootLocation != null && packageLocation != null && rootLocation.isPrefixOf(packageLocation);
+	}
+
 	protected void updateStore(IJavaProject[] checkedProjects) {
 
 		if (fCustomButton.getSelection()) {
