@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -18,6 +18,7 @@ import org.eclipse.core.runtime.ISafeRunnable;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.OperationCanceledException;
 import org.eclipse.core.runtime.Platform;
+import org.eclipse.core.runtime.SafeRunner;
 import org.eclipse.core.runtime.Status;
 
 import org.eclipse.jface.text.Assert;
@@ -32,6 +33,8 @@ import org.eclipse.ui.PlatformUI;
 import org.eclipse.jdt.core.IClassFile;
 import org.eclipse.jdt.core.ICompilationUnit;
 import org.eclipse.jdt.core.IJavaElement;
+import org.eclipse.jdt.core.ISourceReference;
+import org.eclipse.jdt.core.JavaModelException;
 import org.eclipse.jdt.core.dom.AST;
 import org.eclipse.jdt.core.dom.ASTNode;
 import org.eclipse.jdt.core.dom.ASTParser;
@@ -58,7 +61,7 @@ public final class ASTProvider {
 	 *
 	 * @since 3.1
 	 */
-	private static final class WAIT_FLAG {
+	public static final class WAIT_FLAG {
 
 		String fName;
 
@@ -242,7 +245,8 @@ public final class ASTProvider {
 		}
 	}
 
-	public static final int AST_LEVEL= AST.JLS3;
+	public static final int SHARED_AST_LEVEL= AST.JLS3;
+	public static final boolean SHARED_AST_STATEMENT_RECOVERY= true;
 
 	private static final String DEBUG_PREFIX= "ASTProvider > "; //$NON-NLS-1$
 
@@ -256,6 +260,17 @@ public final class ASTProvider {
 	private boolean fIsReconciling;
 	private IWorkbenchPart fActiveEditor;
 
+	
+	/**
+	 * Returns the Java plug-in's AST provider.
+	 * 
+	 * @return the AST provider
+	 * @since 3.2
+	 */
+	public static ASTProvider getASTProvider() {
+		return JavaPlugin.getDefault().getASTProvider();
+	}
+	
 	/**
 	 * Creates a new AST provider.
 	 */
@@ -286,19 +301,21 @@ public final class ASTProvider {
 		synchronized (this) {
 			fActiveEditor= editor;
 			fActiveJavaElement= javaElement;
+			cache(null, javaElement);
 		}
 
 		if (DEBUG)
 			System.out.println(getThreadName() + " - " + DEBUG_PREFIX + "active editor is: " + toString(javaElement)); //$NON-NLS-1$ //$NON-NLS-2$
 
 		synchronized (fReconcileLock) {
-			if (fIsReconciling && !fReconcilingJavaElement.equals(javaElement)) {
+			if (fIsReconciling && (fReconcilingJavaElement == null || !fReconcilingJavaElement.equals(javaElement))) {
+				fIsReconciling= false;
+				fReconcilingJavaElement= null;
+			} else if (javaElement == null) {
 				fIsReconciling= false;
 				fReconcilingJavaElement= null;
 			}
 		}
-
-		cache(null, javaElement);
 	}
 
 	/**
@@ -430,11 +447,14 @@ public final class ASTProvider {
 	 *
 	 * @param je				the Java element
 	 * @param waitFlag			{@link #WAIT_YES}, {@link #WAIT_NO} or {@link #WAIT_ACTIVE_ONLY}
-	 * @param progressMonitor	the progress monitor
+	 * @param progressMonitor	the progress monitor or <code>null</code>
 	 * @return					the AST or <code>null</code> if the AST is not available
 	 */
 	public CompilationUnit getAST(IJavaElement je, WAIT_FLAG waitFlag, IProgressMonitor progressMonitor) {
-		Assert.isTrue(je != null && (je.getElementType() == IJavaElement.CLASS_FILE || je.getElementType() == IJavaElement.COMPILATION_UNIT));
+		if (je == null)
+			return null;
+		
+		Assert.isTrue(je.getElementType() == IJavaElement.CLASS_FILE || je.getElementType() == IJavaElement.COMPILATION_UNIT);
 
 		if (progressMonitor != null && progressMonitor.isCanceled())
 			return null;
@@ -472,7 +492,7 @@ public final class ASTProvider {
 
 				// Check whether active element is still valid
 				synchronized (this) {
-					if (activeElement == fActiveJavaElement) {
+					if (activeElement == fActiveJavaElement && fAST != null) {
 						if (DEBUG)
 							System.out.println(getThreadName() + " - " + DEBUG_PREFIX + "...got AST for: " + je.getElementName()); //$NON-NLS-1$ //$NON-NLS-2$
 
@@ -481,6 +501,7 @@ public final class ASTProvider {
 				}
 				return getAST(je, waitFlag, progressMonitor);
 			} catch (InterruptedException e) {
+				return null; // thread has been interrupted don't compute AST
 			}
 		} else if (waitFlag == WAIT_NO || (waitFlag == WAIT_ACTIVE_ONLY && !(isActiveElement && fAST == null)))
 			return null;
@@ -496,8 +517,14 @@ public final class ASTProvider {
 			else if (DEBUG && ast != null)
 				System.err.println(getThreadName() + " - " + DEBUG_PREFIX + "created AST for: " + je.getElementName()); //$NON-NLS-1$ //$NON-NLS-2$
 		} finally {
-			if (isActiveElement)
-				reconciled(ast, je);
+			if (isActiveElement) {
+				if (fAST != null) {
+					if (DEBUG)
+						System.out.println(getThreadName() + " - " + DEBUG_PREFIX + "Ignore created AST for " + je.getElementName() + "- AST from reconciler is newer"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+					reconciled(fAST, je, null);
+				} else
+					reconciled(ast, je, null);
+			}
 		}
 
 		return ast;
@@ -515,7 +542,7 @@ public final class ASTProvider {
 	 * @param wait				<code>true</code> if the client wants to wait for the result,
 	 * 								<code>null</code> will be returned if the AST is not ready and
 	 * 								the client does not want to wait
-	 * @param progressMonitor	the progress monitor
+	 * @param progressMonitor	the progress monitor or <code>null</code>
 	 * @return					the AST or <code>null</code> if the AST is not available
 	 * @deprecated As of 3.1, use {@link #getAST(IJavaElement, WAIT_FLAG, IProgressMonitor)}
 	 */
@@ -535,7 +562,7 @@ public final class ASTProvider {
 	 */
 	private boolean isReconciling(IJavaElement javaElement) {
 		synchronized (fReconcileLock) {
-			return javaElement.equals(fReconcilingJavaElement) && fIsReconciling;
+			return javaElement != null && javaElement.equals(fReconcilingJavaElement) && fIsReconciling;
 		}
 	}
 
@@ -547,9 +574,19 @@ public final class ASTProvider {
 	 * @return AST
 	 */
 	private CompilationUnit createAST(IJavaElement je, final IProgressMonitor progressMonitor) {
-		final ASTParser parser = ASTParser.newParser(AST_LEVEL);
+		if (!hasSource(je))
+			return null;
+		
+		if (progressMonitor != null && progressMonitor.isCanceled())
+			return null;
+		
+		final ASTParser parser = ASTParser.newParser(SHARED_AST_LEVEL);
 		parser.setResolveBindings(true);
+		parser.setStatementsRecovery(SHARED_AST_STATEMENT_RECOVERY);
 
+		if (progressMonitor != null && progressMonitor.isCanceled())
+			return null;
+		
 		if (je.getElementType() == IJavaElement.COMPILATION_UNIT)
 			parser.setSource((ICompilationUnit)je);
 		else if (je.getElementType() == IJavaElement.CLASS_FILE)
@@ -560,14 +597,14 @@ public final class ASTProvider {
 
 		final CompilationUnit root[]= new CompilationUnit[1]; 
 		
-		Platform.run(new ISafeRunnable() {
+		SafeRunner.run(new ISafeRunnable() {
 			public void run() {
 				try {
+					if (progressMonitor != null && progressMonitor.isCanceled())
+						root[0]= null;
 					root[0]= (CompilationUnit)parser.createAST(progressMonitor);
 				} catch (OperationCanceledException ex) {
 					root[0]= null;
-				} catch (IllegalStateException ex) {
-					root[0]= null;
 				}
 			}
 			public void handleException(Throwable ex) {
@@ -583,6 +620,26 @@ public final class ASTProvider {
 		return root[0];
 	}
 	
+	/**
+	 * Checks whether the given Java element has accessible source.
+	 * 
+	 * @param je the Java element to test
+	 * @return <code>true</code> if the element has source
+	 * @since 3.2
+	 */
+	private boolean hasSource(IJavaElement je) {
+		if (je == null || !je.exists())
+			return false;
+		
+		try {
+			return je instanceof ISourceReference && ((ISourceReference)je).getSource() != null;
+		} catch (JavaModelException ex) {
+			IStatus status= new Status(IStatus.ERROR, JavaUI.ID_PLUGIN, IStatus.OK, "Error in JDT Core during AST creation", ex);  //$NON-NLS-1$
+			JavaPlugin.getDefault().getLog().log(status);
+		}
+		return false;
+	}
+	
 	/**
 	 * Disposes this AST provider.
 	 */
@@ -595,21 +652,21 @@ public final class ASTProvider {
 		disposeAST();
 
 		synchronized (fWaitLock) {
-			fWaitLock.notify();
+			fWaitLock.notifyAll();
 		}
 	}
 
 	/*
 	 * @see org.eclipse.jdt.internal.ui.text.java.IJavaReconcilingListener#reconciled(org.eclipse.jdt.core.dom.CompilationUnit)
 	 */
-	void reconciled(CompilationUnit ast, IJavaElement javaElement) {
+	void reconciled(CompilationUnit ast, IJavaElement javaElement, IProgressMonitor progressMonitor) {
 
 		if (DEBUG)
 			System.out.println(getThreadName() + " - " + DEBUG_PREFIX + "reconciled: " + toString(javaElement) + ", AST: " + toString(ast)); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
 
 		synchronized (fReconcileLock) {
 
-			fIsReconciling= false;
+			fIsReconciling= progressMonitor != null && progressMonitor.isCanceled();
 			if (javaElement == null || !javaElement.equals(fReconcilingJavaElement)) {
 
 				if (DEBUG)
@@ -634,5 +691,6 @@ public final class ASTProvider {
 		else
 			return Thread.currentThread().toString();
 	}
+	
 }
 
