@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -23,6 +23,8 @@ import org.eclipse.ui.IEditorInput;
 
 import org.eclipse.jdt.core.ICompilationUnit;
 
+import org.eclipse.jdt.internal.corext.util.JavaModelUtil;
+
 import org.eclipse.jdt.ui.IWorkingCopyManager;
 import org.eclipse.jdt.ui.IWorkingCopyManagerExtension;
 
@@ -84,8 +86,30 @@ public class WorkingCopyManager implements IWorkingCopyManager, IWorkingCopyMana
 	 * @see org.eclipse.jdt.ui.IWorkingCopyManager#getWorkingCopy(org.eclipse.ui.IEditorInput)
 	 */
 	public ICompilationUnit getWorkingCopy(IEditorInput input) {
+		return getWorkingCopy(input, true);
+	}
+
+	/**
+	 * Returns the working copy remembered for the compilation unit encoded in the
+	 * given editor input.
+	 * <p>
+	 * Note: This method must not be part of the public {@link IWorkingCopyManager} API.
+	 * </p>
+	 *
+	 * @param input the editor input
+	 * @param primaryOnly if <code>true</code> only primary working copies will be returned
+	 * @return the working copy of the compilation unit, or <code>null</code> if the
+	 *   input does not encode an editor input, or if there is no remembered working
+	 *   copy for this compilation unit
+	 * @since 3.2
+	 */
+	public ICompilationUnit getWorkingCopy(IEditorInput input, boolean primaryOnly) {
 		ICompilationUnit unit= fMap == null ? null : (ICompilationUnit) fMap.get(input);
-		return unit != null ? unit : fDocumentProvider.getWorkingCopy(input);
+		if (unit == null)
+			unit= fDocumentProvider.getWorkingCopy(input);
+		if (unit != null && (!primaryOnly || JavaModelUtil.isPrimary(unit)))
+			return unit;
+		return null;
 	}
 
 	/*
