@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -37,6 +37,7 @@ import org.eclipse.swt.widgets.Combo;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Label;
+import org.eclipse.swt.widgets.Link;
 import org.eclipse.swt.widgets.Shell;
 import org.eclipse.swt.widgets.Text;
 import org.eclipse.swt.widgets.Widget;
@@ -46,9 +47,9 @@ import org.eclipse.jface.dialogs.IDialogSettings;
 import org.eclipse.jface.dialogs.MessageDialog;
 import org.eclipse.jface.resource.JFaceResources;
 
-import org.eclipse.ui.internal.preferences.WorkingCopyManager;
 import org.eclipse.ui.preferences.IWorkbenchPreferenceContainer;
 import org.eclipse.ui.preferences.IWorkingCopyManager;
+import org.eclipse.ui.preferences.WorkingCopyManager;
 
 import org.eclipse.ui.forms.events.ExpansionAdapter;
 import org.eclipse.ui.forms.events.ExpansionEvent;
@@ -164,6 +165,8 @@ public abstract class OptionsConfigurationBlock {
 		}
 	}
 	
+	private static final String REBUILD_COUNT_KEY= "preferences_build_requested"; //$NON-NLS-1$
+	
 	private static final String SETTINGS_EXPANDED= "expanded"; //$NON-NLS-1$
 
 	protected final ArrayList fCheckBoxes;
@@ -188,6 +191,8 @@ public abstract class OptionsConfigurationBlock {
 
 	private Map fDisabledProjectSettings; // null when project specific settings are turned off
 	
+	private int fRebuildCount; /// used to prevent multiple dialogs that ask for a rebuild
+	
 	public OptionsConfigurationBlock(IStatusChangeListener context, IProject project, Key[] allKeys, IWorkbenchPreferenceContainer container) {
 		fContext= context;
 		fProject= project;
@@ -230,7 +235,9 @@ public abstract class OptionsConfigurationBlock {
 		fTextBoxes= new ArrayList(2);
 		fLabels= new HashMap();
 		fExpandedComposites= new ArrayList();
-	}
+		
+		fRebuildCount= getRebuildCount();
+	}	
 	
 	protected final IWorkbenchPreferenceContainer getPreferenceContainer() {
 		return fContainer;
@@ -257,6 +264,16 @@ public abstract class OptionsConfigurationBlock {
 		}
 	}
 	
+	private int getRebuildCount() {
+		return fManager.getWorkingCopy(new DefaultScope().getNode(JavaUI.ID_PLUGIN)).getInt(REBUILD_COUNT_KEY, 0);
+	}
+	
+	private void incrementRebuildCount() {
+		fRebuildCount++;
+		fManager.getWorkingCopy(new DefaultScope().getNode(JavaUI.ID_PLUGIN)).putInt(REBUILD_COUNT_KEY, fRebuildCount);
+	}
+	
+	
 	protected void settingsUpdated() {
 	}
 	
@@ -335,6 +352,48 @@ public abstract class OptionsConfigurationBlock {
 		return checkBox;
 	}
 	
+	protected Button addCheckBoxWithLink(Composite parent, String label, Key key, String[] values, int indent, int widthHint, SelectionListener listener) {
+		ControlData data= new ControlData(key, values);
+		
+		GridData gd= new GridData(GridData.FILL, GridData.FILL, true, false);
+		gd.horizontalSpan= 3;
+		gd.horizontalIndent= indent;
+		
+		Composite composite= new Composite(parent, SWT.NONE);
+		GridLayout layout= new GridLayout();
+		layout.marginHeight= 0;
+		layout.marginWidth= 0;
+		layout.numColumns= 2;
+		composite.setLayout(layout);
+		composite.setLayoutData(gd);
+		
+		Button checkBox= new Button(composite, SWT.CHECK);
+		checkBox.setFont(JFaceResources.getDialogFont());
+		checkBox.setData(data);
+		checkBox.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, false, false));
+		checkBox.addSelectionListener(getSelectionListener());
+		
+		gd= new GridData(GridData.FILL, GridData.CENTER, true, false);
+		gd.widthHint= widthHint;
+		
+		Link link= new Link(composite, SWT.NONE);
+		link.setText(label);
+		link.setLayoutData(gd);
+		if (listener != null) {
+			link.addSelectionListener(listener);
+		}
+		
+		makeScrollableCompositeAware(link);
+		makeScrollableCompositeAware(checkBox);
+		
+		String currValue= getValue(key);
+		checkBox.setSelection(data.getSelection(currValue) == 0);
+		
+		fCheckBoxes.add(checkBox);
+		
+		return checkBox;
+	}
+	
 	protected Combo addComboBox(Composite parent, String label, Key key, String[] values, String[] valueLabels, int indent) {
 		GridData gd= new GridData(GridData.FILL, GridData.CENTER, true, false, 2, 1);
 		gd.horizontalIndent= indent;
@@ -572,7 +631,7 @@ public abstract class OptionsConfigurationBlock {
 	}
 
 	/**
-	 * Retuens the value as actually stored in the preference store.
+	 * Returns the value as actually stored in the preference store.
 	 * @param key
 	 * @return the value as actually stored in the preference store.
 	 */
@@ -588,7 +647,7 @@ public abstract class OptionsConfigurationBlock {
 	
 	
 	protected String[] getTokens(String text, String separator) {
-		StringTokenizer tok= new StringTokenizer(text, separator); //$NON-NLS-1$
+		StringTokenizer tok= new StringTokenizer(text, separator); 
 		int nTokens= tok.countTokens();
 		String[] res= new String[nTokens];
 		for (int i= 0; i < res.length; i++) {
@@ -627,6 +686,7 @@ public abstract class OptionsConfigurationBlock {
 				}
 				fDisabledProjectSettings= null;
 				updateControls();
+				validateSettings(null, null, null);
 			} else {
 				fDisabledProjectSettings= new IdentityHashMap();
 				for (int i= 0; i < fAllKeys.length; i++) {
@@ -639,6 +699,10 @@ public abstract class OptionsConfigurationBlock {
 		}
 	}
 	
+	public boolean areSettingsEnabled() {
+		return fDisabledProjectSettings == null || fProject == null;
+	}
+	
 	
 	public boolean performOk() {
 		return processChanges(fContainer);
@@ -649,15 +713,21 @@ public abstract class OptionsConfigurationBlock {
 	}
 	
 	protected boolean processChanges(IWorkbenchPreferenceContainer container) {
-
 		IScopeContext currContext= fLookupOrder[0];
-	
-		List /* <PropertyChange>*/ changedOptions= new ArrayList();
+		
+		List /* <Key>*/ changedOptions= new ArrayList();
 		boolean needsBuild= getChanges(currContext, changedOptions);
 		if (changedOptions.isEmpty()) {
 			return true;
 		}
-		
+		if (needsBuild) {
+			int count= getRebuildCount();
+			if (count > fRebuildCount) {
+				needsBuild= false; // build already requested
+				fRebuildCount= count;
+			}
+		}
+
 		boolean doBuild= false;
 		if (needsBuild) {
 			String[] strings= getFullBuildDialogStrings(fProject == null);
@@ -672,8 +742,9 @@ public abstract class OptionsConfigurationBlock {
 			}
 		}
 		if (container != null) {
-			// no need to apply the changes to the original store: will be done by the page  container
+			// no need to apply the changes to the original store: will be done by the page container
 			if (doBuild) { // post build
+				incrementRebuildCount();
 				container.registerUpdateJob(CoreUtility.getBuildJob(fProject));
 			}
 		} else {
