@@ -1,18 +1,17 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.jdt.internal.ui.search;
 
-import java.text.Collator;
+import com.ibm.icu.text.Collator;
 import java.util.Comparator;
-
 import org.eclipse.ui.IWorkingSet;
 
 class WorkingSetsComparator implements Comparator {
@@ -29,13 +28,13 @@ class WorkingSetsComparator implements Comparator {
 		if (o1 instanceof IWorkingSet[]) {
 			IWorkingSet[] workingSets= (IWorkingSet[])o1;
 			if (workingSets.length > 0)
-				name1= workingSets[0].getName();
+				name1= workingSets[0].getLabel();
 		}
 
 		if (o2 instanceof IWorkingSet[]) {
 			IWorkingSet[] workingSets= (IWorkingSet[])o1;
 			if (workingSets.length > 0)
-				name2= workingSets[0].getName();
+				name2= workingSets[0].getLabel();
 		}
 
 		return fCollator.compare(name1, name2);
