@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -29,75 +29,56 @@ import org.eclipse.jface.text.contentassist.IContextInformation;
 import org.eclipse.jface.text.link.LinkedPosition;
 import org.eclipse.jface.text.link.LinkedPositionGroup;
 
-import org.eclipse.jdt.core.CorrectionEngine;
 import org.eclipse.jdt.core.ICompilationUnit;
 import org.eclipse.jdt.core.JavaModelException;
 import org.eclipse.jdt.core.compiler.IProblem;
 import org.eclipse.jdt.core.dom.AST;
 import org.eclipse.jdt.core.dom.ASTNode;
-import org.eclipse.jdt.core.dom.Annotation;
-import org.eclipse.jdt.core.dom.AnnotationTypeDeclaration;
-import org.eclipse.jdt.core.dom.AnnotationTypeMemberDeclaration;
-import org.eclipse.jdt.core.dom.ArrayInitializer;
 import org.eclipse.jdt.core.dom.Block;
-import org.eclipse.jdt.core.dom.BodyDeclaration;
-import org.eclipse.jdt.core.dom.ChildListPropertyDescriptor;
 import org.eclipse.jdt.core.dom.ClassInstanceCreation;
 import org.eclipse.jdt.core.dom.CompilationUnit;
-import org.eclipse.jdt.core.dom.EnumConstantDeclaration;
-import org.eclipse.jdt.core.dom.EnumDeclaration;
 import org.eclipse.jdt.core.dom.Expression;
 import org.eclipse.jdt.core.dom.FieldAccess;
-import org.eclipse.jdt.core.dom.FieldDeclaration;
 import org.eclipse.jdt.core.dom.IBinding;
 import org.eclipse.jdt.core.dom.IMethodBinding;
 import org.eclipse.jdt.core.dom.ITypeBinding;
 import org.eclipse.jdt.core.dom.IVariableBinding;
-import org.eclipse.jdt.core.dom.Initializer;
-import org.eclipse.jdt.core.dom.MemberValuePair;
 import org.eclipse.jdt.core.dom.MethodDeclaration;
 import org.eclipse.jdt.core.dom.MethodInvocation;
 import org.eclipse.jdt.core.dom.Modifier;
-import org.eclipse.jdt.core.dom.NormalAnnotation;
 import org.eclipse.jdt.core.dom.QualifiedName;
 import org.eclipse.jdt.core.dom.ReturnStatement;
 import org.eclipse.jdt.core.dom.SimpleName;
 import org.eclipse.jdt.core.dom.SimpleType;
-import org.eclipse.jdt.core.dom.SingleMemberAnnotation;
-import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
-import org.eclipse.jdt.core.dom.StringLiteral;
-import org.eclipse.jdt.core.dom.StructuralPropertyDescriptor;
 import org.eclipse.jdt.core.dom.SuperConstructorInvocation;
 import org.eclipse.jdt.core.dom.SuperFieldAccess;
 import org.eclipse.jdt.core.dom.SuperMethodInvocation;
-import org.eclipse.jdt.core.dom.ThisExpression;
 import org.eclipse.jdt.core.dom.Type;
 import org.eclipse.jdt.core.dom.TypeDeclaration;
-import org.eclipse.jdt.core.dom.VariableDeclarationFragment;
-import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
 import org.eclipse.jdt.core.dom.Modifier.ModifierKeyword;
 import org.eclipse.jdt.core.dom.rewrite.ASTRewrite;
-import org.eclipse.jdt.core.dom.rewrite.ListRewrite;
+import org.eclipse.jdt.core.formatter.IndentManipulation;
 
 import org.eclipse.jdt.internal.corext.dom.ASTNodeFactory;
 import org.eclipse.jdt.internal.corext.dom.ASTNodes;
 import org.eclipse.jdt.internal.corext.dom.Bindings;
+import org.eclipse.jdt.internal.corext.fix.IFix;
+import org.eclipse.jdt.internal.corext.fix.Java50Fix;
 import org.eclipse.jdt.internal.corext.util.JdtFlags;
 import org.eclipse.jdt.internal.corext.util.Messages;
-import org.eclipse.jdt.internal.corext.util.Strings;
 
 import org.eclipse.jdt.ui.text.java.IInvocationContext;
 import org.eclipse.jdt.ui.text.java.IProblemLocation;
 
 import org.eclipse.jdt.internal.ui.JavaPlugin;
 import org.eclipse.jdt.internal.ui.JavaPluginImages;
+import org.eclipse.jdt.internal.ui.fix.Java50CleanUp;
 import org.eclipse.jdt.internal.ui.text.correction.LinkedCorrectionProposal.ILinkedModeProposal;
 
 /**
   */
 public class ModifierCorrectionSubProcessor {
 
-	private static final String ADD_SUPPRESSWARNINGS_ID= "org.eclipse.jdt.ui.correction.addSuppressWarnings"; //$NON-NLS-1$
 
 	public static final int TO_STATIC= 1;
 	public static final int TO_VISIBLE= 2;
@@ -159,7 +140,7 @@ public class ModifierCorrectionSubProcessor {
 			typeBinding= varDecl.getDeclaringClass();
 			name= binding.getName();
 			isLocalVar= !varDecl.isField();
-			bindingDecl= Bindings.getVariableDeclaration(varDecl);
+			bindingDecl= varDecl.getVariableDeclaration();
 		} else if (binding instanceof ITypeBinding) {
 			typeBinding= (ITypeBinding) binding;
 			bindingDecl= typeBinding.getTypeDeclaration();
@@ -194,8 +175,7 @@ public class ModifierCorrectionSubProcessor {
 					excludedModifiers= Modifier.FINAL;
 					break;
 				default:
-					Assert.isTrue(false, "not supported"); //$NON-NLS-1$
-					return;
+					throw new IllegalArgumentException("not supported"); //$NON-NLS-1$
 			}
 			ICompilationUnit targetCU= isLocalVar ? cu : ASTResolving.findCompilationUnitForBinding(cu, context.getASTRoot(), typeBinding.getTypeDeclaration());
 			if (targetCU != null) {
@@ -203,10 +183,8 @@ public class ModifierCorrectionSubProcessor {
 				proposals.add(new ModifierChangeCompletionProposal(label, targetCU, bindingDecl, selectedNode, includedModifiers, excludedModifiers, relevance, image));
 			}
 		}
-		if (kind == TO_VISIBLE && bindingDecl.getKind() == IBinding.VARIABLE /* &&Modifier.isPrivate(bindingDecl.getModifiers())*/) {
-			if (selectedNode instanceof SimpleName || selectedNode instanceof FieldAccess && ((FieldAccess) selectedNode).getExpression() instanceof ThisExpression) {
-				UnresolvedElementsSubProcessor.getVariableProposals(context, problem, proposals);
-			}
+		if (kind == TO_VISIBLE && bindingDecl.getKind() == IBinding.VARIABLE) {
+			UnresolvedElementsSubProcessor.getVariableProposals(context, problem, (IVariableBinding) bindingDecl, proposals);
 		}
 	}
 
@@ -223,7 +201,7 @@ public class ModifierCorrectionSubProcessor {
 
 
 		if (kind == TO_VISIBLE && problem.getProblemId() != IProblem.OverridingNonVisibleMethod) {
-			IMethodBinding defining= Bindings.findMethodDefininition(method, false);
+			IMethodBinding defining= Bindings.findOverriddenMethod(method, false);
 			if (defining != null) {
 				int excludedModifiers= Modifier.PRIVATE | Modifier.PROTECTED | Modifier.PUBLIC;
 				int includedModifiers= JdtFlags.getVisibilityCode(defining);
@@ -282,7 +260,7 @@ public class ModifierCorrectionSubProcessor {
 
 		IBinding binding= ((SimpleName) selectedNode).resolveBinding();
 		if (binding instanceof IVariableBinding) {
-			binding= Bindings.getVariableDeclaration((IVariableBinding) binding);
+			binding= ((IVariableBinding) binding).getVariableDeclaration();
 			Image image= JavaPluginImages.get(JavaPluginImages.IMG_CORRECTION_CHANGE);
 			String label= Messages.format(CorrectionMessages.ModifierCorrectionSubProcessor_changemodifiertofinal_description, binding.getName());
 			proposals.add(new ModifierChangeCompletionProposal(label, cu, binding, selectedNode, Modifier.FINAL, 0, 5, image));
@@ -352,6 +330,9 @@ public class ModifierCorrectionSubProcessor {
 					break;
 				case IProblem.IllegalModifierForMethod:
 					excludedModifiers= ~(Modifier.PUBLIC | Modifier.PROTECTED | Modifier.PRIVATE | Modifier.STATIC | Modifier.ABSTRACT | Modifier.FINAL | Modifier.NATIVE | Modifier.STRICTFP);
+					if (((IMethodBinding) binding).isConstructor()) {
+						excludedModifiers |= Modifier.STATIC;
+					}
 					break;
 				case IProblem.IllegalModifierForVariable:
 					excludedModifiers= ~Modifier.FINAL;
@@ -600,60 +581,30 @@ public class ModifierCorrectionSubProcessor {
 
 		IBinding binding= ((SimpleName) selectedNode).resolveBinding();
 		if (binding instanceof IVariableBinding) {
-			binding= Bindings.getVariableDeclaration((IVariableBinding) binding);
+			binding= ((IVariableBinding) binding).getVariableDeclaration();
 			Image image= JavaPluginImages.get(JavaPluginImages.IMG_CORRECTION_CHANGE);
 			String label= Messages.format(CorrectionMessages.ModifierCorrectionSubProcessor_changemodifiertofinal_description, binding.getName());
 			proposals.add(new ModifierChangeCompletionProposal(label, cu, binding, selectedNode, Modifier.FINAL, 0, 5, image));
 		}
 	}
 	
-	public static void addOverrideAnnotationProposal(IInvocationContext context, IProblemLocation problem, Collection proposals) {
-		ASTRewriteCorrectionProposal proposal= getMissingAnnotationsProposal(context, problem, "Override"); //$NON-NLS-1$
-		if (proposal != null) {
-			proposal.setDisplayName(CorrectionMessages.ModifierCorrectionSubProcessor_addoverrideannotation);
-			proposal.setImage(JavaPluginImages.get(JavaPluginImages.IMG_CORRECTION_CHANGE));
+	public static void addOverrideAnnotationProposal(IInvocationContext context, IProblemLocation problem, Collection proposals) throws CoreException {
+		IFix fix= Java50Fix.createAddOverrideAnnotationFix(context.getASTRoot(), problem);
+		if (fix != null) {
+			Image image= JavaPluginImages.get(JavaPluginImages.IMG_CORRECTION_CHANGE);
+			FixCorrectionProposal proposal= new FixCorrectionProposal(fix, new Java50CleanUp(Java50CleanUp.ADD_OVERRIDE_ANNOATION), 5, image, context);
 			proposals.add(proposal);
 		}
 	}
 	
-	public static void addDeprecatedAnnotationProposal(IInvocationContext context, IProblemLocation problem, Collection proposals) {
-		ASTRewriteCorrectionProposal proposal= getMissingAnnotationsProposal(context, problem, "Deprecated"); //$NON-NLS-1$
-		if (proposal != null) {
-			proposal.setDisplayName(CorrectionMessages.ModifierCorrectionSubProcessor_adddeprecatedannotation);
-			proposal.setImage(JavaPluginImages.get(JavaPluginImages.IMG_CORRECTION_CHANGE));
+	public static void addDeprecatedAnnotationProposal(IInvocationContext context, IProblemLocation problem, Collection proposals) throws CoreException {
+		IFix fix= Java50Fix.createAddDeprectatedAnnotation(context.getASTRoot(), problem);
+		if (fix != null) {
+			Image image= JavaPluginImages.get(JavaPluginImages.IMG_CORRECTION_CHANGE);
+			FixCorrectionProposal proposal= new FixCorrectionProposal(fix, new Java50CleanUp(Java50CleanUp.ADD_DEPRECATED_ANNOTATION), 5, image, context);
 			proposals.add(proposal);
 		}
 	}
-		
-	private static ASTRewriteCorrectionProposal getMissingAnnotationsProposal(IInvocationContext context, IProblemLocation problem, String annotationName) {
-		ICompilationUnit cu= context.getCompilationUnit();
-
-		ASTNode selectedNode= problem.getCoveringNode(context.getASTRoot());
-		ASTNode declaringNode= null;		
-		if (selectedNode instanceof MethodDeclaration) {
-			declaringNode= selectedNode;
-		} else if (selectedNode instanceof SimpleName) {
-			StructuralPropertyDescriptor locationInParent= selectedNode.getLocationInParent();
-			if (locationInParent == MethodDeclaration.NAME_PROPERTY || locationInParent == TypeDeclaration.NAME_PROPERTY) {
-				declaringNode= selectedNode.getParent();
-			} else if (locationInParent == VariableDeclarationFragment.NAME_PROPERTY) {
-				declaringNode= selectedNode.getParent().getParent();
-			}
-		}
-		if (declaringNode instanceof BodyDeclaration) {
-			BodyDeclaration declaration= (BodyDeclaration) declaringNode;
-			AST ast= declaration.getAST();
-			ASTRewrite rewrite= ASTRewrite.create(ast);
-			ListRewrite listRewrite= rewrite.getListRewrite(declaration, declaration.getModifiersProperty());
-
-			Annotation newAnnotation= ast.newMarkerAnnotation();
-			newAnnotation.setTypeName(ast.newSimpleName(annotationName));
-			listRewrite.insertFirst(newAnnotation, null);
-			
-			return new ASTRewriteCorrectionProposal("", cu, rewrite, 5, null); //$NON-NLS-1$
-		}
-		return null;
-	}
 
 	private static final String KEY_MODIFIER= "modifier"; //$NON-NLS-1$
 	
@@ -714,14 +665,14 @@ public class ModifierCorrectionSubProcessor {
 
 					int end= currentPosition.offset + currentPosition.length; // current end position
 					int k= end;
-					while (k < documentLen && Strings.isIndentChar(document.getChar(k))) {
+					while (k < documentLen && IndentManipulation.isIndentChar(document.getChar(k))) {
 						k++;
 					}
-					// first remove space then replace range (remove space can destoy empty positon)
+					// first remove space then replace range (remove space can destroy empty position)
 					document.replace(end, k - end, new String()); // remove extra spaces
 					document.replace(currentPosition.offset, currentPosition.length, new String());
 				} else {
-					// first then replace range the insert space (insert space can destoy empty positon)
+					// first then replace range the insert space (insert space can destroy empty position)
 					document.replace(currentPosition.offset, currentPosition.length, ModifierKeyword.fromFlagValue(fModifier).toString());
 					int end= currentPosition.offset + currentPosition.length; // current end position
 					if (end < documentLen && !Character.isWhitespace(document.getChar(end))) {
@@ -743,7 +694,7 @@ public class ModifierCorrectionSubProcessor {
 		public boolean validate(IDocument document, int offset, DocumentEvent event) { return false; }
 	}
 	
-	public static void installLinkedVisibilityProposals(LinkedCorrectionProposal proposal, ASTRewrite rewrite, List modifiers) {
+	public static void installLinkedVisibilityProposals(LinkedCorrectionProposal proposal, ASTRewrite rewrite, List modifiers, boolean inInterface) {
 		ASTNode modifier= findVisibilityModifier(modifiers);
 		if (modifier != null) {
 			int selected= ((Modifier) modifier).getKeyword().toFlagValue();
@@ -751,7 +702,7 @@ public class ModifierCorrectionSubProcessor {
 			proposal.addLinkedPositionProposal(KEY_MODIFIER, new ModifierLinkedModeProposal(selected));
 			
 			// add all others
-			int[] flagValues= { Modifier.PUBLIC, 0, Modifier.PROTECTED, Modifier.PRIVATE };
+			int[] flagValues= inInterface ? new int[] { Modifier.PUBLIC, 0 } : new int[] { Modifier.PUBLIC, 0, Modifier.PROTECTED, Modifier.PRIVATE };
 			for (int i= 0; i < flagValues.length; i++) {
 				if (flagValues[i] != selected) {
 					proposal.addLinkedPositionProposal(KEY_MODIFIER,  new ModifierLinkedModeProposal(flagValues[i]));
@@ -773,182 +724,4 @@ public class ModifierCorrectionSubProcessor {
 		}
 		return null;
 	}
-
-	public static final boolean hasSuppressWarningsProposal(int problemId) {
-		return CorrectionEngine.getWarningToken(problemId) != null; // Surpress warning annotations
-	}
-	
-	
-	public static void addSuppressWarningsProposals(IInvocationContext context, IProblemLocation problem, Collection proposals) {
-		if (problem.isError()) {
-			return;
-		}
-		String warningToken= CorrectionEngine.getWarningToken(problem.getProblemId());
-		if (warningToken == null) {
-			return;
-		}
-		/*for (Iterator iter= proposals.iterator(); iter.hasNext();) {
-			Object element= iter.next();
-			if (element instanceof ICommandAccess && ADD_SUPPRESSWARNINGS_ID.equals(((ICommandAccess) element).getCommandId())) {
-				return; // only one at a time
-			}
-		}*/
-		
-		ASTNode node= problem.getCoveringNode(context.getASTRoot());
-		if (node == null) {
-			return;
-		}
-		if (node.getLocationInParent() == VariableDeclarationFragment.NAME_PROPERTY) {
-			ASTNode parent= node.getParent();
-			if (parent.getLocationInParent() == VariableDeclarationStatement.FRAGMENTS_PROPERTY) {
-				addSuppressWarningsProposal(context.getCompilationUnit(), parent.getParent(), warningToken, -2, proposals);
-				return;
-			}
-		} else if (node.getLocationInParent() == SingleVariableDeclaration.NAME_PROPERTY) {
-			addSuppressWarningsProposal(context.getCompilationUnit(), node.getParent(), warningToken, -2, proposals);
-			return;
-		}
-		
-		ASTNode target= ASTResolving.findParentBodyDeclaration(node);
-		if (target != null) {
-			addSuppressWarningsProposal(context.getCompilationUnit(), target, warningToken, -3, proposals);
-		}
-	}
-	
-	private static String getFirstFragmentName(List fragments) {
-		if (fragments.size() > 0) {
-			return ((VariableDeclarationFragment) fragments.get(0)).getName().getIdentifier();
-		}
-		return new String();
-	}
-	
-	private static void addSuppressWarningsProposal(ICompilationUnit cu, ASTNode node, String warningToken, int relevance, Collection proposals) {
-
-		ChildListPropertyDescriptor property= null;
-		String name;
-		switch (node.getNodeType()) {
-			case ASTNode.SINGLE_VARIABLE_DECLARATION:
-				property= SingleVariableDeclaration.MODIFIERS2_PROPERTY;
-				name= ((SingleVariableDeclaration) node).getName().getIdentifier();
-				break;
-			case ASTNode.VARIABLE_DECLARATION_STATEMENT:
-				property= VariableDeclarationStatement.MODIFIERS2_PROPERTY;
-				name= getFirstFragmentName(((VariableDeclarationStatement) node).fragments());
-				break;
-			case ASTNode.TYPE_DECLARATION:
-				property= TypeDeclaration.MODIFIERS2_PROPERTY;
-				name= ((TypeDeclaration) node).getName().getIdentifier();
-				break;
-			case ASTNode.ANNOTATION_TYPE_DECLARATION:
-				property= AnnotationTypeDeclaration.MODIFIERS2_PROPERTY;
-				name= ((AnnotationTypeDeclaration) node).getName().getIdentifier();
-				break;
-			case ASTNode.ENUM_DECLARATION:
-				property= EnumDeclaration.MODIFIERS2_PROPERTY;
-				name= ((EnumDeclaration) node).getName().getIdentifier();
-				break;
-			case ASTNode.FIELD_DECLARATION:	
-				property= FieldDeclaration.MODIFIERS2_PROPERTY;
-				name= getFirstFragmentName(((FieldDeclaration) node).fragments());
-				break;
-			case ASTNode.INITIALIZER:
-				property= Initializer.MODIFIERS2_PROPERTY;
-				name= CorrectionMessages.ModifierCorrectionSubProcessor_suppress_warnings_initializer_label;
-				break;
-			case ASTNode.METHOD_DECLARATION:
-				property= MethodDeclaration.MODIFIERS2_PROPERTY;
-				name= ((MethodDeclaration) node).getName().getIdentifier() + "()"; //$NON-NLS-1$
-				break;
-			case ASTNode.ANNOTATION_TYPE_MEMBER_DECLARATION:
-				property= AnnotationTypeMemberDeclaration.MODIFIERS2_PROPERTY;
-				name= ((AnnotationTypeMemberDeclaration) node).getName().getIdentifier() + "()"; //$NON-NLS-1$
-				break;
-			case ASTNode.ENUM_CONSTANT_DECLARATION:
-				property= EnumConstantDeclaration.MODIFIERS2_PROPERTY;
-				name= ((EnumConstantDeclaration) node).getName().getIdentifier();
-				break;
-			default:
-				JavaPlugin.logErrorMessage("SuppressWarning quick fix: wrong node kind: " + node.getNodeType()); //$NON-NLS-1$
-				return;
-		}
-		
-		AST ast= node.getAST();
-		ASTRewrite rewrite= ASTRewrite.create(ast);
-		
-		StringLiteral newStringLiteral= ast.newStringLiteral();
-		newStringLiteral.setLiteralValue(warningToken);
-		
-		Annotation existing= findExistingAnnotation((List) node.getStructuralProperty(property));
-		if (existing == null) {
-			ListRewrite listRewrite= rewrite.getListRewrite(node, property);
-			
-			SingleMemberAnnotation newAnnot= ast.newSingleMemberAnnotation();
-			newAnnot.setTypeName(ast.newSimpleName("SuppressWarnings")); //$NON-NLS-1$
-
-			newAnnot.setValue(newStringLiteral);
-			
-			listRewrite.insertFirst(newAnnot, null);
-		} else if (existing instanceof SingleMemberAnnotation) {
-			SingleMemberAnnotation annotation= (SingleMemberAnnotation) existing;
-			Expression value= annotation.getValue();
-			if (!addSuppressArgument(rewrite, value, newStringLiteral)) {
-				rewrite.set(existing, SingleMemberAnnotation.VALUE_PROPERTY, newStringLiteral, null);
-			}
-		} else if (existing instanceof NormalAnnotation) {
-			NormalAnnotation annotation= (NormalAnnotation) existing;
-			Expression value= findValue(annotation.values());
-			if (!addSuppressArgument(rewrite, value, newStringLiteral)) {
-				ListRewrite listRewrite= rewrite.getListRewrite(annotation, NormalAnnotation.VALUES_PROPERTY);
-				MemberValuePair pair= ast.newMemberValuePair();
-				pair.setName(ast.newSimpleName("value")); //$NON-NLS-1$
-				pair.setValue(newStringLiteral);
-				listRewrite.insertFirst(pair, null);
-			}	
-		}
-		String label= Messages.format(CorrectionMessages.ModifierCorrectionSubProcessor_suppress_warnings_label, new String[] { warningToken, name });
-		Image image= JavaPluginImages.get(JavaPluginImages.IMG_OBJS_ANNOTATION);
-		ASTRewriteCorrectionProposal proposal= new ASTRewriteCorrectionProposal(label, cu, rewrite, relevance, image);
-		proposal.setCommandId(ADD_SUPPRESSWARNINGS_ID);
-		proposals.add(proposal);
-	}
-	
-	private static boolean addSuppressArgument(ASTRewrite rewrite, Expression value, StringLiteral newStringLiteral) {
-		if (value instanceof ArrayInitializer) {
-			ListRewrite listRewrite= rewrite.getListRewrite(value, ArrayInitializer.EXPRESSIONS_PROPERTY);
-			listRewrite.insertLast(newStringLiteral, null);
-		} else if (value instanceof StringLiteral) {
-			ArrayInitializer newArr= rewrite.getAST().newArrayInitializer();
-			newArr.expressions().add(rewrite.createMoveTarget(value));
-			newArr.expressions().add(newStringLiteral);
-			rewrite.replace(value, newArr, null);
-		} else {
-			return false;
-		}
-		return true;
-	}
-	
-	
-	private static Expression findValue(List keyValues) {
-		for (int i= 0, len= keyValues.size(); i < len; i++) {
-			MemberValuePair curr= (MemberValuePair) keyValues.get(i);
-			if ("value".equals(curr.getName().getIdentifier())) { //$NON-NLS-1$
-				return curr.getValue();
-			}
-		}
-		return null;
-	}
-	
-	private static Annotation findExistingAnnotation(List modifiers) {
-		for (int i= 0, len= modifiers.size(); i < len; i++) {
-			Object curr= modifiers.get(i);
-			if (curr instanceof NormalAnnotation || curr instanceof SingleMemberAnnotation) {
-				Annotation annotation= (Annotation) curr;
-				String fullyQualifiedName= annotation.getTypeName().getFullyQualifiedName();
-				if ("SuppressWarnings".equals(fullyQualifiedName) || "java.lang.SuppressWarnings".equals(fullyQualifiedName)) { //$NON-NLS-1$ //$NON-NLS-2$
-					return annotation;
-				}
-			}
-		}
-		return null;
-	}
 }
