@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -47,11 +47,11 @@ import org.eclipse.jdt.core.dom.ITypeBinding;
 import org.eclipse.jdt.core.dom.MethodDeclaration;
 import org.eclipse.jdt.core.dom.Modifier;
 import org.eclipse.jdt.core.dom.rewrite.ASTRewrite;
+import org.eclipse.jdt.core.dom.rewrite.ImportRewrite;
 import org.eclipse.jdt.core.dom.rewrite.ListRewrite;
 import org.eclipse.jdt.core.formatter.CodeFormatter;
 
 import org.eclipse.jdt.internal.corext.codemanipulation.CodeGenerationSettings;
-import org.eclipse.jdt.internal.corext.codemanipulation.ImportsStructure;
 import org.eclipse.jdt.internal.corext.codemanipulation.StubUtility2;
 import org.eclipse.jdt.internal.corext.dom.ASTNodes;
 import org.eclipse.jdt.internal.corext.dom.NodeFinder;
@@ -59,7 +59,6 @@ import org.eclipse.jdt.internal.corext.template.java.SignatureUtil;
 import org.eclipse.jdt.internal.corext.util.CodeFormatterUtil;
 import org.eclipse.jdt.internal.corext.util.JavaModelUtil;
 import org.eclipse.jdt.internal.corext.util.Strings;
-import org.eclipse.jdt.internal.corext.util.WorkingCopyUtil;
 
 import org.eclipse.jdt.internal.ui.JavaPlugin;
 import org.eclipse.jdt.internal.ui.JavaPluginImages;
@@ -104,15 +103,15 @@ public class AnonymousTypeCompletionProposal extends JavaTypeCompletionProposal
 		return start;
 	}
 
-	private boolean createStubs(StringBuffer buffer, ImportsStructure structure) throws CoreException {
-		if (structure == null)
+	private boolean createStubs(StringBuffer buffer, ImportRewrite importRewrite) throws CoreException {
+		if (importRewrite == null)
 			return false;
 		if (fSuperType == null)
 			return true;
 		ICompilationUnit copy= null;
 		try {
 			final String name= "Type" + System.currentTimeMillis(); //$NON-NLS-1$
-			copy= WorkingCopyUtil.getNewWorkingCopy(fCompilationUnit);
+			copy= fCompilationUnit.getPrimary().getWorkingCopy(null);
 			final StringBuffer contents= new StringBuffer();
 			int start= 0;
 			int end= 0;
@@ -194,7 +193,7 @@ public class AnonymousTypeCompletionProposal extends JavaTypeCompletionProposal
 							key= keys[index];
 							for (int offset= 0; offset < bindings.length; offset++) {
 								if (key.equals(bindings[offset].getKey())) {
-									stub= StubUtility2.createImplementationStub(copy, rewrite, structure, bindings[offset], binding.getName(), binding.isInterface(), settings);
+									stub= StubUtility2.createImplementationStub(copy, rewrite, importRewrite, bindings[offset], binding.getName(), binding.isInterface(), settings);
 									if (stub != null)
 										rewriter.insertFirst(stub, null);
 									break;
@@ -252,7 +251,7 @@ public class AnonymousTypeCompletionProposal extends JavaTypeCompletionProposal
 		return false;
 	}
 
-	protected boolean updateReplacementString(IDocument document, char trigger, int offset, ImportsStructure impStructure) throws CoreException, BadLocationException {
+	protected boolean updateReplacementString(IDocument document, char trigger, int offset, ImportRewrite impRewrite) throws CoreException, BadLocationException {
 		String replacementString= getReplacementString();
 
 		// construct replacement text: an expression to be formatted
@@ -263,7 +262,7 @@ public class AnonymousTypeCompletionProposal extends JavaTypeCompletionProposal
 			buf.append(')');
 		}
 
-		if (!createStubs(buf, impStructure)) {
+		if (!createStubs(buf, impRewrite)) {
 			return false;
 		}
 		if (document.getChar(offset) != ')')
@@ -275,7 +274,7 @@ public class AnonymousTypeCompletionProposal extends JavaTypeCompletionProposal
 		IRegion region= document.getLineInformationOfOffset(getReplacementOffset());
 		int indent= Strings.computeIndentUnits(document.get(region.getOffset(), region.getLength()), project);
 
-		String replacement= CodeFormatterUtil.format(CodeFormatter.K_EXPRESSION, buf.toString(), 0, null, lineDelim, fSuperType.getJavaProject());
+		String replacement= CodeFormatterUtil.format(CodeFormatter.K_EXPRESSION, buf.toString(), 0, null, lineDelim, project);
 		replacement= Strings.changeIndent(replacement, 0, project, CodeFormatterUtil.createIndentString(indent, project), lineDelim);
 		setReplacementString(replacement.substring(replacement.indexOf('(') + 1));
 
