@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -48,7 +48,7 @@ import org.eclipse.jdt.internal.ui.util.SWTUtil;
 /**
  * A list with a button bar.
  * Typical buttons are 'Add', 'Remove', 'Up' and 'Down'.
- * List model is independend of widget creation.
+ * List model is independent of widget creation.
  * DialogFields controls are: Label, List and Composite containing buttons.
  */
 public class TreeListDialogField extends DialogField {
@@ -114,9 +114,9 @@ public class TreeListDialogField extends DialogField {
 
 	/**
 	* Sets the index of the 'remove' button in the button label array passed in
-	* the constructor. The behaviour of the button marked as the 'remove' button
-	* will then behandled internally. (enable state, button invocation
-	* behaviour)
+	* the constructor. The behavior of the button marked as the 'remove' button
+	* will then be handled internally. (enable state, button invocation
+	* behavior)
 	*/
 	public void setRemoveButtonIndex(int removeButtonIndex) {
 		Assert.isTrue(removeButtonIndex < fButtonLabels.length);
@@ -125,9 +125,9 @@ public class TreeListDialogField extends DialogField {
 
 	/**
 	* Sets the index of the 'up' button in the button label array passed in the
-	* constructor. The behaviour of the button marked as the 'up' button will
-	* then behandled internally.
-	* (enable state, button invocation behaviour)
+	* constructor. The behavior of the button marked as the 'up' button will
+	* then be handled internally.
+	* (enable state, button invocation behavior)
 	*/
 	public void setUpButtonIndex(int upButtonIndex) {
 		Assert.isTrue(upButtonIndex < fButtonLabels.length);
@@ -136,9 +136,9 @@ public class TreeListDialogField extends DialogField {
 
 	/**
 	* Sets the index of the 'down' button in the button label array passed in
-	* the constructor. The behaviour of the button marked as the 'down' button
+	* the constructor. The behavior of the button marked as the 'down' button
 	* will then be handled internally. (enable state, button invocation
-	* behaviour)
+	* behavior)
 	*/
 	public void setDownButtonIndex(int downButtonIndex) {
 		Assert.isTrue(downButtonIndex < fButtonLabels.length);
@@ -153,13 +153,9 @@ public class TreeListDialogField extends DialogField {
 		fViewerSorter= viewerSorter;
 	}
 	
-	/**
-	* Sets the viewerSorter.
-	* @param viewerSorter The viewerSorter to set
-	*/
 	public void setTreeExpansionLevel(int level) {
 		fTreeExpandLevel= level;
-		if (fTree != null) {
+		if (isOkToUse(fTreeControl) && fTreeExpandLevel > 0) {
 			fTree.expandToLevel(level);
 		}
 	}	
@@ -244,12 +240,12 @@ public class TreeListDialogField extends DialogField {
 		}
 	}
 
-	// ------ ui creation
+	// ------ UI creation
 
 	/**
 	* Returns the tree control. When called the first time, the control will be
 	* created.
-	* @param The parent composite when called the first time, or <code>null</code>
+	* @param parent The parent composite when called the first time, or <code>null</code>
 	* after.
 	*/
 	public Control getTreeControl(Composite parent) {
@@ -264,7 +260,6 @@ public class TreeListDialogField extends DialogField {
 					handleKeyPressed(e);
 				}
 			});
-			fTree.setAutoExpandLevel(99);
 			fTree.setContentProvider(fTreeViewerAdapter);
 			fTree.setLabelProvider(fLabelProvider);
 			fTree.addSelectionChangedListener(fTreeViewerAdapter);
@@ -302,18 +297,19 @@ public class TreeListDialogField extends DialogField {
 
 	protected TreeViewer createTreeViewer(Composite parent) {
 		Tree tree= new Tree(parent, getTreeStyle());
+		tree.setFont(parent.getFont());
 		return new TreeViewer(tree);
 	}
 
 	protected Button createButton(Composite parent, String label, SelectionListener listener) {
 		Button button= new Button(parent, SWT.PUSH);
+		button.setFont(parent.getFont());
 		button.setText(label);
 		button.addSelectionListener(listener);
 		GridData gd= new GridData();
 		gd.horizontalAlignment= GridData.FILL;
 		gd.grabExcessHorizontalSpace= true;
 		gd.verticalAlignment= GridData.BEGINNING;
-		gd.heightHint= SWTUtil.getButtonHeigthHint(button);
 		gd.widthHint= SWTUtil.getButtonWidthHint(button);
 
 		button.setLayoutData(gd);
@@ -322,6 +318,7 @@ public class TreeListDialogField extends DialogField {
 
 	private Label createSeparator(Composite parent) {
 		Label separator= new Label(parent, SWT.NONE);
+		separator.setFont(parent.getFont());
 		separator.setVisible(false);
 		GridData gd= new GridData();
 		gd.horizontalAlignment= GridData.FILL;
@@ -334,7 +331,7 @@ public class TreeListDialogField extends DialogField {
 	/**
 	* Returns the composite containing the buttons. When called the first time, the control
 	* will be created.
-	* @param The parent composite when called the first time, or <code>null</code>
+	* @param parent The parent composite when called the first time, or <code>null</code>
 	* after.
 	*/
 	public Composite getButtonBox(Composite parent) {
@@ -350,7 +347,9 @@ public class TreeListDialogField extends DialogField {
 				}
 			};
 
-			Composite contents= new Composite(parent, SWT.NULL);
+			Composite contents= new Composite(parent, SWT.NONE);
+			contents.setFont(parent.getFont());
+			
 			GridLayout layout= new GridLayout();
 			layout.marginWidth= 0;
 			layout.marginHeight= 0;
@@ -418,7 +417,7 @@ public class TreeListDialogField extends DialogField {
 	* Updates the enable state of the all buttons
 	*/
 	protected void updateButtonState() {
-		if (fButtonControls != null) {
+		if (fButtonControls != null && isOkToUse(fTreeControl) && fTreeControl.isEnabled()) {
 			ISelection sel= fTree.getSelection();
 			for (int i= 0; i < fButtonControls.length; i++) {
 				Button button= fButtonControls[i];
@@ -496,7 +495,7 @@ public class TreeListDialogField extends DialogField {
 	public void setElements(List elements) {
 		fElements= new ArrayList(elements);
 		refresh();
-		if (fTree != null) {
+		if (isOkToUse(fTreeControl)) {
 			fTree.expandToLevel(fTreeExpandLevel);
 		}
 		dialogFieldChanged();
@@ -531,7 +530,7 @@ public class TreeListDialogField extends DialogField {
 		int idx= fElements.indexOf(oldElement);
 		if (idx != -1) {
 			fElements.set(idx, newElement);
-			if (fTree != null) {
+			if (isOkToUse(fTreeControl)) {
 				List selected= getSelectedElements();
 				if (selected.remove(oldElement)) {
 					selected.add(newElement);
@@ -553,22 +552,23 @@ public class TreeListDialogField extends DialogField {
 	/**
 	* Adds an element at the end of the tree list.
 	*/
-	public void addElement(Object element) {
+	public boolean addElement(Object element) {
 		if (fElements.contains(element)) {
-			return;
+			return false;
 		}
 		fElements.add(element);
-		if (fTree != null) {
+		if (isOkToUse(fTreeControl)) {
 			fTree.add(fParentElement, element);
 			fTree.expandToLevel(element, fTreeExpandLevel);
 		}
 		dialogFieldChanged();
+		return true;
 	}
 
 	/**
 	* Adds elements at the end of the tree list.
 	*/
-	public void addElements(List elements) {
+	public boolean addElements(List elements) {
 		int nElements= elements.size();
 
 		if (nElements > 0) {
@@ -581,15 +581,19 @@ public class TreeListDialogField extends DialogField {
 					elementsToAdd.add(elem);
 				}
 			}
-			fElements.addAll(elementsToAdd);
-			if (fTree != null) {
-				fTree.add(fParentElement, elementsToAdd.toArray());
-				for (int i= 0; i < elementsToAdd.size(); i++) {
-					fTree.expandToLevel(elementsToAdd.get(i), fTreeExpandLevel);
+			if (!elementsToAdd.isEmpty()) {
+				fElements.addAll(elementsToAdd);
+				if (isOkToUse(fTreeControl)) {
+					fTree.add(fParentElement, elementsToAdd.toArray());
+					for (int i= 0; i < elementsToAdd.size(); i++) {
+						fTree.expandToLevel(elementsToAdd.get(i), fTreeExpandLevel);
+					}
 				}
-			}
-			dialogFieldChanged();
+				dialogFieldChanged();
+				return true;
+			}			
 		}
+		return false;
 	}
 
 	/**
@@ -600,7 +604,7 @@ public class TreeListDialogField extends DialogField {
 			return;
 		}
 		fElements.add(index, element);
-		if (fTree != null) {
+		if (isOkToUse(fTreeControl)) {
 			fTree.add(fParentElement, element);
 			if (fTreeExpandLevel != -1) {
 				fTree.expandToLevel(element, fTreeExpandLevel);
@@ -626,7 +630,7 @@ public class TreeListDialogField extends DialogField {
 	*/
 	public void removeElement(Object element) throws IllegalArgumentException {
 		if (fElements.remove(element)) {
-			if (fTree != null) {
+			if (isOkToUse(fTreeControl)) {
 				fTree.remove(element);
 			}
 			dialogFieldChanged();
@@ -641,7 +645,7 @@ public class TreeListDialogField extends DialogField {
 	public void removeElements(List elements) {
 		if (elements.size() > 0) {
 			fElements.removeAll(elements);
-			if (fTree != null) {
+			if (isOkToUse(fTreeControl)) {
 				fTree.remove(elements.toArray());
 			}
 			dialogFieldChanged();
@@ -657,7 +661,7 @@ public class TreeListDialogField extends DialogField {
 
 	public void selectElements(ISelection selection) {
 		fSelectionWhenEnabled= selection;
-		if (fTree != null) {
+		if (isOkToUse(fTreeControl)) {
 			fTree.setSelection(selection, true);
 		}
 	}
@@ -697,7 +701,8 @@ public class TreeListDialogField extends DialogField {
 	* Refreshes the tree.
 	*/
 	public void refresh() {
-		if (fTree != null) {
+		super.refresh();
+		if (isOkToUse(fTreeControl)) {
 			fTree.refresh();
 		}
 	}
@@ -706,9 +711,18 @@ public class TreeListDialogField extends DialogField {
 	* Refreshes the tree.
 	*/
 	public void refresh(Object element) {
-		if (fTree != null) {
+		if (isOkToUse(fTreeControl)) {
 			fTree.refresh(element);
 		}
+	}
+	
+	/**
+	* Updates the element.
+	*/
+	public void update(Object element) {
+		if (isOkToUse(fTreeControl)) {
+			fTree.update(element, null);
+		}
 	}	
 
 	// ------- list maintenance
@@ -800,7 +814,7 @@ public class TreeListDialogField extends DialogField {
 	*/
 	public List getSelectedElements() {
 		ArrayList result= new ArrayList();
-		if (fTree != null) {
+		if (isOkToUse(fTreeControl)) {
 			ISelection selection= fTree.getSelection();
 			if (selection instanceof IStructuredSelection) {
 				Iterator iter= ((IStructuredSelection)selection).iterator();
@@ -813,7 +827,7 @@ public class TreeListDialogField extends DialogField {
 	}
 	
 	public void expandElement(Object element, int level) {
-		if (fTree != null) {
+		if (isOkToUse(fTreeControl)) {
 			fTree.expandToLevel(element, level);
 		}
 	}
