@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -13,8 +13,8 @@ package org.eclipse.jface.viewers;
 import java.util.ArrayList;
 import java.util.List;
 
+import org.eclipse.core.runtime.ListenerList;
 import org.eclipse.jface.util.Assert;
-import org.eclipse.jface.util.ListenerList;
 import org.eclipse.jface.util.SafeRunnable;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.SelectionEvent;
@@ -40,7 +40,7 @@ public class CheckboxTreeViewer extends TreeViewer implements ICheckable {
     /**
      * List of check state listeners (element type: <code>ICheckStateListener</code>).
      */
-    private ListenerList checkStateListeners = new ListenerList(3);
+    private ListenerList checkStateListeners = new ListenerList();
 
     /**
      * Last item clicked on, or <code>null</code> if none.
@@ -154,10 +154,12 @@ public class CheckboxTreeViewer extends TreeViewer implements ICheckable {
                 Object data = item.getData();
                 if (data != null) {
                     TreeItem ti = (TreeItem) item;
-                    if (ti.getChecked())
-                        checked.put(data, data);
-                    if (ti.getGrayed())
-                        grayed.put(data, data);
+                    if (ti.getChecked()) {
+						checked.put(data, data);
+					}
+                    if (ti.getGrayed()) {
+						grayed.put(data, data);
+					}
                 }
             }
             gatherState(checked, grayed, item);
@@ -169,8 +171,9 @@ public class CheckboxTreeViewer extends TreeViewer implements ICheckable {
      */
     public boolean getChecked(Object element) {
         Widget widget = findItem(element);
-        if (widget instanceof TreeItem)
-            return ((TreeItem) widget).getChecked();
+        if (widget instanceof TreeItem) {
+			return ((TreeItem) widget).getChecked();
+		}
         return false;
     }
 
@@ -243,8 +246,9 @@ public class CheckboxTreeViewer extends TreeViewer implements ICheckable {
                         !state));
             }
             lastClickedItem = null;
-        } else
-            super.handleDoubleSelect(event);
+        } else {
+			super.handleDoubleSelect(event);
+		}
     }
 
     /* (non-Javadoc)
@@ -263,8 +267,9 @@ public class CheckboxTreeViewer extends TreeViewer implements ICheckable {
                 fireCheckStateChanged(new CheckStateChangedEvent(this, data,
                         item.getChecked()));
             }
-        } else
-            super.handleSelect(event);
+        } else {
+			super.handleSelect(event);
+		}
     }
 
     /**
@@ -280,8 +285,9 @@ public class CheckboxTreeViewer extends TreeViewer implements ICheckable {
             Item item = items[i];
             if (item instanceof TreeItem && ((TreeItem) item).getChecked()) {
                 Object data = item.getData();
-                if (data != null)
-                    result.add(data);
+                if (data != null) {
+					result.add(data);
+				}
             }
             internalCollectChecked(result, item);
         }
@@ -300,8 +306,9 @@ public class CheckboxTreeViewer extends TreeViewer implements ICheckable {
             Item item = items[i];
             if (item instanceof TreeItem && ((TreeItem) item).getGrayed()) {
                 Object data = item.getData();
-                if (data != null)
-                    result.add(data);
+                if (data != null) {
+					result.add(data);
+				}
             }
             internalCollectGrayed(result, item);
         }
@@ -551,4 +558,30 @@ public class CheckboxTreeViewer extends TreeViewer implements ICheckable {
         }
         return false;
     }
+
+    /**
+     * Sets to the given value the checked state for all elements in this viewer.
+     *
+     * @param state <code>true</code> if the element should be checked,
+     *  and <code>false</code> if it should be unchecked
+     *  
+     *  @since 3.2
+     */
+	public void setAllChecked(boolean state) {
+		setAllChecked(state,  getTree().getItems());
+		
+	}
+
+	/**
+	 * Set the checked state of items and thier children to state.
+	 * @param state
+	 * @param items
+	 */
+	private void setAllChecked(boolean state, TreeItem[] items) {
+		for (int i = 0; i < items.length; i++) {
+			items[i].setChecked(state);			
+			TreeItem[] children = items[i].getItems();
+			setAllChecked(state, children);
+		}
+	}
 }
