@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,12 +10,14 @@
  *******************************************************************************/
 package org.eclipse.jface.viewers;
 
-import java.text.MessageFormat;
+import java.text.MessageFormat;	// Not using ICU to support standalone JFace scenario
 
 import org.eclipse.jface.resource.ImageDescriptor;
 import org.eclipse.jface.resource.ImageRegistry;
 import org.eclipse.jface.resource.JFaceResources;
 import org.eclipse.swt.SWT;
+import org.eclipse.swt.events.FocusEvent;
+import org.eclipse.swt.events.FocusListener;
 import org.eclipse.swt.events.KeyAdapter;
 import org.eclipse.swt.events.KeyEvent;
 import org.eclipse.swt.events.SelectionAdapter;
@@ -74,6 +76,12 @@ public abstract class DialogCellEditor extends CellEditor {
      */
     private Button button;
 
+	/**
+	 * Listens for 'focusLost' events and  fires the 'apply' event as long
+	 * as the focus wasn't lost because the dialog was opened.
+	 */
+	private FocusListener buttonFocusListener;
+
     /**
      * The value of this cell editor; initially <code>null</code>.
      */
@@ -92,15 +100,17 @@ public abstract class DialogCellEditor extends CellEditor {
         public void layout(Composite editor, boolean force) {
             Rectangle bounds = editor.getClientArea();
             Point size = button.computeSize(SWT.DEFAULT, SWT.DEFAULT, force);
-            if (contents != null)
-                contents.setBounds(0, 0, bounds.width - size.x, bounds.height);
+            if (contents != null) {
+				contents.setBounds(0, 0, bounds.width - size.x, bounds.height);
+			}
             button.setBounds(bounds.width - size.x, 0, size.x, bounds.height);
         }
 
         public Point computeSize(Composite editor, int wHint, int hHint,
                 boolean force) {
-            if (wHint != SWT.DEFAULT && hHint != SWT.DEFAULT)
-                return new Point(wHint, hHint);
+            if (wHint != SWT.DEFAULT && hHint != SWT.DEFAULT) {
+				return new Point(wHint, hHint);
+			}
             Point contentsSize = contents.computeSize(SWT.DEFAULT, SWT.DEFAULT,
                     force);
             Point buttonSize = button.computeSize(SWT.DEFAULT, SWT.DEFAULT,
@@ -208,6 +218,9 @@ public abstract class DialogCellEditor extends CellEditor {
         button.setFont(font);
 
         button.addKeyListener(new KeyAdapter() {
+            /* (non-Javadoc)
+             * @see org.eclipse.swt.events.KeyListener#keyReleased(org.eclipse.swt.events.KeyEvent)
+             */
             public void keyReleased(KeyEvent e) {
                 if (e.character == '\u001b') { // Escape
                     fireCancelEditor();
@@ -216,9 +229,20 @@ public abstract class DialogCellEditor extends CellEditor {
         });
 
         button.addSelectionListener(new SelectionAdapter() {
+            /* (non-Javadoc)
+             * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
+             */
             public void widgetSelected(SelectionEvent event) {
-                Object newValue = openDialogBox(editor);
-                if (newValue != null) {
+            	// Remove the button's focus listener since it's guaranteed
+            	// to lose focus when the dialog opens
+            	button.removeFocusListener(getButtonFocusListener());
+                
+            	Object newValue = openDialogBox(editor);
+            	
+            	// Re-add the listener once the dialog closes
+            	button.addFocusListener(getButtonFocusListener());
+
+            	if (newValue != null) {
                     boolean newValidState = isCorrect(newValue);
                     if (newValidState) {
                         markDirty();
@@ -239,6 +263,21 @@ public abstract class DialogCellEditor extends CellEditor {
     }
 
     /* (non-Javadoc)
+     * 
+     * Override in order to remove the button's focus listener if the celleditor
+     * is deactivating.
+     * 
+     * @see org.eclipse.jface.viewers.CellEditor#deactivate()
+     */
+    public void deactivate() {
+    	if (button != null && !button.isDisposed()) {
+    		button.removeFocusListener(getButtonFocusListener());
+    	}
+    	
+		super.deactivate();
+	}
+
+	/* (non-Javadoc)
      * Method declared on CellEditor.
      */
     protected Object doGetValue() {
@@ -251,9 +290,39 @@ public abstract class DialogCellEditor extends CellEditor {
      */
     protected void doSetFocus() {
         button.setFocus();
+        
+        // add a FocusListener to the button
+        button.addFocusListener(getButtonFocusListener());
     }
 
-    /* (non-Javadoc)
+    /**
+     * Return a listener for button focus.
+     * @return FocusListener
+     */
+    private FocusListener getButtonFocusListener() {
+    	if (buttonFocusListener == null) {
+    		buttonFocusListener = new FocusListener() {
+
+				/* (non-Javadoc)
+				 * @see org.eclipse.swt.events.FocusListener#focusGained(org.eclipse.swt.events.FocusEvent)
+				 */
+				public void focusGained(FocusEvent e) {
+					// Do nothing
+				}
+
+				/* (non-Javadoc)
+				 * @see org.eclipse.swt.events.FocusListener#focusLost(org.eclipse.swt.events.FocusEvent)
+				 */
+				public void focusLost(FocusEvent e) {
+					fireApplyEditorValue();
+				}
+    		};
+    	}
+    	
+    	return buttonFocusListener;
+	}
+
+	/* (non-Javadoc)
      * Method declared on CellEditor.
      */
     protected void doSetValue(Object value) {
@@ -302,12 +371,14 @@ public abstract class DialogCellEditor extends CellEditor {
      * @param value the new value of this cell editor
      */
     protected void updateContents(Object value) {
-        if (defaultLabel == null)
-            return;
+        if (defaultLabel == null) {
+			return;
+		}
 
         String text = "";//$NON-NLS-1$
-        if (value != null)
-            text = value.toString();
+        if (value != null) {
+			text = value.toString();
+		}
         defaultLabel.setText(text);
     }
 }
