@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -189,7 +189,7 @@ public class BundleHost extends AbstractBundle {
 		}
 		if (Debug.DEBUG && Debug.DEBUG_GENERAL) {
 			if ((state & (STARTING | ACTIVE | STOPPING | RESOLVED)) == 0) {
-				Debug.println("Bundle.checkLoader() called when state != STARTING | ACTIVE | STOPPING | RESOLVED: " + this); //$NON-NLS-1$ //$NON-NLS-2$
+				Debug.println("Bundle.checkLoader() called when state != STARTING | ACTIVE | STOPPING | RESOLVED: " + this); //$NON-NLS-1$ 
 				Debug.printStackTrace(new Exception("Stack trace")); //$NON-NLS-1$
 			}
 		}
@@ -197,7 +197,7 @@ public class BundleHost extends AbstractBundle {
 		BundleLoader loader = getBundleLoader();
 		if (loader == null) {
 			if (Debug.DEBUG && Debug.DEBUG_GENERAL) {
-				Debug.println("Bundle.checkLoader() called when loader == null: " + this); //$NON-NLS-1$ //$NON-NLS-2$
+				Debug.println("Bundle.checkLoader() called when loader == null: " + this); //$NON-NLS-1$ 
 				Debug.printStackTrace(new Exception("Stack trace")); //$NON-NLS-1$
 			}
 			return null;
@@ -223,7 +223,7 @@ public class BundleHost extends AbstractBundle {
 		}
 		BundleLoader loader = checkLoader();
 		if (loader == null)
-			throw new ClassNotFoundException(NLS.bind(Msg.BUNDLE_CNFE_NOT_RESOLVED, getBundleData().getLocation(), name)); //$NON-NLS-1$
+			throw new ClassNotFoundException(NLS.bind(Msg.BUNDLE_CNFE_NOT_RESOLVED, getBundleData().getLocation(), name)); 
 		return (loader.loadClass(name));
 	}
 
@@ -279,12 +279,15 @@ public class BundleHost extends AbstractBundle {
 	 * @param persistent if true persistently record the bundle was started.
 	 */
 	protected void startWorker(boolean persistent) throws BundleException {
+		if (persistent) {
+			setStatus(Constants.BUNDLE_STARTED, true);
+		}
 		long start = 0;
 		if (framework.active) {
 			if ((state & (STARTING | ACTIVE)) != 0) {
 				return;
 			}
-
+			boolean shouldStart = false;
 			try {
 				if (state == INSTALLED) {
 					if (!framework.packageAdmin.resolveBundles(new Bundle[] {this})) {
@@ -295,15 +298,13 @@ public class BundleHost extends AbstractBundle {
 				if (Debug.DEBUG && Debug.DEBUG_GENERAL) {
 					Debug.println("Bundle: Active sl = " + framework.startLevelManager.getStartLevel() + "; Bundle " + getBundleId() + " sl = " + getStartLevel()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
 				}
-
-				if (getStartLevel() <= framework.startLevelManager.getStartLevel()) {
+				shouldStart = getStartLevel() <= framework.startLevelManager.getStartLevel();
+				if (shouldStart) {
 					//STARTUP TIMING Start here					
 					if (Debug.DEBUG) {
-						if (Debug.MONITOR_ACTIVATION) {
-							BundleWatcher bundleStats = framework.adaptor.getBundleWatcher();
-							if (bundleStats != null)
-								bundleStats.startActivation(this);
-						}
+						BundleWatcher bundleStats = framework.adaptor.getBundleWatcher();
+						if (bundleStats != null)
+							bundleStats.watchBundle(this, BundleWatcher.START_ACTIVATION);
 						if (Debug.DEBUG_BUNDLE_TIME) {
 							start = System.currentTimeMillis();
 							System.out.println("Starting " + getSymbolicName()); //$NON-NLS-1$
@@ -337,25 +338,19 @@ public class BundleHost extends AbstractBundle {
 					if (state == UNINSTALLED) {
 						context.close();
 						context = null;
-						throw new BundleException(NLS.bind(Msg.BUNDLE_UNINSTALLED_EXCEPTION, getBundleData().getLocation())); //$NON-NLS-1$
+						throw new BundleException(NLS.bind(Msg.BUNDLE_UNINSTALLED_EXCEPTION, getBundleData().getLocation())); 
 					}
 				}
 			} finally {
-				if (Debug.DEBUG && state == ACTIVE) {
-					if (Debug.MONITOR_ACTIVATION) {
-						BundleWatcher bundleStats = framework.adaptor.getBundleWatcher();
-						if (bundleStats != null)
-							bundleStats.endActivation(this);
-					}
+				if (Debug.DEBUG && shouldStart) {
+					BundleWatcher bundleStats = framework.adaptor.getBundleWatcher();
+					if (bundleStats != null)
+						bundleStats.watchBundle(this, BundleWatcher.END_ACTIVATION);
 					if (Debug.DEBUG_BUNDLE_TIME)
 						System.out.println("End starting " + getSymbolicName() + " " + (System.currentTimeMillis() - start)); //$NON-NLS-1$ //$NON-NLS-2$
 				}
 			}
 		}
-
-		if (persistent) {
-			setStatus(Constants.BUNDLE_STARTED, true);
-		}
 	}
 
 	/**
@@ -390,7 +385,11 @@ public class BundleHost extends AbstractBundle {
 			if ((state & (STOPPING | RESOLVED | INSTALLED)) != 0) {
 				return;
 			}
-
+			if (Debug.DEBUG) {
+				BundleWatcher bundleStats = framework.adaptor.getBundleWatcher();
+				if (bundleStats != null)
+					bundleStats.watchBundle(this, BundleWatcher.START_DEACTIVATION);
+			}
 			state = STOPPING;
 			framework.publishBundleEvent(BundleEvent.STOPPING, this);
 			try {
@@ -408,6 +407,11 @@ public class BundleHost extends AbstractBundle {
 				}
 
 				framework.publishBundleEvent(BundleEvent.STOPPED, this);
+				if (Debug.DEBUG) {
+					BundleWatcher bundleStats = framework.adaptor.getBundleWatcher();
+					if (bundleStats != null)
+						bundleStats.watchBundle(this, BundleWatcher.END_DEACTIVATION);
+				}
 			}
 		}
 	}
@@ -505,7 +509,7 @@ public class BundleHost extends AbstractBundle {
 					// then we cannot attach a fragment into the middle
 					// of the fragment chain.
 					if (loader != null) {
-						throw new BundleException(NLS.bind(Msg.BUNDLE_LOADER_ATTACHMENT_ERROR, fragments[i].getSymbolicName(), getSymbolicName())); //$NON-NLS-1$
+						throw new BundleException(NLS.bind(Msg.BUNDLE_LOADER_ATTACHMENT_ERROR, fragments[i].getSymbolicName(), getSymbolicName())); 
 					}
 					newFragments[i] = fragment;
 					inserted = true;
@@ -522,16 +526,12 @@ public class BundleHost extends AbstractBundle {
 		return getLoaderProxy().getBundleLoader();
 	}
 
-	protected BundleLoaderProxy getLoaderProxy() {
-		if (proxy == null) {
-			synchronized (this) {
-				if (proxy == null) {
-					BundleDescription bundleDescription = getBundleDescription();
-					proxy = new BundleLoaderProxy(this, bundleDescription);
-					bundleDescription.setUserObject(proxy);
-				}
-			}
-		}
+	protected synchronized BundleLoaderProxy getLoaderProxy() {
+		if (proxy != null)
+			return proxy;
+		BundleDescription bundleDescription = getBundleDescription();
+		proxy = new BundleLoaderProxy(this, bundleDescription);
+		bundleDescription.setUserObject(proxy);
 		return proxy;
 	}
 
