@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,6 +11,9 @@
 package org.eclipse.osgi.framework.internal.core;
 
 import java.io.*;
+import java.lang.reflect.Field;
+import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
 import java.net.*;
 import java.security.*;
 import java.util.*;
@@ -54,8 +57,7 @@ public class Framework implements EventDispatcher, EventPublisher {
 	protected ServiceRegistry serviceRegistry; //TODO This is duplicated from the adaptor, do we really gain ?
 	/** next free service id. */
 	protected long serviceid;
-	/** the VM profile (execution environment */
-	private String vmProfile;
+
 	/*
 	 * The following EventListeners objects keep track of event listeners
 	 * by BundleContext.  Each element is a EventListeners that is the list
@@ -86,7 +88,7 @@ public class Framework implements EventDispatcher, EventPublisher {
 	String[] bootDelegation;
 	String[] bootDelegationStems;
 	boolean bootDelegateAll = false;
-	boolean contextBootDelegation = "true".equals(System.getProperty("osgi.context.bootdelegation", "true")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+	boolean contextBootDelegation = "true".equals(FrameworkProperties.getProperty("osgi.context.bootdelegation", "true")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
 
 	/**
 	 * The AliasMapper used to alias OS Names.
@@ -95,7 +97,11 @@ public class Framework implements EventDispatcher, EventPublisher {
 	protected ConditionalPermissionAdminImpl condPermAdmin;
 	SecureAction secureAction = new SecureAction();
 	// cache of AdminPermissions keyed by Bundle ID
-	private HashMap adminPermissions;
+	private HashMap adminPermissions = new HashMap();
+	
+	// we need to hold these so that we can unregister them at shutdown
+	private StreamHandlerFactory streamHandlerFactory;
+	private ContentHandlerFactory contentHandlerFactory;
 
 	/**
 	 * Constructor for the Framework instance. This method initializes the
@@ -183,9 +189,9 @@ public class Framework implements EventDispatcher, EventPublisher {
 		if (Profile.PROFILE && Profile.STARTUP)
 			Profile.logTime("Framework.initialze()", "done createSystemBundle"); //$NON-NLS-1$ //$NON-NLS-2$
 		/* install URLStreamHandlerFactory */
-		URL.setURLStreamHandlerFactory(new StreamHandlerFactory(systemBundle.context, adaptor));
+		installURLStreamHandlerFactory(systemBundle.context, adaptor);
 		/* install ContentHandlerFactory for OSGi URLStreamHandler support */
-		URLConnection.setContentHandlerFactory(new ContentHandlerFactory(systemBundle.context));
+		installContentHandlerFactory(systemBundle.context, adaptor);
 		if (Profile.PROFILE && Profile.STARTUP)
 			Profile.logTime("Framework.initialze()", "done new URLStream/Content HandlerFactory"); //$NON-NLS-1$//$NON-NLS-2$
 		/* create bundle objects for all installed bundles. */
@@ -225,7 +231,7 @@ public class Framework implements EventDispatcher, EventPublisher {
 	 *  
 	 */
 	protected void initializeProperties(Properties adaptorProperties) {
-		properties = System.getProperties();
+		properties = FrameworkProperties.getProperties();
 		Enumeration enumKeys = adaptorProperties.propertyNames();
 		while (enumKeys.hasMoreElements()) {
 			String key = (String) enumKeys.nextElement();
@@ -271,70 +277,9 @@ public class Framework implements EventDispatcher, EventPublisher {
 			}
 		}
 		value = properties.getProperty(Constants.FRAMEWORK_LANGUAGE);
-		if (value == null) {
-			value = properties.getProperty(Constants.JVM_USER_LANGUAGE);
-			// set default locale for VM
-			if (value != null) {
-				properties.put(Constants.FRAMEWORK_LANGUAGE, value);
-				StringTokenizer tokenizer = new StringTokenizer(value, "_"); //$NON-NLS-1$
-				int segments = tokenizer.countTokens();
-				try {
-					switch (segments) {
-						case 2 :
-							Locale userLocale = new Locale(tokenizer.nextToken(), tokenizer.nextToken());
-							Locale.setDefault(userLocale);
-							break;
-						case 3 :
-							userLocale = new Locale(tokenizer.nextToken(), tokenizer.nextToken(), tokenizer.nextToken());
-							Locale.setDefault(userLocale);
-							break;
-					}
-				} catch (NoSuchElementException e) {
-					// fall through and use the default
-				}
-			}
-		}
-		setExecutionEnvironment();
-	}
-
-	private void setExecutionEnvironment() {
-		String value = properties.getProperty(Constants.FRAMEWORK_EXECUTIONENVIRONMENT, ""); //$NON-NLS-1$
-		String j2meConfig = properties.getProperty(Constants.J2ME_MICROEDITION_CONFIGURATION);
-		String j2meProfile = properties.getProperty(Constants.J2ME_MICROEDITION_PROFILES);
-		StringBuffer ee = new StringBuffer(value);
-		if (j2meConfig != null && j2meConfig.length() > 0 && j2meProfile != null && j2meProfile.length() > 0) {
-			// save the vmProfile based off of the config and profile
-			vmProfile = j2meConfig + '_' + j2meProfile;
-			int ic = value.indexOf(j2meConfig);
-			// append the profile only if it is not already present
-			if (!(ic >= 0) || !(ic + j2meConfig.length() < value.length() && value.charAt(ic + j2meConfig.length()) == '/') || !(value.startsWith(j2meProfile, ic + j2meConfig.length() + 1))) {
-				if (ee.length() > 0) {
-					ee.append(',');
-				}
-				ee.append(j2meConfig).append('/').append(j2meProfile);
-			}
-
-		} else if (value.length() > 0) {
-			// just use the first EE defined as our profile
-			StringTokenizer st = new StringTokenizer(value, ","); //$NON-NLS-1$
-			vmProfile = st.nextToken().replace('/', '_');
-		} else {
-			String javaSpecVersion = properties.getProperty("java.specification.version"); //$NON-NLS-1$
-			// set the profile and EE based off of the java.specification.version
-			// TODO We assume J2SE here.  need to support other profiles J2ME/J2EE ...
-			if (javaSpecVersion != null) {
-				StringTokenizer st = new StringTokenizer(javaSpecVersion, " _-"); //$NON-NLS-1$
-				javaSpecVersion = st.nextToken();
-				vmProfile = "J2SE-" + javaSpecVersion; //$NON-NLS-1$
-				int index = value.indexOf(vmProfile);
-				if (index < 0) {
-					if (ee.length() > 0)
-						ee.append(',');
-					ee.append(vmProfile);
-				}
-			}
-		}
-		properties.put(Constants.FRAMEWORK_EXECUTIONENVIRONMENT, ee.toString());
+		if (value == null)
+			// set the value of the framework language property
+			properties.put(Constants.FRAMEWORK_LANGUAGE, Locale.getDefault().getLanguage());
 	}
 
 	private void setBootDelegation() {
@@ -364,21 +309,7 @@ public class Framework implements EventDispatcher, EventPublisher {
 	}
 
 	private void loadVMProfile() {
-		InputStream in = findVMProfile();
-		Properties profileProps = new Properties();
-		if (in != null) {
-			try {
-				profileProps.load(new BufferedInputStream(in));
-			} catch (IOException e) {
-				// do nothing
-			} finally {
-				try {
-					in.close();
-				} catch (IOException ee) {
-					// do nothing
-				}
-			}
-		}
+		Properties profileProps = findVMProfile();
 		String systemExports = properties.getProperty(Constants.OSGI_FRAMEWORK_SYSTEM_PACKAGES);
 		// set the system exports property using the vm profile; only if the property is not already set
 		if (systemExports == null) {
@@ -396,12 +327,59 @@ public class Framework implements EventDispatcher, EventPublisher {
 				properties.put(Constants.OSGI_BOOTDELEGATION, profileBootDelegation); // override with the profile value
 		} else if (Constants.OSGI_BOOTDELEGATION_NONE.equals(type))
 			properties.remove(Constants.OSGI_BOOTDELEGATION); // remove the bootdelegation property in case it was set
+		// set the org.osgi.framework.executionenvironment property according to the java profile
+		if (properties.getProperty(Constants.FRAMEWORK_EXECUTIONENVIRONMENT) == null) {
+			// get the ee from the java profile; if no ee is defined then try the java profile name
+			String ee = profileProps.getProperty(Constants.FRAMEWORK_EXECUTIONENVIRONMENT, profileProps.getProperty(Constants.OSGI_JAVA_PROFILE_NAME));
+			if (ee != null)
+				properties.put(Constants.FRAMEWORK_EXECUTIONENVIRONMENT, ee);
+		}
 	}
 
-	private InputStream findVMProfile() {
+	private Properties findVMProfile() {
+		Properties result = new Properties();
+		// Find the VM profile name using J2ME properties
+		String j2meConfig = properties.getProperty(Constants.J2ME_MICROEDITION_CONFIGURATION);
+		String j2meProfiles = properties.getProperty(Constants.J2ME_MICROEDITION_PROFILES);
+		String vmProfile = null;
+		if (j2meConfig != null && j2meConfig.length() > 0 && j2meProfiles != null && j2meProfiles.length() > 0) {
+			// save the vmProfile based off of the config and profile
+			// use the last profile; assuming that is the highest one
+			String[] j2meProfileList = ManifestElement.getArrayFromList(j2meProfiles, " "); //$NON-NLS-1$
+			if (j2meProfileList != null && j2meProfileList.length > 0)
+				vmProfile = j2meConfig + '_' + j2meProfileList[j2meProfileList.length - 1];
+		} else {
+			// No J2ME properties; use J2SE properties
+			// Note that the CDC spec appears not to require VM implementations to set the
+			// javax.microedition properties!!  So we will try to fall back to the 
+			// java.specification.name property, but this is pretty ridiculous!!
+			String javaSpecVersion = properties.getProperty("java.specification.version"); //$NON-NLS-1$
+			// set the profile and EE based off of the java.specification.version
+			// TODO We assume J2ME Foundation and J2SE here.  need to support other profiles J2EE ...
+			if (javaSpecVersion != null) {
+				StringTokenizer st = new StringTokenizer(javaSpecVersion, " _-"); //$NON-NLS-1$
+				javaSpecVersion = st.nextToken();
+				String javaSpecName = properties.getProperty("java.specification.name"); //$NON-NLS-1$
+				if ("J2ME Foundation Specification".equals(javaSpecName)) //$NON-NLS-1$
+					vmProfile = "CDC-" + javaSpecVersion + "_Foundation-" + javaSpecVersion; //$NON-NLS-1$ //$NON-NLS-2$
+				else {
+					// look for JavaSE if 1.6 or greater; otherwise look for J2SE
+					Version v16 = new Version("1.6"); //$NON-NLS-1$
+					String javaEdition = "J2SE-"; //$NON-NLS-1$
+					try {
+						Version vEdition = new Version(javaSpecVersion);
+						if (v16.compareTo(vEdition) <= 0)
+							javaEdition = "JavaSE-"; //$NON-NLS-1$
+					} catch (IllegalArgumentException e) {
+						// do nothing
+					}
+					vmProfile = javaEdition + javaSpecVersion;
+				}
+			}
+		}
 		URL url = null;
 		// check for the java profile property for a url
-		String propJavaProfile = System.getProperty(Constants.OSGI_JAVA_PROFILE);
+		String propJavaProfile = FrameworkProperties.getProperty(Constants.OSGI_JAVA_PROFILE);
 		if (propJavaProfile != null)
 			try {
 				// we assume a URL
@@ -413,16 +391,36 @@ public class Framework implements EventDispatcher, EventPublisher {
 			// look for a profile in the system bundle based on the vm profile
 			String javaProfile = vmProfile + ".profile"; //$NON-NLS-1$
 			url = systemBundle.getEntry(javaProfile);
-			if (url == null)
-				url = getClass().getResource(javaProfile);
+			if (url == null) {
+				// Check the ClassLoader in case we're launched off the Java boot classpath
+				ClassLoader loader=getClass().getClassLoader();
+				url = loader==null ? ClassLoader.getSystemResource(javaProfile) : loader.getResource(javaProfile);
+			}
 		}
-		if (url != null)
+		if (url != null) {
+			InputStream in = null;
 			try {
-				return url.openStream();
+				in = url.openStream();
+				result.load(new BufferedInputStream(in));
 			} catch (IOException e) {
 				// TODO consider logging ...
+			} finally {
+				if (in != null)
+					try {
+						in.close();
+					} catch (IOException ee) {
+						// do nothing
+					}
 			}
-		return null;
+		}
+		// set the profile name if it does not provide one
+		if (result.getProperty(Constants.OSGI_JAVA_PROFILE_NAME) == null)
+			if (vmProfile != null)
+				result.put(Constants.OSGI_JAVA_PROFILE_NAME, vmProfile.replace('_', '/'));
+			else
+				// last resort; default to the absolute minimum profile name
+				result.put(Constants.OSGI_JAVA_PROFILE_NAME, "OSGi/Minimum-1.0"); //$NON-NLS-1$
+		return result;
 	}
 
 	/**
@@ -475,6 +473,8 @@ public class Framework implements EventDispatcher, EventPublisher {
 		condPermAdmin = null;
 		packageAdmin = null;
 		adaptor = null;
+		uninstallURLStreamHandlerFactory();
+		uninstallContentHandlerFactory();
 	}
 
 	/**
@@ -523,6 +523,7 @@ public class Framework implements EventDispatcher, EventPublisher {
 		 * this must be done first according to section 4.19.2 from the OSGi R3 spec.  
 		 */
 		systemBundle.state = AbstractBundle.STOPPING;
+		publishBundleEvent(BundleEvent.STOPPING, systemBundle); // need to send system bundle stopping event 
 		/* call the FrameworkAdaptor.frameworkStopping method first */
 		try {
 			adaptor.frameworkStopping(systemBundle.getContext());
@@ -549,7 +550,6 @@ public class Framework implements EventDispatcher, EventPublisher {
 	 * @param bundledata the BundleData of the Bundle to create
 	 */
 	AbstractBundle createAndVerifyBundle(BundleData bundledata) throws BundleException {
-		// TODO Verify the manifest... for example that the same package is imported twice 
 		// Check for a bundle already installed with the same symbolic name and version.
 		if (bundledata.getSymbolicName() != null) {
 			AbstractBundle installedBundle = getBundleBySymbolicName(bundledata.getSymbolicName(), bundledata.getVersion());
@@ -574,6 +574,8 @@ public class Framework implements EventDispatcher, EventPublisher {
 	 *             Environment.
 	 */
 	protected boolean verifyExecutionEnvironment(Dictionary manifest) throws BundleException {
+		if (!Boolean.valueOf(FrameworkProperties.getProperty(Constants.ECLIPSE_EE_INSTALL_VERIFY, Boolean.TRUE.toString())).booleanValue()) //$NON-NLS-1$
+			return true;
 		String headerValue = (String) manifest.get(Constants.BUNDLE_REQUIREDEXECUTIONENVIRONMENT);
 		/* If no required EE is in the manifest return true */
 		if (headerValue == null) {
@@ -583,7 +585,7 @@ public class Framework implements EventDispatcher, EventPublisher {
 		if (bundleRequiredEE.length == 0) {
 			return true;
 		}
-		String systemEE = System.getProperty(Constants.FRAMEWORK_EXECUTIONENVIRONMENT);
+		String systemEE = FrameworkProperties.getProperty(Constants.FRAMEWORK_EXECUTIONENVIRONMENT);
 		if (systemEE != null && !systemEE.equals("")) { //$NON-NLS-1$
 			ManifestElement[] systemEEs = ManifestElement.parseHeader(Constants.BUNDLE_REQUIREDEXECUTIONENVIRONMENT, systemEE);
 			for (int i = 0; i < systemEEs.length; i++) {
@@ -820,6 +822,11 @@ public class Framework implements EventDispatcher, EventPublisher {
 		try {
 			BundleData bundledata = storage.begin();
 			bundle = createAndVerifyBundle(bundledata);
+			if (Debug.DEBUG) {
+				BundleWatcher bundleStats = adaptor.getBundleWatcher();
+				if (bundleStats != null)
+					bundleStats.watchBundle(bundle, BundleWatcher.START_INSTALLING);
+			}
 			try {
 				// Select the native code paths for the bundle;
 				// this is not done by the adaptor because this
@@ -855,6 +862,12 @@ public class Framework implements EventDispatcher, EventPublisher {
 				}
 				bundle.close();
 				throw error;
+			} finally {
+				if (Debug.DEBUG) {
+					BundleWatcher bundleStats = adaptor.getBundleWatcher();
+					if (bundleStats != null)
+						bundleStats.watchBundle(bundle, BundleWatcher.END_INSTALLING);
+				}
 			}
 			/* bundle has been successfully installed */
 			bundles.add(bundle);
@@ -1255,21 +1268,21 @@ public class Framework implements EventDispatcher, EventPublisher {
 
 	// gets AdminPermission objects from a cache to reduce the number of AdminPermission
 	// objects that are created.
-	private synchronized AdminPermission getAdminPermission(Bundle bundle, String action) {
-		if (adminPermissions == null)
-			adminPermissions = new HashMap();
-		Long ID = new Long(bundle.getBundleId());
-		HashMap bundlePermissions = (HashMap) adminPermissions.get(ID);
-		if (bundlePermissions == null) {
-			bundlePermissions = new HashMap();
-			adminPermissions.put(ID, bundlePermissions);
+	private AdminPermission getAdminPermission(Bundle bundle, String action) {
+		synchronized(adminPermissions) {
+			Long ID = new Long(bundle.getBundleId());
+			HashMap bundlePermissions = (HashMap) adminPermissions.get(ID);
+			if (bundlePermissions == null) {
+				bundlePermissions = new HashMap();
+				adminPermissions.put(ID, bundlePermissions);
+			}
+			AdminPermission result = (AdminPermission) bundlePermissions.get(action);
+			if (result == null) {
+				result = new AdminPermission(bundle, action);
+				bundlePermissions.put(action, result);
+			}
+			return result;
 		}
-		AdminPermission result = (AdminPermission) bundlePermissions.get(action);
-		if (result == null) {
-			result = new AdminPermission(bundle, action);
-			bundlePermissions.put(action, result);
-		}
-		return result;
 	}
 
 	/**
@@ -1575,4 +1588,199 @@ public class Framework implements EventDispatcher, EventPublisher {
 		}
 		throw new BundleException(Msg.BUNDLE_NATIVECODE_MATCH_EXCEPTION);
 	}
+
+	private static Field getStaticField(Class clazz, Class type) {
+		Field[] fields = clazz.getDeclaredFields();
+		for (int i = 0; i < fields.length; i++)
+			if (Modifier.isStatic(fields[i].getModifiers()) && fields[i].getType().equals(type)) {
+				fields[i].setAccessible(true);
+				return fields[i];
+			}
+		return null;
+	}
+
+	private void installContentHandlerFactory(BundleContext context, FrameworkAdaptor frameworkAdaptor) {
+		ContentHandlerFactory chf = new ContentHandlerFactory(context, frameworkAdaptor);
+		try {
+			// first try the standard way
+			URLConnection.setContentHandlerFactory(chf);
+		} catch (Error err) {
+			// ok we failed now use more drastic means to set the factory
+			try {
+				forceContentHandlerFactory(chf);
+			} catch (Exception ex) {
+				// this is unexpected, log the exception and throw the original error
+				adaptor.getFrameworkLog().log(new FrameworkEvent(FrameworkEvent.ERROR, context.getBundle(), ex));
+				throw err;
+			}
+		}
+		contentHandlerFactory = chf;
+	}
+
+	private static void forceContentHandlerFactory(ContentHandlerFactory chf) throws Exception{
+		Field factoryField = getStaticField(URLConnection.class, java.net.ContentHandlerFactory.class);
+		if (factoryField == null)
+			throw new Exception("Could not find ContentHandlerFactory field"); //$NON-NLS-1$
+		synchronized(URLConnection.class) {
+			java.net.ContentHandlerFactory factory = (java.net.ContentHandlerFactory) factoryField.get(null);
+			// doing a null check here just in case, but it would be really strange if it was null, 
+			// because we failed to set the factory normally!!
+			
+			if (factory != null) {
+				try {
+					factory.getClass().getMethod("isMultiplexing", null); //$NON-NLS-1$
+					Method register = factory.getClass().getMethod("register", new Class[] {Object.class}); //$NON-NLS-1$
+					register.invoke(factory, new Object[] {chf});
+				} catch (NoSuchMethodException e) {
+					// current factory does not support multiplexing, ok we'll wrap it
+					chf.setParentFactory(factory);
+					factory = chf;
+				}
+			}
+			// null out the field so that we can successfully call setContentHandlerFactory			
+			factoryField.set(null, null);
+			// always attempt to clear the handlers cache
+			// This allows an optomization for the single framework use-case
+			resetContentHandlers();					
+			URLConnection.setContentHandlerFactory(factory);
+		}
+	}
+
+	private void uninstallContentHandlerFactory() {
+		try {
+			Field factoryField = getStaticField(URLConnection.class, java.net.ContentHandlerFactory.class);
+			if (factoryField == null)
+				return;  // oh well, we tried.
+			synchronized(URLConnection.class) {
+				java.net.ContentHandlerFactory factory = (java.net.ContentHandlerFactory) factoryField.get(null);
+				
+				if (factory == contentHandlerFactory) {
+					factory = (java.net.ContentHandlerFactory) contentHandlerFactory.designateSuccessor();
+				} else {
+					Method unregister = factory.getClass().getMethod("unregister", new Class[] {Object.class}); //$NON-NLS-1$
+					unregister.invoke(factory, new Object[] {contentHandlerFactory});
+				}
+				// null out the field so that we can successfully call setContentHandlerFactory									
+				factoryField.set(null, null);
+				// always attempt to clear the handlers cache
+				// This allows an optomization for the single framework use-case
+				// Note that the call to setContentHandlerFactory below may clear this cache
+				// but we want to be sure to clear it here just incase the parent is null.
+				// In this case the call below would not occur.
+				// Also it appears most java libraries actually do not clear the cache
+				// when setContentHandlerFactory is called, go figure!!
+				resetContentHandlers();
+				if (factory != null) 
+					URLConnection.setContentHandlerFactory(factory);
+			}
+		} catch (Exception e) {
+			// ignore and continue closing the framework
+		}		
+	}
+
+	private static void resetContentHandlers() throws IllegalAccessException {
+		Field handlersField = getStaticField(URLConnection.class, Hashtable.class);
+		if (handlersField != null) {
+			Hashtable handlers = (Hashtable) handlersField.get(null);
+			if (handlers != null)
+				handlers.clear();
+		}
+	}
+
+	private void installURLStreamHandlerFactory(BundleContext context, FrameworkAdaptor frameworkAdaptor) {
+		StreamHandlerFactory shf = new StreamHandlerFactory(context, frameworkAdaptor);
+		try {
+			// first try the standard way
+			URL.setURLStreamHandlerFactory(shf);
+		} catch (Error err) {
+			try {
+				// ok we failed now use more drastic means to set the factory
+				forceURLStreamHandlerFactory(shf);
+			} catch (Exception ex) {
+				adaptor.getFrameworkLog().log(new FrameworkEvent(FrameworkEvent.ERROR, context.getBundle(), ex));
+				throw err;
+			}
+		}
+		streamHandlerFactory = shf;
+	}
+
+	private static void forceURLStreamHandlerFactory(StreamHandlerFactory shf) throws Exception{
+		Field factoryField = getStaticField(URL.class, URLStreamHandlerFactory.class);
+		if (factoryField == null)
+			throw new Exception("Could not find URLStreamHandlerFactory field"); //$NON-NLS-1$
+		// look for a lock to synchronize on
+		Object lock = getURLStreamHandlerFactoryLock();
+		synchronized(lock) {
+			URLStreamHandlerFactory factory = (URLStreamHandlerFactory) factoryField.get(null);
+			// doing a null check here just in case, but it would be really strange if it was null, 
+			// because we failed to set the factory normally!!
+			if (factory != null) {
+				try {
+					factory.getClass().getMethod("isMultiplexing", null); //$NON-NLS-1$
+					Method register = factory.getClass().getMethod("register", new Class[] {Object.class}); //$NON-NLS-1$
+					register.invoke(factory, new Object[] {shf});
+				} catch (NoSuchMethodException e) {
+					// current factory does not support multiplexing, ok we'll wrap it
+					shf.setParentFactory(factory);
+					factory = shf;
+				}
+			}
+			factoryField.set(null, null);
+			// always attempt to clear the handlers cache
+			// This allows an optomization for the single framework use-case
+			resetURLStreamHandlers();
+			URL.setURLStreamHandlerFactory(factory);
+		}
+	}
+
+	private void uninstallURLStreamHandlerFactory() {
+		try {
+			Field factoryField = getStaticField(URL.class, URLStreamHandlerFactory.class);
+			if (factoryField == null)
+				return; // oh well, we tried
+			Object lock = getURLStreamHandlerFactoryLock();
+			synchronized (lock) {
+				URLStreamHandlerFactory factory = (URLStreamHandlerFactory) factoryField.get(null);
+				if (factory == streamHandlerFactory) {
+					factory = (URLStreamHandlerFactory) streamHandlerFactory.designateSuccessor();
+				} else {
+					Method unregister = factory.getClass().getMethod("unregister", new Class[] {Object.class}); //$NON-NLS-1$
+					unregister.invoke(factory, new Object[] {streamHandlerFactory});
+				}
+				factoryField.set(null, null);
+				// always attempt to clear the handlers cache
+				// This allows an optomization for the single framework use-case
+				// Note that the call to setURLStreamHandlerFactory below may clear this cache
+				// but we want to be sure to clear it here just in case the parent is null.
+				// In this case the call below would not occur.
+				resetURLStreamHandlers();
+				if (factory != null) 
+					URL.setURLStreamHandlerFactory(factory);
+			}
+		} catch (Exception e) {
+			// ignore and continue closing the framework
+		}
+	}
+
+	private static Object getURLStreamHandlerFactoryLock() throws IllegalAccessException {
+		Object lock;
+		try {
+			Field streamHandlerLockField = URL.class.getDeclaredField("streamHandlerLock"); //$NON-NLS-1$
+			streamHandlerLockField.setAccessible(true);
+			lock = streamHandlerLockField.get(null);
+		} catch (NoSuchFieldException noField) {
+			// could not find the lock, lets sync on the class object
+			lock = URL.class;
+		}
+		return lock;
+	}
+	
+	private static void resetURLStreamHandlers() throws IllegalAccessException {
+		Field handlersField = getStaticField(URL.class, Hashtable.class);
+		if (handlersField != null) {
+			Hashtable handlers = (Hashtable) handlersField.get(null);
+			if (handlers != null)
+				handlers.clear();
+		}
+	}
 }
