@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,22 +10,50 @@
  *******************************************************************************/
 package org.eclipse.pde.internal.core;
 
-import java.io.*;
+import java.io.IOException;
+import java.io.InputStream;
 import java.net.MalformedURLException;
 import java.net.URL;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.Iterator;
+import java.util.Locale;
+import java.util.Map;
+import java.util.Vector;
 
-import org.eclipse.core.resources.*;
-import org.eclipse.core.runtime.*;
-import org.eclipse.jdt.core.*;
-import org.eclipse.pde.core.*;
-import org.eclipse.pde.core.plugin.*;
-import org.eclipse.pde.internal.core.bundle.*;
-import org.eclipse.pde.internal.core.feature.*;
-import org.eclipse.pde.internal.core.ibundle.*;
-import org.eclipse.pde.internal.core.ifeature.*;
-import org.eclipse.pde.internal.core.plugin.*;
-import org.eclipse.team.core.*;
+import org.eclipse.core.resources.IFile;
+import org.eclipse.core.resources.IFolder;
+import org.eclipse.core.resources.IProject;
+import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.IResourceChangeEvent;
+import org.eclipse.core.resources.IResourceChangeListener;
+import org.eclipse.core.resources.IResourceDelta;
+import org.eclipse.core.resources.IResourceDeltaVisitor;
+import org.eclipse.core.resources.IWorkspace;
+import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.IPath;
+import org.eclipse.core.runtime.Path;
+import org.eclipse.jdt.core.JavaCore;
+import org.eclipse.pde.core.IModel;
+import org.eclipse.pde.core.IModelProviderEvent;
+import org.eclipse.pde.core.IModelProviderListener;
+import org.eclipse.pde.core.plugin.IFragmentModel;
+import org.eclipse.pde.core.plugin.IPluginModel;
+import org.eclipse.pde.core.plugin.IPluginModelBase;
+import org.eclipse.pde.core.plugin.ISharedExtensionsModel;
+import org.eclipse.pde.internal.core.bundle.BundleFragmentModel;
+import org.eclipse.pde.internal.core.bundle.BundlePluginModel;
+import org.eclipse.pde.internal.core.bundle.WorkspaceBundleModel;
+import org.eclipse.pde.internal.core.feature.WorkspaceFeatureModel;
+import org.eclipse.pde.internal.core.ibundle.IBundlePluginModel;
+import org.eclipse.pde.internal.core.ibundle.IBundlePluginModelBase;
+import org.eclipse.pde.internal.core.ifeature.IFeatureModel;
+import org.eclipse.pde.internal.core.plugin.WorkspaceExtensionsModel;
+import org.eclipse.pde.internal.core.plugin.WorkspaceFragmentModel;
+import org.eclipse.pde.internal.core.plugin.WorkspacePluginModel;
+import org.eclipse.pde.internal.core.plugin.WorkspacePluginModelBase;
+import org.eclipse.team.core.RepositoryProvider;
 
 public class WorkspaceModelManager
 		implements
@@ -93,37 +121,28 @@ public class WorkspaceModelManager
 	}
 	
 	public static boolean isBinaryFeatureProject(IProject project) {
-		if (isFeatureProject(project)){
-			try {
-				String binary = project.getPersistentProperty(PDECore.EXTERNAL_PROJECT_PROPERTY);
-				if (binary != null) {
-					RepositoryProvider provider = RepositoryProvider.getProvider(project);
-					return provider==null || provider instanceof BinaryRepositoryProvider;
-				}
-			} catch (CoreException e) {
-				PDECore.logException(e);
-			}
-		}
-		return false;
+		return isFeatureProject(project) && isBinaryProject(project);
 	}
 	
 	public static boolean isBinaryPluginProject(IProject project) {
-		if (isPluginProject(project)){
-			try {
-				String binary = project.getPersistentProperty(PDECore.EXTERNAL_PROJECT_PROPERTY);
-				if (binary != null) {
-					RepositoryProvider provider = RepositoryProvider.getProvider(project);
-					return provider==null || provider instanceof BinaryRepositoryProvider;
-				}
-			} catch (CoreException e) {
-				PDECore.logException(e);
+		return isPluginProject(project) && isBinaryProject(project);
+	}
+
+	public static boolean isBinaryProject(IProject project) {
+		try {
+			String binary = project.getPersistentProperty(PDECore.EXTERNAL_PROJECT_PROPERTY);
+			if (binary != null) {
+				RepositoryProvider provider = RepositoryProvider.getProvider(project);
+				return provider==null || provider instanceof BinaryRepositoryProvider;
 			}
+		} catch (CoreException e) {
+			PDECore.logException(e);
 		}
 		return false;
 	}
-	
+
 	public static boolean isNonBinaryPluginProject(IProject project) {
-		return isPluginProject(project) && !isBinaryPluginProject(project);
+		return isPluginProject(project) && !isBinaryProject(project);
 	}
 	
 	public static boolean isJavaPluginProject(IProject project) {
@@ -296,7 +315,13 @@ public class WorkspaceModelManager
 			if (model instanceof WorkspacePluginModelBase && filename.equals("MANIFEST.MF")) { //$NON-NLS-1$
 				addWorkspaceModel(file.getProject(), true);
 			} else if (model instanceof IBundlePluginModelBase && (filename.equals("plugin.xml") || filename.equals("fragment.xml"))) { //$NON-NLS-1$ //$NON-NLS-2$
-				addWorkspaceModel(file.getProject(), false);
+				WorkspaceExtensionsModel extensions = new WorkspaceExtensionsModel(file);
+				((IBundlePluginModelBase)model).setExtensionsModel(extensions);
+				extensions.setBundleModel((IBundlePluginModelBase)model);
+				loadModel(extensions, false);
+				// no need to fire any notifications if the plugin.xml/fragment.xml
+				// of a bundle is modified.
+				return;
 			}
 		} else if ((IResourceDelta.CONTENT & delta.getFlags()) != 0) {
 			if (model instanceof IBundlePluginModelBase) {
@@ -308,6 +333,7 @@ public class WorkspaceModelManager
 					if (extensions == null) {
 						extensions = new WorkspaceExtensionsModel(file);
 						((IBundlePluginModelBase)model).setExtensionsModel(extensions);
+						((WorkspaceExtensionsModel)extensions).setBundleModel((IBundlePluginModelBase)model);
 					}
 					loadModel(extensions, reload);
 					// no need to fire any notifications if the plugin.xml/fragment.xml
@@ -374,7 +400,7 @@ public class WorkspaceModelManager
 				fFragmentModels.put(project, models[i]);
 			}
 		}
-				fFeatureModels = Collections.synchronizedMap(new HashMap());
+		fFeatureModels = Collections.synchronizedMap(new HashMap());
 		
 		IWorkspace workspace = PDECore.getWorkspace();
 		IProject[] projects = workspace.getRoot().getProjects();
@@ -386,7 +412,7 @@ public class WorkspaceModelManager
 		}
 
 		PDECore.getWorkspace().addResourceChangeListener(this, IResourceChangeEvent.PRE_CLOSE);
-		JavaCore.addPreProcessingResourceChangedListener(this);
+		JavaCore.addPreProcessingResourceChangedListener(this, IResourceChangeEvent.POST_CHANGE);
 		fInitialized = true;
 	}
 	
@@ -412,15 +438,16 @@ public class WorkspaceModelManager
 	/* (non-Javadoc)
 	 * @see org.eclipse.pde.core.IModelProvider#removeModelProviderListener(org.eclipse.pde.core.IModelProviderListener)
 	 */
-	public void removeModelProviderListener(IModelProviderListener listener) {
+	public synchronized void removeModelProviderListener(IModelProviderListener listener) {
 		fListeners.remove(listener);
 	}
 
 	/* (non-Javadoc)
 	 * @see org.eclipse.pde.core.IModelProvider#addModelProviderListener(org.eclipse.pde.core.IModelProviderListener)
 	 */
-	public void addModelProviderListener(IModelProviderListener listener) {
-		fListeners.add(listener);
+	public synchronized void addModelProviderListener(IModelProviderListener listener) {
+		if (!fListeners.contains(listener))
+			fListeners.add(listener);
 	}
 	
 	/* (non-Javadoc)
@@ -514,7 +541,7 @@ public class WorkspaceModelManager
 		}
 		
 		workspace.addResourceChangeListener(this, IResourceChangeEvent.PRE_CLOSE);
-		JavaCore.addPreProcessingResourceChangedListener(this);
+		JavaCore.addPreProcessingResourceChangedListener(this, IResourceChangeEvent.POST_CHANGE);
 		
 		fModelsLocked = false;
 		fInitialized = true;
@@ -722,7 +749,7 @@ public class WorkspaceModelManager
 		}
 	}
 	
-	private void fireModelProviderEvent(ModelProviderEvent event) {
+	private synchronized void fireModelProviderEvent(ModelProviderEvent event) {
 		for (Iterator iter = fListeners.iterator(); iter.hasNext();) {
 			((IModelProviderListener) iter.next()).modelsChanged(event);
 		}
