@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2004 IBM Corporation and others.
+ * Copyright (c) 2003, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -14,8 +14,16 @@ public interface IBundle {
 	
 	void setHeader(String key, String value);
 	
+	void renameHeader(String key, String newKey);
+	
 	String getHeader(String key);
+	
+	IManifestHeader getManifestHeader(String key);
     
     IBundleModel getModel();
     
+    String getLocalization();
+    
+    void setLocalization(String localization);
+    
 }
