@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -9,51 +9,56 @@
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.pde.internal.ui.editor.schema;
-import org.eclipse.pde.core.IModelChangedEvent;
-import org.eclipse.pde.internal.core.ischema.*;
-import org.eclipse.pde.internal.ui.PDEPlugin;
-import org.eclipse.pde.internal.ui.PDEUIMessages;
-import org.eclipse.pde.internal.ui.editor.*;
-import org.eclipse.swt.graphics.Image;
 
+import org.eclipse.jface.viewers.ILabelProvider;
+import org.eclipse.pde.core.IModelChangedEvent;
+import org.eclipse.pde.internal.core.ischema.IDocumentSection;
+import org.eclipse.pde.internal.core.ischema.ISchema;
+import org.eclipse.pde.internal.core.ischema.ISchemaAttribute;
+import org.eclipse.pde.internal.core.ischema.ISchemaComplexType;
+import org.eclipse.pde.internal.core.ischema.ISchemaElement;
+import org.eclipse.pde.internal.core.ischema.ISchemaObject;
+import org.eclipse.pde.internal.core.ischema.ISchemaType;
+import org.eclipse.pde.internal.ui.PDEUIMessages;
+import org.eclipse.pde.internal.ui.editor.FormOutlinePage;
+import org.eclipse.pde.internal.ui.editor.PDEFormEditor;
 
 public class SchemaFormOutlinePage extends FormOutlinePage {
-	private Object[] topics;
+	private Object[] fTopics;
+
 	public Object[] getChildren(Object parent) {
-		ISchema schema = (ISchema) editor.getAggregateModel();
-		if (schema.isValid()) {
+		ISchema schema = (ISchema) fEditor.getAggregateModel();
+		if (schema != null && schema.isValid()) {
 			if (parent instanceof SchemaFormPage) {
 				return getMarkup();
 			}
 			if (parent instanceof ISchemaElement) {
 				return getAttributes((ISchemaElement) parent);
 			}
-			if (parent instanceof SchemaDocPage) {
+			if (parent instanceof SchemaOverviewPage) {
 				return getTopics();
 			}
 		}
 		return super.getChildren(parent);
 	}
-	class OutlineLabelProvider extends BasicLabelProvider {
+	
+	class SchemaLabelProvider extends BasicLabelProvider {
 		public String getText(Object obj) {
 			String label = getObjectLabel(obj);
-			if (label != null)
-				return label;
-			return super.getText(obj);
-		}
-		public Image getImage(Object obj) {
-			Image image = PDEPlugin.getDefault().getLabelProvider().getImage(
-					obj);
-			if (image != null)
-				return image;
-			return super.getImage(obj);
+			return (label == null) ? super.getText(obj) : label;
 		}
 	}
+
 	public SchemaFormOutlinePage(PDEFormEditor editor) {
 		super(editor);
 	}
+	
+	protected ILabelProvider createLabelProvider() {
+		return new SchemaLabelProvider();
+	}
+
 	protected Object[] createTopics() {
-		ISchema schema = (ISchema) editor.getAggregateModel();
+		ISchema schema = (ISchema) fEditor.getAggregateModel();
 		IDocumentSection[] sections = schema.getDocumentSections();
 		Object[] result = new Object[sections.length + 1];
 		result[0] = schema;
@@ -62,18 +67,21 @@ public class SchemaFormOutlinePage extends FormOutlinePage {
 		}
 		return result;
 	}
-	Object[] getAttributes(ISchemaElement element) {
+
+	private Object[] getAttributes(ISchemaElement element) {
 		ISchemaType type = element.getType();
 		if (type instanceof ISchemaComplexType) {
 			return ((ISchemaComplexType) type).getAttributes();
 		}
 		return new Object[0];
 	}
-	Object[] getMarkup() {
-		ISchema schema = (ISchema) editor.getAggregateModel();
+
+	private Object[] getMarkup() {
+		ISchema schema = (ISchema) fEditor.getAggregateModel();
 		return schema.getElements();
 	}
-	String getObjectLabel(Object obj) {
+
+	protected String getObjectLabel(Object obj) {
 		if (obj instanceof ISchema) {
 			return PDEUIMessages.SchemaEditor_topic_overview;
 		}
@@ -93,28 +101,31 @@ public class SchemaFormOutlinePage extends FormOutlinePage {
 		}
 		return null;
 	}
+
 	Object[] getTopics() {
-		if (topics == null) {
-			topics = createTopics();
+		if (fTopics == null) {
+			fTopics = createTopics();
 		}
-		return topics;
+		return fTopics;
 	}
+
 	protected String getParentPageId(Object item) {
 		if (item instanceof ISchemaElement || item instanceof ISchemaAttribute)
 			return SchemaFormPage.PAGE_ID;
 		if (item instanceof IDocumentSection || item instanceof ISchema)
-			return SchemaDocPage.PAGE_ID;
+			return SchemaOverviewPage.PAGE_ID;
 		return super.getParentPageId(item);
 	}
+
 	public void modelChanged(IModelChangedEvent event) {
 		if (event.getChangeType() == IModelChangedEvent.WORLD_CHANGED) {
-			topics = null;
-			treeViewer.refresh();
+			fTopics = null;
+			fTreeViewer.refresh();
 			return;
 		}
 		Object object = event.getChangedObjects()[0];
 		if (event.getChangeType() == IModelChangedEvent.CHANGE) {
-			treeViewer.update(object, null);
+			fTreeViewer.update(object, null);
 		} else {
 			// find the parent
 			Object parent = null;
@@ -122,15 +133,11 @@ public class SchemaFormOutlinePage extends FormOutlinePage {
 				parent = ((ISchemaObject) object).getParent();
 			}
 			if (parent != null) {
-				if (parent instanceof ISchema) {
-					//parent =
-					// formPage.getEditor().getPage(SchemaEditor.DEFINITION_PAGE);
-				}
-				treeViewer.refresh(parent);
-				treeViewer.expandToLevel(parent, 2);
+				fTreeViewer.refresh(parent);
+				fTreeViewer.expandToLevel(parent, 2);
 			} else {
-				treeViewer.refresh();
-				treeViewer.expandAll();
+				fTreeViewer.refresh();
+				fTreeViewer.expandAll();
 			}
 		}
 	}
