@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -18,13 +18,14 @@ import org.eclipse.core.runtime.Preferences;
 import org.eclipse.jface.action.Action;
 import org.eclipse.jface.action.IToolBarManager;
 import org.eclipse.jface.action.Separator;
+import org.eclipse.jface.viewers.IStructuredSelection;
 import org.eclipse.osgi.util.NLS;
 import org.eclipse.pde.core.plugin.IPlugin;
 import org.eclipse.pde.core.plugin.IPluginModel;
 import org.eclipse.pde.core.plugin.IPluginModelBase;
-import org.eclipse.pde.internal.builders.DependencyLoop;
-import org.eclipse.pde.internal.builders.DependencyLoopFinder;
 import org.eclipse.pde.internal.core.PDECore;
+import org.eclipse.pde.internal.core.builders.DependencyLoop;
+import org.eclipse.pde.internal.core.builders.DependencyLoopFinder;
 import org.eclipse.pde.internal.ui.IHelpContextIds;
 import org.eclipse.pde.internal.ui.IPreferenceConstants;
 import org.eclipse.pde.internal.ui.PDEPlugin;
@@ -91,9 +92,9 @@ public class DependenciesView extends PageBookView implements
 
 		public ShowLoopsAction() {
 			super("", AS_PUSH_BUTTON); //$NON-NLS-1$
-			setText(PDEUIMessages.DependenciesView_ShowLoopsAction_label); //$NON-NLS-1$
-			setDescription(PDEUIMessages.DependenciesView_ShowLoopsAction_description); //$NON-NLS-1$
-			setToolTipText(PDEUIMessages.DependenciesView_ShowLoopsAction_tooltip); //$NON-NLS-1$
+			setText(PDEUIMessages.DependenciesView_ShowLoopsAction_label); 
+			setDescription(PDEUIMessages.DependenciesView_ShowLoopsAction_description); 
+			setToolTipText(PDEUIMessages.DependenciesView_ShowLoopsAction_tooltip); 
 			setImageDescriptor(PDEPluginImages.DESC_DEP_LOOP);
 			setDisabledImageDescriptor(PDEPluginImages.DESC_DEP_LOOP_DISABLED);
 			setEnabled(false);
@@ -113,9 +114,9 @@ public class DependenciesView extends PageBookView implements
 
 		public ShowCalleesAction() {
 			super("", AS_RADIO_BUTTON); //$NON-NLS-1$
-			setText(PDEUIMessages.DependenciesView_ShowCalleesAction_label); //$NON-NLS-1$
-			setDescription(PDEUIMessages.DependenciesView_ShowCalleesAction_description); //$NON-NLS-1$
-			setToolTipText(PDEUIMessages.DependenciesView_ShowCalleesAction_tooltip); //$NON-NLS-1$
+			setText(PDEUIMessages.DependenciesView_ShowCalleesAction_label); 
+			setDescription(PDEUIMessages.DependenciesView_ShowCalleesAction_description); 
+			setToolTipText(PDEUIMessages.DependenciesView_ShowCalleesAction_tooltip); 
 			setImageDescriptor(PDEPluginImages.DESC_CALLEES);
 			setDisabledImageDescriptor(PDEPluginImages.DESC_CALLEES_DISABLED);
 		}
@@ -134,9 +135,9 @@ public class DependenciesView extends PageBookView implements
 	class ShowCallersAction extends Action {
 		public ShowCallersAction() {
 			super("", AS_RADIO_BUTTON); //$NON-NLS-1$
-			setText(PDEUIMessages.DependenciesView_ShowCallersAction_label); //$NON-NLS-1$
-			setDescription(PDEUIMessages.DependenciesView_ShowCallersAction_description); //$NON-NLS-1$
-			setToolTipText(PDEUIMessages.DependenciesView_ShowCallersAction_tooltip); //$NON-NLS-1$
+			setText(PDEUIMessages.DependenciesView_ShowCallersAction_label); 
+			setDescription(PDEUIMessages.DependenciesView_ShowCallersAction_description); 
+			setToolTipText(PDEUIMessages.DependenciesView_ShowCallersAction_tooltip); 
 			setImageDescriptor(PDEPluginImages.DESC_CALLERS);
 			setDisabledImageDescriptor(PDEPluginImages.DESC_CALLERS_DISABLED);
 		}
@@ -155,9 +156,9 @@ public class DependenciesView extends PageBookView implements
 	class ShowListAction extends Action {
 		public ShowListAction() {
 			super("", AS_RADIO_BUTTON); //$NON-NLS-1$
-			setText(PDEUIMessages.DependenciesView_ShowListAction_label); //$NON-NLS-1$
-			setDescription(PDEUIMessages.DependenciesView_ShowListAction_description); //$NON-NLS-1$
-			setToolTipText(PDEUIMessages.DependenciesView_ShowListAction_tooltip); //$NON-NLS-1$
+			setText(PDEUIMessages.DependenciesView_ShowListAction_label); 
+			setDescription(PDEUIMessages.DependenciesView_ShowListAction_description); 
+			setToolTipText(PDEUIMessages.DependenciesView_ShowListAction_tooltip); 
 			setImageDescriptor(PDEPluginImages.DESC_FLAT_LAYOUT);
 			setDisabledImageDescriptor(PDEPluginImages.DESC_FLAT_LAYOUT_DISABLED);
 		}
@@ -177,9 +178,9 @@ public class DependenciesView extends PageBookView implements
 
 		public ShowTreeAction() {
 			super("", AS_RADIO_BUTTON); //$NON-NLS-1$
-			setText(PDEUIMessages.DependenciesView_ShowTreeAction_label); //$NON-NLS-1$
-			setDescription(PDEUIMessages.DependenciesView_ShowTreeAction_description); //$NON-NLS-1$
-			setToolTipText(PDEUIMessages.DependenciesView_ShowTreeAction_tooltip); //$NON-NLS-1$
+			setText(PDEUIMessages.DependenciesView_ShowTreeAction_label); 
+			setDescription(PDEUIMessages.DependenciesView_ShowTreeAction_description); 
+			setToolTipText(PDEUIMessages.DependenciesView_ShowTreeAction_tooltip); 
 			setImageDescriptor(PDEPluginImages.DESC_HIERARCHICAL_LAYOUT);
 			setDisabledImageDescriptor(PDEPluginImages.DESC_HIERARCHICAL_LAYOUT_DISABLED);
 		}
@@ -522,10 +523,15 @@ public class DependenciesView extends PageBookView implements
 	 * @see org.eclipse.ui.part.PageBookView#showPageRec(org.eclipse.ui.part.PageBookView.PageRec)
 	 */
 	protected void showPageRec(PageRec pageRec) {
+		IPage currPage = getCurrentPage();
+		IStructuredSelection selection = null;
+		if (currPage instanceof DependenciesViewPage)
+			selection = ((DependenciesViewPage)currPage).getSelection();
 		IPage p = pageRec.page;
 		((DependenciesViewPage) p).setInput(fInput);
 		super.showPageRec(pageRec);
 		updateTitle(fInput);
+		((DependenciesViewPage) p).setSelection(selection);
 	}
 
 	void updateTitle(Object newInput) {
@@ -536,18 +542,18 @@ public class DependenciesView extends PageBookView implements
 					newInput);
 			String title;
 			if (getCurrentContributingPart() == PART_CALLEES_TREE) {
-				title = NLS.bind(PDEUIMessages.DependenciesView_callees_tree_title, name); //$NON-NLS-1$
+				title = NLS.bind(PDEUIMessages.DependenciesView_callees_tree_title, name); 
 			} else if (getCurrentContributingPart() == PART_CALLEES_LIST) {
-				title = NLS.bind(PDEUIMessages.DependenciesView_callees_list_title, name); //$NON-NLS-1$
+				title = NLS.bind(PDEUIMessages.DependenciesView_callees_list_title, name); 
 			} else if (getCurrentContributingPart() == PART_CALLERS_TREE) {
-				title = NLS.bind(PDEUIMessages.DependenciesView_callers_tree_title, name); //$NON-NLS-1$
+				title = NLS.bind(PDEUIMessages.DependenciesView_callers_tree_title, name); 
 			} else {
-				title = NLS.bind(PDEUIMessages.DependenciesView_callers_list_title, name); //$NON-NLS-1$
+				title = NLS.bind(PDEUIMessages.DependenciesView_callers_list_title, name); 
 			}
 			if(fLoops != NO_LOOPS){
 				title = title + " " + PDEUIMessages.DependenciesView_cycles_title; //$NON-NLS-1$
 			}
-			setContentDescription(title); //$NON-NLS-1$
+			setContentDescription(title); 
 		}
 		setTitleToolTip(getTitle());
 	}
