@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -26,7 +26,6 @@ import org.eclipse.jface.viewers.LabelProvider;
 import org.eclipse.jface.viewers.TableViewer;
 import org.eclipse.jface.viewers.ViewerSorter;
 import org.eclipse.jface.wizard.WizardPage;
-import org.eclipse.pde.internal.core.ClasspathUtilCore;
 import org.eclipse.pde.internal.core.ModelEntry;
 import org.eclipse.pde.internal.core.PDECore;
 import org.eclipse.pde.internal.core.PluginModelManager;
@@ -34,6 +33,7 @@ import org.eclipse.pde.internal.ui.IHelpContextIds;
 import org.eclipse.pde.internal.ui.PDEPluginImages;
 import org.eclipse.pde.internal.ui.PDEUIMessages;
 import org.eclipse.pde.internal.ui.elements.DefaultContentProvider;
+import org.eclipse.pde.internal.ui.wizards.plugin.ClasspathComputer;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.layout.GridData;
@@ -111,8 +111,8 @@ public class RequiredPluginsContainerPage
 	 */
 	public RequiredPluginsContainerPage() {
 		super("requiredPluginsContainerPage"); //$NON-NLS-1$
-		setTitle(PDEUIMessages.RequiredPluginsContainerPage_title); //$NON-NLS-1$
-		setDescription(PDEUIMessages.RequiredPluginsContainerPage_desc); //$NON-NLS-1$
+		setTitle(PDEUIMessages.RequiredPluginsContainerPage_title); 
+		setDescription(PDEUIMessages.RequiredPluginsContainerPage_desc); 
 		projectImage =
 			PlatformUI.getWorkbench().getSharedImages().getImage(
 				IDE.SharedImages.IMG_OBJ_PROJECT);
@@ -134,7 +134,7 @@ public class RequiredPluginsContainerPage
 		Composite container = new Composite(parent, SWT.NULL);
 		container.setLayout(new GridLayout());
 		Label label = new Label(container, SWT.NULL);
-		label.setText(PDEUIMessages.RequiredPluginsContainerPage_label); //$NON-NLS-1$
+		label.setText(PDEUIMessages.RequiredPluginsContainerPage_label); 
 		viewer =
 			new TableViewer(
 				container,
@@ -200,11 +200,11 @@ public class RequiredPluginsContainerPage
 		}
 
 		if (entry == null) {
-			entry = ClasspathUtilCore.createContainerEntry();
+			entry = ClasspathComputer.createContainerEntry();
 			PluginModelManager mng = PDECore.getDefault().getModelManager();
 			ModelEntry entry = mng.findEntry(javaProject.getProject());
 			if (entry!=null) {
-				IClasspathContainer container = entry.getClasspathContainer();
+				IClasspathContainer container = entry.getClasspathContainer(false);
 				if (container!=null)
 					realEntries = container.getClasspathEntries();
 			}
