@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -15,17 +15,16 @@ import javax.xml.parsers.FactoryConfigurationError;
 import javax.xml.parsers.ParserConfigurationException;
 
 import org.eclipse.pde.core.plugin.IPluginModelBase;
+import org.eclipse.pde.internal.core.exports.FeatureExportInfo;
 import org.eclipse.pde.internal.ui.PDEPluginImages;
-import org.eclipse.pde.internal.ui.build.FeatureExportInfo;
 import org.eclipse.pde.internal.ui.build.PluginExportJob;
 import org.eclipse.ui.progress.IProgressConstants;
 import org.w3c.dom.DOMException;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 
-public class PluginExportWizard extends BaseExportWizard {
+public class PluginExportWizard extends AntGeneratingExportWizard {
 	private static final String STORE_SECTION = "PluginExportWizard"; //$NON-NLS-1$
-	private AdvancedPluginExportPage fPage2;
 
 	public PluginExportWizard() {
 		setDefaultPageImageDescriptor(PDEPluginImages.DESC_PLUGIN_EXPORT_WIZ);
@@ -39,23 +38,15 @@ public class PluginExportWizard extends BaseExportWizard {
 		return STORE_SECTION;
 	}
 	
-	public void addPages() {
-		super.addPages();
-		fPage2 = new AdvancedPluginExportPage("plugin-sign"); //$NON-NLS-1$
-		addPage(fPage2);
-	}
-	
 	protected void scheduleExportJob() {
 		FeatureExportInfo info = new FeatureExportInfo();
-		info.toDirectory = fPage1.doExportToDirectory();
-		info.useJarFormat = fPage1.useJARFormat();
-		info.exportSource = fPage1.doExportSource();
-		info.destinationDirectory = fPage1.getDestination();
-		info.zipFileName = fPage1.getFileName();
-		info.javacSource = fPage1.getJavacSource();
-		info.javacTarget = fPage1.getJavacTarget();
-		info.items = ((ExportWizardPageWithTable)fPage1).getSelectedItems();
-		info.signingInfo = fPage1.useJARFormat() ? fPage2.getSigningInfo() : null;
+		info.toDirectory = fPage.doExportToDirectory();
+		info.useJarFormat = fPage.useJARFormat();
+		info.exportSource = fPage.doExportSource();
+		info.destinationDirectory = fPage.getDestination();
+		info.zipFileName = fPage.getFileName();
+		info.items = fPage.getSelectedItems();
+		info.signingInfo = fPage.useJARFormat() ? fPage.getSigningInfo() : null;
 		
 		PluginExportJob job = new PluginExportJob(info);
 		job.setUser(true);
@@ -78,15 +69,13 @@ public class PluginExportWizard extends BaseExportWizard {
 			
 			Element export = doc.createElement("pde.exportPlugins"); //$NON-NLS-1$
 			export.setAttribute("plugins", getPluginIDs()); //$NON-NLS-1$
-			export.setAttribute("destination", fPage1.getDestination()); //$NON-NLS-1$
-			String filename = fPage1.getFileName();
+			export.setAttribute("destination", fPage.getDestination()); //$NON-NLS-1$
+			String filename = fPage.getFileName();
 			if (filename != null)
 				export.setAttribute("filename", filename); //$NON-NLS-1$
 			export.setAttribute("exportType", getExportOperation());  //$NON-NLS-1$
-			export.setAttribute("useJARFormat", Boolean.toString(fPage1.useJARFormat()));  //$NON-NLS-1$
-			export.setAttribute("exportSource", Boolean.toString(fPage1.doExportSource()));  //$NON-NLS-1$
-			export.setAttribute("source", fPage1.getJavacSource()); //$NON-NLS-1$
-			export.setAttribute("target", fPage1.getJavacTarget()); //$NON-NLS-1$
+			export.setAttribute("useJARFormat", Boolean.toString(fPage.useJARFormat()));  //$NON-NLS-1$
+			export.setAttribute("exportSource", Boolean.toString(fPage.doExportSource()));  //$NON-NLS-1$
 			target.appendChild(export);
 			return doc;
 		} catch (DOMException e) {
@@ -98,7 +87,7 @@ public class PluginExportWizard extends BaseExportWizard {
 	
 	private String getPluginIDs() {
 		StringBuffer buffer = new StringBuffer();
-		Object[] objects = ((ExportWizardPageWithTable)fPage1).getSelectedItems();
+		Object[] objects = fPage.getSelectedItems();
 		for (int i = 0; i < objects.length; i++) {
 			Object object = objects[i];
 			if (object instanceof IPluginModelBase) {
