@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005 IBM Corporation and others.
+ * Copyright (c) 2005, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -22,7 +22,7 @@ import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.jface.viewers.ISelection;
 import org.eclipse.jface.viewers.StructuredSelection;
-import org.eclipse.pde.internal.PDE;
+import org.eclipse.pde.internal.core.natures.PDE;
 import org.eclipse.pde.internal.core.site.WorkspaceSiteModel;
 import org.eclipse.pde.internal.core.util.CoreUtility;
 import org.eclipse.pde.internal.ui.IPDEUIConstants;
@@ -59,7 +59,7 @@ public class NewSiteProjectCreationOperation extends WorkspaceModifyOperation {
 			InvocationTargetException, InterruptedException {
 		int numUnits = fWebLocation == null ? 3 : 4;
 		
-		monitor.beginTask(PDEUIMessages.NewSiteWizard_creatingProject, numUnits); //$NON-NLS-1$
+		monitor.beginTask(PDEUIMessages.NewSiteWizard_creatingProject, numUnits); 
 
 		CoreUtility.createProject(fProject, fPath, monitor);		
 		fProject.open(monitor);
@@ -74,7 +74,7 @@ public class NewSiteProjectCreationOperation extends WorkspaceModifyOperation {
 			monitor.worked(1);
 		}
 	
-		monitor.subTask(PDEUIMessages.NewSiteWizard_creatingManifest); //$NON-NLS-1$
+		monitor.subTask(PDEUIMessages.NewSiteWizard_creatingManifest); 
 		IFile file = createSiteManifest();
 		monitor.worked(1);
 		
@@ -143,7 +143,7 @@ public class NewSiteProjectCreationOperation extends WorkspaceModifyOperation {
 		writer.println("				xmlFile.load(\"site.xml\");"); //$NON-NLS-1$
 		writer.println("				stylesheet.load(\""+fWebLocation+"/site.xsl\");"); //$NON-NLS-1$ //$NON-NLS-2$
 		writer.println("			} else {"); //$NON-NLS-1$
-		writer.println("				alert(\"" + PDEUIMessages.SiteHTML_loadError + "\");"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
+		writer.println("				alert(\"" + PDEUIMessages.SiteHTML_loadError + "\");"); //$NON-NLS-1$ //$NON-NLS-2$ 
 		writer.println("			}"); //$NON-NLS-1$
 		writer.println("			xmlFile.addEventListener(\"load\", transform, false);"); //$NON-NLS-1$
 		writer.println("			stylesheet.addEventListener(\"load\", transform, false);"); //$NON-NLS-1$
