@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -80,11 +80,15 @@ public class Main {
     private boolean initialize = false;
     private Process showProcess = null;
     private boolean splashDown = false;
-    private final Thread endSplashHandler = new Thread() {
-        public void run() {
-            takeDownSplash();
-        }
-    };
+    public final class EndSplashHandler extends Thread {
+    	public void run() {
+    		takeDownSplash();
+    	}
+    	public OutputStream getOutputStream() {
+			return showProcess == null ? null : showProcess.getOutputStream();
+		}
+    }
+    private final Thread endSplashHandler = new EndSplashHandler();
 
     // command line args
     private static final String FRAMEWORK = "-framework"; //$NON-NLS-1$
@@ -145,7 +149,6 @@ public class Main {
     private static final String PROP_LOGFILE = "osgi.logfile"; //$NON-NLS-1$
     private static final String PROP_REQUIRED_JAVA_VERSION = "osgi.requiredJavaVersion"; //$NON-NLS-1$
     private static final String PROP_PARENT_CLASSLOADER = "osgi.parentClassloader"; //$NON-NLS-1$
-    private static final String PROP_EOF = "eof"; //$NON-NLS-1$
 	private static final String PROP_NL = "osgi.nl";  //$NON-NLS-1$
     private static final String PROP_NOSHUTDOWN = "osgi.noShutdown"; //$NON-NLS-1$
     private static final String PROP_DEBUG = "osgi.debug"; //$NON-NLS-1$	
@@ -245,7 +248,6 @@ public class Main {
     /**
      * Executes the launch.
      * 
-     * @return the result of performing the launch
      * @param args command-line arguments
      * @exception Exception thrown if a problem occurs during the launch
      */
@@ -310,8 +312,8 @@ public class Main {
 			rootURLs[0] = source.getLocation();
             System.arraycopy(bootPath, 0, rootURLs, 1, bootPath.length);
             // replace the security policy
-            Policy eclispePolicy = new EclipsePolicy(Policy.getPolicy(), rootURLs);
-            Policy.setPolicy(eclispePolicy);
+            Policy eclipsePolicy = new EclipsePolicy(Policy.getPolicy(), rootURLs);
+            Policy.setPolicy(eclipsePolicy);
             if (setSM)
                 System.setSecurityManager(sm);
         }
@@ -360,7 +362,7 @@ public class Main {
             if (!compatible) {
                 // any non-zero value should do it - 14 used to be used for version incompatibility in Eclipse 2.1 
                 System.getProperties().put(PROP_EXITCODE, "14"); //$NON-NLS-1$
-                System.getProperties().put(PROP_EXITDATA, "Required Java version: "+ requiredVersion + ". Available: " + availableVersion + '.'); //$NON-NLS-1$ //$NON-NLS-2$
+                System.getProperties().put(PROP_EXITDATA, "<title>Incompatible JVM</title>Version " + availableVersion + " of the JVM is not suitable for this product. Version: "+ requiredVersion + " or greater is required."); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
             }
             return compatible;
         } catch (SecurityException e) {
@@ -414,6 +416,8 @@ public class Main {
         for (int i = 0; i < encodedLength; i++) {
             byte b = encodedBytes[i];
             if (b == '%') {
+            	if(i+2 >= encodedLength)
+            		throw new IllegalArgumentException("Malformed URL (\""+urlString+"\"): % must be followed by 2 digits.");  //$NON-NLS-1$//$NON-NLS-2$
                 byte enc1 = encodedBytes[++i];
                 byte enc2 = encodedBytes[++i];
                 b = (byte) ((hexToByte(enc1) << 4) + hexToByte(enc2));
@@ -461,7 +465,7 @@ public class Main {
     private URL[] getDevPath(URL base) throws IOException {
         ArrayList result = new ArrayList(5);
         if (inDevelopmentMode)
-            addDevEntries(base, result); //$NON-NLS-1$
+            addDevEntries(base, result);
         //The jars from the base always need to be added, even when running in dev mode (bug 46772)
         addBaseJars(base, result);
         return (URL[]) result.toArray(new URL[result.size()]);
@@ -579,7 +583,7 @@ public class Main {
                     addEntry(base, result);
                 }
                 URL url = null;
-                if (string.startsWith(FILE_SCHEME)) //$NON-NLS-1$
+                if (string.startsWith(FILE_SCHEME))
                     url = new File(string.substring(5)).toURL();
                 else
                     url = new URL(string);
@@ -666,7 +670,7 @@ public class Main {
                 return candidate.getName().equals(target) || candidate.getName().startsWith(target + "_"); //$NON-NLS-1$
             }
         };
-        File[] candidates = new File(start).listFiles(filter); //$NON-NLS-1$
+        File[] candidates = new File(start).listFiles(filter);
         if (candidates == null)
             return null;
         String[] arrays = new String[candidates.length];
@@ -761,7 +765,7 @@ public class Main {
     private static URL buildURL(String spec, boolean trailingSlash) {
         if (spec == null)
             return null;
-        boolean isFile = spec.startsWith(FILE_SCHEME); //$NON-NLS-1$
+        boolean isFile = spec.startsWith(FILE_SCHEME);
         try {
             if (isFile) {
                 File toAdjust = new File(spec.substring(5));
@@ -1009,7 +1013,7 @@ public class Main {
         if (exitData == null || data == null)
             return;
         // sync call to the launcher
-        runCommand(true, new String[] {EXITDATA, exitData, data}); //$NON-NLS-1$
+        runCommand(true, new String[] {EXITDATA, exitData, data});
     }
 
     /**
@@ -1538,7 +1542,7 @@ public class Main {
         if (location == null)
             return;
         // async call to the launcher
-        showProcess = runCommand(false, new String[] {SHOWSPLASH, showSplash, location}); //$NON-NLS-1$
+        showProcess = runCommand(false, new String[] {SHOWSPLASH, showSplash, location});
     }
     
     private Process runCommand(boolean block, String[] extraArgs) {
@@ -1555,7 +1559,7 @@ public class Main {
         if (debug) {
             System.out.print("runCommand:\n    ");//$NON-NLS-1$ 
             for (int i = 0; i < args.length; i++) {
-                System.out.print('<' + args[i] + '>');//$NON-NLS-1$ //$NON-NLS-2$ 
+                System.out.print('<' + args[i] + '>');
             }
             System.out.println();
         }
@@ -1836,7 +1840,7 @@ public class Main {
      */
     private String resolve(String urlString) {
         // handle the case where people mistakenly spec a refererence: url.
-        if (urlString.startsWith(REFERENCE_SCHEME)) { //$NON-NLS-1$
+        if (urlString.startsWith(REFERENCE_SCHEME)) {
             urlString = urlString.substring(10);
             System.getProperties().put(PROP_FRAMEWORK, urlString);
         }
@@ -2025,27 +2029,25 @@ public class Main {
     }
 
     private void mergeProperties(Properties destination, Properties source) {
-        if (destination == null || source == null)
-            return;
-        for (Enumeration e = source.keys(); e.hasMoreElements();) {
-            String key = (String) e.nextElement();
-            if (key.equals(PROP_CLASSPATH)) {
-                String destinationClasspath = destination.getProperty(PROP_CLASSPATH);
-                String sourceClasspath = source.getProperty(PROP_CLASSPATH);
-                if (destinationClasspath == null)
-                    destinationClasspath = sourceClasspath;
-                else
-                    destinationClasspath = destinationClasspath + sourceClasspath;
-                destination.put(PROP_CLASSPATH, destinationClasspath);
-                continue;
-            }
-            if (!key.equals(PROP_EOF)) {
-                String value = source.getProperty(key);
-                if (destination.getProperty(key) == null)
-                    destination.put(key, value);
-            }
-        }
-    }
+		if (destination == null || source == null)
+			return;
+		for (Enumeration e = source.keys(); e.hasMoreElements();) {
+			String key = (String) e.nextElement();
+			if (key.equals(PROP_CLASSPATH)) {
+				String destinationClasspath = destination.getProperty(PROP_CLASSPATH);
+				String sourceClasspath = source.getProperty(PROP_CLASSPATH);
+				if (destinationClasspath == null)
+					destinationClasspath = sourceClasspath;
+				else
+					destinationClasspath = destinationClasspath + sourceClasspath;
+				destination.put(PROP_CLASSPATH, destinationClasspath);
+				continue;
+			}
+			String value = source.getProperty(key);
+			if (destination.getProperty(key) == null)
+				destination.put(key, value);
+		}
+	}
 
     private void setupVMProperties() {
         if (vm != null)
@@ -2067,6 +2069,19 @@ public class Main {
         }
     }
 
+    /*
+     * NOTE: It is ok here for EclipsePolicy to use 1.4 methods because the methods
+     * that it calls them from don't exist in Foundation so they will never be called. A more
+     * detailed explanation from Tom:
+     * 
+     * They will never get called because in a pre 1.4 VM the methods 
+     * getPermissions(CodeSource) and implies(ProtectionDomain, Permission) are 
+     * undefined on the Policy class which is what EclipsePolicy extends.  EclipsePolicy 
+     * implements these two methods so it can proxy them to the parent Policy.  
+     * But since these methods are not actually defined on Policy in a pre-1.4 VM 
+     * nobody will actually call them (unless they casted the policy to EclipsePolicy and 
+     * called our methods)
+     */
     private class EclipsePolicy extends Policy {
         // The policy that this EclipsePolicy is replacing
         private Policy policy;
