@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -165,6 +165,15 @@ public Browser(Composite parent, int style) {
 	OS.PtRealizeWidget(webHandle);
 }
 
+/**
+ * Clears all session cookies from all current Browser instances.
+ * 
+ * @since 3.2
+ */
+public static void clearSessions () {
+	// TODO
+}
+
 static int webProc(int handle, int data, int info) {
 	Display display = Display.getCurrent();
 	int parent = OS.PtWidgetParent (handle);
