@@ -136,6 +136,11 @@ protected int QueryInterface(int riid, int ppvObject) {
 		AddRef();
 		return COM.S_OK;
 	}
+    if (COM.IsEqualGUID(guid, COM.IIDIInternetSecurityManager)) {
+        COM.MoveMemory(ppvObject, new int[] {iInternetSecurityManager.getAddress()}, 4);
+        AddRef();
+        return COM.S_OK;
+    }
 	if (COM.IsEqualGUID(guid, COM.IIDIOleCommandTarget)) {
 		COM.MoveMemory(ppvObject, new int[] {iOleCommandTarget.getAddress()}, 4);
 		AddRef();
@@ -160,7 +165,8 @@ int GetDropTarget(int pDropTarget, int ppDropTarget) {
 }
 
 int GetExternal(int ppDispatch) {
-	return COM.E_NOTIMPL;
+	OS.MoveMemory(ppDispatch, new int[] {0}, 4);
+	return COM.S_FALSE;
 }
 
 int GetHostInfo(int pInfo) {
