@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,6 +12,7 @@ package org.eclipse.swt.dnd;
 
 
 import org.eclipse.swt.*;
+import org.eclipse.swt.custom.*;
 import org.eclipse.swt.widgets.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.carbon.*;
@@ -69,33 +70,33 @@ import org.eclipse.swt.internal.carbon.*;
  */
 public class DropTarget extends Widget {
 
-	private Control control;
-	private Listener controlListener;
-	private Transfer[] transferAgents = new Transfer[0];
-	private DragUnderEffect effect;
+	Control control;
+	Listener controlListener;
+	Transfer[] transferAgents = new Transfer[0];
+	DragAndDropEffect effect;
 
 	// Track application selections
-	private TransferData selectedDataType;
-	private int selectedOperation;
+	TransferData selectedDataType;
+	int selectedOperation;
 	
 	// workaround - There is no event for "operation changed" so track operation based on key state
-	private int keyOperation = -1;
+	int keyOperation = -1;
 	
 	// workaround - Simulate events when mouse is not moving
-	private long dragOverStart;
-	private Runnable dragOverHeartbeat;
-	private DNDEvent dragOverEvent;
+	long dragOverStart;
+	Runnable dragOverHeartbeat;
+	DNDEvent dragOverEvent;
 	
 	// workaround - OS events are relative to the application, not the control.
 	// Track which control is the current target to determine when drag and
 	// drop enters or leaves a widget.
-	private static DropTarget CurrentDropTarget = null;
+	static DropTarget CurrentDropTarget = null;
 	
-	private static final String DROPTARGETID = "DropTarget"; //$NON-NLS-1$
-	private static final int DRAGOVER_HYSTERESIS = 50;
+	static final String DROPTARGETID = "DropTarget"; //$NON-NLS-1$
+	static final int DRAGOVER_HYSTERESIS = 50;
 	
-	private static Callback DragTrackingHandler;
-	private static Callback DragReceiveHandler;
+	static Callback DragTrackingHandler;
+	static Callback DragReceiveHandler;
 	
 	static {
 		DragTrackingHandler = new Callback(DropTarget.class, "DragTrackingHandler", 4); //$NON-NLS-1$
@@ -165,13 +166,15 @@ public DropTarget(Control control, int style) {
 		}
 	});
 	
-	// Drag under effect
+	// Drag and drop effect
 	if (control instanceof Tree) {
-		effect = new TreeDragUnderEffect((Tree)control);
+		effect = new TreeDragAndDropEffect((Tree)control);
 	} else if (control instanceof Table) {
-		effect = new TableDragUnderEffect((Table)control);
+		effect = new TableDragAndDropEffect((Table)control);
+	} else if (control instanceof StyledText) {
+		effect = new StyledTextDragAndDropEffect((StyledText)control);
 	} else {
-		effect = new NoDragUnderEffect(control);
+		effect = new NoDragAndDropEffect(control);
 	}
 
 	dragOverHeartbeat = new Runnable() {
@@ -199,10 +202,11 @@ public DropTarget(Control control, int style) {
 				event.dataType = selectedDataType;
 				event.operations = dragOverEvent.operations;
 				event.detail  = selectedOperation;
+				event.item = effect.getItem(event.x, event.y);
 				selectedDataType = null;
 				selectedOperation = DND.DROP_NONE;				
 				notifyListeners(DND.DragOver, event);
-				effect.show(event.feedback, event.x, event.y);
+				effect.showDropTargetEffect(event.feedback, event.x, event.y);
 				if (event.dataType != null) {
 					for (int i = 0; i < allowedTypes.length; i++) {
 						if (allowedTypes[i].type == event.dataType.type) {
@@ -227,13 +231,13 @@ static int checkStyle (int style) {
 	return style;
 }
 
-private static int DragReceiveHandler(int theWindow, int handlerRefCon, int theDrag) {
+static int DragReceiveHandler(int theWindow, int handlerRefCon, int theDrag) {
 	DropTarget target = FindDropTarget(theWindow, theDrag);
 	if (target == null) return OS.noErr;
 	return target.dragReceiveHandler(theWindow, handlerRefCon, theDrag);   
 }
 
-private static int DragTrackingHandler(int message, int theWindow, int handlerRefCon, int theDrag) {
+static int DragTrackingHandler(int message, int theWindow, int handlerRefCon, int theDrag) {
 	if (message == OS.kDragTrackingLeaveHandler || message == OS.kDragTrackingEnterHandler) {
 		CurrentDropTarget = null;
 		return OS.noErr;
@@ -253,7 +257,7 @@ private static int DragTrackingHandler(int message, int theWindow, int handlerRe
 	return target.dragTrackingHandler(message, theWindow, handlerRefCon, theDrag);   
 }
 
-private static DropTarget FindDropTarget(int theWindow, int theDrag) {
+static DropTarget FindDropTarget(int theWindow, int theDrag) {
 	Display display = Display.findDisplay(Thread.currentThread());
 	if (display == null || display.isDisposed()) return null;
 	Point mouse = new Point();
@@ -323,9 +327,9 @@ protected void checkSubclass () {
 	}
 }
 
-private int dragReceiveHandler(int theWindow, int handlerRefCon, int theDrag) {
+int dragReceiveHandler(int theWindow, int handlerRefCon, int theDrag) {
 	updateDragOverHover(0, null);
-	effect.show(DND.FEEDBACK_NONE, 0, 0);
+	effect.showDropTargetEffect(DND.FEEDBACK_NONE, 0, 0);
 
 	if (keyOperation == -1) return OS.dragNotAcceptedErr;
 
@@ -333,7 +337,8 @@ private int dragReceiveHandler(int theWindow, int handlerRefCon, int theDrag) {
 	event.widget = this;
 	event.time = (int)System.currentTimeMillis();
 	event.detail = DND.DROP_NONE;
-	notifyListeners(DND.DragLeave, event);		
+	notifyListeners(DND.DragLeave, event);
+	
 	event = new DNDEvent();
 	if (!setEventData(theDrag, event)) {
 		return OS.dragNotAcceptedErr;
@@ -347,6 +352,7 @@ private int dragReceiveHandler(int theWindow, int handlerRefCon, int theDrag) {
 	selectedDataType = null;
 	selectedOperation = DND.DROP_NONE;
 	notifyListeners(DND.DropAccept, event);
+	
 	if (event.dataType != null) {
 		for (int i = 0; i < allowedDataTypes.length; i++) {
 			if (allowedDataTypes[i].type == event.dataType.type) {
@@ -363,7 +369,7 @@ private int dragReceiveHandler(int theWindow, int handlerRefCon, int theDrag) {
 		return OS.dragNotAcceptedErr;
 	}
 	// ask drag source for dropped data
-	byte[][] data  = data = new byte[0][];
+	byte[][] data  = new byte[0][];
 	// locate all the items with data of the desired type 
 	short[] numItems = new short[1];
 	OS.CountDragItems(theDrag, numItems);
@@ -410,11 +416,11 @@ private int dragReceiveHandler(int theWindow, int handlerRefCon, int theDrag) {
 	return (selectedOperation == DND.DROP_NONE) ? OS.dragNotAcceptedErr : OS.noErr;
 }
 
-private int dragTrackingHandler(int message, int theWindow, int handlerRefCon, int theDrag) {
+int dragTrackingHandler(int message, int theWindow, int handlerRefCon, int theDrag) {
 	
 	if (message == OS.kDragTrackingLeaveWindow) {
 		updateDragOverHover(0, null);
-		effect.show(DND.FEEDBACK_NONE, 0, 0);
+		effect.showDropTargetEffect(DND.FEEDBACK_NONE, 0, 0);
 		OS.SetThemeCursor(OS.kThemeArrowCursor);
 		if (keyOperation == -1) return OS.dragNotAcceptedErr;
 		keyOperation = -1;
@@ -485,7 +491,7 @@ private int dragTrackingHandler(int message, int theWindow, int handlerRefCon, i
 	
 	OS.SetDragDropAction(theDrag, opToOsOp(selectedOperation));
 	
-	effect.show(event.feedback, event.x, event.y);
+	effect.showDropTargetEffect(event.feedback, event.x, event.y);
 
 	switch (selectedOperation) {
 		case DND.DROP_COPY:
@@ -517,7 +523,7 @@ public Control getControl () {
 	return control;
 }
 
-private int getOperationFromKeyState(int theDrag) {
+int getOperationFromKeyState(int theDrag) {
 	short[] modifiers = new short[1];
 	OS.GetDragModifiers(theDrag, modifiers, null, null);
 	boolean option = (modifiers[0] & OS.optionKey) == OS.optionKey;
@@ -537,19 +543,7 @@ public Transfer[] getTransfer() {
 	return transferAgents;
 }
 
-public void notifyListeners (int eventType, Event event) {
-	org.eclipse.swt.graphics.Point coordinates = new org.eclipse.swt.graphics.Point(event.x, event.y);
-	coordinates = control.toControl(coordinates);
-	if (this.control instanceof Tree) {
-		event.item = ((Tree)control).getItem(coordinates);
-	}
-	if (this.control instanceof Table) {
-		event.item = ((Table)control).getItem(coordinates);
-	}
-	super.notifyListeners(eventType, event);
-}
-
-private void onDispose () {	
+void onDispose () {	
 	if (control == null)
 		return;
 	if (controlListener != null)
@@ -560,7 +554,7 @@ private void onDispose () {
 	control = null;
 }
 
-private int opToOsOp(int operation) {
+int opToOsOp(int operation) {
 	int osOperation = 0;
 	if ((operation & DND.DROP_COPY) != 0){
 		osOperation |= OS.kDragActionCopy;
@@ -574,7 +568,7 @@ private int opToOsOp(int operation) {
 	return osOperation;
 }
 
-private int osOpToOp(int osOperation){
+int osOpToOp(int osOperation){
 	int operation = 0;
 	if ((osOperation & OS.kDragActionCopy) != 0){
 		operation |= DND.DROP_COPY;
@@ -618,7 +612,7 @@ public void removeDropListener(DropTargetListener listener) {
 	removeListener (DND.DropAccept, listener);
 }
 
-private boolean setEventData(int theDrag, DNDEvent event) {
+boolean setEventData(int theDrag, DNDEvent event) {
 	if (theDrag == 0) return false;
 	
 	// get allowed operations
@@ -708,6 +702,7 @@ private boolean setEventData(int theDrag, DNDEvent event) {
 	event.dataType = dataTypes[0];
 	event.operations = operations;
 	event.detail = operation;
+	event.item = effect.getItem(event.x, event.y);
 	
 	return true;
 }
@@ -730,7 +725,7 @@ public void setTransfer(Transfer[] transferAgents){
 	this.transferAgents = transferAgents;
 }
 
-private void updateDragOverHover(long delay, DNDEvent event) {
+void updateDragOverHover(long delay, DNDEvent event) {
 	if (delay == 0) {
 		dragOverStart = 0;
 		dragOverEvent = null;
