@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -12,6 +12,7 @@ package org.eclipse.swt.dnd;
 
 
 import org.eclipse.swt.*;
+import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.widgets.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.gtk.*;
@@ -57,9 +58,9 @@ import org.eclipse.swt.internal.gtk.*;
  *	// This example will allow the text to be copied or moved to the drop target
  *	int operations = DND.DROP_MOVE | DND.DROP_COPY;
  *	
- *	DragSource source = new DragSource (label, operations);
+ *	DragSource source = new DragSource(label, operations);
  *	source.setTransfer(types);
- *	source.addDragListener (new DragSourceListener() {
+ *	source.addDragListener(new DragSourceListener() {
  *		public void dragStart(DragSourceEvent e) {
  *			// Only start the drag if there is actually text in the
  *			// label - this text will be what is dropped on the target.
@@ -67,7 +68,7 @@ import org.eclipse.swt.internal.gtk.*;
  *				event.doit = false;
  *			}
  *		};
- *		public void dragSetData (DragSourceEvent event) {
+ *		public void dragSetData(DragSourceEvent event) {
  *			// A drop has been performed, so provide the data of the 
  *			// requested type.
  *			// (Checking the type of the requested data is only 
@@ -95,22 +96,29 @@ import org.eclipse.swt.internal.gtk.*;
 public class DragSource extends Widget {
 
 	// info for registering as a drag source
-	private Control control;
-	private Listener controlListener;
-	private Transfer[] transferAgents = new Transfer[0];
+	Control control;
+	Listener controlListener;
+	Transfer[] transferAgents = new Transfer[0];
 
-	private static final String DRAGSOURCEID = "DragSource"; //$NON-NLS-1$
+	int /*long*/ targetList;
+	
+	//workaround - remember action performed for DragEnd
+	boolean moveData = false;
+	
+	DragAndDropEffect effect;
+	static final String DRAGSOURCEID = "DragSource"; //$NON-NLS-1$
 		
 	static Callback DragGetData;
 	static Callback DragEnd;
 	static Callback DragDataDelete;
 	static {
-		DragGetData = new Callback(DragSource.class, "DragGetData", 6);	
-		DragEnd = new Callback(DragSource.class, "DragEnd", 3);
-		DragDataDelete = new Callback(DragSource.class, "DragDataDelete", 3);
+		DragGetData = new Callback(DragSource.class, "DragGetData", 5);	 //$NON-NLS-1$
+		if (DragGetData.getAddress() == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
+		DragEnd = new Callback(DragSource.class, "DragEnd", 2); //$NON-NLS-1$
+		if (DragEnd.getAddress() == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
+		DragDataDelete = new Callback(DragSource.class, "DragDataDelete", 2); //$NON-NLS-1$
+		if (DragDataDelete.getAddress() == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
 	}
-	int targetList;
-	boolean movePerformed;
 	
 /**
  * Creates a new <code>DragSource</code> to handle dragging from the specified <code>Control</code>.
@@ -124,13 +132,17 @@ public class DragSource extends Widget {
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
  *    <li>ERROR_INVALID_SUBCLASS - if this class is not an allowed subclass</li>
+ * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_CANNOT_INIT_DRAG - unable to initiate drag source; this will occur if more than one
  *        drag source is created for a control or if the operating system will not allow the creation
  *        of the drag source</li>
  * </ul>
- *
- * @see DragSource#dispose
+ * 
+ * <p>NOTE: ERROR_CANNOT_INIT_DRAG should be an SWTException, since it is a
+ * recoverable error, but can not be changed due to backward compatability.</p>
+ * 
+ * @see Widget#dispose
  * @see DragSource#checkSubclass
  * @see DND#DROP_NONE
  * @see DND#DROP_COPY
@@ -140,16 +152,17 @@ public class DragSource extends Widget {
 public DragSource(Control control, int style) {
 	super (control, checkStyle(style));
 	this.control = control;
-	if (control.getData(DRAGSOURCEID) != null)
+	if (DragGetData == null || DragEnd == null || DragDataDelete == null) {
 		DND.error(DND.ERROR_CANNOT_INIT_DRAG);
+	}
+	if (control.getData(DRAGSOURCEID) != null) {
+		DND.error(DND.ERROR_CANNOT_INIT_DRAG);
+	}
 	control.setData(DRAGSOURCEID, this);
-	
-	byte[] buffer = Converter.wcsToMbcs(null, "drag_data_get", true);
-	OS.g_signal_connect(control.handle, buffer, DragGetData.getAddress(), 0);	
-	buffer = Converter.wcsToMbcs(null, "drag_end", true);
-	OS.g_signal_connect(control.handle, buffer, DragEnd.getAddress(), 0);
-	buffer = Converter.wcsToMbcs(null, "drag_data_delete", true);
-	OS.g_signal_connect(control.handle, buffer, DragDataDelete.getAddress(), 0);
+
+	OS.g_signal_connect(control.handle, OS.drag_data_get, DragGetData.getAddress(), 0);	
+	OS.g_signal_connect(control.handle, OS.drag_end, DragEnd.getAddress(), 0);
+	OS.g_signal_connect(control.handle, OS.drag_data_delete, DragDataDelete.getAddress(), 0);
 	
 	controlListener = new Listener () {
 		public void handleEvent (Event event) {
@@ -168,6 +181,15 @@ public DragSource(Control control, int style) {
 	control.addListener (SWT.Dispose, controlListener);
 	control.addListener (SWT.DragDetect, controlListener);
 	
+	// DND effect
+	if (control instanceof Tree) {
+		effect = new TreeDragAndDropEffect((Tree)control);
+	} else if (control instanceof Table) {
+		effect = new TableDragAndDropEffect((Table)control);
+	} else {
+		effect = new NoDragAndDropEffect(control);
+	}
+	
 	this.addListener(SWT.Dispose, new Listener() {
 		public void handleEvent(Event e) {
 			onDispose();
@@ -175,28 +197,39 @@ public DragSource(Control control, int style) {
 	});
 }
 
-static DragSource FindDragSource(int handle) {
+static int checkStyle (int style) {
+	if (style == SWT.NONE) return DND.DROP_MOVE;
+	return style;
+}
+
+static int /*long*/ DragDataDelete(int /*long*/ widget, int /*long*/ context){
+	DragSource source = FindDragSource(widget);
+	if (source == null) return 0;
+	source.dragDataDelete(widget, context);
+	return 0;
+}
+
+static int /*long*/ DragEnd(int /*long*/ widget, int /*long*/ context){
+	DragSource source = FindDragSource(widget);
+	if (source == null) return 0;
+	source.dragEnd(widget, context);
+	return 0;
+}
+	
+static int /*long*/ DragGetData(int /*long*/ widget, int /*long*/ context, int /*long*/ selection_data,  int /*long*/ info, int /*long*/ time){
+	DragSource source = FindDragSource(widget);
+	if (source == null) return 0;
+	source.dragGetData(widget, context, selection_data, (int)/*64*/info, (int)/*64*/time);
+	return 0;
+}
+
+static DragSource FindDragSource(int /*long*/ handle) {
 	Display display = Display.findDisplay(Thread.currentThread());
 	if (display == null || display.isDisposed()) return null;
 	Widget widget = display.findWidget(handle);
 	if (widget == null) return null;
 	return (DragSource)widget.getData(DRAGSOURCEID);
 }
-static int DragEnd(int widget, int context, int data){
-	DragSource source = FindDragSource(widget);
-	if (source == null) return 0;
-	return source.dragEnd(widget, context, data);
-}	
-static int DragGetData(int widget, int context, int selection_data,  int info, int time, int data){
-	DragSource source = FindDragSource(widget);
-	if (source == null) return 0;
-	return source.dragGetData(widget, context, selection_data, info, time, data);
-}
-static int DragDataDelete(int widget, int context, int data){
-	DragSource source = FindDragSource(widget);
-	if (source == null) return 0;
-	return source.dragDataDelete(widget, context, data);
-}
 
 /**
  * Adds the listener to the collection of listeners who will
@@ -235,11 +268,6 @@ public void addDragListener(DragSourceListener listener) {
 	addListener (DND.DragEnd, typedListener);
 }
 
-static int checkStyle (int style) {
-	if (style == SWT.NONE) return DND.DROP_MOVE;
-	return style;
-}
-
 protected void checkSubclass () {
 	String name = getClass().getName ();
 	String validName = DragSource.class.getName();
@@ -248,35 +276,31 @@ protected void checkSubclass () {
 	}
 }
 
-/**
- * Returns the Control which is registered for this DragSource.  This is the control that the 
- * user clicks in to initiate dragging.
- *
- * @return the Control which is registered for this DragSource
- */
-public Control getControl () {
-	return control;
-}
-
-private void drag(Event dragEvent) {
+void drag(Event dragEvent) {
+	moveData = false;	
 	DNDEvent event = new DNDEvent();
 	event.widget = this;	
+	event.x = dragEvent.x;
+	event.y = dragEvent.y;
 	event.time = dragEvent.time;
 	event.doit = true;
-	try {
-		notifyListeners(DND.DragStart, event);
-	} catch (Throwable e) {
-		event.doit = false;
-	}
-	if (!event.doit) return;
-	if (transferAgents == null || transferAgents.length == 0) return;
+	notifyListeners(DND.DragStart, event);
+	if (!event.doit || transferAgents == null || transferAgents.length == 0) return;
 	if (targetList == 0) return;
 	
 	int actions = opToOsOp(getStyle());
-	OS.gtk_drag_begin(control.handle, targetList, actions, 1, 0);
+	ImageData imageData = effect.getDragSourceImage(dragEvent.x, dragEvent.y);
+	int /*long*/ context = OS.gtk_drag_begin(control.handle, targetList, actions, 1, 0);
+	if (context != 0 && imageData != null) {
+		Image image = new Image(getDisplay(), imageData);
+		int /*long*/ pixbuf = createPixbuf(image);
+		OS.gtk_drag_set_icon_pixbuf(context, pixbuf, 0, 0);
+		OS.g_object_unref(pixbuf);
+		image.dispose();
+	}
 }
 
-int dragEnd(int widget, int context, int data){
+void dragEnd(int /*long*/ widget, int /*long*/ context){
 	/*
 	 * Bug in GTK.  If a drag is initiated using gtk_drag_begin and the 
 	 * mouse is released immediately, the mouse and keyboard remain
@@ -289,73 +313,76 @@ int dragEnd(int widget, int context, int data){
 	OS.gdk_pointer_ungrab(OS.GDK_CURRENT_TIME); 
 	OS.gdk_keyboard_ungrab(OS.GDK_CURRENT_TIME);
 	
-	int op = DND.DROP_NONE;
+	int operation = DND.DROP_NONE;
 	if (context != 0) {
 		GdkDragContext gdkDragContext = new GdkDragContext ();
 		OS.memmove(gdkDragContext, context, GdkDragContext.sizeof);
 		if (gdkDragContext.dest_window != 0) { //NOTE: if dest_window is 0, drag was aborted
-			op = osOpToOp(gdkDragContext.action);
-			if (movePerformed) op = DND.DROP_MOVE;
+			if (moveData) {
+				operation = DND.DROP_MOVE;
+			} else {
+				operation = osOpToOp(gdkDragContext.action);
+				if (operation == DND.DROP_MOVE) operation = DND.DROP_NONE;
+			}
 		}
 	}	
 	
 	DNDEvent event = new DNDEvent();
 	event.widget = this;
-	event.doit = op != 0;
-	event.detail = op; 
-	
-	try {
-		notifyListeners(DND.DragEnd, event);
-	} catch (Throwable e) {
-		return 0;
-	}
-	movePerformed = false;
-	return 1;	
+	//event.time = ???
+	event.doit = operation != 0;
+	event.detail = operation; 
+	notifyListeners(DND.DragEnd, event);
+	moveData = false;	
 }	
 
-int dragGetData(int widget, int context, int selection_data,  int info, int time, int data){
-	if (selection_data == 0) return 0;	
+void dragGetData(int /*long*/ widget, int /*long*/ context, int /*long*/ selection_data,  int info, int time){
+	if (selection_data == 0) return;	
 	GtkSelectionData gtkSelectionData = new GtkSelectionData();
 	OS.memmove(gtkSelectionData, selection_data, GtkSelectionData.sizeof);
-	if (gtkSelectionData.target == 0) return 0;
-	TransferData tdata = new TransferData();
-	tdata.type = gtkSelectionData.target;
-	tdata.pValue = gtkSelectionData.data;
-	tdata.length = gtkSelectionData.length;
-	tdata.format = gtkSelectionData.format;
+	if (gtkSelectionData.target == 0) return;
+	
+	TransferData transferData = new TransferData();
+	transferData.type = gtkSelectionData.target;
+	transferData.pValue = gtkSelectionData.data;
+	transferData.length = gtkSelectionData.length;
+	transferData.format = gtkSelectionData.format;
 		
 	DNDEvent event = new DNDEvent();
 	event.widget = this;
 	event.time = time; 
-	event.dataType = tdata; 
-	try {
-		notifyListeners(DND.DragSetData, event);
-	} catch (Throwable e) {
-		return 0;
-	}
-	if (event.data == null) return 0;
+	event.dataType = transferData; 
+	notifyListeners(DND.DragSetData, event);
 		
 	Transfer transfer = null;
 	for (int i = 0; i < transferAgents.length; i++) {
-		transfer = transferAgents[i];
-		if (transfer.isSupportedType(event.dataType)) break;
+		if (transferAgents[i].isSupportedType(transferData)) {
+			transfer = transferAgents[i];
+			break;
+		}
 	}
-	if (transfer == null) return 0;
-	transfer.javaToNative(event.data, event.dataType);
-	if (event.dataType.result == 0) return 0;
-	
-	OS.gtk_selection_data_set(selection_data, event.dataType.type, event.dataType.format, event.dataType.pValue, event.dataType.length);
-	return 1;	
-}
-int dragDataDelete(int widget, int context, int data){
-	movePerformed = true;
-	return 1;
+	if (transfer == null) return;
+	transfer.javaToNative(event.data, transferData);
+	if (transferData.result != 1) return;
+	OS.gtk_selection_data_set(selection_data, transferData.type, transferData.format, transferData.pValue, transferData.length);
+	OS.g_free(transferData.pValue);
+	return;	
 }
 
-public Display getDisplay () {
-	if (control == null) DND.error(SWT.ERROR_WIDGET_DISPOSED);
-	return control.getDisplay ();
+void dragDataDelete(int /*long*/ widget, int /*long*/ context){
+	moveData = true;
 }
+
+/**
+ * Returns the Control which is registered for this DragSource.  This is the control that the 
+ * user clicks in to initiate dragging.
+ *
+ * @return the Control which is registered for this DragSource
+ */
+public Control getControl () {
+	return control;
+}
+
 /**
  * Returns the list of data types that can be transferred by this DragSource.
  *
@@ -365,11 +392,11 @@ public Transfer[] getTransfer(){
 	return transferAgents;
 }
 
-private void onDispose() {
-	if (control == null)
-		return;
-	if (targetList != 0) 
+void onDispose() {
+	if (control == null) return;
+	if (targetList != 0) {
 		OS.gtk_target_list_unref(targetList);
+	}
 	targetList = 0;
 	if (controlListener != null) {
 		control.removeListener(SWT.Dispose, controlListener);
@@ -381,7 +408,7 @@ private void onDispose() {
 	transferAgents = null;
 }
 
-private int opToOsOp(int operation){
+int opToOsOp(int operation){
 	int osOperation = 0;
 	
 	if ((operation & DND.DROP_COPY) == DND.DROP_COPY)
@@ -393,7 +420,8 @@ private int opToOsOp(int operation){
 	
 	return osOperation;
 }
-private int osOpToOp(int osOperation){
+
+int osOpToOp(int osOperation){
 	int operation = DND.DROP_NONE;
 	
 	if ((osOperation & OS.GDK_ACTION_COPY) == OS.GDK_ACTION_COPY)
@@ -429,6 +457,7 @@ public void removeDragListener(DragSourceListener listener) {
 	removeListener (DND.DragSetData, listener);
 	removeListener (DND.DragEnd, listener);
 }
+
 /**
  * Specifies the list of data types that can be transferred by this DragSource.
  * The application must be able to provide data to match each of these types when
@@ -463,7 +492,7 @@ public void setTransfer(Transfer[] transferAgents){
 		}	
 	}
 	
-	int pTargets = OS.g_malloc(targets.length * GtkTargetEntry.sizeof);
+	int /*long*/ pTargets = OS.g_malloc(targets.length * GtkTargetEntry.sizeof);
 	for (int i = 0; i < targets.length; i++) {
 		OS.memmove(pTargets + i*GtkTargetEntry.sizeof, targets[i], GtkTargetEntry.sizeof);		
 	}			
@@ -473,4 +502,60 @@ public void setTransfer(Transfer[] transferAgents){
 		OS.g_free(targets[i].target);
 	}
 }
+
+static int /*long*/ createPixbuf(Image image) {
+	int [] w = new int [1], h = new int [1];
+ 	OS.gdk_drawable_get_size (image.pixmap, w, h);
+	int /*long*/ colormap = OS.gdk_colormap_get_system ();
+	int /*long*/ pixbuf;
+	boolean hasMask = image.mask != 0 && OS.gdk_drawable_get_depth (image.mask) == 1;
+	if (hasMask) {
+		pixbuf = OS.gdk_pixbuf_new (OS.GDK_COLORSPACE_RGB, true, 8, w [0], h [0]);
+		if (pixbuf == 0) SWT.error (SWT.ERROR_NO_HANDLES);
+		OS.gdk_pixbuf_get_from_drawable (pixbuf, image.pixmap, colormap, 0, 0, 0, 0, w [0], h [0]);
+		int /*long*/ maskPixbuf = OS.gdk_pixbuf_new(OS.GDK_COLORSPACE_RGB, false, 8, w [0], h [0]);
+		if (maskPixbuf == 0) SWT.error (SWT.ERROR_NO_HANDLES);
+		OS.gdk_pixbuf_get_from_drawable(maskPixbuf, image.mask, 0, 0, 0, 0, 0, w [0], h [0]);
+		int stride = OS.gdk_pixbuf_get_rowstride(pixbuf);
+		int /*long*/ pixels = OS.gdk_pixbuf_get_pixels(pixbuf);
+		byte[] line = new byte[stride];
+		int maskStride = OS.gdk_pixbuf_get_rowstride(maskPixbuf);
+		int /*long*/ maskPixels = OS.gdk_pixbuf_get_pixels(maskPixbuf);
+		byte[] maskLine = new byte[maskStride];
+		for (int y=0; y<h[0]; y++) {
+			int /*long*/ offset = pixels + (y * stride);
+			OS.memmove(line, offset, stride);
+			int /*long*/ maskOffset = maskPixels + (y * maskStride);
+			OS.memmove(maskLine, maskOffset, maskStride);
+			for (int x=0; x<w[0]; x++) {
+				if (maskLine[x * 3] == 0) {
+					line[x * 4 + 3] = 0;
+				}
+			}
+			OS.memmove(offset, line, stride);
+		}
+		OS.g_object_unref(maskPixbuf);
+	} else {
+		ImageData data = image.getImageData ();
+		boolean hasAlpha = data.getTransparencyType () == SWT.TRANSPARENCY_ALPHA;
+		pixbuf = OS.gdk_pixbuf_new (OS.GDK_COLORSPACE_RGB, hasAlpha, 8, w [0], h [0]);
+		if (pixbuf == 0) SWT.error (SWT.ERROR_NO_HANDLES);
+		OS.gdk_pixbuf_get_from_drawable (pixbuf, image.pixmap, colormap, 0, 0, 0, 0, w [0], h [0]);
+		if (hasAlpha) {
+			byte [] alpha = data.alphaData;
+			int stride = OS.gdk_pixbuf_get_rowstride (pixbuf);
+			int /*long*/ pixels = OS.gdk_pixbuf_get_pixels (pixbuf);
+			byte [] line = new byte [stride];
+			for (int y = 0; y < h [0]; y++) {
+				int /*long*/ offset = pixels + (y * stride);
+				OS.memmove (line, offset, stride);
+				for (int x = 0; x < w [0]; x++) {
+					line [x*4+3] = alpha [y*w [0]+x];
+				}
+				OS.memmove (offset, line, stride);
+			}
+		}
+	}
+	return pixbuf;
+}
 }
