@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -32,8 +32,10 @@ public class TextTransfer extends ByteArrayTransfer {
 	private static TextTransfer _instance = new TextTransfer();
 	private static final String COMPOUND_TEXT = "COMPOUND_TEXT"; //$NON-NLS-1$
 	private static final String UTF8_STRING = "UTF8_STRING"; //$NON-NLS-1$
+	private static final String STRING = "STRING"; //$NON-NLS-1$
 	private static final int COMPOUND_TEXT_ID = registerType(COMPOUND_TEXT);
 	private static final int UTF8_STRING_ID = registerType(UTF8_STRING);
+	private static final int STRING_ID = registerType(STRING);
 
 private TextTransfer() {}
 
@@ -62,13 +64,13 @@ public void javaToNative (Object object, TransferData transferData) {
 		DND.error(DND.ERROR_INVALID_DATA);
 	}
 	String string = (String)object;
-	byte[] buffer = Converter.wcsToMbcs (null, string, true);
+	byte[] utf8 = Converter.wcsToMbcs (null, string, true);
 	if  (transferData.type ==  COMPOUND_TEXT_ID) {
 		int /*long*/[] encoding = new int /*long*/[1];
 		int[] format = new int[1];
 		int /*long*/[] ctext = new int /*long*/[1];
 		int[] length = new int[1];
-		boolean result = OS.gdk_utf8_to_compound_text(buffer, encoding, format, ctext, length);
+		boolean result = OS.gdk_utf8_to_compound_text(utf8, encoding, format, ctext, length);
 		if (!result) return;
 		transferData.type = encoding[0];
 		transferData.format = format[0];
@@ -77,15 +79,24 @@ public void javaToNative (Object object, TransferData transferData) {
 		transferData.result = 1;
 	} 
 	if (transferData.type == UTF8_STRING_ID) {
-		int /*long*/ pValue = OS.g_malloc(buffer.length);
+		int /*long*/ pValue = OS.g_malloc(utf8.length);
 		if (pValue ==  0) return;
-		OS.memmove(pValue, buffer, buffer.length);
+		OS.memmove(pValue, utf8, utf8.length);
 		transferData.type = UTF8_STRING_ID;
 		transferData.format = 8;
-		transferData.length = buffer.length - 1;
+		transferData.length = utf8.length - 1;
 		transferData.pValue = pValue;
 		transferData.result = 1;
 	}
+	if (transferData.type == STRING_ID) {
+		int /*long*/ string_target = OS.gdk_utf8_to_string_target(utf8);
+		if (string_target ==  0) return;
+		transferData.type = STRING_ID;
+		transferData.format = 8;
+		transferData.length = OS.strlen(string_target);
+		transferData.pValue = string_target;
+		transferData.result = 1;
+	}
 }
 
 /**
@@ -99,38 +110,28 @@ public void javaToNative (Object object, TransferData transferData) {
  */
 public Object nativeToJava(TransferData transferData){
 	if (!isSupportedType(transferData) ||  transferData.pValue == 0) return null;
-	byte[] buffer = null;
-	if (transferData.type == COMPOUND_TEXT_ID) { 	
-		int /*long*/[] list = new int /*long*/[1];
-		int count = OS.gdk_text_property_to_utf8_list(transferData.type, transferData.format, transferData.pValue, transferData.length, list);
-		if (count == 0) return null;
-		int /*long*/[] ptr = new int /*long*/[1];
-		OS.memmove(ptr, list[0], OS.PTR_SIZEOF);
-		int length = OS.strlen(ptr[0]);
-		buffer = new byte[length];
-		OS.memmove(buffer, ptr[0], length);
-		OS.g_strfreev(list[0]);
-	}
-	if (transferData.type == UTF8_STRING_ID) {
-		int size = transferData.format * transferData.length / 8;
-		if (size == 0) return null;
-		buffer = new byte[size];
-		OS.memmove(buffer, transferData.pValue, size);
-	}
-	if (buffer == null) return null;
-	// convert byte array to a string
-	char [] unicode = Converter.mbcsToWcs (null, buffer);
+	int /*long*/[] list = new int /*long*/[1];
+	int count = OS.gdk_text_property_to_utf8_list(transferData.type, transferData.format, transferData.pValue, transferData.length, list);
+	if (count == 0) return null;
+	int /*long*/[] ptr = new int /*long*/[1];
+	OS.memmove(ptr, list[0], OS.PTR_SIZEOF);
+	int length = OS.strlen(ptr[0]);
+	byte[] utf8 = new byte[length];
+	OS.memmove(utf8, ptr[0], length);
+	OS.g_strfreev(list[0]);
+	// convert utf8 byte array to a unicode string
+	char [] unicode = Converter.mbcsToWcs (null, utf8);
 	String string = new String (unicode);
 	int end = string.indexOf('\0');
 	return (end == -1) ? string : string.substring(0, end);
 }
 
 protected int[] getTypeIds() {
-	return new int[] {UTF8_STRING_ID, COMPOUND_TEXT_ID};
+	return new int[] {UTF8_STRING_ID, COMPOUND_TEXT_ID, STRING_ID};
 }
 
 protected String[] getTypeNames() {
-	return new String[] {UTF8_STRING, COMPOUND_TEXT};
+	return new String[] {UTF8_STRING, COMPOUND_TEXT, STRING};
 }
 
 boolean checkText(Object object) {
