@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,9 +12,9 @@ package org.eclipse.swt.dnd;
 
 
 import org.eclipse.swt.*;
+import org.eclipse.swt.custom.*;
 import org.eclipse.swt.widgets.*;
-import org.eclipse.swt.graphics.*;
-import org.eclipse.swt.internal.Callback;
+import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.motif.*;
 
 /**
@@ -70,44 +70,44 @@ import org.eclipse.swt.internal.motif.*;
  */
 public class DropTarget extends Widget {
 	
-	private Control control;
-	private Listener controlListener;
-	private Transfer[] transferAgents = new Transfer[0];
-	private DragUnderEffect effect;
+	Control control;
+	Listener controlListener;
+	Transfer[] transferAgents = new Transfer[0];
+	DragAndDropEffect effect;
 	
 	// Track application selections
-	private TransferData selectedDataType;
-	private int selectedOperation;
+	TransferData selectedDataType;
+	int selectedOperation;
 	
 	// workaround - Simulate events when the mouse is not moving
-	private long dragOverStart;
-	private Runnable dragOverHeartbeat;
-	private DNDEvent dragOverEvent;
+	long dragOverStart;
+	Runnable dragOverHeartbeat;
+	DNDEvent dragOverEvent;
 	
 	// workaround - The data required in the transferProc callback is not available 
 	// so store it from the dropProc callback 
-	private XmDropProcCallbackStruct droppedEventData;
-	private int dropTransferObject;
+	XmDropProcCallbackStruct droppedEventData;
+	int dropTransferObject;
 	
 	// workaround - The DND operation can time out so temporarily set the
 	// time out to be very long while the application processes the data and
 	// restore it to the previous value when done.
-	private int selectionTimeout;
+	int selectionTimeout;
 	
 	// workaround - A drop target is still active even when it is not visible.
 	// This causes problems when the widget is not visible but
 	// overlaps with a visible widget.  Deregister drop target when it is
 	// not visible.
-	private boolean registered = false;
+	boolean registered = false;
 	
-	private static int DELETE_TYPE = Transfer.registerType("DELETE\0"); //$NON-NLS-1$
-	private static int NULL_TYPE = Transfer.registerType("NULL\0"); //$NON-NLS-1$
-	private static final String DROPTARGETID = "DropTarget"; //$NON-NLS-1$
-	private static final int DRAGOVER_HYSTERESIS = 50;
+	static int DELETE_TYPE = Transfer.registerType("DELETE\0"); //$NON-NLS-1$
+	static int NULL_TYPE = Transfer.registerType("NULL\0"); //$NON-NLS-1$
+	static final String DROPTARGETID = "DropTarget"; //$NON-NLS-1$
+	static final int DRAGOVER_HYSTERESIS = 50;
 	
-	private static Callback DropProc;
-	private static Callback DragProc;
-	private static Callback TransferProc;
+	static Callback DropProc;
+	static Callback DragProc;
+	static Callback TransferProc;
 	
 	static {
 		DropProc = new Callback(DropTarget.class, "DropProcCallback", 3); //$NON-NLS-1$
@@ -175,12 +175,20 @@ public DropTarget(Control control, int style) {
 					} else {
 						int[] args = new int[]{OS.XmNdropSiteActivity,   OS.XmDROP_SITE_ACTIVE,};
 						OS.XmDropSiteUpdate(DropTarget.this.control.handle, args, args.length/2);
+						if (DropTarget.this.control instanceof Label) {
+							int formHandle = OS.XtParent (DropTarget.this.control.handle);
+							OS.XmDropSiteUpdate(formHandle, args, args.length / 2);
+						}
 					}
 					break;
 				}
 				case SWT.Hide: {
 					int[] args = new int[]{OS.XmNdropSiteActivity,   OS.XmDROP_SITE_INACTIVE,};
 					OS.XmDropSiteUpdate(DropTarget.this.control.handle, args, args.length/2);	
+					if (DropTarget.this.control instanceof Label) {
+						int formHandle = OS.XtParent (DropTarget.this.control.handle);
+						OS.XmDropSiteUpdate(formHandle, args, args.length / 2);
+					}
 					break;
 				}
 			}
@@ -206,11 +214,13 @@ public DropTarget(Control control, int style) {
 
 	// Drag under effect
 	if (control instanceof Tree) {
-		effect = new TreeDragUnderEffect((Tree)control);
+		effect = new TreeDragAndDropEffect((Tree)control);
 	} else if (control instanceof Table) {
-		effect = new TableDragUnderEffect((Table)control);
+		effect = new TableDragAndDropEffect((Table)control);
+	} else if (control instanceof StyledText) {
+		effect = new StyledTextDragAndDropEffect((StyledText)control);
 	} else {
-		effect = new NoDragUnderEffect(control);
+		effect = new NoDragAndDropEffect(control);
 	}
 
 	if (control.isVisible()) registerDropTarget();
@@ -240,9 +250,10 @@ public DropTarget(Control control, int style) {
 				event.dataType = selectedDataType;
 				event.operations = dragOverEvent.operations;
 				event.detail  = selectedOperation;
+				event.item = effect.getItem(dragOverEvent.x, dragOverEvent.y);
 				notifyListeners(DND.DragOver, event);
 				
-				effect.show(event.feedback, event.x, event.y);
+				effect.showDropTargetEffect(event.feedback, event.x, event.y);
 				
 				selectedDataType = null;
 				if (event.dataType != null) {
@@ -271,7 +282,7 @@ static int checkStyle (int style) {
 	return style;
 }
 
-private static int DragProcCallback(int widget, int client_data, int call_data) {
+static int DragProcCallback(int widget, int client_data, int call_data) {
 	DropTarget target = FindDropTarget(widget);
 	if (target != null) {
 		target.dragProcCallback(widget, client_data, call_data);
@@ -279,7 +290,7 @@ private static int DragProcCallback(int widget, int client_data, int call_data)
 	return 0;
 }
 
-private static int DropProcCallback(int widget, int client_data, int call_data) {
+static int DropProcCallback(int widget, int client_data, int call_data) {
 	DropTarget target = FindDropTarget(widget);
 	if (target != null) {
 		target.dropProcCallback(widget, client_data, call_data);
@@ -287,7 +298,7 @@ private static int DropProcCallback(int widget, int client_data, int call_data)
 	return 0;
 }
 
-private static DropTarget FindDropTarget(int handle) {
+static DropTarget FindDropTarget(int handle) {
 	Display display = Display.findDisplay(Thread.currentThread());
 	if (display == null || display.isDisposed()) return null;
 	Widget widget = display.findWidget(handle);
@@ -295,7 +306,7 @@ private static DropTarget FindDropTarget(int handle) {
 	return (DropTarget)widget.getData(DROPTARGETID);
 }
 
-private static int TransferProcCallback(int widget, int client_data, int pSelection, int pType, int pValue, int pLength, int pFormat) {
+static int TransferProcCallback(int widget, int client_data, int pSelection, int pType, int pValue, int pLength, int pFormat) {
 	DropTarget target = FindDropTarget(client_data);
 	if (target != null) {
 		target.transferProcCallback(widget, client_data, pSelection, pType, pValue, pLength, pFormat);
@@ -353,14 +364,14 @@ protected void checkSubclass () {
 	}
 }
 
-private void dragProcCallback(int widget, int client_data, int call_data) {
+void dragProcCallback(int widget, int client_data, int call_data) {
 	if (call_data == 0) return;
 	XmDragProcCallbackStruct callbackData = new XmDragProcCallbackStruct();
 	OS.memmove(callbackData, call_data, XmDragProcCallbackStruct.sizeof);
 
 	if (callbackData.reason == OS.XmCR_DROP_SITE_LEAVE_MESSAGE) {
 		updateDragOverHover(0, null);
-		effect.show(DND.FEEDBACK_NONE, 0, 0);
+		effect.showDropTargetEffect(DND.FEEDBACK_NONE, 0, 0);
 		
 		if (callbackData.dropSiteStatus == OS.XmDROP_SITE_INVALID) {
 			return;
@@ -428,7 +439,7 @@ private void dragProcCallback(int widget, int client_data, int call_data) {
 		selectedOperation = event.detail;
 	}
 	
-	effect.show(event.feedback, event.x, event.y);
+	effect.showDropTargetEffect(event.feedback, event.x, event.y);
 
 	callbackData.dropSiteStatus = (byte)OS.XmDROP_SITE_VALID;
 	callbackData.operation = opToOsOp(selectedOperation);
@@ -439,7 +450,7 @@ private void dragProcCallback(int widget, int client_data, int call_data) {
 	}
 }
 
-private void dropProcCallback(int widget, int client_data, int call_data) {
+void dropProcCallback(int widget, int client_data, int call_data) {
 	if (call_data == 0) return;
 	droppedEventData = new XmDropProcCallbackStruct();
 	OS.memmove(droppedEventData, call_data, XmDropProcCallbackStruct.sizeof);	
@@ -522,47 +533,7 @@ public Transfer[] getTransfer() {
 	return transferAgents;
 }
 
-public void notifyListeners (int eventType, Event event) {
-	Point coordinates = new Point(event.x, event.y);
-	coordinates = control.toControl(coordinates);
-	if (this.control instanceof Tree) {
-		Tree tree = (Tree)control;
-		event.item = tree.getItem(coordinates);
-		if (event.item == null) {
-			Rectangle area = tree.getClientArea();
-			if (area.contains(coordinates)) {
-				// Scan across the width of the tree.
-				for (int x1 = area.x; x1 < area.x + area.width; x1++) {
-					Point pt = new Point(x1, coordinates.y);
-					event.item = tree.getItem(pt);
-					if (event.item != null) {
-						break;
-					}
-				}
-			}
-		}
-	}
-	if (this.control instanceof Table) {
-		Table table = (Table)control;
-		event.item = table.getItem(coordinates);
-		if (event.item == null) {
-			Rectangle area = table.getClientArea();
-			if (area.contains(coordinates)) {
-				// Scan across the width of the tree.
-				for (int x1 = area.x; x1 < area.x + area.width; x1++) {
-					Point pt = new Point(x1, coordinates.y);
-					event.item = table.getItem(pt);
-					if (event.item != null) {
-						break;
-					}
-				}
-			}
-		}
-	}
-	super.notifyListeners(eventType, event);
-}
-
-private void onDispose() {
+void onDispose() {
 	if (control == null) return;
 	if (controlListener != null) {
 		Control c = control;
@@ -578,7 +549,7 @@ private void onDispose() {
 	control = null;
 	transferAgents = null;
 }
-private byte opToOsOp(int operation){
+byte opToOsOp(int operation){
 	byte osOperation = OS.XmDROP_NOOP;
 	
 	if ((operation & DND.DROP_COPY) == DND.DROP_COPY)
@@ -590,7 +561,7 @@ private byte opToOsOp(int operation){
 	
 	return osOperation;
 }
-private int osOpToOp(byte osOperation){
+int osOpToOp(byte osOperation){
 	int operation = DND.DROP_NONE;
 	
 	if ((osOperation & OS.XmDROP_COPY) == OS.XmDROP_COPY)
@@ -602,7 +573,7 @@ private int osOpToOp(byte osOperation){
 	
 	return operation;
 }
-private void registerDropTarget() {
+void registerDropTarget() {
 	if (control == null || control.isDisposed() || registered) return;
 
 	int[] args = new int[]{
@@ -645,6 +616,10 @@ private void registerDropTarget() {
 	}
 	
 	OS.XmDropSiteRegister(control.handle, args, args.length / 2);
+	if (control instanceof Label) {
+		int formHandle = OS.XtParent (control.handle);
+		OS.XmDropSiteRegister(formHandle, args, args.length / 2);
+	}
 	registered = true;
 }
 
@@ -676,7 +651,7 @@ public void removeDropListener(DropTargetListener listener) {
 	removeListener (DND.DropAccept, listener);
 }
 
-private boolean setEventData(byte ops, byte op, int dragContext, short x, short y, int timestamp, DNDEvent event) {
+boolean setEventData(byte ops, byte op, int dragContext, short x, short y, int timestamp, DNDEvent event) {
 	//get allowed operations
 	int style = getStyle();
 	int operations = osOpToOp(ops) & style;
@@ -737,7 +712,7 @@ private boolean setEventData(byte ops, byte op, int dragContext, short x, short
 
 	short [] root_x = new short [1];
 	short [] root_y = new short [1];	
-	OS.XtTranslateCoords (this.control.handle, (short) x, (short) y, root_x, root_y);
+	OS.XtTranslateCoords (this.control.handle, x, y, root_x, root_y);
 	
 	event.widget = this;
 	event.x = root_x[0];
@@ -748,7 +723,7 @@ private boolean setEventData(byte ops, byte op, int dragContext, short x, short
 	event.dataType = dataTypes[0];
 	event.operations = operations;
 	event.detail = operation;
-	
+	event.item = effect.getItem(event.x, event.y);
 	return true;
 }
 
@@ -796,11 +771,15 @@ public void setTransfer(Transfer[] transferAgents){
 	};
 
 	OS.XmDropSiteUpdate(control.handle, args, args.length / 2);
+	if (control instanceof Label) {
+		int formHandle = OS.XtParent (control.handle);
+		OS.XmDropSiteUpdate(formHandle, args, args.length / 2);
+	}
 	
 	OS.XtFree(pImportTargets);
 	
 }
-private void transferProcCallback(int widget, int client_data, int pSelection, int pType, int pValue, int pLength, int pFormat) {
+void transferProcCallback(int widget, int client_data, int pSelection, int pType, int pValue, int pLength, int pFormat) {
 	if (pType == 0 || pValue == 0 || pLength == 0 || pFormat == 0) return;
 	
 	int[] type = new int[1];
@@ -859,13 +838,18 @@ private void transferProcCallback(int widget, int client_data, int pSelection, i
 	}
 }
 
-private void unregisterDropTarget() {
+void unregisterDropTarget() {
 	if (control == null || control.isDisposed() || !registered) return;
 	OS.XmDropSiteUnregister(control.handle);
+	if (control instanceof Label) {
+		int formHandle = OS.XtParent(control.handle);
+		OS.XmDropSiteUnregister(formHandle);
+	}
+	
 	registered = false;
 }
 
-private void updateDragOverHover(long delay, DNDEvent event) {
+void updateDragOverHover(long delay, DNDEvent event) {
 	if (delay == 0) {
 		dragOverStart = 0;
 		dragOverEvent = null;
