@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -58,6 +58,9 @@ final public class OleFrame extends Composite
 	private static String CHECK_FOCUS = "OLE_CHECK_FOCUS"; //$NON-NLS-1$
 	private static String HHOOK = "OLE_HHOOK"; //$NON-NLS-1$
 	private static String HHOOKMSG = "OLE_HHOOK_MSG"; //$NON-NLS-1$
+
+	private static boolean ignoreNextKey;
+	private static final short [] ACCENTS = new short [] {'~', '`', '\'', '^', '"'};
 	
 /**
  * Create an OleFrame child widget using style bits
@@ -66,15 +69,16 @@ final public class OleFrame extends Composite
  * @param parent a composite widget (cannot be null)
  * @param style the bitwise OR'ing of widget styles
  *
- * @exception SWTError
- * <ul><li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
- *     <li>ERROR_ERROR_NULL_ARGUMENT when the parent is null
- *     <li>ERROR_INTERFACES_NOT_INITIALIZED when unable to create callbacks for OLE Interfaces</ul>
+ * @exception IllegalArgumentException <ul>
+ *     <li>ERROR_NULL_ARGUMENT when the parent is null
+ * </ul>
+ * @exception SWTException <ul>
+ *     <li>ERROR_THREAD_INVALID_ACCESS when called from the wrong thread
+ * </ul>
  *
  */
 public OleFrame(Composite parent, int style) {
-	// set up widget attributes
-	super(parent, style | SWT.CLIP_CHILDREN | SWT.CLIP_SIBLINGS);
+	super(parent, style);
 	
 	createCOMInterfaces();
 
@@ -121,7 +125,7 @@ private static void initCheckFocus (final Display display) {
 				// ignore popup menus and dialogs
 				int hwnd = OS.GetFocus();
 				while (hwnd != 0) {
-					int ownerHwnd = OS.GetWindow(hwnd, 4/*GW_OWNER*/);
+					int ownerHwnd = OS.GetWindow(hwnd, OS.GW_OWNER);
 					if (ownerHwnd != 0) {			
 						display.timerExec(time, timer[0]);
 						return;
@@ -155,6 +159,7 @@ private static void initMsgHook(Display display) {
 	if (display.getData(HHOOK) != null) return;
 	final Callback callback = new Callback(OleFrame.class, "getMsgProc", 3); //$NON-NLS-1$
 	int address = callback.getAddress();
+	if (address == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
 	int threadId = OS.GetCurrentThreadId();
 	final int hHook = OS.SetWindowsHookEx(OS.WH_GETMESSAGE, address, 0, threadId);
 	if (hHook == 0) {
@@ -193,11 +198,65 @@ static int getMsgProc(int code, int wParam, int lParam) {
 			if (widget != null && widget instanceof OleClientSite) {
 				OleClientSite site = (OleClientSite)widget;
 				if (site.handle == hwnd) {
-					OleFrame frame = site.frame;
-					if (frame.translateOleAccelerator(msg)) {
+					boolean consumed = false;
+					/* Allow activeX control to translate accelerators except when a menu is active. */
+					int thread = OS.GetWindowThreadProcessId(msg.hwnd, null);
+					GUITHREADINFO  lpgui = new GUITHREADINFO();
+					lpgui.cbSize = GUITHREADINFO.sizeof;
+					boolean rc = OS.GetGUIThreadInfo(thread, lpgui);
+					int mask = OS.GUI_INMENUMODE | OS.GUI_INMOVESIZE | OS.GUI_POPUPMENUMODE | OS.GUI_SYSTEMMENUMODE;
+					if (!rc || (lpgui.flags & mask) == 0) {
+						OleFrame frame = site.frame;
+						consumed = frame.translateOleAccelerator(msg);
+					}
+					boolean accentKey = false;
+					switch (msg.message) {
+						case OS.WM_KEYDOWN:
+						case OS.WM_SYSKEYDOWN:
+							if (!OS.IsWinCE) {
+								/* 
+								* Bug in Windows. The high bit in the result of MapVirtualKey() on
+								* Windows NT is bit 32 while the high bit on Windows 95 is bit 16.
+								* They should both be bit 32.  The fix is to test the right bit.
+								*/
+								int mapKey = OS.MapVirtualKey (msg.wParam, 2);
+								if (mapKey != 0) {
+									accentKey = (mapKey & (OS.IsWinNT ? 0x80000000 : 0x8000)) != 0;
+									if (!accentKey) {
+										for (int i=0; i<ACCENTS.length; i++) {
+											int value = OS.VkKeyScan (ACCENTS [i]);
+											if (value != -1 && (value & 0xFF) == msg.wParam) {
+												int state = value >> 8;
+												if ((OS.GetKeyState (OS.VK_SHIFT) < 0) == ((state & 0x1) != 0) &&
+													(OS.GetKeyState (OS.VK_CONTROL) < 0) == ((state & 0x2) != 0) &&
+													(OS.GetKeyState (OS.VK_MENU) < 0) == ((state & 0x4) != 0)) {
+														if ((state & 0x7) != 0) accentKey = true;
+														break;
+												}
+											}
+										}
+									}
+								}
+							}
+					}
+					/* Allow OleClientSite to process key events before activeX control */
+					if (!consumed && !accentKey && !ignoreNextKey) {
+						int hwndOld = msg.hwnd;
+						msg.hwnd = site.handle;
+						consumed = OS.DispatchMessage (msg) == 1;
+						msg.hwnd = hwndOld;
+					}
+					switch (msg.message) {
+						case OS.WM_KEYDOWN:
+						case OS.WM_SYSKEYDOWN: ignoreNextKey = accentKey;
+					}
+
+					if (consumed) {
 						// In order to prevent this message from also being processed
 						// by the application, zero out message, wParam and lParam
-						OS.MoveMemory(lParam + 4, new int[] {OS.WM_NULL, 0, 0}, 12);
+						msg.message = OS.WM_NULL;
+						msg.wParam = msg.lParam = 0;
+						OS.MoveMemory(lParam, msg, MSG.sizeof);
 						return 0;
 					}
 				}
