@@ -1,17 +1,17 @@
 /*******************************************************************************
-* Copyright (c) 2000, 2005 IBM Corporation and others. All rights reserved.
-* The contents of this file are made available under the terms
-* of the GNU Lesser General Public License (LGPL) Version 2.1 that
-* accompanies this distribution (lgpl-v21.txt).  The LGPL is also
-* available at http://www.gnu.org/licenses/lgpl.html.  If the version
-* of the LGPL at http://www.gnu.org is different to the version of
-* the LGPL accompanying this distribution and there is any conflict
-* between the two license versions, the terms of the LGPL accompanying
-* this distribution shall govern.
-* 
-* Contributors:
-*     IBM Corporation - initial API and implementation
-*******************************************************************************/
+ * Copyright (c) 2000, 2006 IBM Corporation and others. All rights reserved.
+ * The contents of this file are made available under the terms
+ * of the GNU Lesser General Public License (LGPL) Version 2.1 that
+ * accompanies this distribution (lgpl-v21.txt).  The LGPL is also
+ * available at http://www.gnu.org/licenses/lgpl.html.  If the version
+ * of the LGPL at http://www.gnu.org is different to the version of
+ * the LGPL accompanying this distribution and there is any conflict
+ * between the two license versions, the terms of the LGPL accompanying
+ * this distribution shall govern.
+ * 
+ * Contributors:
+ *     IBM Corporation - initial API and implementation
+ *******************************************************************************/
 
 #ifdef NATIVE_STATS
 extern int OS_nativeFunctionCount;
@@ -26,6 +26,8 @@ extern char* OS_nativeFunctionNames[];
 
 typedef enum {
 	Call_FUNC,
+	GDK_1EVENT_1TYPE_FUNC,
+	GDK_1EVENT_1WINDOW_FUNC,
 	GDK_1WINDOWING_1X11_FUNC,
 	GInterfaceInfo_1sizeof_FUNC,
 	GPollFD_1sizeof_FUNC,
@@ -36,6 +38,8 @@ typedef enum {
 	GTK_1SCROLLED_1WINDOW_1SCROLLBAR_1SPACING_FUNC,
 	GTK_1SCROLLED_1WINDOW_1VSCROLLBAR_FUNC,
 	GTK_1TEXTVIEW_1IM_1CONTEXT_FUNC,
+	GTK_1TOOLTIPS_1SET_1ACTIVE_FUNC,
+	GTK_1TOOLTIPS_1TIP_1WINDOW_FUNC,
 	GTK_1WIDGET_1HEIGHT_FUNC,
 	GTK_1WIDGET_1REQUISITION_1HEIGHT_FUNC,
 	GTK_1WIDGET_1REQUISITION_1WIDTH_FUNC,
@@ -51,6 +55,7 @@ typedef enum {
 	GTypeQuery_1sizeof_FUNC,
 	GdkColor_1sizeof_FUNC,
 	GdkDragContext_1sizeof_FUNC,
+	GdkEventAny_1sizeof_FUNC,
 	GdkEventButton_1sizeof_FUNC,
 	GdkEventCrossing_1sizeof_FUNC,
 	GdkEventExpose_1sizeof_FUNC,
@@ -70,6 +75,12 @@ typedef enum {
 	GtkAdjustment_1sizeof_FUNC,
 	GtkAllocation_1sizeof_FUNC,
 	GtkBorder_1sizeof_FUNC,
+	GtkCellRendererPixbufClass_1sizeof_FUNC,
+	GtkCellRendererPixbuf_1sizeof_FUNC,
+	GtkCellRendererTextClass_1sizeof_FUNC,
+	GtkCellRendererText_1sizeof_FUNC,
+	GtkCellRendererToggleClass_1sizeof_FUNC,
+	GtkCellRendererToggle_1sizeof_FUNC,
 	GtkColorSelectionDialog_1sizeof_FUNC,
 	GtkCombo_1sizeof_FUNC,
 	GtkFileSelection_1sizeof_FUNC,
@@ -95,8 +106,11 @@ typedef enum {
 	XEvent_1sizeof_FUNC,
 	XExposeEvent_1sizeof_FUNC,
 	XFocusChangeEvent_1sizeof_FUNC,
+	XRenderPictureAttributes_1sizeof_FUNC,
 	XVisibilityEvent_1sizeof_FUNC,
 	XWindowChanges_1sizeof_FUNC,
+	X_1EVENT_1TYPE_FUNC,
+	X_1EVENT_1WINDOW_FUNC,
 	_1Call_FUNC,
 	_1GDK_1DISPLAY_FUNC,
 	_1GDK_1PIXMAP_1XID_FUNC,
@@ -106,11 +120,17 @@ typedef enum {
 	_1GTK_1IS_1BUTTON_FUNC,
 	_1GTK_1IS_1CELL_1RENDERER_1PIXBUF_FUNC,
 	_1GTK_1IS_1CELL_1RENDERER_1TEXT_FUNC,
+	_1GTK_1IS_1CELL_1RENDERER_1TOGGLE_FUNC,
 	_1GTK_1IS_1CONTAINER_FUNC,
 	_1GTK_1IS_1IMAGE_1MENU_1ITEM_FUNC,
+	_1GTK_1IS_1PLUG_FUNC,
 	_1GTK_1STOCK_1CANCEL_FUNC,
 	_1GTK_1STOCK_1OK_FUNC,
+	_1GTK_1TYPE_1CELL_1RENDERER_1PIXBUF_FUNC,
+	_1GTK_1TYPE_1CELL_1RENDERER_1TEXT_FUNC,
+	_1GTK_1TYPE_1CELL_1RENDERER_1TOGGLE_FUNC,
 	_1GTK_1TYPE_1FIXED_FUNC,
+	_1GTK_1TYPE_1WIDGET_FUNC,
 	_1GTK_1WIDGET_1FLAGS_FUNC,
 	_1GTK_1WIDGET_1HAS_1DEFAULT_FUNC,
 	_1GTK_1WIDGET_1HAS_1FOCUS_FUNC,
@@ -142,6 +162,15 @@ typedef enum {
 	_1XListProperties_FUNC,
 	_1XQueryTree_FUNC,
 	_1XReconfigureWMWindow_FUNC,
+	_1XRenderComposite_FUNC,
+	_1XRenderCreatePicture_FUNC,
+	_1XRenderFindStandardFormat_FUNC,
+	_1XRenderFindVisualFormat_FUNC,
+	_1XRenderFreePicture_FUNC,
+	_1XRenderQueryExtension_FUNC,
+	_1XRenderQueryVersion_FUNC,
+	_1XRenderSetPictureClipRectangles_FUNC,
+	_1XRenderSetPictureTransform_FUNC,
 	_1XSendEvent_FUNC,
 	_1XSetErrorHandler_FUNC,
 	_1XSetIOErrorHandler_FUNC,
@@ -151,6 +180,14 @@ typedef enum {
 	_1XTestFakeKeyEvent_FUNC,
 	_1XTestFakeMotionEvent_FUNC,
 	_1XWarpPointer_FUNC,
+	_1atk_1object_1add_1relationship_FUNC,
+	_1call_FUNC,
+	_1dlclose_FUNC,
+	_1dlopen_FUNC,
+	_1dlsym_FUNC,
+	_1g_1cclosure_1new_FUNC,
+	_1g_1closure_1ref_FUNC,
+	_1g_1closure_1unref_FUNC,
 	_1g_1filename_1from_1uri_FUNC,
 	_1g_1filename_1from_1utf8_FUNC,
 	_1g_1filename_1to_1uri_FUNC,
@@ -199,6 +236,8 @@ typedef enum {
 	_1g_1set_1prgname_FUNC,
 	_1g_1signal_1connect_FUNC,
 	_1g_1signal_1connect_1after_FUNC,
+	_1g_1signal_1connect_1closure_FUNC,
+	_1g_1signal_1connect_1closure_1by_1id_FUNC,
 	_1g_1signal_1emit_1by_1name__I_3B_FUNC,
 	_1g_1signal_1emit_1by_1name__I_3BI_FUNC,
 	_1g_1signal_1emit_1by_1name__I_3BII_FUNC,
@@ -236,6 +275,8 @@ typedef enum {
 	_1gdk_1atom_1name_FUNC,
 	_1gdk_1beep_FUNC,
 	_1gdk_1bitmap_1create_1from_1data_FUNC,
+	_1gdk_1cairo_1region_FUNC,
+	_1gdk_1cairo_1set_1source_1color_FUNC,
 	_1gdk_1color_1white_FUNC,
 	_1gdk_1colormap_1alloc_1color_FUNC,
 	_1gdk_1colormap_1free_1colors_FUNC,
@@ -247,6 +288,7 @@ typedef enum {
 	_1gdk_1drag_1status_FUNC,
 	_1gdk_1draw_1arc_FUNC,
 	_1gdk_1draw_1drawable_FUNC,
+	_1gdk_1draw_1image_FUNC,
 	_1gdk_1draw_1layout_FUNC,
 	_1gdk_1draw_1layout_1with_1colors_FUNC,
 	_1gdk_1draw_1line_FUNC,
@@ -255,6 +297,7 @@ typedef enum {
 	_1gdk_1draw_1point_FUNC,
 	_1gdk_1draw_1polygon_FUNC,
 	_1gdk_1draw_1rectangle_FUNC,
+	_1gdk_1drawable_1get_1depth_FUNC,
 	_1gdk_1drawable_1get_1image_FUNC,
 	_1gdk_1drawable_1get_1size_FUNC,
 	_1gdk_1drawable_1get_1visible_1region_FUNC,
@@ -287,9 +330,9 @@ typedef enum {
 	_1gdk_1gc_1set_1line_1attributes_FUNC,
 	_1gdk_1gc_1set_1stipple_FUNC,
 	_1gdk_1gc_1set_1subwindow_FUNC,
+	_1gdk_1gc_1set_1tile_FUNC,
+	_1gdk_1gc_1set_1ts_1origin_FUNC,
 	_1gdk_1gc_1set_1values_FUNC,
-	_1gdk_1image_1get_FUNC,
-	_1gdk_1image_1get_1pixel_FUNC,
 	_1gdk_1keyboard_1ungrab_FUNC,
 	_1gdk_1keymap_1get_1default_FUNC,
 	_1gdk_1keymap_1translate_1keyboard_1state_FUNC,
@@ -306,6 +349,7 @@ typedef enum {
 	_1gdk_1pixbuf_1get_1rowstride_FUNC,
 	_1gdk_1pixbuf_1get_1width_FUNC,
 	_1gdk_1pixbuf_1new_FUNC,
+	_1gdk_1pixbuf_1new_1from_1file_FUNC,
 	_1gdk_1pixbuf_1render_1pixmap_1and_1mask_FUNC,
 	_1gdk_1pixbuf_1render_1to_1drawable_FUNC,
 	_1gdk_1pixbuf_1render_1to_1drawable_1alpha_FUNC,
@@ -344,10 +388,12 @@ typedef enum {
 	_1gdk_1text_1property_1to_1utf8_1list_FUNC,
 	_1gdk_1unicode_1to_1keyval_FUNC,
 	_1gdk_1utf8_1to_1compound_1text_FUNC,
+	_1gdk_1utf8_1to_1string_1target_FUNC,
 	_1gdk_1visual_1get_1system_FUNC,
 	_1gdk_1window_1add_1filter_FUNC,
 	_1gdk_1window_1at_1pointer_FUNC,
 	_1gdk_1window_1begin_1paint_1rect_FUNC,
+	_1gdk_1window_1clear_1area_FUNC,
 	_1gdk_1window_1destroy_FUNC,
 	_1gdk_1window_1end_1paint_FUNC,
 	_1gdk_1window_1focus_FUNC,
@@ -376,6 +422,7 @@ typedef enum {
 	_1gdk_1window_1set_1accept_1focus_FUNC,
 	_1gdk_1window_1set_1back_1pixmap_FUNC,
 	_1gdk_1window_1set_1cursor_FUNC,
+	_1gdk_1window_1set_1debug_1updates_FUNC,
 	_1gdk_1window_1set_1decorations_FUNC,
 	_1gdk_1window_1set_1events_FUNC,
 	_1gdk_1window_1set_1icon_FUNC,
@@ -392,6 +439,7 @@ typedef enum {
 	_1gdk_1x11_1drawable_1get_1xdisplay_FUNC,
 	_1gdk_1x11_1drawable_1get_1xid_FUNC,
 	_1gdk_1x11_1screen_1get_1window_1manager_1name_FUNC,
+	_1gdk_1x11_1screen_1lookup_1visual_FUNC,
 	_1gdk_1x11_1visual_1get_1xvisual_FUNC,
 	_1gtk_1accel_1group_1new_FUNC,
 	_1gtk_1accel_1groups_1activate_FUNC,
@@ -404,7 +452,9 @@ typedef enum {
 	_1gtk_1arrow_1set_FUNC,
 	_1gtk_1bin_1get_1child_FUNC,
 	_1gtk_1box_1set_1child_1packing_FUNC,
+	_1gtk_1box_1set_1spacing_FUNC,
 	_1gtk_1button_1clicked_FUNC,
+	_1gtk_1button_1get_1relief_FUNC,
 	_1gtk_1button_1new_FUNC,
 	_1gtk_1button_1set_1relief_FUNC,
 	_1gtk_1cell_1renderer_1get_1size_FUNC,
@@ -444,6 +494,7 @@ typedef enum {
 	_1gtk_1drag_1dest_1unset_FUNC,
 	_1gtk_1drag_1finish_FUNC,
 	_1gtk_1drag_1get_1data_FUNC,
+	_1gtk_1drag_1set_1icon_1pixbuf_FUNC,
 	_1gtk_1drawing_1area_1new_FUNC,
 	_1gtk_1editable_1copy_1clipboard_FUNC,
 	_1gtk_1editable_1cut_1clipboard_FUNC,
@@ -460,6 +511,7 @@ typedef enum {
 	_1gtk_1editable_1set_1position_FUNC,
 	_1gtk_1entry_1get_1invisible_1char_FUNC,
 	_1gtk_1entry_1get_1layout_FUNC,
+	_1gtk_1entry_1get_1layout_1offsets_FUNC,
 	_1gtk_1entry_1get_1max_1length_FUNC,
 	_1gtk_1entry_1get_1text_FUNC,
 	_1gtk_1entry_1get_1visibility_FUNC,
@@ -471,7 +523,14 @@ typedef enum {
 	_1gtk_1entry_1set_1max_1length_FUNC,
 	_1gtk_1entry_1set_1text_FUNC,
 	_1gtk_1entry_1set_1visibility_FUNC,
+	_1gtk_1entry_1text_1index_1to_1layout_1index_FUNC,
 	_1gtk_1events_1pending_FUNC,
+	_1gtk_1expander_1get_1expanded_FUNC,
+	_1gtk_1expander_1get_1label_1widget_FUNC,
+	_1gtk_1expander_1new_FUNC,
+	_1gtk_1expander_1set_1expanded_FUNC,
+	_1gtk_1expander_1set_1label_FUNC,
+	_1gtk_1expander_1set_1label_1widget_FUNC,
 	_1gtk_1file_1chooser_1add_1filter_FUNC,
 	_1gtk_1file_1chooser_1dialog_1new_FUNC,
 	_1gtk_1file_1chooser_1get_1current_1folder_FUNC,
@@ -516,6 +575,9 @@ typedef enum {
 	_1gtk_1hseparator_1new_FUNC,
 	_1gtk_1icon_1factory_1lookup_1default_FUNC,
 	_1gtk_1icon_1set_1render_1icon_FUNC,
+	_1gtk_1icon_1source_1free_FUNC,
+	_1gtk_1icon_1source_1new_FUNC,
+	_1gtk_1icon_1source_1set_1pixbuf_FUNC,
 	_1gtk_1im_1context_1filter_1keypress_FUNC,
 	_1gtk_1im_1context_1focus_1in_FUNC,
 	_1gtk_1im_1context_1focus_1out_FUNC,
@@ -534,6 +596,7 @@ typedef enum {
 	_1gtk_1image_1set_1from_1pixbuf_FUNC,
 	_1gtk_1image_1set_1from_1pixmap_FUNC,
 	_1gtk_1init_1check_FUNC,
+	_1gtk_1label_1get_1layout_FUNC,
 	_1gtk_1label_1get_1mnemonic_1keyval_FUNC,
 	_1gtk_1label_1new_FUNC,
 	_1gtk_1label_1new_1with_1mnemonic_FUNC,
@@ -588,8 +651,23 @@ typedef enum {
 	_1gtk_1notebook_1set_1show_1tabs_FUNC,
 	_1gtk_1notebook_1set_1tab_1pos_FUNC,
 	_1gtk_1object_1sink_FUNC,
+	_1gtk_1paint_1arrow_FUNC,
+	_1gtk_1paint_1box_FUNC,
+	_1gtk_1paint_1box_1gap_FUNC,
+	_1gtk_1paint_1check_FUNC,
+	_1gtk_1paint_1expander_FUNC,
+	_1gtk_1paint_1extension_FUNC,
+	_1gtk_1paint_1flat_1box_FUNC,
 	_1gtk_1paint_1focus_FUNC,
 	_1gtk_1paint_1handle_FUNC,
+	_1gtk_1paint_1hline_FUNC,
+	_1gtk_1paint_1layout_FUNC,
+	_1gtk_1paint_1option_FUNC,
+	_1gtk_1paint_1shadow_FUNC,
+	_1gtk_1paint_1shadow_1gap_FUNC,
+	_1gtk_1paint_1slider_FUNC,
+	_1gtk_1paint_1tab_FUNC,
+	_1gtk_1paint_1vline_FUNC,
 	_1gtk_1plug_1get_1id_FUNC,
 	_1gtk_1plug_1new_FUNC,
 	_1gtk_1progress_1bar_1new_FUNC,
@@ -614,6 +692,7 @@ typedef enum {
 	_1gtk_1rc_1style_1set_1color_1flags_FUNC,
 	_1gtk_1scale_1set_1digits_FUNC,
 	_1gtk_1scale_1set_1draw_1value_FUNC,
+	_1gtk_1scrolled_1window_1add_1with_1viewport_FUNC,
 	_1gtk_1scrolled_1window_1get_1hadjustment_FUNC,
 	_1gtk_1scrolled_1window_1get_1policy_FUNC,
 	_1gtk_1scrolled_1window_1get_1shadow_1type_FUNC,
@@ -639,14 +718,24 @@ typedef enum {
 	_1gtk_1spin_1button_1set_1wrap_FUNC,
 	_1gtk_1style_1get_1base_FUNC,
 	_1gtk_1style_1get_1bg_FUNC,
+	_1gtk_1style_1get_1bg_1gc_FUNC,
 	_1gtk_1style_1get_1black_FUNC,
+	_1gtk_1style_1get_1black_1gc_FUNC,
 	_1gtk_1style_1get_1dark_FUNC,
+	_1gtk_1style_1get_1dark_1gc_FUNC,
 	_1gtk_1style_1get_1fg_FUNC,
+	_1gtk_1style_1get_1fg_1gc_FUNC,
 	_1gtk_1style_1get_1font_1desc_FUNC,
 	_1gtk_1style_1get_1light_FUNC,
+	_1gtk_1style_1get_1light_1gc_FUNC,
+	_1gtk_1style_1get_1mid_1gc_FUNC,
 	_1gtk_1style_1get_1text_FUNC,
+	_1gtk_1style_1get_1text_1aa_1gc_FUNC,
+	_1gtk_1style_1get_1text_1gc_FUNC,
+	_1gtk_1style_1get_1white_1gc_FUNC,
 	_1gtk_1style_1get_1xthickness_FUNC,
 	_1gtk_1style_1get_1ythickness_FUNC,
+	_1gtk_1style_1render_1icon_FUNC,
 	_1gtk_1target_1list_1new_FUNC,
 	_1gtk_1target_1list_1unref_FUNC,
 	_1gtk_1text_1buffer_1copy_1clipboard_FUNC,
@@ -696,8 +785,10 @@ typedef enum {
 	_1gtk_1toolbar_1insert_1widget_FUNC,
 	_1gtk_1toolbar_1new_FUNC,
 	_1gtk_1toolbar_1set_1orientation_FUNC,
+	_1gtk_1tooltips_1data_1get_FUNC,
 	_1gtk_1tooltips_1disable_FUNC,
 	_1gtk_1tooltips_1enable_FUNC,
+	_1gtk_1tooltips_1force_1window_FUNC,
 	_1gtk_1tooltips_1new_FUNC,
 	_1gtk_1tooltips_1set_1tip_FUNC,
 	_1gtk_1tree_1model_1get__III_3II_FUNC,
@@ -724,6 +815,7 @@ typedef enum {
 	_1gtk_1tree_1path_1next_FUNC,
 	_1gtk_1tree_1path_1prev_FUNC,
 	_1gtk_1tree_1path_1up_FUNC,
+	_1gtk_1tree_1selection_1count_1selected_1rows_FUNC,
 	_1gtk_1tree_1selection_1get_1selected_FUNC,
 	_1gtk_1tree_1selection_1get_1selected_1rows_FUNC,
 	_1gtk_1tree_1selection_1path_1is_1selected_FUNC,
@@ -753,6 +845,9 @@ typedef enum {
 	_1gtk_1tree_1view_1column_1get_1fixed_1width_FUNC,
 	_1gtk_1tree_1view_1column_1get_1reorderable_FUNC,
 	_1gtk_1tree_1view_1column_1get_1resizable_FUNC,
+	_1gtk_1tree_1view_1column_1get_1sizing_FUNC,
+	_1gtk_1tree_1view_1column_1get_1sort_1indicator_FUNC,
+	_1gtk_1tree_1view_1column_1get_1sort_1order_FUNC,
 	_1gtk_1tree_1view_1column_1get_1spacing_FUNC,
 	_1gtk_1tree_1view_1column_1get_1visible_FUNC,
 	_1gtk_1tree_1view_1column_1get_1width_FUNC,
@@ -763,13 +858,18 @@ typedef enum {
 	_1gtk_1tree_1view_1column_1set_1cell_1data_1func_FUNC,
 	_1gtk_1tree_1view_1column_1set_1clickable_FUNC,
 	_1gtk_1tree_1view_1column_1set_1fixed_1width_FUNC,
+	_1gtk_1tree_1view_1column_1set_1min_1width_FUNC,
 	_1gtk_1tree_1view_1column_1set_1reorderable_FUNC,
 	_1gtk_1tree_1view_1column_1set_1resizable_FUNC,
 	_1gtk_1tree_1view_1column_1set_1sizing_FUNC,
+	_1gtk_1tree_1view_1column_1set_1sort_1indicator_FUNC,
+	_1gtk_1tree_1view_1column_1set_1sort_1order_FUNC,
 	_1gtk_1tree_1view_1column_1set_1title_FUNC,
 	_1gtk_1tree_1view_1column_1set_1visible_FUNC,
 	_1gtk_1tree_1view_1column_1set_1widget_FUNC,
+	_1gtk_1tree_1view_1create_1row_1drag_1icon_FUNC,
 	_1gtk_1tree_1view_1expand_1row_FUNC,
+	_1gtk_1tree_1view_1get_1background_1area_FUNC,
 	_1gtk_1tree_1view_1get_1bin_1window_FUNC,
 	_1gtk_1tree_1view_1get_1cell_1area_FUNC,
 	_1gtk_1tree_1view_1get_1column_FUNC,
@@ -790,6 +890,7 @@ typedef enum {
 	_1gtk_1tree_1view_1scroll_1to_1point_FUNC,
 	_1gtk_1tree_1view_1set_1cursor_FUNC,
 	_1gtk_1tree_1view_1set_1drag_1dest_1row_FUNC,
+	_1gtk_1tree_1view_1set_1enable_1search_FUNC,
 	_1gtk_1tree_1view_1set_1headers_1visible_FUNC,
 	_1gtk_1tree_1view_1set_1model_FUNC,
 	_1gtk_1tree_1view_1set_1rules_1hint_FUNC,
@@ -808,6 +909,7 @@ typedef enum {
 	_1gtk_1widget_1create_1pango_1layout__I_3B_FUNC,
 	_1gtk_1widget_1destroy_FUNC,
 	_1gtk_1widget_1event_FUNC,
+	_1gtk_1widget_1get_1accessible_FUNC,
 	_1gtk_1widget_1get_1child_1visible_FUNC,
 	_1gtk_1widget_1get_1default_1direction_FUNC,
 	_1gtk_1widget_1get_1default_1style_FUNC,
@@ -830,17 +932,21 @@ typedef enum {
 	_1gtk_1widget_1modify_1font_FUNC,
 	_1gtk_1widget_1modify_1style_FUNC,
 	_1gtk_1widget_1modify_1text_FUNC,
+	_1gtk_1widget_1queue_1resize_FUNC,
 	_1gtk_1widget_1realize_FUNC,
 	_1gtk_1widget_1remove_1accelerator_FUNC,
 	_1gtk_1widget_1reparent_FUNC,
+	_1gtk_1widget_1set_1app_1paintable_FUNC,
 	_1gtk_1widget_1set_1default_1direction_FUNC,
 	_1gtk_1widget_1set_1direction_FUNC,
 	_1gtk_1widget_1set_1double_1buffered_FUNC,
 	_1gtk_1widget_1set_1name_FUNC,
+	_1gtk_1widget_1set_1parent_1window_FUNC,
 	_1gtk_1widget_1set_1redraw_1on_1allocate_FUNC,
 	_1gtk_1widget_1set_1sensitive_FUNC,
 	_1gtk_1widget_1set_1size_1request_FUNC,
 	_1gtk_1widget_1set_1state_FUNC,
+	_1gtk_1widget_1set_1style_FUNC,
 	_1gtk_1widget_1shape_1combine_1mask_FUNC,
 	_1gtk_1widget_1show_FUNC,
 	_1gtk_1widget_1show_1now_FUNC,
@@ -848,6 +954,7 @@ typedef enum {
 	_1gtk_1widget_1size_1request_FUNC,
 	_1gtk_1widget_1style_1get__I_3B_3II_FUNC,
 	_1gtk_1widget_1style_1get__I_3B_3JI_FUNC,
+	_1gtk_1widget_1translate_1coordinates_FUNC,
 	_1gtk_1widget_1unrealize_FUNC,
 	_1gtk_1window_1activate_1default_FUNC,
 	_1gtk_1window_1add_1accel_1group_FUNC,
@@ -881,10 +988,17 @@ typedef enum {
 	_1pango_1attr_1list_1insert_FUNC,
 	_1pango_1attr_1list_1new_FUNC,
 	_1pango_1attr_1list_1unref_FUNC,
+	_1pango_1attr_1rise_1new_FUNC,
 	_1pango_1attr_1shape_1new_FUNC,
 	_1pango_1attr_1strikethrough_1new_FUNC,
 	_1pango_1attr_1underline_1new_FUNC,
 	_1pango_1attr_1weight_1new_FUNC,
+	_1pango_1cairo_1create_1layout_FUNC,
+	_1pango_1cairo_1font_1map_1create_1context_FUNC,
+	_1pango_1cairo_1font_1map_1get_1default_FUNC,
+	_1pango_1cairo_1font_1map_1new_FUNC,
+	_1pango_1cairo_1layout_1path_FUNC,
+	_1pango_1cairo_1show_1layout_FUNC,
 	_1pango_1context_1get_1base_1dir_FUNC,
 	_1pango_1context_1get_1language_FUNC,
 	_1pango_1context_1get_1metrics_FUNC,
@@ -917,7 +1031,10 @@ typedef enum {
 	_1pango_1layout_1context_1changed_FUNC,
 	_1pango_1layout_1get_1alignment_FUNC,
 	_1pango_1layout_1get_1attributes_FUNC,
+	_1pango_1layout_1get_1context_FUNC,
+	_1pango_1layout_1get_1indent_FUNC,
 	_1pango_1layout_1get_1iter_FUNC,
+	_1pango_1layout_1get_1justify_FUNC,
 	_1pango_1layout_1get_1line_FUNC,
 	_1pango_1layout_1get_1line_1count_FUNC,
 	_1pango_1layout_1get_1log_1attrs_FUNC,
@@ -938,7 +1055,10 @@ typedef enum {
 	_1pango_1layout_1new_FUNC,
 	_1pango_1layout_1set_1alignment_FUNC,
 	_1pango_1layout_1set_1attributes_FUNC,
+	_1pango_1layout_1set_1auto_1dir_FUNC,
 	_1pango_1layout_1set_1font_1description_FUNC,
+	_1pango_1layout_1set_1indent_FUNC,
+	_1pango_1layout_1set_1justify_FUNC,
 	_1pango_1layout_1set_1single_1paragraph_1mode_FUNC,
 	_1pango_1layout_1set_1spacing_FUNC,
 	_1pango_1layout_1set_1tabs_FUNC,
@@ -960,6 +1080,7 @@ typedef enum {
 	memmove__ILorg_eclipse_swt_internal_gtk_GTypeInfo_2I_FUNC,
 	memmove__ILorg_eclipse_swt_internal_gtk_GdkEventButton_2I_FUNC,
 	memmove__ILorg_eclipse_swt_internal_gtk_GtkAdjustment_2_FUNC,
+	memmove__ILorg_eclipse_swt_internal_gtk_GtkCellRendererClass_2_FUNC,
 	memmove__ILorg_eclipse_swt_internal_gtk_GtkFixed_2_FUNC,
 	memmove__ILorg_eclipse_swt_internal_gtk_GtkTargetEntry_2I_FUNC,
 	memmove__ILorg_eclipse_swt_internal_gtk_GtkWidgetClass_2_FUNC,
@@ -978,6 +1099,7 @@ typedef enum {
 	memmove__Lorg_eclipse_swt_internal_gtk_GTypeQuery_2II_FUNC,
 	memmove__Lorg_eclipse_swt_internal_gtk_GdkColor_2II_FUNC,
 	memmove__Lorg_eclipse_swt_internal_gtk_GdkDragContext_2II_FUNC,
+	memmove__Lorg_eclipse_swt_internal_gtk_GdkEventAny_2II_FUNC,
 	memmove__Lorg_eclipse_swt_internal_gtk_GdkEventButton_2II_FUNC,
 	memmove__Lorg_eclipse_swt_internal_gtk_GdkEventCrossing_2II_FUNC,
 	memmove__Lorg_eclipse_swt_internal_gtk_GdkEventExpose_2II_FUNC,
@@ -993,6 +1115,7 @@ typedef enum {
 	memmove__Lorg_eclipse_swt_internal_gtk_GdkVisual_2I_FUNC,
 	memmove__Lorg_eclipse_swt_internal_gtk_GtkAdjustment_2I_FUNC,
 	memmove__Lorg_eclipse_swt_internal_gtk_GtkBorder_2II_FUNC,
+	memmove__Lorg_eclipse_swt_internal_gtk_GtkCellRendererClass_2I_FUNC,
 	memmove__Lorg_eclipse_swt_internal_gtk_GtkColorSelectionDialog_2I_FUNC,
 	memmove__Lorg_eclipse_swt_internal_gtk_GtkCombo_2I_FUNC,
 	memmove__Lorg_eclipse_swt_internal_gtk_GtkFileSelection_2I_FUNC,
