@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others. All rights reserved.
+ * Copyright (c) 2000, 2006 IBM Corporation and others. All rights reserved.
  * The contents of this file are made available under the terms
  * of the GNU Lesser General Public License (LGPL) Version 2.1 that
  * accompanies this distribution (lgpl-v21.txt).  The LGPL is also
@@ -8,6 +8,9 @@
  * the LGPL accompanying this distribution and there is any conflict
  * between the two license versions, the terms of the LGPL accompanying
  * this distribution shall govern.
+ * 
+ * Contributors:
+ *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.swt.internal.gtk;
 
@@ -15,12 +18,12 @@ package org.eclipse.swt.internal.gtk;
 public class GdkDragContext {     
    public int protocol;
    public boolean is_source;
-   public int source_window;
-   public int dest_window;
-   public int targets;
+   public int /*long*/ source_window;
+   public int /*long*/ dest_window;
+   public int /*long*/ targets;
    public int actions;
    public int suggested_action;
    public int action; 
    public int start_time;
-   public static final int sizeof = 34;
+   public static final int sizeof = OS.GdkDragContext_sizeof();
 }
