@@ -1,10 +1,10 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -12,9 +12,9 @@ package org.eclipse.swt.internal.motif;
 
  
 public class XRectangle {
-	public short x, y, width, height;
+	public short x;
+	public short y;
+	public short width;
+	public short height;
 	public static final int sizeof = 8;
-public String toString () {
-	return "XRectangle {" + x + ", " + y + ", " + width + ", " + height + "}";
-}
 }
