@@ -1,10 +1,10 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,8 +13,11 @@ package org.eclipse.swt.internal.motif;
  
 public class XtWidgetGeometry {
 	public int request_mode;
-	public int x, y;
-	public int width, height, border_width;
+	public int x;
+	public int y;
+	public int width;
+	public int height;
+	public int border_width;
 	public int sibling;
 	public int stack_mode; 
 	public static final int sizeof = 24;
