@@ -1,23 +1,24 @@
 /*******************************************************************************
-* Copyright (c) 2000, 2005 IBM Corporation and others.
-* All rights reserved. This program and the accompanying materials
-* are made available under the terms of the Eclipse Public License v1.0
-* which accompanies this distribution, and is available at
-* http://www.eclipse.org/legal/epl-v10.html
-*
-* Contributors:
-*     IBM Corporation - initial API and implementation
-*******************************************************************************/
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
+ * which accompanies this distribution, and is available at
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors:
+ *    IBM Corporation - initial API and implementation
+ *******************************************************************************/
 
 #include "swt.h"
 #include "os_stats.h"
 
 #ifdef NATIVE_STATS
 
-int OS_nativeFunctionCount = 653;
-int OS_nativeFunctionCallCount[653];
+int OS_nativeFunctionCount = 694;
+int OS_nativeFunctionCallCount[694];
 char * OS_nativeFunctionNames[] = {
 	"AbortDoc",
+	"ActivateActCtx",
 	"ActivateKeyboardLayout",
 	"AdjustWindowRectEx",
 	"AlphaBlend",
@@ -56,6 +57,8 @@ char * OS_nativeFunctionNames[] = {
 	"CopyImage",
 	"CreateAcceleratorTableA",
 	"CreateAcceleratorTableW",
+	"CreateActCtxA",
+	"CreateActCtxW",
 	"CreateBitmap",
 	"CreateCaret",
 	"CreateCompatibleBitmap",
@@ -112,6 +115,10 @@ char * OS_nativeFunctionNames[] = {
 	"DrawTextA",
 	"DrawTextW",
 	"DrawThemeBackground",
+	"DrawThemeEdge",
+	"DrawThemeIcon",
+	"DrawThemeParentBackground",
+	"DrawThemeText",
 	"Ellipse",
 	"EnableMenuItem",
 	"EnableScrollBar",
@@ -215,6 +222,8 @@ char * OS_nativeFunctionNames[] = {
 	"GetMessageTime",
 	"GetMessageW",
 	"GetMetaRgn",
+	"GetModuleFileNameA",
+	"GetModuleFileNameW",
 	"GetModuleHandleA",
 	"GetModuleHandleW",
 	"GetMonitorInfoA",
@@ -266,12 +275,23 @@ char * OS_nativeFunctionNames[] = {
 	"GetTextExtentPoint32W",
 	"GetTextMetricsA",
 	"GetTextMetricsW",
+	"GetThemeBackgroundContentRect",
+	"GetThemeBackgroundExtent",
+	"GetThemeColor",
+	"GetThemeInt",
+	"GetThemeMargins",
+	"GetThemeMetric",
+	"GetThemePartSize",
+	"GetThemeRect",
+	"GetThemeSysSize",
+	"GetThemeTextExtent",
 	"GetTickCount",
 	"GetUpdateRect",
 	"GetUpdateRgn",
 	"GetVersionExA",
 	"GetVersionExW",
 	"GetWindow",
+	"GetWindowDC",
 	"GetWindowLongA",
 	"GetWindowLongW",
 	"GetWindowOrgEx",
@@ -295,11 +315,20 @@ char * OS_nativeFunctionNames[] = {
 	"HeapAlloc",
 	"HeapFree",
 	"HideCaret",
+	"HitTestThemeBackground",
 	"IIDFromString",
 	"ImageList_1Add",
 	"ImageList_1AddMasked",
+	"ImageList_1BeginDrag",
 	"ImageList_1Create",
 	"ImageList_1Destroy",
+	"ImageList_1DragEnter",
+	"ImageList_1DragLeave",
+	"ImageList_1DragMove",
+	"ImageList_1DragShowNolock",
+	"ImageList_1Draw",
+	"ImageList_1EndDrag",
+	"ImageList_1GetDragImage",
 	"ImageList_1GetIcon",
 	"ImageList_1GetIconSize",
 	"ImageList_1GetImageCount",
@@ -331,6 +360,7 @@ char * OS_nativeFunctionNames[] = {
 	"InsertMenuItemA",
 	"InsertMenuItemW",
 	"InsertMenuW",
+	"InternetSetOption",
 	"IntersectClipRect",
 	"IntersectRect",
 	"InvalidateRect",
@@ -361,6 +391,8 @@ char * OS_nativeFunctionNames[] = {
 	"LoadStringA",
 	"LoadStringW",
 	"LocalFree",
+	"LockWindowUpdate",
+	"MCIWndRegisterClass",
 	"MapVirtualKeyA",
 	"MapVirtualKeyW",
 	"MapWindowPoints__IILorg_eclipse_swt_internal_win32_POINT_2I",
@@ -393,6 +425,7 @@ char * OS_nativeFunctionNames[] = {
 	"MoveMemory__I_3DI",
 	"MoveMemory__I_3FI",
 	"MoveMemory__I_3II",
+	"MoveMemory__I_3JI",
 	"MoveMemory__I_3SI",
 	"MoveMemory__Lorg_eclipse_swt_internal_win32_BITMAPINFOHEADER_2_3BI",
 	"MoveMemory__Lorg_eclipse_swt_internal_win32_DRAWITEMSTRUCT_2II",
@@ -421,6 +454,7 @@ char * OS_nativeFunctionNames[] = {
 	"MoveMemory__Lorg_eclipse_swt_internal_win32_NMTTDISPINFOW_2II",
 	"MoveMemory__Lorg_eclipse_swt_internal_win32_NMTVCUSTOMDRAW_2II",
 	"MoveMemory__Lorg_eclipse_swt_internal_win32_NMTVDISPINFO_2II",
+	"MoveMemory__Lorg_eclipse_swt_internal_win32_NMTVITEMCHANGE_2II",
 	"MoveMemory__Lorg_eclipse_swt_internal_win32_NMUPDOWN_2II",
 	"MoveMemory__Lorg_eclipse_swt_internal_win32_POINT_2II",
 	"MoveMemory__Lorg_eclipse_swt_internal_win32_SCRIPT_1ITEM_2II",
@@ -438,6 +472,7 @@ char * OS_nativeFunctionNames[] = {
 	"MoveMemory___3DII",
 	"MoveMemory___3FII",
 	"MoveMemory___3III",
+	"MoveMemory___3JII",
 	"MoveMemory___3SII",
 	"MoveToEx",
 	"MsgWaitForMultipleObjectsEx",
@@ -506,6 +541,7 @@ char * OS_nativeFunctionNames[] = {
 	"SHSipPreference",
 	"SaveDC",
 	"ScreenToClient",
+	"ScriptApplyDigitSubstitution",
 	"ScriptBreak",
 	"ScriptCPtoX",
 	"ScriptCacheGetHeight",
@@ -514,8 +550,10 @@ char * OS_nativeFunctionNames[] = {
 	"ScriptGetLogicalWidths",
 	"ScriptGetProperties",
 	"ScriptItemize",
+	"ScriptJustify",
 	"ScriptLayout",
 	"ScriptPlace",
+	"ScriptRecordDigitSubstitution",
 	"ScriptShape",
 	"ScriptTextOut",
 	"ScriptXtoCP",
@@ -533,6 +571,7 @@ char * OS_nativeFunctionNames[] = {
 	"SendMessageA__IIILorg_eclipse_swt_internal_win32_LVHITTESTINFO_2",
 	"SendMessageA__IIILorg_eclipse_swt_internal_win32_LVITEM_2",
 	"SendMessageA__IIILorg_eclipse_swt_internal_win32_MARGINS_2",
+	"SendMessageA__IIILorg_eclipse_swt_internal_win32_POINT_2",
 	"SendMessageA__IIILorg_eclipse_swt_internal_win32_REBARBANDINFO_2",
 	"SendMessageA__IIILorg_eclipse_swt_internal_win32_RECT_2",
 	"SendMessageA__IIILorg_eclipse_swt_internal_win32_SIZE_2",
@@ -558,6 +597,7 @@ char * OS_nativeFunctionNames[] = {
 	"SendMessageW__IIILorg_eclipse_swt_internal_win32_LVHITTESTINFO_2",
 	"SendMessageW__IIILorg_eclipse_swt_internal_win32_LVITEM_2",
 	"SendMessageW__IIILorg_eclipse_swt_internal_win32_MARGINS_2",
+	"SendMessageW__IIILorg_eclipse_swt_internal_win32_POINT_2",
 	"SendMessageW__IIILorg_eclipse_swt_internal_win32_REBARBANDINFO_2",
 	"SendMessageW__IIILorg_eclipse_swt_internal_win32_RECT_2",
 	"SendMessageW__IIILorg_eclipse_swt_internal_win32_SIZE_2",
@@ -577,6 +617,7 @@ char * OS_nativeFunctionNames[] = {
 	"SetActiveWindow",
 	"SetBkColor",
 	"SetBkMode",
+	"SetBrushOrgEx",
 	"SetCapture",
 	"SetCaretPos",
 	"SetClipboardData",
