@@ -1,21 +1,19 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.swt.internal.ole.win32;
 
-public final class CONTROLINFO
-{
-	public int   cb;
-	public int   hAccel;
+public final class CONTROLINFO {
+	public int cb;
+	public int hAccel;
 	public short cAccel;
-	public int   dwFlags;
-	
+	public int dwFlags;
 	public static final int sizeof = 16;
 }
