@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -17,13 +17,11 @@ import org.eclipse.swt.*;
 /**
  * <code>TextLayout</code> is a graphic object that represents
  * styled text.
- *<p>
+ * <p>
  * Instances of this class provide support for drawing, cursor
  * navigation, hit testing, text wrapping, alignment, tab expansion
  * line breaking, etc.  These are aspects required for rendering internationalized text.
- * </p>
- * 
- * <p>
+ * </p><p>
  * Application code must explicitly invoke the <code>TextLayout#dispose()</code> 
  * method to release the operating system resources managed by each instance
  * when those instances are no longer required.
@@ -58,6 +56,14 @@ public final class TextLayout extends Resource {
 					length += 1;
 					ptrLength += 1;
 				}
+				if (style.metrics != null) {
+					length += 3;
+					ptrLength += 12;
+				}
+				if (style.rise != 0) {
+					length += 1;
+					ptrLength += 4;
+				}
 			}
 			if (font == null) font = defaultFont;
 			boolean synthesize = false;
@@ -134,6 +140,41 @@ public final class TextLayout extends Resource {
 				ptr1 += sizes[index];
 				index++;
 			}
+			if (style != null && style.metrics != null) {
+				GlyphMetrics metrics = style.metrics; 
+				buffer[0] = OS.Long2Fix(metrics.ascent);
+				tags[index] = OS.kATSUAscentTag;
+				sizes[index] = 4;
+				values[index] = ptr1;
+				OS.memcpy(values[index], buffer, sizes[index]);
+				ptr1 += sizes[index];
+				index++;
+				
+				buffer[0] = OS.Long2Fix(metrics.descent);
+				tags[index] = OS.kATSUDescentTag;
+				sizes[index] = 4;
+				values[index] = ptr1;
+				OS.memcpy(values[index], buffer, sizes[index]);
+				ptr1 += sizes[index];
+				index++;
+				
+				buffer[0] = OS.Long2Fix(metrics.width);
+				tags[index] = OS.kATSUImposeWidthTag;
+				sizes[index] = 4;
+				values[index] = ptr1;
+				OS.memcpy(values[index], buffer, sizes[index]);
+				ptr1 += sizes[index];
+				index++;
+			}
+			if (style != null && style.rise != 0) {
+				buffer[0] = OS.Long2Fix(style.rise);
+				tags[index] = OS.kATSUCrossStreamShiftTag;
+				sizes[index] = 4;
+				values[index] = ptr1;
+				OS.memcpy(values[index], buffer, sizes[index]);
+				ptr1 += sizes[index];
+				index++;
+			}
 			if (foreground != null) {
 				RGBColor rgb = new RGBColor ();
 				float[] color = foreground.handle;
@@ -156,6 +197,10 @@ public final class TextLayout extends Resource {
 			OS.ATSUDisposeStyle(atsuStyle);
 			atsuStyle = 0;
 		}
+
+		public String toString () {
+			return "StyleItem {" + start + ", " + style + "}";
+		}
 	}
 	
 	Font font;
@@ -163,13 +208,15 @@ public final class TextLayout extends Resource {
 	int textPtr;
 	StyleItem[] styles;
 	int layout;
-	int spacing, ascent, descent;	
+	int spacing, ascent, descent, indent;
+	int indentStyle;
 	int[] tabs;
 	int[] segments;
 	int tabsPtr;
-	int[] breaks, lineX, lineWidth, lineHeight, lineAscent;
+	int[] breaks, hardBreaks, lineX, lineWidth, lineHeight, lineAscent;
 
 	static final int TAB_COUNT = 32;
+	static final char ZWS = '\u200B';
 	
 /**	 
  * Constructs a new instance of this class on the given device.
@@ -194,7 +241,9 @@ public TextLayout (Device device) {
 	if (buffer[0] == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 	layout = buffer[0];
 	setLayoutControl(OS.kATSULineDirectionTag, OS.kATSULeftToRightBaseDirection, 1);
-	OS.ATSUSetHighlightingMethod(layout, 1, new ATSUUnhighlightData());
+	setLayoutControl(OS.kATSULineLayoutOptionsTag, OS.kATSLineLastNoJustification, 4);
+	setLayoutControl(OS.kATSULineLayoutOptionsTag, OS.kATSLineUseDeviceMetrics, 4);
+	OS.ATSUSetHighlightingMethod(layout, OS.kRedrawHighlighting, new ATSUUnhighlightData());
 	ascent = descent = -1;
 	text = "";
 	styles = new StyleItem[2];
@@ -208,67 +257,116 @@ void checkLayout() {
 
 void computeRuns() {
 	if (breaks != null) return;
-	int length = text.length();
-	if (length != 0) {
-		for (int i = 0; i < styles.length - 1; i++) {
-			StyleItem run = styles[i];
-			run.createStyle(font);
-			int runLength = styles[i + 1].start - run.start;
-			OS.ATSUSetRunStyle(layout, run.atsuStyle, run.start, runLength);
+	int textLength = text.length();
+	char[] chars = new char[textLength + 1];
+	text.getChars(0, textLength, chars, 1);
+	chars[0] = ZWS;
+	int breakCount = 1;
+	for (int i = 0; i < chars.length; i++) {
+		char c = chars[i];
+		if (c == '\n' || c == '\r') {
+			breakCount++;
 		}
-		int[] buffer = new int[1];
-		if (ascent != -1) {
-			OS.ATSUGetLayoutControl(layout, OS.kATSULineAscentTag, 4, buffer, null);
-			int ptr = OS.NewPtr(4);
-			buffer[0] = OS.Long2Fix(Math.max(ascent, OS.Fix2Long(buffer[0])));
-			OS.memcpy(ptr, buffer, 4);
-			int[] tags = new int[]{OS.kATSULineAscentTag};
-			int[] sizes = new int[]{4};
-			int[] values = new int[]{ptr};
-			OS.ATSUSetLineControls(layout, 0, tags.length, tags, sizes, values);
-			OS.DisposePtr(ptr);
-		}
-		if (descent != -1) {
-			OS.ATSUGetLayoutControl(layout, OS.kATSULineDescentTag, 4, buffer, null);
-			int ptr = OS.NewPtr(4);
-			buffer[0] = OS.Long2Fix(Math.max(descent, OS.Fix2Long(buffer[0])));
-			OS.memcpy(ptr, buffer, 4);
-			int[] tags = new int[]{OS.kATSULineDescentTag};
-			int[] sizes = new int[]{4};
-			int[] values = new int[]{ptr};
-			OS.ATSUSetLineControls(layout, 0, tags.length, tags, sizes, values);
-			OS.DisposePtr(ptr);
-		}
-		OS.ATSUGetLayoutControl(layout, OS.kATSULineWidthTag, 4, buffer, null);
-		int wrapWidth = OS.Fix2Long(buffer[0]);
-		int width = wrapWidth == 0 ? 0x7fff : wrapWidth;
-		OS.ATSUBatchBreakLines(layout, 0, OS.kATSUToTextEnd, OS.Long2Fix(width), buffer);
-		int count = Math.max(0, buffer[0]);
-		breaks = new int[count + 1];
-		OS.ATSUGetSoftLineBreaks(layout, 0, OS.kATSUToTextEnd, count, breaks, buffer);
-		breaks[count] = length;
-	} else {
-		breaks = new int[1];
 	}
+	hardBreaks = new int [breakCount];
+	breakCount = 0;
+	for (int i = 0; i < chars.length; i++) {
+		char c = chars[i];
+		if (c == '\n' || c == '\r') {
+			chars[i] = ZWS;
+			hardBreaks[breakCount++] = i;
+		}
+	}
+	hardBreaks[breakCount] = translateOffset(textLength);
+	int newTextPtr = OS.NewPtr(chars.length * 2);
+	OS.memcpy(newTextPtr, chars, chars.length * 2);
+	OS.ATSUSetTextPointerLocation(layout, newTextPtr, 0, chars.length, chars.length);
+	OS.ATSUSetTransientFontMatching(layout, true);
+	if (textPtr != 0) OS.DisposePtr(textPtr);
+	textPtr = newTextPtr;
+
+	int[] buffer = new int[1];
+	for (int i = 0; i < styles.length - 1; i++) {
+		StyleItem run = styles[i];
+		run.createStyle(font);
+		//set the defaut font in the ZWS when text is empty fixes text metrics
+		int start = textLength != 0 ? translateOffset(run.start) : 0;
+		int runLength = translateOffset(styles[i + 1].start) - start;
+		OS.ATSUSetRunStyle(layout, run.atsuStyle, start, runLength);
+	}
+	int ptr = OS.NewPtr(12);
+	buffer = new int[]{OS.Long2Fix(indent), 0, 0};
+	OS.memcpy(ptr, buffer, 12);
+	int[] tags = new int[]{OS.kATSUImposeWidthTag, OS.kATSUAscentTag, OS.kATSUDescentTag};
+	int[] sizes = new int[]{4, 4, 4};
+	int[] values = new int[]{ptr, ptr + 4, ptr + 8};
+	OS.ATSUCreateStyle(buffer);
+	indentStyle = buffer[0];
+	OS.ATSUSetAttributes(indentStyle, tags.length, tags, sizes, values);
+	OS.DisposePtr(ptr);
+	OS.ATSUSetRunStyle(layout, indentStyle, 0, 1);
+	for (int i = 0; i < hardBreaks.length-1; i++) {
+		int offset = hardBreaks[i];
+		OS.ATSUSetRunStyle(layout, indentStyle, offset, 1);
+	}
+	OS.ATSUGetLayoutControl(layout, OS.kATSULineWidthTag, 4, buffer, null);
+	int wrapWidth = buffer[0];
+	for (int i=0, start=0; i<hardBreaks.length; i++) {
+		int hardBreak = hardBreaks[i];
+		buffer[0] = 0;
+		if (wrapWidth != 0) OS.ATSUBatchBreakLines(layout, start, hardBreak - start, wrapWidth, buffer);
+		OS.ATSUSetSoftLineBreak(layout, hardBreak);
+		start = hardBreak;
+	}
+	OS.ATSUGetSoftLineBreaks(layout, 0, OS.kATSUToTextEnd, 0, null, buffer);
+	int count = buffer[0];
+	breaks = new int[count];
+	OS.ATSUGetSoftLineBreaks(layout, 0, OS.kATSUToTextEnd, count, breaks, null);
 	int lineCount = breaks.length;
 	lineX = new int[lineCount];
 	lineWidth = new int[lineCount];
 	lineHeight = new int[lineCount];
 	lineAscent = new int[lineCount];
-	if (length != 0) {
-		ATSTrapezoid trapezoid = new ATSTrapezoid();
-		for (int i=0, start=0; i<lineCount; i++) {
-			int lineBreak = breaks[i];
-			int lineLength = skipHardBreak(lineBreak) - start;
-			OS.ATSUGetGlyphBounds(layout, 0, 0, start, lineLength == 0 ? 1 : lineLength, (short)OS.kATSUseDeviceOrigins, 1, trapezoid, null);
-			lineX[i] = OS.Fix2Long(trapezoid.lowerLeft_x);
-			lineAscent[i] = -OS.Fix2Long(trapezoid.upperRight_y);
-			if (lineLength != 0) {
-				lineWidth[i] = OS.Fix2Long(trapezoid.upperRight_x) - OS.Fix2Long(trapezoid.upperLeft_x);
-			}
-			lineHeight[i] = OS.Fix2Long(trapezoid.lowerRight_y) + lineAscent[i];
-			start = lineBreak;
+	ATSTrapezoid trapezoid = new ATSTrapezoid();
+	for (int i=0, start=0; i<lineCount; i++) {
+		if (ascent != -1) {
+			ptr = OS.NewPtr(4);
+			buffer[0] = OS.kATSUseLineHeight;
+			OS.memcpy(ptr, buffer, 4);
+			tags = new int[]{OS.kATSULineAscentTag};
+			sizes = new int[]{4};
+			values = new int[]{ptr};
+			OS.ATSUSetLineControls(layout, start, tags.length, tags, sizes, values);
+			OS.ATSUGetLineControl(layout, start, OS.kATSULineAscentTag, 4, buffer, null);
+			buffer[0] = OS.Long2Fix(Math.max(ascent, OS.Fix2Long(buffer[0])));
+			OS.memcpy(ptr, buffer, 4);
+			OS.ATSUSetLineControls(layout, start, tags.length, tags, sizes, values);
+			OS.DisposePtr(ptr);
 		}
+		if (descent != -1) {
+			ptr = OS.NewPtr(4);
+			buffer[0] = OS.kATSUseLineHeight;
+			OS.memcpy(ptr, buffer, 4);
+			tags = new int[]{OS.kATSULineDescentTag};
+			sizes = new int[]{4};
+			values = new int[]{ptr};
+			OS.ATSUSetLineControls(layout, start, tags.length, tags, sizes, values);
+			OS.ATSUGetLineControl(layout, start, OS.kATSULineDescentTag, 4, buffer, null);
+			buffer[0] = OS.Long2Fix(Math.max(descent, OS.Fix2Long(buffer[0])));
+			OS.memcpy(ptr, buffer, 4);
+			OS.ATSUSetLineControls(layout, start, tags.length, tags, sizes, values);
+			OS.DisposePtr(ptr);
+		}
+		int lineBreak = breaks[i];
+		int lineLength = lineBreak - start;
+		OS.ATSUGetGlyphBounds(layout, 0, 0, start, lineLength, (short)OS.kATSUseDeviceOrigins, 1, trapezoid, null);
+		lineX[i] = OS.Fix2Long(trapezoid.lowerLeft_x);
+		lineAscent[i] = -OS.Fix2Long(trapezoid.upperRight_y);
+		if (lineLength != 0) {
+			lineWidth[i] = OS.Fix2Long(trapezoid.lowerRight_x) - OS.Fix2Long(trapezoid.lowerLeft_x);
+		}
+		lineHeight[i] = OS.Fix2Long(trapezoid.lowerRight_y) + lineAscent[i] + spacing;
+		start = lineBreak;
 	}
 }
 
@@ -288,6 +386,8 @@ public void dispose() {
 	textPtr = 0;
 	if (tabsPtr != 0) OS.DisposePtr(tabsPtr);
 	tabsPtr = 0;
+	if (indentStyle != 0) OS.ATSUDisposeStyle(indentStyle);
+	indentStyle = 0;
 	device = null;
 }
 
@@ -336,16 +436,14 @@ public void draw(GC gc, int x, int y, int selectionStart, int selectionEnd, Colo
 	if (gc.isDisposed()) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	if (selectionForeground != null && selectionForeground.isDisposed()) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	if (selectionBackground != null && selectionBackground.isDisposed()) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
-	int length = text.length();
+	int length = translateOffset(text.length());
 	if (length == 0) return;
 	setLayoutControl(OS.kATSUCGContextTag, gc.handle, 4);
 	boolean hasSelection = selectionStart <= selectionEnd && selectionStart != -1 && selectionEnd != -1;
 	OS.CGContextSaveGState(gc.handle);
 	boolean restoreColor = false;
 	if (hasSelection && selectionBackground != null) {
-		int[] response = new int[1];
-		int err = OS.Gestalt(OS.gestaltSystemVersion, response);
-		if (err == OS.noErr && ((response[0] & 0xffff) >= 0x1030)) {
+		if (OS.VERSION >= 0x1030) {
 			restoreColor = true;
 			int color = OS.CGColorCreate(device.colorspace, selectionBackground.handle);
 			setLayoutControl(OS.kATSULineHighlightCGColorTag, color, 4);
@@ -368,19 +466,17 @@ public void draw(GC gc, int x, int y, int selectionStart, int selectionEnd, Colo
 			gc.getClipping(clipping);
 			rect = clipping.getBounds();
 		}
-		int start = run.start;
-		int end = j + 1 < styles.length ? styles[j + 1].start - 1 : length;
+		int start = translateOffset(run.start);
+		int end = j + 1 < styles.length ? translateOffset(styles[j + 1].start - 1) : length;
 		for (int i=0, lineStart=0, lineY = 0; i<breaks.length; i++) {
 			int lineBreak = breaks[i];
 			int lineEnd = lineBreak - 1;
 			if (!(start > lineEnd || end < lineStart)) {
 				int highStart = Math.max(lineStart, start);
 				int highEnd = Math.min(lineEnd, end);
-				int highLen = skipHardBreak(highEnd) - highStart + 1;
+				int highLen = highEnd - highStart + 1;
 				if (highLen > 0) {
-					OS.ATSUGetTextHighlight(layout, lineX[i], lineY, highStart, highLen, region.handle);
-					OS.OffsetRgn(region.handle, (short)0, (short)(lineY + lineAscent[i]));
-					OS.OffsetRgn(region.handle, (short)x, (short)y);
+					OS.ATSUGetTextHighlight(layout, OS.Long2Fix(x), OS.Long2Fix(y + lineY + lineAscent[i]), highStart, highLen, region.handle);
 					region.intersect(clipping);
 					gc.setClipping(region);
 					gc.fillRectangle(rect);
@@ -399,13 +495,15 @@ public void draw(GC gc, int x, int y, int selectionStart, int selectionEnd, Colo
 		region.dispose();
 	}
 
+	selectionStart = translateOffset(selectionStart);
+	selectionEnd = translateOffset(selectionEnd);
 	OS.CGContextScaleCTM(gc.handle, 1, -1);
 	OS.CGContextSetFillColor(gc.handle, gc.data.foreground);
 	int drawX = OS.Long2Fix(x);
 	int drawY = y;
 	for (int i=0, start=0; i<breaks.length; i++) {
 		int lineBreak = breaks[i];
-		int lineLength = skipHardBreak(lineBreak) - start;
+		int lineLength = lineBreak - start;
 		if (lineLength > 0) {
 			int fixYDraw = OS.Long2Fix(-(drawY + lineAscent[i]));
 			OS.ATSUDrawText(layout, start, lineLength, drawX, fixYDraw);
@@ -429,6 +527,8 @@ void freeRuns() {
 		StyleItem run = styles[i];
 		run.freeStyle();
 	}
+	if (indentStyle != 0) OS.ATSUDisposeStyle(indentStyle);
+	indentStyle = 0;
 	breaks = lineX = lineWidth = lineHeight = lineAscent = null;
 }
 
@@ -519,6 +619,16 @@ public Rectangle getBounds(int start, int end) {
 	if (start > end) return new Rectangle(0, 0, 0, 0);
 	start = Math.min(Math.max(0, start), length - 1);
 	end = Math.min(Math.max(0, end), length - 1);
+	start = translateOffset(start);
+	end = translateOffset(end);
+	for (int i = 0; i < hardBreaks.length; i++) {
+		if (start == hardBreaks[i]) {
+			if (start > 0) start--;
+		}
+		if (end == hardBreaks[i]) {
+			if (end > 0) end--;
+		}
+	}
 	int rgn = OS.NewRgn();
 	Rect rect = new Rect();
 	Rect rect1 = new Rect();
@@ -528,11 +638,10 @@ public Rectangle getBounds(int start, int end) {
 		if (!(start > lineEnd || end < lineStart)) {
 			int highStart = Math.max(lineStart, start);
 			int highEnd = Math.min(lineEnd, end);
-			int highLen = skipHardBreak(highEnd) - highStart + 1;
+			int highLen = highEnd - highStart + 1;
 			if (highLen > 0) {
-				OS.ATSUGetTextHighlight(layout, lineX[i], lineY, highStart, highLen, rgn);
+				OS.ATSUGetTextHighlight(layout, 0, OS.Long2Fix(lineY + lineAscent[i]), highStart, highLen, rgn);
 				OS.GetRegionBounds(rgn, rect1);
-				OS.OffsetRect(rect1, (short)0, (short)(lineY + lineAscent[i]));
 				OS.UnionRect(rect, rect1, rect);
 			}
 		}
@@ -578,6 +687,40 @@ public Font getFont () {
 	return font;
 }
 
+/**
+* Returns the receiver's indent.
+*
+* @return the receiver's indent
+* 
+* @exception SWTException <ul>
+*    <li>ERROR_GRAPHIC_DISPOSED - if the receiver has been disposed</li>
+* </ul>
+* 
+* @since 3.2
+*/
+public int getIndent () {
+	checkLayout();	
+	return indent;
+}
+
+/**
+* Returns the receiver's justification.
+*
+* @return the receiver's justification
+* 
+* @exception SWTException <ul>
+*    <li>ERROR_GRAPHIC_DISPOSED - if the receiver has been disposed</li>
+* </ul>
+* 
+* @since 3.2
+*/
+public boolean getJustify () {
+	checkLayout();
+	int[] buffer = new int[1];
+	OS.ATSUGetLayoutControl(layout, OS.kATSULineJustificationFactorTag, 4, buffer, null);
+	return buffer[0] == OS.kATSUFullJustification;
+}
+
 /**
  * Returns the embedding level for the specified character offset. The
  * embedding level is usually used to determine the directionality of a
@@ -597,6 +740,7 @@ public int getLevel(int offset) {
 	computeRuns();
 	int length = text.length();
 	if (!(0 <= offset && offset <= length)) SWT.error(SWT.ERROR_INVALID_RANGE);
+	offset = translateOffset(offset);	
 	int level = 0;
 	//TODO
 	return level;
@@ -617,7 +761,9 @@ public int[] getLineOffsets() {
 	checkLayout ();
 	computeRuns();
 	int[] offsets = new int[breaks.length + 1];
-	System.arraycopy(breaks, 0, offsets, 1, breaks.length);
+	for (int i = 1; i < offsets.length; i++) {
+		offsets[i] = untranslateOffset(breaks[i - 1]);	
+	}
 	return offsets;
 }
 
@@ -640,6 +786,7 @@ public int getLineIndex(int offset) {
 	computeRuns();
 	int length = text.length();
 	if (!(0 <= offset && offset <= length)) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
+	offset = translateOffset(offset);
 	for (int i=0; i<breaks.length-1; i++) {
 		int lineBreak = breaks[i];
 		if (lineBreak > offset) return i;
@@ -669,7 +816,9 @@ public Rectangle getLineBounds(int lineIndex) {
 	for (int i=0; i<lineIndex; i++) {
 		lineY += lineHeight[i];
 	}
-	return new Rectangle(lineX[lineIndex], lineY, lineWidth[lineIndex], lineHeight[lineIndex]);
+	int lineX = this.lineX[lineIndex];
+	int lineWidth = this.lineWidth[lineIndex];
+	return new Rectangle(lineX, lineY, lineWidth, lineHeight[lineIndex] - spacing);
 }
 
 /**
@@ -746,13 +895,21 @@ public Point getLocation(int offset, boolean trailing) {
 	int length = text.length();
 	if (!(0 <= offset && offset <= length)) SWT.error(SWT.ERROR_INVALID_RANGE);
 	if (length == 0) return new Point(0, 0);
+	offset = translateOffset(offset);
+	for (int i = 0; i < hardBreaks.length; i++) {
+		if (offset == hardBreaks[i]) {
+			trailing = true;
+			if (offset > 0) offset--;
+			break;
+		}
+	}
 	int lineY = 0;
 	for (int i=0; i<breaks.length-1; i++) {
 		int lineBreak = breaks[i];
 		if (lineBreak > offset) break;
 		lineY += lineHeight[i];
 	}
-	if (offset != length && text.charAt(offset) != '\n' && trailing) offset++;
+	if (trailing) offset++;
 	ATSUCaret caret = new ATSUCaret();
 	OS.ATSUOffsetToPosition(layout, offset, !trailing, caret, null, null);
 	return new Point(Math.min(OS.Fix2Long(caret.fX), OS.Fix2Long(caret.fDeltaX)), lineY);
@@ -786,6 +943,7 @@ int _getOffset (int offset, int movement, boolean forward) {
 	int length = text.length();
 	if (!(0 <= offset && offset <= length)) SWT.error(SWT.ERROR_INVALID_RANGE);
 	if (length == 0) return 0;
+	offset = translateOffset(offset);
 	int[] newOffset = new int[1];
 	int type = OS.kATSUByCharacter;
 	switch (movement) {
@@ -794,6 +952,7 @@ int _getOffset (int offset, int movement, boolean forward) {
 	}
 	if (forward) {
 		OS.ATSUNextCursorPosition(layout, offset, type, newOffset);
+		newOffset[0] = untranslateOffset(newOffset[0]);
 		if (movement == SWT.MOVEMENT_WORD) {
 			while (newOffset[0] < length && Compatibility.isWhitespace(text.charAt(newOffset[0]))) {
 				newOffset[0]++;
@@ -801,6 +960,7 @@ int _getOffset (int offset, int movement, boolean forward) {
 		}
 	} else {
 		OS.ATSUPreviousCursorPosition(layout, offset, type, newOffset);
+		newOffset[0] = untranslateOffset(newOffset[0]);
 		if (movement == SWT.MOVEMENT_WORD) {
 			while (newOffset[0] > 0 && !Compatibility.isWhitespace(text.charAt(newOffset[0] - 1))) {
 				newOffset[0]--;
@@ -869,10 +1029,10 @@ public int getOffset(int x, int y, int[] trailing) {
 	if (trailing != null && trailing.length < 1) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	int length = text.length();
 	if (length == 0) return 0;
-	int lineY = 0, start = 0;
-	for (int i=0; i<breaks.length-1; i++) {
-		int lineBreak = breaks[i];
-		int height = lineHeight[i];
+	int lineY = 0, start = 0, lineIndex;
+	for (lineIndex=0; lineIndex<breaks.length-1; lineIndex++) {
+		int lineBreak = breaks[lineIndex];
+		int height = lineHeight[lineIndex];
 		if (lineY + height > y) break;
 		lineY += height;
 		start = lineBreak;
@@ -882,7 +1042,13 @@ public int getOffset(int x, int y, int[] trailing) {
 	OS.ATSUPositionToOffset(layout, OS.Long2Fix(x), OS.Long2Fix(y - lineY), offset, leading, null);
 	if (trailing != null) trailing[0] = (leading[0] ? 0 : 1);
 	if (!leading[0]) offset[0]--;
-	return offset[0];
+	for (int i = 0; i < hardBreaks.length; i++) {
+		if (offset[0] == hardBreaks[i]) {
+			offset[0]++;
+			break;
+		}
+	}
+	return Math.min(untranslateOffset(offset[0]), length - 1);
 }
 
 /**
@@ -923,6 +1089,38 @@ public int getPreviousOffset (int index, int movement) {
 	return _getOffset(index, movement, false);
 }
 
+/**
+ * Gets the ranges of text that are associated with a <code>TextStyle</code>.
+ *
+ * @return the ranges, an array of offsets representing the start and end of each
+ * text style. 
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_GRAPHIC_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ * 
+ * @see #getStyles()
+ * 
+ * @since 3.2
+ */
+public int[] getRanges () {
+	checkLayout();
+	int[] result = new int[styles.length * 2];
+	int count = 0;
+	for (int i=0; i<styles.length - 1; i++) {
+		if (styles[i].style != null) {
+			result[count++] = styles[i].start;
+			result[count++] = styles[i + 1].start - 1;
+		}
+	}
+	if (count != result.length) {
+		int[] newResult = new int[count];
+		System.arraycopy(result, 0, newResult, 0, count);
+		result = newResult;
+	}
+	return result;
+}
+
 /**
  * Returns the text segments offsets of the receiver.
  *
@@ -977,6 +1175,36 @@ public TextStyle getStyle (int offset) {
 	return null;
 }
 
+/**
+ * Gets all styles of the receiver.
+ *
+ * @return the styles
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_GRAPHIC_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ * 
+ * @see #getRanges()
+ * 
+ * @since 3.2
+ */
+public TextStyle[] getStyles () {
+	checkLayout();
+	TextStyle[] result = new TextStyle[styles.length];
+	int count = 0;
+	for (int i=0; i<styles.length; i++) {
+		if (styles[i].style != null) {
+			result[count++] = styles[i].style;
+		}
+	}
+	if (count != result.length) {
+		TextStyle[] newResult = new TextStyle[count];
+		System.arraycopy(result, 0, newResult, 0, count);
+		result = newResult;
+	}
+	return result;
+}
+
 /**
  * Returns the tab list of the receiver.
  *
@@ -1030,6 +1258,7 @@ public int getWidth () {
  * This method gets the dispose state for the text layout.
  * When a text layout has been disposed, it is an error to
  * invoke any other method using the text layout.
+ * </p>
  *
  * @return <code>true</code> when the text layout is disposed and <code>false</code> otherwise
  */
@@ -1045,7 +1274,7 @@ public boolean isDisposed () {
  * The default alignment is <code>SWT.LEFT</code>.  Note that the receiver's
  * width must be set in order to use <code>SWT.RIGHT</code> or <code>SWT.CENTER</code>
  * alignment.
- *</p>
+ * </p>
  *
  * @param alignment the new alignment 
  *
@@ -1167,10 +1396,48 @@ public void setFont (Font font) {
 	this.font = font;
 }
 
+/**
+ * Sets the indent of the receiver. This indent it applied of the first line of 
+ * each paragraph.  
+ *
+ * @param indent new indent
+ * 
+ * @exception SWTException <ul>
+ *    <li>ERROR_GRAPHIC_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ * 
+ * @since 3.2
+ */
+public void setIndent (int indent) {
+	checkLayout ();
+	if (indent < 0) return;
+	if (this.indent == indent) return;
+	freeRuns();
+	this.indent = indent;
+}
+
+/**
+ * Sets the justification of the receiver. Note that the receiver's
+ * width must be set in order to use justification. 
+ *
+ * @param justify new justify
+ * 
+ * @exception SWTException <ul>
+ *    <li>ERROR_GRAPHIC_DISPOSED - if the receiver has been disposed</li>
+ * </ul>
+ * 
+ * @since 3.2
+ */
+public void setJustify (boolean justify) {
+	checkLayout ();
+	if (justify == getJustify()) return;
+	freeRuns();
+	setLayoutControl(OS.kATSULineJustificationFactorTag, justify ? OS.kATSUFullJustification : OS.kATSUNoJustification, 4);
+}
+
 /**
  * Sets the orientation of the receiver, which must be one
  * of <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.RIGHT_TO_LEFT</code>.
- * <p>
  *
  * @param orientation new orientation style
  * 
@@ -1196,11 +1463,12 @@ public void setOrientation(int orientation) {
  * override the default behaviour of the bidirectional algorithm.
  * Bidirectional reordering can happen within a text segment but not 
  * between two adjacent segments.
+ * <p>
  * Each text segment is determined by two consecutive offsets in the 
  * <code>segments</code> arrays. The first element of the array should 
  * always be zero and the last one should always be equals to length of
  * the text.
- * <p>
+ * </p>
  * 
  * @param segments the text segments offset
  * 
@@ -1241,6 +1509,7 @@ public void setSpacing (int spacing) {
 	checkLayout();
 	if (spacing < 0) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	if (this.spacing == spacing) return;
+	freeRuns();
 	this.spacing = spacing;
 }
 
@@ -1268,7 +1537,7 @@ public void setStyle (TextStyle style, int start, int end) {
 	int high = styles.length;
 	while (high - low > 1) {
 		int index = (high + low) / 2;
-		if (start <= styles[index].start) {
+		if (styles[index + 1].start > start) {
 			high = index;
 		} else {
 			low = index;
@@ -1285,42 +1554,47 @@ public void setStyle (TextStyle style, int start, int end) {
 		}
 	}
 	freeRuns();
-	int count = 0, i;
-	StyleItem[] newStyles = new StyleItem[styles.length + 2];
-	for (i = 0; i < styles.length; i++) {
-		StyleItem item = styles[i];
-		if (item.start >= start) break;
-		newStyles[count++] = item;
+	int modifyStart = high;
+	int modifyEnd = modifyStart;
+	while (modifyEnd < styles.length) {
+		if (styles[modifyEnd + 1].start > end) break;
+		modifyEnd++;
 	}
-	StyleItem newItem = new StyleItem();
-	newItem.start = start;
-	newItem.style = style;
-	newStyles[count++] = newItem;
-	if (styles[i].start > end) {
-		newItem = new StyleItem();
-		newItem.start = end + 1;
-		newItem.style = styles[i -1].style;
-		newStyles[count++] = newItem;
-	} else {
-		for (; i<styles.length; i++) {
-			StyleItem item = styles[i];
-			if (item.start > end) break;
+	if (modifyStart == modifyEnd) {
+		int styleStart = styles[modifyStart].start; 
+		int styleEnd = styles[modifyEnd + 1].start - 1;
+		if (styleStart == start && styleEnd == end) {
+			styles[modifyStart].style = style;
+			return;
 		}
-		if (end != styles[i].start - 1) {
-			i--;
-			styles[i].start = end + 1;
+		if (styleStart != start && styleEnd != end) {
+			StyleItem[] newStyles = new StyleItem[styles.length + 2];
+			System.arraycopy(styles, 0, newStyles, 0, modifyStart + 1);
+			StyleItem item = new StyleItem();
+			item.start = start;
+			item.style = style;
+			newStyles[modifyStart + 1] = item;	
+			item = new StyleItem();
+			item.start = end + 1;
+			item.style = styles[modifyStart].style;
+			newStyles[modifyStart + 2] = item;
+			System.arraycopy(styles, modifyEnd + 1, newStyles, modifyEnd + 3, styles.length - modifyEnd - 1);
+			styles = newStyles;
+			return;
 		}
 	}
-	for (; i<styles.length; i++) {
-		StyleItem item = styles[i];
-		if (item.start > end) newStyles[count++] = item;
-	}
-	if (newStyles.length != count) {
-		styles = new StyleItem[count];
-		System.arraycopy(newStyles, 0, styles, 0, count);
-	} else {
-		styles = newStyles;
-	}
+	if (start == styles[modifyStart].start) modifyStart--;
+	if (end == styles[modifyEnd + 1].start - 1) modifyEnd++;
+	int newLength = styles.length + 1 - (modifyEnd - modifyStart - 1);
+	StyleItem[] newStyles = new StyleItem[newLength];
+	System.arraycopy(styles, 0, newStyles, 0, modifyStart + 1);	
+	StyleItem item = new StyleItem();
+	item.start = start;
+	item.style = style;
+	newStyles[modifyStart + 1] = item;
+	styles[modifyEnd].start = end + 1;
+	System.arraycopy(styles, modifyEnd, newStyles, modifyStart + 2, styles.length - modifyEnd);
+	styles = newStyles;
 }
 
 /**
@@ -1392,15 +1666,6 @@ public void setText (String text) {
 	if (text.equals(this.text)) return;
 	freeRuns();
 	this.text = text;
-	int length = text.length();
-	if (length != 0) {
-		char[] chars = new char[length];
-		text.getChars(0, length, chars, 0);
-		textPtr = OS.NewPtr(length * 2);
-		OS.memcpy(textPtr, chars, length * 2);
-		OS.ATSUSetTextPointerLocation(layout, textPtr, 0, length, length);
-		OS.ATSUSetTransientFontMatching(layout, true);
-	}
 	styles = new StyleItem[2];
 	styles[0] = new StyleItem();
 	styles[1] = new StyleItem();
@@ -1431,22 +1696,6 @@ public void setWidth (int width) {
 	setLayoutControl(OS.kATSULineWidthTag, OS.Long2Fix(Math.max(0, width)), 4);
 }
 
-int skipHardBreak(int lineBreak) {
-	if (true) return lineBreak;
-	while (lineBreak > 0) {
-		char c = text.charAt(lineBreak - 1);
-		switch (c) {
-			case '\r':
-			case '\n':
-				break;
-			default:
-				return lineBreak;
-		}
-		lineBreak--;
-	}
-	return lineBreak;
-}
-
 /**
  * Returns a string containing a concise, human-readable
  * description of the receiver.
@@ -1458,4 +1707,18 @@ public String toString () {
 	return "TextLayout {" + layout + "}";
 }
 
-} 
+/*
+ *  Translate a client offset to an internal offset
+ */
+int translateOffset (int offset) {
+	return offset + 1;
+}
+
+/*
+ *  Translate an internal offset to a client offset
+ */
+int untranslateOffset (int offset) {
+	return Math.max(0, offset - 1);
+}
+
+}
