@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -32,6 +32,12 @@ import org.eclipse.swt.graphics.*;
  * behavior is undefined if they are used with subclasses of <code>Composite</code> other
  * than <code>Canvas</code>.
  * </p><p>
+ * Note: The <code>CENTER</code> style, although undefined for composites, has the
+ * same value as <code>EMBEDDED</code> (which is used to embed widgets from other
+ * widget toolkits into SWT).  On some operating systems (GTK, Motif), this may cause
+ * the children of this composite to be obscured.  The <code>EMBEDDED</code> style
+ * is for use by other widget toolkits and should normally never be used.
+ * </p><p>
  * This class may be subclassed by custom control implementors
  * who are building controls that are constructed from aggregates
  * of other controls.
@@ -43,7 +49,7 @@ public class Composite extends Scrollable {
 	Layout layout;
 	Control[] tabList;
 	int scrolledVisibleRgn, siblingsVisibleRgn;
-	int layoutCount = 0;
+	int layoutCount, backgroundMode;
 
 Composite () {
 	/* Do nothing */
@@ -84,6 +90,30 @@ public Composite (Composite parent, int style) {
 }
 
 Control [] _getChildren () {
+	if (OS.HIVIEW) {
+		short [] buffer = new short [1];
+		OS.CountSubControls (handle, buffer);
+		int count = buffer [0];
+		Control [] children = new Control [count];
+		int i = 0, j = 0;
+		int child = OS.HIViewGetFirstSubview (handle);
+		while (i < count) {
+			if (child != 0) {
+				Widget widget = display.getWidget (child);
+				if (widget != null && widget != this) {
+					if (widget instanceof Control) {
+						children [j++] = (Control) widget;
+					}
+				}
+			}
+			child = OS.HIViewGetNextView (child);
+			i++;
+		}
+		if (j == count) return children;
+		Control [] newChildren = new Control [j];
+		System.arraycopy (children, 0, newChildren, 0, j);
+		return newChildren;
+	}
 	short [] count = new short [1];
 	OS.CountSubControls (handle, count);
 	if (count [0] == 0) return new Control [0];
@@ -223,7 +253,9 @@ Control [] computeTabList () {
 
 void createHandle () {
 	state |= CANVAS | GRAB;
-	if ((style & (SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL)) != 0) {
+	boolean scrolled = (style & (SWT.H_SCROLL | SWT.V_SCROLL)) != 0;
+	if (!scrolled)  state |= THEME_BACKGROUND;
+	if (scrolled || (style & SWT.BORDER) != 0) {
 		createScrolledHandle (parent.handle);
 	} else {
 		createHandle (parent.handle);
@@ -253,21 +285,24 @@ void createScrolledHandle (int parentHandle) {
 	handle = outControl [0];
 }
 
-void drawBackground (int control) {
+void drawBackground (int control, int context) {
 	if (control == scrolledHandle) {
+		Composite parent = this;
+		Shell shell = getShell ();
+		if (shell != this) parent = this.parent;
 		if ((style & SWT.NO_FOCUS) == 0 && hooksKeys ()) {
-			drawFocus (control, hasFocus () && drawFocusRing (), hasBorder (), getParentBackground (), inset ());
+			parent.drawFocus (control, context, hasFocus () && drawFocusRing (), hasBorder (), inset ());
 		} else {
 			if (hasBorder ()) {
-				drawFocus (control, false, hasBorder (), getParentBackground (), inset ());
+				parent.drawFocus (control, context, false, hasBorder (), inset ());
 			} else {
-				drawBackground (control, getParentBackground ());
+				parent.fillBackground (control, context, null);
 			}
 		}
 	} else {
 		if ((state & CANVAS) != 0) {
 			if ((style & SWT.NO_BACKGROUND) == 0) {
-				drawBackground (control, background);
+				fillBackground (control, context, null);
 			}	
 		}
 	}
@@ -283,6 +318,10 @@ void enableWidget (boolean enabled) {
 	super.enableWidget (enabled);
 }
 
+Composite findDeferredControl () {
+	return layoutCount > 0 ? this : parent.findDeferredControl ();
+}
+
 void fixTabList (Control control) {
 	if (tabList == null) return;
 	int count = 0;
@@ -304,9 +343,34 @@ void fixTabList (Control control) {
 	tabList = newList;
 }
 
+/**
+ * Returns the receiver's background drawing mode. This
+ * will be one of the following constants defined in class
+ * <code>SWT</code>:
+ * <code>INHERIT_NONE</code>, <code>INHERIT_DEFAULT</code>,
+ * <code>INHERTIT_FORCE</code>.
+ *
+ * @return the background mode
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see SWT
+ * 
+ * @since 3.2
+ */
+public int getBackgroundMode () {
+	checkWidget ();
+	return backgroundMode;
+}
+
 /**
  * Returns a (possibly empty) array containing the receiver's children.
- * Children are returned in the order that they are drawn.
+ * Children are returned in the order that they are drawn.  The topmost
+ * control appears at the beginning of the array.  Subsequent controls
+ * draw beneath this control and appear later in the array.
  * <p>
  * Note: This is not the actual structure used by the receiver
  * to maintain its list of children, so modifying the array will
@@ -466,7 +530,11 @@ int kEventControlSetFocusPart (int nextHandler, int theEvent, int userData) {
 				if ((style & SWT.NO_FOCUS) == 0 && hooksKeys ()) {
 					short [] part = new short [1];
 					OS.GetEventParameter (theEvent, OS.kEventParamControlPart, OS.typeControlPartCode, null, 2, null, part);
-					drawFocusClipped (scrolledHandle, part [0] != OS.kControlFocusNoPart && drawFocusRing (), hasBorder (), getParentBackground (), inset ());
+					if (OS.HIVIEW) {
+						redrawWidget (scrolledHandle, false);
+					} else {
+						drawFocusClipped (scrolledHandle, part [0] != OS.kControlFocusNoPart && drawFocusRing (), hasBorder (), getParentBackground (), inset ());
+					}
 				}
 			}
 		}
@@ -511,14 +579,15 @@ int kEventRawKeyPressed (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
-int kEventTextInputUnicodeForKeyEvent (int nextHandler, int theEvent, int userData) {
-	int result = super.kEventTextInputUnicodeForKeyEvent (nextHandler, theEvent, userData);
+int kEventUnicodeKeyPressed (int nextHandler, int theEvent, int userData) {
+	int result = super.kEventUnicodeKeyPressed (nextHandler, theEvent, userData);
 	if ((state & CANVAS) != 0) {
 		int [] keyboardEvent = new int [1];
 		OS.GetEventParameter (theEvent, OS.kEventParamTextInputSendKeyboardEvent, OS.typeEventRef, null, keyboardEvent.length * 4, null, keyboardEvent);
 		int [] keyCode = new int [1];
 		OS.GetEventParameter (keyboardEvent [0], OS.kEventParamKeyCode, OS.typeUInt32, null, keyCode.length * 4, null, keyCode);
 		switch (keyCode [0]) {
+			case 76: /* KP Enter */
 			case 36: /* Return */
 				/*
 				* Feature in the Macintosh.  The default behaviour when the return key is pressed is
@@ -564,7 +633,7 @@ void invalidateChildrenVisibleRegion (int control) {
  */
 public boolean isLayoutDeferred () {
 	checkWidget ();
-	return layoutCount > 0 || parent.isLayoutDeferred ();
+	return findDeferredControl () != null;
 }
 
 boolean isTabGroup () {
@@ -743,16 +812,18 @@ Point minimumSize (int wHint, int Hint, boolean changed) {
 	return new Point (width, height);
 }
 
-void releaseChildren () {
+void releaseChildren (boolean destroy) {
 	Control [] children = _getChildren ();
 	for (int i=0; i<children.length; i++) {
 		Control child = children [i];
-		if (!child.isDisposed ()) child.releaseResources ();
+		if (child != null && !child.isDisposed ()) {
+			child.release (false);
+		}
 	}
+	super.releaseChildren (destroy);
 }
 
 void releaseWidget () {
-	releaseChildren ();
 	super.releaseWidget ();
 	if (scrolledVisibleRgn != 0) OS.DisposeRgn (scrolledVisibleRgn);
 	if (siblingsVisibleRgn != 0) OS.DisposeRgn (siblingsVisibleRgn);
@@ -777,6 +848,32 @@ void resetVisibleRegion (int control) {
 	super.resetVisibleRegion (control);
 }
 
+/**
+ * Sets the background drawing mode to the argument which should
+ * be one of the following constants defined in class <code>SWT</code>:
+ * <code>INHERIT_NONE</code>, <code>INHERIT_DEFAULT</code>,
+ * <code>INHERIT_FORCE</code>.
+ *
+ * @param mode the new background mode
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see SWT
+ * 
+ * @since 3.2
+ */
+public void setBackgroundMode (int mode) {
+	checkWidget ();
+	backgroundMode = mode;
+	Control [] children = _getChildren ();
+	for (int i = 0; i < children.length; i++) {
+		children [i].updateBackgroundMode ();
+	}
+}
+
 int setBounds (int x, int y, int width, int height, boolean move, boolean resize, boolean events) {
 	int result = super.setBounds (x, y, width, height, move, resize, events);
 	if (layout != null && (result & RESIZED) != 0) {
@@ -836,13 +933,16 @@ public void setLayout (Layout layout) {
 public void setLayoutDeferred (boolean defer) {
 	if (!defer) {
 		if (--layoutCount == 0) {
-			if (!isLayoutDeferred ()) updateLayout (true);
+			if ((state & LAYOUT_CHILD) != 0 || (state & LAYOUT_NEEDED) != 0) {
+				updateLayout (true);
+			}
 		}
 	} else {
 		layoutCount++;
 	}
 }
 
+
 boolean setScrollBarVisible (ScrollBar bar, boolean visible) {
 	boolean changed = super.setScrollBarVisible (bar, visible);
 	if (changed && layout != null) {
@@ -896,11 +996,6 @@ public void setTabList (Control [] tabList) {
 	this.tabList = tabList;
 }
 
-void setZOrder () {
-	super.setZOrder ();
-	if (scrolledHandle != 0) OS.HIViewAddSubview (scrolledHandle, handle);
-}
-
 int traversalCode (int key, int theEvent) {
 	if ((state & CANVAS) != 0) {
 		if ((style & SWT.NO_FOCUS) != 0) return 0;
@@ -909,14 +1004,27 @@ int traversalCode (int key, int theEvent) {
 	return super.traversalCode (key, theEvent);
 }
 
+void updateBackgroundMode () {
+	super.updateBackgroundMode ();
+	Control [] children = _getChildren ();
+	for (int i = 0; i < children.length; i++) {
+		children [i].updateBackgroundMode ();
+	}
+}
+
 void updateLayout (boolean all) {
-	if (isLayoutDeferred ()) return;
+	Composite parent = findDeferredControl ();
+	if (parent != null) {
+		parent.state |= LAYOUT_CHILD;
+		return;
+	}
 	if ((state & LAYOUT_NEEDED) != 0) {
 		boolean changed = (state & LAYOUT_CHANGED) != 0;
 		state &= ~(LAYOUT_NEEDED | LAYOUT_CHANGED);
 		layout.layout (this, changed);
 	}
 	if (all) {
+		state &= ~LAYOUT_CHILD;
 		Control [] children = _getChildren ();
 		for (int i=0; i<children.length; i++) {
 			children [i].updateLayout (all);
