@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,6 +12,7 @@ package org.eclipse.swt.widgets;
 
 import org.eclipse.swt.internal.carbon.OS;
 import org.eclipse.swt.internal.carbon.Rect;
+import org.eclipse.swt.internal.carbon.CGPoint;
 
 import org.eclipse.swt.*;
 import org.eclipse.swt.events.*;
@@ -37,15 +38,15 @@ import org.eclipse.swt.graphics.*;
 public class Link extends Control {
 	String text;
 	TextLayout layout;
-	Color linkColor, linkDisabledColor;
+	Color linkColor, disabledColor;
 	Point [] offsets;
 	Point selection;
 	String [] ids;
 	int [] mnemonics;
 	int focusIndex;
 	
-    static final RGB LINK_FOREGROUND = new RGB (0, 51, 153);
-    static final RGB LINK_DISABLED_FOREGROUND = new RGB (172, 168, 153);
+	static final RGB LINK_FOREGROUND = new RGB (0, 51, 153);
+	static final RGB LINK_DISABLED_FOREGROUND = new RGB (172, 168, 153);
 
 /**
  * Constructs a new instance of this class given its parent
@@ -142,6 +143,7 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 }
 
 void createHandle () {
+	state |= THEME_BACKGROUND;
 	int features = OS.kControlSupportsFocus | OS.kControlGetsFocusOnClick;
 	int [] outControl = new int [1];
 	int window = OS.GetControlOwner (parent.handle);
@@ -150,7 +152,7 @@ void createHandle () {
 	handle = outControl [0];
 	layout = new TextLayout (display);
 	linkColor = new Color (display, LINK_FOREGROUND);
-    linkDisabledColor = new Color (display, LINK_DISABLED_FOREGROUND);
+	disabledColor = new Color (display, LINK_DISABLED_FOREGROUND);
 	offsets = new Point [0];
 	ids = new String [0];
 	mnemonics = new int [0]; 
@@ -165,14 +167,14 @@ void createWidget () {
 	//TODO accessibility
 }
 
-void drawBackground (int control) {
-	drawBackground (control, background);
+void drawBackground (int control, int context) {
+	fillBackground (control, context, null);
 	if (!hasFocus () || !drawFocusRing () || focusIndex == -1) return;
 	int [] outMetric = new int [1];
 	OS.GetThemeMetric (OS.kThemeMetricFocusRectOutset, outMetric);
 	outMetric[0]--;
 	Rect r = new Rect (), bounds = new Rect();
-	OS.GetControlBounds (control, bounds);
+	if (!OS.HIVIEW) OS.GetControlBounds (control, bounds);
 	Rectangle [] rects = getRectangles (focusIndex);
 	for (int i = 0; i < rects.length; i++) {
 		Rectangle rect = rects [i];
@@ -184,7 +186,7 @@ void drawBackground (int control) {
 	}
 }
 
-void drawWidget (int control, int damageRgn, int visibleRgn, int theEvent) {
+void drawWidget (int control, int context, int damageRgn, int visibleRgn, int theEvent) {
 	GCData data = new GCData ();
 	data.paintEvent = theEvent;
 	data.visibleRgn = visibleRgn;
@@ -197,20 +199,21 @@ void drawWidget (int control, int damageRgn, int visibleRgn, int theEvent) {
 	}
 	// temporary code to disable text selection
 	selStart = selEnd = -1;
+	if ((state & DISABLED) != 0) gc.setForeground (disabledColor);
 	layout.draw (gc, 0, 0, selStart, selEnd, null, null);
 	gc.dispose ();
-	super.drawWidget (control, damageRgn, visibleRgn, theEvent);
+	super.drawWidget (control, context, damageRgn, visibleRgn, theEvent);
 }
 
 void enableWidget (boolean enabled) {
-    super.enableWidget (enabled);
-    TextStyle linkStyle = new TextStyle (null, enabled ? linkColor : linkDisabledColor, null);
-    linkStyle.underline = true;
-    for (int i = 0; i < offsets.length; i++) {
-        Point point = offsets [i];
-        layout.setStyle (linkStyle, point.x, point.y);
-    }
-    redraw ();
+	super.enableWidget (enabled);
+	TextStyle linkStyle = new TextStyle (null, enabled ? linkColor : disabledColor, null);
+	linkStyle.underline = true;
+	for (int i = 0; i < offsets.length; i++) {
+		Point point = offsets [i];
+		layout.setStyle (linkStyle, point.x, point.y);
+	}
+	redraw ();
 }
 
 String getNameText () {
@@ -248,8 +251,7 @@ Rectangle [] getRectangles (int linkIndex) {
 
 /**
  * Returns the receiver's text, which will be an empty
- * string if it has never been set or if the receiver is
- * a <code>SEPARATOR</code> label.
+ * string if it has never been set.
  *
  * @return the receiver's text
  *
@@ -275,22 +277,32 @@ int kEventControlSetFocusPart (int nextHandler, int theEvent, int userData) {
 
 int kEventMouseDown (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventMouseDown (nextHandler, theEvent, userData);
+	if (result == OS.noErr) return result;
 	short [] button = new short [1];
 	OS.GetEventParameter (theEvent, OS.kEventParamMouseButton, OS.typeMouseButton, null, 2, null, button);
 	int [] clickCount = new int [1];
 	OS.GetEventParameter (theEvent, OS.kEventParamClickCount, OS.typeUInt32, null, 4, null, clickCount);
 	if (button [0] == 1 && clickCount [0] == 1) {
-		int sizeof = org.eclipse.swt.internal.carbon.Point.sizeof;
-		org.eclipse.swt.internal.carbon.Point pt = new org.eclipse.swt.internal.carbon.Point ();
-		OS.GetEventParameter (theEvent, OS.kEventParamMouseLocation, OS.typeQDPoint, null, sizeof, null, pt);
-		Rect rect = new Rect ();
-		int window = OS.GetControlOwner (handle);
-		OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
-		int x = pt.h - rect.left;
-		int y = pt.v - rect.top;
-		OS.GetControlBounds (handle, rect);
-		x -= rect.left;
-		y -= rect.top;
+		int x, y;
+		if (OS.HIVIEW) {
+			CGPoint pt = new CGPoint ();
+			OS.GetEventParameter (theEvent, OS.kEventParamWindowMouseLocation, OS.typeHIPoint, null, CGPoint.sizeof, null, pt);
+			OS.HIViewConvertPoint (pt, 0, handle);
+			x = (int) pt.x;
+			y = (int) pt.y;
+		} else {
+			int sizeof = org.eclipse.swt.internal.carbon.Point.sizeof;
+			org.eclipse.swt.internal.carbon.Point pt = new org.eclipse.swt.internal.carbon.Point ();
+			OS.GetEventParameter (theEvent, OS.kEventParamMouseLocation, OS.typeQDPoint, null, sizeof, null, pt);
+			Rect rect = new Rect ();
+			int window = OS.GetControlOwner (handle);
+			OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
+			x = pt.h - rect.left;
+			y = pt.v - rect.top;
+			OS.GetControlBounds (handle, rect);
+			x -= rect.left;
+			y -= rect.top;
+		}
 		int offset = layout.getOffset (x, y, null);
 		int oldSelectionX = selection.x;
 		int oldSelectionY = selection.y;
@@ -319,9 +331,11 @@ int kEventMouseDown (int nextHandler, int theEvent, int userData) {
 	}
 	
 	int window = OS.GetControlOwner (handle);
-	int port = OS.GetWindowPort (window);
+	int port = OS.HIVIEW ? -1 : OS.GetWindowPort (window);
 	int [] outModifiers = new int [1];
 	short [] outResult = new short [1];
+	CGPoint pt = new CGPoint ();
+	Rect rect = new Rect ();
 	org.eclipse.swt.internal.carbon.Point outPt = new org.eclipse.swt.internal.carbon.Point ();
 	while (outResult [0] != OS.kMouseTrackingMouseUp) {
 		OS.TrackMouseLocationWithOptions (port, 0, OS.kEventDurationForever, outPt, outModifiers, outResult);
@@ -331,13 +345,22 @@ int kEventMouseDown (int nextHandler, int theEvent, int userData) {
 			case OS.kMouseTrackingMouseDragged: {
 				int chord = OS.GetCurrentEventButtonState ();
 				if ((chord & 0x01) != 0) {
-					Rect rect = new Rect ();
-					OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
-					int x = outPt.h;
-					int y = outPt.v;
-					OS.GetControlBounds (handle, rect);
-					x -= rect.left;
-					y -= rect.top;
+					int x, y;
+					if (OS.HIVIEW) {
+						OS.GetWindowBounds (window, (short) OS.kWindowStructureRgn, rect);
+						pt.x = outPt.h - rect.left;
+						pt.y = outPt.v - rect.top;
+						OS.HIViewConvertPoint (pt, 0, handle);
+						x = (int) pt.x;
+						y = (int) pt.y;
+					} else {
+						OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
+						x = outPt.h;
+						y = outPt.v;
+						OS.GetControlBounds (handle, rect);
+						x -= rect.left;
+						y -= rect.top;
+					}
 					int oldSelection = selection.y;
 					selection.y = layout.getOffset (x, y, null);
 					if (selection.y != oldSelection) {
@@ -349,7 +372,7 @@ int kEventMouseDown (int nextHandler, int theEvent, int userData) {
 						}
 						Rectangle rectangle = layout.getBounds (oldSelection, newSelection);
 						redraw (rectangle.x, rectangle.y, rectangle.width, rectangle.height, false);
-						update ();
+						if (!OS.HIVIEW) update ();
 					}
 				}
 				break;
@@ -365,17 +388,27 @@ int kEventMouseDown (int nextHandler, int theEvent, int userData) {
 
 int kEventMouseMoved (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventMouseMoved (nextHandler, theEvent, userData);
-	int sizeof = org.eclipse.swt.internal.carbon.Point.sizeof;
-	org.eclipse.swt.internal.carbon.Point pt = new org.eclipse.swt.internal.carbon.Point ();
-	OS.GetEventParameter (theEvent, OS.kEventParamMouseLocation, OS.typeQDPoint, null, sizeof, null, pt);
-	Rect rect = new Rect ();
-	int window = OS.GetControlOwner (handle);
-	OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
-	int x = pt.h - rect.left;
-	int y = pt.v - rect.top;
-	OS.GetControlBounds (handle, rect);
-	x -= rect.left;
-	y -= rect.top;
+	if (result == OS.noErr) return result;
+	int x, y;
+	if (OS.HIVIEW) {
+		CGPoint pt = new CGPoint ();
+		OS.GetEventParameter (theEvent, OS.kEventParamWindowMouseLocation, OS.typeHIPoint, null, CGPoint.sizeof, null, pt);
+		OS.HIViewConvertPoint (pt, 0, handle);
+		x = (int) pt.x;
+		y = (int) pt.y;
+	} else {
+		int sizeof = org.eclipse.swt.internal.carbon.Point.sizeof;
+		org.eclipse.swt.internal.carbon.Point pt = new org.eclipse.swt.internal.carbon.Point ();
+		OS.GetEventParameter (theEvent, OS.kEventParamMouseLocation, OS.typeQDPoint, null, sizeof, null, pt);
+		Rect rect = new Rect ();
+		int window = OS.GetControlOwner (handle);
+		OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
+		x = pt.h - rect.left;
+		y = pt.v - rect.top;
+		OS.GetControlBounds (handle, rect);
+		x -= rect.left;
+		y -= rect.top;
+	}
 	for (int j = 0; j < offsets.length; j++) {
 		Rectangle [] rects = getRectangles (j);
 		for (int i = 0; i < rects.length; i++) {
@@ -392,21 +425,31 @@ int kEventMouseMoved (int nextHandler, int theEvent, int userData) {
 
 int kEventMouseUp (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventMouseUp (nextHandler, theEvent, userData);
+	if (result == OS.noErr) return result;
 	if (focusIndex == -1) return result;
 	short [] button = new short [1];
 	OS.GetEventParameter (theEvent, OS.kEventParamMouseButton, OS.typeMouseButton, null, 2, null, button);
 	if (button [0] == 1) {
-		int sizeof = org.eclipse.swt.internal.carbon.Point.sizeof;
-		org.eclipse.swt.internal.carbon.Point pt = new org.eclipse.swt.internal.carbon.Point ();
-		OS.GetEventParameter (theEvent, OS.kEventParamMouseLocation, OS.typeQDPoint, null, sizeof, null, pt);
-		Rect rect = new Rect ();
-		int window = OS.GetControlOwner (handle);
-		OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
-		int x = pt.h - rect.left;
-		int y = pt.v - rect.top;
-		OS.GetControlBounds (handle, rect);
-		x -= rect.left;
-		y -= rect.top;
+		int x, y;
+		if (OS.HIVIEW) {
+			CGPoint pt = new CGPoint ();
+			OS.GetEventParameter (theEvent, OS.kEventParamWindowMouseLocation, OS.typeHIPoint, null, CGPoint.sizeof, null, pt);
+			OS.HIViewConvertPoint (pt, 0, handle);
+			x = (int) pt.x;
+			y = (int) pt.y;
+		} else {
+			int sizeof = org.eclipse.swt.internal.carbon.Point.sizeof;
+			org.eclipse.swt.internal.carbon.Point pt = new org.eclipse.swt.internal.carbon.Point ();
+			OS.GetEventParameter (theEvent, OS.kEventParamMouseLocation, OS.typeQDPoint, null, sizeof, null, pt);
+			Rect rect = new Rect ();
+			int window = OS.GetControlOwner (handle);
+			OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
+			x = pt.h - rect.left;
+			y = pt.v - rect.top;
+			OS.GetControlBounds (handle, rect);
+			x -= rect.left;
+			y -= rect.top;
+		}
 		Rectangle [] rects = getRectangles (focusIndex);
 		for (int i = 0; i < rects.length; i++) {
 			Rectangle rectangle = rects [i];
@@ -421,10 +464,10 @@ int kEventMouseUp (int nextHandler, int theEvent, int userData) {
 	return result;
 }
 
-int kEventTextInputUnicodeForKeyEvent (int nextHandler, int theEvent, int userData) {
-	int result = super.kEventTextInputUnicodeForKeyEvent (nextHandler, theEvent, userData);
-	if (focusIndex == -1) return result;
+int kEventUnicodeKeyPressed (int nextHandler, int theEvent, int userData) {
+	int result = super.kEventUnicodeKeyPressed (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
+	if (focusIndex == -1) return result;
 	int [] keyboardEvent = new int [1];
 	OS.GetEventParameter (theEvent, OS.kEventParamTextInputSendKeyboardEvent, OS.typeEventRef, null, keyboardEvent.length * 4, null, keyboardEvent);
 	int [] keyCode = new int [1];
@@ -464,8 +507,8 @@ void releaseWidget () {
 	layout = null;
 	if (linkColor != null) linkColor.dispose ();
 	linkColor = null;
-    if (linkDisabledColor != null) linkDisabledColor.dispose ();
-    linkDisabledColor = null;
+	if (disabledColor != null) disabledColor.dispose ();
+	disabledColor = null;
 	offsets = null;
 	ids = null;
 	mnemonics = null;
@@ -690,8 +733,8 @@ public void setText (String string) {
 	layout.setText (parse (string));
 	focusIndex = offsets.length > 0 ? 0 : -1 ;
 	selection.x = selection.y = -1;
-    boolean enabled = (state & DISABLED) == 0;
-    TextStyle linkStyle = new TextStyle (null, enabled ? linkColor : linkDisabledColor, null);
+	boolean enabled = (state & DISABLED) == 0;
+	TextStyle linkStyle = new TextStyle (null, enabled ? linkColor : disabledColor, null);
 	linkStyle.underline = true;
 	for (int i = 0; i < offsets.length; i++) {
 		Point point = offsets [i];
