@@ -51,7 +51,7 @@ public class Menu extends Widget {
 	 */
 	int handle;
 	short id;
-	int x, y;
+	int x, y, itemCount;
 //	int width, height;
 	boolean hasLocation, modified, closed;
 	MenuItem [] items;
@@ -278,24 +278,26 @@ void createHandle () {
 
 void createItem (MenuItem item, int index) {
 	checkWidget ();
-	int count = OS.CountMenuItems (handle);
-	if (!(0 <= index && index <= count)) error (SWT.ERROR_INVALID_RANGE);
+	if (!(0 <= index && index <= itemCount)) error (SWT.ERROR_INVALID_RANGE);
 	int attributes = OS.kMenuItemAttrAutoRepeat | OS.kMenuItemAttrCustomDraw;
 	if ((item.style & SWT.SEPARATOR) != 0) attributes = OS.kMenuItemAttrSeparator;
 	int result = OS.InsertMenuItemTextWithCFString (handle, 0, (short) index, attributes, 0);
 	if (result != OS.noErr) {
 		error (SWT.ERROR_ITEM_NOT_ADDED);
 	}
-	if (count == items.length) {
+	if (itemCount == items.length) {
 		MenuItem [] newItems = new MenuItem [items.length + 4];
 		System.arraycopy (items, 0, newItems, 0, items.length);
 		items = newItems;
 	}
-	System.arraycopy (items, index, items, index + 1, count - index);
+	System.arraycopy (items, index, items, index + 1, itemCount++ - index);
 	items [index] = item;
 	modified = true;
 	int emptyMenu = item.createEmptyMenu ();
-	if (emptyMenu != 0) OS.SetMenuItemHierarchicalMenu (handle, (short) (index + 1), emptyMenu);
+	if (emptyMenu != 0) {
+		OS.SetMenuItemHierarchicalMenu (handle, (short) (index + 1), emptyMenu);
+		OS.ReleaseMenu (emptyMenu);
+	}
 }
 
 void createWidget () {
@@ -305,17 +307,15 @@ void createWidget () {
 }
 
 void destroyItem (MenuItem item) {
-	int count = OS.CountMenuItems (handle);
 	int index = 0;
-	while (index < count) {
+	while (index < itemCount) {
 		if (items [index] == item) break;
 		index++;
 	}
-	if (index == count) return;
-	System.arraycopy (items, index + 1, items, index, --count - index);
-	items [count] = null;
-	if (count == 0) items = new MenuItem [4];
-	if (item.menu == null) item.destroyEmptyMenu (index);
+	if (index == itemCount) return;
+	System.arraycopy (items, index + 1, items, index, --itemCount - index);
+	items [itemCount] = null;
+	if (itemCount == 0) items = new MenuItem [4];
 	modified = true;
 	OS.DeleteMenuItem (handle, (short) (index + 1));
 }
@@ -324,7 +324,6 @@ void destroyWidget () {
 	int theMenu = handle;
 	releaseHandle ();
 	if (theMenu != 0) {
-		OS.DeleteMenu (OS.GetMenuID (theMenu));
 		OS.DisposeMenu (theMenu);
 	}
 }
@@ -401,8 +400,7 @@ public boolean getEnabled () {
  */
 public MenuItem getItem (int index) {
 	checkWidget ();
-	int count = OS.CountMenuItems (handle);
-	if (!(0 <= index && index < count)) error (SWT.ERROR_INVALID_RANGE);
+	if (!(0 <= index && index < itemCount)) error (SWT.ERROR_INVALID_RANGE);
 	return items [index];
 }
 
@@ -418,7 +416,7 @@ public MenuItem getItem (int index) {
  */
 public int getItemCount () {
 	checkWidget ();
-	return OS.CountMenuItems (handle);
+	return itemCount;
 }
 
 /**
@@ -439,9 +437,21 @@ public int getItemCount () {
  */
 public MenuItem [] getItems () {
 	checkWidget ();
-	int count = OS.CountMenuItems (handle);
-	MenuItem [] result = new MenuItem [count];
-	System.arraycopy (items, 0, result, 0, count);
+	MenuItem [] result = new MenuItem [itemCount];
+	int index = 0;
+	if (items != null) {
+		for (int i = 0; i < itemCount; i++) {
+			MenuItem item = items [i];
+			if (item != null && !item.isDisposed ()) {
+				result [index++] = item;
+			}
+		}
+	}
+	if (index != result.length) {
+		MenuItem [] newItems = new MenuItem[index];
+		System.arraycopy(result, 0, newItems, 0, index);
+		result = newItems;
+	}
 	return result;
 }
 
@@ -604,8 +614,10 @@ int kEventMenuClosed (int nextHandler, int theEvent, int userData) {
 //		item.x = item.y = item.width = item.height = 0;
 //	}
 	sendEvent (SWT.Hide);
-	if (hooks (SWT.Hide)) {
-		getShell().update (true);
+	if (!OS.HIVIEW) {
+		if (hooks (SWT.Hide)) {
+			getShell().update (true);
+		}
 	}
 	return OS.eventNotHandledErr;
 }
@@ -753,8 +765,10 @@ int kEventMenuOpening (int nextHandler, int theEvent, int userData) {
 	closed = false;
 	sendEvent (SWT.Show);
 	modified = false;
-	if (hooks (SWT.Show)) {
-		getShell().update (true);
+	if (!OS.HIVIEW) {
+		if (hooks (SWT.Show)) {
+			getShell().update (true);
+		}
 	}
 	return OS.eventNotHandledErr;
 }
@@ -768,8 +782,10 @@ int kEventMenuTargetItem (int nextHandler, int theEvent, int userData) {
 		if (index [0] > 0) lastTarget = items [index [0] - 1];
 		if (lastTarget != null) {
 			lastTarget.sendEvent (SWT.Arm);
-			if (lastTarget.hooks (SWT.Arm)) {
-				getShell().update (true);
+			if (!OS.HIVIEW) {
+				if (lastTarget.hooks (SWT.Arm)) {
+					getShell().update (true);
+				}
 			}
 		}
 	}
@@ -796,8 +812,7 @@ int kEventMenuTargetItem (int nextHandler, int theEvent, int userData) {
 public int indexOf (MenuItem item) {
 	checkWidget ();
 	if (item == null) error (SWT.ERROR_NULL_ARGUMENT);
-	int count = OS.CountMenuItems (handle);
-	for (int i=0; i<count; i++) {
+	for (int i=0; i<itemCount; i++) {
 		if (items [i] == item) return i;
 	}
 	return -1;
@@ -864,12 +879,17 @@ int modifierIndex (String accelText) {
 	return -1;
 }
 
-void releaseChild () {
-	super.releaseChild ();
-	if (cascade != null) cascade.setMenu (null);
-	if ((style & SWT.BAR) != 0 && this == parent.menuBar) {
-		parent.setMenuBar (null);
+void releaseChildren (boolean destroy) {
+	if (items != null) {
+		for (int i=0; i<items.length; i++) {
+			MenuItem item = items [i];
+			if (item != null && !item.isDisposed ()) {
+				item.release (false);
+			}
+		}
+		items = null;
 	}
+	super.releaseChildren (destroy);
 }
 
 void releaseHandle () {
@@ -877,13 +897,15 @@ void releaseHandle () {
 	handle = 0;
 }
 
-void releaseWidget () {
-	int count = OS.CountMenuItems (handle);
-	for (int i=0; i<count; i++) {
-		MenuItem item = items [i];
-		if (!item.isDisposed ()) item.releaseResources ();
+void releaseParent () {
+	super.releaseParent ();
+	if (cascade != null) cascade.setMenu (null);
+	if ((style & SWT.BAR) != 0 && this == parent.menuBar) {
+		parent.setMenuBar (null);
 	}
-	items = null;
+}
+
+void releaseWidget () {
 	super.releaseWidget ();
 	display.removeMenu (this);
 	parent = null;
