@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -55,9 +55,18 @@ import org.eclipse.swt.graphics.*;
  * of these changes are also possible.
  * </li>
  * </ul>
- * </p>
- * <p>
- * Note: The styles supported by this class must be treated
+ * </p><p>
+ * The <em>modality</em> of an instance may be specified using
+ * style bits. The modality style bits are used to determine
+ * whether input is blocked for other shells on the display.
+ * The <code>PRIMARY_MODAL</code> style allows an instance to block
+ * input to its parent. The <code>APPLICATION_MODAL</code> style
+ * allows an instance to block input to every other shell in the
+ * display. The <code>SYSTEM_MODAL</code> style allows an instance
+ * to block input to all shells, including shells belonging to
+ * different applications.
+ * </p><p>
+ * Note: The styles supported by this class are treated
  * as <em>HINT</em>s, since the window manager for the
  * desktop on which the instance is visible has ultimate
  * control over the appearance and behavior of decorations
@@ -68,6 +77,11 @@ import org.eclipse.swt.graphics.*;
  * more restrictive modality style that is supported. For
  * example, if <code>PRIMARY_MODAL</code> is not supported,
  * it would be upgraded to <code>APPLICATION_MODAL</code>.
+ * A modality style may also be "downgraded" to a less
+ * restrictive style. For example, most operating systems
+ * no longer support <code>SYSTEM_MODAL</code> because
+ * it can freeze up the desktop, so this is typically
+ * downgraded to <code>APPLICATION_MODAL</code>.
  * <dl>
  * <dt><b>Styles:</b></dt>
  * <dd>BORDER, CLOSE, MIN, MAX, NO_TRIM, RESIZE, TITLE, ON_TOP, TOOL</dd>
@@ -104,7 +118,7 @@ import org.eclipse.swt.graphics.*;
  */
 public class Shell extends Decorations {
 	int shellHandle, windowGroup;
-	boolean resized, moved, drawing, reshape, update, activate, disposed, opened;
+	boolean resized, moved, drawing, reshape, update, deferDispose, activate, active, disposed, opened;
 	int invalRgn;
 	Control lastActive;
 	Region region;
@@ -423,7 +437,8 @@ public Rectangle computeTrim (int x, int y, int width, int height) {
 
 void createHandle () {
 	state |= CANVAS | GRAB | HIDDEN;
-	int attributes = OS.kWindowStandardHandlerAttribute; // | OS.kWindowCompositingAttribute;
+	int attributes = OS.kWindowStandardHandlerAttribute;
+	if (OS.HIVIEW) attributes |= OS.kWindowCompositingAttribute;
 	if ((style & SWT.NO_TRIM) == 0) {
 		if ((style & SWT.CLOSE) != 0) attributes |= OS.kWindowCloseBoxAttribute;
 		if ((style & SWT.MIN) != 0) attributes |= OS.kWindowCollapseBoxAttribute;
@@ -442,11 +457,11 @@ void createHandle () {
 	}
 	int windowClass = OS.kDocumentWindowClass;
 	if ((style & (SWT.CLOSE | SWT.TITLE)) == 0) windowClass = OS.kSheetWindowClass;
-//	if ((style & (SWT.APPLICATION_MODAL | SWT.PRIMARY_MODAL | SWT.SYSTEM_MODAL)) != 0) {
-//		if ((style & SWT.CLOSE) == 0)  {
-//			windowClass = (style & SWT.TITLE) != 0 ? OS.kMovableModalWindowClass : OS.kModalWindowClass;
-//		}
-//	}
+	if ((style & (SWT.APPLICATION_MODAL | SWT.PRIMARY_MODAL | SWT.SYSTEM_MODAL)) != 0) {
+		if ((style & (SWT.CLOSE | SWT.MAX | SWT.MIN)) == 0)  {
+			windowClass = (style & SWT.TITLE) != 0 ? OS.kMovableModalWindowClass : OS.kModalWindowClass;
+		}
+	}
 	Monitor monitor = getMonitor ();
 	Rectangle rect = monitor.getClientArea ();
 	int width = rect.width * 5 / 8;
@@ -461,8 +476,12 @@ void createHandle () {
 	OS.RepositionWindow (shellHandle, 0, OS.kWindowCascadeOnMainScreen);
 //	OS.SetThemeWindowBackground (shellHandle, (short) OS.kThemeBrushDialogBackgroundActive, false);
 	int [] theRoot = new int [1];
-	OS.CreateRootControl (shellHandle, theRoot);
-	OS.GetRootControl (shellHandle, theRoot);
+	if (OS.HIVIEW) {
+		OS.CreateRootControl (shellHandle, theRoot);
+		OS.GetRootControl (shellHandle, theRoot);
+	} else {
+		OS.HIViewFindByID (OS.HIViewGetRoot (shellHandle), OS.kHIViewWindowContentID (), theRoot);
+	}
 	if (theRoot [0] == 0) error (SWT.ERROR_NO_HANDLES);
 	if ((style & (SWT.H_SCROLL | SWT.V_SCROLL)) != 0) {
 		createScrolledHandle (theRoot [0]);
@@ -474,15 +493,18 @@ void createHandle () {
 	OS.CreateWindowGroup (OS.kWindowGroupAttrHideOnCollapse, outGroup);
 	if (outGroup [0] == 0) error (SWT.ERROR_NO_HANDLES);
 	windowGroup = outGroup [0];
-	if (parent != null) {
-		Shell shell = parent.getShell ();
-		int parentGroup = shell.windowGroup;
-		OS.SetWindowGroup (shellHandle, parentGroup);
-		OS.SetWindowGroupParent (windowGroup, parentGroup);
+	int parentGroup;
+	if ((style & SWT.ON_TOP) != 0) {
+		parentGroup = OS.GetWindowGroupOfClass (OS.kFloatingWindowClass);		
 	} else {
-		int parentGroup = OS.GetWindowGroupOfClass (windowClass);
-		OS.SetWindowGroupParent (windowGroup, parentGroup);
+		if (parent != null) {
+			parentGroup = parent.getShell ().windowGroup;
+		} else {
+			parentGroup = OS.GetWindowGroupOfClass (windowClass);
+		}
 	}
+	OS.SetWindowGroup (shellHandle, parentGroup);
+	OS.SetWindowGroupParent (windowGroup, parentGroup);
 	OS.SetWindowGroupOwner (windowGroup, shellHandle);
 	CGPoint inMinLimits = new CGPoint (), inMaxLimits = new CGPoint ();
 	OS.GetWindowResizeLimits (shellHandle, inMinLimits, inMaxLimits);
@@ -516,16 +538,32 @@ void destroyWidget () {
 	* causes a segment fault when an application is exiting.  This
 	* seems to happen to large applications.  The fix is to avoid
 	* calling HideWindow() when a shell is about to be disposed.
+	* 
+	* Bug in the Macintosh.  Disposing a window from kEventWindowDeactivated
+	* causes a segment fault. The fix is to defer disposing the window until
+	* the event loop becomes idle.
 	*/
-//	OS.HideWindow (shellHandle);
+	Display display = this.display;
+	Composite parent = this.parent;
+	while (!deferDispose && parent != null) {
+		Shell shell = parent.getShell ();
+		deferDispose = shell.deferDispose;
+		parent = shell.parent;
+	}
+	if (deferDispose) OS.HideWindow (shellHandle);
 	releaseHandle ();
 	if (theWindow != 0) {
-		OS.DisposeWindow (theWindow);
+		if (deferDispose) {
+			display.addDisposeWindow (theWindow);
+		} else {
+			OS.DisposeWindow (theWindow);
+		}
 	} 
 }
 
-void drawWidget (int control, int damageRgn, int visibleRgn, int theEvent) {
-	super.drawWidget (control, damageRgn, visibleRgn, theEvent);
+void drawWidget (int control, int context, int damageRgn, int visibleRgn, int theEvent) {
+	super.drawWidget (control, context, damageRgn, visibleRgn, theEvent);
+	
 	/*
 	* Bug in the Macintosh. In kEventWindowGetRegion, 
 	* Carbon assumes the origin of the Region is (0, 0)
@@ -543,26 +581,39 @@ void drawWidget (int control, int damageRgn, int visibleRgn, int theEvent) {
 	boolean limit = region.contains(rgnRect.right - 1, rgnRect.bottom - 1);
 	if (origin && limit) return;
 	
-	int[] context = new int [1];
-	int port = OS.GetWindowPort (shellHandle);
-	Rect portRect = new Rect ();
-	OS.GetPortBounds (port, portRect);
-	OS.QDBeginCGContext (port, context);
-	OS.CGContextScaleCTM (context [0], 1, -1);
-	OS.CGContextTranslateCTM (context [0], 0, portRect.top - portRect.bottom);
+	int port = 0; 
+	int [] buffer = null;
+	if (!OS.HIVIEW) {
+		buffer = new int [1];
+		Rect portRect = new Rect ();
+		port = OS.GetWindowPort (shellHandle);
+		OS.GetPortBounds (port, portRect);
+		OS.QDBeginCGContext (port, buffer);
+		context = buffer [0];
+		OS.CGContextScaleCTM (context, 1, -1);
+		OS.CGContextTranslateCTM (context, 0, portRect.top - portRect.bottom);
+	}
 	CGRect cgRect = new CGRect ();
 	cgRect.width = 1;
 	cgRect.height = 1;
 	if (!origin) {
-		OS.CGContextClearRect (context [0], cgRect);
+		OS.CGContextClearRect (context, cgRect);
 	}
 	if (!limit) {
 		cgRect.x = rgnRect.right - 1;
 		cgRect.y = rgnRect.bottom - 1;
-		OS.CGContextClearRect (context [0], cgRect);
+		OS.CGContextClearRect (context, cgRect);
 	}
-	OS.CGContextSynchronize (context [0]);
-	OS.QDEndCGContext (port, context);
+	OS.CGContextSynchronize (context);
+	if (!OS.HIVIEW) OS.QDEndCGContext (port, buffer);
+}
+
+Control findBackgroundControl () {
+	return background != null || backgroundImage != null ? this : null;
+}
+
+Composite findDeferredControl () {
+	return layoutCount > 0 ? this : null;
 }
 
 Cursor findCursor () {
@@ -598,13 +649,6 @@ public void forceActive () {
 	OS.SetFrontProcessWithOptions (new int [] {0, OS.kCurrentProcess}, OS.kSetFrontProcessFrontWindowOnly);
 }
 
-public Rectangle getClientArea () {
-	checkWidget();
-	Rect rect = new Rect ();
-	OS.GetWindowBounds (shellHandle, (short) OS.kWindowContentRgn, rect);
-	return new Rectangle (0, 0, rect.right - rect.left, rect.bottom - rect.top);
-}
-
 public Rectangle getBounds () {
 	checkWidget();
 	Rect rect = new Rect ();
@@ -755,6 +799,10 @@ public Point getSize () {
 	return new Point (rect.right - rect.left, rect.bottom - rect.top);
 }
 
+float getThemeAlpha () {
+	return 1;
+}
+
 public boolean getVisible () {
 	checkWidget();
     return OS.IsWindowVisible (shellHandle);
@@ -781,6 +829,7 @@ void hookEvents () {
 		OS.kEventClassWindow, OS.kEventWindowHitTest,
 		OS.kEventClassWindow, OS.kEventWindowShown,
 		OS.kEventClassWindow, OS.kEventWindowUpdate,
+		OS.kEventClassWindow, OS.kEventWindowGetClickModality,
 	};
 	int windowTarget = OS.GetWindowEventTarget (shellHandle);
 	OS.InstallEventHandler (windowTarget, windowProc, mask1.length / 2, mask1, shellHandle, null);
@@ -802,6 +851,12 @@ void invalidateVisibleRegion (int control) {
 }
 
 void invalWindowRgn (int window, int rgn) {
+	if (OS.HIVIEW) {
+		display.needsPaint = true;
+		if (invalRgn == 0) invalRgn = OS.NewRgn();
+		OS.UnionRgn (rgn, invalRgn, invalRgn);
+		return;
+	}
 	/*
 	* Bug in the Macintosh.  InvalidWindowRgn() will not invalidate
 	* the window when it is called from the default kEventWindowUpdate
@@ -827,11 +882,6 @@ boolean isEnabledCursor () {
 	return true;
 }
 
-public boolean isLayoutDeferred () {
-	checkWidget ();
-	return layoutCount > 0;
-}
-
 public boolean isVisible () {
 	checkWidget();
 	return getVisible ();
@@ -849,13 +899,16 @@ int kEventWindowActivated (int nextHandler, int theEvent, int userData) {
 	int [] outScope = new int [1];
 	OS.GetWindowActivationScope (shellHandle, outScope); 
 	if (outScope [0] == OS.kWindowActivationScopeNone) return result;
-	display.setMenuBar (menuBar);
-	if (menuBar != null) OS.DrawMenuBar ();
-	activate = true;
-	sendEvent (SWT.Activate);
-	if (isDisposed ()) return result;
-	restoreFocus ();
-	activate = false;
+	if (!active) {
+		active = true;
+		display.setMenuBar (menuBar);
+		if (menuBar != null) OS.DrawMenuBar ();
+		activate = true;
+		sendEvent (SWT.Activate);
+		if (isDisposed ()) return result;
+		if (!restoreFocus () && !traverseGroup (true)) setFocus ();
+		activate = false;
+	}
 	return result;
 }
 
@@ -905,25 +958,28 @@ int kEventWindowCollapsed (int nextHandler, int theEvent, int userData) {
 int kEventWindowDeactivated (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventWindowDeactivated (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
-	//TEMPORARY CODE - should be send, but causes a GP
-	Display display = this.display;
-	postEvent (SWT.Deactivate);
-	if (isDisposed ()) return result;
-	saveFocus ();
-	if (savedFocus != null) {
-		/*
-		* Bug in the Macintosh.  When ClearKeyboardFocus() is called,
-		* the control that has focus gets two kEventControlSetFocus
-		* events indicating that focus was lost.  The fix is to ignore
-		* both of these and send the focus lost event explicitly.
-		*/
-		display.ignoreFocus = true;
-		OS.ClearKeyboardFocus (shellHandle);
-		display.ignoreFocus = false;
-		//TEMPORARY CODE - should be send, but causes a GP
-		if (!savedFocus.isDisposed ()) savedFocus.sendFocusEvent (SWT.FocusOut, true);
+	if (active) {
+		active = false;
+		deferDispose = true;
+		Display display = this.display;
+		sendEvent (SWT.Deactivate);
+		if (isDisposed ()) return result;
+		saveFocus ();
+		if (savedFocus != null) {
+			/*
+			* Bug in the Macintosh.  When ClearKeyboardFocus() is called,
+			* the control that has focus gets two kEventControlSetFocus
+			* events indicating that focus was lost.  The fix is to ignore
+			* both of these and send the focus lost event explicitly.
+			*/
+			display.ignoreFocus = true;
+			OS.ClearKeyboardFocus (shellHandle);
+			display.ignoreFocus = false;
+			if (!savedFocus.isDisposed ()) savedFocus.sendFocusEvent (SWT.FocusOut, false);
+		}
+		display.setMenuBar (null);
+		deferDispose = false;
 	}
-	display.setMenuBar (null);
 	return result;
 }
 
@@ -946,6 +1002,20 @@ int kEventWindowExpanded (int nextHandler, int theEvent, int userData) {
 	return result;
 }
 
+int kEventWindowGetClickModality (int nextHandler, int theEvent, int userData) {
+	/*
+	* Bug in the Macintosh.  When the user clicks in an Expose window that is
+	* disabled because of a modal dialog, the modal dialog does not come to
+	* front and is obscured by the Expose window.  The fix is to select the
+	* modal dialog.
+	*/
+	int result = OS.CallNextEventHandler (nextHandler, theEvent);
+	int [] modal = new int [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamModalWindow, OS.typeWindowRef, null, 4, null, modal);
+	if (modal [0] != 0) OS.SelectWindow (modal [0]);
+	return result;
+}
+
 int kEventWindowGetRegion (int nextHandler, int theEvent, int userData) {
 	int result = super.kEventWindowGetRegion (nextHandler, theEvent, userData);
 	if (result == OS.noErr) return result;
@@ -1084,7 +1154,9 @@ public void open () {
 	bringToTop (false);
 	setVisible (true);
 	if (isDisposed ()) return;
-	if (!restoreFocus () && !traverseGroup (true)) setFocus ();
+	if (active) {
+		if (!restoreFocus () && !traverseGroup (true)) setFocus ();
+	}
 }
 
 void register () {
@@ -1094,8 +1166,15 @@ void register () {
 	display.addWidget (theRoot [0], this);
 }
 
-void releaseChild () {
-	/* Do nothing */
+void releaseChildren (boolean destroy) {
+	Shell [] shells = getShells ();
+	for (int i=0; i<shells.length; i++) {
+		Shell shell = shells [i];
+		if (shell != null && !shell.isDisposed ()) {
+			shell.dispose ();
+		}
+	}
+	super.releaseChildren (destroy);
 }
 
 void releaseHandle () {
@@ -1103,18 +1182,13 @@ void releaseHandle () {
 	shellHandle = 0;
 }
 
-void releaseShells () {
-	Shell [] shells = getShells ();
-	for (int i=0; i<shells.length; i++) {
-		Shell shell = shells [i];
-		if (!shell.isDisposed ()) shell.dispose ();
-	}
+void releaseParent () {
+	/* Do nothing */
 }
 
 void releaseWidget () {
-	disposed = true;
-	releaseShells ();
 	super.releaseWidget ();
+	disposed = true;
 	if (windowGroup != 0) OS.ReleaseWindowGroup (windowGroup);
 	display.updateQuitMenu ();
 	if (invalRgn != 0) OS.DisposeRgn (invalRgn);
@@ -1405,6 +1479,7 @@ public void setRegion (Region region) {
 		reshape = true;
 	} else {
 		OS.ReshapeCustomWindow (shellHandle);
+		if (OS.HIVIEW) redrawWidget (handle, true);
 	}
 }
 
@@ -1437,21 +1512,6 @@ void setWindowVisible (boolean visible) {
 	if (visible) {
 		sendEvent (SWT.Show);
 		if (isDisposed ()) return;
-		opened = true;
-		if (!moved) {
-			moved = true;
-			sendEvent (SWT.Move);
-			if (isDisposed ()) return;
-		}
-		if (!resized) {
-			resized = true;
-			sendEvent (SWT.Resize);
-			if (isDisposed ()) return;
-			if (layout != null) {
-				markLayout (false, false);
-				updateLayout (false);
-			}
-		}
 		int inModalKind = OS.kWindowModalityNone;
 		if ((style & SWT.PRIMARY_MODAL) != 0) inModalKind = OS.kWindowModalityWindowModal;
 		if ((style & SWT.APPLICATION_MODAL) != 0) inModalKind = OS.kWindowModalityAppModal;
@@ -1473,12 +1533,26 @@ void setWindowVisible (boolean visible) {
 		OS.RetainWindow (shellHandle);
 		OS.ShowWindow (shellHandle);
 		OS.ReleaseWindow (shellHandle);
-		if (!isDisposed()) {
-			if (minimized != OS.IsWindowCollapsed (shellHandle)) {
-				OS.CollapseWindow (shellHandle, minimized);
-			}
-			if ((style & SWT.ON_TOP) != 0) {
-				OS.SetWindowActivationScope (shellHandle, scope [0]);
+		if (isDisposed()) return;
+		if (minimized != OS.IsWindowCollapsed (shellHandle)) {
+			OS.CollapseWindow (shellHandle, minimized);
+		}
+		if ((style & SWT.ON_TOP) != 0) {
+			OS.SetWindowActivationScope (shellHandle, scope [0]);
+		}
+		opened = true;
+		if (!moved) {
+			moved = true;
+			sendEvent (SWT.Move);
+			if (isDisposed ()) return;
+		}
+		if (!resized) {
+			resized = true;
+			sendEvent (SWT.Resize);
+			if (isDisposed ()) return;
+			if (layout != null) {
+				markLayout (false, false);
+				updateLayout (false);
 			}
 		}
 	} else {
