@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -13,6 +13,7 @@ package org.eclipse.swt.widgets;
  
 import org.eclipse.swt.internal.carbon.OS;
 import org.eclipse.swt.internal.carbon.Rect;
+import org.eclipse.swt.internal.carbon.CGPoint;
 
 import org.eclipse.swt.*;
 import org.eclipse.swt.events.*;
@@ -177,8 +178,10 @@ int actionProc (int theControl, int partCode) {
 	}
 	OS.SetControl32BitValue (handle, value);
 	sendEvent (SWT.Selection, event);
-	Shell shell = getShell ();
-	shell.update (true);
+	if (!OS.HIVIEW) {
+		Shell shell = getShell ();
+		shell.update (true);
+	}
 	return 0;
 }
 
@@ -208,8 +211,8 @@ void createHandle () {
 	handle = outControl [0];
 }
 
-void drawBackground (int control) {
-	drawBackground (control, background);
+void drawBackground (int control, int context) {
+	fillBackground (control, context, null);
 }
 
 /**
@@ -342,17 +345,28 @@ int kEventMouseDown (int nextHandler, int theEvent, int userData) {
 		OS.GetGlobalMouse (outPt);
 		Rect rect = new Rect ();
 		int window = OS.GetControlOwner (handle);
-		OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
-		int x = outPt.h - rect.left;
-		int y = outPt.v - rect.top;
-		OS.GetControlBounds (parent.handle, rect);
+		int x, y;
+		if (OS.HIVIEW) {
+			CGPoint pt = new CGPoint ();
+			pt.x = outPt.h;
+			pt.y = outPt.v;
+			OS.HIViewConvertPoint (pt, 0, handle);
+			x = (int) pt.x;
+			y = (int) pt.y;
+			OS.GetWindowBounds (window, (short) OS.kWindowStructureRgn, rect);
+		} else {
+			OS.GetControlBounds (handle, rect);
+			x = outPt.h - rect.left;
+			y = outPt.v - rect.top;
+			OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
+		}
 		x -= rect.left;
 		y -=  rect.top;
 		short [] button = new short [1];
 		OS.GetEventParameter (theEvent, OS.kEventParamMouseButton, OS.typeMouseButton, null, 2, null, button);
 		int chord = OS.GetCurrentEventButtonState ();
 		int modifiers = OS.GetCurrentEventKeyModifiers ();
-		sendMouseEvent (SWT.MouseUp, button [0], chord, (short)x, (short)y, modifiers, true);
+		sendMouseEvent (SWT.MouseUp, button [0], true, chord, (short)x, (short)y, modifiers);
 	}
 	tracking = false;
 	return status;
@@ -520,7 +534,7 @@ public void setThumb (int value) {
  * Sets the receiver's selection, minimum value, maximum
  * value, thumb, increment and page increment all at once.
  * <p>
- * Note: This is equivalent to setting the values individually
+ * Note: This is similar to setting the values individually
  * using the appropriate methods, but may be implemented in a 
  * more efficient fashion on some platforms.
  * </p>
