@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,6 +13,7 @@ package org.eclipse.swt.widgets;
  
 import org.eclipse.swt.internal.carbon.OS;
 import org.eclipse.swt.internal.carbon.Rect;
+import org.eclipse.swt.internal.carbon.CGRect;
 
 import org.eclipse.swt.*;
 import org.eclipse.swt.events.*;
@@ -33,11 +34,13 @@ import org.eclipse.swt.graphics.*;
  * </p><p>
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>(none)</dd>
+ * <dd>TOP, BOTTOM</dd>
  * <dt><b>Events:</b></dt>
  * <dd>Selection</dd>
  * </dl>
  * <p>
+ * Note: Only one of the styles TOP and BOTTOM may be specified.
+ * </p><p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
  * </p>
  */
@@ -110,6 +113,7 @@ public void addSelectionListener(SelectionListener listener) {
 }
 
 static int checkStyle (int style) {
+	style = checkBits (style, SWT.TOP, SWT.BOTTOM, 0, 0, 0, 0);
 	/*
 	* Even though it is legal to create this widget
 	* with scroll bars, they serve no useful purpose
@@ -125,37 +129,68 @@ protected void checkSubclass () {
 }
 
 public Point computeSize (int wHint, int hHint, boolean changed) {
-	checkWidget ();
-	// NEEDS WORK
-	int count = OS.GetControl32BitMaximum (handle);
-	int width = 100 * count, height = 0;
-	Point size;
-	if (layout != null) {
-		size = layout.computeSize (this, wHint, hHint, changed);
-	} else {
-		size = minimumSize (wHint, hHint, changed);
+	Point size = super.computeSize (wHint, hHint, changed);
+	if (wHint == SWT.DEFAULT && items.length > 0) {
+		int width = 0;
+		GC gc = new GC (this);
+		for (int i = 0; i < items.length; i++) {
+			if (items [i] != null) {
+				width += items [i].calculateWidth (gc);
+			}
+		}
+		gc.dispose ();
+		Rectangle trim = computeTrim (0, 0, width, 0);
+		size.x = Math.max (trim.width, size.x);
 	}
-	if (size.x == 0) size.x = DEFAULT_WIDTH;
-	if (size.y == 0) size.y = DEFAULT_HEIGHT;
-	if (wHint != SWT.DEFAULT) size.x = wHint;
-	if (hHint != SWT.DEFAULT) size.y = hHint;
-	width = Math.max (width, size.x);
-	height = Math.max (height, size.y);
-	Rectangle trim = computeTrim (0, 0, width, height);
-	width = trim.width;  height = trim.height;
-	return new Point (width, height);
+	return size;
 }
 
 public Rectangle computeTrim (int x, int y, int width, int height) {
 	checkWidget ();
-	Rect bounds = new Rect ();
-	OS.GetControlBounds (handle, bounds);
+	if (OS.HIVIEW) {
+		CGRect oldBounds = new CGRect (), bounds = oldBounds;
+		OS.HIViewGetFrame (handle, oldBounds);
+		int MIN_SIZE = 100;
+		if (oldBounds.width < MIN_SIZE || oldBounds.height < MIN_SIZE) {
+			OS.HIViewSetDrawingEnabled (handle, false);
+			bounds = new CGRect ();
+			bounds.width = bounds.height = 100;
+			OS.HIViewSetFrame (handle, bounds);
+		}
+		Rect client = new Rect ();
+		OS.GetTabContentRect (handle, client);
+		if (oldBounds.width < MIN_SIZE || oldBounds.height < MIN_SIZE) {
+			OS.HIViewSetFrame (handle, oldBounds);
+			OS.HIViewSetDrawingEnabled (handle, drawCount == 0);
+		}
+		x -= client.left;
+		y -= client.top;
+		width += (int) bounds.width - (client.right - client.left);
+		height += (int) bounds.height - (client.bottom - client.top);
+		Rect inset = getInset ();
+		x -= inset.left;
+		y -= inset.top;
+		width += inset.left + inset.right;
+		height += inset.top + inset.bottom;
+		return new Rectangle (-client.left, -client.top, width, height);
+	}
+	Rect bounds, oldBounds = new Rect ();
+	OS.GetControlBounds (handle, oldBounds);
+	boolean fixBounds = (oldBounds.right - oldBounds.left) < 100 || (oldBounds.bottom - oldBounds.top) < 100;
+	if (fixBounds) {
+		bounds = new Rect ();
+		bounds.right = bounds.bottom = 100;
+		OS.SetControlBounds (handle, bounds);
+	} else {
+		bounds = oldBounds;
+	}
 	Rect client = new Rect ();
-	OS.GetControlData (handle, (short)OS.kControlEntireControl, OS.kControlTabContentRectTag, Rect.sizeof, client, null);
-	x -= Math.max (3, client.left - bounds.left);
-	y -= Math.max (34, client.top - bounds.top);
-	width += Math.max (6, (bounds.right - bounds.left) - (client.right - client.left));
-	height += Math.max (37, (bounds.bottom - bounds.top) - (client.bottom - client.top));
+	OS.GetTabContentRect (handle, client);
+	if (fixBounds) OS.SetControlBounds (handle, oldBounds);
+	x -= client.left - bounds.left;
+	y -= client.top - bounds.top;
+	width += (bounds.right - bounds.left) - (client.right - client.left);
+	height += (bounds.bottom - bounds.top) - (client.bottom - client.top);
 	Rect inset = getInset ();
 	x -= inset.left;
 	y -= inset.top;
@@ -167,7 +202,8 @@ public Rectangle computeTrim (int x, int y, int width, int height) {
 void createHandle () {
 	int [] outControl = new int [1];
 	int window = OS.GetControlOwner (parent.handle);
-	OS.CreateTabsControl (window, new Rect (), (short)OS.kControlTabSizeLarge, (short)OS.kControlTabDirectionNorth, (short) 0, 0, outControl);
+	short direction = (style & SWT.BOTTOM) != 0 ? (short)OS.kControlTabDirectionSouth : (short)OS.kControlTabDirectionNorth;
+	OS.CreateTabsControl (window, new Rect (), (short)OS.kControlTabSizeLarge, direction, (short) 0, 0, outControl);
 	if (outControl [0] == 0) error (SWT.ERROR_NO_HANDLES);
 	handle = outControl [0];
 }
@@ -211,34 +247,44 @@ void destroyItem (TabItem item) {
 		if (items [index] == item) break;
 		index++;
 	}
-	if (index == count) return;	// not found
+	if (index == count) return;
+	redrawWidget (handle, false);
 	int selectionIndex = OS.GetControl32BitValue (handle) - 1;
 	--count;
 	OS.SetControl32BitMaximum (handle, count);
-	if (count == 0) {
-		items = new TabItem [4];
-		return;
-	}
 	System.arraycopy (items, index + 1, items, index, count - index);
 	items [count] = null;
-	if (index == selectionIndex) {
-		setSelection (Math.max (0, selectionIndex - 1));
-		selectionIndex = getSelectionIndex ();
-		if (selectionIndex != -1) {
-			Event event = new Event ();
-			event.item = items [selectionIndex];
-			sendEvent (SWT.Selection, event);
-			// the widget could be destroyed at this point
-		}
+	if (count == 0) {
+		items = new TabItem [4];
 	}
+	for (int i = index; i < count; i++) {
+		items [i].update ();
+	}
+	if (count > 0 && index == selectionIndex) {
+		setSelection (Math.max (0, selectionIndex - 1), true);
+	}
+	invalidateVisibleRegion (handle);
 }
 
 public Rectangle getClientArea () {
 	checkWidget ();
+	if (OS.HIVIEW) {
+		Rect client = new Rect ();
+		if (OS.GetControlData (handle, (short)OS.kControlEntireControl, OS.kControlTabContentRectTag, Rect.sizeof, client, null) != OS.noErr) {
+			return new Rectangle(0, 0, 0, 0);
+		}
+		int x = Math.max (0, client.left);
+		int y = Math.max (0, client.top);
+		int width = Math.max (0, client.right - client.left);
+		int height = Math.max (0, client.bottom - client.top);
+		return new Rectangle (x, y, width, height);
+	}
 	Rect bounds = new Rect ();
 	OS.GetControlBounds (handle, bounds);
 	Rect client = new Rect ();
-	OS.GetControlData (handle, (short)OS.kControlEntireControl, OS.kControlTabContentRectTag, Rect.sizeof, client, null);
+	if (OS.GetControlData (handle, (short)OS.kControlEntireControl, OS.kControlTabContentRectTag, Rect.sizeof, client, null) != OS.noErr) {
+		return new Rectangle(0, 0, 0, 0);
+	}
 	int x = Math.max (0, client.left - bounds.left);
 	int y = Math.max (0, client.top - bounds.top);
 	int width = Math.max (0, client.right - client.left);
@@ -346,6 +392,10 @@ public int getSelectionIndex () {
 	return OS.GetControl32BitValue (handle) - 1;
 }
 
+float getThemeAlpha () {
+	return 0.25f * parent.getThemeAlpha ();
+}
+
 /**
  * Searches the receiver's list starting at the first item
  * (index 0) until an item is found that is equal to the 
@@ -398,7 +448,8 @@ Point minimumSize (int wHint, int hHint, boolean flushCache) {
 }
 
 Rect getInset () {
-	return display.tabFolderInset;
+	if (OS.VERSION >= 0x1020) return super.getInset();
+	return (style & SWT.BOTTOM) != 0 ? display.tabFolderSouthInset : display.tabFolderNorthInset;
 }
 
 int kEventControlApplyBackground (int nextHandler, int theEvent, int userData) {
@@ -439,14 +490,35 @@ int kEventControlHit (int nextHandler, int theEvent, int userData) {
 	return OS.noErr;
 }
 
-void releaseWidget () {
+int kEventControlSetFocusPart (int nextHandler, int theEvent, int userData) {
+	short [] part = new short [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamControlPart, OS.typeControlPartCode, null, 2, null, part);
+	if (part [0] == OS.kControlFocusNoPart || part [0] == OS.kControlFocusNextPart) {
+		return super.kEventControlSetFocusPart (nextHandler, theEvent, userData);
+	}
+	return OS.eventNotHandledErr;
+}
+
+void releaseChildren (boolean destroy) {
+	if (items != null) {
+		for (int i=0; i<items.length; i++) {
+			TabItem item = items [i];
+			if (item != null && !item.isDisposed ()) {
+				item.release (false);
+			}
+		}
+		items = null;
+	}
+	super.releaseChildren (destroy);
+}
+
+void removeControl (Control control) {
+	super.removeControl (control);
 	int count = OS.GetControl32BitMaximum (handle);
 	for (int i=0; i<count; i++) {
 		TabItem item = items [i];
-		if (!item.isDisposed ()) item.releaseResources ();
+		if (item.control == control) item.setControl (null);
 	}
-	items = null;
-	super.releaseWidget ();
 }
 
 /**
@@ -474,8 +546,8 @@ public void removeSelectionListener (SelectionListener listener) {
 	eventTable.unhook (SWT.DefaultSelection,listener);	
 }
 
-int setBounds (int c, int x, int y, int width, int height, boolean move, boolean resize, boolean events) {
-	int result = super.setBounds(c, x, y, width, height, move, resize, events);
+int setBounds (int x, int y, int width, int height, boolean move, boolean resize, boolean events) {
+	int result = super.setBounds(x, y, width, height, move, resize, events);
 	if ((result & RESIZED) != 0) {
 		int index = OS.GetControl32BitValue (handle) - 1;
 		if (index != -1) {
@@ -489,6 +561,29 @@ int setBounds (int c, int x, int y, int width, int height, boolean move, boolean
 	return result;
 }
 
+/**
+ * Sets the receiver's selection to the given item.
+ * The current selected is first cleared, then the new item is
+ * selected.
+ *
+ * @param item the item to select
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the item is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.2
+ */
+public void setSelection (TabItem item) {
+	checkWidget ();
+	if (item == null) error (SWT.ERROR_NULL_ARGUMENT);
+	setSelection (new TabItem [] {item});
+}
+
 /**
  * Sets the receiver's selection to be the given array of items.
  * The current selected is first cleared, then the new items are
@@ -496,6 +591,9 @@ int setBounds (int c, int x, int y, int width, int height, boolean move, boolean
  *
  * @param items the array of items
  *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the items array is null</li>
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -505,19 +603,19 @@ public void setSelection (TabItem [] items) {
 	checkWidget ();
 	if (items == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (items.length == 0) {
-		setSelection (-1);
-		return;
-	}
-	for (int i=items.length - 1; i>=0; --i) {
-		int index = indexOf (items [i]);
-		if (index != -1) setSelection (index);
+		setSelection (-1, false);
+	} else {
+		for (int i=items.length - 1; i>=0; --i) {
+			int index = indexOf (items [i]);
+			if (index != -1) setSelection (index, false);
+		}
 	}
 }
 
 /**
  * Selects the item at the given zero-relative index in the receiver. 
  * If the item at the index was already selected, it remains selected.
- * The current selected is first cleared, then the new items are
+ * The current selection is first cleared, then the new items are
  * selected. Indices that are out of range are ignored.
  *
  * @param index the index of the item to select
@@ -529,6 +627,8 @@ public void setSelection (TabItem [] items) {
  */
 public void setSelection (int index) {
 	checkWidget ();
+	int count = OS.GetControl32BitMaximum (handle);
+	if (!(0 <= index && index < count)) return;
 	setSelection (index, false);
 }
 
