@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -40,11 +40,13 @@ import org.eclipse.swt.internal.carbon.Rect;
  */
 public class Tracker extends Widget {
 	Control parent;
-	boolean tracking, stippled;
+	boolean tracking, cancelled, stippled;
 	Cursor clientCursor, resizeCursor;
 	Rectangle [] rectangles, proportions;
+	Rectangle bounds;
 	int cursorOrientation = SWT.NONE;
 	boolean inEvent = false;
+	int window, oldX, oldY;
 		
 	/*
 	* The following values mirror step sizes on Windows
@@ -156,35 +158,57 @@ public void addControlListener (ControlListener listener) {
 	checkWidget ();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	TypedListener typedListener = new TypedListener (listener);
-	addListener (SWT.Move,typedListener);
+	addListener (SWT.Resize, typedListener);
+	addListener (SWT.Move, typedListener);
+}
+
+/**
+ * Adds the listener to the collection of listeners who will
+ * be notified when keys are pressed and released on the system keyboard, by sending
+ * it one of the messages defined in the <code>KeyListener</code>
+ * interface.
+ *
+ * @param listener the listener which should be notified
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see KeyListener
+ * @see #removeKeyListener
+ */
+public void addKeyListener(KeyListener listener) {
+	checkWidget();
+	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
+	TypedListener typedListener = new TypedListener (listener);
+	addListener(SWT.KeyUp,typedListener);
+	addListener(SWT.KeyDown,typedListener);
 }
 
 Point adjustMoveCursor () {
-	Rectangle bounds = computeBounds ();
 	int newX = bounds.x + bounds.width / 2;
 	int newY = bounds.y;
 	/*
-	 * Convert to screen coordinates iff needed
+	 * Convert to screen coordinates if needed
  	 */
 	if (parent != null) {
-		Rect rect = new Rect ();
-		OS.GetControlBounds (parent.handle, rect);
-		newX += rect.left; 
-		newY += rect.top; 
-		int window = OS.GetControlOwner (parent.handle);
-		OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
-		newX += rect.left; 
-		newY += rect.top; 
+		Point pt = parent.toDisplay (newX, newY);
+		newX = pt.x;
+		newY = pt.y;
 	}
 	CGPoint pt = new CGPoint ();
-	pt.x = newX;  pt.y = newY;
+	pt.x = newX;
+	pt.y = newY;
 	OS.CGWarpMouseCursorPosition (pt);
 	return new Point ((int) pt.x, (int) pt.y);
 }
 
-Point adjustResizeCursor () {
+Point adjustResizeCursor (boolean movePointer) {
 	int newX, newY;
-	Rectangle bounds = computeBounds ();
 
 	if ((cursorOrientation & SWT.LEFT) != 0) {
 		newX = bounds.x;
@@ -203,21 +227,19 @@ Point adjustResizeCursor () {
 	}
 
 	/*
-	 * Convert to screen coordinates iff needed
+	 * Convert to screen coordinates if needed
  	 */
 	if (parent != null) {
-		Rect rect = new Rect ();
-		OS.GetControlBounds (parent.handle, rect);
-		newX += rect.left; 
-		newY += rect.top; 
-		int window = OS.GetControlOwner (parent.handle);
-		OS.GetWindowBounds (window, (short) OS.kWindowContentRgn, rect);
-		newX += rect.left; 
-		newY += rect.top; 
+		Point pt = parent.toDisplay (newX, newY);
+		newX = pt.x;
+		newY = pt.y; 
+	}
+	if (movePointer) {
+		CGPoint pt = new CGPoint ();
+		pt.x = newX;
+		pt.y = newY;
+		OS.CGWarpMouseCursorPosition (pt);
 	}
-	CGPoint pt = new CGPoint ();
-	pt.x = newX;  pt.y = newY;
-	OS.CGWarpMouseCursorPosition (pt);
 
 	/*
 	* If the client has not provided a custom cursor then determine
@@ -261,7 +283,7 @@ Point adjustResizeCursor () {
 		resizeCursor = newCursor;
 	}
 		
-	return new Point ((int) pt.x, (int) pt.y);
+	return new Point (newX, newY);
 }
 
 static int checkStyle (int style) {
@@ -304,31 +326,27 @@ Rectangle computeBounds () {
 
 Rectangle [] computeProportions (Rectangle [] rects) {
 	Rectangle [] result = new Rectangle [rects.length];
-	Rectangle bounds = computeBounds ();
+	bounds = computeBounds ();
 	for (int i = 0; i < rects.length; i++) {
 		int x = 0, y = 0, width = 0, height = 0;
 		if (bounds.width != 0) {
 			x = (rects [i].x - bounds.x) * 100 / bounds.width;
 			width = rects [i].width * 100 / bounds.width;
+		} else {
+			width = 100;
 		}
 		if (bounds.height != 0) {
 			y = (rects [i].y - bounds.y) * 100 / bounds.height;
 			height = rects [i].height * 100 / bounds.height;
+		} else {
+			height = 100;
 		}
 		result [i] = new Rectangle (x, y, width, height);			
 	}
 	return result;
 }
 
-
-void drawRectangles (int window, boolean erase) {
-	if (parent != null) {
-		if (parent.isDisposed ()) return;
-		Shell shell = parent.getShell ();
-		shell.update (true);
-	} else {
-		display.update ();
-	}
+void drawRectangles (int window, Rectangle [] rects, boolean erase) {
 	int[] context = new int [1];
 	int port = OS.GetWindowPort (window);
 	Rect portRect = new Rect ();
@@ -337,10 +355,16 @@ void drawRectangles (int window, boolean erase) {
 	OS.CGContextScaleCTM (context [0], 1, -1);
 	OS.CGContextTranslateCTM (context [0], 0, portRect.top - portRect.bottom);
 	CGRect cgRect = new CGRect ();
-	for (int i=0; i<rectangles.length; i++) {
-		Rectangle rect = rectangles [i];
-		cgRect.x = rect.x;
-		cgRect.y = rect.y;
+	Point parentOrigin;
+	if (parent != null) {
+		parentOrigin = display.map (parent, null, 0, 0);
+	} else {
+		parentOrigin = new Point (0, 0);	
+	}
+	for (int i=0; i<rects.length; i++) {
+		Rectangle rect = rects [i];
+		cgRect.x = rect.x + parentOrigin.x;
+		cgRect.y = rect.y + parentOrigin.y;
 		cgRect.width = rect.width;
 		cgRect.height = rect.height;
 		if (erase) {
@@ -370,8 +394,15 @@ void drawRectangles (int window, boolean erase) {
  * </ul>
  */
 public Rectangle [] getRectangles () {
-	checkWidget ();
-	return rectangles;
+	checkWidget();
+	int length = 0;
+	if (rectangles != null) length = rectangles.length;
+	Rectangle [] result = new Rectangle [length];
+	for (int i = 0; i < length; i++) {
+		Rectangle current = rectangles [i];
+		result [i] = new Rectangle (current.x, current.y, current.width, current.height);
+	}
+	return result;
 }
 
 /**
@@ -389,11 +420,267 @@ public boolean getStippled () {
 	return stippled;
 }
 
+int kEventMouse (int eventKind, int nextHandler, int theEvent, int userData) {
+	int sizeof = org.eclipse.swt.internal.carbon.Point.sizeof;
+	org.eclipse.swt.internal.carbon.Point where = new org.eclipse.swt.internal.carbon.Point ();
+	OS.GetEventParameter (theEvent, OS.kEventParamMouseLocation, OS.typeQDPoint, null, sizeof, null, where);
+	int newX = where.h, newY = where.v;	
+	if (newX != oldX || newY != oldY) {
+		Rectangle [] oldRectangles = rectangles;
+		Rectangle [] rectsToErase = new Rectangle [rectangles.length];
+		for (int i = 0; i < rectangles.length; i++) {
+			Rectangle current = rectangles [i];
+			rectsToErase [i] = new Rectangle (current.x, current.y, current.width, current.height);
+		}
+		Event event = new Event ();
+		event.x = newX;
+		event.y = newY;
+		if ((style & SWT.RESIZE) != 0) {
+			boolean orientationInit = resizeRectangles (newX - oldX, newY - oldY);
+			inEvent = true;
+			sendEvent (SWT.Resize, event);
+			inEvent = false;
+			/*
+			* It is possible (but unlikely), that application
+			* code could have disposed the widget in the move
+			* event.  If this happens, return false to indicate
+			* that the tracking has failed.
+			*/
+			if (isDisposed ()) {
+				cancelled = true;
+				return OS.noErr;
+			}
+			boolean draw = false;
+			/*
+			 * It is possible that application code could have
+			 * changed the rectangles in the resize event.  If this
+			 * happens then only redraw the tracker if the rectangle
+			 * values have changed.
+			 */
+			if (rectangles != oldRectangles) {
+				int length = rectangles.length;
+				if (length != rectsToErase.length) {
+					draw = true;
+				} else {
+					for (int i = 0; i < length; i++) {
+						if (!rectangles [i].equals (rectsToErase [i])) {
+							draw = true;
+							break;
+						}
+					}
+				}
+			}
+			else {
+				draw = true;
+			}
+			if (draw) {
+				drawRectangles (window, rectsToErase, true);
+				drawRectangles (window, rectangles, false);
+			}
+			Point cursorPos = adjustResizeCursor (orientationInit);
+			newX = cursorPos.x;
+			newY = cursorPos.y;
+		} else {
+			moveRectangles (newX - oldX, newY - oldY);
+			inEvent = true;
+			sendEvent (SWT.Move, event);
+			inEvent = false;
+			/*
+			* It is possible (but unlikely), that application
+			* code could have disposed the widget in the move
+			* event.  If this happens, return false to indicate
+			* that the tracking has failed.
+			*/
+			if (isDisposed ()) {
+				cancelled = true;
+				return OS.noErr;
+			}
+			boolean draw = false;
+			/*
+			 * It is possible that application code could have
+			 * changed the rectangles in the move event.  If this
+			 * happens then only redraw the tracker if the rectangle
+			 * values have changed.
+			 */
+			if (rectangles != oldRectangles) {
+				int length = rectangles.length;
+				if (length != rectsToErase.length) {
+					draw = true;
+				} else {
+					for (int i = 0; i < length; i++) {
+						if (!rectangles [i].equals (rectsToErase [i])) {
+							draw = true;
+							break;
+						}
+					}
+				}
+			} else {
+				draw = true;
+			}
+			if (draw) {
+				drawRectangles (window, rectsToErase, true);
+				drawRectangles (window, rectangles, false);
+			}
+		}
+		oldX = newX;  oldY = newY;
+	}
+	tracking = eventKind != OS.kEventMouseUp;
+	return 0;
+}
+
+int kEventMouseDragged (int nextHandler, int theEvent, int userData) {
+	return kEventMouse (OS.kEventMouseDragged, nextHandler, theEvent, userData);
+}
+
+int kEventRawKeyPressed (int nextHandler, int theEvent, int userData) {
+	if (!sendKeyEvent (SWT.KeyDown, theEvent)) return OS.noErr;
+	int [] keyCode = new int [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamKeyCode, OS.typeUInt32, null, keyCode.length * 4, null, keyCode);
+	int [] modifiers = new int [1];
+	OS.GetEventParameter (theEvent, OS.kEventParamKeyModifiers, OS.typeUInt32, null, 4, null, modifiers);
+	int stepSize = (modifiers [0] & OS.controlKey) != 0 ? STEPSIZE_SMALL : STEPSIZE_LARGE;
+	int xChange = 0, yChange = 0;
+	switch (keyCode [0]) {
+		case 53: /* Esc */
+			cancelled = true;
+			tracking = false;
+			break;
+		case 76: /* KP Enter */
+		case 36: /* Return */
+			tracking = false;
+			break;
+		case 123: /* Left arrow */
+			xChange = -stepSize;
+			break;
+		case 124: /* Right arrow */
+			xChange = stepSize;
+			break;
+		case 126: /* Up arrow */
+			yChange = -stepSize;
+			break;
+		case 125: /* Down arrow */
+			yChange = stepSize;
+			break;
+	}
+	if (xChange != 0 || yChange != 0) {
+		Rectangle [] oldRectangles = rectangles;
+		Rectangle [] rectsToErase = new Rectangle [rectangles.length];
+		for (int i = 0; i < rectangles.length; i++) {
+			Rectangle current = rectangles [i];
+			rectsToErase [i] = new Rectangle (current.x, current.y, current.width, current.height);
+		}
+		Event event = new Event ();
+		int newX = oldX + xChange;
+		int newY = oldY + yChange;
+		event.x = newX;
+		event.y = newY;
+		Point cursorPos;
+		if ((style & SWT.RESIZE) != 0) {
+			resizeRectangles (xChange, yChange);
+			inEvent = true;
+			sendEvent (SWT.Resize, event);
+			inEvent = false;
+			/*
+			* It is possible (but unlikely) that application
+			* code could have disposed the widget in the move
+			* event.  If this happens return false to indicate
+			* that the tracking has failed.
+			*/
+			if (isDisposed ()) {
+				cancelled = true;
+				return OS.noErr;
+			}
+			boolean draw = false;
+			/*
+			 * It is possible that application code could have
+			 * changed the rectangles in the resize event.  If this
+			 * happens then only redraw the tracker if the rectangle
+			 * values have changed.
+			 */
+			if (rectangles != oldRectangles) {
+				int length = rectangles.length;
+				if (length != rectsToErase.length) {
+					draw = true;
+				} else {
+					for (int i = 0; i < length; i++) {
+						if (!rectangles [i].equals (rectsToErase [i])) {
+							draw = true;
+							break;
+						}
+					}
+				}
+			} else {
+				draw = true;
+			}
+			if (draw) {
+				drawRectangles (window, rectsToErase, true);
+				drawRectangles (window, rectangles, false);
+			}
+			cursorPos = adjustResizeCursor (true);
+		} else {
+			moveRectangles (xChange, yChange);
+			inEvent = true;
+			sendEvent (SWT.Move, event);
+			inEvent = false;
+			/*
+			* It is possible (but unlikely) that application
+			* code could have disposed the widget in the move
+			* event.  If this happens return false to indicate
+			* that the tracking has failed.
+			*/
+			if (isDisposed ()) {
+				cancelled = true;
+				return OS.noErr;
+			}
+			boolean draw = false;
+			/*
+			 * It is possible that application code could have
+			 * changed the rectangles in the move event.  If this
+			 * happens then only redraw the tracker if the rectangle
+			 * values have changed.
+			 */
+			if (rectangles != oldRectangles) {
+				int length = rectangles.length;
+				if (length != rectsToErase.length) {
+					draw = true;
+				} else {
+					for (int i = 0; i < length; i++) {
+						if (!rectangles [i].equals (rectsToErase [i])) {
+							draw = true;
+							break;
+						}
+					}
+				}
+			} else {
+				draw = true;
+			}
+			if (draw) {
+				drawRectangles (window, rectsToErase, true);
+				drawRectangles (window, rectangles, false);
+			}
+			cursorPos = adjustMoveCursor ();
+		}
+		oldX = cursorPos.x;
+		oldY = cursorPos.y;
+	}
+	return 0;
+}
+
+int kEventMouseMoved (int nextHandler, int theEvent, int userData) {
+	return kEventMouse (OS.kEventMouseMoved, nextHandler, theEvent, userData);
+}
+
+int kEventMouseUp (int nextHandler, int theEvent, int userData) {
+	return kEventMouse (OS.kEventMouseUp, nextHandler, theEvent, userData);
+}
+
 void moveRectangles (int xChange, int yChange) {
-	if (xChange < 0 && ((style & SWT.LEFT) == 0)) return;
-	if (xChange > 0 && ((style & SWT.RIGHT) == 0)) return;
-	if (yChange < 0 && ((style & SWT.UP) == 0)) return;
-	if (yChange > 0 && ((style & SWT.DOWN) == 0)) return;
+	if (xChange < 0 && ((style & SWT.LEFT) == 0)) xChange = 0;
+	if (xChange > 0 && ((style & SWT.RIGHT) == 0)) xChange = 0;
+	if (yChange < 0 && ((style & SWT.UP) == 0)) yChange = 0;
+	if (yChange > 0 && ((style & SWT.DOWN) == 0)) yChange = 0;
+	if (xChange == 0 && yChange == 0) return;
+	bounds.x += xChange; bounds.y += yChange;
 	for (int i = 0; i < rectangles.length; i++) {
 		rectangles [i].x += xChange;
 		rectangles [i].y += yChange;
@@ -415,11 +702,25 @@ void moveRectangles (int xChange, int yChange) {
 public boolean open () {
 	checkWidget ();
 	if (rectangles == null) return false;
-	boolean cancelled = false;
+	cancelled = false;
 	tracking = true;
-	int window = display.createOverlayWindow ();
+	window = display.createOverlayWindow ();
 	OS.ShowWindow (window);
-	drawRectangles (window, false);
+	drawRectangles (window, rectangles, false);
+	
+	/*
+	* If exactly one of UP/DOWN is specified as a style then set the cursor
+	* orientation accordingly (the same is done for LEFT/RIGHT styles below).
+	*/
+	int vStyle = style & (SWT.UP | SWT.DOWN);
+	if (vStyle == SWT.UP || vStyle == SWT.DOWN) {
+		cursorOrientation |= vStyle;
+	}
+	int hStyle = style & (SWT.LEFT | SWT.RIGHT);
+	if (hStyle == SWT.LEFT || hStyle == SWT.RIGHT) {
+		cursorOrientation |= hStyle;
+	}
+	
 	Point cursorPos;
 	if (OS.StillDown ()) {
 		org.eclipse.swt.internal.carbon.Point pt = new org.eclipse.swt.internal.carbon.Point ();
@@ -427,130 +728,39 @@ public boolean open () {
 		cursorPos = new Point (pt.h, pt.v);
 	} else {
 		if ((style & SWT.RESIZE) != 0) {
-			cursorPos = adjustResizeCursor ();
+			cursorPos = adjustResizeCursor (true);
 		} else {
 			cursorPos = adjustMoveCursor ();
 		}
 	}
-	
-	int oldX = cursorPos.x, oldY = cursorPos.y;
-	/*
-	* Tracker behaves like a Dialog with its own OS event loop.
-	*/
-	Event event = new Event ();
+	oldX = cursorPos.x;
+	oldY = cursorPos.y;
+
+	/* Tracker behaves like a Dialog with its own OS event loop. */
 	int [] outEvent  = new int [1];
 	while (tracking && !cancelled) {
 		int status = OS.ReceiveNextEvent (0, null, OS.kEventDurationNoWait, true, outEvent);
 		if (status != OS.noErr) continue;
-		int eventClass = OS.GetEventClass (outEvent [0]);
-		int eventKind = OS.GetEventKind (outEvent [0]);
-		int newX = oldX, newY = oldY;	
+		int event = outEvent [0];
+		int eventClass = OS.GetEventClass (event);
+		int eventKind = OS.GetEventKind (event);
+		int nextHandler = 0;
 		switch (eventClass) {
-			case OS.kEventClassMouse: {
+			case OS.kEventClassMouse:
 				switch (eventKind) {
-					case OS.kEventMouseUp:
-					case OS.kEventMouseMoved:
-					case OS.kEventMouseDragged:
-						int sizeof = org.eclipse.swt.internal.carbon.Point.sizeof;
-						org.eclipse.swt.internal.carbon.Point where = new org.eclipse.swt.internal.carbon.Point ();
-						OS.GetEventParameter (outEvent [0], OS.kEventParamMouseLocation, OS.typeQDPoint, null, sizeof, null, where);
-						newX = where.h;
-						newY = where.v;	
-						if (newX != oldX || newY != oldY) {
-							drawRectangles (window, true);
-							event.x = newX;
-							event.y = newY;
-							if ((style & SWT.RESIZE) != 0) {
-							   resizeRectangles (newX - oldX, newY - oldY);
-								cursorPos = adjustResizeCursor ();
-								newX = cursorPos.x; newY = cursorPos.y;
-								inEvent = true;
-								sendEvent (SWT.Resize, event);
-							} else {
-								moveRectangles (newX - oldX, newY - oldY);
-								inEvent = true;
-								sendEvent (SWT.Move, event);
-							}
-							inEvent = false;
-							/*
-							* It is possible (but unlikely), that application
-							* code could have disposed the widget in the move
-							* event.  If this happens, return false to indicate
-							* that the tracking has failed.
-							*/
-							if (isDisposed ()) return false;
-							drawRectangles (window, false);
-							oldX = newX;  oldY = newY;
-						}
-						tracking = eventKind != OS.kEventMouseUp;
-						break;
+					case OS.kEventMouseUp: kEventMouseUp (nextHandler, event, 0); break;
+					case OS.kEventMouseMoved: kEventMouseMoved (nextHandler, event, 0); break;
+					case OS.kEventMouseDragged: kEventMouseDragged (nextHandler, event, 0); break;
 				}
 				break;
-			}
-			case OS.kEventClassKeyboard: {
+			case OS.kEventClassKeyboard:
 				switch (eventKind) {
-					case OS.kEventRawKeyDown:
-					case OS.kEventRawKeyModifiersChanged:
-					case OS.kEventRawKeyRepeat: {
-						int [] keyCode = new int [1];
-						OS.GetEventParameter (outEvent [0], OS.kEventParamKeyCode, OS.typeUInt32, null, keyCode.length * 4, null, keyCode);
-						int [] modifiers = new int [1];
-						OS.GetEventParameter (outEvent [0], OS.kEventParamKeyModifiers, OS.typeUInt32, null, 4, null, modifiers);
-						int stepSize = (modifiers [0] & OS.controlKey) != 0 ? STEPSIZE_SMALL : STEPSIZE_LARGE;
-						int xChange = 0, yChange = 0;
-						switch (keyCode [0]) {
-							case 53: /* Esc */
-								cancelled = true;
-								tracking = false;
-								break;
-							case 36: /* Return */
-								tracking = false;
-								break;
-							case 123: /* Left arrow */
-								xChange = -stepSize;
-								break;
-							case 124: /* Right arrow */
-								xChange = stepSize;
-								break;
-							case 126: /* Up arrow */
-								yChange = -stepSize;
-								break;
-							case 125: /* Down arrow */
-								yChange = stepSize;
-								break;
-						}
-						if (xChange != 0 || yChange != 0) {
-							drawRectangles (window, true);
-							newX = oldX + xChange;
-							newY = oldY + yChange;
-							event.x = newX;
-							event.y = newY;
-							if ((style & SWT.RESIZE) != 0) {
-								resizeRectangles (xChange, yChange);
-								cursorPos = adjustResizeCursor ();
-								inEvent = true;
-								sendEvent (SWT.Resize, event);
-							} else {
-								moveRectangles (xChange, yChange);
-								cursorPos = adjustMoveCursor ();
-								inEvent = true;
-								sendEvent (SWT.Move, event);
-							}
-							inEvent = false;
-							/*
-							* It is possible (but unlikely) that application
-							* code could have disposed the widget in the move
-							* event.  If this happens return false to indicate
-							* that the tracking has failed.
-							*/
-							if (isDisposed ()) return false;
-							drawRectangles (window, false);
-							oldX = cursorPos.x;  oldY = cursorPos.y;
-						}
-						break;
-					}
+					case OS.kEventRawKeyDown: kEventRawKeyDown (nextHandler, event, 0); break;
+					case OS.kEventRawKeyModifiersChanged: kEventRawKeyModifiersChanged (nextHandler, event, 0); break;
+					case OS.kEventRawKeyRepeat: kEventRawKeyRepeat (nextHandler, event, 0); break;
+					case OS.kEventRawKeyUp: kEventRawKeyUp (nextHandler, event, 0); break;
 				}
-			}
+				break;
 		}
 		/*
 		* Don't dispatch mouse and key events in general, EXCEPT once this
@@ -561,16 +771,19 @@ public boolean open () {
 			if (eventClass == OS.kEventClassMouse) dispatch = false;
 			if (eventClass == OS.kEventClassKeyboard) dispatch = false;
 		}
-		if (dispatch) OS.SendEventToEventTarget (outEvent [0], OS.GetEventDispatcherTarget ());
-		OS.ReleaseEvent (outEvent [0]);
+		if (dispatch) OS.SendEventToEventTarget (event, OS.GetEventDispatcherTarget ());
+		OS.ReleaseEvent (event);
 		if (clientCursor != null && resizeCursor == null) {
 			display.setCursor (clientCursor.handle);
 		}
 	}
-	drawRectangles (window, true);
+	if (!isDisposed()) {
+		drawRectangles (window, rectangles, true);
+	}
 	OS.DisposeWindow (window);
 	tracking = false;
 	display.grabControl = null;
+	window = 0;
 	return !cancelled;
 }
 
@@ -578,7 +791,7 @@ public boolean open () {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is moved or resized.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -595,24 +808,138 @@ public void removeControlListener (ControlListener listener) {
 	checkWidget ();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (eventTable == null) return;
+	eventTable.unhook (SWT.Resize, listener);
 	eventTable.unhook (SWT.Move, listener);
 }
 
-void resizeRectangles (int xChange, int yChange) {
+/**
+ * Removes the listener from the collection of listeners who will
+ * be notified when keys are pressed and released on the system keyboard.
+ *
+ * @param listener the listener which should no longer be notified
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see KeyListener
+ * @see #addKeyListener
+ */
+public void removeKeyListener(KeyListener listener) {
+	checkWidget();
+	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (eventTable == null) return;
+	eventTable.unhook(SWT.KeyUp, listener);
+	eventTable.unhook(SWT.KeyDown, listener);
+}
+
+/*
+ * Returns true if the pointer's orientation was initialized in some dimension,
+ * and false otherwise.
+ */
+boolean resizeRectangles (int xChange, int yChange) {
+	boolean orientationInit = false;
 	/*
 	* If the cursor orientation has not been set in the orientation of
 	* this change then try to set it here.
 	*/
 	if (xChange < 0 && ((style & SWT.LEFT) != 0) && ((cursorOrientation & SWT.RIGHT) == 0)) {
-		cursorOrientation |= SWT.LEFT;
-	} else if (xChange > 0 && ((style & SWT.RIGHT) != 0) && ((cursorOrientation & SWT.LEFT) == 0)) {
-		cursorOrientation |= SWT.RIGHT;
-	} else if (yChange < 0 && ((style & SWT.UP) != 0) && ((cursorOrientation & SWT.DOWN) == 0)) {
-		cursorOrientation |= SWT.UP;
-	} else if (yChange > 0 && ((style & SWT.DOWN) != 0) && ((cursorOrientation & SWT.UP) == 0)) {
-		cursorOrientation |= SWT.DOWN;
+		if ((cursorOrientation & SWT.LEFT) == 0) {
+			cursorOrientation |= SWT.LEFT;
+			orientationInit = true;
+		}
 	}
-	Rectangle bounds = computeBounds ();
+	if (xChange > 0 && ((style & SWT.RIGHT) != 0) && ((cursorOrientation & SWT.LEFT) == 0)) {
+		if ((cursorOrientation & SWT.RIGHT) == 0) {
+			cursorOrientation |= SWT.RIGHT;
+			orientationInit = true;
+		}
+	} 
+	if (yChange < 0 && ((style & SWT.UP) != 0) && ((cursorOrientation & SWT.DOWN) == 0)) {
+		if ((cursorOrientation & SWT.UP) == 0) {
+			cursorOrientation |= SWT.UP;
+			orientationInit = true;
+		}
+	} 
+	if (yChange > 0 && ((style & SWT.DOWN) != 0) && ((cursorOrientation & SWT.UP) == 0)) {
+		if ((cursorOrientation & SWT.DOWN) == 0) {
+			cursorOrientation |= SWT.DOWN;
+			orientationInit = true;
+		}
+	}
+	
+	/*
+	 * If the bounds will flip about the x or y axis then apply the adjustment
+	 * up to the axis (ie.- where bounds width/height becomes 0), change the
+	 * cursor's orientation accordingly, and flip each Rectangle's origin (only
+	 * necessary for > 1 Rectangles) 
+	 */
+	if ((cursorOrientation & SWT.LEFT) != 0) {
+		if (xChange > bounds.width) {
+			if ((style & SWT.RIGHT) == 0) return orientationInit;
+			cursorOrientation |= SWT.RIGHT;
+			cursorOrientation &= ~SWT.LEFT;
+			bounds.x += bounds.width;
+			xChange -= bounds.width;
+			bounds.width = 0;
+			if (proportions.length > 1) {
+				for (int i = 0; i < proportions.length; i++) {
+					Rectangle proportion = proportions [i];
+					proportion.x = 100 - proportion.x - proportion.width;
+				}
+			}
+		}
+	} else if ((cursorOrientation & SWT.RIGHT) != 0) {
+		if (bounds.width < -xChange) {
+			if ((style & SWT.LEFT) == 0) return orientationInit;
+			cursorOrientation |= SWT.LEFT;
+			cursorOrientation &= ~SWT.RIGHT;
+			xChange += bounds.width;
+			bounds.width = 0;
+			if (proportions.length > 1) {
+				for (int i = 0; i < proportions.length; i++) {
+					Rectangle proportion = proportions [i];
+					proportion.x = 100 - proportion.x - proportion.width;
+				}
+			}
+		}
+	}
+	if ((cursorOrientation & SWT.UP) != 0) {
+		if (yChange > bounds.height) {
+			if ((style & SWT.DOWN) == 0) return orientationInit;
+			cursorOrientation |= SWT.DOWN;
+			cursorOrientation &= ~SWT.UP;
+			bounds.y += bounds.height;
+			yChange -= bounds.height;
+			bounds.height = 0;
+			if (proportions.length > 1) {
+				for (int i = 0; i < proportions.length; i++) {
+					Rectangle proportion = proportions [i];
+					proportion.y = 100 - proportion.y - proportion.height;
+				}
+			}
+		}
+	} else if ((cursorOrientation & SWT.DOWN) != 0) {
+		if (bounds.height < -yChange) {
+			if ((style & SWT.UP) == 0) return orientationInit;
+			cursorOrientation |= SWT.UP;
+			cursorOrientation &= ~SWT.DOWN;
+			yChange += bounds.height;
+			bounds.height = 0;
+			if (proportions.length > 1) {
+				for (int i = 0; i < proportions.length; i++) {
+					Rectangle proportion = proportions [i];
+					proportion.y = 100 - proportion.y - proportion.height;
+				}
+			}
+		}
+	}
+	
+	// apply the bounds adjustment
 	if ((cursorOrientation & SWT.LEFT) != 0) {
 		bounds.x += xChange;
 		bounds.width -= xChange;
@@ -625,10 +952,6 @@ void resizeRectangles (int xChange, int yChange) {
 	} else if ((cursorOrientation & SWT.DOWN) != 0) {
 		bounds.height += yChange;
 	}
-	/*
-	* The following are conditions under which the resize should not be applied
-	*/
-	if (bounds.width < 0 || bounds.height < 0) return;
 	
 	Rectangle [] newRects = new Rectangle [rectangles.length];
 	for (int i = 0; i < rectangles.length; i++) {
@@ -639,7 +962,8 @@ void resizeRectangles (int xChange, int yChange) {
 			proportion.width * bounds.width / 100,
 			proportion.height * bounds.height / 100);
 	}
-	rectangles = newRects;	
+	rectangles = newRects;
+	return orientationInit;
 }
 
 /**
@@ -667,8 +991,8 @@ public void setCursor (Cursor newCursor) {
  *
  * @param rectangles the bounds of the rectangles to be drawn
  *
-  * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the set of rectangles is null</li>
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the set of rectangles is null or contains a null rectangle</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -678,7 +1002,13 @@ public void setCursor (Cursor newCursor) {
 public void setRectangles (Rectangle [] rectangles) {
 	checkWidget ();
 	if (rectangles == null) error (SWT.ERROR_NULL_ARGUMENT);
-	this.rectangles = rectangles;
+	int length = rectangles.length;
+	this.rectangles = new Rectangle [length];
+	for (int i = 0; i < length; i++) {
+		Rectangle current = rectangles [i];
+		if (current == null) error (SWT.ERROR_NULL_ARGUMENT);
+		this.rectangles [i] = new Rectangle (current.x, current.y, current.width, current.height);
+	}
 	proportions = computeProportions (rectangles);
 }
 
@@ -696,4 +1026,5 @@ public void setStippled (boolean stippled) {
 	checkWidget ();
 	this.stippled = stippled;
 }
+
 }
