@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -20,13 +20,13 @@ import org.eclipse.swt.events.*;
 
 /**
  * Instances of this class represent a column in a tree widget.
- *  <dl>
+ * <p><dl>
  * <dt><b>Styles:</b></dt>
  * <dd>LEFT, RIGHT, CENTER</dd>
  * <dt><b>Events:</b></dt>
  * <dd> Move, Resize, Selection</dd>
  * </dl>
- * <p>
+ * </p><p>
  * Note: Only one of the styles LEFT, RIGHT and CENTER may be specified.
  * </p><p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
@@ -36,8 +36,9 @@ import org.eclipse.swt.events.*;
  */
 public class TreeColumn extends Item {
 	Tree parent;
-	int id, lastWidth, iconRef;
+	int id, lastWidth, lastPosition, iconRef;
 	boolean resizable;
+	String toolTipText;
 
 /**
  * Constructs a new instance of this class given its parent
@@ -95,10 +96,11 @@ public TreeColumn (Tree parent, int style) {
  *
  * @param parent a composite control which will be the parent of the new instance (cannot be null)
  * @param style the style of control to construct
- * @param index the index to store the receiver in its parent
+ * @param index the zero-relative index to store the receiver in its parent
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the parent (inclusive)</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -185,6 +187,11 @@ protected void checkSubclass () {
 	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
 }
 
+void destroyWidget () {
+	parent.destroyItem (this);
+	releaseHandle ();
+}
+
 /**
  * Returns a value which describes the position of the
  * text or image in the receiver. The value will be one of
@@ -224,6 +231,33 @@ public Tree getParent () {
 	return parent;
 }
 
+/**
+ * Gets the moveable attribute. A column that is
+ * not moveable cannot be reordered by the user 
+ * by dragging the header but may be reordered 
+ * by the programmer.
+ *
+ * @return the moveable attribute
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @see Tree#getColumnOrder()
+ * @see Tree#setColumnOrder(int[])
+ * @see TreeColumn#setMoveable(boolean)
+ * @see SWT#Move
+ * 
+ * @since 3.2
+ */
+public boolean getMoveable () {
+	checkWidget ();
+	int [] flags = new int [1];
+	OS.GetDataBrowserPropertyFlags (parent.handle, id, flags);
+	return (flags [0] & OS.kDataBrowserListViewMovableColumn) != 0;
+}
+
 /**
  * Gets the resizable attribute. A column that is
  * not resizable cannot be dragged by the user but
@@ -241,6 +275,24 @@ public boolean getResizable () {
 	return resizable;
 }
 
+/**
+ * Returns the receiver's tool tip text, or null if it has
+ * not been set.
+ *
+ * @return the receiver's tool tip text
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.2
+ */
+public String getToolTipText () {
+	checkWidget ();
+	return toolTipText;
+}
+
 /**
  * Gets the width of the receiver.
  *
@@ -255,7 +307,7 @@ public int getWidth () {
 	checkWidget ();
 	short [] width = new short [1];
 	OS.GetDataBrowserTableViewNamedColumnWidth (parent.handle, id, width);
-	return Math.max (0, width [0]);
+	return Math.max (0, width [0] + parent.getLeftDisclosureInset (id));
 }
 
 /**
@@ -273,38 +325,44 @@ public void pack () {
 	checkWidget ();
 	GC gc = new GC (parent);
 	int width = gc.stringExtent (text).x;
-	if (iconRef != 0) {
+	if (iconRef != 0 || (image != null && OS.VERSION >= 0x1040)) {
 		/* Note that the image is stretched to the header height */
 		width += parent.headerHeight;
-		if (text.length () != 0) width += 3;
-	}
-	if ((parent.style & SWT.VIRTUAL) == 0) {
-		int index = parent.indexOf (this);
-		width = Math.max (width, calculateWidth(parent.getItems (null), index, gc, width));
+		if (text.length () != 0) width += parent.getGap ();
 	}
+	int index = parent.indexOf (this);
+	width = Math.max (width, calculateWidth (parent.childIds, index, gc, width));
+
 	gc.dispose ();
-	setWidth (width + Tree.EXTRA_WIDTH);
+	setWidth (width + parent.getInsetWidth (id, true));
 }
 
-int calculateWidth (TreeItem[] items, int index, GC gc, int width) {
-	for (int i=0; i<items.length; i++) {
-		TreeItem item = items [i];
-		width = Math.max (width, item.calculateWidth (index, gc));
-		if (item.getExpanded ()) {
-			width = Math.max (width, calculateWidth(parent.getItems(item), index, gc, width));
+int calculateWidth (int[] ids, int index, GC gc, int width) {
+	int max = width;
+	if (ids == null) return max;
+	for (int i=0; i<ids.length; i++) {
+		TreeItem item = parent._getItem (ids [i], false);
+		if (item != null && item.cached) {
+			max = Math.max (max, item.calculateWidth (index, gc));
+			if (item.getExpanded ()) {
+				max = Math.max (max, calculateWidth (item.childIds, index, gc, max));
+			}
 		}
 	}
-	return width;
+	return max;
 }
 
-void releaseChild () {
-	super.releaseChild ();
-	parent.destroyItem (this);
+void releaseHandle () {
+	super.releaseHandle ();
+	id = -1;
+	parent = null;
 }
 
 void releaseWidget () {
 	super.releaseWidget ();
-	parent = null;
+	if (parent.sortColumn == this) {
+		parent.sortColumn = null;
+	}
 	if (iconRef != 0) OS.ReleaseIconRef (iconRef);
 	iconRef = 0;
 }
@@ -362,10 +420,16 @@ public void removeSelectionListener(SelectionListener listener) {
 void resized (int newWidth) {
 	lastWidth = newWidth;
 	sendEvent (SWT.Resize);
-	int index = parent.indexOf(this);
-	for (int j = index + 1; j < parent.columnCount; j++) {
-		TreeColumn column = parent.columns [j];
-		column.sendEvent (SWT.Move);
+	if (isDisposed ()) return;
+	boolean moved = false;
+	int [] order = parent.getColumnOrder ();
+	TreeColumn [] columns = parent.getColumns ();
+	for (int i=0; i<order.length; i++) {
+		TreeColumn column = columns [order [i]];
+		if (moved && !column.isDisposed ()) {
+			column.sendEvent (SWT.Move);
+		}
+		if (column == this) moved = true;
 	}
 }
 
@@ -404,11 +468,46 @@ public void setImage (Image image) {
 	}
 	super.setImage (image);
 	if (image != null) {
-		iconRef = createIconRef (image);
+		if (OS.VERSION < 0x1040) {
+			iconRef = createIconRef (image);
+		}
 	}
 	updateHeader ();
 }
 
+/**
+ * Sets the moveable attribute.  A column that is
+ * moveable can be reordered by the user by dragging
+ * the header. A column that is not moveable cannot be 
+ * dragged by the user but may be reordered 
+ * by the programmer.
+ *
+ * @param moveable the moveable attribute
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @see Tree#setColumnOrder(int[])
+ * @see Tree#getColumnOrder()
+ * @see TreeColumn#getMoveable()
+ * @see SWT#Move
+ * 
+ * @since 3.2
+ */
+public void setMoveable (boolean moveable) {
+	checkWidget ();
+	int [] flags = new int [1];
+	OS.GetDataBrowserPropertyFlags (parent.handle, id, flags);
+	if (moveable) {
+		flags [0] |= OS.kDataBrowserListViewMovableColumn;
+	} else {
+		flags [0] &= ~OS.kDataBrowserListViewMovableColumn;
+	}
+	OS.SetDataBrowserPropertyFlags (parent.handle, id, flags [0]);
+}
+
 /**
  * Sets the resizable attribute.  A column that is
  * not resizable cannot be dragged by the user but
@@ -434,6 +533,24 @@ public void setText (String string) {
 	updateHeader ();
 }
 
+/**
+ * Sets the receiver's tool tip text to the argument, which
+ * may be null indicating that no tool tip text should be shown.
+ *
+ * @param string the new tool tip text (or null)
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.2
+ */
+public void setToolTipText (String string) {
+	checkWidget();
+	toolTipText = string;
+}
+
 /**
  * Sets the width of the receiver.
  *
@@ -446,6 +563,13 @@ public void setText (String string) {
  */
 public void setWidth (int width) {
 	checkWidget ();
+	/*
+	* Feature in the Macintosh. The data browser widget adds the left inset
+	* of the disclosure column to the specified width making the column too
+	* wide. The fix is to subtract this value from the column width.
+	*/
+	width -= parent.getLeftDisclosureInset (id);
+	if (width < 0) width = 0;
 	OS.SetDataBrowserTableViewNamedColumnWidth (parent.handle, id, (short) width);
 	updateHeader ();
 	if (width != lastWidth) resized (width);
@@ -472,8 +596,15 @@ void updateHeader () {
 		desc.minimumWidth = desc.maximumWidth = width [0];
 	}
 	desc.titleString = str;
-	desc.btnContentInfo_contentType = (short) (iconRef != 0 ? OS.kControlContentIconRef : OS.kControlContentTextOnly);
-	desc.btnContentInfo_iconRef = iconRef;
+	if (OS.VERSION < 0x1040) {
+		desc.btnContentInfo_contentType = (short) (iconRef != 0 ? OS.kControlContentIconRef : OS.kControlContentTextOnly);
+		desc.btnContentInfo_iconRef = iconRef;
+	} else {
+		if (image != null) {
+			desc.btnContentInfo_contentType = OS.kControlContentCGImageRef;
+			desc.btnContentInfo_iconRef = image.handle;
+		}
+	}
 	OS.SetDataBrowserListViewHeaderDesc (parent.handle, id, desc);
 	OS.CFRelease (str);
 }
