@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -65,10 +65,24 @@ public abstract class Widget {
 	static final int MOVED            = 1 << 6;
 	static final int RESIZED          = 1 << 7;
 	static final int EXPANDING        = 1 << 8;
-	static final int LAYOUT_NEEDED    = 1 << 9;
-	static final int LAYOUT_CHANGED   = 1 << 10;
-	static final int IGNORE_WHEEL     = 1 << 11;
+	static final int IGNORE_WHEEL     = 1 << 9;
+	static final int PARENT_BACKGROUND = 1 << 10;
+	static final int THEME_BACKGROUND = 1 << 11;
+	
+	/* A layout was requested on this widget */
+	static final int LAYOUT_NEEDED	= 1<<12;
+	
+	/* The preferred size of a child has changed */
+	static final int LAYOUT_CHANGED = 1<<13;
+	
+	/* A layout was requested in this widget hierachy */
+	static final int LAYOUT_CHILD = 1<<14;
 
+	/* More global state flags */
+	static final int RELEASED = 1<<15;
+	static final int DISPOSE_SENT = 1<<16;
+	
+	/* Default size for widgets */
 	static final int DEFAULT_WIDTH	= 64;
 	static final int DEFAULT_HEIGHT	= 64;
 	
@@ -120,9 +134,10 @@ int actionProc (int theControl, int partCode) {
 
 /**
  * Adds the listener to the collection of listeners who will
- * be notifed when an event of the given type occurs. When the
+ * be notified when an event of the given type occurs. When the
  * event does occur in the widget, the listener is notified by
- * sending it the <code>handleEvent()</code> message.
+ * sending it the <code>handleEvent()</code> message. The event
+ * type is one of the event constants defined in class <code>SWT</code>.
  *
  * @param eventType the type of event to listen for
  * @param listener the listener which should be notified when the event occurs
@@ -136,13 +151,19 @@ int actionProc (int theControl, int partCode) {
  * </ul>
  *
  * @see Listener
+ * @see SWT
  * @see #removeListener
+ * @see #notifyListeners
  */
-public void addListener (int eventType, Listener handler) {
+public void addListener (int eventType, Listener listener) {
 	checkWidget();
-	if (handler == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
+	_addListener (eventType, listener);
+}
+
+void _addListener (int eventType, Listener listener) {
 	if (eventTable == null) eventTable = new EventTable ();
-	eventTable.hook (eventType, handler);
+	eventTable.hook (eventType, listener);
 }
 
 int callPaintEventHandler (int control, int damageRgn, int visibleRgn, int theEvent, int nextHandler) {
@@ -151,7 +172,7 @@ int callPaintEventHandler (int control, int damageRgn, int visibleRgn, int theEv
 
 /**
  * Adds the listener to the collection of listeners who will
- * be notifed when the widget is disposed. When the widget is
+ * be notified when the widget is disposed. When the widget is
  * disposed, the listener is notified by sending it the
  * <code>widgetDisposed()</code> message.
  *
@@ -200,16 +221,18 @@ void calculateVisibleRegion (int control, int visibleRgn, boolean clipChildren)
 		int tempControl = control, lastControl = 0;
 		while (tempControl != root) {
 			OS.GetControlRegion (tempControl, (short) OS.kControlStructureMetaPart, tempRgn);
+			if (OS.HIVIEW) OS.HIViewConvertRegion (tempRgn, tempControl, root);
 			OS.SectRgn (tempRgn, visibleRgn, visibleRgn);
 			if (OS.EmptyRgn (visibleRgn)) break;
 			if (clipChildren || tempControl != control) {
 				OS.CountSubControls (tempControl, count);
 				for (int i = 0; i < count [0]; i++) {
-					OS.GetIndexedSubControl (tempControl, (short)(i + 1), outControl);
+					OS.GetIndexedSubControl (tempControl, (short)(OS.HIVIEW ? count [0] - i : i + 1), outControl);
 					int child = outControl [0];
 					if (child == lastControl) break;
 					if (!OS.IsControlVisible (child)) continue;
 					OS.GetControlRegion (child, (short) OS.kControlStructureMetaPart, tempRgn);
+					if (OS.HIVIEW) OS.HIViewConvertRegion (tempRgn, child, root);
 					OS.UnionRgn (tempRgn, childRgn, childRgn);
 				}
 			}
@@ -313,6 +336,10 @@ int colorProc (int inControl, int inMessage, int inDrawDepth, int inDrawInColor)
 	return OS.eventNotHandledErr;
 }
 
+boolean contains (int shellX, int shellY) {
+	return true;
+}
+
 int controlProc (int nextHandler, int theEvent, int userData) {
 	int eventKind = OS.GetEventKind (theEvent);
 	switch (eventKind) {
@@ -327,7 +354,19 @@ int controlProc (int nextHandler, int theEvent, int userData) {
 		case OS.kEventControlSetCursor:				return kEventControlSetCursor (nextHandler, theEvent, userData);
 		case OS.kEventControlSetFocusPart:			return kEventControlSetFocusPart (nextHandler, theEvent, userData);
 		case OS.kEventControlTrack:					return kEventControlTrack (nextHandler, theEvent, userData);
-		case OS.kEventControlGetFocusPart:			return kEventControlGetFocusPart (nextHandler, theEvent, userData); 
+		case OS.kEventControlGetFocusPart:			return kEventControlGetFocusPart (nextHandler, theEvent, userData);
+		case OS.kEventControlHitTest:				return kEventControlHitTest (nextHandler, theEvent, userData);
+		case OS.kEventControlGetClickActivation:	return kEventControlGetClickActivation (nextHandler, theEvent, userData);
+	}
+	return OS.eventNotHandledErr;
+}
+
+int accessibilityProc (int nextHandler, int theEvent, int userData) {
+	int eventKind = OS.GetEventKind (theEvent);
+	switch (eventKind) {
+		case OS.kEventAccessibleGetChildAtPoint:	return kEventAccessibleGetChildAtPoint (nextHandler, theEvent, userData);
+		case OS.kEventAccessibleGetAllAttributeNames:	return kEventAccessibleGetAllAttributeNames (nextHandler, theEvent, userData);
+		case OS.kEventAccessibleGetNamedAttribute:	return kEventAccessibleGetNamedAttribute (nextHandler, theEvent, userData);
 	}
 	return OS.eventNotHandledErr;
 }
@@ -598,76 +637,16 @@ public void dispose () {
 	* Note:  It is valid to attempt to dispose a widget
 	* more than once.  If this happens, fail silently.
 	*/
-	if (isDisposed()) return;
+	if (isDisposed ()) return;
 	if (!isValidThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
-	releaseChild ();
-	releaseWidget ();
-	destroyWidget ();
+	release (true);
 }
 
-void drawBackground (int control) {
+void drawBackground (int control, int context) {
 	/* Do nothing */
 }
 
-void drawBackground (int control, float [] background) {
-	Rect rect = new Rect ();
-	OS.GetControlBounds (control, rect);
-	if (background != null) {
-		OS.RGBForeColor (toRGBColor (background));
-		OS.PaintRect (rect);
-	} else {
-		OS.SetThemeBackground((short) OS.kThemeBrushDialogBackgroundActive, (short) 0, true);
-		OS.EraseRect (rect);
-	}
-}
-
-void drawFocus (int control, boolean hasFocus, boolean hasBorder, float[] background, Rect inset) {
-	drawBackground (control, background);
-	Rect rect = new Rect ();
-	OS.GetControlBounds (control, rect);
-	rect.left += inset.left;
-	rect.top += inset.top;
-	rect.right -= inset.right;
-	rect.bottom -= inset.bottom;
-	int state;
-	if (OS.IsControlEnabled (control)) {
-		state = OS.IsControlActive (control) ? OS.kThemeStateActive : OS.kThemeStateInactive;
-	} else {
-		state = OS.IsControlActive (control) ? OS.kThemeStateUnavailable : OS.kThemeStateUnavailableInactive;
-	}
-	if (hasFocus) {
-		if (hasBorder) OS.DrawThemeEditTextFrame (rect, state);
-		OS.DrawThemeFocusRect (rect, true);
-	} else {
-		/*
-		* This code is intentionaly commented.
-		*  
-		* NOTE: the focus ring is erased by drawBackground() above. 
-		*/
-//		OS.DrawThemeFocusRect (rect, false);
-		if (hasBorder) OS.DrawThemeEditTextFrame (rect, state);
-	}
-}
-
-void drawFocusClipped (int control, boolean hasFocus, boolean hasBorder, float[] background, Rect inset) {
-	int visibleRgn = getVisibleRegion (control, true);
-	if (!OS.EmptyRgn (visibleRgn)) {
-		int [] currentPort = new int [1];
-		OS.GetPort (currentPort);
-		int window = OS.GetControlOwner (control);
-		int port = OS.GetWindowPort (window);
-		OS.SetPort (port);
-		int oldClip = OS.NewRgn ();
-		OS.GetClip (oldClip);
-		OS.SetClip (visibleRgn);
-		drawFocus (control, hasFocus, hasBorder, background, inset);
-		OS.SetClip (oldClip);
-		OS.SetPort (currentPort [0]);
-	}
-	OS.DisposeRgn (visibleRgn);
-}
-
-void drawWidget (int control, int damageRgn, int visibleRgn, int theEvent) {
+void drawWidget (int control, int context, int damageRgn, int visibleRgn, int theEvent) {
 }
 
 void error (int code) {
@@ -690,7 +669,17 @@ int fixMnemonic (char [] buffer) {
 	return j;
 }
 
-Rect getControlBounds (int control) {
+Rectangle getControlBounds (int control) {
+	if (OS.HIVIEW) {
+		CGRect rect = new CGRect ();
+		OS.HIViewGetFrame (control, rect);
+		Rect inset = getInset ();
+		rect.x -= inset.left;
+		rect.y -= inset.top;
+		rect.width += inset.right + inset.left;
+		rect.height += inset.bottom + inset.top;
+		return new Rectangle ((int) rect.x, (int) rect.y, (int) rect.width, (int) rect.height);
+	}
 	Rect rect = new Rect();
 	OS.GetControlBounds (control, rect);
 	int window = OS.GetControlOwner (control);
@@ -708,10 +697,18 @@ Rect getControlBounds (int control) {
 	rect.top -= inset.top;
 	rect.right += inset.right;
 	rect.bottom += inset.bottom;
-	return rect;
+	return new Rectangle (rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top);
 }
 
-Rect getControlSize (int control) {
+Point getControlSize (int control) {
+	if (OS.HIVIEW) {
+		CGRect rect = new CGRect ();
+		OS.HIViewGetFrame (control, rect);
+		Rect inset = getInset ();
+		int width = (int) rect.width + inset.left + inset.right;
+		int height = (int) rect.height + inset.top + inset.bottom;
+		return new Point (width, height);
+	}
 	Rect rect = new Rect ();
 	OS.GetControlBounds (control, rect);
 	Rect inset = getInset ();
@@ -719,7 +716,7 @@ Rect getControlSize (int control) {
 	rect.top -= inset.top;
 	rect.right += inset.right;
 	rect.bottom += inset.bottom;
-	return rect;
+	return new Point (rect.right - rect.left, rect.bottom - rect.top);
 }
 
 /**
@@ -906,15 +903,18 @@ boolean isEnabled () {
 /**
  * Returns <code>true</code> if there are any listeners
  * for the specified event type associated with the receiver,
- * and <code>false</code> otherwise.
+ * and <code>false</code> otherwise. The event type is one of
+ * the event constants defined in class <code>SWT</code>.
  *
- * @param	eventType the type of event
+ * @param eventType the type of event
  * @return true if the event is hooked
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ *
+ * @see SWT
  */
 public boolean isListening (int eventType) {
 	checkWidget();
@@ -945,6 +945,18 @@ int itemNotificationProc (int browser, int item, int message) {
 	return OS.noErr;
 }
 
+int kEventAccessibleGetChildAtPoint (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
+int kEventAccessibleGetAllAttributeNames (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
+int kEventAccessibleGetNamedAttribute (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
 int kEventProcessCommand (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
@@ -976,30 +988,46 @@ int kEventControlDeactivate (int nextHandler, int theEvent, int userData) {
 int kEventControlDraw (int nextHandler, int theEvent, int userData) {
 	int [] theControl = new int [1];
 	OS.GetEventParameter (theEvent, OS.kEventParamDirectObject, OS.typeControlRef, null, 4, null, theControl);
-	if (getDrawCount (theControl [0]) > 0) return OS.noErr;
 	int [] region = new int [1];	
 	OS.GetEventParameter (theEvent, OS.kEventParamRgnHandle, OS.typeQDRgnHandle, null, 4, null, region);
-	int visibleRgn = getVisibleRegion (theControl [0], true);
-	OS.SectRgn(region [0], visibleRgn, visibleRgn);
-	if (!OS.EmptyRgn (visibleRgn)) {
-		int [] port = new int [1];
-		OS.GetPort (port);
-		OS.LockPortBits (port [0]);
-//		OS.QDSetDirtyRegion (port, visibleRgn);
-		int oldClip = OS.NewRgn ();
-		OS.GetClip (oldClip);
-		OS.SetClip (visibleRgn);
-		drawBackground (theControl [0]);
+	if (OS.HIVIEW) {
+		boolean oldInPaint = display.inPaint;
+		display.inPaint = true;
+		int[] context = new int [1];
+		OS.GetEventParameter (theEvent, OS.kEventParamCGContextRef, OS.typeCGContextRef, null, 4, null, context);
+		int visibleRgn = region [0];
+		drawBackground (theControl [0], context [0]);
 		callPaintEventHandler (theControl [0], region [0], visibleRgn, theEvent, nextHandler);
-		drawWidget (theControl [0], region [0], visibleRgn, theEvent);
-		OS.SetClip (oldClip);
-		OS.DisposeRgn (oldClip);
-		OS.UnlockPortBits (port [0]);
+		drawWidget (theControl [0], context [0], region [0], visibleRgn, theEvent);
+		display.inPaint = oldInPaint;
+	} else {
+		if (getDrawCount (theControl [0]) > 0) return OS.noErr;
+		int visibleRgn = getVisibleRegion (theControl [0], true);
+		OS.SectRgn(region [0], visibleRgn, visibleRgn);
+		if (!OS.EmptyRgn (visibleRgn)) {
+			int [] port = new int [1];
+			OS.GetPort (port);
+			OS.LockPortBits (port [0]);
+//			OS.QDSetDirtyRegion (port, visibleRgn);
+			int oldClip = OS.NewRgn ();
+			OS.GetClip (oldClip);
+			OS.SetClip (visibleRgn);
+			drawBackground (theControl [0], 0);
+			callPaintEventHandler (theControl [0], region [0], visibleRgn, theEvent, nextHandler);
+			drawWidget (theControl [0], 0, region [0], visibleRgn, theEvent);
+			OS.SetClip (oldClip);
+			OS.DisposeRgn (oldClip);
+			OS.UnlockPortBits (port [0]);
+		}
+		OS.DisposeRgn (visibleRgn);
 	}
-	OS.DisposeRgn (visibleRgn);
 	return OS.noErr;
 }
 
+int kEventControlGetClickActivation (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
 int kEventControlGetFocusPart (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
@@ -1008,6 +1036,10 @@ int kEventControlHit (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
+int kEventControlHitTest (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
 int kEventControlSetCursor (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
@@ -1157,6 +1189,10 @@ int kEventWindowExpanded (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
 
+int kEventWindowGetClickModality (int nextHandler, int theEvent, int userData) {
+	return OS.eventNotHandledErr;
+}
+
 int kEventWindowGetRegion (int nextHandler, int theEvent, int userData) {
 	return OS.eventNotHandledErr;
 }
@@ -1221,7 +1257,9 @@ int mouseProc (int nextHandler, int theEvent, int userData) {
 /**
  * Notifies all of the receiver's listeners for events
  * of the given type that one such event has occurred by
- * invoking their <code>handleEvent()</code> method.
+ * invoking their <code>handleEvent()</code> method.  The
+ * event type is one of the event constants defined in class
+ * <code>SWT</code>.
  *
  * @param eventType the type of event which has occurred
  * @param event the event data
@@ -1230,6 +1268,10 @@ int mouseProc (int nextHandler, int theEvent, int userData) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see SWT
+ * @see #addListener
+ * @see #removeListener
  */
 public void notifyListeners (int eventType, Event event) {
 	checkWidget();
@@ -1245,7 +1287,45 @@ void postEvent (int eventType, Event event) {
 	sendEvent (eventType, event, false);
 }
 
+void redrawChildren (int control) {
+	int child = OS.HIViewGetFirstSubview (control);
+	while (child != 0) {
+		OS.HIViewSetNeedsDisplay (child, true);
+		redrawChildren (child);
+		child = OS.HIViewGetNextView (child);
+	}
+}
+
+void redrawChildren (int control, int rgn) {
+	int child = OS.HIViewGetFirstSubview (control);
+	while (child != 0) {
+		OS.HIViewConvertRegion(rgn, control, child);
+		OS.HIViewSetNeedsDisplayInRegion (child, rgn, true);
+		redrawChildren (child, rgn);
+		OS.HIViewConvertRegion(rgn, child, control);
+		child = OS.HIViewGetNextView (child);
+	}
+}
+
 void redrawWidget (int control, boolean children) {
+	if (OS.HIVIEW) {
+		if (display.inPaint) {
+			int rgn = OS.NewRgn ();
+			Rect rect = new Rect ();
+			OS.GetControlBounds (control, rect);
+			rect.right += rect.left;
+			rect.bottom += rect.top;
+			rect.top = rect.left = 0;
+			OS.RectRgn (rgn, rect);
+			OS.HIViewConvertRegion (rgn, control, 0);
+			invalWindowRgn (0, rgn);
+			OS.DisposeRgn (rgn);
+		} else {
+			OS.HIViewSetNeedsDisplay (control, true);
+			if (children) redrawChildren (control);
+		}
+		return;
+	}
 	if (!isDrawing (control)) return;
 	int window = OS.GetControlOwner (control);
 	int visibleRgn = getVisibleRegion (control, !children);
@@ -1254,6 +1334,21 @@ void redrawWidget (int control, boolean children) {
 }
 
 void redrawWidget (int control, int x, int y, int width, int height, boolean children) {
+	if (OS.HIVIEW) {
+		int rgn = OS.NewRgn ();
+		Rect rect = new Rect ();
+		OS.SetRect (rect, (short) x, (short) y, (short) (x + width), (short) (y + height));
+		OS.RectRgn (rgn, rect);
+		if (display.inPaint) {
+			OS.HIViewConvertRegion (rgn, control, 0);
+			invalWindowRgn (0, rgn);
+		} else {
+			OS.HIViewSetNeedsDisplayInRegion (control, rgn, true);
+			if (children) redrawChildren (control, rgn);
+		}
+		OS.DisposeRgn (rgn);
+		return;
+	}
 	if (!isDrawing (control)) return;
 	Rect rect = new Rect ();
 	OS.GetControlBounds (control, rect);
@@ -1273,8 +1368,28 @@ void redrawWidget (int control, int x, int y, int width, int height, boolean chi
 void register () {
 }
 
-void releaseChild () {
-	/* Do nothing */
+void release (boolean destroy) {
+	if ((state & DISPOSE_SENT) == 0) {
+		state |= DISPOSE_SENT;
+		sendEvent (SWT.Dispose);
+	}
+	if ((state & DISPOSED) == 0) {
+		releaseChildren (destroy);
+	}
+	if ((state & RELEASED) == 0) {
+		state |= RELEASED;
+		if (destroy) {
+			releaseParent ();
+			releaseWidget ();
+			destroyWidget ();
+		} else {
+			releaseWidget ();
+			releaseHandle ();
+		}
+	}
+}
+
+void releaseChildren (boolean destroy) {
 }
 
 void releaseHandle () {
@@ -1282,13 +1397,11 @@ void releaseHandle () {
 	display = null;
 }
 
-void releaseResources () {
-	releaseWidget ();
-	releaseHandle ();
+void releaseParent () {
+	/* Do nothing */
 }
 
 void releaseWidget () {
-	sendEvent (SWT.Dispose);
 	deregister ();
 	eventTable = null;
 	data = null;
@@ -1296,7 +1409,8 @@ void releaseWidget () {
 
 /**
  * Removes the listener from the collection of listeners who will
- * be notifed when an event of the given type occurs.
+ * be notified when an event of the given type occurs. The event
+ * type is one of the event constants defined in class <code>SWT</code>.
  *
  * @param eventType the type of event to listen for
  * @param listener the listener which should no longer be notified when the event occurs
@@ -1310,7 +1424,9 @@ void releaseWidget () {
  * </ul>
  *
  * @see Listener
+ * @see SWT
  * @see #addListener
+ * @see #notifyListeners
  */
 public void removeListener (int eventType, Listener handler) {
 	checkWidget();
@@ -1321,7 +1437,7 @@ public void removeListener (int eventType, Listener handler) {
 
 /**
  * Removes the listener from the collection of listeners who will
- * be notifed when an event of the given type occurs.
+ * be notified when an event of the given type occurs.
  * <p>
  * <b>IMPORTANT:</b> This method is <em>not</em> part of the SWT
  * public API. It is marked public only so that it can be shared
@@ -1352,7 +1468,7 @@ protected void removeListener (int eventType, SWTEventListener handler) {
 
 /**
  * Removes the listener from the collection of listeners who will
- * be notifed when the widget is disposed.
+ * be notified when the widget is disposed.
  *
  * @param listener the listener which should no longer be notified when the receiver is disposed
  *
@@ -1452,64 +1568,93 @@ boolean sendKeyEvent (int type, Event event) {
 }
 
 int setBounds (int control, int x, int y, int width, int height, boolean move, boolean resize, boolean events) {
-	/* Compute the old bounds */
-	Rect oldBounds = new Rect ();
-	OS.GetControlBounds (control, oldBounds);
-	int [] theRoot = new int [1];
-	int window = OS.GetControlOwner (control);
-	OS.GetRootControl (window, theRoot);
-	int [] parentHandle = new int [1];
-	OS.GetSuperControl (control, parentHandle);
-	Rect parentRect = new Rect ();
-	if (parentHandle [0] != theRoot [0]) {
-		OS.GetControlBounds (parentHandle [0], parentRect);
-		OS.OffsetRect (oldBounds, (short) -parentRect.left, (short) -parentRect.top);
-	}
-	Rect inset = getInset ();
-	oldBounds.left -= inset.left;
-	oldBounds.top -= inset.top;
-	oldBounds.right += inset.right;
-	oldBounds.bottom += inset.bottom;
+	boolean sameOrigin = true, sameExtent = true;
+	if (OS.HIVIEW) {
+		CGRect oldBounds = new CGRect ();
+		OS.HIViewGetFrame (control, oldBounds);
+		Rect inset = getInset ();
+		oldBounds.x -= inset.left;
+		oldBounds.y -= inset.top;
+		oldBounds.width += inset.left + inset.right;
+		oldBounds.height += inset.top + inset.bottom;
+		if (!move) {
+			x = (int) oldBounds.x;
+			y = (int) oldBounds.y;
+		}
+		if (!resize) {
+			width = (int) oldBounds.width;
+			height = (int) oldBounds.height;
+		}
+		CGRect newBounds = new CGRect ();
+		newBounds.x = x + inset.left;
+		newBounds.y = y + inset.top;
+		newBounds.width = width - inset.right - inset.left;
+		newBounds.height = height - inset.bottom - inset.top;
+		sameOrigin = newBounds.x == oldBounds.x && newBounds.y == oldBounds.y;
+		sameExtent = newBounds.width == oldBounds.width && newBounds.height == oldBounds.height;
+		if (sameOrigin && sameExtent) return 0;
+		OS.HIViewSetFrame (control, newBounds);
+		invalidateVisibleRegion (control);
+	} else {
+		/* Compute the old bounds */
+		Rect oldBounds = new Rect ();
+		OS.GetControlBounds (control, oldBounds);
+		int [] theRoot = new int [1];
+		int window = OS.GetControlOwner (control);
+		OS.GetRootControl (window, theRoot);
+		int [] parentHandle = new int [1];
+		OS.GetSuperControl (control, parentHandle);
+		Rect parentRect = new Rect ();
+		if (parentHandle [0] != theRoot [0]) {
+			OS.GetControlBounds (parentHandle [0], parentRect);
+			OS.OffsetRect (oldBounds, (short) -parentRect.left, (short) -parentRect.top);
+		}
+		Rect inset = getInset ();
+		oldBounds.left -= inset.left;
+		oldBounds.top -= inset.top;
+		oldBounds.right += inset.right;
+		oldBounds.bottom += inset.bottom;
+		
+		/* Compute the new bounds */
+		if (!move) {
+			x = oldBounds.left;
+			y = oldBounds.top;
+		}
+		if (!resize) {
+			width = oldBounds.right - oldBounds.left;
+			height = oldBounds.bottom - oldBounds.top;
+		}	
+		Rect newBounds = new Rect ();
+		newBounds.left = (short) (parentRect.left + x + inset.left);
+		newBounds.top = (short) (parentRect.top + y + inset.top);
+		newBounds.right = (short) (newBounds.left + width - inset.right - inset.left);
+		newBounds.bottom = (short) (newBounds.top + height - inset.bottom - inset.top);	
+		if (newBounds.bottom < newBounds.top) newBounds.bottom = newBounds.top;
+		if (newBounds.right < newBounds.left) newBounds.right = newBounds.left;
 	
-	/* Compute the new bounds */
-	if (!move) {
-		x = oldBounds.left;
-		y = oldBounds.top;
-	}
-	if (!resize) {
-		width = oldBounds.right - oldBounds.left;
-		height = oldBounds.bottom - oldBounds.top;
-	}	
-	Rect newBounds = new Rect ();
-	newBounds.left = (short) (parentRect.left + x + inset.left);
-	newBounds.top = (short) (parentRect.top + y + inset.top);
-	newBounds.right = (short) (newBounds.left + width - inset.right - inset.left);
-	newBounds.bottom = (short) (newBounds.top + height - inset.bottom - inset.top);	
-	if (newBounds.bottom < newBounds.top) newBounds.bottom = newBounds.top;
-	if (newBounds.right < newBounds.left) newBounds.right = newBounds.left;
-
-	/* Get bounds again, since the one above is in SWT coordinates */
-	OS.GetControlBounds (control, oldBounds);
+		/* Get bounds again, since the one above is in SWT coordinates */
+		OS.GetControlBounds (control, oldBounds);
+		
+		/* Check if anything changed */
+		sameOrigin = newBounds.left == oldBounds.left && newBounds.top == oldBounds.top;
+		sameExtent = (newBounds.right - newBounds.left) == (oldBounds.right - oldBounds.left) && (newBounds.bottom - newBounds.top) == (oldBounds.bottom - oldBounds.top);
+		if (sameOrigin && sameExtent) return 0;
 	
-	/* Check if anything changed */
-	boolean sameOrigin = newBounds.left == oldBounds.left && newBounds.top == oldBounds.top;
-	boolean sameExtent = (newBounds.right - newBounds.left) == (oldBounds.right - oldBounds.left) && (newBounds.bottom - newBounds.top) == (oldBounds.bottom - oldBounds.top);
-	if (sameOrigin && sameExtent) return 0;
-
-	/* Apply changes and invalidate appropriate rectangles */
-	int tempRgn = 0;
-	boolean visible = OS.IsControlVisible (control);
-	if (visible) {
-		tempRgn = OS.NewRgn ();
-		OS.GetControlRegion (control, (short) OS.kControlStructureMetaPart, tempRgn);
-		invalWindowRgn (window, tempRgn);
-	}
-	OS.SetControlBounds (control, newBounds);
-	invalidateVisibleRegion (control);
-	if (visible) {
-		OS.GetControlRegion (control, (short) OS.kControlStructureMetaPart, tempRgn);
-		invalWindowRgn (window, tempRgn);
-		OS.DisposeRgn(tempRgn);
+		/* Apply changes and invalidate appropriate rectangles */
+		int tempRgn = 0;
+		boolean visible = OS.IsControlVisible (control);
+		if (visible) {
+			tempRgn = OS.NewRgn ();
+			OS.GetControlRegion (control, (short) OS.kControlStructureMetaPart, tempRgn);
+			invalWindowRgn (window, tempRgn);
+		}
+		OS.SetControlBounds (control, newBounds);
+		invalidateVisibleRegion (control);
+		if (visible) {
+			OS.GetControlRegion (control, (short) OS.kControlStructureMetaPart, tempRgn);
+			invalWindowRgn (window, tempRgn);
+			OS.DisposeRgn(tempRgn);
+		}
 	}
 	
 	/* Send events */
@@ -1804,37 +1949,48 @@ boolean setKeyState (Event event, int type, int theEvent) {
 }
 
 void setVisible (int control, boolean visible) {
-	int visibleRgn = 0;
-	boolean drawing = getDrawCount (control) == 0;
-	if (drawing && !visible) visibleRgn = getVisibleRegion (control, false);
-	OS.SetControlVisibility (control, visible, false);
-	invalidateVisibleRegion (control);
-	if (drawing && visible) visibleRgn = getVisibleRegion (control, false);
-	if (drawing) {
-		int window = OS.GetControlOwner (control);
-		invalWindowRgn (window, visibleRgn);
-		OS.DisposeRgn (visibleRgn);
+	if (OS.HIVIEW) {
+		OS.HIViewSetVisible (control, visible);
+		invalidateVisibleRegion (control);
+	} else {
+		int visibleRgn = 0;
+		boolean drawing = getDrawCount (control) == 0;
+		if (drawing && !visible) visibleRgn = getVisibleRegion (control, false);
+		OS.SetControlVisibility (control, visible, false);
+		invalidateVisibleRegion (control);
+		if (drawing && visible) visibleRgn = getVisibleRegion (control, false);
+		if (drawing) {
+			int window = OS.GetControlOwner (control);
+			invalWindowRgn (window, visibleRgn);
+			OS.DisposeRgn (visibleRgn);
+		}
 	}
 }
 
 void setZOrder (int control, int otheControl, boolean above) {
-	int inOp = above ?  OS.kHIViewZOrderBelow :  OS.kHIViewZOrderAbove;
-	int oldRgn = 0;
-	boolean drawing = isDrawing (control);
-	if (drawing) oldRgn = getVisibleRegion (control, false);
-	OS.HIViewSetZOrder (control, inOp, otheControl);
-	invalidateVisibleRegion (control);
-	if (drawing) {
-		int newRgn = getVisibleRegion (control, false);
-		if (above) {
-			OS.DiffRgn (newRgn, oldRgn, newRgn);
-		} else {
-			OS.DiffRgn (oldRgn, newRgn, newRgn);
+	if (OS.HIVIEW) {
+		int inOp = above ?  OS.kHIViewZOrderAbove :  OS.kHIViewZOrderBelow;
+		OS.HIViewSetZOrder (control, inOp, otheControl);
+		invalidateVisibleRegion (control);
+	} else {
+		int inOp = above ?  OS.kHIViewZOrderBelow :  OS.kHIViewZOrderAbove;
+		int oldRgn = 0;
+		boolean drawing = isDrawing (control);
+		if (drawing) oldRgn = getVisibleRegion (control, false);
+		OS.HIViewSetZOrder (control, inOp, otheControl);
+		invalidateVisibleRegion (control);
+		if (drawing) {
+			int newRgn = getVisibleRegion (control, false);
+			if (above) {
+				OS.DiffRgn (newRgn, oldRgn, newRgn);
+			} else {
+				OS.DiffRgn (oldRgn, newRgn, newRgn);
+			}
+			int window = OS.GetControlOwner (control);
+			invalWindowRgn (window, newRgn);
+			OS.DisposeRgn (oldRgn);
+			OS.DisposeRgn (newRgn);
 		}
-		int window = OS.GetControlOwner (control);
-		invalWindowRgn (window, newRgn);
-		OS.DisposeRgn (oldRgn);
-		OS.DisposeRgn (newRgn);
 	}
 }
 
@@ -1889,6 +2045,7 @@ int windowProc (int nextHandler, int theEvent, int userData) {
 		case OS.kEventWindowHitTest:			return kEventWindowHitTest (nextHandler, theEvent, userData);
 		case OS.kEventWindowShown:				return kEventWindowShown (nextHandler, theEvent, userData);
 		case OS.kEventWindowUpdate:				return kEventWindowUpdate (nextHandler, theEvent, userData);
+		case OS.kEventWindowGetClickModality:	return kEventWindowGetClickModality (nextHandler, theEvent, userData);
 	}
 	return OS.eventNotHandledErr;
 }
