@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -16,9 +16,10 @@ import org.eclipse.swt.graphics.*;
 import java.io.*;
 
 final class WinBMPFileFormat extends FileFormat {
+	static final int BMPFileHeaderSize = 14;
+	static final int BMPHeaderFixedSize = 40;
 	int importantColors;
 	Point pelsPerMeter = new Point(0, 0);
-	public static final int BMPHeaderFixedSize = 40;
 
 /**
  * Compress numBytes bytes of image data from src, storing in dest
@@ -321,10 +322,11 @@ int decompressRLE8Data(byte[] src, int numBytes, int stride, byte[] dest, int de
 }
 boolean isFileFormat(LEDataInputStream stream) {
 	try {
-		byte[] header = new byte[2];
+		byte[] header = new byte[18];
 		stream.read(header);
 		stream.unread(header);
-		return header[0] == 0x42 && header[1] == 0x4D;
+		int infoHeaderSize = (header[14] & 0xFF) | ((header[15] & 0xFF) << 8) | ((header[16] & 0xFF) << 16) | ((header[17] & 0xFF) << 24);
+		return header[0] == 0x42 && header[1] == 0x4D && infoHeaderSize >= BMPHeaderFixedSize;
 	} catch (Exception e) {
 		return false;
 	}
@@ -387,6 +389,9 @@ ImageData[] loadFromByteStream() {
 	} catch (Exception e) {
 		SWT.error(SWT.ERROR_IO, e);
 	}
+	int width = (infoHeader[4] & 0xFF) | ((infoHeader[5] & 0xFF) << 8) | ((infoHeader[6] & 0xFF) << 16) | ((infoHeader[7] & 0xFF) << 24);
+	int height = (infoHeader[8] & 0xFF) | ((infoHeader[9] & 0xFF) << 8) | ((infoHeader[10] & 0xFF) << 16) | ((infoHeader[11] & 0xFF) << 24);
+	int bitCount = (infoHeader[14] & 0xFF) | ((infoHeader[15] & 0xFF) << 8);
 	PaletteData palette = loadPalette(infoHeader);
 	if (inputStream.getPosition() < fileHeader[4]) {
 		// Seek to the specified offset
@@ -402,9 +407,6 @@ ImageData[] loadFromByteStream() {
 	int xPelsPerMeter = (infoHeader[24] & 0xFF) | ((infoHeader[25] & 0xFF) << 8) | ((infoHeader[26] & 0xFF) << 16) | ((infoHeader[27] & 0xFF) << 24);
 	int yPelsPerMeter = (infoHeader[28] & 0xFF) | ((infoHeader[29] & 0xFF) << 8) | ((infoHeader[30] & 0xFF) << 16) | ((infoHeader[31] & 0xFF) << 24);
 	this.pelsPerMeter = new Point(xPelsPerMeter, yPelsPerMeter);
-	int width = (infoHeader[4] & 0xFF) | ((infoHeader[5] & 0xFF) << 8) | ((infoHeader[6] & 0xFF) << 16) | ((infoHeader[7] & 0xFF) << 24);
-	int height = (infoHeader[8] & 0xFF) | ((infoHeader[9] & 0xFF) << 8) | ((infoHeader[10] & 0xFF) << 16) | ((infoHeader[11] & 0xFF) << 24);
-	int bitCount = (infoHeader[14] & 0xFF) | ((infoHeader[15] & 0xFF) << 8);
 	int type = (this.compression == 1 /*BMP_RLE8_COMPRESSION*/) || (this.compression == 2 /*BMP_RLE4_COMPRESSION*/) ? SWT.IMAGE_BMP_RLE : SWT.IMAGE_BMP;
 	return new ImageData[] {
 		ImageData.internal_new(
@@ -431,7 +433,7 @@ PaletteData loadPalette(byte[] infoHeader) {
 	if (depth <= 8) {
 		int numColors = (infoHeader[32] & 0xFF) | ((infoHeader[33] & 0xFF) << 8) | ((infoHeader[34] & 0xFF) << 16) | ((infoHeader[35] & 0xFF) << 24);
 		if (numColors == 0) {
-			numColors = 1 << ((infoHeader[14] & 0xFF) | ((infoHeader[15] & 0xFF) << 8));
+			numColors = 1 << depth;
 		} else {
 			if (numColors > 256)
 				numColors = 256;
@@ -464,8 +466,8 @@ PaletteData paletteFromBytes(byte[] bytes, int numColors) {
  * Answer a byte array containing the BMP representation of
  * the given device independent palette.
  */
-byte[] paletteToBytes(PaletteData pal) {
-	int n = (pal.colors.length < 256) ? pal.colors.length : 256;
+static byte[] paletteToBytes(PaletteData pal) {
+	int n = pal.colors == null ? 0 : (pal.colors.length < 256 ? pal.colors.length : 256);
 	byte[] bytes = new byte[n * 4];
 	int offset = 0;
 	for (int i = 0; i < n; i++) {
@@ -567,7 +569,8 @@ int unloadDataNoCompression(ImageData image, OutputStream out) {
  * Unload a DeviceIndependentImage using Windows .BMP format into the given
  * byte stream.
  */
-void unloadIntoByteStream(ImageData image) {
+void unloadIntoByteStream(ImageLoader loader) {
+	ImageData image = loader.data[0];
 	byte[] rgbs;
 	int numCols;
 	if (!((image.depth == 1) || (image.depth == 4) || (image.depth == 8) ||
@@ -590,7 +593,7 @@ void unloadIntoByteStream(ImageData image) {
 		rgbs = paletteToBytes(pal);
 	}
 	// Fill in file header, except for bfsize, which is done later.
-	int headersSize = 54;
+	int headersSize = BMPFileHeaderSize + BMPHeaderFixedSize;
 	int[] fileHeader = new int[5];
 	fileHeader[0] = 0x4D42;	// Signature
 	fileHeader[1] = 0; // File size - filled in later
@@ -605,10 +608,10 @@ void unloadIntoByteStream(ImageData image) {
 	// the stream and fill in the details later.
 	ByteArrayOutputStream out = new ByteArrayOutputStream();
 	unloadData(image, out, comp);
-	byte[] compressedData = out.toByteArray();
+	byte[] data = out.toByteArray();
 	
 	// Calculate file size
-	fileHeader[1] = fileHeader[4] + compressedData.length;
+	fileHeader[1] = fileHeader[4] + data.length;
 
 	// Write the headers
 	try {
@@ -621,13 +624,13 @@ void unloadIntoByteStream(ImageData image) {
 		SWT.error(SWT.ERROR_IO, e);
 	}
 	try {
-		outputStream.writeInt(WinBMPFileFormat.BMPHeaderFixedSize);
+		outputStream.writeInt(BMPHeaderFixedSize);
 		outputStream.writeInt(image.width);
 		outputStream.writeInt(image.height);
 		outputStream.writeShort(1);
 		outputStream.writeShort((short)image.depth);
 		outputStream.writeInt(comp);
-		outputStream.writeInt(compressedData.length);
+		outputStream.writeInt(data.length);
 		outputStream.writeInt(pelsPerMeter.x);
 		outputStream.writeInt(pelsPerMeter.y);
 		outputStream.writeInt(numCols);
@@ -647,7 +650,7 @@ void unloadIntoByteStream(ImageData image) {
 
 	// Unload the data
 	try {
-		outputStream.write(compressedData);
+		outputStream.write(data);
 	} catch (IOException e) {
 		SWT.error(SWT.ERROR_IO, e);
 	}
