@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -30,11 +30,13 @@ import org.eclipse.swt.graphics.*;
  * </p><p>
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>(none)</dd>
+ * <dd>TOP, BOTTOM</dd>
  * <dt><b>Events:</b></dt>
  * <dd>Selection</dd>
  * </dl>
  * <p>
+ * Note: Only one of the styles TOP and BOTTOM may be specified.
+ * </p><p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
  * </p>
  */
@@ -92,7 +94,7 @@ public TabFolder(Composite parent, int style) {
 	addListener (SWT.MouseUp, listener);	
 	addListener (SWT.MouseHover, listener);
 	addListener (SWT.Paint, listener);
-	addListener (SWT.Resize, listener);
+//	addListener (SWT.Resize, listener);
 	addListener (SWT.Traverse, listener);
 	addListener (SWT.KeyDown, listener);
 	addListener (SWT.FocusIn, listener);
@@ -130,6 +132,7 @@ public void addSelectionListener(SelectionListener listener) {
 	addListener(SWT.DefaultSelection,typedListener);
 }
 static int checkStyle (int style) {
+	style = checkBits (style, SWT.TOP, SWT.BOTTOM, 0, 0, 0, 0);
 	/*
 	* Even though it is legal to create this widget
 	* with scroll bars, they serve no useful purpose
@@ -143,29 +146,14 @@ protected void checkSubclass () {
 	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
 }
 public Point computeSize (int wHint, int hHint, boolean changed) {
-	checkWidget();
-	int width = CLIENT_MARGIN_WIDTH * 2 + TabItem.SHADOW_WIDTH * 2;
-	int height = 0;
-
+	Point size = super.computeSize (wHint, hHint, changed);
 	if (items.length > 0) {
 		TabItem lastItem = items[items.length-1];
-		width = Math.max (width, lastItem.x + lastItem.width);
+		int border = getBorderWidth ();
+		int width = lastItem.x + lastItem.width + border * 2 + CLIENT_MARGIN_WIDTH * 2 + TabItem.SHADOW_WIDTH * 2;
+		size.x = Math.max (width, size.x);
 	}
-	Point size;
-	Layout layout = getLayout();
-	if (layout != null) {
-		size = layout.computeSize (this, wHint, hHint, changed);
-	} else {
-		size = minimumSize (wHint, hHint, changed);
-	}
-	if (size.x == 0) size.x = DEFAULT_WIDTH;
-	if (size.y == 0) size.y = DEFAULT_HEIGHT;
-	if (wHint != SWT.DEFAULT) size.x = wHint;
-	if (hHint != SWT.DEFAULT) size.y = hHint;
-	width = Math.max (width, size.x);
-	height = Math.max (height, size.y);
-	Rectangle trim = computeTrim (0, 0, width, height);
-	return new Point (trim.width, trim.height);
+	return size;
 }
 public Rectangle computeTrim (int x, int y, int width, int height) {
 	checkWidget();
@@ -265,8 +253,11 @@ void destroyChild (TabItem item) {
 /**
  * Dispose the items of the receiver
  */
-void doDispose() {
+void doDispose(Event event) {
+	if (inDispose) return;
 	inDispose = true;
+	notifyListeners(SWT.Dispose, event);
+	event.type = SWT.None;
 	// items array is resized during TabItem.dispose
 	// it is length 0 if the last item is removed
 	while (items.length > 0) {						
@@ -310,8 +301,8 @@ void drawBorder(Event event) {
 	int wClient = clientArea.width;
 	int hClient = clientArea.height;
 	int x, y, x1, y1;
-	final Color HighlightShadow = getDisplay().getSystemColor(SWT.COLOR_WIDGET_HIGHLIGHT_SHADOW);
-	final Color LightShadow = getDisplay().getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW);
+	final Color HighlightShadow = display.getSystemColor(SWT.COLOR_WIDGET_HIGHLIGHT_SHADOW);
+	final Color LightShadow = display.getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW);
 
 	// Draw the left line
 	gc.setForeground(HighlightShadow);
@@ -339,7 +330,7 @@ void drawBorder(Event event) {
 	}
 
 	// Draw the right and bottom black lines
-	gc.setForeground(getDisplay().getSystemColor(SWT.COLOR_WIDGET_FOREGROUND));
+	gc.setForeground(display.getSystemColor(SWT.COLOR_WIDGET_FOREGROUND));
 	gc.drawLine((x = xClient - CLIENT_MARGIN_WIDTH),
 		(y = yClient + hClient + CLIENT_MARGIN_WIDTH),
 		(x1 = xClient + wClient + CLIENT_MARGIN_WIDTH),
@@ -352,7 +343,7 @@ void drawBorder(Event event) {
 
 
 	// There is a dark gray line above the bottom back line
-	gc.setForeground(getDisplay().getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW));
+	gc.setForeground(display.getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW));
 	gc.drawLine(x, y, x1, y);
 	// On the right there is a dark gray line, left of the black one
 	gc.drawLine(x1, y-1, x1, y1);
@@ -369,18 +360,18 @@ void drawBorder(Event event) {
  */
 void drawPlainButton(GC gc, int xPos, int yPos, int size) {
 	Color rightBottomColor = getForeground();
-	Color leftTopColor = getDisplay().getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW);
-	Color rightBottomInnerColor = getDisplay().getSystemColor(SWT.COLOR_WIDGET_DARK_SHADOW);
-	Color leftTopInnerColor = getDisplay().getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW);	
+	Color leftTopColor = display.getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW);
+	Color rightBottomInnerColor = display.getSystemColor(SWT.COLOR_WIDGET_DARK_SHADOW);
+	Color leftTopInnerColor = display.getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW);	
 	int upper = yPos;
 	int left = xPos;
 	int lower = yPos + size - 1;
 	int right = xPos + size - 1;
 
 	if (scrollButtonDown) {						// draw the button in the pressed down state?
-		rightBottomColor = getDisplay().getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW);	
-		leftTopColor = getDisplay().getSystemColor(SWT.COLOR_WIDGET_DARK_SHADOW);
-		rightBottomInnerColor = getDisplay().getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW);
+		rightBottomColor = display.getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW);	
+		leftTopColor = display.getSystemColor(SWT.COLOR_WIDGET_DARK_SHADOW);
+		rightBottomInnerColor = display.getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW);
 		leftTopInnerColor = getForeground();
 	}		
 	gc.fillRectangle(left, upper, right - left, lower - upper);
@@ -544,6 +535,17 @@ public TabItem [] getItems() {
 	System.arraycopy(items, 0, tabItems, 0, items.length);
 	return tabItems;
 }
+char getMnemonic (String string) {
+	int index = 0;
+	int length = string.length ();
+	do {
+		while ((index < length) && (string.charAt (index) != '&')) index++;
+		if (++index >= length) return '\0';
+		if (string.charAt (index) != '&') return string.charAt (index);
+		index++;
+	} while (index < length);
+ 	return '\0';
+}
 /** 
  * Returns the area where the two scroll buttons are drawn.
  */
@@ -598,14 +600,14 @@ public String getToolTipText () {
 void handleEvents (Event event){
 	switch (event.type) {
 		case SWT.Dispose:
-			doDispose();
+			doDispose(event);
 			break;
 		case SWT.Paint:
 			paint(event);
 			break;
-		case SWT.Resize:
-			resize();
-			break;
+//		case SWT.Resize:
+//			resize();
+//			break;
 		case SWT.MouseDown:
 			mouseDown(event);
 			break;
@@ -623,7 +625,8 @@ void handleEvents (Event event){
 			focus(event);
 			break;
 		case SWT.KeyDown:
-			//do nothing - this callback exists so that widget is included in tab order
+			// this callback is always needed so that widget is included in tab order
+			keyDown(event);
 			break;
 		default:
 			break;
@@ -712,7 +715,36 @@ void itemChanged(TabItem item) {
 		redrawScrollButtons();
 	}	
 }
-
+/** 
+ * A key was pressed.  If one of the tab-selection keys that is not a traversal
+ * was pressed then change tabs accordingly.
+ */
+void keyDown(Event event) {
+	int count = items.length;
+	if (count <= 1) return;
+	switch (event.keyCode) {
+		case SWT.ARROW_RIGHT:
+			if (selectedIndex < items.length - 1) {
+				setSelection(selectedIndex + 1, true);
+			}
+			break;
+		case SWT.ARROW_LEFT:
+			if (selectedIndex > 0) {
+				setSelection(selectedIndex - 1, true);
+			}
+			break;
+		case SWT.HOME:
+			if (selectedIndex > 0) {
+				setSelection(0, true);
+			}
+			break;
+		case SWT.END:
+			if (selectedIndex < count - 1) {
+				setSelection(count - 1, true);
+			}
+			break;
+	}
+}
 /**
  * Layout the items and store the client area size.
  */
@@ -770,6 +802,20 @@ Point minimumSize (int wHint, int hHint, boolean flushCache) {
 	}
 	return new Point (width, height);
 }
+boolean mnemonicHit (char key) {
+	for (int i = 0; i < items.length; i++) {
+		char mnemonic = getMnemonic (items[i].getText ());
+		if (mnemonic != '\0') {
+			if (Character.toUpperCase (key) == Character.toUpperCase (mnemonic)) {
+				if (forceFocus ()) {
+					if (i != selectedIndex) setSelection(i, true);
+					return true;
+				}
+			}
+		}
+	}
+	return false;
+}
 /** 
  * A mouse button was pressed down. 
  * If one of the tab scroll buttons was hit, scroll in the appropriate 
@@ -930,6 +976,13 @@ void redrawSelectionChange(int oldSelection, int newSelection) {
 void redrawTabs() {
 	redraw(0, 0, super.getClientArea().width, getClientArea().y);
 }
+void removeControl (Control control) {
+	super.removeControl (control);
+	for (int i=0; i<items.length; i++) {
+		TabItem item = items [i];
+		if (item.control == control) item.setControl (null);
+	}
+}
 /**
  * Removes the listener from the collection of listeners who will
  * be notified when the receiver's selection changes.
@@ -992,6 +1045,11 @@ void scrollRight() {
 		}
 	}	
 }
+boolean setBounds (int x, int y, int width, int height, boolean move, boolean resize) {
+	boolean changed = super.setBounds (x, y, width, height, move, resize);
+	if (changed && resize) resize ();
+	return changed;
+}
 public void setFont(Font font) {
 	checkWidget();
 	if (font != null && font.equals(getFont())) return;
@@ -1002,7 +1060,7 @@ public void setFont(Font font) {
 /**
  * Selects the item at the given zero-relative index in the receiver. 
  * If the item at the index was already selected, it remains selected.
- * The current selected is first cleared, then the new items are
+ * The current selection is first cleared, then the new items are
  * selected. Indices that are out of range are ignored.
  *
  * @param index the index of the item to select
@@ -1014,8 +1072,31 @@ public void setFont(Font font) {
  */
 public void setSelection(int index) {
 	checkWidget();
+	if (!(0 <= index && index < items.length)) return;
 	setSelection(index, false);
 }
+/**
+ * Sets the receiver's selection to the given item.
+ * The current selected is first cleared, then the new item is
+ * selected.
+ *
+ * @param item the item to select
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the item is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.2
+ */
+public void setSelection(TabItem item) {
+	checkWidget();
+	if (item == null) error(SWT.ERROR_NULL_ARGUMENT);
+	setSelection(new TabItem[]{item});
+}
 /**
  * Sets the receiver's selection to be the given array of items.
  * The current selected is first cleared, then the new items are
@@ -1023,6 +1104,9 @@ public void setSelection(int index) {
  *
  * @param items the array of items
  *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the items array is null</li>
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -1085,72 +1169,26 @@ void traversal(Event event) {
 		case SWT.TRAVERSE_RETURN:
 		case SWT.TRAVERSE_TAB_NEXT:
 		case SWT.TRAVERSE_TAB_PREVIOUS:
-			event.doit = true;
-			break;
 		case SWT.TRAVERSE_MNEMONIC:
-			event.doit = mnemonicTraversal(event);
-			if (event.doit) event.detail = SWT.TRAVERSE_NONE;
-			break;
 		case SWT.TRAVERSE_PAGE_NEXT:
 		case SWT.TRAVERSE_PAGE_PREVIOUS:
-			event.doit = pageTraversal(event);
-			if (event.doit) event.detail = SWT.TRAVERSE_NONE;
-			break;
-		case SWT.TRAVERSE_ARROW_NEXT:
-			if (selectedIndex < items.length - 1) {
-				setSelection(selectedIndex + 1, true);
-			}
 			event.doit = true;
-			event.detail = SWT.TRAVERSE_NONE;
-			break;
-		case SWT.TRAVERSE_ARROW_PREVIOUS: 
-			if (selectedIndex > 0) {
-				setSelection(selectedIndex - 1, true);
-			}
-			event.doit = true;
-			event.detail = SWT.TRAVERSE_NONE;
-			break;
 	}
 }
-
-boolean pageTraversal(Event event) {
-	int count = getItemCount ();
+boolean traverseItem (boolean next) {
+	return false;
+}
+boolean traversePage (boolean next) {
+	int count = items.length;
 	if (count == 0) return false;
-	int index = getSelectionIndex ();
+	int index = selectedIndex;
 	if (index == -1) {
 		index = 0;
 	} else {
-		int offset = (event.detail == SWT.TRAVERSE_PAGE_NEXT) ? 1 : -1;
+		int offset = next ? 1 : -1;
 		index = (index + offset + count) % count;
 	}
 	setSelection (index, true);
 	return true;
 }
-
-boolean mnemonicTraversal (Event event) {
-	char key = event.character;
-	for (int i = 0; i < items.length; i++) {
-		if (items[i] != null) {
-			char mnemonic = getMnemonic (items[i].getText ());
-			if (mnemonic != '\0') {
-				if (Character.toUpperCase (key) == Character.toUpperCase (mnemonic)) {
-					setSelection(i, true);
-					return true;
-				}
-			}
-		}
-	}
-	return false;
-}
-char getMnemonic (String string) {
-	int index = 0;
-	int length = string.length ();
-	do {
-		while ((index < length) && (string.charAt (index) != '&')) index++;
-		if (++index >= length) return '\0';
-		if (string.charAt (index) != '&') return string.charAt (index);
-		index++;
-	} while (index < length);
- 	return '\0';
-}
 }
