@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -35,6 +35,7 @@ import org.eclipse.swt.events.*;
  */
 public class MenuItem extends Item {
 	Menu parent, menu;
+	int /*long*/ groupHandle;
 	int accelerator;
 	
 /**
@@ -94,10 +95,11 @@ public MenuItem (Menu parent, int style) {
  *
  * @param parent a menu control which will be the parent of the new instance (cannot be null)
  * @param style the style of control to construct
- * @param index the index to store the receiver in its parent
+ * @param index the zero-relative index to store the receiver in its parent
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the parent (inclusive)</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -122,11 +124,11 @@ public MenuItem (Menu parent, int style, int index) {
 	createWidget (index);
 }
 
-void addAccelerator (int accelGroup) {
+void addAccelerator (int /*long*/ accelGroup) {
 	updateAccelerator (accelGroup, true);
 }
 
-void addAccelerators (int accelGroup) {
+void addAccelerators (int /*long*/ accelGroup) {
 	addAccelerator (accelGroup);
 	if (menu != null) menu.addAccelerators (accelGroup);
 }
@@ -185,7 +187,7 @@ public void addHelpListener (HelpListener listener) {
 
 /**
  * Adds the listener to the collection of listeners who will
- * be notified when the control is selected, by sending
+ * be notified when the menu item is selected, by sending
  * it one of the messages defined in the <code>SelectionListener</code>
  * interface.
  * <p>
@@ -219,6 +221,10 @@ static int checkStyle (int style) {
 	return checkBits (style, SWT.PUSH, SWT.CHECK, SWT.RADIO, SWT.SEPARATOR, SWT.CASCADE, 0);
 }
 
+protected void checkSubclass () {
+	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
+}
+
 void createHandle (int index) {
 	state |= HANDLE;
 	byte [] buffer = new byte [1];
@@ -228,8 +234,24 @@ void createHandle (int index) {
 			handle = OS.gtk_separator_menu_item_new ();
 			break;
 		case SWT.RADIO:
-//			handle = OS.gtk_radio_menu_item_new_with_label (0, buffer);
-//			break;
+			/*
+			* Feature in GTK.  In GTK, radio button must always be part of
+			* a radio button group.  In a GTK radio group, one button is always
+			* selected.  This means that it is not possible to have a single
+			* radio button that is unselected.  This is necessary to allow
+			* applications to implement their own radio behavior or use radio
+			* buttons outside of radio groups.  The fix is to create a hidden
+			* radio button for each radio button we create and add them
+			* to the same group.  This allows the visible button to be
+			* unselected.
+			*/
+			groupHandle = OS.gtk_radio_menu_item_new (0);
+			if (groupHandle == 0) error (SWT.ERROR_NO_HANDLES);
+			OS.g_object_ref (groupHandle);
+			OS.gtk_object_sink (groupHandle);
+			int /*long*/ group = OS.gtk_radio_menu_item_get_group (groupHandle);
+			handle = OS.gtk_radio_menu_item_new_with_label (group, buffer);
+			break;
 		case SWT.CHECK:
 			handle = OS.gtk_check_menu_item_new_with_label (buffer);
 			break;
@@ -240,10 +262,10 @@ void createHandle (int index) {
 	}
 	if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 	if ((style & SWT.SEPARATOR) == 0) {
-		int label = OS.gtk_bin_get_child (handle);
+		int /*long*/ label = OS.gtk_bin_get_child (handle);
 		OS.gtk_accel_label_set_accel_widget (label, 0);
 	}
-	int parentHandle = parent.handle;
+	int /*long*/ parentHandle = parent.handle;
 	boolean enabled = OS.GTK_WIDGET_SENSITIVE (parentHandle);     
 	if (!enabled) OS.GTK_WIDGET_SET_FLAGS (parentHandle, OS.GTK_SENSITIVE);
 	OS.gtk_menu_shell_insert (parentHandle, handle, index);
@@ -251,12 +273,18 @@ void createHandle (int index) {
 	OS.gtk_widget_show (handle);
 }
 
+void fixMenus (Decorations newParent) {
+	if (menu != null) menu.fixMenus (newParent);
+}
+
 /**
- * Return the widget accelerator.  An accelerator is the bit-wise
+ * Returns the widget accelerator.  An accelerator is the bit-wise
  * OR of zero or more modifier masks and a key. Examples:
  * <code>SWT.CONTROL | SWT.SHIFT | 'T', SWT.ALT | SWT.F2</code>.
+ * The default value is zero, indicating that the menu item does
+ * not have an accelerator.
  *
- * @return the accelerator
+ * @return the accelerator or 0
  *
  * </ul>
  * @exception SWTException <ul>
@@ -269,24 +297,31 @@ public int getAccelerator () {
 	return accelerator;
 }
 
-int getAccelGroup () {
+int /*long*/ getAccelGroup () {
 	Menu menu = parent;
 	while (menu != null && menu.cascade != null) {
 		menu = menu.cascade.parent;
 	}
+	if (menu == null) return 0;
 	Decorations shell = menu.parent;
 	return shell.menuBar == menu ? shell.accelGroup : 0;
 }
 
-public Display getDisplay () {
-	Menu parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
+/*public*/ Rectangle getBounds () {
+	checkWidget();
+	if (!OS.GTK_WIDGET_MAPPED (handle)) {
+		return new Rectangle (0, 0, 0, 0);
+	}
+	int x = OS.GTK_WIDGET_X (handle);
+	int y = OS.GTK_WIDGET_Y (handle);
+	int width = OS.GTK_WIDGET_WIDTH (handle);
+	int height = OS.GTK_WIDGET_HEIGHT (handle);
+	return new Rectangle (x, y, width, height);
 }
 
 /**
  * Returns <code>true</code> if the receiver is enabled, and
- * <code>false</code> otherwise. A disabled control is typically
+ * <code>false</code> otherwise. A disabled menu item is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -301,7 +336,7 @@ public Display getDisplay () {
  */
 public boolean getEnabled () {
 	checkWidget();
-	return OS.GTK_WIDGET_SENSITIVE(handle);
+	return OS.GTK_WIDGET_SENSITIVE (handle);
 }
 
 /**
@@ -358,7 +393,7 @@ public boolean getSelection () {
 	return OS.gtk_check_menu_item_get_active(handle);
 }
 
-int gtk_activate (int widget) {
+int /*long*/ gtk_activate (int /*long*/ widget) {
 	if ((style & SWT.CASCADE) != 0 && menu != null) return 0;
 	/*
 	* Bug in GTK.  When an ancestor menu is disabled and
@@ -369,7 +404,7 @@ int gtk_activate (int widget) {
 	*/
 	if (!isEnabled ()) return 0;
 	Event event = new Event ();
-	int ptr = OS.gtk_get_current_event ();
+	int /*long*/ ptr = OS.gtk_get_current_event ();
 	if (ptr != 0) {
 		GdkEvent gdkEvent = new GdkEvent ();
 		OS.memmove (gdkEvent, ptr, GdkEvent.sizeof);
@@ -396,37 +431,37 @@ int gtk_activate (int widget) {
 	return 0;
 }
 
-int gtk_select (int item) {
+int /*long*/ gtk_select (int /*long*/ item) {
 	parent.selectedItem = this;
-	postEvent (SWT.Arm);
+	sendEvent (SWT.Arm);
 	return 0;
 }
 
-int gtk_show_help (int widget, int helpType) {
-	boolean hooks = hooks (SWT.Help);
-	if (hooks) {
+int /*long*/ gtk_show_help (int /*long*/ widget, int /*long*/ helpType) {
+	boolean handled = hooks (SWT.Help);
+	if (handled) {
 		postEvent (SWT.Help);
 	} else {
-		hooks = parent.sendHelpEvent (helpType);
+		handled = parent.sendHelpEvent (helpType);
+	}
+	if (handled) {
+		OS.gtk_menu_shell_deactivate (parent.handle);
+		return 1;
 	}
-	if (hooks) OS.gtk_menu_shell_deactivate (parent.handle);
 	return 0;
 }
 
 void hookEvents () {
 	super.hookEvents ();
-	Display display = getDisplay ();
-	int windowProc2 = display.windowProc2;
-	int windowProc3 = display.windowProc3;
-	OS.g_signal_connect (handle, OS.activate, windowProc2, ACTIVATE);
-	OS.g_signal_connect (handle, OS.select, windowProc2, SELECT);
-	OS.g_signal_connect (handle, OS.show_help, windowProc3, SHOW_HELP);
+	OS.g_signal_connect_closure (handle, OS.activate, display.closures [ACTIVATE], false);
+	OS.g_signal_connect_closure (handle, OS.select, display.closures [SELECT], false);
+	OS.g_signal_connect_closure_by_id (handle, display.signalIds [SHOW_HELP], 0, display.closures [SHOW_HELP], false);
 }
 
 /**
  * Returns <code>true</code> if the receiver is enabled and all
  * of the receiver's ancestors are enabled, and <code>false</code>
- * otherwise. A disabled control is typically not selectable from the
+ * otherwise. A disabled menu item is typically not selectable from the
  * user interface and draws with an inactive or "grayed" look.
  *
  * @return the receiver's enabled state
@@ -442,8 +477,16 @@ public boolean isEnabled () {
 	return getEnabled () && parent.isEnabled ();
 }
 
-void releaseChild () {
-	super.releaseChild ();
+void releaseChildren (boolean destroy) {
+	if (menu != null) {
+		menu.release (false);
+		menu = null;
+	}
+	super.releaseChildren (destroy);
+}
+
+void releaseParent () {
+	super.releaseParent ();
 	if (menu != null) {
 		if (menu.selectedItem == this) menu.selectedItem = null;
 		menu.dispose ();
@@ -452,20 +495,20 @@ void releaseChild () {
 }
 
 void releaseWidget () {
-	if (menu != null) menu.releaseResources ();
-	menu = null;
 	super.releaseWidget ();
-	int accelGroup = getAccelGroup ();
+	int /*long*/ accelGroup = getAccelGroup ();
 	if (accelGroup != 0) removeAccelerator (accelGroup);
+	if (groupHandle != 0) OS.g_object_unref (groupHandle);
+	groupHandle = 0;
 	accelerator = 0;
 	parent = null;
 }
 
-void removeAccelerator (int accelGroup) {
+void removeAccelerator (int /*long*/ accelGroup) {
 	updateAccelerator (accelGroup, false);
 }
 
-void removeAccelerators (int accelGroup) {
+void removeAccelerators (int /*long*/ accelGroup) {
 	removeAccelerator (accelGroup);
 	if (menu != null) menu.removeAccelerators (accelGroup);
 }
@@ -474,7 +517,7 @@ void removeAccelerators (int accelGroup) {
  * Removes the listener from the collection of listeners who will
  * be notified when the arm events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -498,7 +541,7 @@ public void removeArmListener (ArmListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the help events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -522,7 +565,7 @@ public void removeHelpListener (HelpListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -557,6 +600,8 @@ void selectRadio () {
  * OR of zero or more modifier masks and a key. Examples:
  * <code>SWT.MOD1 | SWT.MOD2 | 'T', SWT.MOD3 | SWT.F2</code>.
  * <code>SWT.CONTROL | SWT.SHIFT | 'T', SWT.ALT | SWT.F2</code>.
+ * The default value is zero, indicating that the menu item does
+ * not have an accelerator.
  *
  * @param accelerator an integer that is the bit-wise OR of masks and a key
  *
@@ -569,7 +614,7 @@ void selectRadio () {
 public void setAccelerator (int accelerator) {
 	checkWidget();
 	if (this.accelerator == accelerator) return;
-	int accelGroup = getAccelGroup ();
+	int /*long*/ accelGroup = getAccelGroup ();
 	if (accelGroup != 0) removeAccelerator (accelGroup);
 	this.accelerator = accelerator;
 	if (accelGroup != 0) addAccelerator (accelGroup);
@@ -577,7 +622,7 @@ public void setAccelerator (int accelerator) {
 
 /**
  * Enables the receiver if the argument is <code>true</code>,
- * and disables it otherwise. A disabled control is typically
+ * and disables it otherwise. A disabled menu item is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -590,16 +635,21 @@ public void setAccelerator (int accelerator) {
  */
 public void setEnabled (boolean enabled) {
 	checkWidget();
+	if (OS.GTK_WIDGET_SENSITIVE (handle) == enabled) return;
+	int /*long*/ accelGroup = getAccelGroup ();
+	if (accelGroup != 0) removeAccelerator (accelGroup);
 	OS.gtk_widget_set_sensitive (handle, enabled);
+	if (accelGroup != 0) addAccelerator (accelGroup);
 }
 
 /**
  * Sets the image the receiver will display to the argument.
  * <p>
- * Note: This feature is not available on all window systems (for example, Window NT),
- * in which case, calling this method will silently do nothing.
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept (for example, Windows NT).
+ * </p>
  *
- * @param menu the image to display
+ * @param image the image to display
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -610,9 +660,18 @@ public void setImage (Image image) {
 	checkWidget();
 	if ((style & SWT.SEPARATOR) != 0) return;
 	super.setImage (image);
-	if ((style & SWT.PUSH) == 0) return;
+	if (!OS.GTK_IS_IMAGE_MENU_ITEM (handle)) return;
 	if (image != null) {
-		int imageHandle = OS.gtk_image_new_from_pixmap (image.pixmap, image.mask);
+		ImageList imageList = parent.imageList;
+		if (imageList == null) imageList = parent.imageList = new ImageList ();
+		int imageIndex = imageList.indexOf (image);
+		if (imageIndex == -1) {
+			imageIndex = imageList.add (image);
+		} else {
+			imageList.put (imageIndex, image);
+		}
+		int /*long*/ pixbuf = imageList.getPixbuf (imageIndex);
+		int /*long*/ imageHandle = OS.gtk_image_new_from_pixbuf (pixbuf);
 		OS.gtk_image_menu_item_set_image (handle, imageHandle);
 		OS.gtk_widget_show (imageHandle);
 	} else {
@@ -626,6 +685,11 @@ public void setImage (Image image) {
  * pull down menu. The sequence of key strokes, button presses
  * and/or button releases that are used to request a pull down
  * menu is platform specific.
+ * <p>
+ * Note: Disposing of a menu item that has a pull down menu
+ * will dispose of the menu.  To avoid this behavior, set the
+ * menu to null before the menu item is disposed.
+ * </p>
  *
  * @param menu the new pull down menu
  *
@@ -659,7 +723,7 @@ public void setMenu (Menu menu) {
 	/* Assign the new menu */
 	Menu oldMenu = this.menu;
 	if (oldMenu == menu) return;
-	int accelGroup = getAccelGroup ();
+	int /*long*/ accelGroup = getAccelGroup ();
 	if (accelGroup != 0) removeAccelerators (accelGroup);
 	if (oldMenu != null) {
 		oldMenu.cascade = null;
@@ -677,6 +741,15 @@ public void setMenu (Menu menu) {
 	if (accelGroup != 0) addAccelerators (accelGroup);
 }
 
+void setOrientation() {
+	if ((parent.style & SWT.RIGHT_TO_LEFT) != 0) {
+		if (handle != 0) {
+			OS.gtk_widget_set_direction (handle, OS.GTK_TEXT_DIR_RTL);
+			OS.gtk_container_forall (handle, display.setDirectionProc, OS.GTK_TEXT_DIR_RTL);		
+		}
+	}
+}
+
 boolean setRadioSelection (boolean value) {
 	if ((style & SWT.RADIO) == 0) return false;
 	if (getSelection () != value) {
@@ -704,6 +777,7 @@ public void setSelection (boolean selected) {
 	if ((style & (SWT.CHECK | SWT.RADIO)) == 0) return;
 	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, ACTIVATE);
 	OS.gtk_check_menu_item_set_active (handle, selected);
+	if ((style & SWT.RADIO) != 0) OS.gtk_check_menu_item_set_active (groupHandle, !selected);
 	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, ACTIVATE);
 }
 
@@ -711,14 +785,14 @@ public void setSelection (boolean selected) {
  * Sets the receiver's text. The string may include
  * the mnemonic character and accelerator text.
  * <p>
- * Mnemonics are indicated by an '&amp' that causes the next
+ * Mnemonics are indicated by an '&amp;' that causes the next
  * character to be the mnemonic.  When the user presses a
  * key sequence that matches the mnemonic, a selection
  * event occurs. On most platforms, the mnemonic appears
  * underlined but may be emphasised in a platform specific
- * manner.  The mnemonic indicator character '&amp' can be
+ * manner.  The mnemonic indicator character '&amp;' can be
  * escaped by doubling it in the string, causing a single
- *'&amp' to be displayed.
+ * '&amp;' to be displayed.
  * </p>
  * <p>
  * Accelerator text is indicated by the '\t' character.
@@ -748,6 +822,7 @@ public void setText (String string) {
 	checkWidget();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if ((style & SWT.SEPARATOR) != 0) return;
+	if (text.equals (string)) return;
 	super.setText (string);
 	String accelString = "";
 	int index = string.indexOf ('\t');
@@ -756,19 +831,19 @@ public void setText (String string) {
 		string = string.substring (0, index);
 	}
 	char [] chars = fixMnemonic (string);
-	byte [] buffer = Converter.wcsToMbcs (null, chars, false);
-	int label = OS.gtk_bin_get_child (handle);
+	byte [] buffer = Converter.wcsToMbcs (null, chars, true);
+	int /*long*/ label = OS.gtk_bin_get_child (handle);
 	OS.gtk_label_set_text_with_mnemonic (label, buffer);
 	buffer = Converter.wcsToMbcs (null, accelString, true);
-	int ptr = OS.g_malloc (buffer.length);
+	int /*long*/ ptr = OS.g_malloc (buffer.length);
 	OS.memmove (ptr, buffer, buffer.length);
-	int oldPtr = OS.GTK_ACCEL_LABEL_ACCEL_STRING (label);
-	OS.GTK_ACCEL_LABEL_ACCEL_STRING (label, ptr);
+	int /*long*/ oldPtr = OS.GTK_ACCEL_LABEL_GET_ACCEL_STRING (label);
+	OS.GTK_ACCEL_LABEL_SET_ACCEL_STRING (label, ptr);
 	if (oldPtr != 0) OS.g_free (oldPtr);
 }
 
-void updateAccelerator (int accelGroup, boolean add) {
-	if (accelerator == 0) return;
+void updateAccelerator (int /*long*/ accelGroup, boolean add) {
+	if (accelerator == 0 || !getEnabled ()) return;
 	int mask = 0;
 	if ((accelerator & SWT.ALT) != 0) mask |= OS.GDK_MOD1_MASK;
 	if ((accelerator & SWT.SHIFT) != 0) mask |= OS.GDK_SHIFT_MASK;
@@ -780,13 +855,16 @@ void updateAccelerator (int accelGroup, boolean add) {
 	} else {
 		switch (keysym) {
 			case '\r': keysym = OS.GDK_Return; break;
-			default: keysym = wcsToMbcs ((char) keysym);
+			default: keysym = Display.wcsToMbcs ((char) keysym);
 		}
 	}
-	if (add) {
-		OS.gtk_widget_add_accelerator (handle, OS.activate, accelGroup, keysym, mask, OS.GTK_ACCEL_VISIBLE);
-	} else {
-		OS.gtk_widget_remove_accelerator (handle, accelGroup, keysym, mask);
+	/* When accel_key is zero, it causes GTK warnings */
+	if (keysym != 0) {
+		if (add) {
+			OS.gtk_widget_add_accelerator (handle, OS.activate, accelGroup, keysym, mask, OS.GTK_ACCEL_VISIBLE);
+		} else {
+			OS.gtk_widget_remove_accelerator (handle, accelGroup, keysym, mask);
+		}
 	}
 }
 }
