@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -21,7 +21,7 @@ import org.eclipse.swt.events.*;
  * objects that represent a range of positive, numeric values. 
  * <p>
  * At any given moment, a given scroll bar will have a 
- * single <em>selection</em> that is considered to be its
+ * single 'selection' that is considered to be its
  * value, which is constrained to be within the range of
  * values the scroll bar represents (that is, between its
  * <em>minimum</em> and <em>maximum</em> values).
@@ -81,6 +81,8 @@ import org.eclipse.swt.events.*;
  */
 public class ScrollBar extends Widget {
 	Scrollable parent;
+	int /*long*/ adjustmentHandle;
+	int detail;
 	
 ScrollBar () {
 }
@@ -101,7 +103,7 @@ ScrollBar (Scrollable parent, int style) {
  * interface.
  * <p>
  * When <code>widgetSelected</code> is called, the event object detail field contains one of the following values:
- * <code>0</code> - for the end of a drag.
+ * <code>SWT.NONE</code> - for the end of a drag.
  * <code>SWT.DRAG</code>.
  * <code>SWT.HOME</code>.
  * <code>SWT.END</code>.
@@ -129,7 +131,7 @@ ScrollBar (Scrollable parent, int style) {
 public void addSelectionListener (SelectionListener listener) {
 	checkWidget ();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
-	TypedListener typedListener = new TypedListener(listener);
+	TypedListener typedListener = new TypedListener (listener);
 	addListener (SWT.Selection,typedListener);
 	addListener (SWT.DefaultSelection,typedListener);
 }
@@ -138,10 +140,9 @@ static int checkStyle (int style) {
 	return checkBits (style, SWT.HORIZONTAL, SWT.VERTICAL, 0, 0, 0, 0);
 }
 
-public Display getDisplay () {
-	Scrollable parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
+void deregister () {
+	super.deregister ();
+	if (adjustmentHandle != 0) display.removeWidget (adjustmentHandle);
 }
 
 /**
@@ -161,13 +162,7 @@ public Display getDisplay () {
  */
 public boolean getEnabled () {
 	checkWidget ();
-	int barHandle;
-	if ((style & SWT.HORIZONTAL) != 0) {
-		barHandle = OS.GTK_SCROLLED_WINDOW_HSCROLLBAR (parent.scrolledHandle);
-	} else {
-		barHandle = OS.GTK_SCROLLED_WINDOW_VSCROLLBAR (parent.scrolledHandle);
-	}
-	if (barHandle != 0) return OS.GTK_WIDGET_SENSITIVE (barHandle);
+	if (handle != 0) return OS.GTK_WIDGET_SENSITIVE (handle);
 	return true;
 }
 
@@ -186,7 +181,7 @@ public boolean getEnabled () {
 public int getIncrement () {
 	checkWidget ();
 	GtkAdjustment adjustment = new GtkAdjustment ();
-	OS.memmove (adjustment, handle);
+	OS.memmove (adjustment, adjustmentHandle);
 	return (int) adjustment.step_increment;
 }
 
@@ -203,7 +198,7 @@ public int getIncrement () {
 public int getMaximum () {
 	checkWidget ();
 	GtkAdjustment adjustment = new GtkAdjustment ();
-	OS.memmove (adjustment, handle);
+	OS.memmove (adjustment, adjustmentHandle);
 	return (int) adjustment.upper;
 }
 
@@ -220,7 +215,7 @@ public int getMaximum () {
 public int getMinimum () {
 	checkWidget ();
 	GtkAdjustment adjustment = new GtkAdjustment ();
-	OS.memmove (adjustment, handle);
+	OS.memmove (adjustment, adjustmentHandle);
 	return (int) adjustment.lower;
 }
 
@@ -239,12 +234,12 @@ public int getMinimum () {
 public int getPageIncrement () {
 	checkWidget ();
 	GtkAdjustment adjustment = new GtkAdjustment ();
-	OS.memmove (adjustment, handle);
+	OS.memmove (adjustment, adjustmentHandle);
 	return (int) adjustment.page_increment;
 }
 
 /**
- * Returns the receiver's parent, which must be scrollable.
+ * Returns the receiver's parent, which must be a Scrollable.
  *
  * @return the receiver's parent
  *
@@ -259,7 +254,7 @@ public Scrollable getParent () {
 }
 
 /**
- * Returns the single <em>selection</em> that is the receiver's value.
+ * Returns the single 'selection' that is the receiver's value.
  *
  * @return the selection
  *
@@ -271,7 +266,7 @@ public Scrollable getParent () {
 public int getSelection () {
 	checkWidget ();
 	GtkAdjustment adjustment = new GtkAdjustment ();
-	OS.memmove (adjustment, handle);
+	OS.memmove (adjustment, adjustmentHandle);
 	return (int) adjustment.value;
 }
 
@@ -290,21 +285,14 @@ public int getSelection () {
  */
 public Point getSize () {
 	checkWidget ();
-	int barHandle = 0;
-	int scrolledHandle = parent.scrolledHandle;
-	if ((style & SWT.HORIZONTAL) != 0) {
-		barHandle = OS.GTK_SCROLLED_WINDOW_HSCROLLBAR (scrolledHandle);
-	} else {
-		barHandle = OS.GTK_SCROLLED_WINDOW_VSCROLLBAR (scrolledHandle);
-	}
-	if (barHandle == 0) return new Point (0,0);
+	if (handle == 0) return new Point (0,0);
 	GtkRequisition requisition = new GtkRequisition ();
-	OS.gtk_widget_size_request (barHandle, requisition);
+	OS.gtk_widget_size_request (handle, requisition);
 	return new Point (requisition.width, requisition.height);
 }
 
 /**
- * Answers the size of the receiver's thumb relative to the
+ * Returns the size of the receiver's thumb relative to the
  * difference between its maximum and minimum values.
  *
  * @return the thumb value
@@ -319,7 +307,7 @@ public Point getSize () {
 public int getThumb () {
 	checkWidget ();
 	GtkAdjustment adjustment = new GtkAdjustment ();
-	OS.memmove (adjustment, handle);
+	OS.memmove (adjustment, adjustmentHandle);
 	return (int) adjustment.page_size;
 }
 
@@ -342,7 +330,7 @@ public int getThumb () {
  */
 public boolean getVisible () {
 	checkWidget ();
-	int scrolledHandle = parent.scrolledHandle;
+	int /*long*/ scrolledHandle = parent.scrolledHandle;
 	int [] hsp = new int [1], vsp = new int [1];
 	OS.gtk_scrolled_window_get_policy (scrolledHandle, hsp, vsp);
 	if ((style & SWT.HORIZONTAL) != 0) {
@@ -352,15 +340,43 @@ public boolean getVisible () {
 	}
 }
 
-int gtk_value_changed (int adjustment) {
-	postEvent (SWT.Selection);
+int /*long*/ gtk_change_value (int /*long*/ widget, int /*long*/ scroll, int /*long*/ value1, int /*long*/ value2) {
+	detail = (int)/*64*/scroll;
+	return 0;
+}
+
+int /*long*/ gtk_value_changed (int /*long*/ adjustment) {
+	Event event = new Event ();
+	switch (detail) {
+		case OS.GTK_SCROLL_NONE:			event.detail = SWT.NONE; break;
+		case OS.GTK_SCROLL_JUMP:			event.detail = SWT.DRAG; break;
+		case OS.GTK_SCROLL_START:			event.detail = SWT.HOME; break;
+		case OS.GTK_SCROLL_END:				event.detail = SWT.END; break;
+		case OS.GTK_SCROLL_PAGE_DOWN:
+		case OS.GTK_SCROLL_PAGE_RIGHT:
+		case OS.GTK_SCROLL_PAGE_FORWARD:	event.detail = SWT.PAGE_DOWN; break;
+		case OS.GTK_SCROLL_PAGE_UP:
+		case OS.GTK_SCROLL_PAGE_LEFT:
+		case OS.GTK_SCROLL_PAGE_BACKWARD:	event.detail = SWT.PAGE_UP; break;
+		case OS.GTK_SCROLL_STEP_DOWN:
+		case OS.GTK_SCROLL_STEP_RIGHT:
+		case OS.GTK_SCROLL_STEP_FORWARD:	event.detail = SWT.ARROW_DOWN; break;
+		case OS.GTK_SCROLL_STEP_UP:
+		case OS.GTK_SCROLL_STEP_LEFT:
+		case OS.GTK_SCROLL_STEP_BACKWARD:	event.detail = SWT.ARROW_UP; break;
+	}
+	detail = OS.GTK_SCROLL_NONE;
+	postEvent (SWT.Selection, event);
+	parent.updateScrollBarValue (this);
 	return 0;
 }
 
 void hookEvents () {
 	super.hookEvents ();
-	Display display = getDisplay ();
-	OS.g_signal_connect (handle, OS.value_changed, display.windowProc2, VALUE_CHANGED);
+	if (OS.GTK_VERSION >= OS.VERSION (2, 6, 0)) {
+		OS.g_signal_connect_closure (handle, OS.change_value, display.closures [CHANGE_VALUE], false);
+	}
+	OS.g_signal_connect_closure (adjustmentHandle, OS.value_changed, display.closures [VALUE_CHANGED], false);
 }
 
 /**
@@ -402,8 +418,13 @@ public boolean isVisible () {
 	return getVisible () && getParent ().isVisible ();
 }
 
-void releaseChild () {
-	super.releaseChild ();
+void register () {
+	super.register ();
+	if (adjustmentHandle != 0) display.addWidget (adjustmentHandle, this);
+}
+
+void releaseParent () {
+	super.releaseParent ();
 	if (parent.horizontalBar == this) parent.horizontalBar = null;
 	if (parent.verticalBar == this) parent.verticalBar = null;
 }
@@ -453,13 +474,7 @@ public void removeSelectionListener (SelectionListener listener) {
  */
 public void setEnabled (boolean enabled) {
 	checkWidget ();
-	int barHandle;
-	if ((style & SWT.HORIZONTAL) != 0) {
-		barHandle = OS.GTK_SCROLLED_WINDOW_HSCROLLBAR (parent.scrolledHandle);
-	} else {
-		barHandle = OS.GTK_SCROLLED_WINDOW_VSCROLLBAR (parent.scrolledHandle);
-	}
-	if (barHandle != 0) OS.gtk_widget_set_sensitive (barHandle, enabled);
+	if (handle != 0) OS.gtk_widget_set_sensitive (handle, enabled);
 }
 
 /**
@@ -479,20 +494,21 @@ public void setIncrement (int value) {
 	checkWidget ();
 	if (value < 1) return;
 	GtkAdjustment adjustment = new GtkAdjustment ();
-	OS.memmove (adjustment, handle);
+	OS.memmove (adjustment, adjustmentHandle);
 	adjustment.step_increment = (float) value;
-	OS.memmove (handle, adjustment);
-	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_adjustment_changed (handle);
-	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.memmove (adjustmentHandle, adjustment);
+	OS.g_signal_handlers_block_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.gtk_adjustment_changed (adjustmentHandle);
+	OS.g_signal_handlers_unblock_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
 /**
- * Sets the maximum value which the receiver will allow
- * to be the argument which must be greater than or
- * equal to zero.
+ * Sets the maximum. If this value is negative or less than or
+ * equal to the minimum, the value is ignored. If necessary, first
+ * the thumb and then the selection are adjusted to fit within the
+ * new range.
  *
- * @param value the new maximum (must be zero or greater)
+ * @param value the new maximum
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -502,22 +518,25 @@ public void setIncrement (int value) {
 public void setMaximum (int value) {
 	checkWidget ();
 	GtkAdjustment adjustment = new GtkAdjustment ();
-	OS.memmove (adjustment, handle);
-	if (value <= adjustment.lower) return;
-	adjustment.upper = (float) value;
-	adjustment.value = Math.min (adjustment.value, value - adjustment.page_size);
-	OS.memmove (handle, adjustment);
-	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_adjustment_changed (handle);
-	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.memmove (adjustment, adjustmentHandle);
+	int minimum = (int) adjustment.lower;
+	if (value <= minimum) return;
+	adjustment.upper = value;
+	adjustment.page_size = Math.min ((int)adjustment.page_size, value - minimum);
+	adjustment.value = Math.min ((int)adjustment.value, (int)(value - adjustment.page_size));
+	OS.memmove (adjustmentHandle, adjustment);
+	OS.g_signal_handlers_block_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.gtk_adjustment_changed (adjustmentHandle);
+	OS.g_signal_handlers_unblock_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
-
+	
 /**
- * Sets the minimum value which the receiver will allow
- * to be the argument which must be greater than or
- * equal to zero.
+ * Sets the minimum value. If this value is negative or greater
+ * than or equal to the maximum, the value is ignored. If necessary,
+ * first the thumb and then the selection are adjusted to fit within
+ * the new range.
  *
- * @param value the new minimum (must be zero or greater)
+ * @param value the new minimum
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -528,14 +547,16 @@ public void setMinimum (int value) {
 	checkWidget ();
 	if (value < 0) return;
 	GtkAdjustment adjustment = new GtkAdjustment ();
-	OS.memmove (adjustment, handle);
-	if (value >= adjustment.upper) return;
-	adjustment.lower = (float) value;
-	adjustment.value = Math.max (adjustment.value, value);
-	OS.memmove (handle, adjustment);
-	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_adjustment_changed (handle);
-	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.memmove (adjustment, adjustmentHandle);
+	int maximum = (int) adjustment.upper;
+	if (value >= maximum) return;
+	adjustment.lower = value;
+	adjustment.page_size = Math.min ((int)adjustment.page_size, maximum - value);
+	adjustment.value = Math.max ((int)adjustment.value, value);
+	OS.memmove (adjustmentHandle, adjustment);
+	OS.g_signal_handlers_block_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.gtk_adjustment_changed (adjustmentHandle);
+	OS.g_signal_handlers_unblock_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
 /**
@@ -544,7 +565,7 @@ public void setMinimum (int value) {
  * are selected to the argument, which must be at least
  * one.
  *
- * @return the page increment (must be greater than zero)
+ * @param value the page increment (must be greater than zero)
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -555,12 +576,12 @@ public void setPageIncrement (int value) {
 	checkWidget ();
 	if (value < 1) return;
 	GtkAdjustment adjustment = new GtkAdjustment ();
-	OS.memmove (adjustment, handle);
+	OS.memmove (adjustment, adjustmentHandle);
 	adjustment.page_increment = (float) value;
-	OS.memmove (handle, adjustment);
-	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_adjustment_changed (handle);
-	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.memmove (adjustmentHandle, adjustment);
+	OS.g_signal_handlers_block_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.gtk_adjustment_changed (adjustmentHandle);
+	OS.g_signal_handlers_unblock_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
 /**
@@ -568,7 +589,7 @@ public void setPageIncrement (int value) {
  * value to the argument which must be greater than or equal
  * to zero.
  *
- * @param value the new selection (must be zero or greater)
+ * @param selection the new selection (must be zero or greater)
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -578,44 +599,44 @@ public void setPageIncrement (int value) {
 public void setSelection (int value) {
 	checkWidget ();
 	value = Math.min (value, getMaximum() - getThumb());
-	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_adjustment_set_value (handle, value);
-	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.g_signal_handlers_block_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.gtk_adjustment_set_value (adjustmentHandle, value);
+	OS.g_signal_handlers_unblock_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
 /**
  * Sets the size of the receiver's thumb relative to the
- * difference between its maximum and minimum values to the
- * argument which must be at least one.
+ * difference between its maximum and minimum values.  This new
+ * value will be ignored if it is less than one, and will be
+ * clamped if it exceeds the receiver's current range.
  *
- * @param value the new thumb value (must be at least one)
+ * @param value the new thumb value, which must be at least one and not
+ * larger than the size of the current range
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- *
- * @see ScrollBar
  */
 public void setThumb (int value) {
 	checkWidget ();
 	if (value < 1) return;
 	GtkAdjustment adjustment = new GtkAdjustment ();
-	OS.memmove (adjustment, handle);
-	if (value > adjustment.upper - adjustment.lower) return;
+	OS.memmove (adjustment, adjustmentHandle);
+	value = (int) Math.min (value, (int)(adjustment.upper - adjustment.lower)); 
 	adjustment.page_size = (double) value;
-	adjustment.value = Math.min (adjustment.value, adjustment.upper - value);
-	OS.memmove (handle, adjustment);
-	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_adjustment_changed (handle);
-	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	adjustment.value = Math.min ((int)adjustment.value, (int)(adjustment.upper - value));
+	OS.memmove (adjustmentHandle, adjustment);
+	OS.g_signal_handlers_block_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.gtk_adjustment_changed (adjustmentHandle);
+	OS.g_signal_handlers_unblock_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
 /**
  * Sets the receiver's selection, minimum value, maximum
  * value, thumb, increment and page increment all at once.
  * <p>
- * Note: This is equivalent to setting the values individually
+ * Note: This is similar to setting the values individually
  * using the appropriate methods, but may be implemented in a 
  * more efficient fashion on some platforms.
  * </p>
@@ -637,22 +658,22 @@ public void setValues (int selection, int minimum, int maximum, int thumb, int i
 	if (minimum < 0) return;
 	if (maximum < 0) return;
 	if (thumb < 1) return;
-	if (maximum - minimum - thumb < 0) return;
 	if (increment < 1) return;
 	if (pageIncrement < 1) return;
+	thumb = Math.min (thumb, maximum - minimum);
 	GtkAdjustment adjustment = new GtkAdjustment ();
-	OS.memmove (adjustment, handle);
+	OS.memmove (adjustment, adjustmentHandle);
 	adjustment.lower = minimum;
 	adjustment.upper = maximum;
 	adjustment.step_increment = increment;
 	adjustment.page_increment = pageIncrement;
 	adjustment.page_size = thumb;
 	adjustment.value = Math.min (Math.max (selection, minimum), maximum - thumb);
-	OS.memmove (handle, adjustment);
-	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_adjustment_changed (handle);
-	OS.gtk_adjustment_value_changed (handle);
-	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.memmove (adjustmentHandle, adjustment);
+	OS.g_signal_handlers_block_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
+	OS.gtk_adjustment_changed (adjustmentHandle);
+	OS.gtk_adjustment_value_changed (adjustmentHandle);
+	OS.g_signal_handlers_unblock_matched (adjustmentHandle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
 /**
@@ -673,24 +694,7 @@ public void setValues (int selection, int minimum, int maximum, int thumb, int i
  */
 public void setVisible (boolean visible) {
 	checkWidget ();
-	int scrolledHandle = parent.scrolledHandle;
-	int [] hsp = new int [1], vsp = new int [1];
-	OS.gtk_scrolled_window_get_policy (scrolledHandle, hsp, vsp);
-	int policy = visible ? OS.GTK_POLICY_ALWAYS : OS.GTK_POLICY_NEVER;
-	if ((style & SWT.HORIZONTAL) != 0) {
-		hsp [0] = policy;
-	} else {
-		vsp [0] = policy;
-	}
-	OS.gtk_scrolled_window_set_policy (scrolledHandle, hsp [0], vsp [0]);
-	
-	/*
-	* Force the container to allocate the size of its children.
-	*/
-	int parentHandle = parent.scrolledHandle;
-	OS.gtk_container_resize_children (parentHandle);
-
-	parent.sendEvent (SWT.Resize);
+	parent.setScrollBarVisible (this, visible);
 }
 
 }
