@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -52,9 +52,18 @@ import org.eclipse.swt.events.*;
  * of these changes are also possible.
  * </li>
  * </ul>
- * </p>
- * <p>
- * Note: The styles supported by this class must be treated
+ * </p><p>
+ * The <em>modality</em> of an instance may be specified using
+ * style bits. The modality style bits are used to determine
+ * whether input is blocked for other shells on the display.
+ * The <code>PRIMARY_MODAL</code> style allows an instance to block
+ * input to its parent. The <code>APPLICATION_MODAL</code> style
+ * allows an instance to block input to every other shell in the
+ * display. The <code>SYSTEM_MODAL</code> style allows an instance
+ * to block input to all shells, including shells belonging to
+ * different applications.
+ * </p><p>
+ * Note: The styles supported by this class are treated
  * as <em>HINT</em>s, since the window manager for the
  * desktop on which the instance is visible has ultimate
  * control over the appearance and behavior of decorations
@@ -65,6 +74,11 @@ import org.eclipse.swt.events.*;
  * more restrictive modality style that is supported. For
  * example, if <code>PRIMARY_MODAL</code> is not supported,
  * it would be upgraded to <code>APPLICATION_MODAL</code>.
+ * A modality style may also be "downgraded" to a less
+ * restrictive style. For example, most operating systems
+ * no longer support <code>SYSTEM_MODAL</code> because
+ * it can freeze up the desktop, so this is typically
+ * downgraded to <code>APPLICATION_MODAL</code>.
  * <dl>
  * <dt><b>Styles:</b></dt>
  * <dd>BORDER, CLOSE, MIN, MAX, NO_TRIM, RESIZE, TITLE, ON_TOP, TOOL</dd>
@@ -100,7 +114,7 @@ import org.eclipse.swt.events.*;
  * @see SWT
  */
 public class Shell extends Decorations {
-	int /*long*/ shellHandle, tooltipsHandle;
+	int /*long*/ shellHandle, tooltipsHandle, tooltipWindow;
 	boolean mapped, moved, resized, opened;
 	int oldX, oldY, oldWidth, oldHeight;
 	int minWidth, minHeight;
@@ -432,7 +446,21 @@ void bringToTop (boolean force) {
 			if (focusHandle != 0 && !OS.GTK_WIDGET_HAS_FOCUS (focusHandle)) return;
 		}
 	}
+	/*
+	* Bug in GTK.  When a shell that is not managed by the window
+	* manage is given focus, GTK gets stuck in "focus follows pointer"
+	* mode when the pointer is within the shell and its parent when
+	* the shell is hidden or disposed. The fix is to use XSetInputFocus()
+	* to assign focus when ever the active shell has not managed by
+	* the window manager.
+	* 
+	* NOTE: This bug is fixed in GTK+ 2.6.8 and above.
+	*/
+	boolean xFocus = false;
 	if (activeShell != null) {
+		if (OS.GTK_VERSION < OS.VERSION (2, 6, 8)) {
+			xFocus = activeShell.isUndecorated ();
+		}
 		display.activeShell = null;
 		display.activePending = true;
 	}
@@ -443,7 +471,7 @@ void bringToTop (boolean force) {
 	* the focus.
 	*/
 	int /*long*/ window = OS.GTK_WIDGET_WINDOW (shellHandle);
-	if ((style & SWT.ON_TOP) != 0 && OS.GDK_WINDOWING_X11 ()) {
+	if ((xFocus || (style & SWT.ON_TOP) != 0) && OS.GDK_WINDOWING_X11 ()) {
 		int /*long*/ xDisplay = OS.gdk_x11_drawable_get_xdisplay (window);
 		int /*long*/ xWindow = OS.gdk_x11_drawable_get_xid (window);
 		OS.gdk_error_trap_push ();
@@ -475,6 +503,10 @@ void checkOpen () {
 	if (!opened) resized = false;
 }
 
+int /*long*/ childStyle () {
+	return 0;
+}
+
 /**
  * Requests that the window manager close the receiver in
  * the same way it would be closed when the user clicks on
@@ -530,12 +562,11 @@ void createHandle (int index) {
 	} else {
 		shellHandle = OS.gtk_plug_new (handle);
 	}
-	if (shellHandle == 0) SWT.error (SWT.ERROR_NO_HANDLES);
+	if (shellHandle == 0) error (SWT.ERROR_NO_HANDLES);
 	if (parent != null) {
 		OS.gtk_window_set_transient_for (shellHandle, parent.topHandle ());
 		OS.gtk_window_set_destroy_with_parent (shellHandle, true);
-		int orientations = SWT.LEFT_TO_RIGHT | SWT.RIGHT_TO_LEFT;
-		if (!((style & ~orientations) == SWT.NONE || (style & (SWT.NO_TRIM | SWT.ON_TOP)) != 0)) {
+		if (!isUndecorated ()) {
 			OS.gtk_window_set_type_hint (shellHandle, OS.GDK_WINDOW_TYPE_HINT_DIALOG);
 		}
 	}
@@ -550,7 +581,11 @@ void createHandle (int index) {
 	} else {
 		OS.gtk_window_set_resizable (shellHandle, false);
 	}
-	createHandle (index, true);
+	vboxHandle = OS.gtk_vbox_new (false, 0);
+	if (vboxHandle == 0) error (SWT.ERROR_NO_HANDLES);
+	createHandle (index, false, true);
+	OS.gtk_container_add (vboxHandle, scrolledHandle);
+	OS.gtk_box_set_child_packing (vboxHandle, scrolledHandle, true, true, 0, OS.GTK_PACK_END);
 	OS.gtk_window_set_title (shellHandle, new byte [1]);
 	if ((style & (SWT.NO_TRIM | SWT.BORDER | SWT.RESIZE)) == 0) {
 		OS.gtk_container_set_border_width (shellHandle, 1);
@@ -563,6 +598,21 @@ void createHandle (int index) {
 	//TEMPORARY CODE
 	if ((style & SWT.ON_TOP) == 0) modal |= (parent != null && (parent.style & bits) != 0);
 	OS.gtk_window_set_modal (shellHandle, modal);
+	/*
+	* Feature in GTK.  Realizing the shell triggers a size allocate event,
+	* which may be confused for a resize event from the window manager if
+	* received too late.  The fix is to realize the window during creation
+	* to avoid confusion.
+	*/
+	OS.gtk_widget_realize (shellHandle);
+}
+
+Control findBackgroundControl () {
+	return (state & BACKGROUND) != 0 || backgroundImage != null ? this : null;
+}
+
+Composite findDeferredControl () {
+	return layoutCount > 0 ? this : null;
 }
 
 boolean hasBorder () {
@@ -571,19 +621,17 @@ boolean hasBorder () {
 
 void hookEvents () {
 	super.hookEvents ();
-	int /*long*/ shellMapProc = display.shellMapProc;
-	int /*long*/ windowProc3 = display.windowProc3;
-	OS.g_signal_connect (shellHandle, OS.map_event, windowProc3, MAP_EVENT);
-	OS.g_signal_connect (shellHandle, OS.unmap_event, windowProc3, UNMAP_EVENT);
-	OS.g_signal_connect (shellHandle, OS.window_state_event, windowProc3, WINDOW_STATE_EVENT);
-	OS.g_signal_connect (shellHandle, OS.size_allocate, windowProc3, SIZE_ALLOCATE);
-	OS.g_signal_connect (shellHandle, OS.configure_event, windowProc3, CONFIGURE_EVENT);
-	OS.g_signal_connect (shellHandle, OS.delete_event, windowProc3, DELETE_EVENT);
-	OS.g_signal_connect (shellHandle, OS.focus_in_event, windowProc3, FOCUS_IN_EVENT);
-	OS.g_signal_connect (shellHandle, OS.focus_out_event, windowProc3, FOCUS_OUT_EVENT);
-	OS.g_signal_connect (shellHandle, OS.map_event, shellMapProc, 0);
-	OS.g_signal_connect (shellHandle, OS.enter_notify_event, windowProc3, ENTER_NOTIFY_EVENT);
-	OS.g_signal_connect (shellHandle, OS.move_focus, windowProc3, MOVE_FOCUS);
+	OS.g_signal_connect_closure_by_id (shellHandle, display.signalIds [MAP_EVENT], 0, display.closures [MAP_EVENT], false);
+	OS.g_signal_connect_closure_by_id (shellHandle, display.signalIds [UNMAP_EVENT], 0, display.closures [UNMAP_EVENT], false);
+	OS.g_signal_connect_closure_by_id (shellHandle, display.signalIds [WINDOW_STATE_EVENT], 0, display.closures [WINDOW_STATE_EVENT], false);
+	OS.g_signal_connect_closure_by_id (shellHandle, display.signalIds [SIZE_ALLOCATE], 0, display.closures [SIZE_ALLOCATE], false);
+	OS.g_signal_connect_closure_by_id (shellHandle, display.signalIds [CONFIGURE_EVENT], 0, display.closures [CONFIGURE_EVENT], false);
+	OS.g_signal_connect_closure_by_id (shellHandle, display.signalIds [DELETE_EVENT], 0, display.closures [DELETE_EVENT], false);
+	OS.g_signal_connect_closure_by_id (shellHandle, display.signalIds [FOCUS_IN_EVENT], 0, display.closures [FOCUS_IN_EVENT], false);
+	OS.g_signal_connect_closure_by_id (shellHandle, display.signalIds [FOCUS_OUT_EVENT], 0, display.closures [FOCUS_OUT_EVENT], false);
+	OS.g_signal_connect_closure_by_id (shellHandle, display.signalIds [MAP_EVENT], 0, display.shellMapProcClosure, false);
+	OS.g_signal_connect_closure_by_id (shellHandle, display.signalIds [ENTER_NOTIFY_EVENT], 0, display.closures [ENTER_NOTIFY_EVENT], false);
+	OS.g_signal_connect_closure (shellHandle, OS.move_focus, display.closures [MOVE_FOCUS], false);
 }
 
 public boolean isEnabled () {
@@ -591,9 +639,10 @@ public boolean isEnabled () {
 	return getEnabled ();
 }
 
-public boolean isLayoutDeferred () {
-	checkWidget ();
-	return layoutCount > 0;
+boolean isUndecorated () {
+	return
+		(style & (SWT.SHELL_TRIM | SWT.BORDER)) == SWT.NONE ||
+		(style & (SWT.NO_TRIM | SWT.ON_TOP)) != 0;
 }
 
 public boolean isVisible () {
@@ -606,7 +655,7 @@ void register () {
 	display.addWidget (shellHandle, this);
 }
 
-void releaseChild () {
+void releaseParent () {
 	/* Do nothing */
 }
 
@@ -614,48 +663,52 @@ int /*long*/ topHandle () {
 	return shellHandle;
 }
 
-int /*long*/ filterProc (int /*long*/ xEvent, int /*long*/ gdkEvent, int /*long*/ data) {
-	/*
-	* Bug in GTK.  When a shell that has no window manager trimmings
-	* is given focus, GTK gets stuck in "focus follows pointer" mode when
-	* the pointer is within the shell and its parent when the shell is disposed.
-	* The fix is to modify the X events that cause this to happen.
-	*/
-	XFocusChangeEvent focusEvent = new XFocusChangeEvent ();
-	OS.memmove (focusEvent, xEvent, 4);
-	switch (focusEvent.type) {
-		case OS.FocusIn: {
-			OS.memmove (focusEvent, xEvent, XFocusChangeEvent.sizeof);
-			if (focusEvent.detail == OS.NotifyPointer) {
-				focusEvent.detail = OS.NotifyNonlinear;
-				OS.memmove (xEvent, focusEvent, XFocusChangeEvent.sizeof);
+void fixActiveShell () {
+	if (display.activeShell == this) {
+		Shell shell = null;
+		if (parent != null && parent.isVisible ()) shell = parent.getShell ();
+		if (shell == null && isUndecorated ()) {
+			Shell [] shells = display.getShells ();
+			for (int i = 0; i < shells.length; i++) {
+				if (shells [i] != null && shells [i].isVisible ()) {
+					shell = shells [i];
+					break;
+				}
 			}
-			break;
-		}
-		case OS.EnterNotify: {
-			XCrossingEvent crossingEvent = new XCrossingEvent ();
-			OS.memmove (crossingEvent, xEvent, XCrossingEvent.sizeof);
-			if (crossingEvent.focus) {
-				crossingEvent.focus = false;
-				OS.memmove (xEvent, crossingEvent, XCrossingEvent.sizeof);
-			}
-			break;
 		}
+		if (shell != null) shell.bringToTop (false);
 	}
-	return 0;
 }
 
 void fixShell (Shell newShell, Control control) {
 	if (this == newShell) return;
 	if (control == lastActive) setActiveControl (null);
-	if (tooltipsHandle != 0) {
-		setToolTipText (control.handle, null);
+	String toolTipText = control.toolTipText;
+	if (toolTipText != null) {
+		control.setToolTipText (this, null, toolTipText);
+		control.setToolTipText (newShell, toolTipText, null);
 	}
-	newShell.setToolTipText (control.handle, control.toolTipText);
+}
+
+void fixStyle (int /*long*/ handle) {
 }
 
 void forceResize () {
-	/* Do nothing.  Shell keeps the size of its children up to date. */
+	forceResize (OS.GTK_WIDGET_WIDTH (vboxHandle), OS.GTK_WIDGET_HEIGHT (vboxHandle));
+}
+
+void forceResize (int width, int height) {
+	int flags = OS.GTK_WIDGET_FLAGS (vboxHandle);
+	OS.GTK_WIDGET_SET_FLAGS (vboxHandle, OS.GTK_VISIBLE);
+	GtkRequisition requisition = new GtkRequisition ();
+	OS.gtk_widget_size_request (vboxHandle, requisition);
+	GtkAllocation allocation = new GtkAllocation ();
+	allocation.width = width;
+	allocation.height = height;
+	OS.gtk_widget_size_allocate (vboxHandle, allocation);
+	if ((flags & OS.GTK_VISIBLE) == 0) {
+		OS.GTK_WIDGET_UNSET_FLAGS (vboxHandle, OS.GTK_VISIBLE);	
+	}
 }
 
 public Point getLocation () {
@@ -689,12 +742,8 @@ public Point getMinimumSize () {
 
 public Point getSize () {
 	checkWidget ();
-	int width = OS.GTK_WIDGET_WIDTH (scrolledHandle);
-	int height = OS.GTK_WIDGET_HEIGHT (scrolledHandle);
-	if (menuBar != null)  {
-		int /*long*/ barHandle = menuBar.handle;
-		height += OS.GTK_WIDGET_HEIGHT (barHandle);
-	}
+	int width = OS.GTK_WIDGET_WIDTH (vboxHandle);
+	int height = OS.GTK_WIDGET_HEIGHT (vboxHandle);
 	return new Point (width + trimWidth (), height + trimHeight ());
 }
 
@@ -903,17 +952,6 @@ int /*long*/ gtk_realize (int /*long*/ widget) {
 	if ((style & SWT.ON_TOP) != 0) {
 		OS.gdk_window_set_override_redirect (window, true);
 	}
-	/*
-	* Bug in GTK.  When a shell that has no window manager trimmings
-	* is given focus, GTK gets stuck in "focus follows pointer" mode when
-	* the pointer is within the shell and its parent when the shell is disposed.
-	* The fix is to modify the X events that cause this to happen.
-	* 
-	* NOTE: This bug is fixed in GTK+ 2.6.8 and above.
-	*/
-	if (OS.GTK_VERSION < OS.VERSION (2, 6, 8)) {
-		OS.gdk_window_add_filter  (window, display.filterProc, shellHandle);
-	}
 	return result;
 }
 
@@ -1059,19 +1097,10 @@ void resizeBounds (int width, int height, boolean notify) {
 		OS.gdk_window_resize (enableWindow, width, height);
 	}
 	int border = OS.gtk_container_get_border_width (shellHandle);
-	int menuHeight = 0;
-	GtkRequisition requisition = new GtkRequisition ();
-	if (menuBar != null) {
-		int /*long*/ menuHandle = menuBar.handle;
-		OS.gtk_widget_size_request (menuHandle, requisition);
-		menuHeight = OS.GTK_WIDGET_REQUISITION_HEIGHT (menuHandle);
-		OS.gtk_widget_set_size_request (menuHandle, width - (border  * 2), menuHeight);
-		height = Math.max (1, height - menuHeight);
-	}
-	OS.gtk_fixed_move (fixedHandle, scrolledHandle, 0, menuHeight);
-	OS.gtk_widget_set_size_request (scrolledHandle, width - (border  * 2), height - (border  * 2));
-	OS.gtk_widget_size_request (fixedHandle, requisition);
-	OS.gtk_container_resize_children (fixedHandle);
+	int boxWidth = width - 2*border;
+	int boxHeight = height - 2*border;
+	OS.gtk_widget_set_size_request (vboxHandle, boxWidth, boxHeight);
+	forceResize (boxWidth, boxHeight);
 	if (notify) {
 		resized = true;
 		sendEvent (SWT.Resize);
@@ -1115,7 +1144,7 @@ int setBounds (int x, int y, int width, int height, boolean move, boolean resize
 	if (resize) {
 		width = Math.max (1, Math.max (minWidth, width - trimWidth ()));
 		height = Math.max (1, Math.max (minHeight, height - trimHeight ()));
-		OS.gtk_window_resize (shellHandle, width, height);
+		if ((style & SWT.RESIZE) != 0) OS.gtk_window_resize (shellHandle, width, height);
 		boolean changed = width != oldWidth || height != oldHeight;
 		if (changed) {
 			oldWidth = width;
@@ -1222,11 +1251,10 @@ void setInitialBounds () {
 	Rectangle rect = monitor.getClientArea ();
 	int width = rect.width * 5 / 8;
 	int height = rect.height * 5 / 8;
-	OS.gtk_widget_set_size_request (scrolledHandle, width, height);
-	OS.gtk_window_resize (shellHandle, width, height);
-	GtkRequisition requisition = new GtkRequisition ();
-	OS.gtk_widget_size_request (fixedHandle, requisition);
-	OS.gtk_container_resize_children (fixedHandle);
+	if ((style & SWT.RESIZE) != 0) {
+		OS.gtk_window_resize (shellHandle, width, height);
+	}
+	resizeBounds (width, height, false);
 }
 
 public void setMaximized (boolean maximized) {
@@ -1259,8 +1287,8 @@ public void setMenuBar (Menu menu) {
 		createAccelGroup ();
 		menuBar.addAccelerators (accelGroup);
 	}
-	int width = OS.GTK_WIDGET_WIDTH (shellHandle);
-	int height = OS.GTK_WIDGET_HEIGHT (shellHandle);
+	int width = OS.GTK_WIDGET_WIDTH (vboxHandle);
+	int height = OS.GTK_WIDGET_HEIGHT (vboxHandle);
 	resizeBounds (width, height, !both);
 }
 
@@ -1352,6 +1380,12 @@ public void setRegion (Region region) {
 	this.region = region;
 }
 
+/*
+ * Shells are never labelled by other widgets, so no initialization is needed.
+ */
+void setRelations() {
+}
+
 public void setText (String string) {
 	super.setText (string);
 
@@ -1388,32 +1422,34 @@ public void setVisible (boolean visible) {
 		* the shell not will be mapped until the parent is
 		* unminimized or shown on the desktop.
 		*/
-		mapped = false;
-		OS.gtk_widget_show (shellHandle);
-		if (isDisposed ()) return;
-		display.dispatchEvents = new int [] {
-			OS.GDK_EXPOSE,
-			OS.GDK_FOCUS_CHANGE,
-			OS.GDK_CONFIGURE,
-			OS.GDK_MAP,
-			OS.GDK_UNMAP,
-			OS.GDK_NO_EXPOSE,
-		};
-		Display display = this.display;
-		display.putGdkEvents();
-		boolean iconic = false;
-		Shell shell = parent != null ? parent.getShell() : null;
-		do {
-			OS.g_main_context_iteration (0, false);
-			if (isDisposed ()) break;
-			iconic = minimized || (shell != null && shell.minimized);
-		} while (!mapped && !iconic);
-		display.dispatchEvents = null;
-		if (isDisposed ()) return;
-		if (!iconic) {
-			update (true);
+		if (!OS.GTK_IS_PLUG (shellHandle)) {
+			mapped = false;
+			OS.gtk_widget_show (shellHandle);
 			if (isDisposed ()) return;
-			adjustTrim ();
+			display.dispatchEvents = new int [] {
+				OS.GDK_EXPOSE,
+				OS.GDK_FOCUS_CHANGE,
+				OS.GDK_CONFIGURE,
+				OS.GDK_MAP,
+				OS.GDK_UNMAP,
+				OS.GDK_NO_EXPOSE,
+			};
+			Display display = this.display;
+			display.putGdkEvents();
+			boolean iconic = false;
+			Shell shell = parent != null ? parent.getShell() : null;
+			do {
+				OS.g_main_context_iteration (0, false);
+				if (isDisposed ()) break;
+				iconic = minimized || (shell != null && shell.minimized);
+			} while (!mapped && !iconic);
+			display.dispatchEvents = null;
+			if (isDisposed ()) return;
+			if (!iconic) {
+				update (true, true);
+				if (isDisposed ()) return;
+				adjustTrim ();
+			}
 		}
 		mapped = true;
 
@@ -1442,14 +1478,23 @@ public void setVisible (boolean visible) {
 				updateLayout (false);
 			}
 		}
-	} else {	
+	} else {
+		fixActiveShell ();
 		OS.gtk_widget_hide (shellHandle);
 		sendEvent (SWT.Hide);
 	}
 }
 
-void setZOrder (Control sibling, boolean above) {
-	 setZOrder (sibling, above, false);
+void setZOrder (Control sibling, boolean above, boolean fixRelations) {
+	/*
+	* Bug in GTK+.  Changing the toplevel window Z-order causes
+	* X to send a resize event.  Before the shell is mapped, these
+	* resize events always have a size of 200x200, causing extra
+	* layout work to occur.  The fix is to modify the Z-order only
+	* if the shell has already been mapped at least once.
+	*/
+	/* Shells are never included in labelled-by relations */
+	if (mapped) setZOrder (sibling, above, false, false);
 }
 
 int /*long*/ shellMapProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/ user_data) {
@@ -1459,10 +1504,17 @@ int /*long*/ shellMapProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/
 }
 
 void showWidget () {
-	OS.gtk_container_add (shellHandle, fixedHandle);
+	OS.gtk_container_add (shellHandle, vboxHandle);
 	if (scrolledHandle != 0) OS.gtk_widget_show (scrolledHandle);
 	if (handle != 0) OS.gtk_widget_show (handle);
-	if (fixedHandle != 0) OS.gtk_widget_show (fixedHandle);
+	if (vboxHandle != 0) OS.gtk_widget_show (vboxHandle);
+}
+
+int /*long*/ sizeAllocateProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/ user_data) {
+	int [] x = new int [1], y = new int [1];
+	OS.gdk_window_get_pointer (0, x, y, null);
+	OS.gtk_window_move (handle, x [0], y [0] + 16);
+	return 0;
 }
 
 boolean traverseEscape () {
@@ -1514,21 +1566,8 @@ public void dispose () {
 	* more than once.  If this happens, fail silently.
 	*/
 	if (isDisposed()) return;
-
-	/*
-	* Feature in GTK.  When the active shell is disposed,
-	* GTK assigns focus temporarily to the root window
-	* unless it has previously been told to do otherwise.
-	* The fix is to make the parent be the active top level
-	* shell when the child shell is disposed.
-	*/
+	fixActiveShell ();
 	OS.gtk_widget_hide (shellHandle);
-	if (parent != null) {
-		if (display.activeShell == this) {
-			Shell shell = parent.getShell ();	
-			shell.bringToTop (false);
-		}
-	}
 	super.dispose ();
 }
 
@@ -1562,12 +1601,8 @@ public Rectangle getBounds () {
 	checkWidget ();
 	int [] x = new int [1], y = new int [1];
 	OS.gtk_window_get_position (shellHandle, x, y);
-	int width = OS.GTK_WIDGET_WIDTH (scrolledHandle);
-	int height = OS.GTK_WIDGET_HEIGHT (scrolledHandle);
-	if (menuBar != null)  {
-		int /*long*/ barHandle = menuBar.handle;
-		height += OS.GTK_WIDGET_HEIGHT (barHandle);
-	}
+	int width = OS.GTK_WIDGET_WIDTH (vboxHandle);
+	int height = OS.GTK_WIDGET_HEIGHT (vboxHandle);
 	return new Rectangle (x [0], y [0], width + trimWidth (), height + trimHeight ());
 }
 
@@ -1576,33 +1611,31 @@ void releaseHandle () {
 	shellHandle = 0;
 }
 
-void releaseShells () {
+void releaseChildren (boolean destroy) {
 	Shell [] shells = getShells ();
 	for (int i=0; i<shells.length; i++) {
 		Shell shell = shells [i];
-		if (!shell.isDisposed ()) shell.releaseResources ();
+		if (shell != null && !shell.isDisposed ()) {
+			shell.release (false);
+		}
 	}
+	super.releaseChildren (destroy);
 }
 
 void releaseWidget () {
-	releaseShells ();
-	destroyAccelGroup ();
 	super.releaseWidget ();
+	destroyAccelGroup ();
 	if (display.activeShell == this) display.activeShell = null;
 	if (tooltipsHandle != 0) OS.g_object_unref (tooltipsHandle);
 	tooltipsHandle = 0;
-	if (OS.GTK_VERSION < OS.VERSION (2, 6, 8) ) {
-		int /*long*/ window = OS.GTK_WIDGET_WINDOW (shellHandle);
-		OS.gdk_window_remove_filter  (window, display.filterProc, shellHandle);
-	}
 	region = null;
 	lastActive = null;
 }
 
-void setToolTipText (int /*long*/ widget, String string) {
+void setToolTipText (int /*long*/ widget, String newString, String oldString) {
 	byte [] buffer = null;
-	if (string != null && string.length () > 0) {
-		buffer = Converter.wcsToMbcs (null, string, true);
+	if (newString != null && newString.length () > 0) {
+		buffer = Converter.wcsToMbcs (null, newString, true);
 	}
 	if (tooltipsHandle == 0) {
 		tooltipsHandle = OS.gtk_tooltips_new ();
@@ -1610,6 +1643,49 @@ void setToolTipText (int /*long*/ widget, String string) {
 		OS.g_object_ref (tooltipsHandle);
 		OS.gtk_object_sink (tooltipsHandle);
 	}
+
+	/*
+	* Feature in GTK.  There is no API to position a tooltip.
+	* The fix is to connect to the size_allocate signal for
+	* the tooltip window and position it before it is mapped.
+	*
+	* Bug in Solaris-GTK.  Invoking gtk_tooltips_force_window()
+	* can cause a crash in older versions of GTK.  The fix is
+	* to avoid this call if the GTK version is older than 2.2.x.
+	*/
+	if (OS.GTK_VERSION >= OS.VERSION (2, 2, 1)) { 
+		OS.gtk_tooltips_force_window (tooltipsHandle);
+	}
+	int /*long*/ tipWindow = OS.GTK_TOOLTIPS_TIP_WINDOW (tooltipsHandle);
+	if (tipWindow != 0 && tipWindow != tooltipWindow) {
+		OS.g_signal_connect (tipWindow, OS.size_allocate, display.sizeAllocateProc, shellHandle);
+		tooltipWindow = tipWindow;
+	}
 	OS.gtk_tooltips_set_tip (tooltipsHandle, widget, buffer, null);
+	
+	/*
+	* Bug in GTK.  If the cursor is inside the window when a new
+	* tooltip is set and the old tooltip is null, the new tooltip
+	* is not displayed until the mouse enters the window.  The
+	* fix is to cause and enter/leave event to happen by creating
+	* a temporary INPUT_ONLY GDK window.
+	*/
+	if ((OS.GTK_WIDGET_FLAGS (widget) & OS.GTK_REALIZED) == 0) return;
+	if ((OS.GTK_WIDGET_FLAGS (widget) & OS.GTK_VISIBLE) == 0) return;
+	if (oldString == null || oldString.length () == 0) {
+		if (newString != null && newString.length () != 0) {
+			int[] x = new int [1], y = new int [1];
+			int /*long*/ window = OS.gdk_window_at_pointer (x, y);
+			if (window != 0) {
+				int /*long*/ [] user_data = new int /*long*/ [1];
+				OS.gdk_window_get_user_data (window, user_data);
+				if (widget == user_data [0]) {
+					int /*long*/ data = OS.gtk_tooltips_data_get (widget);
+					OS.GTK_TOOLTIPS_SET_ACTIVE (tooltipsHandle, data);
+					OS.gtk_tooltips_set_tip (tooltipsHandle, widget, buffer, null);
+				}
+			}
+		}
+	}
 }
 }
