@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -21,7 +21,7 @@ import org.eclipse.swt.events.*;
  * objects that represent a range of positive, numeric values. 
  * <p>
  * At any given moment, a given slider will have a 
- * single <em>selection</em> that is considered to be its
+ * single 'selection' that is considered to be its
  * value, which is constrained to be within the range of
  * values the slider represents (that is, between its
  * <em>minimum</em> and <em>maximum</em> values).
@@ -65,6 +65,7 @@ import org.eclipse.swt.events.*;
  * @see ScrollBar
  */
 public class Slider extends Control {
+	int detail;
 
 /**
  * Constructs a new instance of this class given its parent
@@ -106,7 +107,7 @@ public Slider (Composite parent, int style) {
  * interface.
  * <p>
  * When <code>widgetSelected</code> is called, the event object detail field contains one of the following values:
- * <code>0</code> - for the end of a drag.
+ * <code>SWT.NONE</code> - for the end of a drag.
  * <code>SWT.DRAG</code>.
  * <code>SWT.HOME</code>.
  * <code>SWT.END</code>.
@@ -145,10 +146,10 @@ static int checkStyle (int style) {
 
 void createHandle (int index) {
 	state |= HANDLE;
-	fixedHandle = OS.gtk_fixed_new ();
+	fixedHandle = OS.g_object_new (display.gtk_fixed_get_type (), 0);
 	if (fixedHandle == 0) error (SWT.ERROR_NO_HANDLES);
 	OS.gtk_fixed_set_has_window (fixedHandle, true);
-	int hAdjustment = OS.gtk_adjustment_new (0, 0, 100, 1, 10, 10);
+	int /*long*/ hAdjustment = OS.gtk_adjustment_new (0, 0, 100, 1, 10, 10);
 	if (hAdjustment == 0) error (SWT.ERROR_NO_HANDLES);
 	if ((style & SWT.HORIZONTAL) != 0) {
 		handle = OS.gtk_hscrollbar_new (hAdjustment);
@@ -156,38 +157,64 @@ void createHandle (int index) {
 		handle = OS.gtk_vscrollbar_new (hAdjustment);
 	}
 	if (handle == 0) error (SWT.ERROR_NO_HANDLES);
-	int parentHandle = parent.parentingHandle ();
-	OS.gtk_container_add (parentHandle, fixedHandle);
+	OS.GTK_WIDGET_SET_FLAGS (handle, OS.GTK_CAN_FOCUS);
 	OS.gtk_container_add (fixedHandle, handle);
-	OS.gtk_widget_show (fixedHandle);
-	OS.gtk_widget_show (handle);
 }
 
-int gtk_value_changed (int adjustment) {
-	postEvent (SWT.Selection);
+int /*long*/ gtk_change_value (int /*long*/ widget, int /*long*/ scroll, int /*long*/ value1, int /*long*/ value2) {
+	detail = (int)/*64*/scroll;
+	return 0;
+}
+
+int /*long*/ gtk_value_changed (int /*long*/ adjustment) {
+	Event event = new Event ();
+	switch (detail) {
+		case OS.GTK_SCROLL_NONE:			event.detail = SWT.NONE; break;
+		case OS.GTK_SCROLL_JUMP:			event.detail = SWT.DRAG; break;
+		case OS.GTK_SCROLL_START:			event.detail = SWT.HOME; break;
+		case OS.GTK_SCROLL_END:				event.detail = SWT.END; break;
+		case OS.GTK_SCROLL_PAGE_DOWN:
+		case OS.GTK_SCROLL_PAGE_RIGHT:
+		case OS.GTK_SCROLL_PAGE_FORWARD:	event.detail = SWT.PAGE_DOWN; break;
+		case OS.GTK_SCROLL_PAGE_UP:
+		case OS.GTK_SCROLL_PAGE_LEFT:
+		case OS.GTK_SCROLL_PAGE_BACKWARD:	event.detail = SWT.PAGE_UP; break;
+		case OS.GTK_SCROLL_STEP_DOWN:
+		case OS.GTK_SCROLL_STEP_RIGHT:
+		case OS.GTK_SCROLL_STEP_FORWARD:	event.detail = SWT.ARROW_DOWN; break;
+		case OS.GTK_SCROLL_STEP_UP:
+		case OS.GTK_SCROLL_STEP_LEFT:
+		case OS.GTK_SCROLL_STEP_BACKWARD:	event.detail = SWT.ARROW_UP; break;
+	}
+	detail = OS.GTK_SCROLL_NONE;
+	postEvent (SWT.Selection, event);
 	return 0;
 }
 
 void hookEvents () {
 	super.hookEvents ();
-	Display display = getDisplay ();
-	OS.g_signal_connect (handle, OS.value_changed, display.windowProc2, VALUE_CHANGED);
+	if (OS.GTK_VERSION >= OS.VERSION (2, 6, 0)) {
+		OS.g_signal_connect_closure (handle, OS.change_value, display.closures [CHANGE_VALUE], false);
+	}
+	OS.g_signal_connect_closure (handle, OS.value_changed, display.closures [VALUE_CHANGED], false);
 }
 
 void register () {
 	super.register ();
-	int hAdjustment = OS.gtk_range_get_adjustment (handle);
-	WidgetTable.put (hAdjustment, this);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	display.addWidget (hAdjustment, this);
 }
 
 void deregister () {
 	super.deregister ();
-	int hAdjustment = OS.gtk_range_get_adjustment (handle);
-	WidgetTable.remove (hAdjustment);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	display.removeWidget (hAdjustment);
 }
 
 public Point computeSize (int wHint, int hHint, boolean changed) {
 	checkWidget();
+	if (wHint != SWT.DEFAULT && wHint < 0) wHint = 0;
+	if (hHint != SWT.DEFAULT && hHint < 0) hHint = 0;
 	Point size = computeNativeSize(handle, wHint, hHint, changed);
 	if ((style & SWT.HORIZONTAL) != 0) {
 		if (wHint == SWT.DEFAULT) size.x = 2 * size.x;
@@ -211,7 +238,7 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
  */
 public int getIncrement () {
 	checkWidget ();
-	int hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	return (int) adjustment.step_increment;
@@ -229,7 +256,7 @@ public int getIncrement () {
  */
 public int getMaximum () {
 	checkWidget ();
-	int hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	return (int) adjustment.upper;
@@ -247,7 +274,7 @@ public int getMaximum () {
  */
 public int getMinimum () {
 	checkWidget ();
-	int hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	return (int) adjustment.lower;
@@ -267,14 +294,14 @@ public int getMinimum () {
  */
 public int getPageIncrement () {
 	checkWidget ();
-	int hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	return (int) adjustment.page_increment;
 }
 
 /**
- * Returns the single <em>selection</em> that is the receiver's value.
+ * Returns the 'selection', which is the receiver's value.
  *
  * @return the selection
  *
@@ -285,7 +312,7 @@ public int getPageIncrement () {
  */
 public int getSelection () {
 	checkWidget ();
-	int hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	return (int) adjustment.value;
@@ -304,7 +331,7 @@ public int getSelection () {
  */
 public int getThumb () {
 	checkWidget ();
-	int hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	return (int) adjustment.page_size;
@@ -357,11 +384,12 @@ public void setIncrement (int value) {
 }
 
 /**
- * Sets the maximum value which the receiver will allow
- * to be the argument which must be greater than or
- * equal to zero.
+ * Sets the maximum. If this value is negative or less than or
+ * equal to the minimum, the value is ignored. If necessary, first
+ * the thumb and then the selection are adjusted to fit within the
+ * new range.
  *
- * @param value the new maximum (must be zero or greater)
+ * @param value the new maximum, which must be greater than the current minimum
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -370,19 +398,27 @@ public void setIncrement (int value) {
  */
 public void setMaximum (int value) {
 	checkWidget ();
-	int minimum = getMinimum ();
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	GtkAdjustment adjustment = new GtkAdjustment ();
+	OS.memmove (adjustment, hAdjustment);
+	int minimum = (int) adjustment.lower;
 	if (value <= minimum) return;
+	adjustment.upper = value;
+	adjustment.page_size = Math.min ((int)adjustment.page_size, value - minimum);
+	adjustment.value = Math.min ((int)adjustment.value, (int)(value - adjustment.page_size));
+	OS.memmove (hAdjustment, adjustment);
 	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_range_set_range (handle, minimum, value);
+	OS.gtk_adjustment_changed (hAdjustment);
 	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
 /**
- * Sets the minimum value which the receiver will allow
- * to be the argument which must be greater than or
- * equal to zero.
+ * Sets the minimum value. If this value is negative or greater
+ * than or equal to the maximum, the value is ignored. If necessary,
+ * first the thumb and then the selection are adjusted to fit within
+ * the new range.
  *
- * @param value the new minimum (must be zero or greater)
+ * @param value the new minimum
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -392,20 +428,36 @@ public void setMaximum (int value) {
 public void setMinimum (int value) {
 	checkWidget ();
 	if (value < 0) return;
-	int maximum = getMaximum ();
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
+	GtkAdjustment adjustment = new GtkAdjustment ();
+	OS.memmove (adjustment, hAdjustment);
+	int maximum = (int) adjustment.upper;
 	if (value >= maximum) return;
+	adjustment.lower = value;
+	adjustment.page_size = Math.min ((int)adjustment.page_size, maximum - value);
+	adjustment.value = Math.max ((int)adjustment.value, value);
+	OS.memmove (hAdjustment, adjustment);
 	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
-	OS.gtk_range_set_range (handle, value, maximum);
+	OS.gtk_adjustment_changed (hAdjustment);
 	OS.g_signal_handlers_unblock_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 }
 
+void setOrientation () {
+	super.setOrientation ();
+	if ((style & SWT.RIGHT_TO_LEFT) != 0) {
+		if ((style & SWT.HORIZONTAL) != 0) {
+			OS.gtk_range_set_inverted (handle, true);
+		}
+	}
+}
+
 /**
  * Sets the amount that the receiver's value will be
  * modified by when the page increment/decrement areas
  * are selected to the argument, which must be at least
  * one.
  *
- * @return the page increment (must be greater than zero)
+ * @param value the page increment (must be greater than zero)
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -421,8 +473,8 @@ public void setPageIncrement (int value) {
 }
 
 /**
- * Sets the single <em>selection</em> that is the receiver's
- * value to the argument which must be greater than or equal
+ * Sets the 'selection', which is the receiver's
+ * value, to the argument which must be greater than or equal
  * to zero.
  *
  * @param value the new selection (must be zero or greater)
@@ -441,27 +493,27 @@ public void setSelection (int value) {
 
 /**
  * Sets the size of the receiver's thumb relative to the
- * difference between its maximum and minimum values to the
- * argument which must be at least one.
+ * difference between its maximum and minimum values.  This new
+ * value will be ignored if it is less than one, and will be
+ * clamped if it exceeds the receiver's current range.
  *
- * @param value the new thumb value (must be at least one)
+ * @param value the new thumb value, which must be at least one and not
+ * larger than the size of the current range
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
- *
- * @see ScrollBar
  */
 public void setThumb (int value) {
 	checkWidget ();
 	if (value < 1) return;
-	int hAdjustment = OS.gtk_range_get_adjustment (handle);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
-	if (value > adjustment.upper - adjustment.lower) return;
+	value = (int) Math.min (value, (int)(adjustment.upper - adjustment.lower));
 	adjustment.page_size = (double) value;
-	adjustment.value = Math.min (adjustment.value, adjustment.upper - value);
+	adjustment.value = Math.min ((int)adjustment.value, (int)(adjustment.upper - value));
 	OS.memmove (hAdjustment, adjustment);
 	OS.g_signal_handlers_block_matched (handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, VALUE_CHANGED);
 	OS.gtk_adjustment_changed (hAdjustment);
@@ -472,7 +524,7 @@ public void setThumb (int value) {
  * Sets the receiver's selection, minimum value, maximum
  * value, thumb, increment and page increment all at once.
  * <p>
- * Note: This is equivalent to setting the values individually
+ * Note: This is similar to setting the values individually
  * using the appropriate methods, but may be implemented in a 
  * more efficient fashion on some platforms.
  * </p>
@@ -494,10 +546,10 @@ public void setValues (int selection, int minimum, int maximum, int thumb, int i
 	if (minimum < 0) return;
 	if (maximum < 0) return;
 	if (thumb < 1) return;
-	if (maximum - minimum - thumb < 0) return;
 	if (increment < 1) return;
 	if (pageIncrement < 1) return;
-	int hAdjustment = OS.gtk_range_get_adjustment (handle);
+	thumb = Math.min (thumb, maximum - minimum);
+	int /*long*/ hAdjustment = OS.gtk_range_get_adjustment (handle);
 	GtkAdjustment adjustment = new GtkAdjustment ();
 	OS.memmove (adjustment, hAdjustment);
 	adjustment.value = Math.min (Math.max (selection, minimum), maximum - thumb);
