@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -19,22 +19,24 @@ import org.eclipse.swt.events.*;
 
 /**
  * Instances of this class represent a column in a table widget.
- *  <dl>
+ * <p><dl>
  * <dt><b>Styles:</b></dt>
  * <dd>LEFT, RIGHT, CENTER</dd>
  * <dt><b>Events:</b></dt>
  * <dd> Move, Resize, Selection</dd>
  * </dl>
- * <p>
+ * </p><p>
  * Note: Only one of the styles LEFT, RIGHT and CENTER may be specified.
  * </p><p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
  * </p>
  */
 public class TableColumn extends Item {
-	int boxHandle, labelHandle, imageHandle;
+	int /*long*/ labelHandle, imageHandle, buttonHandle;
 	Table parent;
-	int modelIndex, lastButton, lastTime;
+	int modelIndex, lastButton, lastTime, lastX, lastWidth;
+	boolean customDraw, useFixedWidth;
+	String toolTipText;
 
 /**
  * Constructs a new instance of this class given its parent
@@ -91,10 +93,11 @@ public TableColumn (Table parent, int style) {
  *
  * @param parent a composite control which will be the parent of the new instance (cannot be null)
  * @param style the style of control to construct
- * @param index the index to store the receiver in its parent
+ * @param index the zero-relative index to store the receiver in its parent
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the parent (inclusive)</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -176,8 +179,13 @@ static int checkStyle (int style) {
 	return checkBits (style, SWT.LEFT, SWT.CENTER, SWT.RIGHT, 0, 0, 0);
 }
 
+protected void checkSubclass () {
+	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
+}
+
 void createWidget (int index) {
 	parent.createItem (this, index);
+	setOrientation ();
 	hookEvents ();
 	register ();
 	text = "";
@@ -185,7 +193,14 @@ void createWidget (int index) {
 
 void deregister() {
 	super.deregister ();
-	WidgetTable.remove (handle);
+	display.removeWidget (handle);
+	if (buttonHandle != 0) display.removeWidget (buttonHandle);
+	if (labelHandle != 0) display.removeWidget (labelHandle);
+}
+
+void destroyWidget () {
+	parent.destroyItem (this);
+	releaseHandle ();
 }
 
 /**
@@ -208,10 +223,29 @@ public int getAlignment () {
 	return SWT.LEFT;
 }
 
-public Display getDisplay () {
-	Table parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
+/**
+ * Gets the moveable attribute. A column that is
+ * not moveable cannot be reordered by the user 
+ * by dragging the header but may be reordered 
+ * by the programmer.
+ *
+ * @return the moveable attribute
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @see Table#getColumnOrder()
+ * @see Table#setColumnOrder(int[])
+ * @see TableColumn#setMoveable(boolean)
+ * @see SWT#Move
+ * 
+ * @since 3.1
+ */
+public boolean getMoveable() {
+	checkWidget();
+	return OS.gtk_tree_view_column_get_reorderable (handle);
 }
 
 /**
@@ -246,6 +280,24 @@ public boolean getResizable () {
 	return OS.gtk_tree_view_column_get_resizable (handle);
 }
 
+/**
+ * Returns the receiver's tool tip text, or null if it has
+ * not been set.
+ *
+ * @return the receiver's tool tip text
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.2
+ */
+public String getToolTipText () {
+	checkWidget();
+	return toolTipText;
+}
+
 /**
  * Gets the width of the receiver.
  *
@@ -258,10 +310,14 @@ public boolean getResizable () {
  */
 public int getWidth () {
 	checkWidget();
+	if (!OS.gtk_tree_view_column_get_visible (handle)) {
+		return 0;
+	}
+	if (useFixedWidth) return OS.gtk_tree_view_column_get_fixed_width (handle);
 	return OS.gtk_tree_view_column_get_width (handle);
 }
 
-int gtk_clicked (int widget) {
+int /*long*/ gtk_clicked (int /*long*/ widget) {
 	/*
 	* There is no API to get a double click on a table column.  Normally, when
 	* the mouse is double clicked, this is indicated by GDK_2BUTTON_PRESS
@@ -270,13 +326,12 @@ int gtk_clicked (int widget) {
 	* and testing for the double click interval.
 	*/
 	boolean doubleClick = false;
-	int eventPtr = OS.gtk_get_current_event ();
+	int /*long*/ eventPtr = OS.gtk_get_current_event ();
 	if (eventPtr != 0) {
 		GdkEventButton gdkEvent = new GdkEventButton ();
 		OS.memmove (gdkEvent, eventPtr, GdkEventButton.sizeof);
 		switch (gdkEvent.type) {
 			case OS.GDK_BUTTON_RELEASE: {
-				Display display = getDisplay ();
 				int clickTime = display.getDoubleClickTime ();
 				int eventTime = gdkEvent.time, eventButton = gdkEvent.button;
 				if (lastButton == eventButton && lastTime != 0 && Math.abs (lastTime - eventTime) <= clickTime) {
@@ -292,10 +347,30 @@ int gtk_clicked (int widget) {
 	return 0;
 }
 
+int /*long*/ gtk_mnemonic_activate (int /*long*/ widget, int /*long*/ arg1) {
+	return parent.gtk_mnemonic_activate (widget, arg1);
+}
+
+int /*long*/ gtk_size_allocate (int /*long*/ widget, int /*long*/ allocation) {
+	useFixedWidth = false;
+	int x = OS.GTK_WIDGET_X (widget);
+	int width = OS.GTK_WIDGET_WIDTH (widget);
+	if (x != lastX) {
+		lastX = x;
+		sendEvent (SWT.Move);
+	}
+	if (width != lastWidth) {
+		lastWidth = width;
+		sendEvent (SWT.Resize);
+	}
+	return 0;
+}
+
 void hookEvents () {
 	super.hookEvents ();
-	Display display = getDisplay ();
-	OS.g_signal_connect (handle, OS.clicked, display.windowProc2, CLICKED);
+	OS.g_signal_connect_closure (handle, OS.clicked, display.closures [CLICKED], false);
+	if (buttonHandle != 0) OS.g_signal_connect_closure_by_id (buttonHandle, display.signalIds [SIZE_ALLOCATE], 0, display.closures [SIZE_ALLOCATE], false);
+	if (labelHandle != 0) OS.g_signal_connect_closure_by_id (labelHandle, display.signalIds [MNEMONIC_ACTIVATE], 0, display.closures [MNEMONIC_ACTIVATE], false);
 }
 
 /**
@@ -311,36 +386,57 @@ void hookEvents () {
  */
 public void pack () {
 	checkWidget();
-	/*
-	* Feature in GTK.  Calling gtk_tree_view_column_set_sizing() with 
-	* GTK_TREE_VIEW_COLUMN_AUTOSIZE causes the tree view
-	* column to be not resizable.  The fix is to reset this afterawards.
-	*/ 
-	boolean resizable = OS.gtk_tree_view_column_get_resizable (handle);
-	OS.gtk_tree_view_column_set_sizing (handle, OS.GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-	OS.gtk_tree_view_column_set_resizable (handle, resizable);
+	int width = 0;
+	if (buttonHandle != 0) {
+		GtkRequisition requisition = new GtkRequisition ();
+		OS.gtk_widget_size_request (buttonHandle, requisition);
+		width = requisition.width;
+	}
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		for (int i=0; i<parent.items.length; i++) {
+			TableItem item = parent.items [i];
+			if (item != null && item.cached) {
+				width = Math.max (width, parent.calculateWidth (handle, item.handle));
+			}
+		}
+	} else {
+		int /*long*/ iter = OS.g_malloc (OS.GtkTreeIter_sizeof ());
+		if (OS.gtk_tree_model_get_iter_first (parent.modelHandle, iter)) {
+			do {
+				width = Math.max (width, parent.calculateWidth (handle, iter));
+			} while (OS.gtk_tree_model_iter_next(parent.modelHandle, iter));
+		}
+		OS.g_free (iter);
+	}
+	setWidth(width);
 }
 
 void register () {
 	super.register ();
-	WidgetTable.put (handle, this);
+	display.addWidget (handle, this);
+	if (buttonHandle != 0) display.addWidget (buttonHandle, this);
+	if (labelHandle != 0) display.addWidget (labelHandle, this);
 }
 
-void releaseChild () {
-	super.releaseChild ();
-	parent.destroyItem (this);
-}
-
-void releaseWidget () {
-	super.releaseWidget ();
+void releaseHandle () {
+	super.releaseHandle ();
+	handle = buttonHandle = labelHandle = imageHandle = 0;
+	modelIndex = -1; 
 	parent = null;
 }
 
+void releaseParent () {
+	super.releaseParent ();
+	if (parent.sortColumn == this) {
+		parent.sortColumn = null;
+	}
+}
+
 /**
  * Removes the listener from the collection of listeners who will
  * be notified when the control is moved or resized.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -365,7 +461,7 @@ public void removeControlListener (ControlListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -401,36 +497,43 @@ public void removeSelectionListener(SelectionListener listener) {
 public void setAlignment (int alignment) {
 	checkWidget();
 	if ((alignment & (SWT.LEFT | SWT.RIGHT | SWT.CENTER)) == 0) return;
-	//TODO - there is OS support
+	int index = parent.indexOf (this);
+	if (index == -1 || index == 0) return;
+	style &= ~(SWT.LEFT | SWT.RIGHT | SWT.CENTER);
+	style |= alignment & (SWT.LEFT | SWT.RIGHT | SWT.CENTER);
+	parent.createRenderers (handle, modelIndex, index == 0, style);
+}
+
+void setFontDescription (int /*long*/ font) {
+	OS.gtk_widget_modify_font (labelHandle, font);
+	OS.gtk_widget_modify_font (imageHandle, font);
 }
 
 public void setImage (Image image) {
 	checkWidget ();
 	super.setImage (image);
 	if (image != null) {
-		OS.gtk_image_set_from_pixmap (imageHandle, image.pixmap, image.mask);
+		ImageList headerImageList = parent.headerImageList;
+		if (headerImageList == null) {
+			headerImageList = parent.headerImageList = new ImageList ();
+		}
+		int imageIndex = headerImageList.indexOf (image);
+		if (imageIndex == -1) imageIndex = headerImageList.add (image);
+		int /*long*/ pixbuf = headerImageList.getPixbuf (imageIndex);
+		OS.gtk_image_set_from_pixbuf (imageHandle, pixbuf);
 		OS.gtk_widget_show (imageHandle);
 	} else {
-		OS.gtk_image_set_from_pixmap (imageHandle, 0, 0);
+		OS.gtk_image_set_from_pixbuf (imageHandle, 0);
 		OS.gtk_widget_hide (imageHandle);
 	}
-
-	/*
-	* Bug in GTK.  For some reason, the column button does not allocate the size 
-	* of its internal children if its bounds is set before the image is set.  The fix is to
-	* force this by calling gtk_widget_size_request() (and throw the results away).
-	*/
-	int parentHandle = OS.gtk_widget_get_parent (boxHandle);
-	if (parentHandle != 0) {
-		GtkRequisition requisition = new GtkRequisition ();
-		OS.gtk_widget_size_request (parentHandle, requisition);
-	}
 }
 
 /**
  * Sets the resizable attribute.  A column that is
- * not resizable cannot be dragged by the user but
- * may be resized by the programmer.
+ * resizable can be resized by the user dragging the
+ * edge of the header.  A column that is not resizable 
+ * cannot be dragged by the user but may be resized 
+ * by the programmer.
  *
  * @param resizable the resize attribute
  *
@@ -444,29 +547,77 @@ public void setResizable (boolean resizable) {
 	OS.gtk_tree_view_column_set_resizable (handle, resizable);
 }
 
+/**
+ * Sets the moveable attribute.  A column that is
+ * moveable can be reordered by the user by dragging
+ * the header. A column that is not moveable cannot be 
+ * dragged by the user but may be reordered 
+ * by the programmer.
+ *
+ * @param moveable the moveable attribute
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @see Table#setColumnOrder(int[])
+ * @see Table#getColumnOrder()
+ * @see TableColumn#getMoveable()
+ * @see SWT#Move
+ * 
+ * @since 3.1
+ */
+public void setMoveable (boolean moveable) {
+	checkWidget();
+	OS.gtk_tree_view_column_set_reorderable (handle, moveable);
+}
+
+void setOrientation() {
+	if ((parent.style & SWT.RIGHT_TO_LEFT) != 0) {
+		if (buttonHandle != 0) {
+			OS.gtk_widget_set_direction (buttonHandle, OS.GTK_TEXT_DIR_RTL);
+			OS.gtk_container_forall (buttonHandle, display.setDirectionProc, OS.GTK_TEXT_DIR_RTL);	
+		}
+	}
+}
+
 public void setText (String string) {
 	checkWidget();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	super.setText (string);
 	char [] chars = fixMnemonic (string);
-	byte [] buffer = Converter.wcsToMbcs (null, chars, false);
+	byte [] buffer = Converter.wcsToMbcs (null, chars, true);
 	OS.gtk_label_set_text_with_mnemonic (labelHandle, buffer);
 	if (string.length () != 0) {
 		OS.gtk_widget_show (labelHandle);
 	} else {
 		OS.gtk_widget_hide (labelHandle);
 	}
+}
 
-	/*
-	* Bug in GTK.  For some reason, the column button does not allocate the size
-	* of its internal children if its bounds is set before the text is set.  The fix is to 
-	* force this by calling gtk_widget_size_request() (and throw the results away).
-	*/
-	int parentHandle = OS.gtk_widget_get_parent (boxHandle);
-	if (parentHandle != 0) {
-		GtkRequisition requisition = new GtkRequisition ();
-		OS.gtk_widget_size_request (parentHandle, requisition);
-	}
+/**
+ * Sets the receiver's tool tip text to the argument, which
+ * may be null indicating that no tool tip text should be shown.
+ *
+ * @param string the new tool tip text (or null)
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.2
+ */
+public void setToolTipText (String string) {
+	checkWidget();
+	Shell shell = parent._getShell ();
+	setToolTipText (shell, string, toolTipText);
+	toolTipText = string;
+}
+
+void setToolTipText (Shell shell, String newString, String oldString) {
+	shell.setToolTipText (buttonHandle, newString, oldString);
 }
 
 /**
@@ -481,8 +632,27 @@ public void setText (String string) {
  */
 public void setWidth (int width) {
 	checkWidget();
-	OS.gtk_tree_view_column_set_sizing (handle, OS.GTK_TREE_VIEW_COLUMN_FIXED);
-	OS.gtk_tree_view_column_set_fixed_width (handle, width);
+	if (width > 0) {
+		useFixedWidth = true;
+		/*
+		* Bug in GTK.  For some reason, calling gtk_tree_view_column_set_visible()
+		* when the parent is not realized fails to show the column. The fix is to
+		* ensure that the table has been realized.
+		*/
+		OS.gtk_widget_realize (parent.handle);
+		boolean sendResize = false;
+		if (!OS.gtk_tree_view_column_get_visible (handle)) {
+			sendResize = OS.gtk_tree_view_column_get_fixed_width (handle) == width;
+		}
+		OS.gtk_tree_view_column_set_fixed_width (handle, width);
+		OS.gtk_tree_view_column_set_visible (handle, true);
+		if (sendResize) sendEvent (SWT.Resize);
+	} else {
+		if (OS.gtk_tree_view_column_get_visible (handle)) {
+			OS.gtk_tree_view_column_set_visible (handle, false);
+			sendEvent (SWT.Resize);
+		}
+	}
 }
 
 }
