@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -68,9 +68,7 @@ public class TreeItem extends Item {
  * @see Widget#getStyle
  */
 public TreeItem (Tree parent, int style) {
-	super (parent, style);
-	this.parent = parent;
-	parent.createItem (this, 0, -1);
+	this (checkNull (parent), 0, style, -1, true);
 }
 
 /**
@@ -90,10 +88,11 @@ public TreeItem (Tree parent, int style) {
  *
  * @param parent a tree control which will be the parent of the new instance (cannot be null)
  * @param style the style of control to construct
- * @param index the index to store the receiver in its parent
+ * @param index the zero-relative index to store the receiver in its parent
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the parent (inclusive)</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -105,10 +104,7 @@ public TreeItem (Tree parent, int style) {
  * @see Widget#getStyle
  */
 public TreeItem (Tree parent, int style, int index) {
-	super (parent, style);
-	if (index < 0) error (SWT.ERROR_INVALID_RANGE);
-	this.parent = parent;
-	parent.createItem (this, 0, index);
+	this (checkNull (parent), 0, style, checkIndex (index), true);
 }
 
 /**
@@ -142,9 +138,7 @@ public TreeItem (Tree parent, int style, int index) {
  * @see Widget#getStyle
  */
 public TreeItem (TreeItem parentItem, int style) {
-	super (checkNull (parentItem).parent, style);
-	this.parent = parentItem.parent;
-	parent.createItem (this, parentItem.handle, -1);
+	this (checkNull (parentItem).parent, parentItem.handle, style, -1, true);
 }
 
 /**
@@ -164,10 +158,11 @@ public TreeItem (TreeItem parentItem, int style) {
  *
  * @param parentItem a tree control which will be the parent of the new instance (cannot be null)
  * @param style the style of control to construct
- * @param index the index to store the receiver in its parent
+ * @param index the zero-relative index to store the receiver in its parent
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the parent (inclusive)</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -179,10 +174,23 @@ public TreeItem (TreeItem parentItem, int style) {
  * @see Widget#getStyle
  */
 public TreeItem (TreeItem parentItem, int style, int index) {
-	super (checkNull (parentItem).parent, style);
-	if (index < 0) error (SWT.ERROR_ITEM_NOT_ADDED);
-	this.parent = parentItem.parent;
-	parent.createItem (this, parentItem.handle, index);
+	this (checkNull (parentItem).parent, parentItem.handle, style, checkIndex (index), true);
+}
+
+TreeItem (Tree parent, int /*long*/ parentIter, int style, int index, boolean create) {
+	super (parent, style);
+	this.parent = parent;
+	if (create) {
+		parent.createItem (this, parentIter, index);
+	} else {
+		handle = OS.g_malloc (OS.GtkTreeIter_sizeof ());
+		OS.gtk_tree_model_iter_nth_child (parent.modelHandle, handle, parentIter, index);
+	}
+}
+
+static int checkIndex (int index) {
+	if (index < 0) SWT.error (SWT.ERROR_INVALID_RANGE);
+	return index;
 }
 
 static TreeItem checkNull (TreeItem item) {
@@ -190,10 +198,96 @@ static TreeItem checkNull (TreeItem item) {
 	return item;
 }
 
+static Tree checkNull (Tree control) {
+	if (control == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
+	return control;
+}
+
 protected void checkSubclass () {
 	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
 }
 
+void clear () {
+	if (parent.currentItem == this) return;
+	if (cached || (parent.style & SWT.VIRTUAL) == 0) {
+		int columnCount = OS.gtk_tree_model_get_n_columns (parent.modelHandle);
+		for (int i=Tree.CHECKED_COLUMN; i<columnCount; i++) {
+			OS.gtk_tree_store_set (parent.modelHandle, handle, i, 0, -1);
+		}
+		/*
+		* Bug in GTK.  When using fixed-height-mode,
+		* row changes do not cause the row to be repainted.  The fix is to
+		* invalidate the row when it is cleared.
+		*/
+		if ((parent.style & SWT.VIRTUAL) != 0) {
+			if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+				redraw ();
+			}
+		}
+	}
+	cached = false;
+	font = null;
+	cellFont = null;
+}
+
+/**
+ * Clears the item at the given zero-relative index in the receiver.
+ * The text, icon and other attributes of the item are set to the default
+ * value.  If the tree was created with the <code>SWT.VIRTUAL</code> style,
+ * these attributes are requested again as needed.
+ *
+ * @param index the index of the item to clear
+ * @param all <code>true</code> if all child items of the indexed item should be
+ * cleared recursively, and <code>false</code> otherwise
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the list minus 1 (inclusive)</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @see SWT#VIRTUAL
+ * @see SWT#SetData
+ * 
+ * @since 3.2
+ */
+public void clear (int index, boolean all) {
+	checkWidget ();
+	parent.clear (handle, index, all);
+}
+
+/**
+ * Clears all the items in the receiver. The text, icon and other
+ * attributes of the items are set to their default values. If the
+ * tree was created with the <code>SWT.VIRTUAL</code> style, these
+ * attributes are requested again as needed.
+ * 
+ * @param all <code>true</code> if all child items should be cleared
+ * recursively, and <code>false</code> otherwise
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @see SWT#VIRTUAL
+ * @see SWT#SetData
+ * 
+ * @since 3.2
+ */
+public void clearAll (boolean all) {
+	checkWidget ();
+	parent.clearAll (all, handle);
+}
+
+void destroyWidget () {
+	parent.releaseItem (this, false);
+	parent.destroyItem (this);
+	releaseHandle ();
+}
+
 /**
  * Returns the receiver's background color.
  *
@@ -264,7 +358,12 @@ public Rectangle getBounds (int index) {
 	checkWidget();
 	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
 	int /*long*/ parentHandle = parent.handle;
-	int /*long*/ column = OS.gtk_tree_view_get_column (parentHandle, index);
+	int /*long*/ column = 0;
+	if (index >= 0 && index < parent.columnCount) {
+		column = parent.columns [index].handle;
+	} else {
+		column = OS.gtk_tree_view_get_column (parentHandle, index);
+	}
 	if (column == 0) return new Rectangle (0, 0, 0, 0);
 	int /*long*/ path = OS.gtk_tree_model_get_path (parent.modelHandle, handle);
 	OS.gtk_widget_realize (parentHandle);
@@ -297,23 +396,7 @@ public Rectangle getBounds (int index) {
 			rect.width -= w [0]  + buffer [0];
 		}
 	}
-	
-	/* 
-	 * In the horizontal direction, the origin of the bin window is 
-	 * not the same as the origin of the scrolled handle.
-	 * The method gtk_tree_view_get_cell_area returns the 
-	 * x coordinate relative to the bin window.  In order to
-	 * get the coordinates relative to the top left corner
-	 * of the client area,  we need to account for the
-	 * horizontal scroll adjustment.
-	 */
-	int[] wx = new int[1];
-	OS.gtk_tree_view_tree_to_widget_coords(parentHandle, rect.x, 0, wx, null);
-	rect.x = wx[0];
-	
-	int border = parent.getBorderWidth ();
-	int headerHeight = parent.getHeaderHeight ();
-	return new Rectangle (rect.x + (index == 0 ? 0 : border), rect.y + (headerHeight == 0 ?  border : headerHeight), rect.width + 1, rect.height + 1);
+	return new Rectangle (rect.x, rect.y, rect.width + 1, rect.height + 1);
 }
 
 /**
@@ -349,9 +432,12 @@ public Rectangle getBounds () {
 	GdkRectangle rect = new GdkRectangle ();
 	OS.gtk_tree_view_get_cell_area (parentHandle, path, column, rect);
 	OS.gtk_tree_path_free (path);
+	int right = rect.x + rect.width;
 
 	int [] x = new int [1], w = new int [1];
+	parent.ignoreSize = true;
 	OS.gtk_cell_renderer_get_size (textRenderer, parentHandle, null, null, null, w, null);
+	parent.ignoreSize = false;
 	rect.width = w [0];
 	int [] buffer = new int [1];
 	if (OS.gtk_tree_view_get_expander_column (parentHandle) == column) {
@@ -373,23 +459,12 @@ public Rectangle getBounds () {
 		OS.gtk_cell_renderer_get_size (pixbufRenderer, parentHandle, null, null, null, w, null);
 		rect.x += w [0] + horizontalSeparator;
 	}
-	
-	/* 
-	 * In the horizontal direction, the origin of the bin window is 
-	 * not the same as the origin of the scrolled handle.
-	 * The method gtk_tree_view_get_cell_area returns the 
-	 * x coordinate relative to the bin window.  In order to
-	 * get the coordinates relative to the top left corner
-	 * of the client area,  we need to account for the
-	 * horizontal scroll adjustment.
-	 */
-	int[] wx = new int[1];
-	OS.gtk_tree_view_tree_to_widget_coords(parentHandle, rect.x, 0, wx, null);
-	rect.x = wx[0];
-	
-	int border = parent.getBorderWidth ();
-	int headerHeight = parent.getHeaderHeight ();
-	return new Rectangle (rect.x + border, rect.y + (headerHeight == 0 ?  border : headerHeight), rect.width + 1, rect.height + 1);
+	if (parent.columnCount > 0) {
+		if (rect.x + rect.width > right) {
+			rect.width = Math.max (0, right - rect.x);
+		}
+	}
+	return new Rectangle (rect.x, rect.y, rect.width + 1, rect.height + 1);
 }
 
 /**
@@ -534,15 +609,15 @@ public Color getForeground (int index) {
  * the <code>CHECK style, return false.
  * <p>
  *
- * @return the grayed state
+ * @return the grayed state of the checkbox
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  */
-public boolean getGrayed() {
-	checkWidget();
+public boolean getGrayed () {
+	checkWidget ();
 	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
 	if ((parent.style & SWT.CHECK) == 0) return false;
 	return grayed;
@@ -571,10 +646,8 @@ public Image getImage () {
 public Image getImage (int index) {
 	checkWidget ();
 	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
-	if (!(0 <= index && index <= parent.columnCount )) return null;
-	int /*long*/ parentHandle = parent.handle;
-	int /*long*/ column = OS.gtk_tree_view_get_column (parentHandle, index);
-	if (column == 0) return null;
+	int count = Math.max (1, parent.getColumnCount ());
+	if (0 > index || index > count - 1) return null;
 	int /*long*/ [] ptr = new int /*long*/ [1];
 	int modelIndex = parent.columnCount == 0 ? Tree.FIRST_COLUMN : parent.columns [index].modelIndex;
 	OS.gtk_tree_model_get (parent.modelHandle, handle, modelIndex + Tree.CELL_PIXBUF, ptr, -1);
@@ -588,7 +661,7 @@ public Image getImage (int index) {
 /**
  * Returns a rectangle describing the size and location
  * relative to its parent of an image at a column in the
- * table.
+ * tree.
  *
  * @param index the index that specifies the column
  * @return the receiver's bounding image rectangle
@@ -605,9 +678,14 @@ public Rectangle getImageBounds (int index) {
 	checkWidget ();
 	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
 	int /*long*/ parentHandle = parent.handle;
-	int /*long*/ column = OS.gtk_tree_view_get_column (parentHandle, index);
+	int /*long*/ column = 0;
+	if (index >= 0 && index < parent.getColumnCount ()) {
+		column = parent.columns [index].handle;
+	} else {
+		column = OS.gtk_tree_view_get_column (parentHandle, index);
+	}
 	if (column == 0) return new Rectangle (0, 0, 0, 0);
-	int /*long*/ pixbufRenderer = parent.getPixbufRenderer (column);	
+	int /*long*/ pixbufRenderer = parent.getPixbufRenderer (column);
 	if (pixbufRenderer == 0)  return new Rectangle (0, 0, 0, 0);
 	GdkRectangle rect = new GdkRectangle ();
 	int /*long*/ path = OS.gtk_tree_model_get_path (parent.modelHandle, handle);
@@ -645,23 +723,7 @@ public Rectangle getImageBounds (int index) {
 		OS.gtk_cell_renderer_get_size (pixbufRenderer, parentHandle, null, null, null, w, null);
 		rect.width = w [0];
 	}
-	
-	/* 
-	 * In the horizontal direction, the origin of the bin window is 
-	 * not the same as the origin of the scrolled handle.
-	 * The method gtk_tree_view_get_cell_area returns the 
-	 * x coordinate relative to the bin window.  In order to
-	 * get the coordinates relative to the top left corner
-	 * of the client area,  we need to account for the
-	 * horizontal scroll adjustment.
-	 */
-	int[] wx = new int[1];
-	OS.gtk_tree_view_tree_to_widget_coords(parentHandle, rect.x, 0, wx, null);
-	rect.x = wx[0];
-
-	int border = parent.getBorderWidth ();
-	int headerHeight = parent.getHeaderHeight ();
-	return new Rectangle (rect.x + (index == 0 ? 0 : border), rect.y + (headerHeight == 0 ?  border : headerHeight), rect.width, rect.height + 1);
+	return new Rectangle (rect.x, rect.y, rect.width, rect.height + 1);
 }
 
 /**
@@ -677,6 +739,7 @@ public Rectangle getImageBounds (int index) {
  */
 public int getItemCount () {
 	checkWidget();
+	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
 	return OS.gtk_tree_model_iter_n_children (parent.modelHandle, handle);
 }
 
@@ -699,15 +762,11 @@ public int getItemCount () {
  */
 public TreeItem getItem (int index) {
 	checkWidget();
-	int length = OS.gtk_tree_model_iter_n_children (parent.modelHandle, handle);
-	if (!(0 <= index && index < length)) error (SWT.ERROR_INVALID_RANGE);
-	int /*long*/ iter = OS.g_malloc (OS.GtkTreeIter_sizeof ());
-	OS.gtk_tree_model_iter_nth_child (parent.modelHandle, iter, handle, index);
-	int [] value = new int [1];
-	OS.gtk_tree_model_get (parent.modelHandle, iter, Tree.ID_COLUMN, value, -1);
-	TreeItem item = parent.items [value [0]];
-	OS.g_free (iter);
-	return item;
+	if (index < 0) error (SWT.ERROR_INVALID_RANGE);
+	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
+	int itemCount = OS.gtk_tree_model_iter_n_children (parent.modelHandle, handle);
+	if (index >= itemCount)  error (SWT.ERROR_INVALID_RANGE);
+	return  parent._getItem (handle, index);
 }
 
 /**
@@ -728,9 +787,17 @@ public TreeItem getItem (int index) {
  */
 public TreeItem [] getItems () {
 	checkWidget();
+	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
 	return parent.getItems (handle);
 }
 
+String getNameText () {
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (!cached) return "*virtual*"; //$NON-NLS-1$
+	}
+	return super.getNameText ();
+}
+
 /**
  * Returns the receiver's parent, which must be a <code>Tree</code>.
  *
@@ -742,7 +809,7 @@ public TreeItem [] getItems () {
  * </ul>
  */
 public Tree getParent () {
-	checkWidget();
+	checkWidget ();
 	return parent;
 }
 
@@ -762,13 +829,12 @@ public TreeItem getParentItem () {
 	checkWidget();
 	int /*long*/ path = OS.gtk_tree_model_get_path (parent.modelHandle, handle);
 	TreeItem item = null;
-	if (OS.gtk_tree_path_get_depth (path) > 1) {
+	int depth = OS.gtk_tree_path_get_depth (path); 
+	if (depth > 1) {
 		OS.gtk_tree_path_up (path);
 		int /*long*/ iter = OS.g_malloc (OS.GtkTreeIter_sizeof ());
 		if (OS.gtk_tree_model_get_iter (parent.modelHandle, iter, path)) {
-			int [] index = new int [1];
-			OS.gtk_tree_model_get (parent.modelHandle, iter, Tree.ID_COLUMN, index, -1);
-			item = parent.items [index [0]];
+			item = parent._getItem (iter);
 		}
 		OS.g_free (iter);
 	}
@@ -799,10 +865,8 @@ public String getText () {
 public String getText (int index) {
 	checkWidget ();
 	if (!parent.checkData (this)) error (SWT.ERROR_WIDGET_DISPOSED);
-	if (!(0 <= index && index <= parent.columnCount )) return ""; //$NON-NLS-1$
-	int /*long*/ parentHandle = parent.handle;
-	int /*long*/ column = OS.gtk_tree_view_get_column (parentHandle, index);
-	if (column == 0) error(SWT.ERROR_CANNOT_GET_TEXT);
+	int count = Math.max (1, parent.getColumnCount ());
+	if (0 > index || index > count - 1) return "";
 	int /*long*/ [] ptr = new int /*long*/ [1];
 	int modelIndex = parent.columnCount == 0 ? Tree.FIRST_COLUMN : parent.columns [index].modelIndex;
 	OS.gtk_tree_model_get (parent.modelHandle, handle, modelIndex + Tree.CELL_TEXT, ptr, -1);
@@ -862,16 +926,38 @@ public int indexOf (TreeItem item) {
 	return index;
 }
 
-void releaseChild () {
-	super.releaseChild ();
-	parent.destroyItem (this);
+void redraw () {
+	int /*long*/ parentHandle = parent.handle;
+	if ((OS.GTK_WIDGET_FLAGS (parentHandle) & OS.GTK_REALIZED) != 0) {
+		int /*long*/ path = OS.gtk_tree_model_get_path (parent.modelHandle, handle);
+		GdkRectangle rect = new GdkRectangle ();
+		OS.gtk_tree_view_get_cell_area (parentHandle, path, 0, rect);
+		OS.gtk_tree_path_free (path);
+		int /*long*/ window = OS.gtk_tree_view_get_bin_window (parentHandle);
+		rect.x = 0;
+		int [] w = new int [1], h = new int [1];
+		OS.gdk_drawable_get_size (window, w, h);
+		rect.width = w [0];
+		OS.gdk_window_invalidate_rect (window, rect, false);
+	}
+}
+
+void releaseChildren (boolean destroy) {
+	if (destroy) {
+		parent.releaseItems (handle);
+	}
+	super.releaseChildren (destroy);
+}
+
+void releaseHandle () {
+	if (handle != 0) OS.g_free (handle);
+	handle = 0;
+	super.releaseHandle ();
+	parent = null;
 }
 
 void releaseWidget () {
 	super.releaseWidget ();
-	if (handle != 0) OS.g_free (handle);
-	handle = 0;
-	parent = null;
 	font = null;
 	cellFont = null;
 }
@@ -894,9 +980,11 @@ public void removeAll () {
 	int [] index = new int [1];
 	while (OS.gtk_tree_model_iter_children (parent.modelHandle, iter, handle)) {
 		OS.gtk_tree_model_get (parent.modelHandle, iter, Tree.ID_COLUMN, index, -1);
-		TreeItem item = parent.items [index [0]];
-		if (item != null && !item.isDisposed ()) {
-			item.dispose ();
+		if (index [0] != -1) {
+			TreeItem item = parent.items [index [0]];
+			if (item != null && !item.isDisposed ()) {
+				item.dispose ();
+			}
 		}
 	}
 	OS.g_free (iter);
@@ -927,6 +1015,16 @@ public void setBackground (Color color) {
 	}
 	GdkColor gdkColor = color != null ? color.handle : null;
 	OS.gtk_tree_store_set (parent.modelHandle, handle, Tree.BACKGROUND_COLUMN, gdkColor, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 }
 
@@ -954,24 +1052,39 @@ public void setBackground (int index, Color color) {
 	if (color != null && color.isDisposed ()) {
 		SWT.error (SWT.ERROR_INVALID_ARGUMENT);
 	}
-	int count = Math.max (1, parent.columnCount);
+	int count = Math.max (1, parent.getColumnCount ());
 	if (0 > index || index > count - 1) return;
-	int /*long*/ parentHandle = parent.handle;
-	int /*long*/ column = OS.gtk_tree_view_get_column (parentHandle, index);
-	if (column == 0) return;
 	int modelIndex = parent.columnCount == 0 ? Tree.FIRST_COLUMN : parent.columns [index].modelIndex;
 	GdkColor gdkColor = color != null ? color.handle : null;
 	OS.gtk_tree_store_set (parent.modelHandle, handle, modelIndex + Tree.CELL_BACKGROUND, gdkColor, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 	
 	if (color != null) {
 		boolean customDraw = (parent.columnCount == 0)  ? parent.firstCustomDraw : parent.columns [index].customDraw;
 		if (!customDraw) {
 			if ((parent.style & SWT.VIRTUAL) == 0) {
+				int /*long*/ parentHandle = parent.handle;
+				int /*long*/ column = 0;
+				if (parent.columnCount > 0) {
+					column = parent.columns [index].handle;
+				} else {
+					column = OS.gtk_tree_view_get_column (parentHandle, index);
+				}
+				if (column == 0) return;
 				int /*long*/ textRenderer = parent.getTextRenderer (column);
 				int /*long*/ imageRenderer = parent.getPixbufRenderer (column);
-				OS.gtk_tree_view_column_set_cell_data_func (column, textRenderer, display.textCellDataProc, parent.handle, 0);
-				OS.gtk_tree_view_column_set_cell_data_func (column, imageRenderer, display.pixbufCellDataProc, parent.handle, 0);
+				OS.gtk_tree_view_column_set_cell_data_func (column, textRenderer, display.cellDataProc, parentHandle, 0);
+				OS.gtk_tree_view_column_set_cell_data_func (column, imageRenderer, display.cellDataProc, parentHandle, 0);
 			}
 			if (parent.columnCount == 0) {
 				parent.firstCustomDraw = true;
@@ -1031,6 +1144,7 @@ public void setExpanded (boolean expanded) {
 		OS.g_signal_handlers_unblock_matched (parent.handle, OS.G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, TEST_COLLAPSE_ROW);
 	}
 	OS.gtk_tree_path_free (path);
+	cached = true;
 }
 
 
@@ -1061,6 +1175,16 @@ public void setFont (Font font){
 	this.font = font;
 	int /*long*/ fontHandle = font != null ? font.handle : 0;
 	OS.gtk_tree_store_set (parent.modelHandle, handle, Tree.FONT_COLUMN, fontHandle, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 }
 
@@ -1088,7 +1212,7 @@ public void setFont (int index, Font font) {
 	if (font != null && font.isDisposed ()) {
 		SWT.error (SWT.ERROR_INVALID_ARGUMENT);
 	}
-	int count = Math.max (1, parent.columnCount);
+	int count = Math.max (1, parent.getColumnCount ());
 	if (0 > index || index > count - 1) return;
 	if (cellFont == null) {
 		cellFont = new Font [count];
@@ -1096,22 +1220,38 @@ public void setFont (int index, Font font) {
 	if (cellFont [index] == font) return;
 	if (cellFont [index] != null && cellFont [index].equals (font)) return;
 	cellFont [index] = font;
-	int /*long*/ parentHandle = parent.handle;
-	int /*long*/ column = OS.gtk_tree_view_get_column (parentHandle, index);
-	if (column == 0) return;
+	
 	int modelIndex = parent.columnCount == 0 ? Tree.FIRST_COLUMN : parent.columns [index].modelIndex;
 	int /*long*/ fontHandle  = font != null ? font.handle : 0;
 	OS.gtk_tree_store_set (parent.modelHandle, handle, modelIndex + Tree.CELL_FONT, fontHandle, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 	
 	if (font != null) {
 		boolean customDraw = (parent.columnCount == 0)  ? parent.firstCustomDraw : parent.columns [index].customDraw;
 		if (!customDraw) {
 			if ((parent.style & SWT.VIRTUAL) == 0) {
+				int /*long*/ parentHandle = parent.handle;
+				int /*long*/ column = 0;
+				if (parent.columnCount > 0) {
+					column = parent.columns [index].handle;
+				} else {
+					column = OS.gtk_tree_view_get_column (parentHandle, index);
+				}
+				if (column == 0) return;
 				int /*long*/ textRenderer = parent.getTextRenderer (column);
 				int /*long*/ imageRenderer = parent.getPixbufRenderer (column);
-				OS.gtk_tree_view_column_set_cell_data_func (column, textRenderer, display.textCellDataProc, parent.handle, 0);
-				OS.gtk_tree_view_column_set_cell_data_func (column, imageRenderer, display.pixbufCellDataProc, parent.handle, 0);
+				OS.gtk_tree_view_column_set_cell_data_func (column, textRenderer, display.cellDataProc, parentHandle, 0);
+				OS.gtk_tree_view_column_set_cell_data_func (column, imageRenderer, display.cellDataProc, parentHandle, 0);
 			}
 			if (parent.columnCount == 0) {
 				parent.firstCustomDraw = true;
@@ -1149,6 +1289,16 @@ public void setForeground (Color color){
 	}
 	GdkColor gdkColor = color != null ? color.handle : null;
 	OS.gtk_tree_store_set (parent.modelHandle, handle, Tree.FOREGROUND_COLUMN, gdkColor, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 }
 
@@ -1176,24 +1326,39 @@ public void setForeground (int index, Color color){
 	if (color != null && color.isDisposed ()) {
 		SWT.error (SWT.ERROR_INVALID_ARGUMENT);
 	}
-	int count = Math.max (1, parent.columnCount);
+	int count = Math.max (1, parent.getColumnCount ());
 	if (0 > index || index > count - 1) return;
-	int /*long*/ parentHandle = parent.handle;
-	int /*long*/ column = OS.gtk_tree_view_get_column (parentHandle, index);
-	if (column == 0) return;
 	int modelIndex = parent.columnCount == 0 ? Tree.FIRST_COLUMN : parent.columns [index].modelIndex;
 	GdkColor gdkColor = color != null ? color.handle : null;
 	OS.gtk_tree_store_set (parent.modelHandle, handle, modelIndex + Tree.CELL_FOREGROUND, gdkColor, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when it is cleared.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 	
 	if (color != null) {
 		boolean customDraw = (parent.columnCount == 0)  ? parent.firstCustomDraw : parent.columns [index].customDraw;
 		if (!customDraw) {
 			if ((parent.style & SWT.VIRTUAL) == 0) {
+				int /*long*/ parentHandle = parent.handle;
+				int /*long*/ column = 0;
+				if (parent.columnCount > 0) {
+					column = parent.columns [index].handle;
+				} else {
+					column = OS.gtk_tree_view_get_column (parentHandle, index);
+				}
+				if (column == 0) return;
 				int /*long*/ textRenderer = parent.getTextRenderer (column);
 				int /*long*/ imageRenderer = parent.getPixbufRenderer (column);
-				OS.gtk_tree_view_column_set_cell_data_func (column, textRenderer, display.textCellDataProc, parent.handle, 0);
-				OS.gtk_tree_view_column_set_cell_data_func (column, imageRenderer, display.pixbufCellDataProc, parent.handle, 0);
+				OS.gtk_tree_view_column_set_cell_data_func (column, textRenderer, display.cellDataProc, parentHandle, 0);
+				OS.gtk_tree_view_column_set_cell_data_func (column, imageRenderer, display.cellDataProc, parentHandle, 0);
 			}
 			if (parent.columnCount == 0) {
 				parent.firstCustomDraw = true;
@@ -1205,10 +1370,10 @@ public void setForeground (int index, Color color){
 }
 
 /**
- * Sets the grayed state of the receiver.
- * <p>
+ * Sets the grayed state of the checkbox for this item.  This state change 
+ * only applies if the Tree was created with the SWT.CHECK style.
  *
- * @param grayed the new grayed state
+ * @param grayed the new grayed state of the checkbox
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -1250,10 +1415,8 @@ public void setImage (int index, Image image) {
 	if (image != null && image.isDisposed()) {
 		error(SWT.ERROR_INVALID_ARGUMENT);
 	}
-	if (!(0 <= index && index <= parent.columnCount )) return;
-	int /*long*/ parentHandle = parent.handle;
-	int /*long*/ column = OS.gtk_tree_view_get_column (parentHandle, index);
-	if (column == 0) return;
+	int count = Math.max (1, parent.getColumnCount ());
+	if (0 > index || index > count - 1) return;
 	int /*long*/ pixbuf = 0;
 	if (image != null) {
 		ImageList imageList = parent.imageList;
@@ -1264,6 +1427,43 @@ public void setImage (int index, Image image) {
 	}
 	int modelIndex = parent.columnCount == 0 ? Tree.FIRST_COLUMN : parent.columns [index].modelIndex;
 	OS.gtk_tree_store_set (parent.modelHandle, handle, modelIndex + Tree.CELL_PIXBUF, pixbuf, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when the image changes.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			if (parent.columnCount == 0) {
+				redraw ();
+			}
+		}
+	}
+	/*
+	* Bug in GTK.  When using fixed-height-mode, GTK does not recalculate the cell renderer width
+	* when the image is changed in the model.  The fix is to force it to recalculate the width if
+	* more space is required.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0 && parent.currentItem == null) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2)) {
+			if (image != null) {
+				int /*long*/parentHandle = parent.handle;
+				int /*long*/ column = OS.gtk_tree_view_get_column (parentHandle, index);
+				int [] w = new int [1];
+				int /*long*/ pixbufRenderer = parent.getPixbufRenderer(column);
+				OS.gtk_tree_view_column_cell_get_position (column, pixbufRenderer, null, w);
+				if (w[0] < image.getBounds().width) {
+					/*
+					 * There is no direct way to clear the cell renderer width so we
+					 * are relying on the fact that it is done as part of modifying
+					 * the style.
+					 */
+					int /*long*/ style = OS.gtk_widget_get_modifier_style (parentHandle);
+					OS.gtk_widget_modify_style (parentHandle, style);
+				}
+			} 
+		}
+	}
 	cached = true;
 }
 
@@ -1296,6 +1496,24 @@ public void setImage (Image [] images) {
 	}
 }
 
+/**
+ * Sets the number of child items contained in the receiver.
+ *
+ * @param count the number of items
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @since 3.2
+ */
+public void setItemCount (int count) {
+	checkWidget ();
+	count = Math.max (0, count);
+	parent.setItemCount (handle, count);
+}
+
 /**
  * Sets the receiver's text at a column
  *
@@ -1315,13 +1533,21 @@ public void setImage (Image [] images) {
 public void setText (int index, String string) {
 	checkWidget ();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
-	if (!(0 <= index && index <= parent.columnCount )) return;
-	int /*long*/ parentHandle = parent.handle;
-	int /*long*/ column = OS.gtk_tree_view_get_column (parentHandle, index);
-	if (column == 0) return;
+	int count = Math.max (1, parent.getColumnCount ());
+	if (0 > index || index > count - 1) return;
 	byte[] buffer = Converter.wcsToMbcs (null, string, true);
 	int modelIndex = parent.columnCount == 0 ? Tree.FIRST_COLUMN : parent.columns [index].modelIndex;
 	OS.gtk_tree_store_set (parent.modelHandle, handle, modelIndex + Tree.CELL_TEXT, buffer, -1);
+	/*
+	* Bug in GTK.  When using fixed-height-mode,
+	* row changes do not cause the row to be repainted.  The fix is to
+	* invalidate the row when the text changes.
+	*/
+	if ((parent.style & SWT.VIRTUAL) != 0) {
+		if (OS.GTK_VERSION >= OS.VERSION (2, 3, 2) && OS.GTK_VERSION < OS.VERSION (2, 6, 3)) {
+			redraw ();
+		}
+	}
 	cached = true;
 }
 
@@ -1353,5 +1579,4 @@ public void setText (String [] strings) {
 		if (string != null) setText (i, string);
 	}
 }
-
 }
