@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -60,79 +60,102 @@ public abstract class Widget {
 	Object data;
 	
 	/* Global state flags */
-	static final int DISPOSED                = 1<<0;
-	static final int CANVAS                  = 1<<1;
-	static final int KEYED_DATA           = 1<<2;
-	static final int HANDLE                   = 1<<3;
-	static final int DISABLED                = 1<<4;
-	static final int MENU                      = 1<<5;
-	static final int OBSCURED              = 1<<6;
-	static final int MOVED                   = 1<<7;
-	static final int RESIZED                  = 1<<8;
-	static final int LAYOUT_NEEDED    = 1<<9;
-	static final int LAYOUT_CHANGED = 1<<10;
-	static final int ZERO_SIZED = 1<<11;
-	static final int HIDDEN = 1<<12;
-	static final int FOREGROUND = 1<<13;
-	static final int BACKGROUND = 1<<14;
-	static final int FONT = 1<<15;
+	static final int DISPOSED = 1<<0;
+	static final int CANVAS = 1<<1;
+	static final int KEYED_DATA = 1<<2;
+	static final int HANDLE = 1<<3;
+	static final int DISABLED = 1<<4;
+	static final int MENU = 1<<5;
+	static final int OBSCURED = 1<<6;
+	static final int MOVED = 1<<7;
+	static final int RESIZED = 1<<8;
+	static final int ZERO_WIDTH = 1<<9;
+	static final int ZERO_HEIGHT = 1<<10;
+	static final int HIDDEN = 1<<11;
+	static final int FOREGROUND = 1<<12;
+	static final int BACKGROUND = 1<<13;
+	static final int FONT = 1<<14;
+	static final int PARENT_BACKGROUND = 1<<15;
+	static final int THEME_BACKGROUND = 1<<16;
 	
-	/* Default widths for widgets */
+	/* A layout was requested on this widget */
+	static final int LAYOUT_NEEDED	= 1<<17;
+	
+	/* The preferred size of a child has changed */
+	static final int LAYOUT_CHANGED = 1<<18;
+	
+	/* A layout was requested in this widget hierachy */
+	static final int LAYOUT_CHILD = 1<<19;
+
+	/* More global state flags */
+	static final int RELEASED = 1<<20;
+	static final int DISPOSE_SENT = 1<<21;
+	
+	/* Default size for widgets */
 	static final int DEFAULT_WIDTH	= 64;
 	static final int DEFAULT_HEIGHT	= 64;
 	
 	/* GTK signals data */
 	static final int ACTIVATE = 1;
 	static final int BUTTON_PRESS_EVENT = 2;
-	static final int BUTTON_RELEASE_EVENT = 3;
-	static final int CHANGED = 4;
-	static final int CLICKED = 5;
-	static final int COMMIT = 6;
-	static final int CONFIGURE_EVENT = 7;
-	static final int DELETE_EVENT = 8;
-	static final int DELETE_RANGE = 9;
-	static final int DELETE_TEXT = 10;
-	static final int ENTER_NOTIFY_EVENT = 11;
-	static final int EVENT = 12;
-	static final int EVENT_AFTER = 13;
-	static final int EXPAND_COLLAPSE_CURSOR_ROW = 14;
-	static final int EXPOSE_EVENT = 15;
-	static final int FOCUS = 16;
-	static final int FOCUS_IN_EVENT = 17;
-	static final int FOCUS_OUT_EVENT = 18;
-	static final int HIDE = 19;
-	static final int INPUT = 20;
-	static final int INSERT_TEXT = 21;
-	static final int KEY_PRESS_EVENT = 22;
-	static final int KEY_RELEASE_EVENT = 23;
-	static final int LEAVE_NOTIFY_EVENT = 24;
-	static final int MAP = 25;
-	static final int MAP_EVENT = 26;
-	static final int MNEMONIC_ACTIVATE = 27;
-	static final int MOTION_NOTIFY_EVENT = 28;
-	static final int MOVE_FOCUS = 29;
-	static final int OUTPUT = 30;
-	static final int POPUP_MENU = 31;
-	static final int PREEDIT_CHANGED = 32;
-	static final int REALIZE = 33;
-	static final int ROW_ACTIVATED = 34;
-	static final int SCROLL_CHILD = 35;
-	static final int SCROLL_EVENT = 36;
-	static final int SELECT = 37;
-	static final int SHOW = 38;
-	static final int SHOW_HELP = 39;
-	static final int SIZE_ALLOCATE = 40;
-	static final int STYLE_SET = 41;
-	static final int SWITCH_PAGE = 42;
-	static final int TEST_COLLAPSE_ROW = 43;
-	static final int TEST_EXPAND_ROW = 44;
-	static final int TOGGLED = 45;
-	static final int UNMAP = 46;
-	static final int UNMAP_EVENT = 47;
-	static final int UNREALIZE = 48;
-	static final int VALUE_CHANGED = 49;
-	static final int VISIBILITY_NOTIFY_EVENT = 50;
-	static final int WINDOW_STATE_EVENT = 51;
+	static final int BUTTON_PRESS_EVENT_INVERSE = 3;
+	static final int BUTTON_RELEASE_EVENT = 4;
+	static final int BUTTON_RELEASE_EVENT_INVERSE = 5;
+	static final int CHANGED = 6;
+	static final int CHANGE_VALUE = 7;
+	static final int CLICKED = 8;
+	static final int COMMIT = 9;
+	static final int CONFIGURE_EVENT = 10;
+	static final int DELETE_EVENT = 11;
+	static final int DELETE_RANGE = 12;
+	static final int DELETE_TEXT = 13;
+	static final int ENTER_NOTIFY_EVENT = 14;
+	static final int EVENT = 15;
+	static final int EVENT_AFTER = 16;
+	static final int EXPAND_COLLAPSE_CURSOR_ROW = 17;
+	static final int EXPOSE_EVENT = 18;
+	static final int EXPOSE_EVENT_INVERSE = 19;
+	static final int FOCUS = 20;
+	static final int FOCUS_IN_EVENT = 21;
+	static final int FOCUS_OUT_EVENT = 22;
+	static final int GRAB_FOCUS = 23;
+	static final int HIDE = 24;
+	static final int INPUT = 25;
+	static final int INSERT_TEXT = 26;
+	static final int KEY_PRESS_EVENT = 27;
+	static final int KEY_RELEASE_EVENT = 28;
+	static final int LEAVE_NOTIFY_EVENT = 29;
+	static final int MAP = 30;
+	static final int MAP_EVENT = 31;
+	static final int MNEMONIC_ACTIVATE = 32;
+	static final int MOTION_NOTIFY_EVENT = 33;
+	static final int MOTION_NOTIFY_EVENT_INVERSE = 34;
+	static final int MOVE_FOCUS = 35;
+	static final int OUTPUT = 36;
+	static final int POPUP_MENU = 37;
+	static final int PREEDIT_CHANGED = 38;
+	static final int REALIZE = 39;
+	static final int ROW_ACTIVATED = 40;
+	static final int SCROLL_CHILD = 41;
+	static final int SCROLL_EVENT = 42;
+	static final int SELECT = 43;
+	static final int SHOW = 44;
+	static final int SHOW_HELP = 45;
+	static final int SIZE_ALLOCATE = 46;
+	static final int STYLE_SET = 47;
+	static final int SWITCH_PAGE = 48;
+	static final int TEST_COLLAPSE_ROW = 49;
+	static final int TEST_EXPAND_ROW = 50;
+	static final int TEXT_BUFFER_INSERT_TEXT = 51;
+	static final int TOGGLED = 52;
+	static final int UNMAP = 53;
+	static final int UNMAP_EVENT = 54;
+	static final int UNREALIZE = 55;
+	static final int VALUE_CHANGED = 56;
+	static final int VISIBILITY_NOTIFY_EVENT = 57;
+	static final int WINDOW_STATE_EVENT = 58;
+	static final int ACTIVATE_INVERSE = 59;
+	static final int LAST_SIGNAL = 60;
 
 /**
  * Prevents uninitialized instances from being created outside the package.
@@ -175,11 +198,17 @@ public Widget (Widget parent, int style) {
 	display = parent.display;
 }
 
+void _addListener (int eventType, Listener listener) {
+	if (eventTable == null) eventTable = new EventTable ();
+	eventTable.hook (eventType, listener);
+}
+
 /**
  * Adds the listener to the collection of listeners who will
- * be notifed when an event of the given type occurs. When the
+ * be notified when an event of the given type occurs. When the
  * event does occur in the widget, the listener is notified by
- * sending it the <code>handleEvent()</code> message.
+ * sending it the <code>handleEvent()</code> message. The event
+ * type is one of the event constants defined in class <code>SWT</code>.
  *
  * @param eventType the type of event to listen for
  * @param listener the listener which should be notified when the event occurs
@@ -193,18 +222,19 @@ public Widget (Widget parent, int style) {
  * </ul>
  *
  * @see Listener
+ * @see SWT
  * @see #removeListener
+ * @see #notifyListeners
  */
-public void addListener (int eventType, Listener handler) {
+public void addListener (int eventType, Listener listener) {
 	checkWidget ();
-	if (handler == null) error (SWT.ERROR_NULL_ARGUMENT);
-	if (eventTable == null) eventTable = new EventTable ();
-	eventTable.hook (eventType, handler);
+	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
+	_addListener (eventType, listener);
 }
 
 /**
  * Adds the listener to the collection of listeners who will
- * be notifed when the widget is disposed. When the widget is
+ * be notified when the widget is disposed. When the widget is
  * disposed, the listener is notified by sending it the
  * <code>widgetDisposed()</code> message.
  *
@@ -228,15 +258,9 @@ public void addDisposeListener (DisposeListener listener) {
 	addListener (SWT.Dispose, typedListener);
 }
 
-int /*long*/ textCellDataProc (int /*long*/ tree_column, int /*long*/ cell, int /*long*/ tree_model, int /*long*/ iter, int /*long*/ data) {
-	return 0;
-}
 int /*long*/ paintWindow () {
 	return 0;
 }
-int /*long*/ pixbufCellDataProc (int /*long*/ tree_column, int /*long*/ cell, int /*long*/ tree_model, int /*long*/ iter, int /*long*/ data) {
-	return 0;
-}
 
 static int checkBits (int style, int int0, int int1, int int2, int int3, int int4, int int5) {
 	int mask = int0 | int1 | int2 | int3 | int4 | int5;
@@ -250,6 +274,10 @@ static int checkBits (int style, int int0, int int1, int int2, int int3, int int
 	return style;
 }
 
+int /*long*/ cellDataProc (int /*long*/ tree_column, int /*long*/ cell, int /*long*/ tree_model, int /*long*/ iter, int /*long*/ data) {
+	return 0;
+}
+
 void checkOpen () {
 	/* Do nothing */
 }
@@ -397,11 +425,9 @@ public void dispose () {
 	* Note:  It is valid to attempt to dispose a widget
 	* more than once.  If this happens, fail silently.
 	*/
-	if (isDisposed()) return;
+	if (isDisposed ()) return;
 	if (!isValidThread ()) error (SWT.ERROR_THREAD_INVALID_ACCESS);
-	releaseChild ();
-	releaseWidget ();
-	destroyWidget ();
+	release (true);
 }
 
 void error (int code) {
@@ -547,6 +573,10 @@ int /*long*/ gtk_changed (int /*long*/ widget) {
 	return 0;
 }
 
+int /*long*/ gtk_change_value (int /*long*/ widget, int /*long*/ scroll, int /*long*/ value1, int /*long*/ value2) {
+	return 0;
+}
+
 int /*long*/ gtk_clicked (int /*long*/ widget) {
 	return 0;
 }
@@ -603,6 +633,10 @@ int /*long*/ gtk_focus_out_event (int /*long*/ widget, int /*long*/ event) {
 	return 0;
 }
 
+int /*long*/ gtk_grab_focus (int /*long*/ widget) {
+	return 0;
+}
+
 int /*long*/ gtk_hide (int /*long*/ widget) {
 	return 0;
 }
@@ -711,6 +745,10 @@ int /*long*/ gtk_test_expand_row (int /*long*/ tree, int /*long*/ iter, int /*lo
 	return 0;
 }
 
+int /*long*/ gtk_text_buffer_insert_text (int /*long*/ widget, int /*long*/ iter, int /*long*/ text, int /*long*/ length) {
+	return 0;
+}
+
 int /*long*/ gtk_timer () {
 	return 0;
 }
@@ -757,10 +795,6 @@ boolean filters (int eventType) {
 	return display.filters (eventType);
 }
 
-int /*long*/ filterProc (int /*long*/ xEvent, int /*long*/ gdkEvent, int /*long*/ data) {
-	return 0;
-}
-
 int /*long*/ fixedMapProc (int /*long*/ widget) {
 	return 0;
 }
@@ -801,23 +835,24 @@ char [] fixMnemonic (String string) {
  * @return <code>true</code> when the widget is disposed and <code>false</code> otherwise
  */
 public boolean isDisposed () {
-	if (handle != 0) return false;
-	if ((state & HANDLE) != 0) return true;
 	return (state & DISPOSED) != 0;
 }
 
 /**
  * Returns <code>true</code> if there are any listeners
  * for the specified event type associated with the receiver,
- * and <code>false</code> otherwise.
+ * and <code>false</code> otherwise. The event type is one of
+ * the event constants defined in class <code>SWT</code>.
  *
- * @param	eventType the type of event
+ * @param eventType the type of event
  * @return true if the event is hooked
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ *
+ * @see SWT
  */
 public boolean isListening (int eventType) {
 	checkWidget ();
@@ -828,12 +863,6 @@ boolean isValidThread () {
 	return getDisplay ().isValidThread ();
 }
 
-boolean isValidWidget () {
-	if (handle != 0) return true;
-	if ((state & HANDLE) != 0) return false;
-	return (state & DISPOSED) == 0;
-}
-
 boolean isValidSubclass() {
 	return Display.isValidClass(getClass());
 }
@@ -884,7 +913,9 @@ boolean mnemonicMatch (int /*long*/ mnemonicHandle, char key) {
 /**
  * Notifies all of the receiver's listeners for events
  * of the given type that one such event has occurred by
- * invoking their <code>handleEvent()</code> method.
+ * invoking their <code>handleEvent()</code> method.  The
+ * event type is one of the event constants defined in class
+ * <code>SWT</code>.
  *
  * @param eventType the type of event which has occurred
  * @param event the event data
@@ -893,6 +924,10 @@ boolean mnemonicMatch (int /*long*/ mnemonicHandle, char key) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see SWT
+ * @see #addListener
+ * @see #removeListener
  */
 public void notifyListeners (int eventType, Event event) {
 	checkWidget();
@@ -913,8 +948,28 @@ void register () {
 	if ((state & HANDLE) != 0) display.addWidget (handle, this);
 }
 
-void releaseChild () {
-	/* Do nothing */
+void release (boolean destroy) {
+	if ((state & DISPOSE_SENT) == 0) {
+		state |= DISPOSE_SENT;
+		sendEvent (SWT.Dispose);
+	}
+	if ((state & DISPOSED) == 0) {
+		releaseChildren (destroy);
+	}
+	if ((state & RELEASED) == 0) {
+		state |= RELEASED;
+		if (destroy) {
+			releaseParent ();
+			releaseWidget ();
+			destroyWidget ();
+		} else {
+			releaseWidget ();
+			releaseHandle ();
+		}
+	}
+}
+
+void releaseChildren (boolean destroy) {
 }
 
 void releaseHandle () {
@@ -923,13 +978,11 @@ void releaseHandle () {
 	display = null;
 }
 
-void releaseResources () {
-	releaseWidget ();
-	releaseHandle ();
+void releaseParent () {
+	/* Do nothing */
 }
 
 void releaseWidget () {
-	sendEvent (SWT.Dispose);
 	deregister ();
 	eventTable = null;
 	data = null;
@@ -937,7 +990,8 @@ void releaseWidget () {
 
 /**
  * Removes the listener from the collection of listeners who will
- * be notifed when an event of the given type occurs.
+ * be notified when an event of the given type occurs. The event
+ * type is one of the event constants defined in class <code>SWT</code>.
  *
  * @param eventType the type of event to listen for
  * @param listener the listener which should no longer be notified when the event occurs
@@ -951,7 +1005,9 @@ void releaseWidget () {
  * </ul>
  *
  * @see Listener
+ * @see SWT
  * @see #addListener
+ * @see #notifyListeners
  */
 public void removeListener (int eventType, Listener handler) {
 	checkWidget ();
@@ -962,7 +1018,7 @@ public void removeListener (int eventType, Listener handler) {
 
 /**
  * Removes the listener from the collection of listeners who will
- * be notifed when an event of the given type occurs.
+ * be notified when an event of the given type occurs.
  * <p>
  * <b>IMPORTANT:</b> This method is <em>not</em> part of the SWT
  * public API. It is marked public only so that it can be shared
@@ -991,9 +1047,17 @@ protected void removeListener (int eventType, SWTEventListener handler) {
 	eventTable.unhook (eventType, handler);
 }
 
+int /*long*/ rendererGetSizeProc (int /*long*/ cell, int /*long*/ handle, int /*long*/ cell_area, int /*long*/ x_offset, int /*long*/ y_offset, int /*long*/ width, int /*long*/ height) {
+	return 0;
+}
+
+int /*long*/ rendererRenderProc (int /*long*/ cell, int /*long*/ window, int /*long*/ handle, int /*long*/ background_area, int /*long*/ cell_area, int /*long*/ expose_area, int /*long*/ flags) {
+	return 0;
+}
+
 /**
  * Removes the listener from the collection of listeners who will
- * be notifed when the widget is disposed.
+ * be notified when the widget is disposed.
  *
  * @param listener the listener which should no longer be notified when the receiver is disposed
  *
@@ -1284,6 +1348,10 @@ int /*long*/ shellMapProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/
 	return 0;
 }
 
+int /*long*/ sizeAllocateProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/ user_data) {
+	return 0;
+}
+
 /**
  * Returns a string containing a concise, human-readable
  * description of the receiver.
@@ -1321,6 +1389,7 @@ int /*long*/ windowProc (int /*long*/ handle, int /*long*/ user_data) {
 		case CHANGED: return gtk_changed (handle);
 		case CLICKED: return gtk_clicked (handle);
 		case HIDE: return gtk_hide (handle);
+		case GRAB_FOCUS: return gtk_grab_focus (handle);
 		case MAP: return gtk_map (handle);
 		case OUTPUT: return gtk_output (handle);
 		case POPUP_MENU: return gtk_popup_menu (handle);
@@ -1337,7 +1406,7 @@ int /*long*/ windowProc (int /*long*/ handle, int /*long*/ user_data) {
 
 int /*long*/ windowProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/ user_data) {
 	switch ((int)/*64*/user_data) {
-		case -EXPOSE_EVENT: {
+		case EXPOSE_EVENT_INVERSE: {
 			GdkEventExpose gdkEvent = new GdkEventExpose ();
 			OS.memmove (gdkEvent, arg0, GdkEventExpose.sizeof);
 			int /*long*/ paintWindow = paintWindow();
@@ -1345,9 +1414,9 @@ int /*long*/ windowProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/ us
 			if (window != paintWindow) return 0;
 			return (state & OBSCURED) != 0 ? 1 : 0;
 		}
-		case -BUTTON_PRESS_EVENT:
-		case -BUTTON_RELEASE_EVENT:
-		case -MOTION_NOTIFY_EVENT: {
+		case BUTTON_PRESS_EVENT_INVERSE:
+		case BUTTON_RELEASE_EVENT_INVERSE:
+		case MOTION_NOTIFY_EVENT_INVERSE: {
 			return 1;
 		}
 		case BUTTON_PRESS_EVENT: return gtk_button_press_event (handle, arg0);
@@ -1397,8 +1466,10 @@ int /*long*/ windowProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/ ar
 
 int /*long*/ windowProc (int /*long*/ handle, int /*long*/ arg0, int /*long*/ arg1, int /*long*/ arg2, int /*long*/ user_data) {
 	switch ((int)/*64*/user_data) {
+		case CHANGE_VALUE: return gtk_change_value (handle, arg0, arg1, arg2);
 		case EXPAND_COLLAPSE_CURSOR_ROW: return gtk_expand_collapse_cursor_row (handle, arg0, arg1, arg2);
 		case INSERT_TEXT: return gtk_insert_text (handle, arg0, arg1, arg2);
+		case TEXT_BUFFER_INSERT_TEXT: return gtk_text_buffer_insert_text (handle, arg0, arg1, arg2);
 		default: return 0;
 	}
 }
