@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -28,26 +28,33 @@ import org.eclipse.swt.*;
  *
  * @see FontData
  */
-public final class Font {
+public final class Font extends Resource {
 	/**
 	 * the handle to the OS font resource
 	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
 	 */
 	public int handle;
 
 	/**
 	 * the code page of the font
 	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
 	 * 
 	 * @since 2.0
 	 */
 	public String codePage;
 
-	/**
-	 * The device where this image was created.
-	 */
-	Device device;
-
 Font () {
 }
 
@@ -194,7 +201,7 @@ static String getCodePage (int xDisplay, int fontList) {
 			OS.memmove(fontStruct,fontPtr,20 * 4);
 			int propPtr = fontStruct.properties;
 			for (int i = 0; i < fontStruct.n_properties; i++) {
-				/* Reef through properties looking for XAFONT */
+				/* Look through properties for XAFONT */
 				int[] prop = new int[2];
 				OS.memmove(prop, propPtr, 8);
 				if (prop[0] == OS.XA_FONT) {
@@ -265,10 +272,11 @@ public FontData[] getFontData() {
 	int xDisplay = device.xDisplay;
 	/*
 	 * Create a font context to iterate over each element in the font list.
-	 * If a font context can not be created, return null.
 	 */
 	int[] buffer = new int[1];
-	if (!OS.XmFontListInitFontContext(buffer, handle)) return null;
+	if (!OS.XmFontListInitFontContext(buffer, handle)) {
+		SWT.error (SWT.ERROR_INVALID_FONT);
+	}
 	int context = buffer[0];
 	XFontStruct fontStruct = new XFontStruct();
 	int fontListEntry;
@@ -335,8 +343,8 @@ public FontData[] getFontData() {
 								/*
 								 * Some font servers, for example, xfstt, do not pass
 								 * reasonable font properties to the client, so we
-								 * cannot construct a FontData for these. Use the font
-								 * name instead and return null if that fails.
+								 * cannot construct a FontData for these. Try to use
+								 * the font name instead.
 								 */
 								int[] fontName = new int[1];
 								OS.memmove(fontName, fontNamePtr [0] + (i * 4), 4);
@@ -357,14 +365,14 @@ public FontData[] getFontData() {
 				}
 			}
 		}
-		if (data.length == 0) return null;
+		if (data.length == 0) SWT.error (SWT.ERROR_INVALID_FONT);
 	} catch (Exception e) {
 		/*
 		 * Some font servers, for example, xfstt, do not pass
 		 * reasonable font properties to the client, so we
-		 * cannot construct a FontData for these. Return null.
+		 * cannot construct a FontData for these.
 		 */
-		return null;
+		SWT.error (SWT.ERROR_INVALID_FONT);
 	} finally {
 		OS.XmFontListFreeFontContext(context);
 	}
@@ -373,7 +381,7 @@ public FontData[] getFontData() {
 
 /**
  * Returns an integer hash code for the receiver. Any two 
- * objects which return <code>true</code> when passed to 
+ * objects that return <code>true</code> when passed to 
  * <code>equals</code> must return the same value for this
  * method.
  *
@@ -405,11 +413,28 @@ void init (Device device, FontData[] fds) {
 		OS.setlocale (OS.LC_CTYPE, buffer);
 	}
 	
+	int fontType = OS.XmFONT_IS_FONTSET;
+	/*
+	* Bug in HPUX.  If the locale is "C" then FontSets do not work
+	* properly.  The fix is to detect this case and use a Font struct
+	* instead.
+	*/
+	if (OS.IsHPUX) {
+		int localePtr = OS.setlocale(OS.LC_CTYPE, null);
+		int length = OS.strlen(localePtr);
+		byte[] buffer = new byte[length];
+		OS.memmove(buffer, localePtr, length);
+		if ("C".equals(new String(Converter.mbcsToWcs(null, buffer)))) {
+			fontType = OS.XmFONT_IS_FONT;
+		}
+	}
+	
 	/* Generate desire font name */
 	Point dpi = null;
 	if (device.setDPI) dpi = device.getDPI();
 	StringBuffer stringBuffer = new StringBuffer();
 	for (int i = 0; i < fds.length; i++) {
+		if (i != 0) stringBuffer.append(',');
 		FontData fd = fds[i];
 		int hRes = fd.horizontalResolution, vRes = fd.verticalResolution;
 		if (dpi != null) {
@@ -417,7 +442,6 @@ void init (Device device, FontData[] fds) {
 			fd.verticalResolution = dpi.y;
 		}
 		stringBuffer.append(fd.getXlfd());
-		stringBuffer.append(',');
 		fd.horizontalResolution = hRes;
 		fd.verticalResolution = vRes;
 	}
@@ -430,32 +454,36 @@ void init (Device device, FontData[] fds) {
 	* font takes a very long time (10 minutes) when there are no Japanese bold
 	* fonts available.  The fix is to wildcard the field weight.
 	*/
-	if (OS.IsAIX && OS.IsDBLocale) {
-		stringBuffer.append(newFd.getXlfd());
-	} else {
-		newFd.weight = firstFd.weight;
-		newFd.slant = firstFd.slant;
-		stringBuffer.append(newFd.getXlfd());
-		newFd.weight = null;
-		newFd.slant = null;		
-		stringBuffer.append(',');
-		stringBuffer.append(newFd.getXlfd());
+	if (fontType == OS.XmFONT_IS_FONTSET) {
+		if (OS.IsAIX && OS.IsDBLocale) {
+			stringBuffer.append(',');
+			stringBuffer.append(newFd.getXlfd());
+		} else {
+			newFd.weight = firstFd.weight;
+			newFd.slant = firstFd.slant;
+			stringBuffer.append(',');
+			stringBuffer.append(newFd.getXlfd());
+			newFd.weight = null;
+			newFd.slant = null;		
+			stringBuffer.append(',');
+			stringBuffer.append(newFd.getXlfd());
+		}
 	}
 	
 	/* Load font list entry */		 
-	boolean warnings = device.getWarnings ();
-	device.setWarnings (false);
+	boolean warnings = device._getWarnings ();
+	device._setWarnings (false);
 	byte[] buffer = Converter.wcsToMbcs(null, stringBuffer.toString() , true);
-	int fontListEntry = OS.XmFontListEntryLoad(device.xDisplay, buffer, OS.XmFONT_IS_FONTSET, OS.XmFONTLIST_DEFAULT_TAG);
-	device.setWarnings (warnings);
+	int fontListEntry = OS.XmFontListEntryLoad(device.xDisplay, buffer, fontType, OS.XmFONTLIST_DEFAULT_TAG);
+	device._setWarnings (warnings);
 	if (fontListEntry != 0) {
 		handle = OS.XmFontListAppendEntry(0, fontListEntry);
 		OS.XmFontListEntryFree(new int[]{fontListEntry});
 		int codesetPtr = OS.nl_langinfo(OS.CODESET);
 		int length = OS.strlen(codesetPtr);
-		byte[] codeset = new byte[length];
-		OS.memmove(codeset, codesetPtr, length);
-		codePage = new String(Converter.mbcsToWcs(null, codeset));
+		buffer = new byte[length];
+		OS.memmove(buffer, codesetPtr, length);
+		codePage = new String(Converter.mbcsToWcs(null, buffer));
 	} else {
 		Font systemFont = device.systemFont;
 		handle = systemFont.handle;
