@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -31,10 +31,29 @@ public final class GCData {
 	public int drawable;
 	public int foreground = -1;
 	public int background = -1;
-	public int fontList;
-	public String codePage;
+	public Image backgroundImage;
+	public Pattern foregroundPattern;
+	public Pattern backgroundPattern;
+	public Font font;
 	public int colormap;
-	public int clipRgn;
+	public int clipRgn, damageRgn;
 	public int lineStyle = SWT.LINE_SOLID;
+	public int lineWidth;
+	public int[] dashes;
 	public int renderTable;
+	public int alpha = 0xFF;
+	public int interpolation = SWT.DEFAULT;
+
+	public int /*long*/ cairo;
+
+	public String string;
+	public int stringWidth = -1;
+	public int stringHeight = -1;
+	public int xmString;
+
+	public String text;
+	public int textWidth = -1;
+	public int textHeight = -1;
+	public int xmText, xmMnemonic;
+	public int drawFlags;
 }
