@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -41,6 +41,7 @@ import org.eclipse.swt.events.*;
  * </p>
  */
 public class Button extends Control {
+	String text = "";
 	Image image, bitmap, disabled;
 	static final byte [] ARM_AND_ACTIVATE;
 	static {
@@ -93,6 +94,70 @@ public class Button extends Control {
 public Button (Composite parent, int style) {
 	super (parent, checkStyle (style));
 }
+void _setImage (Image image) {
+	int labelPixmap = OS.XmUNSPECIFIED_PIXMAP;
+	int labelInsensitivePixmap = OS.XmUNSPECIFIED_PIXMAP;
+	if (bitmap != null) bitmap.dispose ();
+	if (disabled != null) disabled.dispose ();
+	bitmap = disabled = null;
+	if (image != null) {
+		if (image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
+		switch (image.type) {
+			case SWT.BITMAP:
+				ImageData data;
+				if (image.mask == 0	&& (data = image.getImageData ()).alpha == -1 && data.alphaData == null && data.transparentPixel == -1) {
+					labelPixmap = image.pixmap;
+					disabled = new Image (display, image, SWT.IMAGE_DISABLE);
+					labelInsensitivePixmap = disabled.pixmap;
+					break;
+				}
+				//FALL THROUGH
+			case SWT.ICON:
+				Rectangle rect = image.getBounds ();
+				bitmap = new Image (display, rect.width, rect.height);
+				GC gc = new GC (bitmap);
+				gc.setBackground (getBackground ());
+				gc.fillRectangle (rect);
+				gc.drawImage (image, 0, 0);
+				gc.dispose ();
+				labelPixmap = bitmap.pixmap;
+				disabled = new Image (display, bitmap, SWT.IMAGE_DISABLE);
+				labelInsensitivePixmap = disabled.pixmap;
+				break;
+			default:
+				error (SWT.ERROR_NOT_IMPLEMENTED);
+		}
+	}
+	int [] argList = {
+		OS.XmNlabelType, image == null ? OS.XmSTRING : OS.XmPIXMAP,
+		OS.XmNlabelPixmap, labelPixmap,
+		OS.XmNlabelInsensitivePixmap, labelInsensitivePixmap,
+	};
+	OS.XtSetValues (handle, argList, argList.length / 2);
+}
+void _setText (String string) {
+	char [] text = new char [string.length ()];
+	string.getChars (0, text.length, text, 0);
+	int mnemonic = fixMnemonic (text);
+	byte [] buffer = Converter.wcsToMbcs (getCodePage (), text, true);
+	int xmString = OS.XmStringParseText (
+		buffer,
+		0,
+		OS.XmFONTLIST_DEFAULT_TAG, 
+		OS.XmCHARSET_TEXT, 
+		null,
+		0,
+		0);	
+	if (xmString == 0) error (SWT.ERROR_CANNOT_SET_TEXT);
+	if (mnemonic == 0) mnemonic = OS.XK_VoidSymbol;
+	int [] argList = {
+		OS.XmNlabelType, OS.XmSTRING,
+		OS.XmNlabelString, xmString,
+		OS.XmNmnemonic, mnemonic,
+	};
+	OS.XtSetValues (handle, argList, argList.length / 2);
+	if (xmString != 0) OS.XmStringFree (xmString);
+}
 /**
  * Adds the listener to the collection of listeners who will
  * be notified when the control is selected, by sending
@@ -126,26 +191,28 @@ public void addSelectionListener(SelectionListener listener) {
 }
 static int checkStyle (int style) {
 	style = checkBits (style, SWT.PUSH, SWT.ARROW, SWT.CHECK, SWT.RADIO, SWT.TOGGLE, 0);
-	if ((style & SWT.PUSH) != 0) {
+	if ((style & (SWT.PUSH | SWT.TOGGLE)) != 0) {
 		return checkBits (style, SWT.CENTER, SWT.LEFT, SWT.RIGHT, 0, 0, 0);
 	}
-	if ((style & (SWT.CHECK | SWT.RADIO | SWT.TOGGLE)) != 0) {
+	if ((style & (SWT.CHECK | SWT.RADIO)) != 0) {
 		return checkBits (style, SWT.LEFT, SWT.RIGHT, SWT.CENTER, 0, 0, 0);
 	}
 	if ((style & SWT.ARROW) != 0) {
+		style |= SWT.NO_FOCUS;
 		return checkBits (style, SWT.UP, SWT.DOWN, SWT.LEFT, SWT.RIGHT, 0, 0);
 	}
 	return style;
 }
 void click () {
-	OS.XtCallActionProc (handle, ARM_AND_ACTIVATE, new XAnyEvent (), null, 0);
+	int event = OS.XtMalloc (XEvent.sizeof);
+	OS.XtCallActionProc (handle, ARM_AND_ACTIVATE, event, null, 0);
+	OS.XtFree (event);
 }
 public Point computeSize (int wHint, int hHint, boolean changed) {
 	checkWidget();
 	int border = getBorderWidth ();
 	int width = border * 2, height = border * 2;
 	if ((style & SWT.ARROW) != 0) {
-		Display display = getDisplay ();
 		width += display.scrolledMarginX;
 		height += display.scrolledMarginY;
 		if (wHint != SWT.DEFAULT) width = wHint + (border * 2);
@@ -173,11 +240,24 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		int [] argList1 = {OS.XmNlabelString, 0};
 		OS.XtGetValues (handle, argList1, argList1.length / 2);
 		int xmString = argList1 [1];
-		if (OS.XmStringEmpty (xmString)) height += getFontHeight ();
-		if (xmString != 0) OS.XmStringFree (xmString);
+		if (xmString != 0) {
+			if (OS.XmStringEmpty (xmString)) {
+				int xmString2 = OS.XmStringCreateLocalized (new byte[]{' ', '\0'});
+				if (xmString2 != 0) {
+					height += OS.XmStringHeight (font.handle, xmString2);
+					OS.XmStringFree(xmString2);
+				}
+			}
+			OS.XmStringFree (xmString);
+		}
 	}
 	if (wHint != SWT.DEFAULT || hHint != SWT.DEFAULT) {	
-		int [] argList4 = new int [] {OS.XmNmarginLeft, 0, OS.XmNmarginRight, 0, OS.XmNmarginTop, 0, OS.XmNmarginBottom, 0};
+		int [] argList4 = new int [] {
+			OS.XmNmarginLeft, 0,
+			OS.XmNmarginRight, 0,
+			OS.XmNmarginTop, 0,
+			OS.XmNmarginBottom, 0,
+		};
 		OS.XtGetValues (handle, argList4, argList4.length / 2);
 		if (wHint != SWT.DEFAULT) width = wHint + argList4 [1] + argList4 [3] + (border * 2);
 		if (hHint != SWT.DEFAULT) height = hHint + argList4 [5] + argList4 [7] + (border * 2);
@@ -185,7 +265,7 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 	return new Point (width, height);
 }
 void createHandle (int index) {
-	state |= HANDLE;
+	if ((style & SWT.PUSH) == 0) state |= THEME_BACKGROUND;
 	int borderWidth = (style & SWT.BORDER) != 0 ? 1 : 0;
 	int parentHandle = parent.handle;
 	
@@ -229,7 +309,6 @@ void createHandle (int index) {
 		* push button look.  The fix is to set the shadow
 		* thickness when ever this resource is changed.
 		*/
-		Display display = getDisplay ();
 		int thickness = display.buttonShadowThickness;
 		int [] argList = {
 			OS.XmNancestorSensitive, 1,
@@ -310,13 +389,13 @@ void createWidget (int index) {
 	OS.XtSetValues (handle, argList, argList.length / 2);
 }
 int defaultBackground () {
-	return getDisplay ().buttonBackground;
+	return display.buttonBackground;
 }
 Font defaultFont () {
-	return getDisplay ().buttonFont;
+	return display.buttonFont;
 }
 int defaultForeground () {
-	return getDisplay ().buttonForeground;
+	return display.buttonForeground;
 }
 /**
  * Returns a value which describes the position of the
@@ -356,9 +435,10 @@ public int getAlignment () {
 }
 boolean getDefault () {
 	if ((style & SWT.PUSH) == 0) return false;
-	int [] argList = {OS.XmNshowAsDefault, 0};
-	OS.XtGetValues (handle, argList, argList.length / 2);
-	return argList [1] != 0;
+//	int [] argList = {OS.XmNshowAsDefault, 0};
+//	OS.XtGetValues (handle, argList, argList.length / 2);
+//	return argList [1] != 0;
+	return this == menuShell ().defaultButton;
 }
 /**
  * Returns the receiver's image if it has one, or null
@@ -403,7 +483,8 @@ public boolean getSelection () {
 }
 /**
  * Returns the receiver's text, which will be an empty
- * string if it has never been set.
+ * string if it has never been set or if the receiver is
+ * an <code>ARROW</code> button.
  *
  * @return the receiver's text
  *
@@ -415,51 +496,11 @@ public boolean getSelection () {
 public String getText () {
 	checkWidget();
 	if ((style & SWT.ARROW) != 0) return "";
-	int [] argList = {OS.XmNlabelString, 0, OS.XmNmnemonic, 0};
-	OS.XtGetValues (handle, argList, argList.length / 2);
-	int xmString = argList [1];
-	int mnemonic = argList [3];
-	if (mnemonic == OS.XK_VoidSymbol) mnemonic = 0;
-	if (xmString == 0) error (SWT.ERROR_CANNOT_GET_TEXT);
-	char [] result = null;	
-	int address = OS.XmStringUnparse (
-		xmString,
-		null,
-		OS.XmCHARSET_TEXT,
-		OS.XmCHARSET_TEXT,
-		null,
-		0,
-		OS.XmOUTPUT_ALL);
-	if (address != 0) {
-		int length = OS.strlen (address);
-		byte [] buffer = new byte [length];
-		OS.memmove (buffer, address, length);
-		OS.XtFree (address);
-		result = Converter.mbcsToWcs (getCodePage (), buffer);
-	}	
-	OS.XmStringFree (xmString);
-	int count = 0;
-	if (mnemonic != 0) count++;
-	for (int i=0; i<result.length-1; i++)
-		if (result [i] == Mnemonic) count++;
-	char [] newResult = result;
-	if ((count != 0) || (mnemonic != 0)) {
-		newResult = new char [result.length + count];
-		int i = 0, j = 0;
-		while (i < result.length) {
-			if ((mnemonic != 0) && (result [i] == mnemonic)) {
-				if (j < newResult.length) newResult [j++] = Mnemonic;
-				mnemonic = 0;
-			}
-			if ((newResult [j++] = result [i++]) == Mnemonic)
-				if (j < newResult.length) newResult [j++] = Mnemonic;
-		}
-	}
-	return new String (newResult);
+	return text;
 }
 void hookEvents () {
 	super.hookEvents ();
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	if ((style & (SWT.CHECK | SWT.RADIO | SWT.TOGGLE)) != 0) {
 		OS.XtAddCallback (handle, OS.XmNvalueChangedCallback, windowProc, VALUE_CHANGED_CALLBACK);
 	} else {
@@ -492,7 +533,7 @@ void releaseWidget () {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -571,59 +612,53 @@ public void setAlignment (int alignment) {
 }
 void setBackgroundPixel (int pixel) {
 	super.setBackgroundPixel (pixel);
+	if ((style & SWT.FLAT) != 0) {
+		int [] argList1 = {OS.XmNbottomShadowColor, 0};
+		OS.XtGetValues (handle, argList1, argList1.length / 2);
+		int [] argList2 = {OS.XmNtopShadowColor, argList1 [1]};
+		OS.XtSetValues (handle, argList2, argList2.length / 2);
+	}
 	int [] argList = {OS.XmNlabelType, 0};
 	OS.XtGetValues (handle, argList, argList.length / 2);
-	if (argList [1] == OS.XmPIXMAP) setBitmap (image);
-}
-void setBitmap (Image image) {
-	int labelPixmap = OS.XmUNSPECIFIED_PIXMAP;
-	int labelInsensitivePixmap = OS.XmUNSPECIFIED_PIXMAP;
-	if (bitmap != null) bitmap.dispose ();
-	if (disabled != null) disabled.dispose ();
-	bitmap = disabled = null;
-	if (image != null) {
-		if (image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
-		Display display = getDisplay ();
-		switch (image.type) {
-			case SWT.BITMAP:
-				labelPixmap = image.pixmap;
-				disabled = new Image (display, image, SWT.IMAGE_DISABLE);
-				labelInsensitivePixmap = disabled.pixmap;
-				break;
-			case SWT.ICON:
-				Rectangle rect = image.getBounds ();
-				bitmap = new Image (display, rect.width, rect.height);
-				GC gc = new GC (bitmap);
-				gc.setBackground (getBackground ());
-				gc.fillRectangle (rect);
-				gc.drawImage (image, 0, 0);
-				gc.dispose ();
-				labelPixmap = bitmap.pixmap;
-				disabled = new Image (display, bitmap, SWT.IMAGE_DISABLE);
-				labelInsensitivePixmap = disabled.pixmap;
-				break;
-			default:
-				error (SWT.ERROR_NOT_IMPLEMENTED);
-		}
-	}
-	int [] argList = {
-		OS.XmNlabelType, OS.XmPIXMAP,
-		OS.XmNlabelPixmap, labelPixmap,
-		OS.XmNlabelInsensitivePixmap, labelInsensitivePixmap,
-	};
-	OS.XtSetValues (handle, argList, argList.length / 2);
+	if (argList [1] == OS.XmPIXMAP) _setImage (image);
 }
+
 void setDefault (boolean value) {
 	if ((style & SWT.PUSH) == 0) return;
 	if (getShell ().parent == null) return;
 	int [] argList = {OS.XmNshowAsDefault, (value ? 1 : 0)};
 	OS.XtSetValues (handle, argList, argList.length / 2);
 }
+public void setFont (Font font) {
+	checkWidget();
+	
+	/*
+	* Bug in Motif. Setting the font in a button widget that does
+	* not have a non-empty string causes GP on UTF-8 locale.
+	* The fix is to set a non-empty string, change the font,
+	* and restore the empty string at the end. 
+	*/	
+	int [] argList1 = {OS.XmNlabelString, 0, OS.XmNlabelType, 0};
+	OS.XtGetValues (handle, argList1, argList1.length / 2);
+	boolean fixString = OS.IsDBLocale && OS.XmStringEmpty (argList1 [1]); 
+	if (fixString) {
+		byte[] buffer = Converter.wcsToMbcs (getCodePage (), "string", true);
+		int xmString = OS.XmStringCreateLocalized (buffer);	
+		int [] argList2 = { 
+			OS.XmNlabelType, OS.XmSTRING,
+			OS.XmNlabelString, xmString,
+		};
+		OS.XtSetValues (handle, argList2, argList2.length / 2);
+		OS.XmStringFree (xmString);
+	}
+	super.setFont (font);
+	if (fixString) OS.XtSetValues (handle, argList1, argList1.length / 2);	
+}
 /**
  * Sets the receiver's image to the argument, which may be
- * null indicating that no image should be displayed.
+ * <code>null</code> indicating that no image should be displayed.
  *
- * @param image the image to display on the receiver (may be null)
+ * @param image the image to display on the receiver (may be <code>null</code>)
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_INVALID_ARGUMENT - if the image has been disposed</li>
@@ -635,7 +670,15 @@ void setDefault (boolean value) {
  */
 public void setImage (Image image) {
 	checkWidget();
-	setBitmap (this.image = image);
+	if (image != null && image.isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
+	if ((style & SWT.ARROW) != 0) return;
+	this.image = image;
+	/* This code is intentionally commented*/
+//	if (image == null || text.length () != 0) {
+//		_setText (text);
+//		return;
+//	}
+	_setImage (image);
 }
 boolean setRadioSelection (boolean value) {
 	if ((style & SWT.RADIO) == 0) return false;
@@ -675,14 +718,14 @@ public void setSelection (boolean selected) {
  * the mnemonic character but must not contain line delimiters.
  * </p>
  * <p>
- * Mnemonics are indicated by an '&amp' that causes the next
+ * Mnemonics are indicated by an '&amp;' that causes the next
  * character to be the mnemonic.  When the user presses a
  * key sequence that matches the mnemonic, a selection
  * event occurs. On most platforms, the mnemonic appears
  * underlined but may be emphasised in a platform specific
- * manner.  The mnemonic indicator character '&amp' can be
+ * manner.  The mnemonic indicator character '&amp;' can be
  * escaped by doubling it in the string, causing a single
- *'&amp' to be displayed.
+ * '&amp;' to be displayed.
  * </p>
  * 
  * @param string the new text
@@ -699,36 +742,20 @@ public void setText (String string) {
 	checkWidget();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if ((style & SWT.ARROW) != 0) return;
-	char [] text = new char [string.length ()];
-	string.getChars (0, text.length, text, 0);
-	int i=0, j=0, mnemonic=0;
-	while (i < text.length) {
-		if ((text [j++] = text [i++]) == Mnemonic) {
-			if (i == text.length) {continue;}
-			if (text [i] == Mnemonic) {i++; continue;}
-			if (mnemonic == 0) mnemonic = text [i];
-			j--;
-		}
-	}
-	while (j < text.length) text [j++] = 0;
-	byte [] buffer = Converter.wcsToMbcs (getCodePage (), text, true);
-	int xmString = OS.XmStringParseText (
-		buffer,
-		0,
-		OS.XmFONTLIST_DEFAULT_TAG, 
-		OS.XmCHARSET_TEXT, 
-		null,
-		0,
-		0);	
-	if (xmString == 0) error (SWT.ERROR_CANNOT_SET_TEXT);
-	if (mnemonic == 0) mnemonic = OS.XK_VoidSymbol;
-	int [] argList = {
-		OS.XmNlabelType, OS.XmSTRING,
-		OS.XmNlabelString, xmString,
-		OS.XmNmnemonic, mnemonic,
-	};
-	OS.XtSetValues (handle, argList, argList.length / 2);
-	if (xmString != 0) OS.XmStringFree (xmString);
+	/*
+	* Feature in Motif.  Motif does not optimize the case
+	* when the same text is set into a button causing
+	* it to flash.  The fix is to test for equality and
+	* do nothing.
+	*/
+	if (text.equals (string)) return;
+	text = string;
+	/* This code is intentionally commented*/
+//	if (text.length () == 0 && image != null) {
+//		_setImage (image);
+//		return;
+//	}
+	_setText (string);
 }
 int traversalCode (int key, XKeyEvent xEvent) {
 	return super.traversalCode (key, xEvent) | SWT.TRAVERSE_MNEMONIC;
@@ -737,7 +764,6 @@ void updateShadows () {
 	if ((style & SWT.FLAT) != 0 && (style & SWT.TOGGLE) != 0) {
 		int [] argList1 = {OS.XmNset, 0};
 		OS.XtGetValues (handle, argList1, argList1.length / 2);
-		Display display = getDisplay ();
 		int pixel = argList1 [1] == OS.XmUNSET ? display.compositeBottomShadow : display.compositeTopShadow;
 		int [] argList2 = {OS.XmNtopShadowColor, pixel};
 		OS.XtSetValues (handle, argList2, argList2.length / 2);
