@@ -1,17 +1,17 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.swt.widgets;
 
  
-import java.io.*; 
+import org.eclipse.swt.graphics.*;
 import org.eclipse.swt.internal.*;
 import org.eclipse.swt.internal.motif.*;
 import org.eclipse.swt.*;
@@ -26,6 +26,8 @@ import org.eclipse.swt.*;
  * <dd>(none)</dd>
  * </dl>
  * <p>
+ * Note: Only one of the styles SAVE and OPEN may be specified.
+ * </p><p>
  * IMPORTANT: This class is intended to be subclassed <em>only</em>
  * within the SWT implementation.
  * </p>
@@ -34,24 +36,15 @@ public class FileDialog extends Dialog {
 	int dialog;
 	String [] filterNames = new String [0];
 	String [] filterExtensions = new String [0];
-	String [] fileNames;
-	String fileName = "";
-	String filterPath = "";
+	String [] fileNames = new String [0];
+	String fileName = ""; //$NON-NLS-1$
+	String filterPath = ""; //$NON-NLS-1$
 	String fullPath;
-	boolean cancel = false;
-	static final String FILTER = "*";
+	static final String FILTER = "*"; //$NON-NLS-1$
+	static final char SEPARATOR = System.getProperty ("file.separator").charAt (0); //$NON-NLS-1$
 
 /**
- * Constructs a new instance of this class given only its
- * parent.
- * <p>
- * Note: Currently, null can be passed in for the parent.
- * This has the effect of creating the dialog on the currently active
- * display if there is one. If there is no current display, the 
- * dialog is created on a "default" display. <b>Passing in null as
- * the parent is not considered to be good coding style,
- * and may not be supported in a future release of SWT.</b>
- * </p>
+ * Constructs a new instance of this class given only its parent.
  *
  * @param parent a shell which will be the parent of the new instance
  *
@@ -79,15 +72,9 @@ public FileDialog (Shell parent) {
  * lists the style constants that are applicable to the class.
  * Style bits are also inherited from superclasses.
  * </p>
- * Note: Currently, null can be passed in for the parent.
- * This has the effect of creating the dialog on the currently active
- * display if there is one. If there is no current display, the 
- * dialog is created on a "default" display. <b>Passing in null as
- * the parent is not considered to be good coding style,
- * and may not be supported in a future release of SWT.</b>
- * </p>
  *
  * @param parent a shell which will be the parent of the new instance
+ * @param style the style of dialog to construct
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
@@ -103,13 +90,13 @@ public FileDialog (Shell parent, int style) {
 }
 
 int cancelPressed (int widget, int client, int call) {
-	cancel = true;
 	OS.XtUnmanageChild (widget);
 	return 0;
 }
 /**
  * Returns the path of the first file that was
- * selected in the dialog relative to the filter path
+ * selected in the dialog relative to the filter path, or an
+ * empty string if no such file has been selected.
  * 
  * @return the relative path of the file
  */
@@ -118,9 +105,8 @@ public String getFileName () {
 }
 
 /**
- * Returns the paths of all files that were selected
- * in the dialog relative to the filter path, or null
- * if none are available.
+ * Returns a (possibly empty) array with the paths of all files
+ * that were selected in the dialog relative to the filter path.
  * 
  * @return the relative paths of the files
  */
@@ -139,19 +125,19 @@ public String [] getFilterExtensions () {
 }
 
 /**
- * Returns the file names which the dialog will
- * use to filter the files it shows.
+ * Returns the names that describe the filter extensions
+ * which the dialog will use to filter the files it shows.
  *
- * @return the file name filter
+ * @return the list of filter names
  */
 public String [] getFilterNames () {
 	return filterNames;
 }
 
 /**
- * Returns the directory path that the dialog will use.
- * File names in this path will appear in the dialog,
- * filtered according to the filter extensions.
+ * Returns the directory path that the dialog will use, or an empty
+ * string if this is not set.  File names in this path will appear
+ * in the dialog, filtered according to the filter extensions.
  *
  * @return the directory path string
  * 
@@ -182,13 +168,15 @@ int itemSelected (int widget, int client, int call) {
 		ptr = buffer [0];
 	}
 	if (ptr == 0) return 0;
+	Display display = parent.getDisplay ();
+	int [] table = new int [] {display.tabMapping, display.crMapping};
 	int address = OS.XmStringUnparse (
 		ptr,
 		null,
 		OS.XmCHARSET_TEXT,
 		OS.XmCHARSET_TEXT,
-		null,
-		0,
+		table,
+		table.length,
 		OS.XmOUTPUT_ALL);
 	if (itemCount == 0) OS.XmStringFree (ptr);
 	if (address == 0) return 0;
@@ -210,13 +198,15 @@ int okPressed (int widget, int client, int call) {
 	OS.XtGetValues (dialog, argList, argList.length / 2);
 	
 	int xmString1 = argList [1];
+	Display display = parent.getDisplay ();
+	int [] table = new int [] {display.tabMapping, display.crMapping};
 	int ptr = OS.XmStringUnparse (
 		xmString1,
 		null,
 		OS.XmCHARSET_TEXT,
 		OS.XmCHARSET_TEXT,
-		null,
-		0,
+		table,
+		table.length,
 		OS.XmOUTPUT_ALL);
 	if (ptr != 0) {
 		int length = OS.strlen (ptr);
@@ -245,8 +235,8 @@ int okPressed (int widget, int client, int call) {
 				null,
 				OS.XmCHARSET_TEXT,
 				OS.XmCHARSET_TEXT,
-				null,
-				0,
+				table,
+				table.length,
 				OS.XmOUTPUT_ALL);
 			if (address != 0) {
 				int length = OS.strlen (address);
@@ -255,7 +245,7 @@ int okPressed (int widget, int client, int call) {
 				OS.XtFree (address);
 				/* Use the character encoding for the default locale */
 				String fullFilename = new String (Converter.mbcsToWcs (null, buffer));
-				int index = fullFilename.lastIndexOf ('/');
+				int index = fullFilename.lastIndexOf (SEPARATOR);
 				fileNames [i] = fullFilename.substring (index + 1, fullFilename.length ());
 				if (fullFilename.equals (fullPath)) match = true;
 			}
@@ -267,18 +257,18 @@ int okPressed (int widget, int client, int call) {
 			/* The user has modified the text field such that it doesn't match any
 			 * of the selected files, so use this value instead
 			 */
-			int index = fullPath.lastIndexOf ('/');
+			int index = fullPath.lastIndexOf (SEPARATOR);
 			fileName = fullPath.substring (index + 1, fullPath.length ());
 			fileNames = new String [] {fileName};
 		}
 	} else {
-		int index = fullPath.lastIndexOf ('/');
+		int index = fullPath.lastIndexOf (SEPARATOR);
 		fileName = fullPath.substring (index + 1, fullPath.length ());
 		fileNames = new String [] {fileName};
 	}
 
 	// if no file selected then go into the current directory
-	if (fileName.equals("")) {
+	if (fileName.equals("")) { //$NON-NLS-1$
 		int [] argList1 = {OS.XmNdirMask, 0};
 		OS.XtGetValues (dialog, argList1, argList1.length / 2);
 		int directoryHandle = argList1[1];
@@ -294,8 +284,8 @@ int okPressed (int widget, int client, int call) {
 		null,
 		OS.XmCHARSET_TEXT,
 		OS.XmCHARSET_TEXT,
-		null,
-		0,
+		table,
+		table.length,
 		OS.XmOUTPUT_ALL);
 	if (ptr != 0) {
 		int length = OS.strlen (ptr);
@@ -306,8 +296,11 @@ int okPressed (int widget, int client, int call) {
 		filterPath = new String (Converter.mbcsToWcs (null, buffer));
 	}
 	OS.XmStringFree (xmString2);
-	if (filterPath.endsWith("/")) {
-		filterPath = filterPath.substring (0, filterPath.length() - 1);
+	int length = filterPath.length ();
+	if (length > 0) {
+		if (filterPath.charAt (length - 1) == SEPARATOR) {
+			filterPath = filterPath.substring (0, length - 1);
+		}
 	}
 
 	this.fullPath = fullPath;
@@ -330,18 +323,6 @@ int okPressed (int widget, int client, int call) {
  * </ul>
  */
 public String open () {
-
-	/* Get the parent */
-	boolean destroyContext;
-	Display appContext = Display.getCurrent ();
-	if (destroyContext = (appContext == null)) appContext = new Display ();
-	int display = appContext.xDisplay;
-	int parentHandle = appContext.shellHandle;
-	if ((parent != null) && (parent.getDisplay () == appContext)) {
-		if (OS.IsAIX) parent.realizeWidget ();		/* Fix for bug 17507 */
-		parentHandle = parent.shellHandle;
-	}
-
 	/* Compute the dialog title */	
 	/*
 	* Feature in Motif.  It is not possible to set a shell
@@ -349,7 +330,7 @@ public String open () {
 	* to be a single space.
 	*/
 	String string = title;
-	if (string.length () == 0) string = " ";
+	if (string.length () == 0) string = " "; //$NON-NLS-1$
 	/* Use the character encoding for the default locale */
 	byte [] buffer1 = Converter.wcsToMbcs (null, string, true);
 	int xmStringPtr1 = OS.XmStringParseText (
@@ -362,7 +343,7 @@ public String open () {
 		0);
 
 	fullPath = null;
-	fileNames = null;
+	fileNames = new String [0];
 	
 	/* Compute the filter */
 	String mask = FILTER;
@@ -389,10 +370,10 @@ public String open () {
 		0);
 
 	/* Compute the filter path */
-	if (filterPath == null) filterPath = "";
-	if (!filterPath.endsWith ("/")) {
-		File dir = new File (filterPath);
-		if (dir.exists () && dir.isDirectory ()) filterPath += '/';
+	if (filterPath == null) filterPath = ""; //$NON-NLS-1$
+	int length = filterPath.length ();
+	if (length == 0 || filterPath.charAt (length - 1) != SEPARATOR) {
+		filterPath += SEPARATOR;
 	}
 	/* Use the character encoding for the default locale */
 	byte [] buffer3 = Converter.wcsToMbcs (null, filterPath, true);
@@ -406,16 +387,25 @@ public String open () {
 		0);
 
 	/* Create the dialog */
+	boolean defaultPos = parent.isVisible ();
+	Display display = parent.display;
 	int [] argList1 = {
 		OS.XmNresizePolicy, OS.XmRESIZE_NONE,
 		OS.XmNdialogStyle, OS.XmDIALOG_PRIMARY_APPLICATION_MODAL,
-		OS.XmNwidth, OS.XDisplayWidth (display, OS.XDefaultScreen (display)) * 4 / 9,
+		OS.XmNwidth, OS.XDisplayWidth (display.xDisplay, OS.XDefaultScreen (display.xDisplay)) * 4 / 9,
 		OS.XmNpathMode, OS.XmPATH_MODE_FULL,
 		OS.XmNdialogTitle, xmStringPtr1,
 		OS.XmNpattern, xmStringPtr2,
 		OS.XmNdirMask, xmStringPtr3,
+		OS.XmNdefaultPosition, defaultPos ? 1 : 0,
 	};
 	/*
+	* Bug in AIX. The dialog does not respond to input, if the parent
+	* is not realized.  The fix is to realize the parent.  
+	*/
+	if (OS.IsAIX) parent.realizeWidget ();
+	int parentHandle = parent.shellHandle;
+	/*
 	* Feature in Linux.  For some reason, the XmCreateFileSelectionDialog()
 	* will not accept NULL for the widget name.  This works fine on the other
 	* Motif platforms and in the other XmCreate calls on Linux.  The fix is
@@ -428,36 +418,80 @@ public String open () {
 	OS.XmStringFree (xmStringPtr1);
 	OS.XmStringFree (xmStringPtr2);
 	OS.XmStringFree (xmStringPtr3);
+	
+	/*
+	 * Can override the selection text field if necessary now that
+	 * its initial value has been computed by the platform dialog.
+	 */
+	if (fileName != null && fileName.length() > 0) {
+		/* Use the character encoding for the default locale */
+		byte [] buffer4 = Converter.wcsToMbcs (null, fileName, true);
+		int xmStringPtr4 = OS.XmStringParseText (
+				buffer4,
+				0,
+				OS.XmFONTLIST_DEFAULT_TAG, 
+				OS.XmCHARSET_TEXT, 
+				null,
+				0,
+				0);
+		int [] argList2 = {OS.XmNdirSpec, 0};
+		OS.XtGetValues (dialog, argList2, argList2.length / 2);
+		int oldDirSpec = argList2 [1];
+		int newDirSpec = OS.XmStringConcat (oldDirSpec, xmStringPtr4);
+		argList2 [1] = newDirSpec;
+		OS.XtSetValues (dialog, argList2, argList2.length / 2);
+		OS.XmStringFree (xmStringPtr4);
+		OS.XmStringFree (oldDirSpec);
+		OS.XmStringFree (newDirSpec);
+	}
 
 	/* Hook the callbacks. */
-	Callback cancelCallback = new Callback (this, "cancelPressed", 3);
+	Callback cancelCallback = new Callback (this, "cancelPressed", 3); //$NON-NLS-1$
 	int cancelAddress = cancelCallback.getAddress ();
+	if (cancelAddress == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
 	OS.XtAddCallback (dialog, OS.XmNcancelCallback, cancelAddress, 0);
-	Callback okCallback = new Callback (this, "okPressed", 3);
+	Callback okCallback = new Callback (this, "okPressed", 3); //$NON-NLS-1$
 	int okAddress = okCallback.getAddress ();
+	if (okAddress == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
 	OS.XtAddCallback (dialog, OS.XmNokCallback, okAddress, 0);
 	Callback selectCallback = null;
 	if ((style & SWT.MULTI) != 0) {
 		child = OS.XmFileSelectionBoxGetChild (dialog, OS.XmDIALOG_LIST);
 		if (child != 0) {
-			int [] argList2 = {OS.XmNselectionPolicy, OS.XmEXTENDED_SELECT};
-			OS.XtSetValues(child, argList2, argList2.length / 2);
-			selectCallback = new Callback (this, "itemSelected", 3);
+			int [] argList3 = {OS.XmNselectionPolicy, OS.XmEXTENDED_SELECT};
+			OS.XtSetValues(child, argList3, argList3.length / 2);
+			selectCallback = new Callback (this, "itemSelected", 3); //$NON-NLS-1$
 			int selectAddress = selectCallback.getAddress ();
 			if (selectAddress == 0) error (SWT.ERROR_NO_MORE_CALLBACKS);
 			OS.XtAddCallback (child, OS.XmNextendedSelectionCallback, selectAddress, 0);
 		}
 	}
 
+	if (!defaultPos) {
+		OS.XtRealizeWidget (dialog);
+		int[] argList4 = new int[] {
+			OS.XmNwidth, 0,
+			OS.XmNheight, 0,
+		};
+		OS.XtGetValues (dialog, argList4, argList4.length / 2);
+		Monitor monitor = parent.getMonitor ();
+		Rectangle bounds = monitor.getBounds ();
+		int x = bounds.x + (bounds.width - argList4 [1]) / 2;
+		int y = bounds.y + (bounds.height - argList4 [3]) / 2;
+		int[] argList5 = new int[] {
+			OS.XmNx, x,
+			OS.XmNy, y,
+		};
+		OS.XtSetValues (dialog, argList5, argList5.length / 2);
+	}
 	OS.XtManageChild (dialog);
 
 	// Should be a pure OS message loop (no SWT AppContext)
 	while (OS.XtIsRealized (dialog) && OS.XtIsManaged (dialog))
-		if (!appContext.readAndDispatch ()) appContext.sleep ();
+		if (!display.readAndDispatch ()) display.sleep ();
 
 	/* Destroy the dialog and update the display. */
 	if (OS.XtIsRealized (dialog)) OS.XtDestroyWidget (dialog);
-	if (destroyContext) appContext.dispose ();
 	okCallback.dispose ();
 	cancelCallback.dispose ();
 	if (selectCallback != null) selectCallback.dispose ();
@@ -480,19 +514,34 @@ public void setFileName (String string) {
  * Set the file extensions which the dialog will
  * use to filter the files it shows to the argument,
  * which may be null.
+ * <p>
+ * The strings are platform specific. For example, on
+ * Windows, an extension filter string is typically of
+ * the form "*.extension", where "*.*" matches all files.
+ * </p>
  *
  * @param extensions the file extension filter
+ * 
+ * @see #setFilterNames to specify the user-friendly
+ * names corresponding to the extensions
  */
 public void setFilterExtensions (String [] extensions) {
 	filterExtensions = extensions;
 }
 
 /**
- * Sets the file names which the dialog will
- * use to filter the files it shows to the argument,
- * which may be null.
+ * Sets the the names that describe the filter extensions
+ * which the dialog will use to filter the files it shows
+ * to the argument, which may be null.
+ * <p>
+ * Each name is a user-friendly short description shown for
+ * its corresponding filter. The <code>names</code> array must
+ * be the same length as the <code>extensions</code> array.
+ * </p>
  *
- * @param names the file name filter
+ * @param names the list of filter names, or null for no filter names
+ * 
+ * @see #setFilterExtensions
  */
 public void setFilterNames (String [] names) {
 	filterNames = names;
@@ -502,7 +551,14 @@ public void setFilterNames (String [] names) {
  * Sets the directory path that the dialog will use
  * to the argument, which may be null. File names in this
  * path will appear in the dialog, filtered according
- * to the filter extensions.
+ * to the filter extensions. If the string is null,
+ * then the operating system's default filter path
+ * will be used.
+ * <p>
+ * Note that the path string is platform dependent.
+ * For convenience, either '/' or '\' can be used
+ * as a path separator.
+ * </p>
  *
  * @param string the directory path
  * 
