@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -227,7 +227,7 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 	return new Point (width, height);
 }
 void createHandle (int index) {
-	state |= HANDLE;
+	state |= THEME_BACKGROUND;
 	int parentHandle = parent.handle;
 	int borderWidth = (style & SWT.BORDER) != 0 ? 1 : 0;
 	if ((style & SWT.SEPARATOR) != 0) {
@@ -404,6 +404,16 @@ void realizeChildren () {
 	super.realizeChildren ();
 	setBitGravity ();
 }
+void redrawWidget (int x, int y, int width, int height, boolean redrawAll, boolean allChildren, boolean trim) {
+	super.redrawWidget (x, y, width, height, redrawAll, allChildren, trim);
+	if (formHandle != 0) { 
+		short [] root_x = new short [1], root_y = new short [1];
+		OS.XtTranslateCoords (handle, (short) x, (short) y, root_x, root_y);
+		short [] label_x = new short [1], label_y = new short [1];
+		OS.XtTranslateCoords (formHandle, (short) 0, (short) 0, label_x, label_y);
+		redrawHandle (root_x [0] - label_x [0], root_y [0] - label_y [0], width, height, redrawAll, formHandle);
+	}
+}
 void register () {
 	super.register ();
 	if (formHandle != 0) display.addWidget (formHandle, this);
@@ -482,8 +492,8 @@ void setBitmap (Image image) {
 		if (image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
 		switch (image.type) {
 			case SWT.BITMAP:
-				ImageData data = image.getImageData ();
-				if (data.alpha == -1 && data.alphaData == null && data.transparentPixel == -1) {
+				ImageData data;
+				if (image.mask == 0	&& (data = image.getImageData ()).alpha == -1 && data.alphaData == null && data.transparentPixel == -1) {
 					labelPixmap = image.pixmap;
 					disabled = new Image (display, image, SWT.IMAGE_DISABLE);
 					labelInsensitivePixmap = disabled.pixmap;
@@ -571,6 +581,10 @@ public void setImage (Image image) {
 	if ((style & SWT.SEPARATOR) != 0) return;
 	setBitmap (this.image = image);
 }
+void setParentBackground () {
+	super.setParentBackground ();
+	if (formHandle != 0) setParentBackground (formHandle);
+}
 /**
  * Sets the receiver's text.
  * <p>
@@ -578,14 +592,14 @@ public void setImage (Image image) {
  * the mnemonic character and line delimiters.
  * </p>
  * <p>
- * Mnemonics are indicated by an '&amp' that causes the next
+ * Mnemonics are indicated by an '&amp;' that causes the next
  * character to be the mnemonic.  When the user presses a
  * key sequence that matches the mnemonic, focus is assigned
  * to the control that follows the label. On most platforms,
  * the mnemonic appears underlined but may be emphasised in a
  * platform specific manner.  The mnemonic indicator character
- *'&amp' can be escaped by doubling it in the string, causing
- * a single '&amp' to be displayed.
+ * '&amp;' can be escaped by doubling it in the string, causing
+ * a single '&amp;' to be displayed.
  * </p>
  * 
  * @param string the new text
