@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -23,7 +23,7 @@ import org.eclipse.swt.events.*;
  * <p>
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>CENTER, LEFT, MULTI, SINGLE, RIGHT, READ_ONLY, WRAP</dd>
+ * <dd>CENTER, LEFT, MULTI, PASSWORD, SINGLE, RIGHT, READ_ONLY, WRAP</dd>
  * <dt><b>Events:</b></dt>
  * <dd>DefaultSelection, Modify, Verify</dd>
  * </dl>
@@ -37,11 +37,23 @@ public class Text extends Scrollable {
 	char echoCharacter;
 	boolean ignoreChange;
 	String hiddenText;
-	XmTextVerifyCallbackStruct textVerify;
 	int drawCount;
 	
 	static final boolean IsGB18030;
+	/**
+	* The maximum number of characters that can be entered
+	* into a text widget.
+	* <p>
+	* Note that this value is platform dependent, based upon
+	* the native widget implementation.
+	* </p>
+	*/
 	public static final int LIMIT;
+	/**
+	* The delimiter used by multi-line text widgets.  When text
+	* is queried and from the widget, it will be delimited using
+	* this delimiter.
+	*/
 	public static final String DELIMITER;
 	
 	/*
@@ -181,7 +193,7 @@ public void addVerifyListener (VerifyListener listener) {
  * @param string the string to be appended
  *
  * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ *    <li>ERROR_NULL_ARGUMENT - if the string is null</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -193,7 +205,6 @@ public void append (String string) {
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	int position = OS.XmTextGetLastPosition (handle);
 	byte [] buffer = Converter.wcsToMbcs (getCodePage (), string, true);
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	OS.XmTextSetInsertionPosition (handle, position);
@@ -203,13 +214,18 @@ public void append (String string) {
 	display.setWarnings(warnings);
 }
 static int checkStyle (int style) {
+	if ((style & SWT.SINGLE) != 0 && (style & SWT.MULTI) != 0) {
+		style &= ~SWT.MULTI;
+	}
 	style = checkBits (style, SWT.LEFT, SWT.CENTER, SWT.RIGHT, 0, 0, 0);
 	if ((style & SWT.SINGLE) != 0) style &= ~(SWT.H_SCROLL | SWT.V_SCROLL | SWT.WRAP);
-	if ((style & SWT.WRAP) != 0) style |= SWT.MULTI;
-	if ((style & (SWT.SINGLE | SWT.MULTI)) != 0) return style;
-	if ((style & (SWT.H_SCROLL | SWT.V_SCROLL)) != 0) {
-		return style | SWT.MULTI;
+	if ((style & SWT.WRAP) != 0) {
+		style |= SWT.MULTI;
+		style &= ~SWT.H_SCROLL;
 	}
+	if ((style & SWT.MULTI) != 0) style &= ~SWT.PASSWORD;
+	if ((style & (SWT.SINGLE | SWT.MULTI)) != 0) return style;
+	if ((style & (SWT.H_SCROLL | SWT.V_SCROLL)) != 0) return style | SWT.MULTI;
 	return style | SWT.SINGLE;
 }
 /**
@@ -248,6 +264,13 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		} else {
 			byte [] buffer = new byte [size + 1];
 			OS.memmove (buffer, ptr, size);
+			boolean wrap = (style & SWT.MULTI) != 0 && (style & SWT.WRAP) != 0;
+			if (wrap && wHint != SWT.DEFAULT) {
+				char[] chars = Converter.mbcsToWcs (getCodePage (), buffer);
+				String text = new String (chars);
+				String wrapped = display.wrapText (text, font, wHint);
+				buffer = Converter.wcsToMbcs (getCodePage (), wrapped, true);
+			}
 			int xmString;
 			if ((style & SWT.SINGLE) != 0) {
 				xmString = OS.XmStringParseText (
@@ -278,6 +301,11 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		}
 		OS.XtFree (ptr);
 	}
+	Rectangle trim = computeTrim (0, 0, width, height);
+	return new Point (trim.width, trim.height);
+}
+public Rectangle computeTrim (int x, int y, int width, int height) {
+	checkWidget();
 	if (horizontalBar != null) {
 		int [] argList1 = {OS.XmNheight, 0};
 		OS.XtGetValues (horizontalBar.handle, argList1, argList1.length / 2);
@@ -288,23 +316,28 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		OS.XtGetValues (verticalBar.handle, argList1, argList1.length / 2);
 		width += argList1 [1] + 4;
 	}
+	if ((style & SWT.MULTI) != 0) height+=4;
 	XRectangle rect = new XRectangle ();
 	OS.XmWidgetGetDisplayRect (handle, rect);
-	width += rect.x * 2;  height += rect.y * 2;
-	if ((style & (SWT.MULTI | SWT.BORDER)) != 0) height++;
-	return new Point (width, height);
-}
-public Rectangle computeTrim (int x, int y, int width, int height) {
-	checkWidget();
-	Rectangle trim = super.computeTrim(x, y, width, height);
-	XRectangle rect = new XRectangle ();
-	OS.XmWidgetGetDisplayRect (handle, rect);
-	trim.x -= rect.x;
-	trim.y -= rect.y;
-	trim.width += rect.x;
-	trim.height += rect.y;	
-	if ((style & (SWT.MULTI | SWT.BORDER)) != 0) trim.height += 3;
-	return trim;
+	x -= rect.x;
+	y -= rect.y;
+	width += rect.x * 2;
+	height += rect.y * 2;
+	int shadow = 0, highlight = 0;
+	if ((style & SWT.MULTI) != 0 || (style & SWT.BORDER) != 0) {
+		int [] argList = new int [] {
+			OS.XmNshadowThickness, 0,
+			OS.XmNhighlightThickness, 0,
+		};
+		OS.XtGetValues (handle, argList, argList.length / 2);
+		shadow = argList [1];
+		highlight = argList [3];
+	}
+	x -= shadow + highlight;
+	y -= shadow + highlight;
+	width += (shadow + highlight) * 2;
+	height += (shadow + highlight) * 2;	
+	return new Rectangle (x, y, width, height);
 }
 /**
  * Copies the selected text.
@@ -324,7 +357,6 @@ public void copy () {
 	OS.XmTextCopy (handle, OS.XtLastTimestampProcessed (xDisplay));
 }
 void createHandle (int index) {
-	state |= HANDLE;
 	int [] argList1 = {
 		OS.XmNverifyBell, 0,
 		OS.XmNeditMode, (style & SWT.SINGLE) != 0 ? OS.XmSINGLE_LINE_EDIT : OS.XmMULTI_LINE_EDIT,
@@ -343,29 +375,40 @@ void createHandle (int index) {
 		if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 		int [] argList2 = new int [] {OS.XmNcursorPositionVisible, 0};
 		OS.XtSetValues (handle, argList2, argList2.length / 2);
-		if ((style & SWT.BORDER) == 0) {
-			int [] argList3 = new int [] {
-				/*
-				* Bug in Motif.  Setting the margin width to zero for
-				* a single line text field causes the field to draw
-				* garbage when the caret is placed at the start of
-				* the widget.  The fix is to not set the margin width.
-				*/
-//				OS.XmNmarginWidth, 0,
-				OS.XmNmarginHeight, 0,
-				OS.XmNshadowThickness, 0,
-			};
-			OS.XtSetValues (handle, argList3, argList3.length / 2);
-		}
 	} else {
 		handle = OS.XmCreateScrolledText (parentHandle, new byte [1], argList1, argList1.length / 2);
 		if (handle == 0) error (SWT.ERROR_NO_HANDLES);
 		scrolledHandle = OS.XtParent (handle);
-	}	
+	}
+	if ((style & SWT.BORDER) == 0) {
+		int [] argList3 = new int [] {
+			/*
+			* Bug in Motif.  Setting the margin width to zero for
+			* a single line text field causes the field to draw
+			* garbage when the caret is placed at the start of
+			* the widget.  The fix is to not set the margin width.
+			*/
+//			OS.XmNmarginWidth, 0,
+			OS.XmNmarginHeight, 0,
+			OS.XmNshadowThickness, 0,
+		};
+		OS.XtSetValues (handle, argList3, argList3.length / 2);
+	}
+	/*
+	* Feature in Motif.  The Text widget is created with a default
+	* drop target.  This is inconsistent with other platforms.
+	* To be consistent, disable the default drop target.
+	*/
+	OS.XmDropSiteUnregister (handle);
 }
 ScrollBar createScrollBar (int type) {
 	return createStandardBar (type);
 }
+void createWidget (int index) {
+	super.createWidget (index);	
+	hiddenText = "";
+	if ((style & SWT.PASSWORD) != 0) setEchoChar ('*');
+}
 /**
  * Cuts the selected text.
  * <p>
@@ -373,9 +416,6 @@ ScrollBar createScrollBar (int type) {
  * clipboard and then deleted from the widget.
  * </p>
  *
- * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -388,25 +428,35 @@ public void cut () {
 	OS.XmTextCut (handle, OS.XtLastTimestampProcessed (xDisplay));
 }
 int defaultBackground () {
-	return getDisplay ().textBackground;
+	return display.textBackground;
 }
 Font defaultFont () {
-	return getDisplay ().textFont;
+	return display.textFont;
 }
 int defaultForeground () {
-	return getDisplay ().textForeground;
+	return display.textForeground;
+}
+boolean dragDetect (int x, int y) {
+	if (hooks (SWT.DragDetect)) {
+		int [] start = new int [1], end = new int [1];
+		OS.XmTextGetSelectionPosition (handle, start, end);
+		if (start [0] == end [0]) return false;
+		int pos = OS.XmTextXYToPos(handle, (short) x, (short) y);
+		return pos > start [0] && pos < end [0];
+	}
+	return false;
+}
+boolean dragOverride () {
+	return true;
 }
 /**
- * Gets the line number of the caret.
+ * Returns the line number of the caret.
  * <p>
  * The line number of the caret is returned.
  * </p>
  *
  * @return the line number
  *
- * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -417,7 +467,8 @@ public int getCaretLineNumber () {
 	return getLineNumber (OS.XmTextGetInsertionPosition (handle));
 }
 /**
- * Gets the location the caret.
+ * Returns a point describing the receiver's location relative
+ * to its parent (or its display if its parent is null).
  * <p>
  * The location of the caret is returned.
  * </p>
@@ -432,19 +483,15 @@ public int getCaretLineNumber () {
 public Point getCaretLocation () {
 	checkWidget();
 	int position;
-	if (textVerify != null) {
-		position = textVerify.currInsert;
-	} else {
-		position = OS.XmTextGetInsertionPosition (handle);
-	}
+	position = OS.XmTextGetInsertionPosition (handle);
 	short [] x = new short [1], y = new short [1];
 	OS.XmTextPosToXY (handle, position, x, y);
 	return new Point (x [0], y [0] - getFontAscent (font.handle));
 }
 /**
- * Gets the position of the caret.
+ * Returns the character position of the caret.
  * <p>
- * The character position of the caret is returned.
+ * Indexing is zero based.
  * </p>
  *
  * @return the position of the caret
@@ -459,7 +506,7 @@ public int getCaretPosition () {
 	return OS.XmTextGetInsertionPosition (handle);
 }
 /**
- * Gets the number of characters.
+ * Returns the number of characters.
  *
  * @return number of characters in the widget
  *
@@ -473,12 +520,14 @@ public int getCharCount () {
 	return OS.XmTextGetLastPosition (handle);
 }
 /**
- * Gets the double click enabled flag.
+ * Returns the double click enabled flag.
  * <p>
  * The double click flag enables or disables the
  * default action of the text widget when the user
  * double clicks.
  * </p>
+ * 
+ * @return whether or not double click is enabled
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -492,25 +541,31 @@ public boolean getDoubleClickEnabled () {
 	return argList [1] != 1;
 }
 /**
- * Gets the echo character.
+ * Returns the echo character.
  * <p>
  * The echo character is the character that is
  * displayed when the user enters text or the
  * text is changed by the programmer.
  * </p>
+ * 
+ * @return the echo character
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #setEchoChar
  */
 public char getEchoChar () {
 	checkWidget();
 	return echoCharacter;
 }
 /**
- * Gets the editable state.
+ * Returns the editable state.
  *
+ * @return whether or not the receiver is editable
+ * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -530,7 +585,7 @@ public boolean getEditable () {
 	return argList [1] != 0;
 }
 /**
- * Gets the number of lines.
+ * Returns the number of lines.
  *
  * @return the number of lines in the widget
  *
@@ -546,7 +601,7 @@ public int getLineCount () {
 	return getLineNumber (lastChar) + 1;
 }
 /**
- * Gets the line delimiter.
+ * Returns the line delimiter.
  *
  * @return a string that is the line delimiter
  *
@@ -554,13 +609,15 @@ public int getLineCount () {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #DELIMITER
  */
 public String getLineDelimiter () {
 	checkWidget();
 	return "\n";
 }
 /**
- * Gets the height of a line.
+ * Returns the height of a line.
  *
  * @return the height of a row of text
  *
@@ -571,7 +628,7 @@ public String getLineDelimiter () {
  */
 public int getLineHeight () {
 	checkWidget();
-	return getFontHeight ();
+	return getFontHeight (font.handle);
 }
 int getLineNumber (int position) {
 	if (position == 0) return 0;
@@ -628,9 +685,10 @@ int getNavigationType () {
 	return buffer [0];
 }
 /**
- * Returns the orientation of the receiver.
+ * Returns the orientation of the receiver, which will be one of the
+ * constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.RIGHT_TO_LEFT</code>.
  *
- * @return the orientation bit.
+ * @return the orientation style
  * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -644,14 +702,17 @@ public int getOrientation () {
 	return style & (SWT.LEFT_TO_RIGHT | SWT.RIGHT_TO_LEFT);
 }
 /**
- * Gets the position of the selected text.
+ * Returns a <code>Point</code> whose x coordinate is the
+ * character position representing the start of the selected
+ * text, and whose y coordinate is the character position
+ * representing the end of the selection. An "empty" selection
+ * is indicated by the x and y coordinates having the same value.
  * <p>
- * Indexing is zero based.  The range of
- * a selection is from 0..N where N is
- * the number of characters in the widget.
+ * Indexing is zero based.  The range of a selection is from
+ * 0..N where N is the number of characters in the widget.
  * </p>
- * 
- * @return the start and end of the selection
+ *
+ * @return a point representing the selection start and end
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -660,9 +721,6 @@ public int getOrientation () {
  */
 public Point getSelection () {
 	checkWidget();
-	if (textVerify != null) {
-		return new Point (textVerify.startPos, textVerify.endPos);
-	}
 	int [] start = new int [1], end = new int [1];
 	OS.XmTextGetSelectionPosition (handle, start, end);
 	if (start [0] == end [0]) {
@@ -671,7 +729,7 @@ public Point getSelection () {
 	return new Point (start [0], end [0]);
 }
 /**
- * Gets the number of selected characters.
+ * Returns the number of selected characters.
  *
  * @return the number of selected characters.
  *
@@ -682,15 +740,12 @@ public Point getSelection () {
  */
 public int getSelectionCount () {
 	checkWidget();
-	if (textVerify != null) {
-		return textVerify.endPos - textVerify.startPos;
-	}
 	int [] start = new int [1], end = new int [1];
 	OS.XmTextGetSelectionPosition (handle, start, end);
 	return end [0] - start [0];
 }
 /**
- * Gets the selected text.
+ * Gets the selected text, or an empty string if there is no current selection.
  *
  * @return the selected text
  * 
@@ -701,7 +756,7 @@ public int getSelectionCount () {
  */
 public String getSelectionText () {
 	checkWidget();
-	if (echoCharacter != '\0' || textVerify != null) {
+	if (echoCharacter != '\0') {
 		Point selection = getSelection ();
 		return getText (selection.x, selection.y);
 	}
@@ -714,7 +769,7 @@ public String getSelectionText () {
 	return new String (Converter.mbcsToWcs (getCodePage (), buffer));
 }
 /**
- * Gets the number of tabs.
+ * Returns the number of tabs.
  * <p>
  * Tab stop spacing is specified in terms of the
  * space (' ') character.  The width of a single
@@ -734,9 +789,10 @@ public int getTabs () {
 	return 8;
 }
 /**
- * Gets the widget text.
+ * Returns the widget text.
  * <p>
- * The text for a text widget is the characters in the widget.
+ * The text for a text widget is the characters in the widget, or
+ * an empty string if this has never been set.
  * </p>
  *
  * @return the widget text
@@ -758,7 +814,8 @@ public String getText () {
 	return new String (Converter.mbcsToWcs (getCodePage (), buffer));
 }
 /**
- * Gets a range of text.
+ * Returns a range of text.  Returns an empty string if the
+ * start of the range is greater than the end.
  * <p>
  * Indexing is zero based.  The range of
  * a selection is from 0..N-1 where N is
@@ -776,19 +833,23 @@ public String getText () {
  */
 public String getText (int start, int end) {
 	checkWidget();
+	if (!(start <= end && 0 <= end)) return "";
+	boolean hasEcho = echoCharacter != '\0';
+	int length = hasEcho ? hiddenText.length () : OS.XmTextGetLastPosition (handle);
+	if (length == 0) return "";
+	start = Math.max (0, start);
+	end = Math.min (end, length - 1);
+	if (hasEcho) return hiddenText.substring (start, end + 1);
 	int numChars = end - start + 1;
-	if (numChars < 0 || start < 0) return "";
-	if (echoCharacter != '\0') {
-		return hiddenText.substring (start, Math.min (hiddenText.length (), end));
+	int bufLength = numChars * OS.MB_CUR_MAX () + 1;
+	byte [] buffer = new byte [bufLength];
+	int code = OS.XmTextGetSubstring (handle, start, numChars, bufLength, buffer);
+	switch (code) {
+		case OS.XmCOPY_FAILED:
+		case OS.XmCOPY_TRUNCATED:
+			error (SWT.ERROR_CANNOT_GET_TEXT);
 	}
-	int length = (numChars * 4 /* MB_CUR_MAX */) + 1;
-	byte [] buffer = new byte [length];
-	int code = OS.XmTextGetSubstring (handle, start, numChars, length, buffer);
-	if (code == OS.XmCOPY_FAILED) return "";
 	char [] unicode = Converter.mbcsToWcs (getCodePage (), buffer);
-	if (code == OS.XmCOPY_TRUNCATED) {
-		numChars = OS.XmTextGetLastPosition (handle) - start;
-	}
 	return new String (unicode, 0, numChars);
 }
 /**
@@ -804,6 +865,8 @@ public String getText (int start, int end) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #LIMIT
  */
 public int getTextLimit () {
 	checkWidget();
@@ -835,7 +898,7 @@ public int getTopIndex () {
 	return argList2 [1];
 }
 /**
- * Gets the top pixel.
+ * Returns the top pixel.
  * <p>
  * The top pixel is the pixel position of the line
  * that is currently at the top of the widget.  On
@@ -863,7 +926,7 @@ boolean hasIMSupport() {
 }
 void hookEvents () {
 	super.hookEvents ();
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	OS.XtAddCallback (handle, OS.XmNactivateCallback, windowProc, ACTIVATE_CALLBACK);
 	OS.XtAddCallback (handle, OS.XmNvalueChangedCallback, windowProc, VALUE_CHANGED_CALLBACK);
 	OS.XtAddCallback (handle, OS.XmNmodifyVerifyCallback, windowProc, MODIFY_VERIFY_CALLBACK);
@@ -880,6 +943,9 @@ int inputContext () {
  *
  * @param string the string
  *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the string is <code>null</code></li>
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -894,17 +960,15 @@ public void insert (String string) {
 		start [0] = end [0] = OS.XmTextGetInsertionPosition (handle);
 	}
 	byte [] buffer = Converter.wcsToMbcs (getCodePage (), string, true);
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	OS.XmTextReplace (handle, start [0], end [0], buffer);
-	int position = start [0] + buffer.length - 1;
+	int position = start [0] + Math.max (0, buffer.length - 1);
 	OS.XmTextSetInsertionPosition (handle, position);
 	display.setWarnings (warnings);
 }
 void overrideTranslations () {
 	if ((style & SWT.SINGLE) != 0) {
-		Display display = getDisplay ();
 		OS.XtOverrideTranslations (handle, display.tabTranslations);
 	}
 }
@@ -922,7 +986,6 @@ void overrideTranslations () {
  */
 public void paste () {
 	checkWidget();
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	/*
@@ -941,7 +1004,6 @@ public void paste () {
 void releaseWidget () {
 	super.releaseWidget ();
 	hiddenText = null;
-	textVerify = null;
 }
 /**
  * Removes the listener from the collection of listeners who will
@@ -970,7 +1032,7 @@ public void removeModifyListener (ModifyListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -994,7 +1056,7 @@ public void removeSelectionListener(SelectionListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is verified.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -1013,37 +1075,8 @@ public void removeVerifyListener (VerifyListener listener) {
 	if (eventTable == null) return;
 	eventTable.unhook (SWT.Verify, listener);	
 }
-void sendIMKeyEvent (int type, XKeyEvent xEvent, byte [] mbcs, char [] chars) {
-	/*
-	* Bug in Motif. On Solaris and Linux, XmImMbLookupString() clears
-	* the characters from the IME. This causes the characters to be
-	* stolen from the text widget. The fix is to detect that the IME
-	* has been cleared and use XmTextInsert() to insert the stolen
-	* characters. This problem does not happen on AIX.
-	*/
-	super.sendIMKeyEvent (type, xEvent, mbcs, chars);
-	if (mbcs == null || xEvent.keycode != 0) return;
-	int [] unused = new int [1];
-	byte [] buffer = new byte [2];
-	int length = OS.XmImMbLookupString (handle, xEvent, buffer, buffer.length, unused, unused);
-	if (length != 0) return;
-	int [] start = new int [1], end = new int [1];
-	OS.XmTextGetSelectionPosition (handle, start, end);
-	if (start [0] == end [0]) {
-		start [0] = end [0] = OS.XmTextGetInsertionPosition (handle);
-	}
-	Display display = getDisplay();
-	boolean warnings = display.getWarnings ();
-	display.setWarnings (false);
-	OS.XmTextReplace (handle, start [0], end [0], mbcs);
-	int index = 0;
-	while (index < chars.length) {
-		if (chars [index] == 0) break;
-		index++;
-	}
-	int position = start [0] + index;
-	OS.XmTextSetInsertionPosition (handle, position);
-	display.setWarnings (warnings);
+boolean sendIMKeyEvent (int type, XKeyEvent xEvent) {
+	return super.sendIMKeyEvent (type, xEvent, handle);
 }
 /**
  * Selects all the text in the receiver.
@@ -1072,7 +1105,6 @@ public void selectAll () {
 	/* Set the selection. */
 	int xDisplay = OS.XtDisplay (handle);
 	if (xDisplay == 0) return;
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	OS.XmTextSetSelection (handle, 0, position, OS.XtLastTimestampProcessed (xDisplay));
@@ -1088,26 +1120,10 @@ boolean setBounds (int x, int y, int width, int height, boolean move, boolean re
 	* the visible region during a resize.  The fix is
 	* to temporarily turn off warnings below.
 	*/
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	boolean changed = super.setBounds (x, y, width, height, move, resize);
 	display.setWarnings(warnings);
-	
-	/*
-	* Bug in Motif.  When the receiver is a Text widget
-	* (not a Text Field) and is resized to be smaller than
-	* the inset that surrounds the text and the selection
-	* is set, the receiver scrolls to the left.  When the
-	* receiver is resized larger, the text is not scrolled
-	* back.  The fix is to detect this case and scroll the
-	* text back.
-	*/
-//	inset := self inset.
-//	nWidth := self dimensionAt: XmNwidth.
-//	self noWarnings: [super resizeWidget].
-//	nWidth > inset x ifTrue: [^self].
-//	self showPosition: self topCharacter
 	return changed;
 }
 /**
@@ -1116,6 +1132,9 @@ boolean setBounds (int x, int y, int width, int height, boolean move, boolean re
  * The double click flag enables or disables the
  * default action of the text widget when the user
  * double clicks.
+ * </p><p>
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept.
  * </p>
  * 
  * @param doubleClick the new double click flag
@@ -1139,8 +1158,9 @@ public void setDoubleClickEnabled (boolean doubleClick) {
  * the echo character to '\0' clears the echo
  * character and redraws the original text.
  * If for any reason the echo character is invalid,
- * the default echo character for the platform
- * is used.
+ * or if the platform does not allow modification
+ * of the echo character, the default echo character
+ * for the platform is used.
  * </p>
  *
  * @param echo the new echo character
@@ -1152,11 +1172,12 @@ public void setDoubleClickEnabled (boolean doubleClick) {
  */
 public void setEchoChar (char echo) {
 	checkWidget();
+	if ((style & SWT.MULTI) != 0) return;
 	if (echoCharacter == echo) return;
 	String newText;
 	if (echo == 0) {
 		newText = hiddenText;
-		hiddenText = null;
+		hiddenText = "";
 	} else {
 		newText = hiddenText = getText();
 	}
@@ -1189,10 +1210,13 @@ public void setEditable (boolean editable) {
 }
 /**
  * Sets the orientation of the receiver, which must be one
- * of the constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.LEFT_TO_RIGHT</code>.
+ * of the constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.RIGHT_TO_LEFT</code>.
  * <p>
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept.
+ * </p>
  *
- * @param orientation new orientation bit
+ * @param orientation new orientation style
  * 
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -1243,7 +1267,6 @@ public void setSelection (int start) {
 	if (xDisplay == 0) return;
 	int position = OS.XmTextGetLastPosition (handle);
 	int nStart = Math.min (Math.max (start, 0), position);
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 //	OS.XmTextSetHighlight (handle, 0, position, OS.XmHIGHLIGHT_NORMAL);
@@ -1254,7 +1277,8 @@ public void setSelection (int start) {
 	display.setWarnings (warnings);
 }
 /**
- * Sets the selection.
+ * Sets the selection to the range specified
+ * by the given start and end indices.
  * <p>
  * Indexing is zero based.  The range of
  * a selection is from 0..N where N is
@@ -1296,19 +1320,26 @@ public void setSelection (int start, int end) {
 	/* Set the selection. */
 	int xDisplay = OS.XtDisplay (handle);
 	if (xDisplay == 0) return;
-	int nStart = Math.min (Math.max (Math.min (start, end), 0), position);
-	int nEnd = Math.min (Math.max (Math.max (start, end), 0), position);
-	Display display = getDisplay ();
+	start = Math.min (Math.max (start, 0), position);
+	end = Math.min (Math.max (end, 0), position);
+	int nStart = Math.min (start, end), nEnd = Math.max (start, end);
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	OS.XmTextSetSelection (handle, nStart, nEnd, OS.XtLastTimestampProcessed (xDisplay));
 
 	/* Force the i-beam to follow the highlight/selection. */
-	OS.XmTextSetInsertionPosition (handle, nEnd);
+	if (start > end) {
+		OS.XmTextSetInsertionPosition (handle, nStart);
+	} else {
+		OS.XmTextSetInsertionPosition (handle, nEnd);
+	}
 	display.setWarnings (warnings);
 }
 /**
- * Sets the selection.
+ * Sets the selection to the range specified
+ * by the given point, where the x coordinate
+ * represents the start index and the y coordinate
+ * represents the end index.
  * <p>
  * Indexing is zero based.  The range of
  * a selection is from 0..N where N is
@@ -1363,7 +1394,7 @@ public void setTabs (int tabs) {
  * SINGLE and the argument contains multiple lines of text, the result of this
  * operation is undefined and may vary from platform to platform.
  *
- * @param text the new text
+ * @param string the new text
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the string is null</li>
@@ -1377,18 +1408,11 @@ public void setText (String string) {
 	checkWidget();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	byte [] buffer = Converter.wcsToMbcs (getCodePage (), string, true);
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	OS.XmTextSetString (handle, buffer);
 	OS.XmTextSetInsertionPosition (handle, 0);
 	display.setWarnings(warnings);
-	/*
-	* Bug in Linux.  When the widget is multi-line
-	* it does not send a Modify to notify the application
-	* that the text has changed.  The fix is to send the event.
-	*/
-	if (OS.IsLinux && (style & SWT.MULTI) != 0) sendEvent (SWT.Modify);
 }
 /**
  * Sets the maximum number of characters that the receiver
@@ -1398,6 +1422,8 @@ public void setText (String string) {
  * creating a read-only text widget.
  * </p><p>
  * To reset this value to the default, use <code>setTextLimit(Text.LIMIT)</code>.
+ * Specifying a limit value larger than <code>Text.LIMIT</code> sets the
+ * receiver's limit to <code>Text.LIMIT</code>.
  * </p>
  *
  * @param limit new text limit
@@ -1409,6 +1435,8 @@ public void setText (String string) {
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
+ * 
+ * @see #LIMIT
  */
 public void setTextLimit (int limit) {
 	checkWidget();
@@ -1446,9 +1474,6 @@ public void setTopIndex (int index) {
  * lines are scrolled until the selection is visible.
  * </p>
  * 
- * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
- * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -1456,7 +1481,6 @@ public void setTopIndex (int index) {
  */
 public void showSelection () {
 	checkWidget();
-	Display display = getDisplay ();
 	boolean warnings = display.getWarnings ();
 	display.setWarnings (false);
 	int position = OS.XmTextGetInsertionPosition (handle);
@@ -1542,6 +1566,7 @@ int XmNmodifyVerifyCallback (int w, int client_data, int call_data) {
 			newText = new String (charBuffer);
 		}
 		if (newText != text) {
+			OS.XtFree(textBlock.ptr);
 			byte [] buffer2 = Converter.wcsToMbcs (codePage, newText, true);
 			int length = buffer2.length;
 			int ptr = OS.XtMalloc (length);
@@ -1552,7 +1577,6 @@ int XmNmodifyVerifyCallback (int w, int client_data, int call_data) {
 		}
 	}
 	OS.memmove (call_data, textVerify, XmTextVerifyCallbackStruct.sizeof);
-	textVerify = null;
 	return result;
 }
 int XmNvalueChangedCallback (int w, int client_data, int call_data) {
