@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -37,11 +37,12 @@ import org.eclipse.swt.events.*;
  */
 public class Tracker extends Widget {
 	Composite parent;
-	Display display;
-	boolean tracking, stippled;
+	boolean tracking, cancelled, stippled;
 	Rectangle [] rectangles, proportions;
+	Rectangle bounds;
 	int cursorOrientation = SWT.NONE;
-	int cursor;
+	int cursor, window, oldX, oldY;
+
 	final static int STEPSIZE_SMALL = 1;
 	final static int STEPSIZE_LARGE = 9;
 
@@ -80,7 +81,6 @@ public class Tracker extends Widget {
 public Tracker (Composite parent, int style) {
 	super (parent, checkStyle (style));
 	this.parent = parent;
-	display = parent.getDisplay ();
 }
 
 /**
@@ -150,30 +150,56 @@ public void addControlListener (ControlListener listener) {
 	checkWidget ();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	TypedListener typedListener = new TypedListener (listener);
-	addListener (SWT.Move,typedListener);
+	addListener (SWT.Resize, typedListener);
+	addListener (SWT.Move, typedListener);
+}
+/**
+ * Adds the listener to the collection of listeners who will
+ * be notified when keys are pressed and released on the system keyboard, by sending
+ * it one of the messages defined in the <code>KeyListener</code>
+ * interface.
+ *
+ * @param listener the listener which should be notified
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see KeyListener
+ * @see #removeKeyListener
+ */
+public void addKeyListener(KeyListener listener) {
+	checkWidget();
+	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
+	TypedListener typedListener = new TypedListener (listener);
+	addListener(SWT.KeyUp,typedListener);
+	addListener(SWT.KeyDown,typedListener);
 }
 
-Point adjustMoveCursor (int xDisplay, int xWindow) {
+Point adjustMoveCursor () {
 	final int unused[] = new int[1];
 	int actualX[] = new int[1];
 	int actualY[] = new int[1];
 	
-	Rectangle bounds = computeBounds ();
 	int newX = bounds.x + bounds.width / 2;
 	int newY = bounds.y;
 	
-	OS.XWarpPointer (xDisplay, OS.None, xWindow, 0, 0, 0, 0, newX, newY);
+	int xDisplay = display.xDisplay;
+	OS.XWarpPointer (xDisplay, OS.None, window, 0, 0, 0, 0, newX, newY);
 	/*
 	 * The call to XWarpPointer does not always place the pointer on the
 	 * exact location that is specified, so do a query (below) to get the
 	 * actual location of the pointer after it has been moved.
 	 */
-	OS.XQueryPointer (xDisplay, xWindow, unused, unused, actualX, actualY, unused, unused, unused);
+	OS.XQueryPointer (xDisplay, window, unused, unused, actualX, actualY, unused, unused, unused);
 	return new Point (actualX[0], actualY[0]);
 }
-Point adjustResizeCursor (int xDisplay, int xWindow) {
+Point adjustResizeCursor () {
 	int newX, newY;
-	Rectangle bounds = computeBounds ();
 
 	if ((cursorOrientation & SWT.LEFT) != 0) {
 		newX = bounds.x;
@@ -194,13 +220,14 @@ Point adjustResizeCursor (int xDisplay, int xWindow) {
 	final int unused[] = new int[1];
 	int actualX[] = new int[1];
 	int actualY[] = new int[1];
-	OS.XWarpPointer (xDisplay, 0, xWindow, 0, 0, 0, 0, newX, newY);
+	int xDisplay = display.xDisplay;
+	OS.XWarpPointer (xDisplay, 0, window, 0, 0, 0, 0, newX, newY);
 	/*
 	 * The call to XWarpPointer does not always place the pointer on the
 	 * exact location that is specified, so do a query (below) to get the
 	 * actual location of the pointer after it has been moved.
 	 */
-	OS.XQueryPointer (xDisplay, xWindow, unused, unused, actualX, actualY, unused, unused, unused);
+	OS.XQueryPointer (xDisplay, window, unused, unused, actualX, actualY, unused, unused, unused);
 	return new Point (actualX[0], actualY[0]);
 }
 static int checkStyle (int style) {
@@ -242,33 +269,38 @@ Rectangle computeBounds () {
 
 Rectangle [] computeProportions (Rectangle [] rects) {
 	Rectangle [] result = new Rectangle [rects.length];
-	Rectangle bounds = computeBounds ();
+	bounds = computeBounds ();
 	for (int i = 0; i < rects.length; i++) {
-		result[i] = new Rectangle (
-			(rects[i].x - bounds.x) * 100 / bounds.width,
-			(rects[i].y - bounds.y) * 100 / bounds.height,
-			rects[i].width * 100 / bounds.width,
-			rects[i].height * 100 / bounds.height);
+		int x = 0, y = 0, width = 0, height = 0;
+		if (bounds.width != 0) {
+			x = (rects [i].x - bounds.x) * 100 / bounds.width;
+			width = rects [i].width * 100 / bounds.width;
+		} else {
+			width = 100;
+		}
+		if (bounds.height != 0) {
+			y = (rects [i].y - bounds.y) * 100 / bounds.height;
+			height = rects [i].height * 100 / bounds.height;
+		} else {
+			height = 100;
+		}
+		result [i] = new Rectangle (x, y, width, height);			
 	}
 	return result;
 }
 
-void drawRectangles () {
-	if (parent != null) {
-		if (parent.isDisposed ()) return;
-		parent.getShell ().update ();
-	} else {
-		display.update ();
-	}
+void drawRectangles (Rectangle [] rects, boolean stippled) {
 	int xDisplay = display.xDisplay;
 	int color = OS.XWhitePixel (xDisplay, 0);
 	int xWindow = OS.XDefaultRootWindow (xDisplay);
 	if (parent != null) {
 		xWindow = OS.XtWindow (parent.handle);
 		if (xWindow == 0) return;
-		int [] argList = {OS.XmNforeground, 0, OS.XmNbackground, 0};
-		OS.XtGetValues (parent.handle, argList, argList.length / 2);
-		color = argList [1] ^ argList [3];
+		int foreground = parent.getForegroundPixel ();
+		Control control = parent.findBackgroundControl ();
+		if (control == null) control = parent;
+		int background = control.getBackgroundPixel ();
+		color = foreground ^ background;
 	}
 	int gc = OS.XCreateGC (xDisplay, xWindow, 0, null);
 	OS.XSetForeground (xDisplay, gc, color);
@@ -282,8 +314,8 @@ void drawRectangles () {
 		OS.XSetFillStyle (xDisplay, gc, OS.FillStippled);
 		OS.XSetLineAttributes (xDisplay, gc, 3, OS.LineSolid, OS.CapButt, OS.JoinMiter);
 	}
-	for (int i=0; i<rectangles.length; i++) {
-		Rectangle rect = rectangles [i];
+	for (int i=0; i<rects.length; i++) {
+		Rectangle rect = rects [i];
 		OS.XDrawRectangle (xDisplay, xWindow, gc, rect.x, rect.y, rect.width, rect.height);
 	}
 	if (stippled) {
@@ -291,9 +323,6 @@ void drawRectangles () {
 	}
 	OS.XFreeGC (xDisplay, gc);
 }
-public Display getDisplay () {
-	return display;
-}
 /**
  * Returns the bounds that are being drawn, expressed relative to the parent
  * widget.  If the parent is a <code>Display</code> then these are screen
@@ -307,8 +336,15 @@ public Display getDisplay () {
  * </ul>
  */
 public Rectangle [] getRectangles () {
-	checkWidget ();
-	return rectangles;
+	checkWidget();
+	int length = 0;
+	if (rectangles != null) length = rectangles.length;
+	Rectangle [] result = new Rectangle [length];
+	for (int i = 0; i < length; i++) {
+		Rectangle current = rectangles [i];
+		result [i] = new Rectangle (current.x, current.y, current.width, current.height);
+	}
+	return result;
 }
 /**
  * Returns <code>true</code> if the rectangles are drawn with a stippled line, <code>false</code> otherwise.
@@ -326,10 +362,12 @@ public boolean getStippled () {
 }
 
 void moveRectangles (int xChange, int yChange) {
-	if (xChange < 0 && ((style & SWT.LEFT) == 0)) return;
-	if (xChange > 0 && ((style & SWT.RIGHT) == 0)) return;
-	if (yChange < 0 && ((style & SWT.UP) == 0)) return;
-	if (yChange > 0 && ((style & SWT.DOWN) == 0)) return;
+	if (xChange < 0 && ((style & SWT.LEFT) == 0)) xChange = 0;
+	if (xChange > 0 && ((style & SWT.RIGHT) == 0)) xChange = 0;
+	if (yChange < 0 && ((style & SWT.UP) == 0)) yChange = 0;
+	if (yChange > 0 && ((style & SWT.DOWN) == 0)) yChange = 0;
+	if (xChange == 0 && yChange == 0) return;
+	bounds.x += xChange; bounds.y += yChange;
 	for (int i = 0; i < rectangles.length; i++) {
 		rectangles [i].x += xChange;
 		rectangles [i].y += yChange;
@@ -352,189 +390,98 @@ public boolean open () {
 	checkWidget ();
 	if (rectangles == null) return false;
 	int xDisplay = display.xDisplay;
-	int xWindow = OS.XDefaultRootWindow (xDisplay);
+	window = OS.XDefaultRootWindow (xDisplay);
 	if (parent != null) {
-		xWindow = OS.XtWindow (parent.handle);
-		if (xWindow == 0) return false;
+		window = OS.XtWindow (parent.handle);
+		if (window == 0) return false;
 	}
-	boolean cancelled = false;
+	cancelled = false;
 	tracking = true;
-	drawRectangles ();
+	update ();
+	drawRectangles (rectangles, stippled);
 	int [] oldX = new int [1], oldY = new int [1];
 	int [] unused = new int [1], mask = new int [1];
-	OS.XQueryPointer (xDisplay, xWindow, unused, unused, oldX, oldY, unused, unused, mask);
+	OS.XQueryPointer (xDisplay, window, unused, unused, oldX, oldY, unused, unused, mask);
+	
+	/*
+	* If exactly one of UP/DOWN is specified as a style then set the cursor
+	* orientation accordingly (the same is done for LEFT/RIGHT styles below).
+	*/
+	int vStyle = style & (SWT.UP | SWT.DOWN);
+	if (vStyle == SWT.UP || vStyle == SWT.DOWN) {
+		cursorOrientation |= vStyle;
+	}
+	int hStyle = style & (SWT.LEFT | SWT.RIGHT);
+	if (hStyle == SWT.LEFT || hStyle == SWT.RIGHT) {
+		cursorOrientation |= hStyle;
+	}
+	
 	Point cursorPos;
 	int mouseMasks = OS.Button1Mask | OS.Button2Mask | OS.Button3Mask;
 	boolean mouseDown = (mask [0] & mouseMasks) != 0;
 	if (!mouseDown) {
 		if ((style & SWT.RESIZE) != 0) {
-			cursorPos = adjustResizeCursor (xDisplay, xWindow);
+			cursorPos = adjustResizeCursor ();
 		} else {
-			cursorPos = adjustMoveCursor (xDisplay, xWindow);
+			cursorPos = adjustMoveCursor ();
 		}
 		oldX [0] = cursorPos.x;  oldY [0] = cursorPos.y;
 	}
+	this.oldX = oldX [0];
+	this.oldY = oldY [0];
 
-	XAnyEvent xEvent = new XAnyEvent ();
-	int [] newX = new int [1], newY = new int [1];
-	int xtContext = OS.XtDisplayToApplicationContext (xDisplay);
-	
-	int ptrGrabResult = OS.XGrabPointer (
-		xDisplay,
-		xWindow,
-		0,
+	int ptrGrabResult = OS.XGrabPointer (xDisplay, window, 0,
 		OS.ButtonPressMask | OS.ButtonReleaseMask | OS.PointerMotionMask,
-		OS.GrabModeAsync,
-		OS.GrabModeAsync,
-		OS.None,
-		OS.None,
-		OS.CurrentTime);
-	int kbdGrabResult = OS.XGrabKeyboard (
-		xDisplay,
-		xWindow,
-		0,
-		OS.GrabModeAsync,
-		OS.GrabModeAsync,
-		OS.CurrentTime);
+		OS.GrabModeAsync, OS.GrabModeAsync, OS.None, OS.None, OS.CurrentTime);
+	int kbdGrabResult = OS.XGrabKeyboard (xDisplay, window, 0,
+		OS.GrabModeAsync, OS.GrabModeAsync, OS.CurrentTime);
 				
-	/*
-	 *  Tracker behaves like a Dialog with its own OS event loop.
-	 */
+	/* Tracker behaves like a Dialog with its own OS event loop. */
+	XAnyEvent anyEvent = new XAnyEvent();
+	int xEvent = OS.XtMalloc (XEvent.sizeof);
+	int dispatch = OS.XtMalloc (4);
+	int xtContext = OS.XtDisplayToApplicationContext (xDisplay);
 	while (tracking) {
 		if (parent != null && parent.isDisposed ()) break;
 		OS.XtAppNextEvent (xtContext, xEvent);
-		switch (xEvent.type) {
-			case OS.MotionNotify:
-				if (cursor != 0) {
-					OS.XChangeActivePointerGrab (
-						xDisplay,
-						OS.ButtonPressMask | OS.ButtonReleaseMask | OS.PointerMotionMask,
-						cursor,
-						OS.CurrentTime);
-				}
-				// fall through
-			case OS.ButtonRelease:
-				OS.XQueryPointer (xDisplay, xWindow, unused, unused, newX, newY, unused, unused, unused);
-				if (oldX [0] != newX [0] || oldY [0] != newY [0]) {
-					drawRectangles ();
-					Event event = new Event ();
-					event.x = newX [0];
-					event.y = newY [0];
-					if ((style & SWT.RESIZE) != 0) {
-						resizeRectangles (newX [0] - oldX [0], newY [0] - oldY [0]);
-						sendEvent (SWT.Resize, event);
-						cursorPos = adjustResizeCursor (xDisplay, xWindow);
-						newX [0] = cursorPos.x; newY [0] = cursorPos.y;
-					} else {
-						moveRectangles (newX [0] - oldX [0], newY [0] - oldY [0]);
-						sendEvent (SWT.Move, event);
-					}
-					/*
-					 * It is possible (but unlikely) that application code
-					 * could have disposed the widget in the move event.
-					 * If this happens then return false to indicate that
-					 * the move failed.
-					 */
-					if (isDisposed ()) {
-						if (ptrGrabResult == OS.GrabSuccess) OS.XUngrabPointer (xDisplay, OS.CurrentTime);
-						if (kbdGrabResult == OS.GrabSuccess) OS.XUngrabKeyboard (xDisplay, OS.CurrentTime);
-						return false;
-					}
-					drawRectangles ();
-					oldX [0] = newX [0];  oldY [0] = newY [0];
-				}
-				tracking = xEvent.type != OS.ButtonRelease;
-				break;
-			case OS.KeyPress:
-				XKeyEvent keyEvent = new XKeyEvent ();
-				OS.memmove (keyEvent, xEvent, XKeyEvent.sizeof);
-				if (keyEvent.keycode != 0) {
-					int [] keysym = new int [1];
-					OS.XLookupString (keyEvent, null, 0, keysym, null);
-					keysym [0] &= 0xFFFF;
-					int xChange = 0, yChange = 0;
-					int stepSize = ((keyEvent.state & OS.ControlMask) != 0) ? STEPSIZE_SMALL : STEPSIZE_LARGE;
-					switch (keysym [0]) {
-						case OS.XK_KP_Enter:
-						case OS.XK_Return:
-							tracking = false;
-							/*
-							 * Eat the subsequent KeyRelease event
-							 */
-							OS.XtAppNextEvent (xtContext, xEvent);
-							break;
-						case OS.XK_Escape:
-						case OS.XK_Cancel:
-							tracking = false;
-							cancelled = true;
-							/*
-							 * Eat the subsequent KeyRelease event
-							 */
-							OS.XtAppNextEvent (xtContext, xEvent);
-							break;
-						case OS.XK_Left:
-							xChange = -stepSize;
-							break;
-						case OS.XK_Right:
-							xChange = stepSize;
-							break;
-						case OS.XK_Up:
-							yChange = -stepSize;
-							break;
-						case OS.XK_Down:
-							yChange = stepSize;
-							break;
-					}
-					if (xChange != 0 || yChange != 0) {
-						drawRectangles ();
-						Event event = new Event ();
-						event.x = oldX[0] + xChange;
-						event.y = oldY[0] + yChange;
-						if ((style & SWT.RESIZE) != 0) {
-							resizeRectangles (xChange, yChange);
-							sendEvent (SWT.Resize, event);
-							cursorPos = adjustResizeCursor (xDisplay, xWindow);
-						} else {
-							moveRectangles (xChange, yChange);
-							sendEvent (SWT.Move, event);
-							cursorPos = adjustMoveCursor (xDisplay, xWindow);
-						}
-						/*
-						 * It is possible (but unlikely) that application code
-						 * could have disposed the widget in the move event.
-						 * If this happens then return false to indicate that
-						 * the move failed.
-						 */
-						if (isDisposed ()) {
-							if (ptrGrabResult == OS.GrabSuccess) OS.XUngrabPointer (xDisplay, OS.CurrentTime);
-							if (kbdGrabResult == OS.GrabSuccess) OS.XUngrabKeyboard (xDisplay, OS.CurrentTime);
-							return false;
-						}
-						drawRectangles ();
-						oldX[0] = cursorPos.x;  oldY[0] = cursorPos.y;
-					}
-				}
-				break;
+		OS.memmove (anyEvent, xEvent, XAnyEvent.sizeof);
+		int widget = OS.XtWindowToWidget (anyEvent.display, anyEvent.window);
+		switch (anyEvent.type) {
+			case OS.MotionNotify: XPointerMotion (widget, 0, xEvent, dispatch); break;
+			case OS.ButtonRelease: XButtonRelease (widget, 0, xEvent, dispatch); break;
+			case OS.KeyPress: XKeyPress (widget, 0, xEvent, dispatch); break;
+			case OS.KeyRelease: XKeyRelease (widget, 0, xEvent, dispatch); break;
+			case OS.ButtonPress:
 			case OS.EnterNotify:
 			case OS.LeaveNotify:
-				/*
-				 * Do not dispatch these
-				 */
+				/* Do not dispatch these */
+				break;
+			case OS.Expose:
+				update ();
+				drawRectangles (rectangles, stippled);
+				OS.XtDispatchEvent (xEvent);
+				drawRectangles (rectangles, stippled);
 				break;
 			default:
 				OS.XtDispatchEvent (xEvent);
 		}
 	}
-	drawRectangles ();
+	if (xEvent != 0) OS.XtFree (xEvent);
+	if (dispatch != 0) OS.XtFree (dispatch);
+	if (!isDisposed()) {
+		update ();
+		drawRectangles (rectangles, stippled);
+	}
 	if (ptrGrabResult == OS.GrabSuccess) OS.XUngrabPointer (xDisplay, OS.CurrentTime);
 	if (kbdGrabResult == OS.GrabSuccess) OS.XUngrabKeyboard (xDisplay, OS.CurrentTime);
+	window = 0;
 	return !cancelled;
 }
 /**
  * Removes the listener from the collection of listeners who will
  * be notified when the control is moved or resized.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -551,23 +498,119 @@ public void removeControlListener (ControlListener listener) {
 	checkWidget ();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (eventTable == null) return;
+	eventTable.unhook (SWT.Resize, listener);
 	eventTable.unhook (SWT.Move, listener);
 }
+/**
+ * Removes the listener from the collection of listeners who will
+ * be notified when keys are pressed and released on the system keyboard.
+ *
+ * @param listener the listener which should no longer be notified
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see KeyListener
+ * @see #addKeyListener
+ */
+public void removeKeyListener(KeyListener listener) {
+	checkWidget();
+	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (eventTable == null) return;
+	eventTable.unhook(SWT.KeyUp, listener);
+	eventTable.unhook(SWT.KeyDown, listener);
+}
 void resizeRectangles (int xChange, int yChange) {
 	/*
-	* If the cursor orientation has not been set in the orientation of
-	* this change then try to set it here.
-	*/
+	 * If the cursor orientation has not been set in the orientation of
+	 * this change then try to set it here.
+	 */
 	if (xChange < 0 && ((style & SWT.LEFT) != 0) && ((cursorOrientation & SWT.RIGHT) == 0)) {
 		cursorOrientation |= SWT.LEFT;
-	} else if (xChange > 0 && ((style & SWT.RIGHT) != 0) && ((cursorOrientation & SWT.LEFT) == 0)) {
+	}
+	if (xChange > 0 && ((style & SWT.RIGHT) != 0) && ((cursorOrientation & SWT.LEFT) == 0)) {
 		cursorOrientation |= SWT.RIGHT;
-	} else if (yChange < 0 && ((style & SWT.UP) != 0) && ((cursorOrientation & SWT.DOWN) == 0)) {
+	}
+	if (yChange < 0 && ((style & SWT.UP) != 0) && ((cursorOrientation & SWT.DOWN) == 0)) {
 		cursorOrientation |= SWT.UP;
-	} else if (yChange > 0 && ((style & SWT.DOWN) != 0) && ((cursorOrientation & SWT.UP) == 0)) {
+	}
+	if (yChange > 0 && ((style & SWT.DOWN) != 0) && ((cursorOrientation & SWT.UP) == 0)) {
 		cursorOrientation |= SWT.DOWN;
 	}
-	Rectangle bounds = computeBounds ();
+	
+	/*
+	 * If the bounds will flip about the x or y axis then apply the adjustment
+	 * up to the axis (ie.- where bounds width/height becomes 0), change the
+	 * cursor's orientation accordingly, and flip each Rectangle's origin (only
+	 * necessary for > 1 Rectangles) 
+	 */
+	if ((cursorOrientation & SWT.LEFT) != 0) {
+		if (xChange > bounds.width) {
+			if ((style & SWT.RIGHT) == 0) return;
+			cursorOrientation |= SWT.RIGHT;
+			cursorOrientation &= ~SWT.LEFT;
+			bounds.x += bounds.width;
+			xChange -= bounds.width;
+			bounds.width = 0;
+			if (proportions.length > 1) {
+				for (int i = 0; i < proportions.length; i++) {
+					Rectangle proportion = proportions [i];
+					proportion.x = 100 - proportion.x - proportion.width;
+				}
+			}
+		}
+	} else if ((cursorOrientation & SWT.RIGHT) != 0) {
+		if (bounds.width < -xChange) {
+			if ((style & SWT.LEFT) == 0) return;
+			cursorOrientation |= SWT.LEFT;
+			cursorOrientation &= ~SWT.RIGHT;
+			xChange += bounds.width;
+			bounds.width = 0;
+			if (proportions.length > 1) {
+				for (int i = 0; i < proportions.length; i++) {
+					Rectangle proportion = proportions [i];
+					proportion.x = 100 - proportion.x - proportion.width;
+				}
+			}
+		}
+	}
+	if ((cursorOrientation & SWT.UP) != 0) {
+		if (yChange > bounds.height) {
+			if ((style & SWT.DOWN) == 0) return;
+			cursorOrientation |= SWT.DOWN;
+			cursorOrientation &= ~SWT.UP;
+			bounds.y += bounds.height;
+			yChange -= bounds.height;
+			bounds.height = 0;
+			if (proportions.length > 1) {
+				for (int i = 0; i < proportions.length; i++) {
+					Rectangle proportion = proportions [i];
+					proportion.y = 100 - proportion.y - proportion.height;
+				}
+			}
+		}
+	} else if ((cursorOrientation & SWT.DOWN) != 0) {
+		if (bounds.height < -yChange) {
+			if ((style & SWT.UP) == 0) return;
+			cursorOrientation |= SWT.UP;
+			cursorOrientation &= ~SWT.DOWN;
+			yChange += bounds.height;
+			bounds.height = 0;
+			if (proportions.length > 1) {
+				for (int i = 0; i < proportions.length; i++) {
+					Rectangle proportion = proportions [i];
+					proportion.y = 100 - proportion.y - proportion.height;
+				}
+			}
+		}
+	}
+	
+	// apply the bounds adjustment
 	if ((cursorOrientation & SWT.LEFT) != 0) {
 		bounds.x += xChange;
 		bounds.width -= xChange;
@@ -580,10 +623,6 @@ void resizeRectangles (int xChange, int yChange) {
 	} else if ((cursorOrientation & SWT.DOWN) != 0) {
 		bounds.height += yChange;
 	}
-	/*
-	* The following are conditions under which the resize should not be applied
-	*/
-	if (bounds.width < 0 || bounds.height < 0) return;
 	
 	Rectangle [] newRects = new Rectangle [rectangles.length];
 	for (int i = 0; i < rectangles.length; i++) {
@@ -619,6 +658,9 @@ public void setCursor (Cursor value) {
  *
  * @param rectangles the bounds of the rectangles to be drawn
  *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the set of rectangles is null or contains a null rectangle</li>
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -626,7 +668,14 @@ public void setCursor (Cursor value) {
  */
 public void setRectangles (Rectangle [] rectangles) {
 	checkWidget ();
-	this.rectangles = rectangles;
+	if (rectangles == null) error (SWT.ERROR_NULL_ARGUMENT);
+	int length = rectangles.length;
+	this.rectangles = new Rectangle [length];
+	for (int i = 0; i < length; i++) {
+		Rectangle current = rectangles [i];
+		if (current == null) error (SWT.ERROR_NULL_ARGUMENT);
+		this.rectangles [i] = new Rectangle (current.x, current.y, current.width, current.height);
+	}
 	proportions = computeProportions (rectangles);
 }
 /**
@@ -643,4 +692,271 @@ public void setStippled (boolean stippled) {
 	checkWidget ();
 	this.stippled = stippled;
 }
+void update () {
+	if (parent != null) {
+		if (parent.isDisposed ()) return;
+		parent.getShell ().update ();
+	} else {
+		display.update ();
+	}
+}
+int XButtonRelease (int w, int client_data, int call_data, int continue_to_dispatch) {
+	return xMouse (OS.ButtonRelease, w, client_data, call_data, continue_to_dispatch);
+}
+
+int XKeyPress (int w, int client_data, int call_data, int continue_to_dispatch) {
+	int result = super.XKeyPress (w, client_data, call_data, continue_to_dispatch);
+	if (result != 0) return result;
+	XKeyEvent keyEvent = new XKeyEvent ();
+	OS.memmove (keyEvent, call_data, XKeyEvent.sizeof);
+	if (keyEvent.keycode != 0) {
+		int [] keysym = new int [1];
+		OS.XLookupString (keyEvent, null, 0, keysym, null);
+		keysym [0] &= 0xFFFF;
+		int xChange = 0, yChange = 0;
+		int stepSize = ((keyEvent.state & OS.ControlMask) != 0) ? STEPSIZE_SMALL : STEPSIZE_LARGE;
+		switch (keysym [0]) {
+			case OS.XK_KP_Enter:
+			case OS.XK_Return:
+				tracking = false;
+				/* Eat the subsequent KeyRelease event */
+				OS.XtAppNextEvent (OS.XtDisplayToApplicationContext (keyEvent.display), call_data);
+				break;
+			case OS.XK_Escape:
+				tracking = false;
+				cancelled = true;
+				/* Eat the subsequent KeyRelease event */
+				OS.XtAppNextEvent (OS.XtDisplayToApplicationContext (keyEvent.display), call_data);
+				break;
+			case OS.XK_Left:
+				xChange = -stepSize;
+				break;
+			case OS.XK_Right:
+				xChange = stepSize;
+				break;
+			case OS.XK_Up:
+				yChange = -stepSize;
+				break;
+			case OS.XK_Down:
+				yChange = stepSize;
+				break;
+		}
+		if (xChange != 0 || yChange != 0) {
+			Rectangle [] oldRectangles = rectangles;
+			boolean oldStippled = stippled;
+			Rectangle [] rectsToErase = new Rectangle [rectangles.length];
+			for (int i = 0; i < rectangles.length; i++) {
+				Rectangle current = rectangles [i];
+				rectsToErase [i] = new Rectangle (current.x, current.y, current.width, current.height);
+			}
+			Event event = new Event ();
+			event.x = oldX + xChange;
+			event.y = oldY + yChange;
+			Point cursorPos;
+			if ((style & SWT.RESIZE) != 0) {
+				resizeRectangles (xChange, yChange);
+				sendEvent (SWT.Resize, event);
+				/*
+				 * It is possible (but unlikely) that application code
+				 * could have disposed the widget in the resize event.
+				 * If this happens then return false to indicate that
+				 * the move failed.
+				 */
+				if (isDisposed ()) {
+					cancelled = true;
+					return 1;
+				}
+				boolean draw = false;
+				/*
+				 * It is possible that application code could have
+				 * changed the rectangles in the resize event.  If this
+				 * happens then only redraw the tracker if the rectangle
+				 * values have changed.
+				 */
+				if (rectangles != oldRectangles) {
+					int length = rectangles.length;
+					if (length != rectsToErase.length) {
+						draw = true;
+					} else {
+						for (int i = 0; i < length; i++) {
+							if (!rectangles [i].equals (rectsToErase [i])) {
+								draw = true;
+								break;
+							}
+						}
+					}
+				} else {
+					draw = true;
+				}
+				if (draw) {
+					drawRectangles (rectsToErase, oldStippled);
+					update ();
+					drawRectangles (rectangles, stippled);
+				}
+				cursorPos = adjustResizeCursor ();
+			} else {
+				moveRectangles (xChange, yChange);
+				sendEvent (SWT.Move, event);
+				/*
+				 * It is possible (but unlikely) that application code
+				 * could have disposed the widget in the move event.
+				 * If this happens then return false to indicate that
+				 * the move failed.
+				 */
+				if (isDisposed ()) {
+					cancelled = true;
+					return 1;
+				}
+				boolean draw = false;
+				/*
+				 * It is possible that application code could have
+				 * changed the rectangles in the move event.  If this
+				 * happens then only redraw the tracker if the rectangle
+				 * values have changed.
+				 */
+				if (rectangles != oldRectangles) {
+					int length = rectangles.length;
+					if (length != rectsToErase.length) {
+						draw = true;
+					} else {
+						for (int i = 0; i < length; i++) {
+							if (!rectangles [i].equals (rectsToErase [i])) {
+								draw = true;
+								break;
+							}
+						}
+					}
+				} else {
+					draw = true;
+				}
+				if (draw) {
+					drawRectangles (rectsToErase, oldStippled);
+					update ();
+					drawRectangles (rectangles, stippled);
+				}
+				cursorPos = adjustMoveCursor ();
+			}
+			oldX = cursorPos.x;
+			oldY = cursorPos.y;
+		}
+	}
+	return result;
+}
+
+int XPointerMotion (int w, int client_data, int call_data, int continue_to_dispatch) {
+	if (cursor != 0) {
+		int xDisplay = display.xDisplay;
+		OS.XChangeActivePointerGrab (xDisplay,
+			OS.ButtonPressMask | OS.ButtonReleaseMask | OS.PointerMotionMask,
+			cursor, OS.CurrentTime);
+	}
+	return xMouse (OS.MotionNotify, w, client_data, call_data, continue_to_dispatch);
+}
+
+int xMouse (int type, int w, int client_data, int call_data, int continue_to_dispatch) {
+	int xDisplay = display.xDisplay;
+	int [] newX = new int [1], newY = new int [1], unused = new int [1];
+	OS.XQueryPointer (xDisplay, window, unused, unused, newX, newY, unused, unused, unused);
+	if (oldX != newX [0] || oldY != newY [0]) {
+		Rectangle [] oldRectangles = rectangles;
+		boolean oldStippled = stippled;
+		Rectangle [] rectsToErase = new Rectangle [rectangles.length];
+		for (int i = 0; i < rectangles.length; i++) {
+			Rectangle current = rectangles [i];
+			rectsToErase [i] = new Rectangle (current.x, current.y, current.width, current.height);
+		}
+		Event event = new Event ();
+		event.x = newX [0];
+		event.y = newY [0];
+		if ((style & SWT.RESIZE) != 0) {
+			resizeRectangles (newX [0] - oldX, newY [0] - oldY);
+			sendEvent (SWT.Resize, event);
+			/*
+			 * It is possible (but unlikely) that application code
+			 * could have disposed the widget in the resize event.
+			 * If this happens then return false to indicate that
+			 * the move failed.
+			 */
+			if (isDisposed ()) {
+				cancelled = true;
+				return 1;
+			}
+			boolean draw = false;
+			/*
+			 * It is possible that application code could have
+			 * changed the rectangles in the resize event.  If this
+			 * happens then only redraw the tracker if the rectangle
+			 * values have changed.
+			 */
+			if (rectangles != oldRectangles) {
+				int length = rectangles.length;
+				if (length != rectsToErase.length) {
+					draw = true;
+				} else {
+					for (int i = 0; i < length; i++) {
+						if (!rectangles [i].equals (rectsToErase [i])) {
+							draw = true;
+							break;
+						}
+					}
+				}
+			} else {
+				draw = true;
+			}
+			if (draw) {
+				drawRectangles (rectsToErase, oldStippled);
+				update ();
+				drawRectangles (rectangles, stippled);
+			}
+			Point cursorPos = adjustResizeCursor ();
+			newX [0] = cursorPos.x;
+			newY [0] = cursorPos.y;
+		} else {
+			moveRectangles (newX [0] - oldX, newY [0] - oldY);
+			sendEvent (SWT.Move, event);
+			/*
+			 * It is possible (but unlikely) that application code
+			 * could have disposed the widget in the move event.
+			 * If this happens then return false to indicate that
+			 * the move failed.
+			 */
+			if (isDisposed ()) {
+				cancelled = true;
+				return 1;
+			}
+			boolean draw = false;
+			/*
+			 * It is possible that application code could have
+			 * changed the rectangles in the move event.  If this
+			 * happens then only redraw the tracker if the rectangle
+			 * values have changed.
+			 */
+			if (rectangles != oldRectangles) {
+				int length = rectangles.length;
+				if (length != rectsToErase.length) {
+					draw = true;
+				} else {
+					for (int i = 0; i < length; i++) {
+						if (!rectangles [i].equals (rectsToErase [i])) {
+							draw = true;
+							break;
+						}
+					}
+				}
+			} else {
+				draw = true;
+			}
+			if (draw) {
+				drawRectangles (rectsToErase, oldStippled);
+				update ();
+				drawRectangles (rectangles, stippled);
+			}
+		}
+		oldX = newX [0];
+		oldY = newY [0];
+	}
+	tracking = type != OS.ButtonRelease;
+	return 0;
+}
+
 }
