@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -41,6 +41,7 @@ import org.eclipse.swt.events.*;
  * </p>
  */
 public class Button extends Control {
+	String text = "";
 	Image image;
 
 /**
@@ -85,13 +86,14 @@ public Button (Composite parent, int style) {
 
 static int checkStyle (int style) {
 	style = checkBits (style, SWT.PUSH, SWT.ARROW, SWT.CHECK, SWT.RADIO, SWT.TOGGLE, 0);
-	if ((style & SWT.PUSH) != 0) {
+	if ((style & (SWT.PUSH | SWT.TOGGLE)) != 0) {
 		return checkBits (style, SWT.CENTER, SWT.LEFT, SWT.RIGHT, 0, 0, 0);
 	}
-	if ((style & (SWT.CHECK | SWT.RADIO | SWT.TOGGLE)) != 0) {
+	if ((style & (SWT.CHECK | SWT.RADIO)) != 0) {
 		return checkBits (style, SWT.LEFT, SWT.RIGHT, SWT.CENTER, 0, 0, 0);
 	}
 	if ((style & SWT.ARROW) != 0) {
+		style |= SWT.NO_FOCUS;
 		return checkBits (style, SWT.UP, SWT.DOWN, SWT.LEFT, SWT.RIGHT, 0, 0);
 	}
 	return style;
@@ -130,34 +132,7 @@ public void addSelectionListener (SelectionListener listener) {
 }
 
 void click () {
-	int rid = OS.PtWidgetRid (handle);
-	if (rid == 0) return;
-	PhEvent_t event = new PhEvent_t ();
-	event.emitter_rid = rid;
-	event.emitter_handle = handle;
-	event.collector_rid = rid;
-	event.collector_handle = handle;
-	event.flags = OS.Ph_EVENT_DIRECT;
-	event.processing_flags = OS.Ph_FAKE_EVENT;
-	event.type = OS.Ph_EV_BUT_PRESS;
-	event.num_rects = 1;
-	PhPointerEvent_t pe = new PhPointerEvent_t ();
-	pe.click_count = 1;
-	pe.buttons = OS.Ph_BUTTON_SELECT;
-	PhRect_t rect = new PhRect_t ();
-	int ptr = OS.malloc (PhEvent_t.sizeof + PhPointerEvent_t.sizeof + PhRect_t.sizeof);
-	OS.memmove (ptr, event, PhEvent_t.sizeof);
-	OS.memmove (ptr + PhEvent_t.sizeof,  rect, PhRect_t.sizeof);
-	OS.memmove (ptr + PhEvent_t.sizeof + PhRect_t.sizeof, pe, PhPointerEvent_t.sizeof);
-	OS.PtSendEventToWidget (handle, ptr);	
-	OS.PtFlush ();
-	event.type = OS.Ph_EV_BUT_RELEASE;
-	event.subtype = OS.Ph_EV_RELEASE_REAL;
-	OS.memmove (ptr, event, PhEvent_t.sizeof);
-	OS.memmove (ptr + PhEvent_t.sizeof,  rect, PhRect_t.sizeof);
-	OS.memmove (ptr + PhEvent_t.sizeof + PhRect_t.sizeof, pe, PhPointerEvent_t.sizeof);
-	OS.PtSendEventToWidget (handle, ptr);	
-	OS.free (ptr);
+	click (handle);
 }
 
 public Point computeSize (int wHint, int hHint, boolean changed) {
@@ -208,7 +183,6 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 
 void createHandle (int index) {
 	state |= HANDLE;
-	Display display = getDisplay ();
 	int parentHandle = parent.parentingHandle ();
 		
 	/* ARROW button */
@@ -348,7 +322,8 @@ String getNameText () {
 
 /**
  * Returns the receiver's text, which will be an empty
- * string if it has never been set.
+ * string if it has never been set or if the receiver is
+ * an <code>ARROW</code> button.
  *
  * @return the receiver's text
  *
@@ -360,49 +335,12 @@ String getNameText () {
 public String getText () {
 	checkWidget();
 	if ((style & SWT.ARROW) != 0) return "";
-	int [] args = {
-		OS.Pt_ARG_TEXT_STRING, 0, 0,
-		OS.Pt_ARG_ACCEL_KEY, 0, 0,
-	};
-	OS.PtGetResources (handle, args.length / 3, args);
-	if (args [1] == 0) return "";
-	int length = OS.strlen (args [1]);
-	byte [] buffer = new byte [length];
-	OS.memmove (buffer, args [1], length);
-	char [] result = Converter.mbcsToWcs (null, buffer);
-	int count = 0;
-	int mnemonic = 0;
-	if (args [4] != 0) {
-		int length2 = OS.strlen (args [4]);
-		if (length2 > 0) {
-			byte [] buffer2 = new byte [length2];
-			OS.memmove (buffer2, args [4], length2);
-			char [] result2 = Converter.mbcsToWcs (null, buffer2);
-			if (result2.length > 0) mnemonic = result2 [0];
-		}
-	}
-	if (mnemonic != 0) count++;
-	for (int i=0; i<result.length-1; i++)
-		if (result [i] == Mnemonic) count++;
-	char [] newResult = result;
-	if ((count != 0) || (mnemonic != 0)) {
-		newResult = new char [result.length + count];
-		int i = 0, j = 0;
-		while (i < result.length) {
-			if ((mnemonic != 0) && (result [i] == mnemonic)) {
-				if (j < newResult.length) newResult [j++] = Mnemonic;
-				mnemonic = 0;
-			}
-			if ((newResult [j++] = result [i++]) == Mnemonic)
-				if (j < newResult.length) newResult [j++] = Mnemonic;
-		}
-	}
-	return new String (newResult);
+	return text;
 }
 
 void hookEvents () {
 	super.hookEvents ();
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	OS.PtAddCallback (handle, OS.Pt_CB_ACTIVATE, windowProc, OS.Pt_CB_ACTIVATE);
 }
 
@@ -450,7 +388,7 @@ void releaseWidget () {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -569,9 +507,9 @@ public void setSelection (boolean selected) {
 
 /**
  * Sets the receiver's image to the argument, which may be
- * null indicating that no image should be displayed.
+ * <code>null</code> indicating that no image should be displayed.
  *
- * @param image the image to display on the receiver (may be null)
+ * @param image the image to display on the receiver (may be <code>null</code>)
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_INVALID_ARGUMENT - if the image has been disposed</li>
@@ -605,14 +543,14 @@ public void setImage (Image image) {
  * the mnemonic character but must not contain line delimiters.
  * </p>
  * <p>
- * Mnemonics are indicated by an '&amp' that causes the next
+ * Mnemonics are indicated by an '&amp;' that causes the next
  * character to be the mnemonic.  When the user presses a
  * key sequence that matches the mnemonic, a selection
  * event occurs. On most platforms, the mnemonic appears
  * underlined but may be emphasised in a platform specific
- * manner.  The mnemonic indicator character '&amp' can be
+ * manner.  The mnemonic indicator character '&amp;' can be
  * escaped by doubling it in the string, causing a single
- *'&amp' to be displayed.
+ * '&amp;' to be displayed.
  * </p>
  * 
  * @param string the new text
@@ -629,22 +567,13 @@ public void setText (String string) {
 	checkWidget();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if ((style & SWT.ARROW) != 0) return;
+	text = string;
 	char [] text = new char [string.length ()];
 	string.getChars (0, text.length, text, 0);
-	int i=0, j=0;
-	char mnemonic=0;
-	while (i < text.length) {
-		if ((text [j++] = text [i++]) == Mnemonic) {
-			if (i == text.length) {continue;}
-			if (text [i] == Mnemonic) {i++; continue;}
-			if (mnemonic == 0) mnemonic = text [i];
-			j--;
-		}
-	}
-	while (j < text.length) text [j++] = 0;
+	char mnemonic = fixMnemonic (text);
 	byte [] buffer = Converter.wcsToMbcs (null, text, true);
-	int ptr = OS.malloc (buffer.length);
-	OS.memmove (ptr, buffer, buffer.length);
+	int ptr1 = OS.malloc (buffer.length);
+	OS.memmove (ptr1, buffer, buffer.length);
 	int ptr2 = 0;
 	if (mnemonic != 0) {
 		byte [] buffer2 = Converter.wcsToMbcs (null, new char []{mnemonic}, true);
@@ -653,18 +582,18 @@ public void setText (String string) {
 	}
 	replaceMnemonic (mnemonic, true, true);
 	int [] args = {
-		OS.Pt_ARG_TEXT_STRING, ptr, 0,
+		OS.Pt_ARG_TEXT_STRING, ptr1, 0,
 		OS.Pt_ARG_LABEL_TYPE, OS.Pt_Z_STRING, 0,
 		OS.Pt_ARG_ACCEL_KEY, ptr2, 0,
 	};
 	OS.PtSetResources (handle, args.length / 3, args);
-	OS.free (ptr);
+	OS.free (ptr1);
 	OS.free (ptr2);
 }
 
 int traversalCode (int key_sym, PhKeyEvent_t ke) {
-	int code = super.traversalCode (key_sym , ke);
-	code |= SWT.TRAVERSE_ARROW_NEXT | SWT.TRAVERSE_ARROW_PREVIOUS | SWT.TRAVERSE_MNEMONIC;
+	int code = super.traversalCode (key_sym, ke);
+	if ((style & SWT.RADIO) != 0) code |= SWT.TRAVERSE_ARROW_NEXT | SWT.TRAVERSE_ARROW_PREVIOUS;
 	return code;
 }
 
