@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -31,6 +31,12 @@ import org.eclipse.swt.graphics.*;
  * behavior is undefined if they are used with subclasses of <code>Composite</code> other
  * than <code>Canvas</code>.
  * </p><p>
+ * Note: The <code>CENTER</code> style, although undefined for composites, has the
+ * same value as <code>EMBEDDED</code> (which is used to embed widgets from other
+ * widget toolkits into SWT).  On some operating systems (GTK, Motif), this may cause
+ * the children of this composite to be obscured.  The <code>EMBEDDED</code> style
+ * is for use by other widget toolkits and should normally never be used.
+ * </p><p>
  * This class may be subclassed by custom control implementors
  * who are building controls that are constructed from aggregates
  * of other controls.
@@ -42,7 +48,7 @@ public class Composite extends Scrollable {
 	Layout layout;
 	Control [] tabList;
 	int cornerHandle;
-	int layoutCount = 0;
+	int layoutCount, backgroundMode;
 	
 Composite () {
 	/* Do nothing */
@@ -270,6 +276,13 @@ void createScrolledHandle (int parentHandle) {
 	createScrollBars ();
 }
 
+void drawBackground (GC gc, int x, int y, int width, int height) {
+	Color oldColor = gc.getBackground();
+	gc.setBackground(getBackground());
+	gc.fillRectangle(x, y, width, height);
+	gc.setBackground(oldColor);
+}
+
 void drawWidget (int widget, int damage) {
 	if ((state & CANVAS) != 0) {
 		if ((style & SWT.NO_BACKGROUND) == 0) {
@@ -319,6 +332,10 @@ void drawWidget (int widget, int damage) {
 	}
 }
 
+Composite findDeferredControl () {
+	return layoutCount > 0 ? this : parent.findDeferredControl ();
+}
+
 void fixTabList (Control control) {
 	if (tabList == null) return;
 	int count = 0;
@@ -436,9 +453,34 @@ int getClipping(int widget, int topWidget, boolean clipChildren, boolean clipSib
 	return widget_tile;
 }
 
+/**
+ * Returns the receiver's background drawing mode. This
+ * will be one of the following constants defined in class
+ * <code>SWT</code>:
+ * <code>INHERIT_NONE</code>, <code>INHERIT_DEFAULT</code>,
+ * <code>INHERTIT_FORCE</code>.
+ *
+ * @return the background mode
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see SWT
+ * 
+ * @since 3.2
+ */
+public int getBackgroundMode () {
+	checkWidget ();
+	return backgroundMode;
+}
+
 /**
  * Returns a (possibly empty) array containing the receiver's children.
- * Children are returned in the order that they are drawn.
+ * Children are returned in the order that they are drawn.  The topmost
+ * control appears at the beginning of the array.  Subsequent controls
+ * draw beneath this control and appear later in the array.
  * <p>
  * Note: This is not the actual structure used by the receiver
  * to maintain its list of children, so modifying the array will
@@ -540,7 +582,7 @@ boolean hooksKeys () {
  */
 public boolean isLayoutDeferred () {
 	checkWidget ();
-	return layoutCount > 0 || parent.isLayoutDeferred ();
+	return findDeferredControl () != null;
 }
 
 /**
@@ -812,12 +854,15 @@ int Pt_CB_OUTBOUND (int widget, int info) {
 	return OS.Pt_CONTINUE;
 }
 
-void releaseChildren () {
+void releaseChildren (boolean destroy) {
 	Control [] children = _getChildren ();
 	for (int i=0; i<children.length; i++) {
 		Control child = children [i];
-		if (!child.isDisposed ()) child.releaseResources ();
+		if (child != null && !child.isDisposed ()) {
+			child.release (false);
+		}
 	}
+	super.releaseChildren (destroy);
 }
 
 void releaseHandle () {
@@ -826,7 +871,6 @@ void releaseHandle () {
 }
 
 void releaseWidget () {
-	releaseChildren ();
 	super.releaseWidget ();
 	layout = null;
 	tabList = null;
@@ -913,6 +957,28 @@ void resizeClientArea (int width, int height, boolean events) {
 	}
 }
 
+/**
+ * Sets the background drawing mode to the argument which should
+ * be one of the following constants defined in class <code>SWT</code>:
+ * <code>INHERIT_NONE</code>, <code>INHERIT_DEFAULT</code>,
+ * <code>INHERIT_FORCE</code>.
+ *
+ * @param mode the new background mode
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see SWT
+ * 
+ * @since 3.2
+ */
+public void setBackgroundMode (int mode) {
+	checkWidget ();
+	backgroundMode = mode;
+}
+
 int setBounds (int x, int y, int width, int height, boolean move, boolean resize, boolean events) {
 	int result = super.setBounds (x, y, width, height, move, resize, false);
 	if ((result & MOVED) != 0) {
@@ -980,7 +1046,9 @@ public void setLayout (Layout layout) {
 public void setLayoutDeferred (boolean defer) {
 	if (!defer) {
 		if (--layoutCount == 0) {
-			if (!isLayoutDeferred ()) updateLayout (true);
+			if ((state & LAYOUT_CHILD) != 0 || (state & LAYOUT_NEEDED) != 0) {
+				updateLayout (true);
+			}
 		}
 	} else {
 		layoutCount++;
@@ -1036,13 +1104,18 @@ int traversalCode (int key_sym, PhKeyEvent_t ke) {
 }
 
 void updateLayout (boolean all) {
-	if (isLayoutDeferred ()) return;
+	Composite parent = findDeferredControl ();
+	if (parent != null) {
+		parent.state |= LAYOUT_CHILD;
+		return;
+	}
 	if ((state & LAYOUT_NEEDED) != 0) {
 		boolean changed = (state & LAYOUT_CHANGED) != 0;
 		state &= ~(LAYOUT_NEEDED | LAYOUT_CHANGED);
 		layout.layout (this, changed);
 	}
 	if (all) {
+		state &= ~LAYOUT_CHILD;
 		Control [] children = _getChildren ();
 		for (int i=0; i<children.length; i++) {
 			children [i].updateLayout (all);
