@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -52,9 +52,18 @@ import org.eclipse.swt.events.*;
  * of these changes are also possible.
  * </li>
  * </ul>
- * </p>
- * <p>
- * Note: The styles supported by this class must be treated
+ * </p><p>
+ * The <em>modality</em> of an instance may be specified using
+ * style bits. The modality style bits are used to determine
+ * whether input is blocked for other shells on the display.
+ * The <code>PRIMARY_MODAL</code> style allows an instance to block
+ * input to its parent. The <code>APPLICATION_MODAL</code> style
+ * allows an instance to block input to every other shell in the
+ * display. The <code>SYSTEM_MODAL</code> style allows an instance
+ * to block input to all shells, including shells belonging to
+ * different applications.
+ * </p><p>
+ * Note: The styles supported by this class are treated
  * as <em>HINT</em>s, since the window manager for the
  * desktop on which the instance is visible has ultimate
  * control over the appearance and behavior of decorations
@@ -65,9 +74,14 @@ import org.eclipse.swt.events.*;
  * more restrictive modality style that is supported. For
  * example, if <code>PRIMARY_MODAL</code> is not supported,
  * it would be upgraded to <code>APPLICATION_MODAL</code>.
+ * A modality style may also be "downgraded" to a less
+ * restrictive style. For example, most operating systems
+ * no longer support <code>SYSTEM_MODAL</code> because
+ * it can freeze up the desktop, so this is typically
+ * downgraded to <code>APPLICATION_MODAL</code>.
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>BORDER, CLOSE, MIN, MAX, NO_TRIM, RESIZE, TITLE</dd>
+ * <dd>BORDER, CLOSE, MIN, MAX, NO_TRIM, RESIZE, TITLE, ON_TOP, TOOL</dd>
  * <dd>APPLICATION_MODAL, MODELESS, PRIMARY_MODAL, SYSTEM_MODAL</dd>
  * <dt><b>Events:</b></dt>
  * <dd>Activate, Close, Deactivate, Deiconify, Iconify</dd>
@@ -101,10 +115,10 @@ import org.eclipse.swt.events.*;
  */
 public class Shell extends Decorations {
 	int shellHandle;
-	Display display;
 	Menu activeMenu;
 	int blockedList;
 	Control lastActive;
+	Region region;
 
 /**
  * Constructs a new instance of this class. This is equivalent
@@ -235,6 +249,9 @@ Shell (Display display, Shell parent, int style, int handle) {
 	if (!display.isValidThread ()) {
 		error (SWT.ERROR_THREAD_INVALID_ACCESS);
 	}
+	if (parent != null && parent.isDisposed ()) {
+		error (SWT.ERROR_INVALID_ARGUMENT);	
+	}
 	this.style = checkStyle (style);
 	this.parent = parent;
 	this.display = display;
@@ -257,7 +274,7 @@ Shell (Display display, Shell parent, int style, int handle) {
  * @param parent a shell which will be the parent of the new instance
  *
  * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if the parent is disposed</li> 
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -265,7 +282,7 @@ Shell (Display display, Shell parent, int style, int handle) {
  * </ul>
  */
 public Shell (Shell parent) {
-	this (parent, SWT.TITLE | SWT.CLOSE | SWT.BORDER);
+	this (parent, SWT.DIALOG_TRIM);
 }
 
 /**
@@ -291,6 +308,9 @@ public Shell (Shell parent) {
  * @param parent a shell which will be the parent of the new instance
  * @param style the style of control to construct
  *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_INVALID_ARGUMENT - if the parent is disposed</li> 
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
  *    <li>ERROR_INVALID_SUBCLASS - if this class is not an allowed subclass</li>
@@ -305,13 +325,15 @@ public Shell (Shell parent) {
  * @see SWT#NO_TRIM
  * @see SWT#SHELL_TRIM
  * @see SWT#DIALOG_TRIM
+ * @see SWT#ON_TOP
+ * @see SWT#TOOL
  * @see SWT#MODELESS
  * @see SWT#PRIMARY_MODAL
  * @see SWT#APPLICATION_MODAL
  * @see SWT#SYSTEM_MODAL
  */
 public Shell (Shell parent, int style) {
-	this (parent != null ? parent.getDisplay () : null, parent, style, 0);
+	this (parent != null ? parent.display : null, parent, style, 0);
 }
 
 public static Shell photon_new (Display display, int handle) {
@@ -348,8 +370,11 @@ public void addShellListener (ShellListener listener) {
 	addListener (SWT.Deactivate, typedListener);
 }
 
-void bringToTop () {
-	OS.PtWidgetToFront (shellHandle);
+void bringToTop (boolean force) {
+	if (!force) {
+		if (display.getActiveShell () == null) return;
+	}
+	OS.PtWindowToFront (shellHandle);
 }
 
 static int checkStyle (int style) {
@@ -381,6 +406,7 @@ void closeWidget () {
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
  * </ul>
  *
+ * @see SWT#Close
  * @see #dispose
  */
 public void close () {
@@ -427,10 +453,10 @@ void createHandle (int index) {
 	} else {
 		int parentHandle = 0;
 		if (parent != null) parentHandle = parent.topHandle ();
-		PhRect_t rect = new PhRect_t ();
-		OS.PhWindowQueryVisible (OS.Ph_QUERY_GRAPHICS, 0, OS.PhInputGroup (0), rect);
-		int width = (short) ((rect.lr_x - rect.ul_x + 1) * 5 / 8);
-		int height = (short) ((rect.lr_y - rect.ul_y + 1) * 5 / 8);
+		Monitor monitor = getMonitor ();
+		Rectangle rect = monitor.getClientArea ();
+		int width = rect.width * 5 / 8;
+		int height = rect.height * 5 / 8;
 		int decorations = 0;
 		int flags =
 			OS.Ph_WM_RENDER_MIN | OS.Ph_WM_RENDER_MAX | OS.Ph_WM_RENDER_RESIZE |
@@ -452,6 +478,7 @@ void createHandle (int index) {
 		int windowState = OS.Ph_WM_STATE_ISFOCUS;
 		if ((style & SWT.ON_TOP) != 0) windowState = OS.Ph_WM_STATE_ISFRONT;
 		int titlePtr = OS.malloc (1);
+		OS.memset(titlePtr, 0, 1);
 		int [] args = {
 			OS.Pt_ARG_WIDTH, width, 0,
 			OS.Pt_ARG_HEIGHT, height, 0,
@@ -478,7 +505,12 @@ void createHandle (int index) {
 		};
 		OS.PtSetResources (scrolledHandle, args.length / 3, args);
 	}
-	int [] args = {OS.Pt_ARG_WIDTH, 0, 0, OS.Pt_ARG_HEIGHT, 0, 0};
+	int trim = SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX;
+	if ((style & SWT.NO_TRIM) != 0 || (style & trim) == 0) {
+		OS.PtSetResource (shellHandle, OS.Pt_ARG_MIN_WIDTH, 0, 0);
+	}
+	OS.PtSetResource (shellHandle, OS.Pt_ARG_MIN_HEIGHT, 0, 0);
+	int [] args = new int [] {OS.Pt_ARG_WIDTH, 0, 0, OS.Pt_ARG_HEIGHT, 0, 0};
 	OS.PtGetResources (shellHandle, args.length / 3, args);
 	resizeBounds (args [1], args [4]);
 }
@@ -488,12 +520,16 @@ void deregister () {
 	WidgetTable.remove (shellHandle);
 }
 
+Composite findDeferredControl () {
+	return layoutCount > 0 ? this : null;
+}
+
 /**
- * Moves the receiver to the top of the drawing order for
- * the display on which it was created (so that all other
- * shells on that display, which are not the receiver's
- * children will be drawn behind it) and forces the window
- * manager to make the shell active.
+ * If the receiver is visible, moves it to the top of the 
+ * drawing order for the display on which it was created 
+ * (so that all other shells on that display, which are not 
+ * the receiver's children will be drawn behind it) and forces 
+ * the window manager to make the shell active.
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -511,7 +547,7 @@ void deregister () {
  */
 public void forceActive () {
 	checkWidget ();
-	bringToTop ();
+	bringToTop (true);
 }
 
 public Rectangle getBounds () {
@@ -530,11 +566,6 @@ public Rectangle getBounds () {
 	return new Rectangle (area.pos_x, area.pos_y, width, height);
 }
 
-public Display getDisplay () {
-	if (display == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return display;
-}
-
 /**
  * Returns the receiver's input method editor mode. This
  * will be the result of bitwise OR'ing together one or
@@ -586,6 +617,56 @@ public boolean getMinimized () {
 	return (args [1] & OS.Ph_WM_STATE_ISICONIFIED) != 0;
 }
 
+/**
+ * Returns a point describing the minimum receiver's size. The
+ * x coordinate of the result is the minimum width of the receiver.
+ * The y coordinate of the result is the minimum height of the
+ * receiver.
+ *
+ * @return the receiver's size
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+public Point getMinimumSize () {
+	checkWidget();
+	int [] args = {OS.Pt_ARG_WINDOW_RENDER_FLAGS, 0, 0};
+	OS.PtGetResources (shellHandle, args.length / 3, args);
+	int flags = args [1];
+	int [] left = new int [1], top = new int [1];
+	int [] right = new int [1], bottom = new int [1];
+	OS.PtFrameSize (flags, 0, left, top, right, bottom);
+	args = new int [] {
+		OS.Pt_ARG_MIN_WIDTH, 0, 0,
+		OS.Pt_ARG_MIN_HEIGHT, 0, 0,
+	};
+	OS.PtGetResources (shellHandle, args.length / 3, args);
+	return new Point (args [1] + left [0] + right [0], args [4] + top [0] + bottom [0]);
+}
+
+/** 
+ * Returns the region that defines the shape of the shell,
+ * or null if the shell has the default shape.
+ *
+ * @return the region that defines the shape of the shell (or null)
+ *	
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @since 3.0
+ *
+ */
+public Region getRegion () {
+	checkWidget ();
+	return region;
+}
+
 public Shell getShell () {
 	checkWidget();
 	return this;
@@ -646,7 +727,7 @@ public Point getSize () {
 
 void hookEvents () {
 	super.hookEvents ();
-	int windowProc = getDisplay ().windowProc;
+	int windowProc = display.windowProc;
 	OS.PtAddCallback (shellHandle, OS.Pt_CB_WINDOW, windowProc, OS.Pt_CB_WINDOW);
 	OS.PtAddCallback (shellHandle, OS.Pt_CB_RESIZE, windowProc, OS.Pt_CB_RESIZE);
 }
@@ -667,8 +748,8 @@ int hotkeyProc (int w, int data, int info) {
  * the display on which it was created (so that all other
  * shells on that display, which are not the receiver's
  * children will be drawn behind it), marks it visible,
- * and sets focus to its default button (if it has one)
- * and asks the window manager to make the shell active.
+ * sets the focus and asks the window manager to make the
+ * shell active.
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -685,8 +766,8 @@ int hotkeyProc (int w, int data, int info) {
  */
 public void open () {
 	checkWidget();
-	bringToTop ();
 	setVisible (true);
+	if (isDisposed ()) return;
 	traverseGroup (true);
 }
 
@@ -699,7 +780,10 @@ int Pt_CB_RESIZE (int widget, int info) {
 	OS.PtGetResources (shellHandle, args.length / 3, args);
 	resizeBounds (args [1], args [4]);
 	sendEvent(SWT.Resize);
-	if (layout != null) layout (false);
+	if (layout != null) {
+		markLayout (false, false);
+		updateLayout (false);
+	}
 	return OS.Pt_CONTINUE;
 }
 
@@ -743,16 +827,20 @@ void realizeWidget() {
 	/* Do nothing */
 }
 
+void releaseParent () {
+	/* Do nothing */
+}
+
 void releaseHandle () {
 	super.releaseHandle ();
 	shellHandle = 0;
 }
 
-void releaseShells () {
+void releaseChildren (boolean destroy) {
 	Shell [] shells = getShells ();
 	for (int i=0; i<shells.length; i++) {
 		Shell shell = shells [i];
-		if (!shell.isDisposed ()) {
+		if (shell != null && !shell.isDisposed ()) {
 			/*
 			* Feature in Photon.  A shell may have child shells that have been
 			* temporarily reparented to NULL because they were shown without
@@ -763,19 +851,19 @@ void releaseShells () {
 			if (shell.parent != null && OS.PtWidgetParent (shell.shellHandle) == 0) {
 				shell.dispose ();
 			} else {
-				shell.releaseResources ();
+				shell.release (false);
 			}
 		}
 	}
+	super.releaseChildren (destroy);
 }
 
 void releaseWidget () {
-	releaseShells ();
 	super.releaseWidget ();
 	if (blockedList != 0) OS.PtUnblockWindows (blockedList);
 	blockedList = 0;
 	lastActive = null;
-	display = null;
+	region = null;
 }
 
 /**
@@ -807,11 +895,11 @@ public void removeShellListener (ShellListener listener) {
 }
 
 /**
- * Moves the receiver to the top of the drawing order for
- * the display on which it was created (so that all other
- * shells on that display, which are not the receiver's
- * children will be drawn behind it) and asks the window
- * manager to make the shell active.
+ * If the receiver is visible, moves it to the top of the 
+ * drawing order for the display on which it was created 
+ * (so that all other shells on that display, which are not 
+ * the receiver's children will be drawn behind it) and asks 
+ * the window manager to make the shell active 
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -829,7 +917,7 @@ public void removeShellListener (ShellListener listener) {
  */
 public void setActive () {
 	checkWidget ();
-	bringToTop ();
+	bringToTop (false);
 }
 
 void setActiveControl (Control control) {
@@ -937,7 +1025,10 @@ int setBounds (int x, int y, int width, int height, boolean move, boolean resize
 		if (!sameExtent & resize) {
 			resizeBounds (newArea.size_w, newArea.size_h);
 			sendEvent(SWT.Resize);
-			if (layout != null) layout (false);
+			if (layout != null) {
+				markLayout (false, false);
+				updateLayout (false);
+			}
 		}
 	}
 	
@@ -1022,6 +1113,89 @@ public void setMinimized (boolean minimized) {
 	}
 }
 
+/**
+ * Sets the shape of the shell to the region specified
+ * by the argument.  When the argument is null, the
+ * default shape of the shell is restored.  The shell
+ * must be created with the style SWT.NO_TRIM in order
+ * to specify a region.
+ *
+ * @param region the region that defines the shape of the shell (or null)
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_INVALID_ARGUMENT - if the region has been disposed</li>
+ * </ul>  
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @since 3.0
+ *
+ */
+public void setRegion (Region region) {
+	checkWidget ();
+	if ((style & SWT.NO_TRIM) == 0) return;
+	if (region != null && region.isDisposed()) error (SWT.ERROR_INVALID_ARGUMENT);
+	// TODO implement setRegion
+	this.region = region;
+}
+
+/**
+ * Sets the receiver's minimum size to the size specified by the arguments.
+ * If the new minimum size is larger than the current size of the receiver,
+ * the receiver is resized to the new minimum size.
+ *
+ * @param width the new minimum width for the receiver
+ * @param height the new minimum height for the receiver
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+public void setMinimumSize (int width, int height) {
+	checkWidget();
+	int [] args = {OS.Pt_ARG_WINDOW_RENDER_FLAGS, 0, 0};
+	OS.PtGetResources (shellHandle, args.length / 3, args);
+	int flags = args [1];
+	int [] left = new int [1], top = new int [1];
+	int [] right = new int [1], bottom = new int [1];
+	OS.PtFrameSize (flags, 0, left, top, right, bottom);
+	width = Math.max (width - left [0] - right [0], 0);
+	height = Math.max (height - top [0] - bottom [0], 0);
+	args = new int [] {
+		OS.Pt_ARG_MIN_WIDTH, width, 0,
+		OS.Pt_ARG_MIN_HEIGHT, height, 0,
+	};
+	OS.PtSetResources (shellHandle, args.length / 3, args);
+}
+
+/**
+ * Sets the receiver's minimum size to the size specified by the argument.
+ * If the new minimum size is larger than the current size of the receiver,
+ * the receiver is resized to the new minimum size.
+ *
+ * @param size the new minimum size for the receiver
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the point is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+public void setMinimumSize (Point size) {
+	checkWidget();
+	if (size == null) error (SWT.ERROR_NULL_ARGUMENT);
+	setMinimumSize (size.x, size.y);
+}
+
 public void setText (String string) {
 	checkWidget();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
@@ -1074,10 +1248,12 @@ public void setVisible (boolean visible) {
 	OS.PtSetResource (shellHandle, OS.Pt_ARG_FLAGS, flags, OS.Pt_DELAY_REALIZE);
 	if (visible) {
 		sendEvent (SWT.Show);
+		if (isDisposed ()) return;
 		OS.PtRealizeWidget (shellHandle);
 	} else {
 		OS.PtUnrealizeWidget (shellHandle);
 		sendEvent(SWT.Hide);
+		if (isDisposed ()) return;
 	}
 
 	/*
