@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -37,9 +37,9 @@ import org.eclipse.swt.events.*;
  */
 public class Tracker extends Widget {
 	Composite parent;
-	Display display;
 	boolean tracking, stippled;
 	Rectangle [] rectangles, proportions;
+	Rectangle bounds;
 	int resizeCursor, clientCursor, clientBitmap;
 	int cursorOrientation = SWT.NONE;
 
@@ -128,7 +128,6 @@ public Tracker (Display display, int style) {
 public Tracker (Composite parent, int style) {
 	super (parent, checkStyle (style));
 	this.parent = parent;
-	display = parent.getDisplay ();
 }
 
 /**
@@ -154,11 +153,38 @@ public void addControlListener(ControlListener listener) {
 	checkWidget();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	TypedListener typedListener = new TypedListener (listener);
-	addListener (SWT.Move,typedListener);
+	addListener (SWT.Resize, typedListener);
+	addListener (SWT.Move, typedListener);
+}
+
+/**
+ * Adds the listener to the collection of listeners who will
+ * be notified when keys are pressed and released on the system keyboard, by sending
+ * it one of the messages defined in the <code>KeyListener</code>
+ * interface.
+ *
+ * @param listener the listener which should be notified
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see KeyListener
+ * @see #removeKeyListener
+ */
+public void addKeyListener (KeyListener listener) {
+	checkWidget();
+	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
+	TypedListener typedListener = new TypedListener (listener);
+	addListener (SWT.KeyUp,typedListener);
+	addListener (SWT.KeyDown,typedListener);
 }
 
 Point adjustMoveCursor () {
-	Rectangle bounds = computeBounds ();
 	int newX = bounds.x + bounds.width / 2;
 	int newY = bounds.y;
 	/*
@@ -176,7 +202,6 @@ Point adjustMoveCursor () {
 
 Point adjustResizeCursor () {
 	int newX, newY;
-	Rectangle bounds = computeBounds ();
 
 	if ((cursorOrientation & SWT.LEFT) != 0) {
 		newX = bounds.x;
@@ -287,24 +312,27 @@ Rectangle computeBounds () {
 
 Rectangle [] computeProportions (Rectangle [] rects) {
 	Rectangle [] result = new Rectangle [rects.length];
-	Rectangle bounds = computeBounds ();
+	bounds = computeBounds ();
 	for (int i = 0; i < rects.length; i++) {
-		result[i] = new Rectangle (
-			(rects[i].x - bounds.x) * 100 / bounds.width,
-			(rects[i].y - bounds.y) * 100 / bounds.height,
-			rects[i].width * 100 / bounds.width,
-			rects[i].height * 100 / bounds.height);
+		int x = 0, y = 0, width = 0, height = 0;
+		if (bounds.width != 0) {
+			x = (rects [i].x - bounds.x) * 100 / bounds.width;
+			width = rects [i].width * 100 / bounds.width;
+		} else {
+			width = 100;
+		}
+		if (bounds.height != 0) {
+			y = (rects [i].y - bounds.y) * 100 / bounds.height;
+			height = rects [i].height * 100 / bounds.height;
+		} else {
+			height = 100;
+		}
+		result [i] = new Rectangle (x, y, width, height);			
 	}
 	return result;
 }
 
-void drawRectangles (Rectangle [] rects) {
-	if (parent != null) {
-		if (parent.isDisposed ()) return;
-		parent.getShell ().update ();
-	} else {
-		display.update ();
-	}
+void drawRectangles (Rectangle [] rects, boolean stippled) {
 	int rid = OS.Ph_DEV_RID;
 	if (parent != null) rid = OS.PtWidgetRid (parent.handle);
 	
@@ -312,7 +340,7 @@ void drawRectangles (Rectangle [] rects) {
 	if (phGC == 0) return;
 	int prevContext = OS.PgSetGC (phGC);
 	OS.PgSetRegion (rid);
-	OS.PgSetDrawMode (OS.Pg_DRAWMODE_XOR);
+	OS.PgSetDrawMode (OS.Pg_DrawModeDSx);
 	OS.PgSetFillColor (0xffffff);
 	
 	int bandWidth = 0;
@@ -335,10 +363,6 @@ void drawRectangles (Rectangle [] rects) {
 	OS.PgDestroyGC (phGC);
 }
 
-public Display getDisplay () {
-	return display;
-}
-
 /**
  * Returns the bounds that are being drawn, expressed relative to the parent
  * widget.  If the parent is a <code>Display</code> then these are screen
@@ -353,7 +377,14 @@ public Display getDisplay () {
  */
 public Rectangle [] getRectangles () {
 	checkWidget();
-	return rectangles;
+	int length = 0;
+	if (rectangles != null) length = rectangles.length;
+	Rectangle [] result = new Rectangle [length];
+	for (int i = 0; i < length; i++) {
+		Rectangle current = rectangles [i];
+		result [i] = new Rectangle (current.x, current.y, current.width, current.height);
+	}
+	return result;
 }
 
 /**
@@ -372,11 +403,11 @@ public boolean getStippled () {
 }
 
 void moveRectangles (int xChange, int yChange) {
-	if (xChange < 0 && ((style & SWT.LEFT) == 0)) return;
-	if (xChange > 0 && ((style & SWT.RIGHT) == 0)) return;
-	if (yChange < 0 && ((style & SWT.UP) == 0)) return;
-	if (yChange > 0 && ((style & SWT.DOWN) == 0)) return;
-	Rectangle bounds = computeBounds ();
+	if (xChange < 0 && ((style & SWT.LEFT) == 0)) xChange = 0;
+	if (xChange > 0 && ((style & SWT.RIGHT) == 0)) xChange = 0;
+	if (yChange < 0 && ((style & SWT.UP) == 0)) yChange = 0;
+	if (yChange > 0 && ((style & SWT.DOWN) == 0)) yChange = 0;
+	if (xChange == 0 && yChange == 0) return;
 	bounds.x += xChange; bounds.y += yChange;
 	for (int i = 0; i < rectangles.length; i++) {
 		rectangles [i].x += xChange;
@@ -440,8 +471,20 @@ public boolean open () {
 	PhEvent_t phEvent = new PhEvent_t ();
 	Event event = new Event ();
 	Point cursorPos;
+	
+	// if exactly one of UP/DOWN is specified as a style then set the cursor
+	// orientation accordingly (the same is done for LEFT/RIGHT styles below)
+	int vStyle = style & (SWT.UP | SWT.DOWN);
+	if (vStyle == SWT.UP || vStyle == SWT.DOWN) {
+		cursorOrientation |= vStyle;
+	}
+	int hStyle = style & (SWT.LEFT | SWT.RIGHT);
+	if (hStyle == SWT.LEFT || hStyle == SWT.RIGHT) {
+		cursorOrientation |= hStyle;
+	}
 
-	drawRectangles (rectangles);
+	update ();
+	drawRectangles (rectangles, stippled);
 	if ((style & SWT.MENU) == 0) {
 		oldX = info.pos_x;
 		oldY = info.pos_y;
@@ -479,26 +522,98 @@ public boolean open () {
 					int newX = pe.pos_x;
 					int newY = pe.pos_y;
 					if (newX != oldX || newY != oldY) {
-						drawRectangles (rectangles);
+						Rectangle [] oldRectangles = rectangles;
+						boolean oldStippled = stippled;
+						Rectangle [] rectsToErase = new Rectangle [rectangles.length];
+						for (int i = 0; i < rectangles.length; i++) {
+							Rectangle current = rectangles [i];
+							rectsToErase [i] = new Rectangle (current.x, current.y, current.width, current.height);
+						}
 						event.x = newX;
 						event.y = newY;
 						if ((style & SWT.RESIZE) != 0) {
 							resizeRectangles (newX - oldX, newY - oldY);
+							sendEvent (SWT.Resize, event);
+							/*
+							* It is possible (but unlikely), that application
+							* code could have disposed the widget in the move
+							* event.  If this happens, return false to indicate
+							* that the tracking has failed.
+							*/
+							if (isDisposed ()) {
+								cancelled = true;
+								break;
+							}
+							boolean draw = false;
+							/*
+							 * It is possible that application code could have
+							 * changed the rectangles in the resize event.  If this
+							 * happens then only redraw the tracker if the rectangle
+							 * values have changed.
+							 */
+							if (rectangles != oldRectangles) {
+								int length = rectangles.length;
+								if (length != rectsToErase.length) {
+									draw = true;
+								} else {
+									for (int i = 0; i < length; i++) {
+										if (!rectangles [i].equals (rectsToErase [i])) {
+											draw = true;
+											break;
+										}
+									}
+								}
+							} else {
+								draw = true;
+							}
+							if (draw) {
+								drawRectangles (rectsToErase, oldStippled);
+								update ();
+								drawRectangles (rectangles, stippled);
+							}
 							cursorPos = adjustResizeCursor ();
 							newX = cursorPos.x; newY = cursorPos.y;
-							sendEvent (SWT.Resize, event);
 						} else {
 							moveRectangles (newX - oldX, newY - oldY);
 							sendEvent (SWT.Move, event);
+							/*
+							* It is possible (but unlikely), that application
+							* code could have disposed the widget in the move
+							* event.  If this happens, return false to indicate
+							* that the tracking has failed.
+							*/
+							if (isDisposed ()) {
+								cancelled = true;
+								break;
+							}
+							boolean draw = false;
+							/*
+							 * It is possible that application code could have
+							 * changed the rectangles in the move event.  If this
+							 * happens then only redraw the tracker if the rectangle
+							 * values have changed.
+							 */
+							if (rectangles != oldRectangles) {
+								int length = rectangles.length;
+								if (length != rectsToErase.length) {
+									draw = true;
+								} else {
+									for (int i = 0; i < length; i++) {
+										if (!rectangles [i].equals (rectsToErase [i])) {
+											draw = true;
+											break;
+										}
+									}
+								}
+							} else {
+								draw = true;
+							}
+							if (draw) {
+								drawRectangles (rectsToErase, oldStippled);
+								update ();
+								drawRectangles (rectangles, stippled);
+							}
 						}
-						/*
-						* It is possible (but unlikely), that application
-						* code could have disposed the widget in the move
-						* event.  If this happens, return false to indicate
-						* that the tracking has failed.
-						*/
-						if (isDisposed ()) return false;
-						drawRectangles (rectangles);
 						oldX = newX;  oldY = newY;
 					}
 					break;
@@ -533,28 +648,100 @@ public boolean open () {
 								break;
 						}
 						if (xChange != 0 || yChange != 0) {
-							drawRectangles (rectangles);
+							Rectangle [] oldRectangles = rectangles;
+							boolean oldStippled = stippled;
+							Rectangle [] rectsToErase = new Rectangle [rectangles.length];
+							for (int i = 0; i < rectangles.length; i++) {
+								Rectangle current = rectangles [i];
+								rectsToErase [i] = new Rectangle (current.x, current.y, current.width, current.height);
+							}
 							int newX = oldX + xChange;
 							int newY = oldY + yChange;
 							event.x = newX;
 							event.y = newY;
 							if ((style & SWT.RESIZE) != 0) {
 								resizeRectangles (xChange, yChange);
-								cursorPos = adjustResizeCursor ();
 								sendEvent (SWT.Resize, event);
+								/*
+								* It is possible (but unlikely) that application
+								* code could have disposed the widget in the move
+								* event.  If this happens return false to indicate
+								* that the tracking has failed.
+								*/
+								if (isDisposed ()) {
+									cancelled = true;
+									break;
+								}
+								boolean draw = false;
+								/*
+								 * It is possible that application code could have
+								 * changed the rectangles in the resize event.  If this
+								 * happens then only redraw the tracker if the rectangle
+								 * values have changed.
+								 */
+								if (rectangles != oldRectangles) {
+									int length = rectangles.length;
+									if (length != rectsToErase.length) {
+										draw = true;
+									} else {
+										for (int i = 0; i < length; i++) {
+											if (!rectangles [i].equals (rectsToErase [i])) {
+												draw = true;
+												break;
+											}
+										}
+									}
+								} else {
+									draw = true;
+								}
+								if (draw) {
+									drawRectangles (rectsToErase, oldStippled);
+									update ();
+									drawRectangles (rectangles, stippled);
+								}
+								cursorPos = adjustResizeCursor ();
 							} else {
 								moveRectangles (xChange, yChange);
-								cursorPos = adjustMoveCursor ();
 								sendEvent (SWT.Move, event);
+								/*
+								* It is possible (but unlikely) that application
+								* code could have disposed the widget in the move
+								* event.  If this happens return false to indicate
+								* that the tracking has failed.
+								*/
+								if (isDisposed ()) {
+									cancelled = true;
+									break;
+								}
+								boolean draw = false;
+								/*
+								 * It is possible that application code could have
+								 * changed the rectangles in the move event.  If this
+								 * happens then only redraw the tracker if the rectangle
+								 * values have changed.
+								 */
+								if (rectangles != oldRectangles) {
+									int length = rectangles.length;
+									if (length != rectsToErase.length) {
+										draw = true;
+									} else {
+										for (int i = 0; i < length; i++) {
+											if (!rectangles [i].equals (rectsToErase [i])) {
+												draw = true;
+												break;
+											}
+										}
+									}
+								} else {
+									draw = true;
+								}
+								if (draw) {
+									drawRectangles (rectsToErase, oldStippled);
+									update ();
+									drawRectangles (rectangles, stippled);
+								}
+								cursorPos = adjustMoveCursor ();
 							}
-							/*
-							* It is possible (but unlikely) that application
-							* code could have disposed the widget in the move
-							* event.  If this happens return false to indicate
-							* that the tracking has failed.
-							*/
-							if (isDisposed ()) return false;
-							drawRectangles (rectangles);
 							oldX = cursorPos.x;  oldY = cursorPos.y;
 						}
 					}
@@ -577,7 +764,11 @@ public boolean open () {
 		}
 		OS.PtEventHandler (buffer);
 	}
-	drawRectangles (rectangles);
+	OS.free (buffer);
+	if (!isDisposed ()) {
+		update ();
+		drawRectangles (rectangles, stippled);
+	}
 	tracking = false;
 	if (region != 0) OS.PtDestroyWidget (region);
 	return !cancelled;
@@ -594,7 +785,7 @@ void releaseWidget () {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is moved or resized.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -611,9 +802,35 @@ public void removeControlListener (ControlListener listener) {
 	checkWidget();
 	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if (eventTable == null) return;
+	eventTable.unhook (SWT.Resize, listener);
 	eventTable.unhook (SWT.Move, listener);
 }
 
+/**
+ * Removes the listener from the collection of listeners who will
+ * be notified when keys are pressed and released on the system keyboard.
+ *
+ * @param listener the listener which should no longer be notified
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see KeyListener
+ * @see #addKeyListener
+ */
+public void removeKeyListener(KeyListener listener) {
+	checkWidget();
+	if (listener == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (eventTable == null) return;
+	eventTable.unhook (SWT.KeyUp, listener);
+	eventTable.unhook (SWT.KeyDown, listener);
+}
+
 void resizeRectangles (int xChange, int yChange) {
 	/*
 	* If the cursor orientation has not been set in the orientation of
@@ -621,14 +838,85 @@ void resizeRectangles (int xChange, int yChange) {
 	*/
 	if (xChange < 0 && ((style & SWT.LEFT) != 0) && ((cursorOrientation & SWT.RIGHT) == 0)) {
 		cursorOrientation |= SWT.LEFT;
-	} else if (xChange > 0 && ((style & SWT.RIGHT) != 0) && ((cursorOrientation & SWT.LEFT) == 0)) {
+	}
+	if (xChange > 0 && ((style & SWT.RIGHT) != 0) && ((cursorOrientation & SWT.LEFT) == 0)) {
 		cursorOrientation |= SWT.RIGHT;
-	} else if (yChange < 0 && ((style & SWT.UP) != 0) && ((cursorOrientation & SWT.DOWN) == 0)) {
+	}
+	if (yChange < 0 && ((style & SWT.UP) != 0) && ((cursorOrientation & SWT.DOWN) == 0)) {
 		cursorOrientation |= SWT.UP;
-	} else if (yChange > 0 && ((style & SWT.DOWN) != 0) && ((cursorOrientation & SWT.UP) == 0)) {
+	}
+	if (yChange > 0 && ((style & SWT.DOWN) != 0) && ((cursorOrientation & SWT.UP) == 0)) {
 		cursorOrientation |= SWT.DOWN;
 	}
-	Rectangle bounds = computeBounds ();
+	
+	/*
+	 * If the bounds will flip about the x or y axis then apply the adjustment
+	 * up to the axis (ie.- where bounds width/height becomes 0), change the
+	 * cursor's orientation accordingly, and flip each Rectangle's origin (only
+	 * necessary for > 1 Rectangles) 
+	 */
+	if ((cursorOrientation & SWT.LEFT) != 0) {
+		if (xChange > bounds.width) {
+			if ((style & SWT.RIGHT) == 0) return;
+			cursorOrientation |= SWT.RIGHT;
+			cursorOrientation &= ~SWT.LEFT;
+			bounds.x += bounds.width;
+			xChange -= bounds.width;
+			bounds.width = 0;
+			if (proportions.length > 1) {
+				for (int i = 0; i < proportions.length; i++) {
+					Rectangle proportion = proportions [i];
+					proportion.x = 100 - proportion.x - proportion.width;
+				}
+			}
+		}
+	} else if ((cursorOrientation & SWT.RIGHT) != 0) {
+		if (bounds.width < -xChange) {
+			if ((style & SWT.LEFT) == 0) return;
+			cursorOrientation |= SWT.LEFT;
+			cursorOrientation &= ~SWT.RIGHT;
+			xChange += bounds.width;
+			bounds.width = 0;
+			if (proportions.length > 1) {
+				for (int i = 0; i < proportions.length; i++) {
+					Rectangle proportion = proportions [i];
+					proportion.x = 100 - proportion.x - proportion.width;
+				}
+			}
+		}
+	}
+	if ((cursorOrientation & SWT.UP) != 0) {
+		if (yChange > bounds.height) {
+			if ((style & SWT.DOWN) == 0) return;
+			cursorOrientation |= SWT.DOWN;
+			cursorOrientation &= ~SWT.UP;
+			bounds.y += bounds.height;
+			yChange -= bounds.height;
+			bounds.height = 0;
+			if (proportions.length > 1) {
+				for (int i = 0; i < proportions.length; i++) {
+					Rectangle proportion = proportions [i];
+					proportion.y = 100 - proportion.y - proportion.height;
+				}
+			}
+		}
+	} else if ((cursorOrientation & SWT.DOWN) != 0) {
+		if (bounds.height < -yChange) {
+			if ((style & SWT.UP) == 0) return;
+			cursorOrientation |= SWT.UP;
+			cursorOrientation &= ~SWT.DOWN;
+			yChange += bounds.height;
+			bounds.height = 0;
+			if (proportions.length > 1) {
+				for (int i = 0; i < proportions.length; i++) {
+					Rectangle proportion = proportions [i];
+					proportion.y = 100 - proportion.y - proportion.height;
+				}
+			}
+		}
+	}
+	
+	// apply the bounds adjustment
 	if ((cursorOrientation & SWT.LEFT) != 0) {
 		bounds.x += xChange;
 		bounds.width -= xChange;
@@ -641,10 +929,6 @@ void resizeRectangles (int xChange, int yChange) {
 	} else if ((cursorOrientation & SWT.DOWN) != 0) {
 		bounds.height += yChange;
 	}
-	/*
-	* The following are conditions under which the resize should not be applied
-	*/
-	if (bounds.width < 0 || bounds.height < 0) return;
 	
 	Rectangle [] newRects = new Rectangle [rectangles.length];
 	for (int i = 0; i < rectangles.length; i++) {
@@ -716,6 +1000,9 @@ void setCursor (int cursorHandle) {
  *
  * @param rectangles the bounds of the rectangles to be drawn
  *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the set of rectangles is null or contains a null rectangle</li>
+ * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
@@ -724,7 +1011,13 @@ void setCursor (int cursorHandle) {
 public void setRectangles (Rectangle [] rectangles) {
 	checkWidget();
 	if (rectangles == null) error (SWT.ERROR_NULL_ARGUMENT);
-	this.rectangles = rectangles;
+	int length = rectangles.length;
+	this.rectangles = new Rectangle [length];
+	for (int i = 0; i < length; i++) {
+		Rectangle current = rectangles [i];
+		if (current == null) error (SWT.ERROR_NULL_ARGUMENT);
+		this.rectangles [i] = new Rectangle (current.x, current.y, current.width, current.height);
+	}
 	proportions = computeProportions (rectangles);
 }
 
@@ -743,4 +1036,12 @@ public void setStippled (boolean stippled) {
 	this.stippled = stippled;
 }
 
+void update () {
+	if (parent != null) {
+		if (parent.isDisposed ()) return;
+		parent.getShell ().update ();
+	} else {
+		display.update ();
+	}
+}
 }
