@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -12,6 +12,7 @@ package org.eclipse.swt.graphics;
 
  
 import org.eclipse.swt.internal.*;
+import org.eclipse.swt.internal.gdip.*;
 import org.eclipse.swt.internal.win32.*;
 import org.eclipse.swt.*;
 
@@ -30,9 +31,16 @@ public abstract class Device implements Drawable {
 	Error [] errors;
 	Object [] objects;
 	
-	/* Palette 
-	* (Warning: This field is platform dependent)
-	*/
+	/**
+	 * Palette 
+	 * (Warning: This field is platform dependent)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
+	 */
 	public int hPalette = 0;
 	int [] colorRefCount;
 	
@@ -42,9 +50,19 @@ public abstract class Device implements Drawable {
 	/* Font Enumeration */
 	int nFonts = 256;
 	LOGFONT [] logFonts;
+	TEXTMETRIC metrics;
+	int[] pixels;
+
+	/* Scripts */
+	int [] scripts;
+
+	/* Advanced Graphics */
+	int [] gdipToken;
 
 	boolean disposed;
 	
+	final static Object CREATE_LOCK = new Object();
+
 	/*
 	* TEMPORARY CODE. When a graphics object is
 	* created and the device parameter is null,
@@ -61,20 +79,35 @@ public abstract class Device implements Drawable {
 	protected static Runnable DeviceFinder;
 	static {
 		try {
-			Class.forName ("org.eclipse.swt.widgets.Display");
+			Class.forName ("org.eclipse.swt.widgets.Display"); //$NON-NLS-1$
 		} catch (Throwable e) {}
 	}	
 
 /*
 * TEMPORARY CODE.
 */
-static Device getDevice () {
+static synchronized Device getDevice () {
 	if (DeviceFinder != null) DeviceFinder.run();
-	Device device = CurrentDevice;
+	Device device = CurrentDevice;	
 	CurrentDevice = null;
 	return device;
 }
-	
+
+/**
+ * Constructs a new instance of this class.
+ * <p>
+ * You must dispose the device when it is no longer required. 
+ * </p>
+ *
+ * @see #create
+ * @see #init
+ * 
+ * @since 3.1
+ */
+public Device() {
+	this(null);
+}
+
 /**
  * Constructs a new instance of this class.
  * <p>
@@ -88,19 +121,21 @@ static Device getDevice () {
  * @see DeviceData
  */
 public Device(DeviceData data) {
-	if (data != null) {
-		debug = data.debug;
-		tracking = data.tracking;
+	synchronized (CREATE_LOCK) {
+		if (data != null) {
+			debug = data.debug;
+			tracking = data.tracking;
+		}
+		create (data);
+		init ();
+		if (tracking) {
+			errors = new Error [128];
+			objects = new Object [128];
+		}
+		
+		/* Initialize the system font slot */
+		systemFont = getSystemFont().handle;
 	}
-	create (data);
-	init ();
-	if (tracking) {
-		errors = new Error [128];
-		objects = new Object [128];
-	}
-	
-	/* Initialize the system font slot */
-	systemFont = getSystemFont().handle;
 }
 
 /**
@@ -126,6 +161,24 @@ protected void checkDevice () {
 	if (disposed) SWT.error(SWT.ERROR_DEVICE_DISPOSED);
 }
 
+void checkGDIP() {
+	if (gdipToken != null) return;
+	if (OS.IsWinCE) SWT.error(SWT.ERROR_NOT_IMPLEMENTED);
+    int oldErrorMode = OS.SetErrorMode (OS.SEM_FAILCRITICALERRORS);
+	try {
+		int [] token = new int [1];
+		GdiplusStartupInput input = new GdiplusStartupInput ();
+		input.GdiplusVersion = 1;
+		if (Gdip.GdiplusStartup (token, input, 0) == 0) {
+			gdipToken = token;
+		}
+	} catch (Throwable t) {
+		SWT.error (SWT.ERROR_NO_GRAPHICS_LIBRARY, t, " [GDI+ is required]"); //$NON-NLS-1$
+	} finally {
+        OS.SetErrorMode (oldErrorMode);
+    }
+}
+
 /**
  * Creates the device in the operating system.  If the device
  * does not have a handle, this method may do nothing depending
@@ -151,7 +204,7 @@ int computePixels(int height) {
 	return pixels;
 }
 
-int computePoints(LOGFONT logFont) {
+int computePoints(LOGFONT logFont, int hFont) {
 	int hDC = internal_new_GC (null);
 	int logPixelsY = OS.GetDeviceCaps(hDC, OS.LOGPIXELSY);
 	int pixels = 0; 
@@ -161,15 +214,12 @@ int computePoints(LOGFONT logFont) {
 		 * is positive, the lfHeight measures the height of the entire
 		 * cell, including internal leading, in logical units. Since the
 		 * height of a font in points does not include the internal leading,
-		 * we must subtract the internal leading, which requires a TEXTMETRIC,
-		 * which in turn requires font creation.
+		 * we must subtract the internal leading, which requires a TEXTMETRIC.
 		 */
-		int hFont = OS.CreateFontIndirect(logFont);
 		int oldFont = OS.SelectObject(hDC, hFont);
-		TEXTMETRIC lptm = new TEXTMETRIC();
+		TEXTMETRIC lptm = OS.IsUnicode ? (TEXTMETRIC)new TEXTMETRICW() : new TEXTMETRICA();
 		OS.GetTextMetrics(hDC, lptm);
 		OS.SelectObject(hDC, oldFont);
-		OS.DeleteObject(hFont);
 		pixels = logFont.lfHeight - lptm.tmInternalLeading;
 	} else {
 		pixels = -logFont.lfHeight;
@@ -230,18 +280,37 @@ void dispose_Object (Object object) {
 
 int EnumFontFamProc (int lpelfe, int lpntme, int FontType, int lParam) {
 	boolean isScalable = (FontType & OS.RASTER_FONTTYPE) == 0;
-	if ((lParam == 1) != isScalable) return 1;
-	
-	/* Add the log font to the list of log fonts */
-	if (nFonts == logFonts.length) {
-		LOGFONT [] newLogFonts = new LOGFONT [logFonts.length + 128];
-		System.arraycopy (logFonts, 0, newLogFonts, 0, nFonts);
-		logFonts = newLogFonts;
+	boolean scalable = lParam == 1;
+	if (isScalable == scalable) {
+		/* Add the log font to the list of log fonts */
+		if (nFonts == logFonts.length) {
+			LOGFONT [] newLogFonts = new LOGFONT [logFonts.length + 128];
+			System.arraycopy (logFonts, 0, newLogFonts, 0, nFonts);
+			logFonts = newLogFonts;
+			int[] newPixels = new int[newLogFonts.length];
+			System.arraycopy (pixels, 0, newPixels, 0, nFonts);
+			pixels = newPixels;
+		}
+		LOGFONT logFont = logFonts [nFonts];
+		if (logFont == null) logFont = OS.IsUnicode ? (LOGFONT)new LOGFONTW () : new LOGFONTA ();
+		OS.MoveMemory (logFont, lpelfe, LOGFONT.sizeof);
+		logFonts [nFonts] = logFont;
+		if (logFont.lfHeight > 0) {
+			/*
+			 * Feature in Windows. If the lfHeight of the LOGFONT structure
+			 * is positive, the lfHeight measures the height of the entire
+			 * cell, including internal leading, in logical units. Since the
+			 * height of a font in points does not include the internal leading,
+			 * we must subtract the internal leading, which requires a TEXTMETRIC,
+			 * which in turn requires font creation.
+			 */
+			OS.MoveMemory(metrics, lpntme, TEXTMETRIC.sizeof);
+			pixels[nFonts] = logFont.lfHeight - metrics.tmInternalLeading;
+		} else {
+			pixels[nFonts] = -logFont.lfHeight;
+		}
+		nFonts++;
 	}
-	LOGFONT logFont = logFonts [nFonts];
-	if (logFont == null) logFont = new LOGFONT ();
-	OS.MoveMemory (logFont, lpelfe, LOGFONT.sizeof);
-	logFonts [nFonts++] = logFont;
 	return 1;
 }
 
@@ -362,7 +431,7 @@ public Point getDPI () {
  * <code>faceName</code> is null, all fonts will be returned.
  *
  * @param faceName the name of the font to look for, or null
- * @param scalable true if scalable fonts should be returned.
+ * @param scalable if true only scalable fonts are returned, otherwise only non-scalable fonts are returned.
  * @return the matching font data
  *
  * @exception SWTException <ul>
@@ -373,13 +442,16 @@ public FontData [] getFontList (String faceName, boolean scalable) {
 	checkDevice ();
 	
 	/* Create the callback */
-	Callback callback = new Callback (this, "EnumFontFamProc", 4);
+	Callback callback = new Callback (this, "EnumFontFamProc", 4); //$NON-NLS-1$
 	int lpEnumFontFamProc = callback.getAddress ();
-		
+	if (lpEnumFontFamProc == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
+	
 	/* Initialize the instance variables */
+	metrics = OS.IsUnicode ? (TEXTMETRIC)new TEXTMETRICW() : new TEXTMETRICA();
+	pixels = new int[nFonts];
 	logFonts = new LOGFONT [nFonts];
 	for (int i=0; i<logFonts.length; i++) {
-		logFonts [i] = new LOGFONT ();
+		logFonts [i] = OS.IsUnicode ? (LOGFONT) new LOGFONTW () : new LOGFONTA ();
 	}
 	nFonts = 0;
 
@@ -403,18 +475,11 @@ public FontData [] getFontList (String faceName, boolean scalable) {
 			 * should enumerate for each available style of that font. Instead, it only enumerates
 			 * once. The fix is to call EnumFontFamilies, which works as expected.
 			 */
-			char [] buffer = new char [] {
-				lf.lfFaceName0,  lf.lfFaceName1,  lf.lfFaceName2,  lf.lfFaceName3,
-				lf.lfFaceName4,  lf.lfFaceName5,  lf.lfFaceName6,  lf.lfFaceName7,
-				lf.lfFaceName8,  lf.lfFaceName9,  lf.lfFaceName10, lf.lfFaceName11,
-				lf.lfFaceName12, lf.lfFaceName13, lf.lfFaceName14, lf.lfFaceName15,
-				lf.lfFaceName16, lf.lfFaceName17, lf.lfFaceName18, lf.lfFaceName19,
-				lf.lfFaceName20, lf.lfFaceName21, lf.lfFaceName22, lf.lfFaceName23,
-				lf.lfFaceName24, lf.lfFaceName25, lf.lfFaceName26, lf.lfFaceName27,
-				lf.lfFaceName28, lf.lfFaceName29, lf.lfFaceName30, lf.lfFaceName31,
-			};
-			TCHAR lpFaceName = new TCHAR (0, new String(buffer), true);
-			OS.EnumFontFamilies (hDC, lpFaceName, lpEnumFontFamProc, scalable ? 1 : 0);
+			if (OS.IsUnicode) {
+				OS.EnumFontFamiliesW (hDC, ((LOGFONTW)lf).lfFaceName, lpEnumFontFamProc, scalable ? 1 : 0);
+			} else {
+				OS.EnumFontFamiliesA (hDC, ((LOGFONTA)lf).lfFaceName, lpEnumFontFamProc, scalable ? 1 : 0);
+			}
 		}
 	} else {
 		/* Use the character encoding for the default locale */
@@ -426,29 +491,51 @@ public FontData [] getFontList (String faceName, boolean scalable) {
 		 */
 		OS.EnumFontFamilies (hDC, lpFaceName, lpEnumFontFamProc, scalable ? 1 : 0);
 	}
+	int logPixelsY = OS.GetDeviceCaps(hDC, OS.LOGPIXELSY);
 	internal_dispose_GC (hDC, null);
 
 	/* Create the fontData from the logfonts */
 	int count = nFonts - offset;
 	FontData [] result = new FontData [count];
 	for (int i=0; i<count; i++) {
-		LOGFONT logFont = logFonts [i+offset];
-		result [i] = FontData.win32_new (logFont, computePoints(logFont));
+		int index = i + offset;
+		result [i] = FontData.win32_new (logFonts [index], Compatibility.round(pixels [index] * 72, logPixelsY));
 	}
 	
 	/* Clean up */
 	callback.dispose ();
 	logFonts = null;
+	pixels = null;
+	metrics = null;
 	return result;
 }
 
+String getLastError () {
+	int error = OS.GetLastError();
+	if (error == 0) return ""; //$NON-NLS-1$
+	return " [GetLastError=0x" + Integer.toHexString(error) + "]"; //$NON-NLS-1$ //$NON-NLS-2$
+}
+
+String getLastErrorText () {
+	int error = OS.GetLastError();
+	if (error == 0) return ""; //$NON-NLS-1$
+	int[] buffer = new int[1];
+	int dwFlags = OS.FORMAT_MESSAGE_ALLOCATE_BUFFER | OS.FORMAT_MESSAGE_FROM_SYSTEM | OS.FORMAT_MESSAGE_IGNORE_INSERTS;
+	int length = OS.FormatMessage(dwFlags, 0, error, OS.LANG_USER_DEFAULT, buffer, 0, 0);
+	if (length == 0) return " [GetLastError=0x" + Integer.toHexString(error) + "]"; //$NON-NLS-1$ //$NON-NLS-2$
+	TCHAR buffer1 = new TCHAR(0, length);
+	OS.MoveMemory(buffer1, buffer[0], length * TCHAR.sizeof);
+	if (buffer[0] != 0) OS.LocalFree(buffer[0]);
+	return buffer1.toString(0, length);
+}
+
 /**
  * Returns the matching standard color for the given
  * constant, which should be one of the color constants
  * specified in class <code>SWT</code>. Any value other
  * than one of the SWT color constants which is passed
  * in will result in the color black. This color should
- * not be free'd because it was allocated by the system,
+ * not be freed because it was allocated by the system,
  * not the application.
  *
  * @param id the color constant
@@ -488,7 +575,7 @@ public Color getSystemColor (int id) {
  * Returns a reasonable font for applications to use.
  * On some platforms, this will match the "default font"
  * or "system font" if such can be found.  This font
- * should not be free'd because it was allocated by the
+ * should not be freed because it was allocated by the
  * system, not the application.
  * <p>
  * Typically, applications which want the default look
@@ -539,6 +626,18 @@ public boolean getWarnings () {
  * @see #create
  */
 protected void init () {
+	if (debug) {
+		if (!OS.IsWinCE) OS.GdiSetBatchLimit(1);
+	}
+
+	/* Initialize scripts list */
+	if (!OS.IsWinCE) {
+		int [] ppSp = new int [1];
+		int [] piNumScripts = new int [1];
+		OS.ScriptGetProperties (ppSp, piNumScripts);
+		scripts = new int [piNumScripts [0]];
+		OS.MoveMemory (scripts, ppSp [0], scripts.length * 4);
+	}
 	
 	/*
 	 * If we're not on a device which supports palettes,
@@ -628,7 +727,7 @@ public abstract int internal_new_GC (GCData data);
  * application code.
  * </p>
  *
- * @param handle the platform specific GC handle
+ * @param hDC the platform specific GC handle
  * @param data the platform specific GC data 
  */
 public abstract void internal_dispose_GC (int hDC, GCData data);
@@ -689,6 +788,11 @@ void new_Object (Object object) {
  * @see #destroy
  */
 protected void release () {
+	if (gdipToken != null) {
+		Gdip.GdiplusShutdown (gdipToken);
+	}
+	gdipToken = null;
+	scripts = null;
 	if (hPalette != 0) OS.DeleteObject (hPalette);
 	hPalette = 0;
 	colorRefCount = null;
@@ -698,11 +802,11 @@ protected void release () {
 
 /**
  * If the underlying window system supports printing warning messages
- * to the console, setting warnings to <code>true</code> prevents these
- * messages from being printed. If the argument is <code>false</code>
+ * to the console, setting warnings to <code>false</code> prevents these
+ * messages from being printed. If the argument is <code>true</code> then
  * message printing is not blocked.
  *
- * @param warnings <code>true</code>if warnings should be handled, and <code>false</code> otherwise
+ * @param warnings <code>true</code>if warnings should be printed, and <code>false</code> otherwise
  *
  * @exception SWTException <ul>
  *    <li>ERROR_DEVICE_DISPOSED - if the receiver has been disposed</li>
