@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -14,6 +14,7 @@ package org.eclipse.swt.graphics;
 import org.eclipse.swt.internal.gdip.*;
 import org.eclipse.swt.internal.win32.*;
 import org.eclipse.swt.*;
+
 import java.io.*;
 
 /**
@@ -69,6 +70,12 @@ public final class Image extends Resource implements Drawable {
 	/**
 	 * specifies whether the receiver is a bitmap or an icon
 	 * (one of <code>SWT.BITMAP</code>, <code>SWT.ICON</code>)
+	 * <p>
+	 * <b>IMPORTANT:</b> This field is <em>not</em> part of the SWT
+	 * public API. It is marked public only so that it can be shared
+	 * within the packages provided by SWT. It is not available on all
+	 * platforms and should never be accessed from application code.
+	 * </p>
 	 */
 	public int type;
 	
@@ -110,6 +117,16 @@ public final class Image extends Resource implements Drawable {
 	 */
 	ImageData data;
 	
+	/**
+	 * width of the image
+	 */
+	int width = -1;
+	
+	/**
+	 * height of the image
+	 */
+	int height = -1;
+	
 	/**
 	 * specifies the default scanline padding
 	 */
@@ -182,9 +199,8 @@ public Image(Device device, int width, int height) {
  *    <li>ERROR_INVALID_ARGUMENT - if the image has been disposed</li>
  * </ul>
  * @exception SWTException <ul>
- *    <li>ERROR_INVALID_IMAGE - if the image is not a bitmap or an icon, or
- *          is otherwise in an invalid state</li>
- *    <li>ERROR_UNSUPPORTED_DEPTH - if the depth of the Image is not supported</li>
+ *    <li>ERROR_INVALID_IMAGE - if the image is not a bitmap or an icon, or is otherwise in an invalid state</li>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the depth of the image is not supported</li>
  * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
@@ -196,9 +212,9 @@ public Image(Device device, Image srcImage, int flag) {
 	this.device = device;
 	if (srcImage == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 	if (srcImage.isDisposed()) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
+	Rectangle rect = srcImage.getBounds();
 	switch (flag) {
 		case SWT.IMAGE_COPY: {
-			Rectangle r = srcImage.getBounds();
 			this.type = srcImage.type;
 			switch (type) {
 				case SWT.BITMAP:
@@ -209,10 +225,12 @@ public Image(Device device, Image srcImage, int flag) {
 					int hdcSource = OS.CreateCompatibleDC(hDC);
 					int hdcDest = OS.CreateCompatibleDC(hDC);
 					int hOldSrc = OS.SelectObject(hdcSource, srcImage.handle);
-					handle = OS.CreateCompatibleBitmap(hdcSource, r.width, r.height);
+					BITMAP bm = new BITMAP();
+					OS.GetObject(srcImage.handle, BITMAP.sizeof, bm);
+					handle = OS.CreateCompatibleBitmap(hdcSource, rect.width, bm.bmBits != 0 ? -rect.height : rect.height);
 					if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 					int hOldDest = OS.SelectObject(hdcDest, handle);
-					OS.BitBlt(hdcDest, 0, 0, r.width, r.height, hdcSource, 0, 0, OS.SRCCOPY);
+					OS.BitBlt(hdcDest, 0, 0, rect.width, rect.height, hdcSource, 0, 0, OS.SRCCOPY);
 					OS.SelectObject(hdcSource, hOldSrc);
 					OS.SelectObject(hdcDest, hOldDest);
 					OS.DeleteDC(hdcSource);
@@ -232,203 +250,77 @@ public Image(Device device, Image srcImage, int flag) {
 					if (OS.IsWinCE) {
 						init(device, srcImage.data);
 					} else {
-						handle = OS.CopyImage(srcImage.handle, OS.IMAGE_ICON, r.width, r.height, 0);
+						handle = OS.CopyImage(srcImage.handle, OS.IMAGE_ICON, rect.width, rect.height, 0);
 						if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 					}
 					break;
 				default:
-					SWT.error(SWT.ERROR_UNSUPPORTED_FORMAT);
+					SWT.error(SWT.ERROR_INVALID_IMAGE);
 			}
 			if (device.tracking) device.new_Object(this);	
 			return;
 		}
 		case SWT.IMAGE_DISABLE: {
-			Rectangle r = srcImage.getBounds();
-			this.type = srcImage.type;
-			byte[] rgbBwBitmapInfo = {
-				40,0,0,0, /* biSize */
-				(byte)(r.width & 0xFF), /* biWidth */
-				(byte)((r.width & 0xFF00) >> 8),
-				(byte)((r.width & 0xFF0000) >> 16),
-				(byte)((r.width & 0xFF000000) >> 24),
-				(byte)(r.height & 0xFF), /* biHeight */
-				(byte)((r.height & 0xFF00) >> 8),
-				(byte)((r.height & 0xFF0000) >> 16),
-				(byte)((r.height & 0xFF000000) >> 24),
-				1,0, /* biPlanes */
-				1,0, /* biBitCount */
-				0,0,0,0, /* biCompression */
-				0,0,0,0, /* biSizeImage */
-				0,0,0,0, /* biXPelsPerMeter */
-				0,0,0,0, /* biYPelsPerMeter */
-				0,0,0,0, /* biClrUsed */
-				0,0,0,0, /* biClrImportant */
-				0,0,0,0, /* First color: black */
-				(byte)0xFF,(byte)0xFF,(byte)0xFF,0 /* Second color: white */
-			};
+			ImageData data = srcImage.getImageData();
+			PaletteData palette = data.palette;
+			RGB[] rgbs = new RGB[3];
+			rgbs[0] = device.getSystemColor(SWT.COLOR_BLACK).getRGB();
+			rgbs[1] = device.getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW).getRGB();
+			rgbs[2] = device.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND).getRGB();
+			ImageData newData = new ImageData(rect.width, rect.height, 8, new PaletteData(rgbs));
+			newData.alpha = data.alpha;
+			newData.alphaData = data.alphaData;
+			newData.maskData = data.maskData;
+			newData.maskPad = data.maskPad;
+			if (data.transparentPixel != -1) newData.transparentPixel = 0;
 
-			/* Get the HDC for the device */
-			int hDC = device.internal_new_GC(null);
-
-			/* Source DC */
-			int hdcSource = OS.CreateCompatibleDC(hDC);
-			if (hdcSource == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-			/* Monochrome (Intermediate) DC */
-			int bwDC = OS.CreateCompatibleDC(hdcSource);
-			if (bwDC == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-			/* Destination DC */
-			int hdcBmp = OS.CreateCompatibleDC(hDC);
-			if (hdcBmp == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-			/* Monochrome (Intermediate) DIB section */
-			int[] pbitsBW = new int[1];
-			int hbmBW = OS.CreateDIBSection(bwDC, rgbBwBitmapInfo, OS.DIB_RGB_COLORS, pbitsBW, 0, 0);
-			if (hbmBW == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-			switch (type) {
-				case SWT.BITMAP:
-					/* Attach the bitmap to the source DC */
-					int hOldSrc = OS.SelectObject(hdcSource, srcImage.handle);
-					/* Create the destination bitmap */
-					handle = OS.CreateCompatibleBitmap(hDC, r.width, r.height);
-					if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-					/* Attach the DIB section and the new bitmap to the DCs */
-					int hOldBw = OS.SelectObject(bwDC, hbmBW);
-					int hOldBmp = OS.SelectObject(hdcBmp, handle);
-					/* BitBlt the bitmap into the monochrome DIB section */
-					OS.BitBlt(bwDC, 0, 0, r.width, r.height, hdcSource, 0, 0, OS.SRCCOPY);
-					/* Paint the destination rectangle in gray */
-					RECT rect = new RECT();
-					rect.left = 0;
-					rect.top = 0;
-					rect.right = r.width;
-					rect.bottom = r.height;
-					OS.FillRect(hdcBmp, rect, OS.GetSysColorBrush(OS.COLOR_3DFACE));
-					/*
-					 * BitBlt the black bits in the monochrome bitmap into
-					 * COLOR_3DHILIGHT bits in the destination DC.
-					 * The magic ROP comes from Charles Petzold's book
-					 */
-					int hb = OS.CreateSolidBrush(OS.GetSysColor(OS.COLOR_3DHILIGHT));
-					int oldBrush = OS.SelectObject(hdcBmp, hb);
-					OS.BitBlt(hdcBmp, 1, 1, r.width, r.height, bwDC, 0, 0, 0xB8074A);
-					/*
-					 * BitBlt the black bits in the monochrome bitmap into 
-					 * COLOR_3DSHADOW bits in the destination DC.
-					 */
-					hb = OS.CreateSolidBrush(OS.GetSysColor(OS.COLOR_3DSHADOW));
-					OS.DeleteObject(OS.SelectObject(hdcBmp, hb));
-					OS.BitBlt(hdcBmp, 0, 0, r.width, r.height, bwDC, 0, 0, 0xB8074A);
-					OS.DeleteObject(OS.SelectObject(hdcBmp, oldBrush));
-					/* Free resources */
-					OS.SelectObject(hdcSource, hOldSrc);
-					OS.SelectObject(hdcBmp, hOldBmp);
-					OS.SelectObject(bwDC, hOldBw);
-					OS.DeleteDC(hdcSource);
-					OS.DeleteDC(bwDC);
-					OS.DeleteDC(hdcBmp);
-					OS.DeleteObject(hbmBW);
-					
-					/* Release the HDC for the device */
-					device.internal_dispose_GC(hDC, null);
-					break;
-				case SWT.ICON:
-					/* Get icon information */
-					ICONINFO iconInfo = new ICONINFO();
-					if (OS.IsWinCE) {
-						GetIconInfo(srcImage, iconInfo);
-					} else {
-						if (!OS.GetIconInfo(srcImage.handle, iconInfo))
-							SWT.error(SWT.ERROR_INVALID_IMAGE);
+			/* Convert the pixels. */
+			int[] scanline = new int[rect.width];
+			int[] maskScanline = null;
+			ImageData mask = null;
+			if (data.maskData != null) mask = data.getTransparencyMask();
+			if (mask != null) maskScanline = new int[rect.width];
+			int redMask = palette.redMask;
+			int greenMask = palette.greenMask;
+			int blueMask = palette.blueMask;
+			int redShift = palette.redShift;
+			int greenShift = palette.greenShift;
+			int blueShift = palette.blueShift;
+			for (int y=0; y<rect.height; y++) {
+				int offset = y * newData.bytesPerLine;
+				data.getPixels(0, y, rect.width, scanline, 0);
+				if (mask != null) mask.getPixels(0, y, rect.width, maskScanline, 0);
+				for (int x=0; x<rect.width; x++) {
+					int pixel = scanline[x];
+					if (!(pixel == data.transparentPixel || (mask != null && maskScanline[x] == 0))) {
+						int red, green, blue;
+						if (palette.isDirect) {
+							red = pixel & redMask;
+							red = (redShift < 0) ? red >>> -redShift : red << redShift;
+							green = pixel & greenMask;
+							green = (greenShift < 0) ? green >>> -greenShift : green << greenShift;
+							blue = pixel & blueMask;
+							blue = (blueShift < 0) ? blue >>> -blueShift : blue << blueShift;
+						} else {
+							red = palette.colors[pixel].red;
+							green = palette.colors[pixel].green;
+							blue = palette.colors[pixel].blue;
+						}
+						int intensity = red * red + green * green + blue * blue;
+						if (intensity < 98304) {
+							newData.data[offset] = (byte)1;
+						} else {
+							newData.data[offset] = (byte)2;	
+						}
 					}
-					int hdcMask = OS.CreateCompatibleDC(hDC);
-					/* Create the destination bitmaps */
-					if (iconInfo.hbmColor == 0)
-						hOldSrc = OS.SelectObject(hdcSource, iconInfo.hbmMask);
-					else
-						hOldSrc = OS.SelectObject(hdcSource, iconInfo.hbmColor);
-					int newHbmp = OS.CreateCompatibleBitmap(hdcSource, r.width, r.height);
-					if (newHbmp == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-					int newHmask = OS.CreateBitmap(r.width, r.height, 1, 1, null);
-					if (newHmask == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-					/* BitBlt the source mask into the destination mask */
-					int hOldMask = OS.SelectObject(hdcMask, newHmask);
-					if (iconInfo.hbmColor != 0)
-						OS.SelectObject(hdcSource, iconInfo.hbmMask);
-					OS.SelectObject(hdcSource, iconInfo.hbmMask);
-					OS.BitBlt(hdcMask, 0, 0, r.width, r.height, hdcSource, 0, 0, OS.SRCCOPY);
-					/* Attach the monochrome DIB section and the destination bitmap to the DCs */
-					hOldBw = OS.SelectObject(bwDC, hbmBW);
-					/* BitBlt the bitmap into the monochrome DIB section */
-					if (iconInfo.hbmColor == 0) {
-						OS.SelectObject(hdcSource, iconInfo.hbmMask);
-						OS.BitBlt(bwDC, 0, 0, r.width, r.height, hdcSource, 0, r.height, OS.SRCCOPY);
-					} else {
-						OS.SelectObject(hdcSource, iconInfo.hbmColor);
-						OS.BitBlt(bwDC, 0, 0, r.width, r.height, hdcSource, 0, 0, OS.SRCCOPY);
-					}
-					/* Paint the destination rectangle in grey */
-					rect = new RECT();
-					rect.left = 0;
-					rect.top = 0;
-					rect.right = r.width;
-					rect.bottom = r.height;
-					hOldBmp = OS.SelectObject(hdcBmp, newHbmp);
-					OS.FillRect(hdcBmp, rect, OS.GetSysColorBrush(OS.COLOR_3DFACE));
-					/*
-					 * BitBlt the black bits in the monochrome bitmap into
-					 * COLOR_3DHILIGHT bits in the destination DC.
-					 * The magic ROP comes from Charles Petzold's book
-					 */
-					hb = OS.CreateSolidBrush(OS.GetSysColor(OS.COLOR_3DSHADOW));
-					oldBrush = OS.SelectObject(hdcBmp, hb);
-					OS.BitBlt(hdcBmp, 0, 0, r.width, r.height, bwDC, 0, 0, 0xB8074A);
-					/* Invert mask into hdcBw */
-					OS.BitBlt(bwDC, 0, 0, r.width, r.height, hdcMask, 0, 0, OS.NOTSRCCOPY);
-					/* Select black brush into destination */
-					hb = OS.CreateSolidBrush(0);
-					OS.DeleteObject(OS.SelectObject(hdcBmp, hb));
-					/*
-					 * Copy black bits from monochrome bitmap into black bits in the
-					 * destination DC.
-					 */
-					OS.BitBlt(hdcBmp, 0, 0, r.width, r.height, bwDC, 0, 0, 0xB8074A);
-					OS.DeleteObject(OS.SelectObject(hdcBmp, oldBrush));
-					/* Free resources */
-					OS.SelectObject(hdcSource, hOldSrc);
-					OS.DeleteDC(hdcSource);
-					OS.SelectObject(bwDC, hOldBw);
-					OS.DeleteDC(bwDC);
-					OS.SelectObject(hdcBmp, hOldBmp);
-					OS.DeleteDC(hdcBmp);
-					OS.SelectObject(hdcMask, hOldMask);
-					OS.DeleteDC(hdcMask);
-					OS.DeleteObject(hbmBW);
-					
-					/* Release the HDC for the device */
-					device.internal_dispose_GC(hDC, null);
-			
-					/* Create the new iconinfo */
-					ICONINFO newIconInfo = new ICONINFO();
-					newIconInfo.fIcon = iconInfo.fIcon;
-					newIconInfo.hbmMask = newHmask;
-					newIconInfo.hbmColor = newHbmp;
-					/* Create the new icon */
-					handle = OS.CreateIconIndirect(newIconInfo);
-					if (handle == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-					/* Free bitmaps */
-					OS.DeleteObject(newHbmp);
-					OS.DeleteObject(newHmask);
-					if (iconInfo.hbmColor != 0)
-						OS.DeleteObject(iconInfo.hbmColor);
-					OS.DeleteObject(iconInfo.hbmMask);
-					break;
-				default:
-					SWT.error(SWT.ERROR_UNSUPPORTED_FORMAT);
+					offset++;
+				}
 			}
-			if (device.tracking) device.new_Object(this);	
+			init (device, newData);
+			if (device.tracking) device.new_Object(this);
 			return;
 		}
 		case SWT.IMAGE_GRAY: {
-			Rectangle r = srcImage.getBounds();
 			ImageData data = srcImage.getImageData();
 			PaletteData palette = data.palette;
 			ImageData newData = data;
@@ -452,23 +344,25 @@ public Image(Device device, Image srcImage, int flag) {
 				for (int i=0; i<rgbs.length; i++) {
 					rgbs[i] = new RGB(i, i, i);
 				}
-				newData = new ImageData(r.width, r.height, 8, new PaletteData(rgbs));
+				newData = new ImageData(rect.width, rect.height, 8, new PaletteData(rgbs));
+				newData.alpha = data.alpha;
+				newData.alphaData = data.alphaData;
 				newData.maskData = data.maskData;
 				newData.maskPad = data.maskPad;
 				if (data.transparentPixel != -1) newData.transparentPixel = 254; 
 
 				/* Convert the pixels. */
-				int[] scanline = new int[r.width];
+				int[] scanline = new int[rect.width];
 				int redMask = palette.redMask;
 				int greenMask = palette.greenMask;
 				int blueMask = palette.blueMask;
 				int redShift = palette.redShift;
 				int greenShift = palette.greenShift;
 				int blueShift = palette.blueShift;
-				for (int y=0; y<r.height; y++) {
+				for (int y=0; y<rect.height; y++) {
 					int offset = y * newData.bytesPerLine;
-					data.getPixels(0, y, r.width, scanline, 0);
-					for (int x=0; x<r.width; x++) {
+					data.getPixels(0, y, rect.width, scanline, 0);
+					for (int x=0; x<rect.width; x++) {
 						int pixel = scanline[x];
 						if (pixel != data.transparentPixel) {
 							int red = pixel & redMask;
@@ -636,11 +530,11 @@ public Image(Device device, ImageData source, ImageData mask) {
  *    <li>ERROR_NULL_ARGUMENT - if the stream is null</li>
  * </ul>
  * @exception SWTException <ul>
- *    <li>ERROR_INVALID_IMAGE - if the image file contains invalid data </li>
- *    <li>ERROR_IO - if an IO error occurs while reading data</li>
- *    <li>ERROR_UNSUPPORTED_DEPTH - if the InputStream describes an image with an unsupported depth</li>
- *    <li>ERROR_UNSUPPORTED_FORMAT - if the image file contains an unrecognized format</li>
- *  * </ul>
+ *    <li>ERROR_IO - if an IO error occurs while reading from the stream</li>
+ *    <li>ERROR_INVALID_IMAGE - if the image stream contains invalid data </li>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the image stream describes an image with an unsupported depth</li>
+ *    <li>ERROR_UNSUPPORTED_FORMAT - if the image stream contains an unrecognized format</li>
+ * </ul>
  * @exception SWTError <ul>
  *    <li>ERROR_NO_HANDLES if a handle could not be obtained for image creation</li>
  * </ul>
@@ -670,9 +564,9 @@ public Image (Device device, InputStream stream) {
  *    <li>ERROR_NULL_ARGUMENT - if the file name is null</li>
  * </ul>
  * @exception SWTException <ul>
+ *    <li>ERROR_IO - if an IO error occurs while reading from the file</li>
  *    <li>ERROR_INVALID_IMAGE - if the image file contains invalid data </li>
- *    <li>ERROR_IO - if an IO error occurs while reading data</li>
- *    <li>ERROR_UNSUPPORTED_DEPTH - if the image file has an unsupported depth</li>
+ *    <li>ERROR_UNSUPPORTED_DEPTH - if the image file describes an image with an unsupported depth</li>
  *    <li>ERROR_UNSUPPORTED_FORMAT - if the image file contains an unrecognized format</li>
  * </ul>
  * @exception SWTError <ul>
@@ -682,8 +576,137 @@ public Image (Device device, InputStream stream) {
 public Image (Device device, String filename) {
 	if (device == null) device = Device.getDevice();
 	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
+	if (filename == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
+	this.device = device;
+	try {
+		device.checkGDIP();
+		int length = filename.length();
+		char[] chars = new char[length+1];
+		filename.getChars(0, length, chars, 0);
+		int bitmap = Gdip.Bitmap_new(chars, false);
+		int status = Gdip.Image_GetLastStatus(bitmap);
+		if (bitmap != 0 && status == 0) {
+			int transparentColor = -1;
+			int error = SWT.ERROR_NO_HANDLES;
+			int width = Gdip.Image_GetWidth(bitmap);
+			int height = Gdip.Image_GetHeight(bitmap);
+			if (filename.toLowerCase().endsWith(".ico")) {
+				this.type = SWT.ICON;
+				int[] hicon = new int[1];
+				Gdip.Bitmap_GetHICON(bitmap, hicon);
+				this.handle = hicon[0];
+			} else {
+				this.type = SWT.BITMAP;
+				int pixelFormat = Gdip.Image_GetPixelFormat(bitmap);
+				switch (pixelFormat) {
+					case Gdip.PixelFormat32bppARGB:
+						int lockedBitmapData = Gdip.BitmapData_new();
+						if (lockedBitmapData == 0) break;
+						Gdip.Bitmap_LockBits(bitmap, 0, 0, pixelFormat, lockedBitmapData);
+						BitmapData bitmapData = new BitmapData();
+						Gdip.MoveMemory(bitmapData, lockedBitmapData, BitmapData.sizeof);
+						int stride = bitmapData.Stride;
+						int pixels = bitmapData.Scan0;
+						byte[] line = new byte[stride];
+		 		 		alphaData = new byte[width * height];
+		 		 		for (int y = 0; y < height; y++) {
+		 		 			OS.MoveMemory(line, pixels + (y * stride), stride);
+		 		 		 	for (int x = 0; x < width; x++) {
+		 		 		 		alphaData[y*width+x] = line[x*4 + 3];
+		 		 		 	}
+		 		 		}
+		 		 		Gdip.Bitmap_UnlockBits(bitmap, lockedBitmapData);
+						Gdip.BitmapData_delete(lockedBitmapData);
+						this.handle = createDIB(width, height, 32);
+						break;
+					case Gdip.PixelFormat1bppIndexed:
+					case Gdip.PixelFormat4bppIndexed:
+					case Gdip.PixelFormat8bppIndexed:
+						int paletteSize = Gdip.Image_GetPaletteSize(bitmap);
+						int hHeap = OS.GetProcessHeap();
+						int palette = OS.HeapAlloc(hHeap, OS.HEAP_ZERO_MEMORY, paletteSize);
+						if (palette == 0) break;
+						Gdip.Image_GetPalette(bitmap, palette, paletteSize);
+						ColorPalette colorPalette = new ColorPalette();
+						Gdip.MoveMemory(colorPalette, palette, ColorPalette.sizeof);
+						int[] entries = new int[colorPalette.Count];
+						OS.MoveMemory(entries, palette + 8, entries.length * 4);
+						OS.HeapFree(hHeap, 0, palette);
+						BITMAPINFOHEADER bmiHeader = new BITMAPINFOHEADER();
+						bmiHeader.biSize = BITMAPINFOHEADER.sizeof;
+						bmiHeader.biWidth = width;
+						bmiHeader.biHeight = -height;
+						bmiHeader.biPlanes = 1;
+						bmiHeader.biBitCount = (short)8;
+						bmiHeader.biCompression = OS.BI_RGB;
+						byte[] bmi = new byte[BITMAPINFOHEADER.sizeof + colorPalette.Count * 4];
+						OS.MoveMemory(bmi, bmiHeader, BITMAPINFOHEADER.sizeof);					
+						int offset = BITMAPINFOHEADER.sizeof;
+						for (int i = 0; i < entries.length; i++) {
+							if (((entries[i] >> 24) & 0xFF) == 0 && (colorPalette.Flags & Gdip.PaletteFlagsHasAlpha) != 0) {
+								transparentPixel = i;
+								transparentColor = entries[i] & 0xFFFFFF;
+							}
+							bmi[offset] = (byte)((entries[i] & 0xFF) >> 0);
+							bmi[offset + 1] = (byte)((entries[i] & 0xFF00) >> 8);
+							bmi[offset + 2] = (byte)((entries[i] & 0xFF0000) >> 16);
+							bmi[offset + 3] = 0;
+							offset += 4;
+						}
+						int[] pBits = new int[1];
+						this.handle = OS.CreateDIBSection(0, bmi, OS.DIB_RGB_COLORS, pBits, 0, 0);						
+						break;
+					case Gdip.PixelFormat16bppGrayScale:
+					case Gdip.PixelFormat16bppRGB555:
+					case Gdip.PixelFormat16bppRGB565:
+						this.handle = createDIB(width, height, 16);
+						break;
+					case Gdip.PixelFormat24bppRGB:
+						this.handle = createDIB(width, height, 24);
+						break;
+					default:
+						this.handle = createDIB(width, height, 32);
+				}
+				if (handle != 0) {
+					int hDC = device.internal_new_GC(null);
+					int srcHDC = OS.CreateCompatibleDC(hDC);
+					int oldSrcBitmap = OS.SelectObject(srcHDC, handle);
+					int graphics = Gdip.Graphics_new(srcHDC);
+					if (graphics != 0) {
+						if (transparentColor != -1) {
+							int color = Gdip.Color_new(transparentColor | 0xFF000000);
+							if (color != 0) {
+								int brush = Gdip.SolidBrush_new(color);
+								if (brush != 0) {
+									Gdip.Graphics_FillRectangle(graphics, brush, 0, 0, width, height);
+									Gdip.SolidBrush_delete(brush);
+								}
+								Gdip.Color_delete(color);
+							}
+						}
+						Rect rect = new Rect();
+						rect.Width = width;
+						rect.Height = height;
+						status = Gdip.Graphics_DrawImage(graphics, bitmap, rect, 0, 0, width, height, Gdip.UnitPixel, 0, 0, 0);
+						if (status != 0) {
+							error = SWT.ERROR_INVALID_IMAGE;
+							OS.DeleteObject(handle);
+							handle = 0;
+						}
+						Gdip.Graphics_delete(graphics);
+					}
+					OS.SelectObject(srcHDC, oldSrcBitmap);
+					OS.DeleteDC(srcHDC);
+					device.internal_dispose_GC(hDC, null);
+				}
+			}
+			Gdip.Bitmap_delete(bitmap);
+			if (handle == 0) SWT.error(error);
+			return;
+		}
+	} catch (SWTException e) {}
 	init(device, new ImageData(filename));
-	if (device.tracking) device.new_Object(this);	
+	if(device.tracking) device.new_Object(this);
 }
 
 /** 
@@ -947,7 +970,7 @@ int[] createGdipImage() {
 			if (iconInfo.hbmMask != 0) OS.DeleteObject(iconInfo.hbmMask);
 			return new int[]{img, pixels};
 		}
-		default: SWT.error(SWT.ERROR_UNSUPPORTED_FORMAT);
+		default: SWT.error(SWT.ERROR_INVALID_IMAGE);
 	}
 	return null;
 }
@@ -1085,14 +1108,17 @@ public Color getBackground() {
  */
 public Rectangle getBounds() {
 	if (isDisposed()) SWT.error(SWT.ERROR_GRAPHIC_DISPOSED);
+	if (width != -1 && height != -1) {
+		return new Rectangle(0, 0, width, height);
+	}
 	switch (type) {
 		case SWT.BITMAP:
 			BITMAP bm = new BITMAP();
 			OS.GetObject(handle, BITMAP.sizeof, bm);
-			return new Rectangle(0, 0, bm.bmWidth, bm.bmHeight);
+			return new Rectangle(0, 0, width = bm.bmWidth, height = bm.bmHeight);
 		case SWT.ICON:
 			if (OS.IsWinCE) {
-				return new Rectangle(0, 0, data.width, data.height);
+				return new Rectangle(0, 0, width = data.width, height = data.height);
 			} else {
 				ICONINFO info = new ICONINFO();
 				OS.GetIconInfo(handle, info);
@@ -1103,10 +1129,10 @@ public Rectangle getBounds() {
 				if (hBitmap == info.hbmMask) bm.bmHeight /= 2;
 				if (info.hbmColor != 0) OS.DeleteObject(info.hbmColor);
 				if (info.hbmMask != 0) OS.DeleteObject(info.hbmMask);
-				return new Rectangle(0, 0, bm.bmWidth, bm.bmHeight);
+				return new Rectangle(0, 0, width = bm.bmWidth, height = bm.bmHeight);
 			}
 		default:
-			SWT.error(SWT.ERROR_UNSUPPORTED_FORMAT);
+			SWT.error(SWT.ERROR_INVALID_IMAGE);
 			return null;
 	}
 }
@@ -1465,7 +1491,7 @@ public ImageData getImageData() {
 			return imageData;
 		}
 		default:
-			SWT.error(SWT.ERROR_UNSUPPORTED_FORMAT);
+			SWT.error(SWT.ERROR_INVALID_IMAGE);
 			return null;
 	}
 }
