@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -45,11 +45,12 @@ public class BidiUtil {
 	 * they are targeted for CLDC.
 	 */
 	//	static Callback callback = new Callback (BidiUtil.class, "windowProc", 4);
-	static final String CLASS_NAME = "org.eclipse.swt.internal.BidiUtil";
+	static final String CLASS_NAME = "org.eclipse.swt.internal.BidiUtil"; //$NON-NLS-1$
 	static Callback callback;
 	static {
 		try {
-			callback = new Callback (Class.forName (CLASS_NAME), "windowProc", 4);
+			callback = new Callback (Class.forName (CLASS_NAME), "windowProc", 4); //$NON-NLS-1$
+			if (callback.getAddress () == 0) SWT.error (SWT.ERROR_NO_MORE_CALLBACKS);
 		} catch (ClassNotFoundException e) {}
 	}
 
@@ -71,8 +72,8 @@ public class BidiUtil {
 	static final int LANG_ARABIC = 0x01;
 	static final int LANG_HEBREW = 0x0d;
 	// code page identifiers
-	static final String CD_PG_HEBREW = "1255";
-	static final String CD_PG_ARABIC = "1256";
+	static final String CD_PG_HEBREW = "1255"; //$NON-NLS-1$
+	static final String CD_PG_ARABIC = "1256"; //$NON-NLS-1$
 	// ActivateKeyboard constants
 	static final int HKL_NEXT = 1;
 	static final int HKL_PREV = 0;
@@ -134,7 +135,7 @@ static int EnumSystemLanguageGroupsProc(int lpLangGrpId, int lpLangGrpIdString,
 public static void drawGlyphs(GC gc, char[] renderBuffer, int[] renderDx, int x, int y) {
 	int length = renderDx.length;
 	
-	if ((OS.WIN32_MAJOR << 16 | OS.WIN32_MINOR) >= (4 << 16 | 10)) {
+	if (!OS.IsWinCE && OS.WIN32_VERSION >= OS.VERSION(4, 10)) {
 		if (OS.GetLayout (gc.handle) != 0) {
 			reverse(renderDx);
 			renderDx[length-1]--;               //fixes bug 40006
@@ -170,9 +171,9 @@ public static char[] getRenderInfo(GC gc, String text, int[] order, byte[] class
 	int[] lpCs = new int[8];
 	int cs = OS.GetTextCharset(gc.handle);
 	boolean isRightOriented = false;
-	if ((OS.WIN32_MAJOR << 16 | OS.WIN32_MINOR) >= (4 << 16 | 10)) {
+	if (!OS.IsWinCE && OS.WIN32_VERSION >= OS.VERSION(4, 10)) {
 		isRightOriented = OS.GetLayout(gc.handle) != 0;
-	}
+	} 
 	OS.TranslateCharsetInfo(cs, lpCs, OS.TCI_SRCCHARSET);
 	TCHAR textBuffer = new TCHAR(lpCs[1], text, false);
 	int byteCount = textBuffer.length();
@@ -306,9 +307,9 @@ public static void getOrderInfo(GC gc, String text, int[] order, byte[] classBuf
 	TCHAR textBuffer = new TCHAR(lpCs[1], text, false);
 	int byteCount = textBuffer.length();
 	boolean isRightOriented = false;
-	if ((OS.WIN32_MAJOR << 16 | OS.WIN32_MINOR) >= (4 << 16 | 10)) {
+	if (!OS.IsWinCE && OS.WIN32_VERSION >= OS.VERSION(4, 10)) {
 		isRightOriented = OS.GetLayout(gc.handle) != 0;
-	}
+	} 
 
 	GCP_RESULTS result = new GCP_RESULTS();
 	result.lStructSize = GCP_RESULTS.sizeof;
@@ -450,7 +451,7 @@ public static boolean isBidiPlatform() {
 	
 	Callback callback = null;
 	try {
-		callback = new Callback (Class.forName (CLASS_NAME), "EnumSystemLanguageGroupsProc", 5);
+		callback = new Callback (Class.forName (CLASS_NAME), "EnumSystemLanguageGroupsProc", 5); //$NON-NLS-1$
 		int lpEnumSystemLanguageGroupsProc = callback.getAddress ();	
 		if (lpEnumSystemLanguageGroupsProc == 0) SWT.error(SWT.ERROR_NO_MORE_CALLBACKS);
 		OS.EnumSystemLanguageGroups(lpEnumSystemLanguageGroupsProc, OS.LGRPID_INSTALLED, 0);
@@ -543,7 +544,8 @@ public static void setKeyboardLanguage(int language) {
  * 	could not be changed
  */
 public static boolean setOrientation (int hwnd, int orientation) {
-	if ((OS.WIN32_MAJOR << 16 | OS.WIN32_MINOR) < (4 << 16 | 10)) return false;
+	if (OS.IsWinCE) return false;
+	if (OS.WIN32_VERSION < OS.VERSION(4, 10)) return false;
 	int bits = OS.GetWindowLong (hwnd, OS.GWL_EXSTYLE);
 	if ((orientation & SWT.RIGHT_TO_LEFT) != 0) {
 		bits |= OS.WS_EX_LAYOUTRTL; 
