@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -42,14 +42,14 @@ import org.eclipse.swt.events.*;
 
 public class Button extends Control {
 	String text = "";
-	Image image, image2;
+	Image image, image2, disabledImage;
 	ImageList imageList;
 	boolean ignoreMouse;
-	static final int ButtonProc;
-	static final TCHAR ButtonClass = new TCHAR (0,"BUTTON", true);
-	static final char [] SCROLLBAR = new char [] {'S', 'C', 'R', 'O', 'L', 'L', 'B', 'A', 'R', 0};
+	static final int MARGIN = 4;
 	static final int CHECK_WIDTH, CHECK_HEIGHT;
 	static final int ICON_WIDTH = 128, ICON_HEIGHT = 128;
+	static final int ButtonProc;
+	static final TCHAR ButtonClass = new TCHAR (0, "BUTTON", true);
 	static {
 		int hBitmap = OS.LoadBitmap (0, OS.OBM_CHECKBOXES);
 		if (hBitmap == 0) {
@@ -108,26 +108,50 @@ public Button (Composite parent, int style) {
 }
 
 void _setImage (Image image) {
-	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
-		OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, 0);
+	if (OS.COMCTL32_MAJOR >= 6) {
 		if (imageList != null) imageList.dispose ();
 		imageList = null;
 		if (image != null) {
 			imageList = new ImageList (style & SWT.RIGHT_TO_LEFT);
-			imageList.add (image);
+			if (OS.IsWindowEnabled (handle)) {
+				imageList.add (image);
+			} else {
+				if (disabledImage != null) disabledImage.dispose ();
+				disabledImage = new Image (display, image, SWT.IMAGE_DISABLE);
+				imageList.add (disabledImage);
+			}
 			BUTTON_IMAGELIST buttonImageList = new BUTTON_IMAGELIST ();
 			buttonImageList.himl = imageList.getHandle ();
-			if ((style & SWT.LEFT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_LEFT;
-			if ((style & SWT.CENTER) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_CENTER;
-			if ((style & SWT.RIGHT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_RIGHT;
-			TCHAR buffer = new TCHAR (getCodePage (), "", true);
-			OS.SetWindowText (handle, buffer);
+			int oldBits = OS.GetWindowLong (handle, OS.GWL_STYLE), newBits = oldBits;
+			newBits &= ~(OS.BS_LEFT | OS.BS_CENTER | OS.BS_RIGHT);
+			if ((style & SWT.LEFT) != 0) newBits |= OS.BS_LEFT;
+			if ((style & SWT.CENTER) != 0) newBits |= OS.BS_CENTER;
+			if ((style & SWT.RIGHT) != 0) newBits |= OS.BS_RIGHT;
+			if (text.length () == 0) {
+				if ((style & SWT.LEFT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_LEFT;
+				if ((style & SWT.CENTER) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_CENTER;
+				if ((style & SWT.RIGHT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_RIGHT;
+			} else {
+				buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_LEFT;
+				buttonImageList.margin_left = computeLeftMargin ();
+				buttonImageList.margin_right = MARGIN;
+				newBits &= ~(OS.BS_CENTER | OS.BS_RIGHT);
+				newBits |= OS.BS_LEFT;
+			}
+			if (newBits != oldBits) {
+				OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
+				OS.InvalidateRect (handle, null, true);
+			}
 			OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, buttonImageList);
 		} else {
-			TCHAR buffer = new TCHAR (getCodePage (), text, true);
-			OS.SetWindowText (handle, buffer);
 			OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, 0);
 		}
+		/*
+		* Bug in Windows.  Under certain cirumstances yet to be
+		* isolated, BCM_SETIMAGELIST does not redraw the control
+		* when a new image is set.  The fix is to force a redraw.
+		*/
+		OS.InvalidateRect (handle, null, true);
 	} else {
 		if (image2 != null) image2.dispose ();
 		image2 = null;
@@ -173,7 +197,6 @@ void _setImage (Image image) {
 					break;
 				}
 			}
-			
 			/*
 			* Feature in Windows.  The button control mirrors its image when the
 			* flag WS_EX_LAYOUTRTL is set. This behaviour is not desirable in SWT.
@@ -195,7 +218,9 @@ void _setImage (Image image) {
 						OS.SelectObject (srcHdc, oldSrcBitmap);
 						OS.DeleteDC (srcHdc);
 					} else {
-						int newBrush = OS.CreateSolidBrush (getBackgroundPixel ());
+						Control control = findBackgroundControl ();
+						if (control == null) control = this;
+						int newBrush = OS.CreateSolidBrush (control.getBackgroundPixel ());
 						int oldBrush = OS.SelectObject (dstHdc, newBrush);
 						OS.PatBlt (dstHdc, 0, 0, rect.width, rect.height, OS.PATCOPY);
 						OS.DrawIconEx (dstHdc, 0, 0, hImage, 0, 0, 0, 0, OS.DI_NORMAL);
@@ -213,8 +238,7 @@ void _setImage (Image image) {
 				}
 			}
 		}
-		int newBits = OS.GetWindowLong (handle, OS.GWL_STYLE);
-		int oldBits = newBits;
+		int newBits = OS.GetWindowLong (handle, OS.GWL_STYLE), oldBits = newBits;
 		newBits &= ~(OS.BS_BITMAP | OS.BS_ICON);
 		newBits |= imageBits;
 		if (newBits != oldBits) OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
@@ -222,6 +246,50 @@ void _setImage (Image image) {
 	}
 }
 
+void _setText (String text) {
+	int oldBits = OS.GetWindowLong (handle, OS.GWL_STYLE), newBits = oldBits;
+	if (OS.COMCTL32_MAJOR >= 6) {
+		newBits &= ~(OS.BS_LEFT | OS.BS_CENTER | OS.BS_RIGHT);
+		if ((style & SWT.LEFT) != 0) newBits |= OS.BS_LEFT;
+		if ((style & SWT.CENTER) != 0) newBits |= OS.BS_CENTER;
+		if ((style & SWT.RIGHT) != 0) newBits |= OS.BS_RIGHT;
+		if (imageList != null) {
+			BUTTON_IMAGELIST buttonImageList = new BUTTON_IMAGELIST ();
+			buttonImageList.himl = imageList.getHandle ();
+			if (text.length () == 0) {
+				if ((style & SWT.LEFT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_LEFT;
+				if ((style & SWT.CENTER) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_CENTER;
+				if ((style & SWT.RIGHT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_RIGHT;
+			} else {
+				buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_LEFT;
+				buttonImageList.margin_left = computeLeftMargin ();
+				buttonImageList.margin_right = MARGIN;
+				newBits &= ~(OS.BS_CENTER | OS.BS_RIGHT);
+				newBits |= OS.BS_LEFT;
+			}
+			OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, buttonImageList);
+		}
+	} else {
+		newBits &= ~(OS.BS_BITMAP | OS.BS_ICON);
+	}
+	if (newBits != oldBits) {
+		OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
+		OS.InvalidateRect (handle, null, true);
+	}
+	/*
+	* Bug in Windows.  When a Button control is right-to-left and
+	* is disabled, the first pixel of the text is clipped.  The fix
+	* is to append a space to the text.
+	*/
+	if ((style & SWT.RIGHT_TO_LEFT) != 0) {
+		if (OS.COMCTL32_MAJOR < 6 || !OS.IsAppThemed ()) {
+			text = OS.IsWindowEnabled (handle) ? text : text + " ";
+		}
+	}
+	TCHAR buffer = new TCHAR (getCodePage (), text, true);
+	OS.SetWindowText (handle, buffer);
+}
+
 /**
  * Adds the listener to the collection of listeners who will
  * be notified when the control is selected, by sending
@@ -286,10 +354,33 @@ void click () {
 	ignoreMouse = false;
 }
 
+int computeLeftMargin () {
+	if (OS.COMCTL32_MAJOR < 6) return MARGIN;
+	if ((style & (SWT.PUSH | SWT.TOGGLE)) == 0) return MARGIN;
+	int margin = 0;
+	if (image != null && text.length () != 0) {
+		Rectangle bounds = image.getBounds ();
+		margin += bounds.width + MARGIN * 2;
+		int oldFont = 0;
+		int hDC = OS.GetDC (handle);
+		int newFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
+		if (newFont != 0) oldFont = OS.SelectObject (hDC, newFont);
+		TCHAR buffer = new TCHAR (getCodePage (), text, true);
+		RECT rect = new RECT ();
+		int flags = OS.DT_CALCRECT | OS.DT_SINGLELINE;
+		OS.DrawText (hDC, buffer, -1, rect, flags);
+		margin += rect.right - rect.left;
+		if (newFont != 0) OS.SelectObject (hDC, oldFont);
+		OS.ReleaseDC (handle, hDC);
+		OS.GetClientRect (handle, rect);
+		margin = Math.max (MARGIN, (rect.right - rect.left - margin) / 2);
+	}
+	return margin;
+}
+
 public Point computeSize (int wHint, int hHint, boolean changed) {
 	checkWidget ();
-	int border = getBorderWidth ();
-	int width = 0, height = 0;
+	int width = 0, height = 0, border = getBorderWidth ();
 	if ((style & SWT.ARROW) != 0) {
 		if ((style & (SWT.UP | SWT.DOWN)) != 0) {
 			width += OS.GetSystemMetrics (OS.SM_CXVSCROLL);
@@ -304,16 +395,24 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		return new Point (width, height);
 	}
 	int extra = 0;
-	boolean hasImage;
-	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
-		BUTTON_IMAGELIST buttonImageList = new BUTTON_IMAGELIST();
-		OS.SendMessage (handle, OS.BCM_GETIMAGELIST, 0, buttonImageList);
-		hasImage = buttonImageList.himl != 0;
-	} else {
+	boolean hasImage = image != null, hasText = true;
+	if (OS.COMCTL32_MAJOR < 6) {
 		int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
-		hasImage = (bits & (OS.BS_BITMAP | OS.BS_ICON)) != 0;		
+		hasImage = (bits & (OS.BS_BITMAP | OS.BS_ICON)) != 0;
+		if (hasImage) hasText = false;
 	}
-	if (!hasImage) {
+	if (hasImage) {
+		if (image != null) {
+			Rectangle rect = image.getBounds ();
+			width = rect.width;
+			if (hasText && text.length () != 0) {
+				width += MARGIN * 2;
+			}
+			height = rect.height;
+			extra = MARGIN * 2;
+		}
+	}
+	if (hasText) {
 		int oldFont = 0;
 		int hDC = OS.GetDC (handle);
 		int newFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
@@ -322,25 +421,18 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		OS.GetTextMetrics (hDC, lptm);
 		int length = text.length ();
 		if (length == 0) {
-			height += lptm.tmHeight;
+			height = Math.max (height, lptm.tmHeight);
 		} else {
-			extra = Math.max (8, lptm.tmAveCharWidth);
+			extra = Math.max (MARGIN * 2, lptm.tmAveCharWidth);
 			TCHAR buffer = new TCHAR (getCodePage (), text, true);
 			RECT rect = new RECT ();
 			int flags = OS.DT_CALCRECT | OS.DT_SINGLELINE;
 			OS.DrawText (hDC, buffer, -1, rect, flags);
 			width += rect.right - rect.left;
-			height += rect.bottom - rect.top;
+			height = Math.max (height, rect.bottom - rect.top);
 		}
 		if (newFont != 0) OS.SelectObject (hDC, oldFont);
 		OS.ReleaseDC (handle, hDC);
-	} else {
-		if (image != null) {
-			Rectangle rect = image.getBounds ();
-			width = rect.width;
-			height = rect.height;
-			extra = 8;
-		}
 	}
 	if ((style & (SWT.CHECK | SWT.RADIO)) != 0) {
 		width += CHECK_WIDTH + extra;
@@ -355,6 +447,14 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 	return new Point (width, height);
 }
 
+void createHandle () {
+	super.createHandle ();
+	if ((style & SWT.PUSH) == 0) state |= THEME_BACKGROUND;
+	if (OS.COMCTL32_MAJOR >= 6) {
+		if ((style & SWT.RADIO) != 0) state |= DRAW_BACKGROUND;
+	}
+}
+
 int defaultBackground () {
 	if ((style & (SWT.PUSH | SWT.TOGGLE)) != 0) {
 		return OS.GetSysColor (OS.COLOR_BTNFACE);
@@ -378,12 +478,45 @@ void enableWidget (boolean enabled) {
 			int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
 			boolean hasImage = (bits & (OS.BS_BITMAP | OS.BS_ICON)) != 0;
 			if (!hasImage) {
-					String string = enabled ? text : text + " ";
-					TCHAR buffer = new TCHAR (getCodePage (), string, true);
-					OS.SetWindowText (handle, buffer);
+				String string = enabled ? text : text + " ";
+				TCHAR buffer = new TCHAR (getCodePage (), string, true);
+				OS.SetWindowText (handle, buffer);
 			}
 		}
 	}
+	/*
+	* Bug in Windows.  When a button has the style BS_CHECKBOX
+	* or BS_RADIOBUTTON, is checked, and is displaying both an
+	* image and some text, when BCM_SETIMAGELIST is used to
+	* assign an image list for each of the button states, the
+	* button does not draw properly.  When the user drags the
+	* mouse in and out of the button, it draws using a blank
+	* image.  The fix is to set the complete image list only
+	* when the button is disabled.
+	*/
+	if (OS.COMCTL32_MAJOR >= 6) {
+		if (imageList != null) {
+			BUTTON_IMAGELIST buttonImageList = new BUTTON_IMAGELIST ();
+			OS.SendMessage (handle, OS.BCM_GETIMAGELIST, 0, buttonImageList);
+			if (imageList != null) imageList.dispose ();
+			imageList = new ImageList (style & SWT.RIGHT_TO_LEFT);
+			if (OS.IsWindowEnabled (handle)) {
+				imageList.add (image);
+			} else {
+				if (disabledImage != null) disabledImage.dispose ();
+				disabledImage = new Image (display, image, SWT.IMAGE_DISABLE);
+				imageList.add (disabledImage);
+			}
+			buttonImageList.himl = imageList.getHandle ();
+			OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, buttonImageList);
+			/*
+			* Bug in Windows.  Under certain cirumstances yet to be
+			* isolated, BCM_SETIMAGELIST does not redraw the control
+			* when an image is set.  The fix is to force a redraw.
+			*/
+			OS.InvalidateRect (handle, null, true);
+		}
+	}
 }
 
 /**
@@ -512,6 +645,8 @@ void releaseWidget () {
 	super.releaseWidget ();
 	if (imageList != null) imageList.dispose ();
 	imageList = null;
+	if (disabledImage != null) disabledImage.dispose ();
+	disabledImage = null;
 	if (image2 != null) image2.dispose ();
 	image2 = null;
 	text = null;
@@ -596,23 +731,33 @@ public void setAlignment (int alignment) {
 	if ((alignment & (SWT.LEFT | SWT.RIGHT | SWT.CENTER)) == 0) return;
 	style &= ~(SWT.LEFT | SWT.RIGHT | SWT.CENTER);
 	style |= alignment & (SWT.LEFT | SWT.RIGHT | SWT.CENTER);
-	int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
-	bits &= ~(OS.BS_LEFT | OS.BS_CENTER | OS.BS_RIGHT);
-	if ((style & SWT.LEFT) != 0) bits |= OS.BS_LEFT;
-	if ((style & SWT.CENTER) != 0) bits |= OS.BS_CENTER;
-	if ((style & SWT.RIGHT) != 0) bits |= OS.BS_RIGHT;
-	OS.SetWindowLong (handle, OS.GWL_STYLE, bits);
-	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+	int oldBits = OS.GetWindowLong (handle, OS.GWL_STYLE), newBits = oldBits;
+	newBits &= ~(OS.BS_LEFT | OS.BS_CENTER | OS.BS_RIGHT);
+	if ((style & SWT.LEFT) != 0) newBits |= OS.BS_LEFT;
+	if ((style & SWT.CENTER) != 0) newBits |= OS.BS_CENTER;
+	if ((style & SWT.RIGHT) != 0) newBits |= OS.BS_RIGHT;
+	if (OS.COMCTL32_MAJOR >= 6) {
 		if (imageList != null) {
 			BUTTON_IMAGELIST buttonImageList = new BUTTON_IMAGELIST ();
 			buttonImageList.himl = imageList.getHandle ();
-			if ((style & SWT.LEFT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_LEFT;
-			if ((style & SWT.CENTER) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_CENTER;
-			if ((style & SWT.RIGHT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_RIGHT;
+			if (text.length () == 0) {
+				if ((style & SWT.LEFT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_LEFT;
+				if ((style & SWT.CENTER) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_CENTER;
+				if ((style & SWT.RIGHT) != 0) buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_RIGHT;
+			} else {
+				buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_LEFT;
+				buttonImageList.margin_left = computeLeftMargin ();
+				buttonImageList.margin_right = MARGIN;
+				newBits &= ~(OS.BS_CENTER | OS.BS_RIGHT);
+				newBits |= OS.BS_LEFT;
+			}
 			OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, buttonImageList);
 		}
 	}
-	OS.InvalidateRect (handle, null, true);
+	if (newBits != oldBits) {
+		OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
+		OS.InvalidateRect (handle, null, true);
+	}
 }
 
 void setDefault (boolean value) {
@@ -656,7 +801,16 @@ boolean setFixedFocus () {
 public void setImage (Image image) {
 	checkWidget ();
 	if (image != null && image.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
-	_setImage (this.image = image);
+	if ((style & SWT.ARROW) != 0) return;
+	this.image = image;
+	/* This code is intentionally commented */
+//	if (OS.COMCTL32_MAJOR < 6) {
+//		if (image == null || text.length () != 0) {
+//			_setText (text);
+//			return;
+//		}
+//	}
+	_setImage (image);
 }
 
 boolean setRadioFocus () {
@@ -706,7 +860,6 @@ public void setSelection (boolean selected) {
 	checkWidget ();
 	if ((style & (SWT.CHECK | SWT.RADIO | SWT.TOGGLE)) == 0) return;
 	int flags = selected ? OS.BST_CHECKED : OS.BST_UNCHECKED;
-	
 	/*
 	* Feature in Windows. When BM_SETCHECK is used
 	* to set the checked state of a radio or check
@@ -726,14 +879,14 @@ public void setSelection (boolean selected) {
  * the mnemonic character but must not contain line delimiters.
  * </p>
  * <p>
- * Mnemonics are indicated by an '&amp' that causes the next
+ * Mnemonics are indicated by an '&amp;' that causes the next
  * character to be the mnemonic.  When the user presses a
  * key sequence that matches the mnemonic, a selection
  * event occurs. On most platforms, the mnemonic appears
  * underlined but may be emphasised in a platform specific
- * manner.  The mnemonic indicator character '&amp' can be
+ * manner.  The mnemonic indicator character '&amp;' can be
  * escaped by doubling it in the string, causing a single
- *'&amp' to be displayed.
+ * '&amp;' to be displayed.
  * </p>
  * 
  * @param string the new text
@@ -750,31 +903,15 @@ public void setText (String string) {
 	checkWidget ();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
 	if ((style & SWT.ARROW) != 0) return;
-	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
-		OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, 0);
-		if (imageList != null) imageList.dispose ();
-		imageList = null;
-	} else {
-		int newBits = OS.GetWindowLong (handle, OS.GWL_STYLE);
-		int oldBits = newBits;
-		newBits &= ~(OS.BS_BITMAP | OS.BS_ICON);
-		if (newBits != oldBits) {
-			OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
-		}
-	}
 	text = string;
-	/*
-	* Bug in Windows.  When a Button control is right-to-left and
-	* is disabled, the first pixel of the text is clipped.  The fix
-	* is to append a space to the text.
-	*/
-	if ((style & SWT.RIGHT_TO_LEFT) != 0) {
-		if (OS.COMCTL32_MAJOR < 6 || !OS.IsAppThemed ()) {
-			string = OS.IsWindowEnabled (handle) ? text : text + " ";
-		}
-	}
-	TCHAR buffer = new TCHAR (getCodePage (), string, true);
-	OS.SetWindowText (handle, buffer);
+	/* This code is intentionally commented */
+//	if (OS.COMCTL32_MAJOR < 6) {
+//		if (text.length () == 0 && image != null) {
+//			_setImage (image);
+//			return;
+//		}
+//	}
+	_setText (string);
 }
 
 int widgetStyle () {
@@ -799,6 +936,29 @@ int windowProc () {
 	return ButtonProc;
 }
 
+
+LRESULT WM_ERASEBKGND (int wParam, int lParam) {
+	LRESULT result = super.WM_ERASEBKGND (wParam, lParam);
+	if (result != null) return result;
+	/*
+	* Bug in Windows.  For some reason, the HBRUSH that
+	* is returned from WM_CTRLCOLOR is misaligned when
+	* the button uses it to draw.  If the brush is a solid
+	* color, this does not matter.  However, if the brush
+	* contains an image, the image is misaligned.  The
+	* fix is to draw the background in WM_ERASEBKGND.
+	*/
+	if (OS.COMCTL32_MAJOR < 6 || !OS.IsAppThemed ()) {
+		if ((style & (SWT.RADIO | SWT.CHECK)) != 0) {
+			if (findImageControl () != null) {
+				drawBackground (wParam);
+				return LRESULT.ONE;
+			}
+		}
+	}
+	return result;
+}
+
 LRESULT WM_GETDLGCODE (int wParam, int lParam) {
 	LRESULT result = super.WM_GETDLGCODE (wParam, lParam);
 	if (result != null) return result;
@@ -847,6 +1007,24 @@ LRESULT WM_SETFOCUS (int wParam, int lParam) {
 	return result;
 }
 
+LRESULT WM_SIZE (int wParam, int lParam) {
+	LRESULT result = super.WM_SIZE (wParam, lParam);
+	if (result != null) return result;
+	if (OS.COMCTL32_MAJOR >= 6) {
+		if ((style & (SWT.PUSH | SWT.TOGGLE)) != 0) {
+			if (imageList != null && text.length () != 0) {
+				BUTTON_IMAGELIST buttonImageList = new BUTTON_IMAGELIST ();
+				OS.SendMessage (handle, OS.BCM_GETIMAGELIST, 0, buttonImageList);
+				buttonImageList.uAlign = OS.BUTTON_IMAGELIST_ALIGN_LEFT;
+				buttonImageList.margin_left = computeLeftMargin ();
+				buttonImageList.margin_right = MARGIN;
+				OS.SendMessage (handle, OS.BCM_SETIMAGELIST, 0, buttonImageList);
+			}
+		}
+	}
+	return result;
+}
+
 LRESULT WM_SYSCOLORCHANGE (int wParam, int lParam) {
 	LRESULT result = super.WM_SYSCOLORCHANGE (wParam, lParam);
 	if (result != null) return result;
@@ -854,22 +1032,6 @@ LRESULT WM_SYSCOLORCHANGE (int wParam, int lParam) {
 	return result;
 }
 
-LRESULT wmColorChild (int wParam, int lParam) {
-	LRESULT result = super.wmColorChild (wParam, lParam);
-	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
-		Control control = findThemeControl ();
-		if (control != null) {
-			OS.SetBkMode (wParam, OS.TRANSPARENT);
-			RECT rect = new RECT ();
-			OS.GetClientRect (control.handle, rect);
-			OS.MapWindowPoints (control.handle, handle, rect, 2);
-			control.drawThemeBackground (wParam, rect);
-			return new LRESULT (OS.GetStockObject (OS.NULL_BRUSH));
-		}
-	}
-	return result;
-}
-
 LRESULT wmCommandChild (int wParam, int lParam) {
 	int code = wParam >> 16;
 	switch (code) {
@@ -891,6 +1053,27 @@ LRESULT wmCommandChild (int wParam, int lParam) {
 	return super.wmCommandChild (wParam, lParam);
 }
 
+LRESULT wmColorChild (int wParam, int lParam) {
+	/*
+	* Bug in Windows.  For some reason, the HBRUSH that
+	* is returned from WM_CTRLCOLOR is misaligned when
+	* the label uses it to draw.  If the brush is a solid
+	* color, this does not matter.  However, if the brush
+	* contains an image, the image is misaligned.  The
+	* fix is to draw the background in WM_ERASEBKGND.
+	*/
+	LRESULT result = super.wmColorChild (wParam, lParam);
+	if (OS.COMCTL32_MAJOR < 6 || !OS.IsAppThemed ()) {
+		if ((style & (SWT.RADIO | SWT.CHECK)) != 0) {
+			if (findImageControl () != null) {
+				OS.SetBkMode (wParam, OS.TRANSPARENT);
+				return new LRESULT (OS.GetStockObject (OS.NULL_BRUSH));
+			}
+		}
+	}
+	return result;
+}
+
 LRESULT wmDrawChild (int wParam, int lParam) {
 	if ((style & SWT.ARROW) == 0) return super.wmDrawChild (wParam, lParam);
 	DRAWITEMSTRUCT struct = new DRAWITEMSTRUCT ();
