@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -75,17 +75,15 @@ public Canvas (Composite parent, int style) {
 	super (parent, style);
 }
 
-/*
-* Not currently used.
-*/
 void clearArea (int x, int y, int width, int height) {
 	checkWidget ();
-	if (OS.IsWindowVisible (handle)) return;
-	RECT rect = new RECT ();
-	OS.SetRect (rect, x, y, x + width, y + height);
-	int hDC = OS.GetDCEx (handle, 0, OS.DCX_CACHE | OS.DCX_CLIPCHILDREN | OS.DCX_CLIPSIBLINGS);
-	drawBackground (hDC, rect);
-	OS.ReleaseDC (handle, hDC);
+	if (OS.IsWindowVisible (handle)) {
+		RECT rect = new RECT ();
+		OS.SetRect (rect, x, y, x + width, y + height);
+		int hDC = OS.GetDCEx (handle, 0, OS.DCX_CACHE | OS.DCX_CLIPCHILDREN | OS.DCX_CLIPSIBLINGS);
+		drawBackground (hDC, rect);
+		OS.ReleaseDC (handle, hDC);
+	}
 }
 
 /**
@@ -111,10 +109,44 @@ public Caret getCaret () {
 	return caret;
 }
 
-void releaseWidget () {
-	if (caret != null) caret.releaseResources ();
-	caret = null;
-	super.releaseWidget ();
+void releaseChildren (boolean destroy) {
+	if (caret != null) {
+		caret.release (false);
+		caret = null;
+	}
+	super.releaseChildren (destroy);
+}
+
+/** 
+ * Fills the interior of the rectangle specified by the arguments,
+ * with the receiver's background. 
+ *
+ * @param gc the gc where the rectangle is to be filled
+ * @param x the x coordinate of the rectangle to be filled
+ * @param y the y coordinate of the rectangle to be filled
+ * @param width the width of the rectangle to be filled
+ * @param height the height of the rectangle to be filled
+ *
+ * @exception IllegalArgumentException <ul>
+ *    <li>ERROR_NULL_ARGUMENT - if the gc is null</li>
+ *    <li>ERROR_INVALID_ARGUMENT - if the gc has been disposed</li>
+ * </ul>
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.2
+ */
+public void drawBackground (GC gc, int x, int y, int width, int height) {
+	checkWidget ();
+	if (gc == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (gc.isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
+	RECT rect = new RECT ();
+	OS.SetRect (rect, x, y, x + width, y + height);
+	int hDC = gc.handle;
+	int pixel = background == -1 ? OS.GetBkColor (hDC) : -1;
+	drawBackground (hDC, rect, pixel);
 }
 
 /**
@@ -157,23 +189,41 @@ public void scroll (int destX, int destY, int x, int y, int width, int height, b
 		}
 	}
 	int deltaX = destX - x, deltaY = destY - y;
-	int flags = OS.SW_INVALIDATE | OS.SW_ERASE;
-	/*
-	* Feature in Windows.  If any child in the widget tree partially
-	* intersects the scrolling rectangle, Windows moves the child
-	* and copies the bits that intersect the scrolling rectangle but
-	* does not redraw the child.
-	* 
-	* Feature in Windows.  When any child in the widget tree does not
-	* intersect the scrolling rectangle but the parent does intersect,
-	* Windows does not move the child.  This is the documented (but
-	* strange) Windows behavior.
-	* 
-	* The fix is to not use SW_SCROLLCHILDREN and move the children
-	* explicitly after scrolling.  
-	*/
-//	if (all) flags |= OS.SW_SCROLLCHILDREN;
-	OS.ScrollWindowEx (handle, deltaX, deltaY, sourceRect, null, 0, null, flags);
+	if (findImageControl () != null) {
+		if (OS.IsWinCE) {
+			OS.InvalidateRect (handle, sourceRect, true);
+		} else {
+			int flags = OS.RDW_ERASE | OS.RDW_FRAME | OS.RDW_INVALIDATE;
+			if (all) flags |= OS.RDW_ALLCHILDREN;
+			OS.RedrawWindow (handle, sourceRect, 0, flags);
+		}
+		OS.OffsetRect (sourceRect, deltaX, deltaY);
+		if (OS.IsWinCE) {
+			OS.InvalidateRect (handle, sourceRect, true);
+		} else {
+			int flags = OS.RDW_ERASE | OS.RDW_FRAME | OS.RDW_INVALIDATE;
+			if (all) flags |= OS.RDW_ALLCHILDREN;
+			OS.RedrawWindow (handle, sourceRect, 0, flags);
+		}
+	} else {
+		int flags = OS.SW_INVALIDATE | OS.SW_ERASE;
+		/*
+		* Feature in Windows.  If any child in the widget tree partially
+		* intersects the scrolling rectangle, Windows moves the child
+		* and copies the bits that intersect the scrolling rectangle but
+		* does not redraw the child.
+		* 
+		* Feature in Windows.  When any child in the widget tree does not
+		* intersect the scrolling rectangle but the parent does intersect,
+		* Windows does not move the child.  This is the documented (but
+		* strange) Windows behavior.
+		* 
+		* The fix is to not use SW_SCROLLCHILDREN and move the children
+		* explicitly after scrolling.  
+		*/
+//		if (all) flags |= OS.SW_SCROLLCHILDREN;
+		OS.ScrollWindowEx (handle, deltaX, deltaY, sourceRect, null, 0, null, flags);
+	}
 	if (all) {
 		Control [] children = _getChildren ();
 		for (int i=0; i<children.length; i++) {
@@ -213,7 +263,7 @@ public void setCaret (Caret caret) {
 	Caret newCaret = caret;
 	Caret oldCaret = this.caret;
 	this.caret = newCaret;
-	if (isFocusControl ()) {
+	if (hasFocus ()) {
 		if (oldCaret != null) oldCaret.killFocus ();
 		if (newCaret != null) {
 			if (newCaret.isDisposed()) error(SWT.ERROR_INVALID_ARGUMENT);
