@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -54,7 +54,7 @@ import org.eclipse.swt.events.*;
 
 public class Combo extends Composite {
 	boolean noSelection, ignoreModify, ignoreCharacter;
-	int visibleCount = 5;
+	int scrollWidth, visibleCount = 5;
 	
 	/**
 	 * the operating system limit for the number of characters
@@ -79,7 +79,7 @@ public class Combo extends Composite {
 	 */
 	static final int CBID_LIST = 1000;
 	static final int CBID_EDIT = 1001;
-	static int EditProc, ListProc;
+	static /*final*/ int EditProc, ListProc;
 	
 	static final int ComboProc;
 	static final TCHAR ComboClass = new TCHAR (0, "COMBOBOX", true);
@@ -121,6 +121,9 @@ public class Combo extends Composite {
  */
 public Combo (Composite parent, int style) {
 	super (parent, checkStyle (style));
+	/* This code is intentionally commented */
+	//if ((style & SWT.H_SCROLL) != 0) this.style |= SWT.H_SCROLL;
+	this.style |= SWT.H_SCROLL;
 }
 
 /**
@@ -145,6 +148,7 @@ public void add (String string) {
 	int result = OS.SendMessage (handle, OS.CB_ADDSTRING, 0, buffer);
 	if (result == OS.CB_ERR) error (SWT.ERROR_ITEM_NOT_ADDED);
 	if (result == OS.CB_ERRSPACE) error (SWT.ERROR_ITEM_NOT_ADDED);
+	if ((style & SWT.H_SCROLL) != 0) setScrollWidth (buffer, true);
 }
 
 /**
@@ -182,6 +186,7 @@ public void add (String string, int index) {
 	if (result == OS.CB_ERRSPACE || result == OS.CB_ERR) {
 		error (SWT.ERROR_ITEM_NOT_ADDED);
 	}
+	if ((style & SWT.H_SCROLL) != 0) setScrollWidth (buffer, true);
 }
 
 /**
@@ -362,14 +367,18 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		OS.GetWindowText (handle, buffer, length + 1);
 		OS.DrawText (hDC, buffer, length, rect, flags);
 		width = Math.max (width, rect.right - rect.left);
-		for (int i=0; i<count; i++) {
-			length = OS.SendMessage (handle, OS.CB_GETLBTEXTLEN, i, 0);
-			if (length != OS.CB_ERR) {
-				if (length + 1 > buffer.length ()) buffer = new TCHAR (cp, length + 1);
-				int result = OS.SendMessage (handle, OS.CB_GETLBTEXT, i, buffer);
-				if (result != OS.CB_ERR) {
-					OS.DrawText (hDC, buffer, length, rect, flags);
-					width = Math.max (width, rect.right - rect.left);
+		if ((style & SWT.H_SCROLL) != 0) {
+			width = Math.max (width, scrollWidth);
+		} else {
+			for (int i=0; i<count; i++) {
+				length = OS.SendMessage (handle, OS.CB_GETLBTEXTLEN, i, 0);
+				if (length != OS.CB_ERR) {
+					if (length + 1 > buffer.length ()) buffer = new TCHAR (cp, length + 1);
+					int result = OS.SendMessage (handle, OS.CB_GETLBTEXT, i, buffer);
+					if (result != OS.CB_ERR) {
+						OS.DrawText (hDC, buffer, length, rect, flags);
+						width = Math.max (width, rect.right - rect.left);
+					}
 				}
 			}
 		}
@@ -412,6 +421,9 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 			height += textHeight + 10;
 		}
 	}
+	if ((style & SWT.SIMPLE) != 0 && (style & SWT.H_SCROLL) != 0) {
+		height += OS.GetSystemMetrics (OS.SM_CYHSCROLL);
+	}
 	return new Point (width, height);
 }
 
@@ -435,7 +447,7 @@ public void copy () {
 
 void createHandle () {
 	super.createHandle ();
-	state &= ~CANVAS;
+	state &= ~(CANVAS | THEME_BACKGROUND);
 
 	/* Get the text and list window procs */
 	int hwndText = OS.GetDlgItem (handle, CBID_EDIT);
@@ -919,6 +931,22 @@ void register () {
  */
 public void remove (int index) {
 	checkWidget ();
+	TCHAR buffer = null;
+	if ((style & SWT.H_SCROLL) != 0) {
+		int length = OS.SendMessage (handle, OS.CB_GETLBTEXTLEN, index, 0);
+		if (length == OS.CB_ERR) {
+			int count = OS.SendMessage (handle, OS.CB_GETCOUNT, 0, 0);
+			if (0 <= index && index < count) error (SWT.ERROR_ITEM_NOT_REMOVED);
+			error (SWT.ERROR_INVALID_RANGE);
+		}
+		buffer = new TCHAR (getCodePage (), length + 1);
+		int result = OS.SendMessage (handle, OS.CB_GETLBTEXT, index, buffer);
+		if (result == OS.CB_ERR) {
+			int count = OS.SendMessage (handle, OS.CB_GETCOUNT, 0, 0);
+			if (0 <= index && index < count) error (SWT.ERROR_ITEM_NOT_REMOVED);
+			error (SWT.ERROR_INVALID_RANGE);
+		}
+	}
 	int length = OS.GetWindowTextLength (handle);
 	int code = OS.SendMessage (handle, OS.CB_DELETESTRING, index, 0);
 	if (code == OS.CB_ERR) {
@@ -926,12 +954,8 @@ public void remove (int index) {
 		if (0 <= index && index < count) error (SWT.ERROR_ITEM_NOT_REMOVED);
 		error (SWT.ERROR_INVALID_RANGE);
 	}
+	if ((style & SWT.H_SCROLL) != 0) setScrollWidth (buffer, true);
 	if (length != OS.GetWindowTextLength (handle)) {
-		/*
-		* It is possible (but unlikely), that application
-		* code could have disposed the widget in the modify
-		* event.  If this happens, just return.
-		*/
 		sendEvent (SWT.Modify);
 		if (isDisposed ()) return;
 	}
@@ -971,17 +995,39 @@ public void remove (int start, int end) {
 	if (!(0 <= start && start <= end && end < count)) {
 		error (SWT.ERROR_INVALID_RANGE);
 	}
-	int length = OS.GetWindowTextLength (handle);
+	int textLength = OS.GetWindowTextLength (handle);
+	RECT rect = null;
+	int hDC = 0, oldFont = 0, newFont = 0, newWidth = 0;
+	if ((style & SWT.H_SCROLL) != 0) {
+		rect = new RECT ();
+		hDC = OS.GetDC (handle);
+		newFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
+		if (newFont != 0) oldFont = OS.SelectObject (hDC, newFont);
+	}
+	int cp = getCodePage ();
+	int flags = OS.DT_CALCRECT | OS.DT_SINGLELINE | OS.DT_NOPREFIX;
 	for (int i=start; i<=end; i++) {
+		TCHAR buffer = null;
+		if ((style & SWT.H_SCROLL) != 0) {
+			int length = OS.SendMessage (handle, OS.CB_GETLBTEXTLEN, start, 0);
+			if (length == OS.CB_ERR) break;
+			buffer = new TCHAR (cp, length + 1);
+			int result = OS.SendMessage (handle, OS.CB_GETLBTEXT, start, buffer);
+			if (result == OS.CB_ERR) break;
+		}
 		int result = OS.SendMessage (handle, OS.CB_DELETESTRING, start, 0);
 		if (result == OS.CB_ERR) error (SWT.ERROR_ITEM_NOT_REMOVED);
+		if ((style & SWT.H_SCROLL) != 0) {
+			OS.DrawText (hDC, buffer, -1, rect, flags);
+			newWidth = Math.max (newWidth, rect.right - rect.left);
+		}
 	}
-	if (length != OS.GetWindowTextLength (handle)) {
-		/*
-		* It is possible (but unlikely), that application
-		* code could have disposed the widget in the modify
-		* event.  If this happens, just return.
-		*/
+	if ((style & SWT.H_SCROLL) != 0) {
+		if (newFont != 0) OS.SelectObject (hDC, oldFont);
+		OS.ReleaseDC (handle, hDC);
+		setScrollWidth (newWidth, false);
+	}
+	if (textLength != OS.GetWindowTextLength (handle)) {
 		sendEvent (SWT.Modify);
 		if (isDisposed ()) return;
 	}
@@ -1035,7 +1081,8 @@ public void removeAll () {
 	checkWidget ();
 	OS.SendMessage (handle, OS.CB_RESETCONTENT, 0, 0);
 	sendEvent (SWT.Modify);
-	// widget could be disposed at this point
+	if (isDisposed ()) return;
+	if ((style & SWT.H_SCROLL) != 0) setScrollWidth (0);
 }
 
 /**
@@ -1223,8 +1270,15 @@ public void select (int index) {
 	}
 }
 
+void setBackgroundImage (int hBitmap) {
+	super.setBackgroundImage (hBitmap);
+	int hwndText = OS.GetDlgItem (handle, CBID_EDIT);
+	if (hwndText != 0) OS.InvalidateRect (hwndText, null, true);
+	int hwndList = OS.GetDlgItem (handle, CBID_LIST);
+	if (hwndList != 0) OS.InvalidateRect (hwndList, null, true);
+}
+
 void setBackgroundPixel (int pixel) {
-	if (background == pixel) return;
 	super.setBackgroundPixel (pixel);
 	int hwndText = OS.GetDlgItem (handle, CBID_EDIT);
 	if (hwndText != 0) OS.InvalidateRect (hwndText, null, true);
@@ -1279,8 +1333,13 @@ void setBounds (int x, int y, int width, int height, int flags) {
 	}
 }
 
+public void setFont (Font font) {
+	checkWidget ();
+	super.setFont (font);
+	if ((style & SWT.H_SCROLL) != 0) setScrollWidth ();
+}
+
 void setForegroundPixel (int pixel) {
-	if (foreground == pixel) return;
 	super.setForegroundPixel (pixel);
 	int hwndText = OS.GetDlgItem (handle, CBID_EDIT);
 	if (hwndText != 0) OS.InvalidateRect (hwndText, null, true);
@@ -1309,15 +1368,13 @@ void setForegroundPixel (int pixel) {
 public void setItem (int index, String string) {
 	checkWidget ();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
+	int selection = OS.SendMessage (handle, OS.CB_GETCURSEL, 0, 0);
 	remove (index);
-	/*
-	* It is possible (but unlikely), that application
-	* code could have disposed the widget in the modify
-	* event that might be sent when the index is removed.
-	* If this happens, just exit.
-	*/
 	if (isDisposed ()) return;
 	add (string, index);
+	if (selection != -1) {
+		OS.SendMessage (handle, OS.CB_SETCURSEL, selection, 0);
+	}
 }
 
 /**
@@ -1340,6 +1397,15 @@ public void setItems (String [] items) {
 	for (int i=0; i<items.length; i++) {
 		if (items [i] == null) error (SWT.ERROR_INVALID_ARGUMENT);
 	}
+	RECT rect = null;
+	int hDC = 0, oldFont = 0, newFont = 0, newWidth = 0;
+	if ((style & SWT.H_SCROLL) != 0) {
+		rect = new RECT ();
+		hDC = OS.GetDC (handle);
+		newFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
+		if (newFont != 0) oldFont = OS.SelectObject (hDC, newFont);
+		setScrollWidth (0);
+	}
 	OS.SendMessage (handle, OS.CB_RESETCONTENT, 0, 0);
 	int codePage = getCodePage ();
 	for (int i=0; i<items.length; i++) {
@@ -1348,9 +1414,19 @@ public void setItems (String [] items) {
 		int code = OS.SendMessage (handle, OS.CB_ADDSTRING, 0, buffer);
 		if (code == OS.CB_ERR) error (SWT.ERROR_ITEM_NOT_ADDED);
 		if (code == OS.CB_ERRSPACE) error (SWT.ERROR_ITEM_NOT_ADDED);
+		if ((style & SWT.H_SCROLL) != 0) {
+			int flags = OS.DT_CALCRECT | OS.DT_SINGLELINE | OS.DT_NOPREFIX;
+			OS.DrawText (hDC, buffer, -1, rect, flags);
+			newWidth = Math.max (newWidth, rect.right - rect.left);
+		}
+	}
+	if ((style & SWT.H_SCROLL) != 0) {
+		if (newFont != 0) OS.SelectObject (hDC, oldFont);
+		OS.ReleaseDC (handle, hDC);
+		setScrollWidth (newWidth + 3);
 	}
-	// widget could be disposed at this point
 	sendEvent (SWT.Modify);
+	// widget could be disposed at this point
 }
 
 /**
@@ -1433,6 +1509,88 @@ public void setOrientation (int orientation) {
 	}
 }
 
+void setScrollWidth () {
+	int newWidth = 0;
+	RECT rect = new RECT ();
+	int newFont, oldFont = 0;
+	int hDC = OS.GetDC (handle);
+	newFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
+	if (newFont != 0) oldFont = OS.SelectObject (hDC, newFont);
+	int cp = getCodePage ();
+	int count = OS.SendMessage (handle, OS.CB_GETCOUNT, 0, 0);
+	int flags = OS.DT_CALCRECT | OS.DT_SINGLELINE | OS.DT_NOPREFIX;
+	for (int i=0; i<count; i++) {
+		int length = OS.SendMessage (handle, OS.CB_GETLBTEXTLEN, i, 0);
+		if (length != OS.CB_ERR) {
+			TCHAR buffer = new TCHAR (cp, length + 1);
+			int result = OS.SendMessage (handle, OS.CB_GETLBTEXT, i, buffer);
+			if (result != OS.CB_ERR) {
+				OS.DrawText (hDC, buffer, -1, rect, flags);
+				newWidth = Math.max (newWidth, rect.right - rect.left);
+			}
+		}
+	}
+	if (newFont != 0) OS.SelectObject (hDC, oldFont);
+	OS.ReleaseDC (handle, hDC);
+	setScrollWidth (newWidth + 3);
+}
+
+void setScrollWidth (int scrollWidth) {
+	this.scrollWidth = scrollWidth;
+	if ((style & SWT.SIMPLE) != 0) {
+		OS.SendMessage (handle, OS.CB_SETHORIZONTALEXTENT, scrollWidth, 0);
+		return;
+	}
+	boolean scroll = false;
+	int count = OS.SendMessage (handle, OS.CB_GETCOUNT, 0, 0);
+	if (count > 3) {
+		int maxWidth = 0;
+		if (OS.IsWinCE || OS.WIN32_VERSION < OS.VERSION (4, 10)) {
+			RECT rect = new RECT ();
+			OS.SystemParametersInfo (OS.SPI_GETWORKAREA, 0, rect, 0);
+			maxWidth = (rect.right - rect.left) / 4;
+		} else {
+			int hmonitor = OS.MonitorFromWindow (handle, OS.MONITOR_DEFAULTTONEAREST);
+			MONITORINFO lpmi = new MONITORINFO ();
+			lpmi.cbSize = MONITORINFO.sizeof;
+			OS.GetMonitorInfo (hmonitor, lpmi);
+			maxWidth = (lpmi.rcWork_right - lpmi.rcWork_left) / 4;
+		}
+		scroll = scrollWidth > maxWidth;
+	}
+	if (scroll) {
+		OS.SendMessage (handle, OS.CB_SETDROPPEDWIDTH, 0, 0);
+		OS.SendMessage (handle, OS.CB_SETHORIZONTALEXTENT, scrollWidth, 0);
+	} else {
+		scrollWidth += OS.GetSystemMetrics (OS.SM_CYHSCROLL);
+		OS.SendMessage (handle, OS.CB_SETDROPPEDWIDTH, scrollWidth, 0);
+		OS.SendMessage (handle, OS.CB_SETHORIZONTALEXTENT, 0, 0);
+	}
+}
+
+void setScrollWidth (TCHAR buffer, boolean grow) {
+	RECT rect = new RECT ();
+	int newFont, oldFont = 0;
+	int hDC = OS.GetDC (handle);
+	newFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
+	if (newFont != 0) oldFont = OS.SelectObject (hDC, newFont);
+	int flags = OS.DT_CALCRECT | OS.DT_SINGLELINE | OS.DT_NOPREFIX;
+	OS.DrawText (hDC, buffer, -1, rect, flags);
+	if (newFont != 0) OS.SelectObject (hDC, oldFont);
+	OS.ReleaseDC (handle, hDC);
+	setScrollWidth (rect.right - rect.left, grow);
+}
+
+void setScrollWidth (int newWidth, boolean grow) {
+	if (grow) {
+		if (newWidth <= scrollWidth) return;
+		setScrollWidth (newWidth + 3);
+	} else {
+		if (newWidth < scrollWidth) return;
+		setScrollWidth ();
+	}
+}
+
 /**
  * Sets the selection in the receiver's text field to the
  * range specified by the argument whose x coordinate is the
@@ -1523,6 +1681,14 @@ public void setTextLimit (int limit) {
 	OS.SendMessage (handle, OS.CB_LIMITTEXT, limit, 0);
 }
 
+void setToolTipText (Shell shell, String string) {
+	int hwndText = OS.GetDlgItem (handle, CBID_EDIT);
+	int hwndList = OS.GetDlgItem (handle, CBID_LIST);
+	if (hwndText != 0) shell.setToolTipText (hwndText, string);
+	if (hwndList != 0) shell.setToolTipText (hwndList, string);
+	shell.setToolTipText (handle, string);
+}
+
 /**
  * Sets the number of items that are visible in the drop
  * down portion of the receiver's list.
@@ -1654,7 +1820,7 @@ int widgetExtStyle () {
 }
 
 int widgetStyle () {
-	int bits = super.widgetStyle () | OS.CBS_AUTOHSCROLL | OS.CBS_NOINTEGRALHEIGHT | OS.WS_VSCROLL;
+	int bits = super.widgetStyle () | OS.CBS_AUTOHSCROLL | OS.CBS_NOINTEGRALHEIGHT | OS.WS_HSCROLL |OS.WS_VSCROLL;
 	if ((style & SWT.SIMPLE) != 0) return bits | OS.CBS_SIMPLE;
 	if ((style & SWT.READ_ONLY) != 0) return bits | OS.CBS_DROPDOWNLIST;
 	return bits | OS.CBS_DROPDOWN;
@@ -1794,6 +1960,7 @@ LRESULT WM_LBUTTONDOWN (int wParam, int lParam) {
 	*/
 	int oldSelection = OS.SendMessage (handle, OS.CB_GETCURSEL, 0, 0);
 	LRESULT result = super.WM_LBUTTONDOWN (wParam, lParam);
+	if (result == LRESULT.ZERO) return result;
 	if ((style & SWT.READ_ONLY) == 0) {
 		int newSelection = OS.SendMessage (handle, OS.CB_GETCURSEL, 0, 0);
 		if (oldSelection != newSelection) {
@@ -1844,39 +2011,48 @@ LRESULT WM_SIZE (int wParam, int lParam) {
 	* match from the list.  The fix is to remember the original
 	* text and reset it after the widget is resized.
 	*/
-	if ((style & SWT.READ_ONLY) != 0 || (style & SWT.DROP_DOWN) == 0) {
-		return super.WM_SIZE (wParam, lParam);
-	}
-	int index = OS.SendMessage (handle, OS.CB_GETCURSEL, 0, 0);
-	boolean redraw = false;
-	TCHAR buffer = null;
-	int [] start = null, end = null;
-	if (index == OS.CB_ERR) {
-		int length = OS.GetWindowTextLength (handle);
-		if (length != 0) {
-			buffer = new TCHAR (getCodePage (), length + 1);
-			OS.GetWindowText (handle, buffer, length + 1);
-			start = new int [1];  end = new int [1];
-			OS.SendMessage (handle, OS.CB_GETEDITSEL, start, end);
-			redraw = drawCount == 0 && OS.IsWindowVisible (handle);
-			if (redraw) setRedraw (false);
+	LRESULT result = null;
+	if ((style & SWT.READ_ONLY) != 0) {
+		result = super.WM_SIZE (wParam, lParam);
+	} else {
+		int index = OS.SendMessage (handle, OS.CB_GETCURSEL, 0, 0);
+		boolean redraw = false;
+		TCHAR buffer = null;
+		int [] start = null, end = null;
+		if (index == OS.CB_ERR) {
+			int length = OS.GetWindowTextLength (handle);
+			if (length != 0) {
+				buffer = new TCHAR (getCodePage (), length + 1);
+				OS.GetWindowText (handle, buffer, length + 1);
+				start = new int [1];  end = new int [1];
+				OS.SendMessage (handle, OS.CB_GETEDITSEL, start, end);
+				redraw = drawCount == 0 && OS.IsWindowVisible (handle);
+				if (redraw) setRedraw (false);
+			}
+		}
+		result = super.WM_SIZE (wParam, lParam);
+		/*
+		* It is possible (but unlikely), that application
+		* code could have disposed the widget in the resize
+		* event.  If this happens, end the processing of the
+		* Windows message by returning the result of the
+		* WM_SIZE message.
+		*/
+		if (isDisposed ()) return result;
+		if (buffer != null) {
+			OS.SetWindowText (handle, buffer);
+			int bits = start [0] | (end [0] << 16);
+			OS.SendMessage (handle, OS.CB_SETEDITSEL, 0, bits);
+			if (redraw) setRedraw (true);
 		}
 	}
-	LRESULT result = super.WM_SIZE (wParam, lParam);
 	/*
-	* It is possible (but unlikely), that application
-	* code could have disposed the widget in the resize
-	* event.  If this happens, end the processing of the
-	* Windows message by returning the result of the
-	* WM_SIZE message.
+	* Feature in Windows.  When CB_SETDROPPEDWIDTH is called with
+	* a width that is smaller than the current size of the combo
+	* box, it is ignored.  This the fix is to set the width after
+	* the combo box has been resized.
 	*/
-	if (isDisposed ()) return result;
-	if (buffer != null) {
-		OS.SetWindowText (handle, buffer);
-		int bits = start [0] | (end [0] << 16);
-		OS.SendMessage (handle, OS.CB_SETEDITSEL, 0, bits);
-		if (redraw) setRedraw (true);
-	}
+	if ((style & SWT.H_SCROLL) != 0) setScrollWidth (scrollWidth);
 	return result; 
 }
 
@@ -1935,10 +2111,12 @@ LRESULT wmClipboard (int hwndText, int msg, int wParam, int lParam) {
 				OS.SendMessage (hwndText, OS.EM_GETSEL, start, end);
 				OS.CallWindowProc (EditProc, hwndText, msg, wParam, lParam);
 				int length = OS.GetWindowTextLength (hwndText);
-				if (length != 0 && start [0] != end [0]) {
+				int [] newStart = new int [1], newEnd = new int [1];
+				OS.SendMessage (hwndText, OS.EM_GETSEL, newStart, newEnd);
+				if (length != 0 && newStart [0] != newEnd [0]) {
 					TCHAR buffer = new TCHAR (getCodePage (), length + 1);
 					OS.GetWindowText (hwndText, buffer, length + 1);
-					newText = buffer.toString (start [0], end [0] - start [0]);
+					newText = buffer.toString (newStart [0], newEnd [0] - newStart [0]);
 				} else {
 					newText = "";
 				}
@@ -2081,4 +2259,31 @@ LRESULT wmIMEChar (int hwnd, int wParam, int lParam) {
 	return new LRESULT (result);
 }
 
+LRESULT wmSysKeyDown (int hwnd, int wParam, int lParam) {
+	/*
+	* Feature in Windows.  When an editable combo box is dropped
+	* down using Alt+Down and the text in the entry field partially
+	* matches an item in the list, Windows selects the item but doesn't
+	* send WM_COMMAND with CBN_SELCHANGE.  The fix is to detect that
+	* the selection has changed and issue the notification.
+	*/
+	int oldSelection = OS.SendMessage (handle, OS.CB_GETCURSEL, 0, 0);
+	LRESULT result = super.wmSysKeyDown (hwnd, wParam, lParam);
+	if (result != null) return result;
+	if ((style & SWT.READ_ONLY) == 0) {
+		if (wParam == OS.VK_DOWN) {
+			int code = callWindowProc (hwnd, OS.WM_SYSKEYDOWN, wParam, lParam);
+			int newSelection = OS.SendMessage (handle, OS.CB_GETCURSEL, 0, 0);
+			if (oldSelection != newSelection) {
+				sendEvent (SWT.Modify);
+				if (isDisposed ()) return LRESULT.ZERO;
+				sendEvent (SWT.Selection);
+				if (isDisposed ()) return LRESULT.ZERO;
+			}
+			return new LRESULT (code);
+		}
+	}
+	return result;
+}
+
 }
