@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -32,6 +32,12 @@ import org.eclipse.swt.graphics.*;
  * behavior is undefined if they are used with subclasses of <code>Composite</code> other
  * than <code>Canvas</code>.
  * </p><p>
+ * Note: The <code>CENTER</code> style, although undefined for composites, has the
+ * same value as <code>EMBEDDED</code> (which is used to embed widgets from other
+ * widget toolkits into SWT).  On some operating systems (GTK, Motif), this may cause
+ * the children of this composite to be obscured.  The <code>EMBEDDED</code> style
+ * is for use by other widget toolkits and should normally never be used.
+ * </p><p>
  * This class may be subclassed by custom control implementors
  * who are building controls that are constructed from aggregates
  * of other controls.
@@ -45,8 +51,8 @@ public class Composite extends Scrollable {
 	int font;
 	WINDOWPOS [] lpwp;
 	Control [] tabList;
-	int layoutCount = 0;
-	
+	int layoutCount, backgroundMode;
+
 /**
  * Prevents uninitialized instances from being created outside the package.
  */
@@ -228,6 +234,13 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 void createHandle () {
 	super.createHandle ();
 	state |= CANVAS;
+	if ((style & (SWT.H_SCROLL | SWT.V_SCROLL)) == 0) {
+		state |= THEME_BACKGROUND;
+	}
+}
+
+Composite findDeferredControl () {
+	return layoutCount > 0 ? this : parent.findDeferredControl ();
 }
 
 Menu [] findMenus (Control control) {
@@ -276,9 +289,34 @@ void fixTabList (Control control) {
 	tabList = newList;
 }
 
+/**
+ * Returns the receiver's background drawing mode. This
+ * will be one of the following constants defined in class
+ * <code>SWT</code>:
+ * <code>INHERIT_NONE</code>, <code>INHERIT_DEFAULT</code>,
+ * <code>INHERTIT_FORCE</code>.
+ *
+ * @return the background mode
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see SWT
+ * 
+ * @since 3.2
+ */
+public int getBackgroundMode () {
+	checkWidget ();
+	return backgroundMode;
+}
+
 /**
  * Returns a (possibly empty) array containing the receiver's children.
- * Children are returned in the order that they are drawn.
+ * Children are returned in the order that they are drawn.  The topmost
+ * control appears at the beginning of the array.  Subsequent controls
+ * draw beneath this control and appear later in the array.
  * <p>
  * Note: This is not the actual structure used by the receiver
  * to maintain its list of children, so modifying the array will
@@ -407,7 +445,7 @@ public boolean getLayoutDeferred () {
  */
 public boolean isLayoutDeferred () {
 	checkWidget ();
-	return layoutCount > 0 || parent.isLayoutDeferred ();
+	return findDeferredControl () != null;
 }
 
 /**
@@ -581,22 +619,34 @@ Point minimumSize (int wHint, int hHint, boolean changed) {
 	return new Point (width, height);
 }
 
-void releaseChildren () {
+boolean redrawChildren () {
+	if (!super.redrawChildren ()) return false;
+	Control [] children = _getChildren ();
+	for (int i=0; i<children.length; i++) {
+		children [i].redrawChildren ();
+	}
+	return true;
+}
+
+void releaseChildren (boolean destroy) {
 	Control [] children = _getChildren ();
 	for (int i=0; i<children.length; i++) {
 		Control child = children [i];
-		if (!child.isDisposed ()) child.releaseResources ();
+		if (child != null && !child.isDisposed ()) {
+			child.release (false);
+		}
 	}
+	super.releaseChildren (destroy);
 }
 
 void releaseWidget () {
-	releaseChildren ();
 	super.releaseWidget ();
 	if ((state & CANVAS) != 0 && (style & SWT.EMBEDDED) != 0) {
 		int hwndChild = OS.GetWindow (handle, OS.GW_CHILD);
 		if (hwndChild != 0) {
 			int threadId = OS.GetWindowThreadProcessId (hwndChild, null);
 			if (threadId != OS.GetCurrentThreadId ()) {
+				OS.ShowWindow (hwndChild, OS.SW_HIDE);
 				OS.SetParent (hwndChild, 0);
 			}
 		}
@@ -675,6 +725,43 @@ void resizeEmbeddedHandle(int embeddedHandle, int width, int height) {
 	}
 }
 
+void sendResize () {
+	setResizeChildren (false);
+	sendEvent (SWT.Resize);
+	if (isDisposed ()) return;
+	if (layout != null) {
+		markLayout (false, false);
+		updateLayout (false, false);
+	}
+	setResizeChildren (true);
+}
+
+/**
+ * Sets the background drawing mode to the argument which should
+ * be one of the following constants defined in class <code>SWT</code>:
+ * <code>INHERIT_NONE</code>, <code>INHERIT_DEFAULT</code>,
+ * <code>INHERIT_FORCE</code>.
+ *
+ * @param mode the new background mode
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ *
+ * @see SWT
+ * 
+ * @since 3.2
+ */
+public void setBackgroundMode (int mode) {
+	checkWidget ();
+	backgroundMode = mode;
+	Control [] children = _getChildren ();
+	for (int i = 0; i < children.length; i++) {
+		children [i].updateBackgroundMode ();		
+	}
+}
+
 boolean setFixedFocus () {
 	checkWidget ();
 	Control [] children = _getChildren ();
@@ -744,7 +831,9 @@ public void setLayout (Layout layout) {
 public void setLayoutDeferred (boolean defer) {
 	if (!defer) {
 		if (--layoutCount == 0) {
-			if (!isLayoutDeferred ()) updateLayout (true, true);
+			if ((state & LAYOUT_CHILD) != 0 || (state & LAYOUT_NEEDED) != 0) {
+				updateLayout (true, true);
+			}
 		}
 	} else {
 		layoutCount++;
@@ -814,13 +903,21 @@ boolean setTabGroupFocus () {
 
 String toolTipText (NMTTDISPINFO hdr) {
 	if ((hdr.uFlags & OS.TTF_IDISHWND) == 0) {
-		return null;
+		String string = null;
+		Shell shell = getShell ();
+		ToolTip toolTip = shell.findToolTip (hdr.idFrom);
+		if (toolTip != null) {
+			string = toolTip.message;
+			if (string == null || string.length () == 0) string = " ";
+		}
+		return string;
 	}
 	int hwnd = hdr.idFrom;
 	if (hwnd == 0) return null;
 	Control control = display.getControl (hwnd);
-	if (control == null) return null;
-	return control.toolTipText;
+	OS.SendMessage (hdr.hwndFrom, OS.TTM_SETTITLE, 0, 0);
+	OS.SendMessage (hdr.hwndFrom, OS.TTM_SETMAXTIPWIDTH, 0, 0x7FFF);
+	return control != null ? control.toolTipText : null;
 }
 
 boolean translateMnemonic (Event event, Control control) {
@@ -840,7 +937,36 @@ boolean translateTraversal (MSG msg) {
 	return super.translateTraversal (msg);
 }
 
-boolean updateFont (Font oldFont, Font newFont) {
+void updateBackgroundColor () {
+	super.updateBackgroundColor ();
+	Control [] children = _getChildren ();
+	for (int i=0; i<children.length; i++) {
+		if ((children [i].state & PARENT_BACKGROUND) != 0) {
+			children [i].updateBackgroundColor ();
+		}
+	}
+}
+
+void updateBackgroundImage () {
+	super.updateBackgroundImage ();
+	Control [] children = _getChildren ();
+	for (int i=0; i<children.length; i++) {
+		if ((children [i].state & PARENT_BACKGROUND) != 0) {
+			children [i].updateBackgroundImage ();
+		}
+	}
+}
+
+void updateBackgroundMode () {
+	super.updateBackgroundMode ();
+	Control [] children = _getChildren ();
+	for (int i = 0; i < children.length; i++) {
+		children [i].updateBackgroundMode ();		
+	}
+}
+
+void updateFont (Font oldFont, Font newFont) {
+	super.updateFont (oldFont, newFont);
 	Control [] children = _getChildren ();
 	for (int i=0; i<children.length; i++) {
 		Control control = children [i];
@@ -848,20 +974,14 @@ boolean updateFont (Font oldFont, Font newFont) {
 			control.updateFont (oldFont, newFont);
 		}
 	}
-	boolean changed = super.updateFont (oldFont, newFont);
-	if (changed) {
-		/*
-		* Call layout() directly so that subclasses that reimplement
-		* this method instead of using a Layout will set the size and
-		* location of their children when the font changes.
-		*/
-		layout (true);
-	}
-	return changed;
 }
 
 void updateLayout (boolean resize, boolean all) {
-	if (isLayoutDeferred ()) return;
+	Composite parent = findDeferredControl ();
+	if (parent != null) {
+		parent.state |= LAYOUT_CHILD;
+		return;
+	}
 	if ((state & LAYOUT_NEEDED) != 0) {
 		boolean changed = (state & LAYOUT_CHANGED) != 0;
 		state &= ~(LAYOUT_NEEDED | LAYOUT_CHANGED);
@@ -870,6 +990,7 @@ void updateLayout (boolean resize, boolean all) {
 		if (resize) setResizeChildren (true);
 	}
 	if (all) {
+		state &= ~LAYOUT_CHILD;
 		Control [] children = _getChildren ();
 		for (int i=0; i<children.length; i++) {
 			children [i].updateLayout (resize, all);
@@ -918,6 +1039,7 @@ LRESULT WM_GETFONT (int wParam, int lParam) {
 
 LRESULT WM_LBUTTONDOWN (int wParam, int lParam) {
 	LRESULT result = super.WM_LBUTTONDOWN (wParam, lParam);
+	if (result == LRESULT.ZERO) return result;
 
 	/* Set focus for a canvas with no children */
 	if ((state & CANVAS) != 0) {
@@ -928,6 +1050,15 @@ LRESULT WM_LBUTTONDOWN (int wParam, int lParam) {
 	return result;
 }
 
+LRESULT WM_NCPAINT (int wParam, int lParam) {
+	LRESULT result = super.WM_NCPAINT (wParam, lParam);
+	if (result != null) return result;
+	if ((state & CANVAS) != 0) {
+		result = wmNCPaint (handle, wParam, lParam);
+	}
+	return result;
+}
+
 LRESULT WM_NOTIFY (int wParam, int lParam) {
 	if (!OS.IsWinCE) {
 		NMHDR hdr = new NMHDR ();
@@ -1062,31 +1193,35 @@ LRESULT WM_PAINT (int wParam, int lParam) {
 	/* Paint the control and the background */
 	PAINTSTRUCT ps = new PAINTSTRUCT ();
 	if (hooks (SWT.Paint)) {
-		
-		/* Get the damage */
-		int [] lpRgnData = null;
-		boolean isComplex = false;
-		boolean exposeRegion = false;
 
-		int rgn = 0;
-		if ((style & (SWT.NO_MERGE_PAINTS | SWT.DOUBLE_BUFFERED)) != 0) {
-			rgn = OS.CreateRectRgn (0, 0, 0, 0);
-			isComplex = OS.GetUpdateRgn (handle, rgn, false) == OS.COMPLEXREGION;
-		}
-
-		if ((style & SWT.NO_MERGE_PAINTS) != 0) {
-			if (isComplex) {
-				int nBytes = OS.GetRegionData (rgn, 0, null);
-				lpRgnData = new int [nBytes / 4];
-				exposeRegion = OS.GetRegionData (rgn, nBytes, lpRgnData) != 0;
-			}
-		}
-	
 		/* Create the paint GC */
 		GCData data = new GCData ();
 		data.ps = ps;
 		data.hwnd = handle;
 		GC gc = GC.win32_new (this, data);
+
+		/* Get the system region for the paint HDC */
+		int sysRgn = 0;
+		if ((style & (SWT.NO_MERGE_PAINTS | SWT.DOUBLE_BUFFERED)) != 0) {
+			sysRgn = OS.CreateRectRgn (0, 0, 0, 0);
+			if (OS.GetRandomRgn (gc.handle, sysRgn, OS.SYSRGN) == 1) {
+				if (OS.WIN32_VERSION >= OS.VERSION (4, 10)) {
+					if ((OS.GetLayout (gc.handle) & OS.LAYOUT_RTL) != 0) {
+						int nBytes = OS.GetRegionData (sysRgn, 0, null);
+						int [] lpRgnData = new int [nBytes / 4];
+						OS.GetRegionData (sysRgn, nBytes, lpRgnData);
+						int newSysRgn = OS.ExtCreateRegion (new float [] {-1, 0, 0, 1, 0, 0}, nBytes, lpRgnData);
+						OS.DeleteObject (sysRgn);
+						sysRgn = newSysRgn;
+					}
+				}
+				if (OS.IsWinNT) {
+					POINT pt = new POINT();
+					OS.MapWindowPoints (0, handle, pt, 1);
+					OS.OffsetRgn (sysRgn, pt.x, pt.y);
+				}
+			}
+		}
 		
 		/* Send the paint event */
 		int width = ps.right - ps.left;
@@ -1097,31 +1232,36 @@ LRESULT WM_PAINT (int wParam, int lParam) {
 			if ((style & SWT.DOUBLE_BUFFERED) != 0) {
 				image = new Image (display, width, height);
 				paintGC = gc;
-				gc = new GC (image, style & SWT.RIGHT_TO_LEFT);
+				gc = new GC (image, paintGC.getStyle () & SWT.RIGHT_TO_LEFT);
 				gc.setForeground (getForeground ());
 				gc.setBackground (getBackground ());
 				gc.setFont (getFont ());
-				if ((style & SWT.NO_BACKGROUND) != 0) {
-					paintGC.copyArea (image, ps.left, ps.top);
-				} else {
-					gc.fillRectangle (0, 0, width, height);
-				}
-				OS.OffsetRgn (rgn, -ps.left, -ps.top);
-				OS.SelectClipRgn (gc.handle, rgn);
-				OS.SetMetaRgn (gc.handle);
+				OS.OffsetRgn (sysRgn, -ps.left, -ps.top);
+				OS.SelectClipRgn (gc.handle, sysRgn);
+				OS.OffsetRgn (sysRgn, ps.left, ps.top);
+				OS.SetMetaRgn (gc.handle);	
 				OS.SetWindowOrgEx (gc.handle, ps.left, ps.top, null);
+				OS.SetBrushOrgEx (gc.handle, ps.left, ps.top, null);
+				if ((style & SWT.NO_BACKGROUND) != 0) {
+					/* This code is intentionally commented because it may be slow to copy bits from the screen */
+					//paintGC.copyArea (image, ps.left, ps.top);
+				} else {
+					RECT rect = new RECT ();
+					OS.SetRect (rect, ps.left, ps.top, ps.right, ps.bottom);
+					drawBackground (gc.handle, rect);
+				}
 			}
 			Event event = new Event ();
-			event.gc = gc;
-			if (isComplex && exposeRegion) {
-				RECT rect = new RECT ();
+			event.gc = gc;			
+			RECT rect = null;
+			if ((style & SWT.NO_MERGE_PAINTS) != 0 && OS.GetRgnBox (sysRgn, rect = new RECT ()) == OS.COMPLEXREGION) {
+				int nBytes = OS.GetRegionData (sysRgn, 0, null);
+				int [] lpRgnData = new int [nBytes / 4];
+				OS.GetRegionData (sysRgn, nBytes, lpRgnData);
 				int count = lpRgnData [2];
 				for (int i=0; i<count; i++) {
-					OS.SetRect (rect,
-						lpRgnData [8 + (i << 2)],
-						lpRgnData [8 + (i << 2) + 1],
-						lpRgnData [8 + (i << 2) + 2],
-						lpRgnData [8 + (i << 2) + 3]);
+					int offset = 8 + (i << 2);
+					OS.SetRect (rect, lpRgnData [offset], lpRgnData [offset + 1], lpRgnData [offset + 2], lpRgnData [offset + 3]);
 					if ((style & (SWT.DOUBLE_BUFFERED | SWT.NO_BACKGROUND)) == 0) {
 						drawBackground (gc.handle, rect);
 					}
@@ -1130,19 +1270,11 @@ LRESULT WM_PAINT (int wParam, int lParam) {
 					event.width = rect.right - rect.left;
 					event.height = rect.bottom - rect.top;
 					event.count = count - 1 - i;
-					/*
-					* It is possible (but unlikely), that application
-					* code could have disposed the widget in the paint
-					* event.  If this happens, attempt to give back the
-					* paint GC anyways because this is a scarce Windows
-					* resource.
-					*/
 					sendEvent (SWT.Paint, event);
-					if (isDisposed ()) break;
 				}
 			} else {
 				if ((style & (SWT.DOUBLE_BUFFERED | SWT.NO_BACKGROUND)) == 0) {
-					RECT rect = new RECT ();
+					if (rect == null) rect = new RECT ();
 					OS.SetRect (rect, ps.left, ps.top, ps.right, ps.bottom);
 					drawBackground (gc.handle, rect);
 				}
@@ -1157,16 +1289,16 @@ LRESULT WM_PAINT (int wParam, int lParam) {
 			if ((style & SWT.DOUBLE_BUFFERED) != 0) {
 				gc.dispose();
 				if (!isDisposed ()) {
-					paintGC.drawImage(image, ps.left, ps.top);
+					paintGC.drawImage (image, ps.left, ps.top);
 				}
-				image.dispose();
+				image.dispose ();
 				gc = paintGC;
 			}
 		}
 		
 		/* Dispose the paint GC */
 		gc.dispose ();
-		if (rgn != 0) OS.DeleteObject (rgn);
+		if (sysRgn != 0) OS.DeleteObject (sysRgn);
 	} else {
 		int hDC = OS.BeginPaint (handle, ps);
 		if ((style & SWT.NO_BACKGROUND) == 0) {
@@ -1178,7 +1310,7 @@ LRESULT WM_PAINT (int wParam, int lParam) {
 	}
 
 	/* Restore the clipping bits */
-	if (!OS.IsWinCE) { 
+	if (!OS.IsWinCE && !isDisposed ()) { 
 		if (newBits != oldBits) {
 			/*
 			* It is possible (but unlikely), that application
@@ -1208,6 +1340,11 @@ LRESULT WM_PRINTCLIENT (int wParam, int lParam) {
 		if (hooks (SWT.Paint) || filters (SWT.Paint)) {
 			GCData data = new GCData ();
 			data.device = display;
+			data.foreground = getForegroundPixel ();
+			Control control = findBackgroundControl ();
+			if (control == null) control = this;
+			data.background = control.getBackgroundPixel ();
+			data.hFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
 			GC gc = GC.win32_new (wParam, data);
 			Event event = new Event ();
 			event.gc = gc;
@@ -1225,6 +1362,7 @@ LRESULT WM_PRINTCLIENT (int wParam, int lParam) {
 }
 
 LRESULT WM_SETFONT (int wParam, int lParam) {
+	if (lParam != 0) OS.InvalidateRect (handle, null, true);
 	return super.WM_SETFONT (font = wParam, lParam);
 }
 
@@ -1261,14 +1399,7 @@ LRESULT WM_SIZE (int wParam, int lParam) {
 			}
 		}
 		if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
-			if (findThemeControl () != null) {
-				int hwndChild = OS.GetWindow (handle, OS.GW_CHILD);
-				while (hwndChild != 0) {
-					int flags = OS.RDW_ERASE | OS.RDW_INVALIDATE | OS.RDW_ALLCHILDREN;
-					OS.RedrawWindow (hwndChild, null, 0, flags);
-					hwndChild = OS.GetWindow (hwndChild, OS.GW_HWNDNEXT);
-				}
-			}
+			if (findThemeControl () != null) redrawChildren ();
 		}
 	}
 
@@ -1280,12 +1411,14 @@ LRESULT WM_SIZE (int wParam, int lParam) {
 }
 
 LRESULT WM_SYSCOLORCHANGE (int wParam, int lParam) {
+	LRESULT result = super.WM_SYSCOLORCHANGE (wParam, lParam);
+	if (result != null) return result;
 	int hwndChild = OS.GetWindow (handle, OS.GW_CHILD);
 	while (hwndChild != 0) {
 		OS.SendMessage (hwndChild, OS.WM_SYSCOLORCHANGE, 0, 0);
 		hwndChild = OS.GetWindow (hwndChild, OS.GW_HWNDNEXT);
 	}
-	return null;
+	return result;
 }
 
 LRESULT WM_SYSCOMMAND (int wParam, int lParam) {
@@ -1326,4 +1459,27 @@ LRESULT WM_SYSCOMMAND (int wParam, int lParam) {
 	return result;
 }
 
+LRESULT wmNCPaint (int hwnd, int wParam, int lParam) {
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+		int bits = OS.GetWindowLong (hwnd, OS.GWL_EXSTYLE);
+		if ((bits & OS.WS_EX_CLIENTEDGE) != 0) {
+			int code = callWindowProc (hwnd, OS.WM_NCPAINT, wParam, lParam);
+			int hDC = OS.GetWindowDC (hwnd);
+			RECT rect = new RECT ();
+			OS.GetWindowRect (hwnd, rect);
+			rect.right -= rect.left;
+			rect.bottom -= rect.top;
+			rect.left = rect.top = 0;
+			int border = OS.GetSystemMetrics (OS.SM_CXEDGE);
+			OS.ExcludeClipRect (hDC, border, border, rect.right - border, rect.bottom - border);
+			int hTheme = OS.OpenThemeData (hwnd, EDIT);
+			OS.DrawThemeBackground (hTheme, hDC, OS.EP_EDITTEXT, OS.ETS_NORMAL, rect, null);
+			OS.CloseThemeData (hwnd);
+			OS.ReleaseDC (hwnd, hDC);
+			return new LRESULT (code);
+		}
+	}
+	return null;
+}
+
 }
