@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -28,11 +28,13 @@ import org.eclipse.swt.graphics.*;
  * </p><p>
  * <dl>
  * <dt><b>Styles:</b></dt>
- * <dd>FLAT</dd>
+ * <dd>FLAT, HORIZONTAL, VERTICAL</dd>
  * <dt><b>Events:</b></dt>
  * <dd>(none)</dd>
  * </dl>
- * <p>
+ * </p><p>
+ * Note: Only one of the styles HORIZONTAL and VERTICAL may be specified.
+ * </p><p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
  * </p>
  */
@@ -55,6 +57,8 @@ public class CoolBar extends Composite {
 	}
 	static final int SEPARATOR_WIDTH = 2;
 	static final int MAX_WIDTH = 0x7FFF;
+	static final int DEFAULT_COOLBAR_WIDTH = 0;
+	static final int DEFAULT_COOLBAR_HEIGHT = 0;
 
 /**
  * Constructs a new instance of this class given its parent
@@ -86,6 +90,24 @@ public class CoolBar extends Composite {
  */
 public CoolBar (Composite parent, int style) {
 	super (parent, checkStyle (style));
+	/*
+	* Ensure that either of HORIZONTAL or VERTICAL is set.
+	* NOTE: HORIZONTAL and VERTICAL have the same values
+	* as H_SCROLL and V_SCROLL so it is necessary to first
+	* clear these bits to avoid scroll bars and then reset
+	* the bits using the original style supplied by the
+	* programmer.
+	* 
+	* NOTE: The CCS_VERT style cannot be applied when the
+	* widget is created because of this conflict.
+	*/
+	if ((style & SWT.VERTICAL) != 0) {
+		this.style |= SWT.VERTICAL;
+		int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
+		OS.SetWindowLong (handle, OS.GWL_STYLE, bits | OS.CCS_VERT);
+	} else {
+		this.style |= SWT.HORIZONTAL;
+	}
 }
 
 int callWindowProc (int hwnd, int msg, int wParam, int lParam) {
@@ -146,16 +168,15 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		rbBand.cbSize = REBARBANDINFO.sizeof;
 		rbBand.fMask = OS.RBBIM_IDEALSIZE | OS.RBBIM_STYLE;
 		int rowWidth = 0;
-		int separator = (style & SWT.FLAT) == 0 ? SEPARATOR_WIDTH : 0;
 		for (int i = 0; i < count; i++) {
 			OS.SendMessage(handle, OS.RB_GETBANDINFO, i, rbBand);
 			if ((rbBand.fStyle & OS.RBBS_BREAK) != 0) {
-				width = Math.max(width, rowWidth - separator);
+				width = Math.max(width, rowWidth);
 				rowWidth = 0;
 			}
-			rowWidth += rbBand.cxIdeal + getMargin (i) + separator;
+			rowWidth += rbBand.cxIdeal + getMargin (i);
 		}
-		width = Math.max(width, rowWidth - separator);
+		width = Math.max(width, rowWidth);
 		if (redraw) {
 			if (OS.COMCTL32_MAJOR >= 6) {
 				OS.DefWindowProc (handle, OS.WM_SETREDRAW, 1, 0);
@@ -165,8 +186,13 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 		}
 		ignoreResize = false;
 	}
-	if (width == 0) width = DEFAULT_WIDTH;
-	if (height == 0) height = DEFAULT_HEIGHT;
+	if (width == 0) width = DEFAULT_COOLBAR_WIDTH;
+	if (height == 0) height = DEFAULT_COOLBAR_HEIGHT;
+	if ((style & SWT.VERTICAL) != 0) {
+		int tmp = width;
+		width = height;
+		height = tmp;
+	}
 	if (wHint != SWT.DEFAULT) width = wHint;
 	if (hHint != SWT.DEFAULT) height = hHint;
 	height += border * 2;
@@ -176,7 +202,7 @@ public Point computeSize (int wHint, int hHint, boolean changed) {
 
 void createHandle () {
 	super.createHandle ();
-	state &= ~CANVAS;
+	state &= ~(CANVAS | THEME_BACKGROUND);
 	
 	/*
 	* Feature in Windows.  When the control is created,
@@ -334,6 +360,30 @@ void destroyItem (CoolItem item) {
 	originalItems = newOriginals;
 }
 
+void drawThemeBackground (int hDC, int hwnd, RECT rect) {
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+		if (background == -1 && (style & SWT.FLAT) != 0) {
+			Control control = findBackgroundControl ();
+			if (control != null && control.backgroundImage != null) {
+				fillBackground (hDC, control.getBackgroundPixel (), rect);
+				return;
+			}
+		}
+	}
+	RECT rect2 = new RECT ();
+	OS.GetClientRect (handle, rect2);
+	OS.MapWindowPoints (handle, hwnd, rect2, 2);
+	POINT lpPoint = new POINT ();
+	OS.SetWindowOrgEx (hDC, -rect2.left, -rect2.top, lpPoint);
+	OS.SendMessage (handle, OS.WM_PRINT, hDC, OS.PRF_CLIENT | OS.PRF_ERASEBKGND);
+	OS.SetWindowOrgEx (hDC, lpPoint.x, lpPoint.y, null);
+}
+
+Control findThemeControl () {
+	if ((style & SWT.FLAT) != 0) return this;
+	return background == -1 && backgroundImage == null ? this : super.findThemeControl ();
+}
+
 int getMargin (int index) {
 	int margin = 0;
 	if (OS.COMCTL32_MAJOR >= 6) {
@@ -349,17 +399,26 @@ int getMargin (int index) {
 		* the rectangle returned by RBS_BANDBORDERS is four pixels too small.
 		* The fix is to add four pixels to the result.
 		*/	
-		margin += rect.left + 4;
+		if ((style & SWT.VERTICAL) != 0) {
+			margin += rect.top + 4;
+		} else {
+			margin += rect.left + 4;
+		}
 	} else {
-		margin += rect.left + rect.right; 
+		if ((style & SWT.VERTICAL) != 0) {
+			margin += rect.top + rect.bottom;
+		} else {
+			margin += rect.left + rect.right;
+		}
+	}
+	if ((style & SWT.FLAT) == 0) {
+		if (!isLastItemOfRow (index)) {
+			margin += CoolBar.SEPARATOR_WIDTH;
+		}
 	}
 	return margin;
 }
 
-Control findThemeControl () {
-	return null;
-}
-
 /**
  * Returns the item that is currently displayed at the given,
  * zero-relative index. Throws an exception if the index is
@@ -505,7 +564,11 @@ public Point [] getItemSizes () {
 			rect.right += margins.cxRightWidth;
 		}
 		if (!isLastItemOfRow(i)) rect.right += separator;
-		sizes [i] = new Point (rect.right - rect.left, rbBand.cyChild);
+		if ((style & SWT.VERTICAL) != 0) {
+			sizes [i] = new Point (rbBand.cyChild, rect.right - rect.left);
+		} else {
+			sizes [i] = new Point (rect.right - rect.left, rbBand.cyChild);
+		}
 	}
 	return sizes;
 }
@@ -633,15 +696,17 @@ void resizeToMaximumWidth (int index) {
 	OS.SendMessage (handle, OS.RB_SETBANDINFO, index, rbBand);
 }	
 
-void releaseWidget () {
-	for (int i=0; i<items.length; i++) {
-		CoolItem item = items [i];
-		if (item != null && !item.isDisposed ()) {
-			item.releaseResources ();
+void releaseChildren (boolean destroy) {
+	if (items != null) {
+		for (int i=0; i<items.length; i++) {
+			CoolItem item = items [i];
+			if (item != null && !item.isDisposed ()) {
+				item.release (false);
+			}
 		}
+		items = null;
 	}
-	items = null;
-	super.releaseWidget();
+	super.releaseChildren (destroy);
 }
 
 void removeControl (Control control) {
@@ -655,8 +720,6 @@ void removeControl (Control control) {
 }
 
 void setBackgroundPixel (int pixel) {
-	if (background == pixel) return;
-	background = pixel;
 	if (pixel == -1) pixel = defaultBackground ();
 	OS.SendMessage (handle, OS.RB_SETBKCOLOR, 0, pixel);
 	setItemColors (OS.SendMessage (handle, OS.RB_GETTEXTCOLOR, 0, 0), pixel);
@@ -676,8 +739,6 @@ void setBackgroundPixel (int pixel) {
 }
 
 void setForegroundPixel (int pixel) {
-	if (foreground == pixel) return;
-	foreground = pixel;
 	if (pixel == -1) pixel = defaultForeground ();
 	OS.SendMessage (handle, OS.RB_SETTEXTCOLOR, 0, pixel);
 	setItemColors (pixel, OS.SendMessage (handle, OS.RB_GETBKCOLOR, 0, 0));
@@ -943,8 +1004,6 @@ LRESULT WM_COMMAND (int wParam, int lParam) {
 
 LRESULT WM_ERASEBKGND (int wParam, int lParam) {
 	LRESULT result = super.WM_ERASEBKGND (wParam, lParam);
-	if (result != null) return result;
-		
 	/*
 	* Feature in Windows.  For some reason, Windows
 	* does not fully erase the area that the cool bar
@@ -957,12 +1016,12 @@ LRESULT WM_ERASEBKGND (int wParam, int lParam) {
 	* WM_ERASEBKGND.  Therefore it is essential to run
 	* the cool bar window proc after the background has
 	* been erased.
-	* 
-	* On XP, this work around is unnecessary because
-	* the background is drawn using NM_CUSTOMDRAW.
 	*/
-	if (OS.COMCTL32_MAJOR < 6) drawBackground (wParam);
-	return null;
+	if (OS.COMCTL32_MAJOR < 6 || !OS.IsAppThemed ()) {
+		drawBackground (wParam);
+		return null;
+	}
+	return result;
 }
 
 LRESULT WM_NOTIFY (int wParam, int lParam) {
@@ -1027,6 +1086,12 @@ LRESULT WM_SIZE (int wParam, int lParam) {
 		if (code == 0) return LRESULT.ZERO;
 		return new LRESULT (code);
 	}
+	//TEMPORARY CODE
+//	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+//		if (background == -1 && (style & SWT.FLAT) == 0) {
+//			OS.InvalidateRect (handle, null, true);
+//		}
+//	}
 	return super.WM_SIZE (wParam, lParam);
 }
 
@@ -1034,7 +1099,16 @@ LRESULT wmNotifyChild (int wParam, int lParam) {
 	NMHDR hdr = new NMHDR ();
 	OS.MoveMemory (hdr, lParam, NMHDR.sizeof);
 	switch (hdr.code) {
-		case OS.RBN_CHILDSIZE:
+		case OS.RBN_BEGINDRAG: {
+			int pos = OS.GetMessagePos ();
+			POINT pt = new POINT ();
+			pt.x = (short) (pos & 0xFFFF);
+			pt.y = (short) (pos >> 16); 
+			OS.ScreenToClient (handle, pt);
+			if (!sendDragEvent (pt.x, pt.y)) return LRESULT.ONE;
+			break;
+		}
+		case OS.RBN_CHILDSIZE: {
 			/*
 			* Bug in Windows.  When Windows sets the size of the rebar band
 			* child and the child is a combo box, the size of the drop down
@@ -1056,27 +1130,39 @@ LRESULT wmNotifyChild (int wParam, int lParam) {
 				}
 			}
 			break;
-		case OS.RBN_HEIGHTCHANGE:
+		}
+		case OS.RBN_HEIGHTCHANGE: {
 			if (!ignoreResize) {
 				Point size = getSize ();
 				int border = getBorderWidth ();
-				int height = OS.SendMessage (handle, OS.RB_GETBARHEIGHT, 0, 0);
-				setSize (size.x, height + (border * 2));
+				int barHeight = OS.SendMessage (handle, OS.RB_GETBARHEIGHT, 0, 0);
+				if ((style & SWT.VERTICAL) != 0) {
+					setSize (barHeight + 2 * border, size.y);
+				} else {
+					setSize (size.x, barHeight + 2 * border);
+				}
 			}
 			break;
-		case OS.RBN_CHEVRONPUSHED:
+		}
+		case OS.RBN_CHEVRONPUSHED: {
 			NMREBARCHEVRON lpnm = new NMREBARCHEVRON ();
 			OS.MoveMemory (lpnm, lParam, NMREBARCHEVRON.sizeof);
 			CoolItem item = items [lpnm.wID];
 			if (item != null) {
 				Event event = new Event();
 				event.detail = SWT.ARROW;
-				event.x = lpnm.left;
-				event.y = lpnm.bottom;
+				if ((style & SWT.VERTICAL) != 0) {
+					event.x = lpnm.right;
+					event.y = lpnm.top;
+				} else {
+					event.x = lpnm.left;
+					event.y = lpnm.bottom;
+				}
 				item.postEvent (SWT.Selection, event);
 			}
 			break;
-		case OS.NM_CUSTOMDRAW:
+		}
+		case OS.NM_CUSTOMDRAW: {
 			/*
 			* Bug in Windows.  On versions of Windows prior to XP,
 			* drawing the background color in NM_CUSTOMDRAW erases
@@ -1084,18 +1170,19 @@ LRESULT wmNotifyChild (int wParam, int lParam) {
 			* in WM_ERASEBKGND.
 			*/
 			if (OS.COMCTL32_MAJOR < 6) break;
-			if (background != -1 || (style & SWT.FLAT) != 0) {
+			if (findBackgroundControl () != null || (style & SWT.FLAT) != 0) {
 				NMCUSTOMDRAW nmcd = new NMCUSTOMDRAW ();
 				OS.MoveMemory (nmcd, lParam, NMCUSTOMDRAW.sizeof);
 				switch (nmcd.dwDrawStage) {
 					case OS.CDDS_PREERASE:
-						return new LRESULT (OS.CDRF_NOTIFYPOSTERASE);
+						return new LRESULT (OS.CDRF_SKIPDEFAULT | OS.CDRF_NOTIFYPOSTERASE);
 					case OS.CDDS_POSTERASE:
 						drawBackground (nmcd.hdc);
 						break;
 				}
 			}
 			break;
+		}
 	}
 	return super.wmNotifyChild (wParam, lParam);
 }
