@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -36,6 +36,16 @@ import org.eclipse.swt.events.*;
 public class MenuItem extends Item {
 	Menu parent, menu;
 	int id, accelerator;
+	/*
+	* Feature in Windows.  On Windows 98, it is necessary
+	* to add 4 pixels to the width of the image or the image
+	* and text are too close.  On other Windows platforms,
+	* this causes the text of the longest item to touch the
+	* accelerator text.  The fix is to use smaller margins
+	* everywhere but on Windows 98.
+	*/
+	final static int MARGIN_WIDTH = OS.IsWin95 ? 2 : 1;
+	final static int MARGIN_HEIGHT = OS.IsWin95 ? 2 : 1;
 
 /**
  * Constructs a new instance of this class given its parent
@@ -94,10 +104,11 @@ public MenuItem (Menu parent, int style) {
  *
  * @param parent a menu control which will be the parent of the new instance (cannot be null)
  * @param style the style of control to construct
- * @param index the index to store the receiver in its parent
+ * @param index the zero-relative index to store the receiver in its parent
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the parent (inclusive)</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -123,8 +134,7 @@ MenuItem (Menu parent, Menu menu, int style, int index) {
 	this.parent = parent;
 	this.menu = menu;	
 	if (menu != null) menu.cascade = this;
-	Decorations shell = parent.parent;
-	shell.add (this);
+	display.addMenuItem (this);
 }
 
 /**
@@ -181,7 +191,7 @@ public void addHelpListener (HelpListener listener) {
 
 /**
  * Adds the listener to the collection of listeners who will
- * be notified when the control is selected, by sending
+ * be notified when the menu item is selected, by sending
  * it one of the messages defined in the <code>SelectionListener</code>
  * interface.
  * <p>
@@ -219,10 +229,15 @@ static int checkStyle (int style) {
 	return checkBits (style, SWT.PUSH, SWT.CHECK, SWT.RADIO, SWT.SEPARATOR, SWT.CASCADE, 0);
 }
 
+void destroyWidget () {
+	parent.destroyItem (this);
+	releaseHandle ();
+}
+
 void fillAccel (ACCEL accel) {
 	accel.fVirt = 0;
 	accel.cmd = accel.key = 0;
-	if (accelerator == 0) return;
+	if (accelerator == 0 || !getEnabled ()) return;
 	int fVirt = OS.FVIRTKEY;
 	int key = accelerator & SWT.KEY_MASK;
 	int vKey = Display.untranslateKey (key);
@@ -239,7 +254,7 @@ void fillAccel (ACCEL accel) {
 			case 27: key = OS.VK_ESCAPE; break;
 			case 127: key = OS.VK_DELETE; break;
 			default: {
-				key = wcsToMbcs ((char) key);
+				key = Display.wcsToMbcs ((char) key);
 				if (key == 0) return;
 				if (OS.IsWinCE) {
 					key = OS.CharUpper ((short) key);
@@ -262,12 +277,18 @@ void fillAccel (ACCEL accel) {
 	if ((accelerator & SWT.CONTROL) != 0) accel.fVirt |= OS.FCONTROL;
 }
 
+void fixMenus (Decorations newParent) {
+	if (menu != null) menu.fixMenus (newParent);
+}
+
 /**
- * Return the widget accelerator.  An accelerator is the bit-wise
+ * Returns the widget accelerator.  An accelerator is the bit-wise
  * OR of zero or more modifier masks and a key. Examples:
  * <code>SWT.CONTROL | SWT.SHIFT | 'T', SWT.ALT | SWT.F2</code>.
+ * The default value is zero, indicating that the menu item does
+ * not have an accelerator.
  *
- * @return the accelerator
+ * @return the accelerator or 0
  *
  * </ul>
  * @exception SWTException <ul>
@@ -280,15 +301,66 @@ public int getAccelerator () {
 	return accelerator;
 }
 
-public Display getDisplay () {
-	Menu parent = this.parent;
-	if (parent == null) error (SWT.ERROR_WIDGET_DISPOSED);
-	return parent.getDisplay ();
+/**
+ * Returns a rectangle describing the receiver's size and location
+ * relative to its parent (or its display if its parent is null).
+ *
+ * @return the receiver's bounding rectangle
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.1
+ */
+/*public*/ Rectangle getBounds () {
+	checkWidget ();
+	if (OS.IsWinCE) return new Rectangle (0, 0, 0, 0);
+	int index = parent.indexOf (this);
+	if (index == -1) return new Rectangle (0, 0, 0, 0);
+	if ((parent.style & SWT.BAR) != 0) {
+		Decorations shell = parent.parent;
+		if (shell.menuBar != parent) {
+			return new Rectangle (0, 0, 0, 0);
+		}
+		int hwndShell = shell.handle;
+		MENUBARINFO info1 = new MENUBARINFO ();
+		info1.cbSize = MENUBARINFO.sizeof;
+		if (!OS.GetMenuBarInfo (hwndShell, OS.OBJID_MENU, 1, info1)) {
+			return new Rectangle (0, 0, 0, 0);
+		}
+		MENUBARINFO info2 = new MENUBARINFO ();
+		info2.cbSize = MENUBARINFO.sizeof;
+		if (!OS.GetMenuBarInfo (hwndShell, OS.OBJID_MENU, index + 1, info2)) {
+			return new Rectangle (0, 0, 0, 0);
+		}
+		int x = info2.left - info1.left;
+		int y = info2.top - info1.top;
+		int width = info2.right - info2.left;
+		int height = info2.bottom - info2.top;
+		return new Rectangle (x, y, width, height);
+	} else {
+		int hMenu = parent.handle;
+		RECT rect1 = new RECT ();
+		if (!OS.GetMenuItemRect (0, hMenu, 0, rect1)) {
+			return new Rectangle (0, 0, 0, 0);
+		}
+		RECT rect2 = new RECT ();
+		if (!OS.GetMenuItemRect (0, hMenu, index, rect2)) {
+			return new Rectangle (0, 0, 0, 0);
+		}
+		int x = rect2.left - rect1.left + 2;
+		int y = rect2.top - rect1.top + 2;
+		int width = rect2.right - rect2.left;
+		int height = rect2.bottom - rect2.top;
+		return new Rectangle (x, y, width, height);
+	}
 }
 
 /**
  * Returns <code>true</code> if the receiver is enabled, and
- * <code>false</code> otherwise. A disabled control is typically
+ * <code>false</code> otherwise. A disabled menu item is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -396,7 +468,7 @@ public boolean getSelection () {
 /**
  * Returns <code>true</code> if the receiver is enabled and all
  * of the receiver's ancestors are enabled, and <code>false</code>
- * otherwise. A disabled control is typically not selectable from the
+ * otherwise. A disabled menu item is typically not selectable from the
  * user interface and draws with an inactive or "grayed" look.
  *
  * @return the receiver's enabled state
@@ -412,11 +484,18 @@ public boolean isEnabled () {
 	return getEnabled () && parent.isEnabled ();
 }
 
-void releaseChild () {
-	super.releaseChild ();
-	if (menu != null) menu.dispose ();
-	menu = null;
-	parent.destroyItem (this);
+void releaseChildren (boolean destroy) {
+	if (menu != null) {
+		menu.release (false);
+		menu = null;
+	}
+	super.releaseChildren (destroy);
+}
+
+void releaseHandle () {
+	super.releaseHandle ();
+	parent = null;
+	id = -1;
 }
 
 void releaseMenu () {
@@ -424,24 +503,26 @@ void releaseMenu () {
 	menu = null;
 }
 
-void releaseWidget () {
-	if (menu != null) menu.releaseResources ();
+void releaseParent () {
+	super.releaseParent ();
+	if (menu != null) menu.dispose ();
 	menu = null;
+}
+
+void releaseWidget () {
 	super.releaseWidget ();
 	if (accelerator != 0) {
 		parent.destroyAccelerators ();
 	}
 	accelerator = 0;
-	Decorations shell = parent.parent;
-	shell.remove (this);
-	parent = null;
+	display.removeMenuItem (this);
 }
 
 /**
  * Removes the listener from the collection of listeners who will
  * be notified when the arm events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -464,7 +545,7 @@ public void removeArmListener (ArmListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the help events are generated for the control.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -487,7 +568,7 @@ public void removeHelpListener (HelpListener listener) {
  * Removes the listener from the collection of listeners who will
  * be notified when the control is selected.
  *
- * @param listener the listener which should be notified
+ * @param listener the listener which should no longer be notified
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
@@ -524,6 +605,8 @@ void selectRadio () {
  * OR of zero or more modifier masks and a key. Examples:
  * <code>SWT.MOD1 | SWT.MOD2 | 'T', SWT.MOD3 | SWT.F2</code>.
  * <code>SWT.CONTROL | SWT.SHIFT | 'T', SWT.ALT | SWT.F2</code>.
+ * The default value is zero, indicating that the menu item does
+ * not have an accelerator.
  *
  * @param accelerator an integer that is the bit-wise OR of masks and a key
  *
@@ -542,7 +625,7 @@ public void setAccelerator (int accelerator) {
 
 /**
  * Enables the receiver if the argument is <code>true</code>,
- * and disables it otherwise. A disabled control is typically
+ * and disables it otherwise. A disabled menu item is typically
  * not selectable from the user interface and draws with an
  * inactive or "grayed" look.
  *
@@ -586,19 +669,34 @@ public void setEnabled (boolean enabled) {
 				info.fState |= bits;
 			}
 			success = OS.SetMenuItemInfo (hMenu, id, false, info);
-			if (!success) error (SWT.ERROR_CANNOT_SET_ENABLED);
+			if (!success) {
+				/*
+				* Bug in Windows.  For some reason SetMenuItemInfo(),
+				* returns a fail code when setting the enabled or
+				* selected state of a default item, but sets the
+				* state anyway.  The fix is to ignore the error.
+				* 
+				* NOTE:  This only happens on Vista.
+				*/
+				if (!OS.IsWinCE && OS.WIN32_VERSION >= OS.VERSION (6, 0)) {
+					success = id == OS.GetMenuDefaultItem (hMenu, OS.MF_BYCOMMAND, OS.GMDI_USEDISABLED);
+				}
+				if (!success) error (SWT.ERROR_CANNOT_SET_ENABLED);
+			}
 		}
 	}
+	parent.destroyAccelerators ();
 	parent.redraw ();
 }
 
 /**
  * Sets the image the receiver will display to the argument.
  * <p>
- * Note: This feature is not available on all window systems (for example, Window NT),
- * in which case, calling this method will silently do nothing.
+ * Note: This operation is a hint and is not supported on
+ * platforms that do not have this concept (for example, Windows NT).
+ * </p>
  *
- * @param menu the image to display
+ * @param image the image to display
  *
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -609,15 +707,25 @@ public void setImage (Image image) {
 	checkWidget ();
 	if ((style & SWT.SEPARATOR) != 0) return;
 	super.setImage (image);
-	if (OS.IsWinCE) return;
-	if ((OS.WIN32_MAJOR << 16 | OS.WIN32_MINOR) < (4 << 16 | 10)) {
+	if (OS.IsWinCE) {
+		if ((OS.IsPPC || OS.IsSP) && parent.hwndCB != 0) {
+			int hwndCB = parent.hwndCB;
+			TBBUTTONINFO info = new TBBUTTONINFO ();
+			info.cbSize = TBBUTTONINFO.sizeof;
+			info.dwMask = OS.TBIF_IMAGE;
+			info.iImage = parent.imageIndex (image);
+			OS.SendMessage (hwndCB, OS.TB_SETBUTTONINFO, id, info);
+		}
+		return;
+	}
+	if (OS.WIN32_VERSION < OS.VERSION (4, 10)) {
 		return;
 	}
-	int hMenu = parent.handle;
 	MENUITEMINFO info = new MENUITEMINFO ();
 	info.cbSize = MENUITEMINFO.sizeof;
 	info.fMask = OS.MIIM_BITMAP;
 	if (image != null) info.hbmpItem = OS.HBMMENU_CALLBACK;
+	int hMenu = parent.handle;
 	OS.SetMenuItemInfo (hMenu, id, false, info);
 	parent.redraw ();
 }
@@ -628,6 +736,11 @@ public void setImage (Image image) {
  * pull down menu. The sequence of key strokes, button presses
  * and/or button releases that are used to request a pull down
  * menu is platform specific.
+ * <p>
+ * Note: Disposing of a menu item that has a pull down menu
+ * will dispose of the menu.  To avoid this behavior, set the
+ * menu to null before the menu item is disposed.
+ * </p>
  *
  * @param menu the new pull down menu
  *
@@ -692,6 +805,26 @@ public void setMenu (Menu menu) {
 			index++;
 		}
 		if (info.dwItemData != id) return;
+		boolean hasBitmap = false, success = false;
+		
+		/*
+		* Bug in Windows.  When GetMenuItemInfo() is used to get the text,
+		* for an item that has a bitmap set using MIIM_BITMAP, the text is
+		* not returned.  This means that when SetMenuItemInfo() is used to
+		* set the submenu and the current menu state, the text is lost.
+		* The fix is to temporarily remove the bitmap and restore it after
+		* the text and submenu have been set.
+		*/
+		if (!OS.IsWinCE && OS.WIN32_VERSION >= OS.VERSION (4, 10)) {
+			info.fMask = OS.MIIM_BITMAP;
+			OS.GetMenuItemInfo (hMenu, index, true, info);
+			hasBitmap = info.hbmpItem != 0;
+			if (hasBitmap) {
+				info.hbmpItem = 0;
+				success = OS.SetMenuItemInfo (hMenu, id, false, info);
+			}
+		}
+		
 		int cch = 128;
 		int hHeap = OS.GetProcessHeap ();
 		int byteCount = cch * TCHAR.sizeof;
@@ -699,7 +832,7 @@ public void setMenu (Menu menu) {
 		info.fMask = OS.MIIM_STATE | OS.MIIM_ID | OS.MIIM_TYPE | OS.MIIM_DATA;
 		info.dwTypeData = pszText;
 		info.cch = cch;
-		boolean success = OS.GetMenuItemInfo (hMenu, index, true, info);
+		success = OS.GetMenuItemInfo (hMenu, index, true, info);
 		if (menu != null) {
 			menu.cascade = this; 
 			info.fMask |= OS.MIIM_SUBMENU;
@@ -708,11 +841,11 @@ public void setMenu (Menu menu) {
 		OS.RemoveMenu (hMenu, index, OS.MF_BYPOSITION);
 		if (OS.IsWinCE) {
 			/*
-			* On WinCE, InsertMenuItem is not available.  SetMenuItemInfo
-			* does not set the menu item state and submenu use InsertMenu
-			* to set these fields and SetMenuItemInfo to set the menu item
-			* data.  NOTE: SetMenuItemInfo is also used to set the string
-			* that was queried from the original menu item.
+			* On WinCE, InsertMenuItem() is not available.  The fix is to
+			* use SetMenuItemInfo() but this call does not set the menu item
+			* state and submenu.  The fix is to use InsertMenu() to insert
+			* the item, SetMenuItemInfo() to set the string and EnableMenuItem()
+			* and CheckMenuItem() to set the state.
 			*/
 			int uIDNewItem = id;
 			int uFlags = OS.MF_BYPOSITION;
@@ -720,7 +853,7 @@ public void setMenu (Menu menu) {
 				uFlags |= OS.MF_POPUP;
 				uIDNewItem = menu.handle;
 			}
-			TCHAR lpNewItem = new TCHAR (0, "", true);
+			TCHAR lpNewItem = new TCHAR (0, " ", true);
 			success = OS.InsertMenu (hMenu, index, uFlags, uIDNewItem, lpNewItem);
 			if (success) {
 				info.fMask = OS.MIIM_DATA | OS.MIIM_TYPE;
@@ -734,6 +867,19 @@ public void setMenu (Menu menu) {
 			}
 		} else {
 			success = OS.InsertMenuItem (hMenu, index, true, info);
+			/*
+			* Restore the bitmap that was removed to work around a problem
+			* in GetMenuItemInfo() and menu items that have bitmaps set with
+			* MIIM_BITMAP.
+			*/
+			if (OS.WIN32_VERSION >= OS.VERSION (4, 10)) {
+				if (hasBitmap) {
+					info.fMask = OS.MIIM_BITMAP;
+					info.hbmpItem = OS.HBMMENU_CALLBACK;
+					success = OS.SetMenuItemInfo (hMenu, id, false, info);
+				}
+			}
+			
 		}
 		if (pszText != 0) OS.HeapFree (hHeap, 0, pszText);
 		if (!success) error (SWT.ERROR_CANNOT_SET_MENU);
@@ -782,7 +928,20 @@ public void setSelection (boolean selected) {
 		info.fState &= ~OS.MFS_CHECKED;
 		if (selected) info.fState |= OS.MFS_CHECKED;
 		success = OS.SetMenuItemInfo (hMenu, id, false, info);
-		if (!success) error (SWT.ERROR_CANNOT_SET_SELECTION);
+		if (!success) {
+			/*
+			* Bug in Windows.  For some reason SetMenuItemInfo(),
+			* returns a fail code when setting the enabled or
+			* selected state of a default item, but sets the
+			* state anyway.  The fix is to ignore the error.
+			* 
+			* NOTE:  This only happens on Vista.
+			*/
+			if (!OS.IsWinCE && OS.WIN32_VERSION >= OS.VERSION (6, 0)) {
+				success = id == OS.GetMenuDefaultItem (hMenu, OS.MF_BYCOMMAND, OS.GMDI_USEDISABLED);
+			}
+			if (!success) error (SWT.ERROR_CANNOT_SET_SELECTION);
+		}
 	}
 	parent.redraw ();
 }
@@ -790,14 +949,14 @@ public void setSelection (boolean selected) {
  * Sets the receiver's text. The string may include
  * the mnemonic character and accelerator text.
  * <p>
- * Mnemonics are indicated by an '&amp' that causes the next
+ * Mnemonics are indicated by an '&amp;' that causes the next
  * character to be the mnemonic.  When the user presses a
  * key sequence that matches the mnemonic, a selection
  * event occurs. On most platforms, the mnemonic appears
  * underlined but may be emphasised in a platform specific
- * manner.  The mnemonic indicator character '&amp' can be
+ * manner.  The mnemonic indicator character '&amp;' can be
  * escaped by doubling it in the string, causing a single
- *'&amp' to be displayed.
+ * '&amp;' to be displayed.
  * </p>
  * <p>
  * Accelerator text is indicated by the '\t' character.
@@ -830,7 +989,7 @@ public void setText (String string) {
 	if (text.equals (string)) return;
 	super.setText (string);
 	int hHeap = OS.GetProcessHeap ();
-	int pszText;
+	int pszText = 0;
 	boolean success = false;
 	if ((OS.IsPPC || OS.IsSP) && parent.hwndCB != 0) {
 		/*
@@ -861,31 +1020,43 @@ public void setText (String string) {
 		info2.pszText = pszText;
 		success = OS.SendMessage (hwndCB, OS.TB_SETBUTTONINFO, id, info2) != 0;
 	} else {
-		/* Use the character encoding for the default locale */
-		TCHAR buffer = new TCHAR (0, string, true);
-		int byteCount = buffer.length () * TCHAR.sizeof;
-		pszText = OS.HeapAlloc (hHeap, OS.HEAP_ZERO_MEMORY, byteCount);
-		OS.MoveMemory (pszText, buffer, byteCount);	
-		int hMenu = parent.handle;
 		MENUITEMINFO info = new MENUITEMINFO ();
 		info.cbSize = MENUITEMINFO.sizeof;
-		info.fMask = OS.MIIM_TYPE;
-		info.fType = widgetStyle ();
-		info.dwTypeData = pszText;
-		success = OS.SetMenuItemInfo (hMenu, id, false, info);
+		int hMenu = parent.handle;
+		
 		/*
 		* Bug in Windows 2000.  For some reason, when MIIM_TYPE is set
 		* on a menu item that also has MIIM_BITMAP, the MIIM_TYPE clears
 		* the MIIM_BITMAP style.  The fix is to reset both MIIM_BITMAP.
 		* Note, this does not happen on Windows 98.
 		*/
-		if (!OS.IsWinCE) {
-			if ((OS.WIN32_MAJOR << 16 | OS.WIN32_MINOR) >= (4 << 16 | 10)) {
-				if (image != null) {
-					info.fMask = OS.MIIM_BITMAP;
-					info.hbmpItem = OS.HBMMENU_CALLBACK;
-					success = OS.SetMenuItemInfo (hMenu, id, false, info);
-				}
+		boolean hasBitmap = false;
+		if (!OS.IsWinCE && OS.WIN32_VERSION >= OS.VERSION (4, 10)) {
+			info.fMask = OS.MIIM_BITMAP;
+			OS.GetMenuItemInfo (hMenu, id, false, info);
+			hasBitmap = info.hbmpItem != 0;
+		}
+		
+		/* Use the character encoding for the default locale */
+		TCHAR buffer = new TCHAR (0, string, true);
+		int byteCount = buffer.length () * TCHAR.sizeof;
+		pszText = OS.HeapAlloc (hHeap, OS.HEAP_ZERO_MEMORY, byteCount);
+		OS.MoveMemory (pszText, buffer, byteCount);	
+		info.fMask = OS.MIIM_TYPE;
+		info.fType = widgetStyle ();
+		info.dwTypeData = pszText;
+		success = OS.SetMenuItemInfo (hMenu, id, false, info);
+
+		/*
+		* Restore the bitmap that was removed to work around a problem
+		* in GetMenuItemInfo() and menu items that have bitmaps set with
+		* MIIM_BITMAP.
+		*/
+		if (!OS.IsWinCE && OS.WIN32_VERSION >= OS.VERSION (4, 10)) {
+			if (hasBitmap) {
+				info.fMask = OS.MIIM_BITMAP;
+				info.hbmpItem = OS.HBMMENU_CALLBACK;
+				success = OS.SetMenuItemInfo (hMenu, id, false, info);
 			}
 		}
 	}
@@ -933,26 +1104,20 @@ LRESULT wmDrawChild (int wParam, int lParam) {
 	DRAWITEMSTRUCT struct = new DRAWITEMSTRUCT ();
 	OS.MoveMemory (struct, lParam, DRAWITEMSTRUCT.sizeof);
 	if (image != null) {
+		GCData data = new GCData();
+		data.device = display;
+		GC gc = GC.win32_new (struct.hDC, data);
 		/*
-		* This code intentionally commented.
+		* Bug in Windows.  When a bitmap is included in the
+		* menu bar, the HDC seems to already include the left
+		* coordinate.  The fix is to ignore this value when
+		* the item is in a menu bar.
 		*/
-//		GC gc = GC.win32_new (struct.hDC, null);
-//		gc.drawImage (image, struct.left, struct.top);
-		int hImage = image.handle;
-		switch (image.type) {
-			case SWT.BITMAP:
-				BITMAP bm = new BITMAP ();
-				OS.GetObject (hImage, BITMAP.sizeof, bm);
-				int hDC = OS.CreateCompatibleDC (struct.hDC);
-				int oldBitmap = OS.SelectObject (hDC, hImage);
-				OS.BitBlt (struct.hDC, struct.left, struct.top + 2, bm.bmWidth, bm.bmHeight, hDC, 0, 0, OS.SRCCOPY);
-				OS.SelectObject (hDC, oldBitmap);
-				OS.DeleteDC (hDC);
-				break;
-			case SWT.ICON:
-				OS.DrawIconEx (struct.hDC, struct.left, struct.top + 2, hImage, 0, 0, 0, 0, OS.DI_NORMAL);
-				break;
-		}
+		int x = (parent.style & SWT.BAR) != 0 ? MARGIN_WIDTH * 2 : struct.left;
+		Image image = getEnabled () ? this.image : new Image (display, this.image, SWT.IMAGE_DISABLE);
+		gc.drawImage (image, x, struct.top + MARGIN_HEIGHT);
+		if (this.image != image) image.dispose ();
+		gc.dispose ();
 	}
 	return null;
 }
@@ -960,20 +1125,43 @@ LRESULT wmDrawChild (int wParam, int lParam) {
 LRESULT wmMeasureChild (int wParam, int lParam) {
 	MEASUREITEMSTRUCT struct = new MEASUREITEMSTRUCT ();
 	OS.MoveMemory (struct, lParam, MEASUREITEMSTRUCT.sizeof);
+	int width = 0, height = 0;
 	if (image != null) {
-		/*
-		* Feature in Windows.  On Windows 98, it is necessary
-		* to add 4 pixels to the width of the image or the image
-		* and text are too close.  On other Windows platforms,
-		* this causes the text of the longest item to touch the
-		* accelerator text.  The fix is to add only 2 pixels in
-		* this case.
-		*/
 		Rectangle rect = image.getBounds ();
-		struct.itemWidth = rect.width + (OS.IsWin95 ? 4 : 2);
-		struct.itemHeight = rect.height + 4;
+		width = rect.width;
+		height = rect.height;
+	} else {
+		/*
+		* Bug in Windows.  If a menu contains items that have
+		* images and can be checked, Windows does not include
+		* the width of the image and the width of the check when
+		* computing the width of the menu.  When the longest item
+		* does not have an image, the label and the accelerator
+		* text can overlap.  The fix is to use SetMenuItemInfo()
+		* to indicate that all items have a bitmap and then include
+		* the width of the widest bitmap in WM_MEASURECHILD.
+		*/
+		MENUINFO lpcmi = new MENUINFO ();
+		lpcmi.cbSize = MENUINFO.sizeof;
+		lpcmi.fMask = OS.MIM_STYLE;
+		int hMenu = parent.handle;
+		OS.GetMenuInfo (hMenu, lpcmi);
+		if ((lpcmi.dwStyle & OS.MNS_CHECKORBMP) == 0) {
+			MenuItem [] items = parent.getItems ();
+			for (int i=0; i<items.length; i++) {
+				MenuItem item = items [i];
+				if (item.image != null) {
+					Rectangle rect = item.image.getBounds ();
+					width = Math.max (width, rect.width); 
+				}
+			}
+		}
+	}
+	if (width != 0 || height != 0) {
+		struct.itemWidth = width + MARGIN_WIDTH * 2;
+		struct.itemHeight = height + MARGIN_HEIGHT * 2;
+		OS.MoveMemory (lParam, struct, MEASUREITEMSTRUCT.sizeof);
 	}
-	OS.MoveMemory (lParam, struct, MEASUREITEMSTRUCT.sizeof);	
 	return null;
 }
 
