@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -51,9 +51,18 @@ import org.eclipse.swt.events.*;
  * of these changes are also possible.
  * </li>
  * </ul>
- * </p>
- * <p>
- * Note: The styles supported by this class must be treated
+ * </p><p>
+ * The <em>modality</em> of an instance may be specified using
+ * style bits. The modality style bits are used to determine
+ * whether input is blocked for other shells on the display.
+ * The <code>PRIMARY_MODAL</code> style allows an instance to block
+ * input to its parent. The <code>APPLICATION_MODAL</code> style
+ * allows an instance to block input to every other shell in the
+ * display. The <code>SYSTEM_MODAL</code> style allows an instance
+ * to block input to all shells, including shells belonging to
+ * different applications.
+ * </p><p>
+ * Note: The styles supported by this class are treated
  * as <em>HINT</em>s, since the window manager for the
  * desktop on which the instance is visible has ultimate
  * control over the appearance and behavior of decorations
@@ -64,6 +73,11 @@ import org.eclipse.swt.events.*;
  * more restrictive modality style that is supported. For
  * example, if <code>PRIMARY_MODAL</code> is not supported,
  * it would be upgraded to <code>APPLICATION_MODAL</code>.
+ * A modality style may also be "downgraded" to a less
+ * restrictive style. For example, most operating systems
+ * no longer support <code>SYSTEM_MODAL</code> because
+ * it can freeze up the desktop, so this is typically
+ * downgraded to <code>APPLICATION_MODAL</code>.
  * <dl>
  * <dt><b>Styles:</b></dt>
  * <dd>BORDER, CLOSE, MIN, MAX, NO_TRIM, RESIZE, TITLE, ON_TOP, TOOL</dd>
@@ -100,15 +114,26 @@ import org.eclipse.swt.events.*;
  */
 public class Shell extends Decorations {
 	Menu activeMenu;
-	int hIMC, hwndMDIClient, toolTipHandle, lpstrTip;
+	ToolTip [] toolTips;
+	int hIMC, hwndMDIClient, lpstrTip, toolTipHandle, balloonTipHandle;
 	int minWidth = SWT.DEFAULT, minHeight = SWT.DEFAULT;
 	int [] brushes;
 	boolean showWithParent;
 	Control lastActive;
 	SHACTIVATEINFO psai;
 	Region region;
+	static /*final*/ int ToolTipProc;
 	static final int DialogProc;
 	static final TCHAR DialogClass = new TCHAR (0, OS.IsWinCE ? "Dialog" : "#32770", true);
+	final static int [] SYSTEM_COLORS = {
+		OS.COLOR_BTNFACE,
+		OS.COLOR_WINDOW,
+		OS.COLOR_BTNTEXT,
+		OS.COLOR_WINDOWTEXT,
+		OS.COLOR_HIGHLIGHT,
+		OS.COLOR_SCROLLBAR,
+	};
+	final static int BRUSHES_SIZE = 32;
 	static {
 		WNDCLASS lpWndClass = new WNDCLASS ();
 		OS.GetClassInfo (0, DialogClass, lpWndClass);
@@ -390,8 +415,16 @@ public void addShellListener (ShellListener listener) {
 	addListener (SWT.Deactivate, typedListener);
 }
 
+int balloonTipHandle () {
+	if (balloonTipHandle == 0) createBalloonTipHandle ();
+	return balloonTipHandle;
+}
+
 int callWindowProc (int hwnd, int msg, int wParam, int lParam) {
 	if (handle == 0) return 0;
+	if (hwnd == toolTipHandle || hwnd == balloonTipHandle) {
+		return OS.CallWindowProc (ToolTipProc, hwnd, msg, wParam, lParam);
+	}
 	if ((style & SWT.TOOL) != 0) {
 		int trim = SWT.TITLE | SWT.CLOSE | SWT.MIN | SWT.MAX | SWT.BORDER | SWT.RESIZE;
 		if ((style & trim) == 0) return OS.DefWindowProc (hwnd, msg, wParam, lParam);
@@ -430,6 +463,33 @@ public void close () {
 	closeWidget ();
 }
 
+void createBalloonTipHandle () {
+	balloonTipHandle = OS.CreateWindowEx (
+		0,
+		new TCHAR (0, OS.TOOLTIPS_CLASS, true),
+		null,
+		OS.TTS_ALWAYSTIP | OS.TTS_BALLOON,
+		OS.CW_USEDEFAULT, 0, OS.CW_USEDEFAULT, 0,
+		handle,
+		0,
+		OS.GetModuleHandle (null),
+		null);
+	if (balloonTipHandle == 0) error (SWT.ERROR_NO_HANDLES);
+	if (ToolTipProc == 0) {
+		ToolTipProc = OS.GetWindowLong (balloonTipHandle, OS.GWL_WNDPROC);
+	}
+	/*
+	* Feature in Windows.  Despite the fact that the
+	* tool tip text contains \r\n, the tooltip will
+	* not honour the new line unless TTM_SETMAXTIPWIDTH
+	* is set.  The fix is to set TTM_SETMAXTIPWIDTH to
+	* a large value.
+	*/
+	OS.SendMessage (balloonTipHandle, OS.TTM_SETMAXTIPWIDTH, 0, 0x7FFF);
+	display.addControl (balloonTipHandle, this);
+	OS.SetWindowLong (balloonTipHandle, OS.GWL_WNDPROC, display.windowProc);
+}
+
 void createHandle () {
 	boolean embedded = handle != 0;
 	
@@ -482,6 +542,74 @@ void createHandle () {
 	}
 }
 
+void createToolTip (ToolTip toolTip) {
+	int id = 0;
+	if (toolTips == null) toolTips = new ToolTip [4];
+	while (id < toolTips.length && toolTips [id] != null) id++;
+	if (id == toolTips.length) {
+		ToolTip [] newToolTips = new ToolTip [toolTips.length + 4];
+		System.arraycopy (toolTips, 0, newToolTips, 0, toolTips.length);
+		toolTips = newToolTips;
+	}
+	toolTips [id] = toolTip;
+	toolTip.id = id + Display.ID_START;
+	if (OS.IsWinCE) return;
+	TOOLINFO lpti = new TOOLINFO ();
+	lpti.cbSize = TOOLINFO.sizeof;
+	lpti.hwnd = handle;
+	lpti.uId = toolTip.id;
+	lpti.uFlags = OS.TTF_TRACK;
+	lpti.lpszText = OS.LPSTR_TEXTCALLBACK;
+	OS.SendMessage (toolTip.hwndToolTip (), OS.TTM_ADDTOOL, 0, lpti);
+}
+
+void createToolTipHandle () {
+	toolTipHandle = OS.CreateWindowEx (
+		0,
+		new TCHAR (0, OS.TOOLTIPS_CLASS, true),
+		null,
+		OS.TTS_ALWAYSTIP,
+		OS.CW_USEDEFAULT, 0, OS.CW_USEDEFAULT, 0,
+		handle,
+		0,
+		OS.GetModuleHandle (null),
+		null);
+	if (toolTipHandle == 0) error (SWT.ERROR_NO_HANDLES);
+	if (ToolTipProc == 0) {
+		ToolTipProc = OS.GetWindowLong (toolTipHandle, OS.GWL_WNDPROC);
+	}
+	/*
+	* Feature in Windows.  Despite the fact that the
+	* tool tip text contains \r\n, the tooltip will
+	* not honour the new line unless TTM_SETMAXTIPWIDTH
+	* is set.  The fix is to set TTM_SETMAXTIPWIDTH to
+	* a large value.
+	*/
+	OS.SendMessage (toolTipHandle, OS.TTM_SETMAXTIPWIDTH, 0, 0x7FFF);
+	display.addControl (toolTipHandle, this);
+	OS.SetWindowLong (toolTipHandle, OS.GWL_WNDPROC, display.windowProc);
+}
+
+void deregister () {
+	super.deregister ();
+	if (toolTipHandle != 0) display.removeControl (toolTipHandle);
+	if (balloonTipHandle != 0) display.removeControl (balloonTipHandle);
+}
+
+void destroyToolTip (ToolTip toolTip) {
+	if (toolTips == null) return;
+	toolTips [toolTip.id - Display.ID_START] = null;
+	if (OS.IsWinCE) return;
+	if (balloonTipHandle != 0) {
+		TOOLINFO lpti = new TOOLINFO ();
+		lpti.cbSize = TOOLINFO.sizeof;
+		lpti.uId = toolTip.id;
+		lpti.hwnd = handle;
+		OS.SendMessage (balloonTipHandle, OS.TTM_DELTOOL, 0, lpti);
+	}
+	toolTip.id = -1;
+}
+
 public void dispose () {
 	/*
 	* This code is intentionally commented.  On some
@@ -514,27 +642,50 @@ void enableWidget (boolean enabled) {
 	}
 }
 
-int findBrush (int pixel) {
-	if (pixel == OS.GetSysColor (OS.COLOR_BTNFACE)) {
-		return OS.GetSysColorBrush (OS.COLOR_BTNFACE);
+int findBrush (int value, int lbStyle) {
+	if (lbStyle == OS.BS_SOLID) {
+		for (int i=0; i<SYSTEM_COLORS.length; i++) {
+			if (value == OS.GetSysColor (SYSTEM_COLORS [i])) {
+				return OS.GetSysColorBrush (SYSTEM_COLORS [i]);
+			}
+		}
 	}
-	if (pixel == OS.GetSysColor (OS.COLOR_WINDOW)) {
-		return OS.GetSysColorBrush (OS.COLOR_WINDOW);
-	}
-	if (brushes == null) brushes = new int [4];
+	if (brushes == null) brushes = new int [BRUSHES_SIZE];
 	LOGBRUSH logBrush = new LOGBRUSH ();
 	for (int i=0; i<brushes.length; i++) {
 		int hBrush = brushes [i];
 		if (hBrush == 0) break;
 		OS.GetObject (hBrush, LOGBRUSH.sizeof, logBrush);
-		if (logBrush.lbColor == pixel) return hBrush;
+		switch (logBrush.lbStyle) {
+			case OS.BS_SOLID:
+				if (lbStyle == OS.BS_SOLID) {
+					if (logBrush.lbColor == value) return hBrush;
+				}
+				break;
+			case OS.BS_PATTERN:
+				if (lbStyle == OS.BS_PATTERN) {
+					if (logBrush.lbHatch == value) return hBrush;
+				}
+				break;
+		}
 	}
 	int length = brushes.length;
 	int hBrush = brushes [--length];
 	if (hBrush != 0) OS.DeleteObject (hBrush);
 	System.arraycopy (brushes, 0, brushes, 1, length);
-	brushes [0] = hBrush = OS.CreateSolidBrush (pixel);
-	return hBrush;
+	switch (lbStyle) {
+		case OS.BS_SOLID:
+			hBrush = OS.CreateSolidBrush (value);
+			break;
+		case OS.BS_PATTERN:
+			hBrush = OS.CreatePatternBrush (value);
+			break;
+	}
+	return brushes [0] = hBrush;
+}
+
+Control findBackgroundControl () {
+	return background != -1 || backgroundImage != null ? this : null;
 }
 
 Cursor findCursor () {
@@ -545,13 +696,20 @@ Control findThemeControl () {
 	return null;
 }
 
+ToolTip findToolTip (int id) {
+	if (toolTips == null) return null;
+	id = id - Display.ID_START;
+	return 0 <= id && id < toolTips.length ? toolTips [id] : null;
+}
+
 void fixShell (Shell newShell, Control control) {
 	if (this == newShell) return;
 	if (control == lastActive) setActiveControl (null);
-	if (toolTipHandle != 0) {
-		setToolTipText (control.handle, null);
+	String toolTipText = control.toolTipText;
+	if (toolTipText != null) {
+		control.setToolTipText (this, null);
+		control.setToolTipText (newShell, toolTipText);
 	}
-	newShell.setToolTipText (control.handle, control.toolTipText);
 }
 
 /**
@@ -597,6 +755,30 @@ public Rectangle getBounds () {
 	return new Rectangle (rect.left, rect.top, width, height);
 }
 
+ToolTip getCurrentToolTip () {
+	if (toolTipHandle != 0) {
+		ToolTip tip = getCurrentToolTip (toolTipHandle);
+		if (tip != null) return tip;
+	}
+	if (balloonTipHandle != 0) {
+		ToolTip tip = getCurrentToolTip (balloonTipHandle);
+		if (tip != null) return tip;
+	}
+	return null;
+}
+
+ToolTip getCurrentToolTip (int hwndToolTip) {
+	if (hwndToolTip == 0) return null;
+	if (OS.SendMessage (hwndToolTip, OS.TTM_GETCURRENTTOOL, 0, 0) != 0) {
+		TOOLINFO lpti = new TOOLINFO ();
+		lpti.cbSize = TOOLINFO.sizeof;
+		if (OS.SendMessage (hwndToolTip, OS.TTM_GETCURRENTTOOL, 0, lpti) != 0) {
+			if ((lpti.uFlags & OS.TTF_IDISHWND) == 0) return findToolTip (lpti.uId);
+		}
+	}
+	return null;
+}
+
 public boolean getEnabled () {
 	checkWidget ();
 	return (state & DISABLED) == 0;
@@ -757,9 +939,8 @@ public Shell [] getShells () {
 	return result;
 }
 
-public boolean isLayoutDeferred () {
-	checkWidget ();
-	return layoutCount > 0;
+Composite findDeferredControl () {
+	return layoutCount > 0 ? this : null;
 }
 
 public boolean isEnabled () {
@@ -846,8 +1027,39 @@ public void open () {
 	if (!restoreFocus () && !traverseGroup (true)) setFocus ();
 }
 
-void releaseChild () {
-	/* Do nothing */
+void register () {
+	super.register ();
+	if (toolTipHandle != 0) display.addControl (toolTipHandle, this);
+	if (balloonTipHandle != 0) display.addControl (balloonTipHandle, this);
+}
+
+void releaseBrushes () {
+	if (brushes != null) {
+		for (int i=0; i<brushes.length; i++) {
+			if (brushes [i] != 0) OS.DeleteObject (brushes [i]);
+		}
+	}
+	brushes = null;
+}
+
+void releaseChildren (boolean destroy) {
+	Shell [] shells = getShells ();
+	for (int i=0; i<shells.length; i++) {
+		Shell shell = shells [i];
+		if (shell != null && !shell.isDisposed ()) {
+			shell.release (false);
+		}
+	}
+	if (toolTips != null) {
+		for (int i=0; i<toolTips.length; i++) {
+			ToolTip toolTip = toolTips [i];
+			if (toolTip != null && !toolTip.isDisposed ()) {
+				toolTip.release (false);
+			}
+		}
+	}
+	toolTips = null;
+	super.releaseChildren (destroy);
 }
 
 void releaseHandle () {
@@ -855,17 +1067,13 @@ void releaseHandle () {
 	hwndMDIClient = 0;
 }
 
-void releaseShells () {
-	Shell [] shells = getShells ();
-	for (int i=0; i<shells.length; i++) {
-		Shell shell = shells [i];
-		if (!shell.isDisposed ()) shell.releaseResources ();
-	}
+void releaseParent () {
+	/* Do nothing */
 }
 
 void releaseWidget () {
-	releaseShells ();
 	super.releaseWidget ();
+	releaseBrushes ();
 	activeMenu = null;
 	display.clearModal (this);
 	if (lpstrTip != 0) {
@@ -873,14 +1081,7 @@ void releaseWidget () {
 		OS.HeapFree (hHeap, 0, lpstrTip);
 	}
 	lpstrTip = 0;
-	toolTipHandle = 0;
-	if (brushes != null) {
-		for (int i=0; i<brushes.length; i++) {
-			int hBrush = brushes [i];
-			if (hBrush != 0) OS.DeleteObject (hBrush);
-		}
-	}
-	brushes = null;
+	toolTipHandle = balloonTipHandle = 0;
 	if (OS.IsDBLocale) {
 		if (hIMC != 0) OS.ImmDestroyContext (hIMC);
 	}
@@ -1177,40 +1378,20 @@ public void setRegion (Region region) {
 
 void setToolTipText (int hwnd, String text) {
 	if (OS.IsWinCE) return;
-	if (toolTipHandle == 0) {
-		toolTipHandle = OS.CreateWindowEx (
-			0,
-			new TCHAR (0, OS.TOOLTIPS_CLASS, true),
-			null,
-			OS.TTS_ALWAYSTIP,
-			OS.CW_USEDEFAULT, 0, OS.CW_USEDEFAULT, 0,
-			handle,
-			0,
-			OS.GetModuleHandle (null),
-			null);
-		if (toolTipHandle == 0) error (SWT.ERROR_NO_HANDLES);	
-		/*
-		* Feature in Windows.  Despite the fact that the
-		* tool tip text contains \r\n, the tooltip will
-		* not honour the new line unless TTM_SETMAXTIPWIDTH
-		* is set.  The fix is to set TTM_SETMAXTIPWIDTH to
-		* a large value.
-		*/
-		OS.SendMessage (toolTipHandle, OS.TTM_SETMAXTIPWIDTH, 0, 0x7FFF);
-	}
 	TOOLINFO lpti = new TOOLINFO ();
 	lpti.cbSize = TOOLINFO.sizeof;
-	lpti.uId = hwnd;
 	lpti.hwnd = handle;
+	lpti.uId = hwnd;
+	int hwndToolTip = toolTipHandle ();
 	if (text == null) {
-		OS.SendMessage (toolTipHandle, OS.TTM_DELTOOL, 0, lpti);
+		OS.SendMessage (hwndToolTip, OS.TTM_DELTOOL, 0, lpti);
 	} else {
-		if (OS.SendMessage (toolTipHandle, OS.TTM_GETTOOLINFO, 0, lpti) != 0) {
-			OS.SendMessage (toolTipHandle, OS.TTM_UPDATE, 0, 0);
+		if (OS.SendMessage (hwndToolTip, OS.TTM_GETTOOLINFO, 0, lpti) != 0) {
+			OS.SendMessage (hwndToolTip, OS.TTM_UPDATE, 0, 0);
 		} else {
 			lpti.uFlags = OS.TTF_IDISHWND | OS.TTF_SUBCLASS;
 			lpti.lpszText = OS.LPSTR_TEXTCALLBACK;
-			OS.SendMessage (toolTipHandle, OS.TTM_ADDTOOL, 0, lpti);
+			OS.SendMessage (hwndToolTip, OS.TTM_ADDTOOL, 0, lpti);
 		}
 	}
 }
@@ -1306,6 +1487,24 @@ public void setVisible (boolean visible) {
 	}
 }
 
+void subclass () {
+	super.subclass ();
+	if (ToolTipProc != 0) {
+		int newProc = display.windowProc;
+		if (toolTipHandle != 0) {
+			OS.SetWindowLong (toolTipHandle, OS.GWL_WNDPROC, newProc);
+		}
+		if (balloonTipHandle != 0) {
+			OS.SetWindowLong (balloonTipHandle, OS.GWL_WNDPROC, newProc);
+		}
+	}
+}
+
+int toolTipHandle () {
+	if (toolTipHandle == 0) createToolTipHandle ();
+	return toolTipHandle;
+}
+
 boolean translateAccelerator (MSG msg) {
 	if (!isEnabled () || !isActive ()) return false;
 	if (menuBar != null && !menuBar.isEnabled ()) return false;
@@ -1319,6 +1518,18 @@ boolean traverseEscape () {
 	return true;
 }
 
+void unsubclass () {
+	super.unsubclass ();
+	if (ToolTipProc != 0) {
+		if (toolTipHandle != 0) {
+			OS.SetWindowLong (toolTipHandle, OS.GWL_WNDPROC, ToolTipProc);
+		}	
+		if (toolTipHandle != 0) {
+			OS.SetWindowLong (toolTipHandle, OS.GWL_WNDPROC, ToolTipProc);
+		}
+	}
+}
+
 void updateModal () {
 	if (Display.TrimEnabled) {
 		setItemEnabled (OS.SC_CLOSE, isActive ());
@@ -1397,6 +1608,32 @@ int windowProc () {
 	return parent != null ? DialogProc : super.windowProc ();
 }
 
+int windowProc (int hwnd, int msg, int wParam, int lParam) {
+	if (handle == 0) return 0;
+	if (hwnd == toolTipHandle || hwnd == balloonTipHandle) {
+		switch (msg) {
+			case OS.WM_TIMER: {
+				if (wParam != ToolTip.TIMER_ID) break;
+				ToolTip tip = getCurrentToolTip (hwnd);
+				if (tip != null && tip.autoHide) {
+					tip.setVisible (false);
+				}
+				break;
+			}
+			case OS.WM_LBUTTONDOWN: {
+				ToolTip tip = getCurrentToolTip (hwnd);
+				if (tip != null) {
+					tip.setVisible (false);
+					tip.postEvent (SWT.Selection);
+				}
+				break;
+			}
+		}
+		return callWindowProc (hwnd, msg, wParam, lParam);
+	}
+	return super.windowProc (hwnd, msg, wParam, lParam);
+}
+
 int widgetStyle () {
 	int bits = super.widgetStyle ();
 	if (handle != 0) return bits | OS.WS_CHILD;
@@ -1473,9 +1710,15 @@ LRESULT WM_ACTIVATE (int wParam, int lParam) {
 		}
 	}
 	
+	/* Process WM_ACTIVATE */
 	LRESULT result = super.WM_ACTIVATE (wParam, lParam);
-	if (parent != null) return LRESULT.ZERO;
-	return result;
+	if ((wParam & 0xFFFF) == 0) {
+		if (lParam == 0 || (lParam != toolTipHandle && lParam != balloonTipHandle)) {
+			ToolTip tip = getCurrentToolTip ();
+			if (tip != null) tip.setVisible (false);
+		}
+	}
+	return parent != null ? LRESULT.ZERO : result;
 }
 
 LRESULT WM_COMMAND (int wParam, int lParam) {
@@ -1527,8 +1770,8 @@ LRESULT WM_DESTROY (int wParam, int lParam) {
 	*/
 	int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
 	if ((bits & OS.WS_CHILD) != 0) {
-		releaseChild ();
-		releaseResources ();
+		releaseParent ();
+		release (false);
 	}
 	return result;
 }
@@ -1536,7 +1779,7 @@ LRESULT WM_DESTROY (int wParam, int lParam) {
 LRESULT WM_ENTERIDLE (int wParam, int lParam) {
 	LRESULT result = super.WM_ENTERIDLE (wParam, lParam);
 	if (result != null) return result;
-	if (OS.IsWinCE) {
+	if (OS.IsWinCE && display.runMessages) {
 		if (display.runAsyncMessages (true)) display.wakeThread ();
 	}
 	return result;
@@ -1636,6 +1879,14 @@ LRESULT WM_MOUSEACTIVATE (int wParam, int lParam) {
 	return null;
 }
 
+LRESULT WM_MOVE (int wParam, int lParam) {
+	LRESULT result = super.WM_MOVE (wParam, lParam);
+	if (result != null) return result;
+	ToolTip tip = getCurrentToolTip ();
+	if (tip != null) tip.setVisible (false);
+	return result;
+}
+
 LRESULT WM_NCHITTEST (int wParam, int lParam) {
 	if (!OS.IsWindowEnabled (handle)) return null;
 	if (!isEnabled () || !isActive ()) {
