@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -41,7 +41,7 @@ import org.eclipse.swt.graphics.*;
 public class ToolBar extends Composite {
 	int lastFocusId;
 	ToolItem [] items;
-	boolean ignoreResize;
+	boolean ignoreResize, ignoreMouse;
 	ImageList imageList, disabledImageList, hotImageList;
 	static final int ToolBarProc;
 	static final TCHAR ToolBarClass = new TCHAR (0, OS.TOOLBARCLASSNAME, true);
@@ -91,8 +91,8 @@ public class ToolBar extends Composite {
  * @see SWT#HORIZONTAL
  * @see SWT#SHADOW_OUT
  * @see SWT#VERTICAL
- * @see Widget#checkSubclass
- * @see Widget#getStyle
+ * @see Widget#checkSubclass()
+ * @see Widget#getStyle()
  */
 public ToolBar (Composite parent, int style) {
 	super (parent, checkStyle (style));
@@ -103,15 +103,36 @@ public ToolBar (Composite parent, int style) {
 	* clear these bits to avoid scroll bars and then reset
 	* the bits using the original style supplied by the
 	* programmer.
+	* 
+	* NOTE: The CCS_VERT style cannot be applied when the
+	* widget is created because of this conflict.
 	*/
 	if ((style & SWT.VERTICAL) != 0) {
 		this.style |= SWT.VERTICAL;
+		int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
+		/*
+		* Feature in Windows.  When a tool bar has the style
+		* TBSTYLE_LIST and has a drop down item, Window leaves
+		* too much padding around the button.  This affects
+		* every button in the tool bar and makes the preferred
+		* height too big.  The fix is to set the TBSTYLE_LIST
+		* when the tool bar contains both text and images.
+		* 
+		* NOTE: Tool bars with CCS_VERT must have TBSTYLE_LIST
+		* set before any item is added or the tool bar does
+		* not lay out properly.  The work around does not run
+		* in this case.
+		*/
+		if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+			if ((style & SWT.RIGHT) != 0) bits |= OS.TBSTYLE_LIST;
+		}
+		OS.SetWindowLong (handle, OS.GWL_STYLE, bits | OS.CCS_VERT);
 	} else {
 		this.style |= SWT.HORIZONTAL;
 	}
 }
 
-int callWindowProc (int msg, int wParam, int lParam) {
+int callWindowProc (int hwnd, int msg, int wParam, int lParam) {
 	if (handle == 0) return 0;
 	/*
 	* Bug in Windows.  For some reason, during the processing
@@ -121,9 +142,9 @@ int callWindowProc (int msg, int wParam, int lParam) {
 	* window proc for WM_SYSCHAR.
 	*/
 	if (msg == OS.WM_SYSCHAR) {
-		return OS.DefWindowProc (handle, msg, wParam, lParam);
+		return OS.DefWindowProc (hwnd, msg, wParam, lParam);
 	}
-	return OS.CallWindowProc (ToolBarProc, handle, msg, wParam, lParam);
+	return OS.CallWindowProc (ToolBarProc, hwnd, msg, wParam, lParam);
 }
 
 static int checkStyle (int style) {
@@ -136,16 +157,7 @@ static int checkStyle (int style) {
 	* A vertical tool bar cannot wrap because TB_SETROWS
 	* fails when the toobar has TBSTYLE_WRAPABLE.
 	*/
-	/*
-	* This code is intentionally commented.
-	*/
-	//if ((style & SWT.VERTICAL) != 0) style &= ~SWT.WRAP;
-	
-	/*
-	* The TB_SETROWS calls are currently commented, so force
-	* the wrap style if this bar is vertical.
-	*/
-	if ((style & SWT.VERTICAL) != 0) style |= SWT.WRAP;
+	if ((style & SWT.VERTICAL) != 0) style &= ~SWT.WRAP;
 		
 	/*
 	* Even though it is legal to create this widget
@@ -157,38 +169,60 @@ static int checkStyle (int style) {
 	return style & ~(SWT.H_SCROLL | SWT.V_SCROLL);
 }
 
+void checkBuffered () {
+	super.checkBuffered ();
+	if (OS.COMCTL32_MAJOR >= 6) style |= SWT.DOUBLE_BUFFERED;
+}
+
 protected void checkSubclass () {
 	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
 }
 
 public Point computeSize (int wHint, int hHint, boolean changed) {
 	checkWidget ();
-	if (layout != null) {
-		return super.computeSize (wHint, hHint, changed);
-	}
 	int width = 0, height = 0;
-	RECT oldRect = new RECT ();
-	OS.GetWindowRect (handle, oldRect);
-	int oldWidth = oldRect.right - oldRect.left;
-	int oldHeight = oldRect.bottom - oldRect.top;
-	int border = getBorderWidth ();
-	int newWidth = wHint == SWT.DEFAULT ? 0x3FFF : wHint + (border * 2);
-	int newHeight = hHint == SWT.DEFAULT ? 0x3FFF : hHint + (border * 2);
-	boolean redraw = drawCount == 0 && OS.IsWindowVisible (handle);
-	ignoreResize = true;
-	if (redraw) OS.UpdateWindow (handle);
-	int flags = OS.SWP_NOACTIVATE | OS.SWP_NOMOVE | OS.SWP_NOREDRAW | OS.SWP_NOZORDER;
-	OS.SetWindowPos (handle, 0, 0, 0, newWidth, newHeight, flags);
-	int count = OS.SendMessage (handle, OS.TB_BUTTONCOUNT, 0, 0);
-	if (count != 0) {
+	if ((style & SWT.VERTICAL) != 0) {
 		RECT rect = new RECT ();
-		OS.SendMessage (handle, OS.TB_GETITEMRECT, count - 1, rect);
-		width = Math.max (width, rect.right);
-		height = Math.max (height, rect.bottom);
+		TBBUTTON lpButton = new TBBUTTON ();
+		int count = OS.SendMessage (handle, OS.TB_BUTTONCOUNT, 0, 0);
+		for (int i=0; i<count; i++) {
+			OS.SendMessage (handle, OS.TB_GETITEMRECT, i, rect);
+			height = Math.max (height, rect.bottom);
+			OS.SendMessage (handle, OS.TB_GETBUTTON, i, lpButton);
+			if ((lpButton.fsStyle & OS.BTNS_SEP) != 0) {
+				TBBUTTONINFO info = new TBBUTTONINFO ();
+				info.cbSize = TBBUTTONINFO.sizeof;
+				info.dwMask = OS.TBIF_SIZE;
+				OS.SendMessage (handle, OS.TB_GETBUTTONINFO, lpButton.idCommand, info);
+				width = Math.max (width, info.cx);
+			} else {
+				width = Math.max (width, rect.right);
+			}
+		}
+	} else {
+		RECT oldRect = new RECT ();
+		OS.GetWindowRect (handle, oldRect);
+		int oldWidth = oldRect.right - oldRect.left;
+		int oldHeight = oldRect.bottom - oldRect.top;
+		int border = getBorderWidth ();
+		int newWidth = wHint == SWT.DEFAULT ? 0x3FFF : wHint + border * 2;
+		int newHeight = hHint == SWT.DEFAULT ? 0x3FFF : hHint + border * 2;
+		boolean redraw = drawCount == 0 && OS.IsWindowVisible (handle);
+		ignoreResize = true;
+		if (redraw) OS.UpdateWindow (handle);
+		int flags = OS.SWP_NOACTIVATE | OS.SWP_NOMOVE | OS.SWP_NOREDRAW | OS.SWP_NOZORDER;
+		SetWindowPos (handle, 0, 0, 0, newWidth, newHeight, flags);
+		int count = OS.SendMessage (handle, OS.TB_BUTTONCOUNT, 0, 0);
+		if (count != 0) {
+			RECT rect = new RECT ();
+			OS.SendMessage (handle, OS.TB_GETITEMRECT, count - 1, rect);
+			width = Math.max (width, rect.right);
+			height = Math.max (height, rect.bottom);
+		}
+		SetWindowPos (handle, 0, 0, 0, oldWidth, oldHeight, flags);
+		if (redraw) OS.ValidateRect (handle, null);
+		ignoreResize = false;
 	}
-	OS.SetWindowPos (handle, 0, 0, 0, oldWidth, oldHeight, flags);
-	if (redraw) OS.ValidateRect (handle, null);
-	ignoreResize = false;
 	
 	/*
 	* From the Windows SDK for TB_SETBUTTONSIZE:
@@ -228,10 +262,12 @@ void createHandle () {
 	* flickering and clearing the TBSTYLE_TRANSPARENT interferes
 	* with the XP theme.
 	*/
-	if (COMCTL32_MAJOR < 6) {
-		int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
-		bits &= ~OS.TBSTYLE_TRANSPARENT;
-		OS.SetWindowLong (handle, OS.GWL_STYLE, bits);
+	if ((style & SWT.FLAT) != 0) {
+		if (OS.COMCTL32_MAJOR < 6 || !OS.IsAppThemed ()) {
+			int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
+			bits &= ~OS.TBSTYLE_TRANSPARENT;
+			OS.SetWindowLong (handle, OS.GWL_STYLE, bits);
+		}
 	}
 
 	/*
@@ -270,7 +306,9 @@ void createHandle () {
 	OS.SendMessage (handle, OS.TB_SETBUTTONSIZE, 0, 0);
 
 	/* Set the extended style bits */
-	OS.SendMessage (handle, OS.TB_SETEXTENDEDSTYLE, 0, OS.TBSTYLE_EX_DRAWDDARROWS);
+	int bits = OS.TBSTYLE_EX_DRAWDDARROWS | OS.TBSTYLE_EX_MIXEDBUTTONS | OS.TBSTYLE_EX_HIDECLIPPEDBUTTONS;
+	if (OS.COMCTL32_MAJOR >= 6) bits |= OS.TBSTYLE_EX_DOUBLEBUFFER;
+	OS.SendMessage (handle, OS.TB_SETEXTENDEDSTYLE, 0, bits);
 }
 
 void createItem (ToolItem item, int index) {
@@ -306,12 +344,7 @@ void createItem (ToolItem item, int index) {
 		error (SWT.ERROR_ITEM_NOT_ADDED);
 	}
 	items [item.id = id] = item;
-	/*
-	* This code is intentionally commented.
-	*/
-//	if ((style & SWT.VERTICAL) != 0) {
-//		OS.SendMessage (handle, OS.TB_SETROWS, count+1, 0);
-//	}
+	if ((style & SWT.VERTICAL) != 0) setRowCount (count + 1);
 	layoutItems ();
 }
 
@@ -352,7 +385,6 @@ void destroyItem (ToolItem item) {
 	item.id = -1;
 	int count = OS.SendMessage (handle, OS.TB_BUTTONCOUNT, 0, 0);
 	if (count == 0) {
-		Display display = getDisplay ();
 		if (imageList != null) {
 			OS.SendMessage (handle, OS.TB_SETIMAGELIST, 0, 0);
 			display.releaseToolImageList (imageList);
@@ -368,15 +400,34 @@ void destroyItem (ToolItem item) {
 		imageList = hotImageList = disabledImageList = null;
 		items = new ToolItem [4];
 	}
-	/*
-	* This code is intentionally commented.
-	*/
-//	if ((style & SWT.VERTICAL) != 0) {
-//		OS.SendMessage (handle, OS.TB_SETROWS, count-1, 0);
-//	}
+	if ((style & SWT.VERTICAL) != 0) setRowCount (count - 1);
 	layoutItems ();
 }
 
+void enableWidget (boolean enabled) {
+	super.enableWidget (enabled);
+	/*
+	* Bug in Windows.  When a tool item with the style
+	* BTNS_CHECK or BTNS_CHECKGROUP is selected and then
+	* disabled, the item does not draw using the disabled
+	* image.  The fix is to use the disabled image in all
+	* image lists for the item.
+	* 
+	* Feature in Windows.  When a tool bar is disabled,
+	* the text draws disabled but the images do not.
+	* The fix is to use the disabled image in all image
+	* lists for all items.
+	*/
+	for (int i=0; i<items.length; i++) {
+		ToolItem item = items [i];
+		if (item != null) {
+			if ((item.style & SWT.SEPARATOR) == 0) {
+				item.updateImages (enabled && item.getEnabled ());
+			}
+		}
+	}
+}
+
 ImageList getDisabledImageList () {
 	return disabledImageList;
 }
@@ -423,7 +474,7 @@ public ToolItem getItem (int index) {
  * @return the item at the given point
  *
  * @exception IllegalArgumentException <ul>
- *    <li>ERROR_NULL_ARGUMENT - if the listener is null</li>
+ *    <li>ERROR_NULL_ARGUMENT - if the point is null</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
@@ -457,7 +508,7 @@ public int getItemCount () {
 }
 
 /**
- * Returns an array of <code>TabItem</code>s which are the items
+ * Returns an array of <code>ToolItem</code>s which are the items
  * in the receiver. 
  * <p>
  * Note: This is not the actual structure used by the receiver
@@ -499,6 +550,9 @@ public ToolItem [] getItems () {
  */
 public int getRowCount () {
 	checkWidget ();
+	if ((style & SWT.VERTICAL) != 0) {
+		return OS.SendMessage (handle, OS.TB_BUTTONCOUNT, 0, 0);
+	}
 	return OS.SendMessage (handle, OS.TB_GETROWS, 0, 0);
 }
 
@@ -527,7 +581,85 @@ public int indexOf (ToolItem item) {
 	return OS.SendMessage (handle, OS.TB_COMMANDTOINDEX, item.id, 0);
 }
 
-void layoutItems () {
+void layoutItems () {	
+	/*
+	* Feature in Windows.  When a tool bar has the style
+	* TBSTYLE_LIST and has a drop down item, Window leaves
+	* too much padding around the button.  This affects
+	* every button in the tool bar and makes the preferred
+	* height too big.  The fix is to set the TBSTYLE_LIST
+	* when the tool bar contains both text and images.
+	* 
+	* NOTE: Tool bars with CCS_VERT must have TBSTYLE_LIST
+	* set before any item is added or the tool bar does
+	* not lay out properly.  The work around does not run
+	* in this case.
+	*/
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+		if ((style & SWT.RIGHT) != 0 && (style & SWT.VERTICAL) == 0) {
+			boolean hasText = false, hasImage = false;
+			for (int i=0; i<items.length; i++) {
+				ToolItem item = items [i];
+				if (item != null) {
+					if (!hasText) hasText = item.text.length () != 0;
+					if (!hasImage) hasImage = item.image != null;
+					if (hasText && hasImage) break;
+				}
+			}
+			int oldBits = OS.GetWindowLong (handle, OS.GWL_STYLE), newBits = oldBits;
+			if (hasText && hasImage) {
+				newBits |= OS.TBSTYLE_LIST;
+			} else {
+				newBits &= ~OS.TBSTYLE_LIST;
+			}
+			if (newBits != oldBits) {
+				setDropDownItems (false);
+				OS.SetWindowLong (handle, OS.GWL_STYLE, newBits);
+				/*
+				* Feature in Windows.  For some reason, when the style
+				* is changed to TBSTYLE_LIST, Windows does not lay out
+				* the tool items.  The fix is to use WM_SETFONT to force
+				* the tool bar to redraw and lay out.
+				*/
+				int hFont = OS.SendMessage (handle, OS.WM_GETFONT, 0, 0);
+				OS.SendMessage (handle, OS.WM_SETFONT, hFont, 0);
+				setDropDownItems (true);
+			}
+		}
+	}
+	
+	if ((style & SWT.WRAP) != 0) {
+		OS.SendMessage (handle, OS.TB_AUTOSIZE, 0, 0);
+	}
+	/*
+	*  When the tool bar is vertical, make the width of each button
+	*  be the width of the widest button in the tool bar.  Note that
+	*  when the tool bar contains a drop down item, it needs to take
+	*  into account extra padding.
+	*/
+	if ((style & SWT.VERTICAL) != 0) {
+		TBBUTTONINFO info = new TBBUTTONINFO ();
+		info.cbSize = TBBUTTONINFO.sizeof;
+		info.dwMask = OS.TBIF_SIZE;
+		int size = OS.SendMessage (handle, OS.TB_GETBUTTONSIZE, 0, 0);
+		info.cx = (short) (size & 0xFFFF);
+		int index = 0;
+		while (index < items.length) {
+			ToolItem item = items [index];
+			if (item != null && (item.style & SWT.DROP_DOWN) != 0) break;
+			index++;
+		}
+		if (index < items.length) {
+			int padding = OS.SendMessage (handle, OS.TB_GETPADDING, 0, 0);
+			info.cx += (padding & 0xFFFF) * 2;
+		}
+		for (int i=0; i<items.length; i++) {
+			ToolItem item = items [i];
+			if (item != null && (item.style & SWT.SEPARATOR) == 0) {
+				OS.SendMessage (handle, OS.TB_SETBUTTONINFO, item.id, info);
+			}
+		}
+	}
 	for (int i=0; i<items.length; i++) {
 		ToolItem item = items [i];
 		if (item != null) item.resizeControl ();
@@ -535,12 +667,12 @@ void layoutItems () {
 }
 
 boolean mnemonicHit (char ch) {
-	int key = wcsToMbcs (ch);
+	int key = Display.wcsToMbcs (ch);
 	int [] id = new int [1];
 	if (OS.SendMessage (handle, OS.TB_MAPACCELERATOR, key, id) == 0) {
 		return false;
 	}
-	if (!setTabGroupFocus ()) return false;
+	if ((style & SWT.FLAT) != 0 && !setTabGroupFocus ()) return false;
 	int index = OS.SendMessage (handle, OS.TB_COMMANDTOINDEX, id [0], 0);
 	if (index == -1) return false;
 	OS.SendMessage (handle, OS.TB_SETHOTITEM, index, 0);
@@ -549,21 +681,37 @@ boolean mnemonicHit (char ch) {
 }
 
 boolean mnemonicMatch (char ch) {
-	int key = wcsToMbcs (ch);
+	int key = Display.wcsToMbcs (ch);
 	int [] id = new int [1];
-	return OS.SendMessage (handle, OS.TB_MAPACCELERATOR, key, id) != 0;
+	if (OS.SendMessage (handle, OS.TB_MAPACCELERATOR, key, id) == 0) {
+		return false;
+	}
+	/*
+	* Feature in Windows.  TB_MAPACCELERATOR matches either the mnemonic
+	* character or the first character in a tool item.  This behavior is
+	* undocumented and unwanted.  The fix is to ensure that the tool item
+	* contains a mnemonic when TB_MAPACCELERATOR returns true.
+	*/
+	int index = OS.SendMessage (handle, OS.TB_COMMANDTOINDEX, id [0], 0);
+	if (index == -1) return false;
+	return findMnemonic (items [id [0]].text) != '\0';
+}
+
+void releaseChildren (boolean destroy) {
+	if (items != null) {
+		for (int i=0; i<items.length; i++) {
+			ToolItem item = items [i];
+			if (item != null && !item.isDisposed ()) {
+				item.release (false);
+			}
+		}
+		items = null;
+	}
+	super.releaseChildren (destroy);
 }
 
 void releaseWidget () {
-	for (int i=0; i<items.length; i++) {
-		ToolItem item = items [i];
-		if (item != null && !item.isDisposed ()) {
-			item.releaseImages ();
-			item.releaseResources ();
-		}
-	}
-	items = null;
-	Display display = getDisplay ();
+	super.releaseWidget ();
 	if (imageList != null) {
 		OS.SendMessage (handle, OS.TB_SETIMAGELIST, 0, 0);
 		display.releaseToolImageList (imageList);
@@ -577,7 +725,51 @@ void releaseWidget () {
 		display.releaseToolDisabledImageList (disabledImageList);
 	}
 	imageList = hotImageList = disabledImageList = null;
-	super.releaseWidget ();
+}
+
+void removeControl (Control control) {
+	super.removeControl (control);
+	for (int i=0; i<items.length; i++) {
+		ToolItem item = items [i];
+		if (item != null && item.control == control) {
+			item.setControl (null);
+		}
+	}
+}
+
+void setBackgroundImage (int hBitmap) {
+	super.setBackgroundImage (hBitmap);
+	setBackgroundTransparent (hBitmap != 0);
+}
+
+void setBackgroundPixel (int pixel) {
+	super.setBackgroundPixel (pixel);
+	setBackgroundTransparent (pixel != -1);
+}
+
+void setBackgroundTransparent (boolean transparent) {
+	/*
+	* Feature in Windows.  When TBSTYLE_TRANSPARENT is set
+	* in a tool bar that is drawing a background, images in
+	* the image list that include transparency information
+	* do not draw correctly.  The fix is to clear and set
+	* TBSTYLE_TRANSPARENT depending on the background color.
+	* 
+	* NOTE:  This work around is unnecessary on XP.  The
+	* TBSTYLE_TRANSPARENT style is never cleared on that
+	* platform.
+	*/
+	if ((style & SWT.FLAT) != 0) {
+		if (OS.COMCTL32_MAJOR < 6 || !OS.IsAppThemed ()) {
+			int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
+			if (!transparent && findBackgroundControl () == null) {
+				bits &= ~OS.TBSTYLE_TRANSPARENT;
+			} else {
+				bits |= OS.TBSTYLE_TRANSPARENT;
+			}
+			OS.SetWindowLong (handle, OS.GWL_STYLE, bits);
+		}
+	}
 }
 
 void setBounds (int x, int y, int width, int height, int flags) {
@@ -605,18 +797,64 @@ void setDefaultFont () {
 	OS.SendMessage (handle, OS.TB_SETBUTTONSIZE, 0, 0);
 }
 
+void setDropDownItems (boolean set) {
+	/*
+	* Feature in Windows.  When the first button in a tool bar
+	* is a drop down item, Window leaves too much padding around
+	* the button.  This affects every button in the tool bar and
+	* makes the preferred height too big.  The fix is clear the
+	* BTNS_DROPDOWN before Windows lays out the tool bar and set
+	* the bit afterwards.
+	* 
+	* NOTE:  This work around only runs when the tool bar contains
+	* both text and images.
+	*/
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+		boolean hasText = false, hasImage = false;
+		for (int i=0; i<items.length; i++) {
+			ToolItem item = items [i];
+			if (item != null) {
+				if (!hasText) hasText = item.text.length () != 0;
+				if (!hasImage) hasImage = item.image != null;
+				if (hasText && hasImage) break;
+			}
+		}
+		if (hasImage && !hasText) {
+			for (int i=0; i<items.length; i++) {
+				ToolItem item = items [i];
+				if (item != null && (item.style & SWT.DROP_DOWN) != 0) {
+					TBBUTTONINFO info = new TBBUTTONINFO ();
+					info.cbSize = TBBUTTONINFO.sizeof;
+					info.dwMask = OS.TBIF_STYLE;
+					OS.SendMessage (handle, OS.TB_GETBUTTONINFO, item.id, info);
+					if (set) {
+						info.fsStyle |= OS.BTNS_DROPDOWN;
+					} else {
+						info.fsStyle &= ~OS.BTNS_DROPDOWN;
+					}
+					OS.SendMessage (handle, OS.TB_SETBUTTONINFO, item.id, info);
+				}
+			}
+		}
+	}
+}
+
 void setDisabledImageList (ImageList imageList) {
 	if (disabledImageList == imageList) return;
 	int hImageList = 0;
 	if ((disabledImageList = imageList) != null) {
 		hImageList = disabledImageList.getHandle ();
 	}
+	setDropDownItems (false);
 	OS.SendMessage (handle, OS.TB_SETDISABLEDIMAGELIST, 0, hImageList);
+	setDropDownItems (true);
 }
 
 public void setFont (Font font) {
 	checkWidget ();
+	setDropDownItems (false);
 	super.setFont (font);
+	setDropDownItems (true);
 	/*
 	* Bug in Windows.  When WM_SETFONT is sent to a tool bar
 	* that contains only separators, causes the bitmap and button
@@ -627,13 +865,14 @@ public void setFont (Font font) {
 	int mask = SWT.PUSH | SWT.CHECK | SWT.RADIO | SWT.DROP_DOWN;
 	while (index < items.length) {
 		ToolItem item = items [index];
-		if (item != null && (item.style & mask) != 0) break;
+		if (item != null && (item.style & mask) != 0) break;		
 		index++;
 	}
 	if (index == items.length) {
 		OS.SendMessage (handle, OS.TB_SETBITMAPSIZE, 0, 0);
 		OS.SendMessage (handle, OS.TB_SETBUTTONSIZE, 0, 0);
 	}
+	layoutItems ();
 }
 
 void setHotImageList (ImageList imageList) {
@@ -642,7 +881,9 @@ void setHotImageList (ImageList imageList) {
 	if ((hotImageList = imageList) != null) {
 		hImageList = hotImageList.getHandle ();
 	}
+	setDropDownItems (false);
 	OS.SendMessage (handle, OS.TB_SETHOTIMAGELIST, 0, hImageList);
+	setDropDownItems (true);
 }
 
 void setImageList (ImageList imageList) {
@@ -651,7 +892,58 @@ void setImageList (ImageList imageList) {
 	if ((this.imageList = imageList) != null) {
 		hImageList = imageList.getHandle ();
 	}
+	setDropDownItems (false);
 	OS.SendMessage (handle, OS.TB_SETIMAGELIST, 0, hImageList);
+	setDropDownItems (true);
+}
+
+public boolean setParent (Composite parent) {
+	checkWidget ();
+	if (!super.setParent (parent)) return false;
+	OS.SendMessage (handle, OS.TB_SETPARENT, parent.handle, 0);
+	return true;
+}
+
+public void setRedraw (boolean redraw) {
+	checkWidget ();
+	setDropDownItems (false);
+	super.setRedraw (redraw);
+	setDropDownItems (true);
+}
+
+void setRowCount (int count) {
+	if ((style & SWT.VERTICAL) != 0) {
+		/*
+		* Feature in Windows.  When the TB_SETROWS is used to set the
+		* number of rows in a tool bar, the tool bar is resized to show
+		* the items.  This is unexpected.  The fix is to save and restore
+		* the current size of the tool bar.
+		*/
+		RECT rect = new RECT ();
+		OS.GetWindowRect (handle, rect);
+		OS.MapWindowPoints (0, parent.handle, rect, 2);
+		ignoreResize = true;
+		/*
+		* Feature in Windows.  When the last button in a tool bar has the
+		* style BTNS_SEP and TB_SETROWS is used to set the number of rows
+		* in the tool bar, depending on the number of buttons, the toolbar
+		* will wrap items with the style BTNS_CHECK, even when the fLarger
+		* flags is used to force the number of rows to be larger than the
+		* number of items.  The fix is to set the number of rows to be two
+		* larger than the actual number of rows in the tool bar.  When items
+		* are being added, as long as the number of rows is at least one
+		* item larger than the count, the tool bar is laid out properly.
+		* When items are being removed, setting the number of rows to be
+		* one more than the item count has no effect.  The number of rows
+		* is already one more causing TB_SETROWS to do nothing.  Therefore,
+		* choosing two instead of one as the row increment fixes both cases.
+		*/
+		count += 2;
+		OS.SendMessage (handle, OS.TB_SETROWS, (1 << 16) | count, 0);
+		int flags = OS.SWP_NOACTIVATE | OS.SWP_NOMOVE | OS.SWP_NOZORDER;
+		SetWindowPos (handle, 0, 0, 0, rect.right - rect.left, rect.bottom - rect.top, flags);
+		ignoreResize = false;
+	}
 }
 
 boolean setTabItemFocus () {
@@ -671,6 +963,14 @@ String toolTipText (NMTTDISPINFO hdr) {
 	if ((hdr.uFlags & OS.TTF_IDISHWND) != 0) {
 		return null;
 	}
+	/*
+	* Bug in Windows.  On Windows XP, when TB_SETHOTITEM is
+	* used to set the hot item, the tool bar control attempts
+	* to display the tool tip, even when the cursor is not in
+	* the hot item.  The fix is to detect this case and fail to
+	* provide the string, causing no tool tip to be displayed.
+	*/
+	if (!hasCursor ()) return ""; //$NON-NLS-1$
 	int index = hdr.idFrom;
 	int hwndToolTip = OS.SendMessage (handle, OS.TB_GETTOOLTIPS, 0, 0);
 	if (hwndToolTip == hdr.hwndFrom) {
@@ -684,11 +984,27 @@ String toolTipText (NMTTDISPINFO hdr) {
 }
 
 int widgetStyle () {
-	int bits = super.widgetStyle () | OS.CCS_NORESIZE | OS.TBSTYLE_TOOLTIPS;
+	int bits = super.widgetStyle () | OS.CCS_NORESIZE | OS.TBSTYLE_TOOLTIPS | OS.TBSTYLE_CUSTOMERASE;
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) bits |= OS.TBSTYLE_TRANSPARENT;
 	if ((style & SWT.SHADOW_OUT) == 0) bits |= OS.CCS_NODIVIDER;
 	if ((style & SWT.WRAP) != 0) bits |= OS.TBSTYLE_WRAPABLE;
 	if ((style & SWT.FLAT) != 0) bits |= OS.TBSTYLE_FLAT;
-	if ((style & SWT.RIGHT) != 0) bits |= OS.TBSTYLE_LIST;
+	/*
+	* Feature in Windows.  When a tool bar has the style
+	* TBSTYLE_LIST and has a drop down item, Window leaves
+	* too much padding around the button.  This affects
+	* every button in the tool bar and makes the preferred
+	* height too big.  The fix is to set the TBSTYLE_LIST
+	* when the tool bar contains both text and images.
+	* 
+	* NOTE: Tool bars with CCS_VERT must have TBSTYLE_LIST
+	* set before any item is added or the tool bar does
+	* not lay out properly.  The work around does not run
+	* in this case.
+	*/
+	if (OS.COMCTL32_MAJOR < 6 || !OS.IsAppThemed ()) {
+		if ((style & SWT.RIGHT) != 0) bits |= OS.TBSTYLE_LIST;
+	}
 	return bits;
 }
 
@@ -700,15 +1016,45 @@ int windowProc () {
 	return ToolBarProc;
 }
 
-LRESULT WM_GETDLGCODE (int wParam, int lParam) {
-	LRESULT result = super.WM_GETDLGCODE (wParam, lParam);
-	/*
-	* Return DLGC_BUTTON so that mnemonics will be
-	* processed without needing to press the ALT key
-	* when the widget has focus.
-	*/
+LRESULT WM_CAPTURECHANGED (int wParam, int lParam) {
+	LRESULT result = super.WM_CAPTURECHANGED (wParam, lParam);
 	if (result != null) return result;
-	return new LRESULT (OS.DLGC_BUTTON);
+	/*
+	* Bug in Windows.  When the tool bar loses capture while an
+	* item is pressed in WM_LBUTTONDOWN, the item remains pressed.
+	* The fix is unpress the item using TB_SETSTATE.
+	*/
+	if (OS.GetKeyState (OS.VK_LBUTTON) < 0) {
+		for (int i=0; i<items.length; i++) {
+			ToolItem item = items [i];
+			if (item != null) {
+				int fsState = OS.SendMessage (handle, OS.TB_GETSTATE, item.id, 0);
+				if ((fsState & OS.TBSTATE_PRESSED) != 0) {
+					fsState &= ~OS.TBSTATE_PRESSED;
+					OS.SendMessage (handle, OS.TB_SETSTATE, item.id, fsState);
+				}
+			}
+		}
+	}
+	return null;
+}
+
+LRESULT WM_CHAR (int wParam, int lParam) {
+	LRESULT result = super.WM_CHAR (wParam, lParam);
+	if (result != null) return result;
+	switch (wParam) {
+		case ' ':
+			int index = OS.SendMessage (handle, OS.TB_GETHOTITEM, 0, 0);
+			if (index != -1) {
+				TBBUTTON lpButton = new TBBUTTON ();
+				int code = OS.SendMessage (handle, OS.TB_GETBUTTON, index, lpButton);
+				if (code != 0) {
+					items [lpButton.idCommand].click (false);
+					return LRESULT.ZERO;
+				}
+			}
+	}
+	return result;
 }
 
 LRESULT WM_COMMAND (int wParam, int lParam) {
@@ -735,21 +1081,45 @@ LRESULT WM_COMMAND (int wParam, int lParam) {
 	return LRESULT.ZERO;
 }
 
+LRESULT WM_ERASEBKGND (int wParam, int lParam) {
+	LRESULT result = super.WM_ERASEBKGND (wParam, lParam);
+	/*
+	* Bug in Windows.  For some reason, NM_CUSTOMDRAW with
+	* CDDS_PREERASE and CDDS_POSTERASE is never sent for
+	* versions of Windows earlier than XP.  The fix is to
+	* draw the background in WM_ERASEBKGND;
+	*/
+	if (findBackgroundControl () != null) {
+		if (OS.COMCTL32_MAJOR < 6) {
+			drawBackground (wParam);
+			return LRESULT.ONE;
+		}
+	}
+	return result;
+}
+
+LRESULT WM_GETDLGCODE (int wParam, int lParam) {
+	LRESULT result = super.WM_GETDLGCODE (wParam, lParam);
+	/*
+	* Return DLGC_BUTTON so that mnemonics will be
+	* processed without needing to press the ALT key
+	* when the widget has focus.
+	*/
+	if (result != null) return result;
+	return new LRESULT (OS.DLGC_BUTTON);
+}
+
 LRESULT WM_KEYDOWN (int wParam, int lParam) {
 	LRESULT result = super.WM_KEYDOWN (wParam, lParam);
 	if (result != null) return result;
 	switch (wParam) {
-		case OS.VK_RETURN:
-		case OS.VK_SPACE:
-			int index = OS.SendMessage (handle, OS.TB_GETHOTITEM, 0, 0);
-			if (index != -1) {
-				TBBUTTON lpButton = new TBBUTTON ();
-				int code = OS.SendMessage (handle, OS.TB_GETBUTTON, index, lpButton);
-				if (code != 0) {
-					items [lpButton.idCommand].click (wParam == OS.VK_RETURN);
-					return LRESULT.ZERO;
-				}
-			}
+		case OS.VK_SPACE:	
+			/*
+			* Ensure that the window proc does not process VK_SPACE
+			* so that it can be handled in WM_CHAR.  This allows the
+			* application the opportunity to cancel the operation.
+			*/
+			return LRESULT.ZERO;
 	}
 	return result;
 }
@@ -762,6 +1132,16 @@ LRESULT WM_KILLFOCUS (int wParam, int lParam) {
 	return super.WM_KILLFOCUS (wParam, lParam);
 }
 
+LRESULT WM_LBUTTONDOWN (int wParam, int lParam) {
+	if (ignoreMouse) return null;
+	return super.WM_LBUTTONDOWN (wParam, lParam);
+}
+
+LRESULT WM_LBUTTONUP (int wParam, int lParam) {
+	if (ignoreMouse) return null;
+	return super.WM_LBUTTONUP (wParam, lParam);
+}
+
 LRESULT WM_NOTIFY (int wParam, int lParam) {
 	/*
 	* Feature in Windows.  When the toolbar window
@@ -797,66 +1177,42 @@ LRESULT WM_SETFOCUS (int wParam, int lParam) {
 
 LRESULT WM_SIZE (int wParam, int lParam) {
 	if (ignoreResize) {
-		int code = callWindowProc (OS.WM_SIZE, wParam, lParam);
+		int code = callWindowProc (handle, OS.WM_SIZE, wParam, lParam);
 		if (code == 0) return LRESULT.ZERO;
 		return new LRESULT (code);
 	}
-	/*
-	* Feature in Windows.  When a tool bar that contains
-	* separators is wrapped, under certain circumstances,
-	* Windows redraws the entire tool bar unnecessarily
-	* when resized no item is moves.  Whether the entire
-	* toolbar is damaged or not seems to depend on the
-	* size of the tool bar and the position of the separators.
-	* The fix is to ensure that the newly exposed areas are
-	* always damaged, and avoid the redraw when no tool item
-	* moves.
-	*/
-	RECT [] rects = null;
-	int rgn = 0, oldCount = 0;
-	boolean fixRedraw = drawCount == 0 &&
-		(style & SWT.WRAP) != 0 &&
-		OS.IsWindowVisible (handle) &&
-		OS.SendMessage (handle, OS.TB_GETROWS, 0, 0) != 1;
-	if (fixRedraw) {
-		rgn = OS.CreateRectRgn (0, 0, 0, 0);
-		OS.GetUpdateRgn (handle, rgn, false);
-		oldCount = OS.SendMessage (handle, OS.TB_BUTTONCOUNT, 0, 0);
-		rects = new RECT [oldCount];
-		for (int i=0; i<oldCount; i++) {
-			rects [i] = new RECT ();
-			OS.SendMessage (handle, OS.TB_GETITEMRECT, i, rects [i]);
-		}
-	}
-	
 	LRESULT result = super.WM_SIZE (wParam, lParam);
-	/*
-	* It is possible (but unlikely), that application
-	* code could have disposed the widget in the resize
-	* event.  If this happens, end the processing of the
-	* Windows message by returning the result of the
-	* WM_SIZE message.
-	*/
 	if (isDisposed ()) return result;
-	
-	if (fixRedraw) {
-		int newCount = OS.SendMessage (handle, OS.TB_BUTTONCOUNT, 0, 0);
-		if (newCount == oldCount) {
-			int index = 0;
-			RECT rect = new RECT ();
-			while (index < newCount) {
-				OS.SendMessage (handle, OS.TB_GETITEMRECT, index, rect);
-				if (!OS.EqualRect (rects [index], rect)) break;
-				index++;
-			}
-			if (index == newCount) {
-				OS.ValidateRect (handle, null);
-				OS.InvalidateRgn (handle, rgn, false);
-			}
+	/*
+	* Bug in Windows.  The code in Windows that determines
+	* when tool items should wrap seems to use the window
+	* bounds rather than the client area.  Unfortunately,
+	* tool bars with the style TBSTYLE_EX_HIDECLIPPEDBUTTONS
+	* use the client area.  This means that buttons which
+	* overlap the border are hidden before they are wrapped.
+	* The fix is to compute TBSTYLE_EX_HIDECLIPPEDBUTTONS
+	* and set it each time the tool bar is resized.
+	*/
+	if ((style & SWT.BORDER) != 0 && (style & SWT.WRAP) != 0) {
+		RECT windowRect = new RECT ();
+		OS.GetWindowRect (handle, windowRect);
+		int index = 0, border = getBorderWidth () * 2; 
+		RECT rect = new RECT ();
+		int count = OS.SendMessage (handle, OS.TB_BUTTONCOUNT, 0, 0);
+		while (index < count) {
+			OS.SendMessage (handle, OS.TB_GETITEMRECT, index, rect);
+			OS.MapWindowPoints (handle, 0, rect, 2);
+			if (rect.right > windowRect.right - border * 2) break;
+			index++;
 		}
-		OS.DeleteObject (rgn);
+		int bits = OS.SendMessage (handle, OS.TB_GETEXTENDEDSTYLE, 0, 0);
+		if (index == count) {
+			bits |= OS.TBSTYLE_EX_HIDECLIPPEDBUTTONS;
+		} else {
+			bits &= ~OS.TBSTYLE_EX_HIDECLIPPEDBUTTONS;
+		}
+		OS.SendMessage (handle, OS.TB_SETEXTENDEDSTYLE, 0, bits);
 	}
-
 	layoutItems ();
 	return result;
 }
@@ -864,16 +1220,17 @@ LRESULT WM_SIZE (int wParam, int lParam) {
 LRESULT WM_WINDOWPOSCHANGING (int wParam, int lParam) {
 	LRESULT result = super.WM_WINDOWPOSCHANGING (wParam, lParam);
 	if (result != null) return result;
+	if (ignoreResize) return result;
 	/*
-	* Feature in Windows.  When a tool bar that contains
-	* separators is wrapped, under certain circumstances,
-	* Windows redraws the entire tool bar unnecessarily
-	* when resized no item is moves.  Whether the entire
-	* toolbar is damaged or not seems to depend on the
-	* size of the tool bar and the position of the separators.
-	* The fix is to ensure that the newly exposed areas are
-	* always damaged, and avoid the redraw when no tool item
-	* moves.
+	* Bug in Windows.  When a flat tool bar is wrapped,
+	* Windows draws a horizontal separator between the
+	* rows.  The tool bar does not draw the first or
+	* the last two pixels of this separator.  When the
+	* toolbar is resized to be bigger, only the new
+	* area is drawn and the last two pixels, which are
+	* blank are drawn over by separator.  This leaves
+	* garbage on the screen.  The fix is to damage the
+	* pixels.
 	*/
 	if (drawCount != 0) return result;
 	if ((style & SWT.WRAP) == 0) return result;
@@ -892,31 +1249,12 @@ LRESULT WM_WINDOWPOSCHANGING (int wParam, int lParam) {
 	OS.SetRect (newRect, 0, 0, lpwp.cx, lpwp.cy);
 	OS.SendMessage (handle, OS.WM_NCCALCSIZE, 0, newRect);
 	int oldWidth = oldRect.right - oldRect.left;
-	int oldHeight = oldRect.bottom - oldRect.top;
 	int newWidth = newRect.right - newRect.left;
-	int newHeight = newRect.bottom - newRect.top;
 	if (newWidth > oldWidth) {
-		/*
-		* Bug in Windows.  When a flat tool bar is wrapped,
-		* Windows draws a horizontal separator between the
-		* rows.  The tool bar does not draw the first or
-		* the last two pixels of this separator.  When the
-		* toolbar is resized to be bigger, only the new
-		* area is drawn and the last two pixels, which are
-		* blank are drawn over by separator.  This leaves
-		* garbage on the screen.  The fix is to damage the
-		* pixels.
-		*/
 		RECT rect = new RECT ();
+		int newHeight = newRect.bottom - newRect.top;
 		OS.SetRect (rect, oldWidth - 2, 0, oldWidth, newHeight);
 		OS.InvalidateRect (handle, rect, false);
-		OS.SetRect (rect, oldRect.right, newRect.top, newRect.right, newRect.bottom);
-		OS.InvalidateRect (handle, rect, true);
-	}
-	if (newHeight > oldHeight) {
-		RECT rect = new RECT ();
-		OS.SetRect (rect, newRect.left, oldRect.bottom, newRect.right, newRect.bottom);
-		OS.InvalidateRect (handle, rect, true);
 	}
 	return result;
 }
@@ -944,7 +1282,56 @@ LRESULT wmNotifyChild (int wParam, int lParam) {
 				event.x = rect.left;
 				event.y = rect.bottom;
 				child.postEvent (SWT.Selection, event);
-				return null;
+			}
+			break;
+		case OS.NM_CUSTOMDRAW:
+			if (OS.COMCTL32_MAJOR < 6) break;
+			/*
+			* Bug in Windows.  For some reason, under the XP Silver
+			* theme, tool bars continue to draw using the gray color
+			* from the default Blue theme.  The fix is to draw the
+			* background.
+			*/
+			NMCUSTOMDRAW nmcd = new NMCUSTOMDRAW ();
+			OS.MoveMemory (nmcd, lParam, NMCUSTOMDRAW.sizeof);
+//			if (drawCount != 0 || !OS.IsWindowVisible (handle)) {
+//				if (!OS.IsWinCE && OS.WindowFromDC (nmcd.hdc) == handle) break;
+//			}
+			switch (nmcd.dwDrawStage) {
+				case OS.CDDS_PREERASE: {
+					/*
+					* Bug in Windows.  When a tool bar does not have the style
+					* TBSTYLE_FLAT, the rectangle to be erased in CDDS_PREERASE
+					* is empty.  The fix is to draw the whole client area.
+					*/
+					int bits = OS.GetWindowLong (handle, OS.GWL_STYLE);
+					if ((bits & OS.TBSTYLE_FLAT) == 0) {
+						drawBackground (nmcd.hdc);
+					} else {
+						RECT rect = new RECT ();
+						OS.SetRect (rect, nmcd.left, nmcd.top, nmcd.right, nmcd.bottom);
+						drawBackground (nmcd.hdc, rect);
+					}
+					return new LRESULT (OS.CDRF_SKIPDEFAULT);
+				}
+			}
+			break;
+		case OS.TBN_HOTITEMCHANGE:
+			if (!OS.IsWinCE) {
+				NMTBHOTITEM lpnmhi = new NMTBHOTITEM ();
+				OS.MoveMemory (lpnmhi, lParam, NMTBHOTITEM.sizeof);
+				switch (lpnmhi.dwFlags) {
+					case OS.HICF_ARROWKEYS:
+			        	RECT client = new RECT ();
+			        	OS.GetClientRect (handle, client);
+			        	int index = OS.SendMessage (handle, OS.TB_COMMANDTOINDEX, lpnmhi.idNew, 0);
+			        	RECT rect = new RECT ();
+			        	OS.SendMessage (handle, OS.TB_GETITEMRECT, index, rect);
+						if (rect.right > client.right || rect.bottom > client.bottom) {
+							return LRESULT.ONE;
+						}
+						break;
+				}
 			}
 			break;
 	}
