@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -18,13 +18,13 @@ import org.eclipse.swt.events.*;
 
 /**
  * Instances of this class represent a column in a tree widget.
- *  <dl>
+ * <p><dl>
  * <dt><b>Styles:</b></dt>
  * <dd>LEFT, RIGHT, CENTER</dd>
  * <dt><b>Events:</b></dt>
  * <dd> Move, Resize, Selection</dd>
  * </dl>
- * <p>
+ * </p><p>
  * Note: Only one of the styles LEFT, RIGHT and CENTER may be specified.
  * </p><p>
  * IMPORTANT: This class is <em>not</em> intended to be subclassed.
@@ -34,7 +34,9 @@ import org.eclipse.swt.events.*;
  */
 public class TreeColumn extends Item {
 	Tree parent;
-	boolean resizable;
+	boolean resizable, moveable;
+	String toolTipText;
+	int id;
 
 /**
  * Constructs a new instance of this class given its parent
@@ -92,10 +94,11 @@ public TreeColumn (Tree parent, int style) {
  *
  * @param parent a composite control which will be the parent of the new instance (cannot be null)
  * @param style the style of control to construct
- * @param index the index to store the receiver in its parent
+ * @param index the zero-relative index to store the receiver in its parent
  *
  * @exception IllegalArgumentException <ul>
  *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
+ *    <li>ERROR_INVALID_RANGE - if the index is not between 0 and the number of elements in the parent (inclusive)</li>
  * </ul>
  * @exception SWTException <ul>
  *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
@@ -182,6 +185,11 @@ protected void checkSubclass () {
 	if (!isValidSubclass ()) error (SWT.ERROR_INVALID_SUBCLASS);
 }
 
+void destroyWidget () {
+	parent.destroyItem (this);
+	releaseHandle ();
+}
+
 /**
  * Returns a value which describes the position of the
  * text or image in the receiver. The value will be one of
@@ -202,6 +210,31 @@ public int getAlignment () {
 	return SWT.LEFT;
 }
 
+/**
+ * Gets the moveable attribute. A column that is
+ * not moveable cannot be reordered by the user 
+ * by dragging the header but may be reordered 
+ * by the programmer.
+ *
+ * @return the moveable attribute
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @see Tree#getColumnOrder()
+ * @see Tree#setColumnOrder(int[])
+ * @see TreeColumn#setMoveable(boolean)
+ * @see SWT#Move
+ * 
+ * @since 3.2
+ */
+public boolean getMoveable () {
+	checkWidget ();
+	return moveable;
+}
+
 String getNameText () {
 	return getText ();
 }
@@ -238,6 +271,24 @@ public boolean getResizable () {
 	return resizable;
 }
 
+/**
+ * Returns the receiver's tool tip text, or null if it has
+ * not been set.
+ *
+ * @return the receiver's tool tip text
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.2
+ */
+public String getToolTipText () {
+	checkWidget();
+	return toolTipText;
+}
+
 /**
  * Gets the width of the receiver.
  *
@@ -276,68 +327,95 @@ public void pack () {
 	int index = parent.indexOf (this);
 	if (index == -1) return;
 	int columnWidth = 0;
-	int hwnd = parent.handle;
+	int hwnd = parent.handle, hwndHeader = parent.hwndHeader;
+	RECT headerRect = new RECT ();
+	OS.SendMessage (hwndHeader, OS.HDM_GETITEMRECT, index, headerRect);
 	int hDC = OS.GetDC (hwnd);
 	int oldFont = 0, newFont = OS.SendMessage (hwnd, OS.WM_GETFONT, 0, 0);
 	if (newFont != 0) oldFont = OS.SelectObject (hDC, newFont);
-	int cp = parent.getCodePage ();		
+	TVITEM tvItem = new TVITEM ();
+	tvItem.mask = OS.TVIF_HANDLE | OS.TVIF_PARAM;
+	tvItem.hItem = OS.SendMessage (hwnd, OS.TVM_GETNEXTITEM, OS.TVGN_ROOT, 0);
+	while (tvItem.hItem != 0) {
+		OS.SendMessage (hwnd, OS.TVM_GETITEM, 0, tvItem);
+		TreeItem item = tvItem.lParam != -1 ? parent.items [tvItem.lParam] : null;
+		if (item != null) {
+			int hFont = item.cellFont != null ? item.cellFont [index] : -1;
+			if (hFont == -1) hFont = item.font;
+			if (hFont != -1) hFont = OS.SelectObject (hDC, hFont);
+			RECT itemRect = item.getBounds (index, true, true, false, false, false, hDC);
+			if (hFont != -1) OS.SelectObject (hDC, hFont);
+			if (parent.hooks (SWT.MeasureItem)) {
+				int nSavedDC = OS.SaveDC (hDC);
+				GCData data = new GCData ();
+				data.device = display;
+				data.hFont = hFont;
+				GC gc = GC.win32_new (hDC, data);
+				Event event = new Event ();
+				event.item = item;
+				event.gc = gc;
+				event.index = index;
+				event.x = itemRect.left;
+				event.y = itemRect.top;
+				event.width = itemRect.right - itemRect.left;
+				event.height = itemRect.bottom - itemRect.top;
+				parent.sendEvent (SWT.MeasureItem, event);
+				event.gc = null;
+				gc.dispose ();
+				OS.RestoreDC (hDC, nSavedDC);
+				if (isDisposed () || parent.isDisposed ()) break;
+				if (event.height > parent.getItemHeight ()) parent.setItemHeight (event.height);
+				//itemRect.left = event.x;
+				itemRect.right = event.x + event.width;
+			}
+			columnWidth = Math.max (columnWidth, itemRect.right - headerRect.left);
+		}
+		tvItem.hItem = OS.SendMessage (hwnd, OS.TVM_GETNEXTITEM, OS.TVGN_NEXTVISIBLE, tvItem.hItem);
+	}
 	RECT rect = new RECT ();
 	int flags = OS.DT_CALCRECT | OS.DT_NOPREFIX;
-	TVITEM tvItem = new TVITEM ();
-	tvItem.mask = OS.TVIF_PARAM;
-	int hItem = OS.SendMessage (hwnd, OS.TVM_GETNEXTITEM, OS.TVGN_ROOT, 0);
-	while (hItem != 0) {
-		hItem = OS.SendMessage (hwnd, OS.TVM_GETNEXTITEM, OS.TVGN_NEXTVISIBLE, hItem);
-		tvItem.hItem = hItem;
-		OS.SendMessage (hwnd, OS.TVM_GETITEM, 0, tvItem);
-		TreeItem item = parent.items [tvItem.lParam];
-		if (index == 0) {
-			rect.left = item.handle;
-			if (OS.SendMessage (hwnd, OS.TVM_GETITEMRECT, 1, rect) != 0) {
-				columnWidth = Math.max (columnWidth, rect.right);
-			}
-		} else {
-			int imageWidth = 0, textWidth = 0;
-			Image image = item.images != null ? item.images [index] : null;
-			if (image != null) {
-				Rectangle bounds = image.getBounds ();
-				imageWidth = bounds.width;
-			}
-			String string = item.strings != null ? item.strings [index] : null;
-			if (string != null) {
-				TCHAR buffer = new TCHAR (cp, string, false);
-				OS.DrawText (hDC, buffer, buffer.length (), rect, flags);
-				textWidth = rect.right - rect.left;
-			}
-			columnWidth = Math.max (columnWidth, imageWidth + textWidth + Tree.INSET * 3);
-		}
-	}
-	TCHAR buffer = new TCHAR (cp, text, true);
+	TCHAR buffer = new TCHAR (parent.getCodePage (), text, false);
 	OS.DrawText (hDC, buffer, buffer.length (), rect, flags);
 	int headerWidth = rect.right - rect.left + Tree.HEADER_MARGIN;
-	if (image != null) {
-		int margin = 0, hwndHeader = parent.hwndHeader;
+	if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) headerWidth += Tree.HEADER_EXTRA;
+	if (image != null || parent.sortColumn == this) {
+		Image headerImage = null;
+		if (parent.sortColumn == this && parent.sortDirection != SWT.NULL) {
+			if (OS.COMCTL32_MAJOR < 6) {
+				headerImage = display.getSortImage (parent.sortDirection);
+			} else {
+				headerWidth += Tree.SORT_WIDTH;
+			}
+		} else {
+			headerImage = image;
+		}
+		if (headerImage != null) {
+			Rectangle bounds = headerImage.getBounds ();
+			headerWidth += bounds.width;
+		}
+		int margin = 0;
 		if (hwndHeader != 0 && OS.COMCTL32_VERSION >= OS.VERSION (5, 80)) {
 			margin = OS.SendMessage (hwndHeader, OS.HDM_GETBITMAPMARGIN, 0, 0);
 		} else {
 			margin = OS.GetSystemMetrics (OS.SM_CXEDGE) * 3;
 		}
-		Rectangle bounds = image.getBounds ();
-		headerWidth += bounds.width + margin * 2;
+		headerWidth += margin * 2;
 	}
 	if (newFont != 0) OS.SelectObject (hDC, oldFont);
 	OS.ReleaseDC (hwnd, hDC);
 	setWidth (Math.max (headerWidth, columnWidth));
 }
 
-void releaseChild () {
-	super.releaseChild ();
-	parent.destroyItem (this);
+void releaseHandle () {
+	super.releaseHandle ();
+	parent = null;
 }
 
-void releaseWidget () {
-	super.releaseWidget ();
-	parent = null;
+void releaseParent () {
+	super.releaseParent ();
+	if (parent.sortColumn == this) {
+		parent.sortColumn = null;
+	}
 }
 
 /**
@@ -421,11 +499,12 @@ public void setAlignment (int alignment) {
 	OS.SendMessage (hwndHeader, OS.HDM_SETITEM, index, hdItem);
 	if (index != 0) {
 		int hwnd = parent.handle;
-		RECT rect = new RECT (), itemRect = new RECT ();
+		parent.forceResize ();
+		RECT rect = new RECT (), headerRect = new RECT ();
 		OS.GetClientRect (hwnd, rect);
-		OS.SendMessage (hwndHeader, OS.HDM_GETITEMRECT, index, itemRect);
-		rect.left = itemRect.left;
-		rect.right = itemRect.right;
+		OS.SendMessage (hwndHeader, OS.HDM_GETITEMRECT, index, headerRect);
+		rect.left = headerRect.left;
+		rect.right = headerRect.right;
 		OS.InvalidateRect (hwnd, rect, true);
 	}
 }
@@ -435,23 +514,66 @@ public void setImage (Image image) {
 	if (image != null && image.isDisposed ()) {
 		error (SWT.ERROR_INVALID_ARGUMENT);
 	}
+	super.setImage (image);
+	if (parent.sortColumn != this || parent.sortDirection != SWT.NULL) {
+		setImage (image, false, false);
+	}
+}
+
+void setImage (Image image, boolean sort, boolean right) {
 	int index = parent.indexOf (this);
 	if (index == -1) return;
-	super.setImage (image);
 	int hwndHeader = parent.hwndHeader;
 	if (hwndHeader == 0) return;
 	HDITEM hdItem = new HDITEM ();
-	hdItem.mask = OS.HDI_FORMAT | OS.HDI_IMAGE;
+	hdItem.mask = OS.HDI_FORMAT | OS.HDI_IMAGE | OS.HDI_BITMAP;
 	OS.SendMessage (hwndHeader, OS.HDM_GETITEM, index, hdItem);
+	hdItem.fmt &= ~OS.HDF_BITMAP_ON_RIGHT;
 	if (image != null) {
-		hdItem.fmt |= OS.LVCFMT_IMAGE;
-		hdItem.iImage = parent.imageIndex (image);
+		if (sort) {
+			hdItem.mask &= ~OS.HDI_IMAGE;
+			hdItem.fmt &= ~OS.HDF_IMAGE;
+			hdItem.fmt |= OS.HDF_BITMAP;
+			hdItem.hbm = image.handle;
+		} else {
+			hdItem.mask &= ~OS.HDI_BITMAP;
+			hdItem.fmt &= ~OS.HDF_BITMAP;
+			hdItem.fmt |= OS.HDF_IMAGE;
+			hdItem.iImage = parent.imageIndexHeader (image);
+		}
+		if (right) hdItem.fmt |= OS.HDF_BITMAP_ON_RIGHT;
 	} else {
-		hdItem.fmt &= ~OS.LVCFMT_IMAGE;
+		hdItem.fmt &= ~(OS.HDF_IMAGE | OS.HDF_BITMAP);
 	}
 	OS.SendMessage (hwndHeader, OS.HDM_SETITEM, index, hdItem);
 }
 
+/**
+ * Sets the moveable attribute.  A column that is
+ * moveable can be reordered by the user by dragging
+ * the header. A column that is not moveable cannot be 
+ * dragged by the user but may be reordered 
+ * by the programmer.
+ *
+ * @param moveable the moveable attribute
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @see Tree#setColumnOrder(int[])
+ * @see Tree#getColumnOrder()
+ * @see TreeColumn#getMoveable()
+ * @see SWT#Move
+ * 
+ * @since 3.2
+ */
+public void setMoveable (boolean moveable) {
+	checkWidget ();
+	this.moveable = moveable;
+}
+
 /**
  * Sets the resizable attribute.  A column that is
  * not resizable cannot be dragged by the user but
@@ -469,9 +591,63 @@ public void setResizable (boolean resizable) {
 	this.resizable = resizable;
 }
 
+void setSortDirection (int direction) {
+	if (OS.COMCTL32_MAJOR >= 6) {
+		int hwndHeader = parent.hwndHeader;
+		if (hwndHeader != 0) {
+			int index = parent.indexOf (this);
+			if (index == -1) return;
+			HDITEM hdItem = new HDITEM ();
+			hdItem.mask = OS.HDI_FORMAT | OS.HDI_IMAGE;
+			OS.SendMessage (hwndHeader, OS.HDM_GETITEM, index, hdItem);
+			switch (direction) {
+				case SWT.UP:
+					hdItem.fmt &= ~(OS.HDF_IMAGE | OS.HDF_SORTDOWN);
+					hdItem.fmt |= OS.HDF_SORTUP;
+					break;
+				case SWT.DOWN:
+					hdItem.fmt &= ~(OS.HDF_IMAGE | OS.HDF_SORTUP);
+					hdItem.fmt |= OS.HDF_SORTDOWN;
+					break;
+				case SWT.NONE:
+					hdItem.fmt &= ~(OS.HDF_SORTUP | OS.HDF_SORTDOWN);
+					if (image != null) {
+						hdItem.fmt |= OS.HDF_IMAGE;
+						hdItem.iImage = parent.imageIndexHeader (image);
+					} else {
+						hdItem.fmt &= ~OS.HDF_IMAGE;
+					}
+					break;
+			}
+			OS.SendMessage (hwndHeader, OS.HDM_SETITEM, index, hdItem);
+			if (OS.COMCTL32_MAJOR >= 6 && OS.IsAppThemed ()) {
+				int hwnd = parent.handle;
+				parent.forceResize ();
+				RECT rect = new RECT (), headerRect = new RECT ();
+				OS.GetClientRect (hwnd, rect);
+				OS.SendMessage (hwndHeader, OS.HDM_GETITEMRECT, index, headerRect);
+				rect.left = headerRect.left;
+				rect.right = headerRect.right;
+				OS.InvalidateRect (hwnd, rect, true);
+			}
+		}
+	} else {
+		switch (direction) {
+			case SWT.UP:
+			case SWT.DOWN:
+				setImage (display.getSortImage (direction), true, true);
+				break;
+			case SWT.NONE:
+				setImage (image, false, false);
+				break;
+		}
+	}
+}
+
 public void setText (String string) {
 	checkWidget ();
 	if (string == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (string.equals (text)) return;
 	int index = parent.indexOf (this);
 	if (index == -1) return;
 	super.setText (string);
@@ -498,6 +674,29 @@ public void setText (String string) {
 	if (result == 0) error (SWT.ERROR_CANNOT_SET_TEXT);
 }
 
+/**
+ * Sets the receiver's tool tip text to the argument, which
+ * may be null indicating that no tool tip text should be shown.
+ *
+ * @param string the new tool tip text (or null)
+ *
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.2
+ */
+public void setToolTipText (String string) {
+	checkWidget();
+	toolTipText = string;
+	int hwndHeaderToolTip = parent.headerToolTipHandle;
+	if (hwndHeaderToolTip == 0) {
+		parent.createHeaderToolTips ();
+		parent.updateHeaderToolTips ();
+	}
+}
+
 /**
  * Sets the width of the receiver.
  *
@@ -518,7 +717,33 @@ public void setWidth (int width) {
 	hdItem.mask = OS.HDI_WIDTH;
 	hdItem.cxy = width;
 	OS.SendMessage (hwndHeader, OS.HDM_SETITEM, index, hdItem);
+	RECT headerRect = new RECT ();
+	OS.SendMessage (hwndHeader, OS.HDM_GETITEMRECT, index, headerRect);
+	parent.forceResize ();
+	int hwnd = parent.handle;
+	RECT rect = new RECT ();
+	OS.GetClientRect (hwnd, rect);
+	rect.left = headerRect.left;
+	OS.InvalidateRect (hwnd, rect, true);
 	parent.setScrollWidth ();
 }
 
+void updateToolTip (int index) {
+	int hwndHeaderToolTip = parent.headerToolTipHandle;
+	if (hwndHeaderToolTip != 0) {
+		int hwndHeader = parent.hwndHeader;
+		RECT rect = new RECT ();
+		if (OS.SendMessage (hwndHeader, OS.HDM_GETITEMRECT, index, rect) != 0) {
+			TOOLINFO lpti = new TOOLINFO ();
+			lpti.cbSize = TOOLINFO.sizeof;
+			lpti.hwnd = hwndHeader;
+			lpti.uId = id;
+			lpti.left = rect.left;
+			lpti.top = rect.top;
+			lpti.right = rect.right;
+			lpti.bottom = rect.bottom;
+			OS.SendMessage (hwndHeaderToolTip, OS.TTM_NEWTOOLRECT, 0, lpti);
+		}
+	}
+}
 }
