@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -17,23 +17,10 @@ import java.util.Map;
 
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.ResourcesPlugin;
-import org.eclipse.core.runtime.IPath;
-import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.core.runtime.IStatus;
-import org.eclipse.core.runtime.MultiStatus;
-import org.eclipse.core.runtime.Path;
-import org.eclipse.team.internal.ccvs.core.CVSException;
-import org.eclipse.team.internal.ccvs.core.CVSProviderPlugin;
-import org.eclipse.team.internal.ccvs.core.CVSStatus;
-import org.eclipse.team.internal.ccvs.core.CVSTag;
-import org.eclipse.team.internal.ccvs.core.ICVSFolder;
-import org.eclipse.team.internal.ccvs.core.ICVSResource;
-import org.eclipse.team.internal.ccvs.core.ICVSRunnable;
-import org.eclipse.team.internal.ccvs.core.Policy;
+import org.eclipse.core.runtime.*;
+import org.eclipse.osgi.util.NLS;
+import org.eclipse.team.internal.ccvs.core.*;
 import org.eclipse.team.internal.ccvs.core.client.listeners.ICommandOutputListener;
-import org.eclipse.team.internal.ccvs.core.client.listeners.IConsoleListener;
-import org.eclipse.team.internal.ccvs.core.resources.CVSWorkspaceRoot;
 
 /**
  * Abstract base class for command requests.
@@ -43,9 +30,11 @@ public abstract class Command extends Request {
 	/*** Command singleton instances ***/
 	public final static Add ADD = new Add();
 	public final static Admin ADMIN = new Admin();
-	public final static Checkout CHECKOUT = new Checkout();
+	public final static Annotate ANNOTATE = new Annotate();
+	public final static Checkout CHECKOUT = new CheckoutWithOverwrite();
 	public final static Commit COMMIT = new Commit();
 	public final static Diff DIFF = new Diff();
+	public final static RDiff RDIFF = new RDiff();
 	public final static Editors EDITORS = new Editors();
 	public final static Import IMPORT = new Import();
 	public final static Log LOG = new Log();
@@ -57,6 +46,8 @@ public abstract class Command extends Request {
 	public final static Tag CUSTOM_TAG = new Tag(true);
 	public final static RTag RTAG = new RTag();
 	public final static Update UPDATE = new Update();
+	public final static Update REPLACE = new UpdateWithOverwrite();
+	public final static SyncUpdate SYNCUPDATE = new SyncUpdate();
 	public final static Version VERSION = new Version();
 	public final static NOOPCommand NOOP = new NOOPCommand();
 	
@@ -86,7 +77,8 @@ public abstract class Command extends Request {
 	/*** Local options: common to many commands ***/
 	// Empty local option array
 	public static final LocalOption[] NO_LOCAL_OPTIONS = new LocalOption[0];
-	// valid for: annotate checkout commit diff export log rdiff remove rtag status tag update  
+	// valid for: annotate checkout commit diff export log rdiff remove rtag status tag update
+    public static final LocalOption RECURSE = new LocalOption("-R"); //$NON-NLS-1$
 	public static final LocalOption DO_NOT_RECURSE = new LocalOption("-l"); //$NON-NLS-1$	
 	// valid for: checkout export update
 	public static final LocalOption PRUNE_EMPTY_DIRECTORIES = new LocalOption("-P"); //$NON-NLS-1$
@@ -255,14 +247,15 @@ public abstract class Command extends Request {
 
 	/**
 	 * Send an array of Resources.
+	 * @param localOptions 
 	 * 
 	 * @see Command#sendFileStructure(ICVSResource,IProgressMonitor,boolean,boolean,boolean)
 	 */
 	protected void sendFileStructure(Session session, ICVSResource[] resources,
-		boolean emptyFolders, IProgressMonitor monitor) throws CVSException {
+		LocalOption[] localOptions, boolean emptyFolders, IProgressMonitor monitor) throws CVSException {
 		checkResourcesManaged(resources);
 		
-		new FileStructureVisitor(session, emptyFolders, true, monitor).visit(session, resources);
+		new FileStructureVisitor(session, localOptions, emptyFolders, true).visit(session, resources, monitor);
 	}
 
 	/**
@@ -280,8 +273,8 @@ public abstract class Command extends Request {
 			else {
 				folder = resources[i].getParent();
 			}
-			if (folder==null || (!folder.isCVSFolder() && folder.exists())) {
-				throw new CVSException(Policy.bind("Command.argumentNotManaged", folder.getName()));//$NON-NLS-1$
+			if (!folder.isCVSFolder() && folder.exists()) {
+				throw new CVSException(NLS.bind(CVSMessages.Command_argumentNotManaged, new String[] { folder.getName() }));
 			}
 		}
 	}
@@ -314,7 +307,7 @@ public abstract class Command extends Request {
 				LocalOption[] lOptions = filterLocalOptions(session, gOptions, localOptions);
 				
 				// print the invocation string to the console
-				if (session.isOutputToConsole() || Policy.DEBUG_CVS_PROTOCOL) {
+				if (session.isOutputToConsole() || Policy.isDebugProtocol()) {
 					IPath commandRootPath;
 					IResource resource = session.getLocalRoot().getIResource();
 					if (resource == null) {
@@ -323,15 +316,13 @@ public abstract class Command extends Request {
 						commandRootPath = resource.getFullPath();
 					}
 					String line = constructCommandInvocationString(commandRootPath, gOptions, lOptions, arguments);
-					if (session.isOutputToConsole()) {
-						IConsoleListener consoleListener = CVSProviderPlugin.getPlugin().getConsoleListener();
-						if (consoleListener != null) consoleListener.commandInvoked(line);
-					}
-					if (Policy.DEBUG_CVS_PROTOCOL) System.out.println("CMD> " + line); //$NON-NLS-1$
+					ConsoleListeners.getInstance().commandInvoked(session, line);
+					if (Policy.isDebugProtocol()) Policy.printProtocolLine("CMD> " + line); //$NON-NLS-1$
 				}
 				
 				// run the command
 				try {
+				    session.setCurrentCommand(Command.this);
 					status[0] = doExecute(session, gOptions, lOptions, arguments, listener, monitor);
 					notifyConsoleOnCompletion(session, status[0], null);
 				} catch (CVSException e) {
@@ -343,18 +334,31 @@ public abstract class Command extends Request {
 				}
 			}
 		};
-		session.getLocalRoot().run(job, pm);
+		if (isWorkspaceModification()) {
+			session.getLocalRoot().run(job, pm);
+		} else {
+			job.run(pm);
+		}
 		return status[0];
 	}
 	
+	/**
+	 * Return whether this command modifies the workspace.
+	 * If <code>true</code> is returned, a scheduling rule on
+	 * the session local root is obtained. Otherwise, no
+	 * scheduling rule is obtained. By default, <code>true</code>
+	 * is returned
+	 * @return whether this command modifies the workspace
+	 */
+	protected boolean isWorkspaceModification() {
+		return true;
+	}
+
 	private void notifyConsoleOnCompletion(Session session, IStatus status, Exception exception) {
-		if (session.isOutputToConsole()) {
-			IConsoleListener consoleListener = CVSProviderPlugin.getPlugin().getConsoleListener();
-			if (consoleListener != null) consoleListener.commandCompleted(status, exception);
-		}
-		if (Policy.DEBUG_CVS_PROTOCOL) {
-			if (status != null) System.out.println("RESULT> " + status.toString()); //$NON-NLS-1$
-			else System.out.println("RESULT> " + exception.toString()); //$NON-NLS-1$
+		ConsoleListeners.getInstance().commandCompleted(session, status, exception);
+		if (Policy.isDebugProtocol()) {
+			if (status != null) Policy.printProtocolLine("RESULT> " + status.toString()); //$NON-NLS-1$
+			else Policy.printProtocolLine("RESULT> " + exception.toString()); //$NON-NLS-1$
 		}
 	}
 
@@ -364,7 +368,7 @@ public abstract class Command extends Request {
 		ICVSResource[] resources = null;
 		/*** setup progress monitor ***/
 		monitor = Policy.monitorFor(monitor);
-		monitor.beginTask(Policy.bind("Command.server"), 100); //$NON-NLS-1$
+		monitor.beginTask(null, 100);
 		Policy.checkCanceled(monitor);
 		try {
 			/*** prepare for command ***/
@@ -386,9 +390,9 @@ public abstract class Command extends Request {
 			// compute the work resources
 			resources = computeWorkResources(session, localOptions, arguments);			
 			Policy.checkCanceled(monitor);
-			// send local working directory state contributes 25% of work
+			// send local working directory state contributes 48% of work
 			resources = sendLocalResourceState(session, globalOptions, localOptions,
-				resources, Policy.subMonitorFor(monitor, 25));
+					resources, Policy.infiniteSubMonitorFor(monitor, 48));
 			Policy.checkCanceled(monitor);
 			// send arguments
 			sendArguments(session, arguments);
@@ -399,11 +403,11 @@ public abstract class Command extends Request {
 			if (listener == null) listener = getDefaultCommandOutputListener();
 
 			/*** execute command and process responses ***/
-			// Processing responses contributes 70% of work.
-			IStatus status = executeRequest(session, listener, Policy.subMonitorFor(monitor, 70));
+			// Processing responses contributes 50% of work.
+			IStatus status = executeRequest(session, listener, Policy.subMonitorFor(monitor, 50));
 
-			// Finished adds last 5% of work.
-			status = commandFinished(session, globalOptions, localOptions, resources, Policy.subMonitorFor(monitor, 5),
+			// Finished adds last 2% of work.
+			status = commandFinished(session, globalOptions, localOptions, resources, Policy.subMonitorFor(monitor, 2),
 				status);
 			return status;
 		} finally {			
@@ -444,7 +448,7 @@ public abstract class Command extends Request {
 				completePath = completePath.append(arguments[i]);
 			}
 			commandLine.append(completePath.toString());
-			commandLine.append('"'); //$NON-NLS-1$
+			commandLine.append('"'); 
 		}
 		return commandLine.toString();
 	}	
@@ -549,6 +553,28 @@ public abstract class Command extends Request {
 			session.sendArgument(option);
 			if (argument != null) session.sendArgument(argument);
 		}
+        public LocalOption[] addTo(LocalOption[] options) {
+            if (this.isElementOf(options)) {
+                return options;
+            }
+            LocalOption[] newOptions = new LocalOption[options.length + 1];
+            System.arraycopy(options, 0, newOptions, 0, options.length);
+            newOptions[options.length] = this;
+            return newOptions;
+        }
+        public LocalOption[] removeFrom(LocalOption[] options) {
+            if (!this.isElementOf(options)) {
+                return options;
+            }
+            List result = new ArrayList();
+            for (int i = 0; i < options.length; i++) {
+                Command.LocalOption option = options[i];
+                if (!option.equals(this)) {
+                    result.add(option);
+                }
+            }
+            return (LocalOption[]) result.toArray(new LocalOption[result.size()]);
+        }
 	}
 	/**
 	 * Options subtype for keyword substitution options.
@@ -583,7 +609,7 @@ public abstract class Command extends Request {
 		 */
 		public static KSubstOption fromFile(IFile file) {
 			if (CVSProviderPlugin.isText(file))
-				return CVSProviderPlugin.getPlugin().getDefaultTextKSubstOption();
+				return getDefaultTextMode();
 			return KSUBST_BINARY;
 		}
 		/**
@@ -593,12 +619,22 @@ public abstract class Command extends Request {
 			return (KSubstOption[]) ksubstOptionMap.values().toArray(new KSubstOption[ksubstOptionMap.size()]);
 		}
 		/**
-		 * Returns the entry line mode string for this instance.
+		 * Returns the entry line mode string for this instance. Note that it might return blank strings
+		 * for certain options. For UI, use {@link #toMode()} which will always return the a string
+		 * containing the keyword substitution.
 		 */
-		public String toMode() {
+		public String toEntryLineMode() {
 			if (KSUBST_TEXT_EXPAND.equals(this)) return ""; //$NON-NLS-1$
 			return getOption();
 		}
+		
+		/**
+		 * Returns the entry line mode string for this instance.
+		 */
+		public String toMode(){
+			return getOption();
+		}
+		
 		/**
 		 * Returns true if the substitution mode requires no data translation
 		 * during file transfer.
@@ -610,15 +646,47 @@ public abstract class Command extends Request {
 		 * Returns a short localized text string describing this mode.
 		 */
 		public String getShortDisplayText() {
-			if (isUnknownMode) return Policy.bind("KSubstOption.unknown.short", option); //$NON-NLS-1$
-			return Policy.bind("KSubstOption." + option + ".short"); //$NON-NLS-1$ //$NON-NLS-2$
+			if (isUnknownMode)
+                return NLS.bind(CVSMessages.KSubstOption_unknown_short, new String[] { option }); 
+            if (option.equals("-kb")) //$NON-NLS-1$
+                return CVSMessages.KSubstOption__kb_short;
+            if (option.equals("-kkv")) //$NON-NLS-1$
+                return CVSMessages.KSubstOption__kkv_short;
+            if (option.equals("-ko")) //$NON-NLS-1$
+                return CVSMessages.KSubstOption__ko_short;
+            if (option.equals("-kk")) //$NON-NLS-1$
+                return CVSMessages.KSubstOption__kk_short;
+            if (option.equals("-kv")) //$NON-NLS-1$
+                return CVSMessages.KSubstOption__kv_short;
+            if (option.equals("-kkvl")) //$NON-NLS-1$
+                return CVSMessages.KSubstOption__kkvl_short;
+            return NLS.bind(CVSMessages.KSubstOption_unknown_short, new String[] { option }); 
 		}
 		/**
 		 * Returns a long localized text string describing this mode.
 		 */
 		public String getLongDisplayText() {
-			if (isUnknownMode) return Policy.bind("KSubstOption.unknown.long", option); //$NON-NLS-1$
-			return Policy.bind("KSubstOption." + option + ".long"); //$NON-NLS-1$ //$NON-NLS-2$
+			if (isUnknownMode)
+                return NLS.bind(CVSMessages.KSubstOption_unknown_long, new String[] { option }); 
+            if (option.equals("-kb")) //$NON-NLS-1$
+                return CVSMessages.KSubstOption__kb_long;
+            if (option.equals("-kkv")) //$NON-NLS-1$
+                return CVSMessages.KSubstOption__kkv_long;
+            if (option.equals("-ko")) //$NON-NLS-1$
+                return CVSMessages.KSubstOption__ko_long;
+            if (option.equals("-kk")) //$NON-NLS-1$
+                return CVSMessages.KSubstOption__kk_long;
+            if (option.equals("-kv")) //$NON-NLS-1$
+                return CVSMessages.KSubstOption__kv_long;
+            if (option.equals("-kkvl")) //$NON-NLS-1$
+                return CVSMessages.KSubstOption__kkvl_long;
+            return NLS.bind(CVSMessages.KSubstOption_unknown_long, new String[] { option }); 
+		}
+		/**
+		 * Return the text mode that will be used by default
+		 */
+		public static KSubstOption getDefaultTextMode() {
+			return CVSProviderPlugin.getPlugin().getDefaultTextKSubstOption();
 		}
 	}
 
@@ -627,7 +695,10 @@ public abstract class Command extends Request {
 	 * Valid for: add commit import
 	 */
 	public static LocalOption makeArgumentOption(LocalOption option, String argument) {
-		return new LocalOption(option.getOption(), argument);  //$NON-NLS-1$
+		if(argument == null) {
+			argument = ""; //$NON-NLS-1$
+		}
+		return new LocalOption(option.getOption(), argument);  
 	}
 	
 	/**
@@ -644,7 +715,7 @@ public abstract class Command extends Request {
 				return new LocalOption("-D", tag.getName()); //$NON-NLS-1$
 			default:
 				// tag must not be HEAD
-				throw new IllegalArgumentException(Policy.bind("Command.invalidTag")); //$NON-NLS-1$
+				throw new IllegalArgumentException(CVSMessages.Command_invalidTag); 
 		}
 	}
 
@@ -705,42 +776,26 @@ public abstract class Command extends Request {
 	}
 	
 	/**
-	 * Execute a CVS command inside an ICVSRunnable passed to Session.run()
-	 * <p>
-	 * Dispatches the commands, retrieves the results, and determines whether or
-	 * not an error occurred.  A listener may be supplied to capture message text
-	 * that would normally be written to the standard error and standard output
-	 * streams of a command line CVS client.
+	 * Execute a CVS command on an array of ICVSResource. This method simply converts
+	 * the ICVSResource to String paths relative to the local root of the session and
+	 * invokes <code>execute(Session, GlobalOption[], LocalOption[], String[], ICommandOutputListener, IProgressMonitor)</code>.
 	 * </p>
+	 * @param session the open CVS session
 	 * @param globalOptions the array of global options, or NO_GLOBAL_OPTIONS
 	 * @param localOptions the array of local options, or NO_LOCAL_OPTIONS
-	 * @param arguments the array of arguments (usually filenames relative to localRoot), or NO_ARGUMENTS
+	 * @param arguments the array of ICVSResource to be operated on
 	 * @param listener the command output listener, or null to discard all messages
 	 * @param monitor the progress monitor
 	 * @return a status code indicating success or failure of the operation
 	 * @throws CVSException if a fatal error occurs (e.g. connection timeout)
+	 * 
+	 * @see Command#execute(Session, GlobalOption[], LocalOption[], String[], ICommandOutputListener, IProgressMonitor)
 	 */
-	public final IStatus execute(GlobalOption[] globalOptions, LocalOption[] localOptions, ICVSResource[] arguments, 
+	public final IStatus execute(Session session, GlobalOption[] globalOptions, LocalOption[] localOptions, ICVSResource[] arguments, 
 		ICommandOutputListener listener, IProgressMonitor pm) throws CVSException {
 		
-		Session openSession = getOpenSession(arguments);
-		String[] stringArguments = convertArgumentsForOpenSession(arguments, openSession);
-		return execute(openSession, globalOptions, localOptions, stringArguments, listener, pm);
-	}
-	
-	protected Session getOpenSession(ICVSResource[] arguments) throws CVSException {
-		// We assume that all the passed resources have the same root
-		Session openSession;
-		if (arguments == null || arguments.length == 0) {
-			// If there are no arguments to the command, assume that the command is rooted at the workspace root
-			openSession = Session.getOpenSession(CVSWorkspaceRoot.getCVSFolderFor(ResourcesPlugin.getWorkspace().getRoot()));
-		} else {
-			openSession = Session.getOpenSession(arguments[0]);
-		}
-		if (openSession == null) {
-			throw new CVSException(Policy.bind("Command.noOpenSession")); //$NON-NLS-1$
-		}
-		return openSession;
+		String[] stringArguments = convertArgumentsForOpenSession(arguments, session);
+		return execute(session, globalOptions, localOptions, stringArguments, listener, pm);
 	}
 	
 	protected String[] convertArgumentsForOpenSession(ICVSResource[] arguments, Session openSession) throws CVSException {
@@ -765,9 +820,9 @@ public abstract class Command extends Request {
 		}
 		if (accumulatedStatus.isOK()) return newStatus;
 		if (newStatus.isOK()) return accumulatedStatus;
-		MultiStatus result = new MultiStatus(CVSProviderPlugin.ID, CVSStatus.INFO,
+		MultiStatus result = new MultiStatus(CVSProviderPlugin.ID, IStatus.INFO,
 				new IStatus[] {accumulatedStatus, newStatus},
-				Policy.bind("Command.warnings", Policy.bind("Command." + getRequestId())), null); //$NON-NLS-1$ //$NON-NLS-2$
+				NLS.bind(CVSMessages.Command_warnings, new String[] { getDisplayText() }), null);
 		return result;
 	}
 }
