@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *     Red Hat Incorporated - is/setExecutable() code
@@ -13,7 +13,9 @@ package org.eclipse.team.internal.ccvs.core.resources;
 
 import java.io.File;
 import java.io.InputStream;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Date;
+import java.util.List;
 
 import org.eclipse.core.resources.*;
 import org.eclipse.core.runtime.*;
@@ -23,7 +25,6 @@ import org.eclipse.team.core.TeamException;
 import org.eclipse.team.internal.ccvs.core.*;
 import org.eclipse.team.internal.ccvs.core.client.Session;
 import org.eclipse.team.internal.ccvs.core.syncinfo.*;
-import org.eclipse.team.internal.ccvs.core.util.Assert;
 
 /**
  * Represents handles to CVS resource on the local file system. Synchronization
@@ -48,7 +49,7 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 		try {
 			((IFile)resource).delete(false /*force*/, true /*keepHistory*/, null);
 		} catch(CoreException e) {
-			throw CVSException.wrapException(resource, NLS.bind(CVSMessages.EclipseFile_Problem_deleting_resource, new String[] { resource.getFullPath().toString(), e.getStatus().getMessage() }), e); //$NON-NLS-1$ //$NON-NLS-2$
+			throw CVSException.wrapException(resource, NLS.bind(CVSMessages.EclipseFile_Problem_deleting_resource, new String[] { resource.getFullPath().toString(), e.getStatus().getMessage() }), e); // 
 		}
 	}
 	
@@ -60,7 +61,7 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
  		try {
 			return getIFile().getContents();
 		} catch (CoreException e) {
- 			throw CVSException.wrapException(resource, NLS.bind(CVSMessages.EclipseFile_Problem_accessing_resource, new String[] { resource.getFullPath().toString(), e.getStatus().getMessage() }), e); //$NON-NLS-1$ //$NON-NLS-2$
+ 			throw CVSException.wrapException(resource, NLS.bind(CVSMessages.EclipseFile_Problem_accessing_resource, new String[] { resource.getFullPath().toString(), e.getStatus().getMessage() }), e); // 
  		}
  	}
 	
@@ -206,7 +207,7 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 				if (t != null) message = t.getMessage();
 			}
 			if (message == null) message = e.getMessage();
-			throw CVSException.wrapException(resource, NLS.bind(CVSMessages.EclipseFile_Problem_writing_resource, new String[] { resource.getFullPath().toString(), message }), e); //$NON-NLS-1$
+			throw CVSException.wrapException(resource, NLS.bind(CVSMessages.EclipseFile_Problem_writing_resource, new String[] { resource.getFullPath().toString(), message }), e); 
 		}
 	}
 			
@@ -455,29 +456,12 @@ public class EclipseFile extends EclipseResource implements ICVSFile {
 		if (entryLine == null) {
 			// cvs commit: the file contents matched the server contents so no entry line was sent
 			if (oldInfo == null) return;
-			Date timeStamp = oldInfo.getTimeStamp();
-			if (timeStamp == null || oldInfo.isMergedWithConflicts()) {
-				// If the entry line has no timestamp, put the file timestamp in the entry line
-				if(! oldInfo.isAdded()) {
-					MutableResourceSyncInfo mutable = oldInfo.cloneMutable();
-					mutable.setTimeStamp(getTimeStamp(), true /* clear merged */);
-					newInfo = mutable;
-				}
-			} else {
-				// reset the file timestamp to the one from the entry line
-			    boolean reset = false;
-			    if (isReadOnly()) {
-			        // We can only set the timestamp if the file is not read-only.
-			        // We will get a situation like this if the timestamp changes due to a time change (e.g. daylight savings)
-			        setReadOnly(false);
-			        reset = true;
-			    }
-				setTimeStamp(timeStamp);
-				if (reset) {
-				    // Make sure we set the file back to read-only
-				    setReadOnly(true);
-				}
-				// (newInfo = null) No need to set the newInfo as there is no sync info change
+			// We should never make the timestamp go backwards so we'll set
+			// the entry line timestamp to match that of the file
+			if(! oldInfo.isAdded()) {
+				MutableResourceSyncInfo mutable = oldInfo.cloneMutable();
+				mutable.setTimeStamp(getTimeStamp(), true /* clear merged */);
+				newInfo = mutable;
 			}
 			// (modified = false) the file will be no longer modified
 		} else if (oldInfo == null) {
