@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -55,7 +55,6 @@ public class RestoreFromRepositoryAction extends WorkspaceTraversalAction {
 	 * contains files that exist on a branch but not in HEAD
 	 */
 	class AtticLogListener extends CommandOutputListener {
-		private static final String ATTIC = "Attic"; //$NON-NLS-1$
 		private static final String RCS_FILE_POSTFIX = ",v"; //$NON-NLS-1$
 		private static final String LOGGING_PREFIX = "Logging "; //$NON-NLS-1$
 		ICVSFolder currentFolder;
@@ -124,11 +123,11 @@ public class RestoreFromRepositoryAction extends WorkspaceTraversalAction {
 		ICVSFile[] files = fetchDeletedFiles(resource);
 		if (files == null) return;
 		if (files.length == 0) {
-			MessageDialog.openInformation(getShell(), CVSUIMessages.RestoreFromRepositoryAction_noFilesTitle, NLS.bind(CVSUIMessages.RestoreFromRepositoryAction_noFilesMessage, new String[] { resource.getName() })); //$NON-NLS-1$ //$NON-NLS-2$
+			MessageDialog.openInformation(getShell(), CVSUIMessages.RestoreFromRepositoryAction_noFilesTitle, NLS.bind(CVSUIMessages.RestoreFromRepositoryAction_noFilesMessage, new String[] { resource.getName() })); // 
 			return;
 		}
 		RestoreFromRepositoryWizard wizard = new RestoreFromRepositoryWizard(resource, files);
-		WizardDialog dialog = new ResizableWizardDialog(shell, wizard);
+		WizardDialog dialog = new ResizableWizardDialog(getShell(), wizard);
 		dialog.setMinimumPageSize(350, 250);
 		dialog.open();
 	}
