@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -14,63 +14,59 @@ import java.lang.reflect.InvocationTargetException;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Properties;
 
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.ResourcesPlugin;
-import org.eclipse.core.runtime.IAdaptable;
-import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.jface.dialogs.IDialogConstants;
-import org.eclipse.jface.dialogs.ProgressMonitorDialog;
+import org.eclipse.core.runtime.*;
+import org.eclipse.jface.dialogs.*;
+import org.eclipse.jface.dialogs.Dialog;
+import org.eclipse.jface.util.PropertyChangeEvent;
+import org.eclipse.jface.window.Window;
+import org.eclipse.osgi.util.NLS;
 import org.eclipse.swt.SWT;
+import org.eclipse.swt.events.SelectionAdapter;
+import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.layout.GridLayout;
-import org.eclipse.swt.widgets.Button;
-import org.eclipse.swt.widgets.Combo;
-import org.eclipse.swt.widgets.Composite;
-import org.eclipse.swt.widgets.Control;
-import org.eclipse.swt.widgets.Event;
-import org.eclipse.swt.widgets.Label;
-import org.eclipse.swt.widgets.Listener;
-import org.eclipse.swt.widgets.Text;
+import org.eclipse.swt.widgets.*;
 import org.eclipse.team.core.RepositoryProvider;
 import org.eclipse.team.core.TeamException;
-import org.eclipse.team.internal.ccvs.core.CVSException;
-import org.eclipse.team.internal.ccvs.core.CVSProviderPlugin;
-import org.eclipse.team.internal.ccvs.core.CVSTeamProvider;
-import org.eclipse.team.internal.ccvs.core.ICVSRepositoryLocation;
-import org.eclipse.team.internal.ccvs.core.IConnectionMethod;
-import org.eclipse.team.internal.ccvs.core.IUserInfo;
+import org.eclipse.team.internal.ccvs.core.*;
 import org.eclipse.team.internal.ccvs.core.connection.CVSRepositoryLocation;
-import org.eclipse.team.internal.ccvs.ui.CVSUIPlugin;
-import org.eclipse.team.internal.ccvs.ui.IHelpContextIds;
+import org.eclipse.team.internal.ccvs.core.util.KnownRepositories;
+import org.eclipse.team.internal.ccvs.ui.*;
 import org.eclipse.team.internal.ccvs.ui.Policy;
-import org.eclipse.team.internal.ui.DetailsDialogWithProjects;
+import org.eclipse.team.internal.ccvs.ui.wizards.ConfigurationWizardMainPage;
+import org.eclipse.team.internal.ui.dialogs.DetailsDialogWithProjects;
+import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.actions.WorkspaceModifyOperation;
 import org.eclipse.ui.dialogs.PropertyPage;
-import org.eclipse.ui.help.WorkbenchHelp;
 
 public class CVSRepositoryPropertiesPage extends PropertyPage {
+	private static final String FAKE_PASSWORD = "*********"; //$NON-NLS-1$
 	ICVSRepositoryLocation location;
 	
 	// Widgets
 	Text userText;
 	Text passwordText;
 	Combo methodType;
-	Label hostLabel;
-	Label pathLabel;
-	Label portLabel;
+	Text hostText;
+	Text pathText;
+	// Port
+	private Text portText;
+	private Button useDefaultPort;
+	private Button useCustomPort;
+	
+	// Caching password
+	private Button allowCachingButton;
+	private boolean allowCaching = false;
 	
 	boolean passwordChanged;
 	boolean connectionInfoChanged;
-	boolean programNameChanged;
-	boolean labelChanged;
 
 	IUserInfo info;
 
-	// Program Name
-	private Text programNameText;
-	private Button useDefaultProgramName;
-	private Button useCustomProgramName;
 	// Label
 	private Button useLocationAsLabel;
 	private Button useCustomLabel;
@@ -101,12 +97,11 @@ public class CVSRepositoryPropertiesPage extends PropertyPage {
 		labelGroup.setLayout(layout);
 		Listener labelListener = new Listener() {
 			public void handleEvent(Event event) {
-				labelChanged = true;
 				updateWidgetEnablements();
 			}
 		};
-		useLocationAsLabel = createRadioButton(labelGroup, Policy.bind("CVSRepositoryPropertiesPage.useLocationAsLabel"), 3); //$NON-NLS-1$
-		useCustomLabel = createRadioButton(labelGroup, Policy.bind("CVSRepositoryPropertiesPage.useCustomLabel"), 1); //$NON-NLS-1$
+		useLocationAsLabel = createRadioButton(labelGroup, CVSUIMessages.CVSRepositoryPropertiesPage_useLocationAsLabel, 3); 
+		useCustomLabel = createRadioButton(labelGroup, CVSUIMessages.CVSRepositoryPropertiesPage_useCustomLabel, 1); 
 		useCustomLabel.addListener(SWT.Selection, labelListener);
 		labelText = createTextField(labelGroup);
 		labelText.addListener(SWT.Modify, labelListener);
@@ -114,71 +109,96 @@ public class CVSRepositoryPropertiesPage extends PropertyPage {
 		// Add some extra space
 		createLabel(composite, "", 3); //$NON-NLS-1$
 		
-		Label label = createLabel(composite, Policy.bind("CVSPropertiesPage.connectionType"), 1); //$NON-NLS-1$
+		createLabel(composite, CVSUIMessages.CVSPropertiesPage_connectionType, 1); 
 		methodType = createCombo(composite);
 		
-		label = createLabel(composite, Policy.bind("CVSPropertiesPage.user"), 1); //$NON-NLS-1$
+		createLabel(composite, CVSUIMessages.CVSPropertiesPage_user, 1); 
 		userText = createTextField(composite);
 		
-		label = createLabel(composite, Policy.bind("CVSPropertiesPage.password"), 1); //$NON-NLS-1$
-		passwordText = createTextField(composite);
-		passwordText.setEchoChar('*');
+		createLabel(composite, CVSUIMessages.CVSPropertiesPage_password, 1); 
+		passwordText = createPasswordField(composite);
+			
+		createLabel(composite, CVSUIMessages.CVSPropertiesPage_host, 1); 
+		hostText = createTextField(composite);
 		
-		label = createLabel(composite, Policy.bind("CVSPropertiesPage.host"), 1); //$NON-NLS-1$
-		hostLabel = createLabel(composite, "", 2); //$NON-NLS-1$
+		createLabel(composite, CVSUIMessages.CVSPropertiesPage_path, 1); 
+		pathText = createTextField(composite);
 		
-		label = createLabel(composite, Policy.bind("CVSPropertiesPage.port"), 1); //$NON-NLS-1$
-		portLabel = createLabel(composite, "", 2); //$NON-NLS-1$
-		
-		label = createLabel(composite, Policy.bind("CVSPropertiesPage.path"), 1); //$NON-NLS-1$
-		pathLabel = createLabel(composite, "", 2); //$NON-NLS-1$
-
-		// Add some extra space
-		createLabel(composite, "", 3); //$NON-NLS-1$
-
-		// Remote CVS program name
+		// Port number
 		// create a composite to ensure the radio buttons come in the correct order
-		Composite programNameGroup = new Composite(composite, SWT.NONE);
+		Composite portGroup = new Composite(composite, SWT.NONE);
 		data = new GridData();
 		data.horizontalSpan = 3;
-		programNameGroup.setLayoutData(data);
+		portGroup.setLayoutData(data);
 		layout = new GridLayout();
 		layout.numColumns = 3;
 		layout.marginHeight = 0;
 		layout.marginWidth = 0;
-		programNameGroup.setLayout(layout);
-		Listener programNameListener = new Listener() {
+		portGroup.setLayout(layout);
+		useDefaultPort = createRadioButton(portGroup, CVSUIMessages.ConfigurationWizardMainPage_useDefaultPort, 3); 
+		useCustomPort = createRadioButton(portGroup, CVSUIMessages.ConfigurationWizardMainPage_usePort, 1); 
+		portText = createTextField(portGroup);
+
+		// Add some extra space
+		createLabel(composite, "", 3); //$NON-NLS-1$
+
+		allowCachingButton = new Button(composite, SWT.CHECK);
+		allowCachingButton.setText(CVSUIMessages.UserValidationDialog_6); 
+		data = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
+		data.horizontalSpan = 3;
+		allowCachingButton.setLayoutData(data);
+		allowCachingButton.addSelectionListener(new SelectionAdapter() {
+			public void widgetSelected(SelectionEvent e) {
+				allowCaching = allowCachingButton.getSelection();
+			}
+		});
+	
+		Composite warningComposite = new Composite(composite, SWT.NONE);
+		layout = new GridLayout();
+		layout.numColumns = 2;
+		layout.marginHeight = 0;
+		layout.marginHeight = 0;
+		warningComposite.setLayout(layout);
+		data = new GridData(GridData.FILL_HORIZONTAL);
+		data.horizontalSpan = 3;
+		warningComposite.setLayoutData(data);
+		Label warningLabel = new Label(warningComposite, SWT.NONE);
+		warningLabel.setImage(Dialog.getImage(Dialog.DLG_IMG_MESSAGE_WARNING));
+		warningLabel.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING | GridData.HORIZONTAL_ALIGN_BEGINNING));
+		Label warningText = new Label(warningComposite, SWT.WRAP);
+		warningText.setText(CVSUIMessages.UserValidationDialog_7); 
+		data = new GridData(GridData.FILL_HORIZONTAL);
+		data.widthHint = 300;
+		warningText.setLayoutData(data);
+		
+		// Add some extra space
+		createLabel(composite, "", 3); //$NON-NLS-1$
+		
+		initializeValues();
+		updateWidgetEnablements();
+		Listener connectionInfoChangedListener = new Listener() {
 			public void handleEvent(Event event) {
-				programNameChanged = true;
+				connectionInfoChanged = true;
 				updateWidgetEnablements();
 			}
 		};
-		useDefaultProgramName = createRadioButton(programNameGroup, Policy.bind("CVSRepositoryPropertiesPage.useDefaultProgramName"), 3); //$NON-NLS-1$
-		useCustomProgramName = createRadioButton(programNameGroup, Policy.bind("CVSRepositoryPropertiesPage.useProgramName"), 1); //$NON-NLS-1$
-		useCustomProgramName.addListener(SWT.Selection, programNameListener);
-		programNameText = createTextField(programNameGroup);
-		programNameText.addListener(SWT.Modify, programNameListener);
-				
-		initializeValues();
-		updateWidgetEnablements();
 		passwordText.addListener(SWT.Modify, new Listener() {
 			public void handleEvent(Event event) {
-				passwordChanged = true;
+				passwordChanged = !passwordText.getText().equals(FAKE_PASSWORD);
 			}
 		});
-		userText.addListener(SWT.Modify, new Listener() {
-			public void handleEvent(Event event) {
-				connectionInfoChanged = true;
-			}
-		});
-		methodType.addListener(SWT.Modify, new Listener() {
-			public void handleEvent(Event event) {
-				connectionInfoChanged = true;
-			}
-		});
-		WorkbenchHelp.setHelp(getControl(), IHelpContextIds.REPOSITORY_LOCATION_PROPERTY_PAGE);
+		userText.addListener(SWT.Modify, connectionInfoChangedListener);
+		methodType.addListener(SWT.Modify, connectionInfoChangedListener);
+		hostText.addListener(SWT.Modify, connectionInfoChangedListener);
+		portText.addListener(SWT.Modify, connectionInfoChangedListener);
+		useCustomPort.addListener(SWT.Selection, connectionInfoChangedListener);
+		pathText.addListener(SWT.Modify, connectionInfoChangedListener);
+		
+        PlatformUI.getWorkbench().getHelpSystem().setHelp(getControl(), IHelpContextIds.REPOSITORY_LOCATION_PROPERTY_PAGE);
+        Dialog.applyDialogFont(parent);
 		return composite;
 	}
+	
 	/**
 	 * Utility method that creates a combo box
 	 *
@@ -218,6 +238,25 @@ public class CVSRepositoryPropertiesPage extends PropertyPage {
 	 */
 	protected Text createTextField(Composite parent) {
 		Text text = new Text(parent, SWT.SINGLE | SWT.BORDER);
+		return layoutTextField(text);
+	}
+	/**
+	 * Create a password field specific for this application
+	 *
+	 * @param parent  the parent of the new text field
+	 * @return the new text field
+	 */
+	protected Text createPasswordField(Composite parent) {
+		Text text = new Text(parent, SWT.SINGLE | SWT.BORDER | SWT.PASSWORD);
+		return layoutTextField(text);
+	}
+	/**
+	 * Layout a text or password field specific for this application
+	 *
+	 * @param parent  the parent of the new text field
+	 * @return the new text field
+	 */
+	protected Text layoutTextField(Text text) {
 		GridData data = new GridData(GridData.FILL_HORIZONTAL);
 		data.verticalAlignment = GridData.CENTER;
 		data.grabExcessVerticalSpace = false;
@@ -226,6 +265,7 @@ public class CVSRepositoryPropertiesPage extends PropertyPage {
 		text.setLayoutData(data);
 		return text;
 	}
+
 	/**
 	 * Utility method to create a radio button
 	 * 
@@ -262,6 +302,7 @@ public class CVSRepositoryPropertiesPage extends PropertyPage {
 	 */
 	private void initializeValues() {
 		passwordChanged = false;
+		connectionInfoChanged = false;
 		
 		IConnectionMethod[] methods = CVSRepositoryLocation.getPluggedInConnectionMethods();
 		for (int i = 0; i < methods.length; i++) {
@@ -271,21 +312,20 @@ public class CVSRepositoryPropertiesPage extends PropertyPage {
 		methodType.select(methodType.indexOf(method));
 		info = location.getUserInfo(true);
 		userText.setText(info.getUsername());
-		passwordText.setText("*********"); //$NON-NLS-1$
-		hostLabel.setText(location.getHost());
+		passwordText.setText(FAKE_PASSWORD);
+		hostText.setText(location.getHost());
 		int port = location.getPort();
 		if (port == ICVSRepositoryLocation.USE_DEFAULT_PORT) {
-			portLabel.setText(Policy.bind("CVSPropertiesPage.defaultPort")); //$NON-NLS-1$
+			useDefaultPort.setSelection(true);
+			useCustomPort.setSelection(false);
+			portText.setEnabled(false);
 		} else {
-			portLabel.setText("" + port); //$NON-NLS-1$
+			useDefaultPort.setSelection(false);
+			useCustomPort.setSelection(true);
+			portText.setText("" + port); //$NON-NLS-1$
 		}
-		pathLabel.setText(location.getRootDirectory());
-		
-		// get the program name
-		String programName = ((CVSRepositoryLocation)location).getRemoteCVSProgramName();
-		programNameText.setText(programName);
-		useDefaultProgramName.setSelection(programName == CVSRepositoryLocation.DEFAULT_REMOTE_CVS_PROGRAM_NAME);
-		useCustomProgramName.setSelection(!useDefaultProgramName.getSelection());
+		pathText.setText(location.getRootDirectory());
+		allowCachingButton.setSelection(location.getUserInfoCached());
 		
 		// get the repository label
 		String label = null;
@@ -294,52 +334,54 @@ public class CVSRepositoryPropertiesPage extends PropertyPage {
 		useLocationAsLabel.setSelection(label == null);
 		useCustomLabel.setSelection(!useLocationAsLabel.getSelection());
 		if (label == null) {
-			label = location.getLocation();
+			label = location.getLocation(true);
 		}
 		labelText.setText(label);
 	}
 	
-	/*
-	 * @see PreferencesPage#performOk
-	 */
-	public boolean performOk() {
-		if (!connectionInfoChanged && !passwordChanged) {
-			if (programNameChanged) {
-				recordNewProgramName((CVSRepositoryLocation)location);
+	private boolean performConnectionInfoChanges() {
+		// Set the caching mode of the location
+		if (!connectionInfoChanged) {
+			location.setAllowCaching(allowCaching);
+			if (!passwordChanged) {
+				((CVSRepositoryLocation)location).updateCache();
 			}
-			if (labelChanged) {
-				recordNewLabel((CVSRepositoryLocation)location);
-			}
-			return true;
 		}
-		info.setUsername(userText.getText());
-		if (passwordChanged) {
-			info.setPassword(passwordText.getText());
-		}
-		final String type = methodType.getText();
-		final String password = passwordText.getText();
-		final boolean[] result = new boolean[] { false };
+		// Don't do anything else if there wasn't a password or connection change
+		if (!passwordChanged && !connectionInfoChanged) return true;
+		
 		try {
+			// Check if the password was the only thing to change.
+			if (passwordChanged && !connectionInfoChanged) {
+				CVSRepositoryLocation oldLocation = (CVSRepositoryLocation)location;
+				oldLocation.setPassword(getNewPassword());
+				if (allowCaching) {
+					oldLocation.updateCache();
+				}
+				passwordChanged = false;
+				return true;
+			}
+		
+			// Otherwise change the connection info and the password
 			// This operation is done inside a workspace operation in case the sharing
 			// info for existing projects is changed
-			new ProgressMonitorDialog(getShell()).run(false, false, new WorkspaceModifyOperation() {
+			if (!(location.getHost().equals(hostText.getText()) && location.getRootDirectory().equals(pathText.getText()))) {
+				// The host or path has changed
+				if (!MessageDialog.openConfirm(getShell(), 
+						CVSUIMessages.CVSRepositoryPropertiesPage_0,  
+						CVSUIMessages.CVSRepositoryPropertiesPage_1)) { 
+					return false;
+				}
+			}
+			final boolean[] result = new boolean[] { false };
+			final ProgressMonitorDialog progressMonitorDialog = new ProgressMonitorDialog(getShell());
+            progressMonitorDialog.run(false, false, new WorkspaceModifyOperation(null) {
 				public void execute(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
 					try {
-						// Check if the password was the only thing to change.
-						if (passwordChanged && !connectionInfoChanged) {
-							CVSRepositoryLocation oldLocation = (CVSRepositoryLocation)location;
-							oldLocation.setPassword(password);
-							oldLocation.updateCache();
-							passwordChanged = false;
-							result[0] = true;
-							return;
-						}
-						
 						// Create a new repository location with the new information
-						CVSRepositoryLocation newLocation = CVSRepositoryLocation.fromString(location.getLocation());
-						newLocation.setMethod(type);
-						newLocation.setUserInfo(info);
-						
+						CVSRepositoryLocation newLocation = CVSRepositoryLocation.fromProperties(createProperties());
+						newLocation.setEncoding(location.getEncoding());
+						location.setAllowCaching(allowCaching);
 						try {
 							// For each project shared with the old location, set connection info to the new one
 							List projects = new ArrayList();
@@ -356,15 +398,15 @@ public class CVSRepositoryPropertiesPage extends PropertyPage {
 							if (projects.size() > 0) {
 								// To do: warn the user
 								DetailsDialogWithProjects dialog = new DetailsDialogWithProjects(
-									getShell(), 
-									Policy.bind("CVSRepositoryPropertiesPage.Confirm_Project_Sharing_Changes_1"), //$NON-NLS-1$
-									Policy.bind("CVSRepositoryPropertiesPage.There_are_projects_in_the_workspace_shared_with_this_repository_2"), //$NON-NLS-1$
-									Policy.bind("CVSRepositoryPropertiesPage.sharedProject", location.toString()), //$NON-NLS-1$
+								    progressMonitorDialog.getShell(), 
+									CVSUIMessages.CVSRepositoryPropertiesPage_Confirm_Project_Sharing_Changes_1, 
+									CVSUIMessages.CVSRepositoryPropertiesPage_There_are_projects_in_the_workspace_shared_with_this_repository_2, 
+									NLS.bind(CVSUIMessages.CVSRepositoryPropertiesPage_sharedProject, new String[] { location.toString() }), 
 									(IProject[]) projects.toArray(new IProject[projects.size()]),
 									true,
-									DetailsDialogWithProjects.DLG_IMG_WARNING);
+									Dialog.DLG_IMG_WARNING);
 								int r = dialog.open();
-								if (r != DetailsDialogWithProjects.OK) {
+								if (r != Window.OK) {
 									result[0] = false;
 									return;
 								}
@@ -387,37 +429,49 @@ public class CVSRepositoryPropertiesPage extends PropertyPage {
 							
 						} finally {
 							// Even if we failed, ensure that the new location appears in the repo view.
-							newLocation = (CVSRepositoryLocation)CVSProviderPlugin.getPlugin().getRepository(newLocation.getLocation());
-							newLocation.updateCache();
+							newLocation = (CVSRepositoryLocation)KnownRepositories.getInstance().addRepository(newLocation, !KnownRepositories.getInstance().isKnownRepository(newLocation.getLocation()));
 						}
 						
-						
 						// Set the location of the page to the new location in case Apply was chosen
 						location = newLocation;
-						
-						if (programNameChanged) {
-							recordNewProgramName((CVSRepositoryLocation)location);
-						}
-						if (labelChanged) {
-							recordNewLabel((CVSRepositoryLocation)location);
-						}
-			
 						connectionInfoChanged = false;
 						passwordChanged = false;
-						programNameChanged = false;
 					} catch (TeamException e) {
 						throw new InvocationTargetException(e);
 					}
 					result[0] = true;
 				}
 			});
+			return result[0];
 		} catch (InvocationTargetException e) {
 			handle(e);
 		} catch (InterruptedException e) {
 		}
-					
-		return result[0];
+		return false; /* we only get here if an exception occurred */
 	}
+	
+	private void performNonConnectionInfoChanges() {
+		recordNewLabel((CVSRepositoryLocation)location);
+	}
+	/*
+	 * @see PreferencesPage#performOk
+	 */
+	public boolean performOk() {
+		if (performConnectionInfoChanges()) {
+			performNonConnectionInfoChanges();
+			return true;
+		}
+		return false;
+	}
+	
+	/* (non-Javadoc)
+	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
+	 */
+	protected void performDefaults() {
+		super.performDefaults();
+		initializeValues();
+	}
+	
 	/**
 	 * Shows the given errors to the user.
 	 */
@@ -429,49 +483,96 @@ public class CVSRepositoryPropertiesPage extends PropertyPage {
 	 * Updates widget enablements and sets error message if appropriate.
 	 */
 	protected void updateWidgetEnablements() {
-		if (useDefaultProgramName.getSelection()) {
-			programNameText.setEnabled(false);
-		} else {
-			programNameText.setEnabled(true);
-		}
 		if (useLocationAsLabel.getSelection()) {
 			labelText.setEnabled(false);
 		} else {
 			labelText.setEnabled(true);
 		}
+		if (useDefaultPort.getSelection()) {
+			portText.setEnabled(false);
+		} else {
+			portText.setEnabled(true);
+		}
 		validateFields();
 	}
 	
 	private void validateFields() {
-		if (programNameText.isEnabled()) {
-			if (programNameText.getText().length() == 0) {
-				setValid(false);
-				return;
-			}
-		}
 		if (labelText.isEnabled()) {
 			if (labelText.getText().length() == 0) {
 				setValid(false);
 				return;
 			}
 		}
+		String user = userText.getText();
+		IStatus status = ConfigurationWizardMainPage.validateUserName(user);
+		if (!isStatusOK(status)) {
+			return;
+		}
+
+		String host = hostText.getText();
+		status = ConfigurationWizardMainPage.validateHost(host);
+		if (!isStatusOK(status)) {
+			return;
+		}
+
+		if (portText.isEnabled()) {
+			String port = portText.getText();
+			status = ConfigurationWizardMainPage.validatePort(port);
+			if (!isStatusOK(status)) {
+				return;
+			}
+		}
+
+		String pathString = pathText.getText();
+		status = ConfigurationWizardMainPage.validatePath(pathString);
+		if (!isStatusOK(status)) {
+			return;
+		}
+		
+		try {
+			CVSRepositoryLocation l = CVSRepositoryLocation.fromProperties(createProperties());
+			if (!l.equals(location) && KnownRepositories.getInstance().isKnownRepository(l.getLocation())) {
+				setErrorMessage(CVSUIMessages.ConfigurationWizardMainPage_0); 
+				setValid(false);
+				return;
+			}
+		} catch (CVSException e) {
+			CVSUIPlugin.log(e);
+			// Let it pass. Creation should fail
+		}
+		
+		setErrorMessage(null);
 		setValid(true);
 	}
 	
-	private void recordNewProgramName(CVSRepositoryLocation location) {
-		// Set the remote program name if appropriate
-		String newProgramName;
-		if (useDefaultProgramName.getSelection()) {
-			newProgramName = CVSRepositoryLocation.DEFAULT_REMOTE_CVS_PROGRAM_NAME;
-		} else {
-			newProgramName = programNameText.getText();
-		}
-		if (!location.getRemoteCVSProgramName().equals(newProgramName)) {
-			CVSProviderPlugin.getPlugin().setCVSProgramName(location, newProgramName);
+	private boolean isStatusOK(IStatus status) {
+		if (!status.isOK()) {
+			setErrorMessage(status.getMessage());
+			setValid(false);
+			return false;
 		}
+		return true;
 	}
 	
 	private void recordNewLabel(CVSRepositoryLocation location) {
+		String newLabel = getNewLabel(location);
+		if (newLabel == null) {
+			String oldLabel = getOldLabel(location);
+			if (oldLabel == null || oldLabel.equals(location.getLocation())) {
+				return;
+			}
+		} else if (newLabel.equals(getOldLabel(location))) {
+			return;
+		}
+		CVSUIPlugin.getPlugin().getRepositoryManager().setLabel(location, newLabel);
+		CVSUIPlugin.broadcastPropertyChange(new PropertyChangeEvent(this, 
+				CVSUIPlugin.P_DECORATORS_CHANGED, null, null));
+
+	}
+	private String getOldLabel(CVSRepositoryLocation location) {
+		return CVSUIPlugin.getPlugin().getRepositoryManager().getRepositoryRootFor(location).getName();
+	}
+	private String getNewLabel(CVSRepositoryLocation location) {
 		String label = null;
 		if (useCustomLabel.getSelection()) {
 			label = labelText.getText();
@@ -479,11 +580,25 @@ public class CVSRepositoryPropertiesPage extends PropertyPage {
 				label = null;
 			}
 		}
-		try {
-			CVSUIPlugin.getPlugin().getRepositoryManager().setLabel(location, label);
-		} catch (CVSException e) {
-			CVSUIPlugin.log(e);
+		return label;
+	}
+	/* internal use only */ String getNewPassword() {
+		return passwordText.getText();
+	}
+	
+	private Properties createProperties() {
+		Properties result = new Properties();
+		result.setProperty("connection", methodType.getText()); //$NON-NLS-1$
+		result.setProperty("user", userText.getText()); //$NON-NLS-1$
+		if (passwordChanged) {
+			result.setProperty("password", passwordText.getText()); //$NON-NLS-1$
 		}
+		result.setProperty("host", hostText.getText()); //$NON-NLS-1$
+		if (useCustomPort.getSelection()) {
+			result.setProperty("port", portText.getText()); //$NON-NLS-1$
+		}
+		result.setProperty("root", pathText.getText()); //$NON-NLS-1$
+		return result;
 	}
 }
 
