@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -12,7 +12,14 @@ package org.eclipse.team.internal.ccvs.ui.repo;
 
 import java.io.IOException;
 import java.lang.reflect.InvocationTargetException;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
 
 import org.eclipse.core.runtime.*;
 import org.eclipse.osgi.util.NLS;
@@ -28,7 +35,7 @@ import org.eclipse.team.internal.ccvs.ui.operations.RemoteLogOperation.LogEntryC
 
 public class RepositoryRoot extends PlatformObject {
 
-	public static final String[] DEFAULT_AUTO_REFRESH_FILES = { ".project" }; //$NON-NLS-1$ //$NON-NLS-2$
+	public static final String[] DEFAULT_AUTO_REFRESH_FILES = { ".project" }; //$NON-NLS-1$ 
 	private static final String DEFINED_MODULE_PREFIX = "module:"; //$NON-NLS-1$
 	
 	ICVSRepositoryLocation root;
@@ -155,12 +162,12 @@ public class RepositoryRoot extends PlatformObject {
 			}
 			FolderSyncInfo info = ((ICVSFolder)resource).getFolderSyncInfo();
 			if (info == null)
-				throw new CVSException(NLS.bind(CVSUIMessages.RepositoryRoot_folderInfoMissing, new String[] { resource.getName() })); //$NON-NLS-1$
+				throw new CVSException(NLS.bind(CVSUIMessages.RepositoryRoot_folderInfoMissing, new String[] { resource.getName() })); 
 			return info.getRepository();
 		} else {
 			FolderSyncInfo info = resource.getParent().getFolderSyncInfo();
 			if (info == null)
-				throw new CVSException(NLS.bind(CVSUIMessages.RepositoryRoot_folderInfoMissing, new String[] { resource.getParent().getName() })); //$NON-NLS-1$
+				throw new CVSException(NLS.bind(CVSUIMessages.RepositoryRoot_folderInfoMissing, new String[] { resource.getParent().getName() })); 
 			String path = new Path(null, info.getRepository()).append(resource.getName()).toString();
 			return path;
 		}
@@ -336,7 +343,7 @@ public class RepositoryRoot extends PlatformObject {
 	public CVSTag[] refreshDefinedTags(ICVSFolder folder, boolean recurse, IProgressMonitor monitor) throws TeamException {
 	    monitor.beginTask(null, 100);
 	    CVSTag[] tags = null;
-	    if (!recurse) {
+	    if (!recurse && !folder.getFolderSyncInfo().isVirtualDirectory()) {
 	        // Only try the auto-refresh file(s) if we are not recursing into sub-folders
 	        tags = fetchTagsUsingAutoRefreshFiles(folder, Policy.subMonitorFor(monitor, 50));
 	    }
@@ -400,7 +407,7 @@ public class RepositoryRoot extends PlatformObject {
 	    String remotePath = getRemotePathFor(folder);
 		String[] filesToRefresh = getAutoRefreshFiles(remotePath);
 		try {
-			monitor.beginTask(null, filesToRefresh.length * 10); //$NON-NLS-1$
+			monitor.beginTask(null, filesToRefresh.length * 10); 
 			List tags = new ArrayList();
 			for (int i = 0; i < filesToRefresh.length; i++) {
 				ICVSRemoteFile file = root.getRemoteFile(filesToRefresh[i], CVSTag.DEFAULT);
