@@ -1,48 +1,43 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.team.internal.ccvs.ui.wizards;
 
-
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import java.util.Properties;
 
-import org.eclipse.core.runtime.IPath;
-import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.core.runtime.Path;
-import org.eclipse.jface.dialogs.IDialogConstants;
-import org.eclipse.jface.dialogs.IDialogSettings;
+import org.eclipse.core.runtime.*;
+import org.eclipse.jface.dialogs.*;
+import org.eclipse.jface.dialogs.Dialog;
 import org.eclipse.jface.resource.ImageDescriptor;
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.layout.FillLayout;
-import org.eclipse.swt.layout.GridData;
-import org.eclipse.swt.layout.GridLayout;
-import org.eclipse.swt.widgets.Button;
-import org.eclipse.swt.widgets.Combo;
-import org.eclipse.swt.widgets.Composite;
-import org.eclipse.swt.widgets.Event;
-import org.eclipse.swt.widgets.Group;
-import org.eclipse.swt.widgets.Listener;
-import org.eclipse.swt.widgets.Text;
-import org.eclipse.team.internal.ccvs.core.IConnectionMethod;
+import org.eclipse.swt.events.SelectionAdapter;
+import org.eclipse.swt.events.SelectionEvent;
+import org.eclipse.swt.graphics.FontMetrics;
+import org.eclipse.swt.graphics.GC;
+import org.eclipse.swt.layout.*;
+import org.eclipse.swt.widgets.*;
+import org.eclipse.team.internal.ccvs.core.*;
 import org.eclipse.team.internal.ccvs.core.connection.CVSRepositoryLocation;
-import org.eclipse.team.internal.ccvs.ui.IHelpContextIds;
-import org.eclipse.team.internal.ccvs.ui.Policy;
-import org.eclipse.ui.help.WorkbenchHelp;
+import org.eclipse.team.internal.ccvs.core.util.KnownRepositories;
+import org.eclipse.team.internal.ccvs.ui.*;
+import org.eclipse.ui.PlatformUI;
 
 /**
  * Wizard page for entering information about a CVS repository location.
  */
 public class ConfigurationWizardMainPage extends CVSWizardPage {
+	private static final String ANONYMOUS_USER = "anonymous"; //$NON-NLS-1$
+	
 	private boolean showValidate;
 	private boolean validate;
 	
@@ -64,11 +59,23 @@ public class ConfigurationWizardMainPage extends CVSWizardPage {
 	private Combo repositoryPathCombo;
 	// Validation
 	private Button validateButton;
+	// Caching password
+	private Button allowCachingButton;
+	private boolean allowCaching = false;
 	
 	private static final int COMBO_HISTORY_LENGTH = 5;
 	
 	private Properties properties = null;
 	
+	// The previously created repository.
+	// It is recorded when asked for and
+	// nulled when the page is changed.
+	private ICVSRepositoryLocation location;
+	
+	// The previously created repository.
+	// It is recorded when fields are changed
+	private ICVSRepositoryLocation oldLocation;
+	
 	// Dialog store id constants
 	private static final String STORE_USERNAME_ID =
 		"ConfigurationWizardMainPage.STORE_USERNAME_ID";//$NON-NLS-1$
@@ -137,48 +144,78 @@ public class ConfigurationWizardMainPage extends CVSWizardPage {
 	 * @param parent  the parent of the created widgets
 	 */
 	public void createControl(Composite parent) {
-		Composite composite = createComposite(parent, 2);
+		Composite composite = createComposite(parent, 2, false);
 		// set F1 help
-		WorkbenchHelp.setHelp(composite, IHelpContextIds.SHARING_NEW_REPOSITORY_PAGE);
+        PlatformUI.getWorkbench().getHelpSystem().setHelp(composite, IHelpContextIds.SHARING_NEW_REPOSITORY_PAGE);
 
 		Listener listener = new Listener() {
 			public void handleEvent(Event event) {
+				if (location != null) {
+					oldLocation = location;
+					location = null;
+				}
+				if (event.widget == hostCombo) {
+					String hostText = hostCombo.getText(); 
+					if (hostText.length() > 0 && hostText.charAt(0) == ':') {
+						try {
+							CVSRepositoryLocation newLocation = CVSRepositoryLocation.fromString(hostText);
+							connectionMethodCombo.setText(newLocation.getMethod().getName());
+							repositoryPathCombo.setText(newLocation.getRootDirectory());
+							int port = newLocation.getPort();
+							if (port == ICVSRepositoryLocation.USE_DEFAULT_PORT) {
+								useDefaultPort.setSelection(true);
+								useCustomPort.setSelection(false);
+							} else {
+								useCustomPort.setSelection(true);
+								useDefaultPort.setSelection(false);
+								portText.setText(String.valueOf(port));	
+							}
+							
+							userCombo.setText(newLocation.getUsername());
+							//passwordText.setText(newLocation.xxx);
+							hostCombo.setText(newLocation.getHost());
+						} catch (CVSException e) {
+							CVSUIPlugin.log(e);
+						}
+					}
+				}
 				updateWidgetEnablements();
 			}
 		};
 		
-		Group g = createGroup(composite, Policy.bind("ConfigurationWizardMainPage.Location_1")); //$NON-NLS-1$
+		Group g = createGroup(composite, CVSUIMessages.ConfigurationWizardMainPage_Location_1); 
 		
 		// Host name
-		createLabel(g, Policy.bind("ConfigurationWizardMainPage.host")); //$NON-NLS-1$
+		createLabel(g, CVSUIMessages.ConfigurationWizardMainPage_host); 
 		hostCombo = createEditableCombo(g);
 		hostCombo.addListener(SWT.Selection, listener);
 		hostCombo.addListener(SWT.Modify, listener);
 		
 		// Repository Path
-		createLabel(g, Policy.bind("ConfigurationWizardMainPage.repositoryPath")); //$NON-NLS-1$
+		createLabel(g, CVSUIMessages.ConfigurationWizardMainPage_repositoryPath); 
 		repositoryPathCombo = createEditableCombo(g);
 		repositoryPathCombo.addListener(SWT.Selection, listener);
 		repositoryPathCombo.addListener(SWT.Modify, listener);
 
-		g = createGroup(composite, Policy.bind("ConfigurationWizardMainPage.Authentication_2")); //$NON-NLS-1$
+		g = createGroup(composite, CVSUIMessages.ConfigurationWizardMainPage_Authentication_2); 
 		
 		// User name
-		createLabel(g, Policy.bind("ConfigurationWizardMainPage.userName")); //$NON-NLS-1$
+		createLabel(g, CVSUIMessages.ConfigurationWizardMainPage_userName); 
 		userCombo = createEditableCombo(g);
 		userCombo.addListener(SWT.Selection, listener);
 		userCombo.addListener(SWT.Modify, listener);
 		
 		// Password
-		createLabel(g, Policy.bind("ConfigurationWizardMainPage.password")); //$NON-NLS-1$
-		passwordText = createTextField(g);
-		passwordText.setEchoChar('*');
+		createLabel(g, CVSUIMessages.ConfigurationWizardMainPage_password); 
+		passwordText = createPasswordField(g);
+		passwordText.addListener(SWT.Modify, listener);
 
-		g = createGroup(composite, Policy.bind("ConfigurationWizardMainPage.Connection_3")); //$NON-NLS-1$
+		g = createGroup(composite, CVSUIMessages.ConfigurationWizardMainPage_Connection_3); 
 		
 		// Connection type
-		createLabel(g, Policy.bind("ConfigurationWizardMainPage.connection")); //$NON-NLS-1$
+		createLabel(g, CVSUIMessages.ConfigurationWizardMainPage_connection); 
 		connectionMethodCombo = createCombo(g);
+		connectionMethodCombo.addListener(SWT.Selection, listener);
 
 		// Port number
 		// create a composite to ensure the radio buttons come in the correct order
@@ -191,8 +228,8 @@ public class ConfigurationWizardMainPage extends CVSWizardPage {
 		layout.marginHeight = 0;
 		layout.marginWidth = 0;
 		portGroup.setLayout(layout);
-		useDefaultPort = createRadioButton(portGroup, Policy.bind("ConfigurationWizardMainPage.useDefaultPort"), 2); //$NON-NLS-1$
-		useCustomPort = createRadioButton(portGroup, Policy.bind("ConfigurationWizardMainPage.usePort"), 1); //$NON-NLS-1$
+		useDefaultPort = createRadioButton(portGroup, CVSUIMessages.ConfigurationWizardMainPage_useDefaultPort, 2); 
+		useCustomPort = createRadioButton(portGroup, CVSUIMessages.ConfigurationWizardMainPage_usePort, 1); 
 		useCustomPort.addListener(SWT.Selection, listener);
 		portText = createTextField(portGroup);
 		portText.addListener(SWT.Modify, listener);
@@ -206,7 +243,7 @@ public class ConfigurationWizardMainPage extends CVSWizardPage {
 			validateButtonTabGroup.setLayout(new FillLayout());
 
 			validateButton = new Button(validateButtonTabGroup, SWT.CHECK);
-			validateButton.setText(Policy.bind("ConfigurationWizardAutoconnectPage.validate")); //$NON-NLS-1$
+			validateButton.setText(CVSUIMessages.ConfigurationWizardAutoconnectPage_validate); 
 			validateButton.addListener(SWT.Selection, new Listener() {
 				public void handleEvent(Event e) {
 					validate = validateButton.getSelection();
@@ -214,11 +251,46 @@ public class ConfigurationWizardMainPage extends CVSWizardPage {
 			});
 		}
 		
+		allowCachingButton = new Button(composite, SWT.CHECK);
+		allowCachingButton.setText(CVSUIMessages.UserValidationDialog_6); 
+		data = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
+		data.horizontalSpan = 3;
+		allowCachingButton.setLayoutData(data);
+		allowCachingButton.addSelectionListener(new SelectionAdapter() {
+			public void widgetSelected(SelectionEvent e) {
+				allowCaching = allowCachingButton.getSelection();
+			}
+		});
+	
+		Composite warningComposite = new Composite(composite, SWT.NONE);
+		layout = new GridLayout();
+		layout.numColumns = 2;
+		layout.marginHeight = 0;
+		layout.marginHeight = 0;
+		warningComposite.setLayout(layout);
+		data = new GridData(GridData.FILL_HORIZONTAL);
+		data.horizontalSpan = 3;
+		warningComposite.setLayoutData(data);
+		Label warningLabel = new Label(warningComposite, SWT.NONE);
+		warningLabel.setImage(Dialog.getImage(Dialog.DLG_IMG_MESSAGE_WARNING));
+		warningLabel.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING | GridData.HORIZONTAL_ALIGN_BEGINNING));
+		Label warningText = new Label(warningComposite, SWT.WRAP);
+		warningText.setText(CVSUIMessages.UserValidationDialog_7); 
+		data = new GridData(GridData.FILL_HORIZONTAL);
+		data.widthHint = 300;
+		GC gc= new GC(composite);
+		gc.setFont(parent.getFont());
+		FontMetrics fontMetrics= gc.getFontMetrics();
+		gc.dispose();
+		data.heightHint= Dialog.convertHeightInCharsToPixels(fontMetrics, 3);
+		warningText.setLayoutData(data);
+		
 		initializeValues();
 		updateWidgetEnablements();
 		hostCombo.setFocus();
 		
 		setControl(composite);
+        Dialog.applyDialogFont(parent);
 	}
 	/**
 	 * Utility method to create an editable combo box
@@ -248,11 +320,10 @@ public class ConfigurationWizardMainPage extends CVSWizardPage {
 		return group;
 	}
 	
-	/**
-	 * @see CVSWizardPage#finish
+	/*
+	 * Create a Proeprties node that contains everything needed to create a repository location
 	 */
-	public boolean finish(IProgressMonitor monitor) {
-		// Set the result to be the current values
+	private Properties createProperties() {
 		Properties result = new Properties();
 		result.setProperty("connection", connectionMethodCombo.getText()); //$NON-NLS-1$
 		result.setProperty("user", userCombo.getText()); //$NON-NLS-1$
@@ -262,20 +333,31 @@ public class ConfigurationWizardMainPage extends CVSWizardPage {
 			result.setProperty("port", portText.getText()); //$NON-NLS-1$
 		}
 		result.setProperty("root", repositoryPathCombo.getText()); //$NON-NLS-1$
-		this.properties = result;
-		
-		saveWidgetValues();
-		
-		return true;
+		return result;
 	}
+	
 	/**
-	 * Returns the properties for the repository connection
-	 * 
-	 * @return the properties or null
+	 * Crate a new location with the information entered on the page.
+	 * The location will exists and can be sed for connecting but is not
+	 * registered for persistance. This method must be called from the UI
+	 * thread.
+	 * @return a location or <code>null</code>
+	 * @throws CVSException
 	 */
-	public Properties getProperties() {
-		return properties;
+	public ICVSRepositoryLocation getLocation() throws CVSException {
+		if (location == null) {
+			if (!isPageComplete()) return null;
+			location = CVSRepositoryLocation.fromProperties(createProperties());
+			if (location.equals(oldLocation)) {
+				location = oldLocation;
+			}
+			location.setAllowCaching(allowCaching);
+			oldLocation = null;
+			saveWidgetValues();
+		}
+		return location;
 	}
+	
 	/**
 	 * Initializes states of the controls.
 	 */
@@ -301,6 +383,7 @@ public class ConfigurationWizardMainPage extends CVSWizardPage {
 					userCombo.add(userNames[i]);
 				}
 			}
+			userCombo.add(ANONYMOUS_USER);
 			if (showValidate) {
 				validate = !settings.getBoolean(STORE_DONT_VALIDATE_ID);
 				validateButton.setSelection(validate);
@@ -317,35 +400,35 @@ public class ConfigurationWizardMainPage extends CVSWizardPage {
 		useDefaultPort.setSelection(true);
 		
 		if(properties != null) {
-			String method = (String)properties.getProperty("connection"); //$NON-NLS-1$
+			String method = properties.getProperty("connection"); //$NON-NLS-1$
 			if (method == null) {
 				connectionMethodCombo.select(0);
 			} else {
 				connectionMethodCombo.select(connectionMethodCombo.indexOf(method));
 			}
 	
-			String user = (String)properties.getProperty("user"); //$NON-NLS-1$
+			String user = properties.getProperty("user"); //$NON-NLS-1$
 			if (user != null) {
 				userCombo.setText(user);
 			}
 	
-			String password = (String)properties.getProperty("password"); //$NON-NLS-1$
+			String password = properties.getProperty("password"); //$NON-NLS-1$
 			if (password != null) {
 				passwordText.setText(password);
 			}
 	
-			String host = (String)properties.getProperty("host"); //$NON-NLS-1$
+			String host = properties.getProperty("host"); //$NON-NLS-1$
 			if (host != null) {
 				hostCombo.setText(host);
 			}
 	
-			String port = (String)properties.getProperty("port"); //$NON-NLS-1$
+			String port = properties.getProperty("port"); //$NON-NLS-1$
 			if (port != null) {
 				useCustomPort.setSelection(true);
 				portText.setText(port);
 			}
 	
-			String repositoryPath = (String)properties.getProperty("root"); //$NON-NLS-1$
+			String repositoryPath = properties.getProperty("root"); //$NON-NLS-1$
 			if (repositoryPath != null) {
 				repositoryPathCombo.setText(repositoryPath);
 			}
@@ -358,11 +441,13 @@ public class ConfigurationWizardMainPage extends CVSWizardPage {
 		// Update history
 		IDialogSettings settings = getDialogSettings();
 		if (settings != null) {
-			String[] userNames = settings.getArray(STORE_USERNAME_ID);
-			if (userNames == null) userNames = new String[0];
-			userNames = addToHistory(userNames, userCombo.getText());
-			settings.put(STORE_USERNAME_ID, userNames);
-
+			String userName = userCombo.getText();
+			if (!userName.equals(ANONYMOUS_USER)) {
+				String[] userNames = settings.getArray(STORE_USERNAME_ID);
+				if (userNames == null) userNames = new String[0];
+				userNames = addToHistory(userNames, userName);
+				settings.put(STORE_USERNAME_ID, userNames);
+			}
 			String[] hostNames = settings.getArray(STORE_HOSTNAME_ID);
 			if (hostNames == null) hostNames = new String[0];
 			hostNames = addToHistory(hostNames, hostCombo.getText());
@@ -410,76 +495,62 @@ public class ConfigurationWizardMainPage extends CVSWizardPage {
 	 */
 	private void validateFields() {
 		String user = userCombo.getText();
-		if (user.length() == 0) {
-			setErrorMessage(null);
-			setPageComplete(false);
-			return;
-		}
-		if ((user.indexOf('@') != -1) || (user.indexOf(':') != -1)) {
-			setErrorMessage(Policy.bind("ConfigurationWizardMainPage.invalidUserName")); //$NON-NLS-1$
-			setPageComplete(false);
+		IStatus status = validateUserName(user);
+		if (!isStatusOK(status)) {
 			return;
 		}
 
 		String host = hostCombo.getText();
-		if (host.length() == 0) {
-			setErrorMessage(null);
-			setPageComplete(false);
-			return;
-		}
-		if (host.indexOf(':') != -1) {
-			setErrorMessage(Policy.bind("ConfigurationWizardMainPage.invalidHostName")); //$NON-NLS-1$
-			setPageComplete(false);
+		status = validateHost(host);
+		if (!isStatusOK(status)) {
 			return;
 		}
 
 		if (portText.isEnabled()) {
-			if (portText.getText().length() == 0) {
-				setErrorMessage(null);
-				setPageComplete(false);
-				return;
-			}
-			try {
-				Integer.parseInt(portText.getText());
-			} catch (NumberFormatException e) {
-				setErrorMessage(Policy.bind("ConfigurationWizardMainPage.invalidPort")); //$NON-NLS-1$
-				setPageComplete(false);
+			String port = portText.getText();
+			status = validatePort(port);
+			if (!isStatusOK(status)) {
 				return;
 			}
 		}
 
-		if (repositoryPathCombo.getText().length() == 0) {
-			setErrorMessage(null);
-			setPageComplete(false);
+		String pathString = repositoryPathCombo.getText();
+		status = validatePath(pathString);
+		if (!isStatusOK(status)) {
 			return;
-		} else {
-			String pathString = repositoryPathCombo.getText();
-			IPath path = new Path(pathString);
-			String[] segments = path.segments();
-			for (int i = 0; i < segments.length; i++) {
-				String string = segments[i];
-				if (string.charAt(0) == ' ' || string.charAt(string.length() -1) == ' ') {
-					setErrorMessage(Policy.bind("ConfigurationWizardMainPage.invalidPathWithSpaces")); //$NON-NLS-1$
-					setPageComplete(false);
-					return;
-				}
-			}
-			// look for // and inform the user that we support use of C:\cvs\root instead of /c//cvs/root
-			if (pathString.indexOf("//") != -1) { //$NON-NLS-1$
-				if (pathString.indexOf("//") == 2) { //$NON-NLS-1$
-					// The user is probably trying to specify a CVSNT path
-					setErrorMessage(Policy.bind("ConfigurationWizardMainPage.useNTFormat")); //$NON-NLS-1$
-				} else {
-					setErrorMessage(Policy.bind("ConfigurationWizardMainPage.invalidPathWithSlashes")); //$NON-NLS-1$
-				}
+		}
+
+		try {
+			CVSRepositoryLocation l = CVSRepositoryLocation.fromProperties(createProperties());
+			if (!l.equals(oldLocation) && KnownRepositories.getInstance().isKnownRepository(l.getLocation())) {
+				setErrorMessage(CVSUIMessages.ConfigurationWizardMainPage_0); 
 				setPageComplete(false);
 				return;
 			}
+		} catch (CVSException e) {
+			CVSUIPlugin.log(e);
+			// Let it pass. Creation should fail
 		}
+		
+		// Everything passed so we're good to go
 		setErrorMessage(null);
 		setPageComplete(true);
 	}
 	
+	private boolean isStatusOK(IStatus status) {
+		if (!status.isOK()) {
+			if (status.getCode() == REQUIRED_FIELD) {
+				// Don't set the message for an empty field
+				setErrorMessage(null);
+			} else {
+				setErrorMessage(status.getMessage());
+			}
+			setPageComplete(false);
+			return false;
+		}
+		return true;
+	}
+	
 	public boolean getValidate() {
 		return validate;
 	}
@@ -489,4 +560,73 @@ public class ConfigurationWizardMainPage extends CVSWizardPage {
 			hostCombo.setFocus();
 		}
 	}
+	
+	public static final int REQUIRED_FIELD = 1;
+	public static final int INVALID_FIELD_CONTENTS = 2;
+	public static final IStatus validateUserName(String user) {
+		if (user.length() == 0) {
+			return new Status(IStatus.ERROR, CVSUIPlugin.ID, REQUIRED_FIELD, CVSUIMessages.ConfigurationWizardMainPage_1, null); 
+		}
+		if ((user.indexOf('@') != -1) || (user.indexOf(':') != -1)) {
+			return new Status(IStatus.ERROR, CVSUIPlugin.ID, INVALID_FIELD_CONTENTS, 
+					CVSUIMessages.ConfigurationWizardMainPage_invalidUserName, null); 
+		}
+		if (user.startsWith(" ") || user.endsWith(" ")) { //$NON-NLS-1$ //$NON-NLS-2$
+			return new Status(IStatus.ERROR, CVSUIPlugin.ID, INVALID_FIELD_CONTENTS, 
+					CVSUIMessages.ConfigurationWizardMainPage_6, null); 
+		}		
+		return Status.OK_STATUS;
+	}
+	public static final IStatus validateHost(String host) {
+		if (host.length() == 0) {
+			return new Status(IStatus.ERROR, CVSUIPlugin.ID, REQUIRED_FIELD, CVSUIMessages.ConfigurationWizardMainPage_2, null); 
+		}
+		if (host.indexOf(':') != -1) {
+			return new Status(IStatus.ERROR, CVSUIPlugin.ID, INVALID_FIELD_CONTENTS, 
+					CVSUIMessages.ConfigurationWizardMainPage_invalidHostName, null); 
+		}
+		if (host.startsWith(" ") || host.endsWith(" ")) { //$NON-NLS-1$ //$NON-NLS-2$
+			return new Status(IStatus.ERROR, CVSUIPlugin.ID, INVALID_FIELD_CONTENTS, 
+					CVSUIMessages.ConfigurationWizardMainPage_5, null); 
+		}
+		return Status.OK_STATUS;
+	}
+	public static final IStatus validatePort(String port) {
+		if (port.length() == 0) {
+			return new Status(IStatus.ERROR, CVSUIPlugin.ID, REQUIRED_FIELD, CVSUIMessages.ConfigurationWizardMainPage_3, null); 
+		}
+		try {
+			Integer.parseInt(port);
+		} catch (NumberFormatException e) {
+			return new Status(IStatus.ERROR, CVSUIPlugin.ID, INVALID_FIELD_CONTENTS, 
+				CVSUIMessages.ConfigurationWizardMainPage_invalidPort, null); 
+		}
+		return Status.OK_STATUS;
+	}
+	public static final IStatus validatePath(String pathString) {
+		if (pathString.length() == 0) {
+			return new Status(IStatus.ERROR, CVSUIPlugin.ID, REQUIRED_FIELD, CVSUIMessages.ConfigurationWizardMainPage_4, null); 
+		}
+		IPath path = new Path(null, pathString);
+		String[] segments = path.segments();
+		for (int i = 0; i < segments.length; i++) {
+			String string = segments[i];
+			if (string.charAt(0) == ' ' || string.charAt(string.length() -1) == ' ') {
+				return new Status(IStatus.ERROR, CVSUIPlugin.ID, INVALID_FIELD_CONTENTS, 
+					CVSUIMessages.ConfigurationWizardMainPage_invalidPathWithSpaces, null); 
+			}
+		}
+		// look for // and inform the user that we support use of C:\cvs\root instead of /c//cvs/root
+		if (pathString.indexOf("//") != -1) { //$NON-NLS-1$
+			if (pathString.indexOf("//") == 2) { //$NON-NLS-1$
+				// The user is probably trying to specify a CVSNT path
+				return new Status(IStatus.ERROR, CVSUIPlugin.ID, INVALID_FIELD_CONTENTS, 
+					CVSUIMessages.ConfigurationWizardMainPage_useNTFormat, null); 
+			} else {
+				return new Status(IStatus.ERROR, CVSUIPlugin.ID, INVALID_FIELD_CONTENTS, 
+					CVSUIMessages.ConfigurationWizardMainPage_invalidPathWithSlashes, null); 
+			}
+		}
+		return Status.OK_STATUS;
+	}
 }
