@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -14,6 +14,7 @@ package org.eclipse.team.internal.ccvs.ui.wizards;
 import org.eclipse.jface.dialogs.Dialog;
 import org.eclipse.jface.resource.ImageDescriptor;
 import org.eclipse.jface.viewers.*;
+import org.eclipse.osgi.util.TextProcessor;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.layout.GridData;
 import org.eclipse.swt.widgets.*;
@@ -28,6 +29,15 @@ import org.eclipse.ui.model.*;
  * If the user selected an existing repo, then getLocation() will return it.
  */
 public class RepositorySelectionPage extends CVSWizardPage {
+	
+	private class DecoratingRepoLabelProvider extends WorkbenchLabelProvider {
+		protected String decorateText(String input, Object element) {
+			//Used to process RTL locales only
+			return TextProcessor.process(input, ":@/"); //$NON-NLS-1$
+		}
+	}
+	
+	
 	private TableViewer table;
 	private Button useExistingRepo;
 	private Button useNewRepo;
@@ -70,16 +80,16 @@ public class RepositorySelectionPage extends CVSWizardPage {
 		// set F1 help
         PlatformUI.getWorkbench().getHelpSystem().setHelp(composite, IHelpContextIds.SHARING_SELECT_REPOSITORY_PAGE);
 		if (extendedDescription == null) {
-			extendedDescription = CVSUIMessages.RepositorySelectionPage_description; //$NON-NLS-1$
+			extendedDescription = CVSUIMessages.RepositorySelectionPage_description; 
 		}
 		createWrappingLabel(composite, extendedDescription, 0 /* indent */, 1 /* columns */); 
 		
-		useNewRepo = createRadioButton(composite, CVSUIMessages.RepositorySelectionPage_useNew, 1); //$NON-NLS-1$
+		useNewRepo = createRadioButton(composite, CVSUIMessages.RepositorySelectionPage_useNew, 1); 
 		
-		useExistingRepo = createRadioButton(composite, CVSUIMessages.RepositorySelectionPage_useExisting, 1); //$NON-NLS-1$
+		useExistingRepo = createRadioButton(composite, CVSUIMessages.RepositorySelectionPage_useExisting, 1); 
 		table = createTable(composite, 1);
 		table.setContentProvider(new WorkbenchContentProvider());
-		table.setLabelProvider(new WorkbenchLabelProvider());
+		table.setLabelProvider(new DecoratingRepoLabelProvider()/*WorkbenchLabelProvider()*/);
         table.addDoubleClickListener(new IDoubleClickListener() {
             public void doubleClick(DoubleClickEvent event) {
                 getContainer().showPage(getNextPage());
