@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
- * 
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,10 +13,16 @@ package org.eclipse.team.internal.ui;
 
 import java.lang.reflect.InvocationTargetException;
 import java.net.URL;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.Hashtable;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
 
 import org.eclipse.core.runtime.*;
 import org.eclipse.core.runtime.jobs.Job;
+import org.eclipse.core.runtime.preferences.InstanceScope;
 import org.eclipse.jface.dialogs.MessageDialogWithToggle;
 import org.eclipse.jface.operation.IRunnableWithProgress;
 import org.eclipse.jface.preference.IPreferenceStore;
@@ -26,11 +32,18 @@ import org.eclipse.jface.util.PropertyChangeEvent;
 import org.eclipse.swt.custom.BusyIndicator;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.widgets.Display;
+import org.eclipse.team.core.RepositoryProviderType;
+import org.eclipse.team.core.subscribers.Subscriber;
+import org.eclipse.team.internal.ui.mapping.StreamMergerDelegate;
+import org.eclipse.team.internal.ui.mapping.WorkspaceTeamStateProvider;
 import org.eclipse.team.internal.ui.synchronize.SynchronizeManager;
 import org.eclipse.team.internal.ui.synchronize.TeamSynchronizingPerspective;
 import org.eclipse.team.internal.ui.synchronize.actions.GlobalRefreshAction;
 import org.eclipse.team.ui.ISharedImages;
 import org.eclipse.team.ui.TeamUI;
+import org.eclipse.team.ui.mapping.ITeamStateProvider;
+import org.eclipse.team.ui.synchronize.TeamStateProvider;
+import org.eclipse.team.ui.synchronize.SubscriberTeamStateProvider;
 import org.eclipse.ui.*;
 import org.eclipse.ui.plugin.AbstractUIPlugin;
 import org.osgi.framework.Bundle;
@@ -56,6 +69,10 @@ public class TeamUIPlugin extends AbstractUIPlugin {
 	
 	private Hashtable imageDescriptors = new Hashtable(20);
 	
+	private WorkspaceTeamStateProvider provider;
+
+	private Map decoratedStateProviders = new HashMap();
+	
 	/**
 	 * Creates a new TeamUIPlugin.
 	 */
@@ -123,9 +140,10 @@ public class TeamUIPlugin extends AbstractUIPlugin {
 	/**
 	 * Initializes the preferences for this plugin if necessary.
 	 */
-	protected void initializePreferences() {
+	protected void initializeDefaultPluginPreferences() {
 		IPreferenceStore store = getPreferenceStore();
 		store.setDefault(IPreferenceIds.SYNCVIEW_VIEW_SYNCINFO_IN_LABEL, false);
+		store.setDefault(IPreferenceIds.SHOW_AUTHOR_IN_COMPARE_EDITOR, false);
 		store.setDefault(IPreferenceIds.SYNCVIEW_COMPRESS_FOLDERS, true);
 		store.setDefault(IPreferenceIds.SYNCVIEW_DEFAULT_LAYOUT, IPreferenceIds.COMPRESSED_LAYOUT);
 		store.setDefault(IPreferenceIds.SYNCVIEW_DEFAULT_PERSPECTIVE, TeamSynchronizingPerspective.ID);
@@ -178,7 +196,6 @@ public class TeamUIPlugin extends AbstractUIPlugin {
 		super.start(context);
 		
 		initializeImages(this);
-		initializePreferences();
 
 		// This is a backwards compatibility check to ensure that repository
 		// provider capability are enabled automatically if an old workspace is
@@ -202,6 +219,8 @@ public class TeamUIPlugin extends AbstractUIPlugin {
 			capabilityInitializer.setPriority(Job.DECORATE);
 			capabilityInitializer.schedule(1000);		
 		}
+		
+		StreamMergerDelegate.start();
 	}
 	
 	/* (non-Javadoc)
@@ -213,6 +232,13 @@ public class TeamUIPlugin extends AbstractUIPlugin {
 		} finally {
 			super.stop(context);
 		}
+		if (provider != null) {
+			provider.dispose();
+		}
+		for (Iterator iter = decoratedStateProviders.values().iterator(); iter.hasNext();) {
+			SubscriberTeamStateProvider sdsp = (SubscriberTeamStateProvider) iter.next();
+			sdsp.dispose();
+		}
 	}
 
 	/**
@@ -339,6 +365,11 @@ public class TeamUIPlugin extends AbstractUIPlugin {
 		createImageDescriptor(plugin, ITeamUIImages.IMG_COMPRESSED_FOLDER);
 		createImageDescriptor(plugin, ITeamUIImages.IMG_SYNC_VIEW);
 		createImageDescriptor(plugin, ITeamUIImages.IMG_HIERARCHICAL);		
+		
+		// Local History Page
+		createImageDescriptor(plugin, ITeamUIImages.IMG_DATES_CATEGORY);
+		createImageDescriptor(plugin, ITeamUIImages.IMG_COMPARE_VIEW);
+		createImageDescriptor(plugin, ITeamUIImages.IMG_LOCALREVISION_TABLE);
 	}
 
     private URL getImageUrl(String relative) {
@@ -376,5 +407,34 @@ public class TeamUIPlugin extends AbstractUIPlugin {
 		} catch (InterruptedException e2) {
 			// Nothing to be done
 		}
-	}	
+	}
+
+	public org.osgi.service.prefs.Preferences getInstancePreferences() {
+		return new InstanceScope().getNode(getBundle().getSymbolicName());
+	}
+
+	public synchronized TeamStateProvider getDecoratedStateProvider(RepositoryProviderType rpt) {
+		TeamStateProvider provider = (TeamStateProvider)decoratedStateProviders.get(rpt.getID());
+		if (provider != null)
+			return provider;
+		Subscriber subscriber = rpt.getSubscriber();
+		if (subscriber != null) {
+			provider = new SubscriberTeamStateProvider(subscriber);
+			decoratedStateProviders.put(rpt.getID(), provider);
+			return provider;
+		}
+		return null;
+	}
+	
+	/**
+	 * Return a decorated state provider that delegates to the appropriate team 
+	 * provider.
+	 * @return a decorated state provider that delegates to the appropriate team 
+	 * provider
+	 */
+	public synchronized ITeamStateProvider getDecoratedStateProvider() {
+		if (provider == null)
+			provider = new WorkspaceTeamStateProvider();
+		return provider;
+	}
 }
