@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -23,6 +23,7 @@ import org.eclipse.core.runtime.ILog;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.MultiStatus;
 import org.eclipse.core.runtime.Platform;
 import org.eclipse.core.runtime.QualifiedName;
 import org.eclipse.core.runtime.Status;
@@ -30,6 +31,7 @@ import org.eclipse.core.runtime.content.IContentDescription;
 import org.eclipse.core.runtime.content.IContentType;
 
 import org.eclipse.core.resources.IEncodedStorage;
+import org.eclipse.core.resources.IResourceStatus;
 import org.eclipse.core.resources.IStorage;
 import org.eclipse.core.resources.ResourcesPlugin;
 
@@ -291,7 +293,7 @@ public class StorageDocumentProvider extends AbstractDocumentProvider implements
 		ILog log= Platform.getLog(bundle);
 
 		if (message != null)
-			log.log(new Status(IStatus.ERROR, PlatformUI.PLUGIN_ID, 0, message, exception));
+			log.log(new Status(IStatus.ERROR, PlatformUI.PLUGIN_ID, IStatus.OK, message, exception));
 		else
 			log.log(exception.getStatus());
 	}
@@ -341,6 +343,21 @@ public class StorageDocumentProvider extends AbstractDocumentProvider implements
 		return super.isReadOnly(element);
 	}
 
+	/*
+	 * @see org.eclipse.ui.texteditor.IDocumentProviderExtension5#isNotSynchronizedException(Object, CoreException)
+	 * @since 3.2
+	 */
+	public boolean isNotSynchronizedException(Object element, CoreException ex) {
+		IStatus status= ex.getStatus(); 
+		if (status == null || status instanceof MultiStatus)
+			return false;
+		
+		if (status.getException() != null)
+			return false;
+		
+		return status.getCode() == IResourceStatus.OUT_OF_SYNC_LOCAL;
+	}
+
 	/*
 	 * @see IDocumentProviderExtension#isModifiable(Object)
 	 * @since 2.0
