@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -15,8 +15,16 @@ import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.Reader;
-import java.io.UnsupportedEncodingException;
 import java.lang.reflect.InvocationTargetException;
+import java.nio.ByteBuffer;
+import java.nio.CharBuffer;
+import java.nio.charset.CharacterCodingException;
+import java.nio.charset.Charset;
+import java.nio.charset.CharsetEncoder;
+import java.nio.charset.CodingErrorAction;
+import java.nio.charset.IllegalCharsetNameException;
+import java.nio.charset.UnmappableCharacterException;
+import java.nio.charset.UnsupportedCharsetException;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -27,25 +35,17 @@ import java.util.NoSuchElementException;
 
 import org.osgi.framework.Bundle;
 
-import org.eclipse.core.filebuffers.FileBuffers;
-import org.eclipse.core.filebuffers.IFileBuffer;
-import org.eclipse.core.filebuffers.IFileBufferListener;
-import org.eclipse.core.filebuffers.IFileBufferManager;
-import org.eclipse.core.filebuffers.ITextFileBuffer;
-import org.eclipse.core.filebuffers.ITextFileBufferManager;
-import org.eclipse.core.filebuffers.manipulation.ContainerCreator;
-import org.eclipse.core.resources.IFile;
-import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.IResourceRuleFactory;
-import org.eclipse.core.resources.IResourceStatus;
-import org.eclipse.core.resources.IWorkspace;
-import org.eclipse.core.resources.ResourcesPlugin;
+import org.eclipse.core.filesystem.EFS;
+import org.eclipse.core.filesystem.IFileInfo;
+import org.eclipse.core.filesystem.IFileStore;
+
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.ILog;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.MultiStatus;
 import org.eclipse.core.runtime.NullProgressMonitor;
 import org.eclipse.core.runtime.Platform;
 import org.eclipse.core.runtime.QualifiedName;
@@ -55,19 +55,36 @@ import org.eclipse.core.runtime.content.IContentDescription;
 import org.eclipse.core.runtime.content.IContentType;
 import org.eclipse.core.runtime.jobs.ISchedulingRule;
 
-import org.eclipse.jface.text.Assert;
-import org.eclipse.jface.text.IDocument;
-import org.eclipse.jface.text.source.IAnnotationModel;
+import org.eclipse.core.resources.IFile;
+import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.IResourceRuleFactory;
+import org.eclipse.core.resources.IResourceStatus;
+import org.eclipse.core.resources.IWorkspace;
+import org.eclipse.core.resources.ResourcesPlugin;
 
-import org.eclipse.ui.internal.editors.text.NLSUtility;
-import org.eclipse.ui.internal.editors.text.UISynchronizationContext;
-import org.eclipse.ui.internal.editors.text.WorkspaceOperationRunner;
+import org.eclipse.core.filebuffers.FileBuffers;
+import org.eclipse.core.filebuffers.IFileBuffer;
+import org.eclipse.core.filebuffers.IFileBufferListener;
+import org.eclipse.core.filebuffers.IFileBufferManager;
+import org.eclipse.core.filebuffers.ITextFileBuffer;
+import org.eclipse.core.filebuffers.ITextFileBufferManager;
+import org.eclipse.core.filebuffers.manipulation.ContainerCreator;
 
 import org.eclipse.jface.operation.IRunnableContext;
 import org.eclipse.jface.operation.IRunnableWithProgress;
+
+import org.eclipse.jface.text.Assert;
+import org.eclipse.jface.text.IDocument;
+import org.eclipse.jface.text.ISynchronizable;
+import org.eclipse.jface.text.source.IAnnotationModel;
+
 import org.eclipse.ui.IEditorInput;
 import org.eclipse.ui.IFileEditorInput;
 import org.eclipse.ui.PlatformUI;
+import org.eclipse.ui.internal.editors.text.JavaFileEditorInput;
+import org.eclipse.ui.internal.editors.text.NLSUtility;
+import org.eclipse.ui.internal.editors.text.UISynchronizationContext;
+import org.eclipse.ui.internal.editors.text.WorkspaceOperationRunner;
 import org.eclipse.ui.part.FileEditorInput;
 import org.eclipse.ui.texteditor.AbstractMarkerAnnotationModel;
 import org.eclipse.ui.texteditor.IDocumentProvider;
@@ -75,6 +92,7 @@ import org.eclipse.ui.texteditor.IDocumentProviderExtension;
 import org.eclipse.ui.texteditor.IDocumentProviderExtension2;
 import org.eclipse.ui.texteditor.IDocumentProviderExtension3;
 import org.eclipse.ui.texteditor.IDocumentProviderExtension4;
+import org.eclipse.ui.texteditor.IDocumentProviderExtension5;
 import org.eclipse.ui.texteditor.IElementStateListener;
 import org.eclipse.ui.texteditor.IElementStateListenerExtension;
 import org.eclipse.ui.texteditor.ISchedulingRuleProvider;
@@ -103,7 +121,7 @@ import org.eclipse.ui.texteditor.ISchedulingRuleProvider;
  *
  * @since 3.0
  */
-public class TextFileDocumentProvider implements IDocumentProvider, IDocumentProviderExtension, IDocumentProviderExtension2, IDocumentProviderExtension3, IStorageDocumentProvider, IDocumentProviderExtension4 {
+public class TextFileDocumentProvider implements IDocumentProvider, IDocumentProviderExtension, IDocumentProviderExtension2, IDocumentProviderExtension3, IDocumentProviderExtension5, IStorageDocumentProvider, IDocumentProviderExtension4 {
 
 	/**
 	 * Operation created by the document provider and to be executed by the providers runnable context.
@@ -137,9 +155,9 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 		}
 	}
 
-	static protected class NullProvider implements IDocumentProvider, IDocumentProviderExtension, IDocumentProviderExtension2, IDocumentProviderExtension3, IDocumentProviderExtension4, IStorageDocumentProvider  {
+	static protected class NullProvider implements IDocumentProvider, IDocumentProviderExtension, IDocumentProviderExtension2, IDocumentProviderExtension3, IDocumentProviderExtension4, IDocumentProviderExtension5, IStorageDocumentProvider  {
 
-		static final private IStatus STATUS_ERROR= new Status(IStatus.ERROR, EditorsUI.PLUGIN_ID, IStatus.INFO, TextEditorMessages.NullProvider_error, null);
+		static final private IStatus STATUS_ERROR= new Status(IStatus.ERROR, EditorsUI.PLUGIN_ID, IStatus.OK, TextEditorMessages.NullProvider_error, null);
 
 		public void connect(Object element) throws CoreException {}
 		public void disconnect(Object element) {}
@@ -167,6 +185,7 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 		public void setProgressMonitor(IProgressMonitor progressMonitor) {}
 		public IProgressMonitor getProgressMonitor() { return new NullProgressMonitor(); }
 		public boolean isSynchronized(Object element) { return true; }
+		public boolean isNotSynchronizedException(Object element, CoreException ex) { return false; }
 		public String getDefaultEncoding() { return null; }
 		public String getEncoding(Object element) { return null; }
 		public void setEncoding(Object element, String encoding) {}
@@ -446,7 +465,7 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 			if (e instanceof CoreException)
 				throw (CoreException) e;
 			String message= (e.getMessage() != null ? e.getMessage() : ""); //$NON-NLS-1$
-			throw new CoreException(new Status(IStatus.ERROR, EditorsUI.PLUGIN_ID, IStatus.ERROR, message, e));
+			throw new CoreException(new Status(IStatus.ERROR, EditorsUI.PLUGIN_ID, IStatus.OK, message, e));
 		} catch (InterruptedException x) {
 			String message= (x.getMessage() != null ? x.getMessage() : ""); //$NON-NLS-1$
 			throw new CoreException(new Status(IStatus.CANCEL, EditorsUI.PLUGIN_ID, IStatus.OK, message, x));
@@ -554,6 +573,31 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 		}
 		return null;
 	}
+	
+	/**
+	 * Sets up the synchronization for the document
+	 * and the annotation mode. 
+	 * 
+	 * @param info the file info
+	 * @since 3.2
+	 */
+	protected void setUpSynchronization(FileInfo info) {
+		if (info == null || info.fTextFileBuffer == null)
+			return;
+		
+		IDocument document= info.fTextFileBuffer.getDocument();
+		IAnnotationModel model= info.fModel;
+
+		if (document instanceof ISynchronizable) {
+			Object lock= ((ISynchronizable)document).getLockObject();
+			if (lock == null) {
+				lock= new Object();
+				((ISynchronizable)document).setLockObject(lock);
+			}
+			if (model instanceof ISynchronizable)
+				((ISynchronizable) model).setLockObject(lock);
+		}
+	}
 
 	/**
 	 * Creates and returns the annotation model for the given file.
@@ -676,7 +720,7 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 				public ISchedulingRule getSchedulingRule() {
 					if (info.fElement instanceof IFileEditorInput) {
 						IFileEditorInput input= (IFileEditorInput) info.fElement;
-						return fResourceRuleFactory.modifyRule((input).getFile());
+						return fResourceRuleFactory.refreshRule((input).getFile());
 					}
 					return null;
 				}
@@ -711,7 +755,7 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 				// -> saveAs was executed with a target that is already open
 				// in another editor
 				// see https://bugs.eclipse.org/bugs/show_bug.cgi?id=85519
-				Status status= new Status(IStatus.WARNING, EditorsUI.PLUGIN_ID, IStatus.ERROR, TextEditorMessages.TextFileDocumentProvider_saveAsTargetOpenInEditor, null);
+				Status status= new Status(IStatus.WARNING, EditorsUI.PLUGIN_ID, IStatus.OK, TextEditorMessages.TextFileDocumentProvider_saveAsTargetOpenInEditor, null);
 				throw new CoreException(status);
 			}
 
@@ -751,6 +795,23 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 					return computeSchedulingRule(file);
 				}
 			};
+		} else if (element instanceof JavaFileEditorInput) {
+
+			final IPath fileLocation= ((JavaFileEditorInput)element).getPath();
+			return new DocumentProviderOperation() {
+				/*
+				 * @see org.eclipse.ui.editors.text.TextFileDocumentProvider.DocumentProviderOperation#execute(org.eclipse.core.runtime.IProgressMonitor)
+				 */
+				public void execute(IProgressMonitor monitor) throws CoreException {
+					createJavaFileFromDocument(monitor, fileLocation, document);
+				}
+				/*
+				 * @see org.eclipse.ui.editors.text.TextFileDocumentProvider.DocumentProviderOperation#getSchedulingRule()
+				 */
+				public ISchedulingRule getSchedulingRule() {
+					return null;
+				}
+			};
 		}
 
 		return null;
@@ -798,17 +859,71 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 		String encoding= getCharsetForNewFile(file, document);
 		try {
 			monitor.beginTask(TextEditorMessages.TextFileDocumentProvider_beginTask_saving, 2000);
-			InputStream stream= new ByteArrayInputStream(document.get().getBytes(encoding));
+			
+			Charset charset;
+			try {
+				charset= Charset.forName(encoding);
+			} catch (UnsupportedCharsetException ex) {
+				String message= NLSUtility.format(TextEditorMessages.DocumentProvider_error_unsupported_encoding_message_arg, encoding);
+				IStatus s= new Status(IStatus.ERROR, EditorsUI.PLUGIN_ID, IStatus.OK, message, ex);
+				throw new CoreException(s);
+			} catch (IllegalCharsetNameException ex) {
+				String message= NLSUtility.format(TextEditorMessages.DocumentProvider_error_illegal_encoding_message_arg, encoding);
+				IStatus s= new Status(IStatus.ERROR, EditorsUI.PLUGIN_ID, IStatus.OK, message, ex);
+				throw new CoreException(s);
+			}
+
+			CharsetEncoder encoder= charset.newEncoder();
+			encoder.onMalformedInput(CodingErrorAction.REPLACE);
+			encoder.onUnmappableCharacter(CodingErrorAction.REPORT);
+
+			InputStream stream;
+			
+			try {
+				byte[] bytes;
+				ByteBuffer byteBuffer= encoder.encode(CharBuffer.wrap(document.get()));
+				if (byteBuffer.hasArray())
+					bytes= byteBuffer.array();
+				else {
+					bytes= new byte[byteBuffer.limit()];
+					byteBuffer.get(bytes);
+				}
+				stream= new ByteArrayInputStream(bytes, 0, byteBuffer.limit());
+			} catch (CharacterCodingException ex) {
+				Assert.isTrue(ex instanceof UnmappableCharacterException);
+				String message= NLSUtility.format(TextEditorMessages.DocumentProvider_error_charset_mapping_failed_message_arg, encoding);
+				IStatus s= new Status(IStatus.ERROR, EditorsUI.PLUGIN_ID, EditorsUI.CHARSET_MAPPING_FAILED, message, null);
+				throw new CoreException(s);
+			}
+			
 			if (!file.exists()) {
 				ContainerCreator creator= new ContainerCreator(file.getWorkspace(), file.getParent().getFullPath());
 				creator.createContainer(new SubProgressMonitor(monitor, 1000));
 				file.create(stream, false, new SubProgressMonitor(monitor, 1000));
 			} else
 				file.setContents(stream, false, false, new SubProgressMonitor(monitor, 1000));
-		} catch (UnsupportedEncodingException x) {
-			String message= NLSUtility.format(TextEditorMessages.Editor_error_unsupported_encoding_message_arg, encoding);
-			IStatus s= new Status(IStatus.ERROR, EditorsUI.PLUGIN_ID, IStatus.OK, message, x);
-			throw new CoreException(s);
+		} finally {
+			monitor.done();
+		}
+	}
+
+	/**
+	 * Creates the given file with the given document content.
+	 *
+	 * @param monitor the progress monitor
+	 * @param location the location where the external file should be created
+	 * @param document the document to be written to the file
+	 * @throws CoreException if the creation of the file fails
+	 * @since 3.2
+	 */
+	private void createJavaFileFromDocument(IProgressMonitor monitor, IPath location, IDocument document) throws CoreException {
+		try {
+			monitor.beginTask(TextEditorMessages.TextFileDocumentProvider_beginTask_saving, 2000);
+			FileBuffers.getTextFileBufferManager().connect(location, monitor);
+			ITextFileBuffer buffer= FileBuffers.getTextFileBufferManager().getTextFileBuffer(location);
+			buffer.getDocument().set(document.get());
+			buffer.commit(monitor, true);
+			FileBuffers.getTextFileBufferManager().disconnect(location, monitor);
 		} finally {
 			monitor.done();
 		}
@@ -839,8 +954,7 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 			// continue with next strategy
 		} finally {
 			try {
-				if (reader != null)
-					reader.close();
+				reader.close();
 			} catch (IOException x) {
 			}
 		}
@@ -880,8 +994,8 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 	public boolean isDeleted(Object element) {
 		FileInfo info= (FileInfo) fFileInfoMap.get(element);
 		if (info != null)  {
-			File file= getSystemFile(info);
-			return file == null ? true : !file.exists();
+			IFileStore fileStore= getFileStore(info);
+			return fileStore == null ? true : !fileStore.fetchInfo().exists();
 		}
 		return getParentProvider().isDeleted(element);
 	}
@@ -1126,6 +1240,21 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 			return info.fTextFileBuffer.isSynchronized();
 		return ((IDocumentProviderExtension3) getParentProvider()).isSynchronized(element);
 	}
+	
+	/*
+	 * @see org.eclipse.ui.texteditor.IDocumentProviderExtension5#isNotSynchronizedException(Object, CoreException)
+	 * @since 3.2
+	 */
+	public boolean isNotSynchronizedException(Object element, CoreException ex) {
+		IStatus status= ex.getStatus(); 
+		if (status == null || status instanceof MultiStatus)
+			return false;
+		
+		if (status.getException() != null)
+			return false;
+		
+		return status.getCode() == IResourceStatus.OUT_OF_SYNC_LOCAL;
+	}
 
 	/*
 	 * @see org.eclipse.ui.editors.text.IStorageDocumentProvider#getDefaultEncoding()
@@ -1179,15 +1308,28 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 	protected void handleCoreException(CoreException exception, String message) {
 		Bundle bundle = Platform.getBundle(PlatformUI.PLUGIN_ID);
 		ILog log= Platform.getLog(bundle);
-		IStatus status= message != null ? new Status(IStatus.ERROR, PlatformUI.PLUGIN_ID, 0, message, exception) : exception.getStatus();
+		IStatus status= message != null ? new Status(IStatus.ERROR, PlatformUI.PLUGIN_ID, IStatus.OK, message, exception) : exception.getStatus();
 		log.log(status);
 	}
 
+	/**
+	 * Returns the file store denoted by the given info.
+	 *
+	 * @param info the element's file info object
+	 * @return the {@link IFileStore} for the given file info
+	 * @since 3.2
+	 */
+	protected IFileStore getFileStore(FileInfo info)  {
+		IPath path= info.fTextFileBuffer.getLocation();
+		return FileBuffers.getFileStoreAtLocation(path);
+	}
+	
 	/**
 	 * Returns the system file denoted by the given info.
 	 *
 	 * @param info the element's file info object
-	 * @return the system for the given file info
+	 * @return the system file for the given file info
+	 * @deprecated As of 3.2, replaced by {@link #getFileStore(org.eclipse.ui.editors.text.TextFileDocumentProvider.FileInfo)}
 	 */
 	protected File getSystemFile(FileInfo info)  {
 		IPath path= info.fTextFileBuffer.getLocation();
@@ -1202,8 +1344,11 @@ public class TextFileDocumentProvider implements IDocumentProvider, IDocumentPro
 	 * @return <code>true</code> iff read-only
 	 */
 	protected boolean isSystemFileReadOnly(FileInfo info)  {
-		File file= getSystemFile(info);
-		return file != null && file.exists() ? !file.canWrite() : false;
+		IFileStore fileStore= getFileStore(info);
+		if (fileStore == null)
+			return false;
+		IFileInfo fileInfo= fileStore.fetchInfo();
+		return fileInfo.exists() && fileInfo.getAttribute(EFS.ATTRIBUTE_READ_ONLY);
 	}
 
 	/**
