@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -19,6 +19,7 @@ import org.eclipse.swt.events.KeyAdapter;
 import org.eclipse.swt.events.KeyEvent;
 import org.eclipse.swt.events.MouseAdapter;
 import org.eclipse.swt.events.MouseEvent;
+import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.Listener;
 import org.eclipse.swt.widgets.Text;
@@ -121,8 +122,9 @@ public class TextActionHandler {
         }
 
         public void propertyChange(PropertyChangeEvent event) {
-            if (activeTextControl != null)
-                return;
+            if (activeTextControl != null) {
+				return;
+			}
             if (event.getProperty().equals(IAction.ENABLED)) {
                 Boolean bool = (Boolean) event.getNewValue();
                 actionHandler.setEnabled(bool.booleanValue());
@@ -141,9 +143,22 @@ public class TextActionHandler {
 
         public void runWithEvent(Event event) {
             if (activeTextControl != null && !activeTextControl.isDisposed()) {
-                activeTextControl.clearSelection();
-                return;
-            }
+				String text = activeTextControl.getText();
+				Point selection = activeTextControl.getSelection();
+				if (selection.y == selection.x) {
+					++selection.y;
+				}
+				if (selection.y > text.length()) {
+					return;
+				}
+				StringBuffer buf = new StringBuffer(text.substring(0,
+						selection.x));
+				buf.append(text.substring(selection.y));
+				activeTextControl.setText(buf.toString());
+				activeTextControl.setSelection(selection.x, selection.x);
+				updateActionsEnableState();
+				return;
+			}
             if (deleteAction != null) {
                 deleteAction.runWithEvent(event);
                 return;
@@ -180,6 +195,7 @@ public class TextActionHandler {
         public void runWithEvent(Event event) {
             if (activeTextControl != null && !activeTextControl.isDisposed()) {
                 activeTextControl.cut();
+                updateActionsEnableState();
                 return;
             }
             if (cutAction != null) {
@@ -216,6 +232,7 @@ public class TextActionHandler {
         public void runWithEvent(Event event) {
             if (activeTextControl != null && !activeTextControl.isDisposed()) {
                 activeTextControl.copy();
+                updateActionsEnableState();
                 return;
             }
             if (copyAction != null) {
@@ -252,6 +269,7 @@ public class TextActionHandler {
         public void runWithEvent(Event event) {
             if (activeTextControl != null && !activeTextControl.isDisposed()) {
                 activeTextControl.paste();
+                updateActionsEnableState();
                 return;
             }
             if (pasteAction != null) {
@@ -288,6 +306,7 @@ public class TextActionHandler {
         public void runWithEvent(Event event) {
             if (activeTextControl != null && !activeTextControl.isDisposed()) {
                 activeTextControl.selectAll();
+                updateActionsEnableState();
                 return;
             }
             if (selectAllAction != null) {
@@ -343,8 +362,9 @@ public class TextActionHandler {
      * @param textControl the inline <code>Text</code> control
      */
     public void addText(Text textControl) {
-        if (textControl == null)
-            return;
+        if (textControl == null) {
+			return;
+		}
 
         activeTextControl = textControl;
         textControl.addListener(SWT.Activate, textControlListener);
@@ -377,8 +397,9 @@ public class TextActionHandler {
      * @param textControl the inline <code>Text</code> control
      */
     public void removeText(Text textControl) {
-        if (textControl == null)
-            return;
+        if (textControl == null) {
+			return;
+		}
 
         textControl.removeListener(SWT.Activate, textControlListener);
         textControl.removeListener(SWT.Deactivate, textControlListener);
@@ -396,19 +417,22 @@ public class TextActionHandler {
      * inline text control.
      *
      * @param action the <code>IAction</code> to run for the
-     *    Copy action, or <code>null</null> if not interested.
+     *    Copy action, or <code>null</code> if not interested.
      */
     public void setCopyAction(IAction action) {
-        if (copyAction == action)
-            return;
+        if (copyAction == action) {
+			return;
+		}
 
-        if (copyAction != null)
-            copyAction.removePropertyChangeListener(copyActionListener);
+        if (copyAction != null) {
+			copyAction.removePropertyChangeListener(copyActionListener);
+		}
 
         copyAction = action;
 
-        if (copyAction != null)
-            copyAction.addPropertyChangeListener(copyActionListener);
+        if (copyAction != null) {
+			copyAction.addPropertyChangeListener(copyActionListener);
+		}
 
         textCopyAction.updateEnabledState();
     }
@@ -419,19 +443,22 @@ public class TextActionHandler {
      * inline text control.
      *
      * @param action the <code>IAction</code> to run for the
-     *    Cut action, or <code>null</null> if not interested.
+     *    Cut action, or <code>null</code> if not interested.
      */
     public void setCutAction(IAction action) {
-        if (cutAction == action)
-            return;
+        if (cutAction == action) {
+			return;
+		}
 
-        if (cutAction != null)
-            cutAction.removePropertyChangeListener(cutActionListener);
+        if (cutAction != null) {
+			cutAction.removePropertyChangeListener(cutActionListener);
+		}
 
         cutAction = action;
 
-        if (cutAction != null)
-            cutAction.addPropertyChangeListener(cutActionListener);
+        if (cutAction != null) {
+			cutAction.addPropertyChangeListener(cutActionListener);
+		}
 
         textCutAction.updateEnabledState();
     }
@@ -442,19 +469,22 @@ public class TextActionHandler {
      * inline text control.
      *
      * @param action the <code>IAction</code> to run for the
-     *    Paste action, or <code>null</null> if not interested.
+     *    Paste action, or <code>null</code> if not interested.
      */
     public void setPasteAction(IAction action) {
-        if (pasteAction == action)
-            return;
+        if (pasteAction == action) {
+			return;
+		}
 
-        if (pasteAction != null)
-            pasteAction.removePropertyChangeListener(pasteActionListener);
+        if (pasteAction != null) {
+			pasteAction.removePropertyChangeListener(pasteActionListener);
+		}
 
         pasteAction = action;
 
-        if (pasteAction != null)
-            pasteAction.addPropertyChangeListener(pasteActionListener);
+        if (pasteAction != null) {
+			pasteAction.addPropertyChangeListener(pasteActionListener);
+		}
 
         textPasteAction.updateEnabledState();
     }
@@ -465,20 +495,23 @@ public class TextActionHandler {
      * inline text control.
      *
      * @param action the <code>IAction</code> to run for the
-     *    Select All action, or <code>null</null> if not interested.
+     *    Select All action, or <code>null</code> if not interested.
      */
     public void setSelectAllAction(IAction action) {
-        if (selectAllAction == action)
-            return;
+        if (selectAllAction == action) {
+			return;
+		}
 
-        if (selectAllAction != null)
-            selectAllAction
+        if (selectAllAction != null) {
+			selectAllAction
                     .removePropertyChangeListener(selectAllActionListener);
+		}
 
         selectAllAction = action;
 
-        if (selectAllAction != null)
-            selectAllAction.addPropertyChangeListener(selectAllActionListener);
+        if (selectAllAction != null) {
+			selectAllAction.addPropertyChangeListener(selectAllActionListener);
+		}
 
         textSelectAllAction.updateEnabledState();
     }
@@ -489,19 +522,22 @@ public class TextActionHandler {
      * inline text control.
      *
      * @param action the <code>IAction</code> to run for the
-     *    Delete action, or <code>null</null> if not interested.
+     *    Delete action, or <code>null</code> if not interested.
      */
     public void setDeleteAction(IAction action) {
-        if (deleteAction == action)
-            return;
+        if (deleteAction == action) {
+			return;
+		}
 
-        if (deleteAction != null)
-            deleteAction.removePropertyChangeListener(deleteActionListener);
+        if (deleteAction != null) {
+			deleteAction.removePropertyChangeListener(deleteActionListener);
+		}
 
         deleteAction = action;
 
-        if (deleteAction != null)
-            deleteAction.addPropertyChangeListener(deleteActionListener);
+        if (deleteAction != null) {
+			deleteAction.addPropertyChangeListener(deleteActionListener);
+		}
 
         textDeleteAction.updateEnabledState();
     }
