@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2003, 2005 IBM Corporation and others.
+ * Copyright (c) 2003, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -18,14 +18,28 @@ import java.util.List;
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IMarker;
 import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.IResourceDelta;
+import org.eclipse.core.resources.mapping.IModelProviderDescriptor;
+import org.eclipse.core.resources.mapping.IResourceChangeDescriptionFactory;
+import org.eclipse.core.resources.mapping.ModelProvider;
+import org.eclipse.core.resources.mapping.ModelStatus;
+import org.eclipse.core.resources.mapping.ResourceChangeValidator;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IAdaptable;
+import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.MultiStatus;
 import org.eclipse.core.runtime.Platform;
 import org.eclipse.core.runtime.QualifiedName;
 import org.eclipse.core.runtime.content.IContentDescription;
 import org.eclipse.core.runtime.content.IContentType;
+import org.eclipse.core.runtime.content.IContentTypeMatcher;
+import org.eclipse.jface.dialogs.ErrorDialog;
+import org.eclipse.jface.dialogs.IDialogConstants;
 import org.eclipse.jface.util.SafeRunnable;
 import org.eclipse.jface.viewers.IStructuredSelection;
+import org.eclipse.osgi.util.NLS;
+import org.eclipse.swt.widgets.Composite;
+import org.eclipse.swt.widgets.Shell;
 import org.eclipse.ui.IEditorDescriptor;
 import org.eclipse.ui.IEditorInput;
 import org.eclipse.ui.IEditorPart;
@@ -268,8 +282,9 @@ public final class IDE {
     public static IEditorPart openEditor(IWorkbenchPage page,
             IEditorInput input, String editorId) throws PartInitException {
         //sanity checks
-        if (page == null)
-            throw new IllegalArgumentException();
+        if (page == null) {
+			throw new IllegalArgumentException();
+		}
 
         // open the editor on the file
         return page.openEditor(input, editorId);
@@ -300,8 +315,9 @@ public final class IDE {
             IEditorInput input, String editorId, boolean activate)
             throws PartInitException {
         //sanity checks
-        if (page == null)
-            throw new IllegalArgumentException();
+        if (page == null) {
+			throw new IllegalArgumentException();
+		}
 
         // open the editor on the file
         return page.openEditor(input, editorId, activate);
@@ -363,8 +379,9 @@ public final class IDE {
 	public static IEditorPart openEditor(IWorkbenchPage page, IFile input,
             boolean activate, boolean determineContentType) throws PartInitException {
         //sanity checks
-        if (page == null)
-            throw new IllegalArgumentException();
+        if (page == null) {
+			throw new IllegalArgumentException();
+		}
 
         // open the editor on the file
         IEditorDescriptor editorDesc = getEditorDescriptor(input, determineContentType);
@@ -393,8 +410,9 @@ public final class IDE {
     public static IEditorPart openEditor(IWorkbenchPage page, IFile input)
             throws PartInitException {
         //sanity checks
-        if (page == null)
-            throw new IllegalArgumentException();
+        if (page == null) {
+			throw new IllegalArgumentException();
+		}
 
         // open the editor on the file
         IEditorDescriptor editorDesc = getEditorDescriptor(input);
@@ -422,8 +440,9 @@ public final class IDE {
     public static IEditorPart openEditor(IWorkbenchPage page, IFile input,
             String editorId) throws PartInitException {
         //sanity checks
-        if (page == null)
-            throw new IllegalArgumentException();
+        if (page == null) {
+			throw new IllegalArgumentException();
+		}
 
         // open the editor on the file
         return page.openEditor(new FileEditorInput(input), editorId);
@@ -453,8 +472,9 @@ public final class IDE {
     public static IEditorPart openEditor(IWorkbenchPage page, IFile input,
             String editorId, boolean activate) throws PartInitException {
         //sanity checks
-        if (page == null)
-            throw new IllegalArgumentException();
+        if (page == null) {
+			throw new IllegalArgumentException();
+		}
 
         // open the editor on the file
         return page.openEditor(new FileEditorInput(input), editorId, activate);
@@ -640,30 +660,35 @@ public final class IDE {
 			IEditorRegistry editorReg, IEditorDescriptor defaultDescriptor)
 			throws PartInitException {
 
-		if (defaultDescriptor != null)
+		if (defaultDescriptor != null) {
 			return defaultDescriptor;
+		}
 
 		IEditorDescriptor editorDesc = defaultDescriptor;
 
 		// next check the OS for in-place editor (OLE on Win32)
-		if (editorReg.isSystemInPlaceEditorAvailable(name))
+		if (editorReg.isSystemInPlaceEditorAvailable(name)) {
 			editorDesc = editorReg
 					.findEditor(IEditorRegistry.SYSTEM_INPLACE_EDITOR_ID);
+		}
 
 		// next check with the OS for an external editor
 		if (editorDesc == null
-				&& editorReg.isSystemExternalEditorAvailable(name))
+				&& editorReg.isSystemExternalEditorAvailable(name)) {
 			editorDesc = editorReg
 					.findEditor(IEditorRegistry.SYSTEM_EXTERNAL_EDITOR_ID);
+		}
 
 		// next lookup the default text editor
-		if (editorDesc == null)
+		if (editorDesc == null) {
 			editorDesc = editorReg
 					.findEditor(IDEWorkbenchPlugin.DEFAULT_TEXT_EDITOR_ID);
+		}
 
 		// if no valid editor found, bail out
-		if (editorDesc == null)
+		if (editorDesc == null) {
 			throw new PartInitException(IDEWorkbenchMessages.IDE_noFileEditorFound);
+		}
 
 		return editorDesc;
 	}
@@ -785,8 +810,9 @@ public final class IDE {
         final boolean[] result = new boolean[1];
         result[0] = true;
 
-        if (resourceRoots.length == 0)
-            return result[0];
+        if (resourceRoots.length == 0) {
+			return result[0];
+		}
 
         Platform.run(new SafeRunnable(IDEWorkbenchMessages.ErrorClosing) {
                     public void run() {
@@ -822,10 +848,11 @@ public final class IDE {
                         if (dirtyEditors.size() > 0) {
                             IWorkbenchWindow w = Workbench.getInstance()
                                     .getActiveWorkbenchWindow();
-                            if (w == null)
-                                w = windows[0];
+                            if (w == null) {
+								w = windows[0];
+							}
                             result[0] = EditorManager.saveAll(dirtyEditors,
-                                    finalConfirm, w);
+                                    finalConfirm, false, false, w);
                         }
                     }
                 });
@@ -898,16 +925,18 @@ public final class IDE {
             String editorID = file.getPersistentProperty(EDITOR_KEY);
             if (editorID != null) {
                 IEditorDescriptor desc = editorReg.findEditor(editorID);
-                if (desc != null)
-                    return desc;
+                if (desc != null) {
+					return desc;
+				}
             }
         } catch (CoreException e) {
             // do nothing
         }
         
 		IContentType contentType = null;
-		if (determineContentType)
-			contentType = getContentType(file);    
+		if (determineContentType) {
+			contentType = getContentType(file);
+		}    
         // Try lookup with filename
         return editorReg.getDefaultEditor(file.getName(), contentType);
     }
@@ -958,8 +987,9 @@ public final class IDE {
 		try {
 			UIStats.start(UIStats.CONTENT_TYPE_LOOKUP, file.getName());
 			IContentDescription contentDescription = file.getContentDescription();
-			if (contentDescription == null)
+			if (contentDescription == null) {
 				return null;
+			}
 			return contentDescription.getContentType();
 		} catch (CoreException e) {
 			return null;		
@@ -967,4 +997,131 @@ public final class IDE {
 			UIStats.end(UIStats.CONTENT_TYPE_LOOKUP, file, file.getName());
 		}
 	}
+	
+	
+	/**
+	 * Guess at the content type of the given file based on the filename.
+	 * 
+	 * @param file the file to test
+	 * @return the content type, or <code>null</code> if it cannot be determined.
+	 * @since 3.2
+	 */
+	public static IContentType guessContentType(IFile file) {
+		String fileName = file.getName();
+		try {
+			UIStats.start(UIStats.CONTENT_TYPE_LOOKUP, fileName);
+			IContentTypeMatcher matcher = file.getProject().getContentTypeMatcher();
+			return matcher.findContentTypeFor(fileName);
+		} catch (CoreException e) {
+			return null;		
+		} finally {
+			UIStats.end(UIStats.CONTENT_TYPE_LOOKUP, file, fileName);
+		}
+	}
+	
+    /**
+     * Prompt the user to inform them of the possible side effects
+     * of an operation on resources. Do not prompt for side effects 
+     * from ignored model providers. A model provider can be ignored if
+     * it is the client calling this API. Any message from the provided
+     * model provider id or any model providers it extends will be ignored. 
+     * @param shell the shell to parent the prompt dialog
+     * @param title the title of the dialog
+     * @param message the message for the dialog
+     * @param delta a delta built using an {@link IResourceChangeDescriptionFactory}
+     * @param ignoreModelProviderIds model providers to be ignored
+     * @param syncExec prompt in a sync exec (required when called from a non-UI thread)
+     * @return whether the user chose to continue
+     * @since 3.2
+     */
+    public static boolean promptToConfirm(final Shell shell, final String title, String message, IResourceDelta delta, String[] ignoreModelProviderIds, boolean syncExec) {
+    	IStatus status = ResourceChangeValidator.getValidator().validateChange(delta, null);
+    	if (status.isOK()) {
+			return true;
+		}
+    	final IStatus displayStatus;
+    	if (status.isMultiStatus()) {
+    		List result = new ArrayList();
+    		IStatus[] children = status.getChildren();
+    		for (int i = 0; i < children.length; i++) {
+				IStatus child = children[i];
+	    		if (!isIgnoredStatus(child, ignoreModelProviderIds)) {
+					result.add(child);
+				}
+			}
+    		if (result.isEmpty()) {
+				return true;
+			}
+    		if (result.size() == 1) {
+    			displayStatus = (IStatus)result.get(0);
+    		} else {
+    			displayStatus = new MultiStatus(status.getPlugin(), status.getCode(), (IStatus[]) result.toArray(new IStatus[result.size()]), status.getMessage(), status.getException());
+    		}
+    	} else {
+    		if (isIgnoredStatus(status, ignoreModelProviderIds)) {
+				return true;
+			}
+    		displayStatus = status;
+    	}
+    	
+    	if (message == null) {
+    		message = IDEWorkbenchMessages.IDE_sideEffectWarning;
+    	}
+    	final String dialogMessage = NLS.bind(IDEWorkbenchMessages.IDE_areYouSure, message);
+    	
+    	final boolean[] result = new boolean[] { false };
+    	Runnable runnable = new Runnable() {
+			public void run() {
+				ErrorDialog dialog = new ErrorDialog(shell, title, dialogMessage, displayStatus, IStatus.ERROR | IStatus.WARNING | IStatus.INFO) {
+					protected void createButtonsForButtonBar(Composite parent) {
+				        createButton(parent, IDialogConstants.YES_ID, IDialogConstants.YES_LABEL,
+				                false);
+						createButton(parent, IDialogConstants.NO_ID, IDialogConstants.NO_LABEL,
+								true);
+				        createDetailsButton(parent);
+					}
+					/* (non-Javadoc)
+					 * @see org.eclipse.jface.dialogs.ErrorDialog#buttonPressed(int)
+					 */
+					protected void buttonPressed(int id) {
+						if (id == IDialogConstants.YES_ID) {
+							super.buttonPressed(IDialogConstants.OK_ID);
+						} else if (id == IDialogConstants.NO_ID) {
+							super.buttonPressed(IDialogConstants.CANCEL_ID);
+						}
+						super.buttonPressed(id);
+					}
+				};
+				int code = dialog.open();
+				result[0] = code == 0;
+			}
+		};
+		if (syncExec) {
+			shell.getDisplay().syncExec(runnable);
+		} else {
+			runnable.run();
+		}
+		return result[0];
+	}
+    
+    private static boolean isIgnoredStatus(IStatus status, String[] ignoreModelProviderIds) {
+    	if (ignoreModelProviderIds == null) {
+			return false;
+		}
+    	if (status instanceof ModelStatus) {
+			ModelStatus ms = (ModelStatus) status;		
+			for (int i = 0; i < ignoreModelProviderIds.length; i++) {
+				String id = ignoreModelProviderIds[i];
+				if (ms.getModelProviderId().equals(id)) {
+					return true;
+				}
+				IModelProviderDescriptor desc = ModelProvider.getModelProviderDescriptor(id);
+				String[] extended = desc.getExtendedModels();
+				if (isIgnoredStatus(status, extended)) {
+					return true;
+				}
+			}
+		}
+		return false;
+	}
 }
