@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -29,8 +29,7 @@ import org.eclipse.jface.action.IStatusLineManager;
 import org.eclipse.jface.action.IToolBarManager;
 import org.eclipse.jface.action.MenuManager;
 import org.eclipse.jface.action.Separator;
-import org.eclipse.jface.action.ToolBarContributionItem;
-import org.eclipse.jface.action.ToolBarManager;
+import org.eclipse.jface.internal.provisional.action.IToolBarContributionItem;
 import org.eclipse.jface.preference.IPreferenceStore;
 import org.eclipse.jface.util.IPropertyChangeListener;
 import org.eclipse.jface.util.PropertyChangeEvent;
@@ -54,6 +53,7 @@ import org.eclipse.ui.internal.ide.actions.BuildSetMenu;
 import org.eclipse.ui.internal.ide.actions.BuildUtilities;
 import org.eclipse.ui.internal.ide.actions.QuickMenuAction;
 import org.eclipse.ui.internal.ide.actions.RetargetActionWithDefault;
+import org.eclipse.ui.internal.provisional.application.IActionBarConfigurer2;
 import org.eclipse.ui.internal.util.StatusLineContributionItem;
 
 /**
@@ -66,6 +66,8 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
     private IWorkbenchAction closeAction;
 
     private IWorkbenchAction closeAllAction;
+    
+    private IWorkbenchAction closeOthersAction;
 
     private IWorkbenchAction closeAllSavedAction;
 
@@ -125,9 +127,9 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
     
     private IWorkbenchAction minimizePartAction;
 
-    private IWorkbenchAction workbenchEditorsAction;
+    private IWorkbenchAction switchToEditorAction;
 
-    private IWorkbenchAction workbookEditorsAction;
+    private IWorkbenchAction openEditorDropDownAction;
 
     private IWorkbenchAction backwardHistoryAction;
 
@@ -186,7 +188,7 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
     private IWorkbenchAction newWizardDropDownAction;
 
     private IWorkbenchAction importResourcesAction;
-
+    
     private IWorkbenchAction exportResourcesAction;
 
     IWorkbenchAction buildAllAction; // Incremental workspace build
@@ -326,8 +328,9 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
         resourceListener = new IResourceChangeListener() {
 			public void resourceChanged(IResourceChangeEvent event) {
 				IResourceDelta delta = event.getDelta();
-				if (delta == null)
+				if (delta == null) {
 					return;
+				}
 				IResourceDelta[] projectDeltas = delta.getAffectedChildren();
 				for (int i = 0; i < projectDeltas.length; i++) {
 					int kind = projectDeltas[i].getKind();
@@ -356,6 +359,7 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
      */
     protected void fillCoolBar(ICoolBarManager coolBar) {
 
+    	IActionBarConfigurer2 actionBarConfigurer = (IActionBarConfigurer2) getActionBarConfigurer();
         { // Set up the context Menu
             IMenuManager popUpMenu = new MenuManager();
             popUpMenu.add(new ActionContributionItem(lockToolBarAction));
@@ -364,8 +368,7 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
         }
         coolBar.add(new GroupMarker(IIDEActionConstants.GROUP_FILE));
         { // File Group
-            IToolBarManager fileToolBar = new ToolBarManager(coolBar
-                    .getStyle());
+            IToolBarManager fileToolBar = actionBarConfigurer.createToolBarManager();
             fileToolBar.add(new Separator(IWorkbenchActionConstants.NEW_GROUP));
             fileToolBar.add(newWizardDropDownAction);
             fileToolBar.add(new GroupMarker(IWorkbenchActionConstants.NEW_EXT));
@@ -386,7 +389,7 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
                     IWorkbenchActionConstants.MB_ADDITIONS));
 
             // Add to the cool bar manager
-            coolBar.add(new ToolBarContributionItem(fileToolBar,
+            coolBar.add(actionBarConfigurer.createToolBarContributionItem(fileToolBar,
                     IWorkbenchActionConstants.TOOLBAR_FILE));
         }
 
@@ -394,8 +397,7 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
 
         coolBar.add(new GroupMarker(IIDEActionConstants.GROUP_NAV));
         { // Navigate group
-            IToolBarManager navToolBar = new ToolBarManager(coolBar
-                    .getStyle());
+            IToolBarManager navToolBar = actionBarConfigurer.createToolBarManager();
             navToolBar.add(new Separator(
                     IWorkbenchActionConstants.HISTORY_GROUP));
             navToolBar
@@ -406,24 +408,23 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
             navToolBar.add(pinEditorContributionItem);
 
             // Add to the cool bar manager
-            coolBar.add(new ToolBarContributionItem(navToolBar,
+            coolBar.add(actionBarConfigurer.createToolBarContributionItem(navToolBar,
                     IWorkbenchActionConstants.TOOLBAR_NAVIGATE));
         }
 
         coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_EDITOR));
      
-        coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_HELP)); //$NON-NLS-1$
+        coolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_HELP));
         
         { // Help group
-            IToolBarManager helpToolBar = new ToolBarManager(coolBar
-                    .getStyle());
-            helpToolBar.add(new Separator(IWorkbenchActionConstants.GROUP_HELP)); //$NON-NLS-1$
+            IToolBarManager helpToolBar = actionBarConfigurer.createToolBarManager();
+            helpToolBar.add(new Separator(IWorkbenchActionConstants.GROUP_HELP));
 //            helpToolBar.add(searchComboItem);
               // Add the group for applications to contribute
             helpToolBar.add(new GroupMarker(IWorkbenchActionConstants.GROUP_APP));              
             // Add to the cool bar manager
-            coolBar.add(new ToolBarContributionItem(helpToolBar,
-                    IWorkbenchActionConstants.TOOLBAR_HELP)); //$NON-NLS-1$
+            coolBar.add(actionBarConfigurer.createToolBarContributionItem(helpToolBar,
+                    IWorkbenchActionConstants.TOOLBAR_HELP));
         }        
 
     }
@@ -454,11 +455,13 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
             MenuManager newMenu = new MenuManager(newText, newId) {
                 public String getMenuText() {
                     String result = super.getMenuText();
-                    if (newQuickMenu == null)
-                        return result;
+                    if (newQuickMenu == null) {
+						return result;
+					}
                     String shortCut = newQuickMenu.getShortCutString();
-                    if (shortCut == null)
-                        return result;
+                    if (shortCut == null) {
+						return result;
+					}
                     return result + "\t" + shortCut; //$NON-NLS-1$
                 }
             };
@@ -571,11 +574,13 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
             MenuManager showInSubMenu = new MenuManager(IDEWorkbenchMessages.Workbench_showIn, "showIn") { //$NON-NLS-1$
                 public String getMenuText() {
                     String result = super.getMenuText();
-                    if (showInQuickMenu == null)
-                        return null;
+                    if (showInQuickMenu == null) {
+						return null;
+					}
                     String shortCut = showInQuickMenu.getShortCutString();
-                    if (shortCut == null)
-                        return result;
+                    if (shortCut == null) {
+						return result;
+					}
                     return result + "\t" + shortCut; //$NON-NLS-1$
                 }
             };
@@ -701,7 +706,8 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
         subMenu.add(activateEditorAction);
         subMenu.add(nextEditorAction);
         subMenu.add(prevEditorAction);
-        subMenu.add(workbookEditorsAction);
+        subMenu.add(switchToEditorAction);
+        subMenu.add(openEditorDropDownAction);
         subMenu.add(new Separator());
         subMenu.add(nextPartAction);
         subMenu.add(prevPartAction);
@@ -717,10 +723,11 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
 		MenuManager menu = new MenuManager(IDEWorkbenchMessages.Workbench_help, IWorkbenchActionConstants.M_HELP);
 		addSeparatorOrGroupMarker(menu, "group.intro"); //$NON-NLS-1$
 		// See if a welcome or intro page is specified
-		if (introAction != null)
+		if (introAction != null) {
 			menu.add(introAction);
-		else if (quickStartAction != null)
+		} else if (quickStartAction != null) {
 			menu.add(quickStartAction);
+		}
 		menu.add(new GroupMarker("group.intro.ext")); //$NON-NLS-1$
 		addSeparatorOrGroupMarker(menu, "group.main"); //$NON-NLS-1$
 		menu.add(helpContentsAction);
@@ -728,8 +735,9 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
 		menu.add(dynamicHelpAction);
 		addSeparatorOrGroupMarker(menu, "group.assist"); //$NON-NLS-1$
 		// See if a tips and tricks page is specified
-		if (tipsAndTricksAction != null)
+		if (tipsAndTricksAction != null) {
 			menu.add(tipsAndTricksAction);
+		}
 		// HELP_START should really be the first item, but it was after
 		// quickStartAction and tipsAndTricksAction in 2.1.
 		menu.add(new GroupMarker(IWorkbenchActionConstants.HELP_START));
@@ -1012,8 +1020,9 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
      * Called when the window is closed.
      */
     public void dispose() {
-        if (isDisposed)
-            return;
+        if (isDisposed) {
+			return;
+		}
     	isDisposed = true;
         getActionBarConfigurer().getStatusLineManager().remove(statusLineItem);
         if (pageListener != null) {
@@ -1044,6 +1053,7 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
         closeAction = null;
         closeAllAction = null;
         closeAllSavedAction = null;
+        closeOthersAction = null;
         saveAction = null;
         saveAllAction = null;
         newWindowAction = null;
@@ -1072,8 +1082,8 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
         activateEditorAction = null;
         maximizePartAction = null;
         minimizePartAction = null;
-        workbenchEditorsAction = null;
-        workbookEditorsAction = null;
+        switchToEditorAction = null;
+        openEditorDropDownAction = null;
         backwardHistoryAction = null;
         forwardHistoryAction = null;
         undoAction = null;
@@ -1137,10 +1147,12 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
      * are preserved in OLE menu merging.
      */
     public boolean isApplicationMenu(String menuId) {
-        if (menuId.equals(IWorkbenchActionConstants.M_FILE))
-            return true;
-        if (menuId.equals(IWorkbenchActionConstants.M_WINDOW))
-            return true;
+        if (menuId.equals(IWorkbenchActionConstants.M_FILE)) {
+			return true;
+		}
+        if (menuId.equals(IWorkbenchActionConstants.M_WINDOW)) {
+			return true;
+		}
         return false;
     }
 
@@ -1149,10 +1161,12 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
      * the workbench creates.
      */
     public boolean isWorkbenchCoolItemId(String id) {
-        if (IWorkbenchActionConstants.TOOLBAR_FILE.equalsIgnoreCase(id))
-            return true;
-        if (IWorkbenchActionConstants.TOOLBAR_NAVIGATE.equalsIgnoreCase(id))
-            return true;
+        if (IWorkbenchActionConstants.TOOLBAR_FILE.equalsIgnoreCase(id)) {
+			return true;
+		}
+        if (IWorkbenchActionConstants.TOOLBAR_NAVIGATE.equalsIgnoreCase(id)) {
+			return true;
+		}
         return false;
     }
 
@@ -1183,7 +1197,7 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
 
         exportResourcesAction = ActionFactory.EXPORT.create(window);
         register(exportResourcesAction);
-
+        
         buildAllAction = IDEActionFactory.BUILD.create(window);
         register(buildAllAction);
 
@@ -1240,6 +1254,9 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
         closeAllAction = ActionFactory.CLOSE_ALL.create(window);
         register(closeAllAction);
 
+        closeOthersAction = ActionFactory.CLOSE_OTHERS.create(window);
+        register(closeOthersAction);
+
         closeAllSavedAction = ActionFactory.CLOSE_ALL_SAVED.create(window);
         register(closeAllSavedAction);
 
@@ -1311,13 +1328,13 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
 		minimizePartAction = ActionFactory.MINIMIZE.create(window);
 		register(minimizePartAction);
         
-        workbenchEditorsAction = ActionFactory.SHOW_OPEN_EDITORS
+        switchToEditorAction = ActionFactory.SHOW_OPEN_EDITORS
                 .create(window);
-        register(workbenchEditorsAction);
+        register(switchToEditorAction);
 
-        workbookEditorsAction = ActionFactory.SHOW_WORKBOOK_EDITORS
+        openEditorDropDownAction = ActionFactory.SHOW_WORKBOOK_EDITORS
                 .create(window);
-        register(workbookEditorsAction);
+        register(openEditorDropDownAction);
 
         hideShowEditorAction = ActionFactory.SHOW_EDITOR.create(window);
         register(hideShowEditorAction);
@@ -1543,8 +1560,9 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
         // this can be triggered by property or resource change notifications
         Runnable update = new Runnable() {
             public void run() {
-                if (isDisposed)
-                	return;
+                if (isDisposed) {
+					return;
+				}
 		    	IWorkspace workspace = ResourcesPlugin.getWorkspace();
 				IProject[] projects = workspace.getRoot().getProjects();
 		    	boolean enabled = BuildUtilities.isEnabled(projects, IncrementalProjectBuilder.INCREMENTAL_BUILD);
@@ -1559,12 +1577,12 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
 		                .getCoolBarManager();
 		        IContributionItem cbItem = coolBarManager
 		                .find(IWorkbenchActionConstants.TOOLBAR_FILE);
-		        if (!(cbItem instanceof ToolBarContributionItem)) {
+		        if (!(cbItem instanceof IToolBarContributionItem)) {
 		            // This should not happen
 		            IDEWorkbenchPlugin.log("File toolbar contribution item is missing"); //$NON-NLS-1$
 		            return;
 		        }
-		        ToolBarContributionItem toolBarItem = (ToolBarContributionItem) cbItem;
+		        IToolBarContributionItem toolBarItem = (IToolBarContributionItem) cbItem;
 		        IToolBarManager toolBarManager = toolBarItem.getToolBarManager();
 		        if (toolBarManager == null) {
 		            // error if this happens, file toolbar assumed to always exist
@@ -1607,13 +1625,13 @@ public final class WorkbenchActionBuilder extends ActionBarAdvisor {
                 .getCoolBarManager();
         IContributionItem cbItem = coolBarManager
                 .find(IWorkbenchActionConstants.TOOLBAR_NAVIGATE);
-        if (!(cbItem instanceof ToolBarContributionItem)) {
+        if (!(cbItem instanceof IToolBarContributionItem)) {
             // This should not happen
             IDEWorkbenchPlugin
                     .log("Navigation toolbar contribution item is missing"); //$NON-NLS-1$
             return;
         }
-        ToolBarContributionItem toolBarItem = (ToolBarContributionItem) cbItem;
+        IToolBarContributionItem toolBarItem = (IToolBarContributionItem) cbItem;
         IToolBarManager toolBarManager = toolBarItem.getToolBarManager();
         if (toolBarManager == null) {
             // error if this happens, navigation toolbar assumed to always exist
