@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2005 IBM Corporation and others.
+ * Copyright (c) 2004, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -18,6 +18,7 @@ import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Label;
+import org.eclipse.ui.forms.FormColors;
 import org.eclipse.ui.forms.events.HyperlinkAdapter;
 import org.eclipse.ui.forms.events.HyperlinkEvent;
 import org.eclipse.ui.forms.widgets.FormText;
@@ -35,6 +36,7 @@ import org.eclipse.ui.internal.intro.impl.model.IntroGroup;
 import org.eclipse.ui.internal.intro.impl.model.IntroHTML;
 import org.eclipse.ui.internal.intro.impl.model.IntroImage;
 import org.eclipse.ui.internal.intro.impl.model.IntroLink;
+import org.eclipse.ui.internal.intro.impl.model.IntroSeparator;
 import org.eclipse.ui.internal.intro.impl.model.IntroText;
 import org.eclipse.ui.internal.intro.impl.model.loader.ContentProviderManager;
 import org.eclipse.ui.internal.intro.impl.model.url.IntroURLParser;
@@ -129,7 +131,8 @@ public class PageWidgetFactory {
         case AbstractIntroElement.IMAGE:
             IntroImage image = (IntroImage) element;
             c = createImage(parent, image);
-            updateLayoutData(c, element);
+            if (c!=null)
+            	updateLayoutData(c, element);
             break;
         case AbstractIntroElement.HTML:
             IntroHTML html = (IntroHTML) element;
@@ -163,6 +166,12 @@ public class PageWidgetFactory {
             c = createContentProvider(parent, provider);
             updateLayoutData(c, element);
             break;
+        case AbstractIntroElement.HR:
+        	IntroSeparator sep = (IntroSeparator)element;
+            c = createSeparator(parent, sep);
+            updateLayoutData(c, element);
+            break;
+        	
         default:
             break;
         }
@@ -188,10 +197,16 @@ public class PageWidgetFactory {
     private Composite createGroup(Composite parent, IntroGroup group) {
         String label = group.getLabel();
         String description = styleManager.getDescription(group);
+        boolean expandable = group.isExpandable();
+        boolean expanded = group.isExpanded();
         Composite client = null;
         Composite control = null;
-        if (description != null || label != null) {
+        if (description != null || label != null || expandable) {
             int style = description != null ? Section.DESCRIPTION : SWT.NULL;
+            if (expandable)
+            	style |= Section.TWISTIE | Section.FOCUS_TITLE | Section.CLIENT_INDENT;
+            if (expanded)
+            	style |= Section.EXPANDED;
             Section section = toolkit.createSection(parent, style);
             if (label != null)
                 section.setText(label);
@@ -239,8 +254,13 @@ public class PageWidgetFactory {
             layout.numColumns = 2;
             container.setLayout(layout);
 
-            Label ilabel = toolkit.createLabel(container, null);
+            //Label ilabel = toolkit.createLabel(container, null);
+            ImageHyperlink ilabel = toolkit.createImageHyperlink(container, SWT.NULL);
             ilabel.setImage(linkImage);
+            ilabel.setHoverImage(styleManager.getImage(link, "hover-icon", //$NON-NLS-1$
+                null));
+            ilabel.setHref(link.getUrl());
+            ilabel.addHyperlinkListener(hyperlinkAdapter);
             TableWrapData td = new TableWrapData();
             td.valign = TableWrapData.TOP;
             td.rowspan = 2;
@@ -336,13 +356,14 @@ public class PageWidgetFactory {
         }
         // for images, do not use default layout. Grab horizontal is not what we
         // want.
-        TableWrapData td = new TableWrapData();
-        ilabel.setLayoutData(td);
+        if (ilabel!=null) {
+        	TableWrapData td = new TableWrapData();
+        	ilabel.setLayoutData(td);
+        }
         return ilabel;
     }
-
-
-    protected Control createContentProvider(Composite parent,
+    
+    public Control createContentProvider(Composite parent,
             IntroContentProvider provider) {
         // If we've already loaded the content provider for this element,
         // retrieve it, otherwise load the class.
@@ -355,6 +376,7 @@ public class PageWidgetFactory {
         layout.leftMargin = 0;
         layout.rightMargin = 0;
         container.setLayout(layout);
+        container.setData(provider);
 
 
         IIntroContentProvider providerClass = ContentProviderManager.getInst()
@@ -385,12 +407,32 @@ public class PageWidgetFactory {
         }
         return container;
     }
+    
+    protected Control createSeparator(Composite parent, IntroSeparator sep) {
+    	String key = sep.getParentPage().getId()+".separator.fg"; //$NON-NLS-1$
+        Color fg = styleManager.getColor(toolkit, key);
+        //Composite l = toolkit.createCompositeSeparator(parent);
+        Composite l = new Composite(parent, SWT.NULL);
+        if (fg!=null)
+        	l.setBackground(fg);
+        else
+        	l.setBackground(toolkit.getColors().getColor(FormColors.SEPARATOR));
+        TableWrapData td = new TableWrapData(TableWrapData.FILL,
+                TableWrapData.FILL);
+        td.grabHorizontal = true;
+        td.maxHeight = 1;
+        l.setLayoutData(td);
+        return l;
+    }
 
     private void colorControl(Control elementControl,
             AbstractBaseIntroElement element) {
         Color fg = styleManager.getColor(toolkit, element);
         if (fg != null)
             elementControl.setForeground(fg);
+        Color bg = styleManager.getBackgrond(toolkit, element);
+        if (bg != null)
+            elementControl.setBackground(bg);
     }
 
 
