@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,8 +10,8 @@
  *******************************************************************************/
 package org.eclipse.ui.actions;
 
-import org.eclipse.jface.action.Action;
 import org.eclipse.jface.action.IAction;
+import org.eclipse.jface.action.LegacyActionTools;
 import org.eclipse.jface.resource.ImageDescriptor;
 import org.eclipse.jface.util.PropertyChangeEvent;
 
@@ -62,7 +62,7 @@ public class LabelRetargetAction extends RetargetAction {
         super(actionID, text, style);
         this.defaultText = text;
         this.defaultToolTipText = text;
-        acceleratorText = extractAcceleratorText(text);
+        acceleratorText = LegacyActionTools.extractAcceleratorText(text);
     }
 
     /**
@@ -71,13 +71,13 @@ public class LabelRetargetAction extends RetargetAction {
     protected void propagateChange(PropertyChangeEvent event) {
         super.propagateChange(event);
         String prop = event.getProperty();
-        if (prop.equals(Action.TEXT)) {
+        if (prop.equals(IAction.TEXT)) {
             String str = (String) event.getNewValue();
             super.setText(appendAccelerator(str));
-        } else if (prop.equals(Action.TOOL_TIP_TEXT)) {
+        } else if (prop.equals(IAction.TOOL_TIP_TEXT)) {
             String str = (String) event.getNewValue();
             super.setToolTipText(str);
-        } else if (prop.equals(Action.IMAGE)) {
+        } else if (prop.equals(IAction.IMAGE)) {
             updateImages(getActionHandler());
         }
     }
@@ -134,7 +134,7 @@ public class LabelRetargetAction extends RetargetAction {
      */
     public void setText(String text) {
         super.setText(text);
-        acceleratorText = extractAcceleratorText(text);
+        acceleratorText = LegacyActionTools.extractAcceleratorText(text);
         defaultText = text;
     }
 
@@ -152,37 +152,19 @@ public class LabelRetargetAction extends RetargetAction {
      * allowed to change the accelerator).
      */
     private String appendAccelerator(String newText) {
-        if (newText == null)
-            return null;
+        if (newText == null) {
+			return null;
+		}
 
         // Remove any accelerator
         String str = removeAcceleratorText(newText);
         // Append our accelerator
-        if (acceleratorText != null)
-            str = str + acceleratorText;
+        if (acceleratorText != null) {
+			str = str + acceleratorText;
+		}
         return str;
     }
 
-    /**
-     * Extracts the accelerator text from the given text.
-     * Returns <code>null</code> if there is no accelerator text,
-     * and the empty string if there is no text after the accelerator delimeter (tab or '@').
-     *
-     * @param text the text for the action
-     * @return the accelerator text including '@' or '\t', or <code>null</code>
-     */
-    private String extractAcceleratorText(String text) {
-        if (text == null)
-            return null;
-
-        int index = text.lastIndexOf('\t');
-        if (index == -1)
-            index = text.lastIndexOf('@');
-        if (index >= 0)
-            return text.substring(index);
-        return null;
-    }
-
     /**
      * Updates the images for this action based on the given handler.
      */
