@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -27,6 +27,11 @@ import org.eclipse.ui.IWorkbenchWindow;
  * The abstract superclass for save actions that depend on the active editor.
  */
 public abstract class BaseSaveAction extends ActiveEditorAction {
+    /*
+     * The view-related code below was added to track the view with focus
+     * in order to support save actions from a view (see bug 10234). 
+     */
+	
     /**
      * List of parts (element type: <code>IWorkbenchPart</code>)
      * against which this class has outstanding property listeners registered.
@@ -36,8 +41,9 @@ public abstract class BaseSaveAction extends ActiveEditorAction {
     private final IPropertyListener propListener = new IPropertyListener() {
         public void propertyChanged(Object source, int propId) {
             if (source == getActiveEditor()) {
-                if (propId == IEditorPart.PROP_DIRTY)
-                    updateState();
+                if (propId == IEditorPart.PROP_DIRTY) {
+					updateState();
+				}
             }
         }
     };
@@ -74,26 +80,31 @@ public abstract class BaseSaveAction extends ActiveEditorAction {
         }
     }
 
-    /*
-     * **********************************************************************************
-     * **********************************************************************************
-     * **********************************************************************************
-     * The code below was added to track the view with focus
-     * in order to support save actions from a view. Remove this
-     * experimental code if the decision is to not allow views to 
-     * participate in save actions (see bug 10234) 
-     */
     private IViewPart activeView;
 
     private final IPropertyListener propListener2 = new IPropertyListener() {
         public void propertyChanged(Object source, int propId) {
             if (source == activeView) {
-                if (propId == IEditorPart.PROP_DIRTY)
-                    updateState();
+                if (propId == IEditorPart.PROP_DIRTY) {
+					updateState();
+				}
             }
         }
     };
 
+    /** the active saveable part is tracked in order to listen to its dirty events */ 
+    private ISaveablePart activeSaveablePart;
+    
+    private final IPropertyListener propListener3 = new IPropertyListener() {
+    	public void propertyChanged(Object source, int propId) {
+    		if (source == activeSaveablePart) {
+    			if (propId == IEditorPart.PROP_DIRTY) {
+    				updateState();
+    			}
+    		}
+    	}
+    };
+    
     /* (non-Javadoc)
      * Method declared on PageEventAction.
      */
@@ -150,13 +161,33 @@ public abstract class BaseSaveAction extends ActiveEditorAction {
      * active page.
      */
     private void updateActiveView() {
-        if (getActivePage() == null)
-            setActiveView(null);
-        else
-            setActiveView(getActivePage().getActivePart());
+        if (getActivePage() == null) {
+			setActiveView(null);
+		} else {
+			setActiveView(getActivePage().getActivePart());
+		}
     }
 
     /**
+	 * 
+	 */
+	private void updateActiveSaveablePart() {
+		if (activeSaveablePart instanceof IWorkbenchPart) {
+			((IWorkbenchPart)activeSaveablePart).removePropertyListener(propListener3);
+			partsWithListeners.remove(activeSaveablePart);
+		}
+		activeSaveablePart = getSaveableView();
+		if (activeSaveablePart == activeView) {
+			// no need to listen to the same part twice
+			activeSaveablePart = null;
+		}
+		if (activeSaveablePart instanceof IWorkbenchPart) {
+			((IWorkbenchPart)activeSaveablePart).addPropertyListener(propListener3);
+			partsWithListeners.add(activeSaveablePart);
+		}
+	}
+
+	/**
      * Set the active editor
      */
     private void setActiveView(IWorkbenchPart part) {
@@ -176,6 +207,7 @@ public abstract class BaseSaveAction extends ActiveEditorAction {
             activeView.addPropertyListener(propListener2);
             partsWithListeners.add(activeView);
         }
+        updateActiveSaveablePart();
     }
 
     protected final ISaveablePart getSaveableView() {
@@ -197,6 +229,7 @@ public abstract class BaseSaveAction extends ActiveEditorAction {
             IWorkbenchPart part = (IWorkbenchPart) it.next();
             part.removePropertyListener(propListener);
             part.removePropertyListener(propListener2);
+            part.removePropertyListener(propListener3);
         }
         partsWithListeners.clear();
     }
