@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -14,7 +14,9 @@ package org.eclipse.ui.internal;
 
 
 import org.eclipse.jface.action.MenuManager;
-import org.eclipse.jface.action.ToolBarManager;
+import org.eclipse.jface.internal.provisional.action.IToolBarManager2;
+import org.eclipse.jface.util.IPropertyChangeListener;
+import org.eclipse.jface.util.PropertyChangeEvent;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.MouseAdapter;
 import org.eclipse.swt.events.MouseEvent;
@@ -25,10 +27,10 @@ import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Menu;
 import org.eclipse.swt.widgets.Shell;
 import org.eclipse.swt.widgets.ToolBar;
-import org.eclipse.ui.IPropertyListener;
 import org.eclipse.ui.IViewReference;
 import org.eclipse.ui.IWorkbenchPart;
 import org.eclipse.ui.internal.dnd.DragUtil;
+import org.eclipse.ui.internal.provisional.presentations.IActionBarPresentationFactory;
 import org.eclipse.ui.presentations.IPresentablePart;
 import org.eclipse.ui.presentations.StackPresentation;
 
@@ -37,13 +39,12 @@ import org.eclipse.ui.presentations.StackPresentation;
  * 
  * TODO: Delete ViewPane and EditorPane, and make PartPane non-abstract.
  */
-public class ViewPane extends PartPane implements IPropertyListener {
+public class ViewPane extends PartPane {
     private boolean busy = false;
 
     // create initially toolbarless bar manager so that actions may be added in the 
     // init method of the view.
-    private ToolBarManager isvToolBarMgr = new PaneToolBarManager(SWT.FLAT
-            | SWT.WRAP);
+    private IToolBarManager2 isvToolBarMgr = null;
 
     private MenuManager isvMenuMgr;
 
@@ -57,16 +58,32 @@ public class ViewPane extends PartPane implements IPropertyListener {
     /**
      * Toolbar manager for the ISV toolbar.
      */
-    class PaneToolBarManager extends ToolBarManager {
-        public PaneToolBarManager(int style) {
-            super(style);
-        }
-
-        protected void relayout(ToolBar toolBar, int oldCount, int newCount) {
-            toolBarResized(toolBar, newCount);
-
-            toolBar.layout();
-        }
+    private class ISVPropListener implements IPropertyChangeListener {    	
+    	private Control toolBar;    	
+    	
+    	/**
+    	 * Constructor
+    	 * @param toolBar
+    	 */
+    	public ISVPropListener (Control toolBar) {
+    		this.toolBar = toolBar;
+    	}
+    	
+    	/* (non-Javadoc)
+    	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
+    	 */
+    	public void propertyChange(PropertyChangeEvent event) {    		
+    		String property = event.getProperty();
+    		Integer newValue = (Integer)event.getNewValue();
+	       	if (IToolBarManager2.PROP_LAYOUT.equals(property)) {
+	       		toolBarResized(toolBar, newValue != null ? newValue.intValue() : 0);	
+	       		if (toolBar instanceof Composite) {
+					((Composite)toolBar).layout();
+				} else {
+					toolBar.getParent().layout();
+				}
+	       	}
+	    }
     }
 
     /**
@@ -83,8 +100,7 @@ public class ViewPane extends PartPane implements IPropertyListener {
             boolean hasMenu = !isEmpty();
             if (hasMenu != hadViewMenu) {
                 hadViewMenu = hasMenu;
-                presentableAdapter
-                        .firePropertyChange(IPresentablePart.PROP_PANE_MENU);
+                firePropertyChange(IPresentablePart.PROP_PANE_MENU);
             }
         }
     }
@@ -94,6 +110,10 @@ public class ViewPane extends PartPane implements IPropertyListener {
      */
     public ViewPane(IViewReference ref, WorkbenchPage page) {
         super(ref, page);
+        IActionBarPresentationFactory actionBarPresentation = ((WorkbenchWindow) page
+				.getWorkbenchWindow()).getActionBarPresentationFactory();
+        
+        isvToolBarMgr = actionBarPresentation.createViewToolBarManager();
     }
 
     /**
@@ -101,21 +121,13 @@ public class ViewPane extends PartPane implements IPropertyListener {
      */
     public void createControl(Composite parent) {
         // Only do this once.
-        if (getControl() != null && !getControl().isDisposed())
-            return;
+        if (getControl() != null && !getControl().isDisposed()) {
+			return;
+		}
 
         super.createControl(parent);
     }
 
-
-    private void recreateToolbars() {
-        // create new toolbars based on the locked vs !locked state
-        createToolBars();
-        // create new toolbars
-        updateActionBars();
-
-    }
-
     /**
      * Create a title bar for the pane.
      * 	- the view icon and title to the far left
@@ -134,9 +146,9 @@ public class ViewPane extends PartPane implements IPropertyListener {
 
     }
 
-    private void toolBarResized(ToolBar toolBar, int newSize) {
+    private void toolBarResized(Control toolBar, int newSize) {
         
-        Control toolbar = isvToolBarMgr.getControl();
+        Control toolbar = isvToolBarMgr.getControl2();
         if (toolbar != null) {
             Control ctrl = getControl();
 
@@ -146,7 +158,7 @@ public class ViewPane extends PartPane implements IPropertyListener {
             toolbar.setVisible(visible);
         }
 
-        presentableAdapter.firePropertyChange(IPresentablePart.PROP_TOOLBAR);
+        firePropertyChange(IPresentablePart.PROP_TOOLBAR);
     }
 
     /**
@@ -157,19 +169,24 @@ public class ViewPane extends PartPane implements IPropertyListener {
         
         // ISV toolbar.
         //			// 1GD0ISU: ITPUI:ALL - Dbl click on view tool cause zoom
-        ToolBar isvToolBar = isvToolBarMgr.createControl(parentControl.getParent());
-
-            isvToolBar.addMouseListener(new MouseAdapter() {
-                public void mouseDoubleClick(MouseEvent event) {
-                    if (event.widget instanceof ToolBar) {
-                    
-	                    if (((ToolBar)event.widget).getItem(new Point(event.x, event.y)) == null)
-	                        doZoom();
-                    }
+        final Control isvToolBar = isvToolBarMgr.createControl2(parentControl.getParent());
+        
+        isvToolBarMgr.addPropertyChangeListener(new ISVPropListener(isvToolBar));
+        
+        isvToolBar.addMouseListener(new MouseAdapter() {
+            public void mouseDoubleClick(MouseEvent event) {
+                if (event.widget instanceof ToolBar) {
+                
+                    if (((ToolBar)event.widget).getItem(new Point(event.x, event.y)) == null) {
+						doZoom();
+					}
                 }
-            });
+            }
+        });
 
+        
         isvToolBar.addListener(SWT.Activate, this);
+        isvToolBar.moveAbove(control);
     }
 
     public void dispose() {
@@ -272,10 +289,11 @@ public class ViewPane extends PartPane implements IPropertyListener {
      */
     public String getCompoundId() {
         IViewReference ref = getViewReference();
-        if (ref != null)
-            return ViewFactory.getKey(ref);
-        else
-            return super.getCompoundId();
+        if (ref != null) {
+			return ViewFactory.getKey(ref);
+		}
+        
+        return super.getCompoundId();
     }
 
     /**
@@ -289,8 +307,9 @@ public class ViewPane extends PartPane implements IPropertyListener {
      * @see ViewActionBars
      */
     public MenuManager getMenuManager() {
-        if (isvMenuMgr == null)
-            isvMenuMgr = new PaneMenuManager();
+        if (isvMenuMgr == null) {
+			isvMenuMgr = new PaneMenuManager();
+		}
         return isvMenuMgr;
     }
 
@@ -310,7 +329,7 @@ public class ViewPane extends PartPane implements IPropertyListener {
     /**
      * @see ViewActionBars
      */
-    public ToolBarManager getToolBarManager() {
+    public IToolBarManager2 getToolBarManager() {
         return isvToolBarMgr;
     }
 
@@ -321,18 +340,6 @@ public class ViewPane extends PartPane implements IPropertyListener {
         return (IViewReference) getPartReference();
     }
 
-    /**
-     * Indicates that a property has changed.
-     *
-     * @param source the object whose property has changed
-     * @param propId the id of the property which has changed; property ids
-     *   are generally defined as constants on the source class
-     */
-    public void propertyChanged(Object source, int propId) {
-        if (propId == IWorkbenchPart.PROP_TITLE)
-            updateTitles();
-    }
-
     /**
      * Sets the fast view state.  If this view is a fast view then
      * various controls like pin and minimize are added to the
@@ -406,12 +413,14 @@ public class ViewPane extends PartPane implements IPropertyListener {
     }
 
     public void showViewMenu(Point location) {
-        if (!hasViewMenu())
-            return;
+        if (!hasViewMenu()) {
+			return;
+		}
 
         // If this is a fast view, it may have been minimized. Do nothing in this case.
-        if (isFastView() && (page.getActiveFastView() != getViewReference()))
-            return;
+        if (isFastView() && (page.getActiveFastView() != getViewReference())) {
+			return;
+		}
 
         Menu aMenu = isvMenuMgr.createContextMenu(getControl().getParent());
         aMenu.setLocation(location.x, location.y);
@@ -427,10 +436,12 @@ public class ViewPane extends PartPane implements IPropertyListener {
      * @see ViewActionBars
      */
     public void updateActionBars() {
-        if (isvMenuMgr != null)
-            isvMenuMgr.updateAll(false);
-        if (isvToolBarMgr != null)
-            isvToolBarMgr.update(false);
+        if (isvMenuMgr != null) {
+			isvMenuMgr.updateAll(false);
+		}
+        if (isvToolBarMgr != null) {
+			isvToolBarMgr.update(false);
+		}
 
     }
 
@@ -438,24 +449,25 @@ public class ViewPane extends PartPane implements IPropertyListener {
      * Update the title attributes.
      */
     public void updateTitles() {
-        presentableAdapter.firePropertyChange(IPresentablePart.PROP_TITLE);
-
+        firePropertyChange(IPresentablePart.PROP_TITLE);
     }
 
     /* (non-Javadoc)
      * @see org.eclipse.ui.internal.PartPane#addSizeMenuItem(org.eclipse.swt.widgets.Menu)
      */
     public void addSizeMenuItem(Menu menu, int index) {
-        if (isMoveable())
-            super.addSizeMenuItem(menu, index);
+        if (isMoveable()) {
+			super.addSizeMenuItem(menu, index);
+		}
     }
 
     /* (non-Javadoc)
      * @see org.eclipse.ui.internal.PartPane#doZoom()
      */
     protected void doZoom() {
-        if (isMoveable())
-            super.doZoom();
+        if (isMoveable()) {
+			super.doZoom();
+		}
     }
 
     /* (non-Javadoc)
@@ -485,9 +497,10 @@ public class ViewPane extends PartPane implements IPropertyListener {
         super.reparent(newParent);
 
         if (isvToolBarMgr != null) {
-            ToolBar bar = isvToolBarMgr.getControl();
+            Control bar = isvToolBarMgr.getControl2();
             if (bar != null) {
                 bar.setParent(newParent);
+                bar.moveAbove(control);
             }
         }
     }
@@ -514,24 +527,25 @@ public class ViewPane extends PartPane implements IPropertyListener {
         Control toolbar = internalGetToolbar();
         
         if (toolbar != null) {
-            toolbar.setVisible(makeVisible && toolbarIsVisible());
+            boolean visible = makeVisible && toolbarIsVisible();
+            toolbar.setVisible(visible);
         }
     }
 
     public boolean toolbarIsVisible() {
-        ToolBarManager toolbarManager = getToolBarManager();
+        IToolBarManager2 toolbarManager = getToolBarManager();
 
         if (toolbarManager == null) {
             return false;
         }
 
-        ToolBar control = toolbarManager.getControl();
+        Control control = toolbarManager.getControl2();
 
         if (control == null || control.isDisposed()) {
             return false;
         }
 
-        return control.getItemCount() > 0;
+        return toolbarManager.getItemCount() > 0;
     }
 
     /*
@@ -541,7 +555,7 @@ public class ViewPane extends PartPane implements IPropertyListener {
     public void setBusy(boolean isBusy) {
         if (isBusy != busy) {
             busy = isBusy;
-            presentableAdapter.firePropertyChange(IPresentablePart.PROP_BUSY);
+            firePropertyChange(IPresentablePart.PROP_BUSY);
         }
     }
 
@@ -557,8 +571,7 @@ public class ViewPane extends PartPane implements IPropertyListener {
      * @see org.eclipse.ui.internal.PartPane#showHighlight()
      */
     public void showHighlight() {
-        presentableAdapter
-                .firePropertyChange(IPresentablePart.PROP_HIGHLIGHT_IF_BACK);
+        firePropertyChange(IPresentablePart.PROP_HIGHLIGHT_IF_BACK);
     }
 
     /* (non-Javadoc)
@@ -580,12 +593,12 @@ public class ViewPane extends PartPane implements IPropertyListener {
         return internalGetToolbar();
     }
     
-    private ToolBar internalGetToolbar() {
+    private Control internalGetToolbar() {
         if (isvToolBarMgr == null) {
             return null;
         }
         
-        return isvToolBarMgr.getControl();        
+        return isvToolBarMgr.getControl2();        
     }
 
     /* (non-Javadoc)
@@ -601,11 +614,23 @@ public class ViewPane extends PartPane implements IPropertyListener {
         return perspective.isCloseable(getViewReference());
     }
     
+    public void showSystemMenu() {
+		if (isFastView()) {
+	        Perspective perspective = page.getActivePerspective();
+	        if (perspective != null) {
+				perspective.getFastViewPane().showSystemMenu();
+			}
+		} else {
+			super.showSystemMenu();	
+		}
+    } 
+    
     public void showPaneMenu() {
 		if (isFastView()) {
 	        Perspective perspective = page.getActivePerspective();
-	        
-	        perspective.getFastViewPane().showSystemMenu();
+			if (perspective != null) {
+				perspective.getFastViewPane().showPaneMenu();
+			}
 		} else {
 			super.showPaneMenu();	
 		}
