@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,6 +10,7 @@
  *******************************************************************************/
 package org.eclipse.ui.part;
 
+import org.eclipse.core.runtime.Assert;
 import org.eclipse.core.runtime.IConfigurationElement;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.ui.IEditorInput;
@@ -19,7 +20,6 @@ import org.eclipse.ui.IPropertyListener;
 import org.eclipse.ui.IWorkbenchPartConstants;
 import org.eclipse.ui.IWorkbenchPartSite;
 import org.eclipse.ui.PartInitException;
-import org.eclipse.ui.internal.misc.Assert;
 
 /**
  * Abstract base implementation of all workbench editors.
@@ -54,7 +54,6 @@ import org.eclipse.ui.internal.misc.Assert;
  *   <li><code>IEditorPart.doSave</code> - to save contents of editor</li>
  *   <li><code>IEditorPart.doSaveAs</code> - to save contents of editor</li>
  *   <li><code>IEditorPart.isSaveAsAllowed</code> - to control Save As</li>
- *   <li><code>IEditorPart.gotoMarker</code> - to make selections based on markers</li>
  * </ul>
  * </p>
  * <p>
@@ -142,10 +141,11 @@ public abstract class EditorPart extends WorkbenchPart implements IEditorPart {
      * @return the tool tip text
      */
     public String getTitleToolTip() {
-        if (editorInput == null)
-            return super.getTitleToolTip();
-        else
-            return editorInput.getToolTipText();
+        if (editorInput == null) {
+			return super.getTitleToolTip();
+		} else {
+			return editorInput.getToolTipText();
+		}
     }
 
     /* (non-Javadoc)
@@ -201,13 +201,48 @@ public abstract class EditorPart extends WorkbenchPart implements IEditorPart {
     }
 
     /**
-     * Sets the input to this editor.
+     * Sets the input to this editor.  This method simply updates the internal
+     * member variable.
+     * 
+     * <p>Unlike most of the other set methods on this class, this method does 
+     * not fire a property change. Clients that call this method from a subclass 
+     * must ensure that they fire an IWorkbenchPartConstants.PROP_INPUT property 
+     * change after calling this method but before leaving whatever public method
+     * they are in. Clients that expose this method as public API must fire
+     * the property change within their implementation of setInput.</p>
+     * 
+     * <p>Note that firing a property change may cause listeners to immediately 
+     * reach back and call methods on this editor. Care should be taken not to 
+     * fire the property change until the editor has fully updated its internal 
+     * state to reflect the new input.</p>
      *
      * @param input the editor input
+     * 
+     * @see #setInputWithNotify(IEditorInput)
      */
     protected void setInput(IEditorInput input) {
         editorInput = input;
     }
+    
+    /**
+     * Sets the input to this editor and fires a PROP_INPUT property change if
+     * the input has changed.  This is the convenience method implementation.
+     * 
+     * <p>Note that firing a property change may cause other objects to reach back
+     * and invoke methods on the editor. Care should be taken not to call this method
+     * until the editor has fully updated its internal state to reflect the
+     * new input.</p>
+     * 
+     * @since 3.2
+     *
+     * @param input the editor input
+     */
+    protected void setInputWithNotify(IEditorInput input) {
+        if (input != editorInput) {
+            editorInput = input;
+            firePropertyChange(PROP_INPUT);
+        }
+    }
 
     /* (non-Javadoc)
      * @see org.eclipse.ui.part.WorkbenchPart#setContentDescription(java.lang.String)
