@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,23 +10,37 @@
  *******************************************************************************/
 package org.eclipse.update.internal.configurator;
 
-import java.io.*;
-import java.lang.reflect.*;
-import java.net.*;
-//PAL nio optional
-//import java.nio.channels.*;
-import java.util.*;
+import java.io.BufferedWriter;
+import java.io.File;
+import java.io.FileFilter;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.OutputStream;
+import java.io.OutputStreamWriter;
+import java.io.SyncFailedException;
+import java.io.UnsupportedEncodingException;
+import java.lang.reflect.InvocationTargetException;
+import java.net.MalformedURLException;
+import java.net.URL;
+import java.net.URLConnection;
+import java.util.ArrayList;
+import java.util.Date;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Properties;
+import java.util.Set;
 
-import javax.xml.parsers.*;
-//PAL cdcFoundation
-//import javax.xml.transform.*;
-//import javax.xml.transform.dom.*;
-//import javax.xml.transform.stream.*;
-import org.eclipse.core.runtime.*;
-import org.eclipse.osgi.service.datalocation.*;
+import javax.xml.parsers.DocumentBuilder;
+import javax.xml.parsers.DocumentBuilderFactory;
+
+import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.FileLocator;
+import org.eclipse.osgi.service.datalocation.Location;
 import org.eclipse.osgi.util.NLS;
-import org.eclipse.update.configurator.*;
-import org.w3c.dom.*;
+import org.eclipse.update.configurator.IPlatformConfiguration;
+import org.w3c.dom.Document;
+import org.w3c.dom.Element;
 
 /**
  * This class is responsible for providing the features and plugins (bundles) to 
@@ -58,6 +72,7 @@ public class PlatformConfiguration implements IPlatformConfiguration, IConfigura
 	//PAL nio optional
 	//private FileLock lock;
 	private Locker lock = null;
+	private static int defaultPolicy = DEFAULT_POLICY_TYPE;
 	private static boolean checkNio = false;
 	private static boolean useNio;
 
@@ -69,7 +84,7 @@ public class PlatformConfiguration implements IPlatformConfiguration, IConfigura
 	private static final String CONFIG_FILE_LOCK_SUFFIX = ".lock"; //$NON-NLS-1$
 	private static final String CONFIG_FILE_TEMP_SUFFIX = ".tmp"; //$NON-NLS-1$
 	private static final String LINKS = "links"; //$NON-NLS-1$
-	private static final String[] BOOTSTRAP_PLUGINS = {}; //$NON-NLS-1$
+	private static final String[] BOOTSTRAP_PLUGINS = {}; 
 
 	private static final String DEFAULT_FEATURE_APPLICATION = "org.eclipse.ui.ide.workbench"; //$NON-NLS-1$
 
@@ -85,6 +100,8 @@ public class PlatformConfiguration implements IPlatformConfiguration, IConfigura
 		
 		// initialize configuration
 		initializeCurrent(platformConfigLocation);
+		if(config != null)
+			setDefaultPolicy();
 
 		// Detect external links. These are "soft link" to additional sites. The link
 		// files are usually provided by external installation programs. They are located
@@ -113,6 +130,23 @@ public class PlatformConfiguration implements IPlatformConfiguration, IConfigura
 		initialize(url);
 	}
 
+	private void setDefaultPolicy() {
+		// Assumption: If the configuration that we initialize with 
+		// has a MANAGED_ONLY policy, then all sites should have default policy 
+		// of MANAGED_ONLY.  
+		ISiteEntry[] sentries = getConfiguredSites();
+		if(sentries != null && sentries.length >0){
+			int policyType = sentries[0].getSitePolicy().getType();
+			if(policyType == ISitePolicy.MANAGED_ONLY){
+				defaultPolicy = policyType;
+			}
+		}
+	}
+
+	public static int getDefaultPolicy(){
+		return defaultPolicy;
+	}
+
 	/*
 	 * @see IPlatformConfiguration#createSiteEntry(URL, ISitePolicy)
 	 */
@@ -354,7 +388,7 @@ public class PlatformConfiguration implements IPlatformConfiguration, IConfigura
 
 	public String getApplicationIdentifier() {
 		// Return the app if defined in system properties
-		String application = System.getProperty(ECLIPSE_APPLICATION);
+		String application = ConfigurationActivator.getBundleContext().getProperty(ECLIPSE_APPLICATION);
 		if (application != null)
 			return application;
 
@@ -379,7 +413,7 @@ public class PlatformConfiguration implements IPlatformConfiguration, IConfigura
 	 */
 	public String getPrimaryFeatureIdentifier() {
 		// Return the product if defined in system properties
-		String primaryFeatureId = System.getProperty(ECLIPSE_PRODUCT);
+		String primaryFeatureId = ConfigurationActivator.getBundleContext().getProperty(ECLIPSE_PRODUCT);
 		if (primaryFeatureId != null) {
 			// check if feature exists
 			IFeatureEntry feature = findConfiguredFeatureEntry(primaryFeatureId);
@@ -755,7 +789,7 @@ public class PlatformConfiguration implements IPlatformConfiguration, IConfigura
 	}
 	private ISiteEntry getRootSite() {
 		// create default site entry for the root
-		ISitePolicy defaultPolicy = createSitePolicy(DEFAULT_POLICY_TYPE, DEFAULT_POLICY_LIST);
+		ISitePolicy defaultPolicy = createSitePolicy(getDefaultPolicy(), DEFAULT_POLICY_LIST);
 		URL siteURL = null;
 		try {
 			siteURL = new URL("platform:/base/"); //$NON-NLS-1$  // try using platform-relative URL
@@ -934,7 +968,7 @@ public class PlatformConfiguration implements IPlatformConfiguration, IConfigura
 		SiteEntry linkSite = (SiteEntry) externalLinkSites.get(siteURL);
 		if (linkSite == null) {
 			// this is a link to a new target so create site for it
-			ISitePolicy linkSitePolicy = createSitePolicy(DEFAULT_POLICY_TYPE, DEFAULT_POLICY_LIST);
+			ISitePolicy linkSitePolicy = createSitePolicy(getDefaultPolicy(), DEFAULT_POLICY_LIST);
 			linkSite = (SiteEntry) createSiteEntry(siteURL, linkSitePolicy);
 		}
 		// update site entry if needed
@@ -1103,10 +1137,8 @@ public class PlatformConfiguration implements IPlatformConfiguration, IConfigura
 	}
 
 	public static URL resolvePlatformURL(URL url) throws IOException {
-		// 19536
 		if (url.getProtocol().equals("platform")) { //$NON-NLS-1$
-			url = Platform.asLocalURL(url);
-			// TODO URL resolution by platform returns url file:d:/path as opposed to file:/d:/path
+			url = FileLocator.toFileURL(url);
 			File f = new File(url.getFile());
 			url = f.toURL();
 		}
