@@ -1,38 +1,69 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.update.internal.core;
 
 
-import java.io.*;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
+import java.io.OutputStreamWriter;
 import java.net.MalformedURLException;
 import java.net.URL;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Date;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.List;
+import java.util.MissingResourceException;
+import java.util.PropertyResourceBundle;
+import java.util.Set;
 
-import org.eclipse.core.boot.BootLoader;
-import org.eclipse.core.boot.IPlatformConfiguration;
-import org.eclipse.core.runtime.*;
-import org.eclipse.update.configuration.*;
-import org.eclipse.update.core.*;
+import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.FileLocator;
+import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.core.runtime.IStatus;
+import org.eclipse.core.runtime.MultiStatus;
+import org.eclipse.osgi.util.NLS;
+import org.eclipse.update.configuration.IActivity;
+import org.eclipse.update.configuration.IConfiguredSite;
+import org.eclipse.update.configuration.IConfiguredSiteChangedListener;
+import org.eclipse.update.configuration.IInstallConfiguration;
+import org.eclipse.update.configuration.IProblemHandler;
+import org.eclipse.update.configurator.ConfiguratorUtils;
+import org.eclipse.update.configurator.IPlatformConfiguration;
+import org.eclipse.update.core.IFeature;
+import org.eclipse.update.core.IFeatureReference;
+import org.eclipse.update.core.IIncludedFeatureReference;
+import org.eclipse.update.core.IPluginEntry;
+import org.eclipse.update.core.ISite;
+import org.eclipse.update.core.ISiteFeatureReference;
+import org.eclipse.update.core.IVerificationListener;
+import org.eclipse.update.core.SiteManager;
+import org.eclipse.update.core.Utilities;
+import org.eclipse.update.core.VersionedIdentifier;
 import org.eclipse.update.core.model.InstallAbortedException;
 import org.eclipse.update.internal.model.ConfiguredSiteModel;
-import org.eclipse.update.internal.model.InstallConfigurationParser;
+import org.eclipse.update.internal.operations.UpdateUtils;
 
 /**
  * A Configured site manages the Configured and unconfigured features of a Site
  */
-public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSite, IWritable {
+public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSite {
 
-	private static final String PRODUCT_SITE_MARKER = ".eclipseproduct";
-	private static final String EXTENSION_SITE_MARKER = ".eclipseextension";
-	private static final String PRIVATE_SITE_MARKER = ".eclipseUM";
+	private static final String PRODUCT_SITE_MARKER = ".eclipseproduct"; //$NON-NLS-1$
+	private static final String EXTENSION_SITE_MARKER = ".eclipseextension"; //$NON-NLS-1$
 
 	// listeners	
 	private ListenersList listeners = new ListenersList();
@@ -81,77 +112,6 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		}
 	}
 
-	/*
-	 * @see IWritable#write(int, PrintWriter)
-	 */
-	public void write(int indent, PrintWriter w) {
-		String gap = ""; //$NON-NLS-1$
-		for (int i = 0; i < indent; i++)
-			gap += " "; //$NON-NLS-1$
-		String increment = ""; //$NON-NLS-1$
-		for (int i = 0; i < IWritable.INDENT; i++)
-			increment += " "; //$NON-NLS-1$
-
-		// CONFIGURATION SITE	
-		w.print(gap + "<" + InstallConfigurationParser.CONFIGURATION_SITE + " ");
-		//$NON-NLS-1$ //$NON-NLS-2$
-		w.println("url=\"" + getSite().getURL().toExternalForm() + "\"");
-		//$NON-NLS-1$ //$NON-NLS-2$
-		w.println(gap + increment + "platformURL=\"" + getPlatformURLString() + "\"");
-		//$NON-NLS-1$ //$NON-NLS-2$
-		w.println(gap + increment + "enable=\"" + (isEnabled() ? "true" : "false") + "\"");
-		//$NON-NLS-1$ //$NON-NLS-2$
-		w.println(gap + increment + "policy=\"" + getConfigurationPolicy().getPolicy() + "\" >");
-		//$NON-NLS-1$ //$NON-NLS-2$
-
-		// configured features ref
-		IFeatureReference[] featuresReferences = getRawConfiguredFeatures();
-		if (featuresReferences != null) {
-			for (int index = 0; index < featuresReferences.length; index++) {
-				IFeatureReference element = featuresReferences[index];
-				w.print(gap + increment + "<" + InstallConfigurationParser.FEATURE + " ");
-				//$NON-NLS-1$ //$NON-NLS-2$
-				// configured = true
-				w.print("configured = \"true\" "); //$NON-NLS-1$
-				// feature URL
-				String URLInfoString = null;
-				if (element.getURL() != null) {
-					ISite featureSite = element.getSite();
-					URLInfoString = UpdateManagerUtils.getURLAsString(featureSite.getURL(), element.getURL());
-					w.print("url=\"" + UpdateManagerUtils.Writer.xmlSafe(URLInfoString) + "\" ");
-					//$NON-NLS-1$ //$NON-NLS-2$
-				}
-				w.println("/>"); //$NON-NLS-1$
-			}
-		}
-
-		// unconfigured features ref
-		featuresReferences = getConfigurationPolicy().getUnconfiguredFeatures();
-		if (featuresReferences != null) {
-			for (int index = 0; index < featuresReferences.length; index++) {
-				IFeatureReference element = featuresReferences[index];
-				w.print(gap + increment + "<" + InstallConfigurationParser.FEATURE + " ");
-				//$NON-NLS-1$ //$NON-NLS-2$
-				// configured = true
-				w.print("configured = \"false\" "); //$NON-NLS-1$
-				// feature URL
-				String URLInfoString = null;
-				if (element.getURL() != null) {
-					ISite featureSite = element.getSite();
-					URLInfoString = UpdateManagerUtils.getURLAsString(featureSite.getURL(), element.getURL());
-					w.print("url=\"" + UpdateManagerUtils.Writer.xmlSafe(URLInfoString) + "\" ");
-					//$NON-NLS-1$ //$NON-NLS-2$
-				}
-				w.println("/>"); //$NON-NLS-1$
-			}
-		}
-
-		// end
-		w.println(gap + "</" + InstallConfigurationParser.CONFIGURATION_SITE + ">");
-		//$NON-NLS-1$ //$NON-NLS-2$
-		w.println(""); //$NON-NLS-1$		
-	}
-
 	/*
 	 * @see IConfiguredSite#install(IFeature,IVerificationListener, IProgressMonitor)
 	 */
@@ -169,15 +129,16 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 
 		// ConfigSite is read only 
 		if (!isUpdatable()) {
-			String errorMessage = Policy.bind("ConfiguredSite.NonInstallableSite", getSite().getURL().toExternalForm());
-			//$NON-NLS-1$
+			String errorMessage = NLS.bind(Messages.ConfiguredSite_NonInstallableSite, (new String[] { getSite().getURL().toExternalForm() }));
+			IStatus status = verifyUpdatableStatus();
+			if (status != null)
+				errorMessage += " " + status.getMessage(); //$NON-NLS-1$
 			throw Utilities.newCoreException(errorMessage, null);
 		}
 
 		// feature is null
 		if (feature == null) {
-			String errorMessage = Policy.bind("ConfiguredSite.NullFeatureToInstall");
-			//$NON-NLS-1$
+			String errorMessage = Messages.ConfiguredSite_NullFeatureToInstall; 
 			throw Utilities.newCoreException(errorMessage, null);
 		}
 
@@ -194,7 +155,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 			installedFeatureRef = getSite().install(feature, optionalFeatures, verificationListener, monitor);
 
 			if (UpdateCore.DEBUG && UpdateCore.DEBUG_SHOW_INSTALL) {
-				UpdateCore.debug("Sucessfully installed: " + installedFeatureRef.getURL().toExternalForm());
+				UpdateCore.debug("Sucessfully installed: " + installedFeatureRef.getURL().toExternalForm()); //$NON-NLS-1$
 			}
 
 			if (installedFeatureRef != null) {
@@ -226,7 +187,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		}
 		// call the configure task	
 		if (installedFeature != null)
-			configure(installedFeature, optionalFeatures, false);
+			configure(installedFeature, optionalFeatures, true);
 		/*callInstallHandler*/
 
 		return installedFeatureRef;
@@ -239,8 +200,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 
 		// ConfigSite is read only
 		if (!isUpdatable()) {
-			String errorMessage = Policy.bind("ConfiguredSite.NonUninstallableSite", getSite().getURL().toExternalForm());
-			//$NON-NLS-1$
+			String errorMessage = NLS.bind(Messages.ConfiguredSite_NonUninstallableSite, (new String[] { getSite().getURL().toExternalForm() }));
 			throw Utilities.newCoreException(errorMessage, null);
 		}
 
@@ -266,15 +226,12 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 				// our UI will check.
 				// For non-UI application, throw error is feature is configured
 				if (getConfigurationPolicy().isConfigured(referenceToRemove)) {
-					IFeature featureToRemove = ((IFeatureReference) referenceToRemove).getFeature(null);
+					IFeature featureToRemove = referenceToRemove.getFeature(null);
 					String featureLabel = (featureToRemove == null) ? null : featureToRemove.getLabel();
-					throw Utilities.newCoreException(Policy.bind("ConfiguredSite.UnableToRemoveConfiguredFeature"
-					//$NON-NLS-1$
-					, featureLabel), null);
+					throw Utilities.newCoreException(NLS.bind(Messages.ConfiguredSite_UnableToRemoveConfiguredFeature, (new String[] { featureLabel })), null);
 				}
 			} else {
-				throw Utilities.newCoreException(Policy.bind("ConfiguredSite.UnableToFindFeature", feature.getURL().toString()),
-				//$NON-NLS-1$
+				throw Utilities.newCoreException(NLS.bind(Messages.ConfiguredSite_UnableToFindFeature, (new String[] { feature.getURL().toString() })),
 				null);
 			}
 
@@ -311,7 +268,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 	private void configure(IFeature feature, IFeatureReference[] optionalFeatures, boolean callInstallHandler) throws CoreException {
 
 		if (feature == null) {
-			UpdateCore.warn("Attempting to configure a null feature in site:" + getSite().getURL().toExternalForm());
+			UpdateCore.warn("Attempting to configure a null feature in site:" + getSite().getURL().toExternalForm()); //$NON-NLS-1$
 			return;
 		}
 
@@ -332,7 +289,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 			} catch (CoreException e) {
 				// will skip any bad children
 				if (!childrenRef[i].isOptional())
-					UpdateCore.warn("Unable to configure child feature: " + childrenRef[i] + " " + e);
+					UpdateCore.warn("Unable to configure child feature: " + childrenRef[i] + " " + e); //$NON-NLS-1$ //$NON-NLS-2$
 			}
 		}
 
@@ -367,12 +324,12 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 					// where children are on the local site
 					try {
 						IFeature installedChildren = optionalfeatures[j].getFeature(null);
-						if (installedChildren.equals(optionalFeatureToConfigure.getFeature(true, null, null))) {
+						if (installedChildren.equals(optionalFeatureToConfigure.getFeature(null))) {
 							childrenToInstall.add(optionalFeatureToConfigure);
 							break;
 						}
 					} catch (CoreException e) {
-						UpdateCore.warn("", e);
+						UpdateCore.warn("", e); //$NON-NLS-1$
 					}
 				}
 			}
@@ -398,7 +355,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		IFeatureReference featureReference = getSite().getFeatureReference(feature);
 
 		if (featureReference == null) {
-			UpdateCore.warn("Unable to retrieve Feature Reference for feature" + feature);
+			UpdateCore.warn("Unable to retrieve Feature Reference for feature" + feature); //$NON-NLS-1$
 			return false;
 		}
 
@@ -408,7 +365,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 
 		// verify no enable parent
 		if (verifyEnableParent && !validateNoConfiguredParents(feature)) {
-			UpdateCore.warn("The feature " + feature.getVersionedIdentifier() + " to disable is needed by another enable feature");
+			UpdateCore.warn("The feature " + feature.getVersionedIdentifier() + " to disable is needed by another enable feature"); //$NON-NLS-1$ //$NON-NLS-2$
 			return false;
 		}
 
@@ -417,8 +374,8 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 			sucessfullyUnconfigured = configPolicy.unconfigure(featureReference, true, true);
 		} catch (CoreException e) {
 			URL url = featureReference.getURL();
-			String urlString = (url != null) ? url.toExternalForm() : "<no feature reference url>";
-			UpdateCore.warn("Unable to unconfigure" + urlString, e);
+			String urlString = (url != null) ? url.toExternalForm() : "<no feature reference url>"; //$NON-NLS-1$
+			UpdateCore.warn("Unable to unconfigure" + urlString, e); //$NON-NLS-1$
 			throw e;
 		}
 		if (sucessfullyUnconfigured) {
@@ -438,11 +395,11 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 			IIncludedFeatureReference[] childrenRef = feature.getIncludedFeatureReferences();
 			for (int i = 0; i < childrenRef.length; i++) {
 				try {
-					IFeature child = childrenRef[i].getFeature(true, null, null); // disable the exact feature
+					IFeature child = childrenRef[i].getFeature(null); // disable the exact feature
 					unconfigure(child, includePatches, true); // check for parent as we should be the only parent.
 				} catch (CoreException e) {
 					// skip any bad children
-					UpdateCore.warn("Unable to unconfigure child feature: " + childrenRef[i] + " " + e);
+					UpdateCore.warn("Unable to unconfigure child feature: " + childrenRef[i] + " " + e); //$NON-NLS-1$ //$NON-NLS-2$
 				}
 			}
 
@@ -456,8 +413,8 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 			return true;
 		} else {
 			URL url = featureReference.getURL();
-			String urlString = (url != null) ? url.toExternalForm() : "<no feature reference url>";
-			UpdateCore.warn("Unable to unconfigure:" + urlString);
+			String urlString = (url != null) ? url.toExternalForm() : "<no feature reference url>"; //$NON-NLS-1$
+			UpdateCore.warn("Unable to unconfigure:" + urlString); //$NON-NLS-1$
 			return false;
 		}
 	}
@@ -479,19 +436,10 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 				if (candidate.equals(feature))
 					continue;
 
-				IImport[] imports = candidate.getImports();
-				for (int j = 0; j < imports.length; j++) {
-					IImport iimport = imports[j];
-					if (iimport.isPatch()) {
-						if (iimport.getVersionedIdentifier().equals(feature.getVersionedIdentifier())) {
-							// bingo - unconfigure this patch
-							unconfigure(candidate, false, false);
-							break;
-						}
-					}
-				}
+				if (UpdateUtils.isPatch(feature, candidate))
+					unconfigure(candidate, false, false);
 			} catch (CoreException e) {
-				UpdateCore.warn("", e);
+				UpdateCore.warn("", e); //$NON-NLS-1$
 			}
 		}
 	}
@@ -576,8 +524,8 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 				// log no feature to unconfigure
 				String url = element.getURL().toString();
 				ISite site = element.getSite();
-				String siteString = (site != null) ? site.getURL().toExternalForm() : Policy.bind("ConfiguredSite.NoSite"); //$NON-NLS-1$
-				UpdateCore.warn(Policy.bind("ConfiguredSite.CannotFindFeatureToUnconfigure", url, siteString), e); //$NON-NLS-1$ 
+				String siteString = (site != null) ? site.getURL().toExternalForm() : Messages.ConfiguredSite_NoSite; 
+				UpdateCore.warn(NLS.bind(Messages.ConfiguredSite_CannotFindFeatureToUnconfigure, (new String[] { url, siteString })), e); 
 			}
 		}
 		//} // end USER_EXCLUDE
@@ -602,7 +550,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 					// notify we cannot find the feature
 					UpdateCore.warn(null, e);
 					String featureString = configuredFeatures[i].getURL().toExternalForm();
-					if (!handler.reportProblem(Policy.bind("ConfiguredSite.CannotFindFeatureToConfigure", featureString))) { //$NON-NLS-1$
+					if (!handler.reportProblem(NLS.bind(Messages.ConfiguredSite_CannotFindFeatureToConfigure, (new String[] { featureString })))) {
 						throw new InterruptedException();
 					}
 				}
@@ -629,19 +577,15 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 							if (!contains(currentFeaturePluginEntry.getVersionedIdentifier(), sitePluginIdentifiers)) {
 								// the plugin defined by the feature
 								// doesn't seem to exist on the site
-								String msg = "Error verifying existence of plugin:" + currentFeaturePluginEntry.getVersionedIdentifier().toString();
-								//$NON-NLS-1$
+								String msg = "Error verifying existence of plugin:" + currentFeaturePluginEntry.getVersionedIdentifier().toString(); //$NON-NLS-1$
 								UpdateCore.log(msg, new Exception());
 
-								String siteString = (site != null) ? site.getURL().toExternalForm() : Policy.bind("ConfiguredSite.NoSite");
-								//$NON-NLS-1$
-								String errorLabel = Policy.bind("ConfiguredSite.CannotFindPluginEntry", currentFeaturePluginEntry.getVersionedIdentifier().toString(), siteString);
-								//$NON-NLS-1$ //$NON-NLS-2$
+								String siteString = (site != null) ? site.getURL().toExternalForm() : Messages.ConfiguredSite_NoSite;	
+								String errorLabel = NLS.bind(Messages.ConfiguredSite_CannotFindPluginEntry, (new String[] { currentFeaturePluginEntry.getVersionedIdentifier().toString(), siteString }));
 								if (handler == null) {
 									throw new InterruptedException(errorLabel);
 								}
-								if (!handler.reportProblem(Policy.bind(errorLabel))) {
-									//$NON-NLS-1$ //$NON-NLS-2$
+								if (!handler.reportProblem(errorLabel)) {
 									throw new InterruptedException();
 								}
 							} // end if not found in site
@@ -765,7 +709,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 	 */
 	public IStatus getBrokenStatus(IFeature feature) {
 
-		IStatus featureStatus = createStatus(IStatus.OK, IFeature.STATUS_HAPPY, "", null);
+		IStatus featureStatus = createStatus(IStatus.OK, IFeature.STATUS_HAPPY, "", null); //$NON-NLS-1$
 
 		// check the Plugins of all the features
 		// every plugin of the feature must be on the site
@@ -773,17 +717,16 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		IPluginEntry[] featuresEntries = feature.getPluginEntries();
 		IPluginEntry[] result = UpdateManagerUtils.diff(featuresEntries, siteEntries);
 		if (result != null && (result.length != 0)) {
-			String msg = Policy.bind("SiteLocal.FeatureUnHappy");
+			String msg = Messages.SiteLocal_FeatureUnHappy; 
 			MultiStatus multi = new MultiStatus(featureStatus.getPlugin(), IFeature.STATUS_UNHAPPY, msg, null);
 
 			for (int k = 0; k < result.length; k++) {
 				VersionedIdentifier id = result[k].getVersionedIdentifier();
-				Object[] values = new String[] { "", "" };
+				Object[] values = new String[] { "", "" }; //$NON-NLS-1$ //$NON-NLS-2$
 				if (id != null) {
 					values = new Object[] { id.getIdentifier(), id.getVersion()};
 				}
-				String msg1 = Policy.bind("ConfiguredSite.MissingPluginsBrokenFeature", values);
-				//$NON-NLS-1$
+				String msg1 = NLS.bind(Messages.ConfiguredSite_MissingPluginsBrokenFeature, values);
 				UpdateCore.warn(msg1);
 				IStatus status = createStatus(IStatus.ERROR, IFeature.STATUS_UNHAPPY, msg1, null);
 				multi.add(status);
@@ -793,7 +736,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 
 		// check os, arch, and ws
 
-		String msg = Policy.bind("SiteLocal.FeatureHappy");
+		String msg = Messages.SiteLocal_FeatureHappy; 
 		return createStatus(IStatus.OK, IFeature.STATUS_HAPPY, msg, null);
 	}
 
@@ -809,7 +752,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		IFeatureReference featureReference = getSite().getFeatureReference(feature);
 		if (featureReference == null) {
 			if (UpdateCore.DEBUG && UpdateCore.DEBUG_SHOW_WARNINGS)
-				UpdateCore.warn("Unable to retrieve featureReference for feature:" + feature);
+				UpdateCore.warn("Unable to retrieve featureReference for feature:" + feature); //$NON-NLS-1$
 			return false;
 		}
 		return getConfigurationPolicy().isConfigured(featureReference);
@@ -820,9 +763,9 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 	 */
 	public String toString() {
 		if (getSite() == null)
-			return "No Site";
+			return "No Site"; //$NON-NLS-1$
 		if (getSite().getURL() == null)
-			return "No URL";
+			return "No URL"; //$NON-NLS-1$
 		return getSite().getURL().toExternalForm();
 	}
 
@@ -836,12 +779,12 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 
 		URL siteURL = getSite().getURL();
 		if (siteURL == null) {
-			verifyStatus = createStatus(IStatus.ERROR, Policy.bind("ConfiguredSite.SiteURLNull"), null); //$NON-NLS-1$
+			verifyStatus = createStatus(IStatus.ERROR, Messages.ConfiguredSite_SiteURLNull, null); 
 			return verifyStatus;
 		}
 
-		if (!"file".equalsIgnoreCase(siteURL.getProtocol())) {
-			verifyStatus = createStatus(IStatus.ERROR, Policy.bind("ConfiguredSite.NonLocalSite"), null); //$NON-NLS-1$
+		if (!"file".equalsIgnoreCase(siteURL.getProtocol())) { //$NON-NLS-1$
+			verifyStatus = createStatus(IStatus.ERROR, Messages.ConfiguredSite_NonLocalSite, null); 
 			return verifyStatus;
 		}
 
@@ -854,24 +797,24 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		// otherwise don't check if we are contained in another site
 		String productName = getProductName(file);
 		if (productName != null) {
-			if (!productName.equals(getProductIdentifier("id", getProductFile()))) {
-				verifyStatus = createStatus(IStatus.ERROR, Policy.bind("ConfiguredSite.NotSameProductId", productName), null); //$NON-NLS-1$
+			if (!productName.equals(getProductIdentifier("id", getProductFile()))) { //$NON-NLS-1$
+				verifyStatus = createStatus(IStatus.ERROR, NLS.bind(Messages.ConfiguredSite_NotSameProductId, (new String[] { productName })), null);
 				return verifyStatus;
 			}
 		} else {
 			File container = getSiteContaining(file);
 			if (container != null) {
-				verifyStatus = createStatus(IStatus.ERROR, Policy.bind("ConfiguredSite.ContainedInAnotherSite", container.getAbsolutePath()), null); //$NON-NLS-1$
+				verifyStatus = createStatus(IStatus.ERROR, NLS.bind(Messages.ConfiguredSite_ContainedInAnotherSite, (new String[] { container.getAbsolutePath() })), null);
 				return verifyStatus;
 			}
 		}
 
 		if (!canWrite(file)) {
-			verifyStatus = createStatus(IStatus.ERROR, Policy.bind("ConfiguredSite.ReadOnlySite"), null); //$NON-NLS-1$
+			verifyStatus = createStatus(IStatus.ERROR, Messages.ConfiguredSite_ReadOnlySite, null); 
 			return verifyStatus;
 		}
 
-		verifyStatus = createStatus(IStatus.OK, "", null);
+		verifyStatus = createStatus(IStatus.OK, "", null); //$NON-NLS-1$
 		setUpdatable(true);
 		return verifyStatus;
 	}
@@ -887,7 +830,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		File tryFile = null;
 		FileOutputStream out = null;
 		try {
-			tryFile = new File(file, "toDelete");
+			tryFile = new File(file, "toDelete"); //$NON-NLS-1$
 			out = new FileOutputStream(tryFile);
 			out.write(0);
 		} catch (IOException e) {
@@ -897,7 +840,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 				if (out != null)
 					out.close();
 			} catch (IOException e) {
-			};
+			}
 			if (tryFile != null)
 				tryFile.delete();
 		}
@@ -914,19 +857,19 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		if (file == null)
 			return null;
 
-		UpdateCore.warn("IsContained: Checking for markers at:" + file);
+		UpdateCore.warn("IsContained: Checking for markers at:" + file); //$NON-NLS-1$
 		if (file.exists() && file.isDirectory()) {
 			File productFile = new File(file, PRODUCT_SITE_MARKER);
 			File extensionFile = new File(file, EXTENSION_SITE_MARKER);
 			if (productFile.exists() || extensionFile.exists())
 				return file;
-			// do not check if a marker exists in the current but start from the parent
-			// the current is analyze by getProductname()
-			if (file.getParentFile() != null) {
-				File privateFile = new File(file.getParentFile(), PRIVATE_SITE_MARKER);
-				if (privateFile.exists())
-					return file.getParentFile();
-			}
+//			// do not check if a marker exists in the current but start from the parent
+//			// the current is analyze by getProductname()
+//			if (file.getParentFile() != null) {
+//				File privateFile = new File(file.getParentFile(), PRIVATE_SITE_MARKER);
+//				if (privateFile.exists())
+//					return file.getParentFile();
+//			}
 		}
 		return getSiteContaining(file.getParentFile());
 	}
@@ -941,7 +884,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		if (file == null)
 			return null;
 
-		File markerFile = new File(file, PRIVATE_SITE_MARKER);
+		File markerFile = new File(file, EXTENSION_SITE_MARKER );
 		if (!markerFile.exists()) {
 			return null;
 		}
@@ -950,25 +893,25 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		String productId = null;
 		String privateId = null;
 		if (productFile != null) {
-			productId = getProductIdentifier("id", productFile);
-			privateId = getProductIdentifier("id", markerFile);
+			productId = getProductIdentifier("id", productFile); //$NON-NLS-1$
+			privateId = getProductIdentifier("id", markerFile); //$NON-NLS-1$
 			if (productId == null) {
-				UpdateCore.warn("Product ID is null at:" + productFile);
+				UpdateCore.warn("Product ID is null at:" + productFile); //$NON-NLS-1$
 				return null;
 			}
 			if (!productId.equalsIgnoreCase(privateId)) {
-				UpdateCore.warn("Product id at" + productFile + " Different than:" + markerFile);
-				String name = getProductIdentifier("name", markerFile);
-				String version = getProductIdentifier("version", markerFile);
-				String markerID = (name == null) ? version : name + ":" + version;
+				UpdateCore.warn("Product id at" + productFile + " Different than:" + markerFile); //$NON-NLS-1$ //$NON-NLS-2$
+				String name = getProductIdentifier("name", markerFile); //$NON-NLS-1$
+				String version = getProductIdentifier("version", markerFile); //$NON-NLS-1$
+				String markerID = (name == null) ? version : name + ":" + version; //$NON-NLS-1$
 				if (markerID == null)
-					markerID = "";
+					markerID = ""; //$NON-NLS-1$
 				return markerID;
 			} else {
 				return privateId;
 			}
 		} else {
-			UpdateCore.warn("Product Marker doesn't exist:" + productFile);
+			UpdateCore.warn("Product Marker doesn't exist:" + productFile); //$NON-NLS-1$
 		}
 
 		return null;
@@ -981,16 +924,23 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		String result = null;
 		if (identifier == null)
 			return result;
+		InputStream in = null;
 		try {
-			InputStream in = new FileInputStream(propertyFile);
+			in = new FileInputStream(propertyFile);
 			PropertyResourceBundle bundle = new PropertyResourceBundle(in);
 			result = bundle.getString(identifier);
 		} catch (IOException e) {
 			if (UpdateCore.DEBUG && UpdateCore.DEBUG_SHOW_INSTALL)
-				UpdateCore.debug("Exception reading property file:" + propertyFile);
+				UpdateCore.debug("Exception reading property file:" + propertyFile); //$NON-NLS-1$
 		} catch (MissingResourceException e) {
 			if (UpdateCore.DEBUG && UpdateCore.DEBUG_SHOW_INSTALL)
-				UpdateCore.debug("Exception reading '" + identifier + "' from property file:" + propertyFile);
+				UpdateCore.debug("Exception reading '" + identifier + "' from property file:" + propertyFile); //$NON-NLS-1$ //$NON-NLS-2$
+		} finally {
+			if (in == null)
+				try {
+					in.close();
+				} catch (IOException e1) {
+				}
 		}
 		return result;
 	}
@@ -1000,16 +950,16 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 	 */
 	private static File getProductFile() {
 
-		String productInstallDirectory = BootLoader.getInstallURL().getFile();
+		String productInstallDirectory = ConfiguratorUtils.getInstallURL().getFile();
 		if (productInstallDirectory != null) {
 			File productFile = new File(productInstallDirectory, PRODUCT_SITE_MARKER);
 			if (productFile.exists()) {
 				return productFile;
 			} else {
-				UpdateCore.warn("Product marker doesn't exist:" + productFile);
+				UpdateCore.warn("Product marker doesn't exist:" + productFile); //$NON-NLS-1$
 			}
 		} else {
-			UpdateCore.warn("Cannot retrieve install URL from BootLoader");
+			UpdateCore.warn("Cannot retrieve install URL from BootLoader"); //$NON-NLS-1$
 		}
 		return null;
 	}
@@ -1021,12 +971,12 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 	boolean createPrivateSiteMarker() {
 		URL siteURL = getSite().getURL();
 		if (siteURL == null) {
-			UpdateCore.warn("Unable to create marker. The Site url is null.");
+			UpdateCore.warn("Unable to create marker. The Site url is null."); //$NON-NLS-1$
 			return false;
 		}
 
-		if (!"file".equalsIgnoreCase(siteURL.getProtocol())) {
-			UpdateCore.warn("Unable to create private marker. The Site is not on the local file system.");
+		if (!"file".equalsIgnoreCase(siteURL.getProtocol())) { //$NON-NLS-1$
+			UpdateCore.warn("Unable to create private marker. The Site is not on the local file system."); //$NON-NLS-1$
 			return false;
 		}
 
@@ -1034,30 +984,37 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		File productFile = getProductFile();
 		boolean success = false;
 		if (productFile != null) {
-			String productId = getProductIdentifier("id", productFile);
-			String productName = getProductIdentifier("name", productFile);
-			String productVer = getProductIdentifier("version", productFile);
+			String productId = getProductIdentifier("id", productFile); //$NON-NLS-1$
+			String productName = getProductIdentifier("name", productFile); //$NON-NLS-1$
+			String productVer = getProductIdentifier("version", productFile); //$NON-NLS-1$
 			if (productId != null) {
-				File file = new File(siteLocation, PRIVATE_SITE_MARKER);
+				File file = new File(siteLocation, EXTENSION_SITE_MARKER);
 				if (!file.exists()) {
-					PrintWriter w = null;
+					OutputStream out = null;
+					OutputStreamWriter outWriter = null;
 					try {
-						OutputStream out = new FileOutputStream(file);
-						OutputStreamWriter outWriter = new OutputStreamWriter(out, "UTF8"); //$NON-NLS-1$
-						BufferedWriter buffWriter = new BufferedWriter(outWriter);
-						w = new PrintWriter(buffWriter);
-						w.println("id=" + productId);
+						out = new FileOutputStream(file);
+						outWriter = new OutputStreamWriter(out, "UTF8"); //$NON-NLS-1$
+						outWriter.write("id=" + productId+"\n"); //$NON-NLS-1$ //$NON-NLS-2$
 						if (productName != null)
-							w.println("name=" + productName);
+							outWriter.write("name=" + productName+"\n"); //$NON-NLS-1$ //$NON-NLS-2$
 						if (productVer != null)
-							w.println("version=" + productVer);
+							outWriter.write("version=" + productVer+"\n"); //$NON-NLS-1$ //$NON-NLS-2$
 						success = true;
 						justCreated = true;
 					} catch (Exception e) {
-						UpdateCore.warn("Unable to create private Marker at:" + file, e);
+						UpdateCore.warn("Unable to create private Marker at:" + file, e); //$NON-NLS-1$
 					} finally {
-						if (w != null)
-							w.close();
+						try {
+							if (outWriter != null)
+								outWriter.close();
+						} catch (IOException e1) {
+						}
+						try {
+							if (out != null)
+								out.close();
+						} catch (IOException e2) {
+						}
 					}
 				}
 			}
@@ -1065,41 +1022,6 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 		return success;
 	}
 
-	/*
-	 * 
-	 */
-	/*package*/
-	boolean removePrivateSiteMarker() {
-
-		if (!justCreated) {
-			UpdateCore.warn("Unable to remove marker. The site was not created during this activity.");
-			return false;
-		}
-
-		URL siteURL = getSite().getURL();
-		if (siteURL == null) {
-			UpdateCore.warn("Unable to remove marker. The Site url is null.");
-			return false;
-		}
-
-		if (!"file".equalsIgnoreCase(siteURL.getProtocol())) {
-			UpdateCore.warn("Unable to remove private marker. The Site is not on the local file system.");
-			return false;
-		}
-
-		String siteLocation = siteURL.getFile();
-		File productFile = getProductFile();
-		File file = new File(siteLocation, PRIVATE_SITE_MARKER);
-		boolean success = false;
-		if (file.exists()) {
-			try {
-				success = file.delete();
-			} catch (Exception e) {
-				UpdateCore.warn("Unable to remove private Marker at:" + file, e);
-			}
-		}
-		return success;
-	}
 
 	/*
 	 * Returns true if the directory of the Site contains
@@ -1122,7 +1044,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 	 * .eclipseextension
 	 */
 	public boolean isPrivateSite() {
-		return containsMarker(PRIVATE_SITE_MARKER);
+		return isExtensionSite();
 	}
 
 	/*
@@ -1131,27 +1053,27 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 	private boolean containsMarker(String marker) {
 		ISite site = getSite();
 		if (site == null) {
-			UpdateCore.warn("Contains Markers:The site is null");
+			UpdateCore.warn("Contains Markers:The site is null"); //$NON-NLS-1$
 			return false;
 		}
 
 		URL url = site.getURL();
 		if (url == null) {
-			UpdateCore.warn("Contains Markers:Site URL is null");
+			UpdateCore.warn("Contains Markers:Site URL is null"); //$NON-NLS-1$
 			return false;
 		}
-		if (!"file".equalsIgnoreCase(url.getProtocol())) {
-			UpdateCore.warn("Contains Markers:Non file protocol");
+		if (!"file".equalsIgnoreCase(url.getProtocol())) { //$NON-NLS-1$
+			UpdateCore.warn("Contains Markers:Non file protocol"); //$NON-NLS-1$
 			return false;
 		}
 		File file = new File(url.getFile());
 		if (!file.exists()) {
-			UpdateCore.warn("Contains Markers:The site doesn't exist:" + file);
+			UpdateCore.warn("Contains Markers:The site doesn't exist:" + file); //$NON-NLS-1$
 			return false;
 		}
 		File extension = new File(file, marker);
 		if (!extension.exists()) {
-			UpdateCore.warn("Contains Markers:The extensionfile does not exist:" + extension);
+			UpdateCore.warn("Contains Markers:The extensionfile does not exist:" + extension); //$NON-NLS-1$
 			return false;
 		}
 		return true;
@@ -1163,7 +1085,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 	public boolean isNativelyLinked() throws CoreException {
 		String platformString = getPlatformURLString();
 		if (platformString == null) {
-			UpdateCore.warn("Unable to retrieve platformString");
+			UpdateCore.warn("Unable to retrieve platformString"); //$NON-NLS-1$
 			return false;
 		}
 
@@ -1172,27 +1094,27 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 			// check if the site exists and is updatable
 			// update configSite
 			URL urlToCheck = new URL(platformString);
-			IPlatformConfiguration runtimeConfig = BootLoader.getCurrentPlatformConfiguration();
+			IPlatformConfiguration runtimeConfig = ConfiguratorUtils.getCurrentPlatformConfiguration();
 			IPlatformConfiguration.ISiteEntry entry = runtimeConfig.findConfiguredSite(urlToCheck);
 			if (entry != null) {
 				return entry.isNativelyLinked();
 			} else {
-				UpdateCore.warn("Unable to retrieve site:" + platformString + " from platform.");
+				UpdateCore.warn("Unable to retrieve site:" + platformString + " from platform."); //$NON-NLS-1$ //$NON-NLS-2$
 			}
 
 			// check by comparing URLs
 			IPlatformConfiguration.ISiteEntry[] sites = runtimeConfig.getConfiguredSites();
 			for (int i = 0; i < sites.length; i++) {
 				siteURL = sites[i].getURL();
-				URL resolvedURL = Platform.resolve(siteURL);
+				URL resolvedURL = FileLocator.resolve(siteURL);
 				if (UpdateManagerUtils.sameURL(resolvedURL, urlToCheck))
 					return true;
 			}
 		} catch (MalformedURLException e) {
-			String msg = Policy.bind("ConfiguredSite.UnableResolveURL", platformString);
+			String msg = NLS.bind(Messages.ConfiguredSite_UnableResolveURL, (new String[] { platformString }));
 			throw Utilities.newCoreException(msg, e);
 		} catch (IOException e) {
-			String msg = Policy.bind("ConfiguredSite.UnableToAccessSite", new Object[] { siteURL });
+			String msg = NLS.bind(Messages.ConfiguredSite_UnableToAccessSite, (new Object[] { siteURL }));
 			throw Utilities.newCoreException(msg, e);
 		}
 
@@ -1204,7 +1126,7 @@ public class ConfiguredSite extends ConfiguredSiteModel implements IConfiguredSi
 	*/
 	private boolean validateNoConfiguredParents(IFeature feature) throws CoreException {
 		if (feature == null) {
-			UpdateCore.warn("ConfigurationPolicy: validate Feature is null");
+			UpdateCore.warn("ConfigurationPolicy: validate Feature is null"); //$NON-NLS-1$
 			return true;
 		}
 
