@@ -1,18 +1,18 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.update.internal.core;
 
 
-import org.eclipse.update.core.IFeatureContentConsumer;
-import org.eclipse.update.core.model.ModelObject;
+import org.eclipse.update.core.*;
+import org.eclipse.update.core.model.*;
 
 /**
  * A default implementation for IFeatureContentConsumer
