@@ -26,6 +26,7 @@ import java.util.Map;
 import java.util.Set;
 
 import org.eclipse.core.runtime.CoreException;
+import org.eclipse.core.runtime.FileLocator;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.Platform;
 import org.eclipse.osgi.util.NLS;
@@ -50,7 +51,6 @@ import org.eclipse.update.core.VersionedIdentifier;
 import org.eclipse.update.core.model.SiteModel;
 import org.eclipse.update.internal.configurator.ConfigurationActivator;
 import org.eclipse.update.internal.configurator.FeatureEntry;
-import org.eclipse.update.internal.configurator.IConfigurationConstants;
 import org.eclipse.update.internal.configurator.PlatformConfiguration;
 import org.eclipse.update.internal.configurator.PluginEntry;
 import org.eclipse.update.internal.configurator.SiteEntry;
@@ -93,8 +93,8 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 		setCurrent(false);
 		
 		if (newLocation == null) {
-			String newFileName = UpdateManagerUtils.getLocalRandomIdentifier(SiteLocal.CONFIG_FILE, now);
-			newLocation = UpdateManagerUtils.getURL(((SiteLocal)SiteManager.getLocalSite()).getLocationURL(), newFileName, null);
+			String newFileName = UpdateManagerUtils.getLocalRandomIdentifier(LocalSite.CONFIG_FILE, now);
+			newLocation = UpdateManagerUtils.getURL(((LocalSite)SiteManager.getLocalSite()).getLocationURL(), newFileName, null);
 		}
 		setLocationURLString(newLocation.toExternalForm());
 		
@@ -135,7 +135,7 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 	 * Returns the default site policy
 	 */
 	private int getDefaultPolicy() {
-		return IConfigurationConstants.DEFAULT_POLICY_TYPE;
+		return PlatformConfiguration.getDefaultPolicy();
 	}
 
 	/**
@@ -561,7 +561,7 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 			// if the URL doesn't exits log it
 			IPlatformConfiguration.ISiteEntry siteEntry = runtimeConfiguration.findConfiguredSite(urlToCheck);
 			if (siteEntry == null) {
-				UpdateCore.warn(NLS.bind(Messages.InstallConfiguration_unableToFindSite, (new String[] { urlToCheck.toExternalForm(), runtimeConfiguration.getConfigurationLocation().toExternalForm() }))); //$NON-NLS-1$
+				UpdateCore.warn(NLS.bind(Messages.InstallConfiguration_unableToFindSite, (new String[] { urlToCheck.toExternalForm(), runtimeConfiguration.getConfigurationLocation().toExternalForm() }))); 
 			}
 		}
 	}
@@ -746,7 +746,7 @@ public class InstallConfiguration extends InstallConfigurationModel implements I
 				String location = fragments[i].getLocation();
 				try {
 					URL locationURL = new URL(location);
-					locationURL = Platform.asLocalURL(Platform.resolve(locationURL));
+					locationURL = FileLocator.toFileURL(FileLocator.resolve(locationURL));
 					list.add(asInstallRelativeURL(locationURL));
 				} catch (IOException e) {
 					// skip bad fragments
