@@ -1,16 +1,20 @@
 /*******************************************************************************
  * Copyright (c) 2000, 2003 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
 package org.eclipse.update.internal.core;
+
 import org.eclipse.core.runtime.CoreException;
-import org.eclipse.update.core.*;
+import org.eclipse.osgi.util.NLS;
+import org.eclipse.update.core.Feature;
+import org.eclipse.update.core.IFeatureContentConsumer;
+import org.eclipse.update.core.Utilities;
 
 /**
  * 
@@ -44,7 +48,7 @@ public class TargetFeature extends Feature {
 	 */
 	public IFeatureContentConsumer getFeatureContentConsumer() throws CoreException {
 		if (this.contentConsumer == null) {
-			throw Utilities.newCoreException( Policy.bind("Feature.NoFeatureContentConsumer", getURL().toExternalForm()), null); //$NON-NLS-1$
+			throw Utilities.newCoreException( NLS.bind(Messages.Feature_NoFeatureContentConsumer, (new String[] { getURL().toExternalForm() })), null);
 		}
 		return contentConsumer;
 	}
