@@ -13,9 +13,11 @@ package org.eclipse.update.internal.ui.wizards;
 import java.lang.reflect.InvocationTargetException;
 import java.net.URL;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashSet;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Set;
 
 import org.eclipse.core.runtime.CoreException;
@@ -62,6 +64,8 @@ import org.eclipse.update.core.ICategory;
 import org.eclipse.update.core.IFeature;
 import org.eclipse.update.core.IFeatureReference;
 import org.eclipse.update.core.IImport;
+import org.eclipse.update.core.IIncludedFeatureReference;
+import org.eclipse.update.core.IPluginEntry;
 import org.eclipse.update.core.ISiteFeatureReference;
 import org.eclipse.update.core.IURLEntry;
 import org.eclipse.update.core.Utilities;
@@ -366,7 +370,7 @@ public class ReviewPage	extends BannerPage {
 					feature.getIncludedFeatureReferences();
 				for (int i = 0; i < irefs.length; i++) {
 					IFeatureReference iref = irefs[i];
-					IFeature ifeature = iref.getFeature(null);
+					IFeature ifeature = UpdateUtils.getIncludedFeature(feature, iref);
 					VersionedIdentifier ivid =
 						ifeature.getVersionedIdentifier();
 					if (ivid.equals(vid))
@@ -428,7 +432,7 @@ public class ReviewPage	extends BannerPage {
 					continue;
 				if (!feature.getVersionedIdentifier().getIdentifier().equals(candidate.getVersionedIdentifier().getIdentifier()))
 					continue;
-				if (!feature.getVersionedIdentifier().getVersion().isGreaterThan(candidate.getVersionedIdentifier().getVersion()))
+				if (!feature.getVersionedIdentifier().getVersion().isGreaterOrEqualTo(candidate.getVersionedIdentifier().getVersion()))
 					return false;
 			}
 			return true;
@@ -584,20 +588,6 @@ public class ReviewPage	extends BannerPage {
 		buttonContainer.setLayout(layout);
 		buttonContainer.setLayoutData(new GridData(GridData.FILL_BOTH));
 
-//		Button button = new Button(buttonContainer, SWT.PUSH);
-//		button.setText(UpdateUI.getString("InstallWizard.ReviewPage.selectAll")); //$NON-NLS-1$
-//		gd =
-//			new GridData(
-//				GridData.HORIZONTAL_ALIGN_FILL
-//					| GridData.VERTICAL_ALIGN_BEGINNING);
-//		button.setLayoutData(gd);
-//		SWTUtil.setButtonDimensionHint(button);
-//		button.addSelectionListener(new SelectionAdapter() {
-//			public void widgetSelected(SelectionEvent e) {
-//				handleSelectAll(true);
-//			}
-//		});
-
 		Button button = new Button(buttonContainer, SWT.PUSH);
 		button.setText(UpdateUIMessages.InstallWizard_ReviewPage_deselectAll); 
 		gd =
@@ -777,6 +767,7 @@ public class ReviewPage	extends BannerPage {
 					getContainer().run(true, true,
 							getCheckStateOperation(event, parent.getDisplay()));
 					getContainer().updateButtons();
+					updateStatusButton();
 				} catch (InvocationTargetException e) {
 					UpdateUI.logException(e);
 				} catch (InterruptedException e) {
@@ -784,17 +775,6 @@ public class ReviewPage	extends BannerPage {
 				}
             }
         });
-
-//      treeViewer.addFilter(new ViewerFilter() {
-//          public boolean select(
-//              Viewer viewer,
-//              Object parentElement,
-//              Object element) {
-//              if (element instanceof SiteBookmark)
-//                  return !((SiteBookmark) element).isWebBookmark();
-//              return true;
-//          }
-//      });
         
       descLabel = new ScrolledFormText(sform, true);
       descLabel.setText(""); //$NON-NLS-1$
@@ -851,65 +831,11 @@ public class ReviewPage	extends BannerPage {
 		});
 	}
     
-    
-//    private void handleSiteChecked(SiteBookmark bookmark, boolean checked) {
-//
-//        bookmark.setSelected(checked);
-//        if (checked)
-//            bookmark.setIgnoredCategories(new String[0]);
-//            
-//        if (checked || bookmark.isSiteConnected())
-//            treeViewer.setSubtreeChecked(bookmark, checked);
-//   
-//        treeViewer.setGrayed(bookmark, false);
-//    }
-//    
-//    private void handleCategoryChecked(SiteCategory category, boolean checked) {
-//        SiteBookmark bookmark = category.getBookmark();
-//
-//        ArrayList array = new ArrayList();
-//
-//        if (bookmark.isSelected()) {
-//            String[] ignored = bookmark.getIgnoredCategories();
-//            for (int i = 0; i < ignored.length; i++)
-//                array.add(ignored[i]);
-//        } else {
-//            Object[] categs = bookmark.getCatalog(true, null);
-//            for (int i = 0; i < categs.length; i++)
-//                array.add(((SiteCategory) categs[i]).getFullName());
-//        }
-//
-//        if (checked) {
-//            array.remove(category.getFullName());
-//        } else {
-//            array.add(category.getFullName());
-//        }
-//
-//        bookmark.setIgnoredCategories((String[]) array.toArray(new String[array
-//                .size()]));
-//
-//        Object[] children = ((TreeContentProvider) treeViewer
-//                .getContentProvider()).getChildren(category.getBookmark());
-//        treeViewer.setChecked(bookmark, array.size() < children.length);
-//        bookmark.setSelected(array.size() < children.length);
-//        
-//        if (checked)
-//            treeViewer.setSubtreeChecked(bookmark, checked);
-//        
-//        treeViewer.setGrayed(bookmark, array.size() > 0
-//                && array.size() < children.length);
-//    }
-//
-//    private void handleFeatureChecked(IInstallFeatureOperation job, boolean checked) {
-//        treeViewer.setGrayed(job, false);
-//    }
-    
     private void handleSelectionChanged(IStructuredSelection ssel) {
-        //boolean enable = false;
+        
         Object item = ssel.getFirstElement();
         String description = null;
         if (item instanceof SiteBookmark) {
-            //enable = !((SiteBookmark) item).isReadOnly();
             description = ((SiteBookmark)item).getDescription();
         } else if (item instanceof SiteCategory) {
             IURLEntry descEntry = ((SiteCategory)item).getCategory().getDescription();
@@ -917,9 +843,6 @@ public class ReviewPage	extends BannerPage {
                 description = descEntry.getAnnotation();
         } else if (item instanceof IInstallFeatureOperation) {
             jobSelected(ssel);
-//            IURLEntry descEntry = ((IInstallFeatureOperation)item).getFeature().getDescription();
-//            if (descEntry != null)
-//                description = descEntry.getAnnotation();
             return;
         }
 
@@ -966,11 +889,9 @@ public class ReviewPage	extends BannerPage {
 			lastDisplayedStatus = null;
 			setErrorMessage(null);
 			setPageComplete(false);
-			validationStatus = null;
+			setValidationStatus(null);
 			problematicFeatures.clear();
 		}
-
-		statusButton.setEnabled(validationStatus != null);
 		treeViewer.setCheckedElements(jobsSelected);
 		//validateSelection();
         treeViewer.refresh();
@@ -978,14 +899,40 @@ public class ReviewPage	extends BannerPage {
         updateItemCount();
 	}
 	
+	private void setValidationStatus(IStatus newValidationStatus) {
+		this.validationStatus = newValidationStatus;
+		updateStatusButton();
+	}
+	
+	private void updateStatusButton() {
+		statusButton.getDisplay().syncExec(new Runnable() {
+			public void run() {
+				boolean newState = validationStatus != null && validationStatus.getSeverity() != IStatus.OK;
+				statusButton.setEnabled(newState);
+			}
+		});
+	}
+	
 	private void updateItemCount() {
 		updateItemCount(-1, -1);
 	}
+	
+	private int getSelectedJobsUniqueCount() {
+		Object[] checkedElements = getSelectedJobs();
+		Set set = new HashSet();
+		for (int i=0; i<checkedElements.length; i++) {
+			IInstallFeatureOperation job = (IInstallFeatureOperation)checkedElements[i];
+			IFeature feature = job.getFeature();
+			if (set.contains(feature))
+				continue;
+			set.add(feature);
+		}
+		return set.size();
+	}
 
 	private void updateItemCount(int checkedCount, int totalCount) {
 		if (checkedCount == -1) {
-			Object[] checkedElements = getSelectedJobs();
-			checkedCount = checkedElements.length;
+			checkedCount = getSelectedJobsUniqueCount();
 		}
 		if (totalCount == -1) {
 			totalCount = jobs.size();
@@ -1088,10 +1035,10 @@ public class ReviewPage	extends BannerPage {
 		IInstallFeatureOperation[] jobs = getSelectedJobs();
 		RequiredFeaturesResult requiredFeaturesResult = ((OperationValidator)OperationsManager
 				.getValidator()).getRequiredFeatures(jobs);
-		validationStatus = requiredFeaturesResult.getStatus();
+		setValidationStatus(requiredFeaturesResult.getStatus());
 		Set requiredFeatures = requiredFeaturesResult.getRequiredFeatures();
 		problematicFeatures.clear();
-
+		
 		Iterator requiredFeaturesIterator = requiredFeatures.iterator();
 		ArrayList toBeInstalled = new ArrayList();
 		
@@ -1139,7 +1086,8 @@ public class ReviewPage	extends BannerPage {
 		if (!toBeInstalled.isEmpty()) {
 			Iterator toBeInstalledIterator = toBeInstalled.iterator();
 			while (toBeInstalledIterator.hasNext()) {
-				treeViewer.setChecked(toBeInstalledIterator.next(), true);			
+				IInstallFeatureOperation current = (IInstallFeatureOperation)toBeInstalledIterator.next();
+				treeViewer.setChecked(current, true);			
 			}
 			return selectRequiredFeatures();
 		} else {
@@ -1157,13 +1105,11 @@ public class ReviewPage	extends BannerPage {
 			}
 
 			setPageComplete(validationStatus == null
-					|| validationStatus.getCode() == IStatus.WARNING);
+					|| validationStatus.getSeverity() == IStatus.WARNING);
 
 			updateWizardMessage();
 			
 			treeViewer.update(getSelectedJobs(), null);
-			statusButton.setEnabled(validationStatus != null && validationStatus.getCode() != IStatus.OK);
-
 			return validationStatus;
 		}
 	}
@@ -1176,7 +1122,7 @@ public class ReviewPage	extends BannerPage {
                 selectedJobs.add(selected[i]);
         return (IInstallFeatureOperation[])selectedJobs.toArray(new IInstallFeatureOperation[selectedJobs.size()]);
 	}
-
+	
 	public void validateSelection(IProgressMonitor monitor) {
 		IInstallFeatureOperation[] jobs;
 
@@ -1188,8 +1134,8 @@ public class ReviewPage	extends BannerPage {
 		});
 		if (monitor.isCanceled()) return;
 		jobs = bag[0];
-		validationStatus = OperationsManager.getValidator()
-				.validatePendingChanges(jobs);
+		setValidationStatus(OperationsManager.getValidator()
+				.validatePendingChanges(jobs));
 		problematicFeatures.clear();
 		if (monitor.isCanceled()) return;
 		if (validationStatus != null) {
@@ -1207,10 +1153,11 @@ public class ReviewPage	extends BannerPage {
 		treeViewer.getControl().getDisplay().syncExec(new Runnable() {
 			public void run() {
 				setPageComplete(validationStatus == null
-						|| validationStatus.getCode() == IStatus.WARNING);
-
+						|| validationStatus.getSeverity() == IStatus.WARNING);
+/*
 				statusButton.setEnabled(validationStatus != null
-						&& validationStatus.getCode() != IStatus.OK);
+						&& validationStatus.getSeverity() != IStatus.OK);
+						*/
 
 				updateWizardMessage();
 			}
@@ -1253,10 +1200,11 @@ public class ReviewPage	extends BannerPage {
 	 * Update status in the wizard status area
 	 */
 	private void updateWizardMessage() {
+		
 		if (validationStatus == null) {
 			lastDisplayedStatus=null;
 			setErrorMessage(null);
-		} else if (validationStatus.getCode() == IStatus.WARNING) {
+		} else if (validationStatus.getSeverity() == IStatus.WARNING) {
 			lastDisplayedStatus=null;
 			setErrorMessage(null);
 			setMessage(validationStatus.getMessage(), IMessageProvider.WARNING);
@@ -1313,7 +1261,7 @@ public class ReviewPage	extends BannerPage {
 	class StatusDialog extends ErrorDialog {
 //		Button detailsButton;
 		public StatusDialog() {
-			super(getContainer().getShell(), UpdateUIMessages.InstallWizard_ReviewPage_invalid_short, null, //$NON-NLS-1$
+			super(getContainer().getShell(), UpdateUIMessages.InstallWizard_ReviewPage_invalid_short, null, 
 					validationStatus, IStatus.OK | IStatus.INFO
 							| IStatus.WARNING | IStatus.ERROR);
 		}
@@ -1426,44 +1374,144 @@ public class ReviewPage	extends BannerPage {
 			}
 	}
 
-	public boolean isFeatureGood(IImport requiredFeature,
-			IInstallFeatureOperation feature) {
+	
+	public boolean isFeatureGood(IImport requiredFeature, IFeature feature) {
+		return isFeatureGood(requiredFeature, feature, new ArrayList());
+	}
+	
+	public boolean isFeatureGood(IImport requiredFeature, IFeature feature, List visitedFeatures) {
 
-		if (!requiredFeature.getVersionedIdentifier().getIdentifier().equals(
-				feature.getFeature().getVersionedIdentifier().getIdentifier())) {
+		if (requiredFeature.getKind() == IImport.KIND_FEATURE) { 
+			if ((!requiredFeature.getVersionedIdentifier().getIdentifier().equals(
+					feature.getVersionedIdentifier().getIdentifier()))) {
+				IIncludedFeatureReference[] iifr = null;
+				try {
+					iifr = feature.getIncludedFeatureReferences();
+				} catch (CoreException e) {
+					UpdateUI.logException(e);
+					//	if we can not get included features then they can not satisfy requirement, so just ignore them
+					return false;
+				}
+				if (iifr == null) {
+					return false;
+				}
+				
+				for(int i = 0; i < iifr.length; i++) {
+					IFeature current;
+					try {
+						current = UpdateUtils.getIncludedFeature(feature, iifr[i]);
+					} catch (CoreException e) {
+						// if we can not get feature then it can not satisfy requirement, so just ignore it
+						UpdateUI.logException(e);
+						continue;
+					}
+					if (!visitedFeatures.contains(current)) {
+						visitedFeatures.add(current);
+						if (isFeatureGood(requiredFeature, current, visitedFeatures)) {
+							return true;
+						}
+					}
+				}
+				
+				return false;
+			}
+
+			int rule = (requiredFeature.getRule() != IImport.RULE_NONE) ? requiredFeature.getRule() : IImport.RULE_COMPATIBLE;
+
+			switch (rule) {
+			case IImport.RULE_PERFECT: return feature.getVersionedIdentifier().getVersion().isPerfect(
+								requiredFeature.getVersionedIdentifier()
+								.getVersion());
+			case IImport.RULE_EQUIVALENT:
+						return feature.getVersionedIdentifier().getVersion()
+						.isEquivalentTo(
+								requiredFeature.getVersionedIdentifier()
+								.getVersion());
+			case IImport.RULE_COMPATIBLE:
+				return feature.getVersionedIdentifier().getVersion()
+						.isCompatibleWith(
+								requiredFeature.getVersionedIdentifier()
+								.getVersion());
+			case IImport.RULE_GREATER_OR_EQUAL:
+						return feature.getVersionedIdentifier().getVersion()
+						.isGreaterOrEqualTo(
+								requiredFeature.getVersionedIdentifier()
+								.getVersion());
+			}
+
+			return false;
+		} else {
+			if ((requiredFeature.getKind() == IImport.KIND_PLUGIN)) { 
+				return checkIfFeatureHasPlugin( requiredFeature, feature);
+			}
 			return false;
 		}
-
-		int rule = (requiredFeature.getRule() != IImport.RULE_NONE) ? requiredFeature
-				.getRule()
-				: IImport.RULE_COMPATIBLE;
-
-		switch (rule) {
-		case IImport.RULE_PERFECT:
-			return feature.getFeature().getVersionedIdentifier().getVersion()
-					.isPerfect(
-							requiredFeature.getVersionedIdentifier()
-									.getVersion());
-		case IImport.RULE_EQUIVALENT:
-			return feature.getFeature().getVersionedIdentifier().getVersion()
-					.isEquivalentTo(
-							requiredFeature.getVersionedIdentifier()
-									.getVersion());
-		case IImport.RULE_COMPATIBLE:
-			return feature.getFeature().getVersionedIdentifier().getVersion()
-					.isCompatibleWith(
-							requiredFeature.getVersionedIdentifier()
-									.getVersion());
-		case IImport.RULE_GREATER_OR_EQUAL:
-			return feature.getFeature().getVersionedIdentifier().getVersion()
-					.isGreaterOrEqualTo(
-							requiredFeature.getVersionedIdentifier()
-									.getVersion());
+	}
+	
+	private boolean checkIfFeatureHasPlugin(IImport requiredFeature, IFeature feature)  {
+		
+		IPluginEntry[] plugins = feature.getPluginEntries();
+		try {			
+			List includedPlugins = getPluginEntriesFromIncludedFeatures(feature, new ArrayList(), new ArrayList());
+			includedPlugins.addAll(Arrays.asList(plugins));
+			plugins = (IPluginEntry[])includedPlugins.toArray( new IPluginEntry[includedPlugins.size()]);
+		} catch( CoreException ce) {
+			UpdateUI.logException(ce);
+			// ignore this plugins can not sutisfy requirement anyways
 		}
-
+		if (plugins == null) {
+			return false;
+		}
+		
+		for(int i = 0; i < plugins.length; i++) {
+			if (isMatch(plugins[i].getVersionedIdentifier(), requiredFeature.getVersionedIdentifier(), requiredFeature.getIdRule())) {
+				return true;
+			}
+		}
+		
 		return false;
 	}
 
+	private List getPluginEntriesFromIncludedFeatures(IFeature feature, List plugins, List visitedFeatures) throws CoreException {
+		IIncludedFeatureReference[] iifr = feature.getIncludedFeatureReferences();
+		for(int i = 0; i < iifr.length; i++) {
+			IFeature current = UpdateUtils.getIncludedFeature( feature, iifr[i]);
+			if (!visitedFeatures.contains(current)) {
+				IPluginEntry[] pluginEntries = current.getPluginEntries();
+				plugins.addAll(Arrays.asList(pluginEntries));
+				visitedFeatures.add(current);
+				getPluginEntriesFromIncludedFeatures(current, plugins, visitedFeatures);
+			}
+		}
+		
+		return plugins;
+	}
+
+	// vid1 = feature
+	// vid2 = requiredFeature
+	private boolean isMatch( VersionedIdentifier vid1, VersionedIdentifier vid2, int rule) {
+		
+		if (!vid1.getIdentifier().equals(vid2.getIdentifier())) {
+			return false;
+		}
+		if ( vid2.getVersion().getMajorComponent() == 0 && vid2.getVersion().getMinorComponent() == 0 && vid2.getVersion().getServiceComponent() == 0 ) {
+			//version is ignored
+			return true;
+		}
+		switch (rule) {
+		case IImport.RULE_PERFECT:
+			return vid1.getVersion().isPerfect(vid2.getVersion());
+		case IImport.RULE_EQUIVALENT:
+			return vid1.getVersion().isEquivalentTo(vid2.getVersion());
+		case IImport.RULE_COMPATIBLE:
+			return vid1.getVersion().isCompatibleWith(vid2.getVersion());
+		case IImport.RULE_GREATER_OR_EQUAL:
+			return vid1.getVersion().isGreaterOrEqualTo(vid2.getVersion());
+		}
+		return false;
+	}
+
+
 	public boolean isFeatureBetter(IInstallFeatureOperation feature,
 			IInstallFeatureOperation currentFeatureSelected) {
 
@@ -1480,9 +1528,8 @@ public class ReviewPage	extends BannerPage {
 			IImport requiredFeature, IInstallFeatureOperation feature,
 			IInstallFeatureOperation currentFeatureSelected) {
 
-		if (isFeatureGood(requiredFeature, feature)
-				&& isFeatureBetter(feature, currentFeatureSelected)) {
-			return feature;
+		if (isFeatureGood(requiredFeature, feature.getFeature()) && isFeatureBetter(feature, currentFeatureSelected)) {
+				return feature;
 		} else {
 			return currentFeatureSelected;
 		}
